/*
 * File: $Id: ClinicalSpectroscopyWorklistPanel.java,v 1.1.1.1 $
 * History
 * Date: 12.12.2016  Authors: Hans Slotboom
 *
 * Changes:
 *
 * Copyright (C) 2016 by
 *
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */
 
package org.spectrim.clinicalcasehandling;
 
import java.awt.Dimension;
import java.awt.event.MouseListener;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JPanel;
import javax.swing.JPopupMenu;

import javax.swing.JScrollPane;
import javax.swing.table.DefaultTableModel;

import javax.swing.BoxLayout;
import javax.swing.BorderFactory;


import java.util.Iterator;
import java.util.Vector;
import org.spectrim.automaticprocessing.AutomaticProcessingFileHandler;
import org.spectrim.batchprocessing.BatchProcessing;


/**
 *
 * JPanel allowing the use to select detected incoming spectral DICOM data
 * It also gives information on the processing status, i.e whether automatic
 * processing has been performed, is being performed or is pending.
 * 
 * @author hans
 */

public class ClinicalSpectroscopyWorklistPanel extends JPanel implements ActionListener
{
    /**
     * Creates a new instance of ClinicalSpectroscopyIncomingWorklistControl
     */
    public ClinicalSpectroscopyWorklistPanel( )
    {

        m_tablePanel = new JPanel( );
        m_tablePanel.setLayout( new BoxLayout( m_tablePanel, BoxLayout.X_AXIS ) );
        m_tablePanel.setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ) );

        m_dicomSpectroIncomingWorklistTable = new DicomSpectroWorklistTable( );

        // JList does not provide a scroll list automatically so we have to
        // add it ourselves.
        m_listScrollPane = new JScrollPane( );
        m_listScrollPane.getViewport( ).setView( m_dicomSpectroIncomingWorklistTable );
        m_listScrollPane.setPreferredSize( new Dimension( 600, 200 ) );

        m_tablePanel.add( m_listScrollPane );

        setLayout( new BoxLayout( this, BoxLayout.X_AXIS ) );

        setMinimumSize( new Dimension( 810, 210 ) );
        setPreferredSize( new Dimension( 810, 210 ) );

        setBorder( BorderFactory.createCompoundBorder(
                   BorderFactory.createLoweredBevelBorder( ),
                   BorderFactory.createTitledBorder( "Detected incoming DICOM spectroscopy files" ) ) );

        add( m_tablePanel );
        BatchProcessing.addActionListener( m_dicomSpectroIncomingWorklistTable );
    }

    /**
     * Returns the selected series name of the selected treenode
     * @return selected series name
     */
    public String getSelectedSeriesName( )
    {
        return m_selectedNodeName;
    }

    /**
     * Returns the selected series name of the selected treenode
     * @return returns the series number of the selected node
     */
    public int getSelectedSeriesNumber( )
    {
        return m_selectedSeriesNumber.intValue( );
    }

    /**
     * Returns the file references of all selected files from the table
     * @return the file system reference of the selected node
     */
    public Vector<String> getSelectedFileReferencesFromTable( )
    {
        return m_dicomSpectroIncomingWorklistTable.getSelectedFileReferencesFromTable( );
    }


    // Notify all listeners that a change has taken place in this object
    protected void notifyActionListeners( int evType, String cmd )
    {
        Iterator it = m_listeners.iterator( );
        // notify registered listeners
        while( it.hasNext( ) )
        {
            ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, evType, cmd ) );
        }
    }

    /**
     * Adds an action listener to the list of action listeners who are notified
     * when something happens in this component
     */
    public void addActionListener( ActionListener e )
    {
        m_listeners.add( e );
    }

    /**
     * Returns the TreeSet contain the file references in the selected series
     * @return a linked list of selected file references
     */
    public Vector<String> getSelectedSeriesFileReferences( )
    {
        return m_dicomSpectroIncomingWorklistTable.getSelectedSeriesFileReferences( );
    }

   
    /**
     * Sets all elements of the table according to the Vector
     * @param worklistElements
     */
    void changeSpectroWorklistDataElement( ClinicalSpectroscopyCaseWorklistDataElement worklistElement )
    {
        m_dicomSpectroIncomingWorklistTable.changeListDataElement( worklistElement );
    }

    /**
     * Sets all elements of the table according to the Vector
     * @param worklistElements
     */
    public void addWorkListDataElement( ClinicalSpectroscopyCaseWorklistDataElement worklistElement )
    {
        
        System.out.println( "####   >>>>   ##### m_dicomSpectroIncomingWorklistData " + m_dicomSpectroIncomingWorklistData.size( ) );
        
        m_dicomSpectroIncomingWorklistData.add( worklistElement );
        m_dicomSpectroIncomingWorklistTable.addRow( worklistElement );

        if ( worklistElement.getSpectroscopyDataType().compareTo( ClinicalSpectroscopyCaseWorklistDataElement.CSI_DATA_TYPE_STRING ) == 0  )
        {
            notifyActionListeners( RECEIVED_CSI_SPECTROSCOPY_DATAFILE_ID,
                                   RECEIVED_CSI_SPECTROSCOPY_DATAFILE_COMMAND );
        }
        else
        {
            notifyActionListeners( RECEIVED_SVS_SPECTROSCOPY_DATAFILE_ID,
                                   RECEIVED_SVS_SPECTROSCOPY_DATAFILE_COMMAND );
        }  
    }


    /**
     * Returns the spectrotroscopy worklist data
     * @return
     */
    public Vector<ClinicalSpectroscopyCaseWorklistDataElement> getAllSpectroWorklistDataElements( )
    {
        return m_dicomSpectroIncomingWorklistData;
    }
    
    /**
     * Synchronizes the table content before displaying
     */
    public void syncTableContent( ) 
    {
        m_dicomSpectroIncomingWorklistTable.clearDicomSpectroWorklistTable();
        for ( int i = 0; i < m_dicomSpectroIncomingWorklistData.size(); i++ )
        {
            m_dicomSpectroIncomingWorklistTable.addRow(m_dicomSpectroIncomingWorklistData.get(i));
        }
    }
    
    @Override
    public void actionPerformed(ActionEvent e) {
        Class sender = e.getClass( );
        
        if ( e.getSource() instanceof AutomaticProcessingFileHandler )
        {
            
            if ( e.getID() == AutomaticProcessingFileHandler.ADD_NEW_FILE_TO_PROCESS )
            {
                String filePath = (String)e.getActionCommand( );
                ClinicalSpectroscopyCaseWorklistDataElement elem = new ClinicalSpectroscopyCaseWorklistDataElement( filePath );
                addWorkListDataElement( elem );
            }
            
//            if ( e.getID( ) == AutomaticProcessingFileHandler.getInstance( ).ADD_NEW_FILE_TO_PROCESS )
//            {
//                
//            }
//            
//            if ( e.getID( ) == AutomaticProcessingFileHandler.getInstance( ).ADD_NEW_FILE_TO_PROCESS )
//            {
//                
//            }
            
        }
    }
    
    

    // Member variables
    // Listeners to textfield changes
    private Vector<ActionListener> m_listeners = new Vector<ActionListener>( );

    private String m_selectedNodeName;
    private Integer m_selectedSeriesNumber;
    // Two sub panels to put the treeview and the listview upon
    //JPanel m_treePanel;
    JPanel m_tablePanel;

    // Contains the list of images belonging to a series study
    DicomSpectroWorklistTable   m_dicomSpectroIncomingWorklistTable;
    DefaultTableModel                   m_tableModel;
    // Object to store the list items of the image series data into
    Object[][]                          m_cellData;

    JScrollPane                         m_listScrollPane;

    // Data structure containing all file informations of the mounted directory
    protected Vector<ClinicalSpectroscopyCaseWorklistDataElement> m_dicomSpectroIncomingWorklistData = new Vector<ClinicalSpectroscopyCaseWorklistDataElement>( );
 
    MouseListener m_treeMouseListener;

    // variable indicating whether series contains image data or else
    boolean m_isImageSeries;

    // variable to indicate that a file system is mounted
    protected boolean m_directoryIsMounted = false;

    protected JPopupMenu m_popupMenu;

    public final static int     RECEIVED_CSI_SPECTROSCOPY_DATAFILE_ID        = 0;
    public final static String  RECEIVED_CSI_SPECTROSCOPY_DATAFILE_COMMAND   = "TABLE_UPDATED_WITH_CSI_DATA_ELEMENT";

    public final static int     RECEIVED_SVS_SPECTROSCOPY_DATAFILE_ID          = 1;
    public final static String  RECEIVED_SVS_SPECTROSCOPY_DATAFILE_COMMAND     = "TABLE_UPDATED_WITH_SVS_DATA_ELEMENT";

    // For proper serialization
    private static final long serialVersionUID = 9982128799622883L;
    
}