/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.colormap;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.text.DecimalFormat;
import org.spectrim.utilities.ViewingTools;

/**
 *
 * @author Nuno
 */
public class ColorBar {

    protected int width;
    protected int height;
    protected int x;
    protected int y;
    protected ColorMap colormap;
    protected int mode = 1;
    protected float valueOnFocus;
    protected int yValueOnFocus;

    /**
     * Default constructor
     */
    public ColorBar(ColorMap cmap) {
        this.colormap = cmap;
    }

    public void setBounds(int x, int y, int width, int height) {
        this.x = x;
        this.y = y;
        this.width = width;
        this.height = height;
    }

    public int[] getDimensions() {
        return new int[]{x, y, width, height};
    }

    public boolean isPointOver(int x, int y) {
        y = y - 2;
        if (x <= (this.x + width)
                && y >= this.y && y <= (this.y + height)) {
            return true;
        } else {
            return false;
        }
    }

    /**
     *
     */
    public void mouseListener(int x, int y) {
        if (isPointOver(x, y)) {
            yValueOnFocus = y + 1;
            y = y - 2;
            valueOnFocus = ((float) (y - this.y) / (float) height)
                    * (colormap.getMin() - colormap.getMax()) + colormap.getMax();
        } else {
            valueOnFocus = Float.NaN;
        }
    }

    /**
     * Paint the Colorbar onto a Graphics object.
     */
    public synchronized void paintComponent(Graphics g) {
        Graphics2D g2D = null;
        if (g instanceof Graphics2D) {
            g2D = (Graphics2D) g;
        } else {
            return;
        }

        DecimalFormat df = new DecimalFormat("#.###");
        DecimalFormat df2 = new DecimalFormat("#");
        float minscale = colormap.getMin();
        float maxscale = colormap.getMax();

        g.setColor(Color.BLACK);
        g.fillRect(x - 2, y - 2, width + 4, height + 4);
        g.setColor(Color.WHITE);
        g.fillRect(x - 1, y - 1, width + 2, height + 2);

        double logmax = 0;
        double logmin = 0;
        if (colormap.getMax() != Float.NaN && colormap.getMin() != Float.NaN && (colormap.getMax() - colormap.getMin()) > 0) {
            logmax = Math.floor(Math.log10(Math.abs(colormap.getMax())));
            logmin = Math.floor(Math.log10(Math.abs(colormap.getMin())));
            if (logmax < logmin) {
                logmax = logmin;
            }

            for (int n = 0; n <= 256; n++) {
                int h = (int) ((float) height * (1 - ((float) n / 256f)));

                float val = (((float) n) * (maxscale - minscale) / 256f) + minscale;

                g.setColor(colormap.getColor(val, 255));
                g.fillRect(x, y, width, h);
                int spacing = 32;
                if (height < 200) {
                    spacing = 64;
                }
                if (n % spacing == 0) {
                    val = (float) (val / Math.pow(10, logmax));
                    if (n == 256 && logmax != 0) {
                        ViewingTools.drawShadowedString(g, "(x10^" + df2.format(logmax) + ")", x, y - 10);
                    }
                    ViewingTools.drawShadowedString(g, "- " + df.format(val), x + width, y + h + 4 - Math.round((256f - (float) n) / 256f));
                }
            }
            if (!Float.isNaN(valueOnFocus)) {
                float val = (float) (valueOnFocus / Math.pow(10, Math.floor(Math.log10(maxscale))));
                ViewingTools.drawShadowedString(g, "- " + df.format(val), x + width, this.yValueOnFocus);
            }
        }
    }

}
