/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.colormap;

import com.jmatio.types.MLArray;
import com.jmatio.types.MLNumericArray;
import java.awt.Color;
import java.io.File;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import org.spectrim.model.GlobalModel;
import org.spectrim.selectionprofile.SelectionProfileCorrection;
import org.spectrim.utilities.MatlabIO;

/**
 *
 * @author Nuno
 */
public class ColorMap {

    private String name;
    private float min;
    private float max;
    private float[] minTriChannel;
    private float[] maxTriChannel;
    private HashMap<String, Float> mapValues = new LinkedHashMap<>();
    private LinkedHashMap<String, double[]> mapValuesTriChannel = new LinkedHashMap<>();
    private boolean isColorBarVisible = true;
    private ColorBar colorbar;
    private GlobalModel model;
//  Color scales
    private int currentScale;
    private ArrayList<ColorScale> colorScales;
//  Mapping functions
    private int currentFunction;
    private ArrayList<MapFunction> functions;
    public boolean isTriChannel = false;

    public ColorMap(GlobalModel model) {
        this.model = model;
        colorbar = new ColorBar(this);
        constructColorScales();
        constructMappingFunctions();
    }

    public ColorMap(ColorMap cpColoMap, GlobalModel model) {
        this.model = model;
        colorbar = new ColorBar(this);
        constructColorScales();
        constructMappingFunctions();
        min = cpColoMap.min;
        max = cpColoMap.max;
        if (cpColoMap.minTriChannel != null) {
            minTriChannel = cpColoMap.minTriChannel.clone();
        }
        if (cpColoMap.maxTriChannel != null) {
            maxTriChannel = cpColoMap.maxTriChannel.clone();
        }
        if (cpColoMap.mapValues != null) {
            mapValues = (HashMap<String, Float>) cpColoMap.mapValues.clone();
        }
        if (cpColoMap.mapValuesTriChannel != null) {
            mapValuesTriChannel = (LinkedHashMap<String, double[]>) cpColoMap.mapValuesTriChannel.clone();
        }
        isColorBarVisible = cpColoMap.isColorBarVisible;
        //  Color scales
        currentScale = cpColoMap.currentScale;
        if (cpColoMap.colorScales != null) {
            colorScales = (ArrayList<ColorScale>) cpColoMap.colorScales.clone();
        }
        //  Mapping functions
        currentFunction = cpColoMap.currentFunction;
        if (cpColoMap.functions != null) {
            functions = (ArrayList<MapFunction>) cpColoMap.functions.clone();
        }
        isTriChannel = cpColoMap.isTriChannel;
    }

    public void setName(String txt) {
        name = txt;
    }

    public String getName() {
        return name;
    }

    private void constructColorScales() {
        colorScales = new ArrayList<>();
        colorScales.add(new ColorScale("Rainbow 1", new Color[]{Color.BLUE, Color.CYAN, Color.YELLOW, Color.RED}));
        colorScales.add(new ColorScale("Rainbow 2", new Color[]{Color.BLUE, Color.CYAN, Color.GREEN, Color.YELLOW, Color.RED}));
        colorScales.add(new ColorScale("Rainbow 3", new Color[]{Color.BLUE, Color.CYAN, Color.YELLOW, Color.RED, Color.MAGENTA}));
        colorScales.add(new ColorScale("Rainbow 4", new Color[]{Color.BLUE, Color.CYAN, Color.GREEN, Color.YELLOW, Color.RED, Color.MAGENTA}));
        colorScales.add(new ColorScale("Black-Red-Blue-White", new Color[]{Color.BLACK, Color.RED, Color.MAGENTA, Color.BLUE, Color.CYAN, Color.WHITE}));
        colorScales.add(new ColorScale("Black-Blue-Red-White", new Color[]{Color.BLACK, Color.BLUE, Color.MAGENTA, Color.RED, Color.YELLOW, Color.WHITE}));
        colorScales.add(new ColorScale("Blue-Red", new Color[]{Color.BLUE, Color.RED}));
        colorScales.add(new ColorScale("Red-Blue", new Color[]{Color.RED, Color.BLUE}));
        colorScales.add(new ColorScale("White-Black", new Color[]{Color.WHITE, Color.BLACK}));
        colorScales.add(new ColorScale("Black-White", new Color[]{Color.BLACK, Color.WHITE}));
        colorScales.add(new ColorScale("Blue-Cyan-White", new Color[]{Color.BLUE, Color.CYAN, Color.WHITE}));
        colorScales.add(new ColorScale("Red-Yellow-White", new Color[]{Color.RED, Color.YELLOW, Color.WHITE}));
    }

    private void constructMappingFunctions() 
    {
        functions = new ArrayList<>( );
        
        functions.add( new MapFunction( "FD Peak SNR", this.model.getSpectra( ), 1, MapFunction.FD_RANGE ) 
        {
            @Override
            public float function( String coord, double... limits ) 
            {
                float valaux;
                valaux = (float) spectra.getPeakSNR( limits[0], limits[1], coord );
                return valaux;
            }
        });
        
        functions.add( new MapFunction( "FD Magnitude Sum", this.model.getSpectra( ), 1, MapFunction.FD_RANGE ) 
        {
            @Override
            public float function( String coord, double... limits ) 
            {
                float valaux;
                valaux = (float) spectra.getMagnitudeSumFD( coord, limits[0], limits[1] );
                return valaux;
            }
        });
        
        functions.add( new MapFunction( "FD Absorption Sum", this.model.getSpectra( ), 1, MapFunction.FD_RANGE ) 
        {
            @Override
            public float function( String coord, double... limits ) 
            {
                float valaux;
                valaux = (float) spectra.getAbsorptionSumFD( coord, limits[0], limits[1] );
                return valaux;
            }
        });
        
        functions.add(new MapFunction( "FD Regions Ratio", this.model.getSpectra( ), 2, MapFunction.FD_RANGE ) 
        {
            @Override
            public float function( String coord, double... limits ) 
            {
                float valaux;
                valaux = (float) spectra.getPeakSNR( limits[0], limits[1], coord );
                valaux /= (float) spectra.getPeakSNR( limits[2], limits[3], coord );
                return valaux;
            }
        });
        
        functions.add( new MapFunction( "Phase", this.model.getSpectra( ) ) 
        {
            @Override
            public float function( String coord, double... limits ) 
            {
                float valaux;
                valaux = (float) spectra.getPhase(coord)[0];
                return valaux;
            }
        });
        
        functions.add( new MapFunction( "TD Noise", this.model.getSpectra( ) ) 
        {
            @Override
            public float function( String coord, double... limits ) 
            {
                float valaux;
                valaux = (float) spectra.getNoiseTD( coord );
                return valaux;
            }
        });
        
        functions.add( new MapFunction("FD Noise", this.model.getSpectra( ) ) 
        {
            @Override
            public float function( String coord, double... limits ) 
            {
                float valaux;
                valaux = (float) spectra.getNoiseFD( coord );
                return valaux;
            }
        });
//        functions.add(new MapFunction("Exp. Fit. T2", this.model.getSpectra()) {
//            @Override
//            public float function(String coord, double... limits) {
//                float valaux;
//                valaux = (float) spectra.expDecayParam(coord)[0];
//                return valaux;
//            }
//        });
//        functions.add(new MapFunction("Exp. Fit Magnitude", this.model.getSpectra()) {
//            @Override
//            public float function(String coord, double... limits) {
//                float valaux;
//                valaux = (float) spectra.expDecayParam(coord)[1];
//                return valaux;
//            }
//        });
//        functions.add(new MapFunction("Exp. Fit Error", this.model.getSpectra()) {
//            @Override
//            public float function(String coord, double... limits) {
//                float valaux;
//                valaux = (float) spectra.expDecayParam(coord)[2];
//                return valaux;
//            }
//        });
        functions.add( new MapFunction("TD Max", this.model.getSpectra( ) ) 
        {
            @Override
            public float function( String coord, double... limits ) 
            {
                float valaux;
                valaux = (float) spectra.getTDStatistics(coord)[0];
                return valaux;
            }
        });
        
        functions.add( new MapFunction( "TD Time of Max", this.model.getSpectra( ) ) 
        {
            @Override
            public float function( String coord, double... limits ) 
            {
                float valaux;
                valaux = (float) spectra.getTDStatistics(coord)[1];
                return valaux;
            }
        });
        
        functions.add( new MapFunction( "TD Mean", this.model.getSpectra( ) ) 
        {
            @Override
            public float function( String coord, double... limits ) 
            {
                float valaux;
                valaux = (float) spectra.getTDStatistics(coord)[2];
                return valaux;
            }
        });
        
        functions.add( new MapFunction( "TD Std. Dev.", this.model.getSpectra( ) ) 
        {
            @Override
            public float function( String coord, double... limits ) 
            {
                float valaux;
                valaux = (float) spectra.getTDStatistics(coord)[3];
                return valaux;
            }
        });
        functions.add( new MapFunction( "TD Skewness", this.model.getSpectra( ) ) 
        {
            @Override
            public float function( String coord, double... limits ) 
            {
                float valaux;
                valaux = (float) spectra.getTDStatistics(coord)[4];
                return valaux;
            }
        });
        
        functions.add( new MapFunction( "TD Kurtosis", this.model.getSpectra( ) ) 
        {
            @Override
            public float function( String coord, double... limits ) 
            {
                float valaux;
                valaux = (float) spectra.getTDStatistics(coord)[5];
                return valaux;
            }
        });
        
        functions.add( new MapFunction( "FD Max", this.model.getSpectra( ) ) 
        {
            @Override
            public float function( String coord, double... limits ) 
            {
                float valaux;
                valaux = (float) spectra.getFDStatistics(coord)[0];
                return valaux;
            }
        });
        
        functions.add( new MapFunction( "FD PPM Max", this.model.getSpectra( ) ) 
        {
            @Override
            public float function( String coord, double... limits ) 
            {
                float valaux;
                valaux = (float) spectra.getFDStatistics(coord)[1];
                return valaux;
            }
        });
        
        functions.add( new MapFunction( "FD Mean", this.model.getSpectra( ) ) 
        {
            @Override
            public float function( String coord, double... limits ) 
            {
                float valaux;
                valaux = (float) spectra.getFDStatistics(coord)[2];
                return valaux;
            }
        });
        
        functions.add( new MapFunction( "FD Std. Dev.", this.model.getSpectra( ) ) 
        {
            @Override
            public float function( String coord, double... limits ) 
            {
                float valaux;
                valaux = (float) spectra.getFDStatistics(coord)[3];
                return valaux;
            }
        });
        
        functions.add( new MapFunction( "FD Skewness", this.model.getSpectra( ) ) 
        {
            @Override
            public float function( String coord, double... limits ) 
            {
                float valaux;
                valaux = (float) spectra.getFDStatistics(coord)[4];
                return valaux;
            }
        });
        functions.add( new MapFunction( "FD Kurtosis", this.model.getSpectra( ) ) 
        {
            @Override
            public float function( String coord, double... limits ) 
            {
                float valaux;
                valaux = (float) spectra.getFDStatistics(coord)[5];
                return valaux;
            }
        });
        
        functions.add( new MapFunction( "TD Mean SNR", this.model.getSpectra( ), 1, MapFunction.TD_RANGE ) 
        {
            @Override
            public float function( String coord, double... limits ) 
            {
                float valaux;
                valaux = (float) spectra.getMeanTDSNR(limits[0], limits[1], coord);
                return valaux;
            }
        });
        
        functions.add( new MapFunction( "Quality Map", this.model.getSpectra( ) ) 
        {
            @Override
            public float function( String coord, double... limits ) 
            {
                float valaux = Float.NaN;
                if ( spectra.getQualityScores( ).containsKey( coord ) ) 
                {
                    valaux = spectra.getQualityScores( ).get(coord).floatValue( );
                }
                return valaux;
            }
        });
        
        functions.add( new MapFunction( "Temperature", this.model.getSpectra( ) ) 
        {
            @Override
            public float function( String coord, double... limits ) 
            {
                float valaux = Float.NaN;
                if ( model.getTemperatures( ).containsKey( coord ) ) 
                {
                    valaux = model.getTemperatures( ).get( coord ).floatValue( );
                }
                return valaux;
            }
        });
        
        functions.add( new MapFunction( "TDFDFit quality number", this.model.getSpectra( ) ) 
        {
            @Override
            public float function( String coord, double... limits ) 
            {
                float valaux = Float.NaN;
                if ( model.getTDFDFitQualityNumbers( ).containsKey( coord ) ) 
                {
                    valaux = model.getTDFDFitQualityNumbers( ).get( coord ).floatValue( );
                }
                return valaux;
            }
        });       
        
//        functions.add(new MapFunction("Alignment Score - Pearson's Correlation coeff.", this.model.getSpectra()) {
//            @Override
//            public float function(String coord, double... limits) {
//                float valaux = Float.NaN;
//                valaux = (float) spectra.getCorrelationAlignment(coord);
//                return valaux;
//            }
//        });
    }

    public ArrayList<String> getColorScalesNames() {
        ArrayList<String> res = new ArrayList<>();
        for (int i = 0; i < colorScales.size(); i++) {
            res.add(colorScales.get(i).getName());
        }
        return res;
    }

    public ArrayList<String> getFunctionsNames() {
        ArrayList<String> res = new ArrayList<>();
        for (int i = 0; i < functions.size(); i++) {
            res.add(functions.get(i).getName());
        }
        return res;
    }

    public MapFunction getFunction(int i) {
        return functions.get(i);
    }

    public int getCurrentFunctionIndex() {
        return currentFunction;
    }

    public MapFunction getCurrentFunction() {
        return functions.get(currentFunction);
    }

    public void setCurrentScale(int currentScale) {
        this.currentScale = currentScale;
        model.updateViewImagePanels();
    }

    public float getMin() {
        return min;
    }

    public float getMax() {
        return max;
    }

    public void clear() {
        mapValues.clear();
        mapValuesTriChannel.clear();
        min = Float.POSITIVE_INFINITY;
        max = Float.NEGATIVE_INFINITY;
        minTriChannel = new float[]{Float.POSITIVE_INFINITY, Float.POSITIVE_INFINITY, Float.POSITIVE_INFINITY};
        maxTriChannel = new float[]{Float.NEGATIVE_INFINITY, Float.NEGATIVE_INFINITY, Float.NEGATIVE_INFINITY};
        model.setGridTransparency(1f);
        model.updateViewImagePanels();
    }

    public boolean isActive() {
        if (!this.isTriChannel) {
            return (!mapValues.isEmpty());
        } else {
            return (!mapValuesTriChannel.isEmpty());
        }
    }

    public boolean isColorBarVisible() {
        return this.isColorBarVisible;
    }

    public void setColorBarVisible(boolean val) {
        this.isColorBarVisible = val;
        model.updateViewImagePanels();
    }

    public void createMap(int function, double... limits) {
        clear();
        Iterator<String> iter = model.getSelectedVoxelsIterator();
        float valaux = 0;
        String coord = "";
        model.setGridTransparency(0.1f);
        currentFunction = function;
        name = this.getFunctionsNames().get(function);

        if (function >= 0 && function < functions.size()) {
            while (iter.hasNext()) {
                coord = (String) iter.next();
                valaux = functions.get(function).function(coord, limits);
                if (valaux < min) {
                    min = valaux;
                }
                if (valaux > max) {
                    max = valaux;
                }
                mapValues.put(coord, valaux);
            }
            model.updateViewImagePanels();
        }
    }

    public HashMap<String, Float> getMapValues(int function, double... limits) {
        clear();
        Iterator<String> iter = model.getSelectedVoxelsIterator();
        float valaux = 0;
        String coord = "";
        model.setGridTransparency(0.1f);
        currentFunction = function;
        name = this.getFunctionsNames().get(function);

        if (function >= 0 && function < functions.size()) 
        {
            while (iter.hasNext()) 
            {
                coord = (String) iter.next();
                valaux = functions.get(function).function(coord, limits);
                if (valaux < min) 
                {
                    min = valaux;
                }
                if (valaux > max) 
                {
                    max = valaux;
                }
                mapValues.put( coord, valaux );
            }
            model.updateViewImagePanels( );
        }
        return mapValues;
    }
    
    
    
    public void setColorMapValues(HashMap<String, Double> values, boolean showJustSelected, String mapName) {
        clear();
        isTriChannel = false;
        model.setGridTransparency(0.1f);
        currentFunction = -1;
        name = mapName;
        if (values != null) {
            if (!values.isEmpty()) {
                Iterator<String> iter = values.keySet().iterator();
                float val;
                String aux;
                while (iter.hasNext()) {
                    aux = iter.next();
                    if ((showJustSelected && model.getSelectedVoxels().contains(aux) || !showJustSelected)) {
                        val = values.get(aux).floatValue();
                        if (val < min) {
                            min = val;
                        }
                        if (val > max) {
                            max = val;
                        }
                        mapValues.put(aux, val);
                    }
                }
                model.updateViewImagePanels( );
            }
        }
    }

    public void setColorMapValues(HashMap<String, Double> values, boolean showJustSelected) {
        this.setColorMapValues(values, showJustSelected, "");
    }

    public void setValueMapFromMAT(String file, String name) {
        clear();
        isTriChannel = false;
        model.setGridTransparency(0.1f);
        HashMap<String, MLArray> maps = MatlabIO.readMAT(file);
        MLNumericArray<Double> ids = (MLNumericArray<Double>) maps.get("Coordinates");
        MLNumericArray<Double> values = (MLNumericArray<Double>) maps.get(name);
        currentFunction = -1;
        this.mapValues.clear();
        this.name = name;
        for (int i = 0; i < ids.getM(); i++) {
            int c1 = ids.getReal(i, 0).intValue() - 1;
            int c2 = ids.getReal(i, 1).intValue() - 1;
            int c3 = ids.getReal(i, 2).intValue() - 1;
            String coord = c1 + "_" + c2 + "_" + c3;
            float val = values.getReal(i).floatValue();
            mapValues.put(coord, val);
            if (val < min) {
                min = val;
            }
            if (val > max) {
                max = val;
            }
        }
        model.updateViewImagePanels();
    }

    public void setColorMapValuesTriChannel(LinkedHashMap<String, double[]> values, boolean showJustSelected) {
        clear();
        isTriChannel = true;
        model.setGridTransparency(0.1f);
        currentFunction = -1;
        name = "";
        if (values != null) {
            if (!values.isEmpty()) {
                Iterator<String> iter = values.keySet().iterator();
                double[] val;
                String aux;
                while (iter.hasNext()) {
                    aux = iter.next();
                    if ((showJustSelected && model.getSelectedVoxels().contains(aux) || !showJustSelected)) {
                        val = values.get(aux);
                        if (val[0] < minTriChannel[0]) {
                            minTriChannel[0] = (float) val[0];
                        }
                        if (val[0] > maxTriChannel[0]) {
                            maxTriChannel[0] = (float) val[0];
                        }
                        if (val[1] < minTriChannel[1]) {
                            minTriChannel[1] = (float) val[1];
                        }
                        if (val[1] > maxTriChannel[1]) {
                            maxTriChannel[1] = (float) val[1];
                        }
                        if (val[2] < minTriChannel[2]) {
                            minTriChannel[2] = (float) val[2];
                        }
                        if (val[2] > maxTriChannel[2]) {
                            maxTriChannel[2] = (float) val[2];
                        }
                        mapValuesTriChannel.put(aux, val);
                    }
                }
                model.updateViewImagePanels();
            }
        }
    }

    public double[] getValueMapStatistics(double... valref) {
        Iterator iter = mapValues.keySet().iterator();
        int count = 0;
        double mean = 0;
        double stdDev = 0;
        double m3 = 0;
        double m4 = 0;
        double skewness = 0;
        double kurtosis = 0;
        double min = Double.POSITIVE_INFINITY;
        double max = Double.NEGATIVE_INFINITY;
        if (valref.length == 0) {
            valref = new double[]{1};
        }
        while (iter.hasNext()) {
            double val = mapValues.get((String) iter.next()) / valref[0];
            mean += val;
            if (val < min) {
                min = val;
            } else if (val > max) {
                max = val;
            };
            count++;
        }
        mean = mean / (double) count;
        iter = mapValues.keySet().iterator();
        while (iter.hasNext()) {
            double val = mapValues.get(iter.next()) / valref[0];
            stdDev += Math.pow((val - mean), 2);
            m3 += Math.pow((val - mean), 3);
            m4 += Math.pow((val - mean), 4);
        }
        skewness = (m3 / count) / Math.pow(Math.sqrt(stdDev / (count - 1)), 3);
        kurtosis = ((m4 / count) / Math.pow(stdDev / count, 2)) - 3;
        stdDev = Math.sqrt(stdDev / count);

        return new double[]{count, min, max, mean, stdDev, skewness, kurtosis};
    }

    public double[] getValueMapSelectionStatistics(double... valref) {
        Iterator iter = mapValues.keySet().iterator();
        int count = 0;
        double mean = 0;
        double stdDev = 0;
        double m3 = 0;
        double m4 = 0;
        double skewness = 0;
        double kurtosis = 0;
        double min = Double.POSITIVE_INFINITY;
        double max = Double.NEGATIVE_INFINITY;
        if (valref.length == 0) {
            valref = new double[]{1};
        }
        while (iter.hasNext()) {
            String coord = (String) iter.next();
            if (model.getSelectedVoxels().contains(coord)) {
                double val = mapValues.get(coord) / valref[0];
                if (!Double.isNaN(val)) {
                    mean += val;
                    if (val < min) {
                        min = val;
                    } else if (val > max) {
                        max = val;
                    };
                    count++;
                }
            }
        }
        mean = mean / (double) count;
        iter = mapValues.keySet().iterator();
        while (iter.hasNext()) {
            String coord = (String) iter.next();
            if (model.getSelectedVoxels().contains(coord)) {
                double val = mapValues.get(coord) / valref[0];
                if (!Double.isNaN(val)) {
                    stdDev += Math.pow((val - mean), 2);
                    m3 += Math.pow((val - mean), 3);
                    m4 += Math.pow((val - mean), 4);
                }
            }
        }
        skewness = (m3 / count) / Math.pow(Math.sqrt(stdDev / (count - 1)), 3);
        kurtosis = ((m4 / count) / Math.pow(stdDev / count, 2)) - 3;
        stdDev = Math.sqrt(stdDev / count);

        return new double[]{count, min, max, mean, stdDev, skewness, kurtosis};
    }

    public void publishValueMapStatistics(boolean onlySelected) {
        if (!mapValues.isEmpty()) {
            double[] stats;
            if (onlySelected) {
                stats = getValueMapSelectionStatistics();
            } else {
                stats = getValueMapStatistics();
            }
            model.printTxt("");
            model.printTxt("Value Map Statistics");
            model.printTxt("Nr Voxels\t" + "Min\t" + "Max\t" + "Mean\t" + "Std Dev\t" + "Skewness\t" + "Kurtosis");
            DecimalFormat format = new DecimalFormat("#.#####");

            model.printNumeric(stats[0] + "\t" + format.format(stats[1]) + "\t" + format.format(stats[2]) + "\t" + format.format(stats[3]) + "\t"
                    + format.format(stats[4]) + "\t" + format.format(stats[5]) + "\t" + format.format(stats[6]), true);
        }
    }

    public Color getColor(float val, int alpha) {
        if (min == max) {
            return colorScales.get(currentScale).getColor(0, alpha);
        }
        val = (val - min) / (max - min);
        return colorScales.get(currentScale).getColor(val, alpha);
    }

    public Color getPolygonColor(int i, int j, int k) {
        Color coloraux = null;
        if (!isTriChannel) {
            if (mapValues.containsKey(i + "_" + j + "_" + k)) {
                coloraux = this.getColor(mapValues.get(i + "_" + j + "_" + k), 127);
            }
        } else {
            if (mapValuesTriChannel.containsKey(i + "_" + j + "_" + k)) {
                double[] valaux = this.mapValuesTriChannel.get(i + "_" + j + "_" + k);
                float r = ((float) (valaux[0] - minTriChannel[0])) / (float) (maxTriChannel[0] - minTriChannel[0]);
                float g = ((float) (valaux[1] - minTriChannel[1])) / (float) (maxTriChannel[1] - minTriChannel[1]);
                float b = ((float) (valaux[2] - minTriChannel[2])) / (float) (maxTriChannel[2] - minTriChannel[2]);
                if (maxTriChannel[0] > 0) {
                    coloraux = new Color(r, g, b, 0.7f);
                }
            }

        }
        return coloraux;
    }

    public double[] getMapValue(int i, int j, int k) {
        if (!isTriChannel) {
            if (mapValues.containsKey(i + "_" + j + "_" + k)) {
                return new double[]{mapValues.get(i + "_" + j + "_" + k)};
            }
        } else {
            if (mapValuesTriChannel.containsKey(i + "_" + j + "_" + k)) {
                return this.mapValuesTriChannel.get(i + "_" + j + "_" + k);
            }
        }
        return null;
    }

    public void setLimits(float min, float max) {
        this.min = min;
        this.max = max;
        model.updateViewImagePanels();
    }

    public void setMin(float min) {
        this.min = min;
        model.updateViewImagePanels();
    }

    public void setMax(float max) {
        this.max = max;
        model.updateViewImagePanels();
    }

    public ColorBar getColorBar() {
        return colorbar;
    }

    public void smoothColorMap() {
        Iterator<String> iter = this.mapValues.keySet().iterator();
        HashMap<String, Float> filteredMap = new HashMap<>();
        float min = Float.POSITIVE_INFINITY;
        float max = Float.NEGATIVE_INFINITY;
        while (iter.hasNext()) {
            String coord = iter.next();
            String[] parts = coord.split("_");
            float newVal = 0f;
            float weightSum = 0f;
            for (int x = -2; x <= 2; x++) {
                for (int y = -2; y <= 2; y++) {
                    String newCoord = (Integer.valueOf(parts[0]) + x) + "_" + (Integer.valueOf(parts[1]) + y) + "_" + parts[2];
                    if (mapValues.containsKey(newCoord)) {
                        if (!mapValues.get(newCoord).isNaN()) {
                            double weight = Math.exp(-(Math.pow(x, 2) + Math.pow(y, 2)) / 1);
                            newVal += weight * mapValues.get(newCoord);
                            weightSum += weight;
                        }
                    }
                }
            }
            newVal /= weightSum;
            if (newVal > max) {
                max = newVal;
            }
            if (newVal < min) {
                min = newVal;
            }
            filteredMap.put(coord, newVal);
        }
        this.min = min;
        this.max = max;
        mapValues = filteredMap;
    }

    public void saveMapToMAT(File file, boolean normalize) {
        MatlabIO matio = new MatlabIO();

        int minX = Integer.MAX_VALUE;
        int maxX = Integer.MIN_VALUE;
        int minY = Integer.MAX_VALUE;
        int maxY = Integer.MIN_VALUE;

        float mean = 0f;
        int nr = 0;
        Iterator<String> iter = this.mapValues.keySet().iterator();
        while (iter.hasNext()) {
            String coord = iter.next();
            String[] coords = coord.split("_");
            int x = Integer.valueOf(coords[0]);
            int y = Integer.valueOf(coords[1]);
            if (x < minX) {
                minX = x;
            }
            if (x > maxX) {
                maxX = x;
            }
            if (y < minY) {
                minY = y;
            }
            if (y > maxY) {
                maxY = y;
            }
            mean += mapValues.get(coord);
            nr++;
        }
        mean /= nr;

        float[][] values = new float[maxX - minX + 1][maxY - minY + 1];
        String[][] coords = new String[maxX - minX + 1][maxY - minY + 1];

        for (int i = minX; i <= maxX; i++) {
            for (int j = minY; j <= maxY; j++) {
                String coord = i + "_" + j + "_" + 0;
                float val = 0f;
                if (mapValues.containsKey(coord)) {
                    if (normalize) {
                        val = mapValues.get(coord) / mean;
                    } else {
                        val = mapValues.get(coord);
                    }
                }
                values[i - minX][j - minY] = val;
                coords[i - minX][j - minY] = coord;
            }
        }

        matio.addArrayToList("Xlim", new double[]{minX, maxX});
        matio.addArrayToList("Ylim", new double[]{minY, maxY});
        matio.addArrayToList("mapVals", values);

        matio.save(file.toString());
    }

    public void correctMapFromTemplate(File file) {
        HashMap<String, MLArray> data = MatlabIO.readMAT(file.toString());
        MLNumericArray<Double> Xlim = (MLNumericArray<Double>) data.get("Xlim");
        MLNumericArray<Double> Ylim = (MLNumericArray<Double>) data.get("Ylim");
        MLNumericArray<Double> values = (MLNumericArray<Double>) data.get("mapVals");
        LinkedHashMap<String, Double> map = new LinkedHashMap<>();

        float min = Float.POSITIVE_INFINITY;
        float max = Float.NEGATIVE_INFINITY;
        for (int i = Xlim.get(0).intValue(); i <= Xlim.get(1).intValue(); i++) {
            for (int j = Ylim.get(0).intValue(); j <= Ylim.get(1).intValue(); j++) {
                float val = values.getReal(i - Xlim.get(0).intValue(), j - Ylim.get(0).intValue()).floatValue();
                String coord = i + "_" + j + "_" + 0;
                if (this.mapValues.containsKey(coord)) {
                    val = mapValues.get(coord) / val;
                    mapValues.put(coord, val);
                    if (val > max) {
                        max = val;
                    }
                    if (val < min) {
                        min = val;
                    }
                } else {
                    this.mapValues.remove(coord);
                }
            }
        }
        this.min = min;
        this.max = max;
    }

    public void correctSelectionProfile(SelectionProfileCorrection profileCorrection, String metab) {
        if (profileCorrection.checkMetaboliteName(metab)) {
            ArrayList<String> coords = new ArrayList<>(mapValues.keySet());
            for (String coord : coords) {
                double correctionFact = profileCorrection.getCorrectionFactor(coord, metab);
                if (Double.isNaN(correctionFact) || mapValues.get(coord) == null || Double.isNaN(mapValues.get(coord))) {
                    mapValues.remove(coord);
                } else {
                    mapValues.put(coord, mapValues.get(coord) / (float) correctionFact);
                }
            }
        }
    }

    public void correctSelectionProfileRatio(SelectionProfileCorrection profileCorrection, String metab1, String metab2) {
        if (profileCorrection.checkMetaboliteName(metab1) && profileCorrection.checkMetaboliteName(metab2)) {
            ArrayList<String> coords = new ArrayList<>(mapValues.keySet());
            for (String coord : coords) {
                double correctionFact1 = profileCorrection.getCorrectionFactor(coord, metab1);
                double correctionFact2 = profileCorrection.getCorrectionFactor(coord, metab2);
                if (Double.isNaN(correctionFact1) || Double.isNaN(correctionFact2) || mapValues.get(coord) == null || Double.isNaN(mapValues.get(coord))) {
                    mapValues.remove(coord);
                } else {
                    mapValues.put(coord, mapValues.get(coord) * (float) correctionFact2 / (float) correctionFact1);
                }
            }
        }
    }

}
