/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.data;

import org.spectrim.dicom.DicomDataLoaderPanel;
import java.awt.Color;
import java.awt.image.Raster;
import java.io.File;
import java.io.FilenameFilter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;
import java.util.TreeMap;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JOptionPane;
import javax.swing.SwingWorker;
import org.dcm4che2.data.DicomObject;
import org.spectrim.batchprocessing.BatchProcessing;
import org.spectrim.io.dicom.DicomHandler;
import org.spectrim.io.dicom.DicomHeader;
import org.spectrim.io.dicom.DicomHeaderLine;
import org.spectrim.io.nifti.NiftiHandler;
import org.spectrim.main.DataLoadOptionFrame;
import org.spectrim.model.GlobalModel;
import org.spectrim.view.ImageSlice;
import org.spectrim.view.ImageStack;
import org.spectrim.view.LocalizerPanel;
import org.spectrim.view.MultiFrameParameters;

/**
 * Class handling the loading of Dicom data (images and spectroscopy).
 * 08.05.2017: adding the possibility to do proper loading of multiple
 * spectroscopy instances.
 * @author nunopbarros
 */
public class FileReaderWorker extends SwingWorker {

    private String selectedFile = null;
    private LinkedList<String> selectedFiles = new LinkedList<String>( );
    private GlobalModel csimodel;
    private DicomDataLoaderPanel dicomManager;
    private FileReaderListener parent;
    private boolean isReferenceFile;
    private int mrsloadingOption = DataLoadOptionFrame.LOAD_SINGLE_INSTANCE;
    private int singleInstanceToLoad = 0;
    private int firstInstanceOfTwoToLoad = 0;
    private int secondInstanceOfTwoToLoad = 1;
    private MultiFrameParameters mfp;
    
    public enum filetype {

        IMAGE, MRS_SIEMENS, MRS_STD, PERFUSION, TENSOR, UNKNOWN, IMAGE_MULTI
    };

    public FileReaderWorker(FileReaderListener parent, String selectedFile, DicomDataLoaderPanel dicomManager, GlobalModel model) {
        this.selectedFile = selectedFile;
        this.dicomManager = dicomManager;
        this.csimodel = model;
        this.parent = parent;
    }

    public FileReaderWorker(FileReaderListener parent, String selectedFile, DicomDataLoaderPanel dicomManager, GlobalModel model, boolean isRef) {
        this.selectedFile = selectedFile;
        this.dicomManager = dicomManager;
        this.csimodel = model;
        this.parent = parent;
        isReferenceFile = isRef;
    }

    /**
     * Constructor that allows the loading of multiple spectroscopy instances
     * in a user definable way; eg if you want to load edited data which is 
     * stored in multiple responses, or add separately stored responses 
     * belonging to one series; With the mrsLoadingOption the user can set
     * how the multiple responses can be loaded.
     * @param parent
     * @param selectedFiles
     * @param dicomManager
     * @param model
     * @param mrsLoadingOption 
     */
    public FileReaderWorker( FileReaderListener parent, LinkedList<String> selectedFiles, 
                             DicomDataLoaderPanel dicomManager, 
                             GlobalModel model, int mrsLoadingOption, 
                             int singleInstanceToLoad ) 
    {
        this.selectedFiles = selectedFiles;
        this.dicomManager = dicomManager;
        this.csimodel = model;
        this.parent = parent;
        this.mrsloadingOption = mrsLoadingOption;
        this.singleInstanceToLoad = singleInstanceToLoad;
    }
    
    public FileReaderWorker( FileReaderListener parent, LinkedList<String> selectedFiles, 
                             DicomDataLoaderPanel dicomManager, 
                             GlobalModel model, int mrsLoadingOption, 
                             int firstResponse, int secondResponse ) 
    {
        this.selectedFiles = selectedFiles;
        this.dicomManager = dicomManager;
        this.csimodel = model;
        this.parent = parent;
        this.mrsloadingOption = mrsLoadingOption;
        this.firstInstanceOfTwoToLoad = firstResponse;
        this.secondInstanceOfTwoToLoad = secondResponse;
    }
    
    
    
    private int getWantedInstanceLocation( LinkedList<String> list, int wantedInstanceNumber )
    {
        DicomObject object;
        int currentInstanceNumber = -1;
        int instanceLocatedAt     = -1;
        for ( int i = 0; i < list.size( ); i++ )
        {
            object = DicomHandler.getDicomObject( list.get( i ) );
            currentInstanceNumber = DicomHandler.getInstanceNr( object );
            if ( wantedInstanceNumber == currentInstanceNumber )
            {
                instanceLocatedAt = i;
            }
        }
        return instanceLocatedAt;
    }
    
    
    @Override
    protected GlobalModel doInBackground() throws Exception {
        ImageStack imStack = new ImageStack();
        Raster imageRaster;
        HashMap<Integer, ImageSlice> ImMap = new HashMap<>();
        DicomHeaderLine headerLine;
        String refSeriesPath;
        File[] listOfFiles = new File[3];
        File fileaux = new File("");
        Long t0 = System.currentTimeMillis();
        
        DicomObject object;
        String imagetype;
        String classUID;
        filetype type;
        boolean PhilipsMultiFrameRefImages = false;

        // Now CSIData must be declared here due to several loading options
//        GlobalModel modelaux = new GlobalModel( );
        
        GlobalModel modelaux = this.csimodel;
        
        CSIData csidata = null;
        CSIData instanceToLoadFurther;
//        
//        if ( csimodel.getCSIData( ) == null )
//        {
//            csidata = null;
//            instanceToLoadFurther = null;
//        }
//        else
//        {
//            csidata = csimodel.getCSIData( );
//        }
//        
        // next four lines added to handle more than one MRS instance
        if ( ( selectedFiles != null ) && ( selectedFiles.size( ) > 0 ) )
        {
            
            if ( mrsloadingOption ==  DataLoadOptionFrame.LOAD_ALL_AND_SUM )
            {                
              selectedFile = selectedFiles.get( getWantedInstanceLocation( selectedFiles, 1 ) );
            }
                     
            if ( mrsloadingOption ==  DataLoadOptionFrame.LOAD_SINGLE_INSTANCE_OF_MULTIPLE )
            {
                selectedFile = selectedFiles.get( getWantedInstanceLocation( selectedFiles, singleInstanceToLoad ) );
            }
            
            if ( mrsloadingOption ==  DataLoadOptionFrame.LOAD_DIFF_1_MINUS_2 )
            {
                selectedFile = selectedFiles.get( getWantedInstanceLocation( selectedFiles, firstInstanceOfTwoToLoad ) );
            }
            
            if ( mrsloadingOption ==  DataLoadOptionFrame.LOAD_DIFF_2_MINUS_1 )
            {
                selectedFile = selectedFiles.get( getWantedInstanceLocation( selectedFiles, secondInstanceOfTwoToLoad ) );
            }
            
            if ( mrsloadingOption ==  DataLoadOptionFrame.LOAD_SINGLE_INSTANCE )
            {
                selectedFile = selectedFiles.get( getWantedInstanceLocation( selectedFiles, singleInstanceToLoad ) );
            }  
        }
        
        
        if ( DicomHandler.isDicom( selectedFile ) ) 
        {
//      Read Dicom Data
            
            object = DicomHandler.getDicomObject(selectedFile);
            imagetype = DicomHandler.getImageType(object);
            classUID = DicomHandler.getSOPClassUID(object);
            switch (classUID) {
                case "1.2.840.10008.5.1.4.1.1.4": // Image
                    type = filetype.IMAGE;
                    break;
                case "1.2.840.10008.5.1.4.1.1.4.2": // Spectro std
                    type = filetype.MRS_STD;
                    break;
                case "1.3.12.2.1107.5.9.1": // Spectro/Tensor Siemens
                    if (imagetype.indexOf("TENSOR") >= 0) {
                        type = filetype.TENSOR;
                        type = filetype.IMAGE;
                    } else {
                        type = filetype.MRS_SIEMENS;
                    }
                    break;
                case "1.2.840.10008.5.1.4.1.1.7": // Perfusion derived maps
                    type = filetype.PERFUSION;
                    type = filetype.IMAGE;
                    break;
                case "1.2.840.10008.5.1.4.1.1.4.1":
                    //enhanced MR image
                    type = filetype.IMAGE_MULTI;
                    break;                        
                default: // Unknown format
                    type = filetype.UNKNOWN;
                    type = filetype.IMAGE; //TEMPORARIOOOO
                    break;
            }
            if (type == filetype.UNKNOWN || type == filetype.PERFUSION || type == filetype.TENSOR) {
                System.out.println("File type not suported!");
                this.cancel(true);
                return null;
            } else {
                
                try {
                    DicomHeader header = DicomHandler.getHeader(object);
                    if (type == filetype.MRS_SIEMENS) {
                        DicomHandler.readSiemensData(object, header);
                    }

                    if (type == filetype.MRS_SIEMENS || type == filetype.MRS_STD) {
                        parent.clearResultsData();
                        // depending on whether multiple mrs instances are
                        // selected or not the data has to be loaded differently
                        if ( ( selectedFiles != null ) && ( selectedFiles.size() > 0 ) )
                        {
                            if ( mrsloadingOption == DataLoadOptionFrame.LOAD_ALL_AND_SUM )
                            {
                                
                                csidata  = new CSIData( header, selectedFiles.get( getWantedInstanceLocation( selectedFiles, 1 ) ) );
                                
                                for ( int i = 2; i <= selectedFiles.size( ); i++ ) 
                                {
                                    int locationOfInstance = getWantedInstanceLocation( selectedFiles, secondInstanceOfTwoToLoad );
                                    
                                    if ( locationOfInstance != -1 )
                                    {
                                        DicomObject object2 = DicomHandler.getDicomObject( selectedFiles.get( locationOfInstance ) );
                                        DicomHeader header2 = DicomHandler.getHeader( object2 );
                                        DicomHandler.readSiemensData( object2, header2 );
                                    
                                        // Normally you also have to read header information to 
                                        // get a complete CSIData object in the end.
                                        // Currently this works, but more properly it would be
                                        // to do the following statement as well:
                                        // DicomHandler.readSiemensData(object2, header2);
                                        int locationOfFurtherInstance = getWantedInstanceLocation( selectedFiles, i );
                                        if ( locationOfFurtherInstance != -1 )
                                        {
                                            instanceToLoadFurther = new CSIData( header2, selectedFiles.get( locationOfFurtherInstance ) );
                                            float[] toAdd = instanceToLoadFurther.getComplexResponse( );
//                                          System.out.println(" Added response : " + i );
                                            csidata.addResponse( toAdd );
                                        }
                                        else
                                        {
                                            JOptionPane.showMessageDialog( null, "Warning One of the two signals was not defined: \n Possibly the loaded signal is not the sum of the signals you intended to load!." );
                                        }
                                    }
                                    else
                                    {
                                        JOptionPane.showMessageDialog( null, "Warning One of the two signals was not defined: \n Possibly the loaded signal is not the sum of signals you intended to load!." );
                                    }
                                    
                                }
                                csidata.conjugate( );
                            }
                            
                            if ( mrsloadingOption == DataLoadOptionFrame.LOAD_SINGLE_INSTANCE )
                            {
                                if ( singleInstanceToLoad == 0 )
                                {
                                    csidata = new CSIData( header, selectedFiles.get( getWantedInstanceLocation( selectedFiles, 0 ) ) );
                                }
                                else
                                {
                                    csidata = new CSIData( header, selectedFiles.get( getWantedInstanceLocation( selectedFiles, singleInstanceToLoad ) ) );
                                    csidata.setResponseToZero( );
                                    
                                    DicomObject object2 = DicomHandler.getDicomObject( selectedFiles.get( getWantedInstanceLocation( selectedFiles, singleInstanceToLoad ) ) );
                                    DicomHeader header2 = DicomHandler.getHeader( object2 );
                                    DicomHandler.readSiemensData(object2, header2);
                                    
                                    instanceToLoadFurther = new CSIData( header2, selectedFiles.get( getWantedInstanceLocation( selectedFiles, singleInstanceToLoad ) ) );
                                    csidata.addResponse( instanceToLoadFurther.getComplexResponse( ) );
                                    if ( csidata.getSeriesDescription( ).toLowerCase( ).contains( new String( "EPSI" ).toLowerCase( ) )  == false )
                                    {
                                        
                                        csidata.conjugate( );
                                    }
                                    
                                }
                            }
                            
                            if ( mrsloadingOption == DataLoadOptionFrame.LOAD_DIFF_1_MINUS_2 )
                            {
                                csidata  = new CSIData( header, selectedFiles.get( getWantedInstanceLocation( selectedFiles, firstInstanceOfTwoToLoad ) ) );
                                
                                for ( int i = 1; i < selectedFiles.size( ); i++ ) 
                                {
                                    int locationOfInstance = getWantedInstanceLocation( selectedFiles, secondInstanceOfTwoToLoad );
                                    if ( locationOfInstance != -1 )
                                    {
                                        DicomObject object2 = DicomHandler.getDicomObject( selectedFiles.get( locationOfInstance ) );
                                        DicomHeader header2 = DicomHandler.getHeader(object2);
                                        DicomHandler.readSiemensData(object2, header2);
                                    
                                        instanceToLoadFurther = new CSIData( header2, selectedFiles.get( getWantedInstanceLocation( selectedFiles, secondInstanceOfTwoToLoad ) ).toString( ) );
                                        float[] toSubtract = instanceToLoadFurther.getComplexResponse( );
                                    
                                        csidata.subtractResponse( toSubtract );
                                        csidata.conjugate( );
                                    }
                                    else
                                    {
                                        JOptionPane.showMessageDialog( null, "Warning One of the two signals was not defined: \n Possibly the loaded signal is not the sum signal you intended to load!." );
                                    }
                                }
                            }
                                                        
                            if ( mrsloadingOption == DataLoadOptionFrame.LOAD_DIFF_2_MINUS_1 )
                            {
                                csidata  = new CSIData( header, selectedFiles.get( getWantedInstanceLocation( selectedFiles, secondInstanceOfTwoToLoad ) ) );
//                              System.out.println( "Selected file 2 : " + selectedFiles.get( secondInstanceOfTwoToLoad - 1 ) );
//                              for ( int i = 1; i < selectedFiles.size( ); i++ ) 
//                              {
                                DicomObject object2 = DicomHandler.getDicomObject( selectedFiles.get( getWantedInstanceLocation( selectedFiles, firstInstanceOfTwoToLoad ) ).toString( ) );
                                DicomHeader header2 = DicomHandler.getHeader( object2 );
                                DicomHandler.readSiemensData( object2, header2 );
                                    
                                instanceToLoadFurther = new CSIData( header2, selectedFiles.get( getWantedInstanceLocation( selectedFiles, firstInstanceOfTwoToLoad ) ).toString( ) );
                                                              
                                float[] toSubtract = instanceToLoadFurther.getComplexResponse( );
                                   
                                csidata.subtractResponse( toSubtract );
                                csidata.invertResponse( );
//                              }
                            }
                            
                            modelaux = csidata.getModel( );
                            
                        }
                        else
                        {              
                            csidata = new CSIData(header, selectedFile.toString()); //Build CSIData with DICOM info
                            modelaux = csidata.getModel();
                        }
                        
                        refSeriesPath = dicomManager.getSeriesPath(modelaux.getReferenceImage(1));
                        if ( refSeriesPath != null ) 
                        {
                            DicomObject objectTemp = DicomHandler.getDicomObject(refSeriesPath);
                            String classUIDTemp = DicomHandler.getSOPClassUID(objectTemp);
                            if ( classUIDTemp.equals( "1.2.840.10008.5.1.4.1.1.4.1" ) ) 
                            {
                                PhilipsMultiFrameRefImages = true;
                            }
                        }
                        if ( !PhilipsMultiFrameRefImages ) 
                        {
                            if (!this.isReferenceFile) {
                                if (dicomManager.isMounted()) {
                                    listOfFiles = new File[3];
                                    refSeriesPath = dicomManager.getSeriesPath(modelaux.getReferenceImage(1));
                                    if (refSeriesPath != null) {
                                        fileaux = new File(refSeriesPath);
                                        listOfFiles[0] = fileaux;
                                    }
                                    refSeriesPath = dicomManager.getSeriesPath(modelaux.getReferenceImage(2));
                                    if (refSeriesPath != null) {
                                        fileaux = new File(refSeriesPath);
                                        listOfFiles[1] = fileaux;
                                    }
                                    refSeriesPath = dicomManager.getSeriesPath(modelaux.getReferenceImage(3));
                                    if (refSeriesPath != null) {
                                        fileaux = new File(refSeriesPath);
                                        listOfFiles[2] = fileaux;
                                    }

                                    if (refSeriesPath != null) {
                                        csidata.m_refSlice = DicomHandler.getInstanceNr(refSeriesPath);
                                    }
                                    if (fileaux.exists()) {
                                        fileaux = fileaux.getParentFile();
                                        LinkedList<File> files = new LinkedList<>(Arrays.asList(fileaux.listFiles(new FilenameFilter() {
                                            @Override
                                            public boolean accept(File file, String string) {
                                                return DicomHandler.isDicom(file.toString() + File.separator + string);
                                            }
                                        })));
                                        for (File file : files) {
                                            imageRaster = DicomHandler.readImageRaster(file.toString());
                                            header = DicomHandler.getHeader(DicomHandler.getDicomObject(file.toString()));
                                            ImMap.put(header.getInstanceNumber(), new ImageSlice(imageRaster, header, file.toString()));
                                        }
                                        Map<Integer, ImageSlice> sortedImMap = new TreeMap<>(ImMap);
                                        imStack.addAll(sortedImMap.values());                                  
                                        modelaux.setImageStack(imStack);
                                    }
                                }
                            }
                        } // experimental Philips
                        else 
                        {
                            if (!this.isReferenceFile) {
                                if (dicomManager.isMounted()) {
                                    ///
                                    File currentfile = new File(refSeriesPath);
                                    mfp = new MultiFrameParameters(currentfile);
                                    int numberOfFrames = mfp.getNumberOfFrames();
                                    int nrOfOrientations = mfp.numberOfOrientations();
                                    mfp.getAllImages();

                                    if (nrOfOrientations == 1) {
                                        for (int i = 0; i < numberOfFrames; i++) {
                                            ImMap.put(mfp.getInstanceNumber(i), new ImageSlice(mfp.getImage(i), mfp, currentfile.toString(), i));
                                        }
                                    } else {
                                        String[] choices = new String[nrOfOrientations];
                                        for (int i = 0; i < nrOfOrientations; i++) {
                                            choices[i] = String.valueOf(i);
                                        }
                                        String input = (String) JOptionPane.showInputDialog(null, "Orientation....",
                                                "Select orientation", JOptionPane.QUESTION_MESSAGE, null,
                                                choices, choices[0]);
                                        ArrayList<Integer> array = mfp.getOrientationIndex(Integer.valueOf(input));
                                        for (int i = 0; i < array.size(); i++) {
                                            int index = array.get(i);
                                            ImMap.put(mfp.getInstanceNumber(index), new ImageSlice(mfp.getImage(index), mfp, currentfile.toString(), index));
                                        }
                                    }
                                    Map<Integer, ImageSlice> sortedImMap = new TreeMap<>(ImMap);
                                    imStack.addAll(sortedImMap.values());
                                    modelaux.setImageStack(imStack);
                                }
                            }
                        }
                    } else if (type == filetype.IMAGE && !this.isReferenceFile) {

                        File currentfile = new File(selectedFile);
                        File[] files = currentfile.getParentFile().listFiles(new FilenameFilter() {
                            @Override
                            public boolean accept(File file, String string) {
//                                return DicomHandler.isDicom(file.toString() + File.separator + string);
                                return DicomHandler.isDicomImage(file.toString() + File.separator + string);
                            }
                        });
                        if (files.length > 0) {
                            String PatientID = DicomHandler.getPatientID(DicomHandler.getDicomObject(files[0].toString()));
                            if (csimodel != null) {
                                if (!csimodel.isCSIDataNull()) {
                                    if (PatientID.matches(csimodel.getPatientID())) {
                                        modelaux = csimodel;
                                        modelaux.removeLocalizers();
                                    } else {
                                        csimodel.removeLocalizers();
                                    }
                                }
                            }
                        }

                        for (File file : files) {
                            imageRaster = DicomHandler.readImageRaster(file.toString());
                            header = DicomHandler.getHeader(DicomHandler.getDicomObject(file.toString()));
                            ImMap.put(header.getInstanceNumber(), new ImageSlice(imageRaster, header, file.toString()));
                        }
                        Map<Integer, ImageSlice> sortedImMap = new TreeMap<>(ImMap);
                        imStack.addAll(sortedImMap.values());
                        modelaux.setImageStack(imStack);
                        String[] refs;
                        if (files.length > 0) {
                            refs = DicomHandler.getReferenceImages(DicomHandler.getDicomObject(files[0].toString()));
                            if (refs != null) {
                                try {
                                    refSeriesPath = dicomManager.getSeriesPath(refs[0]);
                                    fileaux = new File(refSeriesPath);
                                    listOfFiles[0] = fileaux;
                                    refSeriesPath = dicomManager.getSeriesPath(refs[1]);
                                    fileaux = new File(refSeriesPath);
                                    listOfFiles[1] = fileaux;
                                    refSeriesPath = dicomManager.getSeriesPath(refs[2]);
                                    fileaux = new File(refSeriesPath);
                                    listOfFiles[2] = fileaux;
                                } catch (Exception e) {
                                }
                            }
                        }
                    }

                    if (type == filetype.IMAGE_MULTI) {
                        // MJ
                        System.out.println("Multi Frame");
                        File currentfile = new File(selectedFile);
                        String PatientID = DicomHandler.getPatientID(DicomHandler.getDicomObject(currentfile.toString()));
                        if (csimodel != null) {
                            if (!csimodel.isCSIDataNull()) {
                                if (PatientID.matches(csimodel.getPatientID())) {
                                    modelaux = csimodel;
                                    modelaux.removeLocalizers();
                                } else {
                                    csimodel.removeLocalizers();
                                }
                            }
                        }
                        mfp = new MultiFrameParameters(currentfile);
                        int numberOfFrames = mfp.getNumberOfFrames();
                        int orient = mfp.numberOfOrientations();
                        mfp.getAllImages();

                        if (orient == 1) {
                            for (int i = 0; i < numberOfFrames; i++) {
                                ImMap.put(mfp.getInstanceNumber(i), new ImageSlice(mfp.getImage(i), mfp, currentfile.toString(), i));
                            }
                        } else {

                            String[] choices = new String[orient];
                            for (int i = 0; i < orient; i++) {
                                choices[i] = String.valueOf(i);
                            }
                            String input = (String) JOptionPane.showInputDialog(null, "Orientation....",
                                    "Select orientation", JOptionPane.QUESTION_MESSAGE, null, choices, choices[0]);
                            ArrayList<Integer> array = mfp.getOrientationIndex(Integer.valueOf(input));
                            for (int i = 0; i < array.size(); i++) {
                                int index = array.get(i);
                                ImMap.put(mfp.getInstanceNumber(index), new ImageSlice(mfp.getImage(index), mfp, currentfile.toString(), index));
                            }

                        }

                        Map<Integer, ImageSlice> sortedImMap = new TreeMap<>(ImMap);
                        imStack.addAll(sortedImMap.values());
                        modelaux.setImageStack(imStack);

                    }

//                      Update Localizers' Image and CSI Grid
                    if (listOfFiles != null && !this.isReferenceFile) {
                        String file_aux;
                        if (!PhilipsMultiFrameRefImages) {
                        for (int i = 0; i < listOfFiles.length && i < 3; i++) {
                            if (listOfFiles[i] != null) {
                                if (listOfFiles[i].isFile()) {
                                    file_aux = listOfFiles[i].toString();
                                    if (DicomHandler.isDicom(file_aux)) {
                                        imageRaster = DicomHandler.readImageRaster(file_aux);
                                        object = DicomHandler.getDicomObject(file_aux);
                                        header = DicomHandler.getHeader(object);
                                        LocalizerPanel loc = new LocalizerPanel();
                                        loc.setImageInfo(new ImageSlice(imageRaster, header, file_aux.toString()));
                                        modelaux.addLocalizerPanel(loc); //Link Localizer with ImageViewerPanel
                                    }
                                }
                            }
                        }
                        }else{
                            /// multi-frame
                            for (int i = 0; i < mfp.numberOfOrientations(); i++) {
                                LocalizerPanel loc = new LocalizerPanel();
                                loc.setModel(modelaux);
                                ArrayList<Integer> array = null;
                                array = mfp.getOrientationIndex(i);
                                int size = array.size() / 2;
                                int index = array.get(size);
                                loc.setImageInfo(new ImageSlice(mfp.getImage(index), mfp, mfp.getFile().toString(), index));
                                loc.computeProjections();
                                switch (i) {
                                    case 0:
                                        loc.setColor(Color.green);
                                        break;
                                    case 1:
                                        loc.setColor(Color.yellow);
                                        break;
                                    case 2:
                                        loc.setColor(Color.red);
                                        break;
                                }
                                modelaux.addLocalizerPanel(loc); //Link Localizer with ImageViewerPanel
                            }
                        }
                    }

                    System.out.println("File Loaded after " + (System.currentTimeMillis() - t0) + " ms");
                } catch (Exception ex) {
                    Logger.getLogger(DicomHandler.class.getName()).log(Level.SEVERE, null, ex);
                }
                return modelaux;
            }
        } else if (this.selectedFile.endsWith(".nii")) {
            if (csimodel == null) {
                modelaux = new GlobalModel();
            } else {
                modelaux = csimodel;
            }
            modelaux.setImageStack(NiftiHandler.getImagesFromNifti(selectedFile));
            return modelaux;
        }
        return null;
    }

    @Override
    public void done() 
    {
        try 
        {
            parent.setWaitLayerActive( false );
            if ( !this.isCancelled( ) ) 
            {
                if ( this.isReferenceFile ) 
                {
                    parent.setCSIReference((GlobalModel) get( ) );
                } 
                else 
                {
                    parent.setGlobalModel((GlobalModel) get( ) );
                }
            }
            BatchProcessing.resume( );//If we are doing BatchProcessing
            //this is the moment that the file loading finishes and batch processing
            // can continue

        } 
        catch ( InterruptedException ignore ) 
        {
            // nothing to do
        } 
        catch ( java.util.concurrent.ExecutionException e ) 
        {
            String why = null;
            Throwable cause = e.getCause( );
            if ( cause != null ) 
            {
                why = cause.getMessage( );
            } 
            else 
            {
                why = e.getMessage( );
            }
            System.err.println( "Error reading file: " + why );
        }
    }
}
