/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.decisionforest.multithreading;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.UnsupportedEncodingException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.TimeZone;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.spectrim.decisionforest.data.DataSet;
import org.spectrim.decisionforest.results.ResultSet;
import org.spectrim.decisionforest.structure.Forest;

/**
 *
 * @author nunopbarros
 */
public class ForestTrainingControler {

    private int[] nrTreesVals;
    private int[] maxSplitsVals;
    private int[] maxDepthVals;
    private int[] nrVarNodeVals;
    private int[] nrChildrenVals;
    private int[] nrIterVals;
    private int[] nrSamplesVals;
    private int[] minSizeVals;
    private String[] dataFiles;
    private String folder;
    private ArrayList<int[]> combinations = new ArrayList<>();
    private int currIndex = -1;
    private ArrayList<DataSet> testsamples = new ArrayList<>();
    private ArrayList<DataSet> trainsamples = new ArrayList<>();
    private long timestamp;
    private int weaklearner;
    private boolean optimizeWeights;

    public void setNrTrees(int[] vals) {
        this.nrTreesVals = vals;
    }

    public void setMaxSplits(int[] vals) {
        this.maxSplitsVals = vals;
    }

    public void setMaxDepth(int[] vals) {
        this.maxDepthVals = vals;
    }

    public void setNrVarNode(int[] vals) {
        this.nrVarNodeVals = vals;
    }

    public void setNrChildren(int[] vals) {
        this.nrChildrenVals = vals;
    }

    public void setNrIter(int[] vals) {
        this.nrIterVals = vals;
    }

    public void setNrSamplesVals(int[] vals) {
        this.nrSamplesVals = vals;
    }

    public void setMinSizeVals(int[] vals) {
        this.minSizeVals = vals;
    }

    public void setFolder(String folder) {
        this.folder = folder;
    }

    public void setFiles(String[] vals) {
        System.out.println("Reading files...");
        this.dataFiles = vals;
        for (int i = 0; i < vals.length; i++) {
            try {
                DataSet datatrain = new DataSet();
                DataSet datatest = new DataSet();
                for (int j = 0; j < this.dataFiles.length; j++) {
                    if (j != i) {
                        datatrain.importtraindata(folder + File.separator + vals[j], 1, true);
                    }
                }
                datatest.importtestdata(folder + File.separator + vals[i], 2);
                trainsamples.add(datatrain);
                testsamples.add(datatest);
            } catch (Exception ex) {
                Logger.getLogger(ForestTrainingControler.class.getName()).log(Level.SEVERE, null, ex);
            }
        }

    }

    public synchronized void testAndSaveForest(Forest forest) {
            String separator = System.getProperty("os.name").compareTo("Linux") == 0 ? "/":"\\";
            int[] indexes = this.combinations.get(currIndex);
            int ntree = nrTreesVals[indexes[0]];
            int maxsplits = maxSplitsVals[indexes[1]];
            int maxdepth = maxDepthVals[indexes[2]];
            int nrvarnode = nrVarNodeVals[indexes[3]];
            int nchildren = nrChildrenVals[indexes[4]];
            int nriter = nrIterVals[indexes[5]];
            int nrsamples = nrSamplesVals[indexes[6]];
            int minsize = minSizeVals[indexes[7]];
            String file = dataFiles[indexes[8]];

            DataSet datatest = testsamples.get(indexes[8]).clone();
            forest.test(datatest, 0,folder + separator + "Results_" + ntree + "_" + maxsplits + "_" + maxdepth + "_" + nrvarnode + "_" + nchildren + "_" + nriter + "_" + nrsamples + "_" + minsize + "_File_" + file,null,false);
//            results.writeResults(folder + "\\" + "Results_" + ntree + "_" + maxsplits + "_" + maxdepth + "_" + nrvarnode + "_" + nchildren + "_" + nriter + "_" + nrsamples + "_" + minsize + "_File_" + file);
            
//            forest.save(folder + "\\" +"Forest_" + ntree + "_" + maxsplits + "_" + maxdepth + "_" + nrvarnode + "_" + nchildren + "_" + nriter + "_" + nrsamples + "_" + minsize + "_File_" + file.replace(".txt", "") + ".data");
            long spenttime = System.nanoTime() - timestamp;
            SimpleDateFormat sdf = new SimpleDateFormat("HH:mm:ss.SSS");
            sdf.setTimeZone(TimeZone.getTimeZone("UTC"));
            String strDate = sdf.format(spenttime / 1000000);
            System.out.println("time spent: " + strDate);
    }

    public void createCombinations() {
        for (int i = 0; i < nrTreesVals.length; i++) {
            for (int q = 0; q < maxSplitsVals.length; q++) {
                for (int j = 0; j < maxDepthVals.length; j++) {
                    for (int k = 0; k < nrVarNodeVals.length; k++) {
                        for (int l = 0; l < nrChildrenVals.length; l++) {
                            for (int m = 0; m < nrIterVals.length; m++) {
                                for (int n = 0; n < nrSamplesVals.length; n++) {
                                    for (int o = 0; o < minSizeVals.length; o++) {
                                        for (int p = 0; p < dataFiles.length; p++) {
                                            combinations.add(new int[]{i, q, j, k, l, m, n, o, p});
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    public void setCurrentIndex(int i) {
        currIndex = i;
    }
    
    public void setWeakLearner(int i){
        weaklearner = i;
    }
    
    public void setOptimizeWeights(boolean optimizeWeights){
        this.optimizeWeights = optimizeWeights;
    }

    public int getMaxIndex() {
        return combinations.size();
    }

    public synchronized void trainNextForest() {
        currIndex++;
        if (currIndex < combinations.size()) {
            int[] indexes = this.combinations.get(currIndex);

            int ntree = nrTreesVals[indexes[0]];
            int maxsplits = maxSplitsVals[indexes[1]];
            int maxdepth = maxDepthVals[indexes[2]];
            int nrvarnode = nrVarNodeVals[indexes[3]];
            int nchildren = nrChildrenVals[indexes[4]];
            int nriter = nrIterVals[indexes[5]];
            int nrsamples = nrSamplesVals[indexes[6]];
            int minsize = minSizeVals[indexes[7]];
            String file = dataFiles[indexes[8]];

            System.out.println("Forest: Ntree:" + ntree + " max splits:" + maxsplits + " max depth:" + maxdepth + " varnode:" + nrvarnode + " nchildren:" + nchildren + " nriter:" + nriter + " nrsamples:" + nrsamples + " minsize:" + minsize + " File:" + file);

            DataSet datatrain = trainsamples.get(indexes[8]).clone();
            timestamp = System.nanoTime();
            final Forest forest = new Forest(ntree, maxdepth, nchildren, nrvarnode, datatrain.nrvar, minsize, maxsplits);
            forest.setControler(this);
            forest.train(datatrain, nriter, nrsamples, datatrain.size(),true,weaklearner,optimizeWeights);
        }
    }
}
