/*
 * File: $Id: DicomDataLoaderPanel.java,v 1.2 2006/05/09 11:25:55 hans Exp $
 * History
 * Date: 02.06.2006  Authors: Hans Slotboom  
 *
 * Changes:
 *
 * Copyright (C) 2006 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */
package org.spectrim.dicom;

import java.awt.Dimension;
import java.awt.Cursor;
import java.awt.Toolkit;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.io.File;

import javax.swing.JFrame;
import javax.swing.JPopupMenu;
import javax.swing.JMenuItem;
import javax.swing.JFileChooser;

import java.util.LinkedList;
import java.util.Iterator;
import java.util.Vector;
import org.spectrim.model.GlobalModel;
 
/**
 *
 * Handles the loading of Dicom data.
 *
 * @author hans
 */
public class DicomDataLoaderPanel extends DicomFileSelectionControl implements ActionListener {

    /** 
     * Creates a new instance of DicomDataLoader. 
     * @param parent
     * @param csimodel
     */
    public DicomDataLoaderPanel( JFrame parent, GlobalModel csimodel ) {
        m_parent = parent;
        m_csimodel = csimodel;
        initialize();
    }
    
        /** 
     * Creates a new instance of DicomDataLoader. 
     * @param parent
     * @param csimodel
     */
    public DicomDataLoaderPanel( JFrame parent ) 
    {
        m_parent = parent;
        m_csimodel = null;
        initialize();
    }
    
    public void setGlobalModel(GlobalModel csimodel) {
        m_csimodel = csimodel;
    }
        
    public void initialize() {
        m_popupMenu = new JPopupMenu();
        
        m_mountDirectoryMenuItem = new JMenuItem("Mount directory");
        m_mountDirectoryMenuItem.addActionListener(this);
        m_mountDirectoryMenuItem.setActionCommand("Mounts a user selected directory.");
        
        m_popupMenu.add(m_mountDirectoryMenuItem);
        
        m_unMountDirectoryMenuItem = new JMenuItem("Unmount directory");
        m_unMountDirectoryMenuItem.addActionListener(this);
        m_unMountDirectoryMenuItem.setActionCommand("Unmounts a user selected directory.");
        m_unMountDirectoryMenuItem.setEnabled(false);
        
        m_popupMenu.add(m_unMountDirectoryMenuItem);
        
//        m_copyAndRenameMountedDicomDir = new JMenuItem("Copy and rename mounted directory ..");
//        m_copyAndRenameMountedDicomDir.addActionListener(this);
//        m_copyAndRenameMountedDicomDir.setActionCommand("Copies and renames the user selected mounted directory.");
//        m_copyAndRenameMountedDicomDir.setEnabled(false);
//
//        m_popupMenu.add(m_copyAndRenameMountedDicomDir);
//
//        m_copyAndRegridsSelectedDicomImageSeries = new JMenuItem("Copy and regrid selected image series ..");
//        m_copyAndRegridsSelectedDicomImageSeries.addActionListener(this);
//        m_copyAndRegridsSelectedDicomImageSeries.setActionCommand("Copies a DICOM image series with user selected regridding of images");
//        m_copyAndRegridsSelectedDicomImageSeries.setEnabled(false);
//
//        m_popupMenu.add(m_copyAndRegridsSelectedDicomImageSeries);
    }
    
    /**
     * Mounts a directory. 
     */
    public void mountDirectory( ) 
    {
        setDicomFiles( );
        
        if ( m_dicomDir != null ) 
        {
            m_mountDirectoryMenuItem.setEnabled( false );
            m_unMountDirectoryMenuItem.setEnabled( true );

            m_directoryIsMounted = true;
        }
    }
    
    /**
     * Called when an action event occurs. 
     * @param e 
     */
    public void mountDirectory( String path ) 
    {
        // to do
        setDicomFiles(path);
        if ( m_dicomDir != null ) 
        {
            m_mountDirectoryMenuItem.setEnabled(false);
            m_unMountDirectoryMenuItem.setEnabled(true);
        
            m_directoryIsMounted = true;

        }
    }
 
    /**
     * Returns the reference to the DicomDirectory object. 
     * @return 
     */
    public DicomDirectory getDicomDirObject( )
        {
        return m_dicomDir;
    }
            
    /**
     * Unmount the currently mounted directory.
     */
    public void unmountDirectory( ) 
    {
        clearDicomDir( );
            
        m_directoryIsMounted = false;
        notifyActionListeners( UNMOUNT_DIRECTORY_EVENT );
            
        m_mountDirectoryMenuItem.setEnabled(true);
        m_unMountDirectoryMenuItem.setEnabled(false);
    }
        
    public boolean isMounted() {
        return m_directoryIsMounted;
        }
        
    /**
     * Called whenever registered components fire action events.
     * @param e 
     */
    @Override
    public void actionPerformed(ActionEvent e) 
    {
        Object sender = e.getSource();

        if ( sender == m_mountDirectoryMenuItem ) 
        {
            mountDirectory( );
        }
        
        if ( sender == m_unMountDirectoryMenuItem ) 
        {

            clearDicomDir( );

            m_directoryIsMounted = false;
            notifyActionListeners(UNMOUNT_DIRECTORY_EVENT);

            m_mountDirectoryMenuItem.setEnabled(true);
            m_unMountDirectoryMenuItem.setEnabled(false);
        }
        
        if (sender == m_dicomDir) 
        {
            setDicomDir(m_dicomDir);
            notifyActionListeners(MOUNT_DIRECTORY_EVENT);
        }
    }
    
    /**
     * Notifies actionEvent to all registered listeners.
     */
    private void notifyActionListeners(int eventType) 
    {
        Iterator it = m_actionListeners.iterator( );
        while ( it.hasNext( ) ) 
        {
            ( (ActionListener) it.next( ) ).actionPerformed( new ActionEvent( this, eventType, "" ) );
        }
    }
    
    /**
     * Evokes the directory selection dialog.
     */
    private void setDicomFiles() 
    {
        this.setCursor( new Cursor(Cursor.WAIT_CURSOR ) );
 
        m_selectedDicomDirectory = getDicomDirectory( );
        
        if ( m_selectedDicomDirectory.getPath( ).compareTo( "" ) != 0 ) 
        {
            if ( m_directoryIsMounted == true ) 
            {
                clearDicomDir( );
            }
            if ( m_dicomDir != null ) 
            {
                m_dicomDir.stopMounting( );
            }
            
            m_dicomDir = new DicomDirectory( m_parent, m_selectedDicomDirectory );
            
            if ( m_dicomDir != null ) 
            {
                m_dicomDir.addActionListener( this );
            }
        }
        this.setCursor( new Cursor( Cursor.DEFAULT_CURSOR ) );
    }
    
    /**
     * Returns the Dicom directory selected by the user.
     */
    private File getDicomDirectory( ) 
    {
        DicomDefaults fdef = DicomDefaults.getInstance( );
        fdef.load( );
        
        String path = fdef.getProperty( "Dicom mount path" );
        
        JFileChooser fileChooser = new JFileChooser( fdef.getProperty( "Dicom mount path" ) + System.getProperty( "file.separator" ) );
        Dimension dimension = Toolkit.getDefaultToolkit().getScreenSize( );
        int x = (int) ( (dimension.getWidth( ) - fileChooser.getWidth( ) ) / 2 );
        int y = (int) ( (dimension.getHeight( ) - fileChooser.getHeight( ) ) / 2 );
        fileChooser.setLocation( x, y );
              
        fileChooser.setName( "Mount Dicom Directory" );
        
        fileChooser.setFileSelectionMode( JFileChooser.DIRECTORIES_ONLY );
        
        int result = fileChooser.showOpenDialog( null );
        
        File selected_dir = new File("");
        
        switch ( result ) 
        {
            case JFileChooser.APPROVE_OPTION:   // Approve Open button was pressed
                                                if ( m_csimodel != null ) 
                                                {
                                                    m_csimodel.clearGrid( );
                                                    m_csimodel.clearColorMap( );
                                                    m_csimodel.clearVoxelSelection( );
                                                    if ( m_csimodel.getImageStack( ) != null ) 
                                                    {
                                                        m_csimodel.clearImageStack( );
                                                    }
                                                    m_csimodel.setBlankImageStack( );
                                                }
                                                selected_dir = fileChooser.getSelectedFile( );
                                                DicomDefaults def = DicomDefaults.getInstance( );
                                                def.setProperty( "Dicom mount path", selected_dir.getPath( ) );
                                                def.save( );
                                                if ( selected_dir != null ) 
                                                {
                                                    m_dicomDirectorySet = true;
                                                }
                                                
                                                break;
            case JFileChooser.CANCEL_OPTION:     // Cancel or close dialog was selected
                                                break;
            case JFileChooser.ERROR_OPTION:      // the selected process did not terminate properly
                                                break;
        }
        
        return selected_dir;
    }

    /**
     * Adds an action listener to the list of listeners.
     * @param e
     */
    @Override
    public void addActionListener(ActionListener e) {
        super.addActionListener(e);
        m_actionListeners.add(e);
    }
    
    /**
     * Returns the imaging modality string of the files in the currently 
     * selected directory.
     * @return 
     */
    public String getImagingModality() {
        String retVal = "none";
        
        if (m_dicomDir != null) {
            retVal = m_dicomDir.getImageModality();
        }
        if (retVal == null) {
            retVal = "undefined";
        }
        return retVal;
            
    }
    
    /**
     * Returns the number of images selected.
     * @return 
     */
    public int getNumberOfSelectedImages() {
        return m_fileSelectedSeriesFileReferences.size();
    }
    
    /**
     * Returns the number of images selected.
     * @param seriesDescription
     * @param sliceNumber
     * @return 
     */
    public Vector getDataFileReferences(String seriesDescription, int sliceNumber) {
        return m_dicomDir.getFileReferences(seriesDescription, sliceNumber);
    }
    
    /**
     * Returns the Dicom file header belonging to fileName.
     * @param fileName
     * @return 
     */
    public byte[] getDicomHeaderAsByteArray(String fileName) {
        DicomImageData imDat = new DicomImageData(m_dicomDir.getPath() + File.separatorChar + fileName);
        return imDat.getDicomHeaderAsByteArray();
    }
    
    /**
     * Returns the image dimensions.
     * @param instanceNumber
     * @param seriesDescription
     * @return 
     */
    public Dimension getImageDimensions(int instanceNumber, String seriesDescription) {
        return m_dicomDir.getImageDimension(seriesDescription, instanceNumber);
    }
    
    /**
     * Returns the image dimensions.
     * @param seriesDescription
     * @return 
     */
    public Dimension getImageDimensions(String seriesDescription) {
        return m_dicomDir.getImageDimension(seriesDescription);
    }
    
    /**
     * Returns the series number of the last selected series.
     * @return 
     */
    public int getCurrentlySelectedSeriesNumber() {
        return m_currentlySelectedSeriesNumber;
    }

    /**
     * Returns the patient name.
     * @return 
     */
    public String getPatientName() {
        return m_patientName;
    }
    
    /**
     * Returns a reference on the current DicomDirectory object.
     * @return 
     */
    public DicomDirectory getDicomDir() {
        return m_dicomDir;
    }
    
    /**
     * Sets the boolean which determines how the data are stored: in one series
     * or in separate series.
     * @param set
     */
    public void setImagesIndifferentSeries(boolean set) {
       m_areImagesFilesStoredInDifferentSeries = set;
    }
    /**
     * Returns the series path, ie the path where the series is located in the
     * file system.
     * @param UID
     * @return 
     */
    public String getSeriesPath(String UID) {
        String res;
        res = m_dicomDir.getFileNameBelongingToMediaStorageSOPInstanceUID(UID);
        return res;
    }
    
//    public void mountDirectory(String inDirectory) {
//        setDicomFiles(inDirectory);
//    }
    private void setDicomFiles(String inDirecotory) {
        m_selectedDicomDirectory = new File(inDirecotory);
    
        if (m_selectedDicomDirectory.getPath().compareTo("") != 0) {
            if (m_directoryIsMounted == true) {
                clearDicomDir();
            }
            if (m_dicomDir != null) {
                m_dicomDir.stopMounting();
            }
            m_dicomDir = new DicomDirectory(m_parent, m_selectedDicomDirectory);

            if (m_dicomDir != null) {
                m_dicomDir.addActionListener(this);
            }
        }
        this.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
    }
    // Attributes
    private final JFrame m_parent;
    private JMenuItem m_mountDirectoryMenuItem;
    private JMenuItem m_unMountDirectoryMenuItem;
    private JMenuItem m_copyAndRenameMountedDicomDir;
    private JMenuItem m_copyAndRegridsSelectedDicomImageSeries;
    // Copies and renames dicom files from a mounted directory to another
//    private JMenuItem m_copyAndRenameMountedDicomDir;
    // Creates symbolic links to the dicom files of the mounted directory
//    private JMenuItem m_copyAndRegridsSelectedDicomImageSeries;
    // Listeners to this object
    private final LinkedList<ActionListener> m_actionListeners = new LinkedList<>();
    private File m_selectedDicomDirectory;
    private DicomDirectory m_dicomDir;
    private boolean m_dicomDirectorySet = false;
    private final String m_defaultPath = null;
    // Events generated by this class
    public final static int MOUNT_DIRECTORY_EVENT = 2;
    public final static int COPY_AND_RENAME_MOUNTED_DIRECTORY_EVENT = 3;
    public final static int CREATE_REGRIDDED_DICOM_FILES_OF_SELECTED_IMAGE_SERIES_EVENT = 4;
    public final static int UNMOUNT_DIRECTORY_EVENT = 9;
    // series description of currently selected series
    private final String m_currentlySelectedSeriesDescription = null;
    private int m_currentlySelectedSeriesNumber;
    private String m_patientName;
    private boolean m_areImagesFilesStoredInDifferentSeries = true;
    // Must be declarated somehow
    private static final long serialVersionUID = 3487412815819393L;
    private GlobalModel m_csimodel;
}