/*
 * File: $Id: DicomFileOverNetworkTransferrer.java,v 1.6 2009/15/05 13:28:39 hans Exp $
 * History
 * Date: 15.05.2009  Authors: Hans Slotboom
 *
 * Changes:
 *
 * Copyright (C) 2009 by
 *
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */


/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.spectrim.dicom;

import java.io.IOException;
import java.io.InputStream;

/**
 *
 * Allows a Dicom file to be transferred over the network to an
 * active Dicom-node
 *
 * @author slotboom
 */
public class DicomFileOverNetworkTransferrer
{

    /**
     * Default constructor of DicomFileOverNetworkTransferrer.
     * @param selectedRemoteDicomNodeNumber
     * @param localDicomFileReference
     */
    public DicomFileOverNetworkTransferrer( int selectedRemoteDicomNodeNumber,
                                            String localDicomFileReference )
    {
        m_selectedDicomNodeNumber = selectedRemoteDicomNodeNumber;
        m_localDicomFileReference = localDicomFileReference;
    }

    /**
     * Replaces all blanks by "\  " in order to be interpreted 
     * correctly by the Windows operating system
     * @param cmd 
     * @return 
     */
    private String generateProperWindowsCommand( String cmd )
    {
        String out = "";
        for ( int i = 0; i < cmd.length( ); i++ )
        {
            if ( cmd.substring( i, i + 1 ).compareTo( "\\" ) == 0 )
            {
                out += "\\" + "\\";
            }
            else
            {
                out += cmd.substring( i, i + 1 );
            }
        }
        return out;
    }
    
    /**
     *
     * Sents a Dicom formatted file over the network to the currently selected
     * remote Dicom network node.
     *
     * @return 
     */
    public int performDicomSendOfLocalDicomFileToSelectedRemoteNode( )
    {

        DicomNetworkNode remotedcmNode = DicomNetworkNodes.getInstance( ).getRemoteDicomNetworkNode( DicomNetworkNodes.getInstance( ).getSelectedRemoteDicomNodeNumber( ) );
        DicomNetworkNode localDcmNode =  DicomNetworkNodes.getInstance( ).getLocalHostDicomNetworkNode( );

        int returnValue = 0;
        String cmd = "";
        
        // All operations take place is the working directory 
        String workingDir = System.getProperty( "user.dir" );
        
        if ( System.getProperty( "os.name" ).toLowerCase( ).compareTo( "linux" ) == 0 )
        {
            cmd = workingDir + "/lib/dcmsnd -L " + localDcmNode.getTransmitAEtitle( ) + ":" + localDcmNode.getDicomTransmitPortNumber( ) +  " "
                  + remotedcmNode.getReceiveAEtitle( ) + "@" + remotedcmNode.getIPaddressr( ) + ":" + remotedcmNode.getDicomReceivePortNumber( ) + " " + m_localDicomFileReference;
        }
        
        if ( System.getProperty( "os.name" ).toLowerCase( ).contains( "windows" ) == true )
        {
            cmd = workingDir + "\\lib\\dcmsnd.bat -L " + localDcmNode.getTransmitAEtitle( ) + ":" + localDcmNode.getDicomTransmitPortNumber( ) + " "
                  + remotedcmNode.getReceiveAEtitle( ) + "@" + remotedcmNode.getIPaddressr( ) + ":" + remotedcmNode.getDicomReceivePortNumber( ) + " " + m_localDicomFileReference;
            cmd = generateProperWindowsCommand( cmd );
        }
        
        //cmd = "./lib/dcmsnd -L DINSPECTRO:104 " + aetitle + "@" + IP_sent_to_number + ":" + dicomListenPortNumber + " " + dicomFileName;

        System.out.println( "Trying following command : " + cmd );
        try
        {       
            Process p = Runtime.getRuntime( ).exec( cmd ); 
     
            InputStream in = p.getInputStream( );
            int c;
            while ( ( c = in.read( ) ) != -1 )
            {
                System.out.print( (char)c ) ;
            }

            returnValue = p.waitFor( );
            in.close( );
        }
        catch ( IOException ioX )
        {
            // not handled yet
            System.out.println( "IOException occurred" );
            return -1;
        }
        catch ( InterruptedException intX )
        {
            // not handled yet
            System.out.println( "InterruptedException occurred" );
            return -1;
        }

        return returnValue;
    }


    // Class attributes
    private int m_selectedDicomNodeNumber = -1;
    private String m_localDicomFileReference = null;
}
