/*
 * File: $Id: DicomFileSiemensSpecificData.java,v 1.1.1.1 2006/04/19 09:38:47 hans Exp $
 * History
 * Date: 10.02.2004, 17:11  Authors: Hans Slotboom  
 *
 * Changes:
 *
 * Copyright (C) 2003 - 2004 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

 
package org.spectrim.dicom;

import org.spectrim.io.dicom.DicomHandler;
import org.spectrim.io.dicom.DicomHeaderLine;
import java.io.BufferedReader;
import java.util.LinkedHashMap;

import java.io.IOException;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.RandomAccessFile;
import java.io.UnsupportedEncodingException;
import java.util.Arrays;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.dcm4che2.data.DicomObject;


/**
 * Contains all Siemens specific data from the Dicom header
 * Is necessary to get insight in measurement parameters, 
 * with which the image in the file was acquired
 * It also stores for spectroscopy the reference images on which
 * the ROI was planned
 *
 * The class parses the dicom file and extracts the Siemens specific 
 * ascii block from it and stores every line in a item of a TreeMap
 * 
 * @author  hans
 */
public class DicomFileSiemensSpecificData 
{
    
    /** 
     * Creates a new instance of DicomFileSiemensSpecificData 
     */
    public DicomFileSiemensSpecificData( String fileRef ) 
    {   
        DicomObject object;
        object = DicomHandler.getDicomObject(fileRef);
        byte[] res;
        if (object.contains(Integer.parseInt("00291120", 16))) {
            res = object.getBytes(Integer.parseInt("00291120", 16));
            try {
                byte[] beginmark = "### ASCCONV BEGIN".getBytes();
                int posaux = 0;
                int startpos = 0;
                int i = 0;
                while (i < res.length) {
                    if (Byte.compare(res[i], beginmark[posaux]) == 0) {
                        posaux++;
                    } else {
                        posaux = 0;
                    }
                    if (posaux == beginmark.length) {
                        startpos = i;
                        i = res.length;
                    }
                    i++;
                }
                res = Arrays.copyOfRange(res, startpos, res.length);
                String stringaux = new String(res, "ASCII");
                String[] lines = stringaux.split("\n");
                String[] parts;
                for (int j = 1; j < lines.length; j++) {
                    parts = lines[j].split("((\\s+)|(\\t))\\=((\\s+)|(\\t))");
                    if (parts.length == 2) {
                        if (!parts[0].isEmpty() && !parts[1].isEmpty()) {
                            if (parts[1].startsWith("\"\"")) {
                                parts[1] = parts[1].replace("\"\"", "");
                            }
                            m_siemensDicomAttributes.put( parts[0], parts[1] );

                        }
                    }
                }



//                byte[] startmark = "### ASCCONV BEGIN".getBytes();
//                Arrays.binarySearch(res, startmark[1]);

//        try (PrintWriter writer = new PrintWriter("E:\\cases\\Baechtold_Michelle_20130228_183203.265000\\csi_se_135_fast\\test.txt", "UTF-8")) {
//            writer.println(test);
//        }       catch (FileNotFoundException ex) {
//                    Logger.getLogger(DicomHandler.class.getName()).log(Level.SEVERE, null, ex);
//                }
//                System.out.println(test);
            } catch (UnsupportedEncodingException ex) {
                Logger.getLogger(DicomHandler.class.getName()).log(Level.SEVERE, null, ex);
            }
        };
        
        
//        boolean asciiPart = false;
//        try
//        {
//            RandomAccessFile in = new RandomAccessFile( fileRef, "r" );
//            String readFromFile = in.readLine( );
//            while ( readFromFile != null )
//            {
//                if ( asciiPart == true )
//                {
//                    int equalSignPos = readFromFile.indexOf( "=" );
//                    
//                    String attribName;
//                    String attribValue;
//                    if ( equalSignPos > 1 )
//                    {
//                        attribName = readFromFile.substring( 0, equalSignPos - 1 );
//                        attribName = attribName.trim( );
//                        attribValue = readFromFile.substring( equalSignPos + 1, readFromFile.length( ) );
//                        attribValue = attribValue.trim( );
//                        m_siemensDicomAttributes.put( attribName, attribValue );
//                    }
//                }
//                
//                if ( readFromFile.lastIndexOf( "### ASCCONV BEGIN" ) != -1 )
//                {
//                    asciiPart = true;
//                }
//                if ( readFromFile.lastIndexOf( "### ASCCONV END ###" ) != -1 )
//                {
//                    asciiPart = false;
//                }
//                
//                readFromFile = in.readLine( );
//            }
//            in.close( );
//        }
//        catch ( FileNotFoundException e )
//        {
//            System.out.println( "DicomFileSiemensSpecificData -> file : " + fileRef + " not found!" );
//        }
//        catch ( IOException ioex )
//        {
//            System.out.println( "DicomFileSiemensSpecificData -> file : " + fileRef + " not found!" );
//        }
    }
    
    /** 
     * Creates a new instance of DicomFileSiemensSpecificData 
     */
    public DicomFileSiemensSpecificData( String sopUID, InputStream in ) 
    {
        boolean asciiPart = false;
        try
        {
            // RandomAccessFile in = new RandomAccessFile( fileRef, "r" );
            
            BufferedReader dataInputStream = new BufferedReader( new InputStreamReader( in ) );

            String readFromFile = dataInputStream.readLine( );
           
            while ( readFromFile != null )
            {
                if ( asciiPart == true )
                {
                    int equalSignPos = readFromFile.indexOf( "=" );
                    
                    String attribName;
                    String attribValue;
                    if ( equalSignPos != -1 )
                    {
                        attribName = readFromFile.substring( 0, equalSignPos - 1 );
                        attribName = attribName.trim( );
                        attribValue = readFromFile.substring( equalSignPos + 1, readFromFile.length( ) );
                        attribValue = attribValue.trim( );
                        m_siemensDicomAttributes.put( attribName, attribValue );
                    }
                }
                
                if ( readFromFile.lastIndexOf( "### ASCCONV BEGIN ###" ) != -1 )
                {
                    asciiPart = true;
                }
                if ( readFromFile.lastIndexOf( "### ASCCONV END ###" ) != -1 )
                {
                    asciiPart = false;
                }
                
                readFromFile = dataInputStream.readLine( );
            }
            in.close( );
        }
        catch ( IOException ioex )
        {
            System.out.println( "DicomFileSiemensSpecificData -> dicom file with SOPinstanceUID : " + sopUID + " not found!" );
        }
    }
    
    
    
    /**
     * Returns the hashtable containing all attributes from the 
     * Dicom file
     */
    public LinkedHashMap getSiemensAttributes( )
    {
        return m_siemensDicomAttributes;
    }
    
    // Attributes
    private LinkedHashMap< String, String > m_siemensDicomAttributes = new LinkedHashMap< String, String >( );
    // Attributes present in the Siemens specific Dicom header 
    public final static String ulVersion = new String( "ulVersion" );
    public final static String tSequenceFileName = new String( "tSequenceFileName" );
    public final static String tProtocolName = new String( "tProtocolName" );
    public final static String tReferenceImage0 = new String( "tReferenceImage" );
    public final static String tReferenceImage1 = new String( "tReferenceImage1" );
    public final static String tReferenceImage2 = new String( "tReferenceImage2" );
    public final static String ucScanRegionPosValid = new String("ucScanRegionPosValid");
    public final static String sProtConsistencyInfo_tBaselineString = new String( "sProtConsistencyInfo.tBaselineString" );
    public final static String sProtConsistencyInfo_flNominalB0 = new String("sProtConsistencyInfo.flNominalB0");
    public final static String sProtConsistencyInfo_flGMax = new String( "sProtConsistencyInfo.flGMax" );
    public final static String sProtConsistencyInfo_flRiseTime = new String( "sProtConsistencyInfo.flRiseTime" );
    public final static String sGRADSPEC_sEddyCompensationX_aflAmplitude_0_ = new String( "sGRADSPEC.sEddyCompensationX.aflAmplitude[0]" );
    public final static String sGRADSPEC_sEddyCompensationX_aflAmplitude_1_ = new String( "sGRADSPEC.sEddyCompensationX.aflAmplitude[1]" );
    public final static String sGRADSPEC_sEddyCompensationX_aflAmplitude_2_ = new String( "sGRADSPEC.sEddyCompensationX.aflAmplitude[2]" );
    public final static String sGRADSPEC_sEddyCompensationX_aflAmplitude_3_ = new String( "sGRADSPEC.sEddyCompensationX.aflAmplitude[3]" );
    public final static String sGRADSPEC_sEddyCompensationX_aflAmplitude_4_ = new String( "sGRADSPEC.sEddyCompensationX.aflAmplitude[4]" );
    public final static String sGRADSPEC_sEddyCompensationX_aflTimeConstant_0_ = new String( "sGRADSPEC.sEddyCompensationX.aflTimeConstant[0]");
    public final static String sGRADSPEC_sEddyCompensationX_aflTimeConstant_1_ = new String( "sGRADSPEC.sEddyCompensationX.aflTimeConstant[1]");
    public final static String sGRADSPEC_sEddyCompensationX_aflTimeConstant_2_ = new String( "sGRADSPEC.sEddyCompensationX.aflTimeConstant[2]");
    public final static String sGRADSPEC_sEddyCompensationX_aflTimeConstant_3_ = new String( "sGRADSPEC.sEddyCompensationX.aflTimeConstant[3]");
    public final static String sGRADSPEC_sEddyCompensationX_aflTimeConstant_4_ = new String( "sGRADSPEC.sEddyCompensationX.aflTimeConstant[4]");
    public final static String sGRADSPEC_sEddyCompensationY_aflAmplitude_0_ = new String( "sGRADSPEC.sEddyCompensationY.aflAmplitude[0]" );
    public final static String sGRADSPEC_sEddyCompensationY_aflAmplitude_1_ = new String( "sGRADSPEC.sEddyCompensationY.aflAmplitude[1]" );
    public final static String sGRADSPEC_sEddyCompensationY_aflAmplitude_2_ = new String( "sGRADSPEC.sEddyCompensationY.aflAmplitude[2]" );
    public final static String sGRADSPEC_sEddyCompensationY_aflAmplitude_3_ = new String( "sGRADSPEC.sEddyCompensationY.aflAmplitude[3]" );
    public final static String sGRADSPEC_sEddyCompensationY_aflAmplitude_4_ = new String( "sGRADSPEC.sEddyCompensationY.aflAmplitude[4]" );
    public final static String sGRADSPEC_sEddyCompensationY_aflTimeConstant_0_ = new String( "sGRADSPEC_sEddyCompensationY.aflTimeConstant[0]" );
    public final static String sGRADSPEC_sEddyCompensationY_aflTimeConstant_1_ = new String( "sGRADSPEC_sEddyCompensationY.aflTimeConstant[1]" );
    public final static String sGRADSPEC_sEddyCompensationY_aflTimeConstant_2_ = new String( "sGRADSPEC_sEddyCompensationY.aflTimeConstant[2]" );
    public final static String sGRADSPEC_sEddyCompensationY_aflTimeConstant_3_ = new String( "sGRADSPEC_sEddyCompensationY.aflTimeConstant[3]" );
    public final static String sGRADSPEC_sEddyCompensationY_aflTimeConstant_4_ = new String( "sGRADSPEC_sEddyCompensationY.aflTimeConstant[4]" );
    public final static String sGRADSPEC_sEddyCompensationZ_aflAmplitude_0_ = new String( "sGRADSPEC.sEddyCompensationZ.aflAmplitude[0]" );
    public final static String sGRADSPEC_sEddyCompensationZ_aflAmplitude_1_ = new String( "sGRADSPEC.sEddyCompensationZ.aflAmplitude[1]" );
    public final static String sGRADSPEC_sEddyCompensationZ_aflAmplitude_2_ = new String( "sGRADSPEC.sEddyCompensationZ.aflAmplitude[2]" );
    public final static String sGRADSPEC_sEddyCompensationZ_aflAmplitude_3_ = new String( "sGRADSPEC.sEddyCompensationZ.aflAmplitude[3]" );
    public final static String sGRADSPEC_sEddyCompensationZ_aflAmplitude_4_ = new String( "sGRADSPEC.sEddyCompensationZ.aflAmplitude[4]" );
    public final static String sGRADSPEC_sEddyCompensationZ_aflTimeConstant_0_ = new String( "sGRADSPEC.sEddyCompensationZ.aflTimeConstant[0]" );
    public final static String sGRADSPEC_sEddyCompensationZ_aflTimeConstant_1_ = new String( "sGRADSPEC.sEddyCompensationZ.aflTimeConstant[1]" );
    public final static String sGRADSPEC_sEddyCompensationZ_aflTimeConstant_2_ = new String( "sGRADSPEC.sEddyCompensationZ.aflTimeConstant[2]" );
    public final static String sGRADSPEC_sEddyCompensationZ_aflTimeConstant_3_ = new String( "sGRADSPEC.sEddyCompensationZ.aflTimeConstant[3]" );
    public final static String sGRADSPEC_sEddyCompensationZ_aflTimeConstant_4_ = new String( "sGRADSPEC.sEddyCompensationZ.aflTimeConstant[4]" );
    public final static String sGRADSPEC_bEddyCompensationValid= new String( "sGRADSPEC.bEddyCompensationValid" );
    public final static String sGRADSPEC_sB0CompensationX_aflAmplitude_0_ = new String( "sGRADSPEC.sB0CompensationX.aflAmplitude[0]" );
    public final static String sGRADSPEC_sB0CompensationX_aflAmplitude_1_ = new String( "sGRADSPEC.sB0CompensationX.aflAmplitude[1]" );
    public final static String sGRADSPEC_sB0CompensationX_aflAmplitude_2_ = new String( "sGRADSPEC.sB0CompensationX.aflAmplitude[2]" );
    public final static String sGRADSPEC_sB0CompensationX_aflTimeConstant_0_= new String( "sGRADSPEC.sB0CompensationX.aflTimeConstant[0]" );
    public final static String sGRADSPEC_sB0CompensationX_aflTimeConstant_1_= new String( "sGRADSPEC.sB0CompensationX.aflTimeConstant[1]" );
    public final static String sGRADSPEC_sB0CompensationX_aflTimeConstant_2_= new String( "sGRADSPEC.sB0CompensationX.aflTimeConstant[2]" );
    public final static String sGRADSPEC_sB0CompensationY_aflAmplitude_0_ = new String( "sGRADSPEC.sB0CompensationY.aflAmplitude[0]" );
    public final static String sGRADSPEC_sB0CompensationY_aflAmplitude_1_ = new String( "sGRADSPEC.sB0CompensationY.aflAmplitude[1]" );
    public final static String sGRADSPEC_sB0CompensationY_aflAmplitude_2_ = new String( "sGRADSPEC.sB0CompensationY.aflAmplitude[2]" );
    public final static String sGRADSPEC_sB0CompensationY_aflTimeConstant_0_ = new String( "sGRADSPEC.sB0CompensationY.aflTimeConstant[0]" );
    public final static String sGRADSPEC_sB0CompensationY_aflTimeConstant_1_ = new String( "sGRADSPEC.sB0CompensationY.aflTimeConstant[1]" );
    public final static String sGRADSPEC_sB0CompensationY_aflTimeConstant_2_ = new String( "sGRADSPEC.sB0CompensationY.aflTimeConstant[2]" );
    public final static String sGRADSPEC_sB0CompensationZ_aflAmplitude_0_ = new String( "sGRADSPEC.sB0CompensationZ.aflAmplitude[0]" );
    public final static String sGRADSPEC_sB0CompensationZ_aflAmplitude_1_ = new String( "sGRADSPEC.sB0CompensationZ.aflAmplitude[1]" );
    public final static String sGRADSPEC_sB0CompensationZ_aflAmplitude_2_ = new String( "sGRADSPEC.sB0CompensationZ.aflAmplitude[2]" );
    public final static String sGRADSPEC_sB0CompensationZ_aflTimeConstant_0_ = new String( "sGRADSPEC.sB0CompensationZ.aflTimeConstant[0]" );
    public final static String sGRADSPEC_sB0CompensationZ_aflTimeConstant_1_ = new String( "sGRADSPEC.sB0CompensationZ.aflTimeConstant[1]" );
    public final static String sGRADSPEC_sB0CompensationZ_aflTimeConstant_2_ = new String( "sGRADSPEC.sB0CompensationZ.aflTimeConstant[2]" );
    public final static String sGRADSPEC_bB0CompensationValid = new String( "sGRADSPEC.bB0CompensationValid" );
    public final static String sGRADSPEC_sCrossTermCompensationXY_aflAmplitude_0_ = new String( "sGRADSPEC.sCrossTermCompensationXY.aflAmplitude[0]" );
    public final static String sGRADSPEC_sCrossTermCompensationXY_aflTimeConstant_0_ = new String( "sGRADSPEC.sCrossTermCompensationXY.aflTimeConstant[0]" );
    public final static String sGRADSPEC_sCrossTermCompensationXZ_aflAmplitude_0_ = new String( "sGRADSPEC.sCrossTermCompensationXZ.aflAmplitude[0]" );
    public final static String sGRADSPEC_sCrossTermCompensationXZ_aflTimeConstant_0_ = new String( "sGRADSPEC.sCrossTermCompensationXZ.aflTimeConstant[0]" );
    public final static String sGRADSPEC_sCrossTermCompensationYZ_aflAmplitude_0_ = new String( "sGRADSPEC.sCrossTermCompensationYZ.aflAmplitude[0]" );
    public final static String sGRADSPEC_sCrossTermCompensationYZ_aflTimeConstant_0_ = new String( "sGRADSPEC.sCrossTermCompensationYZ.aflTimeConstant[0]" );
    public final static String sGRADSPEC_bCrossTermCompensationValid = new String( "sGRADSPEC.bCrossTermCompensationValid" );
    public final static String sGRADSPEC_lOffsetX= new String( "sGRADSPEC.lOffsetX" );
    public final static String sGRADSPEC_lOffsetY= new String( "sGRADSPEC.lOffsetY" );
    public final static String sGRADSPEC_lOffsetZ= new String( "sGRADSPEC.lOffsetZ" );
    public final static String sGRADSPEC_bOffsetValid = new String( "sGRADSPEC.bOffsetValid" );
    public final static String sGRADSPEC_lDelayX = new String( "sGRADSPEC.lDelayX" );
    public final static String sGRADSPEC_lDelayY = new String( "sGRADSPEC.lDelayY" );
    public final static String sGRADSPEC_lDelayZ = new String( "sGRADSPEC.lDelayZ" );
    public final static String sGRADSPEC_bDelayValid= new String( "sGRADSPEC.bDelayValid");
    public final static String sGRADSPEC_flSensitivityX= new String( "sGRADSPEC.flSensitivityX" );
    public final static String sGRADSPEC_flSensitivityY= new String( "sGRADSPEC.flSensitivityY" );
    public final static String sGRADSPEC_flSensitivityZ= new String( "sGRADSPEC.flSensitivityZ" );
    public final static String sGRADSPEC_bSensitivityValid = new String( "sGRADSPEC.bSensitivityValid" );
    public final static String sGRADSPEC_alShimCurrent_0_ = new String( "sGRADSPEC.alShimCurrent[0]" );
    public final static String sGRADSPEC_alShimCurrent_1_ = new String( "sGRADSPEC.alShimCurrent[1]" );
    public final static String sGRADSPEC_alShimCurrent_2_ = new String( "sGRADSPEC.alShimCurrent[2]" );
    public final static String sGRADSPEC_alShimCurrent_3_ = new String( "sGRADSPEC.alShimCurrent[3]" );
    public final static String sGRADSPEC_alShimCurrent_4_ = new String( "sGRADSPEC.alShimCurrent[4]" );
    public final static String sGRADSPEC_bShimCurrentValid = new String( "sGRADSPEC.bShimCurrentValid" );
    public final static String sGRADSPEC_ucMode = new String( "sGRADSPEC.ucMode" );
    public final static String sTXSPEC_asNucleusInfo_0_tNucleus = new String( "sTXSPEC.asNucleusInfo[0].tNucleus" );
    public final static String sTXSPEC_asNucleusInfo_0_lFrequency = new String( "sTXSPEC.asNucleusInfo[0].lFrequency" );
    public final static String sTXSPEC_asNucleusInfo_0_bFrequencyValid = new String( "sTXSPEC.asNucleusInfo[0].bFrequencyValid" );
    public final static String sTXSPEC_asNucleusInfo_0_flReferenceAmplitude = new String( "sTXSPEC.asNucleusInfo[0].flReferenceAmplitude" );
    public final static String sTXSPEC_asNucleusInfo_0_bReferenceAmplitudeValid = new String( "sTXSPEC.asNucleusInfo[0].bReferenceAmplitudeValid" );
    public final static String sTXSPEC_asNucleusInfo_0_flAmplitudeCorrection = new String( "sTXSPEC.asNucleusInfo[0].flAmplitudeCorrection");
    public final static String sTXSPEC_asNucleusInfo_0_bAmplitudeCorrectionValid= new String( "sTXSPEC.asNucleusInfo[0].bAmplitudeCorrectionValid");
    public final static String sTXSPEC_asNucleusInfo_1_bFrequencyValid= new String( "sTXSPEC.asNucleusInfo[1].bFrequencyValid" );
    public final static String sTXSPEC_asNucleusInfo_1_bReferenceAmplitudeValid= new String( "sTXSPEC.asNucleusInfo[1].bReferenceAmplitudeValid" );
    public final static String sTXSPEC_asNucleusInfo_1_bAmplitudeCorrectionValid= new String( "sTXSPEC.asNucleusInfo[1].bAmplitudeCorrectionValid" );
    public final static String sTXSPEC_aRFPULSE_0_tName = new String("sTXSPEC.aRFPULSE[0].tName" );
    public final static String sTXSPEC_aRFPULSE_0_bAmplitudeValid = new String( "sTXSPEC.aRFPULSE[0].bAmplitudeValid" );
    public final static String sTXSPEC_aRFPULSE_0_flAmplitude = new String( "sTXSPEC.aRFPULSE[0].flAmplitude" );
    public final static String sTXSPEC_aRFPULSE_1_tName = new String( "sTXSPEC.aRFPULSE[1].tName" );
    public final static String sTXSPEC_aRFPULSE_1_bAmplitudeValid = new String( "sTXSPEC.aRFPULSE[1].bAmplitudeValid" );
    public final static String sTXSPEC_aRFPULSE_1_flAmplitude = new String( "sTXSPEC.aRFPULSE[1].flAmplitude" );
    public final static String sTXSPEC_aRFPULSE_2_tName = new String( "sTXSPEC.aRFPULSE[2].tName" );
    public final static String sTXSPEC_aRFPULSE_2_bAmplitudeValid = new String( "sTXSPEC.aRFPULSE[2].bAmplitudeValid" );
    public final static String sTXSPEC_aRFPULSE_2_flAmplitude = new String( "sTXSPEC.aRFPULSE[2].flAmplitude" );
    public final static String sTXSPEC_aRFPULSE_3_tName = new String( "sTXSPEC.aRFPULSE[3].tName");
    public final static String sTXSPEC_aRFPULSE_3_bAmplitudeValid = new String( "sTXSPEC.aRFPULSE[3].bAmplitudeValid" );
    public final static String sTXSPEC_aRFPULSE_3_flAmplitude = new String( "sTXSPEC.aRFPULSE[3].flAmplitude" );
    public final static String sTXSPEC_aRFPULSE_4_tName = new String( "sTXSPEC.aRFPULSE[4].tName" );
    public final static String sTXSPEC_aRFPULSE_4_bAmplitudeValid = new String( "sTXSPEC.aRFPULSE[4].bAmplitudeValid" );
    public final static String sTXSPEC_aRFPULSE_4_flAmplitude = new String( "sTXSPEC.aRFPULSE[4].flAmplitude" );
    public final static String sTXSPEC_aRFPULSE_5_tName = new String( "sTXSPEC.aRFPULSE[5].tName" );
    public final static String sTXSPEC_aRFPULSE_5_bAmplitudeValid = new String( "sTXSPEC.aRFPULSE[5].bAmplitudeValid" );
    public final static String sTXSPEC_aRFPULSE_5_flAmplitude = new String( "sTXSPEC.aRFPULSE[5].flAmplitude" );
    public final static String sTXSPEC_aRFPULSE_6_tName = new String( "sTXSPEC.aRFPULSE[6].tName" );
    public final static String sTXSPEC_aRFPULSE_6_bAmplitudeValid = new String( "sTXSPEC.aRFPULSE[6].bAmplitudeValid" );
    public final static String sTXSPEC_aRFPULSE_6_flAmplitude = new String( "sTXSPEC.aRFPULSE[6].flAmplitude" );
    public final static String sTXSPEC_lNoOfTraPulses = new String( "sTXSPEC.lNoOfTraPulses" );
    public final static String sTXSPEC_lBTB1ParallelCapacity = new String( "sTXSPEC.lBTB1ParallelCapacity" );
    public final static String sTXSPEC_lBTB1SerialCapacity = new String( "sTXSPEC.lBTB1SerialCapacity" );
    public final static String sTXSPEC_lBTB2ParallelCapacity = new String( "sTXSPEC.lBTB2ParallelCapacity" );
    public final static String sTXSPEC_lBTB2SerialCapacity = new String( "sTXSPEC.lBTB2SerialCapacity" );
    public final static String sTXSPEC_bBTBValid = new String( "sTXSPEC.bBTBValid" );
    public final static String sTXSPEC_flKDynMagnitudeMin = new String( "sTXSPEC.flKDynMagnitudeMin" );
    public final static String sTXSPEC_flKDynMagnitudeMax = new String( "sTXSPEC.flKDynMagnitudeMax" );
    public final static String sTXSPEC_flKDynMagnitudeClipLow = new String( "sTXSPEC.flKDynMagnitudeClipLow" );
    public final static String sTXSPEC_flKDynMagnitudeClipHigh = new String( "sTXSPEC.flKDynMagnitudeClipHigh" );
    public final static String sTXSPEC_flKDynPhaseMax = new String( "sTXSPEC.flKDynPhaseMax" );
    public final static String sTXSPEC_flKDynPhaseClip = new String( "sTXSPEC.flKDynPhaseClip" );
    public final static String sTXSPEC_bKDynValid = new String( "sTXSPEC.bKDynValid" );
    public final static String sTXSPEC_ucRFPulseType = new String( "sTXSPEC.ucRFPulseType" );
    public final static String sTXSPEC_ucExcitMode = new String( "sTXSPEC.ucExcitMode" );
    public final static String sTXSPEC_ucSimultaneousExcitation = new String( "sTXSPEC.ucSimultaneousExcitation" );
    public final static String sRXSPEC_lGain = new String( "sRXSPEC.lGain" );
    public final static String sRXSPEC_bGainValid = new String( "sRXSPEC.bGainValid" );
    public final static String sRXSPEC_aFFT_SCALE_0_lRxChannel = new String( "sRXSPEC.aFFT_SCALE[0].lRxChannel" );
    public final static String sRXSPEC_aFFT_SCALE_0_flFactor = new String( "sRXSPEC.aFFT_SCALE[0].flFactor" );
    public final static String sRXSPEC_aFFT_SCALE_0_bValid = new String( "sRXSPEC.aFFT_SCALE[0].bValid" );
    public final static String sRXSPEC_bVariCapVoltagesValid = new String( "sRXSPEC.bVariCapVoltagesValid" );
    public final static String sRXSPEC_alDwellTime_0_ = new String( "sRXSPEC.alDwellTime[0]" );
    public final static String sAdjFreSpec_ulMode = new String( "sAdjFreSpec.ulMode" );
    public final static String sAdjShimSpec_ulMode = new String( "sAdjShimSpec.ulMode" );
    public final static String sAdjWatSupSpec_ulMode = new String( "sAdjWatSupSpec.ulMode" );
    public final static String alTR_0_ = new String( "alTR[0]" );
    public final static String lContrasts = new String( "lContrasts");
    public final static String alTE_0_ = new String( "alTE[0]");
    public final static String lCombinedEchoes = new String("lCombinedEchoes");
    public final static String sSliceArray_asSlice_0_sNormal_dTra = new String( "sSliceArray.asSlice[0].sNormal.dTra" );
    public final static String sSliceArray_asSlice_0_dThickness = new String( "sSliceArray.asSlice[0].dThickness" );
    public final static String sSliceArray_asSlice_0_dPhaseFOV = new String( "sSliceArray.asSlice[0].dPhaseFOV" );
    public final static String sSliceArray_asSlice_0_dReadoutFOV = new String( "sSliceArray.asSlice[0].dReadoutFOV" );
    public final static String sSliceArray_lSize = new String( "sSliceArray.lSize" );
    public final static String sSliceArray_lConc = new String( "sSliceArray.lConc" );
    public final static String sSliceArray_ucMode = new String( "sSliceArray.ucMode" );
    public final static String sSliceArray_sTSat_dThickness = new String( "sSliceArray.sTSat.dThickness" );
    public final static String sGroupArray_asGroup_0_nSize = new String( "sGroupArray.asGroup[0].nSize" );
    public final static String sGroupArray_asGroup_0_dDistFact = new String( "sGroupArray.asGroup[0].dDistFact" );
    public final static String sGroupArray_anMember_1_ = new String( "sGroupArray.anMember[1]");
    public final static String sGroupArray_lSize = new String( "sGroupArray.lSize" );
    public final static String sGroupArray_sPSat_dThickness = new String( "sGroupArray.sPSat.dThickness" );
    public final static String sNavigatorPara_ucRespComp = new String( "sNavigatorPara.ucRespComp" );
    public final static String sPrepPulses_ucFatSat = new String( "sPrepPulses.ucFatSat" );
    public final static String sPrepPulses_ucWaterSat = new String( "sPrepPulses.ucWaterSat" );
    public final static String sPrepPulses_ucInversion = new String( "sPrepPulses.ucInversion" );
    public final static String sPrepPulses_ucSatRecovery = new String( "sPrepPulses.ucSatRecovery" );
    public final static String sPrepPulses_ucFatSatMode = new String( "sPrepPulses.ucFatSatMode" );
    public final static String sKSpace_lBaseResolution = new String( "sKSpace.lBaseResolution" );
    public final static String sKSpace_lPhaseEncodingLines = new String( "sKSpace.lPhaseEncodingLines" );
    public final static String sKSpace_dPhaseResolution = new String( "sKSpace.dPhaseResolution" );
    public final static String sKSpace_lPartitions = new String( "sKSpace.lPartitions" );
    public final static String sKSpace_lImagesPerSlab = new String( "sKSpace.lImagesPerSlab" );
    public final static String sKSpace_dSliceResolution = new String( "sKSpace.dSliceResolution" );
    public final static String sKSpace_ucPhasePartialFourier = new String( "sKSpace.ucPhasePartialFourier" );
    public final static String sKSpace_ucSlicePartialFourier = new String( "sKSpace.ucSlicePartialFourier" );
    public final static String sKSpace_ucAveragingMode = new String( "sKSpace.ucAveragingMode" );
    public final static String sKSpace_ucMultiSliceMode = new String( "sKSpace.ucMultiSliceMode" );
    public final static String sKSpace_ucDimension = new String( "sKSpace.ucDimension" );
    public final static String sKSpace_unReordering = new String( "sKSpace.unReordering" );
    public final static String sFastImaging_lEPIFactor = new String( "sFastImaging.lEPIFactor" );
    public final static String sFastImaging_lTurboFactor = new String( "sFastImaging.lTurboFactor" );
    public final static String sFastImaging_lSegments = new String( "sFastImaging.lSegments" );
    public final static String sFastImaging_ulEnableRFSpoiling = new String( "sFastImaging.ulEnableRFSpoiling" );
    public final static String sPhysioImaging_lSignal1 = new String( "sPhysioImaging.lSignal1" );
    public final static String sPhysioImaging_lMethod1 = new String( "sPhysioImaging.lMethod1" );
    public final static String sPhysioImaging_lSignal2 = new String( "sPhysioImaging.lSignal2" );
    public final static String sPhysioImaging_lMethod2 = new String( "sPhysioImaging.lMethod2" );
    public final static String sPhysioImaging_lPhases = new String( "sPhysioImaging.lPhases " );
    public final static String sPhysioImaging_lRetroGatedImages = new String( "sPhysioImaging.lRetroGatedImages");
    public final static String sPhysioImaging_sPhysioECG_lTriggerPulses = new String( "sPhysioImaging.sPhysioECG.lTriggerPulses");
    public final static String sPhysioImaging_sPhysioECG_lTriggerWindow = new String( "sPhysioImaging.sPhysioECG.lTriggerWindow");
    public final static String sPhysioImaging_sPhysioECG_lArrhythmiaDetection = new String( "sPhysioImaging.sPhysioECG.lArrhythmiaDetection");
    public final static String sPhysioImaging_sPhysioECG_lCardiacGateOnThreshold = new String( "sPhysioImaging.sPhysioECG.lCardiacGateOnThreshold");
    public final static String sPhysioImaging_sPhysioECG_lCardiacGateOffThreshold = new String( "sPhysioImaging.sPhysioECG.lCardiacGateOffThreshold");
    public final static String sPhysioImaging_sPhysioPulse_lTriggerPulses = new String( "sPhysioImaging.sPhysioPulse.lTriggerPulses");
    public final static String sPhysioImaging_sPhysioPulse_lTriggerWindow = new String( "sPhysioImaging.sPhysioPulse.lTriggerWindow" );
    public final static String sPhysioImaging_sPhysioPulse_lCardiacGateOnThreshold = new String( "sPhysioImaging.sPhysioPulse.lCardiacGateOnThreshold" );
    public final static String sPhysioImaging_sPhysioPulse_lCardiacGateOffThreshold = new String( "sPhysioImaging.sPhysioPulse.lCardiacGateOffThreshold" );
    public final static String sPhysioImaging_sPhysioExt_lTriggerPulses = new String( "sPhysioImaging.sPhysioExt.lTriggerPulses" );
    public final static String sPhysioImaging_sPhysioExt_lTriggerWindow = new String( "sPhysioImaging.sPhysioExt.lTriggerWindow" );
    public final static String sPhysioImaging_sPhysioExt_lCardiacGateOnThreshold = new String( "sPhysioImaging.sPhysioExt.lCardiacGateOnThreshold" );
    public final static String sPhysioImaging_sPhysioExt_lCardiacGateOffThreshold = new String( "sPhysioImaging.sPhysioExt.lCardiacGateOffThreshold" );
    public final static String sPhysioImaging_sPhysioResp_lRespGateThreshold = new String( "sPhysioImaging.sPhysioResp.lRespGateThreshold" );
    public final static String sPhysioImaging_sPhysioResp_lRespGatePhase = new String( "sPhysioImaging.sPhysioResp.lRespGatePhase" );
    public final static String sPhysioImaging_sPhysioResp_dGatingRatio = new String( "sPhysioImaging.sPhysioResp.dGatingRatio" );
    public final static String sSpecPara_lVectorSize = new String( "sSpecPara.lVectorSize" );
    public final static String sSpecPara_lPreparingScans = new String( "sSpecPara.lPreparingScans" );
    public final static String sSpecPara_lPhaseCyclingType = new String( "sSpecPara.lPhaseCyclingType" );
    public final static String sSpecPara_lPhaseEncodingType = new String( "sSpecPara.lPhaseEncodingType" );
    public final static String sSpecPara_lRFExcitationBandwidth = new String( "sSpecPara.lRFExcitationBandwidth" );
    public final static String sSpecPara_sVoI_sPosition_dSag = new String( "sSpecPara.sVoI.sPosition.dSag" );
    public final static String sSpecPara_sVoI_sPosition_dCor = new String( "sSpecPara.sVoI.sPosition.dCor" );
    public final static String sSpecPara_sVoI_sPosition_dTra = new String( "sSpecPara.sVoI.sPosition.dTra" );
    public final static String sSpecPara_sVoI_sNormal_dSag = new String( "sSpecPara.sVoI.sNormal.dSag" );
    public final static String sSpecPara_sVoI_sNormal_dCor = new String( "sSpecPara.sVoI.sNormal.dCor" );
    public final static String sSpecPara_sVoI_sNormal_dTra = new String( "sSpecPara.sVoI.sNormal.dTra" );
    public final static String sSpecPara_sVoI_dThickness = new String( "sSpecPara.sVoI.dThickness" );
    public final static String sSpecPara_sVoI_dPhaseFOV = new String( "sSpecPara.sVoI.dPhaseFOV" );
    public final static String sSpecPara_sVoI_dReadoutFOV = new String( "sSpecPara.sVoI.dReadoutFOV" );
    public final static String sSpecPara_sVoI_dInPlaneRot = new String( "sSpecPara.sVoI.dInPlaneRot" );
    public final static String sSpecPara_ucVoIValid = new String( "sSpecPara.ucVoIValid" );
    public final static String sSpecPara_ucRemoveOversampling = new String( "sSpecPara.ucRemoveOversampling" );
    public final static String sSpecPara_lDecouplingType = new String( "sSpecPara.lDecouplingType" );
    public final static String sSpecPara_lNOEType = new String( "sSpecPara.lNOEType" );
    public final static String sSpecPara_lExcitationType = new String( "sSpecPara.lExcitationType" );
    public final static String sSpecPara_dDeltaFrequency = new String( "sSpecPara.dDeltaFrequency" );
    public final static String sDiffusion_ulMode = new String( "sDiffusion.ulMode" );
    public final static String sAngio_ucPCFlowMode = new String( " sAngio.ucPCFlowMode" );
    public final static String sAngio_ucTOFInflow = new String( "sAngio.ucTOFInflow" );
    public final static String sEllipticalFilter_ucMode = new String( "sEllipticalFilter.ucMode" );
    public final static String sPat_lAccelFactPE = new String( "sPat.lAccelFactPE" );
    public final static String sPat_lAccelFact3D = new String( "sPat.lAccelFact3D" );
    public final static String sPat_ucPATMode = new String( "sPat.ucPATMode" );
    public final static String sPat_ucRefScanMode = new String( "sPat.ucRefScanMode" );
    public final static String ucEnableIntro = new String( "ucEnableIntro" );
    public final static String ucDisableChangeStoreImages = new String( "ucDisableChangeStoreImages" );
    public final static String ucReconstructionMode = new String( "ucReconstructionMode" );
    public final static String ucPHAPSMode = new String( "ucPHAPSMode" );
    public final static String ucDixon = new String("ucDixon");
    public final static String lAverages = new String("lAverages");
    public final static String dFlipAngleDegree = new String("dFlipAngleDegree");
    public final static String lScanTimeSec = new String("lScanTimeSec");
    public final static String lTotalScanTimeSec = new String("lTotalScanTimeSec");
    public final static String dRefSNR = new String( "dRefSNR" );
    public final static String dRefSNR_VOI = new String( "dRefSNR_VOI" );
    public final static String aulServicePara_0_ = new String( "aulServicePara[0]" );
    public final static String sCOIL_SELECT_MEAS_asList_0_sCoilElementID_tCoilID = new String( "sCOIL_SELECT_MEAS.asList[0].sCoilElementID.tCoilID" );
    public final static String sCOIL_SELECT_MEAS_asList_0_sCoilElementID_lCoilCopy = new String( "sCOIL_SELECT_MEAS.asList[0].sCoilElementID.lCoilCopy" );
    public final static String sCOIL_SELECT_MEAS_asList_0_sCoilElementID_tElement = new String( "sCOIL_SELECT_MEAS.asList[0].sCoilElementID.tElement" );
    public final static String sCOIL_SELECT_MEAS_asList_0_lElementSelected = new String( "sCOIL_SELECT_MEAS.asList[0].lElementSelected" );
    public final static String sCOIL_SELECT_MEAS_asList_0_lRxChannelConnected = new String( "sCOIL_SELECT_MEAS.asList[0].lRxChannelConnected" );
    public final static String sCOIL_SELECT_MEAS_sCOILPLUGS_aulPlugId_0_ = new String( "sCOIL_SELECT_MEAS.sCOILPLUGS.aulPlugId[0]" );
    public final static String sCOIL_SELECT_MEAS_sCOILPLUGS_aulPlugId_1_ = new String( "sCOIL_SELECT_MEAS.sCOILPLUGS.aulPlugId[1]" );
    public final static String sCOIL_SELECT_MEAS_sCOILPLUGS_aulPlugId_2_ = new String( "sCOIL_SELECT_MEAS.sCOILPLUGS.aulPlugId[2]" );
    public final static String sCOIL_SELECT_MEAS_sCOILPLUGS_aulPlugId_3_ = new String( "sCOIL_SELECT_MEAS.sCOILPLUGS.aulPlugId[3]" );
    public final static String sCOIL_SELECT_MEAS_sCOILPLUGS_aulPlugId_4_ = new String( "sCOIL_SELECT_MEAS.sCOILPLUGS.aulPlugId[4]" );
    public final static String sCOIL_SELECT_MEAS_sCOILPLUGS_auiNmbrOfNibbles_0_ = new String( "sCOIL_SELECT_MEAS.sCOILPLUGS.auiNmbrOfNibbles[0]" );
    public final static String sCOIL_SELECT_MEAS_sCOILPLUGS_auiNmbrOfNibbles_1_ = new String( "sCOIL_SELECT_MEAS.sCOILPLUGS.auiNmbrOfNibbles[1]" );
    public final static String sCOIL_SELECT_MEAS_sCOILPLUGS_auiNmbrOfNibbles_2_ = new String( "sCOIL_SELECT_MEAS.sCOILPLUGS.auiNmbrOfNibbles[2]" );
    public final static String sCOIL_SELECT_MEAS_sCOILPLUGS_auiNmbrOfNibbles_3_ = new String( "sCOIL_SELECT_MEAS.sCOILPLUGS.auiNmbrOfNibbles[3]" );
    public final static String sCOIL_SELECT_MEAS_sCOILPLUGS_auiNmbrOfNibbles_4_ = new String( "sCOIL_SELECT_MEAS.sCOILPLUGS.auiNmbrOfNibbles[4]" );
    public final static String sEFISPEC_bEFIDataValid =  new String( "sEFISPEC.bEFIDataValid" );
}
