/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.dicom;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import org.apache.commons.jci.monitor.FilesystemAlterationListener;
import org.apache.commons.jci.monitor.FilesystemAlterationObserver;
import org.dcm4che2.data.DicomObject;
import org.dcm4che2.io.DicomOutputStream;
import org.spectrim.automaticprocessing.AutomaticProcessingFileHandler;
import org.spectrim.clinicalcasehandling.ClinicalSpectroscopyCaseWorklistData;
import org.spectrim.data.FileReaderWorker;
import org.spectrim.io.dicom.DicomHandler;
import org.spectrim.io.dicom.DicomHeader;
import org.spectrim.io.dicom.DicomHeaderLine;

/**
 *
 * @author slotboom
 */
public class DicomFileSystemAlterationListener implements FilesystemAlterationListener {

   /**
    * Default constructor: add class ClinicalSpectroscopyCaseWorklistData as action listener
    */
    public DicomFileSystemAlterationListener( )
    {
        ClinicalSpectroscopyCaseWorklistData instance = ClinicalSpectroscopyCaseWorklistData.getInstance( );
        m_actionListeners.add(instance);
    }
    
    @Override
    public void onStart(FilesystemAlterationObserver arg0) {
        //System.out.println( "DicomFileSystemAlterationListener::onStart" );
    }

    /**
     * Is called whenever a file is created.
     *
     * @param incomingFile
     * @param arg0
     */
    @Override
    public void onFileCreate(File incomingFile) 
    {
//        System.out.println("");
//        System.out.println("==================================================");

        DicomNetworkNodeDefaults netDefs = DicomNetworkNodeDefaults.getInstance();

//        System.out.println("Renaming and moving file : " + incomingFile.getAbsoluteFile());

        String outPath = netDefs.getProperty(DicomNetworkNodeDefaults.CURRENT_RENAME_TO_DICOM_DIRECTORY);

        // only files are treated; directories are ignored!
        // only files whose name contains .dcm are treated
        if ((incomingFile.isFile() == true) && ((incomingFile.getName().endsWith(".dcm") == true) || (incomingFile.getName().matches("^.+?\\d$") == true))) 
        {
            DicomObject dcmObj = DicomHandler.getDicomObject(incomingFile.getParentFile().getAbsolutePath() + File.separatorChar + incomingFile.getName());
            if (dcmObj != null) 
            {
                String patientName = eliminateIllegalFileSystemCharacters(DicomHandler.getPatientName(dcmObj));
                String seriesDescription = eliminateIllegalFileSystemCharacters(DicomHandler.getSeriesDescription(dcmObj));
                String seriesNumber = DicomHandler.getSeriesNumber(dcmObj) + "";
                String studyDate = DicomHandler.getStudyDate(dcmObj);
                String instanceNumber = String.format("%04d", DicomHandler.getInstanceNr(dcmObj));
                String birthDate = DicomHandler.getPatientBirthDate(dcmObj);
                String studyTime = DicomHandler.getStudyTime(dcmObj);
                
                int posDot = studyTime.indexOf(".");
                if (studyTime.contains(".")) 
                {
                    studyTime = studyTime.substring(0, posDot);
                } else 
                {
                    studyTime = "undefined";
                }

                String patientfolder = patientName + "-" + birthDate;
                String studyfolder = studyDate + "-" + studyTime;
                String seriesfolder = seriesNumber + "-" + seriesDescription;

                String outputFilename = instanceNumber;

                String fileToSave = outPath + File.separator + patientfolder
                        + File.separator + studyfolder + File.separator + seriesfolder + File.separator + outputFilename + ".dcm";
                File newfile = new File(fileToSave);
                newfile.getParentFile().mkdirs();
                
                if (AutomaticProcessingFileHandler.isActive()) 
                {
                        String imagetype = DicomHandler.getImageType(dcmObj);
                        String classUID = DicomHandler.getSOPClassUID(dcmObj);
                        FileReaderWorker.filetype type;
                        switch (classUID) {
                            case "1.2.840.10008.5.1.4.1.1.4": // Image
                                type = FileReaderWorker.filetype.IMAGE;
                                break;
                            case "1.2.840.10008.5.1.4.1.1.4.2": // Spectro std
                                type = FileReaderWorker.filetype.MRS_STD;
                                break;
                            case "1.3.12.2.1107.5.9.1": // Spectro/Tensor Siemens
                                if (imagetype.indexOf("TENSOR") >= 0) {
                                    type = FileReaderWorker.filetype.TENSOR;
                                } else {
                                    type = FileReaderWorker.filetype.MRS_SIEMENS;
                                }
                                break;
                            case "1.2.840.10008.5.1.4.1.1.7": // Perfusion derived maps
                                type = FileReaderWorker.filetype.PERFUSION;
                                break;
                            default: // Unknown format
                                type = FileReaderWorker.filetype.UNKNOWN;
                                break;
                        }
                        if (type == FileReaderWorker.filetype.MRS_SIEMENS || type == FileReaderWorker.filetype.MRS_STD) {
                            DicomHeader header = DicomHandler.getHeader(dcmObj);
                            ArrayList<String> refUIDs = new ArrayList<>();
                            if (type == FileReaderWorker.filetype.MRS_SIEMENS) {
                                DicomHandler.readSiemensData(dcmObj, header);
                                for (int j = 0; j < 3; j++) {
                                    String tagAux = "tReferenceImage" + j;
                                    DicomHeaderLine headerLine = (DicomHeaderLine) header.get(tagAux);
                                    if (headerLine != null) {
                                        refUIDs.add(headerLine.Value.replace("\"", ""));
                                    }
                                }
                            } else {
                                for (int j = 0; j < 3; j++) {
                                    String tagAux = "(0008,1140)[" + j + "]->(0008,1155)";
                                    DicomHeaderLine headerLine = (DicomHeaderLine) header.get(tagAux);
                                    if (headerLine != null) {
                                        refUIDs.add(headerLine.Value.replace("\"", ""));
                                    }
                                }
                            }
                            
                            notifyActionListeners( fileToSave, NOTIFY_DICOM_FILE_RENAMED, NOTIFY_DICOM_FILE_RENAMED_ID );
                            
                            AutomaticProcessingFileHandler.getInstance().processSpectroscopyFile(newfile,refUIDs);
                        }
                    }
                
                try 
                {
                    DicomOutputStream dos = new DicomOutputStream(new File(fileToSave));
                    dos.writeDicomFile(dcmObj);
                    dos.close();
                    File parentFile = incomingFile.getParentFile();
                    incomingFile.delete( );
                    if ( parentFile.list( ).length == 0 )
                    {
                        parentFile.delete( );
                    }
                } 
                catch ( Exception e ) 
                {
                    System.out.println( e.getMessage( ) );
                    System.exit( 0 );
                }
            }

//            DicomDecoder dicom_deco = new DicomDecoder( incomingFile.getParentFile( ).getAbsolutePath( ) + System.getProperty( "file.separator" ), incomingFile.getName( ) );
//
//            FileInfo dicom_fileinfo = null;
//            try
//            {
//                dicom_fileinfo = dicom_deco.getFileInfo( );
//
//                m_patientName = dicom_fileinfo.getPatientName( );
//                
//                m_examinationDate = dicom_fileinfo.getStudyDate( );
//                m_examinationTime = dicom_fileinfo.getStudyTime( );
//                
//                m_seriesDescription = dicom_fileinfo.getSeriesDescription( );
//
//                m_seriesNumber = new Integer( dicom_fileinfo.getSeriesNumber( ).trim( ) ).intValue( );
//                m_instanceNumber = new Integer( dicom_fileinfo.getInstanceNumber( ).trim( ) ).intValue( );
//
//                
//                // Variable necessary for logging dicomtags of incoming DICOM files
//                m_newSeriesInstanceUID = dicom_fileinfo.getSeriesInstanceUID( );
//                m_newSeriesDescription = dicom_fileinfo.getSeriesDescription( );
//                
//                String restPath1 = m_patientName + "_" + m_examinationDate + "_" + m_examinationTime;
//
//                restPath1 = eliminateIllegalFileSystemCharacters( restPath1 );
//
//                File directory1 = new File( outPath + System.getProperty( "file.separator" ) + restPath1 );
//
//                if ( directory1.exists( ) == false )
//                {
//                    directory1.mkdir( );
//                }
//
//                String restPath2 = outPath +
//                                   System.getProperty( "file.separator" ) +
//                                   restPath1 +
//                                   System.getProperty( "file.separator" ) +
//                                   m_seriesDescription;
//
//                restPath2 = eliminateIllegalFileSystemCharacters( restPath2 );
//
//                File directory2 = new File( restPath2 );
//
//                if ( directory2.exists( ) == false )
//                {
//                    directory2.mkdir( );
//                }
//
//                String fileName = m_patientName + "_" + m_seriesDescription + "_" + m_seriesNumber + "_" + m_instanceNumber + ".dcm";
//
//
//                fileName = eliminateIllegalFileSystemCharacters( fileName );
//
//                //File fileOut = new File( restPath2 + System.getProperty( "file.separator" ) + fileName );
//
//                //System.out.println( restPath2 + fileName );
//
//                m_outgoingFile = new File( restPath2, fileName );
//                m_outgoingFile.createNewFile( );
//
//                // incomingFile.renameTo( outgoingFile );
//                try 
//                {
//                    FileUtils.renameFile( incomingFile, m_outgoingFile );
//                } 
//                catch ( Exception ex ) 
//                {
//                    Logger.getLogger( DicomFileSystemAlterationListener.class.getName( ) ).log(Level.SEVERE, null, ex);
//                }
//                
//                notifyActionListeners( NOTIFY_DICOM_FILE_RENAMED, NOTIFY_DICOM_FILE_RENAMED_ID );
//                
////                if ( ( m_newSeriesInstanceUID.compareTo( m_lastSeriesInstanceUID ) != 0 ) &&
////                     ( m_newSeriesDescription.compareTo( m_lastSeriesDescription ) != 0 ) )
////                {
//                    
//                    System.out.println( "Old series uid is : " + m_lastSeriesInstanceUID );
//                    System.out.println( "New series uid is : " + m_newSeriesInstanceUID );
//                    
//                    if ( dicom_fileinfo.getImageModality( ).compareTo( "MR" ) ==  0 )
//                    {
//                        writeMRIRenamingLogFile( dicom_fileinfo );
//                    }
//                    if ( dicom_fileinfo.getImageModality( ).compareTo( "CT" ) ==  0 )
//                    {
//                        writeCTRenamingLogFile( dicom_fileinfo );
//                    }
//                    if ( dicom_fileinfo.getImageModality( ).compareTo( "XA" ) ==  0 )
//                    {
//                        writeFluoroscopyRenamingLogFile( dicom_fileinfo );
//                    }
//                    
//                    m_lastSeriesInstanceUID =  m_newSeriesInstanceUID;
//                    m_lastSeriesDescription = m_newSeriesDescription;
////                }
//                
//                // System.out.println( "Moved file to  ** : " + m_outgoingFile.getAbsoluteFile( ) );
//
//            }
//            catch ( IOException e )
//            {
//                
//                dicom_deco.closeInputStream( );
//                
//                // non handled exception
//                e.printStackTrace( );
//                System.out.println( e.getMessage( ) );
//                System.out.println( "Non handled exception" );
//
//                try
//                {
//                    //String exceptionPath = outPath + System.getProperty( "file.separator" ) + "non_renamed" + System.getProperty( "file.separator" );
//
//                    File exceptionPath = new File( outPath  + System.getProperty( "file.separator" ) + "non_handled_compressed_files" );
//                    exceptionPath.mkdir( );
//
//                    m_outgoingFile = new File( exceptionPath.getAbsolutePath( ) + System.getProperty( "file.separator" ), incomingFile.getName( ) );
//                    m_outgoingFile.createNewFile( );
//                    try 
//                    {
//                        // incomingFile.renameTo( outgoingFile );
//                        FileUtils.renameFile( incomingFile, m_outgoingFile );
//                    } 
//                    catch ( Exception ex ) 
//                    {
//                        Logger.getLogger(DicomFileSystemAlterationListener.class.getName()).log(Level.SEVERE, null, ex);
//                    }
//                    
//                    
//                    System.out.println( " ** Moved file to  : " + m_outgoingFile.getAbsoluteFile( ) );
//
//                    notifyActionListeners( NOTIFY_DICOM_UNEXPECTED_EOF, NOTIFY_DICOM_UNEXPECTED_EOF_ID );
//
//                    // non handled exception
//                    e.printStackTrace( );
//                    System.out.println( e.getMessage( ) );
//                    System.out.println( "Exception : No renaming possible due to non supported tranfer syntax uid !" );
//                }
//                catch ( IOException ioe )
//                {
//                   // non handled exceptionasdas
//                   System.out.println( "Unhandled exception in DicomFileSystemAlterationListener::handleExistingFilesInDirectory( )" );
//                }
//            }
        } 
        else 
        {
//            System.out.println("simply delete incoming file...");
            incomingFile.delete( );
        }
    }

    /**
     * Writes the information to be logged in to the logging information.
     *
     * @param fInfo
     */
    public void writeMRIRenamingLogFile(FileInfo fInfo) 
    {
        try 
        {
            DecimalFormat formatter = new DecimalFormat("###.00");

            String logFileName = m_homeDir + System.getProperty("file.separator") + "logDicomRenamedFiles.txt";

            File logFile = new File(logFileName);

            System.out.println("Writing to log file : " + logFileName);

            FileWriter fileWriter;

            if ( logFile.exists( ) == false ) 
            {
                fileWriter = new FileWriter(logFileName);
            } 
            else 
            {
                fileWriter = new FileWriter(logFileName, true);
            }

            fileWriter.append(fInfo.getPatientName() + ",");
            fileWriter.append(fInfo.getSeriesNumber().trim() + ",");
            fileWriter.append(fInfo.getSeriesDescription() + ",");
            fileWriter.append(fInfo.getAcquisitionDate() + ",");
            fileWriter.append(fInfo.getAcquisitionTime() + ",");
            fileWriter.append(fInfo.getPixelWidth() + ",");
            fileWriter.append(fInfo.getPixelHeight() + ",");
            fileWriter.append(fInfo.getPixelDepth() + ",");
            fileWriter.append(fInfo.getSliceThickness() + ",");
            fileWriter.append(fInfo.getSeriesNumber().trim() + ",");
            fileWriter.append(fInfo.getTR() + ",");
            fileWriter.append(fInfo.getEchoTime() + ",");
            fileWriter.append(fInfo.getImageWidth() + ",");
            fileWriter.append(fInfo.getImageHeight() + ",");
            fileWriter.append(fInfo.getSeriesInstanceUID());
            fileWriter.write("\r\n");

            fileWriter.close();
        } 
        catch (IOException e) 
        {
            System.out.println("Error creating file : " + "~/logFileName.txt");
        }
    }

    /**
     * Writes the CT information to be logged in to the logging information
     *
     * @param fInfo
     */
    public void writeCTRenamingLogFile(FileInfo fInfo) 
    {
        try 
        {

            DecimalFormat formatter = new DecimalFormat("###.00");

            String logFileName = m_homeDir + System.getProperty("file.separator") + "logCTDicomRenamedFiles.txt";

            File logFile = new File(logFileName);

            System.out.println("Writing to log file : " + logFileName);

            FileWriter fileWriter;

            if (logFile.exists() == false) {
                fileWriter = new FileWriter(logFileName);
            } else {
                fileWriter = new FileWriter(logFileName, true);
            }

            fileWriter.append(fInfo.getSeriesInstanceUID() + ",");

            fileWriter.append(fInfo.getPatientName() + ",");
            fileWriter.append(fInfo.getSeriesNumber().trim() + ",");
            fileWriter.append(fInfo.getSeriesDescription() + ",");
            fileWriter.append(fInfo.getAcquisitionDate() + ",");
            fileWriter.append(fInfo.getAcquisitionTime() + ",");

            fileWriter.append(fInfo.get_CT_CTDIvol() + ",");
            fileWriter.append(fInfo.get_CTXRayTubeCurrent() + ",");
            fileWriter.append(fInfo.get_CT_KVP() + ",");
            fileWriter.append(fInfo.get_CT_ExposureTime() + ",");
            fileWriter.append(fInfo.get_CT_ExposureModulationType() + ",");

            fileWriter.append(fInfo.get_CT_DistanceSourceToDetector() + ",");
            fileWriter.append(fInfo.get_CT_DistanceSourceToPatient() + ",");
            fileWriter.append(fInfo.get_CT_DataCollectionDiameter() + ",");
            fileWriter.append(fInfo.get_CT_ReconstructionTargetCenterPatient() + ",");

            fileWriter.append(fInfo.get_CT_FilterType() + ",");
            fileWriter.append(fInfo.get_CT_ConvolutionKernel() + ",");

            fileWriter.append(fInfo.getPixelWidth() + ",");
            fileWriter.append(fInfo.getPixelHeight() + ",");
            fileWriter.append(fInfo.getPixelDepth() + ",");
            fileWriter.append(fInfo.getSliceThickness() + ",");

            fileWriter.append(fInfo.get_CT_ContrastBolusIngredientConcentration() + ",");
            fileWriter.append(fInfo.get_CT_ContrastFlowRate() + ",");
            fileWriter.append(fInfo.get_CT_ContrastBolusStartTime() + ",");
            fileWriter.append(fInfo.get_CT_ContrastBolusStopTime() + ",");
            fileWriter.append(fInfo.get_CT_ContrastBolusTotalDose() + ",");

            fileWriter.write("\r\n");

            fileWriter.close();
        } 
        catch (IOException e) 
        {
            System.out.println("Error creating file : " + "~/logCTFileName.txt");
        }
    }

    /**
     * Writes the CT information to be logged in to the logging information
     *
     * @param fInfo
     */
    public void writeFluoroscopyRenamingLogFile(FileInfo fInfo) 
    {
        try 
        {
            DecimalFormat formatter = new DecimalFormat("###.00");

            String logFileName = m_homeDir + System.getProperty("file.separator") + "logFluoroscopyDicomRenamedFiles.txt";

            File logFile = new File(logFileName);

            System.out.println("Writing to log file : " + logFileName);

            FileWriter fileWriter;

            if (logFile.exists() == false) 
            {
                fileWriter = new FileWriter(logFileName);
            } else {
                fileWriter = new FileWriter(logFileName, true);
            }

            fileWriter.append(fInfo.getPatientName() + ",");
            fileWriter.append(fInfo.getSeriesNumber().trim() + ",");
            fileWriter.append(fInfo.getSeriesDescription() + ",");
            fileWriter.append(fInfo.getAcquisitionDate() + ",");
            fileWriter.append(fInfo.getAcquisitionTime() + ",");
            fileWriter.append(fInfo.getPixelWidth() + ",");
            fileWriter.append(fInfo.getPixelHeight() + ",");
            fileWriter.append(fInfo.getPixelDepth() + ",");
            fileWriter.append(fInfo.getSliceThickness() + ",");
            fileWriter.append(fInfo.getSeriesNumber().trim() + ",");
            fileWriter.append(fInfo.getTR() + ",");
            fileWriter.append(fInfo.getEchoTime() + ",");
            fileWriter.append(fInfo.getImageWidth() + ",");
            fileWriter.append(fInfo.getImageHeight() + ",");
            fileWriter.append(fInfo.getSeriesInstanceUID());
            fileWriter.write("\r\n");

            fileWriter.close();
        } 
        catch (IOException e) 
        {
            System.out.println("Error creating file : " + "~/logCTFileName.txt");
        }
    }

    @Override
    public void onFileChange(File arg0) 
    {
        System.out.println("DicomFileSystemAlterationListener::onFileChange " + arg0.getName());
    }

    @Override
    public void onFileDelete(File arg0) 
    {
        // System.out.println( "DicomFileSystemAlterationListener::onFileDelete " + arg0.getName( ) );
    }

    @Override
    public void onDirectoryCreate(File arg0) 
    {
        // System.out.println( "DicomFileSystemAlterationListener::onDirectoryCreate " + arg0.getName( ) );
    }

    @Override
    public void onDirectoryChange(File arg0) 
    {
        // System.out.println( "DicomFileSystemAlterationListener::onDirectoryChange" + arg0.getName( ) );
    }

    @Override
    public void onDirectoryDelete(File arg0) 
    {
        //System.out.println( "DicomFileSystemAlterationListener::onDirectoryDelete " + arg0.getName( ) );
    }

    @Override
    public void onStop(FilesystemAlterationObserver arg0) 
    {
        //System.out.println( "DicomFileSystemAlterationListener::onStop" );
    }

    /**
     * Replaces all blanks by "\ " in order to be interpreted correctly by the
     * Windows operating system
     *
     * @param cmd
     * @return
     */
    private String eliminateIllegalFileSystemCharacters(String cmd) 
    {

        // first get rid ofm blancs
        cmd = cmd.trim();

        cmd = cmd.replace(' ', '_');
        cmd = cmd.replace(',', '_');
        cmd = cmd.replace(';', '_');
        cmd = cmd.replace('%', '_');
        //cmd.replace( '*', '_' );

        cmd = cmd.replace('é', 'e'); // problematic on some linux systems
        cmd = cmd.replace('è', 'e'); // problematic on some linux systems
        cmd = cmd.replace('ö', 'o'); // problematic on some linux systems
        cmd = cmd.replace('ü', 'u'); // problematic on some linux systems
        cmd = cmd.replace('à', 'a'); // problematic on some linux systems
        cmd = cmd.replace('ä', 'a'); // problematic on some linux systems
        cmd = cmd.replace('Ö', 'O'); // problematic on some linux systems
        cmd = cmd.replace('Ü', 'U'); // problematic on some linux systems

        cmd = cmd.replace('@', '_'); // problematic on some linux systems
        cmd = cmd.replace('#', '_'); // problematic on some linux systems

        cmd = cmd.replace('?', '_');  // problematic on some linux systems

        cmd = cmd.replace('¢', 'c'); // problematic on some linux systems
        cmd = cmd.replace('!', '_');  // problematic on some linux systems
        cmd = cmd.replace('§', '_');  // problematic on some linux systems
        cmd = cmd.replace('¦', '_');  // problematic on some linux systems

        cmd = cmd.replace('$', '_');  // problematic on some linux systems
        cmd = cmd.replace('[', '_');  // problematic on some linux systems
        cmd = cmd.replace(']', '_');  // problematic on some linux systems

        cmd = cmd.replace('^', '_');  // problematic on some linux systems
        cmd = cmd.replace('~', '_');  // problematic on some linux systems

        cmd = cmd.replace('ç', 'c');  // problematic on some linux systems
        cmd = cmd.replace('~', '_');  // problematic on some linux systems
        cmd = cmd.replace('"', '_');  // problematic on some linux systems

        cmd = cmd.replace("____", "_");
        cmd = cmd.replace("___", "_");
        cmd = cmd.replace("__", "_");

        return cmd;
    }

    /**
     * Adds an ActionListener to the list of listeners.
     * @param e
     */
    public void addActionListener( ActionListener e ) 
    {
        m_actionListeners.add(e);
    }

    /**
     * Notifies actionEvent to all registered listeners.
     */
    private void notifyActionListeners( String fileRef, String actionCommand, int actionID) 
    {
        Iterator it = m_actionListeners.iterator( );
        while (it.hasNext()) 
        {
            ( (ActionListener) it.next( ) ).actionPerformed( new ActionEvent( fileRef, actionID, actionCommand ) );
        }
    }

    /**
     * @return The last outgoing file which was received.
     */
    public File getOutgoingFile( ) 
    {
        return m_outgoingFile;
    }

    // Attribute of the class
    private String  m_patientName;
    private String  m_examinationDate;
    private String  m_examinationTime;
    private String  m_seriesDescription;
    private int     m_seriesNumber;
    private int     m_instanceNumber;
    private File    m_outgoingFile;

    // this class should notify all action listeners whenever a DICOM file enters
    HashSet<ActionListener> m_actionListeners = new HashSet<ActionListener>( );

    static final public String NOTIFY_DICOM_FILE_RENAMED    = "DICOM_FILE_RENAMED";
    static final public String NOTIFY_DICOM_UNEXPECTED_EOF  = "DICOM_FILE_UNEXPECTED_EOF";
    static final public int NOTIFY_DICOM_FILE_RENAMED_ID    = 0;
    static final public int NOTIFY_DICOM_UNEXPECTED_EOF_ID  = 1;

    private String m_lastSeriesInstanceUID = "0";
    private String m_newSeriesInstanceUID = "-1";

    private String m_lastSeriesDescription = "0";
    private String m_newSeriesDescription = "-1";

    private String m_homeDir = System.getProperty("user.home");
    
}
