/*
 * File: $Id: DicomNetworkFiletransferHandler.java,v 1.6 2009/12/06 13:28:39 hans Exp $
 * History
 * Date: 12.06.2009  Authors: Hans Slotboom
 *
 * Changes:
 *
 * Copyright (C) 2009 by
 *
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */
package org.spectrim.dicom;

import org.spectrim.lib.DualProgressBar;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.text.DecimalFormat;
import java.util.ArrayList;

import java.util.Iterator;
import java.util.LinkedList;
import javax.swing.JFrame;

/**
 *
 * Class dedicated to listening to incoming Dicom files from other Dicom nodes.
 *
 * @author slotboom
 */
import org.apache.commons.jci.monitor.FilesystemAlterationMonitor;
import org.dcm4che2.data.DicomObject;
import org.dcm4che2.io.DicomOutputStream;
import org.spectrim.automaticprocessing.AutomaticProcessingFileHandler;
import org.spectrim.data.FileReaderWorker;
import org.spectrim.io.dicom.DicomHandler;
import org.spectrim.io.dicom.DicomHeader;
import org.spectrim.io.dicom.DicomHeaderLine;

public class DicomNetworkFileTransferAndDicomFileRenamingHandler implements ActionListener 
{

    public DicomNetworkFileTransferAndDicomFileRenamingHandler( JFrame parent, String AETitle, int portNumber, String localDicomFilePath,
                                                                boolean renameDirectory, String renameToFilePath, boolean deleteAfterRename) 
    {
        m_parent = parent;
        m_AETitle = AETitle;
        m_portNumber = portNumber;
        m_localDicomFilePath = localDicomFilePath;

        m_renameDirectory = renameDirectory;
        m_renameToFilePath = renameToFilePath;
        m_deleteAfterRename = deleteAfterRename;
    }

    /**
     * Starts the handler of all incoming Dicom files, and files present in the
     * incoming directory.
     *
     * Renaming starts at startup ends at program end (at first)
     *
     */
    public void startDicomFileRenamerService() {

        // First look whether the directory is empty before starting the
        // filesystemalteration listener..
        if ((m_localDicomFilePath != null) && (m_localDicomFilePath.length()) != 0) {
            m_localDicomFilePathAsFile = new File(m_localDicomFilePath);

            if (m_localDicomFilePathAsFile != null) {
                if ((m_localDicomFilePathAsFile.exists()) && (m_localDicomFilePathAsFile.isDirectory())) {
//                    if ( m_localDicomFilePathAsFile.length( ) > 0 )
//                    {
                    serviceStart(m_localDicomFilePathAsFile);
//                    }

                }
            }
        }

    }

    /**
     * Starts the DicomFileAlterationMonitoring using a
     *
     * org.apache.commons.jci.monitor.FilesystemAlterationMonitor
     *
     */
    private void startDicomFileAlterationMonitoring() {
        m_fileSystemAlterationMonitor = new FilesystemAlterationMonitor();

        m_dicomFileAlterationListener = new DicomFileSystemAlterationListener();

        m_dicomFileAlterationListener.addActionListener(this);

        m_fileSystemAlterationMonitor.addListener(new File(m_localDicomFilePath), m_dicomFileAlterationListener);

        m_fileSystemAlterationMonitor.start();
    }

    /**
     *
     * @param dicom_dir
     */
    private void serviceStart(final File dicom_dir) {
        // Get the list of files in the selected dicom directory
        m_dcm_filelist = dicom_dir.list();
        if (m_dcm_filelist != null) {
            if (m_dcm_filelist.length > 0) {
                m_dualProgressBar = new DualProgressBar( m_parent, 100, 100, new Dimension( 900, 150 ), false );
                m_dualProgressBar.setTitle( "Analyzing Dicom Directory" );

                m_dualProgressBar.setUpperProgress(0);
                m_dualProgressBar.setUpperLabel("        Total progress " );
                m_dualProgressBar.setUpperProgressVisible(false);
                m_dualProgressBar.setLowerLabel( "<HTML><BR>     Analyzing Dicom image : " + m_dcm_filelist[0] + "</HTML>" );

                final org.spectrim.lib.SwingWorker worker = new org.spectrim.lib.SwingWorker() {
                    public Object construct() {
                        handleExistingFilesInDirectory(dicom_dir);
                        return null;
                    }

                    // runs in the event dispatching thread
                    @Override
                    public void finished() {
                        // After calculation set enabled again
                        m_dualProgressBar.setVisible(false);
                        startDicomNetworkFiletransferHandler();
                    }
                };
                worker.start();
            } else {
                startDicomNetworkFiletransferHandler();
            }
        }
    }

    /**
     * Analyzes the dicom directory; i.e. that all dicom file headers are read
     * and the file header object (FileInfo) objects are stored in a HashTable
     * The files are renamed.
     */
    private void handleExistingFilesInDirectory(File dicom_dir) {

        for (int i = 0; i < m_dcm_filelist.length; i++) {

            m_dualProgressBar.setLowerProgress((int) (100.0 * (double) i / m_dcm_filelist.length));
            m_dualProgressBar.setLowerLabel("    Analyzing Dicom image : " + m_dcm_filelist[i]);

            File file = new File(dicom_dir.getPath() + File.separatorChar + m_dcm_filelist[i]);
            File outgoingFile;
            String outputFolder = DicomNetworkNodeDefaults.getInstance().getProperty(DicomNetworkNodeDefaults.CURRENT_RENAME_TO_DICOM_DIRECTORY);

            // only files are treated; directories are ignored!
            if ((file.isFile() == true) && !file.getName().endsWith(".nii") && ((file.getName().endsWith(".dcm") == true) && !file.getName().matches("^.+?\\d$"))) {
                DicomObject dcmObj = DicomHandler.getDicomObject(dicom_dir.getPath() + File.separatorChar + m_dcm_filelist[i]);
                if (dcmObj != null) {
                    String patientName = eliminateIllegalFileSystemCharacters(DicomHandler.getPatientName(dcmObj));
                    String seriesDescription = eliminateIllegalFileSystemCharacters(DicomHandler.getSeriesDescription(dcmObj));
                    String seriesNumber = DicomHandler.getSeriesNumber(dcmObj) + "";
                    String studyDate = DicomHandler.getStudyDate(dcmObj);
                    String instanceNumber = String.format("%04d", DicomHandler.getInstanceNr(dcmObj));
                    String birthDate = DicomHandler.getPatientBirthDate(dcmObj);
                    String studyTime = DicomHandler.getStudyTime(dcmObj);
                    int posDot = studyTime.indexOf(".");
                    if (studyTime.contains(".")) {
                        studyTime = studyTime.substring(0, posDot);
                    } else {
                        studyTime = "undefined";
                    }

                    String patientfolder = patientName + "-" + birthDate;
                    String studyfolder = studyDate + "-" + studyTime;
                    String seriesfolder = seriesNumber + "-" + seriesDescription;

                    String outputFilename = instanceNumber;

                    String fileToSave = outputFolder + File.separator + patientfolder
                            + File.separator + studyfolder + File.separator + seriesfolder + File.separator + outputFilename + ".dcm";
                    File newfile = new File(fileToSave);
                    newfile.getParentFile().mkdirs();
//                    notifyActionListeners(newfile, NOTIFY_DICOM_FILE_RENAMED, NOTIFY_DICOM_FILE_RENAMED_ID);

                    if (AutomaticProcessingFileHandler.isActive()) {
                        String imagetype = DicomHandler.getImageType(dcmObj);
                        String classUID = DicomHandler.getSOPClassUID(dcmObj);
                        FileReaderWorker.filetype type;
                        switch (classUID) {
                            case "1.2.840.10008.5.1.4.1.1.4": // Image
                                type = FileReaderWorker.filetype.IMAGE;
                                break;
                            case "1.2.840.10008.5.1.4.1.1.4.2": // Spectro std
                                type = FileReaderWorker.filetype.MRS_STD;
                                break;
                            case "1.3.12.2.1107.5.9.1": // Spectro/Tensor Siemens
                                if (imagetype.indexOf("TENSOR") >= 0) {
                                    type = FileReaderWorker.filetype.TENSOR;
                                } else {
                                    type = FileReaderWorker.filetype.MRS_SIEMENS;
                                }
                                break;
                            case "1.2.840.10008.5.1.4.1.1.7": // Perfusion derived maps
                                type = FileReaderWorker.filetype.PERFUSION;
                                break;
                            default: // Unknown format
                                type = FileReaderWorker.filetype.UNKNOWN;
                                break;
                        }
                        if (type == FileReaderWorker.filetype.MRS_SIEMENS || type == FileReaderWorker.filetype.MRS_STD) {
                            DicomHeader header = DicomHandler.getHeader(dcmObj);
                            ArrayList<String> refUIDs = new ArrayList<>();
                            if (type == FileReaderWorker.filetype.MRS_SIEMENS) {
                                DicomHandler.readSiemensData(dcmObj, header);
                                for (int j = 0; j < 3; j++) {
                                    String tagAux = "tReferenceImage" + j;
                                    DicomHeaderLine headerLine = (DicomHeaderLine) header.get(tagAux);
                                    if (headerLine != null) {
                                        refUIDs.add(headerLine.Value.replace("\"", ""));
                                    }
                                }
                            } else {
                                for (int j = 0; j < 3; j++) {
                                    String tagAux = "(0008,1140)[" + j + "]->(0008,1155)";
                                    DicomHeaderLine headerLine = (DicomHeaderLine) header.get(tagAux);
                                    if (headerLine != null) {
                                        refUIDs.add(headerLine.Value.replace("\"", ""));
                                    }
                                }
                            }
                            AutomaticProcessingFileHandler.getInstance().processSpectroscopyFile(newfile,refUIDs);
                        }
                    }

                    try {
                        DicomOutputStream dos = new DicomOutputStream(new File(fileToSave));
                        dos.writeDicomFile(dcmObj);
                        dos.close();
                        file.delete();
                    } catch (Exception e) {
                        System.out.println(e.getMessage());
                        System.exit(0);
                    }

                }

//                DicomDecoder dicom_deco = new DicomDecoder(dicom_dir.getPath() + File.separatorChar, m_dcm_filelist[i]);
//
//                FileInfo dicom_fileinfo = null;
//                try {
//                    dicom_fileinfo = dicom_deco.getFileInfo();
//
//                    m_patientName = dicom_fileinfo.getPatientName();
//                    m_examinationDate = dicom_fileinfo.getStudyDate();
//
//                    m_examinationTime = dicom_fileinfo.getStudyTime();
//
//                    m_seriesDescription = dicom_fileinfo.getSeriesDescription();
//
//                    m_seriesNumber = new Integer(dicom_fileinfo.getSeriesNumber().trim()).intValue();
//                    m_instanceNumber = new Integer(dicom_fileinfo.getInstanceNumber().trim()).intValue();
//
//                    // Variable necessary for logging dicomtags of incoming DICOM files
//                    m_newSeriesInstanceUID = dicom_fileinfo.getSeriesInstanceUID();
//                    m_newSeriesDescription = dicom_fileinfo.getSeriesDescription();
//
//                    String restPath1 = m_patientName + "_" + m_examinationDate + "_" + m_examinationTime;
//
//                    restPath1 = eliminateIllegalFileSystemCharacters(restPath1);
//
//                    File directory1 = new File(m_renameToFilePath + System.getProperty("file.separator") + restPath1);
//
//                    if (directory1.exists() == false) {
//                        directory1.mkdir();
//                    }
//
//                    String restPath2 = m_renameToFilePath
//                            + System.getProperty("file.separator")
//                            + restPath1
//                            + System.getProperty("file.separator")
//                            + m_seriesDescription;
//
//                    restPath2 = eliminateIllegalFileSystemCharacters(restPath2);
//
//                    File directory2 = new File(restPath2);
//
//                    if (directory2.exists() == false) {
//                        directory2.mkdir();
//                    }
//
//                    String fileName = m_patientName + "_" + m_seriesDescription + "_" + m_seriesNumber + "_" + m_instanceNumber + ".dcm";
//
//                    fileName = eliminateIllegalFileSystemCharacters(fileName);
//
//                    //File fileOut = new File( restPath2 + System.getProperty( "file.separator" ) + fileName );
//                    System.out.println(restPath2 + fileName);
//
//                    File fl = new File(restPath2, fileName);
//                    fl.createNewFile();
//                    try {
//                        //file.renameTo( fl );
//                        FileUtils.renameFile(file, fl);
//                    } catch (Exception ex) {
//                        Logger.getLogger(DicomNetworkFileTransferAndDicomFileRenamingHandler.class.getName()).log(Level.SEVERE, null, ex);
//                    }
//
//
//                    // System.out.println( "Moved file to  ** : " + file.getAbsoluteFile( ) );
//                    if ((m_newSeriesInstanceUID.compareTo(m_lastSeriesInstanceUID) != 0)
//                            && (m_newSeriesDescription.compareTo(m_lastSeriesDescription) != 0)) {
//
//                        System.out.println("Old series uid is : " + m_lastSeriesInstanceUID);
//                        System.out.println("New series uid is : " + m_newSeriesInstanceUID);
//
//                        if (dicom_fileinfo.getImageModality().compareTo("MR") == 0) {
//                            writeMRIRenamingLogFile(dicom_fileinfo);
//                        }
//                        if (dicom_fileinfo.getImageModality().compareTo("CT") == 0) {
//                            writeCTRenamingLogFile(dicom_fileinfo);
//                        }
//                        if (dicom_fileinfo.getImageModality().compareTo("XA") == 0) {
//                            writeFluoroscopyRenamingLogFile(dicom_fileinfo);
//                        }
//
//                        m_lastSeriesInstanceUID = m_newSeriesInstanceUID;
//                        m_lastSeriesDescription = m_newSeriesDescription;
//                    }
//
//                } catch (IOException e) {
//                    // non handled exception
//                    e.printStackTrace();
//                    System.out.println(e.getMessage());
//                    System.out.println("Non handled exception");
//
//                    try {
//                        //String exceptionPath = outPath + System.getProperty( "file.separator" ) + "non_renamed" + System.getProperty( "file.separator" );
//
//                        File exceptionPath = new File(m_renameToFilePath + System.getProperty("file.separator") + "non_handled_compressed_files");
//                        exceptionPath.mkdir();
//
//                        outgoingFile = new File(exceptionPath.getAbsolutePath() + System.getProperty("file.separator"), file.getName());
//                        outgoingFile.createNewFile();
//
//                        try {
//
//                            // file.renameTo( outgoingFile );
//                            FileUtils.renameFile(file, outgoingFile);
//                        } catch (Exception ex) {
//                            Logger.getLogger(DicomNetworkFileTransferAndDicomFileRenamingHandler.class.getName()).log(Level.SEVERE, null, ex);
//                        }
//
//                        System.out.println("Moved file to  : " + outgoingFile.getAbsoluteFile());
//
//                        // non handled exception
//                        e.printStackTrace();
//                        System.out.println(e.getMessage());
//                        System.out.println("Exception : No renaming possible due to non supported tranfer syntax uid !");
//                    } catch (IOException ioe) {
//                        // non handled exception
//                        System.out.println("Unhandled exception in DicomNetworkFileTransferAndDicomFileRenamingHandler::handleExistingFilesInDirectory( )");
//                    }
//                }
            } else {
                file.delete();
            }
        }
    }

    /**
     * Replaces all blanks by "\ " in order to be interpreted correctly by the
     * Windows operating system
     *
     * @param cmd
     * @return
     */
    private String eliminateIllegalFileSystemCharacters(String cmd) {

        // first get rid ofm blancs
        cmd = cmd.trim();

        cmd = cmd.replace(' ', '_');
        cmd = cmd.replace(',', '_');
        cmd = cmd.replace(';', '_');
        cmd = cmd.replace('%', '_');
        cmd = cmd.replace('*', '_');

        cmd = cmd.replace('é', 'e'); // problematic on some linux systems
        cmd = cmd.replace('è', 'e'); // problematic on some linux systems
        cmd = cmd.replace('ö', 'o'); // problematic on some linux systems
        cmd = cmd.replace('ü', 'u'); // problematic on some linux systems
        cmd = cmd.replace('à', 'a'); // problematic on some linux systems
        cmd = cmd.replace('ä', 'a'); // problematic on some linux systems
        cmd = cmd.replace('Ö', 'O'); // problematic on some linux systems
        cmd = cmd.replace('Ü', 'U'); // problematic on some linux systems

        cmd = cmd.replace('@', '_'); // problematic on some linux systems
        cmd = cmd.replace('#', '_'); // problematic on some linux systems

        cmd = cmd.replace('?', '_');  // problematic on some linux systems

        cmd = cmd.replace('¢', 'c'); // problematic on some linux systems
        cmd = cmd.replace('!', '_');  // problematic on some linux systems
        cmd = cmd.replace('§', '_');  // problematic on some linux systems
        cmd = cmd.replace('¦', '_');  // problematic on some linux systems
        cmd = cmd.replace('|', '_');  // problematic on some linux systems

        cmd = cmd.replace('$', '_');  // problematic on some linux systems
        cmd = cmd.replace('[', '_');  // problematic on some linux systems
        cmd = cmd.replace(']', '_');  // problematic on some linux systems
        cmd = cmd.replace(':', '_');  // problematic on some linux systems

        cmd = cmd.replace('^', '_');  // problematic on some linux systems
        cmd = cmd.replace('~', '_');  // problematic on some linux systems

        cmd = cmd.replace('ç', 'c');  // problematic on some linux systems
        cmd = cmd.replace('~', '_');  // problematic on some linux systems
        cmd = cmd.replace('"', '_');  // problematic on some linux systems
        cmd = cmd.replace('<', '_');  // problematic on some linux systems
        cmd = cmd.replace('>', '_');  // problematic on some linux systems
        cmd = cmd.replace("____", "_");
        cmd = cmd.replace("___", "_");
        cmd = cmd.replace("__", "_");

        return cmd;
    }

    /**
     * Starts the file renaming and file system alteration listener
     */
    public void startFileAlterationAndRenamingThread() {
        final org.spectrim.lib.SwingWorker worker = new org.spectrim.lib.SwingWorker() {

            public Object construct() {
                System.out.println("File alteration monitor - thread started");
                startDicomFileAlterationMonitoring();
                return null;
            }

            // runs in the event dispatching thread
            @Override
            public void finished() {
                // After calculation set enabled again
                System.out.println("File alteration monitor - thread stopped");
            }
        };
        worker.start();
    }

    /**
     * Starts the handler of incoming Dicom files
     */
    public void startDicomNetworkFiletransferHandler() {

        // first start the filealteration listener; this will react to changes
        // in the file system and will react appropriatly to it
        startFileAlterationAndRenamingThread();

        // start the DICOM file receiving with Dicom network protocol ..
        startDicomReceiveThread();

    }

    /**
     * Starts the code which runs in a separate thread.
     *
     * @return
     */
    private int dicomReceiveTreadCode() {

        // Now start the Dicom receive tool
        DicomNetworkNode remotedcmNode = DicomNetworkNodes.getInstance().getRemoteDicomNetworkNode(DicomNetworkNodes.getInstance().getSelectedRemoteDicomNodeNumber());
        DicomNetworkNode localDcmNode = DicomNetworkNodes.getInstance().getLocalHostDicomNetworkNode();

        // All operations take place is the working directory 
        String workingDir = System.getProperty( "user.dir" );
        String cmd = "";

        if ( System.getProperty("os.name").toLowerCase().compareTo("linux") == 0) 
        {
            cmd = workingDir + "/lib/dcmrcv " + localDcmNode.getReceiveAEtitle() + ":" + localDcmNode.getDicomReceivePortNumber() + " "
                    + "-dest " + m_localDicomFilePath;
        }

        if (System.getProperty("os.name").toLowerCase().contains("windows") == true) {
            cmd = workingDir + "\\lib\\dcmrcv.bat " + localDcmNode.getReceiveAEtitle() + ":" + localDcmNode.getDicomReceivePortNumber() + " "
                    + "-dest " + m_localDicomFilePath;

            cmd = generateProperWindowsCommand(cmd);
        }

        System.out.println("Trying following system command : " + cmd);

        int returnValue;

        try {

            Process p = Runtime.getRuntime().exec(cmd);

            InputStream in = p.getInputStream();
            int c;
            while ((c = in.read()) != -1) {
                System.out.print((char) c);
            }

            returnValue = p.waitFor();
            in.close();
        } catch (IOException ioX) {
            // not handled yet
            System.out.println("IOException occurred");
            return -1;
        } catch (InterruptedException intX) {
            // not handled yet
            System.out.println("InterruptedException occurred");
            return -1;
        }

        return 0;
    }

    /**
     * Starts the dicom receiving thread. Code allows listening of incoming
     * DICOM files via the network with the DICOM protocol.
     */
    private void startDicomReceiveThread() {
        final org.spectrim.lib.SwingWorker worker = new org.spectrim.lib.SwingWorker() {

            public Object construct() {
                System.out.println("Dicom network file receiving : thread started");
                dicomReceiveTreadCode();
                return null;
            }

            // runs in the event dispatching thread
            @Override
            public void finished() {
                // After calculation set enabled again
                System.out.println("Dicom network file receiving : thread finished");
            }
        };
        worker.start();
    }

    /**
     * Replaces all blanks by "\ " in order to be interpreted correctly by the
     * Windows operating system
     *
     * @param cmd
     * @return
     */
    private String generateProperWindowsCommand(String cmd) {
        String out = "";
        for (int i = 0; i < cmd.length(); i++) {
            if (cmd.substring(i, i + 1).compareTo("\\") == 0) {
                out += "\\" + "\\";
            } else {
                out += cmd.substring(i, i + 1);
            }
        }
        return out;
    }

    /**
     *
     * @param arg0
     */
    public void actionPerformed(ActionEvent e) {

        Object sender = e.getSource();

        if (sender == m_localDicomFilePath) {

        }

        if (sender == m_dicomFileAlterationListener) {
            if (e.getID() == DicomFileSystemAlterationListener.NOTIFY_DICOM_FILE_RENAMED_ID) {
                Object obj = m_dicomFileAlterationListener.getOutgoingFile();
                notifyActionListeners(obj, NOTIFY_DICOM_FILE_RENAMED, NOTIFY_DICOM_FILE_RENAMED_ID);
            }
        }

        if (sender == m_dicomFileAlterationListener) {
            if (e.getID() == DicomFileSystemAlterationListener.NOTIFY_DICOM_UNEXPECTED_EOF_ID) {
                Object obj = m_dicomFileAlterationListener.getOutgoingFile();
                notifyActionListeners(obj, NOTIFY_DICOM_FILE_UNEXPECTED_EOF, NOTIFY_DICOM_FILE_UNEXPECTED_EOF_ID);
            }
        }

    }

    /**
     * Adds an actionlistener to the list of listeners
     */
    public void addActionListener(ActionListener e) {
        m_actionListeners.add(e);
    }

    /**
     * Notifies actionEvent to all registered listeners
     */
    private void notifyActionListeners(Object obj, String actionCommand, int actionID) {
        Iterator it = m_actionListeners.iterator();
        while (it.hasNext()) {
            ((ActionListener) it.next()).actionPerformed(new ActionEvent(obj, actionID, actionCommand));
        }
    }

    /**
     * Writes the information to be logged in to the logging information
     *
     * @param fInfo
     */
    public void writeMRIRenamingLogFile(FileInfo fInfo) {
        try {
            DecimalFormat formatter = new DecimalFormat("###.00");

            String logFileName = m_homeDir + System.getProperty("file.separator") + "logMRDicomRenamedFiles.txt";

            File logFile = new File(logFileName);

            System.out.println("Writing to log file : " + logFileName);

            FileWriter fileWriter;

            if (logFile.exists() == false) {
                fileWriter = new FileWriter(logFileName);
            } else {
                fileWriter = new FileWriter(logFileName, true);
            }

            fileWriter.append(fInfo.getPatientName() + ",");
            fileWriter.append(fInfo.getSeriesNumber().trim() + ",");
            fileWriter.append(fInfo.getSeriesDescription() + ",");
            fileWriter.append(fInfo.getAcquisitionDate() + ",");
            fileWriter.append(fInfo.getAcquisitionTime() + ",");
            fileWriter.append(fInfo.getPixelWidth() + ",");
            fileWriter.append(fInfo.getPixelHeight() + ",");
            fileWriter.append(fInfo.getPixelDepth() + ",");
            fileWriter.append(fInfo.getSliceThickness() + ",");
            fileWriter.append(fInfo.getSeriesNumber().trim() + ",");
            fileWriter.append(fInfo.getTR() + ",");
            fileWriter.append(fInfo.getEchoTime() + ",");
            fileWriter.append(fInfo.getImageWidth() + ",");
            fileWriter.append(fInfo.getImageHeight() + ",");
            fileWriter.append(fInfo.getSeriesInstanceUID());
            fileWriter.write("\r\n");

            fileWriter.close();
        } catch (IOException e) {
            System.out.println("Error creating file : " + "~/logMRFileName.txt");
        }
    }

    /**
     * Writes the CT information to be logged in to the logging information
     *
     * @param fInfo
     */
    public void writeCTRenamingLogFile(FileInfo fInfo) {
        try {

            DecimalFormat formatter = new DecimalFormat("###.00");

            String logFileName = m_homeDir + System.getProperty("file.separator") + "logCTDicomRenamedFiles.txt";

            File logFile = new File(logFileName);

            System.out.println("Writing to log file : " + logFileName);

            FileWriter fileWriter;

            if (logFile.exists() == false) {
                fileWriter = new FileWriter(logFileName);
            } else {
                fileWriter = new FileWriter(logFileName, true);
            }

            fileWriter.append(fInfo.getSeriesInstanceUID() + ",");

            fileWriter.append(fInfo.getPatientName() + ",");
            fileWriter.append(fInfo.getSeriesNumber().trim() + ",");
            fileWriter.append(fInfo.getSeriesDescription() + ",");
            fileWriter.append(fInfo.getAcquisitionDate() + ",");
            fileWriter.append(fInfo.getAcquisitionTime() + ",");

            fileWriter.append(fInfo.get_CT_CTDIvol() + ",");
            fileWriter.append(fInfo.get_CTXRayTubeCurrent() + ",");
            fileWriter.append(fInfo.get_CT_KVP() + ",");
            fileWriter.append(fInfo.get_CT_ExposureTime() + ",");
            fileWriter.append(fInfo.get_CT_ExposureModulationType() + ",");

            fileWriter.append(fInfo.get_CT_DistanceSourceToDetector() + ",");
            fileWriter.append(fInfo.get_CT_DistanceSourceToPatient() + ",");
            fileWriter.append(fInfo.get_CT_DataCollectionDiameter() + ",");
            fileWriter.append(fInfo.get_CT_ReconstructionTargetCenterPatient() + ",");

            fileWriter.append(fInfo.get_CT_FilterType() + ",");
            fileWriter.append(fInfo.get_CT_ConvolutionKernel() + ",");

            fileWriter.append(fInfo.getPixelWidth() + ",");
            fileWriter.append(fInfo.getPixelHeight() + ",");
            fileWriter.append(fInfo.getPixelDepth() + ",");
            fileWriter.append(fInfo.getSliceThickness() + ",");

            fileWriter.append(fInfo.get_CT_ContrastBolusIngredientConcentration() + ",");
            fileWriter.append(fInfo.get_CT_ContrastFlowRate() + ",");
            fileWriter.append(fInfo.get_CT_ContrastBolusStartTime() + ",");
            fileWriter.append(fInfo.get_CT_ContrastBolusStopTime() + ",");
            fileWriter.append(fInfo.get_CT_ContrastBolusTotalDose() + ",");

            fileWriter.write("\r\n");

            fileWriter.close();
        } catch (IOException e) {
            System.out.println("Error creating file : " + "~/logCTFileName.txt");
        }
    }

    /**
     * Writes the CT information to be logged in to the logging information
     *
     * @param fInfo
     */
    public void writeFluoroscopyRenamingLogFile(FileInfo fInfo) {
        try {
            DecimalFormat formatter = new DecimalFormat("###.00");

            String logFileName = m_homeDir + System.getProperty("file.separator") + "logFluoroscopyDicomRenamedFiles.txt";

            File logFile = new File(logFileName);

            System.out.println("Writing to log file : " + logFileName);

            FileWriter fileWriter;

            if (logFile.exists() == false) {
                fileWriter = new FileWriter(logFileName);
            } else {
                fileWriter = new FileWriter(logFileName, true);
            }

            fileWriter.append(fInfo.getPatientName() + ",");
            fileWriter.append(fInfo.getSeriesNumber().trim() + ",");
            fileWriter.append(fInfo.getSeriesDescription() + ",");
            fileWriter.append(fInfo.getAcquisitionDate() + ",");
            fileWriter.append(fInfo.getAcquisitionTime() + ",");
            fileWriter.append(fInfo.getPixelWidth() + ",");
            fileWriter.append(fInfo.getPixelHeight() + ",");
            fileWriter.append(fInfo.getPixelDepth() + ",");
            fileWriter.append(fInfo.getSliceThickness() + ",");
            fileWriter.append(fInfo.getSeriesNumber().trim() + ",");
            fileWriter.append(fInfo.getTR() + ",");
            fileWriter.append(fInfo.getEchoTime() + ",");
            fileWriter.append(fInfo.getImageWidth() + ",");
            fileWriter.append(fInfo.getImageHeight() + ",");
            fileWriter.append(fInfo.getSeriesInstanceUID());
            fileWriter.write("\r\n");

            fileWriter.close();
        } catch (IOException e) {
            System.out.println("Error creating file : " + "~/logCTFileName.txt");
        }
    }

    // Attributes
    private JFrame m_parent;
    private String m_AETitle;
    private int m_portNumber;

    private String m_localDicomFilePath;

    private File m_localDicomFilePathAsFile;
    private boolean m_renameDirectory;

    private String m_renameToFilePath;

    private boolean m_deleteAfterRename;

    private FilesystemAlterationMonitor m_fileSystemAlterationMonitor;
    private DicomFileSystemAlterationListener m_dicomFileAlterationListener;
    private String[] m_dcm_filelist;
    private DualProgressBar m_dualProgressBar;

    private String m_patientName;
    private String m_examinationDate;
    private String m_examinationTime;
    private String m_seriesDescription;

    private int m_seriesNumber;
    private int m_instanceNumber;

    // this class should notify all action listeners whenever a DICOM file enters
    LinkedList<ActionListener> m_actionListeners = new LinkedList<ActionListener>();

    static final public String NOTIFY_DICOM_FILE_RENAMED = "DICOM_FILE_RENAMED";
    static final public int NOTIFY_DICOM_FILE_RENAMED_ID = 0;

    static final public String NOTIFY_DICOM_FILE_UNEXPECTED_EOF = "DICOM_FILE_UNEXPECTED_EOF";
    static final public int NOTIFY_DICOM_FILE_UNEXPECTED_EOF_ID = 1;

    private String m_lastSeriesInstanceUID = "0";
    private String m_newSeriesInstanceUID = "-1";
    private String m_lastSeriesDescription = "0";
    private String m_newSeriesDescription = "-1";

    private String m_homeDir = System.getProperty("user.home");
}
