/*
 * DicomNetworkListenerPropertiesPanel.java
 *
 * Created on 15. Juni 2009, 09:42
 */

package org.spectrim.dicom;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.PrintStream;
import java.util.Iterator;
import java.util.LinkedList;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
   
/**
 *
 * @author  slotboom
 */
public class DicomNetworkListenerPropertiesPanel extends javax.swing.JPanel implements ActionListener
{

    /** 
     * Creates new form DicomNetworkListenerPropertiesPanel
     * Panel allowing the user to set properties of the Dicom server listening
     * part of the local host
     */
    public DicomNetworkListenerPropertiesPanel( ) 
    {
        initComponents( );
        DicomNetworkNodeDefaults defs = DicomNetworkNodeDefaults.getInstance( );
        
        m_incomingDicomPathLabel.setText( defs.getProperty( DicomNetworkNodeDefaults.CURRENT_RENAME_INPUT_PATH ) );
        m_renameToLocalPathLabel.setText( defs.getProperty( DicomNetworkNodeDefaults.CURRENT_RENAME_TO_DICOM_DIRECTORY ) );
        
        m_AETitleTextField.setText( defs.getProperty( DicomNetworkNodeDefaults.LOCAL_HOST_DICOM_RECEIVE_AETITLE ) );
        m_localPortnumberTextField.setText( defs.getProperty( DicomNetworkNodeDefaults.LOCAL_HOST_DICOM_RECEIVE_PORT ) );

        m_outputStream = new JTextFieldAndErrorLogFileOutputStream( new ByteArrayOutputStream( ) );

        m_outputStream.setJTextAreaToWriteTo( m_textAreaWithPopUpMenu );
        m_outputStream.setLoggingToFile( true );

        m_printStream  = new PrintStream( m_outputStream );

        System.setOut(  m_printStream );
        System.setErr(  m_printStream );

    }

    /** 
     * Creates new form DicomNetworkListenerPropertiesPanel 
     */
    public DicomNetworkListenerPropertiesPanel( JFrame parent ) 
    {
        initComponents( );
        DicomNetworkNodeDefaults defs = DicomNetworkNodeDefaults.getInstance( );
        
        m_incomingDicomPathLabel.setText( defs.getProperty( DicomNetworkNodeDefaults.CURRENT_RENAME_INPUT_PATH ) );
        m_renameToLocalPathLabel.setText( defs.getProperty( DicomNetworkNodeDefaults.CURRENT_RENAME_TO_DICOM_DIRECTORY ) );
        File indir = new File( defs.getProperty( DicomNetworkNodeDefaults.CURRENT_RENAME_INPUT_PATH  ) );
        System.out.println( "Indir " +  indir.getAbsolutePath( ) );

        if ( indir.exists( ) == false )
        {
            System.out.println( "Indir does not exist!" +  indir.getAbsolutePath( ) );
            // Create the base dir
            File baseDir = new File( System.getProperty( "user.home" ) + System.getProperty( "file.separator" ) + "JMRUI-Siemens-DICOM" + System.getProperty( "file.separator" ) );
            if ( baseDir.exists( ) == false )
            {
                baseDir.mkdir( );
            }

            // appearently the file does not exist: create a default one in user.home
            m_incomingDicomPathLabel.setText( System.getProperty( "user.home" ) + System.getProperty( "file.separator" ) + "JMRUI-Siemens-DICOM" + System.getProperty("file.separator" ) + "IN" );

            File test = new File( System.getProperty( "user.home" ) + System.getProperty( "file.separator" ) + "JMRUI-Siemens-DICOM" + System.getProperty("file.separator" ) + "IN" );
            if ( test.exists( ) == false )
            {
                test.mkdir( );
            }
            defs.setProperty( DicomNetworkNodeDefaults.CURRENT_RENAME_INPUT_PATH, System.getProperty( "user.home" ) + System.getProperty( "file.separator" ) + "JMRUI-Siemens-DICOM" + System.getProperty("file.separator" ) + "IN" );
            DicomNetworkNodeDefaults.getInstance( ).save( );
        }
        else
        {
            m_incomingDicomPathLabel.setText( indir.getAbsolutePath( ) );
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////////////////

        File outdir = new File( defs.getProperty( DicomNetworkNodeDefaults.CURRENT_RENAME_TO_DICOM_DIRECTORY ) );
        System.out.println( "Outdir " +  outdir.getAbsolutePath( ) );

        if ( outdir.exists( ) == false )
        {
            System.out.println( "Outdir does not exist!" +  outdir.getAbsolutePath( ) );
            // Create the base dir : should always be possible
            File baseDir = new File( System.getProperty( "user.home" ) + System.getProperty( "file.separator" ) + "JMRUI-Siemens-DICOM" + System.getProperty( "file.separator" ) );
            if ( baseDir.exists( ) == false )
            {
                baseDir.mkdir( );
            }

            // appearently the file does not exist: create a default one in user.home
            m_renameToLocalPathLabel.setText( System.getProperty( "user.home" ) + System.getProperty( "file.separator" ) + "JMRUI-Siemens-DICOM" + System.getProperty( "file.separator" ) + "OUT" );
            File test = new File( System.getProperty( "user.home" ) + System.getProperty( "file.separator" ) + "JMRUI-Siemens-DICOM" + System.getProperty( "file.separator" ) + "OUT" );
            if ( test.exists( ) == false )
            {
                test.mkdir( );
            }

            defs.setProperty( DicomNetworkNodeDefaults.CURRENT_RENAME_TO_DICOM_DIRECTORY, System.getProperty( "user.home" ) + System.getProperty( "file.separator" ) + "JMRUI-Siemens-DICOM" + System.getProperty( "file.separator" ) + "OUT" );
            DicomNetworkNodeDefaults.getInstance( ).save( );
        }
        else
        {
            m_renameToLocalPathLabel.setText( outdir.getAbsolutePath( ) );
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////////////////

        File tempdir = new File( defs.getProperty( DicomNetworkNodeDefaults.CURRENT_INTERMEDIATE_DICOM_CREATION_DIRECTORY ) );
        System.out.println( "Tempdir " +  tempdir.getAbsolutePath( ) );

        if ( tempdir.exists( ) == false )
        {
            System.out.println( "Tempdir does not exist!" +  tempdir.getAbsolutePath( ) );
            File baseDir = new File( System.getProperty( "user.home" ) + System.getProperty( "file.separator" ) + "JMRUI-Siemens-DICOM" + System.getProperty( "file.separator" ) );
            if ( baseDir.exists( ) == false )
            {
                baseDir.mkdir( );
            }

            // appearently the file does not exist: create a default one in user.home
            m_temporaryDICOMDirectoryLabel.setText( System.getProperty( "user.home" ) + System.getProperty( "file.separator" ) + "JMRUI-Siemens-DICOM" + System.getProperty( "file.separator" ) + "TEMP" );

            File test = new File( System.getProperty( "user.home" ) + System.getProperty( "file.separator" ) + "JMRUI-Siemens-DICOM" + System.getProperty( "file.separator" ) + "TEMP" + System.getProperty( "file.separator" ) );

            if ( test.exists( ) == false )
            {
                test.mkdir( );
            }

            defs.setProperty( DicomNetworkNodeDefaults.CURRENT_INTERMEDIATE_DICOM_CREATION_DIRECTORY, System.getProperty( "user.home" ) + System.getProperty( "file.separator" ) + "JMRUI-Siemens-DICOM" + System.getProperty( "file.separator" ) + "TEMP" );
            DicomNetworkNodeDefaults.getInstance( ).save( );
        }
        else
        {
            m_temporaryDICOMDirectoryLabel.setText( tempdir.getAbsolutePath( ) );
        }

        // m_renameToLocalPathLabel.setText( defs.getProperty( DicomNetworkNodeDefaults.CURRENT_RENAME_TO_DICOM_DIRECTORY ) );


        m_AETitleTextField.setText( defs.getProperty( DicomNetworkNodeDefaults.LOCAL_HOST_DICOM_RECEIVE_AETITLE ) );
        m_localPortnumberTextField.setText( defs.getProperty( DicomNetworkNodeDefaults.LOCAL_HOST_DICOM_RECEIVE_PORT ) );
        m_parent = parent;

        m_outputStream = new JTextFieldAndErrorLogFileOutputStream( new ByteArrayOutputStream( ) );

        m_outputStream.setJTextAreaToWriteTo( m_textAreaWithPopUpMenu );
        m_outputStream.setLoggingToFile( true );

        m_printStream  = new PrintStream( m_outputStream );

        System.setOut( m_printStream );
        System.setErr( m_printStream );

        // Test popup menu:
        JPopupMenu popUp = new JPopupMenu( );

        JMenuItem clearTextField = new JMenuItem( "Clear" );

        m_textAreaWithPopUpMenu.add( popUp );
    }
    
    /**
     * Adds an actionlistener to the list of ActionListeners
     * @param aThis
     */
    void addActionListener( ActionListener listener  ) 
    {
        m_actionListeners.add( listener );
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings( "unchecked" )
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jProgressBar1 = new javax.swing.JProgressBar();
        jPanel1 = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        m_AETitleTextField = new javax.swing.JTextField();
        jLabel2 = new javax.swing.JLabel();
        m_localPortnumberTextField = new javax.swing.JTextField();
        m_setIncomingDicomFilePathButton = new javax.swing.JButton();
        m_renameIncomingDicomFilesCheckbox = new javax.swing.JCheckBox();
        m_incomingDicomPathLabel = new javax.swing.JLabel();
        m_setDirectoryToPutRenamedDicomsIntoButton = new javax.swing.JButton();
        m_deleteDicomFileAfterRenamingCheckBox = new javax.swing.JCheckBox();
        jLabel3 = new javax.swing.JLabel();
        jLabel4 = new javax.swing.JLabel();
        m_renameToLocalPathLabel = new javax.swing.JLabel();
        m_setTemporaryDirectoryButton = new javax.swing.JButton();
        jLabel5 = new javax.swing.JLabel();
        m_temporaryDICOMDirectoryLabel = new javax.swing.JLabel();
        jPanel2 = new javax.swing.JPanel();
        m_startDicomNetworkListenerButton = new javax.swing.JButton();
        m_textAreaScrollPane = new javax.swing.JScrollPane();
        m_textAreaWithPopUpMenu = new org.spectrim.lib.JTextAreaWithPopUpMenu();
        m_closeButton = new javax.swing.JButton();

        setPreferredSize(new java.awt.Dimension(950, 610));

        jPanel1.setBorder(javax.swing.BorderFactory.createTitledBorder("Dicom Listener Properties"));

        jLabel1.setText("Local AE-title");

        jLabel2.setText("Local port number");

        m_setIncomingDicomFilePathButton.setText("Set DICOM incoming path");
        m_setIncomingDicomFilePathButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_setIncomingDicomFilePathButtonActionPerformed(evt);
            }
        });

        m_renameIncomingDicomFilesCheckbox.setSelected(true);
        m_renameIncomingDicomFilesCheckbox.setText("Rename incoming Dicom file(s)");
        m_renameIncomingDicomFilesCheckbox.setEnabled(false);

        m_incomingDicomPathLabel.setText(">> incoming dicom files  path <<");

        m_setDirectoryToPutRenamedDicomsIntoButton.setText("Set DICOM renaming path");
        m_setDirectoryToPutRenamedDicomsIntoButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_setDirectoryToPutRenamedDicomsIntoButtonActionPerformed(evt);
            }
        });

        m_deleteDicomFileAfterRenamingCheckBox.setSelected(true);
        m_deleteDicomFileAfterRenamingCheckBox.setText("Delete incominig DICOM file(s) after renaming");
        m_deleteDicomFileAfterRenamingCheckBox.setEnabled(false);

        jLabel3.setText("Local dicom path (incoming)");

        jLabel4.setText("Local dicom path (renamed)");

        m_renameToLocalPathLabel.setText(">> dicom file path (renaming) <<");

        m_setTemporaryDirectoryButton.setText("Set DICOM temporary path");
        m_setTemporaryDirectoryButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_setTemporaryDirectoryButtonActionPerformed(evt);
            }
        });

        jLabel5.setText("Local dicom temp path");

        m_temporaryDICOMDirectoryLabel.setText(">> dicom file path (renaming) <<");

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel1Layout.createSequentialGroup()
                        .addContainerGap()
                        .addComponent(m_renameIncomingDicomFilesCheckbox, javax.swing.GroupLayout.DEFAULT_SIZE, 524, Short.MAX_VALUE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(m_deleteDicomFileAfterRenamingCheckBox))
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addComponent(m_setIncomingDicomFilePathButton, javax.swing.GroupLayout.DEFAULT_SIZE, 288, Short.MAX_VALUE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addComponent(m_setDirectoryToPutRenamedDicomsIntoButton, javax.swing.GroupLayout.PREFERRED_SIZE, 292, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(m_setTemporaryDirectoryButton, javax.swing.GroupLayout.PREFERRED_SIZE, 320, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jLabel1)
                            .addComponent(jLabel2)
                            .addComponent(jLabel3)
                            .addComponent(jLabel4)
                            .addComponent(jLabel5))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(m_temporaryDICOMDirectoryLabel, javax.swing.GroupLayout.DEFAULT_SIZE, 687, Short.MAX_VALUE)
                            .addComponent(m_localPortnumberTextField, javax.swing.GroupLayout.DEFAULT_SIZE, 687, Short.MAX_VALUE)
                            .addComponent(m_AETitleTextField, javax.swing.GroupLayout.DEFAULT_SIZE, 687, Short.MAX_VALUE)
                            .addComponent(m_incomingDicomPathLabel, javax.swing.GroupLayout.DEFAULT_SIZE, 687, Short.MAX_VALUE)
                            .addComponent(m_renameToLocalPathLabel, javax.swing.GroupLayout.DEFAULT_SIZE, 687, Short.MAX_VALUE))))
                .addContainerGap())
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(jLabel1)
                    .addComponent(m_AETitleTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_localPortnumberTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel2))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel3, javax.swing.GroupLayout.PREFERRED_SIZE, 17, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(m_incomingDicomPathLabel))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(m_renameToLocalPathLabel)
                    .addComponent(jLabel4))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_temporaryDICOMDirectoryLabel)
                    .addComponent(jLabel5))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_renameIncomingDicomFilesCheckbox)
                    .addComponent(m_deleteDicomFileAfterRenamingCheckBox))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_setIncomingDicomFilePathButton)
                    .addComponent(m_setTemporaryDirectoryButton)
                    .addComponent(m_setDirectoryToPutRenamedDicomsIntoButton)))
        );

        jPanel2.setBorder(javax.swing.BorderFactory.createTitledBorder("Incoming Dicom file  (network)"));

        m_startDicomNetworkListenerButton.setText("Start Dicom network listener..");
        m_startDicomNetworkListenerButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_startDicomNetworkListenerButtonActionPerformed(evt);
            }
        });

        m_textAreaScrollPane.setHorizontalScrollBarPolicy(javax.swing.ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
        m_textAreaScrollPane.setVerticalScrollBarPolicy(javax.swing.ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);

        m_textAreaWithPopUpMenu.setColumns(20);
        m_textAreaWithPopUpMenu.setRows(5);
        m_textAreaScrollPane.setViewportView(m_textAreaWithPopUpMenu);

        javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(m_startDicomNetworkListenerButton)
                .addContainerGap(674, Short.MAX_VALUE))
            .addComponent(m_textAreaScrollPane, javax.swing.GroupLayout.DEFAULT_SIZE, 930, Short.MAX_VALUE)
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addComponent(m_startDicomNetworkListenerButton)
                .addGap(18, 18, 18)
                .addComponent(m_textAreaScrollPane, javax.swing.GroupLayout.PREFERRED_SIZE, 265, javax.swing.GroupLayout.PREFERRED_SIZE))
        );

        m_closeButton.setText("Close");
        m_closeButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_closeButtonActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(jPanel2, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(jPanel1, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(12, Short.MAX_VALUE))
            .addGroup(layout.createSequentialGroup()
                .addGap(252, 252, 252)
                .addComponent(m_closeButton, javax.swing.GroupLayout.PREFERRED_SIZE, 423, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(279, Short.MAX_VALUE))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addComponent(jPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jPanel2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(m_closeButton)
                .addContainerGap())
        );
    }// </editor-fold>//GEN-END:initComponents
/**
 * Called whenever the m_setIncomingDicomFilePathButton is pressed
 * @param evt
 */
private void m_setIncomingDicomFilePathButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_setIncomingDicomFilePathButtonActionPerformed

        
        String currPath = DicomNetworkNodeDefaults.getInstance( ).getProperty( DicomNetworkNodeDefaults.CURRENT_RENAME_INPUT_PATH );
        File incomingDicomFilesPath;

        if ( ( currPath != null ) &&  ( ( new File( currPath ) ).exists( )  == true ) )
        {
            incomingDicomFilesPath =  getDicomDirectory( new File( currPath ) );
        }
        else
        {
            incomingDicomFilesPath =  getDicomDirectory( null );
        }

        m_incomingDicomPathLabel.setText( incomingDicomFilesPath.getAbsolutePath( ) );
        DicomNetworkNodeDefaults.getInstance( ).setProperty( DicomNetworkNodeDefaults.CURRENT_RENAME_INPUT_PATH,  incomingDicomFilesPath.getAbsolutePath( ) );
        DicomNetworkNodeDefaults.getInstance( ).save( );
}//GEN-LAST:event_m_setIncomingDicomFilePathButtonActionPerformed

private void m_closeButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_closeButtonActionPerformed
        notifyActionListeners( this, HIDE_FRAME_STRING, HIDE_FRAME_ID );
}//GEN-LAST:event_m_closeButtonActionPerformed

private void m_setDirectoryToPutRenamedDicomsIntoButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_setDirectoryToPutRenamedDicomsIntoButtonActionPerformed

        String currPath = DicomNetworkNodeDefaults.getInstance( ).getProperty( DicomNetworkNodeDefaults.CURRENT_RENAME_TO_DICOM_DIRECTORY );

        File renameDicomFilesToPath;

        if ( ( currPath != null ) && ( new File( currPath ) ).exists( ) == true )
        {
            renameDicomFilesToPath =  getDicomDirectory( new File( currPath ) );
        }
        else
        {
            renameDicomFilesToPath =  getDicomDirectory( null );
        }

        m_renameToLocalPathLabel.setText( renameDicomFilesToPath.getAbsolutePath( ) );
        DicomNetworkNodeDefaults.getInstance( ).setProperty( DicomNetworkNodeDefaults.CURRENT_RENAME_TO_DICOM_DIRECTORY,  renameDicomFilesToPath.getAbsolutePath( ) );
        DicomNetworkNodeDefaults.getInstance( ).save( );
}//GEN-LAST:event_m_setDirectoryToPutRenamedDicomsIntoButtonActionPerformed

/**
 *
 * User wants the Dicom network listener to start.
 *
 * @param evt
 */
private void m_startDicomNetworkListenerButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_startDicomNetworkListenerButtonActionPerformed
        

        System.out.println( "Started: m_startDicomNetworkListenerButtonActionPerformed" );
        String in_path = DicomNetworkNodeDefaults.getInstance( ).getProperty( DicomNetworkNodeDefaults.CURRENT_RENAME_INPUT_PATH );
        String rename_path = DicomNetworkNodeDefaults.getInstance( ).getProperty( DicomNetworkNodeDefaults.CURRENT_RENAME_TO_DICOM_DIRECTORY );

        System.out.println( "Started: in path and out path defined" );
        m_dicomNetworkFiletransferHandler = new DicomNetworkFileTransferAndDicomFileRenamingHandler( m_parent, "DICOMSPECTRO", 104, in_path,
                                                                                                     true, rename_path, true );

        m_dicomNetworkFiletransferHandler.addActionListener( this );

        // start listening!
        m_dicomNetworkFiletransferHandler.startDicomFileRenamerService( );
}//GEN-LAST:event_m_startDicomNetworkListenerButtonActionPerformed

private void m_setTemporaryDirectoryButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_setTemporaryDirectoryButtonActionPerformed
   String currPath = DicomNetworkNodeDefaults.getInstance( ).getProperty( DicomNetworkNodeDefaults.CURRENT_INTERMEDIATE_DICOM_CREATION_DIRECTORY );
    File tempDicomFilesPath;

    if ( ( currPath != null ) &&  ( ( new File( currPath ) ).exists( )  == true ) )
    {
        tempDicomFilesPath =  getDicomDirectory( new File( currPath ) );
    }
    else
    {
        tempDicomFilesPath =  getDicomDirectory( null );
    }

    m_temporaryDICOMDirectoryLabel.setText( tempDicomFilesPath.getAbsolutePath( ) );
    DicomNetworkNodeDefaults.getInstance( ).setProperty( DicomNetworkNodeDefaults.CURRENT_INTERMEDIATE_DICOM_CREATION_DIRECTORY,  tempDicomFilesPath.getAbsolutePath( ) );
    DicomNetworkNodeDefaults.getInstance( ).save( );
}//GEN-LAST:event_m_setTemporaryDirectoryButtonActionPerformed


    /**
     * Notifies actionEvent to all registered listeners
     */
    private void notifyActionListeners( Object obj, String eventString, int eventType )
    {
        Iterator it = m_actionListeners.iterator( );
        while( it.hasNext( ) )
        {
            ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( obj, eventType, eventString ) );
        }
    }
 

    /**
     * Returns the dicomdirectory selected by the user
     */
    private File getDicomDirectory( File startLocation )
    {
        JFileChooser fileChooser = new JFileChooser( startLocation );
        try
        {
            // Create a File object containing the canonical path of the
            // desired directory
            File f = new File( new File( "." ).getCanonicalPath( ) );
            // Set the current directory
            fileChooser.setCurrentDirectory( f );
        }
        catch ( IOException e )
        {
            // not implemented
        }

        fileChooser.setFileSelectionMode( JFileChooser.DIRECTORIES_ONLY );

        int result = fileChooser.showOpenDialog( this );

        File selected_dir = new File( "" );

        switch ( result )
        {
            case JFileChooser.APPROVE_OPTION:   // Approve Open button was pressed
                                                selected_dir = fileChooser.getSelectedFile( );
                                                break;
           case JFileChooser.CANCEL_OPTION:     // Cancel or close dialog was selected
                                                break;
           case JFileChooser.ERROR_OPTION:      // the selected process did not terminate properly
                                                break;
        }

        return selected_dir;
    }

    /**
     *
     * Is called whenever a registered component raises an ActionEvent
     *
     * @param arg0
     */
    public void actionPerformed( ActionEvent e )
    {
        Object sender = e.getSource( );

        if ( sender instanceof File )
        {
            if ( e.getID( ) == DicomNetworkFileTransferAndDicomFileRenamingHandler.NOTIFY_DICOM_FILE_RENAMED_ID )
            {
                notifyActionListeners( e.getSource( ), NOTIFY_DICOM_FILE_RENAMED, NOTIFY_DICOM_FILE_RENAMED_ID );
            }
            
            if ( e.getID( ) == DicomNetworkFileTransferAndDicomFileRenamingHandler.NOTIFY_DICOM_FILE_UNEXPECTED_EOF_ID )
            {
                notifyActionListeners( e.getSource( ), NOTIFY_DICOM_FILE_UNEXPECTED_EOF, NOTIFY_DICOM_FILE_UNEXPECTED_EOF_ID );
            }
        }
    }



    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JProgressBar jProgressBar1;
    private javax.swing.JTextField m_AETitleTextField;
    private javax.swing.JButton m_closeButton;
    private javax.swing.JCheckBox m_deleteDicomFileAfterRenamingCheckBox;
    private javax.swing.JLabel m_incomingDicomPathLabel;
    private javax.swing.JTextField m_localPortnumberTextField;
    private javax.swing.JCheckBox m_renameIncomingDicomFilesCheckbox;
    private javax.swing.JLabel m_renameToLocalPathLabel;
    private javax.swing.JButton m_setDirectoryToPutRenamedDicomsIntoButton;
    private javax.swing.JButton m_setIncomingDicomFilePathButton;
    private javax.swing.JButton m_setTemporaryDirectoryButton;
    private javax.swing.JButton m_startDicomNetworkListenerButton;
    private javax.swing.JLabel m_temporaryDICOMDirectoryLabel;
    private javax.swing.JScrollPane m_textAreaScrollPane;
    private org.spectrim.lib.JTextAreaWithPopUpMenu m_textAreaWithPopUpMenu;
    // End of variables declaration//GEN-END:variables

    private LinkedList<ActionListener> m_actionListeners = new LinkedList<ActionListener>( );

    private DicomNetworkFileTransferAndDicomFileRenamingHandler m_dicomNetworkFiletransferHandler;

    static final public String HIDE_FRAME_STRING            = "HIDE_FRAME";
    static final public String NOTIFY_DICOM_FILE_RENAMED    = "DICOM_FILE_RENAMED";

    
    
    static final public String  NOTIFY_DICOM_FILE_UNEXPECTED_EOF = "UNEXPECTED_EOF";
    public static final int     NOTIFY_DICOM_FILE_UNEXPECTED_EOF_ID = 2;
    
    
    public final static int HIDE_FRAME_ID                   = 0;
    public static final int NOTIFY_DICOM_FILE_RENAMED_ID    = 1;

    private JFrame m_parent;
    private PrintStream m_printStream;
    private JTextFieldAndErrorLogFileOutputStream m_outputStream;
    
}