/**
 * File: $Id: MultiDicomSliceView.java,v 1.2 2006/11/09 10:02:27 hans Exp $
 * History
 * Date: 17.03.2003, 11:21  Authors: Hans Slotboom  
 *
 * Changes:
 *      22.04.2003 - Created from MultiImageViewer class
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

package org.spectrim.dicom;


import org.spectrim.imagegraphicobjects.Plane3D;
import java.awt.Dimension;
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.print.PrinterJob;
import java.awt.Polygon;
import java.awt.Rectangle;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.BorderFactory;
import javax.swing.JFileChooser;

import java.text.NumberFormat;
import java.text.DecimalFormat;

import java.util.Vector;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Enumeration;
import java.util.LinkedHashMap;

import java.awt.Point;

import java.awt.event.MouseListener;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;

import java.awt.print.Book;
import java.awt.print.Printable;
import java.awt.print.PageFormat;
import java.awt.print.PrinterException;

import org.spectrim.imagegraphicobjects.PolygonEvent;
import org.spectrim.imagegraphicobjects.PolygonListener;
import org.spectrim.imagegraphicobjects.PolygonUtils;
import org.spectrim.imagegraphicobjects.PolygonParameterDisplayer;
import org.spectrim.imagegraphicobjects.Point3D;
import org.spectrim.imagegraphicobjects.RegioPolygon;

import org.spectrim.lib.FileUtils;
import org.spectrim.lib.Globals;
import org.spectrim.lib.Dialogs;
import org.spectrim.lib.MessageBox;
import org.spectrim.lib.Tiff;
import org.spectrim.lib.RegFile;
import org.spectrim.lib.MemTools;
import org.spectrim.lib.QuestionDialog;
import org.spectrim.lib.ImageUtils;


import org.spectrim.imageprocessing.MomentsAboutTheMean;
import org.spectrim.mrspectrumfitting.MRFittingDefaults;
import org.spectrim.imageprocessing.SCANalyzeDefaults;
import org.spectrim.signalprocessing.SignalTools;

import java.awt.image.BufferedImage;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import javax.imageio.ImageIO;
import javax.swing.JComponent;
import org.spectrim.lib.CustomFileDialog;

/**
 * Created from MultiImageViewer - 23-04-2003 - code cleanup reasons.
 *
 * @author  slotboom
 */
public class MultiDicomSliceView extends JPanel implements ActionListener, PolygonListener, MouseListener, Printable
{
    
    /**
     * Default constructor.
     */
    public MultiDicomSliceView( )
    {
        m_parent = null;
        m_defaultPath = null;
        m_fileSeparator = System.getProperty( "file.separator" );
        
        setLayout( null );
        setPreferredSize( new Dimension( 517, 517 ) );
    }

    /**
     * 
     * Sets the parent of this frame.
     * 
     * @param parent
     */
    public void setParent( JFrame parent )
    {
        m_parent = parent;
    }

    /** 
     * Creates a new instance of MultiDicomSliceView. 
     * @param parent
     * @param defaultPath
     */
    public MultiDicomSliceView( JFrame parent, String defaultPath )
    {
        m_parent = parent;
        m_defaultPath = defaultPath;
        m_fileSeparator = System.getProperty( "file.separator" );
        setLayout( null );
        setPreferredSize( new Dimension( 517, 517 ) );
    }

    /**
     * Sets the measure distance boolean object.
     * @param measureDistanceNow
     */
    public void setMeasureDistance( boolean measureDistanceNow )
    {
        
        if ( m_currentView != null )
        {
            m_currentView.setMeasureDistanceMode( measureDistanceNow );
        }
    }

    /**
     * Sets the color table of all DicomSliceCanvasPolyDraw-typed views.
     * @param colorTableType
     */
    public void setGlobalColorTable( int colorTableType )
    {
        for ( int k = 0; k < getNumberOfSliceViews( ); k++ )
        {
             DicomSliceCanvasPolyDraw c = getSlice( k );
             c.setColormapType( colorTableType );
             c.update( true );
        }
    }

    /**
     * Shows the scales in all images.
     * @param showScales
     */
    public void showImagePixelScales( boolean showScales )
    {
        for ( int k = 0; k < getNumberOfSliceViews( ); k++ )
        {
             DicomSliceCanvasPolyDraw c = getSlice( k );
             c.scaleSetVisible( showScales );
             c.update( true );
        }
    }

    /**
     * Enables synchronized paging.
     */
    public void enableSyncedDisplayMode( )
    {
        m_synchronizeViews = true;
    }

    /**
     * Disables synchronized paging.
     */
    public void disableSyncedDisplayMode( )
    {
        m_synchronizeViews = false;
    }

    /**
     * Checks whether all currently displayed images belong to
     * 1- the same patient
     * 2- the same examination
     * @return true if displayed images all belong to the same 
     *         patient and same examination
     */
    public int checkValidityOfDicomExport( )
    {
        // No images are displayed: the image makes no sense since it can
        // not be associated to a patient on the MR console
        if ( m_sliceViews.isEmpty( ) )
        {
            return 0;
        }
        // Only one image is displayed, no conflict between different patients
        // and studies can occur.
        if ( m_sliceViews.size( ) == 1 )
        {
            return 1;
        }
        if ( m_sliceViews.size( ) > 1 )
        {
            DicomSliceCanvasPolyDraw canvas0 = m_sliceViews.get( 0 );
            DicomSliceCanvasPolyDraw canvasi;
            
            String patName = canvas0.getDicomFileInfo( ).getPatientName( );
            
            // test whether all images are from the same patient
            for ( int i = 1; i < m_sliceViews.size( ); i++ )
            {
                canvasi = m_sliceViews.get( i );
                if ( patName.compareTo( canvasi.getDicomFileInfo( ).getPatientName( ) ) != 0 )
                {
                    return 2;
                }
            }
            
            String patBirthDate = canvas0.getDicomFileInfo( ).getPatientBirthDate( );
            
            // test whether all images are from the same patient
            for ( int i = 1; i < m_sliceViews.size( ); i++ )
            {
                canvasi = m_sliceViews.get( i );
                if ( patBirthDate.compareTo( canvasi.getDicomFileInfo( ).getPatientBirthDate( ) ) != 0 )
                {
                    return 3;
                }
            }
            
            // also check the study instance UID
            String patStudyUID = canvas0.getDicomFileInfo( ).getStudyInstanceUID( );
            
            // test whether all images are from the same patient
            for ( int i = 1; i < m_sliceViews.size( ); i++ )
            {
                canvasi = m_sliceViews.get( i );
                if ( patStudyUID.compareTo( canvasi.getDicomFileInfo( ).getStudyInstanceUID( ) ) != 0 )
                {
                    return 4;
                }
            }
        }
        
        return 1;
    }
    
    /**
     * Handles the layout of the MultiDicomSliceView.
     */    
    public void setupLayout( ) 
    {
        this.removeAll( );
        
        int gap = 5;	// gap between images
        float n = (float)m_sliceViews.size( );
        if ( n == 0 ) return;
	
        int cols = (int)Math.ceil( Math.sqrt( n ) );
        int rows = (int)Math.ceil( n / (float)cols );
	
        setLayout( null );
        
        setSize(  (int)( m_globalZoomFactor * cols * ( m_maxImageWidth + 5 ) ), (int)( m_globalZoomFactor * rows * ( m_maxImageHeight + 5 ) ) );
        setPreferredSize( new Dimension( (int)(m_globalZoomFactor * cols * ( m_maxImageWidth + 5 ) ) , 
                          (int)( m_globalZoomFactor * rows * ( m_maxImageHeight + 5 ) ) ) );
        
        
        Iterator it = m_sliceViews.iterator( );
        
        int canvas_cntr = 0;
        
        while ( it.hasNext( ) )
        {
            DicomSliceCanvasPolyDraw canvas = (DicomSliceCanvasPolyDraw)it.next( );
            
            canvas.setRedrawImage( true );
            
            this.add( canvas );
            Point p = getTopLeftPosition( canvas_cntr );
            canvas.setLocation( p );
            
            canvas.setZoom( m_globalZoomFactor );
            
            canvas_cntr++;
        }
         
        doLayout( ); 
    }

    
    /**
     * Returns the top, left coordinate of the DicomSliceCanvasPolydraw.
     * @param viewNumber
     * @return 
     */    
    public Point getTopLeftPosition( int viewNumber ) 
    {
        int gap = 5;	/* gap between images */
        float n = (float)m_sliceViews.size( );
        if ( n == 0 ) return new Point( 0, 0 );
        
        int cols = (int)Math.ceil( Math.sqrt( n ) );
        int rows = (int)Math.ceil( n / (float)cols );
        
        int cntr = 0;
        
        for ( int i = 0; i < cols; i++ )
        {
            for ( int j = 0; j < rows; j++ )
            {
                if ( cntr == viewNumber )
                {
                    int leftPosition = (int)( i * ( m_maxImageWidth + 5 ) * m_globalZoomFactor );
                    int topPosition =  (int)( j * ( m_maxImageHeight + 5 ) * m_globalZoomFactor );
                    return new Point(  leftPosition, topPosition );
                }
                cntr++;
            }
        }
        
        return null;
    }

    
    /**
     * Returns the top, left coordinate of the DicomSliceCanvasPolydraw.
     * @param viewNumber
     * @return 
     */    
    public Point getTopLeftPositionRotated( int viewNumber ) 
    {
        int gap = 5;	/* gap between images */
        float n = (float)m_sliceViews.size( );
        if ( n == 0 ) return new Point( 0, 0 );
        
        int cols = (int)Math.ceil( Math.sqrt( n ) );
        int rows = (int)Math.ceil( n / (float)cols );
        
        int cntr = 0;
        
        for ( int i = 0; i < cols; i++ )
        {
            for ( int j = 0; j < rows; j++ )
            {
                if ( cntr == viewNumber )
                {
                    int topPosition = (int)( i * ( m_maxImageWidth + 5 ) * m_globalZoomFactor );
                    int leftPosition =  (int)( j * ( m_maxImageHeight + 5 ) * m_globalZoomFactor );
                    return new Point(  topPosition, leftPosition );
                }
                cntr++;
            }
        }
        
        return null;
    }

    
    
    /**
     * Set the path from which files are loaded.
     * @param loadPath
     */
    public void setLoadPath( String loadPath )
    {
        m_loadPath = loadPath;
    }
    
    /**
     * Adds a new DicomSliceCanvasPolyDraw to the vector of already 
     * loaded views.
     * @param dicomImageSeries
     * @param d
     * @param title
     */
    public void addNewView( DicomImageSeriesData dicomImageSeries, 
                            Dimension d, String title ) 
    {
        
        DicomSliceCanvasPolyDraw c = new DicomSliceCanvasPolyDraw( m_parent );
        c.setTitle( title );
        c.addPolygonListener( this );
        c.addMouseListener( this );
        c.setRegioReference( true );
        c.setDicomImages( dicomImageSeries );
        c.enableImagePaging( true );
        c.enableZoomPopupMenuItem( false );
        
        m_sliceViews.addElement( c );
         
        //www this.add( c );
        setCurrentView( c );
        
        c.displayDicomImage( 0 );
        
        
        if ( m_maxImageHeight < d.height )
        {
            m_maxImageHeight = d.height;
        }
        
        if ( m_maxImageWidth < d.width )
        {
            m_maxImageWidth = d.width;
        }
        
        
        //www c.setZoom( m_globalZoomFactor );
        
        c.setPreferredSize( new Dimension( (int)( m_globalZoomFactor * d.width ), (int)( m_globalZoomFactor * d.height ) ) );
        c.setMinimumSize( new Dimension( (int)( m_globalZoomFactor * d.width ),  (int)( m_globalZoomFactor * d.height ) ) );
        c.setMaximumSize( new Dimension( (int)( m_globalZoomFactor * d.width ), (int)( m_globalZoomFactor * d.height ) ) );
  
        // c.setDicomFileInfo( finfo );
        c.addActionListener( this );
        
        int max = c.getMaxRawDataValue( );
        int min = c.getMinRawDataValue( );
        
        if ( max > m_maxPixelValueOfAllLoadedImages )
        {
            m_maxPixelValueOfAllLoadedImages = max;
        }
        
        if ( min < m_minPixelValueOfAllLoadedImages )
        {
            m_minPixelValueOfAllLoadedImages = min;
        }
        
        c.displayDicomImage( dicomImageSeries.getNumberOfImages( )/2 );
        
    }


    /**
     * Adds a new DicomSliceCanvasPolyDraw to the vector of already 
     * loaded views.
     * @param dicomImageSeries
     * @param d
     * @param title
     * @param displayImageNumber
     */
    public void addNewView( DicomImageSeriesData dicomImageSeries, 
                            Dimension d, String title, int displayImageNumber )
    {

        DicomSliceCanvasPolyDraw c = new DicomSliceCanvasPolyDraw( m_parent );
        c.setTitle( title );
        c.addPolygonListener( this );
        c.addMouseListener( this );
        c.setRegioReference( true );
        c.setDicomImages( dicomImageSeries );
        c.enableImagePaging( true );
        c.enableZoomPopupMenuItem( false );

        m_sliceViews.addElement( c );

        //www this.add( c );
        setCurrentView( c );

        //c.displayDicomImage( 0 );


        if ( m_maxImageHeight < d.height )
        {
            m_maxImageHeight = d.height;
        }

        if ( m_maxImageWidth < d.width )
        {
            m_maxImageWidth = d.width;
        }


        //www c.setZoom( m_globalZoomFactor );

        c.setPreferredSize( new Dimension( (int)( m_globalZoomFactor * d.width ), (int)( m_globalZoomFactor * d.height ) ) );
        c.setMinimumSize( new Dimension( (int)( m_globalZoomFactor * d.width ),  (int)( m_globalZoomFactor * d.height ) ) );
        c.setMaximumSize( new Dimension( (int)( m_globalZoomFactor * d.width ), (int)( m_globalZoomFactor * d.height ) ) );

        // c.setDicomFileInfo( finfo );
        c.addActionListener( this );

        int max = c.getMaxRawDataValue( );
        int min = c.getMinRawDataValue( );

        if ( max > m_maxPixelValueOfAllLoadedImages )
        {
            m_maxPixelValueOfAllLoadedImages = max;
        }

        if ( min < m_minPixelValueOfAllLoadedImages )
        {
            m_minPixelValueOfAllLoadedImages = min;
        }

        c.displayDicomImage( displayImageNumber );

    }


    /**
     * This routine creates a detailed report of all views of the this 
     * multi view
     *
     * It:
     * ===
     *
     * - asks for a file name
     * - iterates over views
     * - iterates over all images of the stack
     * - iterates over all regions per image.
     *
     */
    public void createDetailMAMReport( )
    {

        DicomSliceCanvasPolyDraw currentBackup = this.m_currentView;

        Iterator it = m_sliceViews.iterator( );
        DecimalFormat formatter = new DecimalFormat( "###.00" );
        boolean createdHeader = false;

        while ( it.hasNext( ) )
        {
            m_currentView = (DicomSliceCanvasPolyDraw)it.next( );
            saveRegions( );
        }

    }
    
    /**
     * Adds a new DicomSliceCanvasPolyDraw to the vector of already 
     * loaded views.
     * @param data
     * @param d
     * @param title
     * @param finfo
     * @param fSiemensInfo
     */
    public void addNewView( int[] data, Dimension d, String title,
                            FileInfo finfo, LinkedHashMap fSiemensInfo )
    {
        DicomSliceCanvasPolyDraw c = new DicomSliceCanvasPolyDraw( m_parent );
        c.setTitle( title );
        c.addPolygonListener( this );
        c.addMouseListener( this );
        c.setRegioReference( true );

        m_sliceViews.addElement( c );
        
        setCurrentView( c );
        setImage( c, data, d.width, d.height );
        
        if ( m_maxImageHeight < d.height )
        {
            m_maxImageHeight = d.height;
        }
        
        if ( m_maxImageWidth < d.width )
        {
            m_maxImageWidth = d.width;
        }
        
        if ( finfo != null )
        {
            c.setDicomFileInfo( finfo );
        }
        else
        {
            // disable the finfo JMenuItem
        }
        
        if ( fSiemensInfo != null )
        {
            c.setDicomSpecificFileInfo( fSiemensInfo );
        }
        else
        {
            // disable the siemens specific JMenuItem
        }
        
        c.addActionListener( this );
        
        int max = c.getMaxRawDataValue( );
        int min = c.getMinRawDataValue( );
        
        if ( max > m_maxPixelValueOfAllLoadedImages )
        {
            m_maxPixelValueOfAllLoadedImages = max;
        }
        
        if ( min < m_minPixelValueOfAllLoadedImages )
        {
            m_minPixelValueOfAllLoadedImages = min;
        }   
    }


    /**
     * Adds a new DicomSliceCanvasPolyDraw to the vector of already 
     * loaded views.
     * @param slice_canvas
     */
    public void addNewView( DicomSliceCanvasPolyDraw slice_canvas )
    {

        // The whole DicomSliceCanvasPolyDraw object has to be deep-copied
        DicomSliceCanvasPolyDraw c = new DicomSliceCanvasPolyDraw( m_parent );
        c.setTitle( slice_canvas.getTitle( ) );
        c.addPolygonListener( this );
        c.addMouseListener( this );
        c.setRegioReference( true );

        // add to the vector of views
        m_sliceViews.addElement( c );
        // add to the relative image panel

        setCurrentView( c );
        setImage( c, slice_canvas.getRawData( ), slice_canvas.getImageWidth( ), slice_canvas.getImageHeight( ) );
        c.setZoom( m_globalZoomFactor );

        c.addActionListener( this );

        int max = c.getMaxRawDataValue( );
        int min = c.getMinRawDataValue( );

        if ( max > m_maxPixelValueOfAllLoadedImages )
        {
            m_maxPixelValueOfAllLoadedImages = max;
        }

        if ( min < m_minPixelValueOfAllLoadedImages )
        {
            m_minPixelValueOfAllLoadedImages = min;
        }
    }

    /**
     * Adds a new DicomSliceCanvasPolyDraw to the vector of already 
     * loaded views
     * @param slice_canvas
     * @param regPol
     */
    public void addNewRelativeView( DicomSliceCanvasPolyDraw slice_canvas, 
                                    RegioPolygon regPol ) 
    {
        m_relativeViewsAreShown = true;
        
        // The whole DicomSliceCanvasPolyDraw object has to be deep-copied
        DicomSliceCanvasPolyDraw c = new DicomSliceCanvasPolyDraw( m_parent );
        c.setTitle( slice_canvas.getTitle( ) );
        c.addPolygonListener( this );
        c.addMouseListener( this );
        c.setRegioReference( true );
        
        // add to the vector of views
        m_sliceViews.addElement( c );
        // add to the relative image panel
	
        setCurrentView( c );
        setImage( c, slice_canvas.getRawData( ), slice_canvas.getImageWidth( ), slice_canvas.getImageHeight( ) );
        c.setZoom( m_globalZoomFactor );
        if ( regPol != null )
        {
            c.addPolygon( regPol );
        }
        
        c.calculateRelativeImageRepresentation( );
        setImage( c, c.getRawData( ), slice_canvas.getImageWidth( ), slice_canvas.getImageHeight( ) );
        c.addActionListener( this );
        
        int max = c.getMaxRawDataValue( );
        int min = c.getMinRawDataValue( );
        
        if ( max > m_maxPixelValueOfAllLoadedImages )
        {
            m_maxPixelValueOfAllLoadedImages = max;
        }
        
        if ( min < m_minPixelValueOfAllLoadedImages )
        {
            m_minPixelValueOfAllLoadedImages = min;
        }
    }
    
    /**
     * Sets the current, ie selected view 
     * ( object : DicomSliceCanvasPolyDraw ).
     * @param c
     */
    public void setCurrentView( DicomSliceCanvasPolyDraw c )
    {
        if ( m_currentView != null )
        {
            m_currentView.setBorder( BorderFactory.createEmptyBorder( ) );
        }
        
        m_currentView = c;
        
        m_currentView.setBorder( BorderFactory.createLineBorder( Color.red, 3 ) );
	
        if ( !m_selectedViews.contains( c )  )
        { 
            m_selectedViews.addElement( c );
        }
        repaint( );
        clipWindowTitle( );
        
        notifyActionListeners( SELECT_IMAGE_EVENT );
    }
    
    /**
     * Defines the image of the DicomSliceCanvasPolyDraw object 
     * (normal image type).
     * @param view
     * @param data
     * @param w
     * @param h
     */
    public void setImage( DicomSliceCanvasPolyDraw view, 
                          int[] data, 
                          int w, int h ) 
    {
        view.setImage( data, w, h, 0 );
        setSize( getPreferredSize( ) );
        // clipWindowTitle( );
        String fileName = view.getTitle( );
        int i = fileName.lastIndexOf( "." );
        if ( i == -1 ) i = fileName.length( );
        String nameRoot = fileName.substring( 0, i );
        String regName = nameRoot + ".reg";
        File f = new File( regName );
        if ( f.exists( ) )
        {
            m_currentView.importPolysFromFile( regName );
        }
    }
    
    /**
     * Clips the window title in case normal image display mode.
     */
    public void clipWindowTitle( ) 
    {
        Dimension d = getSize( );
        int w = d.width - 200;
        if ( w < 0 ) w = 0;
        Font f = getFont( );
        if ( f == null ) return;
        FontMetrics fm = getFontMetrics( f );
        int k = 0;
        if ( m_currentView != null )
        {
            String S = m_currentView.getTitle( );
            while ( fm.stringWidth( S.substring( k ) ) > w ) k++;
            m_parent.setTitle( S.substring( k ) );
        }
    }


    /**
     * Method iterating over all displayed images.
     */
    public void saveAllRegionsOfAllViews( )
    {
        DicomSliceCanvasPolyDraw currentBackup = this.m_currentView;

        Iterator it = m_sliceViews.iterator( );
        DecimalFormat formatter = new DecimalFormat( "###.00" );
        boolean createdHeader = false;

        while ( it.hasNext( ) )
        {
            m_currentView = (DicomSliceCanvasPolyDraw)it.next( );
            saveRegions( );
        }
    }



    /**
     * Saves all ROI information to a file.
     */
    public void saveAllROIInformationToFile( )
    {
        
        Iterator it = m_sliceViews.iterator( );
        DecimalFormat formatter = new DecimalFormat( "###.###" );

        String fileName = getFilePathAndName( m_currentView.getImageSeriesData( ).getFileInfo( 0 ).getSliceLocation( )     + "_" +
                                              m_currentView.getImageSeriesData( ).getFileInfo( 0 ).getSeriesDescription( ) + "_" + "_verbose_DTPA_rep.txt", "", "Save" );

        FileWriter fileWriter;
        try
        {
            fileWriter = new FileWriter( fileName );

            // First: write the header

            FileInfo fInfo = m_currentView.getImageSeriesData( ).getFileInfo( 0 );

            fileWriter.write( "Patient name       : " + fInfo.getPatientName( ) + "\r\n");
            fileWriter.write( "Patient ID         : " + fInfo.getPatientID( ) + "\r\n");
            fileWriter.write( "Date of Birth      : " + fInfo.getPatientBirthDate( ) + " " );
            fileWriter.write( "Age                : " + fInfo.getPatientAge( ) + "\r\n" );
            fileWriter.write( "Examination Date   : " + fInfo.getAcquisitionDate( ) + "\r\n" );
            fileWriter.write( "Examination Time   : " + fInfo.getAcquisitionTime( ) + "\r\n" );
            fileWriter.write( "Series Number      : " + fInfo.getSeriesNumber( ).trim( ) + "\r\n" );
            fileWriter.write( "Series Description : " + fInfo.getSeriesDescription( ) + "\r\n" );
            fileWriter.write( "Slice position     : " + fInfo.getSliceLocation( ) + "\r\n" );

            while ( it.hasNext( ) )
            {
                m_currentView = (DicomSliceCanvasPolyDraw)it.next( );
                m_currentView.writeROIInformation( fileWriter );

            }
            fileWriter.close( );
        }
        catch ( IOException ex )
        {
            Logger.getLogger( MultiDicomSliceView.class.getName( ) ).log( Level.SEVERE, null, ex );
        }
    }
    
    /**
     * Saves all ROI to computer aided database file.
     * @param diseaseLabel
     * @param deleteFile
     * @param to_predict
     */
    public void saveAllROIInformationToCAD_dBase_File( String diseaseLabel, 
                                                       boolean deleteFile, 
                                                       boolean to_predict )
    {
     
        if ( deleteFile == true )
        {
            if ( to_predict == false )
            {
                String databaseFileName = System.getProperty( "user.home" ) +
                                          System.getProperty( "file.separator" ) + "CAD" +
                                          System.getProperty( "file.separator" ) + "CAD_dbases_all" +
                                          System.getProperty( "file.separator" ) + diseaseLabel + ".txt";
        
                File f = new File( databaseFileName );
                if ( f.exists( ) == true )
                {
                    f.delete( );
                }
            }
        }
        else
        {
            if ( to_predict == true )
            {
                String databaseFileName = System.getProperty( "user.home" ) +
                                          System.getProperty( "file.separator" ) + "CAD" +
                                          System.getProperty( "file.separator" ) + "CAD_entry_2_predict" +
                                          System.getProperty( "file.separator" ) + diseaseLabel + ".txt";
                
                File f = new File( databaseFileName );
                if ( f.exists( ) == true )
                {
                    f.delete( );
                }
            }
        }    
        
        //create first a container class containing all information 
        HashMap<String, LinkedList<MomentsAboutTheMean> > tpmInformationHashMap 
                = new HashMap< >( );  
             
        FileInfo fInfo = m_currentView.getImageSeriesData( ).getFileInfo( 0 );
        String patientAge = fInfo.getPatientAge( );
        String table_start = null;
        table_start = table_start + "tissue_type,"  + diseaseLabel + ",";
        table_start = table_start + "pat_name,"     + fInfo.getPatientName( ) + ",";
        table_start = table_start + "pat_id,"       + fInfo.getPatientID( ) + ",";
        table_start = table_start + "d_o_b,"        + fInfo.getPatientBirthDate( ) + ",";
        table_start = table_start + "age,"          + fInfo.getPatientAge( ) + ",";
        table_start = table_start + "exam_date,"    + fInfo.getAcquisitionDate( ) + ",";
        table_start = table_start + "exam_time,"    + fInfo.getAcquisitionTime( ) + ",";
        table_start = table_start + "ser_numb,"     + fInfo.getSeriesNumber( ).trim( ) + ",";
        table_start = table_start + "ser_desc,"     + fInfo.getSeriesDescription( ) + ",";
        table_start = table_start + "slice_pos,"    + fInfo.getSliceLocation( ) + ",";
        
        // Declaration of temporary information container classes to make
        LinkedList<MomentsAboutTheMean> origTPMVals = new LinkedList<MomentsAboutTheMean>( );

        LinkedList<MomentsAboutTheMean> averTPMVals = new LinkedList<MomentsAboutTheMean>( );
        LinkedList<MomentsAboutTheMean> diffTPMVals = new LinkedList<MomentsAboutTheMean>( );
        LinkedList<MomentsAboutTheMean> variTPMVals = new LinkedList<MomentsAboutTheMean>( );
        LinkedList<MomentsAboutTheMean> stdeTPMVals = new LinkedList<MomentsAboutTheMean>( );
        LinkedList<MomentsAboutTheMean> skewTPMVals = new LinkedList<MomentsAboutTheMean>( );
        LinkedList<MomentsAboutTheMean> kurtTPMVals = new LinkedList<MomentsAboutTheMean>( );
        LinkedList<MomentsAboutTheMean> vavaTPMVals = new LinkedList<MomentsAboutTheMean>( );
        
        LinkedList<MomentsAboutTheMean> cmeaTPMVals = new LinkedList<MomentsAboutTheMean>( );
        LinkedList<MomentsAboutTheMean> cconTPMVals = new LinkedList<MomentsAboutTheMean>( );
        LinkedList<MomentsAboutTheMean> cdisTPMVals = new LinkedList<MomentsAboutTheMean>( );
        LinkedList<MomentsAboutTheMean> chomTPMVals = new LinkedList<MomentsAboutTheMean>( );
        LinkedList<MomentsAboutTheMean> casmTPMVals = new LinkedList<MomentsAboutTheMean>( );
        LinkedList<MomentsAboutTheMean> ceneTPMVals = new LinkedList<MomentsAboutTheMean>( );
        LinkedList<MomentsAboutTheMean> cmaxTPMVals = new LinkedList<MomentsAboutTheMean>( );
        LinkedList<MomentsAboutTheMean> centTPMVals = new LinkedList<MomentsAboutTheMean>( );
        LinkedList<MomentsAboutTheMean> cvarTPMVals = new LinkedList<MomentsAboutTheMean>( );
        LinkedList<MomentsAboutTheMean> ccorTPMVals = new LinkedList<MomentsAboutTheMean>( );
  
        Iterator it = m_sliceViews.iterator( );
        
        // Collect complete texture parameterinformation of all texture parameter maps
        while ( it.hasNext( ) )
        {
            m_currentView = (DicomSliceCanvasPolyDraw)it.next( );
            
            LinkedList<MomentsAboutTheMean> momentsAboutTheMean = m_currentView.getROI_MAM_Information( );
            
            switch ( momentsAboutTheMean.getFirst( ).getTPMTypeNumber( ) )
            {
                case MomentsAboutTheMean.TPM_TYPE_ORIG: origTPMVals = momentsAboutTheMean; break;
                case MomentsAboutTheMean.TPM_TYPE_AVER: averTPMVals = momentsAboutTheMean; break;
                case MomentsAboutTheMean.TPM_TYPE_DIFF: diffTPMVals = momentsAboutTheMean; break;
                case MomentsAboutTheMean.TPM_TYPE_VARI: variTPMVals = momentsAboutTheMean; break;
                case MomentsAboutTheMean.TPM_TYPE_STDE: stdeTPMVals = momentsAboutTheMean; break;
                case MomentsAboutTheMean.TPM_TYPE_SKEW: skewTPMVals = momentsAboutTheMean; break;
                case MomentsAboutTheMean.TPM_TYPE_KURT: kurtTPMVals = momentsAboutTheMean; break;
                case MomentsAboutTheMean.TPM_TYPE_VAVA: vavaTPMVals = momentsAboutTheMean; break;
                
                case MomentsAboutTheMean.TPM_TYPE_CMEA: cmeaTPMVals = momentsAboutTheMean; break;
                case MomentsAboutTheMean.TPM_TYPE_CCON: cconTPMVals = momentsAboutTheMean; break;
                case MomentsAboutTheMean.TPM_TYPE_CDIS: cdisTPMVals = momentsAboutTheMean; break;
                case MomentsAboutTheMean.TPM_TYPE_CHOM: chomTPMVals = momentsAboutTheMean; break;
                case MomentsAboutTheMean.TPM_TYPE_CASM: casmTPMVals = momentsAboutTheMean; break;
                case MomentsAboutTheMean.TPM_TYPE_CENE: ceneTPMVals = momentsAboutTheMean; break;
                case MomentsAboutTheMean.TPM_TYPE_CMAX: cmaxTPMVals = momentsAboutTheMean; break;
                case MomentsAboutTheMean.TPM_TYPE_CENT: centTPMVals = momentsAboutTheMean; break;
                case MomentsAboutTheMean.TPM_TYPE_CVAR: cvarTPMVals = momentsAboutTheMean; break;
                case MomentsAboutTheMean.TPM_TYPE_CCOR: ccorTPMVals = momentsAboutTheMean; break;                  
            }
        }
        
        // All information has been gathered; now write 
        // the information to a database text file
        FileWriter fileWriter;
        String databaseFileName;
        if ( to_predict == false )
        {
            databaseFileName = System.getProperty( "user.home" ) +
                               System.getProperty( "file.separator" ) + "CAD" +
                               System.getProperty( "file.separator" ) + "CAD_dbases_all" +
                               System.getProperty( "file.separator" ) + diseaseLabel + 
                               ".txt";
        }
        else
        {
            databaseFileName = System.getProperty( "user.home" ) +
                               System.getProperty( "file.separator" ) + "CAD" +
                               System.getProperty( "file.separator" ) + "CAD_entry_2_predict" +
                               System.getProperty( "file.separator" ) + diseaseLabel + 
                               ".txt";
        }
        
        File f = new File( databaseFileName );
        
        if ( ( f.exists( ) == true ) && ( to_predict == true ) )
        {
            f.delete( );
        }
        
        
        if ( !f.exists( ) && !f.isDirectory( ) ) 
        {
            try
            {
                // the database does not exist so create it
                f.createNewFile( );
                
                fileWriter = new FileWriter( databaseFileName, true );
                String labels   = "tissue_type,file_ref,pat_age,ser_i_uid,pat_id,norm_const,poly_name,pixe_numb,exam_volu,";
                labels          += "orig_mean,orig_var,orig_stde,orig_skew,orig_kurt,orig_vava,";
                labels          += "aver_mean,aver_var,aver_stde,aver_skew,aver_kurt,aver_vava,";
                labels          += "vari_mean,vari_var,vari_stde,vari_skew,vari_kurt,vari_vava,";
                labels          += "stde_mean,stde_var,stde_stde,stde_skew,stde_kurt,stde_vava,";
                labels          += "skew_mean,skew_var,skew_stde,skew_skew,skew_kurt,skew_vava,";
                labels          += "kurt_mean,kurt_var,kurt_stde,kurt_skew,kurt_kurt,kurt_vava,";
                labels          += "vava_mean,vava_var,vava_stde,vava_skew,vava_kurt,vava_vava,";
                labels          += "cmea_mean,cmea_var,cmea_stde,cmea_skew,cmea_kurt,cmea_vava,";
                labels          += "ccon_mean,ccon_var,ccon_stde,ccon_skew,ccon_kurt,ccon_vava,";
                labels          += "cdis_mean,cdis_var,cdis_stde,cdis_skew,cdis_kurt,cdis_vava,";
                labels          += "chom_mean,chom_var,chom_stde,chom_skew,chom_kurt,chom_vava,";
                labels          += "casm_mean,casm_var,casm_stde,casm_skew,casm_kurt,casm_vava,";
                labels          += "cene_mean,cene_var,cene_stde,cene_skew,cene_kurt,cene_vava,";
                labels          += "cmax_mean,cmax_var,cmax_stde,cmax_skew,cmax_kurt,cmax_vava,";
                labels          += "cent_mean,cent_var,cent_stde,cent_skew,cent_kurt,cent_vava,";
                labels          += "cvar_mean,cvar_var,cvar_stde,cvar_skew,cvar_kurt,cvar_vava,";
                labels          += "ccor_mean,ccor_var,ccor_stde,ccor_skew,ccor_kurt,ccor_vava\n";
                fileWriter.write( labels );
                fileWriter.close( );
            }
            catch ( IOException ex )
            {
                // do nothing
                ex.printStackTrace( );
            }
        }
        
        try
        {
            fileWriter = new FileWriter( databaseFileName, true );
            
            for ( int i = 0; i < origTPMVals.size( ); i++ )
            {
                
               MomentsAboutTheMean orig = origTPMVals.get( i );
               MomentsAboutTheMean aver = averTPMVals.get( i );
               MomentsAboutTheMean vari = variTPMVals.get( i );
               MomentsAboutTheMean stde = stdeTPMVals.get( i );
               MomentsAboutTheMean skew = skewTPMVals.get( i );
               MomentsAboutTheMean kurt = kurtTPMVals.get( i );
               MomentsAboutTheMean vava = vavaTPMVals.get( i );
               MomentsAboutTheMean cmea = cmeaTPMVals.get( i );
               MomentsAboutTheMean ccon = cconTPMVals.get( i );
               MomentsAboutTheMean cdis = cdisTPMVals.get( i );
               MomentsAboutTheMean chom = chomTPMVals.get( i );
               MomentsAboutTheMean casm = casmTPMVals.get( i );
               MomentsAboutTheMean cene = ceneTPMVals.get( i );
               MomentsAboutTheMean cmax = cmaxTPMVals.get( i );
               MomentsAboutTheMean cent = centTPMVals.get( i );
               MomentsAboutTheMean cvar = cvarTPMVals.get( i );
               MomentsAboutTheMean ccor = ccorTPMVals.get( i );
               
               // Do writing!
               fileWriter.write("" + diseaseLabel + "," );
               fileWriter.write( "" + orig.getDicomImageFileReference( ) + "," );
               fileWriter.write( "" + patientAge + ",");
               fileWriter.write( "" + orig.getInstanceSeriesUID( ) + ",");
               fileWriter.write( "" + orig.getPatientID( ) + "," );
               
               fileWriter.write( "" + orig.getNormalizationConstant( ) + "," );
               fileWriter.write( "" + orig.getPolygonName( ) + "," );
               fileWriter.write( "" + orig.getNumberOfROIPixels( ) + "," );
               fileWriter.write( "" + orig.getExaminedVolumeSize( ) + "," );
               // orig
               fileWriter.write( "" + orig.getMean( ) + "," );
               fileWriter.write( "" + orig.getVariance( ) + "," );
               fileWriter.write( "" + orig.getStDev( ) + "," );
               fileWriter.write( "" + orig.getSkewness( ) + "," );
               fileWriter.write( "" + orig.getKurtosis( ) + "," );
               fileWriter.write( "" + orig.getVarianceOfVariance( ) + "," );
               
               // aver
               fileWriter.write( "" + aver.getMean( ) + "," );
               fileWriter.write( "" + aver.getVariance( ) + "," );
               fileWriter.write( "" + aver.getStDev( ) + "," );
               fileWriter.write( "" + aver.getSkewness( ) + "," );
               fileWriter.write( "" + aver.getKurtosis( ) + "," );
               fileWriter.write( "" + aver.getVarianceOfVariance( ) + "," );
               
               // vari
               fileWriter.write( "" + vari.getMean( ) + "," );
               fileWriter.write( "" + vari.getVariance( ) + "," );
               fileWriter.write( "" + vari.getStDev( ) + "," );
               fileWriter.write( "" + vari.getSkewness( ) + "," );
               fileWriter.write( "" + vari.getKurtosis( ) + "," );
               fileWriter.write( "" + vari.getVarianceOfVariance( ) + "," );
               
               // stde
               fileWriter.write( "" + stde.getMean( ) + "," );
               fileWriter.write( "" + stde.getVariance( ) + "," );
               fileWriter.write( "" + stde.getStDev( ) + "," );
               fileWriter.write( "" + stde.getSkewness( ) + "," );
               fileWriter.write( "" + stde.getKurtosis( ) + "," );
               fileWriter.write( "" + stde.getVarianceOfVariance( ) + "," );
               
               // skew
               fileWriter.write( "" + skew.getMean( ) + "," );
               fileWriter.write( "" + skew.getVariance( ) + "," );
               fileWriter.write( "" + skew.getStDev( ) + "," );
               fileWriter.write( "" + skew.getSkewness( ) + "," );
               fileWriter.write( "" + skew.getKurtosis( ) + "," );
               fileWriter.write( "" + skew.getVarianceOfVariance( ) + "," );
               
               // kurt
               fileWriter.write( "" + kurt.getMean( ) + "," );
               fileWriter.write( "" + kurt.getVariance( ) + "," );
               fileWriter.write( "" + kurt.getStDev( ) + "," );
               fileWriter.write( "" + kurt.getSkewness( ) + "," );
               fileWriter.write( "" + kurt.getKurtosis( ) + "," );
               fileWriter.write( "" + kurt.getVarianceOfVariance( ) + "," );
               
               // vava
               fileWriter.write( "" + vava.getMean( ) + "," );
               fileWriter.write( "" + vava.getVariance( ) + "," );
               fileWriter.write( "" + vava.getStDev( ) + "," );
               fileWriter.write( "" + vava.getSkewness( ) + "," );
               fileWriter.write( "" + vava.getKurtosis( ) + "," );
               fileWriter.write( "" + vava.getVarianceOfVariance( ) + "," );
               
               // cmea
               fileWriter.write( "" + cmea.getMean( ) + "," );
               fileWriter.write( "" + cmea.getVariance( ) + "," );
               fileWriter.write( "" + cmea.getStDev( ) + "," );
               fileWriter.write( "" + cmea.getSkewness( ) + "," );
               fileWriter.write( "" + cmea.getKurtosis( ) + "," );
               fileWriter.write( "" + cmea.getVarianceOfVariance( ) + "," );
               
               // ccon
               fileWriter.write( "" + ccon.getMean( ) + "," );
               fileWriter.write( "" + ccon.getVariance( ) + "," );
               fileWriter.write( "" + ccon.getStDev( ) + "," );
               fileWriter.write( "" + ccon.getSkewness( ) + "," );
               fileWriter.write( "" + ccon.getKurtosis( ) + "," );
               fileWriter.write( "" + ccon.getVarianceOfVariance( ) + "," );
               
               // cdis
               fileWriter.write( "" + cdis.getMean( ) + "," );
               fileWriter.write( "" + cdis.getVariance( ) + "," );
               fileWriter.write( "" + cdis.getStDev( ) + "," );
               fileWriter.write( "" + cdis.getSkewness( ) + "," );
               fileWriter.write( "" + cdis.getKurtosis( ) + "," );
               fileWriter.write( "" + cdis.getVarianceOfVariance( ) + "," );
               
               // chom
               fileWriter.write( "" + chom.getMean( ) + "," );
               fileWriter.write( "" + chom.getVariance( ) + "," );
               fileWriter.write( "" + chom.getStDev( ) + "," );
               fileWriter.write( "" + chom.getSkewness( ) + "," );
               fileWriter.write( "" + chom.getKurtosis( ) + "," );
               fileWriter.write( "" + chom.getVarianceOfVariance( ) + "," );
               
               // casm
               fileWriter.write( "" + casm.getMean( ) + "," );
               fileWriter.write( "" + casm.getVariance( ) + "," );
               fileWriter.write( "" + casm.getStDev( ) + "," );
               fileWriter.write( "" + casm.getSkewness( ) + "," );
               fileWriter.write( "" + casm.getKurtosis( ) + "," );
               fileWriter.write( "" + casm.getVarianceOfVariance( ) + "," );
               
               // cene
               fileWriter.write( "" + cene.getMean( ) + "," );
               fileWriter.write( "" + cene.getVariance( ) + "," );
               fileWriter.write( "" + cene.getStDev( ) + "," );
               fileWriter.write( "" + cene.getSkewness( ) + "," );
               fileWriter.write( "" + cene.getKurtosis( ) + "," );
               fileWriter.write( "" + cene.getVarianceOfVariance( ) + "," );
               
               // cmax
               fileWriter.write( "" + cmax.getMean( ) + "," );
               fileWriter.write( "" + cmax.getVariance( ) + "," );
               fileWriter.write( "" + cmax.getStDev( ) + "," );
               fileWriter.write( "" + cmax.getSkewness( ) + "," );
               fileWriter.write( "" + cmax.getKurtosis( ) + "," );
               fileWriter.write( "" + cmax.getVarianceOfVariance( ) + "," );
               
               // cent
               fileWriter.write( "" + cent.getMean( ) + "," );
               fileWriter.write( "" + cent.getVariance( ) + "," );
               fileWriter.write( "" + cent.getStDev( ) + "," );
               fileWriter.write( "" + cent.getSkewness( ) + "," );
               fileWriter.write( "" + cent.getKurtosis( ) + "," );
               fileWriter.write( "" + cent.getVarianceOfVariance( ) + "," );
               
               // cvar
               fileWriter.write( "" + cvar.getMean( ) + "," );
               fileWriter.write( "" + cvar.getVariance( ) + "," );
               fileWriter.write( "" + cvar.getStDev( ) + "," );
               fileWriter.write( "" + cvar.getSkewness( ) + "," );
               fileWriter.write( "" + cvar.getKurtosis( ) + "," );
               fileWriter.write( "" + cvar.getVarianceOfVariance( ) + "," );
               
                // ccor
               fileWriter.write( "" + ccor.getMean( ) + "," );
               fileWriter.write( "" + ccor.getVariance( ) + "," );
               fileWriter.write( "" + ccor.getStDev( ) + "," );
               fileWriter.write( "" + ccor.getSkewness( ) + "," );
               fileWriter.write( "" + ccor.getKurtosis( ) + "," );
               fileWriter.write( "" + ccor.getVarianceOfVariance( ) + "\n" );
            }
            
            fileWriter.close( );
        }
        catch ( IOException ex )
        {
            // do nothing
        }
    }
    
    /**
     * Saves information of an image time series to a database file.
     */
    public void saveAllROIInformationToCAD_dBase_File_ForTimeSeries( 
                                                        String diseaseLabel, 
                                                        boolean deleteFile, 
                                                        boolean to_predict )
    {
     
        if ( deleteFile == true )
        {
            if ( to_predict == false )
            {
                String databaseFileName = System.getProperty( "user.home" ) +
                                          System.getProperty( "file.separator" ) + "CAD" +
                                          System.getProperty( "file.separator" ) + "CAD_dbases_all" +
                                          System.getProperty( "file.separator" ) + diseaseLabel + ".txt";
        
                File f = new File( databaseFileName );
                if ( f.exists( ) == true )
                {
                    f.delete( );
                }
            }
        }
        else
        {
            if ( to_predict == true )
            {
                String databaseFileName = System.getProperty( "user.home" ) +
                                          System.getProperty( "file.separator" ) + "CAD" +
                                          System.getProperty( "file.separator" ) + "CAD_entry_2_predict" +
                                          System.getProperty( "file.separator" ) + diseaseLabel + ".txt";
                
                File f = new File( databaseFileName );
                if ( f.exists( ) == true )
                {
                    f.delete( );
                }
            }
        }    
        
        //create first a container class containing all information 
        HashMap<String, LinkedList<MomentsAboutTheMean> > tpmInformationHashMap = new HashMap<String, LinkedList<MomentsAboutTheMean> >( );  
             
        FileInfo fInfo = m_currentView.getImageSeriesData( ).getFileInfo( 0 );
        String patientAge = fInfo.getPatientAge( );
        String table_start = null;
        table_start = table_start + "tissue_type,"     + diseaseLabel + ",";
        table_start = table_start + "pat_name,"     + fInfo.getPatientName( ) + ",";
        table_start = table_start + "pat_id,"       + fInfo.getPatientID( ) + ",";
        table_start = table_start + "d_o_b,"        + fInfo.getPatientBirthDate( ) + ",";
        table_start = table_start + "age,"          + fInfo.getPatientAge( ) + ",";
        table_start = table_start + "exam_date,"    + fInfo.getAcquisitionDate( ) + ",";
        table_start = table_start + "exam_time,"    + fInfo.getAcquisitionTime( ) + ",";
        table_start = table_start + "ser_numb,"     + fInfo.getSeriesNumber( ).trim( ) + ",";
        table_start = table_start + "ser_desc,"     + fInfo.getSeriesDescription( ) + ",";
        table_start = table_start + "slice_pos,"    + fInfo.getSliceLocation( ) + ",";
   
        // Declaration of temporary information container classes to make
        LinkedList<MomentsAboutTheMean> origTPMVals = new LinkedList<MomentsAboutTheMean>( );

        LinkedList<MomentsAboutTheMean> averTPMVals = new LinkedList<MomentsAboutTheMean>( );
        LinkedList<MomentsAboutTheMean> diffTPMVals = new LinkedList<MomentsAboutTheMean>( );
        LinkedList<MomentsAboutTheMean> variTPMVals = new LinkedList<MomentsAboutTheMean>( );
        LinkedList<MomentsAboutTheMean> stdeTPMVals = new LinkedList<MomentsAboutTheMean>( );
        LinkedList<MomentsAboutTheMean> skewTPMVals = new LinkedList<MomentsAboutTheMean>( );
        LinkedList<MomentsAboutTheMean> kurtTPMVals = new LinkedList<MomentsAboutTheMean>( );
        LinkedList<MomentsAboutTheMean> vavaTPMVals = new LinkedList<MomentsAboutTheMean>( );
        
        LinkedList<MomentsAboutTheMean> cmeaTPMVals = new LinkedList<MomentsAboutTheMean>( );
        LinkedList<MomentsAboutTheMean> cconTPMVals = new LinkedList<MomentsAboutTheMean>( );
        LinkedList<MomentsAboutTheMean> cdisTPMVals = new LinkedList<MomentsAboutTheMean>( );
        LinkedList<MomentsAboutTheMean> chomTPMVals = new LinkedList<MomentsAboutTheMean>( );
        LinkedList<MomentsAboutTheMean> casmTPMVals = new LinkedList<MomentsAboutTheMean>( );
        LinkedList<MomentsAboutTheMean> ceneTPMVals = new LinkedList<MomentsAboutTheMean>( );
        LinkedList<MomentsAboutTheMean> cmaxTPMVals = new LinkedList<MomentsAboutTheMean>( );
        LinkedList<MomentsAboutTheMean> centTPMVals = new LinkedList<MomentsAboutTheMean>( );
        LinkedList<MomentsAboutTheMean> cvarTPMVals = new LinkedList<MomentsAboutTheMean>( );
        LinkedList<MomentsAboutTheMean> ccorTPMVals = new LinkedList<MomentsAboutTheMean>( );
  
        Iterator it = m_sliceViews.iterator( );
      
        // Collect complete texture parameterinformation of all texture parameter maps
        while ( it.hasNext( ) )
        {
            m_currentView = (DicomSliceCanvasPolyDraw)it.next( );
            
            LinkedList<MomentsAboutTheMean> momentsAboutTheMean = m_currentView.getROI_MAM_Information( );
            
            switch ( momentsAboutTheMean.getFirst( ).getTPMTypeNumber( ) )
            {
                case MomentsAboutTheMean.TPM_TYPE_ORIG: origTPMVals = momentsAboutTheMean; break;
                case MomentsAboutTheMean.TPM_TYPE_AVER: averTPMVals = momentsAboutTheMean; break;
                case MomentsAboutTheMean.TPM_TYPE_DIFF: diffTPMVals = momentsAboutTheMean; break;
                case MomentsAboutTheMean.TPM_TYPE_VARI: variTPMVals = momentsAboutTheMean; break;
                case MomentsAboutTheMean.TPM_TYPE_STDE: stdeTPMVals = momentsAboutTheMean; break;
                case MomentsAboutTheMean.TPM_TYPE_SKEW: skewTPMVals = momentsAboutTheMean; break;
                case MomentsAboutTheMean.TPM_TYPE_KURT: kurtTPMVals = momentsAboutTheMean; break;
                case MomentsAboutTheMean.TPM_TYPE_VAVA: vavaTPMVals = momentsAboutTheMean; break;
                
                case MomentsAboutTheMean.TPM_TYPE_CMEA: cmeaTPMVals = momentsAboutTheMean; break;
                case MomentsAboutTheMean.TPM_TYPE_CCON: cconTPMVals = momentsAboutTheMean; break;
                case MomentsAboutTheMean.TPM_TYPE_CDIS: cdisTPMVals = momentsAboutTheMean; break;
                case MomentsAboutTheMean.TPM_TYPE_CHOM: chomTPMVals = momentsAboutTheMean; break;
                case MomentsAboutTheMean.TPM_TYPE_CASM: casmTPMVals = momentsAboutTheMean; break;
                case MomentsAboutTheMean.TPM_TYPE_CENE: ceneTPMVals = momentsAboutTheMean; break;
                case MomentsAboutTheMean.TPM_TYPE_CMAX: cmaxTPMVals = momentsAboutTheMean; break;
                case MomentsAboutTheMean.TPM_TYPE_CENT: centTPMVals = momentsAboutTheMean; break;
                case MomentsAboutTheMean.TPM_TYPE_CVAR: cvarTPMVals = momentsAboutTheMean; break;
                case MomentsAboutTheMean.TPM_TYPE_CCOR: ccorTPMVals = momentsAboutTheMean; break;                  
            }
        }
        
        // All information has been gathered; now write 
        // the information to a database text file
        FileWriter fileWriter;
        String databaseFileName;
        if ( to_predict == false )
        {
            databaseFileName = System.getProperty( "user.home" ) +
                               System.getProperty( "file.separator" ) + "CAD" +
                               System.getProperty( "file.separator" ) + "CAD_dbases_all" +
                               System.getProperty( "file.separator" ) + diseaseLabel + 
                               ".txt";
        }
        else
        {
            databaseFileName = System.getProperty( "user.home" ) +
                               System.getProperty( "file.separator" ) + "CAD" +
                               System.getProperty( "file.separator" ) + "CAD_entry_2_predict" +
                               System.getProperty( "file.separator" ) + diseaseLabel + 
                               ".txt";
        }
        
        File f = new File( databaseFileName );
        
        if ( ( f.exists( ) == true ) && ( to_predict == true ) )
        {
            f.delete( );
        }
        
        
        if ( !f.exists( ) && !f.isDirectory( ) ) 
        {
            try
            {
                // the database does not exist so create it
                f.createNewFile( );
                
                fileWriter = new FileWriter( databaseFileName, true );
                String labels   = "tissue_type,file_ref,pat_age,ser_i_uid,pat_id,norm_const,poly_name,pixe_numb,exam_volu,perf_phase,";
                labels          += "orig_mean,orig_var,orig_stde,orig_skew,orig_kurt,orig_vava,";
                labels          += "diff_mean,diff_var,diff_stde,diff_skew,diff_kurt,diff_vava,";
                labels          += "aver_mean,aver_var,aver_stde,aver_skew,aver_kurt,aver_vava,";
                labels          += "vari_mean,vari_var,vari_stde,vari_skew,vari_kurt,vari_vava,";
                labels          += "stde_mean,stde_var,stde_stde,stde_skew,stde_kurt,stde_vava,";
                labels          += "skew_mean,skew_var,skew_stde,skew_skew,skew_kurt,skew_vava,";
                labels          += "kurt_mean,kurt_var,kurt_stde,kurt_skew,kurt_kurt,kurt_vava,";
                labels          += "vava_mean,vava_var,vava_stde,vava_skew,vava_kurt,vava_vava,";
                labels          += "cmea_mean,cmea_var,cmea_stde,cmea_skew,cmea_kurt,cmea_vava,";
                labels          += "ccon_mean,ccon_var,ccon_stde,ccon_skew,ccon_kurt,ccon_vava,";
                labels          += "cdis_mean,cdis_var,cdis_stde,cdis_skew,cdis_kurt,cdis_vava,";
                labels          += "chom_mean,chom_var,chom_stde,chom_skew,chom_kurt,chom_vava,";
                labels          += "casm_mean,casm_var,casm_stde,casm_skew,casm_kurt,casm_vava,";
                labels          += "cene_mean,cene_var,cene_stde,cene_skew,cene_kurt,cene_vava,";
                labels          += "cmax_mean,cmax_var,cmax_stde,cmax_skew,cmax_kurt,cmax_vava,";
                labels          += "cent_mean,cent_var,cent_stde,cent_skew,cent_kurt,cent_vava,";
                labels          += "cvar_mean,cvar_var,cvar_stde,cvar_skew,cvar_kurt,cvar_vava,";
                labels          += "ccor_mean,ccor_var,ccor_stde,ccor_skew,ccor_kurt,ccor_vava\n";
                fileWriter.write( labels );
                fileWriter.close( );
            }
            catch ( IOException ex )
            {
                // do nothing
                ex.printStackTrace( );
            }
        }
        
        try
        {
            fileWriter = new FileWriter( databaseFileName, true );
            
            for ( int i = 0; i < origTPMVals.size( ); i++ )
            {
                
               MomentsAboutTheMean orig = origTPMVals.get( i );
               MomentsAboutTheMean diff = diffTPMVals.get( i );
               MomentsAboutTheMean aver = averTPMVals.get( i );
               MomentsAboutTheMean vari = variTPMVals.get( i );
               MomentsAboutTheMean stde = stdeTPMVals.get( i );
               MomentsAboutTheMean skew = skewTPMVals.get( i );
               MomentsAboutTheMean kurt = kurtTPMVals.get( i );
               MomentsAboutTheMean vava = vavaTPMVals.get( i );
               MomentsAboutTheMean cmea = cmeaTPMVals.get( i );
               MomentsAboutTheMean ccon = cconTPMVals.get( i );
               MomentsAboutTheMean cdis = cdisTPMVals.get( i );
               MomentsAboutTheMean chom = chomTPMVals.get( i );
               MomentsAboutTheMean casm = casmTPMVals.get( i );
               MomentsAboutTheMean cene = ceneTPMVals.get( i );
               MomentsAboutTheMean cmax = cmaxTPMVals.get( i );
               MomentsAboutTheMean cent = centTPMVals.get( i );
               MomentsAboutTheMean cvar = cvarTPMVals.get( i );
               MomentsAboutTheMean ccor = ccorTPMVals.get( i );
               
               // Do writing!
               fileWriter.write( "" + diseaseLabel + "," );
               fileWriter.write( "" + orig.getDicomImageFileReference( ) + "," );
               fileWriter.write( "" + patientAge + ",");
               fileWriter.write( "" + orig.getInstanceSeriesUID( ) + ",");
               fileWriter.write( "" + orig.getPatientID( ) + "," );
               
               fileWriter.write( "" + orig.getNormalizationConstant( ) + "," );
               fileWriter.write( "" + orig.getPolygonName( ) + "," );
               fileWriter.write( "" + orig.getNumberOfROIPixels( ) + "," );
               fileWriter.write( "" + orig.getExaminedVolumeSize( ) + "," );
               
               if ( ( 4 * i / origTPMVals.size( ) ) == 0 )
               {
                    fileWriter.write( "" + "baseline" + "," );
               }
               if ( ( 4 * i / origTPMVals.size( ) ) == 1 )
               {
                    fileWriter.write( "" + "inflow" + "," );
               }
               if ( ( 4 * i / origTPMVals.size( ) ) == 2 )
               {
                    fileWriter.write( "" + "outflow" + "," );
               }
               if ( ( 4 * i / origTPMVals.size( ) ) == 3 )
               {
                    fileWriter.write( "" + "reperfusion" + "," );
               }
               
               // orig
               fileWriter.write( "" + orig.getMean( ) + "," );
               fileWriter.write( "" + orig.getVariance( ) + "," );
               fileWriter.write( "" + orig.getStDev( ) + "," );
               fileWriter.write( "" + orig.getSkewness( ) + "," );
               fileWriter.write( "" + orig.getKurtosis( ) + "," );
               fileWriter.write( "" + orig.getVarianceOfVariance( ) + "," );
               
               // diff
               fileWriter.write( "" + diff.getMean( ) + "," );
               fileWriter.write( "" + diff.getVariance( ) + "," );
               fileWriter.write( "" + diff.getStDev( ) + "," );
               fileWriter.write( "" + diff.getSkewness( ) + "," );
               fileWriter.write( "" + diff.getKurtosis( ) + "," );
               fileWriter.write( "" + diff.getVarianceOfVariance( ) + "," );
               
               // aver
               fileWriter.write( "" + aver.getMean( ) + "," );
               fileWriter.write( "" + aver.getVariance( ) + "," );
               fileWriter.write( "" + aver.getStDev( ) + "," );
               fileWriter.write( "" + aver.getSkewness( ) + "," );
               fileWriter.write( "" + aver.getKurtosis( ) + "," );
               fileWriter.write( "" + aver.getVarianceOfVariance( ) + "," );
               
               
               // vari
               fileWriter.write( "" + vari.getMean( ) + "," );
               fileWriter.write( "" + vari.getVariance( ) + "," );
               fileWriter.write( "" + vari.getStDev( ) + "," );
               fileWriter.write( "" + vari.getSkewness( ) + "," );
               fileWriter.write( "" + vari.getKurtosis( ) + "," );
               fileWriter.write( "" + vari.getVarianceOfVariance( ) + "," );
               
               // stde
               fileWriter.write( "" + stde.getMean( ) + "," );
               fileWriter.write( "" + stde.getVariance( ) + "," );
               fileWriter.write( "" + stde.getStDev( ) + "," );
               fileWriter.write( "" + stde.getSkewness( ) + "," );
               fileWriter.write( "" + stde.getKurtosis( ) + "," );
               fileWriter.write( "" + stde.getVarianceOfVariance( ) + "," );
               
               // skew
               fileWriter.write( "" + skew.getMean( ) + "," );
               fileWriter.write( "" + skew.getVariance( ) + "," );
               fileWriter.write( "" + skew.getStDev( ) + "," );
               fileWriter.write( "" + skew.getSkewness( ) + "," );
               fileWriter.write( "" + skew.getKurtosis( ) + "," );
               fileWriter.write( "" + skew.getVarianceOfVariance( ) + "," );
               
               // kurt
               fileWriter.write( "" + kurt.getMean( ) + "," );
               fileWriter.write( "" + kurt.getVariance( ) + "," );
               fileWriter.write( "" + kurt.getStDev( ) + "," );
               fileWriter.write( "" + kurt.getSkewness( ) + "," );
               fileWriter.write( "" + kurt.getKurtosis( ) + "," );
               fileWriter.write( "" + kurt.getVarianceOfVariance( ) + "," );
               
               // vava
               fileWriter.write( "" + vava.getMean( ) + "," );
               fileWriter.write( "" + vava.getVariance( ) + "," );
               fileWriter.write( "" + vava.getStDev( ) + "," );
               fileWriter.write( "" + vava.getSkewness( ) + "," );
               fileWriter.write( "" + vava.getKurtosis( ) + "," );
               fileWriter.write( "" + vava.getVarianceOfVariance( ) + "," );
               
               // cmea
               fileWriter.write( "" + cmea.getMean( ) + "," );
               fileWriter.write( "" + cmea.getVariance( ) + "," );
               fileWriter.write( "" + cmea.getStDev( ) + "," );
               fileWriter.write( "" + cmea.getSkewness( ) + "," );
               fileWriter.write( "" + cmea.getKurtosis( ) + "," );
               fileWriter.write( "" + cmea.getVarianceOfVariance( ) + "," );
               
               // ccon
               fileWriter.write( "" + ccon.getMean( ) + "," );
               fileWriter.write( "" + ccon.getVariance( ) + "," );
               fileWriter.write( "" + ccon.getStDev( ) + "," );
               fileWriter.write( "" + ccon.getSkewness( ) + "," );
               fileWriter.write( "" + ccon.getKurtosis( ) + "," );
               fileWriter.write( "" + ccon.getVarianceOfVariance( ) + "," );
               
               // cdis
               fileWriter.write( "" + cdis.getMean( ) + "," );
               fileWriter.write( "" + cdis.getVariance( ) + "," );
               fileWriter.write( "" + cdis.getStDev( ) + "," );
               fileWriter.write( "" + cdis.getSkewness( ) + "," );
               fileWriter.write( "" + cdis.getKurtosis( ) + "," );
               fileWriter.write( "" + cdis.getVarianceOfVariance( ) + "," );
               
               // chom
               fileWriter.write( "" + chom.getMean( ) + "," );
               fileWriter.write( "" + chom.getVariance( ) + "," );
               fileWriter.write( "" + chom.getStDev( ) + "," );
               fileWriter.write( "" + chom.getSkewness( ) + "," );
               fileWriter.write( "" + chom.getKurtosis( ) + "," );
               fileWriter.write( "" + chom.getVarianceOfVariance( ) + "," );
               
               // casm
               fileWriter.write( "" + casm.getMean( ) + "," );
               fileWriter.write( "" + casm.getVariance( ) + "," );
               fileWriter.write( "" + casm.getStDev( ) + "," );
               fileWriter.write( "" + casm.getSkewness( ) + "," );
               fileWriter.write( "" + casm.getKurtosis( ) + "," );
               fileWriter.write( "" + casm.getVarianceOfVariance( ) + "," );
               
               // cene
               fileWriter.write( "" + cene.getMean( ) + "," );
               fileWriter.write( "" + cene.getVariance( ) + "," );
               fileWriter.write( "" + cene.getStDev( ) + "," );
               fileWriter.write( "" + cene.getSkewness( ) + "," );
               fileWriter.write( "" + cene.getKurtosis( ) + "," );
               fileWriter.write( "" + cene.getVarianceOfVariance( ) + "," );
               
               // cmax
               fileWriter.write( "" + cmax.getMean( ) + "," );
               fileWriter.write( "" + cmax.getVariance( ) + "," );
               fileWriter.write( "" + cmax.getStDev( ) + "," );
               fileWriter.write( "" + cmax.getSkewness( ) + "," );
               fileWriter.write( "" + cmax.getKurtosis( ) + "," );
               fileWriter.write( "" + cmax.getVarianceOfVariance( ) + "," );
               
               // cent
               fileWriter.write( "" + cent.getMean( ) + "," );
               fileWriter.write( "" + cent.getVariance( ) + "," );
               fileWriter.write( "" + cent.getStDev( ) + "," );
               fileWriter.write( "" + cent.getSkewness( ) + "," );
               fileWriter.write( "" + cent.getKurtosis( ) + "," );
               fileWriter.write( "" + cent.getVarianceOfVariance( ) + "," );
               
               // cvar
               fileWriter.write( "" + cvar.getMean( ) + "," );
               fileWriter.write( "" + cvar.getVariance( ) + "," );
               fileWriter.write( "" + cvar.getStDev( ) + "," );
               fileWriter.write( "" + cvar.getSkewness( ) + "," );
               fileWriter.write( "" + cvar.getKurtosis( ) + "," );
               fileWriter.write( "" + cvar.getVarianceOfVariance( ) + "," );
               
                // ccor
               fileWriter.write( "" + ccor.getMean( ) + "," );
               fileWriter.write( "" + ccor.getVariance( ) + "," );
               fileWriter.write( "" + ccor.getStDev( ) + "," );
               fileWriter.write( "" + ccor.getSkewness( ) + "," );
               fileWriter.write( "" + ccor.getKurtosis( ) + "," );
               fileWriter.write( "" + ccor.getVarianceOfVariance( ) + "\n" );
            }
            
            fileWriter.close( );
        }
        catch ( IOException ex )
        {
            // do nothing
        }
    }
    
    /**
     * Saves all textural information into a CAD database: one instance 
     * (data row) per patient.
     * @param deseaseLabel
     * @param deleteFile
     * @param to_predict
     */
    public void saveAllROIInformationToCAD_dBase_File_1_Entry_Per_Patient( String deseaseLabel, 
                                                                           boolean deleteFile, 
                                                                           boolean to_predict )
    {
     
        if ( deleteFile == true )
        {
            if ( to_predict == false )
            {
                String databaseFileName = System.getProperty( "user.home" ) +
                                      System.getProperty( "file.separator" ) + "CAD" +
                                      System.getProperty( "file.separator" ) + "CAD_dbases_1_i_pp" +
                                      System.getProperty( "file.separator" ) + deseaseLabel + "_1_inst_pp.txt";
        
                File f = new File( databaseFileName );
                if ( f.exists( ) == true )
                {
                    f.delete( );
                }
            }
            
            if ( to_predict == true )
            {
                String databaseFileName = System.getProperty( "user.home" ) +
                                          System.getProperty( "file.separator" ) + "CAD" +
                                          System.getProperty( "file.separator" ) + "CAD_entry_2_predict" +
                                          System.getProperty( "file.separator" ) + deseaseLabel + ".txt";
                
                File f = new File( databaseFileName );
                if ( f.exists( ) == true )
                {
                    f.delete( );
                }
            }
        }
        
        
        
//        if ( deleteFile == true )
//        {
//            String databaseFileName = System.getProperty( "user.home" ) +
//                                      System.getProperty( "file.separator" ) + "CAD" +
//                                      System.getProperty( "file.separator" ) + "CAD_dbases_1_i_pp" +
//                                      System.getProperty( "file.separator" ) + deseaseLabel + "_1_inst_pp.txt";
//        
//            File f = new File( databaseFileName );
//            f.delete( );
//        }
        
        
        //create first a container class containing all information 
        HashMap<String, LinkedList<MomentsAboutTheMean> > tpmInformationHashMap = new HashMap<String, LinkedList<MomentsAboutTheMean> >( );  
             
        FileInfo fInfo = m_currentView.getImageSeriesData( ).getFileInfo( 0 );

        float volumePerPixel= ( new Double( fInfo.getPixelHeight( ) ).floatValue( ) ) * ( new Double( fInfo.getPixelWidth( )).floatValue() ) * ( ( new Double( fInfo.getPixelDepth( ) ) )).floatValue( );
        String patientAge= fInfo.getPatientAge( );
        
        String table_start = null;
        table_start = table_start + "tissue_type,"     + deseaseLabel + ",";
        table_start = table_start + "pat_name,"     + fInfo.getPatientName( ) + ",";
        table_start = table_start + "pat_id,"       + fInfo.getPatientID( ) + ",";
        table_start = table_start + "d_o_b,"        + fInfo.getPatientBirthDate( ) + ",";
        table_start = table_start + "age,"          + fInfo.getPatientAge( ) + ",";
        table_start = table_start + "exam_date,"    + fInfo.getAcquisitionDate( ) + ",";
        table_start = table_start + "exam_time,"    + fInfo.getAcquisitionTime( ) + ",";
        table_start = table_start + "ser_numb,"     + fInfo.getSeriesNumber( ).trim( ) + ",";
        table_start = table_start + "ser_desc,"     + fInfo.getSeriesDescription( ) + ",";
        table_start = table_start + "slice_pos,"    + fInfo.getSliceLocation( ) + ",";
   
        // Declaration of temporary information container classes to make
        LinkedList<MomentsAboutTheMean> origTPMVals = new LinkedList<>( );

        LinkedList<MomentsAboutTheMean> meanTPMVals = new LinkedList<>( );
        LinkedList<MomentsAboutTheMean> diffTPMVals = new LinkedList<>( );
        LinkedList<MomentsAboutTheMean> variTPMVals = new LinkedList<>( );
        LinkedList<MomentsAboutTheMean> stdeTPMVals = new LinkedList<>( );
        LinkedList<MomentsAboutTheMean> skewTPMVals = new LinkedList<>( );
        LinkedList<MomentsAboutTheMean> kurtTPMVals = new LinkedList<>( );
        LinkedList<MomentsAboutTheMean> vavaTPMVals = new LinkedList<>( );
        
        LinkedList<MomentsAboutTheMean> cmeaTPMVals = new LinkedList<>( );
        LinkedList<MomentsAboutTheMean> cconTPMVals = new LinkedList<>( );
        LinkedList<MomentsAboutTheMean> cdisTPMVals = new LinkedList<>( );
        LinkedList<MomentsAboutTheMean> chomTPMVals = new LinkedList<>( );
        LinkedList<MomentsAboutTheMean> casmTPMVals = new LinkedList<>( );
        LinkedList<MomentsAboutTheMean> ceneTPMVals = new LinkedList<>( );
        LinkedList<MomentsAboutTheMean> cmaxTPMVals = new LinkedList<>( );
        LinkedList<MomentsAboutTheMean> centTPMVals = new LinkedList<>( );
        LinkedList<MomentsAboutTheMean> cvarTPMVals = new LinkedList<>( );
        LinkedList<MomentsAboutTheMean> ccorTPMVals = new LinkedList<>( );
  
        Iterator it = m_sliceViews.iterator( );
        
        // Collect complete texture parameterinformation of all texture parameter maps
        while ( it.hasNext( ) )
        {
            m_currentView = (DicomSliceCanvasPolyDraw)it.next( );
            
            LinkedList<MomentsAboutTheMean> momentsAboutTheMean = m_currentView.getROI_MAM_Information( );
            
            switch ( momentsAboutTheMean.getFirst( ).getTPMTypeString( ) )
            {
                case MomentsAboutTheMean.TPM_TYPE_STRING_ORIG: origTPMVals = momentsAboutTheMean; break;
                case MomentsAboutTheMean.TPM_TYPE_STRING_AVER: meanTPMVals = momentsAboutTheMean; break;
                case MomentsAboutTheMean.TPM_TYPE_STRING_DIFF: diffTPMVals = momentsAboutTheMean; break;
                case MomentsAboutTheMean.TPM_TYPE_STRING_VARI: variTPMVals = momentsAboutTheMean; break;
                case MomentsAboutTheMean.TPM_TYPE_STRING_STDE: stdeTPMVals = momentsAboutTheMean; break;
                case MomentsAboutTheMean.TPM_TYPE_STRING_SKEW: skewTPMVals = momentsAboutTheMean; break;
                case MomentsAboutTheMean.TPM_TYPE_STRING_KURT: kurtTPMVals = momentsAboutTheMean; break;
                case MomentsAboutTheMean.TPM_TYPE_STRING_VAVA: vavaTPMVals = momentsAboutTheMean; break;
                
                case MomentsAboutTheMean.TPM_TYPE_STRING_CMEA: cmeaTPMVals = momentsAboutTheMean; break;
                case MomentsAboutTheMean.TPM_TYPE_STRING_CCON: cconTPMVals = momentsAboutTheMean; break;
                case MomentsAboutTheMean.TPM_TYPE_STRING_CDIS: cdisTPMVals = momentsAboutTheMean; break;
                case MomentsAboutTheMean.TPM_TYPE_STRING_CHOM: chomTPMVals = momentsAboutTheMean; break;
                case MomentsAboutTheMean.TPM_TYPE_STRING_CASM: casmTPMVals = momentsAboutTheMean; break;
                case MomentsAboutTheMean.TPM_TYPE_STRING_CENE: ceneTPMVals = momentsAboutTheMean; break;
                case MomentsAboutTheMean.TPM_TYPE_STRING_CMAX: cmaxTPMVals = momentsAboutTheMean; break;
                case MomentsAboutTheMean.TPM_TYPE_STRING_CENT: centTPMVals = momentsAboutTheMean; break;
                case MomentsAboutTheMean.TPM_TYPE_STRING_CVAR: cvarTPMVals = momentsAboutTheMean; break;
                case MomentsAboutTheMean.TPM_TYPE_STRING_CCOR: ccorTPMVals = momentsAboutTheMean; break;                  
            }
        }
        
        MomentsAboutTheMean weightAv_origTPMVals;
                            weightAv_origTPMVals = MomentsAboutTheMean.ComputeWeightedAverages( origTPMVals, volumePerPixel,
                                                                                                MomentsAboutTheMean.TPM_TYPE_ORIG,
                                                                                                MomentsAboutTheMean.TPM_TYPE_STRING_ORIG );
        
        MomentsAboutTheMean weightAv_meanTPMVals = MomentsAboutTheMean.ComputeWeightedAverages( meanTPMVals, volumePerPixel ,
                                                                                                MomentsAboutTheMean.TPM_TYPE_AVER,
                                                                                                MomentsAboutTheMean.TPM_TYPE_STRING_AVER );
        
        MomentsAboutTheMean weightAv_diffTPMVals = MomentsAboutTheMean.ComputeWeightedAverages( diffTPMVals, volumePerPixel ,
                                                                                                MomentsAboutTheMean.TPM_TYPE_DIFF,
                                                                                                MomentsAboutTheMean.TPM_TYPE_STRING_DIFF );
        
        
        MomentsAboutTheMean weightAv_variTPMVals = MomentsAboutTheMean.ComputeWeightedAverages( variTPMVals, volumePerPixel ,
                                                                                                MomentsAboutTheMean.TPM_TYPE_VARI,
                                                                                                MomentsAboutTheMean.TPM_TYPE_STRING_VARI );
        
        MomentsAboutTheMean weightAv_stdeTPMVals = MomentsAboutTheMean.ComputeWeightedAverages( stdeTPMVals, volumePerPixel,
                                                                                                MomentsAboutTheMean.TPM_TYPE_STDE,
                                                                                                MomentsAboutTheMean.TPM_TYPE_STRING_STDE );
        
        MomentsAboutTheMean weightAv_skewTPMVals = MomentsAboutTheMean.ComputeWeightedAverages( skewTPMVals, volumePerPixel ,
                                                                                                MomentsAboutTheMean.TPM_TYPE_SKEW,
                                                                                                MomentsAboutTheMean.TPM_TYPE_STRING_SKEW );
        
        MomentsAboutTheMean weightAv_kurtTPMVals = MomentsAboutTheMean.ComputeWeightedAverages( kurtTPMVals, volumePerPixel ,
                                                                                                MomentsAboutTheMean.TPM_TYPE_KURT,
                                                                                                MomentsAboutTheMean.TPM_TYPE_STRING_KURT );
        
        MomentsAboutTheMean weightAv_vavaTPMVals = MomentsAboutTheMean.ComputeWeightedAverages( vavaTPMVals, volumePerPixel ,
                                                                                                MomentsAboutTheMean.TPM_TYPE_VAVA,
                                                                                                MomentsAboutTheMean.TPM_TYPE_STRING_VAVA );
        
        MomentsAboutTheMean weightAv_cmeaTPMVals = MomentsAboutTheMean.ComputeWeightedAverages( cmeaTPMVals, volumePerPixel ,
                                                                                                MomentsAboutTheMean.TPM_TYPE_CMEA,
                                                                                                MomentsAboutTheMean.TPM_TYPE_STRING_CMEA );
        
        MomentsAboutTheMean weightAv_cconTPMVals = MomentsAboutTheMean.ComputeWeightedAverages( cconTPMVals, volumePerPixel ,
                                                                                                MomentsAboutTheMean.TPM_TYPE_CCON,
                                                                                                MomentsAboutTheMean.TPM_TYPE_STRING_CCON );
        
        MomentsAboutTheMean weightAv_cdisTPMVals = MomentsAboutTheMean.ComputeWeightedAverages( cdisTPMVals, volumePerPixel ,
                                                                                                MomentsAboutTheMean.TPM_TYPE_CDIS,
                                                                                                MomentsAboutTheMean.TPM_TYPE_STRING_CDIS );
        
        MomentsAboutTheMean weightAv_chomTPMVals = MomentsAboutTheMean.ComputeWeightedAverages( chomTPMVals, volumePerPixel ,
                                                                                                MomentsAboutTheMean.TPM_TYPE_CHOM,
                                                                                                MomentsAboutTheMean.TPM_TYPE_STRING_CHOM );
        
        MomentsAboutTheMean weightAv_casmTPMVals = MomentsAboutTheMean.ComputeWeightedAverages( casmTPMVals, volumePerPixel ,
                                                                                                MomentsAboutTheMean.TPM_TYPE_CASM,
                                                                                                MomentsAboutTheMean.TPM_TYPE_STRING_CASM );
        
        MomentsAboutTheMean weightAv_ceneTPMVals = MomentsAboutTheMean.ComputeWeightedAverages( ceneTPMVals, volumePerPixel ,
                                                                                                MomentsAboutTheMean.TPM_TYPE_CENE,
                                                                                                MomentsAboutTheMean.TPM_TYPE_STRING_CENE );
        
        MomentsAboutTheMean weightAv_cmaxTPMVals = MomentsAboutTheMean.ComputeWeightedAverages( cmaxTPMVals, volumePerPixel ,
                                                                                                MomentsAboutTheMean.TPM_TYPE_CMAX,
                                                                                                MomentsAboutTheMean.TPM_TYPE_STRING_CMAX );
        
        MomentsAboutTheMean weightAv_centTPMVals = MomentsAboutTheMean.ComputeWeightedAverages( centTPMVals, volumePerPixel ,
                                                                                                MomentsAboutTheMean.TPM_TYPE_CENT,
                                                                                                MomentsAboutTheMean.TPM_TYPE_STRING_CENT );
        
        MomentsAboutTheMean weightAv_cvarTPMVals = MomentsAboutTheMean.ComputeWeightedAverages( cvarTPMVals, volumePerPixel ,
                                                                                                MomentsAboutTheMean.TPM_TYPE_CVAR,
                                                                                                MomentsAboutTheMean.TPM_TYPE_STRING_CVAR );
        
        MomentsAboutTheMean weightAv_ccorTPMVals = MomentsAboutTheMean.ComputeWeightedAverages( ccorTPMVals, volumePerPixel ,
                                                                                                MomentsAboutTheMean.TPM_TYPE_CCOR,
                                                                                                MomentsAboutTheMean.TPM_TYPE_STRING_CCOR );
        
        // All information has been gathered; now write 
        // the information to a database text file
        FileWriter fileWriter;
        
        String databaseFileName = null;
        
        if ( to_predict == false )
        {
            databaseFileName = System.getProperty( "user.home" ) +
                                      System.getProperty( "file.separator" ) + "CAD" +
                                      System.getProperty( "file.separator" ) + "CAD_dbases_1_i_pp" +
                                      System.getProperty( "file.separator" ) + deseaseLabel + "_1_inst_pp.txt";
        }
            
        if ( to_predict == true )
        {
            databaseFileName = System.getProperty( "user.home" ) +
                                      System.getProperty( "file.separator" ) + "CAD" +
                                      System.getProperty( "file.separator" ) + "CAD_entry_2_predict" +
                                      System.getProperty( "file.separator" ) + deseaseLabel + ".txt";    
        }
       
        File f = new File( databaseFileName );
        
        if ( !f.exists( ) && !f.isDirectory( ) ) 
        { 
            try
            {
                // the database does not exist so create it
                f.createNewFile( );
                
                fileWriter = new FileWriter( databaseFileName, true );
                String labels   = "tissue_type,file_ref,pat_age,ser_i_uid,pat_id,norm_const,poly_name,pixe_numb,exam_volu,";
                labels          += "orig_mean,orig_var,orig_stde,orig_skew,orig_kurt,orig_vava,";
                labels          += "vari_mean,vari_var,vari_stde,vari_skew,vari_kurt,vari_vava,";
                labels          += "stde_mean,stde_var,stde_stde,stde_skew,stde_kurt,stde_vava,";
                labels          += "skew_mean,skew_var,skew_stde,skew_skew,skew_kurt,skew_vava,";
                labels          += "kurt_mean,kurt_var,kurt_stde,kurt_skew,kurt_kurt,kurt_vava,";
                labels          += "vava_mean,vava_var,vava_stde,vava_skew,vava_kurt,vava_vava,";
                labels          += "cmea_mean,cmea_var,cmea_stde,cmea_skew,cmea_kurt,cmea_vava,";
                labels          += "ccon_mean,ccon_var,ccon_stde,ccon_skew,ccon_kurt,ccon_vava,";
                labels          += "cdis_mean,cdis_var,cdis_stde,cdis_skew,cdis_kurt,cdis_vava,";
                labels          += "chom_mean,chom_var,chom_stde,chom_skew,chom_kurt,chom_vava,";
                labels          += "casm_mean,casm_var,casm_stde,casm_skew,casm_kurt,casm_vava,";
                labels          += "cene_mean,cene_var,cene_stde,cene_skew,cene_kurt,cene_vava,";
                labels          += "cmax_mean,cmax_var,cmax_stde,cmax_skew,cmax_kurt,cmax_vava,";
                labels          += "cent_mean,cent_var,cent_stde,cent_skew,cent_kurt,cent_vava,";
                labels          += "cvar_mean,cvar_var,cvar_stde,cvar_skew,cvar_kurt,cvar_vava,";
                labels          += "ccor_mean,ccor_var,ccor_stde,ccor_skew,ccor_kurt,ccor_vava\n";
                fileWriter.write( labels );
                fileWriter.close( );
            }
            catch ( IOException ex )
            {
                // do nothing
                ex.printStackTrace( );
            }
        }
        
        try
        {
            fileWriter = new FileWriter( databaseFileName, true );
            
            
                
//            MomentsAboutTheMean orig = origTPMVals.get( i );
//            MomentsAboutTheMean vari = variTPMVals.get( i );
//            MomentsAboutTheMean stde = stdeTPMVals.get( i );
//            MomentsAboutTheMean skew = skewTPMVals.get( i );
//            MomentsAboutTheMean kurt = kurtTPMVals.get( i );
//            MomentsAboutTheMean vava = vavaTPMVals.get( i );
//            MomentsAboutTheMean cmea = cmeaTPMVals.get( i );
//            MomentsAboutTheMean ccon = cconTPMVals.get( i );
//            MomentsAboutTheMean cdis = cdisTPMVals.get( i );
//            MomentsAboutTheMean chom = chomTPMVals.get( i );
//            MomentsAboutTheMean casm = casmTPMVals.get( i );
//            MomentsAboutTheMean cene = ceneTPMVals.get( i );
//            MomentsAboutTheMean cmax = cmaxTPMVals.get( i );
//            MomentsAboutTheMean cent = centTPMVals.get( i );
//            MomentsAboutTheMean cvar = cvarTPMVals.get( i );
//            MomentsAboutTheMean ccor = ccorTPMVals.get( i );
               
            // Do writing!
            fileWriter.write( "" + deseaseLabel + "," );
            fileWriter.write( "" + weightAv_origTPMVals.getDicomImageFileReference( ) + "," );
            fileWriter.write( "" + patientAge + "," );
            
            fileWriter.write( "" + weightAv_origTPMVals.getInstanceSeriesUID( ) + ",");
            fileWriter.write( "" + weightAv_origTPMVals.getPatientID( ) + "," );
             
            fileWriter.write( "" + weightAv_origTPMVals.getNormalizationConstant( ) + "," );
            fileWriter.write( "" + weightAv_origTPMVals.getPolygonName( ) + "," );
            fileWriter.write( "" + weightAv_origTPMVals.getNumberOfROIPixels( ) + "," );
            fileWriter.write( "" + weightAv_origTPMVals.getExaminedVolumeSize( ) + "," );
               
            // orig
            fileWriter.write( "" + weightAv_origTPMVals.getMean( ) + "," );
            fileWriter.write( "" + weightAv_origTPMVals.getVariance( ) + "," );
            fileWriter.write( "" + weightAv_origTPMVals.getStDev( ) + "," );
            fileWriter.write( "" + weightAv_origTPMVals.getSkewness( ) + "," );
            fileWriter.write( "" + weightAv_origTPMVals.getKurtosis( ) + "," );
            fileWriter.write( "" + weightAv_origTPMVals.getVarianceOfVariance( ) + "," );
               
            // vari
            fileWriter.write( "" + weightAv_variTPMVals.getMean( ) + "," );
            fileWriter.write( "" + weightAv_variTPMVals.getVariance( ) + "," );
            fileWriter.write( "" + weightAv_variTPMVals.getStDev( ) + "," );
            fileWriter.write( "" + weightAv_variTPMVals.getSkewness( ) + "," );
            fileWriter.write( "" + weightAv_variTPMVals.getKurtosis( ) + "," );
            fileWriter.write( "" + weightAv_variTPMVals.getVarianceOfVariance( ) + "," );
              
            // stde
            fileWriter.write( "" + weightAv_stdeTPMVals.getMean( ) + "," );
            fileWriter.write( "" + weightAv_stdeTPMVals.getVariance( ) + "," );
            fileWriter.write( "" + weightAv_stdeTPMVals.getStDev( ) + "," );
            fileWriter.write( "" + weightAv_stdeTPMVals.getSkewness( ) + "," );
            fileWriter.write( "" + weightAv_stdeTPMVals.getKurtosis( ) + "," );
            fileWriter.write( "" + weightAv_stdeTPMVals.getVarianceOfVariance( ) + "," );
               
            // skew
            fileWriter.write( "" + weightAv_skewTPMVals.getMean( ) + "," );
            fileWriter.write( "" + weightAv_skewTPMVals.getVariance( ) + "," );
            fileWriter.write( "" + weightAv_skewTPMVals.getStDev( ) + "," );
            fileWriter.write( "" + weightAv_skewTPMVals.getSkewness( ) + "," );
            fileWriter.write( "" + weightAv_skewTPMVals.getKurtosis( ) + "," );
            fileWriter.write( "" + weightAv_skewTPMVals.getVarianceOfVariance( ) + "," );
               
            // kurt
            fileWriter.write( "" + weightAv_kurtTPMVals.getMean( ) + "," );
            fileWriter.write( "" + weightAv_kurtTPMVals.getVariance( ) + "," );
            fileWriter.write( "" + weightAv_kurtTPMVals.getStDev( ) + "," );
            fileWriter.write( "" + weightAv_kurtTPMVals.getSkewness( ) + "," );
            fileWriter.write( "" + weightAv_kurtTPMVals.getKurtosis( ) + "," );
            fileWriter.write( "" + weightAv_kurtTPMVals.getVarianceOfVariance( ) + "," );
               
            // vava
            fileWriter.write( "" + weightAv_vavaTPMVals.getMean( ) + "," );
            fileWriter.write( "" + weightAv_vavaTPMVals.getVariance( ) + "," );
            fileWriter.write( "" + weightAv_vavaTPMVals.getStDev( ) + "," );
            fileWriter.write( "" + weightAv_vavaTPMVals.getSkewness( ) + "," );
            fileWriter.write( "" + weightAv_vavaTPMVals.getKurtosis( ) + "," );
            fileWriter.write( "" + weightAv_vavaTPMVals.getVarianceOfVariance( ) + "," );
               
            // cmea
            fileWriter.write( "" + weightAv_cmeaTPMVals.getMean( ) + "," );
            fileWriter.write( "" + weightAv_cmeaTPMVals.getVariance( ) + "," );
            fileWriter.write( "" + weightAv_cmeaTPMVals.getStDev( ) + "," );
            fileWriter.write( "" + weightAv_cmeaTPMVals.getSkewness( ) + "," );
            fileWriter.write( "" + weightAv_cmeaTPMVals.getKurtosis( ) + "," );
            fileWriter.write( "" + weightAv_cmeaTPMVals.getVarianceOfVariance( ) + "," );
               
            // ccon
            fileWriter.write( "" + weightAv_cconTPMVals.getMean( ) + "," );
            fileWriter.write( "" + weightAv_cconTPMVals.getVariance( ) + "," );
            fileWriter.write( "" + weightAv_cconTPMVals.getStDev( ) + "," );
            fileWriter.write( "" + weightAv_cconTPMVals.getSkewness( ) + "," );
            fileWriter.write( "" + weightAv_cconTPMVals.getKurtosis( ) + "," );
            fileWriter.write( "" + weightAv_cconTPMVals.getVarianceOfVariance( ) + "," );
               
            // cdis
            fileWriter.write( "" + weightAv_cdisTPMVals.getMean( ) + "," );
            fileWriter.write( "" + weightAv_cdisTPMVals.getVariance( ) + "," );
            fileWriter.write( "" + weightAv_cdisTPMVals.getStDev( ) + "," );
            fileWriter.write( "" + weightAv_cdisTPMVals.getSkewness( ) + "," );
            fileWriter.write( "" + weightAv_cdisTPMVals.getKurtosis( ) + "," );
            fileWriter.write( "" + weightAv_cdisTPMVals.getVarianceOfVariance( ) + "," );
               
            // chom
            fileWriter.write( "" + weightAv_chomTPMVals.getMean( ) + "," );
            fileWriter.write( "" + weightAv_chomTPMVals.getVariance( ) + "," );
            fileWriter.write( "" + weightAv_chomTPMVals.getStDev( ) + "," );
            fileWriter.write( "" + weightAv_chomTPMVals.getSkewness( ) + "," );
            fileWriter.write( "" + weightAv_chomTPMVals.getKurtosis( ) + "," );
            fileWriter.write( "" + weightAv_chomTPMVals.getVarianceOfVariance( ) + "," );
               
            // casm
            fileWriter.write( "" + weightAv_casmTPMVals.getMean( ) + "," );
            fileWriter.write( "" + weightAv_casmTPMVals.getVariance( ) + "," );
            fileWriter.write( "" + weightAv_casmTPMVals.getStDev( ) + "," );
            fileWriter.write( "" + weightAv_casmTPMVals.getSkewness( ) + "," );
            fileWriter.write( "" + weightAv_casmTPMVals.getKurtosis( ) + "," );
            fileWriter.write( "" + weightAv_casmTPMVals.getVarianceOfVariance( ) + "," );
               
            // cene
            fileWriter.write( "" + weightAv_ceneTPMVals.getMean( ) + "," );
            fileWriter.write( "" + weightAv_ceneTPMVals.getVariance( ) + "," );
            fileWriter.write( "" + weightAv_ceneTPMVals.getStDev( ) + "," );
            fileWriter.write( "" + weightAv_ceneTPMVals.getSkewness( ) + "," );
            fileWriter.write( "" + weightAv_ceneTPMVals.getKurtosis( ) + "," );
            fileWriter.write( "" + weightAv_ceneTPMVals.getVarianceOfVariance( ) + "," );
               
            // cmax
            fileWriter.write( "" + weightAv_cmaxTPMVals.getMean( ) + "," );
            fileWriter.write( "" + weightAv_cmaxTPMVals.getVariance( ) + "," );
            fileWriter.write( "" + weightAv_cmaxTPMVals.getStDev( ) + "," );
            fileWriter.write( "" + weightAv_cmaxTPMVals.getSkewness( ) + "," );
            fileWriter.write( "" + weightAv_cmaxTPMVals.getKurtosis( ) + "," );
            fileWriter.write( "" + weightAv_cmaxTPMVals.getVarianceOfVariance( ) + "," );
               
            // cent
            fileWriter.write( "" + weightAv_centTPMVals.getMean( ) + "," );
            fileWriter.write( "" + weightAv_centTPMVals.getVariance( ) + "," );
            fileWriter.write( "" + weightAv_centTPMVals.getStDev( ) + "," );
            fileWriter.write( "" + weightAv_centTPMVals.getSkewness( ) + "," );
            fileWriter.write( "" + weightAv_centTPMVals.getKurtosis( ) + "," );
            fileWriter.write( "" + weightAv_centTPMVals.getVarianceOfVariance( ) + "," );
               
            // cvar
            fileWriter.write( "" + weightAv_cvarTPMVals.getMean( ) + "," );
            fileWriter.write( "" + weightAv_cvarTPMVals.getVariance( ) + "," );
            fileWriter.write( "" + weightAv_cvarTPMVals.getStDev( ) + "," );
            fileWriter.write( "" + weightAv_cvarTPMVals.getSkewness( ) + "," );
            fileWriter.write( "" + weightAv_cvarTPMVals.getKurtosis( ) + "," );
            fileWriter.write( "" + weightAv_cvarTPMVals.getVarianceOfVariance( ) + "," );
               
            // ccor
            fileWriter.write( "" + weightAv_ccorTPMVals.getMean( ) + "," );
            fileWriter.write( "" + weightAv_ccorTPMVals.getVariance( ) + "," );
            fileWriter.write( "" + weightAv_ccorTPMVals.getStDev( ) + "," );
            fileWriter.write( "" + weightAv_ccorTPMVals.getSkewness( ) + "," );
            fileWriter.write( "" + weightAv_ccorTPMVals.getKurtosis( ) + "," );
            fileWriter.write( "" + weightAv_ccorTPMVals.getVarianceOfVariance( ) + "\n" );
            
            
            fileWriter.close( );
        }
        catch ( IOException ex )
        {
            // do nothing
        }
    }
    
    
    
    
    /**
     * Saves all information to a Mongo DB (obsolete).
     */
    public void saveAllROIInformationToMongoDB( )
    {
        
        Iterator it = m_sliceViews.iterator( );
        DecimalFormat formatter = new DecimalFormat( "###.###" );

        String fileName = getFilePathAndName( m_currentView.getImageSeriesData( ).getFileInfo( 0 ).getSliceLocation( )     + "_" +
                                              m_currentView.getImageSeriesData( ).getFileInfo( 0 ).getSeriesDescription( ) + "_" + "_verbose_DTPA_rep.txt", "", "Save" );
//        MongoClient mongoClient = null;
//        try
//        {
//            mongoClient = new MongoClient( "cray1.insel.ch" );
//            DB db = mongoClient.getDB( "zeph" );
//            db.authenticate("slotboom", "lakritze".toCharArray());
//            db.command("ping").throwOnError();
//            List<BasicDBObject> data = new ArrayList<BasicDBObject>( );
//
//            while ( it.hasNext( ) )
//            {
//
//                m_currentView = (DicomSliceCanvasPolyDraw)it.next( );
//                m_currentView.writeROIInformationToMongoDB( data );
//
//            }
//        } catch (UnknownHostException ex) {
//            ex.printStackTrace();
//        } finally {
//            if( mongoClient != null ) {
//                mongoClient.close( );
//            }
//        }
//        
//        for( Map map : extractor.getResult() ) {
//            dao.save( dao.mapToMongo( map ) );
//        }
    }

    /**
     * Return the normalization factor of the 
     * @param endBaseLine
     * @param perfMaximum
     * @param beginReperfusion
     * @return
     */
    public float getNormalizeOnTimeSeriesFactor( int endBaseLine, 
                                                 int perfMaximum, 
                                                 int beginReperfusion )
    {
        // First select the image series containing the difference images
        DicomSliceCanvasPolyDraw diffView = this.getView( 1 );

        float averageOfDiffenceImages = 0f;

        int   cntr = 0;
        float meanPixelValue = 0f;
        float cbv_surrogate_marker = 0f;
        
        float globMax = -Float.MIN_VALUE;
        int   maxSliceNumber = -1;
        int[] medians = new int[ beginReperfusion - endBaseLine + 1 ];
        int[] means   = new int[ beginReperfusion - endBaseLine + 1 ];
        
        for ( int i = endBaseLine; i <= perfMaximum; i++ )
        {
            int[] values  = diffView.getPixelValues( i, diffView.getSelectedPolygon( ) );

            // medians[i-endBaseLine] = SignalTools.getMedian( values );
            // means[i-endBaseLine]   = (int)SignalTools.getAverage( values );

            meanPixelValue = SignalTools.getAverage( values );
            
            cbv_surrogate_marker = cbv_surrogate_marker + (float)meanPixelValue;

        }
        
        for ( int i = perfMaximum; i <= beginReperfusion; i++ )
        {
            int[] values  = diffView.getPixelValues( i, diffView.getSelectedPolygon( ) );

            // medians[i-endBaseLine] = SignalTools.getMedian( values );
            // means[i-endBaseLine]   = (int)SignalTools.getAverage( values );

            meanPixelValue = SignalTools.getAverage( values );
            
            cbv_surrogate_marker = cbv_surrogate_marker + (float)meanPixelValue;
            
        }
        
        cbv_surrogate_marker = cbv_surrogate_marker / ( beginReperfusion - endBaseLine );
 
        float factor = 1000f / cbv_surrogate_marker;

        System.out.println( "Average mult factor for difference difference " + factor + 
                            " cbv_surrogate_marker " + cbv_surrogate_marker );

        return factor;

    }



    /**
     * Normalizes the raw data and recalculates the difference 
     * and texture maps again.
     * @param viewNumber
     * @return 
     */
    public float normalize( int viewNumber )
    {

        DicomSliceCanvasPolyDraw rawView = this.getView( viewNumber );

        //int[] values1 = rawView.getPixelsOfCurrentlySelectedROI( );

        int[] values  = rawView.getPixelValues( 0, rawView.getSelectedPolygon( ) );

        int medianSignalValue = SignalTools.getMedian( values );

        //int medianSignalValue1 = SignalTools.getMedian( values1 );
        
        System.out.println( "View type number to normalize    : " + viewNumber );
        System.out.println( "Image number number to normalize : " + rawView.getCurrentlyDisplayedImageNumber( ) );

        System.out.println( "Median is  : " + medianSignalValue );
        // System.out.println( "Median1 is : " + medianSignalValue1 );


        float factor = 1000f / medianSignalValue;

        SCANalyzeDefaults defs = SCANalyzeDefaults.getInstance( );

        defs.setProperty( SCANalyzeDefaults.LAST_DTPA_NORMALIZATION_FACTOR ,  ( "" + factor ) );
        defs.save( );

        return factor;
    }

    /**
     * Method iterating over all displayed images.
     * @param verbose
     * @param beginBaseLine
     * @param endBaseLine
     * @param maxPerf
     * @param startReperfusion
     */
    public void saveAllROIInformationToFile( boolean verbose, 
                                             int beginBaseLine, 
                                             int endBaseLine, 
                                             int maxPerf, 
                                             int startReperfusion )
    {

        Iterator it = m_sliceViews.iterator( );
        DecimalFormat formatter = new DecimalFormat( "###.###" );

        String fileName = getFilePathAndName( m_currentView.getImageSeriesData( ).getFileInfo( 0 ).getSliceLocation( ) + "_" +
                                              m_currentView.getImageSeriesData( ).getFileInfo( 0 ).getSeriesDescription( ) + "_" + "_DTPA_rep.txt", "", "Save" );

        FileWriter fileWriter;
        try
        {
            fileWriter = new FileWriter( fileName );

            // First: write the header

            FileInfo fInfo = m_currentView.getImageSeriesData( ).getFileInfo( 0 );

            fileWriter.write( "Patient name       : " + fInfo.getPatientName( ) + "\r\n" );
            fileWriter.write( "Patient ID         : " + fInfo.getPatientID( ) + "\r\n" );
            fileWriter.write( "Date of Birth      : " + fInfo.getPatientBirthDate( ) + " " );
            fileWriter.write( "Age                : " + fInfo.getPatientAge( ) + "\r\n" );
            fileWriter.write( "Examination Date   : " + fInfo.getAcquisitionDate( ) + "\r\n" );
            fileWriter.write( "Examination Time   : " + fInfo.getAcquisitionTime( ) + "\r\n" );
            fileWriter.write( "Series Number      : " + fInfo.getSeriesNumber( ).trim( ) + "\r\n\r\n" );
            fileWriter.write( "Series Description : " + fInfo.getSeriesDescription( ) + "\r\n\r\n" );

            while ( it.hasNext( ) )
            {
                m_currentView = (DicomSliceCanvasPolyDraw)it.next( );
                m_currentView.writeROIInformation( fileWriter, verbose, beginBaseLine, endBaseLine, maxPerf, startReperfusion );

            }
            fileWriter.close( );
        }
        catch ( IOException ex )
        {
            Logger.getLogger( MultiDicomSliceView.class.getName( ) ).log( Level.SEVERE, null, ex );
        }

    }


    /**
     * Returns the filename and directory selected by the user.
     * @param defaultString
     * @param subDirString
     * @param buttonText
     * @return 
     */
    protected String getFilePathAndName( String defaultString, 
                                         String subDirString, 
                                         String buttonText )
    {
        JFileChooser fileChooser = new JFileChooser( );
        String selectedFilePathAndName = "";

        // Create a File object containing the canonical path of the
        // desired directory

        String filePath;

        SCANalyzeDefaults defs = SCANalyzeDefaults.getInstance( );
        defs.load( );
        
        filePath = (String)defs.get( SCANalyzeDefaults.LAST_DTPA_PATH );
        

        if ( filePath == null )
        {
            filePath = m_currentView.getImageSeriesData( ).getFileInfo( 0 ).getDirectory( );
        }

        String patientName = m_currentView.getImageSeriesData( ).getFileInfo( 0 ).getPatientName( );
        String examinationDate = m_currentView.getImageSeriesData( ).getFileInfo( 0 ).getStudyDate( );

        String outputPath = filePath + System.getProperty( "file.separator" ) + subDirString;

        File f = new File( outputPath );
        if ( f.exists( ) == false )
        {
            f.mkdir( );
        }

        // Set the current directory
        fileChooser.setCurrentDirectory( f );

        int result;
        fileChooser.setSelectedFile( new File( patientName + "_" + examinationDate + "_" + defaultString ) );

        result = fileChooser.showDialog( this, buttonText );

        File selected_file = new File( "" );

        switch ( result )
        {
            case JFileChooser.APPROVE_OPTION:   // Approve Open button was pressed
                                                selected_file = fileChooser.getSelectedFile( );
                                                selectedFilePathAndName = selected_file.getPath( );
                                                defs.setProperty( SCANalyzeDefaults.LAST_DTPA_PATH,
                                                                  new File( selectedFilePathAndName ).getParent( ) );
                                                defs.save( );

                                                break;
            case JFileChooser.CANCEL_OPTION:    // Cancel or close dialog was selected
                                                break;
            case JFileChooser.ERROR_OPTION:     // the selected process did not terminate properly
                                                break;
        }

        return selectedFilePathAndName;
    }
    
    /**
     * Saves the user defined regions in the normal (non relative) image group.
     */
    public void saveRegions( ) 
    {

        Globals globs = Globals.getInstance( );

        if ( m_currentView != null )
        {
            // save to reg file
            String fileName = m_currentView.getTitle( );
            int i = fileName.lastIndexOf( "." );
            if (i == -1) i = fileName.length( );
            String nameRoot = fileName.substring( 0, i );
            String regName = nameRoot + ".reg";
            // check if such a file already exists
            if ( FileUtils.fileExists( regName ) ) 
            {
                QuestionDialog qDlg = new QuestionDialog( m_parent );
                if (! qDlg.ask( "File exists. Ok to overwrite?" ) ) return;
                FileUtils.fileDelete( regName );
            }
            RegFile f = new RegFile( regName, m_currentView.getImageWidth( ), m_currentView.getImageHeight( ) );
            if ( f.open( "rw" ) ) 
            {
                f.writeHeader( );
                for ( int k = 0; k < m_currentView.getNumberOfPolygons( ); k++ ) 
                {
                    f.writePolygonData( m_currentView.getPolygon( k ), m_currentView.getZoom( ) );
                }
                boolean state = f.finish( );
                if ( globs.getConfirmAfterSave( ) )
                {
                    if ( state )
                    { 
                        Dialogs.messageBox( m_parent, "Succesfully saved", "Save ROIs", MessageBox.MB_OK ); 
                    }
                    else 
                    {
                        Dialogs.messageBox( m_parent, "Save failed", "Save ROIs", MessageBox.MB_OK );
                    }
                }
            }
            String tifName = nameRoot + ".ar1";
            System.out.print( "Writing tiff image " + tifName + "..." );
            int[] rData = createRegionMap( );
            Tiff t = new Tiff( );
            t.setImage( rData, m_currentView.getImageWidth( ), m_currentView.getImageHeight( ) );
            if ( Tiff.TIFF_OK != t.writeGrayLevel( tifName, 256 ) )
            {
                System.out.println( "failed" );
            }
            else 
            {
                System.out.println( "ok" );
            }
        }
    }
    
    /**
     * Creates a region map of the currently selected image.
     * @return 
     */    
    public int[] createRegionMap( )
    {
        int imSize = m_currentView.getImageWidth( ) * m_currentView.getImageHeight( );
        int numRegions = m_currentView.getNumberOfPolygons( );
        int[] data = new int[imSize];
        RegioPolygon p = null;
        for ( int y = 0; y < m_currentView.getImageHeight( ); y++ )
        {
            int i = y * m_currentView.getImageWidth( );
            int yz = (int)( y * m_currentView.getZoom( ) );
            for ( int x = 0; x < m_currentView.getImageWidth( ); x++ )
            {
                int xz = (int)( x * m_currentView.getZoom( ) );
                int k = 0;
                while ( k < numRegions )
                {
                    p = m_currentView.getPolygon( k );
                    if ( p.contains( xz, yz ) ) break;
                    else k++;
                }
                if ( k < numRegions )
                {
                    data[ i + x ] = p.m_index;
                    // System.out.println("region "+p.m_index+" at ("+x+","+y+")");
                }
                else
                { 
                    data[ i + x ] = 0;
                }
            }
        }
        return data;
    }
    
    /**
     * Closes the selected DicomSliceCanvasPolyDraw views.
     */
    public void closeSelectedViews( )
    {
//        if ( m_sliceViews.size( ) == m_selectedViews.size( ) )
//        {
//            closeAllViews( );
//            return;
//        }
        if ( m_sliceViews.size( ) > 1 )
        {
            int i = 0;
            for ( int k = 0; k < m_selectedViews.size( ); k++ )
            {
                DicomSliceCanvasPolyDraw c = m_selectedViews.elementAt( k );
                i = Math.min( m_sliceViews.indexOf( c ), m_sliceViews.size( ) - 2 );
                m_sliceViews.removeElement( c );
                remove( c );
            }

            m_selectedViews.removeAllElements( );

            setCurrentView( m_sliceViews.elementAt( i ) );
        
            setupLayout( );
        }
        else
        {
            removeAllViews( );
            repaint( );
        }
    }
    
    /**
     * Closes all the normal views and terminates the application
     * Note this routine does not quit what is expected from it name
     * must be changed in the future.
     */
    public void closeAllViews( ) 
    {
        if ( m_zoomWin != null )
        {
            m_zoomWin.dispose( );
        }
        if ( m_exitOnClose )
        {
            System.exit( 0 );
        }
        else
        {
            m_parent.dispose( );
        }
    }
    
    /**
     * Sets the global zoom factor. 
     * @param zoomFactor
     */
    public void setGlobalZoomFactor( float zoomFactor )
    {
        
        System.out.println( "zoom Factor : " + zoomFactor );

        m_globalZoomFactor = zoomFactor;
        setupLayout( );
    }

    /**
     * Returns the global zoom factor. 
     * @return 
     */
    public float getGlobalZoomFactor( )
    {
        return m_globalZoomFactor;
    }

    /**
     * Returns the reference the the currentView object.
     * @return 
     */
    public DicomSliceCanvasPolyDraw getCurrentView( )
    {
        return m_currentView;
    }
    
    /**
     * Creates a report file containing region information of all currently 
     * displayed DicomSliceCanvasPolyDraw objects, where m_currentDisplay 
     * equals NORMAL_DISPLAY.
     */
    public void createRegioReportFile( )
    {
        String file_name = "";
        
        if ( this.getCurrentView( ) != null )
        {
            DicomSliceCanvasPolyDraw sliceCanvas = getCurrentView( );
            FileInfo finfo = sliceCanvas.getDicomFileInfo( );
            String default_name = finfo.getPatientName( ).replace( ' ','_' ) + "_" + "Slice_" + finfo.getInstanceNumber( ).trim( ) + ".inf";
            
            file_name = getFilePathAndName( default_name );
        }
        
        if ( file_name.length( ) != 0 )
        {
            try
            {
                FileWriter fileWriter = new FileWriter( file_name );
                Iterator it = m_sliceViews.iterator( );
                DecimalFormat formatter = new DecimalFormat( "###.00" );
                boolean createdHeader = false;
            
                while ( it.hasNext( ) )
                {
                    DicomSliceCanvasPolyDraw canvas = (DicomSliceCanvasPolyDraw)it.next( );
                    FileInfo finfo = canvas.getDicomFileInfo( );
                    if ( createdHeader == false )
                    {
                        fileWriter.write( "Patient name :" + finfo.getPatientName( ) + "\t");
                        fileWriter.write( "Birthdate : " + finfo.getPatientBirthDate( ) + " " );
                        fileWriter.write( "(Age : " + finfo.getPatientAge() + ")\r\n\r\n" );
                        fileWriter.write( "Examination date : " + finfo.getAcquisitionDate( ) + "\t" );
                        fileWriter.write( "Examination time : " + finfo.getAcquisitionTime( ) + "\r\n\r\n" );
                        fileWriter.write( "Slice position : " + finfo.getSliceLocation( ) + "\r\n" );
                        createdHeader = true;
                    }
                
                    fileWriter.write( "File short name : " + canvas.getShortImageTitle( ) + "     " );
                    fileWriter.write( "Slice number : " + finfo.getInstanceNumber( ).trim( ) +
                                      " (" + finfo.getSliceLocation( ) + ")\r\n\r\n" );
                    for ( int i = 0; i < canvas.getNumberOfPolygons( ); i++ )
                    {
                        canvas.calculateROIParameters( i );
                        fileWriter.write( "roi# " + i + " - " + canvas.getPolygonName( i ) + "\t" );
                    
                        String msg = finfo.getPatientName( ).replace(' ','_') + "\t" + canvas.getShortImageTitle( ).replace( '-', '_' )+ "\t" + finfo.getAcquisitionDate( ) + "\t" + "\tAv:\t" + formatter.format( canvas.m_av ) + "\tSt.dev:\t" + formatter.format( canvas.m_stdev ) + "\tPixels:\t" + formatter.format( canvas.m_roiSize ) + "\tVolume:\t" + formatter.format( canvas.m_volumePerPixel * canvas.m_roiSize / 1000.0 ) + "\tcm3" ;                
                        fileWriter.write( msg );
                        fileWriter.write( "\r\n" );
                    }
                    fileWriter.write( "\r\n" );
                }
                fileWriter.close( );
            }
            catch ( IOException e )
            {
                System.out.println( "Error creating file : " + file_name );
            }
        }
    }
    
    
    /**
     * Returns the filename and directory selected by the user.
     */
    private String getFilePathAndName( String defaultString )
    {       
        JFileChooser fileChooser = new JFileChooser( );
        String selectedFilePathAndName = "";

        Globals globs = Globals.getInstance( );
        // Create a File object containing the canonical path of the 
        // desired directory
        File f = new File( globs.imageDestinationPath );
        
        // Set the current directory
        fileChooser.setCurrentDirectory( f );
            
        fileChooser.setSelectedFile( new File( defaultString ) );
        int result = fileChooser.showSaveDialog( this );
           
        File selected_file = new File( "" );
            
        switch ( result )
        {
            case JFileChooser.APPROVE_OPTION:   // Approve Open button was pressed
                                                selected_file = fileChooser.getSelectedFile( );
                                                selectedFilePathAndName = selected_file.getPath( ); 
                                                break;
            case JFileChooser.CANCEL_OPTION:    // Cancel or close dialog was selected
                                                break;
            case JFileChooser.ERROR_OPTION:     // the selected process did not terminate properly
                                                break;
        }
        
        return selectedFilePathAndName; 
    }
    
    /**
     * Returns a reference to a DicomSliceCanvasPolyDraw object having number 
     *  slice_number.
     * @param slice_number
     * @return 
     */
    public DicomSliceCanvasPolyDraw getSlice( int slice_number )
    {
        return m_sliceViews.elementAt( slice_number );
    }
    
    /**
     * Returns the number of normalSliceViews.
     * @return 
     */
    public int getNumberOfSliceViews( )
    {
        return m_sliceViews.size( );
    }
    
    /**
     * Mirrors the currently selected region and add this mirror to the list
     * of polygons belonging to the DicomSliceCanvasPolyDraw object.
     */
    public void mirrorRegion( )
    {
        if ( m_currentView.getSelectedPolygon( ) == null )
        {
            Dialogs.messageBox( m_parent, 
                                "Select a region to mirror", "Error", 
                                MessageBox.MB_OK );
            return;
        }
        m_currentView.addMirroredPolygon( );
    }
    
    /**
     * Deletes the regions from a normal image view.
     */
    public void deleteSelectedRegion( ) 
    {
        if ( m_currentView != null )
        {
            m_currentView.deleteSelectedPoly( );
        }
    }
    
    /**
     * Deletes all the polygons defined for the currently selected normal view.
     */
    public void deleteAllRegionsOfCurrentView( ) 
    {
        if ( m_currentView != null )
        {
            m_currentView.deleteAllPolygonsOfCurrentlyDisplayedImageOnly( );
        }
    }
    
    /**
     * Deletes all the polygons defined for the currently selected normal view.
     */
    public void deleteAllRegionsOfAllViews( ) 
    {
        
        for ( int i = 0; i< m_sliceViews.size( ); i++ )
        {
            DicomSliceCanvasPolyDraw p = m_sliceViews.get( i );
            p.deleteAllPolygonsOfAllImagesWithoutNotification( );
        }
        
    }
     
    /**
     * 
     * Utility method giving caller the possibility to access information
     * about the currently displayed and selected DicomSliceCanvasPolydraw
     *   
     * @return
     */
    public DicomSliceCanvasPolyDraw getCurrentSliceCanvasPolyDraw( )
    {
        return m_currentView;
    }

    /**
     * Calculates the region properties: number of pixels, average pixel value
     * and standard deviation.
     */
    public void regionProperties( ) 
    {
        // get a reference on the selected polygon        

        float   av = 0.0f;
        float   stdev = 0.0f;
        int     roiSize = 0;
        
        float zoom = 1f / m_currentView.getZoom( );
        RegioPolygon rpol = m_currentView.getSelectedPolygon( );
        
        
        if ( rpol != null )
        {

            Polygon roi = (Polygon)rpol.scaled( zoom );

            Rectangle bb = roi.getBounds( );
            if ( bb.x < 0 )
            {    
                bb.x = 0;
            }
            if ( bb.x + bb.width >= m_currentView.getImageWidth( ) ) 
            {
                bb.width = m_currentView.getImageWidth( ) - bb.x - 1;
            }
            if ( bb.y < 0 )
            {
                bb.y = 0;
            }
        
            if ( bb.y + bb.height >= m_currentView.getImageHeight( ) ) 
            {
                bb.height = m_currentView.getImageHeight( ) - bb.y - 1;
            }
        
            int imageSize = m_currentView.getImageWidth( ) * m_currentView.getImageHeight( );

            roiSize = 0;
        
            int[] rawData = m_currentView.getRawData( );
            // Calculate the average of the ROI
            for ( int y = bb.y; y <= bb.y + bb.height; y++ ) 
            {
                int yoffset = y * m_currentView.getImageWidth( );
                for ( int x = bb.x; x <= bb.x + bb.width; x++ ) 
                {
                    if ( roi.contains( x, y ) )
                    {
                        roiSize++;
                        int xyoffset = yoffset;
                        av += rawData[ x + m_currentView.getImageWidth( ) * y ];
                    }
                }
            }

            av = av / roiSize;

            // Calculate the standard deviation of the current ROI
            float diff = 0.0f;
            // reset the roi size
            roiSize = 0;
        
            for ( int y = bb.y; y <= bb.y + bb.height; y++ )
            {
                int yoffset = y * m_currentView.getImageWidth( );
                for ( int x = bb.x; x <= bb.x + bb.width; x++ )
                {
                    if ( roi.contains( x, y ) )
                    {
                        roiSize++;
                        diff = av - rawData[ x + m_currentView.getImageWidth( ) * y ];
                        stdev += diff * diff;
                    }
                }
            }

            stdev = (float)Math.sqrt( (double)stdev / (float)roiSize ) ;
        
        
            // has to be changed
            PolygonParameterDisplayer polDisp = new PolygonParameterDisplayer( this, roiSize, av, stdev );
        }
        else
        {
            Dialogs.messageBox( m_parent, "Cannot compute ROI properties: no ROI selected!", "Cannot compute ROI propperties" );
        }
    }
     
    /**
     * Method called as soon as the selected polygon has changed.
     * @param e
     */
    @Override
    public void polygonChanged( PolygonEvent e ) 
    {
        m_currentView.calculateSelectedROIParameters( );
        DecimalFormat formatter = new DecimalFormat( "###.#" );
        m_polygonInformation = "<HTML> <FONT COLOR = BLACK>Region name: <FONT COLOR = GREEN>'" + ( (RegioPolygon)e.polygon ).m_label + 
                     "'|<FONT COLOR = BLUE>    Average pixel value: <FONT COLOR = RED>" + 
                     formatter.format( m_currentView.m_av ) + 
                     "|<FONT COLOR = BLUE>     St.dev : <FONT COLOR = RED>" + 
                     formatter.format(m_currentView.m_stdev) + "|<FONT COLOR = BLUE>     Pixels  : <FONT COLOR = RED>" +
                     m_currentView.m_roiSize + "|<FONT COLOR = BLUE>    Volume: <FONT COLOR = RED>" + 
                     formatter.format( m_currentView.m_roiSize * m_currentView.m_volumePerPixel / 1000.0  ) + " cm3 </HTML>";
        
        Iterator it = m_actionListeners.iterator( );
        while( it.hasNext( ) )
        {
            ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, POLYGON_CHANGE_ACTION_EVENT, "" ) );
        }
    }    
    
    @Override
    public void polygonDrag( PolygonEvent e ) 
    {
        // not implemented yet!
    }
    
    @Override
    public void polygonRemoved( PolygonEvent e ) 
    {
        System.out.println( "Polygon removed event." );
    }
    
    /**
     * This method is called the moment a polygon is selected.
     * @param e
     */
    @Override
    public void polygonSelect( PolygonEvent e ) 
    {
        
        m_selectedPolygon = (RegioPolygon)e.polygon;
        
        if ( m_selectedPolygon != null )
        {
            m_currentView.calculateSelectedROIParameters( );
        
            m_selectedPolygon = (RegioPolygon)e.polygon;
        
            DecimalFormat formatter = new DecimalFormat( "###.#" );
            m_polygonInformation = "<HTML> <FONT COLOR = BLACK>Region name: <FONT COLOR = GREEN>'" + ( (RegioPolygon)e.polygon ).m_label + 
                         "'|<FONT COLOR = BLUE>    Average pixel value: <FONT COLOR = RED>" + 
                         formatter.format( m_currentView.m_av ) + 
                         "|<FONT COLOR = BLUE>     St.dev : <FONT COLOR = RED>" + 
                         formatter.format(m_currentView.m_stdev) + "|<FONT COLOR = BLUE>     Pixels  : <FONT COLOR = RED>" +
                         m_currentView.m_roiSize + "|<FONT COLOR = BLUE>    Volume: <FONT COLOR = RED>" + 
                         formatter.format( m_currentView.m_roiSize * m_currentView.m_volumePerPixel / 1000.0  ) + " cm3 </HTML>";
        
            Iterator it = m_actionListeners.iterator( );
            while( it.hasNext( ) )
            {
                ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, POLYGON_SELECT_ACTION_EVENT, "" ) );
            }
        }
    }
    
    /**
     * This method is called when a mouse button is clicked.
     * @param mouseEvent
     */
    @Override
    public void mouseClicked( MouseEvent mouseEvent )
    {
        // not implemented yet!
    }
    
    @Override
    public void mouseEntered( MouseEvent mouseEvent ) 
    {
        // not implemented yet!
    }
    
    @Override
    public void mouseExited( MouseEvent mouseEvent )
    {
        // not implemented yet!
    }
    
    /** 
     * Called whenever a mouse button is pressed.
     * @param e
     */
    @Override
    public void mousePressed( MouseEvent e )
    {                
        if ( ( e.getModifiers( ) & e.BUTTON1_MASK ) > 0 ) 
        {
            DicomSliceCanvasPolyDraw c = ( DicomSliceCanvasPolyDraw )e.getSource( );
            setCurrentView( c );
            if ( m_zoomWin != null )
            {
                m_zoomWin.setImage( c );
            }
            if ( ( e.getModifiers( ) & e.SHIFT_MASK ) == 0 )
            {
                m_selectedViews.removeAllElements( );
            }
            if ( !m_selectedViews.contains( c ) ) 
            {
                m_selectedViews.addElement( c );
            }
        }
    }
    
    /**
     * Called whenever the mouse button is released.
     * @param mouseEvent
     */
    @Override
    public void mouseReleased( MouseEvent mouseEvent )
    {
        int modifiers = mouseEvent.getModifiers( );
	if ( ( modifiers & mouseEvent.BUTTON2_MASK ) > 0 ) 
        {
            int brightness = getCurrentView( ).getBrightness( );
            int contrast = getCurrentView( ).getContrast( );
            if ( ( m_groupBrightness != brightness ) || ( m_groupContrast != contrast ) )
            {
            
                m_groupBrightness = brightness;
                m_groupContrast = contrast;
            
                for ( int k = 0; k < getNumberOfSliceViews( ); k++ ) 
                {
                    DicomSliceCanvasPolyDraw c = getSlice( k );
                    
                    if ( m_equalize == true )
                    {
                        c.setBrightness( brightness );
                        c.setContrast( contrast );
                        c.setForcedMinMaxRawDataValues( m_minPixelValueOfAllLoadedImages, 
                                                        m_maxPixelValueOfAllLoadedImages );
                        c.setForcedMinMaxRawDataValue( true );
                        c.update( true );
                    }
                    else
                    {
                        c.setForcedMinMaxRawDataValue( false );
                        c.update( true );
                    }
                }
                // next line commented out: after windowing layout set back to m_zoom = 1.0;
                //setupLayout( );
            }
        }
    }
    
    /**
     * Is called in response to some event, eg by pressing a button.
     */
    @Override
    public void actionPerformed( ActionEvent e ) 
    {
        Object sender = e.getSource( );
        if ( sender == m_zoomWin ) 
        {
            m_zoomWin = null;
        }
        if ( sender instanceof DicomSliceCanvasPolyDraw )
        {

            if ( e.getID( ) == DicomSliceCanvasPolyDraw.PROJECT_VOLUME_IN_OTHER_IMAGE_STACKS_EVENT )
            {
               DicomSliceCanvasPolyDraw eventEnvoker = (DicomSliceCanvasPolyDraw)e.getSource( );

               int imageWidthInPoints = eventEnvoker.getImageWidth( );
               int imageHeightInPoints = eventEnvoker.getImageHeight( );

               ArrayList< LinkedList<boolean[][]> > list = eventEnvoker.getImageVolumeOfInterestOverlayList( );

               for ( int i = 0; i < list.size( ) ; i++ )
               {

                    System.out.println( "Overlay of image : " + i );
                    LinkedList<boolean[][]> overlaysList = (LinkedList<boolean[][]>)list.get( i );

                    for ( int kk = 0; kk < overlaysList.size( ); kk++ )
                    {
                        boolean[][] overlay =  overlaysList.get( kk );

                        for ( int j = 0; j < imageWidthInPoints; j++ )
                        {
                            for ( int k = 0; k < imageHeightInPoints; k++ )
                            {
                                if ( overlay[j][k] == true )
                                {

                                    Point3D point = eventEnvoker.compute3DCoordinate( i, j, k );

                                    Iterator it = m_sliceViews.iterator( );
                                    while ( it.hasNext( ) )
                                    {
                                        DicomSliceCanvasPolyDraw canvas = (DicomSliceCanvasPolyDraw)it.next( );

                                        LinkedHashMap< String, Plane3D > geoObjmap2 = canvas.getImageSeriesData( ).getGeometricImagePlanesObjects( );

                                        if ( canvas != eventEnvoker )
                                        {
                                            double minDist = Double.MAX_VALUE;
                                            int sliceCounter = 0;
                                            int minDistSliceNumber = -1;

                                            Iterator it2 = geoObjmap2.keySet( ).iterator( );
                                            Plane3D minPlane = null;

                                            while ( it2.hasNext( ) )
                                            {
                                                Plane3D plane = geoObjmap2.get( it2.next( ) );
                                                double distance = plane.getDistanceToPlane( point );
                                                if ( distance < minDist )
                                                {
                                                    minDist      = distance;
                                                    minDistSliceNumber = sliceCounter;
                                                    minPlane = plane;
                                                }
                                                sliceCounter++;
                                            }

                                            Point p2D = minPlane.computeClosestPoint2D( point );
                                            // System.out.println( "Minimal slice number " + minDistSliceNumber + "  2D point x=" + p2D.x + " y=" + p2D.y );
                                            if ( p2D != null )
                                            {
                                                canvas.setVolumeElementPixel( kk, minDistSliceNumber, p2D );
                                            }
                                       }
                                   }
                               }
                           }
                       }
                   }
               }

               // force update of the other canvasses
               Iterator it = m_sliceViews.iterator( );
               while ( it.hasNext( ) )
               {
                   DicomSliceCanvasPolyDraw canvas = (DicomSliceCanvasPolyDraw)it.next( );
                   if ( canvas != eventEnvoker )
                   {
                       canvas.drawImage( false );
                   }
               }

            }

            if ( e.getID( ) == DicomSliceCanvasPolyDraw.MOUSE_DOUBLE_CLICKED_EVENT )
            {
                
                DicomSliceCanvasPolyDraw view = (DicomSliceCanvasPolyDraw)e.getSource( );
                //LinkedHashMap< String, Plane3D > geoObjmap = view.getImageSeriesData( ).getGeometricImagePlanesObjects( );
                Point3D p = view.getLastDoubleClickedPoint( );

                // some test code ..
//                Iterator it = geoObjmap.keySet( ).iterator( );
//                while( it.hasNext( ) )
//                {
//                    Plane3D plane = geoObjmap.get( it.next( ) );
//                    double distance = plane.getDistanceToPlane( p );
//                    System.out.println( "Distance of double clicked point to image " + distance );
//                }

                Iterator it = m_sliceViews.iterator( );


                while ( it.hasNext( ) )
                {
                    DicomSliceCanvasPolyDraw canvas = (DicomSliceCanvasPolyDraw)it.next( );
                    LinkedHashMap< String, Plane3D > geoObjmap2 = canvas.getImageSeriesData( ).getGeometricImagePlanesObjects( );
                    double minDist = Double.MAX_VALUE;
                    int sliceCounter = 0;
                    int minDistSlice = -1;
                    Iterator it2 = geoObjmap2.keySet( ).iterator( );
                    Plane3D minPlane = null;
                    while ( it2.hasNext( ) )
                    {
                        Plane3D plane = geoObjmap2.get( it2.next( ) );
                        double distance = plane.getDistanceToPlane( p );
                        if ( distance < minDist )
                        {
                            minDist      = distance;
                            minDistSlice = sliceCounter;
                            minPlane = plane;
                        }
                        sliceCounter++;
                        // System.out.println( "Distance of double clicked point to image " + distance );
                        
                    }
                    
                    if ( view != canvas )
                    {
                        canvas.displayDicomImage( minDistSlice );
                        Point p2D = minPlane.computeClosestPoint2D( p );
                        
                        canvas.setComputedDoubleClicked2DPoint( p2D );
                        
                    
                        canvas.update( true );
                    }
                    
                    //System.out.println( "-------------------------------------------" );
                }
            }

            if ( e.getID( ) == DicomSliceCanvasPolyDraw.MOUSE_MOVED_EVENT )
            {
                Iterator it = m_sliceViews.iterator( );
                while ( it.hasNext( ) )
                {
                    DicomSliceCanvasPolyDraw sliceCanvas = (DicomSliceCanvasPolyDraw)it.next( );
                    if ( sender == sliceCanvas )
                    {
                        sliceCanvas.setToolTipText( getHtmlToolTipText( sliceCanvas.m_X, sliceCanvas.m_Y ) );
                    }
                }
            }

            if ( e.getID( ) == DicomSliceCanvasPolyDraw.ADD_ROI_MODE_SELECTED_EVENT )
            {
                this.notifyActionListeners( ADD_ROI_MODE_SELECTED_EVENT );
            }
            if ( e.getID( ) == DicomSliceCanvasPolyDraw.CUT_ROI_MODE_SELECTED_EVENT )
            {
                this.notifyActionListeners( CUT_ROI_MODE_SELECTED_EVENT );
            }
            if ( e.getID( ) == DicomSliceCanvasPolyDraw.DELETE_ROI_MODE_SELECTED_EVENT )
            {
                this.notifyActionListeners( DELETE_ROI_MODE_SELECTED_EVENT );
            }
            if ( e.getID( ) == DicomSliceCanvasPolyDraw.SELECT_ROI_MODE_SELECTED_EVENT )
            {
                this.notifyActionListeners( SELECT_ROI_MODE_SELECTED_EVENT );
            }
            if ( e.getID( ) == DicomSliceCanvasPolyDraw.MOVE_ROI_MODE_SELECTED_EVENT )
            {
                this.notifyActionListeners( MOVE_ROI_MODE_SELECTED_EVENT  );
            }
            if ( e.getID( ) == DicomSliceCanvasPolyDraw.RESIZE_ROI_MODE_SELECTED_EVENT )
            {
                this.notifyActionListeners( RESIZE_ROI_MODE_SELECTED_EVENT );
            }

            if ( e.getID( ) == DicomSliceCanvasPolyDraw.POLYGON_CLOSED_EVENT )
            {
                m_workingPolygon = m_currentView.getSelectedPolygon( );
                notifyActionListeners( POLYGON_CLOSED_EVENT );
            }
            
            
            if ( e.getID( ) == DicomSliceCanvasPolyDraw.POLYGON_CUT_EVENT )
            {
                
                m_workingPolygon = m_currentView.getLastCutPolygon( );
                                
                notifyActionListeners( POLYGON_CUT_EVENT );
                System.out.println( "e.getID( ) == DicomSliceCanvasPolyDraw.POLYGON_CUT_EVENT" );
            }
            
            if ( e.getID( ) == DicomSliceCanvasPolyDraw.POLYGON_CHANGED_EVENT )
            {
                
                m_workingPolygon = m_currentView.getSelectedPolygon( );
                notifyActionListeners( POLYGON_CHANGED_EVENT );

            }

            if ( e.getID( ) == DicomSliceCanvasPolyDraw.POLYGON_REMOVED_EVENT )
            {
                System.out.println( "MULTIDICOMVIEW::Polygon removed event" );
                if ( m_currentView.getSelectedPolygon( ) != null )
                {
                    m_workingPolygon = new RegioPolygon( m_currentView.getSelectedPolygon( ) );
                    String lastRemovedROILabel = m_workingPolygon.getLabel( );
                
                    // String lastRemovedROILabel = getCurrentView( ).getLastRemovedRegioPolygonFileName( );
                
                    int deletedROIBelongsToImageNumber = getCurrentView( ).getCurrentlyDisplayedImageNumber( );
                
                    if ( m_isTimeArrayMode == false )
                    {
                        handleRemoveAllROIsWithSameName( lastRemovedROILabel, deletedROIBelongsToImageNumber );
                    }
                    else
                    {
                        handleRemoveAllROIsWithSameName( lastRemovedROILabel, deletedROIBelongsToImageNumber );
                    }
                
                }
            }
            if ( e.getID( ) == DicomSliceCanvasPolyDraw.POLYGON_RENAME_EVENT )
            {
                m_workingPolygon = m_currentView.getSelectedPolygon( );
                notifyActionListeners( POLYGON_RENAME_EVENT );
            }

            if ( e.getID( ) == DicomSliceCanvasPolyDraw.POLYGON_FROM_FILE_LOAD_EVENT )
            {
                
                // get a reference to the loaded object and add them to the other views
                // only if m_isTimeArrayMode = true!
                if ( m_isTimeArrayMode == true )
                {

                    RegioPolygon p = m_currentView.getLastLoadedPolygon( );
                    int loadedPolygonSliceNumber = m_currentView.getLastLoadedPolygonSliceNumber( );

                    System.out.println( "Polygon loaded event!! for slice number : " + loadedPolygonSliceNumber );

                    if ( ( p!= null ) && ( loadedPolygonSliceNumber > -1 ) )
                    {
                        for ( int i = 0; i < m_sliceViews.size( ); i++ )
                        {
                            DicomSliceCanvasPolyDraw c = m_sliceViews.get( i );

                            if ( c != m_currentView )
                            {
                                c.addPolygonNoTrans( loadedPolygonSliceNumber, new RegioPolygon( p ) );
                            }

                        }
                    }
                }

            }

            if ( m_synchronizeViews == true )
            {


                if ( e.getID( ) == DicomSliceCanvasPolyDraw.DISPLAY_PREVIOUS_IMAGE_OF_SLICE_STACK_EVENT )
                {
                    boolean canAllBeDisplayed = true;
                    // first check can all views do display next of previous image?
                    for ( int i = 0; i < m_sliceViews.size( ); i++ )
                    {
                        DicomSliceCanvasPolyDraw c = m_sliceViews.get( i );
                        if ( c.canNextSmallerImageNumberBeDisplayed(  ) == false )
                        {
                            canAllBeDisplayed = false;
                        }
                    }

                    if ( canAllBeDisplayed == true )
                    {
                        for ( int i = 0; i < m_sliceViews.size( ); i++ )
                        {
                            DicomSliceCanvasPolyDraw c = m_sliceViews.get( i );
                            if ( c != (DicomSliceCanvasPolyDraw)sender )
                            {
                                c.displayPreviousDicomImage( );
                            }
                        }
                    }
                    else
                    {
                        ((DicomSliceCanvasPolyDraw)sender).displayNextDicomImage( );
                    }

                }

                if ( ( e.getID( ) == DicomSliceCanvasPolyDraw.DISPLAY_NEXT_IMAGE_OF_SLICE_STACK_EVENT ) )
                {

                    boolean canAllBeDisplayed = true;
                    // first check can all views do display next of previous image?
                    for ( int i = 0; i < m_sliceViews.size( ); i++ )
                    {
                        DicomSliceCanvasPolyDraw c = m_sliceViews.get( i );
                        if ( c.canNextLargerImageNumberBeDisplayed( )  == false )
                        {
                            canAllBeDisplayed = false;
                        }
                    }

                    if ( canAllBeDisplayed == true )
                    {
                        for ( int i = 0; i < m_sliceViews.size( ); i++ )
                        {
                            DicomSliceCanvasPolyDraw c = m_sliceViews.get( i );
                            if ( c != (DicomSliceCanvasPolyDraw)sender )
                            {
                                c.displayNextDicomImage( );
                            }
                        }
                    }
                    else
                    {
                        ((DicomSliceCanvasPolyDraw)sender).displayPreviousDicomImage( );
                    }
                }
            }
        }


    }
    
    
    /**
     * Defines whether tool tip texts should be shown or not.
     * @param show
     */
    public void setShowToolTipText( boolean show )
    {
        m_showToolTipText = show;
    }
    
    /**
     * This method provides the tool tip text for the image views. 
     * @param x_coord
     * @param y_coord
     * @return 
     */ 
    public String getHtmlToolTipText( int x_coord, int y_coord )
    {
        if ( m_showToolTipText == true )
        {
            String htmlTooltipText = "<HTML>";
            DecimalFormat formatter = new DecimalFormat( "###.##" );
            htmlTooltipText += "  ( x, y )  =  ( " + x_coord + ", " + y_coord + " ) " + "<BR>"; 
            
            for ( Enumeration it = m_sliceViews.elements( ); it.hasMoreElements( ); )
            {
                DicomSliceCanvasPolyDraw view = (DicomSliceCanvasPolyDraw)it.nextElement( );
                htmlTooltipText += view.getShortImageTitle( );
                htmlTooltipText += " = ";
                
                htmlTooltipText += formatter.format( view.getRawPixelValue( (int)( x_coord / view.getZoom( )),(int)( y_coord / view.getZoom( ) ) ) );
                
                
                htmlTooltipText += getUnitOfImageType( view.getShortImageTitle( ) );
                htmlTooltipText += "<BR>";
                
                Point3D p = view.compute3DCoordinate( x_coord, y_coord );
                
                if ( p != null )
                {
                    NumberFormat format = new DecimalFormat( "##.#" );

                    htmlTooltipText += " x = " + format.format( p.m_x1 ) + 
                                       " y = " + format.format( p.m_x2 ) + 
                                       " z = "   + format.format( p.m_x3 ) + "<BR>";
                }
                
            }
            htmlTooltipText += "</HTML>";
            return htmlTooltipText;
        }
        else
        {
            return "";
        }
    }
    
    /*
    * This method return the unit of belonging to a image type.
    */
    private String getUnitOfImageType( String shortImageTitle )
    {
        if ( m_relativeViewsAreShown == false )
        {
            if ( ( shortImageTitle.equals( "--ta" ) ) || 
                 ( shortImageTitle.equals( "--tp" ) ) ||
                 ( shortImageTitle.equals( "-mtt" ) ) )
            {
                return " [sec]";
            }
        
            if ( shortImageTitle.equals( "-cbf" )  )
            {
                return " [ml/( 100gr.min)]";
            }
        
            if ( shortImageTitle.equals( "-cbv" ) )
            {
                return " [ml/100gr]";
            }
        
            if ( shortImageTitle.equals( "-ref" ) )
            {
                return " [a.u]";
            }

            return "";
        }
        else
        {
            return "[ % ]";
        }
   }
    
    /**
     * Imports regions for the normal images.
     */
    public void importRegions( )
    {
        // file dialog to import additional reg files
        m_customFileDialog = new CustomFileDialog( m_parent, m_loadPath, "*.reg" );
        m_customFileDialog.openFiles( "*.reg", true );
        if ( m_customFileDialog.getSelectedFile( ) != null )
        {
	    String fileName = m_customFileDialog.getDirectory( ) + m_fileSeparator + m_customFileDialog.getSelectedFile( );
	    m_currentView.importPolysFromFile( fileName );
        }
    }
    
    /**
     * Sets the default path where the custom file dialog is pointing to.
     * @param defaultPath
     */
    public void setDefaultPath( String defaultPath )
    {
        m_defaultPath = defaultPath;
    }
    
    /**
     * Returns the default path where the custom file dialog is pointing to.
     * @return 
     */
    public String getDefaultPath(  )
    {
        return m_defaultPath;
    }
    
    /**
     * Segments the currently selected view.
     * @param loThreshold
     * @param hiThreshold
     */
    public void segmentCurrentView( int loThreshold, int hiThreshold ) 
    {
        float zoom = m_currentView.getZoom( );
        int w = m_currentView.getImageWidth( );
        int h = m_currentView.getImageHeight( );
        int[][] segVol = MemTools.mallocInt2( h, w );
        if ( segVol == null )
        {
            Dialogs.messageBox( m_parent, "Not enough memory for operation" );
            return;
        }

        // m_label the pixels

        //System.out.println( "Labeling pixels ..." );

        for ( int k = 1; k < h - 1; k++ )
        {
            int off = k * w;
            int[] rawData = m_currentView.getRawData( );
            for ( int l = 1; l < w - 1; l++ )
            {
                int pix = rawData[off++];
                if ( pix >= loThreshold && pix <= hiThreshold )
                segVol[k][l] = -1;
            }
        }
	
        // System.out.println("creating borders ...");

        int polygonIndex = 1;
        Point startingPoint = new Point( );
        PolygonUtils pUtils = new PolygonUtils( );
        while ( null != ( startingPoint = pUtils.getPolygonStart( segVol, startingPoint ) ) )
        {
            RegioPolygon P = pUtils.regionBorder( segVol, startingPoint, polygonIndex );
            P.setScalingFactor( zoom );
            // P.reduceVertices( 40.0 );
            m_currentView.addPolygon( P );
            polygonIndex++;
        }

        m_currentView.repaint( );
    }
    
    /**
     * Opens a zoomed normal view.
     */
    public void openZoomedView( ) 
    {
        if ( m_zoomWin == null )
        {
            m_zoomWin = new ZoomedDicomSliceCanvasPolydraw( m_parent, m_currentView );
            m_zoomWin.addActionListener( this );
        }
        else
        {
            m_zoomWin.setImage( m_currentView );
            m_zoomWin.addActionListener( this );
        }
    }
    
    /**
     * Determines the volume of the region of interest enclosed by the selected
     * polygon; For Dicom format files, the routine returns the number of pixels
     * within the region.
     */
    public void regionVolume( ) 
    {
        RegioPolygon p = m_currentView.getSelectedPolygon( );
        if ( p == null )
        {
            Dialogs.messageBox( m_parent, "Select a region", "Error", MessageBox.MB_OK );
            return;
        }
        float z = m_currentView.getZoom( );
        String fileName = m_currentView.getTitle( );
        double[] dims = ImageUtils.getPixelDimensions( fileName, m_customFileDialog.getImageFormatInfo( ) );
        int volP = Math.round( p.getInnerVolume( ) / ( z * z ) );
        long volMM = Math.round(p.getInnerVolumeMM( dims ) / (double)( z * z ) );
        Dialogs.messageBox( m_parent, "Volume = " + volMM + " mm^3  (" + volP + " pixels)" );
    }
    
    /**
     * Returns the polygon information string. 
     * @return 
     */
    public String getPolygonInformation( )
    {
        return m_polygonInformation;
    }
    
    /**
     * Returns a reference to the currently selected RegioPolygon.
     * @return 
     */
    public RegioPolygon getSelectedPolygon( )
    {
        return m_selectedPolygon;
    }
    
    /**
     * Removes all views from the panel.
     */
    public synchronized void removeAllViews( )
    {

        System.out.println( "removeAllViews( ) called!" );

        int numberOfImages = getNumberOfSliceViews( );
        for ( int k =  numberOfImages; k > 0; k-- )
        {
            DicomSliceCanvasPolyDraw c = getSlice( k - 1 );
            m_sliceViews.remove( c );
            m_selectedViews.remove( c );
            remove( c );
        }

        // Clear the data containing structures
        m_sliceViews.removeAllElements( );
        m_selectedViews.removeAllElements( );
        
        m_sliceViews.clear( );
        m_selectedViews.clear( );
        
        m_currentView = null;
        m_workingPolygon = null;
        
        m_selectedPolygon = null;
        m_actionListeners = new LinkedList<>( );
        m_sliceViews = new Vector<>( );
        m_selectedViews = new Vector<>( );
    }
    
    
    /**
     * If equalize is set true all slice canvases will be shown with the 
     * same window scaling.
     * @param equalize
     */
    public void equalizeScale( boolean equalize )
    {
        // Set the member variable in order to be able to set additional
        // windowing
        m_equalize = equalize;
        
        for ( int k = 0; k < getNumberOfSliceViews( ); k++ ) 
        {
            DicomSliceCanvasPolyDraw c = getSlice( k );
                    
            if ( m_equalize == true )
            {
                c.setForcedMinMaxRawDataValues( m_minPixelValueOfAllLoadedImages, 
                                                m_maxPixelValueOfAllLoadedImages );
                c.setForcedMinMaxRawDataValue( true );
                c.update( true );
            }
            else
            {
                c.setForcedMinMaxRawDataValue( false );
                c.update( true );
            }
        }
        setupLayout( );
    }

    
    /**
     * Prints the currently visible part of this object.
     */
    private void paintComponent( Graphics2D g )
    {
        int viewNumber = 0;
        
        Iterator it = m_sliceViews.iterator( );
        while ( it.hasNext( ) )
        {
            DicomSliceCanvasPolyDraw sliceCanvas = (DicomSliceCanvasPolyDraw)it.next( );
            Point point = getTopLeftPosition( viewNumber );
            sliceCanvas.setImagePosition( point.y, point.x );
            sliceCanvas.paintComponent( (Graphics)g );
            sliceCanvas.setImagePosition( 0, 0 );
            viewNumber++;
        }
    }



//    public void printUnderLinux( )
//    {
//       /* Construct the print request specification.
//		* The print data is a Printable object.
//		* the request additonally specifies a job name, 2 copies, and
//		* landscape orientation of the media.
//		*/
//		PrintRequestAttributeSet aset = new HashPrintRequestAttributeSet();
//		aset.add(OrientationRequested.LANDSCAPE);
//		aset.add( new Copies( 2 ) );
//		aset.add( new JobName( "My job", null ) );
//
//		/* Create a print job */
//		PrinterJob pj = PrinterJob.getPrinterJob( );
//		pj.setPrintable( this );
//		/* locate a print service that can handle the request */
//		PrintService[] services =
//			PrinterJob.lookupPrintServices( );
//
//		if ( services.length > 0 )
//        {
//			System.out.println( "selected printer " + services[0].getName( ) );
//			try
//            {
//				pj.setPrintService( services[0] );
//				pj.pageDialog( aset );
//				if( pj.printDialog( aset ) )
//                {
//					pj.print( aset );
//				}
//			}
//            catch ( PrinterException pe )
//            {
//				System.err.println( pe );
//			}
//		}
//
//    }

    /**
     * Prints the currently displayed images
     */
//    public void print( )
//    {
//        //--- Create a new PrinterJob object
//        Print2DPrinterJob printJob = new Print2DPrinterJob( );
//
//        //--- Create a new book to add pages to
//        Book book = new Book( );
//
//        //--- Add the document page using a landscape page format
//        PageFormat documentPageFormat = new PageFormat( );
//        documentPageFormat.setOrientation( PageFormat.LANDSCAPE );
//        book.append ( this, documentPageFormat );
//
//        //--- Tell the printJob to use the Book as the Pageable object
//        printJob.setPageable( book );
//
//        //--- Show the print dialog box. If the user clicks the
//        //--- print button, we then proceed to print, else we cancel
//        //--- the process.
//        try
//        {
////            if ( printJob.printDialog( ) )
//            {
//                printJob.print( );
////            }
//        }
//        catch ( PrinterException e )
//        {
//        }
//    }



    /**
     * Prints the currently displayed images.
     */
    public void print( )
    {
        //--- Create a new PrinterJob object
        PrinterJob printJob = PrinterJob.getPrinterJob( );

        //--- Create a new book to add pages to
        Book book = new Book( );

        //--- Add the document page using a landscape page format
        PageFormat documentPageFormat = new PageFormat( );
        documentPageFormat.setOrientation( PageFormat.LANDSCAPE );
        book.append ( this, documentPageFormat );

        //--- Tell the printJob to use the Book as the Pageable object
        //printJob.setPageable( book );

        //--- Show the print dialog box. If the user clicks the
        //--- print button, we then proceed to print, else we cancel
        //--- the process.
        try
        {
            if ( printJob.printDialog( ) )
            {
                printJob.print( );
            }
        }
        catch ( PrinterException e )
        {
        }
    }

    /**
     * Prints the spectrum.
     * @param g
     * @param pf
     */
    @Override
    public int print( Graphics g, PageFormat pf, int pageIndex ) throws PrinterException
    {
        if ( pageIndex > 0 )
        {
            return Printable.NO_SUCH_PAGE;
        }

        Graphics2D g2d = (Graphics2D)g;
        g2d.translate( pf.getImageableX( ), pf.getImageableY( ) );

        Rectangle componentBounds = this.getBounds(null);
        double scaleX = pf.getImageableWidth( ) / componentBounds.width;
        double scaleY = pf.getImageableHeight( ) / componentBounds.height;

        if ( scaleX < 1 || scaleY < 1 )
        {

            if ( scaleX < scaleY )
            {
                scaleY = scaleX;
            }
            else
            {
                scaleX = scaleY;
            }
        }

        g2d.scale( scaleX, scaleY );

        paintComponent( g2d );

        return Printable.PAGE_EXISTS;
    }

    /**
     *
     * Writes an image of JComponent on graphic context.
     *
     * @param component
     * @param format
     * @param file
     * @param cropX
     * @param cropY
     * @param cropWidth
     * @param cropHeight
     * @throws java.io.IOException
     */
    private void writeImage(  JComponent component, String format,
                              File file, int cropX, int cropY, 
                              int cropWidth, int cropHeight) throws IOException 
    {
        BufferedImage image = new BufferedImage( cropWidth, cropHeight,
                                                 BufferedImage.TYPE_3BYTE_BGR );
        Graphics g = image.createGraphics( );
        g.translate( cropX, cropY );
        component.printAll( g );
        if ( g != null )
        {
            g.dispose( );
        }
        ImageIO.write( image, format, file );
    }
    

    /**
     * Saves the DicomSliceCanvasPolyDraw object in DICOM format on disk
     * NOTE: Currently only the normal images are stored on hard disk
     * This routine can simply be extended to save the relative images
     * as well to disk.
     */
    public void saveAllDicomImagesAsTrueDicom( )
    {
        for ( int k = 0; k < m_selectedViews.size( ); k++ )
        {
            DicomSliceCanvasPolyDraw c = m_selectedViews.elementAt( k );
            String fileName = c.getTitle( );
            int i = fileName.lastIndexOf( "." );
            if ( i == -1 ) i = fileName.length( );
            fileName = fileName.substring( 0, i ) + "SAVED.dcm";
            if ( 0 == c.writeAsTrueDicom( fileName ) )
            {
                System.out.println( "Dicom file : " + fileName + " saved!" );
            }
            else
            {
                System.out.println( "ERROR: DICOM NOT saved" );
            }
        }
        Dialogs.messageBox( m_parent, "" + m_selectedViews.size( ) + " DICOM files written!",
                            "Image Export                     ", MessageBox.MB_OK );
    }


    /**
     * Saves the DicomSliceCanvasPolyDraw object in tiff format on disk
     * NOTE: Currently only the normal images are stored on hard disk
     * This routine can simply be extended to save the relative images
     * as well to disk .
     */
    public void saveAllDicomImagesAsTif( )
    {
        for ( int k = 0; k < m_selectedViews.size( ); k++ )
        {
            DicomSliceCanvasPolyDraw c = m_selectedViews.elementAt( k );
            String fileName = c.getTitle( );
            int i = fileName.lastIndexOf( "." );
            if ( i == -1 ) i = fileName.length( );
            fileName = fileName.substring( 0, i ) + ".tif";
            if ( 0 == c.writeAsTiff( fileName ) )
            {
                System.out.println( "TIF file : " + fileName + " saved!" );
            }
            else
            {
                System.out.println( "ERROR: TIF NOT saved" );
            }
        }
        Dialogs.messageBox( m_parent, "" + m_selectedViews.size( ) + " TIF files written!", "Image Export                     ", MessageBox.MB_OK );
    }
    
    /**
     * Adds an action listener to the list of action listeners. 
     * @param e
     */
    public void addActionListener( ActionListener e )
    {
        m_actionListeners.add( e );
    }
    
 
    /**
     * Notify listeners that something in the object has changed.
     */
    private void notifyActionListeners( int eventType )
    {
        Iterator it = m_actionListeners.iterator( );
        while( it.hasNext( ) )
        {
            ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, eventType, "" ) );
        }
    }
    
    /**
     * Sends image as secondary capture to the selected Dicom node.
     * @param aetitle
     * @param IP_sent_to_number
     * @param dicomListenPortNumber
     * @param dicomFileName name of Dicom file which has to be sent to IP-number
     */
    public void sentSecondaryCaptureToDefaultDicomNode( String aetitle, 
                                                        String IP_sent_to_number, 
                                                        String dicomListenPortNumber, 
                                                        String dicomFileName )
    {
        int returnValue = 0;
        String cmd = "";
        //
        String workingDir = System.getProperty( "user.dir" );
        
        cmd = workingDir + "./lib/dcmsnd.bat -L DCMSND:11113 " + aetitle + "@" + IP_sent_to_number + ":" + dicomListenPortNumber + " " + dicomFileName;

        //cmd = "./lib/dcmsnd -L DINSPECTRO:104 " + aetitle + "@" + IP_sent_to_number + ":" + dicomListenPortNumber + " " + dicomFileName;

        System.out.println( "trying following system command : " + cmd );

        try
        {
            Process p = Runtime.getRuntime( ).exec( cmd );

            InputStream in = p.getInputStream( );
            int c;
            while ( ( c = in.read( ) ) != -1 )
            {
                System.out.print( (char)c ) ;
            }

            returnValue = p.waitFor( );
            in.close( );
        }
        catch ( IOException ioX )
        {
            // not handled yet
            System.out.println( "IOException occurred" );
        }
        catch ( InterruptedException intX )
        {
            // not handled yet
            System.out.println( "InterruptedException occurred" );
        }
    }


    /**
     * Converts a jpg typed images to a Dicom file.
     *
     * @param jpgFileName
     * @param configFileName
     * @param dicomFileName
     */
    public void convertSecondaryCaptureToDicom( String jpgFileName, 
                                                String configFileName, 
                                                String dicomFileName )
    {
        int returnValue;
        String cmd;

        DicomSliceCanvasPolyDraw draw = getCurrentSliceCanvasPolyDraw( );
        String patInfo = draw.getDicomFileInfo( ).getPatientName( );
        patInfo = patInfo + " (DOB:" + draw.getDicomFileInfo( ).getPatientBirthDate( ) + ")";
        String accessionNumber = "Accession number : " + draw.getDicomFileInfo( ).getAccessionNumber( );
        String studyDate = "Date " + draw.getDicomFileInfo( ).getStudyDate( );

        BufferedImage img = null;
        try
        {
            img = ImageIO.read( new File( jpgFileName ) );

            Graphics g = img.getGraphics( );

            g.setColor( Color.BLUE );
            g.drawString( patInfo + " / " + studyDate + " / " + accessionNumber, 1, img.getHeight( ) - 25 );
            g.drawString( "Secondary Capture by " + System.getProperty( "user.name" ) + " - For scientific purposes only - DIN-Bildlabor - Inselspital Bern", 1, img.getHeight( ) - 10 );

            ImageIO.write( img, "jpg", new File( (jpgFileName + "out.jpg" ) ) );

        } catch ( IOException e )
        {
            // nop
            System.out.printf( "IOException occurred \n" );
            e.printStackTrace();
        }
        
        cmd = "./lib/jpg2dcm -c " + configFileName + " --uid-prefix " + " 1.3.12.2.1107.5.2 " + jpgFileName +"out.jpg" + " " + dicomFileName + "\n";
        
        System.out.println( "Processing following system command : " + cmd );

        try
        {
            Process p = Runtime.getRuntime( ).exec( cmd );

            InputStream in = p.getInputStream( );
            int c;
            while ( ( c = in.read( ) ) != -1 )
            {
                System.out.print( (char)c ) ;
            }

            returnValue = p.waitFor( );
            in.close( );
        }
        catch ( IOException ioX )
        {
            // not handled yet
            System.out.println( "IOException occurred" );
        }
        catch ( InterruptedException intX )
        {
            // not handled yet
            System.out.println( "InterruptedException occurred" );
        }

    }

    /**
     * Creates a screen shot of the currently displayed images.
     * @param imageFile
     */
    public void createScreenshot( File imageFile )
    {
        
        // ask user to define a file is not defined yet
        if ( imageFile == null )
        {
            m_screenshotFileReference = getScreenShotFileReference( );
        }
        else
        {
            m_screenshotFileReference = imageFile;
        }
        
        
        if ( m_screenshotFileReference.getName( ).compareTo( "" ) != 0 )
        {
        
            Dimension dim = this.getSize( );
            
            Point pos = this.getLocationOnScreen( );
                
            //Rectangle screenRect = new Rectangle( pos.x, pos.y, dim.width, dim.height );
            try
            {
                writeImage( this, "jpg", m_screenshotFileReference , 0, 0, dim.width, dim.height );
            }
            catch ( IOException e )
            {
                System.out.println( "Could not write screenshot to filesystem!" );
                e.printStackTrace( );
            }
        }
            
    }


    /**
     * Creates a automatic configuration file which is used to perform the
     * the jpg -> Dicom conversion: This method creates all necessary tags
     * needed for a valid dcm-file.
     * @param jpg2DcmFileAndPath
     * @param seriesDescription
     * @return 
     */
    public int createJpg2DcmConfigFile( String jpg2DcmFileAndPath, 
                                        String seriesDescription  )
    {
        try
        {
        
            FileWriter fileWriter = new FileWriter( jpg2DcmFileAndPath );
            fileWriter.write( "# jpg2dcm Default Configuration for encapsulating JPEG Baseline streams into\n" );
            fileWriter.write( "# DICOM Secondary Capture Image objects\n" );
            fileWriter.write( "# (s. DICOM Part 3, A.32.7 Video Photographic Image IOD)\n\n");
            fileWriter.write( "# Patient Module Attributes \n");
            fileWriter.write( "# Patient's Name\n");
            fileWriter.write( "00100010:" + m_currentView.getDicomFileInfo( ).getPatientName( ) + "\n" );
            fileWriter.write( "# Patient ID\n");
            fileWriter.write( "00100020:" + m_currentView.getDicomFileInfo( ).getPatientID( ) + "\n" );
            fileWriter.write( "# Issuer of Patient ID\n" );
            fileWriter.write( "#00100021:" );
            fileWriter.write( "# Patient's Birth Date\n" );
            fileWriter.write( "00100030:" + m_currentView.getDicomFileInfo( ).getPatientBirthDate( ) + "\n" );
            fileWriter.write( "# Patient's Sex\n" );
            fileWriter.write( "00100040:" + m_currentView.getDicomFileInfo( ).getPatientSex( ) + "\n\n" );
            fileWriter.write( "# General Study Module Attributes\n" );
            fileWriter.write( "# Study Instance UID\n" );
            fileWriter.write( "#0020000D:" + m_currentView.getDicomFileInfo( ).getStudyInstanceUID( ) + "\n" );
            fileWriter.write( "# Study Date\n" );
            fileWriter.write( "00080020:" + m_currentView.getDicomFileInfo( ).getStudyDate( ) + "\n" );
            fileWriter.write( "# Study Time\n" );
            fileWriter.write( "00080030:" + m_currentView.getDicomFileInfo( ).getStudyTime( ) + "\n" );
            fileWriter.write( "# Referring Physician's Name\n" );
            fileWriter.write( "00080090:" + "DIN-Bildlabor\n" );
            fileWriter.write( "# Study ID\n" );
            fileWriter.write( "00200010:" + m_currentView.getDicomFileInfo( ).getStudyID( ) + "\n" );
            fileWriter.write( "# Accession Number\n" );
            fileWriter.write( "00080050:" + m_currentView.getDicomFileInfo( ).getAccessionNumber( ) + "\n" );
            fileWriter.write( "# Study Description\n" );
            fileWriter.write( "#00081030:" + m_currentView.getDicomFileInfo( ).getStudyDescription( ) + "\n" );
            fileWriter.write( "\n" );
            fileWriter.write( "# General Series Module Attributes\n" );
            fileWriter.write( "# Modality\n" );
            fileWriter.write( "00080060:MR\n" ); // for the time being keep this setting: most important is that it does display well
            fileWriter.write( "# Series Instance UID\n" );
            fileWriter.write( "0020000E:" +  m_currentView.getDicomFileInfo( ).getSeriesInstanceUID( ) + "\n" );

            fileWriter.write( "# Series description\n" );
            fileWriter.write( "0008103E:" + seriesDescription + "\n" );

            fileWriter.write( "# Series Number\n" );
            fileWriter.write( "00200011:" + m_currentView.getDicomFileInfo( ).getSeriesNumber( ).trim( ) + "\n" );

            fileWriter.write( "# General Equipment Module Attributes" + "\n" );
            fileWriter.write( "# Manufacturer" + "\n" );
            fileWriter.write( "00080070:" + m_currentView.getDicomFileInfo( ).getManufacturer( ) + "\n" );

            fileWriter.write( "# SC Equipment Module Attributes\n" );
            fileWriter.write( "# Conversion Type\n" );
            fileWriter.write( "00080064:SI" );

            fileWriter.write( "# General Image Module Attributes\n" );
            fileWriter.write( "# Instance Number\n" );
            fileWriter.write( "00200013:" + "2000" + "\n" );

            fileWriter.write( "# Image Pixel Module Attributes (detected from JPEG file, if not specified)" + "\n" );
            fileWriter.write( "# Samples per Pixel\n");
            fileWriter.write( "#00280002:" + m_currentView.getDicomFileInfo( ).getSamplesPerPixel( ) + "\n" );
            fileWriter.write( "# Photometric Interpretation\n");
            fileWriter.write( "#00280004:" + m_currentView.getDicomFileInfo( ).getPhotometricInterpretation( ) + "\n" );
            fileWriter.write( "# Planar Configuration\n" );
            fileWriter.write( "#00280006:" + m_currentView.getDicomFileInfo().getPlanarConfiguration() + "\n" );
            fileWriter.write( "# Rows" );
            fileWriter.write( "#00280010:" + m_currentView.getDicomFileInfo( ).getImageHeight( ) + "\n" );
            fileWriter.write( "# Columns\n" );
            fileWriter.write( "#00280011:" + m_currentView.getDicomFileInfo( ).getImageWidth( ) + "\n" );
            fileWriter.write( "# Bits Allocated\n");
            fileWriter.write( "#00280100:" + m_currentView.getDicomFileInfo( ).getFileType( ) + "\n" );
            fileWriter.write( "# Bits Stored\n" );
            fileWriter.write( "#00280101:" + m_currentView.getDicomFileInfo( ).getBitsStored( ) +"\n" );
            fileWriter.write( "# High Bit\n" );
            fileWriter.write( "#00280102:" + m_currentView.getDicomFileInfo( ).getHighBit( ) + "\n" );
            fileWriter.write( "# Pixel Representation\n" );
            fileWriter.write( "#00280103:" + m_currentView.getDicomFileInfo( ).getPixelRepresentation( ) + "\n\n" );
            fileWriter.write( "# SOP Common Module Attributes\n" );
            fileWriter.write( "# SOP Class UID\n" );
            fileWriter.write( "00080016:1.2.840.10008.5.1.4.1.1.7\n" ); // secondary capture
            fileWriter.write( "# SOP Instance UID\n");
            fileWriter.write( "#00080018:" + "sop_instance_iud" + "\n" );
            fileWriter.close( );
        }
        catch ( IOException e )
        {
            System.out.println( "Error creating jpg2dcm config file : " + jpg2DcmFileAndPath );
            return -1;
        }
        return 0;
    }

    /**
     * Returns the file selected by the user.
     */
    private File getScreenShotFileReference( )
    {   
        MRFittingDefaults fdef = MRFittingDefaults.getInstance( );
        JFileChooser fileChooser = new JFileChooser( fdef.getProperty( "File name and path to store screenshot into" ) );
        String str = getCurrentView( ).getDicomFileInfo( ).getPatientName( ) + "_" + 
                     getCurrentView( ).getDicomFileInfo( ).getSeriesDate( ) + "_" +
                     getCurrentView( ).getDicomFileInfo( ).getSeriesTime( ) + "_" +
                     getCurrentView( ).getDicomFileInfo( ).getSeriesDescription( );
        File f = new File( str + ".jpg" );
        fileChooser.setSelectedFile( f );
        
        int result = fileChooser.showOpenDialog( this );
        
        m_screenshotFileReference = new File( "" );
        
        switch ( result )
        {
            case JFileChooser.APPROVE_OPTION:   // Approve Open button was pressed
                                                m_screenshotFileReference = fileChooser.getSelectedFile( );
                                                m_screenshotFileReference = new File( m_screenshotFileReference.getPath( ) );
                                                break;
           case JFileChooser.CANCEL_OPTION:     // Cancel or close dialog was selected
                                                break;
           case JFileChooser.ERROR_OPTION:      // the selected process did not terminate properly
                                                break;
        }
        
        return m_screenshotFileReference;
    }
    
    
    
    /**
     * Change all polygons according to last changed polygon.
     */
    public void changeAllPolygonsInAllViewsAccordingly( )
    {

        if ( m_isTimeArrayMode == false )
        {
            
            if ( m_workingPolygon != null )
            {
                int selectedImageNumber = m_currentView.getDisplayedImageNumber( );
                
                for ( int i = 0; i < m_sliceViews.size( ); i++ )
                {
                    DicomSliceCanvasPolyDraw c = m_sliceViews.get( i );

                    for ( int k = 0; k < c.getNumberOfImages( ); k++ )
                    {
                        
                        if ( k == selectedImageNumber )
                        {
                            LinkedList<RegioPolygon> polys = c.getPolygonList( k );

                            for ( int j = 0; j < polys.size( ); j++ )
                            {
                                RegioPolygon pol = polys.get( j );

                                if ( pol.getLabel( ).compareTo( m_workingPolygon.getLabel( ) ) == 0 )
                                {
                                    System.out.println( "Deleting!" + k );
                                    polys.remove( j );
                                
                                    RegioPolygon poly = new RegioPolygon( m_workingPolygon );
                                    polys.add( poly );
                                }
                            }
                        
                        
                        // force update
                        c.update( true );
                        }
                    }
                }
            }
        }
        
        if ( m_isTimeArrayMode == true )
        {
            
            if ( m_workingPolygon != null )
            {
                int selectedImageNumber = m_currentView.getDisplayedImageNumber( );
                
                for ( int i = 0; i < m_sliceViews.size( ); i++ )
                {
                    DicomSliceCanvasPolyDraw c = m_sliceViews.get( i );

                    for ( int k = 0; k < c.getNumberOfImages( ); k++ )
                    {
                        
                        LinkedList<RegioPolygon> polys = c.getPolygonList( k );
                        if ( polys.size( ) != 0 )
                        {
                            for ( int j = 0; j < polys.size( ); j++ )
                            {
                                RegioPolygon pol = polys.get( j );

                                if ( pol.getLabel( ).compareTo( m_workingPolygon.getLabel( ) ) == 0 )
                                {
                                    System.out.println( "Deleting!" + k );
                                    polys.remove( j );

                                    RegioPolygon poly = new RegioPolygon( m_workingPolygon );
                                    polys.add( poly );
                                }
                            }
                        }
                        else
                        {
                            polys.add( m_workingPolygon );
                        }

                        // force update
                        c.update( true );
                    }
                }
            }
        }
    }
    
    /**
     * Handles polygon removal!
     * If time image series are loaded there is other behavior than 
     * if a slice stack is loaded.
     * @param roiName
     */
    private void handleRemoveAllROIsWithSameName( String roiName, 
                                                  int imageNumber )
    {
        if ( m_isTimeArrayMode == false )
        {
            for ( int i = 0; i < m_sliceViews.size( ); i++ )
            {
                DicomSliceCanvasPolyDraw c = m_sliceViews.get( i );

                Vector< LinkedList <RegioPolygon> > polVec = c.getPolygonsVector( );

                // this selects the polygon list belonging to the selected 
                LinkedList <RegioPolygon> polList = polVec.get( imageNumber );
                for ( int k = 0; k < polList.size( ); k++ )
                {
                    RegioPolygon p = polList.get( k );
                    if ( p.getLabel( ).compareTo( roiName ) == 0 )
                    {
                        polList.remove( p );
                        break;
                    }
                }

                c.update( true );
            }
        }
        
        if ( m_isTimeArrayMode == true )
        {
            for ( int i = 0; i < m_sliceViews.size( ); i++ )
            {
                DicomSliceCanvasPolyDraw c = m_sliceViews.get( i );

                Vector< LinkedList <RegioPolygon> > polVec = c.getPolygonsVector( );

                // this selects the polygon list belonging to the selected
                for ( int j = 0; j < polVec.size( ); j++ )
                {
                    LinkedList <RegioPolygon> polList = polVec.get( j );

                    for ( int k = 0; k < polList.size( ); k++ )
                    {
                        RegioPolygon p = polList.get( k );
                        if ( p.getLabel( ).compareTo( roiName ) == 0 )
                        {
                            polList.remove( p );
                            // break;
                        }
                    }
                }
    //            for ( int j = 0; j < c.getNumberOfImages( ) ; j++ )
    //            {
    //
    //                LinkedList <RegioPolygon> polList = polVec.get( j );
    //                for ( int k = 0; k < polList.size( ); k++ )
    //                {
    //                    RegioPolygon p = polList.get( k );
    //                    if ( p.getLabel( ).compareTo( roiName ) == 0 )
    //                    {
    //                        polList.remove( p );
    //                        break;
    //                    }
    //                }
    //
    //            }
                c.update( true );
            }
        }
        
    }


    /**
     * Adds the last added.
     * @param addNotToSelf
     */
    public void addWorkingPolygonToAllViews( boolean addNotToSelf )
    {
        if ( m_isTimeArrayMode == false )
        {
            for ( int i = 0; i < m_sliceViews.size( ); i++ )
            {
                DicomSliceCanvasPolyDraw c = m_sliceViews.get( i );
                if ( c != getCurrentView( ) )
                {
                    RegioPolygon poly = new RegioPolygon( m_workingPolygon );
                    c.addPolygon( poly );
                }
            }
        }
        else
        {
            for ( int i = 0; i < m_sliceViews.size( ); i++ )
            {
                DicomSliceCanvasPolyDraw c = m_sliceViews.get( i );
                if ( addNotToSelf == true )
                {
                    if ( c != getCurrentView( ) )
                    {
                        for ( int j = 0; j < c.getNumberOfImages( ); j++ )
                        {
                            if ( j != c.getCurrentlyDisplayedImageNumber( ) )
                            {
                                c.addPolygon( j, new RegioPolygon( m_workingPolygon ) );
                            }
                        }
                        c.addPolygon( c.getCurrentlyDisplayedImageNumber( ), new RegioPolygon( m_workingPolygon ) );
                    }
                }
                else
                {
                    for ( int j = 0; j < c.getNumberOfImages( ); j++ )
                    {
                        if ( j != c.getCurrentlyDisplayedImageNumber( ) )
                        {
                            c.addPolygon( j, new RegioPolygon( m_workingPolygon ) );
                        }
                    }
                    c.addPolygon( c.getCurrentlyDisplayedImageNumber( ), new RegioPolygon( m_workingPolygon ) );
                }
            }
            // Repeat for the currently selected view as well
            for ( int i = 0; i < m_sliceViews.size( ); i++ )
            {
                DicomSliceCanvasPolyDraw c = m_sliceViews.get( i );

                if ( c == getCurrentView( ) )
                {
                    
                    for ( int j = 0; j < c.getNumberOfImages( ); j++ )
                    {
                        if ( c.getCurrentlyDisplayedImageNumber( ) != j )
                        {
                            c.addPolygonNoTrans( j, new RegioPolygon( m_workingPolygon  ) );
                        }
                    }
                }
            }
        }
    }
    
    /**
     * Returns the last added polygon in a MultiDicoomSliceView context)! 
     * @return 
     */
    public RegioPolygon getWorkingPolygon( )
    {
        return m_workingPolygon;
    }
    
    /**
     * Returns the last added polygon in a MultiDicoomSliceView context)! 
     * @param lastAddedPolygon
     */
    public void setWorkingPolygon( RegioPolygon lastAddedPolygon )
    {
        m_workingPolygon = new RegioPolygon( lastAddedPolygon );
    }
    
    /**
     * Cuts all ROIs according to the last changed polygon in all views to same.
     */
    public void cutAllROIsAccordingToLastChangedPolygonOfAllViews( ) 
    {
        if ( m_isTimeArrayMode == true )
        {
            syncAllRegionsAccordingly( );
        }
        
    }
    
    /**
     * Synchronizes all views according to currently selected.
     */
    public void syncAllRegionsAccordingly( )
    {
        
        this.getCurrentSliceCanvasPolyDraw( ).replicatePolygonListOfImageNumberToAllOther( );
        
        for ( int i = 0; i < m_sliceViews.size( ); i++ )
        {

           DicomSliceCanvasPolyDraw view = m_sliceViews.get( i );

           if ( view != this.getCurrentSliceCanvasPolyDraw( ) )
           {
                view.setVectorOfArrayListOfPolygons( getCurrentSliceCanvasPolyDraw( ).getAllPolygonsVector( ) );
            
                // force visual update
                view.update( true );
           }

        }
    }
    
    /**
     * Cut all ROIs according to the last changed polygon of all views.
     * @param polygons 
     */
    public void cutAllROIsAccordingToLastChangedPolygonOfAllViews( Vector< LinkedList< RegioPolygon > > polygons ) 
    {
        if ( m_isTimeArrayMode == true )
        {
            syncAllRegionsAccordingly( polygons );
        }
        
    }
    
    /**
     * Synchronize all regions of all views using given list of polygons. 
     * @param polygons 
     */
    public void syncAllRegionsAccordingly( Vector< LinkedList < RegioPolygon > > polygons )
    {
        
        for ( int i = 0; i < m_sliceViews.size( ); i++ )
        {
         
            DicomSliceCanvasPolyDraw view = m_sliceViews.get( i );

            view.setVectorOfArrayListOfPolygons( polygons );
            
            // force visual update
            view.update( true );
        }
    }
    
    /**
     * Adds the last added polygon from file to all views.
     */
    public void addWorkingPolygonToAllViewsFromFile( )
    {
        if ( m_isTimeArrayMode == false )
        {
            for ( int i = 0; i < m_sliceViews.size( ); i++ )
            {
                DicomSliceCanvasPolyDraw c = m_sliceViews.get( i );
                if ( c != getCurrentView( ) )
                {
                    RegioPolygon poly = new RegioPolygon( m_workingPolygon );
                    c.addPolygon( poly );
                }
            }
        }
        else
        {
//            for ( int i = 0; i < m_sliceViews.size( ); i++ )
//            {
//                DicomSliceCanvasPolyDraw c = m_sliceViews.get( i );
//
//                if ( c != getCurrentView( ) )
//                {
//                    for ( int j = 0; j < c.getNumberOfImages( ); j++ )
//                    {
//                        if ( j != c.getCurrentlyDisplayedImageNumber( ) )
//                        {
//                            c.addPolygon( j, new RegioPolygon( m_lastAddedPolygon ) );
//                        }
//                    }
//                    c.addPolygon( c.getCurrentlyDisplayedImageNumber( ), new RegioPolygon( m_lastAddedPolygon ) );
//                }
//            }

            // Repeat for the currently selected view as well
            for ( int i = 0; i < m_sliceViews.size( ); i++ )
            {
                DicomSliceCanvasPolyDraw c = m_sliceViews.get( i );

//                if ( c == getCurrentView( ) )
//                {
                    
                    for ( int j = 0; j < c.getNumberOfImages( ); j++ )
                    {
//                        if ( c.getCurrentlyDisplayedImageNumber( ) != j )
//                        {
                            c.addPolygonNoTrans( j, new RegioPolygon( m_workingPolygon  ) );
//                        }
                    }
//                }
            }
        }
    }
    
    
    /**
     * Renames the images with the same name into a new name; not possible is 
     * to rename only one roi within one slice, but that is currently not a use
     * case.
     * @param oldPolygonName
     * @param newPolygonName
     */
    public void renameAllROIsAccordingToLastChangedPolygonOfAllViews( String oldPolygonName,
                                                                      String newPolygonName )
    {

        for ( int i = 0; i < m_sliceViews.size( ); i++ )
        {
            DicomSliceCanvasPolyDraw c = m_sliceViews.get( i );

            for ( int j = 0; j < c.getNumberOfImages( ); j++ )
            {
                LinkedList<RegioPolygon> polys = c.getPolygonList( j );

                for ( int k = 0; k < polys.size( ); k++ )
                {
                    RegioPolygon polygonToTest = polys.get( k );

                    if ( polygonToTest.getLabel( ).compareTo( oldPolygonName ) == 0 )
                    {

                        System.out.println( "renaming poly (" + k + ")" + "imageNumber = " +  j + "  view number = " + i );

                        polygonToTest.setLabel( newPolygonName );
                    }
                }
            }
            // force update
            c.update( true );
        }

    }
    
    /**
     * Renames the images with the same name into a new name; not possible is 
     * to rename only one roi within one slice, but that is currently not a use
     * case.
     * @param oldPolygonName
     */
    public void renameAllROIsAccordingToLastChangedPolygonOfAllViews( String oldPolygonName )
    {
        RegioPolygon poly = m_currentView.getSelectedPolygon( );

        //if ( m_isTimeArrayMode == true )
        //{

            System.out.println( "renameAllROIsAccordingToLastChangedPolygonOfAllViews: poly.label -> " + poly.getLabel( ) );
            System.out.println( "renameAllROIsAccordingToLastChangedPolygonOfAllViews: oldPolygonLabel -> " + oldPolygonName );

            for ( int i = 0; i < m_sliceViews.size( ); i++ )
            {
                DicomSliceCanvasPolyDraw c = m_sliceViews.get( i );
                
                for ( int j = 0; j < c.getNumberOfImages( ); j++ )
                {
                    LinkedList<RegioPolygon> polys = c.getPolygonList( j );

                    for ( int k = 0; k < polys.size( ); k++ )
                    {
                        RegioPolygon polygonToTest = polys.get( k );

                        if ( polygonToTest.getLabel( ).compareTo( oldPolygonName ) == 0 )
                        {

                            System.out.println( "renaming poly (" + k + ")" + "imageNumber = " +  j + "  view number = " + i );

                            polygonToTest.setLabel( poly.getLabel( ) );
                        }
                    }
                }
                // force update
                c.update( true );
            }
        //}
    }

    /**
     * Returns the view according to viewNumber
     * (first added = 0;
     *  last added = size( ) )
     * @param viewNumber
     * @return
     */
    public DicomSliceCanvasPolyDraw getView( int viewNumber )
    {
        return m_sliceViews.get( viewNumber );
    }

    /**
     *
     * Returns whether this is a time series.
     *
     * @param timeSeries
     */
    public void isTimeSeries( boolean timeSeries )
    {
        m_isTimeArrayMode = timeSeries;
    }

    /**
     * This method copies all regions of the selected view to all other
     * DicomSliceCanvasPolyDraw object of the MultiDicomSliceView. 
     * @param polysToAdd
     */
    public void copyAllROIsToOtherViews( LinkedList<RegioPolygon> polysToAdd )
    {
        
        DicomSliceCanvasPolyDraw cur = this.getCurrentView( );
        Vector< LinkedList<RegioPolygon> > polVec = cur.getPolygonsVector( );
        
        // Iterate over all DicomSliceCanvasPolyDraw views
        
        for ( int i = 0; i < m_sliceViews.size( ); i++ )
        {
            DicomSliceCanvasPolyDraw c = m_sliceViews.get( i );
            
            // regions must be propagated from the selected control 
            // to the other controls
            if ( !c.equals( cur ) )
            {
                for ( int j = 0; j < cur.getNumberOfImages( ); j++ )
                {
                    
                    LinkedList<RegioPolygon> polList = polVec.get( j );
                    
                    for ( int k = 0; k < polList.size( ); k++ )
                    {
                        RegioPolygon p = new RegioPolygon( polList.get( k ) );
                        c.addPolygon( j, p );
                    }
                }
            }
            c.update( true );
        }
    }
    
    /**
     * Set all regions in all views according to given list of RegioPolygons.
     * @param definedPolygons
     */
    public void setAllRegionsOfAllViews( LinkedList<RegioPolygon> definedPolygons )
    {
        
        // Iterate over all DicomSliceCanvasPolyDraw views
        
        for ( int i = 0; i < m_sliceViews.size( ); i++ )
        {
            DicomSliceCanvasPolyDraw c = m_sliceViews.get( i );
            
            for ( int j = 0; j < c.getNumberOfImages( ); j++ )
            {

                //LinkedList<RegioPolygon> polList = polVec.get( j );

                for ( int k = 0; k < definedPolygons.size( ); k++ )
                {
                    RegioPolygon p = new RegioPolygon( definedPolygons.get( k ) );
                    c.addPolygon( j, p );
                }
            }
        
            c.update( true );
        }
    }
    
    /**
     * Set the adding arbitrary shaped ROIs mode on.
     */
    public void setToAddArbitraryShapeROIsMode( ) 
    {
        m_sliceViews.get(0).setMode( DicomSliceCanvasPolyDraw.POLYGON_DRAW_MODE );
    }
    
    /**
     * Set the adding circular shaped ROIs mode on.
     */
    public void setToAddCircularShapeROIsMode( ) 
    {
        m_sliceViews.get(0).setMode( DicomSliceCanvasPolyDraw.ADD_SINGLE_CIRCULAR_ROI_MODE );
    }
    
    // Attributes for this classs
    
    // Listeners to textfield changes
    private LinkedList<ActionListener> m_actionListeners = new LinkedList<>( );
    
    private File m_screenshotFileReference;
    
    // Attributes     
    private JFrame m_parent;
    private ZoomedDicomSliceCanvasPolydraw m_zoomWin;
    
    
    private boolean m_exitOnClose;
    
    //vector of DicomSliceCanvasPolyDraw objects within the view
    private Vector<DicomSliceCanvasPolyDraw> m_sliceViews = new Vector<>( );
    private Vector<DicomSliceCanvasPolyDraw> m_selectedViews = new Vector<>( );
    private DicomSliceCanvasPolyDraw m_currentView; 
    
    int m_maxPixelValueOfAllLoadedImages = Integer.MIN_VALUE;
    int m_minPixelValueOfAllLoadedImages = Integer.MAX_VALUE;
    
    // Member determining the zoom factor of all images in the multiview
    private float m_globalZoomFactor = 1.0f;
    
    private int m_maxImageHeight = -1;
    private int m_maxImageWidth = -1;
    
    private String m_defaultPath;
    private String m_loadPath;
    
    private CustomFileDialog m_customFileDialog;
    private final String m_fileSeparator;
    
    private String m_polygonInformation;
    private RegioPolygon m_selectedPolygon;
    
    public final static int POLYGON_CHANGE_ACTION_EVENT = 0;
    public final static int POLYGON_SELECT_ACTION_EVENT = 1;
    
    // mode related events
    public final static int ADD_ROI_MODE_SELECTED_EVENT = 2;
    public final static int CUT_ROI_MODE_SELECTED_EVENT = 3;
    public final static int SELECT_ROI_MODE_SELECTED_EVENT = 4;
    public final static int DELETE_ROI_MODE_SELECTED_EVENT = 5;
    public final static int MOVE_ROI_MODE_SELECTED_EVENT = 6;
    public final static int RESIZE_ROI_MODE_SELECTED_EVENT = 7;
    
    public final static int SELECT_IMAGE_EVENT = 100;

    public final static int POLYGON_CLOSED_EVENT    = 1234;
    public final static int POLYGON_CHANGED_EVENT   = 1235;
    public final static int POLYGON_RENAME_EVENT    = 1236;
    public final static int POLYGON_CUT_EVENT       = 1237;

    boolean m_showToolTipText = true;
    boolean m_relativeViewsAreShown = false;
    boolean m_equalize  = false;
    // members to trace changes in brightness and contrast
    int m_groupBrightness = 0;
    int m_groupContrast = 0;  
    
    
    Graphics g_stored;

    private boolean m_synchronizeViews = false;


    public final static int GRAY_VALUE  = 0;
    public final static int RAINBOW     = 1;
    public final static int INV_RAINBOW = 2;

    private RegioPolygon m_workingPolygon;

    //private RegioPolygon m_lastChangedPolygon;

    // For proper serialization
    private static final long serialVersionUID = 1481295122349123L;


    private boolean m_isTimeArrayMode = false;
    
    HashMap<String,LinkedList<MomentsAboutTheMean>> m_all_MAM_Of_All_Views = new HashMap< >( );

    
    
    
      
}
