/**
 * File: $Id: SpectroscopicVoxelProjectionDicomSliceCanvasPolyDraw.java,v 1.1.1.1 2006/04/19 09:38:47 hans Exp $
 * History
 * Date: 15.10.2004, 2:10  Authors: Hans Slotboom  
 *
 * Changes:
 *      15.10.2004 - Starting implementation
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */


package org.spectrim.dicom;

import javax.swing.JFrame;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;

import java.awt.Graphics;
import java.awt.Point;
import java.awt.Polygon;
import java.awt.Rectangle;

import java.text.DecimalFormat;

import java.io.FileWriter;
import java.io.IOException;
import java.util.LinkedHashMap;
import javax.swing.JMenuItem;
import org.spectrim.imagegraphicobjects.Point3D;
import org.spectrim.imagegraphicobjects.PolygonUtils;
import org.spectrim.imagegraphicobjects.RegioPolygon;
import org.spectrim.imagegraphicobjects.Spectroscopic3DVoxel;
import org.spectrim.lib.Dialogs;

/**
 * SpectroscopicVoxelProjectionDicomSliceCanvasPolyDraw is a specialized sub- 
 * class to display image stacks in which spectroscopic information is 
 * projected.
 * @author  hans
 */
public class SpectroscopicVoxelProjectionDicomSliceCanvasPolyDraw extends DicomSliceCanvasPolyDraw implements MouseListener, ActionListener
{
        
    /** 
     * Creates a new instance of 
     * SpectroscopicVoxelProjectionDicomSliceCanvasPolyDraw. 
     * @param parent
     */
    public SpectroscopicVoxelProjectionDicomSliceCanvasPolyDraw( JFrame parent )
    {
        super( parent );

        m_rotateAbout90Degrees.setEnabled( false );
        m_flipVertically.setEnabled( false );
        m_flipHorizontally.setEnabled( false );
    }
    
    /** 
     * Creates a new instance of 
     * SpectroscopicVoxelProjectionDicomSliceCanvasPolyDraw. 
     * @param parent
     * @param voxel
     */
    public SpectroscopicVoxelProjectionDicomSliceCanvasPolyDraw( JFrame parent, 
                                                                 Spectroscopic3DVoxel voxel )
    {
        super( parent );
        m_specVoxel = voxel;

        m_rotateAbout90Degrees.setEnabled( false );
        m_flipVertically.setEnabled( false );
        m_flipHorizontally.setEnabled( false );
       
    }


    /**
     * Creates a new instance of 
     * SpectroscopicVoxelProjectionDicomSliceCanvasPolyDraw.
     * @param parent
     * @param voxel
     * @param siemensSpecificSpectroscopyInformation
     * @param dicomSpectrumFileInfo
     */
    public SpectroscopicVoxelProjectionDicomSliceCanvasPolyDraw( JFrame parent, 
                                        Spectroscopic3DVoxel voxel,
    		                        LinkedHashMap siemensSpecificSpectroscopyInformation,
    		                        FileInfo dicomSpectrumFileInfo )
    {
        super( parent );
        m_specVoxel = voxel;

        m_siemensSpecificSpectroscopyInformation = siemensSpecificSpectroscopyInformation;
        m_spectroscopyDICOMFileInfo = dicomSpectrumFileInfo;

        addAdditionalPopupMenuItems( );

        m_minimumImageNumberToFocus = this.getMinimumImageNumberToFocus( );
        m_maximumImageNumberToFocus = this.getNumberOfImages( );

        m_rotateAbout90Degrees.setEnabled( false );
        m_flipVertically.setEnabled( false );
        m_flipHorizontally.setEnabled( false );
    }



    /**
     * Creates a spectroscopic report text file for this object.
     *
     */
    private void createSpecVoxelReportFile( )
    {

    	String imageCharacteresitcsInSVoxelFileName = getFilePathAndName( getDicomFileInfo( ).getSeriesDescription( ) + "_" + "image_characteristics.txt" , "ima_props_within_spec_volume", "Save" );
        if ( m_siemensSpecificSpectroscopyInformation != null )
        {
            if ( imageCharacteresitcsInSVoxelFileName.length( ) != 0 )
            {

                try
                {
                    // file name is valid

                    DecimalFormat formatter = new DecimalFormat( "###.0000" );

                    FileWriter fileWriter = new FileWriter( imageCharacteresitcsInSVoxelFileName );
                    FileInfo fInfo = m_dicomImageSeriesData.getFileInfo( 0 );

                    fileWriter.write( "Patient name             : " + fInfo.getPatientName( ) + "\r\n");
                    fileWriter.write( "Patient ID               : " + fInfo.getPatientID( ) + "\r\n");
                    fileWriter.write( "Date of Birth            : " + fInfo.getPatientBirthDate( ) + " " );
                    fileWriter.write( "Age                      : " + fInfo.getPatientAge( ) + "\r\n" );
                    fileWriter.write( "Examination Date         : " + fInfo.getAcquisitionDate( ) + "\r\n" );
                    fileWriter.write( "Examination Time         : " + fInfo.getAcquisitionTime( ) + "\r\n" );
                    fileWriter.write( "Series Number            : " + fInfo.getSeriesNumber( ) + "\r\n" );
                    fileWriter.write( "Series Description       : " + fInfo.getSeriesDescription( ) + "\r\n" );
                    fileWriter.write( "Accession Number         : " + fInfo.getAccessionNumber( ) + "\r\n" );
                    fileWriter.write( "SOP Instane UID          : " + fInfo.getSOPInstanceUID( ) + "\r\n\r\n" );

                    String voxelDimensions = "Spectroscopic voxel size : [ " +  m_siemensSpecificSpectroscopyInformation.get( "sSpecPara.sVoI.dThickness" ) + ", ";
                    voxelDimensions += m_siemensSpecificSpectroscopyInformation.get( "sSpecPara.sVoI.dPhaseFOV" ) + ", ";
                    voxelDimensions += m_siemensSpecificSpectroscopyInformation.get( "sSpecPara.sVoI.dReadoutFOV" ) + " ]\r\n";

                    fileWriter.write( voxelDimensions );

                    int currentImageNumber = m_currentlyDisplayedImageNumber;

                    int totalPixels = 0;
                    float weightedMean = 0f;
                    for ( int i = 0; i < m_numberOfImages; i++ )
                    {

                        displayDicomImage( i );

                        drawImage( true );

                        m_av = 0;
                        m_stdev = 0;
                        m_roiSize = 0;

                        deleteAllPolygonsOfCurrentlyDisplayedImageOnly( );
                        computeCrossSectionRegioPolygon( );
                        computeImageCharacteristicsWithinVoxel( );

                        if ( m_roiSize != 0 )
                        {
                            System.out.println( m_SROI_infostring );
                            fileWriter.write( "Instance_number:   "  + "\t" + ( i + 1 )     + "\t" );
                            fileWriter.write( "Average_pixel_value:" + "\t" + formatter.format( m_av )      + "\t");
                            fileWriter.write( "St_dev_pixel_value:"  + "\t" + formatter.format( m_stdev ) + "\t" );
                            fileWriter.write( "Number_of_pixels: "   + "\t" + m_roiSize + "\t" );
                            fileWriter.write( "Volume_of_pixels: "   + "\t" + formatter.format( m_roiSize * m_volumePerPixel ) + "\t" );
                            fileWriter.write( "\r\n" );
                            totalPixels += m_roiSize;
                            weightedMean += m_roiSize * m_av;
                        }
                    }

                    fileWriter.write( "\r\n" );
                    fileWriter.write( "Total_number_of_pixels: "   + "\t" + totalPixels + "\t" + "Total volume" + "\t" + formatter.format( m_volumePerPixel * totalPixels ) + "\t" + "Mean Pixel Intesity" + "\t" + formatter.format( weightedMean/totalPixels ) );

                    fileWriter.close( );

                    m_currentlyDisplayedImageNumber = currentImageNumber;
                    displayDicomImage( m_currentlyDisplayedImageNumber );
                }
                catch ( IOException e )
                {
                    System.out.println( "Error creating file : " + imageCharacteresitcsInSVoxelFileName );
                }
            }
        }
        else
        {
            Dialogs.messageBox( m_parent, "No spectrum defined.", "Cannot create report File!" );
        }
    }

    /**
     * Adds specific pop up menu functionality for this specialized object.
     */
    private void addAdditionalPopupMenuItems( )
    {
    	m_createImagePixelCharacteristicsAtSpectroVoxelLocation = new JMenuItem( "Image Properties at Spectro voxel .." );
    	m_createImagePixelCharacteristicsAtSpectroVoxelLocation.addActionListener( this );
        m_popupmenu.addSeparator( );

        m_popupmenu.add( m_createImagePixelCharacteristicsAtSpectroVoxelLocation );

        if ( m_isJMRUIRelease  == false )
        {
            m_showSpectroscopyDicomFileInfoMenuItem = new JMenuItem( "Display dicom-tags of spectrum" );
            m_showSpectroscopyDicomFileInfoMenuItem.addActionListener( this );

            m_showSpectroscopySiemensDicomFileSpecificInfoMenuItem = new JMenuItem( "Display Siemens specific dicom-tags of spectrum" );
            m_showSpectroscopySiemensDicomFileSpecificInfoMenuItem.addActionListener( this );

            m_popupmenu.add( m_showSpectroscopyDicomFileInfoMenuItem );
            m_popupmenu.add( m_showSpectroscopySiemensDicomFileSpecificInfoMenuItem );

            m_popupmenu.addSeparator( );
        }
    }
    
    /**
     * Routine for painting the component.
     */
    @Override
    public synchronized void paintComponent( Graphics g ) 
    {
        // Drawing code for voxel projected in plane
        
        // Should only be computed once
        if ( m_specVoxel != null )
        {
            
            if ( m_regionOfPlaneWithinVoxel == null )
            {
                computeCrossSectionRegioPolygon( );
            }
            
            super.paintComponent( g );
            
            if ( m_specVoxel != null )
            {
                m_specVoxel.paintComponent( g, m_topLeftPoint, m_dirVecX, m_dirVecY, 
                                            m_FOVX, m_FOVY, (int)(m_zoom * m_imageWidth), (int)(m_zoom * m_imageHeight),
                                            m_infoString, m_SROI_infostring );
            }
        }
        else
        {
            super.paintComponent( g );
            g.drawString( "No voxel defined", 
                          (int)( m_zoom * m_imageWidth / 2 ) - 40, 
                          (int)( m_zoom * m_imageHeight ) - 5 );
        }   
    }
    
    /**
     * Computes the regio polygon which encapsulates the part of the image which
     * is within the spectroscopic voxel.
     */
    private void computeCrossSectionRegioPolygon( )
    {
        int[][] segVol;

        System.out.println( "computeCrossSectionRegioPolygon( ) start" );

        if ( m_specVoxel != null )
        {
            segVol = new int[ (int)(m_imageWidth * m_zoom)][(int)(m_imageHeight * m_zoom) ];
        
            for ( int j = 0; j < (int)(m_imageHeight * m_zoom); j++ )
            {
                for ( int i = 0; i < (int)(m_imageWidth * m_zoom); i++ )
                {
                    Point3D p = this.compute3DCoordinate( i, j );
                    if ( m_specVoxel.isPointWithin3DVoxel( p ) == true )
                    {
                        segVol[j][i] = -1;
                    }
                }
            }
        
            int added = 0;
            int rejected = 0;
        
            int polygonIndex = 1;
            Point startingPoint = new Point( );
            PolygonUtils pUtils = new PolygonUtils( );
        
            while ( null != ( startingPoint = pUtils.getPolygonStart( segVol, startingPoint ) ) ) 
            {
                m_regionOfPlaneWithinVoxel = pUtils.regionBorder( segVol, startingPoint, polygonIndex );
                // m_regionOfPlaneWithinVoxel.setScalingFactor( this.m_zoom );
                
                m_regionOfPlaneWithinVoxel.setLabel( "S-ROI" );
                
                // P.reduceVertices( 40.0 );
            
                if ( m_regionOfPlaneWithinVoxel.getInnerVolume( ) > 0 )
                {
                    System.out.println( "computeCrossSectionRegioPolygon( ) innerVol > 0" );
                    addPolygon( m_regionOfPlaneWithinVoxel );
                    computeImageCharacteristicsWithinVoxel( );
                }
                else
                {
                    m_regionOfPlaneWithinVoxel = null;
                }
                polygonIndex++;
            }
        }
        
        if ( m_regionOfPlaneWithinVoxel == null )
        {
            m_infoString = "No cross section";
            m_SROI_infostring = "";
        }
        
        segVol = null;
    }
    
    /**
     * Compute image characteristics within voxel.
     */
    private void computeImageCharacteristicsWithinVoxel( )
    {
        
        RegioPolygon p = getPolygon( "S-ROI" );
        
        float zoom = 1f / getZoom( );
        
        if ( p != null )
        {
            Polygon roi = (Polygon)p.scaled( zoom );
        
            if ( roi != null )
            {
                Rectangle bb = roi.getBounds( );
                if ( bb.x < 0 )
                {    
                    bb.x = 0;
                }
                if ( bb.x + bb.width >= m_imageWidth ) 
                {
                    bb.width = m_imageWidth - bb.x - 1;
                }
                if ( bb.y < 0 )
                {
                    bb.y = 0;
                }
        
                if ( bb.y + bb.height >= m_imageHeight ) 
                {
                    bb.height = m_imageHeight - bb.y - 1;
                }
        
                int imageSize = m_imageWidth * m_imageHeight;

                m_roiSize = 0;
                m_av = 0f;
                this.m_stdev = 0f;
            
                // Calculate the average of the ROI
                for ( int y = bb.y; y <= bb.y + bb.height; y++ ) 
                {
                    int yoffset = y * m_imageWidth;
                    for ( int x = bb.x; x <= bb.x + bb.width; x++ ) 
                    {
                        if ( roi.contains( x, y ) ) 
                        {
                            m_roiSize++;
                            int xyoffset = yoffset;
                            m_av += m_rawData[ x + m_imageWidth * y ];
                        }
                    }
                }

                m_av = m_av / m_roiSize;

                // Calculate the standard deviation of the current ROI
                float diff = 0.0f;
                // reset the roi size
                m_roiSize = 0;
                m_pixelsTissue = 0;
                m_mixedPixels = 0;
                m_pixelsCSF = 0;
                
                float liquorFraction = 0f;
                float tissueFraction = 0f;
                m_tissueFraction = 0f;
                m_CSFFraction = 0f;
                
                for ( int y = bb.y; y <= bb.y + bb.height; y++ )
                {
                    int yoffset = y * m_imageWidth;
                    for ( int x = bb.x; x <= bb.x + bb.width; x++ )
                    {
                        if ( roi.contains( x, y ) )
                        {
                            m_roiSize++;
                            diff = m_av - m_rawData[ x + m_imageWidth * y ];
                            m_stdev += diff * diff;
                        }
                    }
                }

                if ( m_roiSize > 0 )
                {
                    m_stdev = (float)Math.sqrt( (double) m_stdev / ( (float)m_roiSize - 1.0 ) ) ;
                    m_infoString = "Has crossection!";
                }
                else
                {
                    m_stdev = 0f;
                }

                if ( m_roiSize  == 0 )
                {
                    m_infoString = "Has no crossection!";
                }
                
                // Calculate the volume of pixel lower and number of pixels 
                // larger than a given, user defined thresshold
                
//                for ( int y = bb.y; y <= bb.y + bb.height; y++ )
//                {
//                    int yoffset = y * m_imageWidth;
//                    for ( int x = bb.x; x <= bb.x + bb.width; x++ )
//                    {
//                        if ( roi.contains( x, y ) )
//                        {
//                            if ( m_rawData[ x + m_imageWidth * y ]  <= m_TissueToEdemaBorder )
//                            {
//                                m_pixelsTissue++;
//                                m_tissueFraction = m_tissueFraction + 1f;
//                            }
//
//                            if ( ( m_rawData[ x + m_imageWidth * y ] > m_TissueToEdemaBorder ) &&
//                                 ( m_rawData[ x + m_imageWidth * y ] < m_EdemaTOLiquorBorder ) )
//                            {
//                                m_mixedPixels++;
//                                liquorFraction =
//                                (float)( m_rawData[ x + m_imageWidth * y ] - m_TissueToEdemaBorder ) / (float)( m_EdemaTOLiquorBorder - m_TissueToEdemaBorder );
//                                tissueFraction = 1f - liquorFraction;
//
//                                m_tissueFraction = m_tissueFraction + tissueFraction;
//                                m_CSFFraction = m_CSFFraction + liquorFraction;
//                            }
//
//                            if ( m_rawData[ x + m_imageWidth * y ] > m_EdemaTOLiquorBorder )
//                            {
//                                m_pixelsCSF++;
//                                m_CSFFraction = m_CSFFraction + 1f;
//                            }
//                        }
//                    }
//                }
//
//                NumberFormat format = new DecimalFormat( "#####.#" );
//
//                m_SROI_infostring = "Av. = " + format.format( m_av ) + " St. dev = "
//                                  + format.format( m_stdev )
//                                  + " Size = " + m_roiSize
//                                  + " Liquor = " + m_pixelsCSF
//                                  + " Tissue & Liquor " + m_mixedPixels
//                                  + " Tissue = " + m_pixelsTissue;
//
//               m_SROI_infostring = "";
//
//               String percentageLiquor =  format.format( m_CSFFraction * 100f / ( m_tissueFraction + m_CSFFraction ) );
//               String percentageTissue =  format.format( m_tissueFraction * 100f / ( m_tissueFraction + m_CSFFraction ) );
//
//               m_infoString = "Tissue : " + format.format( m_tissueFraction ) + " (" + percentageTissue + " %)"
//                               + " Liquor :" + format.format( m_CSFFraction ) + " (" + percentageLiquor + " %)";
//               m_infoString = "";
            }
        }
        else
        {
            m_infoString = "Has no crossection!";
        }
    }
    
    
    /**
     * Returns the spectroscopic voxel (if any) else returns null.
     * @return 
     */
    public Spectroscopic3DVoxel getSpectroscopicVoxel( )
    {
        return m_specVoxel;
    }
    
    /** 
     * MouseListener implementation.
     * @param e 
     */
    @Override
    public void mouseClicked( MouseEvent e ) 
    {  
        super.mouseClicked( e );
        
        // Remove the old cross section of the voxel with the image
        deleteAllPolygonsOfCurrentlyDisplayedImageOnly( );
        
        deleteAllPolygonsOfAllImages();

        m_regionOfPlaneWithinVoxel = null;
        
        // should only be computed once
        if ( m_specVoxel != null )
        {
            computeCrossSectionRegioPolygon( );
        }
        else
        {
            getGraphics( ).drawString( "No voxel defined", 
                                       (int)( m_zoom * m_imageWidth / 2 ) - 40, 
                                       (int)( m_zoom * m_imageHeight ) - 5 );
        }
        
        paintComponent( this.getGraphics( ) );
    }
    
    /** 
     * MouseListener implementation.
     * @param e */
    @Override
    public void mouseEntered( MouseEvent e ) 
    {
        super.mouseEntered( e );
    }
    
    /** 
     * MouseListener implementation. 
     * @param e
     */
    @Override
    public void mouseExited( MouseEvent e ) 
    {
        super.mouseExited( e );
    }
    
    /** 
     * MouseListener implementation. 
     * @param e
     */
    @Override
    public void mousePressed( MouseEvent e ) 
    {
        super.mousePressed( e );
    }
    
    /** 
     * MouseListener implementation 
     * @param e
     */
    @Override
    public void mouseReleased( MouseEvent e ) 
    {
        super.mouseReleased( e );
    }
    
     /**
     *  Called when eg a button is pressed or a menu item is selected etc.
     * @param e
     */
    @Override
    public void actionPerformed( ActionEvent e ) 
    {
        Object sender = e.getSource( );

        if ( sender == m_createImagePixelCharacteristicsAtSpectroVoxelLocation )
        {
        	createSpecVoxelReportFile( );
        }

        if ( sender == m_showSpectroscopyDicomFileInfoMenuItem )
        {
        	DicomHeaderViewer headerViewer = new DicomHeaderViewer( m_parent, 
                                                 m_spectroscopyDICOMFileInfo );
        }
        
//        if ( ( sender == m_simpleThressholdingSegmentationMenuItem ) || ( sender == this.m_segmentSpectroFrame ) )
//        {
//            //Base class will not be called in this case
//            if ( sender == m_simpleThressholdingSegmentationMenuItem )
//            {
//                m_segmentSpectroFrame = new SimpleThressholdSegmentationAnalyzerFrame( m_parent, m_rawData, getDicomFileInfo( ), m_siemensSpecificData, m_regionOfPlaneWithinVoxel, m_numberOfImages );
//                m_segmentSpectroFrame.addActionListener( this );
//            }
//            if ( sender == m_segmentSpectroFrame )
//            {
//                if  ( e.getID( ) == SimpleThressholdSegmentationAnalyzerFrame.ACCEPT_EVENT )
//                {
//                    LinkedList persPolys = m_segmentSpectroFrame.getPersistentPolygons( );
//
//                    LinkedList<RegioPolygon> polygonList = m_polygonsVector.get( m_currentlyDisplayedImage );
//
//                    for ( int i = 0 ; i < persPolys.size( ) ; i++ )
//                    {
//                        ( (RegioPolygon)persPolys.get( i ) ).setScalingFactor( 0.5f );
//                        polygonList.addLast( (RegioPolygon)persPolys.get( i ) );
//                    }
//                }
//
//                LinkedList<RegioPolygon> polygonList = m_polygonsVector.get( m_currentlyDisplayedImage );
//
//                // update the context menu status
//                if ( polygonList.size( ) > 0 )
//                {
//
//                    m_selectedPolygon = polygonList.get( 0 );
//                    setSelectedPolygonIndex( 1 );
//
//                    m_renameRegion.setEnabled( true );
//
//                    m_showHistogramOfSelectedRegioPolygon.setEnabled( true );
//
//                    if ( polygonList.size( ) > 1 )
//                    {
//                        m_tTestRegioPolygon.setEnabled( true );
//                    }
//                    else
//                    {
//                        m_tTestRegioPolygon.setEnabled( false );
//                    }
//
//                    m_copyRegioPolygon.setEnabled( true );
//
//                    if ( polygonList.size( ) > 1 )
//                    {
//                        m_copyAllRegioPolygons.setEnabled( true );
//                        m_deleteAllRegioPolygonsOfCurrentlyDisplayedImageOnly.setEnabled( true );
//                    }
//
//                    if ( polygonList.size( ) == 1 )
//                    {
//                        setMode( DRAG_MODE  );
//                    }
//                    else
//                    {
//                        setMode( SELECT_MODE );
//                    }
//                }
//
//                update( );
//
//            }
//        }
//        else
//        {
//        // Base class is called
          super.actionPerformed( e );
//
//            if ( ( sender == m_halfImageDisplay ) || ( sender == m_doubleImageDisplay ) )
//            {
//                m_regionOfPlaneWithinVoxel = null;
//                deleteAllPolygonsOfCurrentlyDisplayedImageOnly( );
//                computeCrossSectionRegioPolygon( );
//                definePagerLocation( );
//            }
//        }
    } 
    
    // Private attributes
    protected Spectroscopic3DVoxel  m_specVoxel;
    protected LinkedHashMap         m_siemensSpecificSpectroscopyInformation;
    protected FileInfo              m_spectroscopyDICOMFileInfo;

    private String                  m_infoString;
    private RegioPolygon            m_regionOfPlaneWithinVoxel;
    
    // Number of pixels of each compartment
    private int                     m_pixelsCSF = 0;        // pixels that are pure CSF
    private int                     m_mixedPixels = 0;      // pixels that are mixes of liquor and tissue
    private int                     m_pixelsTissue = 0;     // pixels that are pure tissue
    
    private float                   m_CSFFraction = 0f;
    private float                   m_tissueFraction = 0f;

    private JMenuItem               m_createImagePixelCharacteristicsAtSpectroVoxelLocation;
    private JMenuItem               m_showSpectroscopyDicomFileInfoMenuItem;
    private JMenuItem               m_showSpectroscopySiemensDicomFileSpecificInfoMenuItem;

    // Thresshold from which on the the pixel is regarded to be a mostly CFS
    private int                     m_TissueToEdemaBorder = 250;
    private int                     m_EdemaTOLiquorBorder = 850;
    
    private String m_SROI_infostring = "";
    private static final long serialVersionUID = 3487115895819393L;

    // code logic for the #jMRUI-release#
    //private boolean m_isJMRUIRelease = true;

}