/*
 * File: $Id: Tools.java,v 1.1.1.1 2006/04/19 09:38:47 hans Exp $
 * History
 * Date: 6. Februar 2003, 09:34    Authors: Martin Buerki
 * Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */
 
package org.spectrim.dicom;

import java.util.LinkedHashMap;

import java.awt.Color;
import java.io.IOException;

/** This class contains static utility methods. */
 public class Tools 
 {
	/** This array contains the 16 hex digits '0'-'F'. */
	public static final char[] hexDigits = { '0','1','2','3','4','5','6',
                                                 '7','8','9','A','B','C','D',
                                                 'E','F'};
	
	/** Converts a Color to an 7 byte hex string starting with '#'. */
	public static String c2hex( Color c ) 
        {
		int i = c.getRGB( );
		char[] buf7 = new char[7];
		buf7[0] = '#';
		for ( int pos = 6; pos >= 1; pos-- ) 
                {
			buf7[pos] = hexDigits[i&0xf];
			i >>>= 4;
		}
		return new String( buf7 );
	}

	/** Converts a float to an 9 byte hex string starting with '#'. */
	public static String f2hex( float f )
        {
		int i = Float.floatToIntBits( f );
		char[] buf9 = new char[9];
		buf9[0] = '#';
		for ( int pos = 8; pos >= 1; pos-- ) 
                {
			buf9[pos] = hexDigits[i&0xf];
			i >>>= 4;
		}
		return new String( buf9 );
	}
        
	public static double[] getMinMax( double[] a ) 
        {
		double min = Double.MAX_VALUE;
		double max = -Double.MAX_VALUE;
		double value;
                
		for ( int i = 0; i < a.length; i++ ) 
                {
			value = a[i];
			if ( value < min )
                        {
                            min = value;
                        }
                        if ( value > max )
                        {
                            max = value;
                        }
		}
                
		double[] minAndMax = new double[2];
		minAndMax[0] = min;
		minAndMax[1] = max;
		return minAndMax;
	}

	public static double[] getMinMax( float[] a )
        {
		double min = Double.MAX_VALUE;
		double max = -Double.MAX_VALUE;
		double value;
		for ( int i = 0; i < a.length; i++ ) 
                {
			value = a[i];
			if ( value < min )
                        {
				min = value;
                        }
                        if ( value > max )
                        {
				max = value;
                        }
		}
		double[] minAndMax = new double[2];
		minAndMax[0] = min;
		minAndMax[1] = max;
		return minAndMax;
	}
	
	/** Converts the float array 'a' to a double array. */
	public static double[] toDouble( float[] a ) 
        {
		int len = a.length;
		double[] d = new double[len];
		for ( int i=0; i<len; i++ )
			d[i] = a[i];
		return d;
	}
	
	/** Converts the double array 'a' to a float array. */
	public static float[] toFloat( double[] a ) 
        {
            int len = a.length;
            float[] f = new float[len];
            for ( int i = 0; i < len; i++ )
            {
                f[i] = (float)a[i];
            }
            return f;
        }
	
	/** Converts carriage returns to line feeds. */
	public static String fixNewLines( String s ) 
        {
            char[] chars = s.toCharArray( );
            for ( int i = 0; i < chars.length; i++ )
            {
                if ( chars[i] == '\r' ) 
                {
                    chars[i] = '\n';
                }
            }
            return new String( chars );
	}

	/**
	* Returns a new double initialized to the value represented by the 
	* specified <code>String</code>.
	*
	* @param      s   the string to be parsed.
	* @return     the double value represented by the string argument.
	* @exception  NumberFormatException  if the string does not contain a
	*             parsable double.
	*/
	public static double parseDouble( String s ) throws NumberFormatException 
        {
		Double d = new Double( s );
		return( d.doubleValue( ) );
	}
        
        
        /**
         * Converts a String to a double
         */
        public static double string2double( String s ) 
        {
            Double d;
            try 
            {
                d = new Double( s );
            }
            catch ( NumberFormatException e ) 
            {
                d = null;
            }
		
            if ( d != null )
            {
                return( d.doubleValue( ) );
            }
            else
            {
                return( 0.0 );
            }
        }
        
        public static String tag2hex( int tag ) 
        {
            char[] buf10 = null;
            
            if ( buf10 == null ) 
            {
                buf10 = new char[11];
                buf10[4] = ',';
                buf10[9] = ' ';
            }
            int pos = 8;
            while ( pos >= 0 ) 
            {
                buf10[pos] = Tools.hexDigits[tag&0xf];
                tag >>>= 4;
                pos--;
                if ( pos==4 )
                { 
                    pos--; // skip coma
                }
            }
            return new String( buf10 );
        }
        
        /**
         * Returns the dicom file info object from which many tags may be obtained
         */
        public static FileInfo getDicomFileInfo( String directoryAndfilename )
        {
            int n = directoryAndfilename.lastIndexOf( System.getProperty( "file.separator" ) );
            String dirName;
            String fName;
            DicomDecoder dd = null;
            if ( n == -1 ) 
            {
                dirName = "." + System.getProperty( "file.separator" );
                fName = directoryAndfilename;
            }    
            else 
            {
                dirName = directoryAndfilename.substring( 0, n + 1 );
                fName = directoryAndfilename.substring( n + 1 );
            }

            try 
            {
                dd = new DicomDecoder( dirName, fName );
                FileInfo fi = dd.getFileInfo( );
            
                return fi;
            }
            catch ( IOException e ) 
            {
                dd.closeInputStream( );
                return null;
            }
        }
        
        /**
         * Returns the dicom file info object from which many tags may be obtained
         */
        public static LinkedHashMap getSiemensSpecificDicomFileInfo( String directoryAndfilename )
        {
            DicomImageData did = new DicomImageData( directoryAndfilename );
            LinkedHashMap hm = did.getSiemensSpecificDataFromFile( );
            if ( hm != null )
            {
                return hm;
            }
            else
            {
                return null;
            }
        }

}
