/*
 * File: $Id: ZoomedDicomSliceCanvasPolydraw.java,v 1.2 2006/11/09 10:02:20 hans Exp $
 * History
 * Date: 3. Dezember 2002, 10:55  Authors: Martin Buerki / Hans Slotboom
 *
 * Changes:
 *
 *      24-04-2003: Created from ZoomWindow class adapted to DicomSliceCanvas &
 *                  DicomSliceCanvasPolydraw
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

package org.spectrim.dicom;


import java.awt.Insets;
import java.awt.Dimension;
import java.awt.Component;

import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowListener;
import java.awt.event.WindowEvent;
import java.awt.event.MouseEvent;
import java.awt.event.KeyEvent;

import java.util.Vector;
import java.util.Iterator;
import java.util.LinkedList;

import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JMenuBar;
import javax.swing.ButtonGroup;

import org.spectrim.imagegraphicobjects.RegioPolygon;


public class ZoomedDicomSliceCanvasPolydraw extends JFrame implements MouseListener, MouseMotionListener, WindowListener, ActionListener 
{
    
    /**
     * Constructor of a ZoomedDicomSliceCanvasPolydraw.
     * @param parent
     * @param ref 
     */
    public ZoomedDicomSliceCanvasPolydraw( JFrame parent, 
                                           DicomSliceCanvasPolyDraw ref ) 
    {       
        super( "Zoomed view" );
        
        m_imageMenu = new JMenu( "Image" );
	m_imageMenu.add( m_itemScaleUp = new JMenuItem( "Zoom +", KeyEvent.VK_P ) );
	m_itemScaleUp.addActionListener( this );
	m_imageMenu.add( m_itemScaleDown = new JMenuItem( "Zoom -", KeyEvent.VK_M ) );
	m_itemScaleDown.addActionListener( this );
        m_imageMenu.add( m_itemShowScale = new JRadioButtonMenuItem( "Show scale" ) );
	m_itemShowScale.addActionListener( this );
        m_imageMenu.addSeparator( ); 
        
        m_imageMenu.add( m_itemGrayScaleMap = new JRadioButtonMenuItem( "Gray Scale" ) );
        m_itemGrayScaleMap.addActionListener( this );
        m_itemGrayScaleMap.setSelected( true );
        m_imageMenu.add( m_itemColorMap = new JRadioButtonMenuItem( "Color" ) );
        m_itemColorMap.addActionListener( this );
        m_imageMenu.add( m_itemColorInverseMap = new JRadioButtonMenuItem( "Color Inverse" ) );
        m_itemColorInverseMap.addActionListener( this );
        
        ButtonGroup m_menuColorTableGroup = new ButtonGroup( );
        m_menuColorTableGroup.add( m_itemGrayScaleMap );
        m_menuColorTableGroup.add( m_itemColorMap );
        m_menuColorTableGroup.add( m_itemColorInverseMap );
        
        m_mainMenu = new JMenuBar( ); 
        m_mainMenu.add( m_imageMenu );
        setJMenuBar( m_mainMenu );
        
	this.parent = parent;
        // this.getContentPane( ).setLayout( new BoxLayout() );
        
        m_mainPanel = new JPanel( );
        
	m_canvas = new DicomSliceCanvasPolyDraw( this );
	m_mainPanel.add( m_canvas );
        // m_mainPanel.setPreferredSize( new Dimension( 512, 512 ) );
	
        m_zoom = 512 / ref.m_imageWidth;
        if ( m_zoom < 1 )
        {
            m_zoom = 1;
        }
        
	m_canvas.addMouseListener( this );
	m_canvas.addMouseMotionListener( this );
        
        m_mainScrollPane = new JScrollPane( m_mainPanel, 
                                            JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
                                            JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS );

        this.getContentPane( ).add( m_mainScrollPane );
	
        // canvas.addPolygonListener(this);
	addWindowListener( this );
               
        pack( );
        setVisible( true );
        
        setImage( ref );
        
        m_canvas.definePagerLocation( );
        m_canvas.enableImagePaging( true );
        
    }
    /**
     * Alternative constructor for a ZoomedDicomSliceCanvasPolydraw.
     * @param parent
     * @param ref 
     */
    public ZoomedDicomSliceCanvasPolydraw( JFrame parent, 
                                           SpectroscopicVoxelProjectionDicomSliceCanvasPolyDraw ref ) 
    {       
        super( "Zoomed view" );
        
        m_imageMenu = new JMenu( "Image" );
	m_imageMenu.add( m_itemScaleUp = new JMenuItem( "Zoom +", KeyEvent.VK_P ) );
	m_itemScaleUp.addActionListener( this );
	m_imageMenu.add( m_itemScaleDown = new JMenuItem( "Zoom -", KeyEvent.VK_M ) );
	m_itemScaleDown.addActionListener( this );
        m_imageMenu.add( m_itemShowScale = new JRadioButtonMenuItem( "Show scale" ) );
	m_itemShowScale.addActionListener( this );
        m_imageMenu.addSeparator( ); 
        
        m_imageMenu.add( m_itemGrayScaleMap = new JRadioButtonMenuItem( "Gray Scale" ) );
        m_itemGrayScaleMap.addActionListener( this );
        m_itemGrayScaleMap.setSelected( true );
        m_imageMenu.add( m_itemColorMap = new JRadioButtonMenuItem( "Color" ) );
        m_itemColorMap.addActionListener( this );
        m_imageMenu.add( m_itemColorInverseMap = new JRadioButtonMenuItem( "Color Inverse" ) );
        m_itemColorInverseMap.addActionListener( this );
        
        ButtonGroup m_menuColorTableGroup = new ButtonGroup( );
        m_menuColorTableGroup.add( m_itemGrayScaleMap );
        m_menuColorTableGroup.add( m_itemColorMap );
        m_menuColorTableGroup.add( m_itemColorInverseMap );
        
        m_mainMenu = new JMenuBar( ); 
        m_mainMenu.add( m_imageMenu );
        setJMenuBar( m_mainMenu );
        
	this.parent = parent;
        // this.getContentPane( ).setLayout( new BoxLayout() );
        
        m_mainPanel = new JPanel( );
        
	m_canvas = new SpectroscopicVoxelProjectionDicomSliceCanvasPolyDraw( this, ref.getSpectroscopicVoxel( ) );
	m_mainPanel.add( m_canvas );
        // m_mainPanel.setPreferredSize( new Dimension( 512, 512 ) );
	
        m_zoom = 512 / ref.m_imageWidth;
        if ( m_zoom < 1 )
        {
            m_zoom = 1;
        }
        
	m_canvas.addMouseListener( this );
	m_canvas.addMouseMotionListener( this );
        
        m_mainScrollPane = new JScrollPane( m_mainPanel, 
                                            JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
                                            JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS );

        this.getContentPane( ).add( m_mainScrollPane );
	
        // canvas.addPolygonListener(this);
	addWindowListener( this );
        pack( );
	setVisible( true );
	
        setImage( ref );
    }    
    
    
    
    /**
     * Sets the number of images of the DicomSliceCanvasPolydraw.
     * @param ref
     */
    public void setImage( DicomSliceCanvasPolyDraw ref ) 
    {
        m_canvas.setImage( ref.m_rawData, ref.m_imageWidth, ref.m_imageHeight, 0 );
        
        m_canvas.setBrightness( ref.getBrightness( ) );
        m_canvas.setContrast( ref.getContrast( ) );
        reference = ref;
        m_canvas.m_zoom = m_zoom;
        copyPolygons( );
        
        // canvas.setSize( canvas.getPreferredSize( ) );
        Insets insets = getInsets( );
        setSize( new Dimension( (int)( m_zoom * ref.m_imageWidth ) + insets.left + insets.right + 18, 
                                (int)( m_zoom * ref.m_imageHeight ) + insets.top + insets.bottom + 41 ) );
        m_ref_zoom = ref.getZoom( );
        zoomFactor = m_ref_zoom / m_zoom;
        m_canvas.drawImage( true );
        setTitle( reference.getTitle( ) );
        setVisible( true );
        
        m_canvas.definePagerLocation( );
        m_canvas.enableImagePaging( true );
    }
    
    
    /**
     * Copies polygons from reference object to this object.
     */
    private void copyPolygons( ) 
    {
        float r = m_zoom / reference.getZoom( );
        m_canvas.deleteAllPolygonsOfCurrentlyDisplayedImageOnly( );
        for ( int k = 0; k < reference.m_polygonsVector.size( ); k++ )
        {
            RegioPolygon p = reference.copyPolygon( k );
            if ( p != null )
            {
                p.setScalingFactor( r );
                m_canvas.addPolygon( p );
            }
        }
    }
    
    /**
     * Copies polygons from reference object to this.
     */
    private void copyPolygonsBack( ) 
    {
        float r = reference.getZoom( )/ m_zoom;
        reference.deleteAllPolygonsOfCurrentlyDisplayedImageOnly( );
        for ( int k = 0; k < m_canvas.m_polygonsVector.size( ); k++ )
        {
            RegioPolygon p = m_canvas.copyPolygon( k );
            if ( p != null )
            {
                p.setScalingFactor( r );
                reference.addPolygon( p );
            }
        }
    }
    
    
/*
    public void polygonChanged(PolygonEvent e) {
        reference.polygonChanged(e);
    }

    public void polygonSelect(PolygonEvent e) {
        reference.polygonSelect(e);
    }

    public void polygonDrag(PolygonEvent e) {
        reference.polygonDrag(e);
    }

    public void polygonRemoved(PolygonEvent e) {
        reference.polygonRemoved(e);
    }
*/
    /**
     * Evokes a mouse evnent based upon an incoming mouse event.
     * @param e
     * @param f
     * @return 
     */
    private MouseEvent scaledMouseEvent( MouseEvent e, float f ) 
    {
	Component src = ( Component )e.getSource( );
	int id = e.getID( );
	long when = e.getWhen( );
	int mod = e.getModifiers( );
	int x = e.getX( );
	int y = e.getY( );
	int cc = e.getClickCount( );
	boolean t = e.isPopupTrigger( );
	return new MouseEvent( src, id, when, mod, (int)(x * f), (int)(y * f), cc, t );
    }
    /* MouseListener implementation */
    
    @Override
    public void mouseClicked( MouseEvent e ) 
    {
        reference.mouseClicked( scaledMouseEvent( e, 1f/zoomFactor ) );
        m_canvas.mouseClicked( e );
    }
    
    
    @Override
    public void mouseEntered( MouseEvent e ) 
    {
        reference.mouseEntered( scaledMouseEvent( e, 1f/zoomFactor ) );
        m_canvas.mouseEntered( e );
    }
    
    @Override
    public void mouseExited( MouseEvent e ) 
    {
        reference.mouseExited( scaledMouseEvent( e, 1f/zoomFactor ) );
        m_canvas.mouseExited( e );
    }
    @Override
    public void mouseDragged( MouseEvent e ) 
    {
        reference.mouseDragged( scaledMouseEvent( e, 1f/zoomFactor ) );
        m_canvas.mouseDragged( e );
    }
    
    @Override
    public void mouseMoved( MouseEvent e ) 
    {
	reference.mouseMoved( scaledMouseEvent( e, 1f/zoomFactor ) );
        m_canvas.mouseMoved( e );
    }
    
    @Override
    public void mousePressed( MouseEvent e ) 
    {
        reference.mousePressed( scaledMouseEvent( e, 1f/zoomFactor ) );
        m_canvas.mousePressed( e );
    }
    
    @Override
    public void mouseReleased( MouseEvent e ) 
    {
        reference.mouseReleased( scaledMouseEvent( e, 1f/zoomFactor ) );
        m_canvas.mouseReleased( e );
    }
    
    /* Window listener implementation */
    
    @Override
    public void windowActivated( WindowEvent e ) 
    {
        // not implemented yet!
    }
    
    @Override
    public void windowClosed( WindowEvent e ) 
    {
        // not implemented yet!
    }
    
    @Override
    public void windowClosing( WindowEvent e ) 
    {
        // copies the polygons back to the reference object 
        copyPolygonsBack( );
        Iterator it = m_listeners.iterator( );
        while( it.hasNext( ) )
	{
            ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, 0, "" ) );
        }
        
    	dispose( );
    }
    
    @Override
    public void windowDeactivated( WindowEvent e ) 
    {
        // not implemented yet!
    }
    
    @Override
    public void windowDeiconified( WindowEvent e ) 
    {
// not implemented yet!
    }
    
    @Override
    public void windowIconified( WindowEvent e ) 
    {
    }
    
    @Override
    public void windowOpened( WindowEvent e ) 
    {
        // not implemented yet!
    }
    
    /**
     * Handles the events of the menu.
     * @param actionEvent
     */
    @Override
    public void actionPerformed( ActionEvent actionEvent ) 
    {
        Object sender = actionEvent.getSource( );
        
        if ( sender == m_itemScaleUp )
        {
            m_zoom = m_zoom * 2;
            zoomFactor = m_ref_zoom / m_zoom;
            m_mainPanel.setSize( (int)( m_zoom * reference.m_imageWidth ),  
                                 (int)( m_zoom * reference.m_imageHeight ) );
            m_canvas.setZoom( m_zoom );
        }
        
        if ( sender == m_itemScaleDown )
        {
            m_zoom = m_zoom / 2;
            zoomFactor = m_ref_zoom / m_zoom;
            if ( m_zoom < 1 )
            {
                m_zoom = 1;
            }
            m_mainPanel.setSize( (int)( m_zoom * reference.m_imageWidth ),
                                 (int)( m_zoom * reference.m_imageHeight ) );
            m_canvas.setZoom( m_zoom );
        }
        
        if ( sender == m_itemShowScale )
        {
            if ( m_itemShowScale.isSelected( ) )
            {
                m_canvas.scaleSetVisible( true );
                m_canvas.update( );
            }
            else
            {
                m_canvas.scaleSetVisible( false );
                m_canvas.update( );
            }   
        }
        
        if ( sender == m_itemGrayScaleMap )
        {
             if ( m_itemGrayScaleMap.isSelected( ) )
            {
                m_canvas.setColormapType( 0 );
                m_canvas.update( true );
            }
        }
        if ( sender == m_itemColorMap )
        {
             if ( m_itemColorMap.isSelected( ) )
            {
                m_canvas.setColormapType( 1 );
                m_canvas.update( true );
            }
            
        }
        if ( sender == m_itemColorInverseMap )
        {
             if ( m_itemColorInverseMap.isSelected( ) )
            {
                m_canvas.setColormapType( 2 );
                m_canvas.update( true );
            }           
        }
    }    
    
    /**
     * Returns a reference on  the DicomSliceCanvasPolyDraw object.
     * @return 
     */
    public DicomSliceCanvasPolyDraw getSliceCanvasPolyDraw( )
    {
        return m_canvas;
    }
    
    /**
     * Adds an action listener to the list of action listeners.
     * @param e
     */
    public void addActionListener( ActionListener e )
    {
	m_listeners.add( e );
    }
    
    
    // Listeners to textfield changes
    private final LinkedList<ActionListener> m_listeners = new LinkedList<>( );
    
    DicomSliceCanvasPolyDraw m_canvas, reference;
    JPanel m_mainPanel;
    JScrollPane m_mainScrollPane;
    
    JFrame parent;
    
    private float m_zoom;
    private float zoomFactor;
    private float m_ref_zoom;
    private final JMenu m_imageMenu;
    private final JMenuItem m_itemScaleUp;
    private final JMenuItem m_itemScaleDown;
    private final JRadioButtonMenuItem m_itemShowScale;
    
    private final JRadioButtonMenuItem m_itemGrayScaleMap;
    private final JRadioButtonMenuItem m_itemColorMap;
    private final JRadioButtonMenuItem m_itemColorInverseMap;
    private final JMenuBar m_mainMenu;
    
    // For proper serialization
    private static final long serialVersionUID = 9982128732688883L;
}
