/*
 * File: $Id: CSIGrid.java,v 1.1.1.1 2006/04/19 09:38:47 hans Exp $
 * History
 * Date: 18.03.2005  Authors: Hans Slotboom
 * Date: 06.03.2014  Authors: Nuno Barros
 * Changes:
 *
 * Copyright (C) 2003 - 2005 by
 *
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */
package org.spectrim.grid;

import org.spectrim.imagegraphicobjects.Point3D;
import org.spectrim.xyplot.XYPlotFrame;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Polygon;
import java.awt.Rectangle;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.TreeMap;
import org.apache.commons.math3.linear.Array2DRowRealMatrix;
import org.apache.commons.math3.linear.ArrayRealVector;
import org.apache.commons.math3.linear.DecompositionSolver;
import org.apache.commons.math3.linear.LUDecomposition;
import org.apache.commons.math3.linear.QRDecomposition;
import org.apache.commons.math3.linear.RealMatrix;
import org.apache.commons.math3.linear.RealVector;
import org.spectrim.geometricobjects.Cube3D;
import org.spectrim.geometricobjects.ImagePlane;
import org.spectrim.geometricobjects.Line;
import org.spectrim.geometricobjects.MeshCube3D;
import org.spectrim.geometricobjects.Polygon3D;
import org.spectrim.model.GlobalModel;
import org.spectrim.model.GlobalModel.SelectionGroup;
import org.spectrim.registration.AffineTransformation;
import org.spectrim.registration.DisplacementField;
import org.spectrim.view.ImageSlice;
import org.spectrim.view.ImageStack;

/**
 * Class that defines a CSI grid.
 *
 * @author hans, nunopbarros
 */
public class CSIGrid {
    // Attributes CSIGrid

    private GlobalModel model;
    public int m_gridSizeDir1, m_gridSizeDir2, m_gridSizeDir3;
    public Cube3D excitationVoxel;
    public MeshCube3D excitationVoxelMesh;
    public HashMap<String, Point3D> excMeshPointsExt = new HashMap<>();
    public HashMap<String, Point3D> excMeshPointsInt = new HashMap<>();
    public HashMap<Integer, ArrayList<PrecisionPolygon>> excMeshPointsExtperSlice = new HashMap<>();
    public HashMap<Integer, ArrayList<PrecisionPolygon>> excMeshPointsIntperSlice = new HashMap<>();
    private Graphics2D m_g;
    private Point3D m_p;
    private Point3D m_r1;
    private Point3D m_r2;
    private double[] AmplificationFact = {1, 1};
    private LinkedHashMap<String, PrecisionPolygon> PolygonMap = new LinkedHashMap<>();
    private LinkedHashMap<String, Cube3D> Grid = new LinkedHashMap<>();
    private LinkedHashMap<String, Cube3D> displacements = new LinkedHashMap<>();
    private final int precisionFact = 1000;
    private Point3D topLeftPoint;
    private Point3D vect1;
    private Point3D vect2;
    private Point3D vect3;
    public ArrayList<Cube3D> saturationBands;
    private Cube3D gridLiner;
    private Double roundingErrAllowed = 0.0001;
    private int meshdivisionlevel = 32;
    private XYPlotFrame plotframe;
    private float gridAlpha = 1.0f;
    private int middleSlice;
    private Point3D FOV;
    private Point3D centerPoint;

    // a few attributes to enable simple segmentation of pixels within the
    // excited volume of the MRS
    Color m_wmSegmentationOverlayColor = new Color( 0f, (153f/255f), 0f, 0.1f );
    Color m_gmSegmentationOverlayColor = new Color( 1f, 0f, 0f, 0.1f );
    Color m_csfSegmentationOverlayColor = new Color( 0, 0, 1f, 0.1f );
    
    int minimum_CSF;
    int maximum_CSF;
    int minimum_GM;
    int maximum_GM;
    int minimum_WM;
    int maximum_WM;
    
    int m_numberOfGMPixelsInVoxel = -1;
    int m_numberOfWMPixelsInVoxel = -1;
    int m_numberOfCSFPixelsInVoxel = -1;
    
    
    HashMap<String, Integer> m_pixelMapSegmentatationOverlay = new HashMap<>( );
    
    /**
     * Creates a new instance of CSIGrid.
     * @param centerPoint
     * @param gridDirectionUnitVector1
     * @param gridDirectionUnitVector2
     * @param gridDirectionUnitVector3
     * @param gridSizeDir1
     * @param gridSizeDir2
     * @param gridSizeDir3
     * @param FOVDir1
     * @param FOVDir2
     * @param FOVDir3
     */
    public CSIGrid( Point3D centerPoint, 
                    Point3D gridDirectionUnitVector1,
                    Point3D gridDirectionUnitVector2, 
                    Point3D gridDirectionUnitVector3,
                    int gridSizeDir1, int gridSizeDir2, int gridSizeDir3,
                    double FOVDir1, double FOVDir2, double FOVDir3 ) 
    {

//        TESTE - INIT
//        centerPoint = new Point3D(100, 100, 10);
//        gridDirectionUnitVector1 = new Point3D(1, 0, 0);
//        gridDirectionUnitVector2 = new Point3D(0, 1, 0);
//        gridDirectionUnitVector3 = new Point3D(0, 0, 1);
//        gridSizeDir1 = 2;
//        gridSizeDir2 = 2;
//        gridSizeDir3 = 1;
//        FOVDir1 = 10;
//        FOVDir2 = 10;
//        FOVDir3 = 5;
//        TESTE - END
        
        this.centerPoint = centerPoint;
        FOV = new Point3D( FOVDir1, FOVDir2, FOVDir3 );
        saturationBands = new ArrayList<>( );

        if ( gridSizeDir1 == 0 ) 
        {
            gridSizeDir1 = 1;
        }
        if ( gridSizeDir2 == 0 ) 
        {
            gridSizeDir2 = 1;
        }
        if ( gridSizeDir3 == 0 ) 
        {
            gridSizeDir3 = 1;
        }

        m_gridSizeDir1 = gridSizeDir1;
        m_gridSizeDir2 = gridSizeDir2;
        m_gridSizeDir3 = gridSizeDir3;

        //      3 DIRECTIONS OF THE VOXEL
        double size1 = FOVDir1 / this.m_gridSizeDir1;
        double size2 = FOVDir2 / this.m_gridSizeDir2;
        double size3 = FOVDir3 / this.m_gridSizeDir3;

        vect1 = gridDirectionUnitVector1.scalarMultiplication( size1 / gridDirectionUnitVector1.length( ) );
        vect2 = gridDirectionUnitVector2.scalarMultiplication( size2 / gridDirectionUnitVector2.length( ) );
        vect3 = gridDirectionUnitVector3.scalarMultiplication( size3 / gridDirectionUnitVector3.length( ) );

//        System.out.println( "direction vect1 [dir1,dir2,dir3] = [" + gridDirectionUnitVector1.m_x1 + ", " + gridDirectionUnitVector1.m_x2 + ", " + gridDirectionUnitVector1.m_x3 + " ]" );
//        System.out.println( "direction vect2 [dir1,dir2,dir3] = [" + gridDirectionUnitVector2.m_x1 + ", " + gridDirectionUnitVector2.m_x2 + ", " + gridDirectionUnitVector2.m_x3 + " ]" );
//        System.out.println( "direction vect3 [dir1,dir2,dir3] = [" + gridDirectionUnitVector3.m_x1 + ", " + gridDirectionUnitVector3.m_x2 + ", " + gridDirectionUnitVector3.m_x3 + " ]" );      
        
        //      TRANSLATION FROM THE CENTER OF THE GRID TO THE CORNER (1st VOXEL)
        topLeftPoint = new Point3D(
                                    - ( this.m_gridSizeDir1 / 2.0 ) * vect1.m_x1
                                    - ( this.m_gridSizeDir2 / 2.0 ) * vect2.m_x1
                                    - ( this.m_gridSizeDir3 / 2.0 ) * vect3.m_x1
                                    + centerPoint.m_x1,
                                    - ( this.m_gridSizeDir1 / 2.0 ) * vect1.m_x2
                                    - ( this.m_gridSizeDir2 / 2.0 ) * vect2.m_x2
                                    - ( this.m_gridSizeDir3 / 2.0 ) * vect3.m_x2
                                    + centerPoint.m_x2,
                                    - ( this.m_gridSizeDir1 / 2.0 ) * vect1.m_x3
                                    - ( this.m_gridSizeDir2 / 2.0 ) * vect2.m_x3
                                    - ( this.m_gridSizeDir3 / 2.0 ) * vect3.m_x3
                                    + centerPoint.m_x3 );
//        System.out.println( " CSIGRID topLeftPoint = [" + topLeftPoint.m_x1 + " , " + topLeftPoint.m_x2 + " , " + topLeftPoint.m_x3 + "]" );
        computeGrid( );
    }

    /**
     * Creates a new instance of CSIGrid.
     * @param shift
     * @param centerPoint
     * @param gridDirectionUnitVector1
     * @param gridDirectionUnitVector2
     * @param gridDirectionUnitVector3
     * @param gridSizeDir1
     * @param gridSizeDir2
     * @param gridSizeDir3
     * @param FOVDir1
     * @param FOVDir2
     * @param FOVDir3
     */
    public CSIGrid( Point3D shift,
                    Point3D centerPoint, 
                    Point3D gridDirectionUnitVector1,
                    Point3D gridDirectionUnitVector2, 
                    Point3D gridDirectionUnitVector3,
                    int gridSizeDir1, int gridSizeDir2, int gridSizeDir3,
                    double FOVDir1, double FOVDir2, double FOVDir3 ) 
    {

//        TESTE - INIT
//        centerPoint = new Point3D(100, 100, 10);
//        gridDirectionUnitVector1 = new Point3D(1, 0, 0);
//        gridDirectionUnitVector2 = new Point3D(0, 1, 0);
//        gridDirectionUnitVector3 = new Point3D(0, 0, 1);
//        gridSizeDir1 = 2;
//        gridSizeDir2 = 2;
//        gridSizeDir3 = 1;
//        FOVDir1 = 10;
//        FOVDir2 = 10;
//        FOVDir3 = 5;
//        TESTE - END
        
        this.centerPoint = centerPoint;
        FOV = new Point3D( FOVDir1, FOVDir2, FOVDir3 );
        saturationBands = new ArrayList<>( );

        if ( gridSizeDir1 == 0 ) 
        {
            gridSizeDir1 = 1;
        }
        if ( gridSizeDir2 == 0 ) 
        {
            gridSizeDir2 = 1;
        }
        if ( gridSizeDir3 == 0 ) 
        {
            gridSizeDir3 = 1;
        }

        m_gridSizeDir1 = gridSizeDir1;
        m_gridSizeDir2 = gridSizeDir2;
        m_gridSizeDir3 = gridSizeDir3;

        //      3 DIRECTIONS OF THE VOXEL
        double size1 = FOVDir1 / this.m_gridSizeDir1;
        double size2 = FOVDir2 / this.m_gridSizeDir2;
        double size3 = FOVDir3 / this.m_gridSizeDir3;

        vect1 = gridDirectionUnitVector1.scalarMultiplication( size1 / gridDirectionUnitVector1.length( ) );
        vect2 = gridDirectionUnitVector2.scalarMultiplication( size2 / gridDirectionUnitVector2.length( ) );
        vect3 = gridDirectionUnitVector3.scalarMultiplication( size3 / gridDirectionUnitVector3.length( ) );

        //      TRANSLATION FROM THE CENTER OF THE GRID TO THE CORNER (1st VOXEL)
        topLeftPoint = new Point3D(
                                    - ( this.m_gridSizeDir1 / 2.0 ) * vect1.m_x1
                                    - ( this.m_gridSizeDir2 / 2.0 ) * vect2.m_x1
                                    - ( this.m_gridSizeDir3 / 2.0 ) * vect3.m_x1
                                    + centerPoint.m_x1
                                    + shift.m_x1,
                                    - ( this.m_gridSizeDir1 / 2.0 ) * vect1.m_x2
                                    - ( this.m_gridSizeDir2 / 2.0 ) * vect2.m_x2
                                    - ( this.m_gridSizeDir3 / 2.0 ) * vect3.m_x2
                                    + centerPoint.m_x2
                                    + shift.m_x2,
                                    - ( this.m_gridSizeDir1 / 2.0 ) * vect1.m_x3
                                    - ( this.m_gridSizeDir2 / 2.0 ) * vect2.m_x3
                                    - ( this.m_gridSizeDir3 / 2.0 ) * vect3.m_x3
                                    + centerPoint.m_x3
                                    + shift.m_x3 
                                   );

        computeGrid( );
    }
    
    
    // copy constructor
    public CSIGrid(CSIGrid copiedCsigrid) {
        this.model = copiedCsigrid.model;
        this.m_gridSizeDir1 = copiedCsigrid.m_gridSizeDir1;
        this.m_gridSizeDir2 = copiedCsigrid.m_gridSizeDir2;
        this.m_gridSizeDir3 = copiedCsigrid.m_gridSizeDir3;
        this.excitationVoxel = copiedCsigrid.excitationVoxel;
        this.excitationVoxelMesh = copiedCsigrid.excitationVoxelMesh;
        this.excMeshPointsExt = new HashMap<>(copiedCsigrid.excMeshPointsExt);
        this.excMeshPointsInt = new HashMap<>(copiedCsigrid.excMeshPointsInt);
        this.excMeshPointsExtperSlice = new HashMap<>(copiedCsigrid.excMeshPointsExtperSlice);
        this.excMeshPointsIntperSlice = new HashMap<>(copiedCsigrid.excMeshPointsIntperSlice);
        this.m_g = copiedCsigrid.m_g;
        this.m_p = copiedCsigrid.m_p;
        this.m_r1 = copiedCsigrid.m_r1;
        this.m_r2 = copiedCsigrid.m_r2;
        this.AmplificationFact = copiedCsigrid.AmplificationFact;
        this.PolygonMap = new LinkedHashMap<>(copiedCsigrid.PolygonMap);
        this.Grid = new LinkedHashMap<>(copiedCsigrid.Grid);
        this.displacements = new LinkedHashMap<>(copiedCsigrid.displacements);
        this.topLeftPoint = copiedCsigrid.topLeftPoint;
        this.vect1 = copiedCsigrid.vect1;
        this.vect2 = copiedCsigrid.vect2;
        this.vect3 = copiedCsigrid.vect3;
        this.saturationBands = copiedCsigrid.saturationBands;
        this.gridLiner = copiedCsigrid.gridLiner;
        this.roundingErrAllowed = copiedCsigrid.roundingErrAllowed;
        this.meshdivisionlevel = copiedCsigrid.meshdivisionlevel;
        this.plotframe = copiedCsigrid.plotframe;
        this.gridAlpha = copiedCsigrid.gridAlpha;
        this.FOV = copiedCsigrid.FOV;
        this.centerPoint = copiedCsigrid.centerPoint;
    }

    public void setModel(GlobalModel model) {
        this.model = model;
//        int sel1 = (int) Math.floor((double) this.m_gridSizeDir1 / 2);
//        int sel2 = (int) Math.floor((double) this.m_gridSizeDir2 / 2);
//        int sel3 = (int) Math.floor((double) this.m_gridSizeDir3 / 2);
//        model.selectVoxel(sel1, sel2, sel3, CSIModel.SELECTION_SINGLE);
    }

    
    /**
     * Computes the green overlay grid of the CSI data 
     */
    private void computeGrid( ) 
    {

        Grid.clear( );
        Cube3D specVoxel;

//     CONSTRUCTION OF THE GRID BASED ON THE 3 DIRECTION VECTORS
        for ( int i = 0; i < this.m_gridSizeDir1; i++ ) 
        {
            for ( int j = 0; j < this.m_gridSizeDir2; j++ ) 
            {
                for ( int k = 0; k < this.m_gridSizeDir3; k++ ) 
                {

                    Point3D p5 = Point3D.vectorCombination( vect1, vect2, vect3, i, j, k );
                    Point3D p3 = Point3D.vectorCombination( vect1, vect2, vect3, i + 1, j + 1, k + 1 );
                    Point3D p6 = Point3D.vectorCombination( vect1, vect2, vect3, i + 1, j, k );
                    Point3D p1 = Point3D.vectorCombination( vect1, vect2, vect3, i, j + 1, k );
                    Point3D p8 = Point3D.vectorCombination( vect1, vect2, vect3, i, j, k + 1 );
                    Point3D p2 = Point3D.vectorCombination( vect1, vect2, vect3, i + 1, j + 1, k );
                    Point3D p4 = Point3D.vectorCombination( vect1, vect2, vect3, i, j + 1, k + 1 );
                    Point3D p7 = Point3D.vectorCombination( vect1, vect2, vect3, i + 1, j, k + 1 );

                    p1.translate( topLeftPoint );
                    p2.translate( topLeftPoint );
                    p3.translate( topLeftPoint );
                    p4.translate( topLeftPoint );
                    p5.translate( topLeftPoint );
                    p6.translate( topLeftPoint );
                    p7.translate( topLeftPoint );
                    p8.translate( topLeftPoint);

                    specVoxel = new Cube3D( p1, p2, p3, p4, p5, p6, p7, p8 );

                    Grid.put( i + "_" + j + "_" + k, specVoxel );
                }
            }
        }

        Point3D p5 = Point3D.vectorCombination( vect1, vect2, vect3, 0, 0, 0);
        Point3D p3 = Point3D.vectorCombination( vect1, vect2, vect3, this.m_gridSizeDir1, this.m_gridSizeDir2, this.m_gridSizeDir3 );
        Point3D p6 = Point3D.vectorCombination( vect1, vect2, vect3, this.m_gridSizeDir1, 0, 0 );
        Point3D p1 = Point3D.vectorCombination( vect1, vect2, vect3, 0, this.m_gridSizeDir2, 0 );
        Point3D p8 = Point3D.vectorCombination( vect1, vect2, vect3, 0, 0, this.m_gridSizeDir3 );
        Point3D p2 = Point3D.vectorCombination( vect1, vect2, vect3, this.m_gridSizeDir1, this.m_gridSizeDir2, 0 );
        Point3D p4 = Point3D.vectorCombination( vect1, vect2, vect3, 0, this.m_gridSizeDir2, this.m_gridSizeDir3 );
        Point3D p7 = Point3D.vectorCombination( vect1, vect2, vect3, this.m_gridSizeDir1, 0, this.m_gridSizeDir3 );
        
        p1.translate( topLeftPoint );
        p2.translate( topLeftPoint );
        p3.translate( topLeftPoint );
        p4.translate( topLeftPoint );
        p5.translate( topLeftPoint );
        p6.translate( topLeftPoint );
        p7.translate( topLeftPoint );
        p8.translate( topLeftPoint );
        
//        System.out.println( "computeGrid p1 = [ " + p1.m_x1 + " , " + p1.m_x2 + " , " + p1.m_x3 + " ] " );  
//        System.out.println( "computeGrid p2 = [ " + p2.m_x1 + " , " + p2.m_x2 + " , " + p2.m_x3 + " ] " );  
//        System.out.println( "computeGrid p3 = [ " + p3.m_x1 + " , " + p3.m_x2 + " , " + p3.m_x3 + " ] " );  
//        System.out.println( "computeGrid p4 = [ " + p4.m_x1 + " , " + p4.m_x2 + " , " + p4.m_x3 + " ] " );  
//        System.out.println( "computeGrid p5 = [ " + p5.m_x1 + " , " + p5.m_x2 + " , " + p5.m_x3 + " ] " );  
//        System.out.println( "computeGrid p6 = [ " + p6.m_x1 + " , " + p6.m_x2 + " , " + p6.m_x3 + " ] " );  
//        System.out.println( "computeGrid p7 = [ " + p7.m_x1 + " , " + p7.m_x2 + " , " + p7.m_x3 + " ] " );  
//        System.out.println( "computeGrid p8 = [ " + p8.m_x1 + " , " + p8.m_x2 + " , " + p8.m_x3 + " ] " );  
        
        gridLiner = new Cube3D( p1, p2, p3, p4, p5, p6, p7, p8 );
    }

    public Point3D getTopLeftPoint( ) 
    {
        return topLeftPoint;
    }

    public Point3D getCenterPoint() {
        return centerPoint;
    }

    
    public Point3D setDir1( Point3D vec1 ) 
    {
        return new Point3D( vec1.m_x1, vec1.m_x2, vec1.m_x3 );
    }

    public Point3D setDir2( Point3D vec2 ) 
    {
        return new Point3D( vec2.m_x1, vec2.m_x2, vec2.m_x3 );
    }

    public Point3D setDir3( Point3D vec3 ) 
    {
        return new Point3D( vec3.m_x1, vec3.m_x2, vec3.m_x3 );
    }
    
    public Point3D getDir1( ) 
    {
        return new Point3D( vect1.m_x1, vect1.m_x2, vect1.m_x3 );
    }

    public Point3D getDir2( ) 
    {
        return new Point3D( vect2.m_x1, vect2.m_x2, vect2.m_x3 );
    }

    public Point3D getDir3() 
    {
        return new Point3D( vect3.m_x1, vect3.m_x2, vect3.m_x3 );
    }

    public double[] getDimsExcitedVolume( ) 
    {
        return excitationVoxel.getDimensions( );
    }
    
    public Point3D getFOV() {
        return FOV;
    }

    public boolean isIntersectionsEmpty() {
        return PolygonMap.isEmpty();
    }

    public void setExcitationVoxel(Cube3D cube) {
        excitationVoxel = cube;
//        excitationVoxelMesh = new MeshCube3D(excitationVoxel, meshdivisionlevel);
//        excMeshPointsExt = excitationVoxelMesh.getExternalPoints();
//        excMeshPointsInt = excitationVoxelMesh.getInternalPoints();
    }

    public int size() {
        return (m_gridSizeDir1 * m_gridSizeDir2 * m_gridSizeDir3);
    }

    public void setAmplificationFact(double d1, double d2) {
        AmplificationFact = new double[2];
        AmplificationFact[0] = d1;
        AmplificationFact[1] = d2;
//        System.out.println("ampl:" + d1 + " " + d2);
    }

    public void setGridAlpha(float gridAlpha) {
        this.gridAlpha = gridAlpha;
    }

    public ArrayList<double[]> getGridLinerProjection(Point3D p, Point3D r1, Point3D r2, int sizeX, int sizeY, float pixelSpacingX, float pixelSpacingY) {
        ArrayList<double[]> projectionLines = new ArrayList<>();
        m_p = p;
        m_r1 = r1;
        m_r2 = r2;

        Point3D vect1aux = r1.Clone().scalarMultiplication(((double) (sizeX) * pixelSpacingX));
        Point3D vect2aux = r2.Clone().scalarMultiplication(((double) (sizeY) * pixelSpacingY));

        projectionLines.addAll(getVoxelProjection(gridLiner, p, r1, r2, vect1aux, vect2aux));

        return projectionLines;
    }

    public ArrayList<double[]> getGridProjection(Point3D p, Point3D r1, Point3D r2, int sizeX, int sizeY, float pixelSpacingX, float pixelSpacingY) {
        ArrayList<double[]> projectionLines = new ArrayList<>();
        m_p = p;
        m_r1 = r1;
        m_r2 = r2;
        Cube3D SpectVoxelaux;

        Point3D vect1aux = r1.Clone().scalarMultiplication(((double) (sizeX) * pixelSpacingX));
        Point3D vect2aux = r2.Clone().scalarMultiplication(((double) (sizeY) * pixelSpacingY));

        for (int i = 0; i < this.m_gridSizeDir1; i++) {
            for (int j = 0; j < this.m_gridSizeDir2; j++) {
                for (int k = 0; k < this.m_gridSizeDir3; k++) {
//                    SpectVoxelaux = Grid.get(i + "_" + j + "_" + k);
                    SpectVoxelaux = getCubeWithDisplacements(i + "_" + j + "_" + k);
                    projectionLines.addAll(getVoxelProjection(SpectVoxelaux, p, r1, r2, vect1aux, vect2aux));
                }
            }
        }

        return projectionLines;
    }

    public ArrayList<double[]> getSelectedVoxelProjection(Point3D p, Point3D r1, Point3D r2, int sizeX, int sizeY, float pixelSpacingX, float pixelSpacingY, SelectionGroup group) {
        ArrayList<double[]> projectionLines = new ArrayList<>();
        String selected;
        Cube3D SpectVoxelaux;
        Point3D vect1aux = r1.Clone().scalarMultiplication(((double) (sizeX) * pixelSpacingX));
        Point3D vect2aux = r2.Clone().scalarMultiplication(((double) (sizeY) * pixelSpacingY));
        if (model.isVoxelSelected()) {
            ArrayList<String> selVoxels = model.getSelectedVoxels();
            for (int i = 0; i < selVoxels.size(); i++) {
                if (model.getSelectedVoxelGroup(i) == group) {
                    selected = (String) selVoxels.get(i);
                    String[] parts = selected.split("_");
//                SpectVoxelaux = Grid.get(parts[0] + "_" + parts[1] + "_" + parts[2]);
                    SpectVoxelaux = getCubeWithDisplacements(parts[0] + "_" + parts[1] + "_" + parts[2]);
                    if (SpectVoxelaux != null) {
                        projectionLines.addAll(getVoxelProjection(SpectVoxelaux, p, r1, r2, vect1aux, vect2aux));
                    }
                }
            }
        }
        return projectionLines;
    }

    public ArrayList<double[]> getExcitationVoxelProjection(Point3D p, Point3D r1, Point3D r2, int sizeX, int sizeY, float pixelSpacingX, float pixelSpacingY) {
        ArrayList<double[]> projectionLines = new ArrayList<>();
        Point3D vect1aux = r1.Clone().scalarMultiplication(((double) (sizeX) * pixelSpacingX));
        Point3D vect2aux = r2.Clone().scalarMultiplication(((double) (sizeY) * pixelSpacingY));
        Cube3D vox = excitationVoxel;
        if (this.displacements.containsKey("EVoxel")) {
            vox = vox.add(displacements.get("EVoxel"));
        }
        projectionLines.addAll(getVoxelProjection(vox, p, r1, r2, vect1aux, vect2aux));

        return projectionLines;
    }

    public ArrayList<double[]> getVoxelProjection(Cube3D SpectVoxelaux, Point3D p, Point3D r1, Point3D r2, Point3D vect1aux, Point3D vect2aux) {
        ArrayList<double[]> projectionLines = new ArrayList<>();
        double[] lineaux = new double[4];
        Point3D c1;
        Point3D c2;
        Point3D c3;
        Point3D c4;
        Point3D c5;
        Point3D c6;
        Point3D c7;
        Point3D c8;
        Point3D P3D = new Point3D();

        double[] point2Daux1;
        double[] point2Daux2;
        double[] point2Daux3;
        double[] point2Daux4;
        double[] point2Daux5;
        double[] point2Daux6;
        double[] point2Daux7;
        double[] point2Daux8;

        c1 = SpectVoxelaux.p1;
        c2 = SpectVoxelaux.p2;
        c3 = SpectVoxelaux.p3;
        c4 = SpectVoxelaux.p4;
        c5 = SpectVoxelaux.p5;
        c6 = SpectVoxelaux.p6;
        c7 = SpectVoxelaux.p7;
        c8 = SpectVoxelaux.p8;

        c1 = P3D.projectPointOnPlane(c1, p, r1, r2);
        c2 = P3D.projectPointOnPlane(c2, p, r1, r2);
        c3 = P3D.projectPointOnPlane(c3, p, r1, r2);
        c4 = P3D.projectPointOnPlane(c4, p, r1, r2);
        c5 = P3D.projectPointOnPlane(c5, p, r1, r2);
        c6 = P3D.projectPointOnPlane(c6, p, r1, r2);
        c7 = P3D.projectPointOnPlane(c7, p, r1, r2);
        c8 = P3D.projectPointOnPlane(c8, p, r1, r2);

        point2Daux1 = coordTransform(c1, p, vect1aux, vect2aux);
        point2Daux2 = coordTransform(c2, p, vect1aux, vect2aux);
        point2Daux3 = coordTransform(c3, p, vect1aux, vect2aux);
        point2Daux4 = coordTransform(c4, p, vect1aux, vect2aux);
        point2Daux5 = coordTransform(c5, p, vect1aux, vect2aux);
        point2Daux6 = coordTransform(c6, p, vect1aux, vect2aux);
        point2Daux7 = coordTransform(c7, p, vect1aux, vect2aux);
        point2Daux8 = coordTransform(c8, p, vect1aux, vect2aux);

        addProjectionLine(projectionLines, point2Daux1[0], point2Daux1[1], point2Daux2[0], point2Daux2[1]);
        addProjectionLine(projectionLines, point2Daux2[0], point2Daux2[1], point2Daux3[0], point2Daux3[1]);
        addProjectionLine(projectionLines, point2Daux3[0], point2Daux3[1], point2Daux4[0], point2Daux4[1]);
        addProjectionLine(projectionLines, point2Daux1[0], point2Daux1[1], point2Daux4[0], point2Daux4[1]);
        addProjectionLine(projectionLines, point2Daux1[0], point2Daux1[1], point2Daux5[0], point2Daux5[1]);
        addProjectionLine(projectionLines, point2Daux6[0], point2Daux6[1], point2Daux2[0], point2Daux2[1]);
        addProjectionLine(projectionLines, point2Daux3[0], point2Daux3[1], point2Daux7[0], point2Daux7[1]);
        addProjectionLine(projectionLines, point2Daux4[0], point2Daux4[1], point2Daux8[0], point2Daux8[1]);
        addProjectionLine(projectionLines, point2Daux5[0], point2Daux5[1], point2Daux6[0], point2Daux6[1]);
        addProjectionLine(projectionLines, point2Daux6[0], point2Daux6[1], point2Daux7[0], point2Daux7[1]);
        addProjectionLine(projectionLines, point2Daux7[0], point2Daux7[1], point2Daux8[0], point2Daux8[1]);
        addProjectionLine(projectionLines, point2Daux8[0], point2Daux8[1], point2Daux5[0], point2Daux5[1]);

        return projectionLines;
    }

    public void addProjectionLine(ArrayList<double[]> projLines, double d1, double d2, double d3, double d4) {
        double[] daux = {d1, d2, d3, d4};
        projLines.add(daux);
    }

    public void GridProjection(Graphics2D g, Point3D p, Point3D r1, Point3D r2, int sizeX, int sizeY, float pixelSpacingX, float pixelSpacingY, int Width, int Height) {
        m_p = p;
        m_r1 = r1;
        m_r2 = r2;
        Cube3D SpectVoxelaux;
        String selected;

        Point3D vect1aux = r1.Clone().scalarMultiplication(((double) (sizeX) * pixelSpacingX) / (double) Width);
        Point3D vect2aux = r2.Clone().scalarMultiplication(((double) (sizeY) * pixelSpacingY) / (double) Height);

        for (int i = 0; i < this.m_gridSizeDir1; i++) {
            for (int j = 0; j < this.m_gridSizeDir2; j++) {
                for (int k = 0; k < this.m_gridSizeDir3; k++) {
//                    SpectVoxelaux = Grid.get(i + "_" + j + "_" + k);
                    SpectVoxelaux = getCubeWithDisplacements(i + "_" + j + "_" + k);
                    VoxelProjection(SpectVoxelaux, g, Color.green, p, r1, r2, vect1aux, vect2aux);
                }
            }
        }

        if (model.isVoxelSelected()) {
            ArrayList<String> selVoxels = model.getSelectedVoxels();
            Color colorSel = new Color(0f, 0f, 0f, 0f);
            for (int i = 0; i < selVoxels.size(); i++) {
                selected = (String) selVoxels.get(i);
                if (model.getSelectedVoxelGroup(i) == GlobalModel.SelectionGroup.BLUE) {
                    colorSel = Color.BLUE;
                } else if (model.getSelectedVoxelGroup(i) == GlobalModel.SelectionGroup.RED) {
                    colorSel = Color.RED;
                }
                String[] parts = selected.split("_");
//                SpectVoxelaux = Grid.get(parts[0] + "_" + parts[1] + "_" + parts[2]);
                SpectVoxelaux = getCubeWithDisplacements(parts[0] + "_" + parts[1] + "_" + parts[2]);
                if (SpectVoxelaux != null) {
                    VoxelProjection(SpectVoxelaux, g, colorSel, p, r1, r2, vect1aux, vect2aux);
                }
            }
        }
        Cube3D vox = excitationVoxel;
        if (this.displacements.containsKey("EVoxel")) {
            vox = vox.add(displacements.get("EVoxel"));
        }
        VoxelProjection(vox, g, Color.red, p, r1, r2, vect1aux, vect2aux);

    }

    public void VoxelProjection(Cube3D SpectVoxelaux, Graphics2D g, Color color, Point3D p, Point3D r1, Point3D r2, Point3D vect1aux, Point3D vect2aux) {
        Point3D c1;
        Point3D c2;
        Point3D c3;
        Point3D c4;
        Point3D c5;
        Point3D c6;
        Point3D c7;
        Point3D c8;
        Point3D P3D = new Point3D();

        double[] point2Daux1;
        double[] point2Daux2;
        double[] point2Daux3;
        double[] point2Daux4;
        double[] point2Daux5;
        double[] point2Daux6;
        double[] point2Daux7;
        double[] point2Daux8;

        c1 = SpectVoxelaux.p1;
        c2 = SpectVoxelaux.p2;
        c3 = SpectVoxelaux.p3;
        c4 = SpectVoxelaux.p4;
        c5 = SpectVoxelaux.p5;
        c6 = SpectVoxelaux.p6;
        c7 = SpectVoxelaux.p7;
        c8 = SpectVoxelaux.p8;

        c1 = P3D.projectPointOnPlane(c1, p, r1, r2);
        c2 = P3D.projectPointOnPlane(c2, p, r1, r2);
        c3 = P3D.projectPointOnPlane(c3, p, r1, r2);
        c4 = P3D.projectPointOnPlane(c4, p, r1, r2);
        c5 = P3D.projectPointOnPlane(c5, p, r1, r2);
        c6 = P3D.projectPointOnPlane(c6, p, r1, r2);
        c7 = P3D.projectPointOnPlane(c7, p, r1, r2);
        c8 = P3D.projectPointOnPlane(c8, p, r1, r2);

        point2Daux1 = coordTransform(c1, p, vect1aux, vect2aux);
        point2Daux2 = coordTransform(c2, p, vect1aux, vect2aux);
        point2Daux3 = coordTransform(c3, p, vect1aux, vect2aux);
        point2Daux4 = coordTransform(c4, p, vect1aux, vect2aux);
        point2Daux5 = coordTransform(c5, p, vect1aux, vect2aux);
        point2Daux6 = coordTransform(c6, p, vect1aux, vect2aux);
        point2Daux7 = coordTransform(c7, p, vect1aux, vect2aux);
        point2Daux8 = coordTransform(c8, p, vect1aux, vect2aux);

        g.setColor(color);
        g.drawLine(double2Int(point2Daux1[0]), double2Int(point2Daux1[1]), double2Int(point2Daux2[0]), double2Int(point2Daux2[1]));
        g.drawLine(double2Int(point2Daux2[0]), double2Int(point2Daux2[1]), double2Int(point2Daux3[0]), double2Int(point2Daux3[1]));
        g.drawLine(double2Int(point2Daux3[0]), double2Int(point2Daux3[1]), double2Int(point2Daux4[0]), double2Int(point2Daux4[1]));
        g.drawLine(double2Int(point2Daux1[0]), double2Int(point2Daux1[1]), double2Int(point2Daux4[0]), double2Int(point2Daux4[1]));
        g.drawLine(double2Int(point2Daux1[0]), double2Int(point2Daux1[1]), double2Int(point2Daux5[0]), double2Int(point2Daux5[1]));
        g.drawLine(double2Int(point2Daux6[0]), double2Int(point2Daux6[1]), double2Int(point2Daux2[0]), double2Int(point2Daux2[1]));
        g.drawLine(double2Int(point2Daux3[0]), double2Int(point2Daux3[1]), double2Int(point2Daux7[0]), double2Int(point2Daux7[1]));
        g.drawLine(double2Int(point2Daux4[0]), double2Int(point2Daux4[1]), double2Int(point2Daux8[0]), double2Int(point2Daux8[1]));
        g.drawLine(double2Int(point2Daux5[0]), double2Int(point2Daux5[1]), double2Int(point2Daux6[0]), double2Int(point2Daux6[1]));
        g.drawLine(double2Int(point2Daux6[0]), double2Int(point2Daux6[1]), double2Int(point2Daux7[0]), double2Int(point2Daux7[1]));
        g.drawLine(double2Int(point2Daux7[0]), double2Int(point2Daux7[1]), double2Int(point2Daux8[0]), double2Int(point2Daux8[1]));
        g.drawLine(double2Int(point2Daux8[0]), double2Int(point2Daux8[1]), double2Int(point2Daux5[0]), double2Int(point2Daux5[1]));
    }

    public int double2Int(double val) {
        int res = (int) Math.round(val);
        return res;
    }

    public void calcIntersections(ImageStack imStack) {
        excMeshPointsExtperSlice.clear();
        excMeshPointsIntperSlice.clear();
        PrecisionPolygon pol = new PrecisionPolygon(precisionFact);
        PolygonMap.clear();
        Cube3D SpectVoxelaux;
        ImageSlice imSlice;
        middleSlice = 0;
        int nr = 0;
        boolean intersectionNotInterrupted = true;

        for (int slice = 0; slice < imStack.size(); slice++) {
            imSlice = imStack.get(slice);
            m_p = imSlice.imagePosition;
            m_r1 = imSlice.imageOrientationX;
            m_r2 = imSlice.imageOrientationY;
            if (size() > 1) {
                pol = CubeIntersection(m_p, m_r1, m_r2, this.gridLiner);
                if (pol != null) {

//                 Spectroscopic Grid
                    for (int i = 0; i < this.m_gridSizeDir1; i++) {
                        for (int j = 0; j < this.m_gridSizeDir2; j++) {
                            for (int k = 0; k < this.m_gridSizeDir3; k++) {
//                                SpectVoxelaux = Grid.get(i + "_" + j + "_" + k);
                                SpectVoxelaux = getCubeWithDisplacements(i + "_" + j + "_" + k);
                                pol = CubeIntersection(m_p, m_r1, m_r2, SpectVoxelaux);
                                if (pol != null) {
                                    PolygonMap.put(slice + "_" + i + "_" + j + "_" + k, pol);
                                }
                            }
                        }
                    }

//                Excited Voxel
                    pol = ExcitedVoxelIntersection(m_p, m_r1, m_r2);
                    if (pol != null) {
                        PolygonMap.put(slice + "_EVoxel", pol);
                        if (intersectionNotInterrupted) {
                            middleSlice += slice;
                            nr++;
                        }
                    } else if (nr > 0) {
                        intersectionNotInterrupted = false;
                    }

//                Saturation Bands
                    for (int i = 0; i < saturationBands.size(); i++) {
                        Cube3D vaux = saturationBands.get(i);
                        pol = CubeIntersection(m_p, m_r1, m_r2, vaux);
                        if (pol != null) {
                            PolygonMap.put(slice + "_SatVoxel" + i, pol);
                        }
                    }
                } else if (nr > 0) {
                    intersectionNotInterrupted = false;
                }

            } else if (size() == 1) {

//                Excited Voxel
                pol = ExcitedVoxelIntersection(m_p, m_r1, m_r2);
                if (pol != null) {
                    PolygonMap.put(slice + "_EVoxel", pol);
                    if (intersectionNotInterrupted) {
                        middleSlice += slice;
                        nr++;
                    }

//                Saturation Bands
                    for (int i = 0; i < saturationBands.size(); i++) {
                        Cube3D vaux = saturationBands.get(i);
                        pol = CubeIntersection(m_p, m_r1, m_r2, vaux);
                        if (pol != null) {
                            PolygonMap.put(slice + "_SatVoxel" + i, pol);
                        }
                    }
                } else if (nr > 0) {
                    intersectionNotInterrupted = false;
                }
            }

        }
        if (nr > 0) {
            middleSlice /= nr;
        }
    }

    public void drawIntersections(Graphics2D g, boolean drawMidPoint, boolean drawSatBands) {
        m_g = g;
        drawIntersections(drawMidPoint, drawSatBands);
    }

    public void drawIntersections(Graphics2D g, boolean drawMidPoint) {
        m_g = g;
        drawIntersections(g, drawMidPoint, true);
    }

    
    /**
     * Updated drawing routine for the spectral grid.
     * Now categorical data (e.g. label data) can be drawn in a color coded way.  
     * @param drawMidPoint
     * @param drawSatBands 
     */
    public void drawIntersections( boolean drawMidPoint, boolean drawSatBands ) 
    {
        int slice = model.getSlice( );
        String currentpol;
        Polygon pol;
        PrecisionPolygon precPol;
        Color coloraux = null;
        m_g.setStroke(new BasicStroke(10));
        AffineTransform transfOrig = (AffineTransform) m_g.getTransform().clone();
        AffineTransform transf = m_g.getTransform();
        double virtzoom = 20;
        transf.scale(1 / virtzoom, 1 / virtzoom);
        this.AmplificationFact[0] = this.AmplificationFact[0] * virtzoom;
        this.AmplificationFact[1] = this.AmplificationFact[1] * virtzoom;
        m_g.setTransform(transf);
        Iterator iter = PolygonMap.keySet( ).iterator( );
        while ( iter.hasNext( ) ) 
        {
            currentpol = (String) iter.next( );
            String[] parts = currentpol.split( "_" );
            if ( size( ) > 1 && parts[0].matches( String.valueOf( slice ) ) && !parts[1].contains("Voxel") ) 
            {
                pol = PolygonMap.get( currentpol ).getRescaledPolygon( this.AmplificationFact[0], this.AmplificationFact[1] );
                if ( model.isColorMapActive( ) || model.isColoredOverlayDisplayOfCSVDataFileModeActive( ) ) 
                {
                    if ( model.isColorMapActive( ) )
                    {
                        coloraux = model.getColorMap( ).getPolygonColor( Integer.valueOf( parts[1] ), 
                                                                         Integer.valueOf( parts[2] ), 
                                                                         Integer.valueOf( parts[3] ) );
                    }
                    
                    if ( model.isColoredOverlayDisplayOfCSVDataFileModeActive( ) == true )
                    {
                        String key = Integer.valueOf( parts[1] ) + "_"  + Integer.valueOf( parts[2] ) +"_"  + Integer.valueOf( parts[3] );
                        coloraux = model.getExternalCSVColorOverlayMap( ).get( key );
                        if ( coloraux == null )
                        {
                            coloraux = new Color(255,255,255,0);
                        }
                    }
                    
                    if ( coloraux != null ) 
                    {
                        m_g.setColor( coloraux );
                        m_g.fillPolygon( pol );

                        if ( model.isValuesVisible( ) && model.getValueMapMode( ) == GlobalModel.STANDARD ) 
                        {
                            float xaux = 0;
                            float yaux = 0;
                            for ( int i = 0; i < pol.npoints; i++ ) 
                            {
                                xaux += pol.xpoints[i];
                                yaux += pol.ypoints[i];
                            }
                            xaux = xaux / pol.npoints;
                            yaux = yaux / pol.npoints;

                            float xleft = 0;
                            float xright = 0;
                            int nleft = 0;
                            int nright = 0;
                            float yup = 0;
                            float ydown = 0;
                            int nup = 0;
                            int ndown = 0;
                            for (int i = 0; i < pol.npoints; i++) 
                            {
                                if (pol.xpoints[i] < xaux) 
                                {
                                    xleft += pol.xpoints[i];
                                    nleft++;
                                } else {
                                    xright += pol.xpoints[i];
                                    nright++;
                                }
                                if (pol.ypoints[i] < yaux) 
                                {
                                    yup += pol.ypoints[i];
                                    nup++;
                                } else 
                                {
                                    ydown += pol.ypoints[i];
                                    ndown++;
                                }
                            }

                            xaux = (xleft / nleft + xright / nright) / 2;
                            yaux = (yup / nup + ydown / ndown) / 2;

                            if ( ( !model.getColorMap( ).isTriChannel ) && ( !model.isColoredOverlayDisplayOfCSVDataFileModeActive( ) ) ) 
                            {
                                double value = model.getColorMap( ).getMapValue( Integer.valueOf( parts[1] ), 
                                                                                 Integer.valueOf( parts[2] ), 
                                                                                 Integer.valueOf( parts[3] ))[0];
                                if (!Double.isInfinite(value) && !Double.isNaN(value) && (model.getColorMap().getMax() != 0 || model.getColorMap().getMin() != 0)) {
                                    double logmax = Math.log10(Math.abs(model.getColorMap().getMax()));
                                    double logmin = Math.log10(Math.abs(model.getColorMap().getMin()));
                                    if (logmax > logmin) {
                                        value = (float) (value / Math.pow(10, Math.floor(logmax)));
                                    } else {
                                        value = (float) (value / Math.pow(10, Math.floor(logmin)));
                                    }
                                    DecimalFormat df = new DecimalFormat("#.##");
                                    String txtval = df.format(value);

                                    Font f = new Font("TimesRoman", Font.PLAIN, 75);
                                    m_g.setFont(f);
                                    FontMetrics fmetrics = m_g.getFontMetrics();
                                    m_g.setColor(Color.black);
                                    m_g.drawString(txtval, xaux - fmetrics.stringWidth(txtval) / 2, yaux + fmetrics.getAscent() / 2);
                                }
                            }
                        }
                    }
                }
                m_g.setColor(new Color(0.02f, 0.98f, 0.4f, gridAlpha));
                m_g.drawPolygon(pol);

            }
        }
        
        if ( ( drawSatBands ) && ( !model.getManufacturer().toUpperCase( ).contains( "PHILIPS" ) ) )
        {
            for ( int i = 0; i < saturationBands.size( ); i++ ) 
            {
                precPol = PolygonMap.get(slice + "_SatVoxel" + i);
                if (precPol != null) 
                {
                    m_g.setStroke(new BasicStroke(10));
                    m_g.setColor(Color.cyan);
                    m_g.drawPolygon(precPol.getRescaledPolygon(this.AmplificationFact[0], this.AmplificationFact[1]));
                    m_g.setColor(new Color(0f, 0.2f, 0.8f, 0.2f));
                    m_g.fillPolygon(precPol.getRescaledPolygon(this.AmplificationFact[0], this.AmplificationFact[1]));
                }
            }
        }
        
        if (size() > 1 && model.isVoxelSelected()) {
            ArrayList<String> selVoxels = model.getSelectedVoxels();
            for (int i = 0; i < selVoxels.size(); i++) {
                currentpol = selVoxels.get(i);
                String[] parts = currentpol.split("_");
                precPol = PolygonMap.get(slice + "_" + parts[0] + "_" + parts[1] + "_" + parts[2]);
                if (precPol != null) {
//                    Graphics2D auxg = (Graphics2D) m_g;
                    m_g.setStroke(new BasicStroke(25));
                    if (model.getSelectedVoxelGroup(i) == GlobalModel.SelectionGroup.BLUE) {
                        m_g.setColor(new Color(0.06f, 0.34f, 1f, 1f));
                    } else {
                        m_g.setColor(new Color(1f, 0.34f, 0.06f, 1f));
                    }
                    pol = precPol.getRescaledPolygon(this.AmplificationFact[0], this.AmplificationFact[1]);
                    m_g.drawPolygon(pol);
                    if (drawMidPoint) {
                        float xaux = 0;
                        float yaux = 0;
                        for (int j = 0; j < pol.npoints; j++) {
                            xaux += pol.xpoints[j];
                            yaux += pol.ypoints[j];
                        }
                        xaux = xaux / pol.npoints;
                        yaux = yaux / pol.npoints;

                        float xleft = 0;
                        float xright = 0;
                        int nleft = 0;
                        int nright = 0;
                        float yup = 0;
                        float ydown = 0;
                        int nup = 0;
                        int ndown = 0;
                        for (int j = 0; j < pol.npoints; j++) {
                            if (pol.xpoints[j] < xaux) {
                                xleft += pol.xpoints[j];
                                nleft++;
                            } else {
                                xright += pol.xpoints[j];
                                nright++;
                            }
                            if (pol.ypoints[j] < yaux) {
                                yup += pol.ypoints[j];
                                nup++;
                            } else {
                                ydown += pol.ypoints[j];
                                ndown++;
                            }
                        }

                        xaux = (xleft / nleft + xright / nright) / 2;
                        yaux = (yup / nup + ydown / ndown) / 2;
                        m_g.setStroke(new BasicStroke(25));
                        m_g.drawLine((int) xaux, (int) yaux - 10, (int) xaux, (int) yaux + 10);
                        m_g.drawLine((int) xaux - 10, (int) yaux, (int) xaux + 10, (int) yaux);
                    }
                    m_g.setStroke(new BasicStroke(1));
                }
            }
        }
        precPol = PolygonMap.get(slice + "_EVoxel");
        if (precPol != null) {
            m_g.setStroke(new BasicStroke(25));
            if ((m_gridSizeDir1 * m_gridSizeDir2 * m_gridSizeDir3) > 1) {
                m_g.setColor(new Color(0.96f, 0.22f, 0.24f, 1f));
            } else {
                m_g.setColor(new Color(0.02f, 0.98f, 0.4f, 1f));
            }
            m_g.drawPolygon(precPol.getRescaledPolygon(this.AmplificationFact[0], this.AmplificationFact[1]));
        }
        precPol = PolygonMap.get(slice + "_EVoxelMesh");
        if (precPol != null) {
            m_g.setStroke(new BasicStroke(25));
            m_g.setColor(new Color(0.2f, 0.2f, 1f, 0.4f));
            m_g.drawPolygon(precPol.getRescaledPolygon(this.AmplificationFact[0], this.AmplificationFact[1]));
        }
        m_g.setStroke(new BasicStroke(12));
        m_g.setColor(new Color(0.05f, 0.9f, 0.05f, 0.8f));
        if (excMeshPointsExtperSlice.containsKey(slice)) {
            ArrayList<PrecisionPolygon> pointlist = excMeshPointsExtperSlice.get(slice);
            for (int i = 0; i < pointlist.size(); i++) {
                Polygon pol2 = pointlist.get(i).getRescaledPolygon(this.AmplificationFact[0], this.AmplificationFact[1]);
                m_g.drawLine(pol2.xpoints[0], pol2.ypoints[0], pol2.xpoints[0], pol2.ypoints[0]);
            }
        }
        m_g.setStroke(new BasicStroke(8));
        m_g.setColor(new Color(0.3f, 0.5f, 1f, 0.6f));
        if (excMeshPointsIntperSlice.containsKey(slice)) {
            ArrayList<PrecisionPolygon> pointlist = excMeshPointsIntperSlice.get(slice);
            for (int i = 0; i < pointlist.size(); i++) {
                Polygon pol2 = pointlist.get(i).getRescaledPolygon(this.AmplificationFact[0], this.AmplificationFact[1]);
                m_g.drawLine(pol2.xpoints[0], pol2.ypoints[0], pol2.xpoints[0], pol2.ypoints[0]);
            }
        }

        this.AmplificationFact[0] = this.AmplificationFact[0] / virtzoom;
        this.AmplificationFact[1] = this.AmplificationFact[1] / virtzoom;
        m_g.setTransform(transfOrig);
    }

    public Font scaleFont(String text, Rectangle2D rect, Graphics2D g, Font pFont) {
        float fontSize = 1.0f;
        Font font = pFont;

        font = g.getFont().deriveFont(fontSize);
        int width = g.getFontMetrics(font).stringWidth(text);
        fontSize = (int) ((rect.getWidth() / width) * fontSize);
        return g.getFont().deriveFont(fontSize);
    }

    public static void wrapTextToPolygon(Graphics2D g, String text, Font font, Color color, java.awt.Shape shape, int x, int y, int border) {
        FontMetrics m = g.getFontMetrics(font);
        java.awt.Shape poly = shape;
        int num = 0;
        String[] words = new String[1];
        if (text.contains(" ")) {
            words = text.split(" ");
        } else {
            words[0] = text;
        }
        int yi = m.getHeight() + border;
        num = 0;
        while (num != words.length) {
            String word = words[num];
            Rectangle rect = new Rectangle((poly.getBounds().width / 2) - (m.stringWidth(word) / 2) + x - border - 1, y + yi, m.stringWidth(word) + (border * 2) + 2, m.getHeight());
            while (!poly.contains(rect)) {
                yi += m.getHeight();
                rect.y = y + yi;
                if (yi >= poly.getBounds().height) {
                    break;
                }
            }
            int i = 1;
            while (true) {
                if (words.length < num + i + 1) {
                    num += i - 1;
                    break;
                }
                rect.width += m.stringWidth(words[num + i]) + (border * 2);
                rect.x -= m.stringWidth(words[num + i]) / 2 - border;
                if (poly.contains(rect)) {
                    word += " " + words[num + i];
                } else {
                    num += i - 1;
                    break;
                }
                i = i + 1;
            }
            if (yi < poly.getBounds().height) {
                g.drawString(word, (poly.getBounds().width / 2) - (m.stringWidth(word) / 2) + x, y + yi);
            } else {
                break;
            }
            yi += m.getHeight();
            num += 1;
        }
    }

    
    public void setSegmentationWMOverlayColor( Color col )
    {
        m_wmSegmentationOverlayColor = col;
        
    }
    
    public void setSegmentationGMOverlayColor( Color col )
    {
        m_gmSegmentationOverlayColor = col;
        
    }
    
    public void setSegmentationCSFOverlayColor( Color col )
    {
        m_csfSegmentationOverlayColor = col;
        
    }
    
    public int getMaximumPixelIntensityOfExcitedVolumeAsInt( ) 
    {
        HashMap<String, Double> dispsmap = new HashMap<>( );
        
        int maximumPixelValue = Integer.MIN_VALUE;
        
        if ( m_pixelMapSegmentatationOverlay.isEmpty( ) )
        {
            prepareImageSegmentColorOverlayComputation( false );
        }
        
        
        for ( Map.Entry<String, Integer> entry : m_pixelMapSegmentatationOverlay.entrySet( ) )
        {
            int intIma = entry.getValue( );

            if ( intIma > maximumPixelValue ) 
            {
                maximumPixelValue = intIma;
            }
        }
       
        return maximumPixelValue;
    }

    public int getMinimumPixelIntensityOfExcitedVolumeAsInt( ) 
    {
        HashMap<String, Double> dispsmap = new HashMap<>( );
        int minimumPixelValue = Integer.MAX_VALUE;
        
        if ( m_pixelMapSegmentatationOverlay.isEmpty() )
        {
            prepareImageSegmentColorOverlayComputation( false );
        }
        
        for ( Map.Entry<String, Integer> entry : m_pixelMapSegmentatationOverlay.entrySet( ) )
        {
            int intIma = entry.getValue( );

            if ( intIma < minimumPixelValue ) 
            {
                minimumPixelValue = intIma;
            }
        }

        return minimumPixelValue;
    }
    
    
     /**
     * This method determines the image pixels that are within the spectroscopic
     * voxel, and segments them in to three groups.
     * It draws the three tissue types as overlay on top of the image.
     * 
     * @param overlayImg 
     * @return  
     */
    public HashMap<String, Integer> prepareImageSegmentColorOverlayComputation( boolean overlayImg )
    {
        
        double[] res;
        String currentpol;
        PrecisionPolygon precPol;
        m_pixelMapSegmentatationOverlay = new HashMap<>();
        HashMap<String, Double> dispsmap = new HashMap<>();
//        TreeMap<Integer, Double> histogram = new TreeMap<>();
        ArrayList<Double> intensities = new ArrayList<>();
      
        if ( this.excMeshPointsInt.isEmpty( ) ) 
        {
            for ( int slice = 0; slice < model.getImageStackSize(); slice++ ) 
            {
                if ( model.isVoxelSelected( ) && size( ) > 1 ) 
                { // CSI
                    Iterator iter2 = model.getSelectedVoxelsIterator();
                    while (iter2.hasNext()) 
                    {
                        currentpol = (String) iter2.next();
                        String[] parts = currentpol.split("_");
                        precPol = PolygonMap.get(slice + "_" + parts[0] + "_" + parts[1] + "_" + parts[2]);
                        getPolygonVoxels(precPol, slice, m_pixelMapSegmentatationOverlay, overlayImg);
                    }

                } 
                else if ( size( ) == 1 ) 
                { 
                    // SVS
                    precPol = PolygonMap.get( slice + "_EVoxel" );
                    getPolygonVoxels( precPol, slice, m_pixelMapSegmentatationOverlay, overlayImg );
                }
            }
        }
        else 
        {
            getPointCloudVoxels( m_pixelMapSegmentatationOverlay, dispsmap );
        }
        
        model.setSegmentationOverlayAvailable( true );
        
        return m_pixelMapSegmentatationOverlay;
    
    }
    
    
    
    /**
     * This method determines the image pixels that are within the spectroscopic
     * voxel, and segments them in to three groups.
     * It draws the three tissue types as overlay on top of the image.
     * 
     * @param overlayImg 
     * @return  
     */
    public HashMap<String, Integer> prepareImageSegmentColorOverlayComputation( boolean overlayImg,
                                                                                int min_CSF, int max_CSF, 
                                                                                int min_GM,  int max_GM,
                                                                                int min_WM,  int max_WM )
    {
        
        minimum_CSF = min_CSF;
        maximum_CSF = max_CSF;
        minimum_GM = min_GM;
        maximum_GM = max_GM;
        minimum_WM = min_WM;
        maximum_WM = max_WM;
        
        double[] res;
        String currentpol;
        PrecisionPolygon precPol;
        m_pixelMapSegmentatationOverlay = new HashMap<>();
        HashMap<String, Double> dispsmap = new HashMap<>();
//        TreeMap<Integer, Double> histogram = new TreeMap<>();
        ArrayList<Double> intensities = new ArrayList<>();
      
        if ( this.excMeshPointsInt.isEmpty( ) ) 
        {
            for ( int slice = 0; slice < model.getImageStackSize(); slice++ ) 
            {
                if ( model.isVoxelSelected( ) && size( ) > 1 ) 
                { // CSI
                    Iterator iter2 = model.getSelectedVoxelsIterator();
                    while (iter2.hasNext()) 
                    {
                        currentpol = (String) iter2.next();
                        String[] parts = currentpol.split("_");
                        precPol = PolygonMap.get(slice + "_" + parts[0] + "_" + parts[1] + "_" + parts[2]);
                        getPolygonVoxels(precPol, slice, m_pixelMapSegmentatationOverlay, overlayImg);
                    }

                } 
                else if ( size( ) == 1 ) 
                { 
                    // SVS
                    precPol = PolygonMap.get( slice + "_EVoxel" );
                    getPolygonVoxels( precPol, slice, m_pixelMapSegmentatationOverlay, overlayImg );
                }
            }
        }
        else 
        {
            getPointCloudVoxels( m_pixelMapSegmentatationOverlay, dispsmap );
        }
        
        model.setSegmentationOverlayAvailable( true );
        
        return m_pixelMapSegmentatationOverlay;
    
    }
    
    
    /**
     * Computes the number of GM,WM, and CSF pisels of the whole dataset.
     */
    public void computeSegementationPixelNumbers( )
    {
        // Reset the image segment voxel counts
        m_numberOfCSFPixelsInVoxel = 0;
        m_numberOfWMPixelsInVoxel = 0;
        m_numberOfGMPixelsInVoxel = 0;
        
        if ( !m_pixelMapSegmentatationOverlay.isEmpty( ) )
        {
            Iterator<String> iter = m_pixelMapSegmentatationOverlay.keySet( ).iterator( );
            int pixelnr = m_pixelMapSegmentatationOverlay.keySet( ).size( );
            
            int[] pixelIntensities = new int[pixelnr];
            
            int curIma = model.getCurrentlyDisplayedImageSliceNumber( );
            int cntr = 0;
            while ( iter.hasNext( ) ) 
            {
                
                String coord = iter.next( );
                String[] parts = coord.split( "_" );
                
                pixelIntensities[cntr] = m_pixelMapSegmentatationOverlay.get( coord );
                 
                int slice_no = new Integer( parts[0] );
                int x = new Integer( parts[1] );
                int y = new Integer( parts[2] );
            
                
                    
                if ( ( pixelIntensities[cntr] >= minimum_CSF ) && 
                     ( pixelIntensities[cntr] <= maximum_CSF ) )
                {
                    m_numberOfCSFPixelsInVoxel++;
                }

                else if ( ( pixelIntensities[cntr] >= minimum_GM ) && 
                          ( pixelIntensities[cntr] <= maximum_GM ) )
                {
                    m_numberOfGMPixelsInVoxel++;
                }

                else if ( ( pixelIntensities[cntr] >= minimum_WM ) && 
                          ( pixelIntensities[cntr] <= maximum_WM ) )
                {
                    m_numberOfWMPixelsInVoxel++;
                }

                cntr++;
            }
        } 
    }
    
    
    /**
     * This routine draws the segmentation overlay on top of the image stack. 
     * @param g 
     */
    public void drawSegmentationOverlay( Graphics2D g, float scaleX, float scaleY, int transX, int scaleFactorPixel ) 
    {
        m_g = g;
        
        ///////
        AffineTransform transfOrig = (AffineTransform) m_g.getTransform( ).clone( );
        
        AffineTransform transf = new AffineTransform();
        transf.scale(1.0, 1.0);
        m_g.setTransform(transf);
        
        ///////
        
        // First compute the segmentation volumes (pixel numbers)
        computeSegementationPixelNumbers( );
        
        // Compute the color-overlay of currently displayed image
        if ( !m_pixelMapSegmentatationOverlay.isEmpty( ) )
        {
            Iterator<String> iter = m_pixelMapSegmentatationOverlay.keySet( ).iterator( );
            int pixelnr = m_pixelMapSegmentatationOverlay.keySet( ).size( );
            
            int[] pixelIntensities = new int[pixelnr];
            
            int curIma = model.getCurrentlyDisplayedImageSliceNumber( );
            int cntr = 0;
            while ( iter.hasNext( ) ) 
            {
                
                String coord = iter.next( );
                String[] parts = coord.split( "_" );
                
                pixelIntensities[cntr] = m_pixelMapSegmentatationOverlay.get( coord );
                 
                int slice_no = new Integer( parts[0] );
                int x = new Integer( parts[1] );
                int y = new Integer( parts[2] );
                
                // x = x * 1000;
                // y = y * 1000;
                
                //Point dest = new Point();
                
                //transf.transform( new Point( (int) (x * AmplificationFact[0]), (int)( y * AmplificationFact[1]) ), dest );
                
                // Point scaledPoint = getScaledPoint(  new Point( x, y ), (float)AmplificationFact[0], (float)AmplificationFact[1] );
                
                Point scaledPoint = getScaledPoint(  new Point( x, y ), scaleX, scaleY );
                
                
                // Point scaledPoint = dest;
                
                g.setColor( Color.red );
                if ( slice_no == curIma )
                {
                    
                    if ( ( pixelIntensities[cntr] > minimum_CSF ) && 
                         ( pixelIntensities[cntr] < maximum_CSF ) )
                    {  
                        g.setColor( m_csfSegmentationOverlayColor );
                        g.drawRect( scaledPoint.x + transX, scaledPoint.y, scaleFactorPixel, scaleFactorPixel ); // should put a red dot there                      
                    }
                    
                    else if ( ( pixelIntensities[cntr] > minimum_GM ) && 
                              ( pixelIntensities[cntr] < maximum_GM ) )
                    {
                        g.setColor( m_gmSegmentationOverlayColor );
                        g.drawRect( scaledPoint.x + transX, scaledPoint.y, scaleFactorPixel, scaleFactorPixel ); // should put a red dot there        
                    }
                    
                    else if ( ( pixelIntensities[cntr] > minimum_WM ) && 
                              ( pixelIntensities[cntr] < maximum_WM ) )
                    {
                        g.setColor( m_wmSegmentationOverlayColor );
                        g.drawRect( scaledPoint.x + transX, scaledPoint.y, scaleFactorPixel, scaleFactorPixel ); // should put a red dot there          
                    }

                }
                
                cntr++;
                
            }
        }
        
        m_g.setTransform( transfOrig );
    }
    
    public Point getScaledPoint( Point input, float rescaleFactX, float rescaleFactY )
    {
        // double zoom = model.getImageViewer().getZoom( );
        
        Point retPoint = new Point( (int) Math.round( ( input.x * rescaleFactX ) ), 
                                    (int) Math.round( ( input.y * rescaleFactY ) ) );
        return retPoint;
    }
    
    
    /**
     * Returns the number of WM pixels in the excited volume.
     * @return 
     */
    public int getNumberOfWMPixelsInExcitedVolume( )
    {
        return this.m_numberOfWMPixelsInVoxel;
    }
    
    /**
     * Returns the number of GM pixels in the excited volume.
     * @return 
     */
    public int getNumberOfGMPixelsInExcitedVolume( )
    {
        return this.m_numberOfGMPixelsInVoxel;
    }
    
    /**
     * Returns the number of GM pixels in the excited volume.
     * @return 
     */
    public int getNumberOfCSFPixelsInExcitedVolume( )
    {
        return this.m_numberOfCSFPixelsInVoxel;
    }
    
     /**
     * Returns the number of GM pixels in the excited volume.
     * @return 
     */
    public int getTotalNumberPixelsInExcitedVolume( )
    {
        return m_pixelMapSegmentatationOverlay.keySet( ).size( );
    }
  
    
//    /**
//     * Sets the thresholds to apply for the segmentation.
//     * @param min_CSF
//     * @param max_CSF
//     * @param min_GM
//     * @param max_GM
//     * @param min_WM
//     * @param max_WM 
//     */
//    public void setTissueSegmentationThressholds( int min_CSF, int max_CSF, 
//                                                  int min_GM, int max_GM,
//                                                  int min_WM, int max_WM )
//    {
//        this.minimum_CSF = min_CSF;
//        this.maximum_CSF = max_CSF;
//        this.minimum_GM = min_GM;
//        this.maximum_GM = max_GM;
//        this.minimum_WM = min_WM;
//        this.maximum_WM = max_WM;
//    }
//    
//    
    public double[] analyseSelectedVoxelsSubImage(boolean overlayImg, double... valref) {
        double[] res;
        String currentpol;
        PrecisionPolygon precPol;
        HashMap<String, Integer> pixelmap = new HashMap<>();
        HashMap<String, Double> dispsmap = new HashMap<>();
        TreeMap<Integer, Double> histogram = new TreeMap<>();
        ArrayList<Double> intensities = new ArrayList<>();
        double meandisplacement = 0;
        double meanintensity = 0;
        double stddevintensity = 0;
        double skewness = 0;
        double kurtosis = 0;
        double m3 = 0;
        double m4 = 0;
        double min = Double.POSITIVE_INFINITY;
        double max = Double.NEGATIVE_INFINITY;
        if (valref.length == 0) {
            valref = new double[]{1};
        }

        if (this.excMeshPointsInt.isEmpty()) {
            for (int slice = 0; slice < model.getImageStackSize(); slice++) {
                if (model.isVoxelSelected() && size() > 1) { // CSI
                    Iterator iter2 = model.getSelectedVoxelsIterator();
                    while (iter2.hasNext()) {
                        currentpol = (String) iter2.next();
                        String[] parts = currentpol.split("_");
                        precPol = PolygonMap.get(slice + "_" + parts[0] + "_" + parts[1] + "_" + parts[2]);
                        getPolygonVoxels(precPol, slice, pixelmap, overlayImg);
                    }

                } else if (size() == 1) { // SVS
                    precPol = PolygonMap.get(slice + "_EVoxel");
                    getPolygonVoxels(precPol, slice, pixelmap, overlayImg);
                }
            }
        } else {
            getPointCloudVoxels(pixelmap, dispsmap);
        }
//        Analyze collected list of pixels
        Iterator<String> iter = pixelmap.keySet().iterator();
        int pixelnr = pixelmap.keySet().size();
        while (iter.hasNext()) {
            String coord = iter.next();
            double intensity = (double) pixelmap.get(coord) / valref[0];
            meanintensity += intensity;
            if (!dispsmap.isEmpty()) {
                meandisplacement += dispsmap.get(coord);
            }
            intensities.add(intensity);
            if (intensity < min) {
                min = intensity;
            } else if (intensity > max) {
                max = intensity;
            }
        }
        meanintensity = meanintensity / pixelnr;
        meandisplacement = meandisplacement / pixelnr;
        
        iter = pixelmap.keySet().iterator();
        while (iter.hasNext()) {
            double val = (double) pixelmap.get(iter.next()) / valref[0];
            stddevintensity += Math.pow((val - meanintensity), 2);
            m3 += Math.pow((val - meanintensity), 3);
            m4 += Math.pow((val - meanintensity), 4);
        }
        skewness = (m3 / pixelnr) / Math.pow(Math.sqrt(stddevintensity / (pixelnr - 1)), 3);
        kurtosis = ((m4 / pixelnr) / Math.pow(stddevintensity / pixelnr, 2)) - 3;
        stddevintensity = Math.sqrt(stddevintensity / pixelnr);
        this.model.printTxt("");
        this.model.printTxt("Intensity Values Statistics");
        if (!dispsmap.isEmpty()) {
            this.model.printTxt("Pixel Count\tMin \tMax \tMean Value\tStdDev Val\tSkewness\tKurtosis\tMean Disp");
            this.model.printNumeric(pixelnr + "\t" + min + "\t" + max + "\t" + meanintensity + "\t" + stddevintensity + "\t" + skewness + "\t" + kurtosis + "\t" + meandisplacement, true);
        } else {
            this.model.printTxt("Pixel Count\tMin \tMax \tMean Value\tStdDev Val\tSkewness\tKurtosis");
            this.model.printNumeric(pixelnr + "\t" + min + "\t" + max + "\t" + meanintensity + "\t" + stddevintensity + "\t" + skewness + "\t" + kurtosis, true);
        }

        res = new double[]{pixelnr, min, max, meanintensity, stddevintensity, skewness, kurtosis};
        return res;
//        histogram = ImageTools.histogram(intensities, (int) Math.round(Math.sqrt(pixelnr)), 0, true);
//        double[] x = new double[histogram.size()];
//        double[] y = new double[histogram.size()];
//        Iterator<Integer> iter2 = histogram.keySet().iterator();
//        int i = 0;
//        while (iter2.hasNext()) {
//            int intens = iter2.next();
//            x[i] = intens;
//            y[i] = histogram.get(intens).floatValue();
//            i++;
//        }
//        TreeMap<Double, Integer> errors = new TreeMap<>();
//        TreeMap<Double, double[]> params = new TreeMap<>();
//        double[] par = new double[5];
//        double[] parOrdered = new double[5];
//        DoubleGaussian dbgaussian = new DoubleGaussian(800, 100, 2000, 100, 0.5);
//        for (int j = 0; j < 10; j++) {
//
//            DoubleGaussianFit fit = new DoubleGaussianFit(dbgaussian, y, x);
//            double min = x[0];
//            double max = x[histogram.size() - 1];
//
//            fit.setStartParameters(new double[]{min + (max - min) / 4, 100, min + 3 * (max - min) / 4, 100, 0.5});
//            DoubleRange[] ranges = new DoubleRange[5];
//
//            ranges[0] = new DoubleRange(min, max);
//            ranges[2] = new DoubleRange(min, max);
//            ranges[1] = new DoubleRange(10, 1000);
//            ranges[3] = new DoubleRange(10, 1000);
//            ranges[4] = new DoubleRange(0, 1);
//            fit.setParameterRanges(ranges);
//            fit.performRandomMinimization(1000, 100, false);
//            par = fit.getMinimizingParameters();
//            double err = fit.getMinimumFunctionValue();
////            this.model.printOutput("\n" + j + " >>" + "Miu1\tSigma1\tMiu2\tSigma2\tRatio\tError");
//            if (par[0] < par[2]) {
////                this.model.printOutpuTSavingtoClipboard(par[0] + "\t" + par[1] + "\t" + par[2] + "\t" + par[3] + "\t" + par[4] + "\t" + err);
//                parOrdered = par;
//            } else {
////                this.model.printOutpuTSavingtoClipboard(par[2] + "\t" + par[3] + "\t" + par[0] + "\t" + par[1] + "\t" + (1d - par[4]) + "\t" + err);
//                parOrdered[0] = par[2];
//                parOrdered[1] = par[3];
//                parOrdered[2] = par[0];
//                parOrdered[3] = par[1];
//                parOrdered[4] = 1 - par[4];
//            }
//            errors.put(err, j);
//            params.put(err, parOrdered);
//            dbgaussian.setParameters(par[0], par[1], par[2], par[3], par[4]);
//            model.openPlotFrame("Histogram analysis");
//            model.addCurve(x, y, Color.BLUE, "Histogram");
//            model.addCurve(x, dbgaussian.getY(x), Color.RED, "Fitted Curve" + j);
//        }
//        System.out.println("x=[");
//        for (int m = 0; m < x.length; m++) {
//            System.out.println(x[m]);
//        }
//        System.out.println("]");
//        System.out.println("fit=[");
//        for (int m = 0; m < x.length; m++) {
//            System.out.println(dbgaussian.getY(x[m]));
//        }
//        System.out.println("]");
//        System.out.println("hist=[");
//        for (int m = 0; m < x.length; m++) {
//            System.out.println(y[m]);
//        }
//        System.out.println("]");
//        this.model.print("\n>>" + "Best Fit:" + (errors.firstEntry().getValue().intValue() + 1) + " out of " + errors.size());
//        double[] bestpar = params.firstEntry().getValue();
//        this.model.print(bestpar[0] + "\t" + bestpar[1] + "\t" + bestpar[2] + "\t" + bestpar[3] + "\t" + bestpar[4] + "\t" + errors.firstEntry().getKey().doubleValue(), true);
//
//
//        double miu1mean = 0;
//        double sigma1mean = 0;
//        double miu2mean = 0;
//        double sigma2mean = 0;
//        double ratiomean = 0;
//        double errormean = 0;
//        double miu1stddev = 0;
//        double sigma1stddev = 0;
//        double miu2mstddev = 0;
//        double sigma2stddev = 0;
//        double ratiostddev = 0;
//        double errorstddev = 0;
//        Iterator<Double> iter3 = params.keySet().iterator();
//        while (iter3.hasNext()) {
//            double err = iter3.next();
//            par = params.get(err);
//            miu1mean += par[0] / errors.size();
//            sigma1mean += par[1] / errors.size();
//            miu2mean += par[2] / errors.size();
//            sigma2mean += par[3] / errors.size();
//            ratiomean += par[4] / errors.size();
//            errormean += err / errors.size();
//        }
//        iter3 = params.keySet().iterator();
//        while (iter3.hasNext()) {
//            double err = iter3.next();
//            par = params.get(err);
//            miu1stddev += Math.pow(par[0] - miu1mean, 2) / errors.size();
//            sigma1stddev += Math.pow(par[1] - sigma1mean, 2) / errors.size();
//            miu2mstddev += Math.pow(par[2] - miu2mean, 2) / errors.size();
//            sigma2stddev += Math.pow(par[3] - sigma2mean, 2) / errors.size();
//            ratiostddev += Math.pow(par[4] - ratiomean, 2) / errors.size();
//            errorstddev += Math.pow(err - errormean, 2) / errors.size();
//        }
//        miu1stddev = Math.sqrt(miu1stddev);
//        sigma1stddev = Math.sqrt(sigma1stddev);
//        miu2mstddev = Math.sqrt(miu2mstddev);
//        sigma2stddev = Math.sqrt(sigma2stddev);
//        ratiostddev = Math.sqrt(ratiostddev);
//        errorstddev = Math.sqrt(errorstddev);
//        this.model.print("\n>>MEAN:\t" + "Miu1\tSigma1\tMiu2\tSigma2\tRatio\tError");
//        this.model.print(miu1mean + "\t" + sigma1mean + "\t" + miu2mean + "\t" + sigma2mean + "\t" + ratiomean + "\t" + errormean);
//        this.model.print("\n>>STDDEV:\t" + "Miu1\tSigma1\tMiu2\tSigma2\tRatio\tError");
//        this.model.print(miu1stddev + "\t" + sigma1stddev + "\t" + miu2mstddev + "\t" + sigma2stddev + "\t" + ratiostddev + "\t" + errorstddev);
//        this.model.print("\n>>COEFFVAR:\t" + "Miu1\tSigma1\tMiu2\tSigma2\tRatio\tError");
//        this.model.print(miu1stddev / miu1mean + "\t" + sigma1stddev / sigma1mean + "\t" + miu2mstddev / miu2mean + "\t" + sigma2stddev / sigma2mean + "\t" + ratiostddev / ratiomean + "\t" + errorstddev / errormean);
//        this.model.print("\n>>TEMP JUST TO COPY! CLASSIC, BEST, COEFFVAR, MEAN, STDDEV");
//        this.model.print(
//                pixelnr + "\t" + meanintensity + "\t" + stddevintensity + "\t" + skewness + "\t" + kurtosis + "\t" + meandisplacement
//                + "\t"
//                + bestpar[0] + "\t" + bestpar[1] + "\t" + bestpar[2] + "\t" + bestpar[3] + "\t" + bestpar[4] + "\t" + errors.firstEntry().getKey().doubleValue()
//                + "\t"
//                + miu1stddev / miu1mean + "\t" + sigma1stddev / sigma1mean + "\t" + miu2mstddev / miu2mean + "\t" + sigma2stddev / sigma2mean + "\t" + ratiostddev / ratiomean + "\t" + errorstddev / errormean
//                + "\t"
//                + miu1mean + "\t" + sigma1mean + "\t" + miu2mean + "\t" + sigma2mean + "\t" + ratiomean + "\t" + errormean
//                + "\t"
//                + miu1stddev + "\t" + sigma1stddev + "\t" + miu2mstddev + "\t" + sigma2stddev + "\t" + ratiostddev + "\t" + errorstddev, true);
    }

    public double[] analyseVoxelSubImage(String voxel, boolean overlayImg, double... valref) {
        double[] res;
        PrecisionPolygon precPol;
        HashMap<String, Integer> pixelmap = new HashMap<>();
        HashMap<String, Double> dispsmap = new HashMap<>();
        TreeMap<Integer, Double> histogram = new TreeMap<>();
        ArrayList<Double> intensities = new ArrayList<>();
        double meandisplacement = 0;
        double meanintensity = 0;
        double stddevintensity = 0;
        double skewness = 0;
        double kurtosis = 0;
        double m3 = 0;
        double m4 = 0;
        double min = Double.POSITIVE_INFINITY;
        double max = Double.NEGATIVE_INFINITY;
        if (valref.length == 0) {
            valref = new double[]{1};
        }

        for (int slice = 0; slice < model.getImageStackSize(); slice++) {
            if (model.isVoxelSelected() && size() > 1) { // CSI
                String[] parts = voxel.split("_");
                precPol = PolygonMap.get(slice + "_" + parts[0] + "_" + parts[1] + "_" + parts[2]);
                getPolygonVoxels(precPol, slice, pixelmap, overlayImg);
            }
        }

//        Analyze collected list of pixels
        Iterator<String> iter = pixelmap.keySet().iterator();
        int pixelnr = pixelmap.keySet().size();
        while (iter.hasNext()) {
            String coord = iter.next();
            double intensity = (double) pixelmap.get(coord) / valref[0];
            meanintensity += intensity;
            if (!dispsmap.isEmpty()) {
                meandisplacement += dispsmap.get(coord);
            }
            intensities.add(intensity);
            if (intensity < min) {
                min = intensity;
            } else if (intensity > max) {
                max = intensity;
            }
        }
        meanintensity = meanintensity / pixelnr;
        meandisplacement = meandisplacement / pixelnr;
        iter = pixelmap.keySet().iterator();
        while (iter.hasNext()) {
            double val = (double) pixelmap.get(iter.next()) / valref[0];
            stddevintensity += Math.pow((val - meanintensity), 2);
            m3 += Math.pow((val - meanintensity), 3);
            m4 += Math.pow((val - meanintensity), 4);
        }
        skewness = (m3 / pixelnr) / Math.pow(Math.sqrt(stddevintensity / (pixelnr - 1)), 3);
        kurtosis = ((m4 / pixelnr) / Math.pow(stddevintensity / pixelnr, 2)) - 3;
        stddevintensity = Math.sqrt(stddevintensity / pixelnr);
//        this.model.printTxt("");
//        this.model.printTxt("Intensity Values Statistics");
//        if (!dispsmap.isEmpty()) {
//            this.model.printTxt("Pixel Count\tMin \tMax \tMean Value\tStdDev Val\tSkewness\tKurtosis\tMean Disp");
//            this.model.printNumeric(pixelnr + "\t" + min + "\t" + max + "\t" + meanintensity + "\t" + stddevintensity + "\t" + skewness + "\t" + kurtosis + "\t" + meandisplacement, true);
//        } else {
//            this.model.printTxt("Pixel Count\tMin \tMax \tMean Value\tStdDev Val\tSkewness\tKurtosis");
//            this.model.printNumeric(pixelnr + "\t" + min + "\t" + max + "\t" + meanintensity + "\t" + stddevintensity + "\t" + skewness + "\t" + kurtosis, true);
//        }

        res = new double[]{min, max, meanintensity, stddevintensity, skewness, kurtosis};
        return res;
//        histogram = ImageTools.histogram(intensities, (int) Math.round(Math.sqrt(pixelnr)), 0, true);
//        double[] x = new double[histogram.size()];
//        double[] y = new double[histogram.size()];
//        Iterator<Integer> iter2 = histogram.keySet().iterator();
//        int i = 0;
//        while (iter2.hasNext()) {
//            int intens = iter2.next();
//            x[i] = intens;
//            y[i] = histogram.get(intens).floatValue();
//            i++;
//        }
//        TreeMap<Double, Integer> errors = new TreeMap<>();
//        TreeMap<Double, double[]> params = new TreeMap<>();
//        double[] par = new double[5];
//        double[] parOrdered = new double[5];
//        DoubleGaussian dbgaussian = new DoubleGaussian(800, 100, 2000, 100, 0.5);
//        for (int j = 0; j < 10; j++) {
//
//            DoubleGaussianFit fit = new DoubleGaussianFit(dbgaussian, y, x);
//            double min = x[0];
//            double max = x[histogram.size() - 1];
//
//            fit.setStartParameters(new double[]{min + (max - min) / 4, 100, min + 3 * (max - min) / 4, 100, 0.5});
//            DoubleRange[] ranges = new DoubleRange[5];
//
//            ranges[0] = new DoubleRange(min, max);
//            ranges[2] = new DoubleRange(min, max);
//            ranges[1] = new DoubleRange(10, 1000);
//            ranges[3] = new DoubleRange(10, 1000);
//            ranges[4] = new DoubleRange(0, 1);
//            fit.setParameterRanges(ranges);
//            fit.performRandomMinimization(1000, 100, false);
//            par = fit.getMinimizingParameters();
//            double err = fit.getMinimumFunctionValue();
////            this.model.printOutput("\n" + j + " >>" + "Miu1\tSigma1\tMiu2\tSigma2\tRatio\tError");
//            if (par[0] < par[2]) {
////                this.model.printOutpuTSavingtoClipboard(par[0] + "\t" + par[1] + "\t" + par[2] + "\t" + par[3] + "\t" + par[4] + "\t" + err);
//                parOrdered = par;
//            } else {
////                this.model.printOutpuTSavingtoClipboard(par[2] + "\t" + par[3] + "\t" + par[0] + "\t" + par[1] + "\t" + (1d - par[4]) + "\t" + err);
//                parOrdered[0] = par[2];
//                parOrdered[1] = par[3];
//                parOrdered[2] = par[0];
//                parOrdered[3] = par[1];
//                parOrdered[4] = 1 - par[4];
//            }
//            errors.put(err, j);
//            params.put(err, parOrdered);
//            dbgaussian.setParameters(par[0], par[1], par[2], par[3], par[4]);
//            model.openPlotFrame("Histogram analysis");
//            model.addCurve(x, y, Color.BLUE, "Histogram");
//            model.addCurve(x, dbgaussian.getY(x), Color.RED, "Fitted Curve" + j);
//        }
//        System.out.println("x=[");
//        for (int m = 0; m < x.length; m++) {
//            System.out.println(x[m]);
//        }
//        System.out.println("]");
//        System.out.println("fit=[");
//        for (int m = 0; m < x.length; m++) {
//            System.out.println(dbgaussian.getY(x[m]));
//        }
//        System.out.println("]");
//        System.out.println("hist=[");
//        for (int m = 0; m < x.length; m++) {
//            System.out.println(y[m]);
//        }
//        System.out.println("]");
//        this.model.print("\n>>" + "Best Fit:" + (errors.firstEntry().getValue().intValue() + 1) + " out of " + errors.size());
//        double[] bestpar = params.firstEntry().getValue();
//        this.model.print(bestpar[0] + "\t" + bestpar[1] + "\t" + bestpar[2] + "\t" + bestpar[3] + "\t" + bestpar[4] + "\t" + errors.firstEntry().getKey().doubleValue(), true);
//
//
//        double miu1mean = 0;
//        double sigma1mean = 0;
//        double miu2mean = 0;
//        double sigma2mean = 0;
//        double ratiomean = 0;
//        double errormean = 0;
//        double miu1stddev = 0;
//        double sigma1stddev = 0;
//        double miu2mstddev = 0;
//        double sigma2stddev = 0;
//        double ratiostddev = 0;
//        double errorstddev = 0;
//        Iterator<Double> iter3 = params.keySet().iterator();
//        while (iter3.hasNext()) {
//            double err = iter3.next();
//            par = params.get(err);
//            miu1mean += par[0] / errors.size();
//            sigma1mean += par[1] / errors.size();
//            miu2mean += par[2] / errors.size();
//            sigma2mean += par[3] / errors.size();
//            ratiomean += par[4] / errors.size();
//            errormean += err / errors.size();
//        }
//        iter3 = params.keySet().iterator();
//        while (iter3.hasNext()) {
//            double err = iter3.next();
//            par = params.get(err);
//            miu1stddev += Math.pow(par[0] - miu1mean, 2) / errors.size();
//            sigma1stddev += Math.pow(par[1] - sigma1mean, 2) / errors.size();
//            miu2mstddev += Math.pow(par[2] - miu2mean, 2) / errors.size();
//            sigma2stddev += Math.pow(par[3] - sigma2mean, 2) / errors.size();
//            ratiostddev += Math.pow(par[4] - ratiomean, 2) / errors.size();
//            errorstddev += Math.pow(err - errormean, 2) / errors.size();
//        }
//        miu1stddev = Math.sqrt(miu1stddev);
//        sigma1stddev = Math.sqrt(sigma1stddev);
//        miu2mstddev = Math.sqrt(miu2mstddev);
//        sigma2stddev = Math.sqrt(sigma2stddev);
//        ratiostddev = Math.sqrt(ratiostddev);
//        errorstddev = Math.sqrt(errorstddev);
//        this.model.print("\n>>MEAN:\t" + "Miu1\tSigma1\tMiu2\tSigma2\tRatio\tError");
//        this.model.print(miu1mean + "\t" + sigma1mean + "\t" + miu2mean + "\t" + sigma2mean + "\t" + ratiomean + "\t" + errormean);
//        this.model.print("\n>>STDDEV:\t" + "Miu1\tSigma1\tMiu2\tSigma2\tRatio\tError");
//        this.model.print(miu1stddev + "\t" + sigma1stddev + "\t" + miu2mstddev + "\t" + sigma2stddev + "\t" + ratiostddev + "\t" + errorstddev);
//        this.model.print("\n>>COEFFVAR:\t" + "Miu1\tSigma1\tMiu2\tSigma2\tRatio\tError");
//        this.model.print(miu1stddev / miu1mean + "\t" + sigma1stddev / sigma1mean + "\t" + miu2mstddev / miu2mean + "\t" + sigma2stddev / sigma2mean + "\t" + ratiostddev / ratiomean + "\t" + errorstddev / errormean);
//        this.model.print("\n>>TEMP JUST TO COPY! CLASSIC, BEST, COEFFVAR, MEAN, STDDEV");
//        this.model.print(
//                pixelnr + "\t" + meanintensity + "\t" + stddevintensity + "\t" + skewness + "\t" + kurtosis + "\t" + meandisplacement
//                + "\t"
//                + bestpar[0] + "\t" + bestpar[1] + "\t" + bestpar[2] + "\t" + bestpar[3] + "\t" + bestpar[4] + "\t" + errors.firstEntry().getKey().doubleValue()
//                + "\t"
//                + miu1stddev / miu1mean + "\t" + sigma1stddev / sigma1mean + "\t" + miu2mstddev / miu2mean + "\t" + sigma2stddev / sigma2mean + "\t" + ratiostddev / ratiomean + "\t" + errorstddev / errormean
//                + "\t"
//                + miu1mean + "\t" + sigma1mean + "\t" + miu2mean + "\t" + sigma2mean + "\t" + ratiomean + "\t" + errormean
//                + "\t"
//                + miu1stddev + "\t" + sigma1stddev + "\t" + miu2mstddev + "\t" + sigma2stddev + "\t" + ratiostddev + "\t" + errorstddev, true);
    }

    public void getPolygonVoxels(PrecisionPolygon precPol, int slice, HashMap<String, Integer> pixelmap, boolean overlayimg) {
        int minx;
        int maxx;
        int miny;
        int maxy;
        int[] xvals;
        int[] yvals;
        Polygon pol;

        if (precPol != null) {

            pol = precPol.getRescaledPolygon(1 / (model.getCurrentImage().pixelSpacingX), 1 / (model.getCurrentImage().pixelSpacingY));
            Rectangle rect = pol.getBounds();
            minx = rect.x;
            miny = rect.y;
            maxx = (int) rect.getMaxX();
            maxy = (int) rect.getMaxY();

            for (int x = minx; x <= maxx; x++) {
                for (int y = miny; y <= maxy; y++) {
                    if (pol.contains(x, y)) {
                        int[] value = new int[model.getCurrentImage().image.getNumBands()];
                        if (overlayimg) {
                            model.getOverlaySlice(slice).image.getPixel(x / 1, y / 1, value);
                        } else {
                            model.getImage(slice).image.getPixel(x / 1, y / 1, value);
                        }
//                        ((WritableRaster) model.getCurrentImage().image).setPixel(x/10, y/10, new double[] {5000,5000,5000}); // To test which part of the image is being included
                        pixelmap.put(slice + "_" + x + "_" + y, value[0]);
                    }
                }
            }
        }
    }

    public void getPointCloudVoxels(HashMap<String, Integer> pixelmap, HashMap<String, Double> dispsmap) {
        this.model.getImageStack().getPixelValuesForPointList(new ArrayList<Point3D>(this.excMeshPointsExt.values()), pixelmap, dispsmap);
        this.model.getImageStack().getPixelValuesForPointList(new ArrayList<Point3D>(this.excMeshPointsInt.values()), pixelmap, dispsmap);
    }

    public int scaleTransfX(double val) {
        int res = (int) Math.round(val * this.AmplificationFact[0]);
        return res;
    }

    public int scaleTransfY(double val) {
        int res = (int) Math.round(val * this.AmplificationFact[1]);
        return res;
    }

    public int[] getVoxelIndex(int x, int y, Point3D p, Point3D r1, Point3D r2) 
    {
        
        int[] res = {-1, -1, -1};
        
        Point3D point = coordTransformInv(x, y, p, r1, r2);

        double[][] matrixData = 
        {
            { vect1.m_x1, vect2.m_x1, vect3.m_x1 },
            { vect1.m_x2, vect2.m_x2, vect3.m_x2 },
            { vect1.m_x3, vect2.m_x3, vect3.m_x3 }
        };
        
        RealVector constants = new ArrayRealVector( new double[]{ point.m_x1 - topLeftPoint.m_x1, 
                                                                  point.m_x2 - topLeftPoint.m_x2, 
                                                                  point.m_x3 - topLeftPoint.m_x3 }, 
                                                                  false );
        
//        System.out.println( "constants 1 = " + constants.getEntry( 0 ) );
//        System.out.println( "constants 2 = " + constants.getEntry( 1 ) );
//        System.out.println( "constants 3 = " + constants.getEntry( 2 ) );
        
        RealMatrix n = new Array2DRowRealMatrix( matrixData );
        DecompositionSolver solver = new LUDecomposition( n ).getSolver( );

        try 
        {
            RealVector solution = solver.solve( constants );
            double[] sol;
            sol = solution.toArray( );
            
//            System.out.println( "sol 1 = " + sol[0] );
//            System.out.println( "sol 2 = " + sol[1] );
//            System.out.println( "sol 3 = " + sol[2] );
            
            if (    sol[0] >= -roundingErrAllowed && sol[0] <= ( this.m_gridSizeDir1 + roundingErrAllowed )
                 && sol[1] >= -roundingErrAllowed && sol[1] <= ( this.m_gridSizeDir2 + roundingErrAllowed )
                 && sol[2] >= -roundingErrAllowed && sol[2] <= ( this.m_gridSizeDir3 + roundingErrAllowed ) ) 
            {
                
                if ( sol[0] < 0 ) 
                {
                    res[0] = 0;
                } else 
                {
                    res[0] = (int) Math.floor( sol[0] );
                }
                if ( sol[1] < 0 ) 
                {
                    res[1] = 0;
                } else {
                    res[1] = (int) Math.floor( sol[1] );
                }
                if ( sol[2] < 0 ) 
                {
                    res[2] = 0;
                } else 
                {
                    res[2] = (int) Math.floor( sol[2] );
                }
                
//                System.out.println( "res 1 = " + res[0] );
//                System.out.println( "res 2 = " + res[1] );
//                System.out.println( "res 3 = " + res[2] );
                
            }
            else
            {
//                System.out.println( "Problematic solution : " );
//                System.out.println( "sol[0] = " + sol[0] );
//                System.out.println( "sol[1] = " + sol[1] );
//                System.out.println( "sol[2] = " + sol[2] );
//                System.out.println( "m_gridSizeDir1 = " + m_gridSizeDir1 + roundingErrAllowed );
//                System.out.println( "m_gridSizeDir2 = " + m_gridSizeDir2 + roundingErrAllowed );
//                System.out.println( "m_gridSizeDir3 = " + m_gridSizeDir3 + roundingErrAllowed );
            }
        } catch ( Exception ex ) 
        {
            System.out.println( "DecompositionSolver error ! " ); 
        }     
        return res;
    }

    public ArrayList<int[]> getVoxelsInside(Polygon region) {
        ArrayList<int[]> voxels = new ArrayList<>();
        int slice = this.model.getSlice();
        Iterator iter = PolygonMap.keySet().iterator();
        String currentpol;
        Polygon pol;
        while (iter.hasNext()) {
            currentpol = (String) iter.next();
            String[] parts = currentpol.split("_");
            if (size() > 1 && parts[0].matches(String.valueOf(slice)) && !parts[1].contains("Voxel")) {
                pol = PolygonMap.get(currentpol).getRescaledPolygon(this.AmplificationFact[0], this.AmplificationFact[1]);
                double meanx = 0;
                double meany = 0;
                for (int i = 0; i < pol.npoints; i++) {
                    meanx += pol.xpoints[i];
                    meany += pol.ypoints[i];
                }
                meanx = meanx / (pol.npoints);
                meany = meany / (pol.npoints);
                if (region.contains(meanx, meany)) {
                    voxels.add(new int[]{Integer.valueOf(parts[1]), Integer.valueOf(parts[2]), Integer.valueOf(parts[3])});
                }
            }
        }
        return voxels;
    }

    public PrecisionPolygon ExcitedVoxelIntersection(Point3D p, Point3D r1, Point3D r2) {
        PrecisionPolygon pol;
        Cube3D vox = excitationVoxel;
        if (this.displacements.containsKey("EVoxel")) {
            if (!this.displacements.get("EVoxel").hasNullPoint()) {
                vox = vox.add(displacements.get("EVoxel"));
            }
        }
        pol = CubeIntersection(p, r1, r2, vox);

        return pol;
    }

//    public PrecisionPolygon ExcitedVoxelMeshIntersection(Point3D p, Point3D r1, Point3D r2) {
//        PrecisionPolygon pol;
//        if (excitationVoxelMesh != null) {
//            pol = CubeIntersection(p, r1, r2, excitationVoxelMesh.getLines());
//            return pol;
//        } else {
//            return null;
//        }
//    }
    public PrecisionPolygon CubeIntersection(Point3D p, Point3D r1, Point3D r2, Cube3D cube) {
        ArrayList<Line> edges = cube.getEdges();
        return CubeIntersection(p, r1, r2, edges);
    }

    public PrecisionPolygon CubeIntersection(Point3D p, Point3D r1, Point3D r2, ArrayList<Line> edges) {
        ArrayList<Point3D> intersections = new ArrayList<>();
        PrecisionPolygon pol = new PrecisionPolygon(precisionFact);

        for (int i = 0; i < edges.size(); i++) {
            PlaneLineIntersection(edges.get(i), intersections, p, r1, r2);
        }

        int nrIntersect = intersections.size();

        if (nrIntersect == 2) {
            pol.addPoint(coordTransform(intersections.get(0), p, r1, r2)[0],
                    coordTransform(intersections.get(0), p, r1, r2)[1]);
            pol.addPoint(coordTransform(intersections.get(1), p, r1, r2)[0],
                    coordTransform(intersections.get(1), p, r1, r2)[1]);
        } else if (nrIntersect > 2) {
            Point3D norm = Point3D.normalize(Point3D.crossProduct(intersections.get(0).subtractPoint(intersections.get(1)), intersections.get(0).subtractPoint(intersections.get(2))));
            double x1 = 0;
            double x2 = 0;
            double x3 = 0;
            for (int i = 0; i < intersections.size(); i++) {
                x1 = x1 + intersections.get(i).m_x1;
                x2 = x2 + intersections.get(i).m_x2;
                x3 = x3 + intersections.get(i).m_x3;
            }
            Point3D midPoint = new Point3D(x1 / nrIntersect, x2 / nrIntersect, x3 / nrIntersect);
            HashMap<Double, Point3D> pointMap = new HashMap<>();
            pointMap.put(0d, intersections.get(0));
            double angle = 0;
            for (int i = 1; i < nrIntersect; i++) {
                angle = intersections.get(0).subtractPoint(midPoint).getAngle(intersections.get(i).subtractPoint(midPoint), norm);
                pointMap.put(angle, intersections.get(i));
            }
            Map<Double, Point3D> sortedPoints = new TreeMap<>(pointMap);
            ArrayList<Point3D> sortedList = new ArrayList<>();
            sortedList.addAll(sortedPoints.values());
            double[] point2Daux;
            for (int i = 0; i < sortedList.size(); i++) {
                point2Daux = coordTransform(sortedList.get(i), p, r1, r2);
                pol.addPoint(point2Daux[0], point2Daux[1]);
            }
        } else {
            pol = null;
        }
        return pol;
    }

    public double[] coordTransform(Point3D c1, Point3D p, Point3D r1, Point3D r2) {
        Point3D point = new Point3D();
        point.m_x1 = (c1.m_x1 - p.m_x1);
        point.m_x2 = (c1.m_x2 - p.m_x2);
        point.m_x3 = (c1.m_x3 - p.m_x3);
        double[][] matrixData = {
            {r1.m_x1, r2.m_x1},
            {r1.m_x2, r2.m_x2},
            {r1.m_x3, r2.m_x3}};
        RealVector constants = new ArrayRealVector(new double[]{point.m_x1, point.m_x2, point.m_x3}, false);
        RealMatrix n = new Array2DRowRealMatrix(matrixData);
        DecompositionSolver solver = new QRDecomposition(n).getSolver();
        double[] res = null;
        try {
            RealVector solution = solver.solve(constants);
            res = new double[2];
//            res[0] = solution.getEntry(0)+1;
//            res[1] = solution.getEntry(1)+1;
            res[0] = solution.getEntry(0);
            res[1] = solution.getEntry(1);

        } catch (Exception ex) {
            ex.printStackTrace();
        }
        return res;
    }

    public static ArrayList<double[]> coordTransform(Polygon3D plg, Point3D p, Point3D r1, Point3D r2) {
        ArrayList<double[]> resPlg = new ArrayList<>();
        for (Point3D c1 : plg.getPoints()) {
            Point3D point = new Point3D();
            point.m_x1 = (c1.m_x1 - p.m_x1);
            point.m_x2 = (c1.m_x2 - p.m_x2);
            point.m_x3 = (c1.m_x3 - p.m_x3);
            double[][] matrixData = {
                {r1.m_x1, r2.m_x1},
                {r1.m_x2, r2.m_x2},
                {r1.m_x3, r2.m_x3}};
            RealVector constants = new ArrayRealVector(new double[]{point.m_x1, point.m_x2, point.m_x3}, false);
            RealMatrix n = new Array2DRowRealMatrix(matrixData);
            DecompositionSolver solver = new QRDecomposition(n).getSolver();
            double[] res = null;
            try {
                RealVector solution = solver.solve(constants);
                res = new double[2];
                //            res[0] = solution.getEntry(0)+1;
                //            res[1] = solution.getEntry(1)+1;
                res[0] = solution.getEntry(0);
                res[1] = solution.getEntry(1);
                resPlg.add(res);
            } catch (Exception ex) {
                ex.printStackTrace();
            }
        }
        return resPlg;
    }

    public Point3D coordTransformInv( int x, int y, Point3D p, Point3D r1, Point3D r2 ) 
    {
        Point3D point = new Point3D();
        double xaux = (double) x / this.AmplificationFact[0];
        double yaux = (double) y / this.AmplificationFact[1];
        point.m_x1 = p.m_x1 + xaux * r1.m_x1 + yaux * r2.m_x1;
        point.m_x2 = p.m_x2 + xaux * r1.m_x2 + yaux * r2.m_x2;
        point.m_x3 = p.m_x3 + xaux * r1.m_x3 + yaux * r2.m_x3;
        return point;
    }

    public void PlaneLineIntersection(Line l, ArrayList<Point3D> intersections, Point3D p, Point3D r1, Point3D r2) {
        PlaneLineIntersection(l.c1, l.c2, intersections, p, r1, r2);
    }

    //Intersection of segment AB with the Plane defined in the class constructor
    public void PlaneLineIntersection(Point3D pA, Point3D pB, ArrayList<Point3D> intersections, Point3D p, Point3D r1, Point3D r2) {
        Point3D res = new Point3D();
        double[][] matrixData = {
            {r1.m_x1, r2.m_x1, pA.m_x1 - pB.m_x1},
            {r1.m_x2, r2.m_x2, pA.m_x2 - pB.m_x2},
            {r1.m_x3, r2.m_x3, pA.m_x3 - pB.m_x3}};
        RealVector constants = new ArrayRealVector(new double[]{pA.m_x1 - p.m_x1, pA.m_x2 - p.m_x2, pA.m_x3 - p.m_x3}, false);
        RealMatrix n = new Array2DRowRealMatrix(matrixData);
        DecompositionSolver solver = new LUDecomposition(n).getSolver();

        try {
            RealVector solution = solver.solve(constants);
            double[] sol;
            sol = solution.toArray();
            if (sol[2] >= -roundingErrAllowed && sol[2] <= (1 + roundingErrAllowed)) {
                res.m_x1 = pA.m_x1 + sol[2] * (pB.m_x1 - pA.m_x1);
                res.m_x2 = pA.m_x2 + sol[2] * (pB.m_x2 - pA.m_x2);
                res.m_x3 = pA.m_x3 + sol[2] * (pB.m_x3 - pA.m_x3);
                intersections.add(res);
            }
        } catch (Exception ex) {
        }
    }

    public void addSaturationBand(Point3D p1, Point3D p2, Point3D p3, Point3D p4, Point3D p5, Point3D p6, Point3D p7, Point3D p8) {
        saturationBands.add(new Cube3D(p1, p2, p3, p4, p5, p6, p7, p8));
    }

    public Cube3D getVoxel(int i, int j, int k) {
//        return Grid.get(i + "_" + j + "_" + k);
        return getCubeWithDisplacements(i + "_" + j + "_" + k);
    }

    public Cube3D getVoxel(String coord) {
        return getCubeWithDisplacements(coord);
    }

    public void applyDisplacements(boolean recalc) {
        DisplacementField disps = model.getImageStack().getDisplacements();
        if (disps != null) {
            if (this.size() > 1) {
                ArrayList<String> keys = new ArrayList<>(this.Grid.keySet());
                for (int i = 0; i < keys.size(); i++) {
                    String key = keys.get(i);
                    Cube3D vox = Grid.get(key);
                    Point3D d1 = disps.getDisplacementForPoint(vox.p1);
                    Point3D d2 = disps.getDisplacementForPoint(vox.p2);
                    Point3D d3 = disps.getDisplacementForPoint(vox.p3);
                    Point3D d4 = disps.getDisplacementForPoint(vox.p4);
                    Point3D d5 = disps.getDisplacementForPoint(vox.p5);
                    Point3D d6 = disps.getDisplacementForPoint(vox.p6);
                    Point3D d7 = disps.getDisplacementForPoint(vox.p7);
                    Point3D d8 = disps.getDisplacementForPoint(vox.p8);
                    Cube3D dispVox = new Cube3D(d1, d2, d3, d4, d5, d6, d7, d8);
                    this.displacements.put(key, dispVox);
                }
            }
            this.excitationVoxelMesh = new MeshCube3D(excitationVoxel, meshdivisionlevel);
            excitationVoxelMesh.applyDisplacements(disps);
            excMeshPointsExt = excitationVoxelMesh.getExternalPoints();
            excMeshPointsInt = excitationVoxelMesh.getInternalPoints();
            Point3D d1 = disps.getDisplacementForPoint(excitationVoxel.p1);
            Point3D d2 = disps.getDisplacementForPoint(excitationVoxel.p2);
            Point3D d3 = disps.getDisplacementForPoint(excitationVoxel.p3);
            Point3D d4 = disps.getDisplacementForPoint(excitationVoxel.p4);
            Point3D d5 = disps.getDisplacementForPoint(excitationVoxel.p5);
            Point3D d6 = disps.getDisplacementForPoint(excitationVoxel.p6);
            Point3D d7 = disps.getDisplacementForPoint(excitationVoxel.p7);
            Point3D d8 = disps.getDisplacementForPoint(excitationVoxel.p8);
            Cube3D dispVox = new Cube3D(d1, d2, d3, d4, d5, d6, d7, d8);
            this.displacements.put("EVoxel", dispVox);
            if (recalc) {
                this.calcIntersections(model.getImageStack());
            }
        }
    }

    public void applyAffine(boolean recalc) {
        AffineTransformation afft = model.getImageStack().getAffine();
        if (afft != null) {
            ArrayList<String> keys = new ArrayList<>(this.Grid.keySet());
            for (int i = 0; i < keys.size(); i++) {
                String key = keys.get(i);
                Cube3D vox = Grid.get(key);
                Grid.put(key, afft.transform(vox));
            }
            excitationVoxel = afft.transform(excitationVoxel);

            excitationVoxelMesh = new MeshCube3D(excitationVoxel, meshdivisionlevel);
            excMeshPointsExt = excitationVoxelMesh.getExternalPoints();
            excMeshPointsInt = excitationVoxelMesh.getInternalPoints();

            gridLiner = afft.transform(gridLiner);
            for (int i = 0; i < saturationBands.size(); i++) {
                saturationBands.set(i, afft.transform(saturationBands.get(i)));
            }
            if (recalc) {
                this.calcIntersections(model.getImageStack());
            }
        }
    }

    private Cube3D getCubeWithDisplacements(String key) {
        Cube3D cube = Grid.get(key);
        if (displacements.containsKey(key)) {
            if (displacements.get(key) != null) {
                Cube3D cubeDisp = displacements.get(key);
                cube = cube.add(cubeDisp);
            }
        }
        return cube;
    }

    public int getMiddleSliceWithIntersection() {
        return middleSlice;
    }
}
