/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.hlsvd;

import java.util.HashMap;
import org.spectrim.tdfdfit.SingleVoxelMeasuredSpectralData;
import org.spectrim.tdfdfit.TDFDFittingDefaults;

/**
 *  Enables the user to call HLSVDPro in a non interactive manner. It does execute the Python implementation of HLSVDPro with a batch of spectrum and HLSVD_server computes them in parallel
 * @author johan
 */
public class HLSVDPro_batch
{
    
    private String[] args = new String[]{ "testsignal", 
                                          "1024", 
                                          "0.2", 
                                          "128.0", 
                                          "x", 
                                          "C:\\Users\\johan\\Documents\\development\\java\\Netbeans\\spectrIm-QMRS\\spectrIm\\native\\target_hlsvd_native_64bits", 
                                          "1024", 
                                          "1024", 
                                          "300", 
                                          "30", 
                                          "1.0", 
                                          "3.5" };    

    int m_numberOfPointsInHLSVDPro = new Integer( TDFDFittingDefaults.getInstance( ).getProperty( TDFDFittingDefaults.HLSVD_NUMBER_OF_DATAPOINTS ) );

    /**
     * Constructs a NonInteractiveHLSVD object.
     * @param signal_re
     * @param signal_im
     * @param hlsvdNumberOfPoints
     * @param batch_size
     * @param acqBandwidth
     * @param resonanceFrequency
     */
    public HLSVDPro_batch( double[] signal_re, double[] signal_im, int hlsvdNumberOfPoints, int batch_size,
                     double acqBandwidth, double resonanceFrequency,
                     double ppmReference )
    {
        
        m_signal_re = new double[signal_re.length];
        m_signal_im = new double[signal_im.length];
        boolean MiltiFreq = true;
        System.arraycopy( signal_re, 0, m_signal_re, 0, signal_re.length );
        System.arraycopy( signal_im, 0, m_signal_im, 0, signal_im.length );
        
        TDFDFittingDefaults defaults = TDFDFittingDefaults.getInstance( );
        defaults.load( );
        
        String tdSampleStepSize             = "" + ( 1.0 / acqBandwidth );
        
        String larmorFrequency              = "" + resonanceFrequency;
        
        String numberOfPointsInHLSVD        = defaults.getProperty( TDFDFittingDefaults.HLSVD_NUMBER_OF_DATAPOINTS );
        String hankelMatrixSize             = defaults.getProperty( TDFDFittingDefaults.HLSVD_HANKEL_MATRIX_SIZE );
        String numberOfHLSVDComponents      = defaults.getProperty( TDFDFittingDefaults.HLSVD_NUMBER_OF_SINUSOIDS );
        // we call hlsvd itself by interchanging high and low point nummers of frequencies.
        m_numberOfPoints                    = hlsvdNumberOfPoints;
        m_hlsvdMinimumFilterFrequency       = defaults.getProperty( TDFDFittingDefaults.HLSVD_MINUMUM_FILTER_FREQUENCY );
        m_hlsvdMaximumFilterFrequency       = defaults.getProperty( TDFDFittingDefaults.HLSVD_MAXIMUM_FILTER_FREQUENCY );
        m_hlsvdRangeFilterFrequency         = defaults.getProperty( TDFDFittingDefaults.HLSVD_RANGES_FILTER_FREQUENCY);
        if(m_hlsvdRangeFilterFrequency=="" || m_hlsvdRangeFilterFrequency.equals("")){
            MiltiFreq = false;
        }
        m_extremeLineWidthThresshold        = defaults.getProperty( TDFDFittingDefaults.HLSVD_EXTREME_LINE_WIDTH_THRESSHOLD ); 
        m_hlsvdFilterMode                   = defaults.getProperty( TDFDFittingDefaults.HLSVD_FILTER_TYPE );
        m_extremeBroadWidth                 = defaults.getProperty( TDFDFittingDefaults.HLSVD_EXTREME_LINE_WIDTH_THRESSHOLD );
        //m_batch_size                        = defaults.getProperty( TDFDFittingDefaults.HLSVD_BATCH_SIZE );
        m_batch_size                        = batch_size;


        HashMap<String, String> arguments = new HashMap<String, String>();
        
        arguments.put("ndp", ""+m_numberOfPoints);
        arguments.put("step",tdSampleStepSize);
        arguments.put("larmor",larmorFrequency);
        arguments.put("hankelSize",hankelMatrixSize);
        arguments.put("hlsvdComponents", numberOfHLSVDComponents);
        
        arguments.put("hlsvdFilterMode", m_hlsvdFilterMode);
        arguments.put("broadComponent", m_extremeBroadWidth);
        arguments.put("batchSize", ""+m_batch_size);

        float totalSpectralWidthPPM = (float)( acqBandwidth * 1.0E6 / resonanceFrequency );
        
        String[] parts = m_hlsvdRangeFilterFrequency.split(";");
        double min_freq;
        double max_freq;
        String final_aux = "";
        String finalRange = "";
        for(int i=0; i<parts.length; i++){
            String[] limits = parts[i].split("#");
            min_freq = Double.parseDouble(limits[0]);
            max_freq = Double.parseDouble(limits[1]);
            min_freq = ( new Double( min_freq ) - ppmReference ) * acqBandwidth / totalSpectralWidthPPM;
            max_freq = ( new Double( max_freq ) - ppmReference ) * acqBandwidth / totalSpectralWidthPPM;
            if(min_freq>max_freq){
                final_aux = max_freq + "#" + min_freq;
            }
            else{
                final_aux = min_freq + "#" + max_freq;
            }
            if(i==0){
                finalRange = final_aux;
            }
            else{
                finalRange = finalRange + ";" + final_aux;
            }
        }
        arguments.put("hlsvdRangeFrequencies", finalRange);
        
        //This will result in a set of vectors of size batch_size*numberOfPointsInHLSVD
        m_hlsvd = new Compute_HLSVDPro_batch( arguments, m_signal_re, m_signal_im );
        m_hlsvd.computeHLSVDPro_batch( );
        m_hlsvdResults = m_hlsvd.getHLSVDResults( );

        int numberOfPoints = Integer.parseInt(numberOfPointsInHLSVD);
        double[] batch_re = new double[numberOfPoints];
        double[] batch_im = new double[numberOfPoints];
        for( int j=0; j<m_batch_size; j++ ){

            batch_re = m_hlsvdResults.getFilteredSignalRealPart(j);
            batch_im = m_hlsvdResults.getFilteredSignalImagenaryPart(j);

            float[] partReal = new float[batch_re.length];
            float[] partImag = new float[batch_im.length];
            for (int i = 0 ; i < batch_im.length; i++)
            {
                partReal[i] = (float) batch_re[i];
                partImag[i] = (float) batch_im[i];
            }

            m_residualWaterResonanceSignal = new SingleVoxelMeasuredSpectralData( numberOfPoints );
            m_residualWaterResonanceSignal.addTimeDomainResponse( partReal, partImag );

            float[] real_water = m_residualWaterResonanceSignal.getAbsorptionChannelTD( ); 
            float[] imag_water = m_residualWaterResonanceSignal.getDispersionChannelTD( );

            // Subtract water signal..
            
            for ( int i = 0; i < real_water.length; i++ )
            {
                signal_re[j*numberOfPoints + i] = signal_re[j*numberOfPoints + i] - real_water[i];
                signal_im[j*numberOfPoints + i] = signal_im[j*numberOfPoints + i] - imag_water[i];
            }
            
            /*for ( int i = 0; i < real_water.length; i++ )
            {
                System.out.println(signal_re[j*numberOfPoints + i]);
            }*/
            m_hlsvdResults.setFilteredSignalRealPart( signal_re, j );
            m_hlsvdResults.setFilteredSignalImagenaryPart( signal_im, j );
        }
    }

    /**
     * Gives calling instance access to the results of the HLSVD.
     * @return 
     */
    public HLSVDResults_batch getHLSVDResults( )
    {
        return m_hlsvdResults;
    }
    
    // Member variables
    private final double[]          m_signal_re;
    private final double[]          m_signal_im;
    private HLSVDResults_batch      m_hlsvdResults;
    private Compute_HLSVDPro_batch  m_hlsvd;
    private final String            m_hlsvdRangeFilterFrequency;
    private final String            m_hlsvdMinimumFilterFrequency;
    private final String            m_hlsvdMaximumFilterFrequency;
    private final String            m_extremeBroadWidth;
    private final String            m_extremeLineWidthThresshold;
    private final String            m_hlsvdFilterMode;
    private SingleVoxelMeasuredSpectralData m_residualWaterResonanceSignal;
    private final int m_batch_size;
    private final int m_numberOfPoints;
    
}