
      subroutine lanczo(yr,yi,n1,k1,np,nit,id7,vr,vi,sv,ndiv,nitf)

c======================================================================
c purpose:
c   solve singular value decomposition problem in an iterative way
c
c usage: 
c   call lanczo(yr,yi,n1,k1,np,nit,id7,vr,vi,sv,ndiv,nitf)
c
c   integer*4 n1,k1,np,nit,id7,ndiv,nitf
c   real*8    yr(n1),yi(n1)
c   real*8    vr(id7,np),vi(id7,np),sv(n1)
c
c description of variables:
c   on entry:
c      yr     - data array (real part of time-domain signal)
c      yi     - data array (imaginary part of time-domain signal)
c      n1     - number of datapoints used in the hlsvd analysis
c      k1     - sizeparameter of hankel datamatrix
c      np     - start value for number of signal singular values
c      nit    - number of iterations entered
c      id7    - integer value for array dimensions 
c
c   on exit:
c      vr     - twodimensional array of eigenvectors (column-wise)
c               real parts
c      vi     - twodimensional array of eigenvectors (column-wise)
c               imaginary parts
c      sv     - array of signal singular values
c      ndiv   - number of singular values found
c      nitf   - final number of iterations
c
c subroutines required:
c   fftwp,lsvlan,evscan,dgtsl
c
c written by w.w.f. pijnappel
c modifications by a. v.d. boogaart
c copyright university of technology delft
c hlsvd version 92.1 (last update 21-4-1992)
c gnuplot monitor added by r. de beer 4-3-2015
c======================================================================

      integer*4 n1,k1,np,nit,id7,ndiv,nitf
      real*8    yr(*),yi(*)
      real*8    vr(id7,*),vi(id7,*),sv(*)

      integer*4 la,ma,na,nry
      
      parameter ( id1=4096,la=id1/2,na=2*(la+1),nry=id1/2,ma=2500 )
      
      integer*4 extra,first,i,i1,ierr,info,iq,j,ji,jj,k,kk,luout,
     +          n,nev,nik,nj,nmmn
      real*8    eps,mum,mup,pse,pst,st1,st2,tot
      real*8    alpha(ma),beta(ma),b0(ma),c0(ma),d0(ma),e0(ma),
     +          lr(la),li(la),v0(na),v1(na),rr(la),ri(la),u(na),w(na)
      real*8    hh(nry,ma),tk(ma,2),tk1(ma,2)
      logical*4 div,low,up,restrt,xy
      
      real*4    per, xvalue, yvalue

c
c la=#datapoints; na=2*(#datapoints+1)
c nry=2*#rows of the datamatrix; ma=#iterations
c
c ma can be changed into another value at any time
c

      luout = 6
      kk    = 0
      nmmn  = 1
      
c
c nmmn is the number of inverse iterations (on from label 80)
c

c      pse = 1.0d-6
      pse = 1.0d-5
      
c
c precision of 1.0d-5 suffices, but for a noiseless signal it may be
c changed into 1.0d-6 in order to then obtain zeroes instead of
c truncating decimals
c
      do 10 i=1,n1
         lr(i)=yr(n1-i+1)
         li(i)=yi(n1-i+1)
   10 continue

      call fftwp(lr,li,n1,n1,n1,1)

      n=2*(n1+1)
      nj=n1

      do 20 i=1,k1
         v1(i)=0.0d0
         v1(n1+1+i)=0.0d0
   20 continue
      do 30 i=1,n1-k1+1
         v1(k1+i)=dsin(dble(i))
         v1(n1+k1+1+i)=dsin(dble(n1-k1+1+i))
   30 continue

c
c extra is an integer value containing the number of additional
c iterations in case of an insufficient number of iterations
c entered by the user. if e.g. extra=5, then first m iterations
c (entered by the user) are done, if the test is negative another 
c five, if the test is still negative another five and so on.
c if one wants a different value for extra, it should be inserted
c here
c     
      extra=nit/10
      if (extra .eq. 0) extra=1
      restrt=.false.
      xy=.false.
      first=1
      nik=nit
      
      per    = 2.0 
      xvalue = 1.0
      yvalue = 0.0      
      open (16,file = "hlsvd_bar_data.txt", status = "unknown")                  
      write (16,*) xvalue, yvalue
      write (16,*) xvalue, per                                
      close (16)  
      
      call bar2gnuplot()         

      open(60,file='track.sv',status='unknown')

   40 call lsvlan(n,first,nik,restrt,xy,v0,v1,u,w,
     +            alpha,beta,lr,li,k1,n1,nj,rr,ri,hh,kk,nry)
                   
      call evscan(nik,n,ma,alpha,beta,nev,low,up,div,ndiv,
     +            tk,tk1,ierr,eps)

      write(60,45) nik,ndiv
   45 format(' after ',i4,' iterations ',i3,' sinvals found')

      per = (100.0*ndiv)/np
      
      if (nik.eq.100) then
        write (*,'("per in lanczos = ",f6.3," %")') per                 
        open (16,file = "hlsvd_bar_data.txt", status = "unknown",        
     +  position = "append")                  
        write (16,*) xvalue, per                          
        close (16)         
      endif
      if (nik.eq.200) then
        write (*,'("per in lanczos = ",f6.3," %")') per
        open (16,file = "hlsvd_bar_data.txt", status = "unknown",
     +  position = "append")                  
        write (16,*) xvalue, per                          
        close (16) 
      endif
      if (nik.eq.500) then
        write (*,'("per in lanczos = ",f6.3," %")') per
        open (16,file = "hlsvd_bar_data.txt", status = "unknown",
     +  position = "append")                  
        write (16,*) xvalue, per                          
        close (16) 
      endif
      if (nik.eq.1000) then
        write (*,'("per in lanczos = ",f6.3," %")') per
        open (16,file = "hlsvd_bar_data.txt", status = "unknown",
     +  position = "append")                  
        write (16,*) xvalue, per                          
        close (16) 
      endif
      if (nik.eq.2000) then
        write (*,'("per in lanczos = ",f6.3," %")') per
        open (16,file = "hlsvd_bar_data.txt", status = "unknown",
     +  position = "append")                  
        write (16,*) xvalue, per                          
        close (16) 
      endif     
      if (ndiv.eq.np) then
        write (*,'("per in lanczos = ",f6.3," %")') per
        open (16,file = "hlsvd_bar_data.txt", status = "unknown",
     +  position = "append")                  
        write (16,*) xvalue, per                          
        close (16) 
      endif      
      
      if (ndiv .lt. np) then
         restrt=.true.
         first=nik+1                           
         nik=nik+extra

c
c added at 21-4-1992
c         
         if (nik.gt.ma) then
           Write(luout,47)
   47      format(/,' ==========================================',/
     +              ' = maximum number of iterations exceeded! =',/
     +              ' ==========================================',/)
           nik=nik-extra
           goto 1234
         endif
         
         goto 40
      endif
      
 1234 continue
c
c
c 
       
      nitf=nik
      
c
c added at 21-4-1992
c
      if(nitf.eq.ma) nitf = nitf-1
c
c
c            

      close(60)

      if(ierr.ne.0) write(*,'( ''error in eigenvaluecomputation'')')
      if(low) write(*,'('' convergence at lower side'')')
      if(up) write(*,'('' convergence at upper side'')')
      write(luout,'(i3,'' singular values asked'')') np
      write(luout,'(i3,'' singular values found'')') ndiv

      do 200 i1=1,np
         mup=-tk(i1,1)*1.6d0
         mum=-tk(i1,1)*0.4d0
         st1=mup-alpha(1)
         st2=mum-alpha(1)
         ji=nitf
         do 50 i=2,nitf
            st1=(mup-alpha(i))-beta(i)*beta(i)/st1
            st2=(mum-alpha(i))-beta(i)*beta(i)/st2
            if ( st1*st2 .le. 0.0d0) then
                ji=i
                goto 60
                endif
   50    continue
   60    continue
         do 150 jj=ji,nitf
            do 70 i=1,jj-1
               e0(i)=beta(i)
               c0(i+1)=e0(i)
               d0(i)=alpha(i)+tk(i1,1)
   70       continue
            d0(jj)=alpha(jj)+tk(i1,1) 
            pst=dabs(d0(jj))
            do 80 i=1,jj-1
               pst=pst+dabs(d0(i))+c0(i+1)+e0(i)
   80       continue
            pst=pst*(2.0d0**(-52))

            do 90 i=1,jj
               b0(i)=pst
   90       continue

            do 130 iq=1,nmmn-1

               call dgtsl(jj,b0,c0,d0,e0,info)

               tot=0.0d0
               do 100 i=1,jj
                  tot=tot+b0(i)*b0(i)
  100          continue

               tot=dsqrt(tot/jj)/pst
               do 110 i=1,jj
                  b0(i)=b0(i)/tot
  110          continue

               do 120 i=1,jj-1
                  e0(i)=beta(i)
                  c0(i+1)=e0(i)
                  d0(i)=alpha(i)+tk(i1,1)
  120          continue
               d0(jj)=alpha(jj)+tk(i1,1) 
  130       continue

            call dgtsl(jj,b0,c0,d0,e0,info)

            tot=0.0d0
            do 140 i=1,jj
               tot=tot+b0(i)*b0(i)
  140       continue
            tot=dsqrt(tot)
            if ( jj .eq. nitf ) then
               ji=jj      
            endif
            tot=dabs((b0(jj)/tot)*beta(jj+1))-pse
            if (tot.le.1.0d-10) then
               ji=jj
               goto 160
            endif
  150    continue
      
c new end

  160    tot=0.0d0
         do 180 j=1,n1-k1+1
            vr(j,i1)=0.0d0
            vi(j,i1)=0.0d0
            do 170 k=1,ji
               vr(j,i1)=vr(j,i1)+hh(j,k)*b0(k)
               vi(j,i1)=vi(j,i1)-hh(n1-k1+j+1,k)*b0(k)
  170       continue
            tot=tot+vr(j,i1)*vr(j,i1)+vi(j,i1)*vi(j,i1)
  180    continue

         tot=dsqrt(tot)
         do 190 i=1,n1-k1+1
            vr(i,i1)=vr(i,i1)/tot
            vi(i,i1)=vi(i,i1)/tot
  190    continue
  200 continue

      do 210 i=1,ndiv
         sv(i)=-tk(i,1)
  210 continue

      return
      end
