/*
 * File: $Id: AdvancedInteractiveSegmentationAnalyzerFrame.java,v 1.1.1.1 2006/04/19 09:38:48 hans Exp $
 * History
 * Date: 03.09.2004  Authors: Hans Slotboom  
 *
 * Changes:
 *
 * Copyright (C) 2004 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */


/*
 * AdvancedInteractiveSegmentationAnalyzerFrame.java
 *
 * Created on April 11, 2008, 8:32 AM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package org.spectrim.imageprocessing;

import org.spectrim.dicom.DicomImageSeriesData;
import org.spectrim.dicom.DicomSliceCanvasPolyDraw;
import org.spectrim.dicom.FileInfo;
import org.spectrim.imagegraphicobjects.RegioPolygon;
import org.spectrim.lib.Dialogs;
import org.spectrim.signalprocessing.SignalTools;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JToggleButton;

/**
 *
 * @author hans
 */
public class AdvancedInteractiveSegmentationAnalyzerFrame extends JFrame implements ActionListener
{

    
    /** 
     * Creates a new instance of AdvancedInteractiveSegmentationAnalyzerFrame
     */
    public AdvancedInteractiveSegmentationAnalyzerFrame( JFrame parent, int[] rawData, FileInfo fInfo, 
                                                         LinkedHashMap siemensSpecific, int numberOfImages,
                                                         int minimumImageNumberToFocus, int maximumImageNumberToFocus )
    {
        m_parent  = parent;

        m_minimumImageNumberToFocus = minimumImageNumberToFocus;
        m_maximumImageNumberToFocus = maximumImageNumberToFocus;

        m_collectionPanel = new JPanel( );
        
        m_collectionPanel.setLayout( new BorderLayout( ) );
        
        setTitle( "Advanced interactive segmentation" );
        
        m_northCollectionPanel = new JPanel( );
        m_northCollectionPanel.setLayout( new BoxLayout( m_northCollectionPanel, BoxLayout.X_AXIS ) );
        
        //////////////////////////////////////////////////////////////////////////////////////////////////////
        //////// NORTH WEST PANEL
        
        m_northWestCollectionPanel = new JPanel( );
        m_northWestCollectionPanel.setLayout( new BoxLayout( m_northWestCollectionPanel, BoxLayout.Y_AXIS ) );
        m_northWestCollectionPanel.setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ) );
        
        
        createBasicTissueTypesDefinitionDicomImagePanel( rawData, fInfo, siemensSpecific, numberOfImages );
        m_northWestCollectionPanel.add( m_dicomInputImagePanel );
        
        createROIModePanelLeft( );
        m_northWestCollectionPanel.add( m_canvasROIModePanelLeft );
        
        createBasicTissueTypesDefinitionROIPropertiesPanel( );
        m_northWestCollectionPanel.add( m_ROIPropertiesScrollPane );
        
        //////////////////////////////////////////////////////////////////////////////////////////////////////
        
        //////// NORTH EAST PANEL
        
        m_northEastCollectionPanel = new JPanel( );
        m_northEastCollectionPanel.setLayout( new BoxLayout( m_northEastCollectionPanel, BoxLayout.Y_AXIS ) );
        m_northEastCollectionPanel.setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ) );
        
        createSegmentationResultsDicomImagePanel( rawData, fInfo, siemensSpecific, numberOfImages );
        m_northEastCollectionPanel.add( m_dicomSegmentationResultsImagePanel );
        
        createROIModePanelRight( );
        m_northEastCollectionPanel.add( m_canvasROIModePanelRight );
        
        createSegmentationResultsROIPropertiesPanel( );
        m_northEastCollectionPanel.add( m_segmentationResultsROIPropertiesScrollPane, BorderLayout.CENTER );
        
        
        // set the north panels
        m_northCollectionPanel.add( m_northWestCollectionPanel );
        m_northCollectionPanel.add( m_northEastCollectionPanel );
        
        //////////////////////////////////////////////////////////////////////////////////////////////////////
        //////// CENTER WEST PANEL
        
        
        m_centerCollectionPanel = new JPanel( );
        m_centerCollectionPanel.setLayout( new BoxLayout( m_centerCollectionPanel, BoxLayout.X_AXIS ) );
        
        // center west-side
        m_centerWestCollectionPanel = new JPanel( );
        m_centerWestCollectionPanel.setLayout( new BoxLayout( m_centerWestCollectionPanel, BoxLayout.Y_AXIS ) );
        m_centerWestCollectionPanel.setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ) );
        
        
        createProbabilitySegmentationProbingRegionPropertiesPanel( );
        m_centerWestCollectionPanel.add( m_probingRegionPropertiesPanel );
        
               
        //////////////////////////////////////////////////////////////////////////////////////////////////////
        //////// CENTER EAST PANEL
       
        m_centerEastCollectionPanel = new JPanel( );
        m_centerEastCollectionPanel.setLayout( new BoxLayout( m_centerEastCollectionPanel, BoxLayout.Y_AXIS ) );
        m_centerEastCollectionPanel.setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ) );
        
        createSegmentationResultsAnalyzerPanel( );  
        m_centerEastCollectionPanel.add( m_segmentationResultsAnalyzerPanel );
         
        m_centerCollectionPanel.add(Box.createHorizontalGlue( ) );
        m_centerCollectionPanel.add( m_centerWestCollectionPanel );
        m_centerCollectionPanel.add(Box.createHorizontalGlue( ) );
        m_centerCollectionPanel.add( m_centerEastCollectionPanel );
        m_centerCollectionPanel.add(Box.createHorizontalGlue( ) );
        
        m_collectionPanel.add( m_northCollectionPanel, BorderLayout.NORTH );       
        m_collectionPanel.add( m_centerCollectionPanel, BorderLayout.CENTER );
        
        
        createCancelApplyPanel( );
        m_collectionPanel.add( m_selectionPanel, BorderLayout.SOUTH );
       
        getContentPane( ).setLayout( new BorderLayout( ) );

        JScrollPane scroller = new JScrollPane( m_collectionPanel );
        this.getContentPane( ).add( scroller, BorderLayout.CENTER );

        setVisible( true );
        
        m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw.setZoom( m_appliedZoom );
        m_segmentationResultsImageDicomSliceCanvasPolydraw.setZoom( m_appliedZoom );


        m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw.setMinimumMaximumFocusImageNumberRange( m_minimumImageNumberToFocus, m_maximumImageNumberToFocus );
        m_segmentationResultsImageDicomSliceCanvasPolydraw.setMinimumMaximumFocusImageNumberRange( m_minimumImageNumberToFocus, m_maximumImageNumberToFocus );

        m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw.addActionListener( m_segmentationResultsImageDicomSliceCanvasPolydraw );
        m_segmentationResultsImageDicomSliceCanvasPolydraw.addActionListener( m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw );
        
        pack( );
        
        // Put the Dialog nicely in the middle of its parent
        Point centerParent = new Point( m_parent.getX( ) + ( m_parent.getWidth( ) / 2 ) - this.getWidth( ) / 2,
                                        m_parent.getY( ) + ( m_parent.getHeight( ) / 2 ) - this.getHeight( ) / 2 );
        
        
        setLocation( centerParent );
        
        m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw.displayDicomImage( numberOfImages / 2 );

    }

    
  //  below new implementation! continue make multislice segementation work!
    
     /** 
     * Creates a new instance of AdvancedInteractiveSegmentationAnalyzerFrame.
     * @param parent
     * @param imageSeriesData
     * @param minimumImageNumberToFocus
     * @param maximumImageNumberToFocus
     */
    public AdvancedInteractiveSegmentationAnalyzerFrame( JFrame parent, 
                                                         DicomImageSeriesData imageSeriesData,
                                                         int minimumImageNumberToFocus, 
                                                         int maximumImageNumberToFocus )
    {
        m_parent  = parent;

        m_minimumImageNumberToFocus = minimumImageNumberToFocus;
        m_maximumImageNumberToFocus = maximumImageNumberToFocus;

        m_collectionPanel = new JPanel( );
        m_collectionPanel.setLayout( new BorderLayout( ) );
        
        setTitle( "Advanced interactive segmentation" );
        
        m_northCollectionPanel = new JPanel( );
        m_northCollectionPanel.setLayout( new BoxLayout( m_northCollectionPanel, BoxLayout.X_AXIS ) );
        //////////////////////////////////////////////////////////////////////////////////////////////////////
        
        //////// NORTH WEST PANEL
        
        m_northWestCollectionPanel = new JPanel( );
        m_northWestCollectionPanel.setLayout( new BoxLayout( m_northWestCollectionPanel, BoxLayout.Y_AXIS ) );
        m_northWestCollectionPanel.setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ) );
        
        
        createBasicTissueTypesDefinitionDicomImagePanel( imageSeriesData );
        m_northWestCollectionPanel.add( m_dicomInputImagePanel );
        
        createROIModePanelLeft( );
        m_northWestCollectionPanel.add( m_canvasROIModePanelLeft );
        
        createBasicTissueTypesDefinitionROIPropertiesPanel( );
        m_northWestCollectionPanel.add( m_ROIPropertiesScrollPane );
        
        //////////////////////////////////////////////////////////////////////////////////////////////////////
        
        //////// NORTH EAST PANEL
        
        m_northEastCollectionPanel = new JPanel( );
        m_northEastCollectionPanel.setLayout( new BoxLayout( m_northEastCollectionPanel, BoxLayout.Y_AXIS ) );
        m_northEastCollectionPanel.setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ) );
        
        createSegmentationResultsDicomImagePanel( imageSeriesData );
        m_northEastCollectionPanel.add( m_dicomSegmentationResultsImagePanel );
        
        createROIModePanelRight( );
        m_northEastCollectionPanel.add( m_canvasROIModePanelRight );
        
        createSegmentationResultsROIPropertiesPanel( );
        m_northEastCollectionPanel.add( m_segmentationResultsROIPropertiesScrollPane, BorderLayout.CENTER );
        
        
        // set the north panels
        m_northCollectionPanel.add( m_northWestCollectionPanel );
        m_northCollectionPanel.add( m_northEastCollectionPanel );
        
        //////////////////////////////////////////////////////////////////////////////////////////////////////
        //////// CENTER WEST PANEL
        
        
        m_centerCollectionPanel = new JPanel( );
        m_centerCollectionPanel.setLayout( new BoxLayout( m_centerCollectionPanel, BoxLayout.X_AXIS ) );
        
        // center west-side
        m_centerWestCollectionPanel = new JPanel( );
        m_centerWestCollectionPanel.setLayout( new BoxLayout( m_centerWestCollectionPanel, BoxLayout.Y_AXIS ) );
        m_centerWestCollectionPanel.setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ) );
        
        
        createProbabilitySegmentationProbingRegionPropertiesPanel( );
        m_centerWestCollectionPanel.add( m_probingRegionPropertiesPanel );
        
               
        //////////////////////////////////////////////////////////////////////////////////////////////////////
        //////// CENTER EAST PANEL
        //////////////////////////////////////////////////////////////////////////////////////////////////////

        m_centerEastCollectionPanel = new JPanel( );
        m_centerEastCollectionPanel.setLayout( new BoxLayout( m_centerEastCollectionPanel, BoxLayout.Y_AXIS ) );
        m_centerEastCollectionPanel.setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ) );
        
        createSegmentationResultsAnalyzerPanel( );  
        m_centerEastCollectionPanel.add( m_segmentationResultsAnalyzerPanel );
         
        m_centerCollectionPanel.add(Box.createHorizontalGlue( ) );
        m_centerCollectionPanel.add( m_centerWestCollectionPanel );
        m_centerCollectionPanel.add(Box.createHorizontalGlue( ) );
        m_centerCollectionPanel.add( m_centerEastCollectionPanel );
        m_centerCollectionPanel.add(Box.createHorizontalGlue( ) );
        
        m_collectionPanel.add( m_northCollectionPanel, BorderLayout.NORTH );       
        m_collectionPanel.add( m_centerCollectionPanel, BorderLayout.CENTER );
        
        
        createCancelApplyPanel( );
        m_collectionPanel.add( m_selectionPanel, BorderLayout.SOUTH );
        
        getContentPane( ).setLayout( new BorderLayout( ) );

        JScrollPane scroller = new JScrollPane( m_collectionPanel );
        this.getContentPane( ).add( scroller, BorderLayout.CENTER );
        
        setVisible( true );
        
        m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw.setZoom( m_appliedZoom );
        m_segmentationResultsImageDicomSliceCanvasPolydraw.setZoom( m_appliedZoom );


        m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw.setMinimumMaximumFocusImageNumberRange( m_minimumImageNumberToFocus, m_maximumImageNumberToFocus );
        m_segmentationResultsImageDicomSliceCanvasPolydraw.setMinimumMaximumFocusImageNumberRange( m_minimumImageNumberToFocus, m_maximumImageNumberToFocus );

        m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw.addActionListener( m_segmentationResultsImageDicomSliceCanvasPolydraw );
        m_segmentationResultsImageDicomSliceCanvasPolydraw.addActionListener( m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw );
        
        
        pack( );
        
        // Put the Dialog nicely in the middle of its parent
        Point centerParent = new Point( m_parent.getX( ) + ( m_parent.getWidth( ) / 2 ) - this.getWidth( ) / 2,
                                        m_parent.getY( ) + ( m_parent.getHeight( ) / 2 ) - this.getHeight( ) / 2 );
        
        
        setLocation( centerParent );
        
        m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw.displayDicomImage( imageSeriesData.getNumberOfImages( ) / 2 );
        m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw.drawImage( false );
        
        m_segmentationResultsImageDicomSliceCanvasPolydraw.displayDicomImage( imageSeriesData.getNumberOfImages( ) / 2 );
        m_segmentationResultsImageDicomSliceCanvasPolydraw.drawImage( false );

    }

    /**
     * Sets the rawdata
     * @param rawData
     */
    public void setImageSeriesPixelData( int[] processedData )
    {
        m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw.setImageSeriesPixelData( processedData );
        m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw.displayDicomImage( m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw.getCurrentlyDisplayedImageNumber( ) );
        m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw.update( true );

        m_segmentationResultsImageDicomSliceCanvasPolydraw.setImageSeriesPixelData( processedData );
        m_segmentationResultsImageDicomSliceCanvasPolydraw.displayDicomImage( m_segmentationResultsImageDicomSliceCanvasPolydraw.getCurrentlyDisplayedImageNumber( ) );
        m_segmentationResultsImageDicomSliceCanvasPolydraw.update( true );

    }

    /**
     * Creates the DicomImagePanel
     */
    private void createBasicTissueTypesDefinitionDicomImagePanel( int[] data, FileInfo finfo, LinkedHashMap siemensSpecificHeader, int numberOfImages )
    {
        m_dicomInputImagePanel = new JPanel( );
        
        m_dicomInputImagePanel.setLayout( new BoxLayout( m_dicomInputImagePanel, BoxLayout.X_AXIS ) );
        m_dicomInputImagePanel.add( Box.createHorizontalGlue( ) );
        
        m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw = new DicomSliceCanvasPolyDraw( this );
        
        if ( finfo.getFileName( ) != null )
        {
            m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw.setTitle( finfo.getFileName( ) );
        }
        else
        {
            setTitle( finfo.getPatientName( ) + "   " + finfo.getPatientBirthDate( ) );
        }
        
        m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw.setImage( data , finfo.getImageWidth( ), finfo.getImageHeight( ), 0, finfo, siemensSpecificHeader );
        m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw.setNumberOfImages( numberOfImages );

        if ( finfo.getImageWidth( ) != 0 )
        {
            m_appliedZoom = 512f / finfo.getImageWidth( );
        }
        
        m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw.enableImagePaging( true );
        
        m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw.displayDicomImage( 0 );
        
        m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw.setPreferredSize( new Dimension( 512, 512 ) );
        m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw.setMaximumSize( new Dimension( 512, 512 ) );
        m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw.setMinimumSize( new Dimension( 512, 512 ) );
        
        m_maxPixelValueOfLoadedImage = SignalTools.getMaximum( data );
        m_minPixelValueOfLoadedImage = SignalTools.getMinimum( data );
       
        m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw.setVisible( true );
        
        m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw.addActionListener( this );
        m_dicomInputImagePanel.add( m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw );
        m_dicomInputImagePanel.add( Box.createHorizontalGlue( ) );
        
        
    }
    
    
    /**
     * Creates the DicomImagePanel based on a image series stack.
     */
    private void createBasicTissueTypesDefinitionDicomImagePanel( DicomImageSeriesData imageSeriesData )
    {
        m_dicomInputImagePanel = new JPanel( );
        
        m_dicomInputImagePanel.setLayout( new BoxLayout( m_dicomInputImagePanel, BoxLayout.X_AXIS ) );
        m_dicomInputImagePanel.add( Box.createHorizontalGlue( ) );
        
        m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw = new DicomSliceCanvasPolyDraw( this );
        
        FileInfo finfo = imageSeriesData.getFileInfo( 0 );
        
        if ( finfo.getFileName( ) != null )
        {
            m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw.setTitle( finfo.getFileName( ) );
        }
        else
        {
            setTitle( finfo.getPatientName( ) + "   " + finfo.getPatientBirthDate( ) );
        }
        
        m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw.setDicomImages( imageSeriesData );

        if ( finfo.getImageWidth( ) != 0 )
        {
            m_appliedZoom = 512f / finfo.getImageWidth( );
        }
        
        m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw.enableImagePaging( true );
        
        m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw.displayDicomImage( 0 );
        
        m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw.setPreferredSize( new Dimension( 512, 512 ) );
        m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw.setMaximumSize( new Dimension( 512, 512 ) );
        m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw.setMinimumSize( new Dimension( 512, 512 ) );
        
        m_maxPixelValueOfLoadedImage = m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw.getMaxRawDataValue( );
        m_minPixelValueOfLoadedImage = m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw.getMinRawDataValue( );
       
        m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw.setVisible( true );
        
        m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw.addActionListener( this );
        m_dicomInputImagePanel.add( m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw );
        m_dicomInputImagePanel.add( Box.createHorizontalGlue( ) );
    }
    
    /**
     * Creates the panel on which the properties of the regions are drawn.
     */
    private void createBasicTissueTypesDefinitionROIPropertiesPanel( )
    {
        m_basicTypesROIPropertiesTable = new RegionPropertiesTable( this, m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw, false );
        m_basicTypesROIPropertiesTable.addActionListener( this );
        
        // JList does not provide a scroll list automatically so we have to 
        // add it ourselves.
        m_ROIPropertiesScrollPane = new JScrollPane( JScrollPane.VERTICAL_SCROLLBAR_ALWAYS , JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED ); 
        m_ROIPropertiesScrollPane.getViewport( ).setView( m_basicTypesROIPropertiesTable );        
        m_ROIPropertiesScrollPane.setPreferredSize( new Dimension( 512, 128 ) );
        m_ROIPropertiesScrollPane.setMaximumSize( new Dimension( 512, 128 ) );
        m_ROIPropertiesScrollPane.setMinimumSize( new Dimension( 512, 128 ) );
        
        m_basicTypesROIPropertiesTable.addActionListener( m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw );
        
    }
    
    /**
     * Creates the panel on which the properties of the regions are drawn
     */
    private void createSegmentationResultsROIPropertiesPanel( )
    {
        m_segmentationResultsROIPropertiesTable = new ImageSegmentPropertiesTable( this,  m_segmentationResultsImageDicomSliceCanvasPolydraw );
        m_segmentationResultsROIPropertiesTable.addActionListener( this );
        
        // JList does not provide a scroll list automatically so we have to 
        // add it ourselves.
        m_segmentationResultsROIPropertiesScrollPane = new JScrollPane( JScrollPane.VERTICAL_SCROLLBAR_ALWAYS , JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED ); 
        m_segmentationResultsROIPropertiesScrollPane.getViewport( ).setView( m_segmentationResultsROIPropertiesTable );        
        m_segmentationResultsROIPropertiesScrollPane.setPreferredSize( new Dimension( 512, 128 ) );
        m_segmentationResultsROIPropertiesScrollPane.setMaximumSize( new Dimension( 512, 128 ) );
        m_segmentationResultsROIPropertiesScrollPane.setMinimumSize( new Dimension( 512, 128 ) );
    }
    
    
    /**
     * Creates the DicomImagePanel on which the resulting regions are displayed
     */
    private void createSegmentationResultsDicomImagePanel( int[] data, FileInfo finfo, LinkedHashMap siemensSpecificHeader, int numberOfImages )
    {
        m_dicomSegmentationResultsImagePanel = new JPanel( );
        
        m_dicomSegmentationResultsImagePanel.setLayout( new BoxLayout( m_dicomSegmentationResultsImagePanel, BoxLayout.X_AXIS ) );
        m_dicomSegmentationResultsImagePanel.add( Box.createHorizontalGlue( ) );
        
        m_segmentationResultsImageDicomSliceCanvasPolydraw = new DicomSliceCanvasPolyDraw( this );
         
        if ( finfo.getFileName( ) != null )
        {
            m_segmentationResultsImageDicomSliceCanvasPolydraw.setTitle( finfo.getFileName( ) );
        }
        else
        {
            setTitle( finfo.getPatientName( ) + "   " + finfo.getPatientBirthDate( ) );
        }
        
        m_segmentationResultsImageDicomSliceCanvasPolydraw.setImage( data , finfo.getImageWidth( ), finfo.getImageHeight( ), 0, finfo, siemensSpecificHeader );
        m_segmentationResultsImageDicomSliceCanvasPolydraw.setNumberOfImages( numberOfImages );

        if ( finfo.getImageWidth( ) != 0 )
        {
            m_appliedZoom = 512f / finfo.getImageWidth( );
        }
        
        m_segmentationResultsImageDicomSliceCanvasPolydraw.enableImagePaging( true );
        
        m_segmentationResultsImageDicomSliceCanvasPolydraw.displayDicomImage( 0 );
        
        m_segmentationResultsImageDicomSliceCanvasPolydraw.setPreferredSize( new Dimension( 512, 512 ) );
        m_segmentationResultsImageDicomSliceCanvasPolydraw.setMaximumSize( new Dimension( 512, 512 ) );
        m_segmentationResultsImageDicomSliceCanvasPolydraw.setMinimumSize( new Dimension( 512, 512 ) );
        
        
        m_segmentationResultsImageDicomSliceCanvasPolydraw.setVisible( true );
        
        m_segmentationResultsImageDicomSliceCanvasPolydraw.addActionListener( this );
        
        m_dicomSegmentationResultsImagePanel.add( m_segmentationResultsImageDicomSliceCanvasPolydraw );
        m_dicomSegmentationResultsImagePanel.add( Box.createHorizontalGlue( ) );
        
        // this view allows image segment manipulation!
        m_segmentationResultsImageDicomSliceCanvasPolydraw.enableImageSegmentDisplayMode( );
    }
    
    
    /**
     * Creates the DicomImagePanel on which the resulting regions are displayed
     */
    private void createSegmentationResultsDicomImagePanel( DicomImageSeriesData imageSeriesData )
    {
        m_dicomSegmentationResultsImagePanel = new JPanel( );
        
        m_dicomSegmentationResultsImagePanel.setLayout( new BoxLayout( m_dicomSegmentationResultsImagePanel, BoxLayout.X_AXIS ) );
        m_dicomSegmentationResultsImagePanel.add( Box.createHorizontalGlue( ) );
        
        m_segmentationResultsImageDicomSliceCanvasPolydraw = new DicomSliceCanvasPolyDraw( this );
        
        FileInfo finfo = imageSeriesData.getFileInfo( 0 );
        
        
        if ( finfo.getFileName( ) != null )
        {
            m_segmentationResultsImageDicomSliceCanvasPolydraw.setTitle( finfo.getFileName( ) );
        }
        else
        {
            setTitle( finfo.getPatientName( ) + "   " + finfo.getPatientBirthDate( ) );
        }
        
        m_segmentationResultsImageDicomSliceCanvasPolydraw.setDicomImages( imageSeriesData );
        
        if ( finfo.getImageWidth( ) != 0 )
        {
            m_appliedZoom = 512f / finfo.getImageWidth( );
        }
        
        m_segmentationResultsImageDicomSliceCanvasPolydraw.enableImagePaging( true );
        
        m_segmentationResultsImageDicomSliceCanvasPolydraw.displayDicomImage( 0 );
        
        m_segmentationResultsImageDicomSliceCanvasPolydraw.setPreferredSize( new Dimension( 512, 512 ) );
        m_segmentationResultsImageDicomSliceCanvasPolydraw.setMaximumSize( new Dimension( 512, 512 ) );
        m_segmentationResultsImageDicomSliceCanvasPolydraw.setMinimumSize( new Dimension( 512, 512 ) );
        
        m_segmentationResultsImageDicomSliceCanvasPolydraw.setVisible( true );
        
        m_segmentationResultsImageDicomSliceCanvasPolydraw.addActionListener( this );
        
        m_dicomSegmentationResultsImagePanel.add( m_segmentationResultsImageDicomSliceCanvasPolydraw );
        m_dicomSegmentationResultsImagePanel.add( Box.createHorizontalGlue( ) );
        
        // this view allows image segment manipulation!
        m_segmentationResultsImageDicomSliceCanvasPolydraw.enableImageSegmentDisplayMode( );
    }
    
    
    
    
    /**
     * Instantiates an apply/ cancel panel
     */
    private void createCancelApplyPanel( )
    {
        // Selection panel definitions
        m_applyButton = new JButton( "Apply" );
        m_applyButton.addActionListener( this );
        m_applyButton.setToolTipText( "Returns a list of ROI objects to calling object" );
        m_cancelButton = new JButton( "Cancel" );
        m_cancelButton.addActionListener( this );
        m_cancelButton.setToolTipText( "Cancels without returning a list of ROI objects to calling object" );
        
        m_selectionPanel = new JPanel( );
        
        m_selectionPanel.setLayout( new BoxLayout( m_selectionPanel, BoxLayout.X_AXIS ) );
        m_selectionPanel.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );
        m_selectionPanel.add( Box.createGlue( ) );
        m_selectionPanel.add( m_applyButton );
        m_selectionPanel.add( Box.createGlue( ) );
        m_selectionPanel.add( m_cancelButton );
        m_selectionPanel.add( Box.createGlue( ) );
    }
    
    
    /**
     * Updates the list with basic tissue types
     */
    private void upDateBasicTissueTypesList( )
    {
        m_basicTypesROIPropertiesTable.setPolygonInformation( );
    }
    
    /**
     * Updates the list of segment tissue types
     */
    private void upDateImageSegmentTypesList( boolean computed )
    {
        LinkedList <ImageParametersWithinARegioPolygon> tissueTypeList = m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw.getAllTissueTypeParametersAsLinkedList( );
       
        m_segmentationResultsAnalyzerPanel.setImageSegmentTypes( tissueTypeList, computed );
       
        m_segmentationColors = m_segmentationResultsAnalyzerPanel.getColors( );
    }
    
    
    /**
     * Is called whenever a registered component raises an ActionEvent.
     * @param e
     */
    @Override
    public void actionPerformed( ActionEvent e ) 
    {
        
        Object sender = e.getSource( );
       
        if ( sender == m_segmentationResultsAnalyzerPanel )
        {
            if ( e.getID( ) == SegmentationResultsAnalyzerPanel.TABLE_SELECTION_CHANGE_EVENT )
            {
                // force the displayers to update as well
                m_segmentationResultsImageDicomSliceCanvasPolydraw.computeStatisticalPropertiesOfImageSegmentsCurrentlyDisplayedImage( m_segmentationResultsImageDicomSliceCanvasPolydraw.getCurrentlyDisplayedImageNumber( ) );
                
                m_segmentationResultsROIPropertiesTable.upDateTableData( m_segmentationResultsImageDicomSliceCanvasPolydraw.getImageSegmentPropertiesofCurrentlyDisplayedImage( ) );
            }
        }
        
        
        if ( e.getSource( ) == m_probingRegionPropertiesPanel )
        {
            // Check whether the user wants to define new  
            if ( e.getID( ) == ProbingRegionPropertiesPanel.GRAPHIC_SEGMENTATION_REGION_DEFINITION_REQUIRED )
            {
                m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw.setMode( DicomSliceCanvasPolyDraw.ADD_ROI_MODE_SELECTED_EVENT );
                Dialogs.messageBox( this, "Draw ROI in the left image control" );
                m_userIsDefiningROIToSegment = true;
            }
            
            if ( e.getID( ) == ProbingRegionPropertiesPanel.REMOVE_GRAPHIC_SEGMENTATION_REGION_REQUIRED )
            {
                m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw.removeRegionToSegment( );
                m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw.repaint( );
                
                m_segmentationResultsImageDicomSliceCanvasPolydraw.removeRegionToSegment( );
                m_segmentationResultsImageDicomSliceCanvasPolydraw.repaint( );
            }
            
            if ( e.getID( ) == ProbingRegionPropertiesPanel.SUBSEGMENT_IN_CURRENTLY_SELECTED_SLICE_ONLY )
            {
                m_segmentationResultsImageDicomSliceCanvasPolydraw.setSubSegmentationScope( ProbingRegionPropertiesPanel.SUBSEGMENT_IN_CURRENTLY_SELECTED_SLICE_ONLY  );
            }
            
            if ( e.getID( ) == ProbingRegionPropertiesPanel.SUBSEGMENT_IN_ALL_SLICES )
            {
                m_segmentationResultsImageDicomSliceCanvasPolydraw.setSubSegmentationScope( ProbingRegionPropertiesPanel.SUBSEGMENT_IN_ALL_SLICES  );
            }
            
            
            if ( e.getID( ) == ProbingRegionPropertiesPanel.START_SEGMENTATION )
            {
                
                // set some defaults
                float lowThreshold = m_probingRegionPropertiesPanel.getValueOfMinimumThresholdSlider( );
                float highThreshold = m_probingRegionPropertiesPanel.getValueOfMaximumThresholdSlider( );
                
                boolean performThresholding = m_probingRegionPropertiesPanel.performThresholdingBeforeSegmentation( );
                boolean performAdvancedSegmentation = m_probingRegionPropertiesPanel.performAdvancedSegmentation( );
                int probeSizeX = m_probingRegionPropertiesPanel.getProbeSizeX( );
                int probeSizeY = m_probingRegionPropertiesPanel.getProbeSizeY( );
                int permutations = m_probingRegionPropertiesPanel.getNumberOfPermutations( );
                int numberOfProbingPixels = m_probingRegionPropertiesPanel.getNumberOfProbingPixels( );
                int subSegmentationApplicationScope = m_probingRegionPropertiesPanel.getSubSegmentScope( );
                
                int pixelRangeComputationType = m_probingRegionPropertiesPanel.getPixelRangeComputationType( );
                float pixelRangeMultiplier = m_probingRegionPropertiesPanel.getRangeMultiplier( );
                
                // set the subsegmentation scope parameter of the DicomSliceCanvasPolyDraw object
                // that allows the display and manipulation of image segments
                m_segmentationResultsImageDicomSliceCanvasPolydraw.setSubSegmentationScope( subSegmentationApplicationScope );
                
                int minROISize = 25;
                int maxROISize = m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw.getImageHeight( ) * m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw.getImageWidth( ) / 2;
                
                m_probabilityBasedSegmentationCalculator = new ProbabilityBasedSegmentationCalculator( m_parent, m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw, 
                                                                                                       lowThreshold, highThreshold, 
                                                                                                       minROISize, maxROISize,
                                                                                                       performThresholding, performAdvancedSegmentation, 
                                                                                                       m_segmentationColors, 
                                                                                                       probeSizeX, probeSizeY, 
                                                                                                       permutations, numberOfProbingPixels,
                                                                                                       pixelRangeComputationType, pixelRangeMultiplier,
                                                                                                       m_minimumImageNumberToFocus, m_maximumImageNumberToFocus,
                                                                                                       m_nordicScalingFactor );
                
                // progress bar listens to events from the calculator
                m_probabilityBasedSegmentationCalculator.addActionListener( m_probingRegionPropertiesPanel );
                
                // >>this<< object also listens to changes in the calculator
                m_probabilityBasedSegmentationCalculator.addActionListener( this );
                
                // Also the part of the GUI the handles the viewing of the results must be notified that the
                // calculator is ready.
                m_probabilityBasedSegmentationCalculator.addActionListener( m_segmentationResultsAnalyzerPanel );
                
                m_probabilityBasedSegmentationCalculator.performSegmentation( );
                
            }
            
        }
        
        if ( e.getSource( ) == m_probabilityBasedSegmentationCalculator )
        {
            if ( e.getID( ) == ProbabilityBasedSegmentationCalculator.PROBABILISTIC_SEGMENTATION_PERFORMED_EVENT )
            {
                upDateImageSegmentTypesList( true );
            }
        }
        
        if ( e.getSource( ) == m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw )
        {
            if ( ( e.getID( ) == DicomSliceCanvasPolyDraw.DISPLAY_NEXT_IMAGE_OF_SLICE_STACK_EVENT ) ||
                 ( e.getID( ) == DicomSliceCanvasPolyDraw.DISPLAY_PREVIOUS_IMAGE_OF_SLICE_STACK_EVENT ) )  
            {
                 m_segmentationResultsImageDicomSliceCanvasPolydraw.computeStatisticalPropertiesOfImageSegmentsCurrentlyDisplayedImage( m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw.getCurrentlyDisplayedImageNumber( ) );
                
                 m_segmentationResultsROIPropertiesTable.upDateTableData( m_segmentationResultsImageDicomSliceCanvasPolydraw.getImageSegmentPropertiesofCurrentlyDisplayedImage( ) );
            }
            
            
            if ( e.getID( ) == DicomSliceCanvasPolyDraw.POLYGON_CLOSED_EVENT )
            {
                if ( m_userIsDefiningROIToSegment == false )
                {
                    upDateBasicTissueTypesList( );
                    upDateImageSegmentTypesList( false );
                }
                else
                {
                    RegioPolygon regPol = m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw.getSelectedPolygon( );
                    RegioPolygon convPol = m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw.convertPolygonToRectangle( regPol );
                    convPol.setLabel( "Area to segment" );
                    
                    m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw.setRegionToSegment( convPol );
                    m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw.deletePoly( regPol );
                    
                    RegioPolygon convPol2 = new RegioPolygon( convPol  );
                    m_segmentationResultsImageDicomSliceCanvasPolydraw.setRegionToSegment( convPol2 );
                    m_segmentationResultsImageDicomSliceCanvasPolydraw.repaint( );
                    
                    m_userIsDefiningROIToSegment = false;
                }
            }
            // handle removal of a polygon as well..
            if ( e.getID( ) == DicomSliceCanvasPolyDraw.POLYGON_REMOVED_EVENT )
            {
                upDateBasicTissueTypesList( );
                upDateImageSegmentTypesList( false );
            }
            
            if ( e.getID( ) == DicomSliceCanvasPolyDraw.POLYGON_CHANGED_EVENT )
            {
                upDateBasicTissueTypesList( );
                upDateImageSegmentTypesList( false );
            }
        }
        
        // handle responses on cancel button pressed event
        if ( sender == m_cancelButton )
        {
            Iterator it = m_listeners.iterator( );
             while( it.hasNext( ) )
             {
                ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, CANCEL_EVENT, "" ) );
             }
            dispose( );
        }
        
        // handle responses on apply button pressed event
        if ( sender == m_applyButton )
        {
            if ( m_applyButtonPressed == false )
            {
                Iterator it = m_listeners.iterator( );
                while( it.hasNext( ) )
                {
                    ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, ACCEPT_EVENT, "" ) );
                }
            }
            m_applyButtonPressed = true;
            dispose( );    
        }
        
        if ( sender == m_segmentationResultsROIPropertiesTable )
        {
            if ( e.getID( ) == ImageSegmentPropertiesTable.CREATE_REPORT_FILE_EVENT )
            {
                m_segmentationResultsImageDicomSliceCanvasPolydraw.createSegmentPropertiesOfAllImagesReportFile( );
            }
        }
        
        
        // handles action events raised by the basic types slice 
        // canvas polydraw object
        if ( sender == m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw )
        {
            
            if ( e.getID( ) == DicomSliceCanvasPolyDraw.ADD_ROI_MODE_SELECTED_EVENT )
            {
                m_addROIButtonLeft.setSelected( true );
            }
            
            if ( e.getID( ) == DicomSliceCanvasPolyDraw.DELETE_ROI_MODE_SELECTED_EVENT )
            {
                m_deleteROIButtonLeft.setSelected( true );
            }
             
            if ( e.getID( ) == DicomSliceCanvasPolyDraw.CUT_ROI_MODE_SELECTED_EVENT )
            {
                m_cutROIButtonLeft.setSelected( true );
            }
            if ( e.getID( ) == DicomSliceCanvasPolyDraw.RESIZE_ROI_MODE_SELECTED_EVENT )
            {
                m_resizeROIButtonLeft.setSelected( true );
            }
            
            if ( e.getID( ) == DicomSliceCanvasPolyDraw.MOVE_ROI_MODE_SELECTED_EVENT )
            {
                m_moveROIButtonLeft.setSelected( true );
            }
            
            if ( e.getID( ) == DicomSliceCanvasPolyDraw.SELECT_ROI_MODE_SELECTED_EVENT )
            {
                m_selectROIButtonLeft.setSelected( true );
            }            
        }
        
        
        if ( sender == m_segmentationResultsImageDicomSliceCanvasPolydraw )
        {    
            if ( ( e.getID( ) == DicomSliceCanvasPolyDraw.DISPLAY_NEXT_IMAGE_OF_SLICE_STACK_EVENT ) ||
                 ( e.getID( ) == DicomSliceCanvasPolyDraw.DISPLAY_PREVIOUS_IMAGE_OF_SLICE_STACK_EVENT ) ) 
            {
                m_segmentationResultsImageDicomSliceCanvasPolydraw.computeStatisticalPropertiesOfImageSegmentsCurrentlyDisplayedImage( m_segmentationResultsImageDicomSliceCanvasPolydraw.getCurrentlyDisplayedImageNumber( ) );
                m_segmentationResultsROIPropertiesTable.upDateTableData( m_segmentationResultsImageDicomSliceCanvasPolydraw.getImageSegmentPropertiesofCurrentlyDisplayedImage( ) );
            }
        }
        
        if ( sender == m_addROIButtonLeft )
        {
            m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw.setMode( DicomSliceCanvasPolyDraw.POLYGON_DRAW_MODE );
        }
       
        if ( sender == m_deleteROIButtonLeft )
        {
            m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw.setMode( DicomSliceCanvasPolyDraw.DELETE_MODE );
        }
        
        if ( sender == m_cutROIButtonLeft )
        {
            m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw.setMode( DicomSliceCanvasPolyDraw.CUT_MODE );
        }
       
        if ( sender == m_resizeROIButtonLeft )
        {
            m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw.setMode( DicomSliceCanvasPolyDraw.RESIZE_MODE );
        }
       
        if ( sender == m_moveROIButtonLeft )
        {
            m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw.setMode( DicomSliceCanvasPolyDraw.DRAG_MODE  );
        }
        
        if ( sender == m_selectROIButtonLeft )
        {
            m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw.setMode( DicomSliceCanvasPolyDraw.SELECT_MODE );
        }
        
        // handles action events raised by the basic types slice 
        // canvas polydraw object
        if ( sender == m_segmentationResultsImageDicomSliceCanvasPolydraw )
        {
            if ( e.getID( ) == DicomSliceCanvasPolyDraw.ADD_ROI_MODE_SELECTED_EVENT )
            {
                m_addROIButtonRight.setSelected( true );
            }
            
            if ( e.getID( ) == DicomSliceCanvasPolyDraw.DELETE_ROI_MODE_SELECTED_EVENT )
            {
                m_deleteROIButtonRight.setSelected( true );
            }
             
            if ( e.getID( ) == DicomSliceCanvasPolyDraw.CUT_ROI_MODE_SELECTED_EVENT )
            {
                m_cutROIButtonRight.setSelected( true );
            }
            if ( e.getID( ) == DicomSliceCanvasPolyDraw.RESIZE_ROI_MODE_SELECTED_EVENT )
            {
                m_resizeROIButtonRight.setSelected( true );
            }
            
            if ( e.getID( ) == DicomSliceCanvasPolyDraw.MOVE_ROI_MODE_SELECTED_EVENT )
            {
                m_moveROIButtonRight.setSelected( true );
            }
            
            if ( e.getID( ) == DicomSliceCanvasPolyDraw.SELECT_ROI_MODE_SELECTED_EVENT )
            {
                m_selectROIButtonRight.setSelected( true );
            }            
        }
        
        if ( sender == m_addROIButtonRight )
        {
            m_segmentationResultsImageDicomSliceCanvasPolydraw.setMode( DicomSliceCanvasPolyDraw.POLYGON_DRAW_MODE );
        }
       
        if ( sender == m_deleteROIButtonRight )
        {
            m_segmentationResultsImageDicomSliceCanvasPolydraw.setMode( DicomSliceCanvasPolyDraw.DELETE_MODE );
        }
        
        if ( sender == m_cutROIButtonRight )
        {
            m_segmentationResultsImageDicomSliceCanvasPolydraw.setMode( DicomSliceCanvasPolyDraw.CUT_MODE );
        }
       
        if ( sender == m_resizeROIButtonRight )
        {
            m_segmentationResultsImageDicomSliceCanvasPolydraw.setMode( DicomSliceCanvasPolyDraw.RESIZE_MODE );
        }
       
        if ( sender == m_moveROIButtonRight )
        {
            m_segmentationResultsImageDicomSliceCanvasPolydraw.setMode( DicomSliceCanvasPolyDraw.DRAG_MODE  );
        }
        
        if ( sender == m_selectROIButtonRight )
        {
            m_segmentationResultsImageDicomSliceCanvasPolydraw.setMode( DicomSliceCanvasPolyDraw.SELECT_MODE );
        }
    }

    /**
     * Creates a panel allowing the user to set the ROI state of the 
     * DicomSliceCanvasPolyDraw control
     */
    private void createROIModePanelLeft( )
    {
        m_canvasROIModePanelLeft = new JPanel( );
        m_canvasROIModePanelLeft.setLayout( new BoxLayout( m_canvasROIModePanelLeft, BoxLayout.X_AXIS ) );
        
        m_canvasROIModePanelLeft.add(Box.createHorizontalGlue( ) );
        
        m_canvasROIModePanelLeft.setPreferredSize( new Dimension( 512, 30 ) );
        m_canvasROIModePanelLeft.setMaximumSize( new Dimension( 512, 30 ) );
        m_canvasROIModePanelLeft.setMinimumSize( new Dimension( 512, 30 ) );
        
        m_addROIButtonLeft = new JToggleButton( "Add ROI" );
        m_addROIButtonLeft.addActionListener( this );
        m_deleteROIButtonLeft = new JToggleButton( "Delete ROI" );
        m_deleteROIButtonLeft.addActionListener( this );
        m_cutROIButtonLeft = new JToggleButton( "Cut" ); 
        m_cutROIButtonLeft.addActionListener( this );
        m_resizeROIButtonLeft = new JToggleButton( "Resize" );
        m_resizeROIButtonLeft.addActionListener( this );
        m_moveROIButtonLeft = new JToggleButton( "Move" );
        m_moveROIButtonLeft.addActionListener( this );
        m_selectROIButtonLeft = new JToggleButton( "Select ROI" );
        m_selectROIButtonLeft.addActionListener( this );
        
        m_canvasROIModePanelLeft.add( Box.createHorizontalGlue( ) );
        m_canvasROIModePanelLeft.add( m_addROIButtonLeft );
        m_canvasROIModePanelLeft.add( m_deleteROIButtonLeft );
        m_canvasROIModePanelLeft.add( m_cutROIButtonLeft );
        m_canvasROIModePanelLeft.add( m_resizeROIButtonLeft );
        m_canvasROIModePanelLeft.add( m_moveROIButtonLeft );
        m_canvasROIModePanelLeft.add( m_selectROIButtonLeft );
        
        m_canvasROIModePanelLeft.add( Box.createHorizontalGlue( ) );
        
        m_buttonGroupROIModesLeft = new ButtonGroup( );

        m_buttonGroupROIModesLeft.add( m_addROIButtonLeft );
        m_buttonGroupROIModesLeft.add( m_deleteROIButtonLeft );
        m_buttonGroupROIModesLeft.add( m_cutROIButtonLeft );
        m_buttonGroupROIModesLeft.add( m_resizeROIButtonLeft );
        m_buttonGroupROIModesLeft.add( m_moveROIButtonLeft );
        m_buttonGroupROIModesLeft.add( m_selectROIButtonLeft );
        
        m_selectROIButtonLeft.setSelected( true );
        
    }
    
    /**
     * Creates a panel allowing the user to set the ROI state of the 
     * DicomSliceCanvasPolyDraw control
     */
    private void createROIModePanelRight( )
    {
        m_canvasROIModePanelRight = new JPanel( );
        m_canvasROIModePanelRight.setLayout( new BoxLayout( m_canvasROIModePanelRight, BoxLayout.X_AXIS ) );
        // m_canvasROIModePanelRight.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );
        m_canvasROIModePanelRight.setPreferredSize( new Dimension( 512, 30 ) );
        m_canvasROIModePanelRight.setMaximumSize( new Dimension( 512, 30 ) );
        m_canvasROIModePanelRight.setMinimumSize( new Dimension( 512, 30 ) );
        
        m_addROIButtonRight = new JToggleButton( "Add ROI" );
        m_addROIButtonRight.addActionListener( this );
        m_deleteROIButtonRight = new JToggleButton( "Delete ROI" );
        m_deleteROIButtonRight.addActionListener( this );
        m_cutROIButtonRight = new JToggleButton( "Cut" ); 
        m_cutROIButtonRight.addActionListener( this );
        m_resizeROIButtonRight = new JToggleButton( "Resize" );
        m_resizeROIButtonRight.addActionListener( this );
        m_moveROIButtonRight = new JToggleButton( "Move" );
        m_moveROIButtonRight.addActionListener( this );
        m_selectROIButtonRight = new JToggleButton( "Select ROI" );
        m_selectROIButtonRight.addActionListener( this );
        
        m_canvasROIModePanelRight.add( Box.createHorizontalGlue( ) );
        
        m_canvasROIModePanelRight.add( m_addROIButtonRight );
        m_canvasROIModePanelRight.add( m_deleteROIButtonRight );
        m_canvasROIModePanelRight.add( m_cutROIButtonRight );
        m_canvasROIModePanelRight.add( m_resizeROIButtonRight );
        m_canvasROIModePanelRight.add( m_moveROIButtonRight );
        m_canvasROIModePanelRight.add( m_selectROIButtonRight );
        
        m_canvasROIModePanelRight.add( Box.createHorizontalGlue( ) );
        
        m_buttonGroupROIModesRight = new ButtonGroup( );

        m_buttonGroupROIModesRight.add( m_addROIButtonRight );
        m_buttonGroupROIModesRight.add( m_deleteROIButtonRight );
        m_buttonGroupROIModesRight.add( m_cutROIButtonRight );
        m_buttonGroupROIModesRight.add( m_resizeROIButtonRight );
        m_buttonGroupROIModesRight.add( m_moveROIButtonRight );
        m_buttonGroupROIModesRight.add( m_selectROIButtonRight );
       
        m_selectROIButtonRight.setSelected( true );
        
    }
    
    /**
     * Adds an actionlistener to the list of action listeners who are notified
     * when something happens within this object
     */
    public void addActionListener( ActionListener e )
    {
        m_listeners.add( e );
    }

    /**
     * Creates the panel that allows the user to set the region size and 
     * number of probing pixels and the number of permutations.
     */
    private void createProbabilitySegmentationProbingRegionPropertiesPanel( ) 
    {
        
        float maxPixVal = m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw.getMaxRawDataValueOfSliceStack( );
        float minPixVal = m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw.getMinRawDataValueOfSliceStack( );
        
        m_probingRegionPropertiesPanel = new ProbingRegionPropertiesPanel( minPixVal, maxPixVal );
        m_probingRegionPropertiesPanel.setPreferredSize( new Dimension( 512, 290 ) );
        m_probingRegionPropertiesPanel.setMaximumSize( new Dimension( 512, 290 ) );
        m_probingRegionPropertiesPanel.setMinimumSize( new Dimension( 512, 290 ) );
        m_probingRegionPropertiesPanel.addActionListener( this );
    }

    /**
     * Creates a panel that allows the user to set the significance level
     * manipulate image segments, allows to view different image segments,
     * i.e. parts of the image that belongs to a certain image type
     */
    private void createSegmentationResultsAnalyzerPanel( ) 
    {
        // The segmentation results analyzer panel must be aware of an instance 
        // of the DicomSliceCanvasPolydraw object to draw the resulting overlays 
        // upon.
        m_segmentationResultsAnalyzerPanel = new SegmentationResultsAnalyzerPanel( m_segmentationResultsImageDicomSliceCanvasPolydraw );
        m_segmentationResultsAnalyzerPanel.setPreferredSize( new Dimension( 512, 275 ) );
        m_segmentationResultsAnalyzerPanel.setMaximumSize( new Dimension( 512, 275 ) );
        m_segmentationResultsAnalyzerPanel.setMinimumSize( new Dimension( 512, 275 ) );
        m_segmentationResultsAnalyzerPanel.addActionListener( this );
        
    }
  
    /**
     * Returns the list of persistent polygons
     */
    public LinkedList getPersistentPolygons( )
    {
        return m_basicTypesROIPropertiesTable.getPersistentPolygons( );
    }
    
    /**
     * Returns the applied zoom
     */
    public float getAppliedZoom( )
    {
        return m_appliedZoom;
    }
    
    // Private attributes of the segmentation frame
    private JPanel m_collectionPanel;
    private JFrame m_parent;

    private JPanel m_northCollectionPanel;
    private JPanel m_northWestCollectionPanel;
    private JPanel m_dicomInputImagePanel;
    
    private JPanel m_northEastCollectionPanel;
    private JPanel m_dicomSegmentationResultsImagePanel;
    
    private RegionPropertiesTable m_basicTypesROIPropertiesTable;
    private JScrollPane m_ROIPropertiesScrollPane;
    
    private ImageSegmentPropertiesTable m_segmentationResultsROIPropertiesTable;
    private JScrollPane m_segmentationResultsROIPropertiesScrollPane;
    private SegmentationResultsAnalyzerPanel m_segmentationResultsAnalyzerPanel;
    
    private JPanel m_selectionPanel; // allows user to cancel/accept operations
    
    private JPanel m_centerCollectionPanel;
    private JPanel m_centerWestCollectionPanel;
    
    
    private JPanel m_canvasROIModePanelLeft;
    private JToggleButton m_addROIButtonLeft;
    private JToggleButton m_deleteROIButtonLeft;
    private JToggleButton m_cutROIButtonLeft;
    private JToggleButton m_resizeROIButtonLeft;
    private JToggleButton m_moveROIButtonLeft;
    private JToggleButton m_selectROIButtonLeft;

    private ButtonGroup m_buttonGroupROIModesLeft;
    
    private JPanel m_canvasROIModePanelRight;
    private JToggleButton m_addROIButtonRight;
    private JToggleButton m_deleteROIButtonRight;
    private JToggleButton m_cutROIButtonRight;
    private JToggleButton m_resizeROIButtonRight;
    private JToggleButton m_moveROIButtonRight;
    private JToggleButton m_selectROIButtonRight;

    private ButtonGroup m_buttonGroupROIModesRight;
    
    private JPanel m_centerEastCollectionPanel;
    
    
    private DicomSliceCanvasPolyDraw m_basicTissueTypesDefinitionDicomSliceCanvasPolydraw;
    private DicomSliceCanvasPolyDraw m_segmentationResultsImageDicomSliceCanvasPolydraw;

    private float m_appliedZoom;

    private int m_maxPixelValueOfLoadedImage;
    private int m_minPixelValueOfLoadedImage;

    private JButton m_applyButton;
    private JButton m_cancelButton;

    private ProbingRegionPropertiesPanel m_probingRegionPropertiesPanel; 

    
    // Listeners to action events in this class
    private LinkedList<ActionListener> m_listeners = new LinkedList<ActionListener>( );
    
    // Events raised by accept and cancel button
    public final static int ACCEPT_EVENT = 0;
    public final static int CANCEL_EVENT = 1;

    private boolean m_applyButtonPressed = false;
    
    // For proper serialization
    private static final long serialVersionUID = 1499343591289333L;
    
    private boolean m_userIsDefiningROIToSegment = false;

    private ProbabilityBasedSegmentationCalculator m_probabilityBasedSegmentationCalculator;
    private Color[] m_segmentationColors;

    private int m_minimumImageNumberToFocus;
    private int m_maximumImageNumberToFocus;
    private float m_nordicScalingFactor = 1f;
    
}
