 /*
 * File: $Id: DicomSliceCanvas.java,v 1.5 2008/10/09 10:52:00 hans Exp $
 * History
 * Date: 09.10.2008  Authors: Hans Slotboom  
 *
 * Changes:
 *
 * Copyright (C) 2008 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */


package org.spectrim.imageprocessing;

import java.util.Vector;
import java.util.LinkedList;
import java.util.Collections;
import java.util.Iterator;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;
import java.awt.event.KeyListener;
import java.awt.event.KeyEvent;


import javax.swing.JTable;
import javax.swing.table.DefaultTableModel;
import javax.swing.JFrame;

import javax.swing.JPopupMenu;
import javax.swing.JMenuItem;

import org.spectrim.lib.ColumnSorter;
import org.spectrim.lib.EnterStringFrame;

import org.spectrim.dicom.DicomSliceCanvasPolyDraw;

import org.spectrim.imagegraphicobjects.RegioPolygon;

/**
 * Table showing an overview of all 
 * @author  hans
 */

public class ImageSegmentPropertiesTable extends JTable implements ActionListener, MouseListener, KeyListener
{
    
    /** 
     * Creates a new instance of ImageSegmentPropertiesTable 
     */
    public ImageSegmentPropertiesTable( JFrame parent, DicomSliceCanvasPolyDraw canvas )
    {
        super( );
        
        m_parent = parent;
        
        m_dicomSliceCanvas = canvas;
        
        m_tableModel = new DefaultTableModel( );
        
        m_tableModel.addColumn( "<HTML><B>Segment name</B></HTML>" );
        m_tableModel.addColumn( "<HTML><B>Pixels</B></HTML>" );
        m_tableModel.addColumn( "<HTML><B>Volume</B></HTML>" );
        m_tableModel.addColumn( "<HTML><B>Average</B></HTML>" );
        m_tableModel.addColumn( "<HTML><B>St. dev.</B></HTML>" );

        setModel( m_tableModel );
        
        setAutoCreateColumnsFromModel( false );
        
        setColumnSelectionAllowed (false );
        setRowSelectionAllowed( true );

        constructObject( );
        
        this.addKeyListener( this );
    }
     
    
    /**
     * Does the rest of the object construction work
     */
    private void constructObject( )
    {
        m_popup = new JPopupMenu( );
        
        m_sortBySegmentName = new JMenuItem( "Sort by segment name" );
        m_sortBySegmentName.addActionListener( this );

        m_sortByNumberOfPixelValue = new JMenuItem( "Sort by number of pixel values" );
        m_sortByNumberOfPixelValue.addActionListener( this );
        
        m_sortBySegmentVolumeValue = new JMenuItem( "Sort by segment volume value" );
        m_sortBySegmentVolumeValue.addActionListener( this );
        
        m_sortByAveragePixelIntensityValue = new JMenuItem( "Sort by average pixel intensity value" );
        m_sortByAveragePixelIntensityValue.addActionListener( this );
        
        m_sortByAveragePixelIntensityStandardDeviationValue = new JMenuItem( "Sort by average pixel intensity standard deviation value" );
        m_sortByAveragePixelIntensityStandardDeviationValue.addActionListener( this );
             
        m_createReportTextFileMenuItem = new JMenuItem( "Create a Report text file .. " );
        m_createReportTextFileMenuItem.addActionListener( this );
        
        m_popup.add( m_sortBySegmentName  );
        m_popup.add( m_sortByNumberOfPixelValue  );
        m_popup.add( m_sortBySegmentVolumeValue  );
        m_popup.add( m_sortByAveragePixelIntensityValue );
        m_popup.add( m_sortByAveragePixelIntensityStandardDeviationValue );
        
        m_popup.addSeparator( );
         
        add( m_popup );
        
        m_popup.add( m_createReportTextFileMenuItem );
        
        addMouseListener( this );
    }
    
    
    /**
     * Is called whenever a registered component raises an ActionEvent
     */
    public void actionPerformed( ActionEvent e ) 
    {
        Object sender = e.getSource( );
        
        if ( sender == m_sortBySegmentName )
        {
            sortAllRowsBy( (DefaultTableModel)this.getModel( ), 0, true );
        }
        
        if ( sender == m_sortByNumberOfPixelValue )
        {
            sortAllRowsBy( (DefaultTableModel)this.getModel( ), 1, true );
        }
        
        if ( sender == m_sortBySegmentVolumeValue )
        {
            sortAllRowsBy( (DefaultTableModel)this.getModel( ), 2, true );
        }
        
        if ( sender == m_sortByAveragePixelIntensityValue )
        {
            sortAllRowsBy( (DefaultTableModel)this.getModel( ), 3, true );
        }
        
        if ( sender == m_sortByAveragePixelIntensityStandardDeviationValue )
        {
            sortAllRowsBy( (DefaultTableModel)this.getModel( ), 4, true );
        }
        
        if ( sender == m_createReportTextFileMenuItem )
        {
            notifyActionListeners( CREATE_REPORT_FILE_EVENT, "create report file of image segments");
        }
        
        
        if ( sender == m_strEnterFrame )
        {
            if ( e.getID( ) == EnterStringFrame.APPLY_EVENT )
            {
                String str = m_strEnterFrame.getText( );
                applySelectedROIRenaming( str );
            }
        }
                
    }    
    
    /**
     * Is called whenever a mouse button is pressed
     */
    public void mousePressed( MouseEvent e ) 
    {
        System.out.println( "Mouse pressed!" );
        if ( e.isPopupTrigger( ) )
        {
            m_popup.show( e.getComponent( ), e.getX( ), e.getY( ) );
        }
        
        setSelectedROIItems( );
        
    }
    
    /**
     * Is called whenever a mouse button is clicked
     */
    
    public void mouseClicked( MouseEvent e ) 
    {
    }
    
    /**
     * Is called whenever a mouse pointer enters
     */
    
    public void mouseEntered( MouseEvent e ) 
    {
    }
    
    /**
     * Is called whenever the mouse pointer exits
     */
    public void mouseExited( MouseEvent e ) 
    {
    }
    
    /**
     * Is called whenever a mouse button is released
     */
    public void mouseReleased( MouseEvent e ) 
    {
        if ( e.isPopupTrigger( ) )
        {
            m_popup.show( e.getComponent( ), e.getX( ), e.getY( ) );
        }
    }
    
    // Regardless of sort order (ascending or descending), null values always appear last.
    // colIndex specifies a column in model.
    public void sortAllRowsBy( DefaultTableModel model, int colIndex, boolean ascending ) 
    {
        Vector data = model.getDataVector( );
        Collections.sort( data, new ColumnSorter( colIndex, ascending ) );
        model.fireTableStructureChanged( );
    }
    
    /**
     * This method updates the segment list with current information
     */
    public void upDateTableData( LinkedList<ImageSegmentProperties> segmentProperties )
    {
        
        // first, clear the table
        int tableSize =  m_tableModel.getRowCount( );
        
        for ( int i = tableSize - 1 ; i >= 0; i-- )
        {
            m_tableModel.removeRow( i );
        }
        
        
        // fill the table again..
        for ( int i = 0; i < segmentProperties.size( ); i++ )
        {
            ImageSegmentProperties segmentProp = segmentProperties.get( i );
            
            m_tableModel.addRow( new Object[]{ segmentProp.getImageSegmentName( ), 
                                               new Integer( segmentProp.getNumberOfPixels( ) ),
                                               new Float( segmentProp.getImageSegmentVolume( ) ), 
                                               new Float( segmentProp.getMeanPixelValue( ) ),
                                               new Float( segmentProp.getStandardDeviationOfPixelValues( ) ),
                                             }
                                );
        }
        
        
    }
    
         
    /**
     * Due to the fact that the segmentation algorithm may find and ROI
     * within an ROI, i.e. the case when a pixel collection is again larger
     * or smaller than the segementation thressholds. In other words, it may
     * find inner or outer borders. In order to add pixel values correctly 
     * together for e.g. averaging, or count volume segments together one needs
     * an algorithm that finds out whether the contour was an inner or an outer
     * border.
     */
    private int determineROISign( float value, int minThresshold, int maxThresshold )
    {
        if ( ( value <= maxThresshold ) && ( value >= minThresshold ) )
        {
            return 1;
        }
        else
        {
            return -1;
        }
    }
    
    /**
     * Sets the selected ROI items in the list, also selected (blue) in the 
     * SliceCanvasPolyDraw
     */
    private void setSelectedROIItems( )
    {
        int[] selectedRows = this.getSelectedRows( );
        String[] selectedROINames = new String[ selectedRows.length ];
        
        for ( int i = 0; i < selectedRows.length; i++ )
        {
            selectedROINames[i] = (String)getValueAt( selectedRows[i], 0 );
        }
        
        m_dicomSliceCanvas.selectPolygons( selectedROINames );
    }
        
    /**
     * Applies the renaming of the roi's
     */
    private void applySelectedROIRenaming( String newName )
    {
        int[] selectedRows = this.getSelectedRows( );
        String[] selectedROINames = new String[ selectedRows.length ];
        
        String cntrString;
        
        for ( int i = 0; i < selectedRows.length; i++ )
        {
            selectedROINames[i] = (String)getValueAt( selectedRows[i], 0 );
        }
        
        for ( int i = 0; i < selectedRows.length; i++ )
        {
            if ( i < 10 )
            {
                cntrString = "0" + i;
            }
            else
            {
                cntrString = "" + i;
            }
            
            m_dicomSliceCanvas.getPolygon( selectedROINames[i] ).setLabel( newName + cntrString );
            m_tableModel.setValueAt( newName + cntrString, selectedRows[i], 0 );
            
        }
        
        m_dicomSliceCanvas.update( );
    }
    
    /**
     * Notify all actionListeners
     */
    private void notifyActionListeners( int eventType, String actionName )
    {
        Iterator it = m_listeners.iterator( );
        while( it.hasNext( ) )
        {
            ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, eventType, actionName ) );
        }
    }
    
    /**
     * Is called wheneve a key is pressed
     */
    public void keyPressed( KeyEvent e ) 
    {
    }
    
    /**
     * Is called wheneve a key is released
     */
    public void keyReleased( KeyEvent e ) 
    {
        
        if ( ( e.getKeyCode( ) == KeyEvent.VK_PAGE_UP ) || ( e.getKeyCode( ) == KeyEvent.VK_PAGE_DOWN ) ||
             ( e.getKeyCode( ) == KeyEvent.VK_DOWN ) || ( e.getKeyCode( ) == KeyEvent.VK_UP ) )
        {
            setSelectedROIItems( );
        }
    }
    
    /**
     * Is called whenever a key is typed
     */
    public void keyTyped( KeyEvent e ) 
    {
    }
    
    /**
     * Returns the linked list to the caller
     */
    public LinkedList getPersistentPolygons( )
    {
        return m_persistentROIList;
    }
    
    /**
     * Returns the list of ROIs that needs to be delete from the list,
     * and from the DicomSliceCanvaPolyDraw 
     */
    public LinkedList getROIsToDeleteNameList( )
    {
        return m_ROIsToDeleteNameList;
    }
    
    /**
     * Adds an actionlistener to the list of action listeners who are notified
     * when something happens with the spinbox
     */
    public void addActionListener( ActionListener e )
    {
        m_listeners.add( e );
    }
    

    
    
    // Private attributes
    
    private JPopupMenu m_popup;
    
    private JMenuItem m_sortBySegmentName;
    private JMenuItem m_sortByNumberOfPixelValue;
    private JMenuItem m_sortBySegmentVolumeValue;
    private JMenuItem m_sortByAveragePixelIntensityValue;
    private JMenuItem m_sortByAveragePixelIntensityStandardDeviationValue;
    
    private JMenuItem m_createReportTextFileMenuItem;
    
    private DefaultTableModel m_tableModel;
    
    private DicomSliceCanvasPolyDraw m_dicomSliceCanvas;
    
    private LinkedList<RegioPolygon> m_persistentROIList = new LinkedList<RegioPolygon>( );
    
    private JFrame m_parent;
    private EnterStringFrame m_strEnterFrame;
    
    public final  static int DELETE_EVENT = 0;
    private LinkedList<String> m_ROIsToDeleteNameList = new LinkedList<String>( );
    
    // Listeners to changes in the segmentation gui control
    private LinkedList<ActionListener> m_listeners = new LinkedList<ActionListener>( );
    
    public final static int CREATE_REPORT_FILE_EVENT = 0;
    
    // Must be declarated somehow
    private static final long serialVersionUID = 3487493391119393L;
}