/*
 * File: $Id: MorphologicFilterFrame.java,v 1.1.1.1 2006/04/19 09:38:48 hans Exp $
 * History
 * Date: 16.02.2007 Hans Slotboom  Changes:
 *
 * Copyright (C) 2007 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */


package org.spectrim.imageprocessing;

import org.spectrim.dicom.DicomImageSeriesData;
import java.awt.Dimension;
import java.awt.BorderLayout;
import java.awt.Point;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JComboBox;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JSlider;
import javax.swing.JTabbedPane;
import javax.swing.BorderFactory;


import java.util.Iterator;
import java.util.LinkedList;
import java.util.LinkedHashMap;


import org.spectrim.dicom.DicomSliceCanvasPolyDraw;
import org.spectrim.dicom.FileInfo;

import org.spectrim.signalprocessing.SignalTools;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * 
 * Frame that enables morphologic filtering of MR-image data.
 * 
 * @author  hans
 */
public class MorphologicFilterFrame extends JFrame implements ActionListener, ChangeListener 
{
    
    /** 
     * Constructor of a new the JFrame MorphologicFilterFrame. 
     * @param parent
     * @param rawData
     * @param fInfo
     * @param siemensSpecificHeader
     * @param numberOfImages
     */
    public MorphologicFilterFrame( JFrame parent, int[] rawData, 
                                   FileInfo fInfo, 
                                   LinkedHashMap siemensSpecificHeader, 
                                   int numberOfImages ) 
    {
        
        m_parent = parent;
        m_fileInfo = fInfo;
        
        m_imagePanel = new JPanel( );
        m_imagePanel.setLayout( new BoxLayout( m_imagePanel, BoxLayout.X_AXIS ) );
        
        m_imagePanel.setMinimumSize( new Dimension( 512, 512 ) );
        m_imagePanel.setMaximumSize( new Dimension( 512, 512 ) );
        m_imagePanel.setPreferredSize( new Dimension( 512, 512 ) );
        
        m_dicomCanvas = new DicomSliceCanvasPolyDraw( this );
        if ( fInfo.getFileName( ) != null )
        {
            m_dicomCanvas.setTitle( fInfo.getFileName( ) );
        }
        else
        {
            setTitle( fInfo.getPatientName( ) + "  " + fInfo.getPatientBirthDate( ) );
        }
        
        m_dicomCanvas.addActionListener( this );
        
        m_dicomCanvas.setNumberOfImages( numberOfImages );
        
        m_originalPixelData = new int[ fInfo.getImageWidth( ) * fInfo.getImageHeight( ) ];
        m_filteredPixelData = new int[ fInfo.getImageWidth( ) * fInfo.getImageHeight( ) ];
        
        System.arraycopy( rawData, 0, m_originalPixelData, 0, rawData.length );
        System.arraycopy( rawData, 0, m_filteredPixelData, 0, rawData.length );
        
        m_rawDataMaxPixVal = SignalTools.getMaximum( rawData );
        m_rawDataMinPixVal = SignalTools.getMinimum( rawData );
        
        m_dicomCanvas.setImage( rawData , fInfo.getImageWidth( ), 
                                fInfo.getImageHeight( ), 0, 
                                fInfo, siemensSpecificHeader );
        
        
        m_imagePanel.add( Box.createHorizontalGlue( ) );
        m_imagePanel.add( m_dicomCanvas );
        m_imagePanel.add( Box.createHorizontalGlue( ) );
        
        m_dicomCanvas.setPreferredSize( new Dimension( 512, 512 ) );
        m_dicomCanvas.setMinimumSize( new Dimension( 512, 512 ) );
        m_dicomCanvas.setMaximumSize( new Dimension( 512, 512 ) );
        
        if ( fInfo.getImageWidth( ) != 0 )
        {
            m_appliedZoom = 512f / fInfo.getImageWidth( );
        }
        
        m_dicomCanvas.enableImagePaging( false );
        
        m_dicomCanvas.setVisible( true );
        
        createFilterTypePanel( );
        createApplyCancelUndoPanel( );
         
        m_collectionPanel = new JPanel( );
        m_collectionPanel.setLayout( new BorderLayout( ) );
        
        m_collectionPanel.add( m_imagePanel, BorderLayout.NORTH );
        m_collectionPanel.add( m_filterFamilyType, BorderLayout.CENTER );
        m_collectionPanel.add( m_selectionPanel, BorderLayout.SOUTH );
        
        getContentPane( ).add( m_collectionPanel );
        setVisible( true );
        m_dicomCanvas.setZoom( m_appliedZoom );
        
        pack( );
        
        // Put the Dialog nicely in the middle of its parent
        Point centerParent = new Point( m_parent.getX( ) + ( m_parent.getWidth( ) / 2 ) - this.getWidth( ) / 2,
                                        m_parent.getY( ) + ( m_parent.getHeight( ) / 2 ) - this.getHeight( ) / 2 );
        
        setLocation( centerParent );
    }
    
    /** 
     * Second constructor to creates a MorphologicFilterFrame JFrame. 
     * @param parent
     * @param imageSeriesData
     */
    public MorphologicFilterFrame( JFrame parent, DicomImageSeriesData imageSeriesData )  
    {
        
        m_parent = parent;
        m_numberOfImages = imageSeriesData.getNumberOfImages( );
        m_fileInfo = imageSeriesData.getFileInfo( m_numberOfImages / 2 );
        
        m_imagePanel = new JPanel( );
        m_imagePanel.setLayout( new BoxLayout( m_imagePanel, BoxLayout.X_AXIS ) );
        
        m_imagePanel.setMinimumSize( new Dimension( 512, 512 ) );
        m_imagePanel.setMaximumSize( new Dimension( 512, 512 ) );
        m_imagePanel.setPreferredSize( new Dimension( 512, 512 ) );
        
        m_dicomCanvas = new DicomSliceCanvasPolyDraw( this );
        
        if ( m_fileInfo.getFileName( ) != null )
        {
            m_dicomCanvas.setTitle( m_fileInfo.getFileName( ) );
        }
        else
        {
            setTitle( m_fileInfo.getPatientName( ) + "  " + m_fileInfo.getPatientBirthDate( ) );
        }
    
        m_dicomCanvas.setDicomImages( imageSeriesData );
        
        m_dicomCanvas.addActionListener( this );
        
        m_dicomCanvas.enableImagePaging( true );
        
        m_dicomCanvas.displayDicomImage( m_numberOfImages / 2 );
        
        m_dicomCanvas.setPreferredSize( new Dimension( 512, 512 ) );
        m_dicomCanvas.setMaximumSize( new Dimension( 512, 512 ) );
        m_dicomCanvas.setMinimumSize( new Dimension( 512, 512 ) );
        
        m_dicomCanvas.addActionListener( this );
        
        m_originalPixelData = new int[ m_numberOfImages * m_fileInfo.getImageWidth( ) * m_fileInfo.getImageHeight( ) ];
        m_filteredPixelData = new int[ m_numberOfImages * m_fileInfo.getImageWidth( ) * m_fileInfo.getImageHeight( ) ];
        
        System.arraycopy( imageSeriesData.getSeriesPixelData( ), 0, m_originalPixelData, 
                          0, imageSeriesData.getSeriesPixelData().length );
        System.arraycopy( imageSeriesData.getSeriesPixelData( ), 0, m_filteredPixelData, 
                          0, imageSeriesData.getSeriesPixelData().length );
        
        m_rawDataMaxPixVal = SignalTools.getMaximum( imageSeriesData.getSeriesPixelData( ) );
        m_rawDataMinPixVal = SignalTools.getMinimum( imageSeriesData.getSeriesPixelData( ) );
               
        m_imagePanel.add( Box.createHorizontalGlue( ) );
        m_imagePanel.add( m_dicomCanvas );
        m_imagePanel.add( Box.createHorizontalGlue( ) );
        
        m_dicomCanvas.setPreferredSize( new Dimension( 512, 512 ) );
        m_dicomCanvas.setMinimumSize( new Dimension( 512, 512 ) );
        m_dicomCanvas.setMaximumSize( new Dimension( 512, 512 ) );
        
        if ( m_fileInfo.getImageWidth( ) != 0 )
        {
            m_appliedZoom = 512f / m_fileInfo.getImageWidth( );
        }
        
        createFilterTypePanel( );
        createApplyCancelUndoPanel( );
         
        m_collectionPanel = new JPanel( );
        m_collectionPanel.setLayout( new BorderLayout( ) );
        
        m_collectionPanel.add( m_imagePanel, BorderLayout.NORTH );
        m_collectionPanel.add( m_filterFamilyType, BorderLayout.CENTER );
        m_collectionPanel.add( m_selectionPanel, BorderLayout.SOUTH );
        
        getContentPane( ).add( m_collectionPanel );
        setVisible( true );
        m_dicomCanvas.setZoom( m_appliedZoom );
        
        pack( );
        
        // Put the Dialog nicely in the middle of its parent
        Point centerParent = new Point( m_parent.getX( ) + ( m_parent.getWidth( ) / 2 ) - this.getWidth( ) / 2,
                                        m_parent.getY( ) + ( m_parent.getHeight( ) / 2 ) - this.getHeight( ) / 2 );
        
        setLocation( centerParent );
    }

    /**
     * Sets the raw data image data attribute.
     * @param processedData
     */
    public void setImageSeriesPixelData( int[] processedData )
    {
        m_originalPixelData = new int[processedData.length];
        m_filteredPixelData = new int[processedData.length];

        System.arraycopy( processedData, 0, m_originalPixelData, 0, processedData.length );
        System.arraycopy( processedData, 0, m_filteredPixelData, 0, processedData.length );

         m_dicomCanvas.setImageSeriesPixelData( processedData );
         m_dicomCanvas.displayDicomImage( m_dicomCanvas.getCurrentlyDisplayedImageNumber( ) );
         m_dicomCanvas.update( true );
    }

    /**
     * Creates a filter selection type panel.
     */
    private void createFilterTypePanel( )
    {
        m_filterFamilyType = new JTabbedPane( JTabbedPane.BOTTOM );
        
        createMorphologicFilterParameterPanel( );
        
        m_filterFamilyType.addTab( "Circular kernal", m_circularKernalPanel );
        m_filterFamilyType.addTab( "Non-circular filter kernal", m_nonCircularKernalPanel );
        
    }
    
    /**
     * Creates the panel allowing the frames basic button operations.
     */
    private void createApplyCancelUndoPanel( )
    {
        m_selectionPanel = new JPanel( );
        
        m_selectionPanel.setLayout( new BoxLayout( m_selectionPanel, BoxLayout.X_AXIS ) );
        m_selectionPanel.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );
        
        m_applyButton = new JButton( "Apply" );
        m_applyAndQuitButton = new JButton( "Apply and Quit" );
        m_undoAllButton = new JButton( "Undo All" );
        m_cancelButton = new JButton( "Cancel" );
        
        m_applyButton.addActionListener( this );
        m_applyAndQuitButton.addActionListener( this );
        m_cancelButton.addActionListener( this );
        m_undoAllButton.addActionListener( this );
        
        m_selectionPanel.add( Box.createGlue( ) );
        m_selectionPanel.add( m_applyButton );
        m_selectionPanel.add( Box.createGlue( ) );
        m_selectionPanel.add( m_applyAndQuitButton );
        m_selectionPanel.add( Box.createGlue( ) );
        m_selectionPanel.add( m_undoAllButton );
        m_selectionPanel.add( Box.createGlue( ) );
        m_selectionPanel.add( m_cancelButton );
        m_selectionPanel.add( Box.createGlue( ) );
    }
      
    
    /**
     * Creates the GUI to set morphologic filter type and filter parameters.
     */
    private void createMorphologicFilterParameterPanel( )
    {
        m_circularKernalPanel = new JPanel( );
        
        m_circularKernalPanel.setLayout( new BoxLayout( m_circularKernalPanel, BoxLayout.Y_AXIS ) );
                
        m_circularKernalRadiusValueLabel = new JLabel( "Filter radius " );
        
        m_circularKernalFilterSizeComboBox = new JComboBox( m_circularKernalFilterSizes );
        m_circularKernalFilterSizeComboBox.setPreferredSize( new Dimension( 100, 25 ) );
        m_circularKernalFilterSizeComboBox.setMaximumSize( new Dimension( 100, 25 ) );
        m_circularKernalFilterSizeComboBox.setMinimumSize( new Dimension( 100, 25 ) );
        
        m_autoApplyButton = new JCheckBox( );
        
        m_circularKernalPanel.add( m_autoApplyButton );
        
        m_circularKernalFilterSizeParameterPanel = new JPanel( );
        m_circularKernalFilterSizeParameterPanel.add( m_circularKernalRadiusValueLabel );
        m_circularKernalFilterSizeParameterPanel.add( m_circularKernalFilterSizeComboBox );
        m_circularKernalFilterSizeParameterPanel.add( Box.createHorizontalGlue( ) );
        
        // Add sub components to collection component
       
        m_circularKernalPanel.add( m_circularKernalFilterSizeParameterPanel );
        
        createThressholdingPanel( m_rawDataMinPixVal, m_rawDataMaxPixVal );
        
        m_circularKernalPanel.add( m_thresholdsDefinitionPanel );

    }   
    
    /**
     * Creates the panel containing the threshold -sliders.
     */
    private void createThressholdingPanel( int defaultMininumPixelValue, 
                                           int defaultMaximumPixelValue )
    {
        m_thresholdsDefinitionPanel = new JPanel( );
        m_thresholdsDefinitionPanel.setLayout( new BoxLayout( m_thresholdsDefinitionPanel, BoxLayout.Y_AXIS ) );
        m_thresholdsDefinitionPanel.setBorder( BorderFactory.createTitledBorder( "Morphologic Filter" ) );
        m_thresholdsDefinitionPanel.setPreferredSize( new Dimension( 524, 150 ) );
        m_thresholdsDefinitionPanel.setMinimumSize( new Dimension( 524, 150 ) );
        m_thresholdsDefinitionPanel.setMaximumSize( new Dimension( 524, 150 ) );
        
        m_minSliderPanel = new JPanel( );
        m_minSliderPanel.setPreferredSize( new Dimension( 512, 55 ) );
        m_minSliderPanel.setMaximumSize( new Dimension( 512, 55 ) );
        m_minSliderPanel.setMinimumSize( new Dimension( 512, 55 ) );
        m_minSliderPanel.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );
        
        m_minSliderPanel.setLayout( new BoxLayout( m_minSliderPanel, BoxLayout.X_AXIS ) );
        
        m_minPixelValueIndicator = new JLabel( "" + defaultMininumPixelValue );
        
        m_minPixelValueIndicator.setPreferredSize( new Dimension( 40, 12 ) );
        m_minPixelValueIndicator.setMaximumSize( new Dimension( 40, 12 ) );
        m_minPixelValueIndicator.setMinimumSize( new Dimension( 40, 12 ) );
        
        m_minSliderPanel.add( m_minPixelValueIndicator );
        
        m_minimumPixelValueSlider = new JSlider( JSlider.HORIZONTAL, 0, 1000, 0 );
	m_minimumPixelValueSlider.setMajorTickSpacing( 250 );
        m_minimumPixelValueSlider.setMinorTickSpacing( 50 );
        m_minimumPixelValueSlider.setPaintTicks( true );
        m_minimumPixelValueSlider.setPaintLabels( true );
        m_minimumPixelValueSlider.setValue( 0 );
        m_minimumPixelValueSlider.addChangeListener( this );
        
        m_minimumPixelValueSlider.setPreferredSize( new Dimension( 390, 50 ) );
        m_minimumPixelValueSlider.setMinimumSize( new Dimension( 390, 50 ) );
        m_minimumPixelValueSlider.setMaximumSize( new Dimension( 390, 50 ) );
        
        m_minPixelLabel = new JLabel( "Minimum [�]" );
        m_minSliderPanel.add( m_minimumPixelValueSlider );
        m_minSliderPanel.add( m_minPixelLabel );
        
        m_maxSliderPanel = new JPanel( );
        m_maxSliderPanel.setLayout( new BoxLayout( m_maxSliderPanel, BoxLayout.X_AXIS ) );
        m_maxSliderPanel.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );
        m_maxPixelValueIndicator = new JLabel( "" + defaultMaximumPixelValue );
        
        m_maxPixelValueIndicator.setPreferredSize( new Dimension( 40, 12 ) );
        m_maxPixelValueIndicator.setMaximumSize( new Dimension( 40, 12 ) );
        m_maxPixelValueIndicator.setMinimumSize( new Dimension( 40, 12 ) );
        
        m_maxSliderPanel.add( m_maxPixelValueIndicator );

        m_maxSliderPanel.setPreferredSize( new Dimension( 512, 55 ) );
        m_maxSliderPanel.setMaximumSize( new Dimension( 512, 55 ) );
        m_maxSliderPanel.setMinimumSize( new Dimension( 512, 55 ) );
        
        m_maximumPixelValueSlider = new JSlider( JSlider.HORIZONTAL, 0, 1000, 1000 );
	m_maximumPixelValueSlider.setMajorTickSpacing( 250 );
        m_maximumPixelValueSlider.setMinorTickSpacing( 50 );
        m_maximumPixelValueSlider.setPaintTicks( true );
        m_maximumPixelValueSlider.setPaintLabels( true );
        m_maximumPixelValueSlider.setValue( 1000 );
        m_maximumPixelValueSlider.addChangeListener( this );
        
        m_maximumPixelValueSlider.setPreferredSize( new Dimension( 390, 50 ) );
        m_maximumPixelValueSlider.setMinimumSize( new Dimension( 390, 50 ) );
        m_maximumPixelValueSlider.setMaximumSize( new Dimension( 390, 50 ) );
        
        m_maxPixelLabel = new JLabel( "Maximum [�]" );
        m_maxSliderPanel.add( m_maximumPixelValueSlider );
        m_maxSliderPanel.add( m_maxPixelLabel );
        
        m_thresholdsDefinitionPanel.add( m_minSliderPanel );
        m_thresholdsDefinitionPanel.add( m_maxSliderPanel );
    }
    
    /** 
     * This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents( ) 
    {                          

        setDefaultCloseOperation( javax.swing.WindowConstants.EXIT_ON_CLOSE );
        pack( );
    }
    
    
    /**
     * Is called whenever a registered component raises an ActionEvent.
     * @param e
     */
    @Override
    public void actionPerformed( ActionEvent e )
    {
                
        if ( e.getSource( ) == this.m_cancelButton )
        {
            dispose( );
        }
        
        if ( e.getSource( ) == this.m_undoAllButton )
        {
            if ( this.m_numberOfImages > 1 )
            {
                m_dicomCanvas.setImageSeriesPixelData( m_originalPixelData );
                m_dicomCanvas.displayDicomImage( m_dicomCanvas.getCurrentlyDisplayedImageNumber( ) );
                m_dicomCanvas.update( true );
            }
            else
            {
                m_dicomCanvas.setImage( m_originalPixelData, m_fileInfo.getImageWidth(), m_fileInfo.getImageHeight( ), 0 );
                m_dicomCanvas.update( true );
            }
            
            System.arraycopy( m_originalPixelData, 0, m_filteredPixelData, 0, m_originalPixelData.length );
            
            m_dicomCanvas.drawImage( false ); 
            
        }
        
        if ( e.getSource( ) == m_applyAndQuitButton )
        {
            
            notifyActionListeners( APPLY_FILTER );
            
            dispose( );
        }
        
        if ( e.getSource( ) == this.m_applyButton )
        {
            if ( this.m_filterFamilyType.getSelectedIndex( ) == 0 ) // simple morphologic filter is selected
            {
                performMorphologicFiltering( );   
            }
        }
    }
    
    /**
     * Performs morphologic filtering of the image pixel data. 
     */    
    private void performMorphologicFiltering( )
    {
        MorphologicFilters morph = new MorphologicFilters( );
        if ( m_numberOfImages > 1 )
        {
             m_filteredPixelData = morph.applySimpleMorphologicFilter( m_filteredPixelData,
                                                                       m_numberOfImages,
                                                                       m_fileInfo.getImageWidth( ), 
                                                                       m_fileInfo.getImageHeight( ),
                                                                       getMinimumThresshold( ),
                                                                       getMaximumThresshold( ),
                                                                       new Integer( (String)m_circularKernalFilterSizeComboBox.getSelectedItem( ) ).intValue( ) );
        
            m_dicomCanvas.setImageSeriesPixelData( m_filteredPixelData );
            m_dicomCanvas.displayDicomImage(m_dicomCanvas.getCurrentlyDisplayedImageNumber( ) );
            m_dicomCanvas.update( true );
        }           
        else
        {
             m_filteredPixelData = morph.applySimpleMorphologicFilter( m_filteredPixelData, 
                                                                       m_fileInfo.getImageWidth( ), 
                                                                       m_fileInfo.getImageHeight( ),
                                                                       getMinimumThresshold( ),
                                                                       getMaximumThresshold( ),
                                                                       new Integer( (String)m_circularKernalFilterSizeComboBox.getSelectedItem( ) ).intValue( ) );  
                                                                      
             m_dicomCanvas.setImage( m_filteredPixelData, 
                                     m_fileInfo.getImageWidth( ), 
                                     m_fileInfo.getImageHeight( ), 0 );
        
            m_dicomCanvas.update( true );
        }

        
    }
    /**
     * Returns the minimum threshold the user selected.
     * @return 
     */
    protected int getMinimumThresshold( )
    {
        return  ( m_rawDataMinPixVal  + (int)( m_minimumPixelValueSlider.getValue( ) * ( m_rawDataMaxPixVal - m_rawDataMinPixVal ) / 1000f ));
    }
    
    /**
     * Returns the maximum threshold the user selected.
     * @return 
     */
    protected int getMaximumThresshold( )
    {
        return  ( m_rawDataMinPixVal  + (int)( m_maximumPixelValueSlider.getValue( ) * ( m_rawDataMaxPixVal - m_rawDataMinPixVal ) / 1000f ));
    }
    
    /**
     * Adds an action listener to the list of action listeners who are notified.
     * @param e
     */
    public void addActionListener( ActionListener e )
    {
	m_listeners.add( e );
    }
    
    /**
     * Notify all actionListeners.
     */
    private void notifyActionListeners( int eventType )
    {
        Iterator it = m_listeners.iterator( );
        while( it.hasNext( ) )
        {
            ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, eventType, "" ) );
        }
    }
    
    
    
    /**
     * Returns the filtered dataset to the called.
     * @return 
     */
    public int[] getFilteredImageData( )
    {
        return m_filteredPixelData;
    }

    
    /**
     * Is called whenever a registered class raises a ChangeEvent.
     */
    @Override
    public void stateChanged( ChangeEvent e ) 
    {
        
        Object sender = e.getSource( );
        
        if ( sender == m_minimumPixelValueSlider ) 
        {
            if ( !m_minimumPixelValueSlider.getValueIsAdjusting( ) )
            {
                notifyActionListeners( MINIMUM_VALUE_CHANGE_EVENT );
                
                if ( this.m_autoApplyButton.isSelected( ) )
                {
                    notifyActionListeners( PERFORM_SEGMENTATION );
                }
            }
            
            m_minPixelValueIndicator.setText( "" +  ( m_rawDataMinPixVal  + (int)( m_minimumPixelValueSlider.getValue( ) * ( m_rawDataMaxPixVal - m_rawDataMinPixVal ) / 1000f )  ) );
        }
        
        if ( sender == m_maximumPixelValueSlider ) 
        {
            if ( !m_maximumPixelValueSlider.getValueIsAdjusting( ) )
            {
                notifyActionListeners( MAXIMUM_VALUE_CHANGE_EVENT );
                
                if ( this.m_autoApplyButton.isSelected( ) )
                {
                    notifyActionListeners( PERFORM_SEGMENTATION );
                }
            }
            
            m_maxPixelValueIndicator.setText( "" + ( m_rawDataMinPixVal  + (int)( m_maximumPixelValueSlider.getValue( ) * ( m_rawDataMaxPixVal - m_rawDataMinPixVal ) / 1000f ) ) );
        }
        
    }
    
    // Variables declaration - do not modify                     
    // End of variables declaration                   
    
    // Listeners to changes in the segmentation gui control
    private LinkedList<ActionListener> m_listeners = new LinkedList<>( );
    
    private JFrame m_parent;
    
    private JPanel m_imagePanel;
    
//    private JRadioButton m_HarmonicMeanFilter;
//    private JRadioButton m_arithmeticMeanRadioButton;
//    private JRadioButton m_contraHarmonicMeanFilter;
//    private JRadioButton m_geometricMeanFilter;
//    private ButtonGroup m_filterTypeButtonGroup;
//    private JLabel m_xMeanFilterSizeLabel;
//    private JComboBox m_xMeanFilterSizeCombobox;
//    
//    private JLabel m_yMeanFilterSizeLabel;
//    private JComboBox m_yMeanFilterSizeCombobox;
    
    private JLabel  m_circularKernalRadiusValueLabel;
    private JComboBox m_circularKernalFilterSizeComboBox;
    
    private JPanel  m_circularKernalPanel;
    private JPanel  m_meanFilterRButtonPanel;
    private JPanel  m_meanFilterParameterPanel;
    
    private JPanel m_circularKernalFilterSizeParameterPanel;
    
    private final String[] m_circularKernalFilterSizes = { "0", "1", "2", "3", "4", "5", "6", "7", 
                                                           "8", "9", "10", "11", "12", "13", "14",
                                                           "15", "16", "17", "18", "19", "20", "21" };
    
    private JTabbedPane m_filterFamilyType;
    
    private JPanel  m_selectionPanel;
    
    private JButton m_applyButton;
    private JButton m_applyAndQuitButton;
    private JButton m_undoAllButton;
    private JButton m_cancelButton;
    
    private JPanel  m_collectionPanel;
    
    private DicomSliceCanvasPolyDraw m_dicomCanvas;
    private FileInfo m_fileInfo;
    
    private float   m_appliedZoom;
    
    private int     m_numberOfImages;
    
    private int[]   m_originalPixelData;
    private int[]   m_filteredPixelData;
    
    // Attributes for order statistics frame
    private JPanel m_nonCircularKernalPanel;
    private JPanel m_orderStatisticsRButtonPanel;
    private JRadioButton m_minFilter;
    private JRadioButton m_maxFilter;
    private JRadioButton m_medianFilter;
    private JRadioButton m_midPointFilter;
    private JRadioButton m_alphatrimmedMeanFilter;
    
    private JPanel m_orderStatisticsFilterParameterPanel;
    
    private JLabel  m_xOrderStatisticsFilterSizeLabel;
    private JLabel  m_yOrderStatisticsFilterSizeLabel;
    private JComboBox m_xOrderStatisticsFilterSizeCombobox;
    private JComboBox m_yOrderStatisticsFilterSizeCombobox;
    
    
    // Event IDs raised by this control
    public final static int APPLY_FILTER = 0;
    
    // Must be declarated somehow
    private static final long serialVersionUID = 34874988952319393L;

    private JPanel m_thresholdsDefinitionPanel;
    private JPanel m_minSliderPanel;
    private JLabel m_minPixelValueIndicator;
    private String m_minPixelValue;
    private JSlider m_minimumPixelValueSlider;
    private JLabel m_minPixelLabel;

    private JPanel m_maxSliderPanel;
    private JLabel m_maxPixelValueIndicator;
    private String m_maxPixelValue;
    private JSlider m_maximumPixelValueSlider;
    private JLabel m_maxPixelLabel;

    private int m_rawDataMinPixVal;
    private int m_rawDataMaxPixVal;
    
    private JCheckBox m_autoApplyButton;
    
    public final static int MAXIMUM_VALUE_CHANGE_EVENT  = 0;
    public final static int MINIMUM_VALUE_CHANGE_EVENT  = 1;
    public final static int PERFORM_SEGMENTATION        = 2;
}
