/*
 * OverlayPropertiesTable.java
 *
 * Created on April 21, 2008, 11:20 AM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package org.spectrim.imageprocessing;

import org.spectrim.lib.ColorEditor;
import org.spectrim.lib.ColorRenderer;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Iterator;
import java.util.LinkedList;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JTable;


/**
 * This class handles the table with overlay properties; this table lists
 * how many different tissue types have been computed and lets the user select
 * for specific tissue types.
 */

public class OverlayPropertiesTable extends JTable implements ActionListener, MouseListener
{


    /**
     * Defines the table of overlay properties which can be displayed in a 
     * DicomSliceCanvasPolydraw object.
     */
    public OverlayPropertiesTable( ) 
    {
        super( );
        
        setDefaultRenderer( Color.class,
                            new ColorRenderer( true ) );

        m_colorEditor = new ColorEditor( );
        m_colorEditor.addActionListener( this );
        
        setDefaultEditor( Color.class,
                          m_colorEditor );
        
        setModel( new OverlayPropertiesTableModel( ) );
        
        setAutoCreateColumnsFromModel( false );
        
        setColumnSelectionAllowed (false );
        setRowSelectionAllowed( true );
        
        addPopupMenu( );
        
        m_selectionListener = new OverlayPropertiesTableSelectionListener( this );
        
        this.getSelectionModel( ).addListSelectionListener( m_selectionListener );
        this.getColumnModel( ).getSelectionModel( ).addListSelectionListener( m_selectionListener );
        
        // listen to mouse events and react to them!
        this.addMouseListener( this );
        
    }


    /**
     *
     * Returns the last selected color.
     *
     * @return
     */
    public Color getLastSelectedColor( )
    {
        return m_lastSelectedColor;
    }


    /**
     * Does the rest of the object construction work.
     */
    private void addPopupMenu( )
    {
        m_popup = new JPopupMenu( );
        
        m_imageSegmentBooleanLogic = new JMenu( "Boolean Operations" );
        
        m_applyLogic_AND_toImageSegments = new JMenuItem( "AND" );
        m_applyLogic_AND_toImageSegments.addActionListener( this );
        m_applyLogic_AND_toImageSegments.setEnabled( false );
        
        m_applyLogic_OR_toImageSegments = new JMenuItem( "OR" );
        m_applyLogic_OR_toImageSegments.addActionListener( this );
        m_applyLogic_OR_toImageSegments.setEnabled( false );
        
        m_applyLogic_EXOR_toImageSegments = new JMenuItem( "EXOR" );
        m_applyLogic_EXOR_toImageSegments.addActionListener( this );
        m_applyLogic_EXOR_toImageSegments.setSelected( false );
        
        m_applyLogic_NOT_toImageSegments = new JMenuItem( "NOT" );
        m_applyLogic_NOT_toImageSegments.addActionListener( this );
        m_applyLogic_NOT_toImageSegments.setEnabled( false );
        
        
        m_imageSegmentBooleanLogic.add( m_applyLogic_AND_toImageSegments  );
        m_imageSegmentBooleanLogic.add( m_applyLogic_OR_toImageSegments  );
        m_imageSegmentBooleanLogic.add( m_applyLogic_EXOR_toImageSegments  );
        m_imageSegmentBooleanLogic.add( m_applyLogic_NOT_toImageSegments );
        
        m_popup.add( m_imageSegmentBooleanLogic );
        
        add( m_popup );
        
    }
    
    /**
     * Notifies all registered components interested in selection 
     * changes in the table.
     * @param eventType
     */
    public void notifyActionListeners( int eventType )
    {
        Iterator it = m_listeners.iterator( );
        while( it.hasNext( ) )
        {
            if ( eventType == OverlayPropertiesTableSelectionListener.ROW_SELECTION_CHANGED )
            {
                ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, OverlayPropertiesTableSelectionListener.ROW_SELECTION_CHANGED, "" ) );
            }
            if ( eventType == OverlayPropertiesTableSelectionListener.COLUMN_SELECTION_CHANGED )
            {
                ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, OverlayPropertiesTableSelectionListener.COLUMN_SELECTION_CHANGED, "" ) );
            }
        }
    }
    
    /**
     * Notifies all other registered components interested in selection 
     * changes in the table.
     * @param eventType
     */
    public void notifyOtherActionListeners( int eventType )
    {
        Iterator it = m_listeners.iterator( );
        while( it.hasNext( ) )
        {
            
           ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, eventType, "" ) );
           
        }
    }
    
    
    /**
     * Adds an action listener to the list of action listeners who are notified
     * when something happens with the spin box.
     * @param e
     */
    public void addActionListener( ActionListener e )
    {
	m_listeners.add( e );
    }
    
    
    /**
     * Is called whenever a registered component raises an ActionEvent.
     * @param e
     */
    @Override
    public void actionPerformed( ActionEvent e ) 
    {
        Object sender = e.getSource( );

        if ( e.getSource( ) instanceof ColorEditor )
        {

             if ( e.getID( ) == ColorEditor.COLOR_SELECTED )
             {
                 System.out.println( "Action event caught from ColorEditor" );
                 m_lastSelectedColor = (Color)m_colorEditor.getCellEditorValue( );
                 notifyOtherActionListeners( OVERLAYCOLOR_CHANGE_EVENT );
             }
        }


        if ( sender == m_applyLogic_AND_toImageSegments )
        {
            notifyOtherActionListeners( PERFORM_BOOLEAN_AND_OPERATION );
        }
        
        if ( sender == m_applyLogic_OR_toImageSegments )
        {
            notifyOtherActionListeners( PERFORM_BOOLEAN_OR_OPERATION );
        }
        
        if ( sender == m_applyLogic_EXOR_toImageSegments )
        {
            notifyOtherActionListeners( PERFORM_BOOLEAN_EXOR_OPERATION );
        }
        
        if ( sender == m_applyLogic_NOT_toImageSegments )
        {
            notifyOtherActionListeners( PERFORM_BOOLEAN_NOT_OPERATION );
        }
        
    }

    /**
     * Is called whenever a registered component raises mouseClicked event.
     * @param e
     */
    @Override
    public void mouseClicked( MouseEvent e ) 
    {
  
    }

    /**
     * The user pressed the table, this may be a change in the selected 
     * row; sent a notification to the listeners to this table.
     * @param e
     */
    @Override
    public void mousePressed( MouseEvent e ) 
    {
        if ( e.isPopupTrigger( ) )
        {
            m_popup.show( e.getComponent( ), e.getX( ), e.getY( ) );
        }
        
        notifySelectionChange( );
    }

    @Override
    public void mouseReleased( MouseEvent e ) 
    {
        if ( e.isPopupTrigger( ) )
        {
            m_popup.show( e.getComponent( ), e.getX( ), e.getY( ) );
        }
        
        notifySelectionChange( );
    }

    @Override
    public void mouseEntered( MouseEvent e ) 
    {
        // not implemented
    }

    @Override
    public void mouseExited( MouseEvent e ) 
    {
        // not implemented
    }

    /**
     * Notifies listeners that a change in table selection was performed.
     */
    private void notifySelectionChange( ) 
    {
         notifyActionListeners( TABLE_SELECTED_EVENT );
         
         if ( ( getSelectedRows( ).length == 0 ) || ( getSelectedRows( ).length > 2 ) )
         {
             m_applyLogic_AND_toImageSegments.setEnabled( false );
             m_applyLogic_OR_toImageSegments.setEnabled( false );
             m_applyLogic_EXOR_toImageSegments.setEnabled( false );
             m_applyLogic_NOT_toImageSegments.setEnabled( false );
             
         }
         
         if ( getSelectedRows( ).length == 1 )
         {
             m_applyLogic_AND_toImageSegments.setEnabled( false );
             m_applyLogic_OR_toImageSegments.setEnabled( false );
             m_applyLogic_EXOR_toImageSegments.setEnabled( false );
             m_applyLogic_NOT_toImageSegments.setEnabled( true );
             
         }
         
         if ( getSelectedRows( ).length == 2 )
         {
             m_applyLogic_AND_toImageSegments.setEnabled( true );
             m_applyLogic_OR_toImageSegments.setEnabled( true );
             m_applyLogic_EXOR_toImageSegments.setEnabled( true );
             m_applyLogic_NOT_toImageSegments.setEnabled( false );
             
         }
         
    }
    
    // Attributes of the class
    
    private static final long serialVersionUID = 99822446327598273L;
    
    private final OverlayPropertiesTableSelectionListener m_selectionListener;
    
    // Listeners to action events in this class
    private final LinkedList<ActionListener> m_listeners = new LinkedList<>( );
    
    public final static int TABLE_SELECTED_EVENT = 0;
    public final static int OVERLAYCOLOR_CHANGE_EVENT = 1;

    private JPopupMenu m_popup;
    
    private JMenu m_imageSegmentBooleanLogic;
    
    private JMenuItem m_applyLogic_AND_toImageSegments;
    private JMenuItem m_applyLogic_OR_toImageSegments;
    private JMenuItem m_applyLogic_EXOR_toImageSegments;
    private JMenuItem m_applyLogic_NOT_toImageSegments;
    
    // possible action events raised by this
    public final static int PERFORM_BOOLEAN_AND_OPERATION = 10;
    public final static int PERFORM_BOOLEAN_OR_OPERATION = 11;
    public final static int PERFORM_BOOLEAN_EXOR_OPERATION = 12;
    public final static int PERFORM_BOOLEAN_NOT_OPERATION = 13;

    private final ColorEditor m_colorEditor;
    private Color m_lastSelectedColor;
}
