/*
 * File: $Id: RegionPropertiesTable.java,v 1.1.1.1 2006/04/19 09:38:48 hans Exp $
 * History
 * Date: 28.09.2004  Authors: Hans Slotboom  
 *
 * Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

/*
 * RegionPropertiesTable.java
 *
 * Class that displays and handles events of a region properties table.
 *
 * Created on September 28, 2004, 7:27 AM
 */

package org.spectrim.imageprocessing;

import java.util.Vector;
import java.util.LinkedList;
import java.util.Collections;
import java.util.Iterator;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;
import java.awt.event.KeyListener;
import java.awt.event.KeyEvent;


import javax.swing.JTable;
import javax.swing.table.DefaultTableModel;
import javax.swing.JFrame;

import javax.swing.JPopupMenu;
import javax.swing.JMenuItem;

import org.spectrim.lib.ColumnSorter;
import org.spectrim.lib.EnterStringFrame;
import org.spectrim.dicom.DicomSliceCanvasPolyDraw;
import org.spectrim.imagegraphicobjects.RegioPolygon;

/**
 * Table giving access to region properties. 
 * @author  hans
 */

public class RegionPropertiesTable extends JTable implements ActionListener, MouseListener, KeyListener
{
    
    /** 
     * Creates a new instance of RegionPropertiesTable. 
     * @param parent
     * @param canvas
     * @param persistence
     */
    public RegionPropertiesTable( JFrame parent, 
                                  DicomSliceCanvasPolyDraw canvas, 
                                  boolean persistence )
    {
        super( );
        
        m_parent = parent;
        m_persistence = persistence;
        
        m_dicomSliceCanvas = canvas;
        
        m_tableModel = new DefaultTableModel( );
        
        m_tableModel.addColumn( "<HTML><B>Region no.</B></HTML>" );
        m_tableModel.addColumn( "<HTML><B>Image no.</B></HTML>" );
        m_tableModel.addColumn( "<HTML><B>ROI name</B></HTML>" );
        m_tableModel.addColumn( "<HTML><B>Pixels</B></HTML>" );
        m_tableModel.addColumn( "<HTML><B>Volume</B></HTML>" );
        m_tableModel.addColumn( "<HTML><B>Average</B></HTML>" );
        m_tableModel.addColumn( "<HTML><B>St. dev.</B></HTML>" );
        
        if ( persistence == true )
        {
            m_tableModel.addColumn( "<HTML><B>Persistent</B></HTML>" );
        }
        
        setModel( m_tableModel );
        
        // define default behavior of the JTable
        setAutoCreateColumnsFromModel( false );
        setColumnSelectionAllowed (false );
        setRowSelectionAllowed( true );

        constructObject( );
        
        addKeyListener( this );
        
    }
     
    
    /**
     * Does the rest of the object construction work.
     */
    private void constructObject( )
    {
        m_popup = new JPopupMenu( );
        
        m_deleteSelectedROIs = new JMenuItem( "Delete selected items" );
        m_deleteSelectedROIs.addActionListener( this );
        
        m_sortByROISize = new JMenuItem( "Sort by ROI size" );
        m_sortByROISize.addActionListener( this );
        
        m_sortByROIName = new JMenuItem( "Sort by ROI name" );
        m_sortByROIName.addActionListener( this );

        m_sortByAverageROIPixelValue = new JMenuItem( "Sort by average ROI pixel value" );
        m_sortByAverageROIPixelValue.addActionListener( this );
        
        m_sortByDeviationROIPixelValue = new JMenuItem( "Sort by deviation ROI pixel value" );
        m_sortByDeviationROIPixelValue.addActionListener( this );
        
        m_setPolygonsPersistentTag = new JMenuItem( "Set selected polygons persistent" );
        m_setPolygonsPersistentTag.addActionListener( this );
        
        m_unsetPolygonsPersistentTag = new JMenuItem( "Set selected polygons non-persistent" );
        m_unsetPolygonsPersistentTag.addActionListener( this );
        
        m_renameSelectedPolygons = new JMenuItem( "Rename selected polygons ..." );
        m_renameSelectedPolygons.addActionListener( this );
        
        m_popup.add( m_deleteSelectedROIs );
        
        m_popup.addSeparator( );
        
        m_popup.add( m_sortByROIName );
        m_popup.add( m_sortByROISize );
        m_popup.add( m_sortByAverageROIPixelValue );
        m_popup.add( m_sortByDeviationROIPixelValue );
        
        m_popup.addSeparator( );
        
        if ( m_persistence == true )
        {
            m_popup.add( m_setPolygonsPersistentTag );
            m_popup.add( m_unsetPolygonsPersistentTag );
        }
        
        m_popup.add( m_renameSelectedPolygons );
        
        add( m_popup );
        
        addMouseListener( this );
    }
    
    
    /**
     * Is called whenever a registered component raises an ActionEvent.
     * @param e
     */
    @Override
    public void actionPerformed( ActionEvent e ) 
    {
        Object sender = e.getSource( );
        if ( sender == m_sortByROISize )
        {
            sortAllRowsBy( (DefaultTableModel)this.getModel( ), 3, true );
        }
        if ( sender == m_sortByROIName )
        {
            sortAllRowsBy( (DefaultTableModel)this.getModel( ), 2, true );
        }
        
        if ( sender == m_sortByAverageROIPixelValue )
        {
            sortAllRowsBy( (DefaultTableModel)this.getModel( ), 5, true );
        }
        
        if ( sender == m_sortByDeviationROIPixelValue )
        {
            sortAllRowsBy( (DefaultTableModel)this.getModel( ), 6, true );
        }
        
        if ( sender == m_setPolygonsPersistentTag )
        {
            setPersistentPolygons( );
        }
        
        if ( sender == m_unsetPolygonsPersistentTag )
        {
            unsetPersistentPolygons( );
        }
        
        if ( sender == m_renameSelectedPolygons )
        {
            renameSelectedPolygons( );
        }
        
        if ( sender == m_strEnterFrame )
        {
            if ( e.getID( ) == EnterStringFrame.APPLY_EVENT )
            {
                String str = m_strEnterFrame.getText( );
                applySelectedROIRenaming( str );
            }
        }
        
        if ( sender == m_deleteSelectedROIs )
        {
            deleteSelectedPolygons( );
        }
        
    }    
    
    /**
     * Is called whenever a mouse button is pressed
     * @param e
     */
    @Override
    public void mousePressed( MouseEvent e ) 
    {
        if ( e.isPopupTrigger( ) )
        {
            m_popup.show( e.getComponent( ), e.getX( ), e.getY( ) );
        }
        
        setSelectedROIItems( );
    }
    
    /**
     * Is called whenever a mouse button is clicked
     */
    @Override
    public void mouseClicked( MouseEvent e ) 
    {
    }
    
    /**
     * Is called whenever a mouse pointer enters.
     * @param e
     */
    @Override
    public void mouseEntered( MouseEvent e ) 
    {
    }
    
    /**
     * Is called whenever the mouse pointer exits.
     * @param e
     */
    @Override
    public void mouseExited( MouseEvent e ) 
    {
    }
    
    /**
     * Is called whenever a mouse button is released.
     * @param e
     */
    @Override
    public void mouseReleased( MouseEvent e ) 
    {
        if ( e.isPopupTrigger( ) )
        {
            m_popup.show( e.getComponent( ), e.getX( ), e.getY( ) );
        }
    }
    
    /** Regardless of sort order (ascending or descending), null values 
     * always appear last colIndex specifies a column in model.
     * @param model
     * @param colIndex
     * @param ascending
     */ 
    public void sortAllRowsBy( DefaultTableModel model, int colIndex, boolean ascending ) 
    {
        Vector data = model.getDataVector( );
        Collections.sort( data, new ColumnSorter( colIndex, ascending ) );
        model.fireTableStructureChanged( );
    }
    
    
    /**
     * This method updates the ROI list with current information.
     * @param minThresshold
     * @param maxThresshold
     */
    public void updateROIList( int minThresshold, int maxThresshold )
    {   
        int totalPixelNumber = 0;
        float totalSegmentVolume = 0f;
        // first clear the table
        int tableSize =  m_tableModel.getRowCount( );
        
        for ( int i = tableSize - 1 ; i >= 0; i-- )
        {
            m_tableModel.removeRow( i );
        }
        
        int cntr = 0;
        
        // fill the table with new information
        for ( int i = 0; i < m_dicomSliceCanvas.getNumberOfImages( ); i++ )
        {
            int  numberOfPolygons = m_dicomSliceCanvas.getNumberOfPolygons( i );
            for ( int j = 0 ; j < numberOfPolygons; j++ )
            {
                ImageParametersWithinARegioPolygon pars = m_dicomSliceCanvas.getPolygonImageInformation( i, j );

                int innerBorders = 0;
                int outerBorders = 0;
            
                int contourType = determineROISign( pars.getMeanPixelValue( ), minThresshold, maxThresshold );
            
                if ( contourType == 1 )
                {
                
                    outerBorders++;
                    if ( this.m_persistence == false )
                    {
                        m_tableModel.addRow( new Object[]{ new Integer( cntr++ ),
                                                           new Integer( i ),
                                                           pars.getROIName( ), 
                                                           new Integer( pars.getNumberOfPixels( ) ),
                                                           new Float( pars.getROIVolume( ) ), 
                                                           new Float( pars.getMeanPixelValue( ) ),
                                                           new Float( pars.getStandardDeviationOfPixelValues( ) ),
                                                        }
                                            );
                    }
                    else
                    {
                        m_tableModel.addRow( new Object[]{ new Integer( cntr++ ),
                                                           new Integer( i ),
                                                           pars.getROIName( ), 
                                                           new Integer( pars.getNumberOfPixels( ) ),
                                                           new Float( pars.getROIVolume( ) ), 
                                                           new Float( pars.getMeanPixelValue( ) ),
                                                           new Float( pars.getStandardDeviationOfPixelValues( ) ),
                                                           "no"
                                                         }
                                            );
                    }
                }
                else
                {
                
                    innerBorders++;
                    if ( m_persistence == false )
                    {
                        m_tableModel.addRow( new Object[]{ new Integer( cntr++ ),
                                                           new Integer( i ),
                                                           pars.getROIName( ),
                                                           new Integer( pars.getNumberOfPixels( ) ),
                                                           "" + "<HTML><FONT COLOR = RED>" + new Float( -1.0f * pars.getROIVolume( ) )+ "</FONT></HTML>",
                                                           new Float( pars.getMeanPixelValue( ) ),
                                                           new Float( pars.getStandardDeviationOfPixelValues( ) )
                                                         }
                                            );
                    }
                    else
                    {
                        m_tableModel.addRow( new Object[]{ new Integer( cntr++ ),
                                                           new Integer( i ),
                                                           pars.getROIName( ),
                                                           new Integer( pars.getNumberOfPixels( ) ),
                                                           "" + "<HTML><FONT COLOR = RED>" + new Float( -1.0f * pars.getROIVolume( ) )+ "</FONT></HTML>",
                                                           new Float( pars.getMeanPixelValue( ) ),
                                                           new Float( pars.getStandardDeviationOfPixelValues( ) ),
                                                           "no"
                                                        }
                                            );
                    }
                }
            
                if ( contourType == 1 )
                {
                    totalPixelNumber += pars.getNumberOfPixels( );
                    totalSegmentVolume += pars.getROIVolume( );
                }
            
                if ( contourType == -1 )
                {
                    totalPixelNumber -= pars.getNumberOfPixels( );
                    totalSegmentVolume -= pars.getROIVolume( );
                }
            }
        }
    }
       
    /**
     * Due to the fact that the segmentation algorithm may find and ROI
     * within an ROI, i.e. the case when a pixel collection is again larger
     * or smaller than the segmentation thresholds. In other words, it may
     * find inner or outer borders. In order to add pixel values correctly 
     * together for e.g. averaging, or count volume segments together one needs
     * an algorithm that finds out whether the contour was an inner or an outer
     * border.
     */
    private int determineROISign( float value, int minThresshold, int maxThresshold )
    {
        if ( ( value <= maxThresshold ) && ( value >= minThresshold ) )
        {
            return 1;
        }
        else
        {
            return -1;
        }
    }
    
    /**
     * Sets the selected ROI items in the list, also selected (blue) in the 
     * SliceCanvasPolyDraw.
     */
    private void setSelectedROIItems( )
    {
        int[] selectedRows = this.getSelectedRows( );
        String[] selectedROINames = new String[ selectedRows.length ];
        
        for ( int i = 0; i < selectedRows.length; i++ )
        {
            selectedROINames[i] = (String)getValueAt( selectedRows[i], 2 );
        }
        
        m_dicomSliceCanvas.selectPolygons( selectedROINames );
    }
    
    /**
     * Launches the rename frame.
     */
    private void renameSelectedPolygons( )
    {
        m_strEnterFrame = new EnterStringFrame( m_parent, "New regio name", "newName", 
                                                          "Enter new name for selected ROI's" );
        m_strEnterFrame.addActionListener( this );
    }
    
    /**
     * Applies the renaming of the roi's.
     */
    private void applySelectedROIRenaming( String newName )
    {
        int[] selectedRows = this.getSelectedRows( );
        String[] selectedROINames = new String[ selectedRows.length ];
        
        String cntrString;
        
        for ( int i = 0; i < selectedRows.length; i++ )
        {
            selectedROINames[i] = (String)getValueAt( selectedRows[i], 2 );
        }
        
        for ( int i = 0; i < selectedRows.length; i++ )
        {
            if ( i < 10 )
            {
                cntrString = "0" + i;
            }
            else
            {
                cntrString = "" + i;
            }
            
            m_dicomSliceCanvas.getPolygon( selectedROINames[i] ).setLabel( newName + cntrString );
            m_tableModel.setValueAt( newName + cntrString, selectedRows[i], 2 );
            
        }
        
        m_dicomSliceCanvas.update( );
    }
    
    
    /**
     * Set the persistency tag of currently displayed regio polygons to 
     * yes, ie they are persistent.
     */
    private void setPersistentPolygons( )
    {
        int[] selectedRows = this.getSelectedRows( );
        String[] selectedROINames = new String[ selectedRows.length ];
        
        for ( int i = 0; i < selectedRows.length; i++ )
        {
            selectedROINames[i] = (String)getValueAt( selectedRows[i], 2 );
        }
        
        for ( int i = 0; i < selectedRows.length; i++ )
        {
            m_persistentROIList.addLast( m_dicomSliceCanvas.getPolygon( selectedROINames[i] ) );
            m_tableModel.setValueAt( "yes", selectedRows[i], 7 );
        }
    }
    
    /**
     * Set the persistency tag of currently displayed regio polygons to 
     * no, i.e. they are persistent
     */
    private void unsetPersistentPolygons( )
    {
        int[] selectedRows = this.getSelectedRows( );
        String[] selectedROINames = new String[ selectedRows.length ];
        
        // list must be determined new
        m_persistentROIList.clear( );
        
        for ( int i = 0; i < selectedRows.length; i++ )
        {
            selectedROINames[i] = (String)getValueAt( selectedRows[i], 2 );
        }
        
        for ( int i = 0; i < selectedRows.length; i++ )
        {
            m_tableModel.setValueAt( "no", selectedRows[i], 7 );
        }
        
        // Build up the list opf remaining persistent regions
        int tableSize =  m_tableModel.getRowCount( );
        
        for ( int i = tableSize - 1 ; i >= 0; i-- )
        {
            if ( ( (String)m_tableModel.getValueAt( i, 7 ) ).compareTo( "yes" ) == 0 )
            {
                m_persistentROIList.addLast( m_dicomSliceCanvas.getPolygon( (String)m_tableModel.getValueAt( i, 1 ) ) );
            }
        }
    }

    /**
     * Deletes the currently selected polygons from the list.
     */
    public void deleteSelectedPolygons( )
    {
        int[] selectedRows = this.getSelectedRows( );
        
        String[] selectedROINames = new String[ selectedRows.length ];
        
        for ( int i = 0; i < selectedRows.length; i++ )
        {
            m_ROIsToDeleteNameList.addLast( (String)getValueAt( selectedRows[i], 2 ) );
        }
        
        for ( int i = selectedRows.length -1 ; i > -1 ; i-- )
        {
            String regionToDelete = (String) m_tableModel.getValueAt( i, 2 );
            // check deletion of only one item if in the list items are removed
            
            m_tableModel.removeRow( selectedRows[i] );
            notifyActionListeners( DELETE_EVENT,  regionToDelete );
           
            
        }
        
        m_tableModel.fireTableStructureChanged( );
        
    }

    /**
     * Notify all actionListeners.
     */
    private void notifyActionListeners( int eventType, String regionName )
    {
        Iterator it = m_listeners.iterator( );
        while( it.hasNext( ) )
        {
            ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, eventType, regionName ) );
        }
    }
    
    /**
     * Is called whenever a key is pressed.
     * @param e
     */
    @Override
    public void keyPressed( KeyEvent e ) 
    {
    }
    
    /**
     * Is called whenever a key is released.
     * @param e
     */
    @Override
    public void keyReleased( KeyEvent e ) 
    {
        
        if ( ( e.getKeyCode( ) == KeyEvent.VK_PAGE_UP ) || ( e.getKeyCode( ) == KeyEvent.VK_PAGE_DOWN ) ||
             ( e.getKeyCode( ) == KeyEvent.VK_DOWN ) || ( e.getKeyCode( ) == KeyEvent.VK_UP ) )
        {
            setSelectedROIItems( );
        }
    }
    
    /**
     * Is called whenever a key is typed.
     * @param e
     */
    @Override
    public void keyTyped( KeyEvent e ) 
    {
    }
    
    /**
     * Returns the linked list to the caller.
     * @return 
     */
    public LinkedList getPersistentPolygons( )
    {
        return m_persistentROIList;
    }
    
    /**
     * Returns the list of ROIs that needs to be delete from the list,
     * and from the DicomSliceCanvaPolyDraw .
     * @return 
     */
    public LinkedList getROIsToDeleteNameList( )
    {
        return m_ROIsToDeleteNameList;
    }
    
    /**
     * Adds an action listener to the list of action listeners who are notified
     * when something happens with the spinbox.
     * @param e
     */
    public void addActionListener( ActionListener e )
    {
	m_listeners.add( e );
    }
    
    /**
     * Sets the polygon information of all the polygons in the 
     * DicomSliceCanvasPolydraw object.
     */
    public void setPolygonInformation( )
    {
        updateROIList( m_dicomSliceCanvas.getMinRawDataValue( ), 
                       m_dicomSliceCanvas.getMaxRawDataValue( ) );
    }
    
    
    // Private attributes
    
    private JPopupMenu m_popup;
    private JMenuItem m_deleteSelectedROIs;
    private JMenuItem m_sortByROISize;
    private JMenuItem m_sortByROIName;
    private JMenuItem m_sortByAverageROIPixelValue;
    private JMenuItem m_sortByDeviationROIPixelValue;
    private JMenuItem m_setPolygonsPersistentTag;
    private JMenuItem m_unsetPolygonsPersistentTag;
    private JMenuItem m_renameSelectedPolygons;
    
    private final DefaultTableModel m_tableModel;
    
    private boolean m_persistence = true;
    
    private final DicomSliceCanvasPolyDraw m_dicomSliceCanvas;
    
    private final LinkedList<RegioPolygon> m_persistentROIList = new LinkedList<>( );
    
    private final JFrame m_parent;
    private EnterStringFrame m_strEnterFrame;
    
    public final  static int DELETE_EVENT = 0;
    private final LinkedList<String> m_ROIsToDeleteNameList = new LinkedList<>( );
    
    // Listeners to changes in the segmentation gui control
    private final LinkedList<ActionListener> m_listeners = new LinkedList<>( );
    
    // Must be declarated somehow
    private static final long serialVersionUID = 3487493391119393L;
}