/*
 * File: $Id: SegmentationGUIControl.java,v 1.1.1.1 2006/04/19 09:38:48 hans Exp $
 * History
 * Date: 14. February 2003, 14:36 Hans Slotboom  
 *
 * Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

 
package org.spectrim.imageprocessing;

/**
 * This GUI control allows for interactive segmentation of images of the
 * DicomSliceCanvasPolydraw. 
 * @author  slotboom
 */

import java.awt.Dimension;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JLabel;
import javax.swing.BoxLayout;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JRadioButton;
import javax.swing.ButtonGroup;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;

import java.text.DecimalFormat;

import java.util.LinkedList;
import java.util.Iterator;

import org.spectrim.dicom.DicomSliceCanvasPolyDraw;

import org.spectrim.lib.FormattedTextField;



public class SegmentationGUIControl extends JPanel implements ActionListener, ChangeListener
{
    
    /** 
     * Creates a new instance of SegmentationGUIControl. 
     * @param title
     * @param defaultMinimum
     * @param defaultMaximum
     */
    public SegmentationGUIControl( String title, 
                                   int defaultMinimum, int defaultMaximum )
    {
        
        m_minPixelValue = defaultMinimum;
        m_maxPixelValue = defaultMaximum;
        
        setLayout( new BoxLayout( this, BoxLayout.X_AXIS ) );
        //setBorder( BorderFactory.createTitledBorder( title ) );
        //setPreferredSize( new Dimension( 524, 240 ) );
        //setMinimumSize( new Dimension( 524, 240 ) );
        //setMaximumSize( new Dimension( 524, 240 ) );
        
        m_segmentationCollectionPanelLeft = new JPanel( );
        m_segmentationCollectionPanelLeft.setLayout( new BoxLayout( m_segmentationCollectionPanelLeft, BoxLayout.Y_AXIS ) );
        m_segmentationCollectionPanelLeft.setBorder( BorderFactory.createTitledBorder( title ) );
        m_segmentationCollectionPanelLeft.setPreferredSize( new Dimension( 524, 150 ) );
        m_segmentationCollectionPanelLeft.setMinimumSize( new Dimension( 524, 150 ) );
        m_segmentationCollectionPanelLeft.setMaximumSize( new Dimension( 524, 150 ) );
        
        m_minSliderPanel = new JPanel( );
        m_minSliderPanel.setPreferredSize( new Dimension( 512, 55 ) );
        m_minSliderPanel.setMaximumSize( new Dimension( 512, 55 ) );
        m_minSliderPanel.setMinimumSize( new Dimension( 512, 55 ) );
        m_minSliderPanel.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );
        
        m_minSliderPanel.setLayout( new BoxLayout( m_minSliderPanel, BoxLayout.X_AXIS ) );
        
        m_minPixelValueIndicator = new JLabel( "" + m_minPixelValue );
        
        m_minPixelValueIndicator.setPreferredSize( new Dimension( 40, 12 ) );
        m_minPixelValueIndicator.setMaximumSize( new Dimension( 40, 12 ) );
        m_minPixelValueIndicator.setMinimumSize( new Dimension( 40, 12 ) );
        
        m_minSliderPanel.add( m_minPixelValueIndicator );
        
        m_minimumPixelValueSlider = new JSlider( JSlider.HORIZONTAL, 0, 1000, 0 );
	m_minimumPixelValueSlider.setMajorTickSpacing( 250 );
        m_minimumPixelValueSlider.setMinorTickSpacing( 50 );
        m_minimumPixelValueSlider.setPaintTicks( true );
        m_minimumPixelValueSlider.setPaintLabels( true );
        m_minimumPixelValueSlider.setValue( defaultMinimum );
        m_minimumPixelValueSlider.addChangeListener( this );
        
        m_minimumPixelValueSlider.setPreferredSize( new Dimension( 390, 50 ) );
        m_minimumPixelValueSlider.setMinimumSize( new Dimension( 390, 50 ) );
        m_minimumPixelValueSlider.setMaximumSize( new Dimension( 390, 50 ) );
        
        m_minPixelLabel = new JLabel( "Minimum [%]" );
        m_minSliderPanel.add( m_minimumPixelValueSlider );
        m_minSliderPanel.add( m_minPixelLabel );
        
        m_maxSliderPanel = new JPanel( );
        m_maxSliderPanel.setLayout( new BoxLayout( m_maxSliderPanel, BoxLayout.X_AXIS ) );
        m_maxSliderPanel.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );
        m_maxPixelValueIndicator = new JLabel( "" + m_maxPixelValue );
        
        m_maxPixelValueIndicator.setPreferredSize( new Dimension( 40, 12 ) );
        m_maxPixelValueIndicator.setMaximumSize( new Dimension( 40, 12 ) );
        m_maxPixelValueIndicator.setMinimumSize( new Dimension( 40, 12 ) );
        
        m_maxSliderPanel.add( m_maxPixelValueIndicator );

        m_maxSliderPanel.setPreferredSize( new Dimension( 512, 55 ) );
        m_maxSliderPanel.setMaximumSize( new Dimension( 512, 55 ) );
        m_maxSliderPanel.setMinimumSize( new Dimension( 512, 55 ) );
        
        m_maximumPixelValueSlider = new JSlider( JSlider.HORIZONTAL, 0, 1000, 1000 );
	m_maximumPixelValueSlider.setMajorTickSpacing( 250 );
        m_maximumPixelValueSlider.setMinorTickSpacing( 50 );
        m_maximumPixelValueSlider.setPaintTicks( true );
        m_maximumPixelValueSlider.setPaintLabels( true );
        m_maximumPixelValueSlider.setValue( defaultMaximum );
        m_maximumPixelValueSlider.addChangeListener( this );
        
        m_maximumPixelValueSlider.setPreferredSize( new Dimension( 390, 50 ) );
        m_maximumPixelValueSlider.setMinimumSize( new Dimension( 390, 50 ) );
        m_maximumPixelValueSlider.setMaximumSize( new Dimension( 390, 50 ) );
        
        m_maxPixelLabel = new JLabel( "Maximum [%]" );
        m_maxSliderPanel.add( m_maximumPixelValueSlider );
        m_maxSliderPanel.add( m_maxPixelLabel );
        
        m_segmentationCollectionPanelLeft.add( m_minSliderPanel );
        m_segmentationCollectionPanelLeft.add( m_maxSliderPanel );
         
        m_segmentationCollectionPanelRight = new JPanel( );
        m_segmentationCollectionPanelRight.setLayout( new BoxLayout( m_segmentationCollectionPanelRight, BoxLayout.Y_AXIS ) );
        m_segmentationCollectionPanelRight.setBorder( BorderFactory.createTitledBorder( title ) );
        m_segmentationCollectionPanelRight.setPreferredSize( new Dimension( 500, 150 ) );
        m_segmentationCollectionPanelRight.setMinimumSize( new Dimension( 500, 150 ) );
        m_segmentationCollectionPanelRight.setMaximumSize( new Dimension( 500, 150 ) );
        
        m_roiMinMaxSizePanel = new JPanel( );
        
        m_roiMinMaxSizePanel.setLayout( new BoxLayout( m_roiMinMaxSizePanel, BoxLayout.X_AXIS ) );
        m_roiMinMaxSizePanel.setBorder( BorderFactory.createEmptyBorder( 5, 12, 5, 5 ) );
        
        m_roiMinMaxSizePanel.add( Box.createHorizontalGlue( ) );
        m_noROISizeLimits = new JRadioButton( "No limited ROI size" );
        m_noROISizeLimits.addActionListener( this );
        
        
        m_roiMinMaxSizePanel.add( m_noROISizeLimits );
        
        m_ROISizeLimits = new JRadioButton( "Limited ROI size" );
        m_ROISizeLimits.addActionListener( this );
        m_ROISizeLimits.setSelected( true );
        
        m_roiMinMaxSizePanel.add( Box.createHorizontalGlue( ) );
        m_roiMinMaxSizePanel.add( m_ROISizeLimits );
        
        m_ROISizeButtonGroup = new ButtonGroup( );
        
        m_ROISizeButtonGroup.add( m_noROISizeLimits );
        m_ROISizeButtonGroup.add( m_ROISizeLimits );
        
        m_minimumROISizeLabel = new JLabel( "Minimum " );
        m_minimumROISize = new FormattedTextField( new DecimalFormat( "0" ), 1.0, 512.0 * 512.0 );
        m_minimumROISize.setValue( 25 );
        m_minimumROISize.setPreferredSize( new Dimension( 50 , 25 ) );
        m_minimumROISize.setMinimumSize( new Dimension( 50 , 25 ) );
        m_minimumROISize.setMaximumSize( new Dimension( 50 , 25 ) );
        
        m_maximumROISizeLabel = new JLabel( "Maximum " );
        m_maximumROISize = new FormattedTextField( new DecimalFormat( "0" ), 1.0 , 256f * 256f );
        m_maximumROISize.setValue( 40000 );
        m_maximumROISize.setPreferredSize( new Dimension( 50 , 25 ) );
        m_maximumROISize.setMinimumSize( new Dimension( 50 , 25 ) );
        m_maximumROISize.setMaximumSize( new Dimension( 50 , 25 ) );
        
        m_roiMinMaxSizePanel.add( Box.createHorizontalGlue( ) );
        m_roiMinMaxSizePanel.add( m_minimumROISizeLabel );
        m_roiMinMaxSizePanel.add( m_minimumROISize );
        m_roiMinMaxSizePanel.add( Box.createHorizontalGlue( ) );
        m_roiMinMaxSizePanel.add( m_maximumROISizeLabel );
        m_roiMinMaxSizePanel.add( m_maximumROISize);
        m_roiMinMaxSizePanel.add( Box.createHorizontalGlue( ) );
        
        m_maximumROISizeLabel.setEnabled( true );
        m_minimumROISizeLabel.setEnabled( true );
        m_maximumROISize.setEnabled( true );
        m_minimumROISize.setEnabled( true );
        
        m_segmentationCollectionPanelRight.add( m_roiMinMaxSizePanel );
        
        m_limitedNumberOfVerticesPanel = new JPanel( );
        m_limitedNumberOfVerticesPanel.setLayout( new BoxLayout( m_limitedNumberOfVerticesPanel, BoxLayout.X_AXIS ) );
        m_limitedNumberOfVerticesPanel.setBorder( BorderFactory.createEmptyBorder( 5, 12, 5, 5 ) );
        
        m_limitNumberOfVerticesButtonGroup = new ButtonGroup( );
        
        m_NotLimitNumberOfVertices =  new JRadioButton( "Not limit number of vertices" );
        m_NotLimitNumberOfVertices.addActionListener( this );
        m_NotLimitNumberOfVertices.setSelected( true );
        
        m_LimitNumberOfVertices = new JRadioButton( "Limit number of vertices" );
        m_LimitNumberOfVertices.addActionListener( this );
        
        m_limitNumberOfVerticesButtonGroup.add( m_NotLimitNumberOfVertices );
        m_limitNumberOfVerticesButtonGroup.add( m_LimitNumberOfVertices );
        
        m_maximumNumberOfVertices = new FormattedTextField( "0", 3, 100000 );
        m_maximumNumberOfVertices.setValue( 1000. );
        m_maximumNumberOfVertices.setPreferredSize( new Dimension( 125 , 25 ) );
        m_maximumNumberOfVertices.setMinimumSize( new Dimension( 125 , 25 ) );
        m_maximumNumberOfVertices.setMaximumSize( new Dimension( 125 , 25 ) );
        m_maximumNumberOfVertices.setEnabled( false );
        
        m_limitedNumberOfVerticesPanel.add( Box.createHorizontalGlue( ) );
        m_limitedNumberOfVerticesPanel.add( m_NotLimitNumberOfVertices );
        m_limitedNumberOfVerticesPanel.add( Box.createHorizontalGlue( ) );
        m_limitedNumberOfVerticesPanel.add( m_LimitNumberOfVertices );
        m_limitedNumberOfVerticesPanel.add( Box.createHorizontalGlue( ) );
        m_limitedNumberOfVerticesPanel.add( m_maximumNumberOfVertices );
        m_limitedNumberOfVerticesPanel.add( Box.createHorizontalGlue( ) );
        
        m_segmentationCollectionPanelRight.add( m_limitedNumberOfVerticesPanel );
        
        
        m_autoApplyPanel = new JPanel( );
        m_autoApplyPanel.setLayout( new BoxLayout( m_autoApplyPanel, BoxLayout.X_AXIS ) );
        m_autoApplyPanel.setBorder( BorderFactory.createEmptyBorder( 5, 12, 5, 12 ) );
        
        m_autoApplyButton = new JRadioButton( "Auto apply changes ");
        m_autoApplyButton.setSelected( true );
        m_autoApplyButton.addActionListener( this );
        m_notAutoApplyButton = new JRadioButton( "Do not auto apply changes " ) ;
        m_notAutoApplyButton.addActionListener( this );
        
        m_applyButton = new JButton( "Apply !" );
        m_applyButton.setEnabled( false );
        m_applyButton.addActionListener( this );
        
        m_autoApplyButtonGroup = new ButtonGroup( );
        
        m_autoApplyButtonGroup.add( m_autoApplyButton );
        m_autoApplyButtonGroup.add( m_notAutoApplyButton );
         
        m_autoApplyPanel.add( m_autoApplyButton );
        m_autoApplyPanel.add( Box.createHorizontalGlue( ) );
        m_autoApplyPanel.add( m_notAutoApplyButton );
        m_autoApplyPanel.add( Box.createHorizontalGlue( ) );
        m_autoApplyPanel.add( m_applyButton );
        
        m_segmentationCollectionPanelRight.add( m_autoApplyPanel );
        
        add( m_segmentationCollectionPanelLeft );
        add( m_segmentationCollectionPanelRight );
    }
    
    /** 
     * Returns a reference to the  DicomSliceCanvasPolyDraw object.
     * @return 
     */
    public DicomSliceCanvasPolyDraw getCanvas( )
    {
        return m_sliceCanvas;
    }
    
    /**
     * Creates a panel allowing the user to set the ROI state of the 
     * DicomSliceCanvasPolyDraw control.
     */
    private void createROIModePanel( )
    {
        m_canvesROIModePanel = new JPanel( );
        m_canvesROIModePanel.setLayout( new BoxLayout( m_canvesROIModePanel, BoxLayout.X_AXIS ) );
        m_canvesROIModePanel.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );
        
        m_addROIButton = new JButton( "Add ROI" );
        m_deleteROIButton = new JButton( "Delete ROI" );
        m_cutROIButton = new JButton( "Cut ROI" ); 
        m_resizeROIButton = new JButton( "Resize ROI" );
        m_moveROIButton = new JButton( "Move ROI" );
        m_selectROIButton = new JButton( "Select ROI" );
        
        m_canvesROIModePanel.add( m_addROIButton );
        m_canvesROIModePanel.add( Box.createHorizontalGlue( ) );
        m_canvesROIModePanel.add( m_deleteROIButton );
        m_canvesROIModePanel.add( m_cutROIButton );
        m_canvesROIModePanel.add( m_resizeROIButton );
        m_canvesROIModePanel.add( m_moveROIButton );
        m_canvesROIModePanel.add( m_selectROIButton );
    }
    
    
    /**
     * Sets the DicomSliceCanvasPolyDraw object on which this control 
     * should work.
     * @param canvas
     */
    public void setSliceCanvasPolyDraw( DicomSliceCanvasPolyDraw canvas )
    {
        m_sliceCanvas = canvas;
        m_sliceCanvas.addChangeListener( this );
        
        m_pixelData = m_sliceCanvas.getImagePixelData( );
        m_minPixelValue = m_sliceCanvas.getMinRawDataValue( );
        m_maxPixelValue = m_sliceCanvas.getMaxRawDataValue( );

    }

    /**
     * Called as a reaction to actions; eg pressing a button.
     * @param e
     */
    @Override
    public void actionPerformed( ActionEvent e ) 
    {
        Object sender = e.getSource( );
        
        if ( sender == this.m_noROISizeLimits )
        {
            m_maximumROISizeLabel.setEnabled( false );
            m_minimumROISizeLabel.setEnabled( false );
            m_maximumROISize.setEnabled( false );
            m_minimumROISize.setEnabled( false );
        }
        
        if ( sender == this.m_ROISizeLimits )
        {
            m_maximumROISizeLabel.setEnabled( true );
            m_minimumROISizeLabel.setEnabled( true );
            m_maximumROISize.setEnabled( true );
            m_minimumROISize.setEnabled( true );
        }

        if ( sender == this.m_notAutoApplyButton )
        {
            m_applyButton.setEnabled( true );
        }

        if ( sender == this.m_autoApplyButton )
        {
            m_applyButton.setEnabled( false );
        }
        if ( sender == m_NotLimitNumberOfVertices )
        {
            m_maximumNumberOfVertices.setEnabled( false );
        }
        if ( sender == m_LimitNumberOfVertices )
        {
            m_maximumNumberOfVertices.setEnabled( true );
        }
        if ( sender == m_applyButton )
        {
            notifyActionListeners( PERFORM_SEGMENTATION );
        }
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 
    }
    
    /**
     * Called when sliders are changed.
     */
    @Override
    public void stateChanged( ChangeEvent e ) 
    {
        
        Object sender = e.getSource( );
        
        if ( sender == m_minimumPixelValueSlider ) 
        {
            if ( !m_minimumPixelValueSlider.getValueIsAdjusting( ) )
            {
                notifyActionListeners( MINIMUM_VALUE_CHANGE_EVENT );
                
                if ( this.m_autoApplyButton.isSelected( ) )
                {
                    notifyActionListeners( PERFORM_SEGMENTATION );
                }
            }
            m_minPixelValueIndicator.setText( "" + (int)( m_minimumPixelValueSlider.getValue( ) * this.m_maxPixelValue / 1000f ) );
        }
        
        if ( sender == m_maximumPixelValueSlider ) 
        {
            if ( !m_maximumPixelValueSlider.getValueIsAdjusting( ) )
            {
                notifyActionListeners( MAXIMUM_VALUE_CHANGE_EVENT );
                
                if ( this.m_autoApplyButton.isSelected( ) )
                {
                    notifyActionListeners( PERFORM_SEGMENTATION );
                }
                
            }
            m_maxPixelValueIndicator.setText( "" + (int)( m_maximumPixelValueSlider.getValue( ) * this.m_maxPixelValue / 1000f ) );
        }
        
        if ( sender == m_sliceCanvas )
        {
            m_pixelData = m_sliceCanvas.getImagePixelData( );
            m_minPixelValue = m_sliceCanvas.getMinRawDataValue( );
            m_maxPixelValue = m_sliceCanvas.getMaxRawDataValue( );
        }
        
    }

    /**
     * Adds an action listener to the list of action listeners. 
     * @param e
     */
    public void addActionListener( ActionListener e )
    {
	listeners.add( e );
    }
    
    /**
     * Notify all actionListeners.
     */
    private void notifyActionListeners( int eventType )
    {
        Iterator it = listeners.iterator( );
        while( it.hasNext( ) )
        {
            ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, eventType, "" ) );
        }
    }
    
    /**
     * Returns the minimum value.
     * @return 
     */
    public int getMinimumValue( ) 
    {
        return (int)(m_minimumPixelValueSlider.getValue( ) * m_maxPixelValue / 1000.0 );
    }
    
    /**
     * Returns the maximum value.
     * @return 
     */
    public int getMaximumValue( ) 
    {
        return (int)(m_maximumPixelValueSlider.getValue( ) * m_maxPixelValue / 1000.0 );
    }

    /**
     * Returns the minimum ROI volume size.
     * @return 
     */
    public int getMinimumROISize( )
    {
        return (int)m_minimumROISize.getValue( );
    }
    
    /**
     * Returns the minimum ROI volume size.
     * @return 
     */
    public int getMaximumROISize( )
    {
        return (int)m_maximumROISize.getValue( );
    }
    
    
    //! The slice canvas on which the segmentation operation has to be performed
    DicomSliceCanvasPolyDraw m_sliceCanvas;
    //! The pixel data of slice canvas
    private int[] m_pixelData;

    // Stores the processed pixel data
    private int[] m_processedPixelData;

    private JPanel m_canvesROIModePanel;
    private JButton m_addROIButton;
    private JButton m_deleteROIButton;
    private JButton m_cutROIButton;
    private JButton m_resizeROIButton;
    private JButton m_moveROIButton;
    private JButton m_selectROIButton;
    
    private final JPanel m_segmentationCollectionPanelRight;
    private final JPanel m_segmentationCollectionPanelLeft;
    
    private int   m_maxPixelValue;
    private int   m_minPixelValue;
    
    private final JLabel m_minPixelValueIndicator;
    private final JLabel m_maxPixelValueIndicator;
    
    private final JLabel m_minimumROISizeLabel;
    private final JLabel m_maximumROISizeLabel;
    
    private final JPanel m_roiMinMaxSizePanel;
    private final FormattedTextField m_minimumROISize;
    private final FormattedTextField m_maximumROISize;
    
    private final JRadioButton m_noROISizeLimits;
    private final JRadioButton m_ROISizeLimits;
    private final ButtonGroup m_ROISizeButtonGroup;
    
    private final JSlider m_minimumPixelValueSlider;
    private final JSlider m_maximumPixelValueSlider;
    
    private final JLabel m_maxPixelLabel;
    private final JLabel m_minPixelLabel;
    
    private final JPanel m_maxSliderPanel;
    private final JPanel m_minSliderPanel;
    
    private final JPanel m_limitedNumberOfVerticesPanel;
    
    private final ButtonGroup m_limitNumberOfVerticesButtonGroup;
    private final JRadioButton m_LimitNumberOfVertices;
    private final JRadioButton m_NotLimitNumberOfVertices;
    
    private final FormattedTextField m_maximumNumberOfVertices;
    
    private final JPanel m_autoApplyPanel;
    private final JRadioButton m_autoApplyButton;
    private final JRadioButton m_notAutoApplyButton;
    private final JButton m_applyButton;
    private final ButtonGroup m_autoApplyButtonGroup;
    
    // Listeners to changes in the segmentation gui control
    private final LinkedList<ActionListener> listeners = new LinkedList<>( );
    
    // Event IDs raised by this control
    public final static int MINIMUM_VALUE_CHANGE_EVENT = 0;
    public final static int MAXIMUM_VALUE_CHANGE_EVENT = 1;

    public final static int PERFORM_SEGMENTATION = 2;
    
    // Must be declarated somehow
    private static final long serialVersionUID = 3487493395819393L;
}