/*
 * File: $Id: SimpleThressholdSegmentationAnalyzerFrame.java,v 1.1.1.1 2006/04/19 09:38:48 hans Exp $
 * History
 * Date: 03.09.2004  Authors: Hans Slotboom  
 *
 * Changes:
 *
 * Copyright (C) 2004 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */


/**
 * Allows for primitive segmentation based on 
 * minimum/maximum simple pixel intensity thresholds.
 *
 * SimpleThressholdSegmentationAnalyzerFrame.java
 *
 * Created on September 17, 2004, 5:05 PM
 */

package org.spectrim.imageprocessing;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Cursor;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.BoxLayout;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.ButtonGroup;
import javax.swing.JToggleButton;


import javax.swing.JScrollPane;

import java.util.LinkedList;
import java.util.Iterator;
import java.util.LinkedHashMap;

import org.spectrim.signalprocessing.SignalTools;

import org.spectrim.dicom.FileInfo;
import org.spectrim.dicom.DicomSliceCanvasPolyDraw;

import org.spectrim.imagegraphicobjects.RegioPolygon;


/**
 * Control for definition of simple thresholding of images.
 * @author  hans
 */
public class SimpleThressholdSegmentationAnalyzerFrame extends JFrame implements ActionListener
{
    
    /** 
     * Creates a new instance of SimpleThressholdSegmentationAnalyzerFrame. 
     * @param parent
     * @param rawData
     * @param fInfo
     * @param siemensSpecific
     * @param numberOfImages
     */
    public SimpleThressholdSegmentationAnalyzerFrame( JFrame parent, 
                                                      int[] rawData, 
                                                      FileInfo fInfo, 
                                                      LinkedHashMap siemensSpecific, 
                                                      int numberOfImages )
    {
        m_parent  = parent;
        
        m_collectionPanel = new JPanel( );
        m_collectionPanel.setLayout( new BorderLayout( ) );
        
        setTitle( "Semi automatic segmentation" );
        
        m_northCollectionPanel = new JPanel( );
        m_northCollectionPanel.setLayout( new BoxLayout( m_northCollectionPanel, BoxLayout.X_AXIS ) );
        
        m_northwestCollectionPanel = new JPanel( );
        m_northwestCollectionPanel.setLayout( new BoxLayout( m_northwestCollectionPanel, BoxLayout.Y_AXIS ) );
        
        createDicomImagePanel( rawData, fInfo, siemensSpecific, numberOfImages );
        m_northwestCollectionPanel.add( m_dicomImagePanel );
        
        m_centerCollectionPanel = new JPanel( );
        m_centerCollectionPanel.setLayout( new BoxLayout( m_centerCollectionPanel, BoxLayout.Y_AXIS ) );
        
        createROIModePanel( );
        m_northwestCollectionPanel.add( m_canvasROIModePanel );
        
        m_northCollectionPanel.add( m_northwestCollectionPanel );
        
        createNorthEastCollectionPanel( );
        
        m_northCollectionPanel.add( m_ROIPropertiesScrollPane );
        
        m_collectionPanel.add( m_northCollectionPanel, BorderLayout.NORTH );
        
        createSegmentationControlPanel( );
        m_centerCollectionPanel.add( this.m_segmentationGUIControl );
        
        m_collectionPanel.add( m_centerCollectionPanel, BorderLayout.CENTER );
        
        
        createCancelApplyPanel( );
        m_collectionPanel.add( m_selectionPanel, BorderLayout.SOUTH );
        
        getContentPane( ).setLayout( new BorderLayout( ) );
        getContentPane( ).add( m_collectionPanel, BorderLayout.CENTER );
        
        setVisible( true ); 
        m_dicomSliceCanvasPolydraw.setZoom( m_appliedZoom );
        
        pack( );
         
        // Put the Dialog nicely in the middle of its parent
        Point centerParent = new Point( m_parent.getX( ) + ( m_parent.getWidth( ) / 2 ) - this.getWidth( ) / 2,
                                        m_parent.getY( ) + ( m_parent.getHeight( ) / 2 ) - this.getHeight( ) / 2 );
        
        
        setLocation( centerParent );
      
    }

    /** 
     * Creates a new instance of SegmentationGUIControl. 
     * @param parent
     * @param rawData
     * @param fInfo
     * @param siemensSpecific
     * @param roi
     * @param numberOfImages
     */
    public SimpleThressholdSegmentationAnalyzerFrame( JFrame parent, 
                                                      int[] rawData, 
                                                      FileInfo fInfo, 
                                                      LinkedHashMap siemensSpecific, 
                                                      RegioPolygon roi, 
                                                      int numberOfImages )
    {
        m_parent  = parent;
        
        m_segmentationWindow = roi;
        
        m_collectionPanel = new JPanel( );
        m_collectionPanel.setLayout( new BorderLayout( ) );
        
        setTitle( "Semi automatic segmentation" );
        
        m_northCollectionPanel = new JPanel( );
        m_northCollectionPanel.setLayout( new BoxLayout( m_northCollectionPanel, BoxLayout.X_AXIS ) );
        
        m_northwestCollectionPanel = new JPanel( );
        m_northwestCollectionPanel.setLayout( new BoxLayout( m_northwestCollectionPanel, BoxLayout.Y_AXIS ) );
        
        createDicomImagePanel( rawData, fInfo, siemensSpecific, numberOfImages );
        m_northwestCollectionPanel.add( m_dicomImagePanel );
        
        m_centerCollectionPanel = new JPanel( );
        m_centerCollectionPanel.setLayout( new BoxLayout( m_centerCollectionPanel, BoxLayout.Y_AXIS ) );
        
        createROIModePanel( );
        m_northwestCollectionPanel.add( m_canvasROIModePanel );
        
        m_northCollectionPanel.add( m_northwestCollectionPanel );
        
        createNorthEastCollectionPanel();
        
        m_northCollectionPanel.add( m_ROIPropertiesScrollPane );
        
        m_collectionPanel.add( m_northCollectionPanel, BorderLayout.NORTH );
        
        createSegmentationControlPanel( );
        m_centerCollectionPanel.add( this.m_segmentationGUIControl );
        
        m_collectionPanel.add( m_centerCollectionPanel, BorderLayout.CENTER );
        
        
        createCancelApplyPanel( );
        m_collectionPanel.add( m_selectionPanel, BorderLayout.SOUTH );
        
        getContentPane( ).setLayout( new BorderLayout( ) );
        getContentPane( ).add( m_collectionPanel, BorderLayout.CENTER );
        
        setVisible( true ); 
        m_dicomSliceCanvasPolydraw.setZoom( m_appliedZoom );
        
        pack( );
        
        // Put the Dialog nicely in the middle of its parent
        Point centerParent = new Point( m_parent.getX( ) + ( m_parent.getWidth( ) / 2 ) - this.getWidth( ) / 2,
                                        m_parent.getY( ) + ( m_parent.getHeight( ) / 2 ) - this.getHeight( ) / 2 );
        
        setLocation( centerParent );
      
    }

    
    /**
     * Creates the panel in the north east side, containing a 
     * RegionPropertiesTable component.
     */
    private void createNorthEastCollectionPanel( )
    {
        m_northeastCollectionPanel = new JPanel( );
        m_northeastCollectionPanel.setLayout( new BoxLayout( m_northeastCollectionPanel, BoxLayout.X_AXIS ) );
        
        m_roiPropertiesTable = new RegionPropertiesTable( this, m_dicomSliceCanvasPolydraw, true );
        m_roiPropertiesTable.addActionListener( this );
        
        // JList does not provide a scroll list automatically so we have to 
        // add it ourselves.
        m_ROIPropertiesScrollPane = new JScrollPane( JScrollPane.VERTICAL_SCROLLBAR_ALWAYS , JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED ); 
        m_ROIPropertiesScrollPane.getViewport( ).setView( m_roiPropertiesTable );        
        m_ROIPropertiesScrollPane.setPreferredSize( new Dimension( 500, 500 ) );
        
    }
    
    /**
     * Creates the DicomImagePanel.
     */
    private void createDicomImagePanel( int[] data, FileInfo finfo, 
                                        LinkedHashMap siemensSpecificHeader, 
                                        int numberOfImages )
    {
        m_dicomImagePanel = new JPanel( );
        
        m_dicomImagePanel.setLayout( new BoxLayout( m_dicomImagePanel, BoxLayout.X_AXIS ) );
        m_dicomImagePanel.add( Box.createHorizontalGlue( ) );
        
        m_dicomSliceCanvasPolydraw = new DicomSliceCanvasPolyDraw( this );
        m_dicomSliceCanvasPolydraw.addActionListener( this );
        
        if ( finfo.getFileName( ) != null )
        {
            m_dicomSliceCanvasPolydraw.setTitle( finfo.getFileName( ) );
        }
        else
        {
            setTitle( finfo.getPatientName( ) + "   " + finfo.getPatientBirthDate( ) );
        }
        
        m_dicomSliceCanvasPolydraw.setImage( data , finfo.getImageWidth( ), finfo.getImageHeight( ), 0, finfo, siemensSpecificHeader );
        
        m_dicomSliceCanvasPolydraw.setNumberOfImages( numberOfImages );
        
        if ( m_segmentationWindow != null )
        {
            m_dicomSliceCanvasPolydraw.addPolygon( m_segmentationWindow );
        }
        
        if ( finfo.getImageWidth( ) != 0 )
        {
            m_appliedZoom = 512f / finfo.getImageWidth( );
        }
        
        m_dicomSliceCanvasPolydraw.enableImagePaging( true );
        
        m_dicomSliceCanvasPolydraw.displayDicomImage( 0 );
        
        m_dicomSliceCanvasPolydraw.setPreferredSize( new Dimension( 512, 512 ) );
        m_dicomSliceCanvasPolydraw.setMaximumSize( new Dimension( 512, 512 ) );
        m_dicomSliceCanvasPolydraw.setMinimumSize( new Dimension( 512, 512 ) );
        
        m_dicomSliceCanvasPolydraw.addActionListener( this );
        
       
        m_maxPixelValueOfLoadedImage = SignalTools.getMaximum( data );
        m_minPixelValueOfLoadedImage = SignalTools.getMinimum( data );
       
        m_dicomSliceCanvasPolydraw.setVisible( true );
        
        m_dicomSliceCanvasPolydraw.addActionListener( this );
        m_dicomImagePanel.add( m_dicomSliceCanvasPolydraw );
        m_dicomImagePanel.add( Box.createHorizontalGlue( ) );
    }
    
    
    /**
     * Returns the applied zoom.
     * @return 
     */
    public float getAppliedZoom( )
    {
        return m_appliedZoom;
    }
    
    
    /**
     * Creates a panel allowing the user to set the ROI state of the 
     * DicomSliceCanvasPolyDraw control.
     */
    private void createROIModePanel( )
    {
        m_canvasROIModePanel = new JPanel( );
        m_canvasROIModePanel.setLayout( new BoxLayout( m_canvasROIModePanel, BoxLayout.X_AXIS ) );
        m_canvasROIModePanel.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );
        
        m_addROIButton = new JToggleButton( "Add ROI" );
        m_addROIButton.addActionListener( this );
        m_deleteROIButton = new JToggleButton( "Delete ROI" );
        m_deleteROIButton.addActionListener( this );
        m_cutROIButton = new JToggleButton( "Cut ROI" ); 
        m_cutROIButton.addActionListener( this );
        m_resizeROIButton = new JToggleButton( "Resize ROI" );
        m_resizeROIButton.addActionListener( this );
        m_moveROIButton = new JToggleButton( "Move ROI" );
        m_moveROIButton.addActionListener( this );
        m_selectROIButton = new JToggleButton( "Select ROI" );
        m_selectROIButton.addActionListener( this );
        
        m_canvasROIModePanel.add( Box.createHorizontalGlue( ) );
        m_canvasROIModePanel.add( m_addROIButton );
        m_canvasROIModePanel.add( m_deleteROIButton );
        m_canvasROIModePanel.add( m_cutROIButton );
        m_canvasROIModePanel.add( m_resizeROIButton );
        m_canvasROIModePanel.add( m_moveROIButton );
        m_canvasROIModePanel.add( m_selectROIButton );
        
        m_buttonGroupROIModes = new ButtonGroup( );

        m_buttonGroupROIModes.add( m_addROIButton );
        m_buttonGroupROIModes.add( m_deleteROIButton );
        m_buttonGroupROIModes.add( m_cutROIButton );
        m_buttonGroupROIModes.add( m_resizeROIButton );
        m_buttonGroupROIModes.add( m_moveROIButton );
        m_buttonGroupROIModes.add( m_selectROIButton );
        
        m_selectROIButton.setSelected( true );
        
    }
    
    /**
     * Creates the SegmentationControlPanel.
     */
    private void createSegmentationControlPanel( )
    {
        m_segmentationGUIControl = new SegmentationGUIControl( "Segmentation Parameters", m_minPixelValueOfLoadedImage, m_maxPixelValueOfLoadedImage );
        m_segmentationGUIControl.addActionListener( this );
    }
    
    
    /**
     * Instantiates an apply/ cancel panel.
     */
    private void createCancelApplyPanel( )
    {
        // Selection panel definitions
        m_applyButton = new JButton( "Apply" );
        m_applyButton.addActionListener( this );
        m_applyButton.setToolTipText( "Returns a list of ROI objects to calling object" );
        m_cancelButton = new JButton( "Cancel" );
        m_cancelButton.addActionListener( this );
        m_cancelButton.setToolTipText( "Cancels eithout returning a list of ROI objects to calling object" );
        
        m_selectionPanel = new JPanel( );
        
        m_selectionPanel.setLayout( new BoxLayout( m_selectionPanel, BoxLayout.X_AXIS ) );
        m_selectionPanel.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );
        m_selectionPanel.add( Box.createGlue( ) );
        m_selectionPanel.add( m_applyButton );
        m_selectionPanel.add( Box.createGlue( ) );
        m_selectionPanel.add( m_cancelButton );
        m_selectionPanel.add( Box.createGlue( ) );
    }
    
    /** 
     * Is called whenever a registered component raises an ActionEvent.
     * @param e
     */
    @Override
    public void actionPerformed( ActionEvent e ) 
    {
        Object sender = e.getSource( );
       
        if ( e.getSource( ) == this.m_dicomSliceCanvasPolydraw )
        {
            if ( e.getID( ) == DicomSliceCanvasPolyDraw.POLYGON_CLOSED_EVENT )
            {
                m_segmentationWindow =  new RegioPolygon( m_dicomSliceCanvasPolydraw.convertCurrentPolygonToRectangle( ) );
            }
            // delete event must also be handled
        }
       
       
       // handle responses on cancel button pressed event
        if ( sender == m_cancelButton )
        {
            Iterator it = m_listeners.iterator( );
             while( it.hasNext( ) )
             {
                ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, CANCEL_EVENT, "" ) );
             }
            dispose( );
        }
        // handle responses on apply button pressed event
        if ( sender == m_applyButton )
        {
            if ( m_applyButtonPressed == false )
            {
                Iterator it = m_listeners.iterator( );
                while( it.hasNext( ) )
                {
                    ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, ACCEPT_EVENT, "" ) );
                }
            }
            m_applyButtonPressed = true;
            this.dispose( );
            
        }
       
        // handles action events raised by the slice canvas polydraw object
        if ( sender == this.m_dicomSliceCanvasPolydraw )
        {
            if ( e.getID( ) == DicomSliceCanvasPolyDraw.ADD_ROI_MODE_SELECTED_EVENT )
            {
                m_addROIButton.setSelected( true );
            }
            
            if ( e.getID( ) == DicomSliceCanvasPolyDraw.DELETE_ROI_MODE_SELECTED_EVENT )
            {
                m_deleteROIButton.setSelected( true );
            }
             
            if ( e.getID( ) == DicomSliceCanvasPolyDraw.CUT_ROI_MODE_SELECTED_EVENT )
            {
                m_cutROIButton.setSelected( true );
            }
            if ( e.getID( ) == DicomSliceCanvasPolyDraw.RESIZE_ROI_MODE_SELECTED_EVENT )
            {
                m_resizeROIButton.setSelected( true );
            }
            
            if ( e.getID( ) == DicomSliceCanvasPolyDraw.MOVE_ROI_MODE_SELECTED_EVENT )
            {
                m_moveROIButton.setSelected( true );
            }
            
            if ( e.getID( ) == DicomSliceCanvasPolyDraw.SELECT_ROI_MODE_SELECTED_EVENT )
            {
                m_selectROIButton.setSelected( true );
            }            
        }
       
        if ( sender == m_addROIButton )
        {
            m_dicomSliceCanvasPolydraw.setMode( DicomSliceCanvasPolyDraw.POLYGON_DRAW_MODE );
        }
       
        if ( sender == m_deleteROIButton )
        {
            m_dicomSliceCanvasPolydraw.setMode( DicomSliceCanvasPolyDraw.DELETE_MODE );
        }
        
        if ( sender == m_cutROIButton )
        {
            m_dicomSliceCanvasPolydraw.setMode( DicomSliceCanvasPolyDraw.CUT_MODE );
        }
       
        if ( sender == m_resizeROIButton )
        {
            m_dicomSliceCanvasPolydraw.setMode( DicomSliceCanvasPolyDraw.RESIZE_MODE );
        }
       
        if ( sender == m_moveROIButton )
        {
            m_dicomSliceCanvasPolydraw.setMode( DicomSliceCanvasPolyDraw.DRAG_MODE  );
        }
        
        if ( sender == m_selectROIButton )
        {
            m_dicomSliceCanvasPolydraw.setMode( DicomSliceCanvasPolyDraw.SELECT_MODE );
        }
        
       if ( sender == m_segCalc )
       {
            m_roiPropertiesTable.updateROIList( m_segmentationGUIControl.getMinimumValue( ), 
                                                m_segmentationGUIControl.getMaximumValue( ) );
           
            if ( m_segmentationWindow != null )
            {
                m_dicomSliceCanvasPolydraw.addPolygon( m_segmentationWindow );
            }
           
            setCursor( new Cursor( Cursor.DEFAULT_CURSOR ) );
       }
        
        if ( sender == m_segmentationGUIControl )
        {
            if ( e.getID( ) == SegmentationGUIControl.PERFORM_SEGMENTATION )
            {
                int minSegVal = m_segmentationGUIControl.getMinimumValue( );
                int maxSegVal = m_segmentationGUIControl.getMaximumValue( );
                int minROISize = m_segmentationGUIControl.getMinimumROISize( );
                int maxROISize = m_segmentationGUIControl.getMaximumROISize( );
                
                m_dicomSliceCanvasPolydraw.deleteAllPolygonsOfCurrentlyDisplayedImageOnly( );
                
                if ( m_segmentationWindow != null )
                {
                    m_segCalc = new SimpleThressholdSegmentationCalculator( this, m_dicomSliceCanvasPolydraw, minSegVal, maxSegVal, minROISize, maxROISize, m_segmentationWindow );
                }
                else
                {
                    m_segCalc = new SimpleThressholdSegmentationCalculator( this, m_dicomSliceCanvasPolydraw, minSegVal, maxSegVal, minROISize, maxROISize );
                }
                
                m_segCalc.addActionListener( this );
                setCursor( new Cursor( Cursor.WAIT_CURSOR ) );
                m_segCalc.performSegmentation( );
            }
        }
       
        if ( sender == m_roiPropertiesTable )
        {
            if ( e.getID( ) == RegionPropertiesTable.DELETE_EVENT )
            {
                LinkedList lst = m_roiPropertiesTable.getROIsToDeleteNameList( );
                for ( int i= 0; i< lst.size(); i++ )
                {
                    m_dicomSliceCanvasPolydraw.deletePolygon( (String)lst.get( i ) );
                }
            }
        }
       
    }
    
    
    /**
     * Adds an action listener to the list of action listeners who are notified
     * when something happens within this object.
     * @param e
     */
    public void addActionListener( ActionListener e )
    {
	m_listeners.add( e );
    }
    
    /**
     * Returns the list of persistent polygons.
     * @return 
     */
    public LinkedList getPersistentPolygons( )
    {
        return m_roiPropertiesTable.getPersistentPolygons( );
    }
    
    // private attributes
    private JFrame m_parent;
    
    private JPanel m_collectionPanel;
    private SegmentationGUIControl m_segmentationGUIControl;
    
    private JPanel m_dicomImagePanel;
    private DicomSliceCanvasPolyDraw m_dicomSliceCanvasPolydraw;
    
    private JPanel m_selectionPanel;
    private JButton m_applyButton;
    private JButton m_cancelButton;
    
    private JPanel m_northCollectionPanel;
    private JPanel m_northwestCollectionPanel;
    
    private JPanel m_northeastCollectionPanel;
    
    public RegionPropertiesTable m_roiPropertiesTable;
    
    private JScrollPane m_ROIPropertiesScrollPane;
    
    // Collects all sub panels to be placed in the center panel of the
    // master layout
    private JPanel m_centerCollectionPanel;
    
    
    // Controls for setting the window state
    private JPanel m_canvasROIModePanel;
    private JToggleButton m_addROIButton;
    private JToggleButton m_deleteROIButton;
    private JToggleButton m_cutROIButton;
    private JToggleButton m_resizeROIButton;
    private JToggleButton m_moveROIButton;
    private JToggleButton m_selectROIButton;
    
    private ButtonGroup m_buttonGroupROIModes;
    
    
    private int m_maxPixelValueOfLoadedImage, m_minPixelValueOfLoadedImage;
    private float m_appliedZoom = 1.0f;
    
    // Listeners to action events in this class
    private LinkedList<ActionListener> m_listeners = new LinkedList<ActionListener>( );
    
    private SimpleThressholdSegmentationCalculator m_segCalc;
    
    // events raised by accept and cancel button
    public final static int ACCEPT_EVENT = 0;
    public final static int CANCEL_EVENT = 1;
    
    // SegmentationWindow
    private RegioPolygon m_segmentationWindow = null;
    private boolean m_applyButtonPressed = false;
    
        // For proper serialization
    private static final long serialVersionUID = 1499324591289333L;
}
