/*
 * OverlayPropertiesTable.java
 *
 * Created on April 21, 2008, 11:20 AM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package org.spectrim.imageprocessing;

import org.spectrim.lib.ColorEditor;
import org.spectrim.lib.ColorRenderer;
import org.spectrim.lib.EnterStringFrame;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Iterator;
import java.util.LinkedList;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JTable;
import javax.swing.table.DefaultTableModel;


/**
 * This class handles the table with overlay properties; this table lists
 * how many different tissue types have been computed and lets the user select
 * for specific tissue types
 */

public class VolumeOfInterestOverlayPropertiesTable extends JTable implements ActionListener, MouseListener
{

   
    public VolumeOfInterestOverlayPropertiesTable( ) 
    {
        super( );
        
        setDefaultRenderer( Color.class,
                            new ColorRenderer( true ) );

        m_colorEditor = new ColorEditor( );

        m_colorEditor.addActionListener( this );

        setDefaultEditor( Color.class,
                          m_colorEditor );
        
        setModel( new VolumeOfInterestOverlayPropertiesTableModel( ) );
        
        setAutoCreateColumnsFromModel( false );
        
        setColumnSelectionAllowed (false );
        setRowSelectionAllowed( true );
        
        addPopupMenu( );
        
        m_selectionListener = new VolumeOfInterestOverlayPropertiesTableSelectionListener( this );
        
        getSelectionModel( ).addListSelectionListener( m_selectionListener );
        getColumnModel( ).getSelectionModel( ).addListSelectionListener( m_selectionListener );
        
        // listen to mouse events and react to them!
        addMouseListener( this );
        
    }


    /**
     * Sets the parent frame for proper displaying of renaming frame if called
     * @param parent
     */
    public void setParentFrame( JFrame parent )
    {
        m_parent = parent;
    }

    /**
     * Does the rest of the object construction work
     */
    private void addPopupMenu( )
    {
        m_popup = new JPopupMenu( );
        
        m_imageVOIBooleanLogicMenu = new JMenu( "Boolean Operations" );
        
        m_applyLogic_AND_toImageSegments = new JMenuItem( "AND" );
        m_applyLogic_AND_toImageSegments.addActionListener( this );
        m_applyLogic_AND_toImageSegments.setEnabled( false );
        
        m_applyLogic_OR_toImageSegments = new JMenuItem( "OR" );
        m_applyLogic_OR_toImageSegments.addActionListener( this );
        m_applyLogic_OR_toImageSegments.setEnabled( false );
        
        m_applyLogic_EXOR_toImageSegments = new JMenuItem( "EXOR" );
        m_applyLogic_EXOR_toImageSegments.addActionListener( this );
        m_applyLogic_EXOR_toImageSegments.setSelected( false );
        
        m_applyLogic_NOT_toImageSegments = new JMenuItem( "NOT" );
        m_applyLogic_NOT_toImageSegments.addActionListener( this );
        m_applyLogic_NOT_toImageSegments.setEnabled( false );
        
        
        m_imageVOIBooleanLogicMenu.add( m_applyLogic_AND_toImageSegments  );
        m_imageVOIBooleanLogicMenu.add( m_applyLogic_OR_toImageSegments  );
        m_imageVOIBooleanLogicMenu.add( m_applyLogic_EXOR_toImageSegments  );
        m_imageVOIBooleanLogicMenu.add( m_applyLogic_NOT_toImageSegments );
        
        m_popup.add( m_imageVOIBooleanLogicMenu );

        m_imageVOIEditMenu = new JMenu( "VOI Edit");

        m_popup.add( m_imageVOIEditMenu );

        m_copyVOIMenuItem = new JMenuItem( "Copy selected VOIs" );
        m_copyVOIMenuItem.addActionListener( this );
        m_imageVOIEditMenu.add(m_copyVOIMenuItem);

        m_pasteVOIMenuItem = new JMenuItem( "Paste selected VOIs" );
        m_pasteVOIMenuItem.addActionListener( this );
        m_imageVOIEditMenu.add(m_pasteVOIMenuItem);

        m_deleteVOIMenuItem = new JMenuItem( "Delete selected VOIs" );
        m_deleteVOIMenuItem.addActionListener( this );
        m_imageVOIEditMenu.add( m_deleteVOIMenuItem );

        m_renameVOIMenuItem = new JMenuItem( "Rename selected VOI" );
        m_renameVOIMenuItem.addActionListener( this );
        m_imageVOIEditMenu.add( m_renameVOIMenuItem );

        add( m_popup );
        
    }
    
    /**
     * Notifies all registered components interested in selection changes in the table
     */
    public void notifyActionListeners( int eventType )
    {
        Iterator it = m_listeners.iterator( );

        while( it.hasNext( ) )
        {
            if ( eventType == VolumeOfInterestOverlayPropertiesTable.TABLE_SELECTED_EVENT )
            {
                ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, VolumeOfInterestOverlayPropertiesTableSelectionListener.ROW_SELECTION_CHANGED, "" ) );
            }
            if ( eventType == VolumeOfInterestOverlayPropertiesTable.DELETE_SELECTED_VOLUME_EVENT )
            {
                ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, VolumeOfInterestOverlayPropertiesTable.DELETE_SELECTED_VOLUME_EVENT , "" ) );
            }

            if ( eventType == VolumeOfInterestOverlayPropertiesTable.RENAME_SELECTED_VOLUME_EVENT )
            {
                ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, VolumeOfInterestOverlayPropertiesTable.RENAME_SELECTED_VOLUME_EVENT , "" ) );
            }
        }

    }
    
    /**
     * Notifies all other registered components interested in selection 
     * changes in the table
     */
    public void notifyOtherActionListeners( int eventType )
    {

        Iterator it = m_listeners.iterator( );
        while( it.hasNext( ) )
        {
           ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, eventType, "" ) );
        }

    }
    
    
    /**
     * Adds an actionlistener to the list of action listeners who are notified
     * when something happens with the spinbox
     */
    public void addActionListener( ActionListener e )
    {
        m_listeners.add( e );
    }
    
    /**
     * 
     * Returns the last selected color.
     * 
     * @return
     */
    public Color getLastSelectedColor( )
    {
        return m_lastSelectedColor;
    }


    /**
     * Is called whenever a registered componenet raises an ActionEvent
     */
    public void actionPerformed( ActionEvent e ) 
    {
        Object sender = e.getSource( );

        if ( e.getSource( ) instanceof ColorEditor )
        {

             if ( e.getID( ) == ColorEditor.COLOR_SELECTED )
             {
                 System.out.println( "Action event caught from ColorEditor" );
                 m_lastSelectedColor = (Color)m_colorEditor.getCellEditorValue( );
                 notifyOtherActionListeners( OVERLAYCOLOR_CHANGE_EVENT );
             }
        }

        if ( sender == m_renameVOIMenuItem )
        {
            DefaultTableModel defModel = (DefaultTableModel)this.getModel( );
            int selRow = getSelectedRow( );

            String initVal = (String)defModel.getValueAt( selRow, 0 );

            m_renameVolumeFrame = new EnterStringFrame( m_parent, "New volume name", initVal, "Rename volume" );
            m_renameVolumeFrame.addActionListener( this );
        }

        if ( sender == m_renameVolumeFrame )
        {
            if ( e.getID( ) == EnterStringFrame.APPLY_EVENT )
            {
                m_newVolumeName = m_renameVolumeFrame.getText( );

                DefaultTableModel defModel = (DefaultTableModel)this.getModel( );
                int selRow = getSelectedRow( );

                defModel.setValueAt( m_newVolumeName, selRow, 0 );

                notifyActionListeners( RENAME_SELECTED_VOLUME_EVENT );
                m_renameVolumeFrame.dispose( );
            }

        }
        if ( sender == m_deleteVOIMenuItem )
        {
            // let the viewer know that the selected volumes need to be removed ..
            notifyActionListeners( DELETE_SELECTED_VOLUME_EVENT );

            DefaultTableModel defModel = (DefaultTableModel)this.getModel( );
            int[] selRows = getSelectedRows( );

            for ( int i = getRowCount( ) - 1; i >= 0; i-- )
            {
                for ( int j = 0; j < selRows.length; j++ )
                {
                    if ( i == selRows[j] )
                    {
                        // remove it ..
                        defModel.removeRow( i );
                    }
                }
            }
        }
    

        if ( sender == m_applyLogic_AND_toImageSegments )
        {
            notifyOtherActionListeners( PERFORM_BOOLEAN_AND_OPERATION );
        }
        
        if ( sender == m_applyLogic_OR_toImageSegments )
        {
            notifyOtherActionListeners( PERFORM_BOOLEAN_OR_OPERATION );
        }
        
        if ( sender == m_applyLogic_EXOR_toImageSegments )
        {
            notifyOtherActionListeners( PERFORM_BOOLEAN_EXOR_OPERATION );
        }
        
        if ( sender == m_applyLogic_NOT_toImageSegments )
        {
            notifyOtherActionListeners( PERFORM_BOOLEAN_NOT_OPERATION );
        }
        
    }

    /**
     * Is called whenever a registered componenet raises mouseClicked event
     */
    public void mouseClicked( MouseEvent e )
    {
  
    }


    /**
     * Returns the new volume name
     * @return
     */
    public String getNewVolumeName( )
    {
        return m_newVolumeName;
    }

    
    /**
     * The user pressed the table, this may be a change in the selected 
     * row. Sent a notification to the listeners to this table
     */
    public void mousePressed( MouseEvent e ) 
    {
        if ( e.isPopupTrigger( ) )
        {
            m_popup.show( e.getComponent( ), e.getX( ), e.getY( ) );
        }
        
        notifySelectionChange( );
    }

    public void mouseReleased( MouseEvent e ) 
    {
        if ( e.isPopupTrigger( ) )
        {
            m_popup.show( e.getComponent( ), e.getX( ), e.getY( ) );
        }
        
        notifySelectionChange( );
    }

    public void mouseEntered( MouseEvent e ) 
    {
        // not implemented
    }

    public void mouseExited( MouseEvent e ) 
    {
        // not implemented
    }

    /**
     * Notifies listeners that a change in table selection was performed
     */
    private void notifySelectionChange( ) 
    {
         notifyActionListeners( TABLE_SELECTED_EVENT );
         
         if ( ( getSelectedRows( ).length == 0 ) || ( getSelectedRows( ).length > 2 ) )
         {
             m_applyLogic_AND_toImageSegments.setEnabled( false );
             m_applyLogic_OR_toImageSegments.setEnabled( false );
             m_applyLogic_EXOR_toImageSegments.setEnabled( false );
             m_applyLogic_NOT_toImageSegments.setEnabled( false );
             
         }
         
         if ( getSelectedRows( ).length == 1 )
         {
             m_applyLogic_AND_toImageSegments.setEnabled( false );
             m_applyLogic_OR_toImageSegments.setEnabled( false );
             m_applyLogic_EXOR_toImageSegments.setEnabled( false );
             m_applyLogic_NOT_toImageSegments.setEnabled( true );
             
         }
         
         if ( getSelectedRows( ).length == 2 )
         {
             m_applyLogic_AND_toImageSegments.setEnabled( true );
             m_applyLogic_OR_toImageSegments.setEnabled( true );
             m_applyLogic_EXOR_toImageSegments.setEnabled( true );
             m_applyLogic_NOT_toImageSegments.setEnabled( false );
             
         }
         
    }
    
    // Attributes of the class
    
    private static final long serialVersionUID = 99822446327598273L;
    
    private VolumeOfInterestOverlayPropertiesTableSelectionListener m_selectionListener;
    
    // Listeners to action events in this class
    private LinkedList<ActionListener> m_listeners = new LinkedList<ActionListener>( );
    
    public final static int TABLE_SELECTED_EVENT            = 0;
    public final static int OVERLAYCOLOR_CHANGE_EVENT       = 1;

    private JPopupMenu      m_popup;
    
    private JMenu           m_imageVOIBooleanLogicMenu;
    
    private JMenuItem       m_applyLogic_AND_toImageSegments;
    private JMenuItem       m_applyLogic_OR_toImageSegments;
    private JMenuItem       m_applyLogic_EXOR_toImageSegments;
    private JMenuItem       m_applyLogic_NOT_toImageSegments;

    private JMenu           m_imageVOIEditMenu;
    private JMenuItem       m_copyVOIMenuItem;
    private JMenuItem       m_pasteVOIMenuItem;
    private JMenuItem       m_deleteVOIMenuItem;
    private JMenuItem       m_renameVOIMenuItem;

    public final static int DELETE_SELECTED_VOLUME_EVENT    = 11;
    public final static int RENAME_SELECTED_VOLUME_EVENT    = 12;

    // possible action events raised by this
    public final static int PERFORM_BOOLEAN_AND_OPERATION   = 20;
    public final static int PERFORM_BOOLEAN_OR_OPERATION    = 21;
    public final static int PERFORM_BOOLEAN_EXOR_OPERATION  = 22;
    public final static int PERFORM_BOOLEAN_NOT_OPERATION   = 23;

    private ColorEditor     m_colorEditor;
    private Color           m_lastSelectedColor;

    private EnterStringFrame  m_renameVolumeFrame;
    private JFrame          m_parent;
    private String          m_newVolumeName;
}
