package org.spectrim.io.dicom;

import org.spectrim.imagegraphicobjects.Point3D;
import com.pixelmed.dicom.DicomException;
import com.pixelmed.display.SourceImage;
import java.awt.image.Raster;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintStream;
import java.io.RandomAccessFile;
import java.io.UnsupportedEncodingException;
import java.nio.channels.FileChannel;
import java.nio.channels.FileLock;
import java.util.Arrays;
import java.util.Iterator;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.dcm4che2.data.DicomElement;
import org.dcm4che2.data.DicomObject;
import org.dcm4che2.data.Tag;
import org.dcm4che2.data.VR;
import org.dcm4che2.io.DicomInputStream;
import org.dcm4che2.io.DicomOutputStream;
import org.dcm4che2.util.TagUtils;

/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
/**
 *
 * @author nunopbarros
 */
public class DicomHandler {

    public static Raster readImageRaster(String file, int index) throws IOException {
        Raster raster;

        try {
            // Hide outputs
            PrintStream originalStream = System.err;
            PrintStream dummyStream = new PrintStream(new OutputStream() {
                public void write(int b) {
                    //NO-OP
                }
            });
            System.setErr(dummyStream);

            raster = new SourceImage(file).getBufferedImage(index).getRaster();

            // Enable outputs again
            System.setErr(originalStream);
        } catch (DicomException ex) {
            throw new IOException("Error: couldn't read dicom image! " + file);
        }
        return raster;
    }

    public static Raster readImageRaster(String file) throws IOException {
        Raster raster;

        try {
            // Hide outputs
            PrintStream originalStream = System.err;
            PrintStream dummyStream = new PrintStream(new OutputStream() {
                public void write(int b) {
                    //NO-OP
                }
            });
            System.setErr(dummyStream);

            raster = new SourceImage(file).getBufferedImage().getRaster();

            // Enable outputs again
            System.setErr(originalStream);
        } catch (DicomException ex) {
            throw new IOException("Error: couldn't read dicom image! " + file);
        }
        return raster;
    }

    public static DicomObject getDicomObject(String file) {
        DicomObject object = null;
        try {
            DicomInputStream dis = new DicomInputStream(new File(file));
            object = dis.readDicomObject();
            dis.close();
        } catch (Exception e) {
            System.out.println(e.getMessage());
        }
        return object;
    }

    public static boolean isDicomImage(String file) {
        if (isDicom(file)) {
            DicomObject object1 = DicomHandler.getDicomObject(file);
            String classUID = DicomHandler.getSOPClassUID(object1);
            if (classUID.equalsIgnoreCase("1.2.840.10008.5.1.4.1.1.4") || classUID.equalsIgnoreCase("1.2.840.10008.5.1.4.1.1.4.1") || classUID.equalsIgnoreCase("1.2.840.10008.5.1.4.1.1.2")) {
                return true;
            }
        }
        return false;
    }

    public static boolean isDicom(String file) {
        return isDicom(new File(file));
    }

    public static boolean isDicom(File file) {
        String text;
        if (file.isFile() && !isLocked(file)) {
            try {
                BufferedReader inputStream = new BufferedReader(new FileReader(file), 4);
                inputStream.skip(128);
                text = inputStream.readLine();
            } catch (FileNotFoundException ex) {
                Logger.getLogger(DicomHandler.class.getName()).log(Level.SEVERE, null, ex);
                return false;
            } catch (IOException ex) {
                Logger.getLogger(DicomHandler.class.getName()).log(Level.SEVERE, null, ex);
                return false;
            }

            if (text.startsWith("DICM")) {
                return true;
            } else {
                return false;
            }
        } else {
            return false;
        }

    }

    public static boolean isLocked(File file) {
        try {
            FileChannel channel = new RandomAccessFile(file, "rw").getChannel();
            FileLock lock = channel.tryLock();
            if (lock == null) {
                return true;
            } else {
                lock.release();
                return false;
            }
        } catch (FileNotFoundException ex) {
            return true;
        } catch (IOException ex) {
            return true;
        }
    }

    public static String[] getReferenceImages(DicomObject object) {
        String[] refs = new String[3];
        DicomElement elem = object.get(Integer.parseInt("00081140", 16));
        if (elem != null) {
            for (int i = 0; i < elem.countItems(); i++) {
                refs[i] = elem.getDicomObject(i).getString(Integer.parseInt("00081155", 16));
            }
            return refs;
        } else {
            return null;
        }

    }

    public static String createInitials(String name, String yr, String UPN) {
        int index = name.indexOf("^");
        String newName = "";
        if (index == -1) {
            newName = name.substring(0, 2)
                    + "_" + UPN + "_" + yr.substring(0, 4);
        } else {
            newName = name.substring(0, 2)
                    + name.substring(index + 1, index + 2) + "_" + UPN + "_" + yr.substring(0, 4);
        }
        return newName.toUpperCase();
    }

    public static String getPatientID(DicomObject object) {
        return object.getString(Integer.parseInt("00100020", 16));
    }

    public static String getPatientName(DicomObject object) {
        return object.getString(Integer.parseInt("00100010", 16));
    }

    public static String getPatientBirthDate(DicomObject object) {
        return object.getString(Integer.parseInt("00100030", 16));
    }

    public static String getFieldStrength(DicomObject object) {
        return object.getString(Integer.parseInt("00180087", 16));
    }

    public static Double getTE(DicomObject object) {
        return object.getDouble(Tag.EchoTime);
    }

    public static String anonymize(DicomObject object, String UPN) {
        object.putString(Integer.parseInt("00080080", 16), VR.LO, ""); //  Institution Name
        object.putString(Integer.parseInt("00080081", 16), VR.ST, ""); //  Institution Address
        object.putString(Integer.parseInt("00080090", 16), VR.PN, ""); //  Referring Physician’s Name
        object.putString(Integer.parseInt("00081010", 16), VR.SH, ""); //  Station Name
        object.putString(Integer.parseInt("00081030", 16), VR.LO, ""); //  Study Description
        object.putString(Integer.parseInt("00081040", 16), VR.LO, ""); //  Institutional Department Name
        object.putString(Integer.parseInt("00081048", 16), VR.PN, ""); //  Physician(s) of Record
        object.putString(Integer.parseInt("00081070", 16), VR.PN, ""); //  Operators’ Name

        String newname = getAnonymizedName(object, UPN);
        object.putString(Integer.parseInt("00100010", 16), VR.PN, newname); //  Patient’s Name
        object.putString(Integer.parseInt("00100020", 16), VR.LO, "1234567890"); //  Patient ID
        object.putString(Integer.parseInt("00100030", 16), VR.DA, "19000101"); //  Patient’s Birth Date
        object.putString(Integer.parseInt("00100040", 16), VR.CS, "O"); //  Patient’s Sex

        object.putString(Integer.parseInt("00101010", 16), VR.AS, "00Y"); //  Patient’s Age
        object.putDouble(Integer.parseInt("00101020", 16), VR.DS, 0.00); //  Patient’s Size
        object.putDouble(Integer.parseInt("00101030", 16), VR.DS, 0); //  Patient’s Weight
        object.putString(Integer.parseInt("00101040", 16), VR.LO, ""); //  Patient’s Address
        object.putString(Integer.parseInt("001021C0", 16), VR.US, ""); //  Pregnancy Status

        object.putString(Integer.parseInt("00204000", 16), VR.LT, ""); //  Image Comments

        object.putString(Integer.parseInt("00321032", 16), VR.PN, ""); //  Requesting Physician
        object.putString(Integer.parseInt("00321033", 16), VR.LO, ""); //  Requesting Service

        return (newname + "_"
                + //initials
                getSeriesDescription(object)
                + // series description
                "_" + object.getString(Integer.parseInt("00200011", 16))
                + // series number
                "_" + object.getString(Integer.parseInt("00200013", 16))); // intance number
    }

    public static String getSeriesDescription(DicomObject object) {
        String res;
        if (object.contains(Integer.parseInt("0008103E", 16))) {
            res = object.getString(Integer.parseInt("0008103E", 16));
        } else {
            res = "NoSeriesDescriptionFound";
        };
        return res;
    }

    public static double getSpacingBetweenSlices(DicomObject object) {
        double res;
        if (object.contains(Integer.parseInt("00180088", 16))) {
            res = object.getDouble(Integer.parseInt("00180088", 16));
        } else {
            res = Double.NaN;
        }
        return res;
    }

    public static String getStudyDate(DicomObject object) {
        String res;
        if (object.contains(Integer.parseInt("00080020", 16))) {
            res = object.getString(Integer.parseInt("00080020", 16));
        } else {
            res = "NoStudyDateFound";
        };
        return res;

    }

    public static String getStudyTime(DicomObject object) {
        String res;
        if (object.contains(Integer.parseInt("00080030", 16))) {
            res = object.getString(Integer.parseInt("00080030", 16));
        } else {
            res = "NoStudyTimeFound";
        };
        return res;
    }

    public static String getSeriesTime(DicomObject object) {
        String res;
        if (object.contains(Integer.parseInt("00080031", 16))) {
            res = object.getString(Integer.parseInt("00080031", 16));
        } else {
            res = "NoSeriesTimeFound";
        };
        return res;
    }

    public static String getManufacturer(DicomObject object) {
        String res;
        if (object.contains(Integer.parseInt("00080070", 16))) {
            res = object.getString(Integer.parseInt("00080070", 16));
        } else {
            res = "NoManufacturer";
        };
        return res;
    }

    public static String getModelName(DicomObject object) {
        String res;
        if (object.contains(Integer.parseInt("00081090", 16))) {
            res = object.getString(Integer.parseInt("00081090", 16));
        } else {
            res = "NoModelNameFound";
        };
        return res;
    }

    public static String getSOPClassUID(DicomObject object) {
        String res;
        if (object.contains(Integer.parseInt("00080016", 16))) {
            res = object.getString(Integer.parseInt("00080016", 16));
        } else {
            res = "NoSOPClassUID";
        };
        return res;

    }

    public static String getSOPInstanceUID(DicomObject object) {
        String res;
        if (object.contains(Integer.parseInt("00080018", 16))) {
            res = object.getString(Integer.parseInt("00080018", 16));
        } else {
            res = "NoSOPInstanceUID";
        };
        return res;
    }

    public static String getSeriesInstanceUID(DicomObject object) {
        String res;
        if (object.contains(Integer.parseInt("0020000E", 16))) {
            res = object.getString(Integer.parseInt("0020000E", 16));
        } else {
            res = "NoSeriesInstanceUID";
        };
        return res;

    }

    public static Point3D getNormalVector(DicomObject object) {
        double[] orientParts = object.getDoubles(Tag.ImageOrientationPatient);
        Point3D xVect = new Point3D(orientParts[0], orientParts[1], orientParts[2]);
        Point3D yVect = new Point3D(orientParts[3], orientParts[4], orientParts[5]);
        return xVect.computeNormalVector(yVect);
    }

    public static String getStudyID(DicomObject object) {
        String res;
        if (object.contains(Integer.parseInt("00200010", 16))) {
            res = object.getString(Integer.parseInt("00200010", 16));
        } else {
            res = "NoStudyIDFound";
        };
        return res;
    }

    public static String getImageType(DicomObject object) {
        String res;
        if (object.contains(Integer.parseInt("00080008", 16))) {
            DicomElement element = object.get(Integer.parseInt("00080008", 16));
            res = element.getValueAsString(null, 0);
        } else {
            res = "NoImageTypeFound";
        };
        return res;
    }

    public static String getAnonymizedName(DicomObject object, String UPN) {
        String res;
        if (object.contains(Integer.parseInt("00100010", 16)) && object.contains(Integer.parseInt("00100030", 16))) {
            String name = object.getString(Integer.parseInt("00100010", 16));
            String year = object.getString(Integer.parseInt("00100030", 16));
            res = createInitials(name, year, UPN);
        } else if (object.contains(Integer.parseInt("00100010", 16))) {
            String name = object.getString(Integer.parseInt("00100010", 16));
            res = createInitials(name, "9999", UPN);
        } else {
            res = "NoNameFound";
        };
        return removeUmlaut(res);
    }

    public static String removeUmlaut(String res) {
        res = res.replace("ä", "ae");
        res = res.replace("Ä", "AE");
        res = res.replace("Ö", "OE");
        res = res.replace("ö", "oe");
        res = res.replace("ü", "ue");
        res = res.replace("Ü", "UE");
        return res;
    }

    public static int getSeriesNumber(DicomObject object) {
        int res;
        if (object.contains(Integer.parseInt("00200011", 16))) {
            res = object.getInt(Integer.parseInt("00200011", 16));
        } else {
            res = 0;
        };
        return res;
    }

    public static String getProtocolName(DicomObject object) {
        String res;
        if (object.contains(Integer.parseInt("00181030", 16))) {
            res = object.getString(Integer.parseInt("00181030", 16));
        } else {
            res = "NoProtocolNameFound";
        };
        return res;
    }

    public static String getContrast(DicomObject object) {
        return object.getString(Integer.parseInt("00180010", 16));
    }

    public static void saveDicom(String file, DicomObject object) {
        try {
            DicomOutputStream dos = new DicomOutputStream(new File(file));
            dos.writeDicomFile(object);
            dos.close();
        } catch (Exception e) {
            System.out.println(e.getMessage());
            System.exit(0);
        }
    }

    public static DicomHeader getHeader(DicomObject object) {
        DicomHeader header = new DicomHeader();
        exploreStruct(object, header, null);
        return header;
    }

    private static void exploreStruct(DicomObject object, DicomHeader header, String Parent) {
        DicomHeaderLine line = new DicomHeaderLine();
        if (object == null) {
            System.out.println(Parent + " is empty!");
        } else {
            Iterator iter = object.datasetIterator();
            while (iter.hasNext()) {
                DicomElement element = (DicomElement) iter.next();
                int tag = element.tag();
                try {
                    line.clear();
                    line.VR = object.vrOf(tag).toString();
                    line.Name = object.nameOf(tag);
                    line.VM = String.valueOf(object.vm(tag));

                    if (TagUtils.toString(tag).contains("(7FE1,1010)")
                            || TagUtils.toString(tag).contains("(5600,0020)")) {
                        line.ValueByte = element.getBytes();
                    }
                    try {
                        line.Value = element.getValueAsString(null, 0);
                    } catch (UnsupportedOperationException ex) {
                        line.Value = "Unsupported";
                    }

                    if (line.VR.equals("SQ")) {
                        if (element.hasDicomObjects()) {
                            if (element.hasItems()) {//if item in metadata
                                line.Value = "Sequence of Items";
                                if (element.countItems() > 1) {
                                    for (int i = 0; i < element.countItems(); i++) {
                                        exploreStruct(element.getDicomObject(i), header, TagUtils.toString(tag) + "[" + String.valueOf(i) + "]");
                                    }
                                } else {
                                    exploreStruct(element.getDicomObject(), header, TagUtils.toString(tag));
                                }

                                element.countItems();
                            }
                        }
                    }
                    if (Parent != null) {
                        header.put(Parent + "->" + TagUtils.toString(tag), line.clone());
                    } else {
                        header.put(TagUtils.toString(tag), line.clone());
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }
    }

    public static double getTableOffset( DicomObject object )
    {
        double tableOffset;
        if ( object.contains(Integer.parseInt( "00189904", 16 ) ) ) 
        {
            tableOffset = object.getDouble( Integer.parseInt( "00189904", 16 ) );
        } 
        else 
        {
            tableOffset = Double.NaN;
        }
        return tableOffset;
    }
    
    public static void readSiemensData( DicomObject object, DicomHeader header ) {
        byte[] res;
        if (object.contains(Integer.parseInt("00291120", 16))) {
            res = object.getBytes(Integer.parseInt("00291120", 16));
            try {
                byte[] beginmark = "### ASCCONV BEGIN".getBytes();
                byte[] endmark = "### ASCCONV END".getBytes();
                int posaux = 0;
                int startpos = 0;
                int endpos = 0;
                int i = 0;
                while (i < res.length) {
                    if (startpos == 0) {
                        if (Byte.compare(res[i], beginmark[posaux]) == 0) {
                            posaux++;
                        } else {
                            posaux = 0;
                        }
                        if (posaux == beginmark.length) {
                            startpos = i;
                            posaux = 0;
                        }
                    } else {
                        if (Byte.compare(res[i], endmark[posaux]) == 0) {
                            posaux++;
                        } else {
                            posaux = 0;
                        }
                        if (posaux == endmark.length) {
                            endpos = i - endmark.length;
                            i = res.length;
                        }
                    }
                    i++;
                }
                res = Arrays.copyOfRange(res, startpos, endpos);
                String stringaux = new String(res, "ASCII");
                String[] lines = stringaux.split("\n");
                String[] parts;
                DicomHeaderLine hline = new DicomHeaderLine();
                for (int j = 1; j < lines.length; j++) {
                    parts = lines[j].split("((\\s+)|(\\t))\\=((\\s+)|(\\t))");
                    if (parts.length == 2) {
                        if (!parts[0].isEmpty() && !parts[1].isEmpty()) {
                            hline.VR = "Siemens";
                            if (parts[1].startsWith("\"\"")) {
                                hline.VR = "Siemens_String";
                                parts[1] = parts[1].replace("\"\"", "");
                            }
                            hline.Value = parts[1];
                            header.put(parts[0], hline.clone());
                        }
                    }
                }

            } catch (UnsupportedEncodingException ex) {
                Logger.getLogger(DicomHandler.class.getName()).log(Level.SEVERE, null, ex);
            }
        };
    }

    public static int getInstanceNr(String file) {
        DicomObject object = null;
        try {
            DicomInputStream dis = new DicomInputStream(new File(file));
            object = dis.readDicomObject();
            dis.close();
        } catch (Exception e) {
            System.out.println(e.getMessage());
            System.exit(0);
        }

        return Integer.valueOf(object.getString(Integer.parseInt("00200013", 16)));
    }

    public static int getInstanceNr(DicomObject object) {
        int res;
        if (object.contains(Integer.parseInt("00200013", 16))) {
            res = object.getInt(Integer.parseInt("00200013", 16));
        } else {
            res = -1;
        };
        return res;
    }
}
