/*
 * File: $Id: BrainVoyagerVOIFileParserAndLoader.java,v 1.2 2006/10/25 08:27:01 hans Exp $
 * History
 * Date: 03.06.2009  Authors: Hans Slotboom
 *
 * Changes:
 *
 * Copyright (C) 2003-2009 by
 *
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

package org.spectrim.lib;

import java.io.IOException;
import java.io.RandomAccessFile;

/**
 *
 * The code in this class allows the parsing of a .voi file of Brainvoyager;
 * The caller can get access to the 3D overlay array of boolean type
 *
 * @author slotboom
 */
public class BrainVoyagerVOIFileParserAndLoader
{
    /**
     * Constructor of BrainVoyagerVOIFileParserAndLoader.
     * @param voiFileReference
     * @param numberOfSlices
     * @param imageWidth
     * @param imageHeight
     */
    public BrainVoyagerVOIFileParserAndLoader( String voiFileReference, int numberOfSlices, int imageWidth, int imageHeight )
    {
        m_voiFileReference = voiFileReference;
        m_brainVoyagerOverlays = new boolean[numberOfSlices][imageWidth][imageHeight];
    }



    /**
     * Returns true if the string can be parsed as Integer, else false
     * @param str
     * @return
     */
    public boolean isParsableToInt( String str )
    {
        try
        {
            Integer.parseInt( str );
            return true;
        }
        catch( NumberFormatException nfe )
        {
            return false;
        }
    }

    /**
     * Loads the voi file.
     * @return the content of the file as a 3D array of boolean-type
     */
    public boolean[][][] loadBrainVoyagerVOIFile( )
    {
        try
        {
            m_voiFile = new RandomAccessFile( m_voiFileReference, "r" );

            String str = m_voiFile.readLine( );
            str = m_voiFile.readLine( );

            // check whether the file is a valid .voi-file
            // if the second line contains the string  "FileVersion:"
            // we assume the file to be valid
            if ( str.contains( "FileVersion:" ) == true )
            {
                m_brainVoyagerOverlays = parseRestOfFile( );
            }
        }
        catch ( IOException e )
        {
            e.printStackTrace( );
            // System.out.println( "IOException while reading " + m_voiFileReference );
            return null;
        }
        return m_brainVoyagerOverlays;
    }


    /**
     * Does the factual parsing of the file.
     * @return
     * @throws java.io.IOException
     */
    public boolean[][][] parseRestOfFile( ) throws IOException
    {

        String str = m_voiFile.readLine( );
        int cntr = 0;
        int x, y, z;
        String dummy1,dummy2,dummy3;

        while ( str != null )
        {
            
            // initialize dummies
            dummy1 = "";
            dummy2 = "";
            dummy3 = "";
            
            if ( str.length( ) > 0 )
            {
                if ( str.lastIndexOf( " " ) > 0 )
                {
                    dummy3 = str.substring( str.lastIndexOf( " " ), str.length( ) );
                    dummy3 = dummy3.trim( );
                
                    // make the string shorter: remove last part
                    str = str.substring( 0, str.lastIndexOf( " " ) );
                }

                if ( str.lastIndexOf( " " ) > 0 )
                {

                    dummy2 = str.substring( str.lastIndexOf( " " ) + 1, str.length( ) );
                    dummy2.trim( );

                    dummy1 = str.substring( 0, str.lastIndexOf( " " ) );
                    dummy1 = dummy1.trim( );
                }
                
                // only if all three strings have non zero length we can try to convert them to integer ...
                if ( ( isParsableToInt( dummy1 ) == true ) && ( isParsableToInt( dummy2 ) == true ) && ( isParsableToInt( dummy3 ) == true ) )
                {
                    x = new Integer( dummy1 ).intValue( );
                    y = new Integer( dummy2 ).intValue( );
                    z = new Integer( dummy3 ).intValue( );
                    // System.out.println( "pixel " + (cntr++) + "  x = " + x +" y =" + y + "z = " + z );
                    m_brainVoyagerOverlays[z][x][y] = true;
                    
                }
            }

            // read the next line from the voi file..
            str = m_voiFile.readLine( );
        }

        // return the values contained in the file..
        return m_brainVoyagerOverlays;
    }



    // Attributes
    private final String  m_voiFileReference;
    private RandomAccessFile  m_voiFile;

    private boolean[][][] m_brainVoyagerOverlays;

}
