/*
 * File: $Id: DualProgressBar.java,v 1.2 2006/11/09 10:39:27 hans Exp $
 * History
 * Date: 30.11.2002 Authors: Hans SLotboom  Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */
package org.spectrim.lib;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.JDialog;
import javax.swing.JProgressBar;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.BoxLayout;
import javax.swing.BorderFactory;
import javax.swing.JButton;

public class DualProgressBar extends JDialog implements ActionListener
{

    public final static int ONE_SECOND = 1000;

    
    public final static int    FORCE_QUIT      = 1;
    public final static String FORCE_QUIT_CMD   = "Force quit";
    
    private JProgressBar    m_upperProgressBar;
    private JLabel          m_upperTaskOutput;

    // two subpanels to put the progressbars and the texts
    private JPanel      m_upperPanel;
    private JPanel      m_lowerPanel;
    private JPanel      m_forceThreadQuitButtonPanel;
    private JButton     m_forceProcessQuitButton;
    
    private JProgressBar            m_lowerProgressBar;
    private JLabel                  m_lowerTaskOutput;
    private DualProgressBarListener listener;

    
    /**
     * Notifies actionEvent to all registered listeners.
     */
    private void notifyActionListeners( Object obj, int eventType, String actionCommand ) 
    {
        Iterator it = m_actionListeners.iterator();
        while ( it.hasNext( ) ) 
        {
            ( ( ActionListener) it.next( ) ).actionPerformed( new ActionEvent( obj, eventType, actionCommand ) );
        }
    }
    
    /**
     * Adds an ActionListener to the list of listeners
     * @param listener
    */
    public void addActionListener( ActionListener listener )
    {
        m_actionListeners.add( listener );
    }
    
    // Listeners to this object
    private Vector<ActionListener> m_actionListeners = new Vector<>( );
    
    
   

    /**
     * Constructor to construct a dual progress bar.
     * @param parent
     * @param total_upper_progress_steps
     * @param total_lower_progress_steps
     * @param prefSize
     * @param displayTerminateProcessesButton
     */
    public DualProgressBar( JFrame parent,
                            int total_upper_progress_steps,
                            int total_lower_progress_steps,
                            Dimension prefSize,
                            boolean displayTerminateProcessesButton ) 
    {

        WindowListener exitListener = new WindowAdapter( ) 
        {
            @Override
            public void windowClosing( WindowEvent e ) 
            {
                if ( listener != null ) 
                {
                    listener.stopMounting( );
                }
                dispose( );
            }
        };
        this.addWindowListener( exitListener );
        m_upperPanel = new JPanel( );

        m_upperPanel.setLayout( new BoxLayout( m_upperPanel, BoxLayout.Y_AXIS ) );

        m_upperProgressBar = new JProgressBar( 0, total_upper_progress_steps );
        m_upperProgressBar.setValue(0);
        m_upperProgressBar.setStringPainted(true);
        m_upperProgressBar.setToolTipText( "Indicates the progress in percent of total process" );

        m_upperPanel.add( m_upperProgressBar );

        m_upperTaskOutput = new JLabel("                              " );
        m_upperTaskOutput.setAlignmentX( JLabel.CENTER_ALIGNMENT );
        m_upperTaskOutput.setAlignmentY( JLabel.CENTER_ALIGNMENT );

        m_upperPanel.add( m_upperTaskOutput );

        m_upperPanel.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );

        m_lowerPanel = new JPanel( );

        m_lowerPanel.setLayout( new BoxLayout( m_lowerPanel, BoxLayout.Y_AXIS ) );

        m_lowerProgressBar = new JProgressBar( 0, total_lower_progress_steps );
        m_lowerProgressBar.setValue( 0 );
        m_lowerProgressBar.setStringPainted( true );
        m_lowerProgressBar.setToolTipText( "Indicates the progress in percent of a sub process" );
        m_lowerPanel.add( m_lowerProgressBar );

        m_lowerTaskOutput = new JLabel( "                              " );
        m_lowerTaskOutput.setAlignmentX( JLabel.CENTER_ALIGNMENT );
        m_lowerTaskOutput.setAlignmentY( JLabel.CENTER_ALIGNMENT );

        m_lowerPanel.add( m_lowerTaskOutput );

        m_lowerPanel.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );

        
        m_forceThreadQuitButtonPanel = new JPanel( );
//        m_forceThreadQuitButtonPanel.setLayout( new BoxLayout( m_forceThreadQuitButtonPanel, BoxLayout.Y_AXIS ) );
//        
//        m_forceThreadQuitButtonPanel.setAlignmentX( JLabel.CENTER_ALIGNMENT );
//        m_forceThreadQuitButtonPanel.setAlignmentY( JLabel.CENTER_ALIGNMENT );
        m_forceProcessQuitButton = new JButton( "Terminate running processes immediately!" );
        m_forceProcessQuitButton.addActionListener( this );
        
        m_forceThreadQuitButtonPanel.setBorder( BorderFactory.createEmptyBorder( 20, 20, 20, 20 ) );
        m_forceThreadQuitButtonPanel.add( m_forceProcessQuitButton );
         
        if ( displayTerminateProcessesButton == true )
        {
            prefSize.height = 10 + ( prefSize.height * 3 )/ 2;
        }
        
        JPanel contentPane = new JPanel( );
        
        setLayout( new BorderLayout( ) );
        
        contentPane.setLayout( new BorderLayout( ) );
        
        contentPane.setPreferredSize( prefSize );
        contentPane.setMinimumSize( prefSize );
        contentPane.setMaximumSize( prefSize );

        contentPane.add( m_upperPanel, BorderLayout.NORTH );
        contentPane.add( m_lowerPanel, BorderLayout.CENTER );
        
        if ( displayTerminateProcessesButton == true )
        {
            contentPane.add( m_forceThreadQuitButtonPanel, BorderLayout.SOUTH ); 
        }
        
        contentPane.setBorder( BorderFactory.createEmptyBorder( 20, 20, 20, 20 ) );

        setContentPane( contentPane );
        pack( );
        
        
        if ( parent != null ) 
        {
            Point centerParent = new Point( parent.getX( ) + ( parent.getWidth( ) - getWidth( ) ) / 2,
                                            parent.getY( ) + ( parent.getHeight( ) - getHeight( ) ) / 2 );
            this.setLocation( centerParent );
        }
        
        setVisible( true );
        
    }

    /**
     * Sets the upper progress bar to indicate value progress.
     * @param progress
     */
    public void setUpperProgress( int progress ) 
    {
        // assure visibility
        if ( progress == 1 ) 
        {
            setVisible(true);
        }
        m_upperProgressBar.setValue( progress );
    }

    /**
     * Sets the upper label to indicate value progress.
     * @param labeltext
     */
    public void setUpperLabel(String labeltext) {
        // assure visibility
//        setVisible(true);
        m_upperTaskOutput.setText(labeltext);
    }

    /**
     * Sets the lower progress bar to indicate value progress.
     * @param progress
     */
    public void setLowerProgress(int progress) {
        // assure visibility
        if (progress == 1) {
            setVisible(true);
        }
        m_lowerProgressBar.setValue(progress);
    }

    /**
     * Sets the lower label to indicate value progress.
     * @param labeltext
     */
    public void setLowerLabel( String labeltext ) {
        // assure visibility
//        setVisible( true );
        m_lowerTaskOutput.setText(labeltext);
    }

    /**
     * Method allowing the upper progress bar to be set visible after
     * construction
     */
    public void setUpperProgressVisible(boolean visible) {
        m_upperTaskOutput.setVisible(visible);
        m_upperProgressBar.setVisible(visible);
    }

    /**
     * Method allowing the upper progress bar to be set visible after
     * construction
     */
    public void setLowerProgressVisible(boolean visible) {
        m_lowerTaskOutput.setVisible(visible);
        m_lowerProgressBar.setVisible(visible);
    }

    public void setListener(DualProgressBarListener listener) {
        this.listener = listener;
    }

    // For proper serialization
    private static final long serialVersionUID = 1499324591289393L;

    @Override
    public void actionPerformed( ActionEvent e ) 
    {
        if ( e.getSource( ) == m_forceProcessQuitButton )
        {
            notifyActionListeners( this, FORCE_QUIT, FORCE_QUIT_CMD );
        }
    }
}
