/*
 * File: $Id: FileUtils.java,v 1.1.1.1 2006/04/19 09:38:48 hans Exp $
 * History
 * Date: 30.11.2002     Authors: Martin Buerki  Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

package org.spectrim.lib;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.RandomAccessFile;

public final class FileUtils 
{

    public static boolean fileExists( String fname ) 
    {
	File f = new File( fname );
	return f.exists( );
    }

    public static boolean isDirectory( String path )
    {
	File f = new File( path );
	return f.isDirectory( );
    }

    public static void fileDelete( String fname )
    {
	File f = new File( fname );
	f.delete( );
    }

    public static String findFirst( String directory, String pattern ) 
    {
	File f = new File( directory );
	GenericFilenameFilter ff = new GenericFilenameFilter( pattern );
	String[] l = f.list( ff );
	if ( l.length == 0 ) 
        {
            return null;
        }
	String firstName = l[0];
	for ( int k=1; k<l.length; k++ ) 
        {
	    if ( Strings.stringCompare(firstName, l[k]) > 0 ) 
            { 
                firstName = l[k]; 
            }
	}
	return firstName;
    }

    public static long getFileSize( String fname )
    {
	File f = new File( fname );
	return f.length( );
    }

    public static long getUncompressedFileSize( String fname ) throws FileNotFoundException, IOException 
    {
        if ( !fileIsZipped( fname ) ) return getFileSize( fname );

        long fsize;
        long n = getFileSize( fname ) - 4;
        fsize = 0;
        RandomAccessFile f = new RandomAccessFile( fname, "r" );
        f.seek(n);
        for ( int k = 0; k < 4; k++ )
        {
            fsize += f.read( ) * ( 1 << ( 8 * k ) );
        }
        f.close( );
        System.out.println( "Dicom file size : " + fsize );
        return fsize;
    }

    public static boolean fileIsZipped( String fileName ) 
    {
        return fileName.endsWith( ".gz" );
    }
    
    /**
     * Copies file "in" to file "out"
     * @param in
     * @param out
     * @throws java.lang.Exception
     */
    public static void copyFile( File in, File out ) throws Exception 
    {
        FileInputStream fis  = new FileInputStream( in );
        FileOutputStream fos = new FileOutputStream( out );
        try 
        {
            byte[] buf = new byte[1024];
            int i = 0;
            while ((i = fis.read(buf)) != -1) 
            {
                fos.write(buf, 0, i);
            }
        } 
        catch ( Exception e ) 
        {
            throw e;
        }
        finally 
        {
            if ( fis != null ) fis.close( );
            if ( fos != null ) fos.close( );
        }
    }
    /**
     * Workaround for not properly working JAVA command File::renameTo()
     * @param in
     * @param out
     * @throws java.lang.Exception
     */
    public static void renameFile( File in, File out ) throws Exception 
    {
        FileInputStream fis  = new FileInputStream( in );
        FileOutputStream fos = new FileOutputStream( out );
        try 
        {
            byte[] buf = new byte[1024];
            int i = 0;
            while ((i = fis.read(buf)) != -1) 
            {
                fos.write(buf, 0, i);
            }
        } 
        catch ( Exception e ) 
        {
            throw e;
        }
        finally 
        {
            if ( fis != null ) fis.close( );
            if ( fos != null ) fos.close( );
        }
        in.delete( );
    }
}