/*
 * File: $Id: Globals.java,v 1.1.1.1 2006/04/19 09:38:48 hans Exp $
 * History
 * Date: 30. November 2002, 08:03  Authors: Martin Buerki
 *
 * Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */


// TODO needs elimination of all code not relavant for SpectrIm

package org.spectrim.lib;

import java.io.RandomAccessFile;
import java.io.IOException;
  
public class Globals
{

    /**
     * The constructor is private: this is a singleton class.
     */
    private Globals( )
    {
        // private default constructor
    }

    /**
     * Returns the one and only instance of OrderedBaseLineRangesDataModel.
     */
    public static Globals getInstance( )
    {
        if ( m_theInstance == null )
        {
            m_theInstance = new Globals( );
        }
        return m_theInstance;
    }
    
    /**
     * Returns the whether there should be a confirmation after saving.
     * @return
     */
    public boolean getConfirmAfterSave( )
    {
        return m_confirmAfterSave;
    }

    /**
     * Sets confirm after save boolean.
     * @param conf
     */
    public void setConfirmAfterSave( boolean conf )
    {
        m_confirmAfterSave = conf;
    }

    // Attributes
    private static Globals  m_theInstance;

    public final String  APP_NAME = "SCANalyze";
    public final String  APP_VERSION = "_ Version 5.0.r625";

    public final String  settingsFileName = ".perf-eval";
    public              boolean m_isJMRUIRelease = true;

    // file formats
    public final static int     NUMARIS     = 0;
    public final static int     ACR_NEMA    = 1;
    public final static int     DICOM       = 2;
    public final static int     CUSTOM      = 3;

    public final String[] fileFormats = { "NUMARIS",
                                          "ACR/NEMA",
                                          "DICOM",
                                          "CUSTOM" };

    // sort order
    public final static int SORTED_BY_POSITION  = 0;
    public final static int SORTED_BY_TIME      = 1;

    // naming schemes
    public final static int NSCH_NONE       = -1;
    public final static int NSCH_SHORT      = 0;
    public final static int NSCH_DICOM      = 1;
    public final static int NSCH_SIEMENS    = 2;
    public final String[] namingSchemes = { "Short (name_a-00001)",
                                            "DICOM (NAME_sequence-0001-0001)",
                                            "Siemens (1234-1-1)" };

    public String dcm2imaCommand            = "/usr/bin/dcm2ima";
    public String xdcm2imaCommand           = "/usr/bin/xdcm2ima";
    public String xdcmsortCommand           = "/usr/bin/xdcmsort";
    public String xdcmsortmoveCommand       = "/usr/bin/xdcmsortmove";

    // Default path where the xdcmsortmove application should get the
    // input dicom images from
    public String imageSourcePath = "~/PerfusionImages";
    
    // Default path where the xdcmsortmove application should put the
    // renamed images
    public String imageDestinationPath = "~/PerfusionImages";
    
    public String defaultPath;
    
    public float    m_defaultDwell;
    public int      m_defaultDelay;
    public int      m_defaultMeanThreshold;
    public boolean  m_defaultConfirmAfterSave;

    public int m_defaultFileFormat   = DICOM;
    public int m_defaultSortOrder    = SORTED_BY_POSITION;
    public int m_defaultNamingScheme = NSCH_DICOM;

    public float    m_relaxivity = 2.7f;
    public float    m_echoTimeMs = 47.0f;
    public float    m_repetitionTime = 1440f;
    public float    m_dwell;
    public int      m_delay;
    public int      m_meanThreshold;
    public boolean  m_confirmAfterSave;
    
    // added for morphologic parameters
    public int      m_morphologicKernalSizeMR = 12;
    public int      m_minimumTressholdPercentageMR = 2;

    public int      m_defaultMorphologicKernalSizeMR = 12;
    public int      m_defaultMinimumTressholdPercentageMR = 2;


    /**
     * Sets the default values.
     */
    public void setDefaultValues( ) 
    {
        m_dwell = m_defaultDwell;
        m_delay = m_defaultDelay;
        m_meanThreshold = m_defaultMeanThreshold;
        m_confirmAfterSave = m_defaultConfirmAfterSave;
        
        m_morphologicKernalSizeMR = m_defaultMorphologicKernalSizeMR;
        m_minimumTressholdPercentageMR = m_defaultMinimumTressholdPercentageMR;
    }
    
    /**
     * Load the user settings (shell commands for dicom tools, dwell, delay,
     * mean threshold etc.) from the file ~user/.perf-eval. If this file
     * does not exist, some default values are used.
     * The loaded values are copied into working variables that can be
     * modified from the OptionsDlg class. Thus the loaded values can always
     * be restored unless the method <code>saveSettings()</code> is called.
     *
     * @see	saveSettings()
     */
    public boolean loadSettings( ) 
    {
	
        RandomAccessFile f;
        boolean state = true;

        String fName = System.getProperty( "user.home" ) +
                       System.getProperty( "file.separator" ) +
                       settingsFileName;

        try
        {
            f = new RandomAccessFile( fName, "r" );
            
            dcm2imaCommand              = f.readLine( );
            xdcm2imaCommand             = f.readLine( );
            xdcmsortCommand             = f.readLine( );
            
            // Get the location where the xdcmsortmove app is located
            xdcmsortmoveCommand         = f.readLine( );
            
            // Get the source path where the xdcmsortmove app should
            // get its input from
            imageSourcePath             = f.readLine( );

            // Get the destination path where the xdcmsortmove app should
            // put its output
            imageDestinationPath        = f.readLine( );
            
            m_defaultDwell              = f.readFloat( );
            m_defaultDelay              = f.readInt( );
            
            // default path where the datafiles are expected to be
            defaultPath                 = f.readLine( );
            
            m_defaultMeanThreshold      = f.readInt( );
            m_defaultConfirmAfterSave   = f.readBoolean( );

            f.close( );
        }
        catch ( IOException e )
        {
            System.out.println( "WARNING: Reading settings failed. Using hard coded values." );
	    
            dcm2imaCommand       = "/usr/bin/dcm2ima";
            xdcm2imaCommand      = "/usr/bin/xdcm2ima";
            xdcmsortCommand      = "/usr/bin/xdcmsort";
            xdcmsortmoveCommand  = "/usr/bin/xdcmsortmove";
            imageSourcePath      = "~/PerfusionImages";
            imageDestinationPath = "~/PerfusionImages";
            
            m_defaultDwell = 1.44f;
            m_defaultDelay = 0;
            m_defaultMeanThreshold = 80;
            m_defaultConfirmAfterSave = false;
            state =  false;
        }
        
        m_dwell = m_defaultDwell;
        m_delay = m_defaultDelay;
        m_meanThreshold = m_defaultMeanThreshold;
        m_confirmAfterSave = m_defaultConfirmAfterSave;
        return state;
    }

    /**
     * Saves the default settings.
     * @return
     */
    public boolean saveSettings( ) 
    {
        RandomAccessFile f;

        String fName = System.getProperty( "user.home" ) +
	    System.getProperty( "file.separator" ) +
	    settingsFileName;

        try
        {
            f = new RandomAccessFile( fName, "rw" );
            f.writeBytes( dcm2imaCommand );
            f.writeByte( '\n' );
            f.writeBytes( xdcm2imaCommand );
            f.writeByte( '\n' );
            f.writeBytes( xdcmsortCommand );
            f.writeByte( '\n' );
            f.writeBytes( xdcmsortmoveCommand );
            f.writeByte( '\n' );
            f.writeBytes( imageSourcePath );
            f.writeByte( '\n' );
            f.writeBytes( imageDestinationPath );
            f.writeByte( '\n' );
            
            f.writeFloat( m_dwell );
            f.writeInt( m_delay );
            f.writeBytes( defaultPath );
            f.writeByte( '\n' );
            f.writeInt( m_meanThreshold );
            f.writeBoolean( m_confirmAfterSave );
            f.close( );
        }
        catch ( IOException e )
        {
            System.out.println( "WARNING: Could not save the settings." );
            // e.printStackTrace( );
            return false;
        }
        return true;
    }

}