/*
 * File: $Id: IntegerRange.java,v 1.1.1.1 2006/04/19 09:38:48 hans Exp $
 * History
 * Date: 17.03.2006, 12:06  Authors: Hans Slotboom  
 *
 * Changes:
 *
 * Copyright (C) 2006 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */
 

/*
 * Range.java
 *
 * Created on March 17, 2006, 9:29 AM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */
 
package org.spectrim.lib;



import java.io.RandomAccessFile;
import java.io.IOException;


/**
 *
 *  Class defining an integer range and its properties
 *
 *  @author hans
 */
public class IntegerRange 
{
    
    /** Creates a new instance of Range */
    public IntegerRange( ) 
    {
    }
    
    /** Creates a new instance of Range */
    public IntegerRange( int minimum, int maximum ) 
    {
        // check whether minimum > maximum; if not the case than
        // automatic correction of the range!
        if ( minimum < maximum )
        {
            m_minimum = minimum;
            m_maximum = maximum;
        }
        else
        {
            m_minimum = maximum;
            m_maximum = minimum;
        }
    }
    
    /**
     * Returns the minimum
     */
    public int getMinimum( )
    {
        return m_minimum;
    }
    
    /**
     * Returns the maximum
     */
    public int getMaximum( )
    {
        return m_maximum;
    }
    
    /**
     * Sets the minimum
     */
    public void setMinimum( int minimum )
    {
        // automatic correction of wrong order
        if ( minimum < m_maximum )
        {
            m_minimum = minimum;
        }
        else
        {
            m_minimum = m_maximum;
            m_maximum = minimum;
        }
    }
    
    /**
     * Sets the maximum
     */
    public void setMaximum( int maximum )
    {
       
        // automatic correction of wrong order
        if ( maximum > m_minimum )
        {
            m_maximum = maximum;
        }
        else
        {
            m_maximum =  m_minimum;
            m_minimum = maximum;
        }
    }
    
    /**
     * Determines whether two integer ranges intersect
     * Returns 0 if intersects 
     * Returns -1 if "ir" is smaller than this
     * Returns +1 if "ir" is larger than this 
     */
    public int doesIntersect( IntegerRange ir )
    {
        int intersect = 0;
        
        // CASE (I)
        if ( ir.getMaximum( ) < this.m_minimum  )
        {
            intersect = -1;
        }
        
        // CASE (II)
        if ( ir.getMinimum( ) > this.m_maximum )
        {
            intersect = +1;
        }
        // CASE (III)
        if ( ( ir.getMinimum( ) < this.m_minimum  ) && ( ( ir.getMaximum( ) > this.m_minimum  ) ) )
        {
            intersect = 0;
        }
        
        // CASE (IV)
        if ( ( ir.getMinimum( ) < this.m_maximum  ) && ( ( ir.getMaximum( ) > this.m_maximum  ) ) )
        {
            intersect = 0;
        }
        
        // CASE (V)
        if ( ( ir.getMinimum( ) > this.m_minimum  ) && ( ( ir.getMaximum( ) < this.m_maximum  ) ) )
        {
            intersect = 0;
        }
        
        return intersect;
    }
    
    /**
     * Writes attributes to RandomAccessFile
     */
    public void write( RandomAccessFile out )
    {
        try
        {
            out.writeUTF( "<IntegerRange>" );
            
            // write the numeric values
            out.writeInt( m_minimum );
            out.writeInt( m_maximum );
            
            out.writeUTF( "</IntegerRange>" );
        }
        catch ( IOException e )
        {
        }
    }
    
    /**
     * Reads attributes to RandomAccessFile
     */
    public void read( RandomAccessFile in )
    {
        try
        {
           // read the opening tag
            String tag = in.readUTF( ); 
            
            m_minimum = in.readInt( );
            m_maximum = in.readInt( );
            
            // read the closing tag
            tag = in.readUTF( );   
        }
        catch ( IOException e )
        {
            System.out.println( "Cannot read IntegerRange from file!" );
        }
    }
    
    // Attributes
    private int m_minimum = Integer.MIN_VALUE;
    private int m_maximum = Integer.MAX_VALUE;
}
