/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.spectrim.lib;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import javax.swing.JComponent;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JTextArea;

/**
 * Allows the user to define a popup menu
 * @author slotboom
 */
public class JTextAreaWithPopUpMenu extends JTextArea implements ActionListener
{

    
    public JTextAreaWithPopUpMenu( )
    {
        // default constructor
        this.setEditable( false );
        initPopupMenu( );
    }

    /**
     * Adds a popup to the this control
     * @param popupMenu
     */
    private void initPopupMenu( )
    {
        m_popupMenu = new JPopupMenu( );
        m_clearTextArea = new JMenuItem( "Clear text area" );
        m_popupMenu.add( m_clearTextArea );
        m_clearTextArea.addActionListener( this );

        addMouseListener(
           new PopupTriggerMouseListener(
                   m_popupMenu,
                   this
           )
        );
    }

    public void actionPerformed( ActionEvent e )
    {
       if (e.getSource( ) == m_clearTextArea )
       {
           removeLines( );
       }
    }

    /**
     * Removes Lines from control
     */
    public void removeLines( )
    {
        setText( "" );
    }

    public static class PopupTriggerMouseListener extends MouseAdapter
    {
        private JPopupMenu popup;
        private JComponent component;

        public PopupTriggerMouseListener( JPopupMenu popup, JComponent component )
        {
            this.popup = popup;
            this.component = component;
        }

        //some systems trigger popup on mouse press, others on mouse release, we want to cater for both
        private void showMenuIfPopupTrigger( MouseEvent e )
        {
            if ( e.isPopupTrigger( ) )
            {
               popup.show( component, e.getX( ) + 3, e.getY( ) + 3 );
            }
        }

        /**
         * Called whenever the mouse button is pressed
         * @param e
         */
        @Override
        public void mousePressed( MouseEvent e )
        {
            showMenuIfPopupTrigger( e );
        }

        /**
         * Called whenever the mouse buttim is released
         * @param e
         */
        @Override
        public void mouseReleased( MouseEvent e )
        {
            showMenuIfPopupTrigger( e );
        }

    }

    //Attributes
    private JPopupMenu m_popupMenu;

    private JMenuItem m_clearTextArea;
    private int m_maxLines = 0;
}


