/*
 * File: $Id: SortedStringList.java,v 1.1.1.1 2006/04/19 09:38:48 hans Exp $
 * History
 * Date: 7. November 2002, 09:40   Authors: Hans Slotboom  
 *
 * Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

package org.spectrim.lib;


import java.util.Vector;

/**
 * Utility class handling sorting of string lists.
 * @author Johannes Slotboom <johannes.slotboom at insel.ch>
 */

public final class SortedStringList 
{

    private Vector<String> items;

    /**
     * Default constructor.
     */
    public SortedStringList( ) 
    {
        items = new Vector<>( );
    }

    /**
     * Alternative constructor based on the list capacity value.
     * @param initialCapacity 
     */
    public SortedStringList( int initialCapacity ) 
    {
        items = new Vector<String>( initialCapacity );
    }

    /**
     * Alternative constructor based on initial list capacity and capacity 
     * increment value.
     * @param initialCapacity 
     * @param capacityIncrement 
     */
    public SortedStringList( int initialCapacity, int capacityIncrement ) 
    {
        items = new Vector<String>( initialCapacity, capacityIncrement );
    }

    /**
     * Alternative constructor based array a String values. 
     * @param elements
     */
    public SortedStringList( String[] elements ) 
    {
        if ( elements != null )
        {
            items = new Vector<String>( elements.length + 5 );
            addItems( elements );
        }
        else
        {
            items = new Vector<String>( 5 );
            addItems( elements );
        }
    }

    /**
     * Returns the size of the SortedStringList-object.
     * @return 
     */
    public int getSize( ) 
    {
        return items.size( );
    }

    /**
     * Returns the capacity of the SortedStringList-object.
     * @return 
     */
    public int getCapacity( ) 
    {
        return items.capacity( );
    }

    /**
     * Returns the String item at location 'index' of the SortedStringList-
     * object.
     * @param index
     * @return 
     */
    public String getItem( int index )
    {
        if ( index<0 || index >= getSize( ) ) return null;
            return items.elementAt( index );
    }

    /**
     * Adds an String element to the String list.
     * @param element
     * @return 
     */
    public int addItem( String element )
    {
        int size = getSize( );
        int k = 0;
        while ( k < size )
        {
            String listElement = items.elementAt( k );
            if ( Strings.stringCompare( element, listElement ) < 0 ) break;
            k++;
        }
        items.insertElementAt( element, k );
        return k;
    }

    /**
     * Adds multiple items of an String array to the list.
     * @param elements 
     */
    public void addItems( String[] elements )
    {
        if ( elements != null )
        {
            for ( int k = 0; k < elements.length; k++ )
            {
                addItem( elements[k] );
            }
        }
    }

    /**
     * Clears the list.
     */
    public void clear( ) 
    {
        items.removeAllElements( );
    }

}