/*
 * File: $Id: SpinBox.java,v 1.1.1.1 2006/04/19 09:38:48 hans Exp $
 * History
 * Date: 04.02.2003  Authors: Hans Slotboom  Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

package org.spectrim.lib;


import java.awt.Dimension;
import java.awt.Insets;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.BoxLayout;
import javax.swing.BorderFactory;

import javax.swing.event.DocumentListener;
import javax.swing.event.DocumentEvent;

import java.text.DecimalFormat;

import java.util.LinkedList;
import java.util.Iterator;

public class SpinBox extends JPanel
{
    /**
     * Constructs a spinbox object.
     * The Spinbox consists of:
     *
     *          String label            title of the spinbox
     *          int step_buttons        number of buttons (one side!)
     *          int stepsizes[]         array of size step_buttons containing the
     *                                  small/medium/large stepsizes
     *          int range[]             defines the minimum and maximum value
     *          int start_value         value appearng in the text field
     *          int digits              number of characters  of the text field
     */
    public SpinBox( String spinboxLabel, int stepButtons, double[] stepSizes,
                    double minValue, double maxValue,
                    String units, double startValue, int digits, String numberFormat )
    {

        this.spinboxLabel = spinboxLabel;
        this.stepButtons = stepButtons;
        for ( int i = 0 ;  i < stepButtons; ++i )
        {
            this.stepSizes[i] = stepSizes[i];
    	}
        this.minValue = minValue;
        this.maxValue = maxValue;
        this.units = units;
        this.startValue = startValue;
        this.digits = digits;
        this.numberFormat = numberFormat;
        sender = this;
	initGUI( );
  
        if ( startValue >= maxValue )
        {
            startValue = maxValue;
            textField.setValue( startValue );
            maxValueSet( );
        }
        
        if ( startValue <= minValue )
        {
            startValue = minValue;
            textField.setValue( startValue );
            minValueSet( );
        }
    }

    /**
     * If max value is set this routine will disable all step-up buttons
     */
    private void maxValueSet( )
    {
        if ( stepUpSmallButton != null )
        {
            stepUpSmallButton.setEnabled( false );
        }
        if ( stepUpMediumButton != null )
        {
            stepUpMediumButton.setEnabled( false );
        }
        if ( stepUpLargeButton != null )
        {
            stepUpLargeButton.setEnabled( false );
        }
    }

    /**
     * If min value is set this routine will disable all step-down buttons
     */
    private void minValueSet( )
    {
        if ( stepDownSmallButton != null )
        {
            stepDownSmallButton.setEnabled( false );
        }
        if ( stepDownMediumButton != null )
        {
            stepDownMediumButton.setEnabled( false );
        }
        if ( stepDownLargeButton != null )
        {
            stepDownLargeButton.setEnabled( false );
        }
    }

    
    /**
     * Mehod to create Stepbuttons and implements ActionListener
     */
    private JButton createButton( String label, final double delta )
    {
        JButton btn = new JButton( label );
	btn.setPreferredSize( new Dimension( 27, 22 ) );
	btn.setMaximumSize( new Dimension( 27, 22 ) );
	btn.setMinimumSize( new Dimension( 27, 22 ) );
	btn.setMargin( new Insets( 0, 0, 0, 0 ) );
	btn.addActionListener( new ActionListener( )
	{
            public void actionPerformed( ActionEvent e )
            {
            	double new_value = textField.getValue( ) + delta;
		textField.setValue( new_value );
                
                if ( textField.isMaximumValueSet( ) == true )
                {
                    maxValueSet( );
                }
                else
                {
                    if ( stepUpSmallButton != null )
                    {
                        stepUpSmallButton.setEnabled( true );
                    }
                    if ( stepUpMediumButton != null )
                    {
                        stepUpMediumButton.setEnabled( true );
                    }
                    if ( stepUpLargeButton != null )
                    {
                        stepUpLargeButton.setEnabled( true );
                    }
                }

                if ( textField.isMinimumValueSet( ) == true )
                {   
                    minValueSet( );
                }
                else
                {
                    if ( stepDownSmallButton != null )
                    {
                        stepDownSmallButton.setEnabled( true );
                    }
                    if ( stepDownMediumButton != null )
                    {    
                        stepDownMediumButton.setEnabled( true );
                    }
                    if ( stepDownLargeButton != null )
                    {
                        stepDownLargeButton.setEnabled( true );
                    }
                }       
            }
	});
        return btn;
    }
    /**
     * Initializes the GUI
     */
    public void initGUI( )
    {

        titleLabel = new JLabel( spinboxLabel );
        setBorder( BorderFactory.createEmptyBorder( 2, 2, 2, 2 ) );
        titleLabel.setPreferredSize( new Dimension( 95, 22 ) );
        titleLabel.setMaximumSize( new Dimension( 95, 22 ) );
        titleLabel.setMinimumSize( new Dimension( 95, 22 ) );

        textField = new FormattedTextField( new DecimalFormat( numberFormat ), minValue, maxValue );
        textField.setValue( startValue );
        textField.setColumns( digits );
        
        textField.setMaximumSize( new Dimension( 95, 22 ) );
        
        setLayout( new BoxLayout( this, BoxLayout.X_AXIS ) );

        add( titleLabel );

        if ( stepButtons >= 3 )
        {
            stepDownLargeButton = createButton( "<<<", -stepSizes[2] );
            add( stepDownLargeButton );
		}
        if ( stepButtons >= 2 )
        {
            stepDownMediumButton = createButton( "<<", -stepSizes[1] );
            add( stepDownMediumButton );
        }

	stepDownSmallButton = createButton( "<", -stepSizes[0] );
	add( stepDownSmallButton );
	add( textField );

        // Add a Documentlistener to the textfield in order to
        // inform registered listeners that something has changed ..
	textField.getDocument( ).addDocumentListener( new DocumentListener( )
        {
            public void changedUpdate( DocumentEvent e )
            {
            }

            public void insertUpdate( DocumentEvent e )
            {
                Iterator it = m_listeners.iterator( );
                while( it.hasNext( ) )
		{
                    ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( sender, 0, "" ) );
                }
            }

            public void removeUpdate( DocumentEvent e )
            {
                Iterator it = m_listeners.iterator( );
                while( it.hasNext( ) )
		{
                    ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( sender, 0, "" ) );
                }
            }
        });

	stepUpSmallButton = createButton( ">", stepSizes[0] );
	add( stepUpSmallButton );

        if ( stepButtons >= 2 )
        {
            stepUpMediumButton = createButton( ">>", stepSizes[1] );
            add( stepUpMediumButton );
        }
        if ( stepButtons >= 3 )
        {
            stepUpLargeButton = createButton( ">>>", stepSizes[2] );
            add( stepUpLargeButton );
        }

        // add the unit label to the east of the border layout
        unitLabel = new JLabel( " " + units );

        unitLabel.setPreferredSize( new Dimension( 35, 22 ) );
        unitLabel.setMaximumSize( new Dimension( 35, 22 ) );

        add( unitLabel );
    }
    
    /**
     * Sets the size of the unit label
     */
    public void setUnitLabelSize( Dimension unitLabelSize )
    {
	unitLabel.setPreferredSize( unitLabelSize );
	unitLabel.setMaximumSize( unitLabelSize );
    }
    
    /**
     * Sets the maximum and preferred size of the text field that displays
     * the currently selected value
     */
    public void setMaximumAndPreferredTextFieldSize( Dimension textFieldSize )
    {
	textField.setMaximumSize( textFieldSize );
        textField.setMinimumSize( textFieldSize );
	textField.setPreferredSize( textFieldSize );
        updateUI();
    }

    /**
     * Sets the maximum and preferred size of the title label
     */
    public void setMaximumAndPreferredTitleLabelSize( Dimension titleLabelSize )
    {
        titleLabel.setMaximumSize( titleLabelSize );
        titleLabel.setMinimumSize( titleLabelSize );
        titleLabel.setPreferredSize( titleLabelSize );
        updateUI();
    }
    
    /**
     * Adds an ActionListener to the list of action listeners who are notified
     * when something happens with the SpinBox.
     * @param e
     */
    public void addActionListener( ActionListener e )
    {
	m_listeners.add( e );
    }
    
    /**
     * Returns the currently displayed text field value
     * @return 
     */
    public double getTextFieldValue( )
    {
        return textField.getValue( );
    }

    /**
     * Sets the maximum displayable value.
     * @param max_display_value
     */
    public void setMaximumValue( double max_display_value )
    {
        maxValue = max_display_value;
        textField.setMaximumValue( maxValue );
    }

    /**
     * Sets the maximum displayable value
     * @param min_display_value
     */
    public void setMinimumValue( double min_display_value )
    {
        minValue = min_display_value;
    }

    /**
     * Set text field value to a starting value.
     */
    public void setTextField( String text )
    {
        textField.setText( text );
    }
    
    private JLabel titleLabel;
    private JLabel unitLabel;
    private FormattedTextField textField;
    private JButton stepUpSmallButton;
    private JButton stepUpMediumButton;
    private JButton stepUpLargeButton;
    private JButton stepDownSmallButton;
    private JButton stepDownMediumButton;
    private JButton stepDownLargeButton;

    private String spinboxLabel;
    private int stepButtons;
    private double[] stepSizes = new double[3];
    private double minValue;
    private double maxValue;
    private String units;
    private double startValue;
    private int digits;
    private String numberFormat;
    public SpinBox sender;
    
    // Listeners to textfield changes
    private LinkedList<ActionListener> m_listeners = new LinkedList<ActionListener>( );
    
    // For proper serialization
    private static final long serialVersionUID = 1482325898889393L;
}
