/*
 * File: $Id: StringDefinitionControl.java,v 1.1.1.1 2006/04/19 09:38:48 hans Exp $
 * History
 * Date: 03.06.2003, 10:24 AM  Authors: Hans Slotboom  
 *
 * Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

package org.spectrim.lib;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Point;

import java.util.LinkedList;
import java.util.Iterator;

import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import javax.swing.JFrame;
import javax.swing.BoxLayout;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.Box;


/**
 * Utility class that enables renaming of a string variable.
 * @author  hans
 */
public class StringDefinitionControl  extends JFrame implements ActionListener
{
    
    /** 
     * Creates a new instance of StringDefinitionControl. 
     * @param title
     * @param description
     * @param defaultStringValue
     * @param parent
     */
    public StringDefinitionControl( String title, String description, String defaultStringValue, JFrame parent ) 
    {
        super( );
        m_parent = parent;
        setTitle( title );
        
        // The content panel contains all subpanels
        m_contentPanel = new JPanel( );
        m_contentPanel.setLayout( new BorderLayout( ) );
        
        // Define the control panel with its inner life
        m_controlPanel = new JPanel( );
        m_controlPanel.setLayout( new BoxLayout( m_controlPanel, BoxLayout.X_AXIS ) );
        m_controlPanel.setBorder( BorderFactory.createEmptyBorder( 10, 10, 10, 10 ) );
        m_controlPanel.setPreferredSize( new Dimension( 430, 60 ) );
        
        m_descriptionLabel = new JLabel( description );
        m_controlPanel.add( m_descriptionLabel );
        
        m_stringDefintionTextField = new JTextField( defaultStringValue );
        m_stringDefintionTextField.setPreferredSize( new Dimension( 350, 25 ) );
        m_stringDefintionTextField.setMaximumSize( new Dimension( 350, 25 ) );
        
        
        m_controlPanel.add( m_stringDefintionTextField );
        
        m_contentPanel.add( m_controlPanel, BorderLayout.CENTER );
        
        // Define the selection Panel with the Apply and Cancel button
        m_applyButton = new JButton( "Apply" );
        m_applyButton.addActionListener( this );
        m_applyButton.setToolTipText( "Applies the selected zerofilling factor to the currently loaded dataset" );
        m_cancelButton = new JButton( "Cancel" );
        m_cancelButton.addActionListener( this );
        m_cancelButton.setToolTipText( "Cancels this operation without applying the selected zerofilling factor" );
        
        m_selectionPanel = new JPanel( );
        
        m_selectionPanel.setLayout( new BoxLayout( m_selectionPanel, BoxLayout.X_AXIS ) );
        m_selectionPanel.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );
        m_selectionPanel.add( Box.createGlue( ) );
        m_selectionPanel.add( m_applyButton );
        m_selectionPanel.add( Box.createGlue( ) );
        m_selectionPanel.add( m_cancelButton );
        m_selectionPanel.add( Box.createGlue( ) );
        
        m_contentPanel.add( m_selectionPanel, BorderLayout.SOUTH );
        
        // add window listener necessary to handle the window closing 
        // event properly
        addWindowListener( 
            new WindowAdapter( )
            {
                @Override
                public void windowClosing( WindowEvent event )
                {
                   dispose( );
                }
            }
        );
        
        
        setContentPane( m_contentPanel );
        
        pack( );
        if ( m_parent != null )
        {
            // Put the Dialog nicely in the middle of its parent
            Point parentLoc = m_parent.getLocation( );
            Point centerParent = new Point( m_parent.getX( ) + ( m_parent.getWidth( ) / 2 ) - this.getWidth( ) / 2,
                                            m_parent.getY( ) + ( m_parent.getHeight( ) / 2 ) - this.getHeight( ) / 2 );
            setLocation( centerParent );
        }
        setVisible( true );
        
    }
    
    /**
     * Called whenever a registered component raises an action event.
     * @param e 
     */
    @Override
    public void actionPerformed( ActionEvent e ) 
    {
        Object sender = e.getSource( );        
        
        // handle responses on cancel button pressed event
        if ( sender == m_cancelButton )
        {
            Iterator it = m_listeners.iterator( );
             while( it.hasNext( ) )
             {
                ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, CANCEL_EVENT, "" ) );
             }
             dispose( );
        }
        
        // handle responses on apply button pressed event
        if ( sender == m_applyButton )
        {
            Iterator it = m_listeners.iterator( );
             while( it.hasNext( ) )
             {
                ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, ACCEPT_EVENT, "" ) );
             }
             dispose( );
        }
        
        
    }
    
    /**
     * Adds an action listener to the list of action listeners who are notified
     * when something happens within this object.
     * @param e
     */
    public void addActionListener( ActionListener e )
    {
        m_listeners.add( e );
    }


    /**
     *
     * Returns the string defined by user.
     *
     * @return string
     */
    public String getRenamedString( )
    { 
        return m_stringDefintionTextField.getText( );
    }


    // Attributes
    
    JFrame m_parent;
    JPanel m_contentPanel;
    JPanel m_controlPanel;
    JPanel m_selectionPanel;
    
    JLabel m_descriptionLabel;
    JTextField m_stringDefintionTextField;
    JButton m_applyButton;
    JButton m_cancelButton;
    
    // Listeners to changes in this class
    private final LinkedList<ActionListener> m_listeners = new LinkedList<>( );
    
    // Possible Events
    public final static int CANCEL_EVENT = 0;
    public final static int ACCEPT_EVENT = 1;
    
    // For proper serialization
    private static final long serialVersionUID = 9982244496663323L;
}
