/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.main;

/**
 * "Spectrim-side" implementation of the MetaboliteFeaturesInterface
 * @author Johannes Slotboom <johannes.slotboom at insel.ch>
 */
public class MetaboliteFeatures implements MetaboliteFeaturesInterface
{
    
    private double area         = 0.0;
    private double amplitude    = 0.0;  // amplitude in time domain at t=0
    private double frqShift     = 0.0;  // offset frequency
    private double damping      = 0.0;  // Lorentzian damping constant
    private double gdamping     = 0.0;  // Gaussian damping constant
    private double stdDevArea   = 0.0;  // CR-bound
    private double phase        = 0.0;
    private double error        = Double.NaN;
  
    public static final int AREA        = 0;
    public static final int AMPLITUDE   = 1;
    public static final int FRQSHIFT    = 2;
    public static final int DAMPING     = 3;
    public static final int STDDEVAMPL  = 4;
    public static final int PHASE       = 5;
    public static final int ERROR       = 6;
    public static final int GDAMPING    = 7;
    public static final int FIT_QUALITY_NUMBER       = 8;
    
    private double m_fitQualityNumber;

    public int additions = 1;
    
    public MetaboliteFeatures( ) 
    {
        // empty constructor
    }

    /**
     * Returns the feature belonging to number 'feature'.
     * @param feature
     * @return 
     */
    @Override
    public double getFeature( int feature ) 
    {
        switch ( feature )
        {
            case AREA:
                    return area;
            case AMPLITUDE:
                    return amplitude;
            case FRQSHIFT:
                    return frqShift;
            case DAMPING:
                    return damping;
            case STDDEVAMPL:
                    return stdDevArea;
            case PHASE:
                    return phase;
            case ERROR:
                    return error;
            case FIT_QUALITY_NUMBER:
                    return m_fitQualityNumber;
            case GDAMPING:
                    return gdamping;        
            default:
                    return Double.NaN;
        }
    }

    /**
     * Returns the number of features to be called.
     * @return 
     */
    @Override
    public int getNbFeatures( ) 
    {
        return 9;
    }

    /**
     * Returns the different names of the features as an array.
     * @return 
     */
    @Override
    public String[] getFeatureNames( ) 
    {
        String[] feats      = new String[9];
        feats[AREA]         = "Area";
        feats[AMPLITUDE]    = "Amplitude";
        feats[FRQSHIFT]     = "FreqShift";
        feats[DAMPING]      = "LDamping";
        feats[STDDEVAMPL]   = "StDevArea";
        feats[PHASE]        = "Phase";
        feats[ERROR]        = "error";
        feats[FIT_QUALITY_NUMBER]        = "FQN";
        feats[GDAMPING]     = "GDamping"; 
        return feats;
    }

    /**
     * Returns the area attribute.
     * @return 
     */
    @Override
    public double getArea( ) 
    {
        return area;
    }

    /**
     * Sets the area attribute.
     * @param area 
     */
    @Override
    public void setArea( double area ) 
    {
        this.area = area;
    }

    /**
     * Returns the amplitude attribute.
     * @return 
     */
    @Override
    public double getAmplitude( ) 
    {
        return amplitude;
    }

    /**
     * Sets the amplitude attribute.
     * @param amplitude 
     */
    @Override
    public void setAmplitude( double amplitude ) 
    {
        this.amplitude = amplitude;
    }

    /**
     * Returns the frequency shift attribute.
     * @return 
     */
    @Override
    public double getFrqShift( )
    {
        return frqShift;
    }

    /**
     * Sets the frequency shift parameter.
     * @param frqShift 
     */
    @Override
    public void setFrqShift( double frqShift ) 
    {
        this.frqShift = frqShift;
    }
    
    /**
     * Returns the Lorentzian damping constant.
     * @return 
     */
    @Override
    public double getDamping( ) 
    {
        return damping;
    }
    /**
     * Sets the Lorentzian damping constant.
     * @param damping 
     */
    @Override
    public void setDamping( double damping ) 
    {
        this.damping = damping;
    }

     /**
     * Returns the Gaussian damping constant.
     * @return 
     */
    @Override
    public double getGaussianDamping( ) 
    {
        return gdamping;
    }
    
    /**
     * Sets the Gaussian damping constant.
     * @param gdamping 
     */
    @Override
    public void setGaussianDamping( double gdamping ) 
    {
        this.gdamping = gdamping;
    }
    
    
    /**
     * Returns the CR-error of the amplitude parameter.
     * @return 
     */
    @Override
    public double getStdDevArea( )
    {
        return stdDevArea;
    }
    /**
     * Sets the CR-error of the amplitude parameter.
     * @param stdDevArea
     */
    @Override
    public void setStdDevArea( double stdDevArea ) 
    {
        this.stdDevArea = stdDevArea;
    }

    /**
     * Returns the phase parameter.
     * @return 
     */
    @Override
    public double getPhase( ) 
    {
        return phase;
    }

    /**
     * Sets the phase parameter.
     * @param phase 
     */
    @Override
    public void setPhase( double phase ) 
    {
        this.phase = phase;
    }

    /**
     * Returns the error parameter.
     * @return 
     */
    @Override
    public double getError( )
    {
        return error;
    }

    /**
     * Sets the error parameter.
     * @param error 
     */
    @Override
    public void setError( double error )
    {
        this.error = error;
    }

    public void setFitTDFDFitQualityNumber( double fitQualityNumber ) 
    {
        m_fitQualityNumber = fitQualityNumber;
    }
    
    public double getFitTDFDFitQualityNumber( ) 
    {
        return m_fitQualityNumber;
    }
    
    public void add( MetaboliteFeatures toAdd )
    {
        this.amplitude  += toAdd.amplitude;
        this.area       += toAdd.area;
        this.damping    += toAdd.damping;
        this.frqShift   += toAdd.frqShift;
        this.gdamping   += toAdd.gdamping;
        this.m_fitQualityNumber += toAdd.m_fitQualityNumber;
        this.phase      += toAdd.phase;
        this.stdDevArea += toAdd.stdDevArea;
        this.error      += toAdd.error;
        additions++;
    }
    
    /**
     * Returns the average if more than one MetaboliteFeatures where added.
     * @return 
     */
    public MetaboliteFeatures getAverageFeatures( )
    {
        MetaboliteFeatures avFeatures = new MetaboliteFeatures( );
        
        avFeatures.amplitude  += amplitude/additions;
        avFeatures.area       += area/additions;
        avFeatures.damping    += damping/additions;
        avFeatures.frqShift   += frqShift/additions;
        avFeatures.gdamping   += gdamping/additions;
        avFeatures.m_fitQualityNumber += m_fitQualityNumber/additions;
        avFeatures.phase      += phase/additions;
        avFeatures.stdDevArea += stdDevArea/additions;
        avFeatures.error      += error/additions;
       
        return avFeatures;
    }
    
}