/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.main;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.channels.FileChannel;
import java.util.Iterator;
import java.util.LinkedList;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.KeyStroke;
import javax.swing.filechooser.FileNameExtensionFilter;
import org.spectrim.data.CSIData;
import org.spectrim.imagegraphicobjects.Point3D;
import org.spectrim.model.GlobalModel;
import org.spectrim.settings.GlobalSettings;
import org.spectrim.signalprocessing.FFT;
import org.spectrim.spectra.Spectra;
import org.spectrim.tdfdfit.TDFDFittingDefaults;

/**
 * Allows the user to select a binary .sid file resulting from EPSI pre-
 * processing using Midas.
 * @author johan
 */
public class MidasPreprocessedEPSIDataImporterParameterSettingsControl extends javax.swing.JFrame
{

    private int[]                   m_dimensions;
    private int                     m_timeSamples = -1;
    private String                  m_lastSelectedEPSIFile = null;
    private final int               m_bytesPerSinglePrecisionFloat = 4;
    private final GlobalModel       m_globalModel;
    private CSIData                 m_csiData;
    private String                  m_imagePath;
    
    private float[]                 m_epsiDataFirst;
    private float[]                 m_epsiDataSecond;
    
    private String                  m_epsiFileReferenceFirst = "Not defined";
    private String                  m_epsiFileReferenceSecond = "Not defined";
    
    private LinkedList<ActionListener> m_actionListeners = new LinkedList<>( );
    
    public final static int         EPSI_MIDAS_PREPROCESSED_DATA_LOADED_ID  = 0;
    public final static String      EPSI_MIDAS_PREPROCESSED_DATA_LOADED_CMD = "EPSI_MIDAS_PREPROCESSED_DATA_LOADED_ID";
    
    public final static int         NO_SPATIAL_FILTERING_AT_IMPORT_ID = 0;
    public final static int         MOVING_FULL_2D_MEDIAN_SPATIAL_FILTERING_AT_IMPORT_ID  = 1;
    public final static int         MOVING_FULL_2D_AVERAGE_SPATIAL_FILTERING_AT_IMPORT_ID = 2;
    public final static int         MOVING_FULL_3D_MEDIAN_SPATIAL_FILTERING_AT_IMPORT_ID  = 3;
    public final static int         MOVING_FULL_3D_AVERAGE_SPATIAL_FILTERING_AT_IMPORT_ID = 4;
    
    public final static int         MOVING_NEAREST_NEIGHBOURS_ONLY_2D_MEDIAN_SPATIAL_FILTERING_AT_IMPORT_ID  = 5;
    public final static int         MOVING_NEAREST_NEIGHBOURS_ONLY_2D_AVERAGE_SPATIAL_FILTERING_AT_IMPORT_ID = 6;
    public final static int         MOVING_NEAREST_NEIGHBOURS_ONLY_3D_MEDIAN_SPATIAL_FILTERING_AT_IMPORT_ID  = 7;
    public final static int         MOVING_NEAREST_NEIGHBOURS_ONLY_3D_AVERAGE_SPATIAL_FILTERING_AT_IMPORT_ID = 8;
    
    private float[] m_current_real_array_to_phase;
    private float[] m_current_imag_array_to_phase;
    private int m_vectorSize = 0;
    public final static int         LOAD_SELECTION_ID = 1000; 
    /**
     * Creates new form MidasPreprocessedEPSIDataImporterParameterSettingsControl.
     * @param model
     */
    public MidasPreprocessedEPSIDataImporterParameterSettingsControl( GlobalModel model  ) 
    {
        
        m_globalModel = model;
        
        if ( m_globalModel.getImageStackSize( ) > 1 )
        {
            m_actionListeners = new LinkedList<ActionListener>( );

            m_dimensions = new int[3];

            m_dimensions[0] =  m_globalModel.getImageStack( ).getDimensions( )[0];
            m_dimensions[1] =  m_globalModel.getImageStack( ).getDimensions( )[1];
            m_dimensions[2] =  m_globalModel.getImageStack( ).getDimensions( )[2];

            initComponents( );

            TDFDFittingDefaults def = TDFDFittingDefaults.getInstance( );
            def.load( );

            m_epsiFileReferenceFirst  = def.getProperty( TDFDFittingDefaults.SELECTED_EPSI_BINARY_FILE_REFERENCE_FIRST );
            m_epsiFileReferenceSecond = def.getProperty( TDFDFittingDefaults.SELECTED_EPSI_BINARY_FILE_REFERENCE_SECOND );
                                        def.save( );

            if ( m_epsiFileReferenceFirst == null )
            {
                m_epsiFileReferenceFirst = "Not defined yet.";
            }

            m_lastSelectedMidasSIDFileNamejTextFieldFirst.setText( m_epsiFileReferenceFirst );

            if ( m_epsiFileReferenceSecond == null )
            {
                m_epsiFileReferenceSecond = "Not defined yet.";
            }

            m_lastSelectedMidasSIDFileNamejTextFieldSecond.setText( m_epsiFileReferenceSecond );

            updateGUIWithValidityOfSelectedFirstEPSIFile( );
            updateGUIWithValidityOfSelectedSecondEPSIFile( );


            File sizeOfFile1InBytes = new File( m_epsiFileReferenceFirst );
            File sizeOfFile2InBytes = new File( m_epsiFileReferenceSecond ); 
            boolean canloadFiles;

            long length1 = -1;
            long length2 = -1;

            if ( ( ( new File( m_epsiFileReferenceFirst ) ).exists( ) == true ) &&
                 ( ( new File( m_epsiFileReferenceSecond ) ).exists( ) == true ) )
            {
                m_importMidasPreprocessedEPSIDatajButton.setEnabled( true );

                length1 = sizeOfFile1InBytes.length( );
                length2 = sizeOfFile2InBytes.length( );
            }

            m_timeSamples = (int)length1 / ( 8 * m_dimensions[0] * m_dimensions[1] * m_dimensions[2] );

            if ( length1 == length2 )
            {
                m_numberOfTDSamplesjTextField.setText( "" + m_timeSamples );
            }

            m_numberOfInplaneXSamplesjTextField.setText( "" + m_dimensions[0] );
            m_numberOfInPlaneYSamplesjTextField.setText( "" + m_dimensions[1] );
            m_numberOfPlanesZDirectionjTextField.setText( "" + m_dimensions[2] );

            m_lastSelectedMidasSIDFileNamejTextFieldFirst.setEnabled(true);
            m_lastSelectedMidasSIDFileNamejTextFieldSecond.setEnabled(false);

            TDFDFittingDefaults defs = TDFDFittingDefaults.getInstance( );
            defs.load( );

            String lastLoadedPPMReferenceValue = defs.getProperty( TDFDFittingDefaults.EPSI_MIDAS_LAST_PPM_REFERENCE_VALUE );
            if ( lastLoadedPPMReferenceValue != null )
            {
                m_referencePPMValuerjTextField.setText( lastLoadedPPMReferenceValue );
            }
            
            String lastLoadedResonanceFrequencyValue = defs.getProperty( TDFDFittingDefaults.EPSI_MIDAS_LAST_LOADED_RESONANCE_FREQUENCY );
            if ( lastLoadedResonanceFrequencyValue != null )
            {
                m_resonanceFrequencyValuerjTextField.setText( lastLoadedResonanceFrequencyValue );
            }
            
            
            String lastLoadedEPSISamplingFrequencyValue = defs.getProperty( TDFDFittingDefaults.EPSI_MIDAS_LAST_LOADED_SAMPLING_FREQUENCY );
            if ( lastLoadedEPSISamplingFrequencyValue != null )
            {
                m_epsiSamplingFrequencyValuerjTextField.setText( lastLoadedEPSISamplingFrequencyValue );
            }
            
            getRootPane( ).setDefaultButton( m_importMidasPreprocessedEPSIDatajButton );
            
            Action loadAndExit = new AbstractAction( ) 
            {
                @Override
                public void actionPerformed( ActionEvent e ) 
                {
                    doImportOfMidasPreprocessedData( );
                }
            };
            
            Action exitNoLoad = new AbstractAction( ) 
            {
                @Override
                public void actionPerformed( ActionEvent e ) 
                {
                   dispose( );
                }
            };
            
            
            getRootPane( ).getInputMap( ).put( KeyStroke.getKeyStroke( "ENTER" ),
                                               "loadAndExit");
            getRootPane( ).getActionMap( ).put( "loadAndExit",
                                                 loadAndExit  );

            getRootPane( ).getInputMap( ).put( KeyStroke.getKeyStroke( "ESCAPE" ),
                                               "Exit without loading" );
            getRootPane( ).getActionMap( ).put( "Exit without loading",
                                                 exitNoLoad );
            
            setDefaultCloseOperation( DISPOSE_ON_CLOSE );
            pack( );
            setVisible( true );
        }
        else
        {
            JOptionPane.showMessageDialog( this, "Cannot import Midas preprocessed EPSI data : \n First load a suitable Midas genrerated metabolite image stack!" );
        }
    }

    
    public static void addKeyBinding( JComponent comp, KeyStroke key, String id, Action action ) 
    {
        comp.getInputMap().put(key, id);
        comp.getActionMap().put(id, action);
    }
    
    
    
    void updateGUIWithValidityOfSelectedFirstEPSIFile( )
    {
        if ( m_epsiFileReferenceFirst.compareTo( "Not defined yet." ) != 0 )
        {
            String imagePath = new File( m_globalModel.getImageStack( ).getFiles( ).get( 0 ) ).getParent( );
            imagePath = new File( imagePath ).getParent( );
            imagePath = new File( imagePath ).getParent( );
            
            String defaultEPSIFile1Path = new File( m_epsiFileReferenceFirst ).getParent( );
            defaultEPSIFile1Path = new File( defaultEPSIFile1Path ).getParent( );
                    
            if ( ( defaultEPSIFile1Path != null ) && ( imagePath != null ) )
            {
                if ( imagePath.compareTo( defaultEPSIFile1Path ) != 0 )
                {
                    m_lastSelectedMidasSIDFileNamejTextFieldFirst.setBackground( Color.orange );
                }
                else
                {
                    m_lastSelectedMidasSIDFileNamejTextFieldFirst.setBackground( Color.green );
                }
            }
        }
    }
    
    void updateGUIWithValidityOfSelectedSecondEPSIFile( )
    {
        if ( m_epsiFileReferenceSecond.compareTo( "Not defined yet." ) != 0 )
        {
            String imagePath2 = new File( m_globalModel.getImageStack( ).getFiles( ).get( 0 ) ).getParent( );
            imagePath2 = new File( imagePath2 ).getParent( );
            imagePath2 = new File( imagePath2 ).getParent( );
            
            String defaultEPSIFile2Path = new File( m_epsiFileReferenceSecond ).getParent( );
            defaultEPSIFile2Path = new File( defaultEPSIFile2Path ).getParent( );
            
            if ( ( defaultEPSIFile2Path != null ) && ( imagePath2 != null  ) )
            {
                if ( imagePath2.compareTo( defaultEPSIFile2Path ) != 0 )
                {
                    m_lastSelectedMidasSIDFileNamejTextFieldSecond.setBackground( Color.orange );
                }
                else
                {
                    m_lastSelectedMidasSIDFileNamejTextFieldSecond.setBackground( Color.green );
                }
            }
        }
    }
    
    
    
    /**
     * Returns a String object pointing to the .sid EPSI file.
     * @return 
     */
    private String getEPSIBinaryDataFileReferenceSecond( ) 
    {
    
        TDFDFittingDefaults defs = TDFDFittingDefaults.getInstance( );
        defs.load( );
        
        m_epsiFileReferenceSecond = defs.getProperty(TDFDFittingDefaults.SELECTED_EPSI_BINARY_FILE_REFERENCE_SECOND );
        
        if ( m_epsiFileReferenceSecond == null )
        {
            defs.setProperty(TDFDFittingDefaults.SELECTED_EPSI_BINARY_FILE_REFERENCE_SECOND, System.getProperty( "user.home" ) );
            m_epsiFileReferenceSecond = System.getProperty( "user.home" ); 
            defs.save( );
        }
        
        JFileChooser fileChooser = new JFileChooser( );
        fileChooser.setCurrentDirectory( new File( m_epsiFileReferenceSecond ) );
        
        // add filters        
        FileNameExtensionFilter filter = new FileNameExtensionFilter( "sid files (*.sid)", "sid" );
        fileChooser.addChoosableFileFilter( filter );
        fileChooser.setFileFilter( filter );
                
        fileChooser.setFileSelectionMode( JFileChooser.FILES_ONLY );
        fileChooser.setDialogTitle( "Please select one \"*.sid\" file." );
        
        int result = fileChooser.showOpenDialog( this );

        switch ( result ) 
        {
            case JFileChooser.APPROVE_OPTION:   // Approve Open button was pressed
                                                defs.setProperty(TDFDFittingDefaults.SELECTED_EPSI_BINARY_FILE_REFERENCE_SECOND, fileChooser.getSelectedFile( ).toString( ) );
                                                defs.save( );
                                                m_lastSelectedMidasSIDFileNamejTextFieldSecond.setText( fileChooser.getSelectedFile( ).toString( ) );
                                                return  fileChooser.getSelectedFile( ).toString( );
            case JFileChooser.CANCEL_OPTION:     // Cancel or close dialog was selected
                                                return  null;
            case JFileChooser.ERROR_OPTION:      // the selected process did not terminate properly
                                                return  null;
        }
        return null;
    }
    
    /**
     * Returns a String object pointing to the .sid EPSI file.
     * @return 
     */
    private String getEPSIBinaryDataFileReferenceFirst( ) 
    {
    
        TDFDFittingDefaults defs = TDFDFittingDefaults.getInstance( );
        defs.load( );
        
        m_epsiFileReferenceFirst = defs.getProperty( TDFDFittingDefaults.SELECTED_EPSI_BINARY_FILE_REFERENCE_FIRST );
        
        if ( m_epsiFileReferenceFirst == null )
        {
            defs.setProperty(TDFDFittingDefaults.SELECTED_EPSI_BINARY_FILE_REFERENCE_FIRST, System.getProperty( "user.home" ) );
            m_epsiFileReferenceFirst = System.getProperty( "user.home" ); 
            defs.save( );
        }
        
        JFileChooser fileChooser = new JFileChooser( );
        fileChooser.setCurrentDirectory( new File( m_epsiFileReferenceFirst ) );
        
        // add filters        
        FileNameExtensionFilter filter = new FileNameExtensionFilter( "sid files (*.sid)", "sid" );
        fileChooser.addChoosableFileFilter( filter );
        fileChooser.setFileFilter( filter );
                
        fileChooser.setFileSelectionMode( JFileChooser.FILES_ONLY );
        fileChooser.setDialogTitle( "Please select one \"*.sid\" file." );
        
        int result = fileChooser.showOpenDialog( this );

        switch ( result ) 
        {
            case JFileChooser.APPROVE_OPTION:   // Approve Open button was pressed
                                                        defs.setProperty( TDFDFittingDefaults.SELECTED_EPSI_BINARY_FILE_REFERENCE_FIRST, fileChooser.getSelectedFile( ).toString( ) );
                                                        defs.save( );
                                                        m_lastSelectedMidasSIDFileNamejTextFieldFirst.setText( fileChooser.getSelectedFile( ).toString( ) );
                                                        return  fileChooser.getSelectedFile( ).toString( );
            case JFileChooser.CANCEL_OPTION:     // Cancel or close dialog was selected
                                                return  null;
            case JFileChooser.ERROR_OPTION:      // the selected process did not terminate properly
                                                return  null;
        }
        return null;
    }
    
    
    
    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        load_option_buttongroup = new javax.swing.ButtonGroup();
        signal_preprocess_buttongroup = new javax.swing.ButtonGroup();
        use_neighbour_type_options_buttongroup = new javax.swing.ButtonGroup();
        jPanel1 = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        m_lastSelectedMidasSIDFileNamejTextFieldFirst = new javax.swing.JTextField();
        m_selectEPSIBinaryFilejFirstButton = new javax.swing.JButton();
        jLabel2 = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();
        jLabel4 = new javax.swing.JLabel();
        jLabel5 = new javax.swing.JLabel();
        m_numberOfTDSamplesjTextField = new javax.swing.JTextField();
        m_numberOfInplaneXSamplesjTextField = new javax.swing.JTextField();
        m_numberOfInPlaneYSamplesjTextField = new javax.swing.JTextField();
        m_numberOfPlanesZDirectionjTextField = new javax.swing.JTextField();
        jLabel6 = new javax.swing.JLabel();
        m_referencePPMValuerjTextField = new javax.swing.JTextField();
        jLabel7 = new javax.swing.JLabel();
        m_lastSelectedMidasSIDFileNamejTextFieldSecond = new javax.swing.JTextField();
        m_selectEPSIBinaryFileSecondjButton = new javax.swing.JButton();
        m_loadSignal_2_minus_Signal_1_jRadioButton = new javax.swing.JRadioButton();
        m_loadSignal_1_jRadioButton_ = new javax.swing.JRadioButton();
        m_loadSignal_2_jRadioButton = new javax.swing.JRadioButton();
        m_loadSignal_1_plus_2_jRadioButton = new javax.swing.JRadioButton();
        m_loadSignal_1_minus_2_jRadioButton = new javax.swing.JRadioButton();
        jLabel19 = new javax.swing.JLabel();
        m_resonanceFrequencyValuerjTextField = new javax.swing.JTextField();
        jLabel20 = new javax.swing.JLabel();
        m_epsiSamplingFrequencyValuerjTextField = new javax.swing.JTextField();
        jPanel2 = new javax.swing.JPanel();
        m_cancelWitoutLoadMidasPreprocessedEPSIDatajButton = new javax.swing.JButton();
        m_importMidasPreprocessedEPSIDatajButton = new javax.swing.JButton();
        jPanel6 = new javax.swing.JPanel();
        jButton1 = new javax.swing.JButton();
        jPanel3 = new javax.swing.JPanel();
        jPanel4 = new javax.swing.JPanel();
        m_signal_As_Is_RadioButton = new javax.swing.JRadioButton();
        m_2DsignalMedianFilterRadioButton = new javax.swing.JRadioButton();
        m_2DsignalAveragingRadioButton = new javax.swing.JRadioButton();
        jLabel8 = new javax.swing.JLabel();
        m_numberOfNeighbourPointsTextField = new javax.swing.JTextField();
        m_3DsignalMedianFilterRadioButton = new javax.swing.JRadioButton();
        m_3DsignalAveragingRadioButton = new javax.swing.JRadioButton();
        m_useAllNeighboursRadioButton = new javax.swing.JRadioButton();
        m_useNearerstNeighboursRadioButton = new javax.swing.JRadioButton();
        jPanel5 = new javax.swing.JPanel();
        m_correctZeroOrderAtImportjCheckBox = new javax.swing.JCheckBox();
        jLabel9 = new javax.swing.JLabel();
        m_minimumPPMRangetextField = new java.awt.TextField();
        jLabel10 = new javax.swing.JLabel();
        m_maximumPPMRangetextField = new java.awt.TextField();
        jLabel11 = new javax.swing.JLabel();
        jLabel12 = new javax.swing.JLabel();
        m_shiftResponseAtImportjCheckBox = new javax.swing.JCheckBox();
        jLabel13 = new javax.swing.JLabel();
        m_targetPPMShifttextField = new java.awt.TextField();
        jLabel14 = new javax.swing.JLabel();
        jLabel15 = new javax.swing.JLabel();
        m_minimumAllowedPPMShiftjTtextField = new java.awt.TextField();
        jLabel16 = new javax.swing.JLabel();
        jLabel17 = new javax.swing.JLabel();
        m_maximumAllowedPPMShiftjTtextField = new java.awt.TextField();
        jLabel18 = new javax.swing.JLabel();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("Acquisition parameters EPSI dataset to import");

        jPanel1.setBorder(javax.swing.BorderFactory.createTitledBorder("File names and data parameters"));

        jLabel1.setText("Selected Midas preprocessed file name:");

        m_lastSelectedMidasSIDFileNamejTextFieldFirst.setText("None");
        m_lastSelectedMidasSIDFileNamejTextFieldFirst.setToolTipText("If background is orange, please verify correctness of EPSI file! If green selection is most likely correct.");
        m_lastSelectedMidasSIDFileNamejTextFieldFirst.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_lastSelectedMidasSIDFileNamejTextFieldFirstActionPerformed(evt);
            }
        });

        m_selectEPSIBinaryFilejFirstButton.setText("Select first signal file ..");
        m_selectEPSIBinaryFilejFirstButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_selectEPSIBinaryFilejFirstButtonActionPerformed(evt);
            }
        });

        jLabel2.setText("Number of TD-samples");

        jLabel3.setText("Number of in plane X samples");

        jLabel4.setText("Number of in plane Y samples");

        jLabel5.setText("Number of planes Z samples");

        m_numberOfTDSamplesjTextField.setText("1024");

        m_numberOfInplaneXSamplesjTextField.setText("65");

        m_numberOfInPlaneYSamplesjTextField.setText("50");

        m_numberOfPlanesZDirectionjTextField.setText("8");

        jLabel6.setText("Reference ppm value");

        m_referencePPMValuerjTextField.setText("3.0");

        jLabel7.setText("Selected Midas preprocessed file name:");

        m_lastSelectedMidasSIDFileNamejTextFieldSecond.setText("None");
        m_lastSelectedMidasSIDFileNamejTextFieldSecond.setToolTipText("If background is orange, please verify correctness of EPSI file! If green selection is most likely correct.");

        m_selectEPSIBinaryFileSecondjButton.setText("Select second signal file ..");
        m_selectEPSIBinaryFileSecondjButton.setToolTipText("");
        m_selectEPSIBinaryFileSecondjButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_selectEPSIBinaryFileSecondjButtonActionPerformed(evt);
            }
        });

        load_option_buttongroup.add(m_loadSignal_2_minus_Signal_1_jRadioButton);
        m_loadSignal_2_minus_Signal_1_jRadioButton.setText("Load signal 2 -  signal 1");
        m_loadSignal_2_minus_Signal_1_jRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_loadSignal_2_minus_Signal_1_jRadioButtonActionPerformed(evt);
            }
        });

        load_option_buttongroup.add(m_loadSignal_1_jRadioButton_);
        m_loadSignal_1_jRadioButton_.setSelected(true);
        m_loadSignal_1_jRadioButton_.setText("Load signal 1");
        m_loadSignal_1_jRadioButton_.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_loadSignal_1_jRadioButton_ActionPerformed(evt);
            }
        });

        load_option_buttongroup.add(m_loadSignal_2_jRadioButton);
        m_loadSignal_2_jRadioButton.setText("Load signal 2");
        m_loadSignal_2_jRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_loadSignal_2_jRadioButtonActionPerformed(evt);
            }
        });

        load_option_buttongroup.add(m_loadSignal_1_plus_2_jRadioButton);
        m_loadSignal_1_plus_2_jRadioButton.setText("Load signal 1 +  signal 2");
        m_loadSignal_1_plus_2_jRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_loadSignal_1_plus_2_jRadioButtonActionPerformed(evt);
            }
        });

        load_option_buttongroup.add(m_loadSignal_1_minus_2_jRadioButton);
        m_loadSignal_1_minus_2_jRadioButton.setText("Load signal 1 -  signal 2");
        m_loadSignal_1_minus_2_jRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_loadSignal_1_minus_2_jRadioButtonActionPerformed(evt);
            }
        });

        jLabel19.setText("Resonance frequency");

        m_resonanceFrequencyValuerjTextField.setText("297223000");

        jLabel20.setText("Sampling frequency");

        m_epsiSamplingFrequencyValuerjTextField.setText("1282.0");

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(jPanel1Layout.createSequentialGroup()
                                .addContainerGap()
                                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                                    .addGroup(javax.swing.GroupLayout.Alignment.LEADING, jPanel1Layout.createSequentialGroup()
                                        .addComponent(jLabel1, javax.swing.GroupLayout.PREFERRED_SIZE, 195, javax.swing.GroupLayout.PREFERRED_SIZE)
                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 18, Short.MAX_VALUE)
                                        .addComponent(m_lastSelectedMidasSIDFileNamejTextFieldFirst, javax.swing.GroupLayout.PREFERRED_SIZE, 723, javax.swing.GroupLayout.PREFERRED_SIZE))
                                    .addGroup(jPanel1Layout.createSequentialGroup()
                                        .addComponent(jLabel7, javax.swing.GroupLayout.PREFERRED_SIZE, 195, javax.swing.GroupLayout.PREFERRED_SIZE)
                                        .addGap(18, 18, 18)
                                        .addComponent(m_lastSelectedMidasSIDFileNamejTextFieldSecond))))
                            .addGroup(jPanel1Layout.createSequentialGroup()
                                .addGap(277, 277, 277)
                                .addComponent(m_loadSignal_1_jRadioButton_)
                                .addGap(18, 18, 18)
                                .addComponent(m_loadSignal_2_jRadioButton)
                                .addGap(18, 18, 18)
                                .addComponent(m_loadSignal_1_plus_2_jRadioButton)
                                .addGap(18, 18, 18)
                                .addComponent(m_loadSignal_2_minus_Signal_1_jRadioButton)
                                .addGap(18, 18, 18)
                                .addComponent(m_loadSignal_1_minus_2_jRadioButton)))
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                            .addComponent(m_selectEPSIBinaryFileSecondjButton, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(m_selectEPSIBinaryFilejFirstButton, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)))
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addContainerGap()
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                            .addGroup(jPanel1Layout.createSequentialGroup()
                                .addComponent(jLabel6)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                .addComponent(m_referencePPMValuerjTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 76, javax.swing.GroupLayout.PREFERRED_SIZE))
                            .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                                .addGroup(jPanel1Layout.createSequentialGroup()
                                    .addComponent(jLabel5)
                                    .addGap(24, 24, 24)
                                    .addComponent(m_numberOfPlanesZDirectionjTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 76, javax.swing.GroupLayout.PREFERRED_SIZE))
                                .addGroup(javax.swing.GroupLayout.Alignment.LEADING, jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                                    .addGroup(jPanel1Layout.createSequentialGroup()
                                        .addComponent(jLabel4)
                                        .addGap(18, 18, 18)
                                        .addComponent(m_numberOfInPlaneYSamplesjTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 76, javax.swing.GroupLayout.PREFERRED_SIZE))
                                    .addGroup(javax.swing.GroupLayout.Alignment.LEADING, jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                                        .addGroup(jPanel1Layout.createSequentialGroup()
                                            .addComponent(jLabel3)
                                            .addGap(18, 18, 18)
                                            .addComponent(m_numberOfInplaneXSamplesjTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 76, javax.swing.GroupLayout.PREFERRED_SIZE))
                                        .addGroup(jPanel1Layout.createSequentialGroup()
                                            .addComponent(jLabel2)
                                            .addGap(50, 50, 50)
                                            .addComponent(m_numberOfTDSamplesjTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 76, javax.swing.GroupLayout.PREFERRED_SIZE)))))
                            .addGroup(jPanel1Layout.createSequentialGroup()
                                .addComponent(jLabel19)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                .addComponent(m_resonanceFrequencyValuerjTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 76, javax.swing.GroupLayout.PREFERRED_SIZE))
                            .addGroup(jPanel1Layout.createSequentialGroup()
                                .addComponent(jLabel20)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                .addComponent(m_epsiSamplingFrequencyValuerjTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 76, javax.swing.GroupLayout.PREFERRED_SIZE)))))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(m_lastSelectedMidasSIDFileNamejTextFieldFirst, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addComponent(m_selectEPSIBinaryFilejFirstButton))
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addGap(3, 3, 3)
                        .addComponent(jLabel1)))
                .addGap(18, 18, 18)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_lastSelectedMidasSIDFileNamejTextFieldSecond, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel7)
                    .addComponent(m_selectEPSIBinaryFileSecondjButton))
                .addGap(18, 18, 18)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(m_loadSignal_1_jRadioButton_)
                    .addComponent(m_loadSignal_2_jRadioButton)
                    .addComponent(m_loadSignal_1_plus_2_jRadioButton)
                    .addComponent(m_loadSignal_2_minus_Signal_1_jRadioButton)
                    .addComponent(m_loadSignal_1_minus_2_jRadioButton))
                .addGap(5, 5, 5)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_resonanceFrequencyValuerjTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel19))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_epsiSamplingFrequencyValuerjTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel20))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_referencePPMValuerjTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel6))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_numberOfTDSamplesjTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel2))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_numberOfInplaneXSamplesjTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel3))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_numberOfInPlaneYSamplesjTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel4))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_numberOfPlanesZDirectionjTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel5)))
        );

        m_cancelWitoutLoadMidasPreprocessedEPSIDatajButton.setText("Cancel without import");
        m_cancelWitoutLoadMidasPreprocessedEPSIDatajButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_cancelWitoutLoadMidasPreprocessedEPSIDatajButtonActionPerformed(evt);
            }
        });

        m_importMidasPreprocessedEPSIDatajButton.setText("Import Midas preprocessed image data");
        m_importMidasPreprocessedEPSIDatajButton.setToolTipText("");
        m_importMidasPreprocessedEPSIDatajButton.setEnabled(false);
        m_importMidasPreprocessedEPSIDatajButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_importMidasPreprocessedEPSIDatajButtonActionPerformed(evt);
            }
        });

        jPanel6.setBorder(javax.swing.BorderFactory.createTitledBorder("Spectral feature filtering"));

        jButton1.setText("Set spectral feature filtering ..");

        javax.swing.GroupLayout jPanel6Layout = new javax.swing.GroupLayout(jPanel6);
        jPanel6.setLayout(jPanel6Layout);
        jPanel6Layout.setHorizontalGroup(
            jPanel6Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel6Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jButton1)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        jPanel6Layout.setVerticalGroup(
            jPanel6Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel6Layout.createSequentialGroup()
                .addComponent(jButton1)
                .addGap(0, 4, Short.MAX_VALUE))
        );

        javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanel2Layout.createSequentialGroup()
                        .addComponent(m_cancelWitoutLoadMidasPreprocessedEPSIDatajButton, javax.swing.GroupLayout.PREFERRED_SIZE, 270, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addComponent(m_importMidasPreprocessedEPSIDatajButton, javax.swing.GroupLayout.PREFERRED_SIZE, 230, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addComponent(jPanel6, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addContainerGap())
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jPanel6, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_importMidasPreprocessedEPSIDatajButton, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(m_cancelWitoutLoadMidasPreprocessedEPSIDatajButton))
                .addContainerGap())
        );

        jPanel3.setBorder(javax.swing.BorderFactory.createTitledBorder("Import and preprocessing options"));

        jPanel4.setBorder(javax.swing.BorderFactory.createTitledBorder("2D-3D Filtering"));

        signal_preprocess_buttongroup.add(m_signal_As_Is_RadioButton);
        m_signal_As_Is_RadioButton.setText("Import as is");
        m_signal_As_Is_RadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_signal_As_Is_RadioButtonActionPerformed(evt);
            }
        });

        signal_preprocess_buttongroup.add(m_2DsignalMedianFilterRadioButton);
        m_2DsignalMedianFilterRadioButton.setSelected(true);
        m_2DsignalMedianFilterRadioButton.setText("Apply 2D moving median at import");
        m_2DsignalMedianFilterRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_2DsignalMedianFilterRadioButtonActionPerformed(evt);
            }
        });

        signal_preprocess_buttongroup.add(m_2DsignalAveragingRadioButton);
        m_2DsignalAveragingRadioButton.setText("Apply 2D moving average at import");
        m_2DsignalAveragingRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_2DsignalAveragingRadioButtonActionPerformed(evt);
            }
        });

        jLabel8.setText("Number of neighbourhood points");

        m_numberOfNeighbourPointsTextField.setText("1");

        signal_preprocess_buttongroup.add(m_3DsignalMedianFilterRadioButton);
        m_3DsignalMedianFilterRadioButton.setText("Apply 3D moving median at import");
        m_3DsignalMedianFilterRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_3DsignalMedianFilterRadioButtonActionPerformed(evt);
            }
        });

        signal_preprocess_buttongroup.add(m_3DsignalAveragingRadioButton);
        m_3DsignalAveragingRadioButton.setText("Apply 3D moving average at import");
        m_3DsignalAveragingRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_3DsignalAveragingRadioButtonActionPerformed(evt);
            }
        });

        use_neighbour_type_options_buttongroup.add(m_useAllNeighboursRadioButton);
        m_useAllNeighboursRadioButton.setText("Use all 2D/3D neighbours");
        m_useAllNeighboursRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_useAllNeighboursRadioButtonActionPerformed(evt);
            }
        });

        use_neighbour_type_options_buttongroup.add(m_useNearerstNeighboursRadioButton);
        m_useNearerstNeighboursRadioButton.setSelected(true);
        m_useNearerstNeighboursRadioButton.setText("Use 2D/3D nearest neighbours only");
        m_useNearerstNeighboursRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_useNearerstNeighboursRadioButtonActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanel4Layout = new javax.swing.GroupLayout(jPanel4);
        jPanel4.setLayout(jPanel4Layout);
        jPanel4Layout.setHorizontalGroup(
            jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel4Layout.createSequentialGroup()
                .addComponent(m_signal_As_Is_RadioButton)
                .addGap(18, 18, 18)
                .addGroup(jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanel4Layout.createSequentialGroup()
                        .addComponent(m_2DsignalMedianFilterRadioButton)
                        .addGap(18, 18, 18)
                        .addComponent(m_2DsignalAveragingRadioButton)
                        .addGap(18, 18, 18)
                        .addComponent(m_numberOfNeighbourPointsTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 53, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(18, 18, 18)
                        .addComponent(jLabel8))
                    .addGroup(jPanel4Layout.createSequentialGroup()
                        .addGroup(jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(m_3DsignalMedianFilterRadioButton)
                            .addComponent(m_useAllNeighboursRadioButton))
                        .addGap(18, 18, 18)
                        .addGroup(jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(m_useNearerstNeighboursRadioButton)
                            .addComponent(m_3DsignalAveragingRadioButton))))
                .addGap(0, 0, Short.MAX_VALUE))
        );
        jPanel4Layout.setVerticalGroup(
            jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel4Layout.createSequentialGroup()
                .addGroup(jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_signal_As_Is_RadioButton)
                    .addComponent(m_2DsignalMedianFilterRadioButton)
                    .addComponent(m_numberOfNeighbourPointsTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel8)
                    .addComponent(m_2DsignalAveragingRadioButton))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_3DsignalMedianFilterRadioButton)
                    .addComponent(m_3DsignalAveragingRadioButton))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_useAllNeighboursRadioButton)
                    .addComponent(m_useNearerstNeighboursRadioButton))
                .addContainerGap(11, Short.MAX_VALUE))
        );

        jPanel5.setBorder(javax.swing.BorderFactory.createTitledBorder("Processing at import"));

        m_correctZeroOrderAtImportjCheckBox.setSelected(true);
        m_correctZeroOrderAtImportjCheckBox.setText("correct zero order phase");
        m_correctZeroOrderAtImportjCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_correctZeroOrderAtImportjCheckBoxActionPerformed(evt);
            }
        });

        jLabel9.setText("Minimum ppm range");

        m_minimumPPMRangetextField.setText("2.8");
        m_minimumPPMRangetextField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_minimumPPMRangetextFieldActionPerformed(evt);
            }
        });

        jLabel10.setText("Maximum ppm range");

        m_maximumPPMRangetextField.setText("3.4");

        jLabel11.setText("ppm");

        jLabel12.setText("ppm");

        m_shiftResponseAtImportjCheckBox.setSelected(true);
        m_shiftResponseAtImportjCheckBox.setText("shift spectrum");
        m_shiftResponseAtImportjCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_shiftResponseAtImportjCheckBoxActionPerformed(evt);
            }
        });

        jLabel13.setText("Target shift [ppm]");

        m_targetPPMShifttextField.setText("3.0");

        jLabel14.setText("ppm");

        jLabel15.setText("Minimum allowed shift");

        m_minimumAllowedPPMShiftjTtextField.setName(""); // NOI18N
        m_minimumAllowedPPMShiftjTtextField.setText("-0.15");
        m_minimumAllowedPPMShiftjTtextField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_minimumAllowedPPMShiftjTtextFieldActionPerformed(evt);
            }
        });

        jLabel16.setText("ppm");

        jLabel17.setText("Maximum allowed shift");

        m_maximumAllowedPPMShiftjTtextField.setName(""); // NOI18N
        m_maximumAllowedPPMShiftjTtextField.setText("0.15");

        jLabel18.setText("ppm");

        javax.swing.GroupLayout jPanel5Layout = new javax.swing.GroupLayout(jPanel5);
        jPanel5.setLayout(jPanel5Layout);
        jPanel5Layout.setHorizontalGroup(
            jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel5Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(m_correctZeroOrderAtImportjCheckBox)
                    .addComponent(m_shiftResponseAtImportjCheckBox))
                .addGap(43, 43, 43)
                .addGroup(jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel9)
                    .addComponent(jLabel13))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanel5Layout.createSequentialGroup()
                        .addComponent(m_targetPPMShifttextField, javax.swing.GroupLayout.PREFERRED_SIZE, 34, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jLabel14))
                    .addGroup(jPanel5Layout.createSequentialGroup()
                        .addComponent(m_minimumPPMRangetextField, javax.swing.GroupLayout.PREFERRED_SIZE, 34, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jLabel11)))
                .addGap(50, 50, 50)
                .addGroup(jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanel5Layout.createSequentialGroup()
                        .addComponent(jLabel10)
                        .addGap(17, 17, 17)
                        .addComponent(m_maximumPPMRangetextField, javax.swing.GroupLayout.PREFERRED_SIZE, 35, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jLabel12))
                    .addGroup(jPanel5Layout.createSequentialGroup()
                        .addComponent(jLabel15)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(m_minimumAllowedPPMShiftjTtextField, javax.swing.GroupLayout.PREFERRED_SIZE, 35, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jLabel16)
                        .addGap(30, 30, 30)
                        .addComponent(jLabel17)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(m_maximumAllowedPPMShiftjTtextField, javax.swing.GroupLayout.PREFERRED_SIZE, 35, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jLabel18)))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        jPanel5Layout.setVerticalGroup(
            jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel5Layout.createSequentialGroup()
                .addGroup(jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                        .addGroup(jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(m_correctZeroOrderAtImportjCheckBox)
                            .addComponent(jLabel9))
                        .addGroup(jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                                .addComponent(jLabel10)
                                .addComponent(jLabel11))
                            .addComponent(m_minimumPPMRangetextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                    .addComponent(m_maximumPPMRangetextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel12))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(m_shiftResponseAtImportjCheckBox)
                    .addGroup(jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                        .addGroup(jPanel5Layout.createSequentialGroup()
                            .addComponent(jLabel13)
                            .addGap(6, 6, 6))
                        .addGroup(jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jLabel15)
                            .addComponent(m_minimumAllowedPPMShiftjTtextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addGroup(jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                                .addComponent(jLabel17)
                                .addComponent(jLabel16))
                            .addComponent(m_maximumAllowedPPMShiftjTtextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(jLabel18)))
                    .addGroup(jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                        .addComponent(m_targetPPMShifttextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addComponent(jLabel14)))
                .addContainerGap(11, Short.MAX_VALUE))
        );

        javax.swing.GroupLayout jPanel3Layout = new javax.swing.GroupLayout(jPanel3);
        jPanel3.setLayout(jPanel3Layout);
        jPanel3Layout.setHorizontalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jPanel4, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .addComponent(jPanel5, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );
        jPanel3Layout.setVerticalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel3Layout.createSequentialGroup()
                .addComponent(jPanel4, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGap(5, 5, 5)
                .addComponent(jPanel5, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
        );

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addGap(2, 2, 2)
                .addComponent(jPanel3, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
            .addComponent(jPanel2, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jPanel3, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addComponent(jPanel2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void m_selectEPSIBinaryFilejFirstButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_selectEPSIBinaryFilejFirstButtonActionPerformed
        
        String testFile = getEPSIBinaryDataFileReferenceFirst( );
        
        if ( testFile != null )
        {
           m_epsiFileReferenceFirst = testFile; 
        }
        
        m_lastSelectedMidasSIDFileNamejTextFieldFirst.setText( m_epsiFileReferenceFirst );
        
        updateGUIWithValidityOfSelectedFirstEPSIFile( );
        
        File sizeOfFile1 = new File( m_epsiFileReferenceFirst );
        long length1 = sizeOfFile1.length( );
        
        m_timeSamples = (int)( length1 / ( 8 * m_dimensions[0] * m_dimensions[1] * m_dimensions[2] ) );
        
        this.m_numberOfTDSamplesjTextField.setText( "" + m_timeSamples );
        
        
        if ( ( ( new File( m_epsiFileReferenceFirst ) ).exists( ) == true ) ||
             ( ( new File( m_epsiFileReferenceSecond ) ).exists( ) == true ) )
        {
            m_importMidasPreprocessedEPSIDatajButton.setEnabled( true );
        }
        
        TDFDFittingDefaults defs = TDFDFittingDefaults.getInstance( );
        defs.load( );
        defs.setProperty( TDFDFittingDefaults.SELECTED_EPSI_BINARY_FILE_REFERENCE_FIRST, m_epsiFileReferenceFirst );
        defs.save( );
        
    }//GEN-LAST:event_m_selectEPSIBinaryFilejFirstButtonActionPerformed

 
    /**
     * Writes data to a binary file.
     * @param array 
     */
    private void fastWriteFloatArrayFirst( float[] array )
    {
        try
        {
            RandomAccessFile aFile = new RandomAccessFile( m_epsiFileReferenceFirst, "rw" );
            FileChannel outChannel = aFile.getChannel( );

            //one float 4 bytes
            ByteBuffer buf = ByteBuffer.allocate( 4 * array.length );
            buf.clear( );
            buf.asFloatBuffer( ).put( array );

            outChannel.write( buf );
            outChannel.close( );

        }
        catch ( IOException ex ) 
        {
            System.err.println( ex.getMessage( ) );
        }
    }        
  
    /**
     * Reads the EPSI preprocessed data file.
     * @param EPSI_SID_data
     * @param fileRef
     * @return 
     */
    public float[] fastReadEPSIBinarySIDFile( float[] EPSI_SID_data, String fileRef )
    {
        try
        {
            System.out.println( "Trying to read : " + fileRef + " .." );
            
            RandomAccessFile rFile = new RandomAccessFile( fileRef, "rw" );

            FileChannel inChannel = rFile.getChannel( );
            ByteBuffer buf_in = ByteBuffer.allocate( EPSI_SID_data.length * 4 ).order(
                                                     ByteOrder.LITTLE_ENDIAN );
            buf_in.clear( );

            inChannel.read( buf_in );

            buf_in.rewind( );
            buf_in.asFloatBuffer( ).get( EPSI_SID_data );

            inChannel.close( );
            m_importMidasPreprocessedEPSIDatajButton.setEnabled( true );
            
            JOptionPane.showMessageDialog( this, "<html>Midas data .sid file loading successful!" + "<br><br>Data dimensions match loaded Dicom metabolite map!</html>" );  
            
            System.out.println( "File: " + fileRef + " successfully read." );
            
        }
        catch ( IOException ex ) 
        {
            System.err.println( ex.getMessage( ) );
        }
        
        return EPSI_SID_data;
    }
    
    
    private void doImportOfMidasPreprocessedData( )
    {
        int[] dimensions = m_globalModel.getImageStack( ).getDimensions( );
        m_dimensions = dimensions;
        
        Point3D imageOrientationX = m_globalModel.getImageStack( ).get( 0 ).imageOrientationX;
        Point3D imageOrientationY = m_globalModel.getImageStack( ).get( 0 ).imageOrientationY;
        
        double pixelSpacingX = m_globalModel.getImageStack( ).get( 0 ).pixelSpacingX;
        double pixelSpacingY = m_globalModel.getImageStack( ).get( 0 ).pixelSpacingY;
          
        int width = m_globalModel.getImageStack( ).get( 0 ).image.getWidth( );
        int height = m_globalModel.getImageStack( ).get( 0 ).image.getHeight( );
                             
        float EPSI_kt_sampling_frequency = new Double( m_epsiSamplingFrequencyValuerjTextField.getText( ) ).floatValue( );
        float TE_EPSI = 68.0f;
        float TR_EPSI = 1500.0f;
        double resonance_frequency_EPSI = new Double( m_resonanceFrequencyValuerjTextField.getText( ) ).floatValue( );
        float referencePPM = new Double( m_referencePPMValuerjTextField.getText( ) ).floatValue( );
        
        TDFDFittingDefaults defs = TDFDFittingDefaults.getInstance( );
        defs.load( );
        
        defs.setProperty( TDFDFittingDefaults.HLSVD_PPM_REFERENCE_VALUE,  "" + 3.0 );
//        defs.setProperty( TDFDFittingDefaults.HLSVD_MINUMUM_FILTER_FREQUENCY,  "" + 4.5 );
//        defs.setProperty( TDFDFittingDefaults.HLSVD_MAXIMUM_FILTER_FREQUENCY,  "" + 5.0 );
        
        defs.setProperty( TDFDFittingDefaults.EPSI_MIDAS_LAST_LOADED_RESONANCE_FREQUENCY, m_resonanceFrequencyValuerjTextField.getText( ) );
        defs.setProperty( TDFDFittingDefaults.EPSI_MIDAS_LAST_LOADED_SAMPLING_FREQUENCY, m_epsiSamplingFrequencyValuerjTextField.getText( ) );
        defs.setProperty( TDFDFittingDefaults.EPSI_MIDAS_LAST_PPM_REFERENCE_VALUE,  m_referencePPMValuerjTextField.getText( ) );
        defs.save( );
        
//        public final static String EPSI_MIDAS_LAST_LOADED_RESONANCE_FREQUENCY  = "EPSI MIDAS LAST RESONANCE FREQUENCY";
//        public final static String EPSI_MIDAS_LAST_LOADED_SAMPLING_FREQUENCY   = "EPSI MIDAS LAST SAMPLING FREQUENCY";
//        public final static String EPSI_MIDAS_LAST_PPM_REFERENCE_VALUE         = "EPSI MIDAS LAST PPM REFERENCE VALUE";
        
     
        GlobalSettings.getInstance( ).setSignalFreeFDMin( referencePPM - EPSI_kt_sampling_frequency * 1000000f / ( 2f * (float)resonance_frequency_EPSI ) );
        GlobalSettings.getInstance( ).setSignalFreeFDMax( referencePPM + EPSI_kt_sampling_frequency * 1000000f / ( 2f * (float)resonance_frequency_EPSI ) );
        
        int floatsToRead;
        
        floatsToRead = 2 * m_dimensions[0] * m_dimensions[1] * m_dimensions[2] * new Integer( m_numberOfTDSamplesjTextField.getText( ) ) ;

        File sizeOfFile1InBytes = new File( m_epsiFileReferenceFirst );
        File sizeOfFile2InBytes = new File( m_epsiFileReferenceSecond ); 
        boolean canloadFiles;
        
        long length1 = sizeOfFile1InBytes.length( );
        long length2 = sizeOfFile2InBytes.length( );
        
        if ( ( sizeOfFile1InBytes.exists( ) && sizeOfFile1InBytes.isFile( ) ) &&
             ( sizeOfFile2InBytes.exists( ) && sizeOfFile2InBytes.isFile( ) ) )
        {
            length1 = sizeOfFile1InBytes.length( );
            length2 = sizeOfFile2InBytes.length( );
            
            if ( length1 != length2 )
            {
                if ( ( m_loadSignal_2_minus_Signal_1_jRadioButton.isSelected( ) == true ) ||
                     ( m_loadSignal_1_minus_2_jRadioButton.isSelected( ) == true ) ||
                     ( m_loadSignal_1_plus_2_jRadioButton.isSelected( ) == true ) )
                {
                    JOptionPane.showMessageDialog( this, "<html>Midas data .sid file loading Failed!" + "<br><br>File size of file 1 unequal to file size of file 2!</html>" );
                    canloadFiles = false;
                }
            }    
        }
        
        int timeSamples;
        
        timeSamples = (int)length1 / ( 8 * m_dimensions[0] * m_dimensions[1] * m_dimensions[2] );
        
        m_numberOfTDSamplesjTextField.setText( "" + timeSamples );
        
        System.out.println( "Number of time samples to read is : " + timeSamples );
        System.out.println( "Total number of floats to read is : " + floatsToRead );
        System.out.println( "Total number of voxels dim 1      : " + m_dimensions[0] );
        System.out.println( "Total number of voxels dim 2      : " + m_dimensions[1] );
        System.out.println( "Total number of voxels dim 3      : " + m_dimensions[2] );
        
        // initialize pointers 
        m_epsiDataFirst = null;
        m_epsiDataSecond = null;
        
        if ( ( m_loadSignal_1_jRadioButton_.isSelected( ) == true ) ||
             ( m_loadSignal_1_minus_2_jRadioButton.isSelected( ) == true ) ||
             ( m_loadSignal_1_plus_2_jRadioButton.isSelected( ) == true ) ||
             ( m_loadSignal_2_minus_Signal_1_jRadioButton.isSelected( ) == true ) )
        {
            m_epsiDataFirst = new float[floatsToRead]; 
            System.out.println( "Reading response 1 ..." );
            m_epsiDataFirst = fastReadEPSIBinarySIDFile( m_epsiDataFirst, m_epsiFileReferenceFirst );
            
            
            //m_epsiDataFirst = correctGuodongsData( m_epsiDataFirst  );
            
            System.out.println( "Reading response 1 done!" );   
        }
        
        if ( ( m_loadSignal_2_minus_Signal_1_jRadioButton.isSelected( ) == true ) ||
             ( m_loadSignal_1_minus_2_jRadioButton.isSelected( ) == true ) ||
             ( m_loadSignal_1_plus_2_jRadioButton.isSelected( ) == true ) ||
             ( m_loadSignal_2_jRadioButton.isSelected( ) == true ) )
                
        {   
            System.out.println( "Reading response 2 ..." );
            m_epsiDataSecond = new float[floatsToRead];
            m_epsiDataSecond = fastReadEPSIBinarySIDFile( m_epsiDataSecond, m_epsiFileReferenceSecond );
            System.out.println( "Reading response 2 done!" );
        }
        
        // output data stucture
        float[] outPut = new float[floatsToRead];
        
        // Before this point the auto phasing needs to take place.
   
        double resonanceFrequency = new Double( m_resonanceFrequencyValuerjTextField.getText( ) );
        
        float spectralWidth = new Float( m_epsiSamplingFrequencyValuerjTextField.getText( ) ).floatValue( );
        
        float ppmReference = new Float( m_referencePPMValuerjTextField.getText( ) ).floatValue( );
        
        m_vectorSize = new Integer( m_numberOfTDSamplesjTextField.getText( ) ).intValue( );
        
        //float zerofillingsize = (float)m_globalModel.getZeroFillingSize( );
        float zerofillingsize = 0;
        float minppm = new Float( m_minimumPPMRangetextField.getText( ) );
        float maxppm = new Float( m_maximumPPMRangetextField.getText( ) );
        
        int minimumppmIndex = m_vectorSize - (int) Math.round( ( ( maxppm - ppmReference ) * resonanceFrequency / ( 1000000 * spectralWidth ) + 0.5d ) * ( m_vectorSize + zerofillingsize - 1 ) );
        int maximumppmIndex = m_vectorSize - (int) Math.round( ( ( minppm - ppmReference ) * resonanceFrequency / ( 1000000 * spectralWidth ) + 0.5d ) * ( m_vectorSize + zerofillingsize - 1 ) );
        
        
        float targetPPMShift  = new Float( m_targetPPMShifttextField.getText( ) );
        float minShiftInPPM = new Float( m_minimumAllowedPPMShiftjTtextField.getText( ) );
        float maxShiftInPPM = new Float( m_maximumAllowedPPMShiftjTtextField.getText( ) );
        
        
        int targetPPMShiftIndex     = m_vectorSize - (int) Math.round( ( ( targetPPMShift - ppmReference ) * resonanceFrequency / ( 1000000 * spectralWidth ) + 0.5d ) * ( m_vectorSize + zerofillingsize - 1 ) );
        int maxAllowedPPMShiftIndex = m_vectorSize - (int) Math.round( ( ( targetPPMShift + minShiftInPPM - ppmReference ) * resonanceFrequency / ( 1000000 * spectralWidth ) + 0.5d ) * ( m_vectorSize + zerofillingsize - 1 ) );
        int minAllowedPPMShiftIndex = m_vectorSize - (int) Math.round( ( ( targetPPMShift + maxShiftInPPM - ppmReference ) * resonanceFrequency / ( 1000000 * spectralWidth ) + 0.5d ) * ( m_vectorSize + zerofillingsize - 1 ) );
             
        if ( m_loadSignal_1_jRadioButton_.isSelected( ) == true )
        {
            System.out.println( "Response 1: Reformatting {abs1,disp1,abs2,disp2,..} to  {abs1,abs2, .. disp1,disp2} format." );
            
            // if user does no preprocessing the outPut is equal to the input
            outPut = m_epsiDataFirst;
            
            if ( m_shiftResponseAtImportjCheckBox.isSelected( )  == true )
            {
                outPut = automaticFrequencyShift( m_epsiDataFirst, targetPPMShiftIndex, minAllowedPPMShiftIndex, maxAllowedPPMShiftIndex );
                m_epsiDataFirst = outPut;
            }
            if ( m_correctZeroOrderAtImportjCheckBox.isSelected( )  == true )
            {
                outPut = automaticCorrectZeroOrderPhase( m_epsiDataFirst, minimumppmIndex, maximumppmIndex );
            }
        }

        if ( m_loadSignal_2_jRadioButton.isSelected( ) == true )
        {
            System.out.println( "Response 2: Reformatting {abs1,disp1,abs2,disp2,..} to  {abs1,abs2, .. disp1,disp2} format." );
            
            // if user does no preprocessing the outPut is equal to the input
            outPut = m_epsiDataSecond;
            
            if ( m_shiftResponseAtImportjCheckBox.isSelected( )  == true )
            {
                outPut = automaticFrequencyShift( m_epsiDataSecond, targetPPMShiftIndex, minAllowedPPMShiftIndex, maxAllowedPPMShiftIndex );
                m_epsiDataSecond = outPut;
            }
            if ( m_correctZeroOrderAtImportjCheckBox.isSelected( )  == true )
            {
                outPut = automaticCorrectZeroOrderPhase( m_epsiDataSecond, minimumppmIndex, maximumppmIndex );
            }
        }
   
        if ( m_loadSignal_1_minus_2_jRadioButton.isSelected( ) == true )
        {
            System.out.println( "Response 1 - Response 2: Reformatting {abs1,disp1,abs2,disp2,..} to  {abs1,abs2, .. disp1,disp2} format." );
            
            float[] dummy1;
            float[] dummy2;
            
            if ( m_shiftResponseAtImportjCheckBox.isSelected( )  == true )
            {
                dummy1 = automaticFrequencyShift( m_epsiDataFirst, targetPPMShiftIndex, minAllowedPPMShiftIndex, maxAllowedPPMShiftIndex );
                dummy2 = automaticFrequencyShift( m_epsiDataSecond, targetPPMShiftIndex, minAllowedPPMShiftIndex, maxAllowedPPMShiftIndex );
            }
            else
            {
                dummy1 = m_epsiDataFirst;
                dummy2 = m_epsiDataSecond;
            }
            
            if ( m_correctZeroOrderAtImportjCheckBox.isSelected( )  == true )
            {
                dummy1 = automaticCorrectZeroOrderPhase( dummy1, minimumppmIndex, maximumppmIndex );
                dummy2 = automaticCorrectZeroOrderPhase( dummy2, minimumppmIndex, maximumppmIndex );
            }
            
            
            for ( int i = 0; i < m_epsiDataFirst.length/2; i++ )
            {                
                outPut[2*i]     = dummy1[2*i] - dummy2[2*i];
                outPut[2*i + 1] = dummy1[2*i + 1] - dummy2[2*i + 1];  
            }
        }
        
        if ( m_loadSignal_2_minus_Signal_1_jRadioButton.isSelected( ) == true )
        {
            System.out.println( "Response 2 - Response 1: Reformatting {abs1,disp1,abs2,disp2,..} to  {abs1,abs2, .. disp1,disp2} format." );
            
            float[] dummy1;
            float[] dummy2;
            
            if ( m_shiftResponseAtImportjCheckBox.isSelected( )  == true )
            {
                dummy1 = automaticFrequencyShift( m_epsiDataFirst, targetPPMShiftIndex, minAllowedPPMShiftIndex, maxAllowedPPMShiftIndex );
                dummy2 = automaticFrequencyShift( m_epsiDataSecond, targetPPMShiftIndex, minAllowedPPMShiftIndex, maxAllowedPPMShiftIndex );
            }
            else
            {
                dummy1 = m_epsiDataFirst;
                dummy2 = m_epsiDataSecond;
            }
            
            if ( m_correctZeroOrderAtImportjCheckBox.isSelected( )  == true )
            {
                dummy1 = automaticCorrectZeroOrderPhase( dummy1, minimumppmIndex, maximumppmIndex );
                dummy2 = automaticCorrectZeroOrderPhase( dummy2, minimumppmIndex, maximumppmIndex );
            }
            
            for ( int i = 0; i < m_epsiDataFirst.length/2; i++ )
            {
                outPut[2*i]     = dummy2[2*i]     - dummy1[2*i];
                outPut[2*i + 1] = dummy2[2*i + 1] - dummy1[2*i + 1];
            }
        }

        if ( m_loadSignal_1_plus_2_jRadioButton.isSelected( ) == true )
        {
            System.out.println( "Response 1 + Response 2: Reformatting {abs1,disp1,abs2,disp2,..} to  {abs1,abs2, .. disp1,disp2} format." );
            
            float[] dummy1;
            float[] dummy2;
            
            if ( m_shiftResponseAtImportjCheckBox.isSelected( )  == true )
            {
                dummy1 = automaticFrequencyShift( m_epsiDataFirst, targetPPMShiftIndex, minAllowedPPMShiftIndex, maxAllowedPPMShiftIndex );
                dummy2 = automaticFrequencyShift( m_epsiDataSecond, targetPPMShiftIndex, minAllowedPPMShiftIndex, maxAllowedPPMShiftIndex );
            }
            else
            {
                dummy1 = m_epsiDataFirst;
                dummy2 = m_epsiDataSecond;
            }
            
            if ( m_correctZeroOrderAtImportjCheckBox.isSelected( )  == true )
            {
                dummy1 = automaticCorrectZeroOrderPhase( dummy1, minimumppmIndex, maximumppmIndex );
                dummy2 = automaticCorrectZeroOrderPhase( dummy2, minimumppmIndex, maximumppmIndex );
            }
            
            for ( int i = 0; i < m_epsiDataFirst.length/2; i++ )
            {
                outPut[2*i]     = dummy1[2*i] + dummy2[2*i];
                outPut[2*i + 1] = dummy1[2*i + 1] + dummy2[2*i + 1];
            }
        }
        
        // Release memory of redundent array ..
        m_epsiDataFirst = null;
        m_epsiDataSecond = null;
        
        if ( m_signal_As_Is_RadioButton.isSelected( ) == true )
        {
            m_csiData = new CSIData( m_globalModel, outPut, 
                                     m_dimensions, timeSamples,
                                     EPSI_kt_sampling_frequency, resonance_frequency_EPSI,
                                     TE_EPSI, TR_EPSI, 
                                     new Integer( m_resonanceFrequencyValuerjTextField.getText( ) ), 
                                     NO_SPATIAL_FILTERING_AT_IMPORT_ID, 
                                     new Integer( m_numberOfNeighbourPointsTextField.getText( ) ).intValue( ),
                                     (float)new Double( m_referencePPMValuerjTextField.getText( ) ).doubleValue( ) );
           
        }
        
        if ( ( m_2DsignalMedianFilterRadioButton.isSelected( ) == true  ) &&
             ( m_useAllNeighboursRadioButton.isSelected( ) == true ) )
        {
            m_csiData = new CSIData( m_globalModel, outPut, 
                                     m_dimensions, timeSamples,
                                     EPSI_kt_sampling_frequency, resonance_frequency_EPSI,
                                     TE_EPSI, TR_EPSI, 
                                     new Integer( m_resonanceFrequencyValuerjTextField.getText( ) ), 
                                     MOVING_FULL_2D_MEDIAN_SPATIAL_FILTERING_AT_IMPORT_ID,
                                     new Integer( m_numberOfNeighbourPointsTextField.getText( ) ).intValue( ),
                                     (float)new Double( m_referencePPMValuerjTextField.getText( ) ).doubleValue( ) );
        }
        
        if ( ( m_2DsignalAveragingRadioButton.isSelected( ) == true ) &&
             (  m_useAllNeighboursRadioButton.isSelected( ) == true ) )
        {
            m_csiData = new CSIData( m_globalModel, outPut, 
                                     m_dimensions, timeSamples,
                                     EPSI_kt_sampling_frequency, resonance_frequency_EPSI,
                                     TE_EPSI, TR_EPSI, 
                                     new Integer( m_resonanceFrequencyValuerjTextField.getText( ) ), 
                                     MOVING_FULL_2D_AVERAGE_SPATIAL_FILTERING_AT_IMPORT_ID,
                                     new Integer( m_numberOfNeighbourPointsTextField.getText( ) ).intValue( ),
                                     (float)new Double( m_referencePPMValuerjTextField.getText( ) ).doubleValue( ) ); 
        }
        
        if ( ( m_3DsignalMedianFilterRadioButton.isSelected( ) == true ) &&
             ( m_useAllNeighboursRadioButton.isSelected( ) == true ) )
        {
            m_csiData = new CSIData( m_globalModel, outPut, 
                                     m_dimensions, timeSamples,
                                     EPSI_kt_sampling_frequency, resonance_frequency_EPSI,
                                     TE_EPSI, TR_EPSI, 
                                     new Integer( m_resonanceFrequencyValuerjTextField.getText( ) ), 
                                     MOVING_FULL_3D_MEDIAN_SPATIAL_FILTERING_AT_IMPORT_ID,
                                     new Integer( m_numberOfNeighbourPointsTextField.getText( ) ).intValue( ),
                                     (float)new Double( m_referencePPMValuerjTextField.getText( ) ).doubleValue( ) );
        }
        
        if ( ( m_3DsignalAveragingRadioButton.isSelected( ) == true ) &&
             (  m_useAllNeighboursRadioButton.isSelected( ) == true ) )
        {
            m_csiData = new CSIData( m_globalModel, outPut, 
                                     m_dimensions, timeSamples,
                                     EPSI_kt_sampling_frequency, resonance_frequency_EPSI,
                                     TE_EPSI, TR_EPSI, 
                                     new Integer( m_resonanceFrequencyValuerjTextField.getText( ) ), 
                                     MOVING_FULL_3D_AVERAGE_SPATIAL_FILTERING_AT_IMPORT_ID,
                                     new Integer( m_numberOfNeighbourPointsTextField.getText( ) ).intValue( ),
                                     (float)new Double( m_referencePPMValuerjTextField.getText( ) ).doubleValue( ) ); 
        }
        
        
        if ( ( m_2DsignalMedianFilterRadioButton.isSelected( ) == true  ) &&
             ( m_useAllNeighboursRadioButton.isSelected( ) == false ) )
        {
            m_csiData = new CSIData( m_globalModel, outPut, 
                                     m_dimensions, timeSamples,
                                     EPSI_kt_sampling_frequency, resonance_frequency_EPSI,
                                     TE_EPSI, TR_EPSI, 
                                     new Integer( m_resonanceFrequencyValuerjTextField.getText( ) ), 
                                     MOVING_NEAREST_NEIGHBOURS_ONLY_2D_MEDIAN_SPATIAL_FILTERING_AT_IMPORT_ID,
                                     new Integer( m_numberOfNeighbourPointsTextField.getText( ) ).intValue( ),
                                     (float)new Double( m_referencePPMValuerjTextField.getText( ) ).doubleValue( ) );
        }
        
        if ( ( m_2DsignalAveragingRadioButton.isSelected( ) == true ) &&
             (  m_useAllNeighboursRadioButton.isSelected( ) == false ) )
        {
            m_csiData = new CSIData( m_globalModel, outPut, 
                                     m_dimensions, timeSamples,
                                     EPSI_kt_sampling_frequency, resonance_frequency_EPSI,
                                     TE_EPSI, TR_EPSI, 
                                     new Integer( m_resonanceFrequencyValuerjTextField.getText( ) ), 
                                     MOVING_NEAREST_NEIGHBOURS_ONLY_2D_AVERAGE_SPATIAL_FILTERING_AT_IMPORT_ID,
                                     new Integer( m_numberOfNeighbourPointsTextField.getText( ) ).intValue( ),
                                     (float)new Double( m_referencePPMValuerjTextField.getText( ) ).doubleValue( ) ); 
        }
        
        if ( ( m_3DsignalMedianFilterRadioButton.isSelected( ) == true ) &&
             ( m_useAllNeighboursRadioButton.isSelected( ) == false ) )
        {
            m_csiData = new CSIData( m_globalModel, outPut, 
                                     m_dimensions, timeSamples,
                                     EPSI_kt_sampling_frequency, resonance_frequency_EPSI,
                                     TE_EPSI, TR_EPSI, 
                                     new Integer( m_resonanceFrequencyValuerjTextField.getText( ) ), 
                                     MOVING_NEAREST_NEIGHBOURS_ONLY_3D_MEDIAN_SPATIAL_FILTERING_AT_IMPORT_ID,
                                     new Integer( m_numberOfNeighbourPointsTextField.getText( ) ).intValue( ),
                                     (float)new Double( m_referencePPMValuerjTextField.getText( ) ).doubleValue( ) );
        }
        
        if ( ( m_3DsignalAveragingRadioButton.isSelected( ) == true ) &&
             (  m_useAllNeighboursRadioButton.isSelected( ) == false ) )
        {
            m_csiData = new CSIData( m_globalModel, outPut, 
                                     m_dimensions, timeSamples,
                                     EPSI_kt_sampling_frequency, resonance_frequency_EPSI,
                                     TE_EPSI, TR_EPSI, 
                                     new Integer( m_resonanceFrequencyValuerjTextField.getText( ) ), 
                                     MOVING_NEAREST_NEIGHBOURS_ONLY_3D_AVERAGE_SPATIAL_FILTERING_AT_IMPORT_ID,
                                     new Integer( m_numberOfNeighbourPointsTextField.getText( ) ).intValue( ),
                                     (float)new Double( m_referencePPMValuerjTextField.getText( ) ).doubleValue( ) ); 
        }
        
        // this filename needs to be set to be able to create .tfr files.
         m_csiData.setFile( m_epsiFileReferenceFirst );
        
        if ( ( m_epsiFileReferenceFirst != null ) && ( m_epsiFileReferenceSecond != null ) )
        {
             m_globalModel.getCSIData( ).setFile( m_epsiFileReferenceFirst );
        }
        
        if ( ( m_epsiFileReferenceFirst == null ) && ( m_epsiFileReferenceSecond != null ) )
        {
             m_globalModel.getCSIData( ).setFile( m_epsiFileReferenceSecond );
        }
        
        m_globalModel.setImageStack( m_globalModel.getImageStack( ) );        
        
        m_globalModel.setSpectraDomain( Spectra.Domain.FD_PPM );
        m_globalModel.setCSIData( m_csiData );
        
        notifyActionListeners( this, 
                               EPSI_MIDAS_PREPROCESSED_DATA_LOADED_ID, 
                               EPSI_MIDAS_PREPROCESSED_DATA_LOADED_CMD );
        
        m_globalModel.setIsManualMidasEPSIImported( true );
        
        // hide the display would be better -> to be implmented
        dispose( );
    }
    
    private void m_importMidasPreprocessedEPSIDatajButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_importMidasPreprocessedEPSIDatajButtonActionPerformed
        doImportOfMidasPreprocessedData( ); 
    }//GEN-LAST:event_m_importMidasPreprocessedEPSIDatajButtonActionPerformed

    private void m_cancelWitoutLoadMidasPreprocessedEPSIDatajButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_cancelWitoutLoadMidasPreprocessedEPSIDatajButtonActionPerformed
        this.dispose( );        // TODO add your handling code here:
    }//GEN-LAST:event_m_cancelWitoutLoadMidasPreprocessedEPSIDatajButtonActionPerformed

    private void m_selectEPSIBinaryFileSecondjButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_selectEPSIBinaryFileSecondjButtonActionPerformed
        
        String testFile = getEPSIBinaryDataFileReferenceSecond( );
        
        if ( testFile != null )
        {
           m_epsiFileReferenceSecond = testFile; 
        }
        
        m_lastSelectedMidasSIDFileNamejTextFieldSecond.setText( m_epsiFileReferenceSecond );
        
        updateGUIWithValidityOfSelectedSecondEPSIFile( );  
        
        File sizeOfFile1 = new File( m_epsiFileReferenceSecond );
        long length1 = sizeOfFile1.length( );

        m_timeSamples = (int)( length1 / ( 8 * m_dimensions[0] * m_dimensions[1] * m_dimensions[2] ) );
        
        this.m_numberOfTDSamplesjTextField.setText( "" + m_timeSamples );
        
    
        if ( ( ( new File( m_epsiFileReferenceFirst ) ).exists( ) == true ) ||
             ( ( new File( m_epsiFileReferenceSecond ) ).exists( ) == true ) )
        {
            m_importMidasPreprocessedEPSIDatajButton.setEnabled( true );
        }
        
        TDFDFittingDefaults def = TDFDFittingDefaults.getInstance( );
        def.load( );
        def.setProperty( TDFDFittingDefaults.SELECTED_EPSI_BINARY_FILE_REFERENCE_SECOND, m_epsiFileReferenceSecond );
        def.save( );
    }//GEN-LAST:event_m_selectEPSIBinaryFileSecondjButtonActionPerformed

    private void m_lastSelectedMidasSIDFileNamejTextFieldFirstActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_lastSelectedMidasSIDFileNamejTextFieldFirstActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_m_lastSelectedMidasSIDFileNamejTextFieldFirstActionPerformed

    private void m_loadSignal_1_jRadioButton_ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_loadSignal_1_jRadioButton_ActionPerformed
        m_lastSelectedMidasSIDFileNamejTextFieldFirst.setEnabled(true);
        m_lastSelectedMidasSIDFileNamejTextFieldSecond.setEnabled(false);
    }//GEN-LAST:event_m_loadSignal_1_jRadioButton_ActionPerformed

    private void m_loadSignal_2_jRadioButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_loadSignal_2_jRadioButtonActionPerformed
        m_lastSelectedMidasSIDFileNamejTextFieldSecond.setEnabled(true);
        m_lastSelectedMidasSIDFileNamejTextFieldFirst.setEnabled(false);
    }//GEN-LAST:event_m_loadSignal_2_jRadioButtonActionPerformed

    private void m_loadSignal_1_plus_2_jRadioButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_loadSignal_1_plus_2_jRadioButtonActionPerformed
        m_lastSelectedMidasSIDFileNamejTextFieldFirst.setEnabled(true);
        m_lastSelectedMidasSIDFileNamejTextFieldSecond.setEnabled(true);
    }//GEN-LAST:event_m_loadSignal_1_plus_2_jRadioButtonActionPerformed

    private void m_loadSignal_2_minus_Signal_1_jRadioButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_loadSignal_2_minus_Signal_1_jRadioButtonActionPerformed
        m_lastSelectedMidasSIDFileNamejTextFieldFirst.setEnabled(true);
        m_lastSelectedMidasSIDFileNamejTextFieldSecond.setEnabled(true);
    }//GEN-LAST:event_m_loadSignal_2_minus_Signal_1_jRadioButtonActionPerformed

    private void m_loadSignal_1_minus_2_jRadioButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_loadSignal_1_minus_2_jRadioButtonActionPerformed
        m_lastSelectedMidasSIDFileNamejTextFieldFirst.setEnabled(true);
        m_lastSelectedMidasSIDFileNamejTextFieldSecond.setEnabled(true);
    }//GEN-LAST:event_m_loadSignal_1_minus_2_jRadioButtonActionPerformed

    private void m_signal_As_Is_RadioButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_signal_As_Is_RadioButtonActionPerformed
        if ( this.m_signal_As_Is_RadioButton.isSelected( ) )
        {
            m_numberOfNeighbourPointsTextField.setEnabled(false);
            m_useAllNeighboursRadioButton.setEnabled(false);
            m_useNearerstNeighboursRadioButton.setEnabled(false);
        }
        
  
        
    }//GEN-LAST:event_m_signal_As_Is_RadioButtonActionPerformed

    private void m_2DsignalMedianFilterRadioButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_2DsignalMedianFilterRadioButtonActionPerformed
       m_numberOfNeighbourPointsTextField.setEnabled(true);
       m_useAllNeighboursRadioButton.setEnabled(true);
       m_useNearerstNeighboursRadioButton.setEnabled(true);
    }//GEN-LAST:event_m_2DsignalMedianFilterRadioButtonActionPerformed

    private void m_2DsignalAveragingRadioButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_2DsignalAveragingRadioButtonActionPerformed
        m_numberOfNeighbourPointsTextField.setEnabled(true);
        m_useAllNeighboursRadioButton.setEnabled(true);
        m_useNearerstNeighboursRadioButton.setEnabled(true);
    }//GEN-LAST:event_m_2DsignalAveragingRadioButtonActionPerformed

    private void m_3DsignalMedianFilterRadioButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_3DsignalMedianFilterRadioButtonActionPerformed
        m_numberOfNeighbourPointsTextField.setEnabled(true);
        m_useAllNeighboursRadioButton.setEnabled(true);
        m_useNearerstNeighboursRadioButton.setEnabled(true);
    }//GEN-LAST:event_m_3DsignalMedianFilterRadioButtonActionPerformed

    private void m_3DsignalAveragingRadioButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_3DsignalAveragingRadioButtonActionPerformed
        m_numberOfNeighbourPointsTextField.setEnabled(true);
        m_useAllNeighboursRadioButton.setEnabled(true);
        m_useNearerstNeighboursRadioButton.setEnabled(true);
    }//GEN-LAST:event_m_3DsignalAveragingRadioButtonActionPerformed

    private void m_useAllNeighboursRadioButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_useAllNeighboursRadioButtonActionPerformed
        m_numberOfNeighbourPointsTextField.setEnabled(true);
        m_useAllNeighboursRadioButton.setEnabled(true);
        m_useNearerstNeighboursRadioButton.setEnabled(true);
    }//GEN-LAST:event_m_useAllNeighboursRadioButtonActionPerformed

    private void m_useNearerstNeighboursRadioButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_useNearerstNeighboursRadioButtonActionPerformed
        m_numberOfNeighbourPointsTextField.setEnabled(true);
        m_useAllNeighboursRadioButton.setEnabled(true);
        m_useNearerstNeighboursRadioButton.setEnabled(true);
    }//GEN-LAST:event_m_useNearerstNeighboursRadioButtonActionPerformed

    private void m_correctZeroOrderAtImportjCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_correctZeroOrderAtImportjCheckBoxActionPerformed
        if ( m_correctZeroOrderAtImportjCheckBox.isSelected( ) == true )
        {
            m_minimumPPMRangetextField.setEnabled( true );
            m_maximumPPMRangetextField.setEnabled( true );
        }
        else
        {
            m_minimumPPMRangetextField.setEnabled( false );
            m_maximumPPMRangetextField.setEnabled( false);
        }
    }//GEN-LAST:event_m_correctZeroOrderAtImportjCheckBoxActionPerformed

    private void m_minimumPPMRangetextFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_minimumPPMRangetextFieldActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_m_minimumPPMRangetextFieldActionPerformed

    private void m_shiftResponseAtImportjCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_shiftResponseAtImportjCheckBoxActionPerformed
       
        if ( m_shiftResponseAtImportjCheckBox.isSelected( ) == true )
        {
            m_minimumAllowedPPMShiftjTtextField.setEnabled( true );
            m_maximumAllowedPPMShiftjTtextField.setEnabled( true );
            this.m_targetPPMShifttextField.setEnabled( true );
        }
        else
        {
            m_minimumAllowedPPMShiftjTtextField.setEnabled( false );
            m_maximumAllowedPPMShiftjTtextField.setEnabled( false );
            m_targetPPMShifttextField.setEnabled( false );
        }
        
    }//GEN-LAST:event_m_shiftResponseAtImportjCheckBoxActionPerformed

    private void m_minimumAllowedPPMShiftjTtextFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_minimumAllowedPPMShiftjTtextFieldActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_m_minimumAllowedPPMShiftjTtextFieldActionPerformed

    /**
     * @param args the command line arguments
     */
    public static void main( String args[] ) 
    {
        /* Set the Nimbus look and feel */
        //<editor-fold defaultstate="collapsed" desc=" Look and feel setting code (optional) ">
        /* If Nimbus (introduced in Java SE 6) is not available, stay with the default look and feel.
         * For details see http://download.oracle.com/javase/tutorial/uiswing/lookandfeel/plaf.html 
         */
        try {
            for (javax.swing.UIManager.LookAndFeelInfo info : javax.swing.UIManager.getInstalledLookAndFeels()) {
                if ("Nimbus".equals(info.getName())) {
                    javax.swing.UIManager.setLookAndFeel(info.getClassName());
                    break;
                }
            }
        } catch (ClassNotFoundException ex) {
            java.util.logging.Logger.getLogger(MidasPreprocessedEPSIDataImporterParameterSettingsControl.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (InstantiationException ex) {
            java.util.logging.Logger.getLogger(MidasPreprocessedEPSIDataImporterParameterSettingsControl.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (IllegalAccessException ex) {
            java.util.logging.Logger.getLogger(MidasPreprocessedEPSIDataImporterParameterSettingsControl.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (javax.swing.UnsupportedLookAndFeelException ex) {
            java.util.logging.Logger.getLogger(MidasPreprocessedEPSIDataImporterParameterSettingsControl.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        }
        //</editor-fold>
        //</editor-fold>

        /* Create and display the form */
        java.awt.EventQueue.invokeLater( new Runnable( ) 
        {
            @Override
            public void run( ) 
            {
//                new MidasPreprocessedEPSIDataImporterParameterSettingsControl().setVisible(true);
            }
        });
    }

    public void addActionListener( ActionListener listener ) 
    {
        m_actionListeners.add( listener );
    }
    
    private void notifyActionListeners( Object obj, int eventType, String actionCommand ) 
    {
        Iterator it = m_actionListeners.iterator( );
        while ( it.hasNext( ) ) 
        {
            ((ActionListener)it.next( ) ).actionPerformed( new ActionEvent( obj, eventType, actionCommand ) );
        }
    }
    
    public CSIData getLoadedCSIData( )
    {
        return m_csiData;
    }
    
   
    public float[] correctGuodongsData( float[] epsiImportedData  )
    {
        System.out.println( "CAUTION: Guodong data correction! .." );
           System.out.println( "Automatic frequency shift correction in progress .." );
        int cntr = 0;
        int outPutCntr = 0;
        
        float[] outputData = new float[epsiImportedData.length];
        
        float[] real = new float[(int)m_vectorSize];
        float[] imag = new float[(int)m_vectorSize];

        while ( cntr < epsiImportedData.length )
        {       
            for ( int i = 0; i < m_globalModel.getVectorSize( ); i++ )
            {
                epsiImportedData[cntr] = -epsiImportedData[cntr];
                cntr++;

                epsiImportedData[cntr] = epsiImportedData[cntr];
                cntr++;
            }  
        }
        return epsiImportedData;
    }
    
    
    
    
    
    public float[] automaticFrequencyShift( float[] epsiImportedData, int targetPPMShiftIndex, 
                                            int minAllowedPPMShiftIndex, int maxAllowedPPMShiftIndex  )
    {
        System.out.println( "Automatic frequency shift correction in progress .." );
        int cntr = 0;
        int outPutCntr = 0;
        int caseCounter = 0;
        
        float[] outputData = new float[epsiImportedData.length];
        
        float[] real = new float[(int)m_vectorSize];
        float[] imag = new float[(int)m_vectorSize];

        while ( cntr < epsiImportedData.length )
        {       
            for ( int i = 0; i < m_globalModel.getVectorSize( ); i++ )
            {
                real[i] = epsiImportedData[cntr];
                cntr++;

                imag[i] = epsiImportedData[cntr];
                cntr++;
            }
            
            float requiredFrequencyShift = getRequiredFrequencyShift( real, imag, targetPPMShiftIndex, 
                                                                      minAllowedPPMShiftIndex, maxAllowedPPMShiftIndex  );
            
            applyRequiredFrequencyShift( requiredFrequencyShift, real, imag );
            
            for ( int i = 0; i < m_globalModel.getVectorSize( ); i++ )
            {
                outputData[outPutCntr] = m_globalModel.getVectorSize( ) * m_current_real_array_to_phase[i];
                outPutCntr++;
            
                outputData[outPutCntr] = m_globalModel.getVectorSize( ) * m_current_imag_array_to_phase[i];
                outPutCntr++;
            }
            
//            if ( ( caseCounter > 12000 ) && ( caseCounter < 12011 ) )
//            {    
//                float[] xAxis = new float[m_globalModel.getVectorSize( )];
//                for ( int i = 0; i < m_globalModel.getVectorSize( ); i++ )
//                {
//                    xAxis[i] = (float)i;
//                }
//            
//                // System.out.println( "Required frequency shift = " + ( requiredFrequencyShift ) + " points."  );
//                for ( int k = 0; k < m_globalModel.getVectorSize( ); k++ )
//                {
//                    m_current_real_array_to_phase[k] = m_current_real_array_to_phase[k] / m_globalModel.getVectorSize( );
//                    m_current_imag_array_to_phase[k] = m_current_imag_array_to_phase[k] / m_globalModel.getVectorSize( );
//                }
//                XYPlotFrame shiftPlotFrame = new XYPlotFrame( "Shift effect", null );
//                Curve originalCurve = new Curve( xAxis, real, Color.black );
//                Curve shiftedCurve = new Curve( xAxis, m_current_real_array_to_phase, Color.red );
//                shiftPlotFrame.addCurve( originalCurve, true ); 
//                shiftPlotFrame.addCurve( shiftedCurve, true );       
//            }
            caseCounter++;
        }
        return outputData;
    }
    
    /**
     * This method performs an automatic zero order phasing on the EPSI
     * imported data
     * @param epsiImportedData
     * @param minppmIndex
     * @param maxppmIndex
     * @return 
     */
    public float[] automaticCorrectZeroOrderPhase( float[] epsiImportedData, int minppmIndex, int maxppmIndex  )
    {
        System.out.println( "Autophasing response (zero order only) in progress" );
        int cntr = 0;
        int outPutCntr = 0;
        int caseCounter = 0;
        
        float[] outputData = new float[epsiImportedData.length];
        
        float[] real = new float[(int)m_vectorSize];
        float[] imag = new float[(int)m_vectorSize];

        while ( cntr < epsiImportedData.length )
        {       
            for ( int i = 0; i < m_globalModel.getVectorSize( ); i++ )
            {
                real[i] = epsiImportedData[cntr];
                cntr++;

                imag[i] = epsiImportedData[cntr];
                cntr++;
            }
            
            float requiredZOPhase = getRequiredZeroOrderPhase( real, imag, minppmIndex, maxppmIndex );
            
            applyZeroOrderPhaseToCurrentResponse( requiredZOPhase, real, imag );
            
            for ( int i = 0; i < m_globalModel.getVectorSize( ); i++ )
            {
                outputData[outPutCntr] = m_current_real_array_to_phase[i];
                outPutCntr++;
            
                outputData[outPutCntr] = m_current_imag_array_to_phase[i];
                outPutCntr++;
            }
            
            // System.out.println( " Response number " + caseCounter++ + " is phased using = " + ( requiredZOPhase * 180f / Math.PI ) + " degrees."  );
            
        }
        
        return outputData;
    }
    
    void applyZeroOrderPhaseToCurrentResponse( float zeroOrderPhase, float[] real, float[] imag )
    {
        
        m_current_real_array_to_phase = new float[m_globalModel.getVectorSize( )];
        m_current_imag_array_to_phase = new float[m_globalModel.getVectorSize( )];

        float cosPhi = (float)Math.cos( (double)zeroOrderPhase );
        float sinPhi = (float)Math.sin( (double)zeroOrderPhase  );
        
        for ( int i = 0; i < real.length; i++ )
        {
            m_current_real_array_to_phase[i] = real[i] * cosPhi - imag[i] * sinPhi;
            m_current_imag_array_to_phase[i] = real[i] * sinPhi + imag[i] * cosPhi;   
        }
    }
   
    /**
     * Nuno's original phasing code which works pretty good on average!
     * @param realSpectrum
     * @param imaginarySpectrum
     * @param minppmIndex
     * @param maxppmIndex
     * @return 
     */
    public float getRequiredZeroOrderPhase( float[] realSpectrum, float[] imaginarySpectrum, int minppmIndex, int maxppmIndex ) 
    {
              
       float phase0 = 0f;
       
       float maxscore = getAbsorptionSumFD( phase0, realSpectrum, imaginarySpectrum, minppmIndex, maxppmIndex );
        
       float phase0max = phase0;
       float auxscore1;
       float auxscore2;
        
       for ( int i = 1; i < 15; i++ ) 
       {
            phase0 += Math.PI * Math.pow(0.5, i);
            auxscore1 = getAbsorptionSumFD( phase0, realSpectrum, imaginarySpectrum, minppmIndex, maxppmIndex );
            phase0 -= Math.PI * 2 * Math.pow(0.5, i);
            auxscore2 = getAbsorptionSumFD( phase0, realSpectrum, imaginarySpectrum, minppmIndex, maxppmIndex );
            if ( auxscore2 > auxscore1 ) 
            {
                if ( auxscore2 > maxscore ) 
                {
                    maxscore = auxscore2;
                    phase0max = phase0;
                }
            } 
            else 
            {
                phase0 += Math.PI * 2 * Math.pow( 0.5, i );
                if ( auxscore1 > maxscore ) 
                {
                    maxscore = auxscore1;
                    phase0max = phase0;
                }
            }
        }
        
        phase0 = phase0max % ( (float)Math.PI * 2 );
        
        if ( phase0 < -(float)Math.PI ) 
        {
            phase0 += ( (float)Math.PI * 2 );
        } 
        else if ( phase0 > (float)Math.PI ) 
        {
            phase0 -= ( (float)Math.PI * 2 );
        }
        
        return phase0;
        
    }
    
    
    /**
     * Apply the frequency shift in TD.
     * @param requiredFrequencyShift
     * @param real
     * @param imag 
     */
    public void applyRequiredFrequencyShift( float requiredFrequencyShift, 
                                             float[] real, float[] imag )
    {
        // simple short cut!
        m_current_real_array_to_phase = new float[real.length];
        m_current_imag_array_to_phase = new float[real.length];
        
        float[][] vaux = FD2TD_EPSI( real, imag );
        
        for ( int i = 0; i < real.length; i++ )
        {
            m_current_real_array_to_phase[i] = vaux[i][0] * (float)Math.cos( i * 2f * Math.PI * requiredFrequencyShift / real.length ) - vaux[i][1] * (float)Math.sin( i * 2f * Math.PI * requiredFrequencyShift / real.length );
            m_current_imag_array_to_phase[i] = vaux[i][0] * (float)Math.sin( i * 2f * Math.PI * requiredFrequencyShift / real.length ) + vaux[i][1] * (float)Math.cos( i * 2f * Math.PI * requiredFrequencyShift / real.length );   
        }
        
        float[][] vauxFD = TD2FD_EPSI( m_current_real_array_to_phase, m_current_imag_array_to_phase );
        
        for ( int i = 0; i < real.length; i++ )
        {
            m_current_real_array_to_phase[i] = vauxFD[i][0];
            m_current_imag_array_to_phase[i] = vauxFD[i][1] ;   
        }
        
    }
    
    public float[][] FD2TD_EPSI( float[] abs, float[] disp ) 
    {
        float[][] vaux = new float[abs.length][2];

        for ( int m = 0; m < abs.length; m++ ) 
        {
            int d = (m + 1 + abs.length / 2) % abs.length;
            
            vaux[d][0] = abs[abs.length - 1 - m];
            vaux[d][1] = disp[disp.length - 1 - m];
        }
        
        FFT fft = new FFT( );
        vaux = fft.fft_1d( vaux );

        return vaux;
    }
    
    public float[][] TD2FD_EPSI( float[] abs, float[] disp ) 
    {
        float[][] vaux = new float[abs.length][2];

        for ( int m = 0; m < abs.length; m++ ) 
        {
            // int d = (m + 1 + abs.length / 2) % abs.length;
            vaux[m][0] = abs[m];
            vaux[m][1] = disp[m];
        }
        
        FFT fft = new FFT( );
        vaux = fft.fft_1d( vaux );

        return vaux;
    }
    
    /**
     * 
     * @param realSpectrum
     * @param imaginarySpectrum
     * @param targetPPMShiftIndex
     * @param minAllowedPPMShiftIndex
     * @param maxAllowedPPMShiftIndex
     * @return 
     */
    public float getRequiredFrequencyShift( float[] realSpectrum, 
                                            float[] imaginarySpectrum, 
                                            int targetPPMShiftIndex, 
                                            int minAllowedPPMShiftIndex, 
                                            int maxAllowedPPMShiftIndex ) 
    {
       float requiredShift  = -1f;       
       float max            = -Float.MAX_VALUE;
       int maxIndex         = -1;
       float shift_x_as_float = 512f;
       
       float[] abs = new float[realSpectrum.length];
       
       for ( int i = minAllowedPPMShiftIndex; i < maxAllowedPPMShiftIndex; i++ )
       {
           
           abs[i] = ( realSpectrum[i] * realSpectrum[i] + imaginarySpectrum[i] * imaginarySpectrum[i] );
           if ( abs[i] > max )
           {
               max = abs[i];
               maxIndex = i;
           }
       }
   
       // Use Lagrange interpolation polynomial: assume second order ...
       if ( ( maxIndex > minAllowedPPMShiftIndex ) && ( maxIndex < maxAllowedPPMShiftIndex ) )
       {
//            System.out.println( "Target index  = " + targetPPMShiftIndex );
//            System.out.println( "Minimum index range = " + minAllowedPPMShiftIndex );
//            System.out.println( "Maximum index range = " + maxAllowedPPMShiftIndex );

            float x1 = maxIndex - 1f;
            float x2 = maxIndex;
            float x3 = maxIndex + 1f;
           
            float k1 = abs[maxIndex-1] / ( ( x1 - x2 ) * ( x1 - x3 ) );
            float k2 =   abs[maxIndex] / ( ( x2 - x1 ) * ( x2 - x3 ) );
            float k3 = abs[maxIndex+1] / ( ( x3 - x2 ) * ( x3 - x1 ) );
           
            shift_x_as_float = ( k1 * ( x2 + x3 ) + k2 * ( x1 + x3 ) + k3 * ( x1 + x2 ) ) / ( 2 * ( k1 + k2 + k3 ) );
       }
       

//       System.out.println( " *  Required shift = " + ( -shift_x_as_float ) );
//       System.out.println( " ** Required shift = " + ( shift_x_as_float - targetPPMShiftIndex ) );
       
       return ( -shift_x_as_float );
        
    }
    
    
    
    /**
     * Algorithm that returns the integral of the absorption time domain signal.
     * @param zeroOrderPhase
     * @param realSpectrum
     * @param imaginarySpectrum
     * @param minppmIndex
     * @param maxppmIndex
     * @return 
     */
    private float getAbsorptionSumFD( float zeroOrderPhase, float[] realSpectrum, float[] imaginarySpectrum, int minppmIndex, int maxppmIndex )
    {
        float absSum = 0f;
        
        float cosPhi = (float)Math.cos( (double)zeroOrderPhase );
        float sinPhi = (float)Math.sin( (double)zeroOrderPhase );

        float[] absorption = new float[realSpectrum.length];
        
        for ( int i = 0; i < absorption.length; i++ )
        {
            if ( i < realSpectrum.length )
            {
                absorption[i] = realSpectrum[i] * cosPhi - imaginarySpectrum[i] * sinPhi;
            }
        }
        
        for ( int i = 0; i < absorption.length; i++ )
        {
            if ( ( i < maxppmIndex ) &&  ( i > minppmIndex ) )
            {
                absSum = absSum + absorption[i];
            }
        }
        
        // System.out.println( "absSum = " + absSum );
        
        return absSum;
    }
    
    public float[] reorderMidasData( float[] rawEPSIData )
    {
        float[] orderedOutputData = new float[rawEPSIData.length];
        
        for ( int i = 0; i < rawEPSIData.length/2 ; i++ )
        { 
            orderedOutputData[2*i]   = rawEPSIData[2*i];
            orderedOutputData[2*i+1] = rawEPSIData[2*i+1];
        }
        
        return orderedOutputData;
        
    }
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton jButton1;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel10;
    private javax.swing.JLabel jLabel11;
    private javax.swing.JLabel jLabel12;
    private javax.swing.JLabel jLabel13;
    private javax.swing.JLabel jLabel14;
    private javax.swing.JLabel jLabel15;
    private javax.swing.JLabel jLabel16;
    private javax.swing.JLabel jLabel17;
    private javax.swing.JLabel jLabel18;
    private javax.swing.JLabel jLabel19;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel20;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel jLabel8;
    private javax.swing.JLabel jLabel9;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JPanel jPanel5;
    private javax.swing.JPanel jPanel6;
    private javax.swing.ButtonGroup load_option_buttongroup;
    private javax.swing.JRadioButton m_2DsignalAveragingRadioButton;
    private javax.swing.JRadioButton m_2DsignalMedianFilterRadioButton;
    private javax.swing.JRadioButton m_3DsignalAveragingRadioButton;
    private javax.swing.JRadioButton m_3DsignalMedianFilterRadioButton;
    private javax.swing.JButton m_cancelWitoutLoadMidasPreprocessedEPSIDatajButton;
    private javax.swing.JCheckBox m_correctZeroOrderAtImportjCheckBox;
    private javax.swing.JTextField m_epsiSamplingFrequencyValuerjTextField;
    private javax.swing.JButton m_importMidasPreprocessedEPSIDatajButton;
    private javax.swing.JTextField m_lastSelectedMidasSIDFileNamejTextFieldFirst;
    private javax.swing.JTextField m_lastSelectedMidasSIDFileNamejTextFieldSecond;
    private javax.swing.JRadioButton m_loadSignal_1_jRadioButton_;
    private javax.swing.JRadioButton m_loadSignal_1_minus_2_jRadioButton;
    private javax.swing.JRadioButton m_loadSignal_1_plus_2_jRadioButton;
    private javax.swing.JRadioButton m_loadSignal_2_jRadioButton;
    private javax.swing.JRadioButton m_loadSignal_2_minus_Signal_1_jRadioButton;
    private java.awt.TextField m_maximumAllowedPPMShiftjTtextField;
    private java.awt.TextField m_maximumPPMRangetextField;
    private java.awt.TextField m_minimumAllowedPPMShiftjTtextField;
    private java.awt.TextField m_minimumPPMRangetextField;
    private javax.swing.JTextField m_numberOfInPlaneYSamplesjTextField;
    private javax.swing.JTextField m_numberOfInplaneXSamplesjTextField;
    private javax.swing.JTextField m_numberOfNeighbourPointsTextField;
    private javax.swing.JTextField m_numberOfPlanesZDirectionjTextField;
    private javax.swing.JTextField m_numberOfTDSamplesjTextField;
    private javax.swing.JTextField m_referencePPMValuerjTextField;
    private javax.swing.JTextField m_resonanceFrequencyValuerjTextField;
    private javax.swing.JButton m_selectEPSIBinaryFileSecondjButton;
    private javax.swing.JButton m_selectEPSIBinaryFilejFirstButton;
    private javax.swing.JCheckBox m_shiftResponseAtImportjCheckBox;
    private javax.swing.JRadioButton m_signal_As_Is_RadioButton;
    private java.awt.TextField m_targetPPMShifttextField;
    private javax.swing.JRadioButton m_useAllNeighboursRadioButton;
    private javax.swing.JRadioButton m_useNearerstNeighboursRadioButton;
    private javax.swing.ButtonGroup signal_preprocess_buttongroup;
    private javax.swing.ButtonGroup use_neighbour_type_options_buttongroup;
    // End of variables declaration//GEN-END:variables


}