/*
 * File: $Id: MRPatientSpectrumDataBaseItem.java,v 1.1.1.1 2006/04/19 09:38:48 hans Exp $
 * History
 * Date: 05.01.2004, 22:06  Authors: Hans Slotboom
 *
 * Changes:
 *
 * Copyright (C) 2003 -2004 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */


/*
 * MRPatientSpectrumDataBaseItem.java
 *
 * Created on January 5, 2004, 10:06 PM
 */

package org.spectrim.patientdatabase;

import java.util.LinkedList;
import java.util.Iterator;

import java.io.IOException;
import java.io.RandomAccessFile; 

/**
 *
 * @author  hans
 */
public class MRPatientSpectrumDataBaseItem 
{    
    /** 
     * Creates a new instance of MRPatientSpectrumDataBaseItem 
     */
    public MRPatientSpectrumDataBaseItem( ) 
    {
        m_dicomRawFileReference = "none";
        m_dicomRawWaterReferenceFileReference = "none";
        
        m_tdfdfitSpectrumToFitFileReference = "none";
        m_tdfdfitWaterReferenceSpectrumFileReference = "none";
        
        m_tdfdfitFitResultFileReference = "none";
        m_tdfdfitParameterResultFileReference = "none";
    }
    
    /**
     * Writes this item to the hard disc
     */
    public void write( RandomAccessFile writer )
    {
        try
        {
            writer.writeUTF( "<MRPatientSpectrumDataBaseItem>" );
        
            writer.writeUTF( m_dataBaseID );
            
            writer.writeUTF( m_patientName );
            
            writer.writeUTF( m_patientBirthDate );
            
            writer.writeUTF( m_examinationNumber );
            
            writer.writeUTF( m_archivDataCarrierNumber );
            
            writer.writeUTF( m_examinationDate );
            
            writer.writeUTF( m_diagnosisDescription );
            
            writer.writeUTF( m_remark );
            
            writer.writeUTF( m_anatomicVoxelLocationDescription );
            
            writer.writeUTF( m_dicomRawFileReference );
            
            writer.writeUTF( m_dicomRawWaterReferenceFileReference );
            
            writer.writeUTF( m_tdfdfitSpectrumToFitFileReference );
            
            writer.writeUTF( m_tdfdfitWaterReferenceSpectrumFileReference );
            
            writer.writeUTF( m_tdfdfitFitResultFileReference );
        
            writer.writeUTF( m_tdfdfitParameterResultFileReference );
            
            writer.writeUTF( "<ImageFileRefences>" );
            
            Iterator it = m_referenceImages.iterator( );
            
            while ( it.hasNext( ) )
            {
                writer.writeUTF( (String)it.next( ) );
            }
            
            writer.writeUTF( "</ImageFileRefences>" );
            writer.writeUTF( "</MRPatientSpectrumDataBaseItem>" );
            
            // Close the file
            writer.close( );
        }
        catch ( IOException e )
        {
            System.out.println( "IOException while writing MRPatientSpectrumDataBaseItem" );
        }
    }
    
    /**
     * Reads the item to the hard disc
     */
    public void read( RandomAccessFile reader )
    {
        try
        {
            m_dataBaseID = reader.readUTF( );
            m_patientName = reader.readUTF( );
                        
            m_patientBirthDate= reader.readUTF( );
            m_examinationNumber= reader.readUTF( );
            m_archivDataCarrierNumber = reader.readUTF( );
            m_examinationDate = reader.readUTF( );
            m_diagnosisDescription = reader.readUTF( );
            m_remark = reader.readUTF( );
            m_anatomicVoxelLocationDescription = reader.readUTF( );
            
            
            m_dicomRawFileReference = reader.readUTF( );
            m_dicomRawWaterReferenceFileReference = reader.readUTF( );
            
            m_tdfdfitSpectrumToFitFileReference = reader.readUTF( );
            m_tdfdfitWaterReferenceSpectrumFileReference = reader.readUTF( );
            
            m_tdfdfitFitResultFileReference = reader.readUTF( );
            m_tdfdfitParameterResultFileReference = reader.readUTF( );
            
            String dummy = reader.readUTF( );
            
            while ( dummy.compareTo( "</ImageFileRefences>" ) != 0 )
            {
                if ( dummy.compareTo( "<ImageFileRefences>" ) != 0 )
                {
                    m_referenceImages.add( dummy );
                }
                dummy = reader.readUTF( );
            }
            
            // Read the closing databaseitem tag
            dummy = reader.readUTF( );
        }
        catch( IOException e )
        {

        }
    }
    
    /**
     * Sets the DICOM raw file reference
     */
    public void setDicomRawFileReference( String fileRef )
    {
        m_dicomRawFileReference = fileRef;
    }

    /**
     * Returns the DICOM raw file reference
     */
    public String getDicomRawFileReference( )
    {
        return m_dicomRawFileReference;
    }

    /**
     * Sets the DICOM raw water reference file reference string
     */
    public void setDicomRawWaterReferenceFileReference( String fileRef )
    {
        m_dicomRawWaterReferenceFileReference = fileRef;
    }

    /**
     * Returns the DICOM raw water reference file reference string
     */
    public String getDicomRawWaterReferenceFileReference( )
    {
        return m_dicomRawWaterReferenceFileReference;
    }
    
    /**
     * Sets the tdfdfit spectrum file reference string
     */
    public void setTdfdfitSpectrumToFitFileReference( String fileRef )
    {
        m_tdfdfitSpectrumToFitFileReference = fileRef;
    }

    /**
     * Returns the tdfdfit spectrum file reference string
     */
    public String getTdfdfitSpectrumToFitFileReference( )
    {
        return m_tdfdfitSpectrumToFitFileReference;
    }
    
    /**
     * Sets the tdfdfit water reference spectrum file reference string
     */
    public void setTdfdfitWaterReferenceSpectrumFileReference( String fileRef )
    {
        m_tdfdfitWaterReferenceSpectrumFileReference = fileRef;
    }

    /**
     * Returns the tdfdfit water reference spectrum file reference string
     */
    public String getTdfdfitWaterReferenceSpectrumFileReference( )
    {
        return m_tdfdfitWaterReferenceSpectrumFileReference;
    }
    
    /**
     * Sets the tdfdfit water reference spectrum file reference string
     */
    public void setTdfdfitFitResultFileReference( String fileRef )
    {
        m_tdfdfitFitResultFileReference = fileRef;
    }

    /**
     * Returns the tdfdfit water reference spectrum file reference string
     */
    public String getTdfdfitFitResultFileReference( )
    {
        return m_tdfdfitFitResultFileReference;
    }
    
    /**
     * Sets the tdfdfit parameter result file reference string
     */
    public void setTdfdfitFitParameterResultFileReference( String fileRef )
    {
        m_tdfdfitParameterResultFileReference = fileRef;
    }

    /**
     * Returns the tdfdfit parameter result file reference string
     */
    public String getTdfdfitFitParameterResultFileReference( )
    {
        return m_tdfdfitParameterResultFileReference;
    }
    
    /**
     * Sets the data base ID (key value)
     */
    public void setDataBaseID( String dBaseID )
    {
        m_dataBaseID = dBaseID;
    }

    /**
     * Returns the data base ID (key value)
     */
    public String getDataBaseID( )
    {
        return m_dataBaseID;
    }

    /**
     * Sets the patient name
     */
    public void setPatientName( String patName )
    {
        m_patientName = patName;
    }

    /**
     * Returns the patient name
     */
    public String getPatientName( )
    {
        return m_patientName;
    }
    
    /**
     * Sets the patient age
     */
    public void setPatientAge( String age )
    {
        m_patientAge = age;
    }

    /**
     * Returns the patient age
     */
    public String getPatientAge( )
    {
        return m_patientAge;
    }
    
    /**
     * Sets the patient ID
     */
    public void setPatientID( String id )
    {
        m_patientID = id;
    }

    /**
     * Returns the patient ID
     */
    public String getPatientID( )
    {
        return m_patientID;
    }
    
    
    
    /**
     * Sets the patients' birthdate 
     */
    public void setPatientDayOfBirth( String birthday )
    {
       m_patientBirthDate = birthday;
    }

    /**
     * Returns the patients' birthdate 
     */
    public String getPatientDayOfBirth( )
    {
        return m_patientBirthDate;
    }
    
    /**
     * Sets the examination number 
     */
    public void setExaminationNumber( String examNumber )
    {
       m_examinationNumber = examNumber;
    }

    /**
     * Returns the examination number 
     */
    public String getExaminationNumber( )
    {
        return m_examinationNumber;
    }

    /**
     * Sets the archive data carrier number 
     */
    public void setArchiveDataCarrierNumber( String archiveNumber )
    {
       m_archivDataCarrierNumber = archiveNumber;
    }
    
    /**
     * Returns the patient weight attribute 
     */
    public String getPatientWeight(  )
    {
       return m_patientWeight;
    }

    /**
     * Sets the patient weight attribute
     */
    public void setPatientWeight( String weight )
    {
       m_patientWeight = weight;
    }

    /**
     * Sets the patient position attribute 
     */
    public void setPatientPosition( String position )
    {
       m_patientPosition = position;
    }
    
    /**
     * Returns the patient position attribute
     */
    public String getPatientPosition(  )
    {
       return m_patientPosition;
    }
    
    
    /**
     * Sets the patient sex attribute 
     */
    public void setPatientSex( String sex )
    {
       m_patientSex = sex;
    }
    
    /**
     * Returns the patient sex attribute
     */
    public String getPatientSex(  )
    {
       return m_patientSex;
    }
    
    /**
     * Returns the archive data carrier number 
     */
    public String getArchiveDataCarrierNumber( )
    {
        return m_archivDataCarrierNumber;
    }

    /**
     * Sets the examination date 
     */
    public void setExaminationDate( String examDate )
    {
       m_examinationDate = examDate;
    }

    /**
     * Returns the examnation date 
     */
    public String getExaminationDate( )
    {
        return m_examinationDate;
    }

    /**
     * Sets the diagnosis description 
     */
    public void setDiagnosisDescription( String diagnosis )
    {
       m_diagnosisDescription = diagnosis;
    }

    /**
     * Returns the diagnosis description 
     */
    public String getDiagnosisDescription( )
    {
        return m_diagnosisDescription;
    }
    
    /**
     * Sets the remark 
     */
    public void setRemark( String remark )
    {
       m_remark = remark;
    }

    /**
     * Returns the remark 
     */
    public String getRemark( )
    {
        return m_remark;
    }
    
    /**
     * Sets the anatomic voxel description 
     */
    public void setAnatomicVoxelLocationDescription( String anatomicVoxelDescription )
    {
       m_anatomicVoxelLocationDescription = anatomicVoxelDescription;
    }

    /**
     * Returns the anatomic voxel description 
     */
    public String getAnatomicVoxelLocationDescription( )
    {
        return m_anatomicVoxelLocationDescription;
    }
    
    /**
     * Adds a filereference to the list of reference files
     */
    public void addReferenceImageFileReference( String imageReference )
    {
        m_referenceImages.add( imageReference );
    }
    
    /**
     * Returns all filereferences of imagereference files 
     */
    public LinkedList getReferenceImageFileReferences( )
    {
        return m_referenceImages;
    }
    
    /**
     * Returns all filereferences of imagereference files 
     */
    public String getReferenceImageFileReference( int referenceIndex )
    {
        return m_referenceImages.get( referenceIndex );
    }
    
    /**
     * Returns the total number of imagereference files 
     */
    public int getReferenceImageFileReference( )
    {
        return m_referenceImages.size( );
    }
    
    // Attributes
    private String m_dataBaseID = "not defined";
    
    // Datafile Related atributes
    private String m_dicomRawFileReference = "not defined";
    private String m_dicomRawWaterReferenceFileReference = "not defined";
    
    private String m_tdfdfitSpectrumToFitFileReference = "not defined";
    private String m_tdfdfitWaterReferenceSpectrumFileReference = "not defined";
    
    private String m_tdfdfitFitResultFileReference = "not defined";
    private String m_tdfdfitParameterResultFileReference = "not defined";
    
    // List of file references to renamed raw Dicom formatfiles
    private LinkedList<String> m_referenceImages; 
    
    // Important other attributes
    private String m_patientName = "not defined";
    private String m_patientBirthDate = "not defined";
    private String m_examinationNumber = "not defined";
    private String m_archivDataCarrierNumber = "not defined";
    
    private String m_examinationDate = "not defined";
    private String m_diagnosisDescription = "not defined";
    private String m_remark = "not defined";
    
    private String m_anatomicVoxelLocationDescription = "not defined";
    
    private String m_patientSex;
    private String m_patientAge;
    private String m_patientID;
    private String m_patientWeight;
    private String m_patientPosition;
    
}
