/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.settings;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.nio.file.Files;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileNameExtensionFilter;
import org.spectrim.main.SpectrImClinicalQMRSBoot;

/**
 * Added variable .spectrIm location in file system.
 * @author nunopbarros /jslotboom
 */
public class GlobalSettings 
{
    private String spectrimRootDir; // = System.getProperty("user.home")+File.separator+".SpectrIm";
    
//  CSIGridPreset
    private String CSIGrdPrstFolder;
    
// R
    private String RExecutable;
    
// Signal Free Regions
    private double signalFreeFDMin;
    private double signalFreeFDMax;
    
    private Properties properties;
    private static  GlobalSettings theInstance = null;
    
    private GlobalSettings( )
    {
        loadProperties( );
    }
    
    public static GlobalSettings getInstance( )
    {
        if ( theInstance == null )
        {
            theInstance = new GlobalSettings( );
        }
        return theInstance;
    }
    
    
    /**
     * Loads all properties needed by spectrIm. If the .spectrIm  data directory 
     * is set, the user is prompted once to set it.
     */
    public void loadProperties( ) 
    {
        String sep = System.getProperty( "file.separator" );
        
        String userDirectory = System.getProperty("user.dir");
        // System.out.println( "userDirectory = " + userDirectory );
        boolean firstTime = false;
        spectrimRootDir = System.getProperty( "user.home" ) + System.getProperty( "file.separator" ) + ".SpectrImClinicalQMRS";
        
        // Open the SpectrImClinicalQMRSBoot file which is located is a
        // subdirectory of the user.home directory.
        // If it does not exist, than it gets created.^
        SpectrImClinicalQMRSBoot spectrImClinicalQMRSBootInstance = SpectrImClinicalQMRSBoot.getInstance( );
        spectrimRootDir = spectrImClinicalQMRSBootInstance.getProperty( SpectrImClinicalQMRSBoot.SPECTRIM_CLINICAL_QMRS_BASE_PATH );
        // Create .spectrIm directory if not existent ..
        if ( ( new File( spectrimRootDir ) ).exists( ) == false )
        {
            
            JFileChooser fileChooser = new JFileChooser( );
            fileChooser.setDialogTitle( "Set location of the .SpectrImClinicalQMRS directory." );
            File currentFolderFeats = new File( System.getProperty( "user.home" ) );
            fileChooser.setCurrentDirectory( currentFolderFeats );
            fileChooser.setFileSelectionMode( JFileChooser.DIRECTORIES_ONLY );
            
            int result = fileChooser.showOpenDialog( null );
            File selected_file;
            
            switch ( result ) 
            {
                case JFileChooser.APPROVE_OPTION:   // Approve Open button was pressed
                                                    spectrimRootDir = fileChooser.getSelectedFile( ).getAbsolutePath( ) + sep + ".SpectrImClinicalQMRS";
                                                    // now set the only property of the boot information.
                                                    spectrImClinicalQMRSBootInstance.setProperty( SpectrImClinicalQMRSBoot.SPECTRIM_CLINICAL_QMRS_BASE_PATH, 
                                                                                                  spectrimRootDir );
                                                    spectrImClinicalQMRSBootInstance.save( );
                                                    firstTime = true;
                                                    break;
                case JFileChooser.CANCEL_OPTION:    // Cancel or close dialog was selected
                                                    break;
                case JFileChooser.ERROR_OPTION:     // the selected process did not
                                                    // terminate properly
                                                    break;
            }
        }
        
        new File(spectrimRootDir+File.separator+"settings").mkdirs();
        new File(spectrimRootDir+File.separator+"CSIGridPrst").mkdirs();
        
//        Properties props = new Properties( );
        properties = new Properties();
        InputStream is = null;

        // First try loading from the current directory
        try 
        {
            File f = new File( spectrimRootDir + File.separator + "settings" + File.separator + "Settings.properties" );
            is = new FileInputStream( f );
        } 
        catch ( Exception e ) 
        {
            is = null;
        }

        try 
        {
            if ( is == null ) 
            {
                // Try loading from classpath
                is = getClass( ).getResourceAsStream( "Settings.properties" );
            }

            // Try loading properties from the file (if found)
            properties.load( is );
        } 
        catch ( Exception e ) 
        {
            
        }
        
        CSIGrdPrstFolder = properties.getProperty("CSIGrdPrstFolder",spectrimRootDir+File.separator+"CSIGridPrst");
        RExecutable = properties.getProperty("RExecutable","");
        
        signalFreeFDMin = Double.valueOf( properties.getProperty( "SignalFreeFDMin", "8.5") );
        signalFreeFDMax = Double.valueOf( properties.getProperty( "SignalFreeFDMax", "10.5") );
        
    }
    
    
    public void saveParamChanges( ) 
    {
        try 
        {
            File f = new File( spectrimRootDir + File.separator + "settings" + File.separator + "Settings.properties" );
            OutputStream out = new FileOutputStream( f );
            properties.store( out, "General SpectrIm properties" );
        } 
        catch ( Exception e ) 
        {
            e.printStackTrace( );
        }
    }
    
    //<editor-fold defaultstate="collapsed" desc="Getters">

    public String getSpectrimRootDir() {
        return spectrimRootDir;
    }
    
    public String getCSIGrdPrstFolder() {
        return CSIGrdPrstFolder;
    }
    
    public String getRExecutable() {
        return RExecutable;
    }
    
    public double getSignalFreeFDMin() {
        return signalFreeFDMin;
    }
    
    public double getSignalFreeFDMax() {
        return signalFreeFDMax;
    }
    
    //</editor-fold>

    //<editor-fold defaultstate="collapsed" desc="Setters">

    public void setCSIGrdPrstFolder(String CSIGrdPrstFolder) {
        properties.setProperty("CSIGrdPrstFolder", CSIGrdPrstFolder);
        this.CSIGrdPrstFolder = CSIGrdPrstFolder;
    }

    public void setRExecutable(String rexe) {
        properties.setProperty("RExecutable", rexe);
        this.RExecutable = rexe;
    }
   
    public void setSignalFreeFDMin(double val) {
        properties.setProperty("SignalFreeFDMin", String.valueOf(val));
        this.signalFreeFDMin = val;
    }

    public void setSignalFreeFDMax(double val) {
        properties.setProperty("SignalFreeFDMax", String.valueOf(val));
        this.signalFreeFDMax = val;
    }
}
