/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.settings.processing;

import org.spectrim.settings.ProcessingSettings;

/**
 * This class incorporates a series actions (e.g. PreprocessingAction, etc.)
 * (Container class to execute a processing pipeline containing several steps actions.)
 * @author nunopbarros
 */
public class ProcessingAction {

    private String name = "";
    private SeriesFilter seriesFilter;
    private PreprocessingAction preprocessingAction;
    private QuantificationAction quantificationAction;
    private ResultsAction resultsAction;

    public ProcessingAction() {
    }

    public ProcessingAction(String name, SeriesFilter seriesFilter, PreprocessingAction preprocessingAction,
            QuantificationAction quantificationAction, ResultsAction resultsAction) {
        this.name = name;
        this.seriesFilter = seriesFilter;
        this.preprocessingAction = preprocessingAction;
        this.quantificationAction = quantificationAction;
        this.resultsAction = resultsAction;
    }

    @Override
    public String toString() {
        String txt = name + ProcessingSettings.separator1;
        if (seriesFilter != null) {
            txt += seriesFilter.getName() + ProcessingSettings.separator1;
        } else {
            txt += "null" + ProcessingSettings.separator1;
        }
        if (preprocessingAction != null) {
            txt += preprocessingAction.getName() + ProcessingSettings.separator1;
        } else {
            txt += "null" + ProcessingSettings.separator1;
        }
        if (quantificationAction != null) {
            txt += quantificationAction.getName() + ProcessingSettings.separator1;
        } else {
            txt += "null" + ProcessingSettings.separator1;
        }
        if (resultsAction != null) {
            txt += resultsAction.getName();
        } else {
            txt += "null";
        }
        return txt;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public SeriesFilter getSeriesFilter() {
        return seriesFilter;
    }

    public void setSeriesFilter(SeriesFilter seriesFilter) {
        this.seriesFilter = seriesFilter;
    }

    public PreprocessingAction getPreprocessingAction() {
        return preprocessingAction;
    }

    public void setPreprocessingAction(PreprocessingAction preprocessingAction) {
        this.preprocessingAction = preprocessingAction;
    }

    /**
     * The quantification action refers to the type of quantification
     * the user wants to perform. 
     * @return 
     */
    public QuantificationAction getQuantificationAction() 
    {
        return quantificationAction;
    }

    public void setQuantificationAction(QuantificationAction quantificationAction) {
        this.quantificationAction = quantificationAction;
    }

    public ResultsAction getResultsAction() {
        return resultsAction;
    }

    public void setResultsAction(ResultsAction resultsAction) {
        this.resultsAction = resultsAction;
    }
}
