/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.spectrim.signalprocessing;


import org.spectrim.imagegraphicobjects.RegioPolygon;

import org.spectrim.xyplot.Curve;


/**
 * Class that computes a histogram of a signal or part of a 1D or 2D
 * signal (rectangular areas only!)
 *
 * For non rectangular areas please use the RegioHistogramCalculator
 *
 * @author  hans
 */


/**
 *
 * @author  hans
 */
public class IntegerSignalHistogramCalculator
{

    /**
     * Creates a new instance of SignalHistogramCalculator for a 
     * one dimensional signal.
     * @param signal
     * @param startIndex
     * @param stopIndex
     * @param histogramRangeDeterminationMethod
     */
    public IntegerSignalHistogramCalculator( int[] signal, int startIndex, 
                                             int stopIndex, 
                                             int histogramRangeDeterminationMethod )
    {
        m_signal = new int[ signal.length ];
        System.arraycopy( signal, 0, m_signal, 0, signal.length );

        m_startIndex = startIndex;
        m_stopIndex = stopIndex;

        m_numberOfResponses = 1;
        m_signalDimensionality = ONE_DIMENSIONAL_SIGNAL;

        m_histogramRangeDeterminationMethod = histogramRangeDeterminationMethod;
    }


    /**
     * Creates a new instance of SignalHistogramCalculator for a 
     * two dimensional signal.
     * @param signal
     * @param numberOfResponses
     * @param numberOfPoints
     * @param startIndex
     * @param stopIndex
     */
    public IntegerSignalHistogramCalculator( int[][] signal, int numberOfResponses, 
                                             int numberOfPoints, int startIndex, 
                                             int stopIndex )
    {
        m_signal = new int[ numberOfResponses * ( stopIndex - startIndex ) ];

        int cntr = 0;

        // create a huge 1D signel from the 2D signal
        for ( int i = 0 ; i < numberOfResponses; i++ )
        {
            for ( int j = startIndex ; j < stopIndex; j++ )
            {
                m_signal[cntr] = signal[i][j];
                cntr++;
            }
        }

        // for 2d signal the signal is recollected before computation therefore
        // the histogram has to be calculated for the complete signal

        m_startIndex = 0;
        m_stopIndex = m_signal.length;
        m_numberOfResponses = numberOfResponses;

        m_signalDimensionality = TWO_DIMENSIONAL_SIGNAL;
        m_histogramRangeDeterminationMethod = HISTOGRAM_RANGE_DETERMINED_BY_TOTAL_SIGNAL;
    }


    /**
     *  Calculates the average pixel value and standard deviation of the
     *  currently active polygon (which encloses a ROI).
     *  @param Polygon roi  the currently selected polygon
     */
    private void calculateStatistics1D( )
    {
        m_intervalSize = m_stopIndex - m_startIndex;

        // Calculate the average of the ROI
        for ( int i = m_startIndex; i < m_stopIndex; i++ )
        {
            m_average += m_signal[i];
        }

        m_average = m_average / m_intervalSize;

        // Calculate the standard deviation of the current ROI
        float diff = 0.0f;
        // reset the roi size

        for ( int i = m_startIndex; i < m_stopIndex; i++ )
        {
            diff = m_average - m_signal[ i ];
            m_stdev += diff * diff;
        }

        m_stdev = (float)Math.sqrt( (double)m_stdev / (float)m_intervalSize) ;

        System.out.println( "Interval size (points) = " + m_intervalSize );
        System.out.println( "Average                = " + m_average );
        System.out.println( "Standard deviation     = " + m_stdev );


        // If HISTOGRAM_RANGE_DETERMINED_BY_TOTAL_IMAGE
        if ( m_histogramRangeDeterminationMethod == this.HISTOGRAM_RANGE_DETERMINED_BY_TOTAL_SIGNAL )
        {
            m_minSignalValue = SignalTools.getMinimum( m_signal );
            m_maxSignalValue = SignalTools.getMaximum( m_signal );
        }
        else
        {
            m_minSignalValue = SignalTools.getMinimum( SignalTools.getArrayFragment( m_signal, m_startIndex, m_stopIndex ) );
            m_maxSignalValue = SignalTools.getMaximum( SignalTools.getArrayFragment( m_signal, m_startIndex, m_stopIndex ) );
        }
    }


    /**
     *  Calculates the average pixel value and standard deviation of the
     *  currently active polygon (which encloses a ROI).
     *  @param Polygon roi  the currently selected polygon
     */
    private void calculateStatistics2D( )
    {

        // Calculate the average of the ROI
        for ( int i = 0; i < m_signal.length; i++ )
        {
            m_average += m_signal[i];
        }

        m_average = m_average / m_signal.length;

        // Calculate the standard deviation of the current ROI
        float diff = 0.0f;

        // reset the roi size
        for ( int i = 0; i < m_signal.length; i++ )
        {
            diff = m_average - m_signal[ i ];
            m_stdev += diff * diff;
        }

        m_stdev = (float)Math.sqrt( (double)m_stdev / (float)m_intervalSize) ;

        System.out.println( "Interval size (points) = " + m_intervalSize );
        System.out.println( "Average                = " + m_average );
        System.out.println( "Standard deviation     = " + m_stdev );


        m_minSignalValue = SignalTools.getMinimum( m_signal );
        m_maxSignalValue = SignalTools.getMaximum( m_signal );
    }



    /**
     * Calculates the histogram of the intensity distribution of the pixels
     * enclosed.
     * @return 
     */
    public Curve calculateHistogram( )
    {
        if ( m_signalDimensionality == ONE_DIMENSIONAL_SIGNAL )
        {
            calculateStatistics1D( );
        }

        if ( m_signalDimensionality == TWO_DIMENSIONAL_SIGNAL )
        {
            calculateStatistics2D( );
        }

        m_rangeWidth = ( m_maxSignalValue - m_minSignalValue );

        if ( m_rangeWidth != 0 )
        {
            m_numberOfClasses = ( m_intervalSize / 10 );

            if ( m_numberOfClasses < 10 )
            {
                m_numberOfClasses = 10;
            }

            if ( m_numberOfClasses > 256 )
            {
                m_numberOfClasses = 256;
            }

            if ( this.m_histogramRangeDeterminationMethod == this.HISTOGRAM_RANGE_DETERMINED_BY_TOTAL_SIGNAL )
            {
                m_numberOfClasses = 256;
            }
            else
            {
                m_numberOfClasses = ( m_intervalSize / 10 );

                if ( m_numberOfClasses < 10 )
                {
                    m_numberOfClasses = 10;
                }

                if ( m_numberOfClasses > 256 )
                {
                    m_numberOfClasses = 256;
                }
            }

            m_classWidth = m_rangeWidth / (float)m_numberOfClasses;

            m_occurences = new int[ m_numberOfClasses + 1];

            for ( int i = m_startIndex; i < m_stopIndex; i++ )
            {

                 float signalValue = m_signal[ i ];
                 int xVal = (int)( (signalValue - m_minSignalValue)  /  m_classWidth );
                 m_occurences[ xVal ]++;
            }

            float[] xVals = new float[ 2 * m_occurences.length ];
            float[] yVals = new float[ 2 * m_occurences.length ];

            for ( int i = 0; i < m_occurences.length; i++ )
            {
                xVals[2*i] = m_minSignalValue + (float)i * m_classWidth;
                yVals[2*i] = m_occurences[i];
                xVals[2*i + 1] = m_minSignalValue + (float)(i+1) * m_classWidth;
                yVals[2*i + 1] = m_occurences[i];
            }

            m_histogram = new Curve( xVals, yVals );
        }

        return m_histogram;
    }


    /**
     * Calculates the histogram of the intensity distribution of the pixels
     * enclosed.
     * @param minSignalValue
     * @param maxSignalValue
     * @param numberOfClasses
     * @return 
     */
    public Curve calculateHistogram( int minSignalValue, int maxSignalValue, int numberOfClasses )
    {

        m_maxSignalValue = maxSignalValue;
        m_minSignalValue = minSignalValue;
        m_numberOfClasses = numberOfClasses;

        m_rangeWidth = ( m_maxSignalValue - m_minSignalValue );

        m_classWidth = m_rangeWidth / (float)m_numberOfClasses;


        m_occurences = new int[ m_numberOfClasses + 1];

        for ( int i = m_startIndex; i < m_stopIndex; i++ )
        {

            float signalValue = m_signal[ i ];
            int xVal = (int)( (signalValue - m_minSignalValue)  /  m_classWidth );
            m_occurences[ xVal ]++;
        }

        float[] xVals = new float[ 2 * m_occurences.length ];
        float[] yVals = new float[ 2 * m_occurences.length ];

        for ( int i = 0; i < m_occurences.length; i++ )
        {
            xVals[2*i] = m_minSignalValue + (float)i * m_classWidth;
            yVals[2*i] = m_occurences[i];
            xVals[2*i + 1] = m_minSignalValue + (float)(i+1) * m_classWidth;
            yVals[2*i + 1] = m_occurences[i];
        }

        m_histogram = new Curve( xVals, yVals );

        return m_histogram;
    }

    /**
     * Returns the average of ROI values.
     * @return 
     */
    public float getAverage( )
    {
       return m_average;
    }

    /**
     * Returns the average of ROI values.
     * @return 
     */
    public float getStandardDeviation( )
    {
       return m_stdev;
    }

    // Private attributes
    private int[] m_signal;
    private int[][] m_signal2D;

    private int m_startIndex;
    private int m_stopIndex;

    private float m_zoom;
    private RegioPolygon m_regPol;

    private float m_rangeWidth;
    private float m_classWidth;

    private int m_numberOfClasses;

    private int m_numberOfResponses = 0;
    private int m_numberOfPoints = 0;


    private int m_minSignalValue = Integer.MAX_VALUE;
    private int m_maxSignalValue = Integer.MIN_VALUE;

    private float m_average = 0f;
    private float m_stdev = 0f;
    private int m_intervalSize = 0;
    private int[] m_occurences;

    private Curve m_histogram;

    private int m_histogramRangeDeterminationMethod = HISTOGRAM_RANGE_DETERMINED_BY_RANGE;

    public final static int HISTOGRAM_RANGE_DETERMINED_BY_RANGE = 0;
    public final static int HISTOGRAM_RANGE_DETERMINED_BY_TOTAL_SIGNAL = 1;

    private int m_signalDimensionality = ONE_DIMENSIONAL_SIGNAL;
    public final static int ONE_DIMENSIONAL_SIGNAL = 2;
    public final static int TWO_DIMENSIONAL_SIGNAL = 3;

}
