/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.signalprocessing;

import org.spectrim.imagegraphicobjects.RegioPolygon;
import org.spectrim.xyplot.Curve;
import java.awt.Color;

/**
 * Class that computes a histogram of a 1D rank order input signal 
 *
 * @author  hans
 */


public class RankSpectrumHistogramCalculator
{

    /**
     * Creates a new instance of SignalHistogramCalculator for a one dimensional signal
     * @param signal
     * @param startIndex
     * @param stopIndex
     * @param histogramRangeDeterminationMethod
     */
    public RankSpectrumHistogramCalculator( int[] signal, int startIndex, int stopIndex, int histogramRangeDeterminationMethod )
    {
        m_signal = new int[ signal.length ];
        System.arraycopy( signal, 0, m_signal, 0, signal.length );

        m_startIndex = startIndex;
        m_stopIndex = stopIndex;

        m_minimumRank = SignalTools.getMinimum( signal );
        m_maximumRank = SignalTools.getMaximum( signal );
        
        m_numberOfResponses = 1;
        m_signalDimensionality = ONE_DIMENSIONAL_SIGNAL;

        m_histogramRangeDeterminationMethod = histogramRangeDeterminationMethod;
    }


    /**
     *  Calculates the average pixel value and standard deviation of the
     *  currently active polygon (which encloses a ROI).
     *  @param Polygon roi  the currently selected polygon
     */
    private void calculateRankOrderStatistics( )
    {
        m_intervalSize = m_stopIndex - m_startIndex;

        // Calculate the average of the ROI
        for ( int i = m_startIndex; i < m_stopIndex; i++ )
        {
            m_average += m_signal[i];
        }

        m_average = m_average / m_intervalSize;

        // Calculate the standard deviation of the current ROI
        float diff = 0.0f;
        // reset the roi size

        for ( int i = m_startIndex; i < m_stopIndex; i++ )
        {
            diff = m_average - m_signal[ i ];
            m_stdev += diff * diff;
        }

        m_stdev = (float)Math.sqrt( (double)m_stdev / (float)m_intervalSize) ;

        System.out.println( "Interval size (points) = " + m_intervalSize );
        System.out.println( "Average                = " + m_average );
        System.out.println( "Standard deviation     = " + m_stdev );


        // If HISTOGRAM_RANGE_DETERMINED_BY_TOTAL_IMAGE
        if ( m_histogramRangeDeterminationMethod == RankSpectrumHistogramCalculator.HISTOGRAM_RANGE_DETERMINED_BY_TOTAL_SIGNAL )
        {
            m_minSignalValue = SignalTools.getMinimum( m_signal );
            m_maxSignalValue = SignalTools.getMaximum( m_signal );
        }
        else
        {
            m_minSignalValue = SignalTools.getMinimum( SignalTools.getArrayFragment( m_signal, m_startIndex, m_stopIndex ) );
            m_maxSignalValue = SignalTools.getMaximum( SignalTools.getArrayFragment( m_signal, m_startIndex, m_stopIndex ) );
        }
    }


    /**
     * Calculates the histogram of the intensity distribution of the pixels
     * enclosed.
     * @return 
     */
    public Curve calculateRankOrderHistogram( )
    {

        // standard number of bin is 1024
        m_rangeWidth = ( m_stopIndex - m_startIndex )/1024f;

        int[] xVals = new int[2048];
        int[] yVals = new int[2048];        
        
        for ( int i=1; i<1023; i++)
        {
            int range_min = i * (int)m_rangeWidth;
            int range_max = ( i + 1 ) * (int)m_rangeWidth;
            int cntr=0;
            for ( int j = 0; j < m_signal.length; j++ )
            {
                if ( ( m_signal[j] > range_min ) && ( m_signal[j] < range_max ) )
                {
                    cntr++;
                }                
            }
            
            xVals[2*i] = range_min;
            yVals[2*i] = cntr;
            xVals[2*i + 1] = range_max;
            yVals[2*i + 1] = cntr;
        }
        
        m_histogram = new Curve( xVals, yVals, Color.BLACK );
        
        return m_histogram;
    }

    // Private attributes
    private final int[] m_signal;
    private int[][] m_signal2D;

    private final int m_startIndex;
    private final int m_stopIndex;

    private float m_zoom;
    private RegioPolygon m_regPol;

    private float m_rangeWidth;
    private float m_classWidth;

    private int m_numberOfClasses;

    private int m_numberOfResponses = 0;
    private final int m_numberOfPoints = 0;


    private int m_minSignalValue = Integer.MAX_VALUE;
    private int m_maxSignalValue = Integer.MIN_VALUE;

    private float m_average = 0f;
    private float m_stdev = 0f;
    private int m_intervalSize = 0;
    private int[] m_occurences;

    private Curve m_histogram;

    private int m_histogramRangeDeterminationMethod = HISTOGRAM_RANGE_DETERMINED_BY_RANGE;

    public final static int HISTOGRAM_RANGE_DETERMINED_BY_RANGE = 0;
    public final static int HISTOGRAM_RANGE_DETERMINED_BY_TOTAL_SIGNAL = 1;

    private int m_signalDimensionality = ONE_DIMENSIONAL_SIGNAL;
    public final static int ONE_DIMENSIONAL_SIGNAL = 2;
    public final static int TWO_DIMENSIONAL_SIGNAL = 3;

    private int m_minimumRank = 0; // contains the lowest rank of the signal
    private int m_maximumRank = 0; // contains the highest rank of the signal
    
}
