/*
 * TwoExponentialsModelRandomMininizer.java
 *
 * Created on April 18, 2006, 10:50 PM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package org.spectrim.signalprocessing;

import org.spectrim.lib.DoubleRange;
import org.spectrim.xyplot.Curve;
import org.spectrim.xyplot.XYPlotFrame;
import java.awt.Color;

/**
 *
 * @author hans
 */
public class TwoExponentialsModelRandomMininizer extends RandomMinimizer 
{
    
    /** Creates a new instance of TwoExponentialsModelRandomMininizer */
    public TwoExponentialsModelRandomMininizer( ) 
    {
        
    }
    
    /**
     * Function to minimize should be defined in this non abstract class
     */
    public double fnc( double[] par )
    {
        if ( par.length == 4 )
        {
            double fncVal = 0;
            double tmp = 0;
        
            for ( int i = 0 ; i < m_xValues.length ; i++ )
            {
                tmp = par[0] * Math.exp( -m_xValues[i] * par[1] ) + par[2]*Math.exp( -m_xValues[i] * par[3] );
                fncVal += ( m_yValues[i] - tmp ) * ( m_yValues[i] - tmp ); 
            }
            
            return fncVal;
        }
        else
        {
            System.out.println( "Illegal dimension of parameter vector : size should be 4" );
            return 0;
        }
    }
    
    /**
     * Computes signal for parameter vector par
     */
    public double[] getSignal( double[] par )
    {
        double[] signal = new double[m_xValues.length];
        
        for ( int i = 0; i < m_xValues.length ; i++ )
        {
            signal[i] = par[0] * Math.exp( -m_xValues[i] * par[1] ) + par[2] * Math.exp( -m_xValues[i] * par[3] );
        }
        
        return signal;
    }
    
    
    /**
     * Sets the x-values for which function fnc should be computed
     */
    public void setXValues( double[] xValues )
    {
        m_xValues = xValues.clone( );
    }

    /**
     * Sets the measured-values for which the model function paramters $
     * should be computed
     */
    public void setYValues( double[] yValues )
    {
        m_yValues = yValues.clone( );
    }
    
        
    /**
     * Some test code to test the functionality of this class
     */ 
    public static void main( String args[] )
    {      

        int npar = 4;
        
        int N = 50;
        int niter_max = 50;
        
        double[] times = new double[ 8 ];
        
        times[0] = 30e-3;
        times[1] = 40e-3;
        times[2] = 50e-3;
        times[3] = 80e-3;
        times[4] = 100e-3;
        times[5] = 200e-3;
        times[6] = 500e-3;
        times[7] = 1500e-3;
        
        
        double[] par = new double[4];
        
        par[0] = 0.7;
        par[1] = 33.33;
        par[2] = 0.1;
        par[3] = 2.0;
        
        
        double[] startPar = new double[4];
        
        startPar[0] = 0.45;
        startPar[1] = 20;
        startPar[2] = 0.2;
        startPar[3] = 3;
        
        DoubleRange[] parameterRanges = new DoubleRange[4];
        
        parameterRanges[0] = new DoubleRange( 0.3, 1.0 );
        parameterRanges[1] = new DoubleRange( 1/10E-3, 1/200E-3 );
        parameterRanges[2] = new DoubleRange( 0.0, 0.5 );
        parameterRanges[3] = new DoubleRange( 1/200E-3, 1/800E-3 );
        
        TwoExponentialsModelRandomMininizer rndMin = new TwoExponentialsModelRandomMininizer( );
        
        rndMin.setXValues( times );
        
        double[] measuredSignal = new double[8];
        
        measuredSignal = rndMin.getSignal( par );
        
        XYPlotFrame signalPlot = new XYPlotFrame( "2 Exponential Signal values", null );
        signalPlot.addCurve( new Curve( times, measuredSignal ), true );
        
        rndMin.setYValues( measuredSignal );
        
        rndMin.setStartParameters( startPar );
        rndMin.setParameterRanges( parameterRanges );
        
        rndMin.performRandomMinimization( N, niter_max, true );
        
        double[] minPar = new double[4];
        
        minPar = rndMin.getMinimizingParameters( );
        
        double[] startSig = new double[8];
        startSig = rndMin.getSignal( startPar );
        
        double[] outSig = new double[8];
        
        outSig = rndMin.getSignal( minPar );
        signalPlot.addCurve( new Curve( times, outSig, Color.RED ), true );
        
        signalPlot.addCurve( new Curve( times, startSig, Color.BLUE ), true );
    }
    
    // Attributes
    double[] m_xValues;
    double[] m_yValues;    
}
