/*
 * File: $Id: BaseLineRangesDefinitionFrame.java,v 1.1.1.1 2006/04/19 09:38:51 hans Exp $
 * History
 * Date: 22.03.2006, 11:26  Authors: Hans Slotboom
 *
 * Changes:
 *
 * Copyright (C) 2006 -    by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */


/*
 * BaseLineRangesDefinitionFrame.java
 *
 * Created on March 16, 2006, 12:09 PM
 */

package org.spectrim.tdfdfit;

 
import java.awt.Color;
import java.awt.event.WindowListener;
import java.awt.event.WindowEvent;

import java.awt.geom.Point2D;


import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;

import java.util.LinkedList;
import java.util.Iterator;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JFileChooser;
import javax.swing.ListSelectionModel;
import javax.swing.filechooser.FileNameExtensionFilter;
import javax.swing.table.DefaultTableModel;
import org.apache.commons.math3.analysis.interpolation.SplineInterpolator;
import org.apache.commons.math3.analysis.polynomials.PolynomialSplineFunction;
import org.spectrim.xyplot.Curve;

/**
 * This GUI allows baseline definition.
 * @author  hans
 */
public class BaseLineSplineSupportPointsDefinitionFrame extends javax.swing.JFrame implements WindowListener, ActionListener
{
    
    /** 
     * Creates new form BaseLineRangesDefinitionFrame. 
     * @param specData
     */
    public BaseLineSplineSupportPointsDefinitionFrame( SingleVoxelMeasuredSpectralData specData ) 
    {
        initComponents( );
        
        m_nPoints = specData.getAbsorptionChannelFD( ).length;
        
        m_differenceReal = new float[m_nPoints];
        m_differenceImag = new float[m_nPoints];
       
        m_splineNodeModelPath = TDFDFittingDefaults.getInstance( ).getProperty( TDFDFittingDefaults.BASELINE_SPLINE_MODEL_PATH );
        m_splineNodeModelName = TDFDFittingDefaults.getInstance( ).getProperty( TDFDFittingDefaults.BASELINE_SPLINE_MODEL_NAME );
        m_splineNodeModelMode = TDFDFittingDefaults.getInstance( ).getProperty( TDFDFittingDefaults.BASELINE_SPLINE_REMOVAL_MODE );
        
        if ( m_splineNodeModelMode == null )
        {
            m_splineNodeModelMode = TDFDFittingDefaults.MAGNITUDE;
            
            TDFDFittingDefaults.getInstance( ).setProperty( TDFDFittingDefaults.BASELINE_SPLINE_REMOVAL_MODE, TDFDFittingDefaults.MAGNITUDE );
            TDFDFittingDefaults.getInstance( ).save( );
        }
        
        // Set the previous Baseline channel mode to be the default mode! 
        TDFDFittingDefaults.getInstance( ).setProperty( TDFDFittingDefaults.DISPLAY_CHANNEL, m_splineNodeModelMode );
        
        if ( m_splineNodeModelMode.compareTo( TDFDFittingDefaults.ABSORPTION )  == 0 )
        {
            m_absorptionModeBLRmovaljRadioButton.setSelected( true );
            TDFDFittingDefaults.getInstance( ).setProperty( TDFDFittingDefaults.DISPLAY_CHANNEL, TDFDFittingDefaults.ABSORPTION );
            TDFDFittingDefaults.getInstance( ).save( );
        }
        else
        {
            this.m_absolutModeBLRemovalRadioButton.setSelected( true );
            TDFDFittingDefaults.getInstance( ).setProperty( TDFDFittingDefaults.DISPLAY_CHANNEL, TDFDFittingDefaults.MAGNITUDE );
            TDFDFittingDefaults.getInstance( ).save( );
        }
        
        m_currentBaselineModel = m_splineNodeModelName;

        if( m_splineNodeModelPath != null )
        {   
            File f = new File( m_splineNodeModelPath );
        
            // Populates the array with names of files and directories
            String[] pathnames = f.list( );

            // For each pathname in the pathnames array
            for ( String pathname : pathnames ) 
            {
                // Print the names of files and directories
                // System.out.println( pathname ); 
                if ( pathname.contains( ".snm" ) == true )
                {
                    m_availableSplineNodeModelsjComboBox.addItem( pathname );
                }
            }

            if ( m_splineNodeModelName != null )
            {
                m_availableSplineNodeModelsjComboBox.setSelectedItem( m_splineNodeModelName );
            }

            m_splineNodePointsTable.setColumnSelectionAllowed( false );
            m_splineNodePointsTable.setRowSelectionAllowed( true );
            m_splineNodePointsTable.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );

            DefaultTableModel dm = (DefaultTableModel)m_splineNodePointsTable.getModel( );

            int rowCount = dm.getRowCount( );

            // Remove rows one by one from the end of the table
            for ( int i = rowCount - 1; i >= 0; i-- ) 
            {
                dm.removeRow( i );
            }

            m_userDefinedSplineNodes.clear( );    
            m_specData = specData;

            m_MRSpectrumBaselineGUIPropertiesDefiner.setModelSpectrum( specData, true );
            m_MRSpectrumBaselineGUIPropertiesDefiner.addActionListener( this );

            setVisible( true );
        }
        // If there is a model loaded than there should be a OrderedBaseLinePointsDataModel object available
        OrderedBaseLinePointsDataModel oBDPM = OrderedBaseLinePointsDataModel.getInstance( );
        oBDPM.deleteAllPoints( );

        // load the last model
        if ( m_splineNodeModelName != null )
        {
            
            try 
            {
                if ( !m_splineNodeModelName.endsWith( ".snm" ) )
                {
                    m_splineNodeModelName = m_splineNodeModelName + ".snm";
                }
                
                String lastSavedModel = m_splineNodeModelPath + System.getProperty( "file.separator" ) + m_splineNodeModelName;
                
                File tempFile = new File( lastSavedModel );
                boolean exists = tempFile.exists( );
                
                if ( exists )
                {
                    RandomAccessFile in = new RandomAccessFile( lastSavedModel, "r" );
                
                    oBDPM.read( in );
                
                    try 
                    {
                        in.close( );
                    } 
                    catch ( IOException ex ) 
                    {
                        Logger.getLogger( BaseLineSplineSupportPointsDefinitionFrame.class.getName( ) ).log( Level.SEVERE, null, ex );
                    }
                
                    for ( int i = 0; i < oBDPM.getSize( ) ; i++ )
                    {
                        float yValue = getCorrespondingYValue( (float)( oBDPM.getPoint( i ).getX( ) ) );
                        m_userDefinedSplineNodes.add( new Point2D.Double( oBDPM.getPoint( i ).getX( ), yValue ) );
                    }
                
                    m_MRSpectrumBaselineGUIPropertiesDefiner.setOrderedSplineNodeList( m_userDefinedSplineNodes );
                
                    for ( int i = 0 ; i < m_userDefinedSplineNodes.size( ); i++ )
                    {
                        DefaultTableModel model = (DefaultTableModel) m_splineNodePointsTable.getModel( );
                        model.addRow( new Object[]{ i, m_userDefinedSplineNodes.get( i ).x, m_userDefinedSplineNodes.get( i ).y } );
                    }
                
                    if ( m_splineNodePointsTable.getModel( ).getRowCount( ) > 0 )
                    {
                        m_splineNodePointsTable.setRowSelectionInterval( 0, 0 );
                    }
                
                    drawSpline( );
                
                    m_availableSplineNodeModelsjComboBox.addItemListener( new ItemListener( ) 
                    {
                        @Override
                        public void itemStateChanged( ItemEvent arg0 ) 
                        {
                            updateGUIAfterSplineModelSelection( );
                        }
                    } );
                }
            }
            catch ( FileNotFoundException ex ) 
            {
                    Logger.getLogger( BaseLineSplineSupportPointsDefinitionFrame.class.getName( ) ).log( Level.SEVERE, null, ex );
            }
        }
    }
    
    /**
     * Returns the y-value of the magnitude spectrum when given the offset 
     * x value.
     * @param xValue
     * @return 
     */
    public float getCorrespondingYValue( float xValue )
    {
//        float[] xaxis = m_specData.getAbsorptionCurveFDPPM( ).getXvalues( );
//        int index = getIndexOfFirstMatchingValue( xValue, xaxis );
//        return ( m_specData.getAbsorptionCurveFDPPM( ).getYvalues( )[index] );
        
        int index;
        float average;
        float[] xaxis;
        if (  m_splineNodeModelMode.compareTo(TDFDFittingDefaults.MAGNITUDE ) == 0 )
        {
            xaxis = m_specData.getMagnitudeCurveFDPPM( ).getXvalues( );
            index = getIndexOfFirstMatchingValue( xValue, xaxis );
        
            // do not return the value of the index itsef, but average of 3 points
            average = m_specData.getMagnitudeCurveFDPPM( ).getYvalues( )[index];
        }
        else
        {
            xaxis = m_specData.getAbsorptionCurveFDPPM( ).getXvalues( );
            index = getIndexOfFirstMatchingValue( xValue, xaxis );
        
            // do not return the value of the index itsef, but average of 3 points
            average = m_specData.getAbsorptionCurveFDPPM( ).getYvalues( )[index];
        }
        
        if (  m_splineNodeModelMode.compareTo(TDFDFittingDefaults.MAGNITUDE ) == 0 )
        {
            if ( ( index > 1 ) && ( index < xaxis.length - 2 ) )
            {
                average +=m_specData.getMagnitudeCurveFDPPM( ).getYvalues( )[index - 1];
                average +=m_specData.getMagnitudeCurveFDPPM( ).getYvalues( )[index + 1];
                average +=m_specData.getMagnitudeCurveFDPPM( ).getYvalues( )[index - 2];
                average +=m_specData.getMagnitudeCurveFDPPM( ).getYvalues( )[index + 2];
            }

            if ( index == 0 )
            {
                average +=m_specData.getMagnitudeCurveFDPPM( ).getYvalues( )[1];
                average +=m_specData.getMagnitudeCurveFDPPM( ).getYvalues( )[2];
                average +=m_specData.getMagnitudeCurveFDPPM( ).getYvalues( )[3];
                average +=m_specData.getMagnitudeCurveFDPPM( ).getYvalues( )[4];
            }            
            if ( index == xaxis.length - 1 )
            {
                average +=m_specData.getMagnitudeCurveFDPPM( ).getYvalues( )[xaxis.length - 2];
                average +=m_specData.getMagnitudeCurveFDPPM( ).getYvalues( )[xaxis.length - 3];
                average +=m_specData.getMagnitudeCurveFDPPM( ).getYvalues( )[xaxis.length - 4];
                average +=m_specData.getMagnitudeCurveFDPPM( ).getYvalues( )[xaxis.length - 5];
            }

            if ( index == 1 )
            {
                average +=m_specData.getMagnitudeCurveFDPPM( ).getYvalues( )[0];
                average +=m_specData.getMagnitudeCurveFDPPM( ).getYvalues( )[2];
                average +=m_specData.getMagnitudeCurveFDPPM( ).getYvalues( )[3];
                average +=m_specData.getMagnitudeCurveFDPPM( ).getYvalues( )[4];
            }            
            if ( index == xaxis.length - 2 )
            {
                average +=m_specData.getMagnitudeCurveFDPPM( ).getYvalues( )[xaxis.length - 1];
                average +=m_specData.getMagnitudeCurveFDPPM( ).getYvalues( )[xaxis.length - 3];
                average +=m_specData.getMagnitudeCurveFDPPM( ).getYvalues( )[xaxis.length - 4];
                average +=m_specData.getMagnitudeCurveFDPPM( ).getYvalues( )[xaxis.length - 5];
            }

            return ( average / 5 );
        }
        else
        {
             if ( ( index > 1 ) && ( index < xaxis.length - 2 ) )
            {
                average +=m_specData.getAbsorptionCurveFDPPM( ).getYvalues( )[index - 1];
                average +=m_specData.getAbsorptionCurveFDPPM( ).getYvalues( )[index + 1];
                average +=m_specData.getAbsorptionCurveFDPPM( ).getYvalues( )[index - 2];
                average +=m_specData.getAbsorptionCurveFDPPM( ).getYvalues( )[index + 2];
            }

            if ( index == 0 )
            {
                average +=m_specData.getAbsorptionCurveFDPPM( ).getYvalues( )[1];
                average +=m_specData.getAbsorptionCurveFDPPM( ).getYvalues( )[2];
                average +=m_specData.getAbsorptionCurveFDPPM( ).getYvalues( )[3];
                average +=m_specData.getAbsorptionCurveFDPPM( ).getYvalues( )[4];
            }            
            if ( index == xaxis.length - 1 )
            {
                average +=m_specData.getAbsorptionCurveFDPPM( ).getYvalues( )[xaxis.length - 2];
                average +=m_specData.getAbsorptionCurveFDPPM( ).getYvalues( )[xaxis.length - 3];
                average +=m_specData.getAbsorptionCurveFDPPM( ).getYvalues( )[xaxis.length - 4];
                average +=m_specData.getAbsorptionCurveFDPPM( ).getYvalues( )[xaxis.length - 5];
            }

            if ( index == 1 )
            {
                average +=m_specData.getAbsorptionCurveFDPPM( ).getYvalues( )[0];
                average +=m_specData.getAbsorptionCurveFDPPM( ).getYvalues( )[2];
                average +=m_specData.getAbsorptionCurveFDPPM( ).getYvalues( )[3];
                average +=m_specData.getAbsorptionCurveFDPPM( ).getYvalues( )[4];
            }            
            if ( index == xaxis.length - 2 )
            {
                average +=m_specData.getAbsorptionCurveFDPPM( ).getYvalues( )[xaxis.length - 1];
                average +=m_specData.getAbsorptionCurveFDPPM( ).getYvalues( )[xaxis.length - 3];
                average +=m_specData.getAbsorptionCurveFDPPM( ).getYvalues( )[xaxis.length - 4];
                average +=m_specData.getAbsorptionCurveFDPPM( ).getYvalues( )[xaxis.length - 5];
            }
             return ( average / 5 );
        }
        
    }
    
    /**
     * Returns the array index of the coordinate containing the maximum value.
     * @param value
     * @param y
     * @return 
     */
    public int getIndexOfFirstMatchingValue( float value, float[] y ) 
    {
	int index = 0;
	
        float current_difference = Float.MAX_VALUE;
        
	for ( int k = 0; k < y.length; k++ ) 
        {
            
            // System.out.println( "Difference = " + Math.abs( y[k] - value ) );
            
            float difference = Math.abs( y[k] - value );
            
	    if ( difference < current_difference ) 
            {
		index = k;
                current_difference = difference;
	    }
	}
        
	return index;
    }
    
    /** 
     * Creates new form BaseLineRangesDefinitionFrame. 
     */
    public BaseLineSplineSupportPointsDefinitionFrame(  ) 
    {
        
        initComponents( );
        
        m_splineNodePointsTable.setColumnSelectionAllowed( false );
        m_splineNodePointsTable.setRowSelectionAllowed( true );
        m_splineNodePointsTable.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
        
        m_userDefinedSplineNodes.clear( );
       
    }
    
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        m_radioButtonGroup = new javax.swing.ButtonGroup();
        jPanel1 = new javax.swing.JPanel();
        m_cancelButton = new javax.swing.JButton();
        m_applyRangesAndQuitButton = new javax.swing.JButton();
        jPanel5 = new javax.swing.JPanel();
        m_deleteSelectedListRangeItemsButton = new javax.swing.JButton();
        m_applyjButton = new javax.swing.JButton();
        jScrollPane2 = new javax.swing.JScrollPane();
        m_splineNodePointsTable = new javax.swing.JTable();
        jPanel2 = new javax.swing.JPanel();
        m_availableSplineNodeModelsjComboBox = new javax.swing.JComboBox<>();
        jLabel1 = new javax.swing.JLabel();
        m_saveCurrentSplineNodesOfDefaultModelAndQuitButton = new javax.swing.JButton();
        m_createNewBaselineSplinejButton = new javax.swing.JButton();
        m_absorptionModeBLRmovaljRadioButton = new javax.swing.JRadioButton();
        m_absolutModeBLRemovalRadioButton = new javax.swing.JRadioButton();
        jPanel3 = new javax.swing.JPanel();
        m_MRSpectrumBaselineGUIPropertiesDefiner = new org.spectrim.tdfdfit.MRSpectrumBaselineGUIPropertiesDefiner();

        setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosed(java.awt.event.WindowEvent evt) {
                formWindowClosed(evt);
            }
            public void windowClosing(java.awt.event.WindowEvent evt) {
                formWindowClosing(evt);
            }
        });

        m_cancelButton.setText(" Cancel & Quit");
        m_cancelButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_cancelButtonActionPerformed(evt);
            }
        });

        m_applyRangesAndQuitButton.setText("Apply & quit");
        m_applyRangesAndQuitButton.setEnabled(false);
        m_applyRangesAndQuitButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_applyRangesAndQuitButtonActionPerformed(evt);
            }
        });

        jPanel5.setBorder(javax.swing.BorderFactory.createTitledBorder("Spline node editing options"));

        m_deleteSelectedListRangeItemsButton.setText("Delete point");
        m_deleteSelectedListRangeItemsButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_deleteSelectedListRangeItemsButtonActionPerformed(evt);
            }
        });

        m_applyjButton.setText("Apply currently selected node points to  signal");
        m_applyjButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_applyjButtonActionPerformed(evt);
            }
        });

        m_splineNodePointsTable.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {null, null, null},
                {null, null, null},
                {null, null, null},
                {null, null, null}
            },
            new String [] {
                "Spline point number", "X-Value", "Y-value"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.Integer.class, java.lang.Float.class, java.lang.Float.class
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }
        });
        jScrollPane2.setViewportView(m_splineNodePointsTable);

        org.jdesktop.layout.GroupLayout jPanel5Layout = new org.jdesktop.layout.GroupLayout(jPanel5);
        jPanel5.setLayout(jPanel5Layout);
        jPanel5Layout.setHorizontalGroup(
            jPanel5Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel5Layout.createSequentialGroup()
                .add(jPanel5Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jPanel5Layout.createSequentialGroup()
                        .add(m_applyjButton)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 947, Short.MAX_VALUE)
                        .add(m_deleteSelectedListRangeItemsButton))
                    .add(jScrollPane2))
                .addContainerGap())
        );
        jPanel5Layout.setVerticalGroup(
            jPanel5Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanel5Layout.createSequentialGroup()
                .addContainerGap()
                .add(jScrollPane2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 92, Short.MAX_VALUE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel5Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(m_deleteSelectedListRangeItemsButton)
                    .add(m_applyjButton))
                .addContainerGap())
        );

        jPanel2.setBorder(javax.swing.BorderFactory.createTitledBorder("Spline model options"));

        m_availableSplineNodeModelsjComboBox.setModel(new javax.swing.DefaultComboBoxModel<>(new String[] { "No model selected yet!" }));
        m_availableSplineNodeModelsjComboBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_availableSplineNodeModelsjComboBoxActionPerformed(evt);
            }
        });

        jLabel1.setText("Current selected baseline model");

        m_saveCurrentSplineNodesOfDefaultModelAndQuitButton.setText("Save defined nodes points  to a baseline spline model ..");
        m_saveCurrentSplineNodesOfDefaultModelAndQuitButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_saveCurrentSplineNodesOfDefaultModelAndQuitButtonActionPerformed(evt);
            }
        });

        m_createNewBaselineSplinejButton.setText("Create a new baseline spline model..");
        m_createNewBaselineSplinejButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_createNewBaselineSplinejButtonActionPerformed(evt);
            }
        });

        m_radioButtonGroup.add(m_absorptionModeBLRmovaljRadioButton);
        m_absorptionModeBLRmovaljRadioButton.setText("Absorption");
        m_absorptionModeBLRmovaljRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_absorptionModeBLRmovaljRadioButtonActionPerformed(evt);
            }
        });

        m_radioButtonGroup.add(m_absolutModeBLRemovalRadioButton);
        m_absolutModeBLRemovalRadioButton.setSelected(true);
        m_absolutModeBLRemovalRadioButton.setText("Magnitude");
        m_absolutModeBLRemovalRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_absolutModeBLRemovalRadioButtonActionPerformed(evt);
            }
        });

        org.jdesktop.layout.GroupLayout jPanel2Layout = new org.jdesktop.layout.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel2Layout.createSequentialGroup()
                .addContainerGap()
                .add(jLabel1)
                .add(18, 18, 18)
                .add(m_availableSplineNodeModelsjComboBox, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 227, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .add(18, 18, 18)
                .add(m_saveCurrentSplineNodesOfDefaultModelAndQuitButton)
                .add(18, 18, 18)
                .add(m_createNewBaselineSplinejButton)
                .add(28, 28, 28)
                .add(m_absorptionModeBLRmovaljRadioButton)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                .add(m_absolutModeBLRemovalRadioButton)
                .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanel2Layout.createSequentialGroup()
                .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(m_availableSplineNodeModelsjComboBox, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel1)
                    .add(m_saveCurrentSplineNodesOfDefaultModelAndQuitButton)
                    .add(m_createNewBaselineSplinejButton)
                    .add(m_absorptionModeBLRmovaljRadioButton)
                    .add(m_absolutModeBLRemovalRadioButton))
                .addContainerGap())
        );

        org.jdesktop.layout.GroupLayout jPanel1Layout = new org.jdesktop.layout.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .add(m_cancelButton)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .add(m_applyRangesAndQuitButton)
                .add(24, 24, 24))
            .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanel1Layout.createSequentialGroup()
                .add(jPanel5, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addContainerGap())
            .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanel2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel1Layout.createSequentialGroup()
                .add(19, 19, 19)
                .add(jPanel2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel5, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(m_cancelButton)
                    .add(m_applyRangesAndQuitButton))
                .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        org.jdesktop.layout.GroupLayout jPanel3Layout = new org.jdesktop.layout.GroupLayout(jPanel3);
        jPanel3.setLayout(jPanel3Layout);
        jPanel3Layout.setHorizontalGroup(
            jPanel3Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(m_MRSpectrumBaselineGUIPropertiesDefiner, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 1333, Short.MAX_VALUE)
        );
        jPanel3Layout.setVerticalGroup(
            jPanel3Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(m_MRSpectrumBaselineGUIPropertiesDefiner, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 434, Short.MAX_VALUE)
        );

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(jPanel1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
            .add(jPanel3, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                .add(jPanel3, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    /**
     * Is called when the delete button is pressed.
     */
    private void m_deleteSelectedListRangeItemsButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_deleteSelectedListRangeItemsButtonActionPerformed
        int selectedIndex = m_splineNodePointsTable.getSelectedRow( );
        if ( selectedIndex > -1 )
        {
            m_userDefinedSplineNodes.remove( selectedIndex );
        }
        
        // clear the whole JTable
        DefaultTableModel model = (DefaultTableModel)m_splineNodePointsTable.getModel( ); 
        int rows = model.getRowCount( ); 
        for( int i = rows - 1; i >= 0; i-- )
        {   
            model.removeRow( i ); 
        }
        
        for ( int i = 1 ; i <= m_userDefinedSplineNodes.size( ); i++ )
        {
            model.addRow( new Object[]{ i, (double)m_userDefinedSplineNodes.get( i-1 ).x, (double)m_userDefinedSplineNodes.get( i-1 ).y } );   
        }
        
        drawSpline( );
        
        
    }//GEN-LAST:event_m_deleteSelectedListRangeItemsButtonActionPerformed

    
    // Now draw the spline again!
     void drawSpline( )
     { 
        // first remove the old one ... 
        m_MRSpectrumBaselineGUIPropertiesDefiner.removeSpline( );
        
        // retrieve the new nodes .. 
        
        m_userDefinedSplineNodes = null;
        
        m_userDefinedSplineNodes = m_MRSpectrumBaselineGUIPropertiesDefiner.getOrderedSplineNodeList( );
        
        if ( m_userDefinedSplineNodes != null )
        {
            
            if ( m_userDefinedSplineNodes.size( ) > 2 )
            {
                // Reset the table content ...
                DefaultTableModel dtm = (DefaultTableModel)m_splineNodePointsTable.getModel( );
                dtm.setRowCount(0);

                // Reset spline point model ...
                // m_userDefinedSplineNodes.clear( );

                // Update the table 
                for ( int i = 0; i < m_userDefinedSplineNodes.size( ); i++ )
                {
                    dtm.addRow( new Object[]{ i, m_userDefinedSplineNodes.get( i ).x, m_userDefinedSplineNodes.get( i ).y } );
                }

                double[] xaxis = new double[ m_userDefinedSplineNodes.size( ) ];
                float[]  xaxisFlt = new float[ m_userDefinedSplineNodes.size( ) ];
                double[] y = new double[ m_userDefinedSplineNodes.size( ) ];

                double minX = m_userDefinedSplineNodes.get( 0 ).x;
                double maxX = m_userDefinedSplineNodes.get( m_userDefinedSplineNodes.size( ) - 1 ).x;

                double[] xInterpolPoints = new double[m_nPoints];
                float[]  xInterpolPointsFlt = new float[m_nPoints];
                
                for ( int i = 0; i < m_nPoints; i++ )
                {
                    xInterpolPoints[i] = maxX - ( maxX - minX ) * i / ( m_nPoints - 1 );
                    xInterpolPointsFlt[i] = (float)xInterpolPoints[i];
                }

                for ( int i = 0; i < m_userDefinedSplineNodes.size( ); i++ )
                {   
                    xaxis[i] = m_userDefinedSplineNodes.get(i).x;
                    xaxisFlt[i] = (float)m_userDefinedSplineNodes.get(i).x;
                    y[i] = m_userDefinedSplineNodes.get(i).y;
                }

                if ( m_userDefinedSplineNodes.size( ) > 3 )
                {

                    m_spline = new float[m_nPoints];

                    SplineInterpolator myInterpolator = new SplineInterpolator( );
                    PolynomialSplineFunction polynomialSplineFunction = myInterpolator.interpolate( xaxis, y );

                    for ( int k = 0; k < m_nPoints; k++ )
                    {
                        m_spline[k] = (float)polynomialSplineFunction.value( xInterpolPoints[ k ] );
                    }

                    m_MRSpectrumBaselineGUIPropertiesDefiner.addSpline( new Curve( xInterpolPointsFlt, m_spline, Color.BLUE, "spline" ), true );
                }
            }
        }
    }
    
    /**
     * Is called when this window is terminated and disposed.
     */
    private void formWindowClosing(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowClosing

        // sent the cancel event to all listeners!
        Iterator it = m_actionListeners.iterator( );
        while( it.hasNext( ) )
        {
            ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, CANCEL_EVENT, "" ) );
        }
        
        this.dispose( );
        
    }//GEN-LAST:event_formWindowClosing

    private void m_applyRangesAndQuitButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_applyRangesAndQuitButtonActionPerformed
        
        m_orderedBaseLinePointsDataModel.deleteAllPoints( );
        
        for ( int i = 0; i < m_userDefinedSplineNodes.size( ) ; i++ )
        {
            m_orderedBaseLinePointsDataModel.addPoint( m_userDefinedSplineNodes.get( i ) );
        }
        
        // System.out.println( "Ordered baseline spline nodes number : " + m_orderedBaseLinePointsDataModel.getSize( ) );
        
        Iterator it = m_actionListeners.iterator( );
        while( it.hasNext( ) )
        {
            ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, ACCEPT_EVENT, "" ) );
        }

        setVisible( false );
    }//GEN-LAST:event_m_applyRangesAndQuitButtonActionPerformed
    
    /**
     * Called when closed.
     * @param evt 
     */
    private void formWindowClosed(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowClosed
        
        // sent the cancel event to all listeners!
        Iterator it = m_actionListeners.iterator( );
        while( it.hasNext( ) )
        {
            ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, CANCEL_EVENT, "" ) );
        }
        
        this.setVisible( false );
    }//GEN-LAST:event_formWindowClosed

    /**
     * Called whenever the cancel button is pressed.
     */
    private void m_cancelButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_cancelButtonActionPerformed
       
        Iterator it = m_actionListeners.iterator( );
        while( it.hasNext( ) )
        {
            ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, CANCEL_EVENT, "" ) );
        }
        
        this.setVisible( false ); 
    }//GEN-LAST:event_m_cancelButtonActionPerformed

    /**
     * Asks the user to set the spline node model to be used!
     * @return 
     */
    private String getCurrentSplineNodeModelFile( ) 
    {
        
        TDFDFittingDefaults defs = TDFDFittingDefaults.getInstance( );
        defs.load( );
        
        String splineNodeModelPath = defs.getProperty( TDFDFittingDefaults.BASELINE_SPLINE_MODEL_PATH );
        String splineNodeModelName = defs.getProperty( TDFDFittingDefaults.BASELINE_SPLINE_MODEL_NAME );
        
        String defaultPathAndName;
        if ( splineNodeModelName != null )
        {
            defaultPathAndName = splineNodeModelPath + System.getProperty( "file.separator" ) + splineNodeModelName;
        }
        else
        {
            defaultPathAndName = splineNodeModelPath;
        }    
        
        if ( defaultPathAndName.endsWith( ".snm.snm" ) == true )
        {
            defaultPathAndName = defaultPathAndName.substring( 0, defaultPathAndName.length( ) - 4 );
        }
        
        JFileChooser fileChooser = new JFileChooser( splineNodeModelPath );
        
        FileNameExtensionFilter filter = new FileNameExtensionFilter( "SNM", "snm" );
        fileChooser.setFileSelectionMode( JFileChooser.FILES_ONLY );
        fileChooser.setFileFilter( filter );
        fileChooser.setDialogTitle( "Please select a spline node model (SNM) file.");
        
        int result = fileChooser.showOpenDialog(this);

        switch ( result ) 
        {
            case JFileChooser.APPROVE_OPTION:    // Approve Open button was pressed
                defs.setProperty( TDFDFittingDefaults.BASELINE_SPLINE_MODEL_NAME, fileChooser.getSelectedFile( ).getName( )  );
                defs.setProperty( TDFDFittingDefaults.BASELINE_SPLINE_MODEL_PATH, fileChooser.getSelectedFile( ).getParent( ) );
                defs.save( );
                return ( fileChooser.getSelectedFile( ).toString( ) );
            case JFileChooser.CANCEL_OPTION:     // Cancel or close dialog was selected
                return null;
            case JFileChooser.ERROR_OPTION:      // the selected process did not terminate properly
                return null;
        }
        return null;
    }
    
    /**
     * Called whenever the add Range button is pressed.
     */ 
    private void m_saveCurrentSplineNodesOfDefaultModelAndQuitButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_saveCurrentSplineNodesOfDefaultModelAndQuitButtonActionPerformed
    
        TDFDFittingDefaults fDef = TDFDFittingDefaults.getInstance( );
        fDef.load( );
        
        String currentSplineNodeModelName = getCurrentSplineNodeModelFile( );
        
        if ( currentSplineNodeModelName != null )
        {
            if ( currentSplineNodeModelName.endsWith( ".snm.snm" ) == true )
            {
                currentSplineNodeModelName = currentSplineNodeModelName.substring( 0, currentSplineNodeModelName.length( ) - 4 );
            }
            
            if ( currentSplineNodeModelName.endsWith( ".snm" ) == false )
            {
                currentSplineNodeModelName = currentSplineNodeModelName + ".snm";
            }
        }
        
        m_orderedBaseLinePointsDataModel.deleteAllPoints( );
        for ( int i = 0; i < m_userDefinedSplineNodes.size( ) ; i++ )
        {
            m_orderedBaseLinePointsDataModel.addPoint( m_userDefinedSplineNodes.get( i ) );
        }
        // System.out.println( "Ordered baseline spline nodes number : " + m_orderedBaseLinePointsDataModel.getSize( ) );
              
        // now write it to a file..
        
        RandomAccessFile out;
        
        try 
        {
            out = new RandomAccessFile( currentSplineNodeModelName, "rw" );
            m_orderedBaseLinePointsDataModel.write(out);
            try 
            {
                out.close( );
            } 
            catch (IOException ex) 
            {
                Logger.getLogger( BaseLineSplineSupportPointsDefinitionFrame.class.getName( ) ).log( Level.SEVERE, null, ex );
            }
        } 
        catch ( FileNotFoundException ex ) 
        {
            Logger.getLogger( BaseLineSplineSupportPointsDefinitionFrame.class.getName( ) ).log( Level.SEVERE, null, ex );
        }
       
        File file = new File( currentSplineNodeModelName );
        
        String modelNameToAddToComboBox = file.getName( ); 
        
        m_availableSplineNodeModelsjComboBox.addItem( modelNameToAddToComboBox );
        m_availableSplineNodeModelsjComboBox.setSelectedIndex( m_availableSplineNodeModelsjComboBox.getItemCount( ) - 1 );
        
        TDFDFittingDefaults fitdef = TDFDFittingDefaults.getInstance( );
        fitdef.load( );
        fitdef.setProperty( TDFDFittingDefaults.BASELINE_SPLINE_MODEL_NAME, modelNameToAddToComboBox );
        fitdef.save( );
             
    }//GEN-LAST:event_m_saveCurrentSplineNodesOfDefaultModelAndQuitButtonActionPerformed

    private void m_applyjButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_applyjButtonActionPerformed
        
//        TDFDFittingDefaults fitdef = TDFDFittingDefaults.getInstance( );
//        fitdef.load( );
//        
//        fitdef.setProperty( TDFDFittingDefaults.DISPLAY_CHANNEL, TDFDFittingDefaults.ABSORPTION );
//        fitdef.save( );
        
        float[] realDisplayed = m_specData.getAbsorptionChannelFD( );
        float[] imagDisplayed = m_specData.getDispersionChannelFD( );
        float[] magnDisplayed = m_specData.getMagnitudeCurveFDPPM( ).getYvalues( );
        
        float[] phaseDisplayed = new float[realDisplayed.length];
        
        for ( int i = 0; i < realDisplayed.length; i++ )
        {
            phaseDisplayed[i] = (float)Math.atan2( realDisplayed[i], imagDisplayed[i] );
        }
        
        float[] difference = new float[realDisplayed.length];
        
        for( int i = 0 ; i < realDisplayed.length; i++ )
        {
            difference[i] = magnDisplayed[i] - m_spline[i];
        }
        
        // re-apply the phase
        for( int i = 0 ; i < realDisplayed.length; i++ )
        {
            m_differenceReal[i] = (float)( difference[i] * Math.sin( phaseDisplayed[i] ) );
            m_differenceImag[i] = (float)( difference[i] * Math.cos( phaseDisplayed[i] ) );
        }
        
        double minX = m_userDefinedSplineNodes.get( 0 ).x;
        double maxX = m_userDefinedSplineNodes.get( m_userDefinedSplineNodes.size( ) - 1 ).x;

        double[] xInterpolPoints = new double[m_nPoints];
        
        m_xInterpolPointsFlt = new float[m_nPoints];
        for ( int i = 0; i < m_nPoints; i++ )
        {
            xInterpolPoints[i] = maxX - ( maxX - minX ) * i / ( m_nPoints - 1 );
            m_xInterpolPointsFlt[i] = (float)xInterpolPoints[i];
        }
        
        m_MRSpectrumBaselineGUIPropertiesDefiner.addCurve( new Curve( m_xInterpolPointsFlt, difference, Color.BLUE, "difference" ), true );
        m_MRSpectrumBaselineGUIPropertiesDefiner.addCurve( new Curve( m_xInterpolPointsFlt, realDisplayed, Color.BLACK, "corrected_real" ), true );
        m_MRSpectrumBaselineGUIPropertiesDefiner.addCurve( new Curve( m_xInterpolPointsFlt, imagDisplayed, Color.ORANGE, "corrected_imag" ), true );
        
        m_applyRangesAndQuitButton.setEnabled( true );
        
    }//GEN-LAST:event_m_applyjButtonActionPerformed

    public float[] getBaselineCorrectedResponseReal( )
    {
        SingleVoxelSpectralData output = new SingleVoxelSpectralData( this.m_differenceReal, this.m_differenceImag, SingleVoxelSpectralData.FREQ_DOMAIN );
        return output.getAbsorptionChannelTD( );
    }
    
    public float[] getBaselineCorrectedResponseImag( )
    {
        SingleVoxelSpectralData output = new SingleVoxelSpectralData( m_differenceReal, m_differenceImag, SingleVoxelSpectralData.FREQ_DOMAIN );
        return output.getDispersionChannelTD( );
    }

    private void m_availableSplineNodeModelsjComboBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_availableSplineNodeModelsjComboBoxActionPerformed

        TDFDFittingDefaults fitdef = TDFDFittingDefaults.getInstance( );
        fitdef.load( );
        
        fitdef.setProperty( TDFDFittingDefaults.BASELINE_SPLINE_MODEL_NAME, (String)m_availableSplineNodeModelsjComboBox.getSelectedItem( ) );
        fitdef.save( );
        
    }//GEN-LAST:event_m_availableSplineNodeModelsjComboBoxActionPerformed

    private void m_createNewBaselineSplinejButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_createNewBaselineSplinejButtonActionPerformed
       // Clear the current model
        OrderedBaseLinePointsDataModel oBDPM = OrderedBaseLinePointsDataModel.getInstance( );
        oBDPM.deleteAllPoints( );
        m_userDefinedSplineNodes.removeAllElements( );
        m_MRSpectrumBaselineGUIPropertiesDefiner.removeCurrentOrderedSplineNodeList( );
        m_MRSpectrumBaselineGUIPropertiesDefiner. removeSpline( );
        ( (DefaultTableModel)m_splineNodePointsTable.getModel( ) ).setRowCount( 0 );
    }//GEN-LAST:event_m_createNewBaselineSplinejButtonActionPerformed

    private void m_absolutModeBLRemovalRadioButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_absolutModeBLRemovalRadioButtonActionPerformed
        TDFDFittingDefaults.getInstance( ).setProperty( TDFDFittingDefaults.BASELINE_SPLINE_REMOVAL_MODE, TDFDFittingDefaults.MAGNITUDE );
        TDFDFittingDefaults.getInstance( ).setProperty( TDFDFittingDefaults.DISPLAY_CHANNEL, TDFDFittingDefaults.MAGNITUDE );
        TDFDFittingDefaults.getInstance( ).save( );
        updateGUIAfterSplineModelSelection( );
        m_MRSpectrumBaselineGUIPropertiesDefiner.setModelSpectrum( m_specData, true );
        handleSplinePointAdded( );
    }//GEN-LAST:event_m_absolutModeBLRemovalRadioButtonActionPerformed

    private void m_absorptionModeBLRmovaljRadioButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_absorptionModeBLRmovaljRadioButtonActionPerformed
       TDFDFittingDefaults.getInstance( ).setProperty( TDFDFittingDefaults.BASELINE_SPLINE_REMOVAL_MODE, TDFDFittingDefaults.ABSORPTION );
       TDFDFittingDefaults.getInstance( ).setProperty( TDFDFittingDefaults.DISPLAY_CHANNEL, TDFDFittingDefaults.ABSORPTION );
       TDFDFittingDefaults.getInstance( ).save( );
       updateGUIAfterSplineModelSelection( );
       m_MRSpectrumBaselineGUIPropertiesDefiner.setModelSpectrum( m_specData, true );
       handleSplinePointAdded( );
    }//GEN-LAST:event_m_absorptionModeBLRmovaljRadioButtonActionPerformed
     
    private void updateGUIAfterSplineModelSelection( )
    {
        String splineNodeModelName = (String)m_availableSplineNodeModelsjComboBox.getSelectedItem( );
        String splineNodeModelPath = TDFDFittingDefaults.getInstance( ).getProperty( TDFDFittingDefaults.BASELINE_SPLINE_MODEL_PATH );
        
        // Clear the current model
        OrderedBaseLinePointsDataModel oBDPM = OrderedBaseLinePointsDataModel.getInstance( );
        oBDPM.deleteAllPoints( );
        m_userDefinedSplineNodes.removeAllElements( );
        
        // load the last model
        if ( ( splineNodeModelName != null ) && ( splineNodeModelName.compareTo( "No model selected yet!" ) != 0 ) )
        {
            try 
            {
                String lastSavedModel = splineNodeModelPath + System.getProperty( "file.separator" ) + splineNodeModelName;
                RandomAccessFile in = new RandomAccessFile( lastSavedModel, "r" );
                
                oBDPM.read( in );
                
                try 
                {
                    in.close( );
                } 
                catch ( IOException ex ) 
                {
                    Logger.getLogger( BaseLineSplineSupportPointsDefinitionFrame.class.getName( ) ).log( Level.SEVERE, null, ex );
                }
                
                for ( int i = 0; i < oBDPM.getSize( ) ; i++ )
                {
                    float yValue = getCorrespondingYValue( (float)oBDPM.getPoint( i ).getX( ) );
                    m_userDefinedSplineNodes.add( new Point2D.Double( (float)oBDPM.getPoint( i ).getX( ), yValue ) );
                }
                
                m_MRSpectrumBaselineGUIPropertiesDefiner.setOrderedSplineNodeList( m_userDefinedSplineNodes );
                
                for ( int i = 0 ; i < m_userDefinedSplineNodes.size( ); i++ )
                {
                    DefaultTableModel model = (DefaultTableModel) m_splineNodePointsTable.getModel( );
                    model.addRow( new Object[]{ i, m_userDefinedSplineNodes.get( i ).x, m_userDefinedSplineNodes.get( i ).y } );
                }
                
                if ( m_splineNodePointsTable.getModel( ).getRowCount( ) > 0 )
                {
                    m_splineNodePointsTable.setRowSelectionInterval( 0, 0 );
                }
                
                drawSpline( );
            } 
            catch ( FileNotFoundException ex ) 
            {
                Logger.getLogger( BaseLineSplineSupportPointsDefinitionFrame.class.getName( ) ).log( Level.SEVERE, null, ex );
            }
        } 
    }
    
    
    /**
     * Adds an action listener to this.
     * @param e
     */
     public void addActionListener( ActionListener e )
    {
        m_actionListeners.add( e );
    }
    
    /**
     * Returns the current range model.
     * @return 
     */
    public OrderedBaseLinePointsDataModel getRangeModel( )
    {
        return m_orderedBaseLinePointsDataModel;
    }
     
    /**
     * Window listener implementation. 
     * @param e
     */
    @Override
    public void windowActivated( WindowEvent e ) 
    {
    }
    
    /**
     * Close window event handler: dispose! 
     * @param e 
     */
    @Override
    public void windowClosed( WindowEvent e ) 
    {
        this.dispose( );
    }
    
    @Override
    public void windowClosing( WindowEvent e ) 
    {
    	this.dispose( );
    }
    
    @Override
    public void windowDeactivated( WindowEvent e ) 
    {
    }
    
    @Override
    public void windowDeiconified( WindowEvent e ) 
    {
    }
    
    @Override
    public void windowIconified( WindowEvent e ) 
    {
    }
    
    @Override
    public void windowOpened( WindowEvent e ) 
    {
    }
    
    private void handleSplinePointAdded( )
    {
        
        if (  m_userDefinedSplineNodes != null )
        {
        
            m_MRSpectrumBaselineGUIPropertiesDefiner.removeSpline( );

            m_userDefinedSplineNodes = m_MRSpectrumBaselineGUIPropertiesDefiner.getOrderedSplineNodeList( );

            // Reset the table content ...
            DefaultTableModel dtm = (DefaultTableModel)m_splineNodePointsTable.getModel( );
            dtm.setRowCount(0);

            // Reset spline point model ...
            // m_userDefinedSplineNodes.clear( );
            double[] xValues  = new double[m_userDefinedSplineNodes.size( )];
            int[]    toRemove = new int[m_userDefinedSplineNodes.size( )];

            for ( int i = 0; i < m_userDefinedSplineNodes.size( ); i++ )
            {
                xValues[i] = m_userDefinedSplineNodes.get( i ).x;
            }

            for ( int i = 0; i < xValues.length; i++ )
            {
                for ( int j = 1; j < xValues.length; j++ )
                {
                    if ( ( i != j ) && ( j > i ) )
                    {
                        if ( xValues[i] == xValues[j] )
                        {
                            toRemove[j] = 1;
                        }
                    }
                }
            }

            for ( int i = toRemove.length - 1; i > 0; i-- )
            {
                if ( toRemove[i] == 1 )
                {
                    m_userDefinedSplineNodes.remove( i );
                    System.out.println( "Removing double node " + i );
                }
            }


            // update the table 
            for ( int i = 0; i < m_userDefinedSplineNodes.size( ); i++ )
            {
                dtm.addRow( new Object[]{ i, m_userDefinedSplineNodes.get( i ).x, m_userDefinedSplineNodes.get( i ).y } );
            }

            double[] xaxis = new double[ m_userDefinedSplineNodes.size( ) ];
            float[]  xaxisFlt = new float[ m_userDefinedSplineNodes.size( ) ];
            double[] y = new double[ m_userDefinedSplineNodes.size( ) ];

            double minX = m_userDefinedSplineNodes.get( 0 ).x;
            double maxX = m_userDefinedSplineNodes.get( m_userDefinedSplineNodes.size( ) - 1 ).x;

            double[] xInterpolPoints = new double[m_nPoints];

            m_xInterpolPointsFlt = new float[m_nPoints];
            for ( int i = 0; i < m_nPoints; i++ )
            {
                xInterpolPoints[i] = maxX - ( maxX - minX ) * i /( m_nPoints - 1 );
                m_xInterpolPointsFlt[i] = (float)xInterpolPoints[i];
            }

            for ( int i = 0; i < m_userDefinedSplineNodes.size( ); i++ )
            {   
                xaxis[i] = m_userDefinedSplineNodes.get(i).x;
                xaxisFlt[i] = (float)m_userDefinedSplineNodes.get(i).x;
                y[i] = m_userDefinedSplineNodes.get(i).y;
            }

            if ( m_userDefinedSplineNodes.size( ) > 3 )
            {
                m_spline = new float[m_nPoints];

                SplineInterpolator myInterpolator = new SplineInterpolator( );
                PolynomialSplineFunction polynomialSplineFunction = myInterpolator.interpolate( xaxis, y );
                for ( int k = 0; k < m_nPoints; k++ )
                {
                    m_spline[k] = (float)polynomialSplineFunction.value( xInterpolPoints[ k ] );
                }
                m_MRSpectrumBaselineGUIPropertiesDefiner.addSpline( new Curve( m_xInterpolPointsFlt, m_spline, Color.BLUE, "spline" ), true );
                m_MRSpectrumBaselineGUIPropertiesDefiner.setSplineNodes( m_userDefinedSplineNodes );
            }
        }
    }

    /**
     * Is called whenever a registered component raises an ActionEvent.
     * @param e
     */
    @Override
    public void actionPerformed( ActionEvent e ) 
    {
        Object sender = e.getSource( );
        
        if ( e.getSource( ) instanceof MRSpectrumBaselineGUIPropertiesDefiner )
        {
            // System.out.println( "Baseline point added!" );
            
            if ( e.getID( ) == MRSpectrumBaselineGUIPropertiesDefiner.SPLINE_POINT_ADDED )
            {   
                handleSplinePointAdded( );
            }
        }
              
        if ( sender instanceof  SingleVoxelSpectrumSeriesViewer ) 
        {
            if ( e.getID( ) == SingleVoxelSpectrumSeriesViewer.BASELINE_POINT_SELECTED_EVENT )
            {
                Point2D.Double p = ( (SingleVoxelSpectrumSeriesViewer)e.getSource( ) ).getLastSelectedPoint( );
                
                OrderedBaseLinePointsDataModel oM = OrderedBaseLinePointsDataModel.getInstance( );
                oM.setArraySize( m_specData.getNumberOfSamplePoints( ) );
                oM.addPoint( p );
            }
        }
        
        if ( sender instanceof  SingleVoxelSpectrumViewer ) 
        {
            if ( e.getID( ) == SingleVoxelSpectrumViewer.BASELINE_POINT_SELECTED_EVENT )
            {
                Point2D.Double p = ( (SingleVoxelSpectrumViewer)e.getSource( ) ).getLastSelectedPoint( );
                OrderedBaseLinePointsDataModel oM = OrderedBaseLinePointsDataModel.getInstance( );
                oM.setArraySize( m_specData.getNumberOfSamplePoints( ) );
                oM.addPoint( p );
            }
        }
    }
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    public javax.swing.JLabel jLabel1;
    public javax.swing.JPanel jPanel1;
    public javax.swing.JPanel jPanel2;
    public javax.swing.JPanel jPanel3;
    public javax.swing.JPanel jPanel5;
    public javax.swing.JScrollPane jScrollPane2;
    public org.spectrim.tdfdfit.MRSpectrumBaselineGUIPropertiesDefiner m_MRSpectrumBaselineGUIPropertiesDefiner;
    public javax.swing.JRadioButton m_absolutModeBLRemovalRadioButton;
    public javax.swing.JRadioButton m_absorptionModeBLRmovaljRadioButton;
    public javax.swing.JButton m_applyRangesAndQuitButton;
    public javax.swing.JButton m_applyjButton;
    public javax.swing.JComboBox<String> m_availableSplineNodeModelsjComboBox;
    public javax.swing.JButton m_cancelButton;
    public javax.swing.JButton m_createNewBaselineSplinejButton;
    public javax.swing.JButton m_deleteSelectedListRangeItemsButton;
    public javax.swing.ButtonGroup m_radioButtonGroup;
    public javax.swing.JButton m_saveCurrentSplineNodesOfDefaultModelAndQuitButton;
    public javax.swing.JTable m_splineNodePointsTable;
    // End of variables declaration//GEN-END:variables
    
    
    private SingleVoxelMeasuredSpectralData m_specData;
    private OrderedBaseLinePointsDataModel m_orderedBaseLinePointsDataModel = OrderedBaseLinePointsDataModel.getInstance( );
    Vector<Point2D.Double> m_userDefinedSplineNodes = new Vector<Point2D.Double>( ); 
    private LinkedList<ActionListener> m_actionListeners = new LinkedList<>( );
    private int m_nPoints;
    
    // Possible Events
    public final static int CANCEL_EVENT    = 0;
    public final static int ACCEPT_EVENT    = 1;
    public final static int PREVIEW_EVENT   = 2;
    
    // Most recent computed spline.
    float[] m_spline;
    float[] m_xInterpolPointsFlt;
    
    float[] m_differenceReal;
    float[] m_differenceImag;
    
    String m_splineNodeModelPath;
    String m_splineNodeModelName;
    String m_splineNodeModelMode;
    
    String m_currentBaselineModel = "";
    
    // For proper serialization
    private static final long serialVersionUID = 64823334441223L;
}
