/*
 * File: $Id: Bspline.java,v 1.1.1.1 2006/04/19 09:38:50 hans Exp $
 * History
 * Date: 14.03.2006  Authors: Hans Slotboom  Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */
 


package org.spectrim.tdfdfit;

import java.awt.geom.Point2D;

/**
 * B splines
 * This applet shows cubic B splines defined by a sequence of control points. 
 * You can move a control point by dragging it with the mouse, add a new one by 
 * clicking the mouse, and delete one by holding down the Shift key while 
 * clicking on it. 
 * Each four successive control points defines a cubic curve. That is, 
 * controls 0123, 1234, 2345 each define a curve. These curves join together 
 * smoothly (the first and second derivatives are continuous). 
 * Unlike a natural cubic spline <natcubic.html>, a B spline has local control. 
 * This means that modifying one control point only affects the part of the 
 * curve near that control point. This is very useful when using the 
 * B splines for designing shapes. 
 *
 * lambert@cse.unsw.edu.au
 * If you want to learn more about the mathematics behind all of this, 
 * I recommend An Introduction to Splines for Use in Computer Graphics and 
 * Geometric Modelling by Bartels, Beatty and Barsky published by Morgan 
 * Kaufman, Los Altos, CA, 1987. 
 *
 * @author  extern / changes hans
 */

public class Bspline extends ControlCurve 
{

    /**
     *  Defines the basis function for a cubic B spline.
     */
    float b( int i, float t ) 
    {
        switch ( i ) 
        {
            case -2:
                        return ( ( ( -t + 3 ) * t - 3 ) * t + 1 ) / 6;
            case -1:
                        return ( ( ( 3 * t - 6 ) * t ) * t + 4 ) / 6;
            case 0:
                    return ( ( ( - 3 * t + 3 ) * t + 3 ) * t + 1 ) / 6;
            case 1:
                    return ( t * t * t ) / 6;
        }
        return 0; //we only get here if an invalid i is specified
    }
  
    /**
     * Evaluates a point on the B spline.
     */
    Point2D.Float p( int i, float t ) 
    {
        float px = 0f;
        float py = 0f;
        for ( int j = -2; j <= 1; j++ )
        {
            //px += b( j , t ) * pts.xpoints[i + j];
            //py += b( j , t ) * pts.ypoints[i + j];
            
            Point2D.Float pnt = (Point2D.Float)m_polygon.get( i + j );
            
            px += b( j , t ) * pnt.getX( );
            py += b( j , t ) * pnt.getY( );
            
        }
        
        return new Point2D.Float( px, py );
    }
    
    /**
     * Computes the BSpline or CatmullRomSpline on the "intervals" grid.
     * @param intervals
     * @return 
     */
    public float[] computeSplineCurve( int intervals )
    {
        float[] spline = new float[intervals];
        int k = 0;
        Point2D.Float q; 
        
        spline[k++] = p( 2, 0f ).y;
        
        // test hs
        // float firstPointY = (float)m_polygon.get( 0 ).getY( );
        // spline[k++] = p( 2, (float)firstPointY ).y;
        
        for ( int i = 2; i < m_polygon.size( ) - 1; i++ ) 
        {
            float beginXindex = (float)( (Point2D.Float)m_polygon.get( i - 2 ) ).getX( );
            float endXindex = (float)( (Point2D.Float)m_polygon.get( i - 1 ) ).getX( );
            
            int XSTEPS = Math.abs( (int)( endXindex - beginXindex ) );
            
            // System.out.println( "XSTEPS = " + XSTEPS );
            
            for ( int j = 1; j <= XSTEPS; j++ ) 
            {
                q = p( i, j / (float)XSTEPS );
                spline[k++] = q.y;
            }
        }
        
        return spline;
    }

}