/*
 * File: $Id: CatmullRomSpline.java,v 1.1.1.1 2006/04/19 09:38:50 hans Exp $
 * History
 * Date: 14.03.2006  Authors: Hans Slotboom  Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */


// Catmull-Rom splines
//
// This applet shows cubic Catmull-Rom splines defined by a sequence of control points. 
// You can move a control point by dragging it with the mouse, add a new one by clicking 
// the mouse, and delete one by holding down the Shift key while clicking on it. 
// Controls 0123 define a cubic curve going from 1 to 2. Controls 1234 define a curve 
// going from 2 to 3 and so on The tangent to both curves at 2 is parallel to the line 
// joining 1 to 3. Unlike a natural cubic spline, a Catmull-Rom spline has local control. 
// This means that modifying one control point only affects the part of the curve 
// near that control point. 


// lambert@cse.unsw.edu.au
// If you want to learn more about the mathematics behind all of this, 
// I recommend An Introduction to Splines for Use in Computer Graphics and 
// Geometric Modelling by Bartels, Beatty and Barsky published by Morgan Kaufman, Los Altos, CA, 1987. 

package org.spectrim.tdfdfit;


public class CatmullRomSpline extends Bspline 
{

    // Catmull-Rom spline is just like a B spline, only with a different basis
    @Override
    float b( int i, float t ) 
    {
        switch ( i ) 
        {
            case -2:
                        return ( ( -t + 2 ) * t - 1 ) * t / 2;
            case -1:
                        return ( ( ( 3 * t - 5 ) * t ) * t + 2 ) / 2;
            case 0:
                        return ( ( -3 * t + 4 ) * t + 1 ) * t / 2;
            case 1:
                        return ( ( t - 1 ) * t * t ) / 2;
        }
        return 0; //we only get here if an invalid i is specified
    }
  
}
