package org.spectrim.tdfdfit;

/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

/**
 * Container class to maintaining fitting parameter ranges.  
 */
public class DefaultFittingParameterRangeValues 
{
    /**
     * Default constructor of DefaultFittingParameterRangeValues. 
     */
    private DefaultFittingParameterRangeValues( ) 
    {
        MRSignalProperties sigProps = MRSignalProperties.getInstance( );
        switch ( sigProps.getNucleusType( ) )
        {
            case MRSignalProperties.PROTON: getProtonDefaults( ); break;
            case MRSignalProperties.CARBON13: setCarbonDefaults( ); break;
            case MRSignalProperties.PHOSPHORUS31: setPhosphorusDefaults( ); break;
            case MRSignalProperties.SODIUM23: setSodiumDefaults( ); break;
            default: getProtonDefaults( );
        }
    }
    
    /**
     * Enables accessing the singleton class.
     * @return 
     */
    public static DefaultFittingParameterRangeValues getInstance( )
    {
        //Hack: set instance to null to enforce reloading!
        theInstance = null;
        if ( theInstance == null )
        {
            theInstance = new DefaultFittingParameterRangeValues( );
        }
        return theInstance;
    }
    
    /**
     * Sets the proton default values which where stored before.
     */
    private void setProtonDefaults( )
    {
        TDFDFittingDefaults defs = TDFDFittingDefaults.getInstance( );
        defs.load( );
        if ( defs.getProperty( TDFDFittingDefaults.IsAreaMaxDeterminedByFactor_1H ) != null )
        {
            String areaMaxDeterminedByFactor = defs.getProperty( TDFDFittingDefaults.IsAreaMaxDeterminedByFactor_1H );
            if ( areaMaxDeterminedByFactor.compareTo( "yes" ) == 0 )
            {
                m_isAreaMaxDeterminedByFactor = true;
            }
            else
            {
                m_isAreaMaxDeterminedByFactor = false;
            }
        }
        else
        {
            defs.setProperty( TDFDFittingDefaults.IsAreaMaxDeterminedByFactor_1H, "yes" );
            defs.save( );
        }
        
        if ( defs.getProperty( TDFDFittingDefaults.AreaMaxFactorDecimalValue_1H ) != null )
        {
            m_areaMaxFactorDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.AreaMaxFactorDecimalValue_1H ) ).floatValue( );
        }
        else
        {
            m_areaMaxFactorDecimalValue = m_areaMaxFactorDecimalValue_1H;
            defs.setProperty( TDFDFittingDefaults.AreaMaxFactorDecimalValue_1H, "" + m_areaMaxFactorDecimalValue );
        }
        ////////////////////
        if ( defs.getProperty( TDFDFittingDefaults.AreaMaxValDecimalValue_1H ) != null )
        {
            m_areaMaxValDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.AreaMaxValDecimalValue_1H ) ).floatValue( );
        }
        else
        {
            m_areaMaxValDecimalValue = m_areaMaxValDecimalValue_1H;
            defs.setProperty( TDFDFittingDefaults.AreaMaxValDecimalValue_1H, "" + m_areaMaxValDecimalValue );
        }
        ////////////////////
        if ( defs.getProperty( TDFDFittingDefaults.AreaMinValDecimalValue_1H ) != null )
        {
            m_areaMinValDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.AreaMinValDecimalValue_1H ) ).floatValue( );
        }
        else
        {
            m_areaMinValDecimalValue = m_areaMinValDecimalValue_1H;
            defs.setProperty( TDFDFittingDefaults.AreaMinValDecimalValue_1H, "" + m_areaMinValDecimalValue );
        }
        ////////////////////
        if ( defs.getProperty( TDFDFittingDefaults.GaussianWidthMaxValDecimalValue_1H ) != null )
        {
            m_gaussianWidthMaxValDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.GaussianWidthMaxValDecimalValue_1H ) ).floatValue( );
        }
        else
        {
            m_gaussianWidthMaxValDecimalValue = m_gaussianWidthMaxValDecimalValue_1H;
            defs.setProperty( TDFDFittingDefaults.GaussianWidthMaxValDecimalValue_1H, "" + m_gaussianWidthMaxValDecimalValue );
        }
        ////////////////////
        if ( defs.getProperty( TDFDFittingDefaults.GaussianWidthMinValDecimalValue_1H ) != null )
        {
            m_gaussianWidthMinValDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.GaussianWidthMinValDecimalValue_1H ) ).floatValue( );
        }
        else
        {
            m_gaussianWidthMinValDecimalValue = m_gaussianWidthMinValDecimalValue_1H;
            defs.setProperty( TDFDFittingDefaults.GaussianWidthMinValDecimalValue_1H, "" + m_gaussianWidthMinValDecimalValue );
        }
        ////////////////////
        if ( defs.getProperty( TDFDFittingDefaults.LorentzianWidthMaxValDecimalValue_1H ) != null )
        {
            m_lorentzianWidthMaxValDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.LorentzianWidthMaxValDecimalValue_1H ) ).floatValue( );
     
        }
        else
        {
            m_lorentzianWidthMaxValDecimalValue = m_lorentzianWidthMaxValDecimalValue_1H;
            defs.setProperty( TDFDFittingDefaults.LorentzianWidthMaxValDecimalValue_1H, "" + m_lorentzianWidthMaxValDecimalValue );
        }
        ////////////////////
        if ( defs.getProperty( TDFDFittingDefaults.LorentzianWidthMinValDecimalValue_1H ) != null )
        {
               m_lorentzianWidthMinValDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.LorentzianWidthMinValDecimalValue_1H ) ).floatValue( );
        }
        else
        {
            m_lorentzianWidthMinValDecimalValue = m_lorentzianWidthMinValDecimalValue_1H;
            defs.setProperty( TDFDFittingDefaults.LorentzianWidthMinValDecimalValue_1H, "" + m_lorentzianWidthMinValDecimalValue );
        }
        ////////////////////
        if ( defs.getProperty( TDFDFittingDefaults.OffsetMaxValDecimalValue_1H ) != null )
        {
            m_offsetMaxValDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.OffsetMaxValDecimalValue_1H ) ).floatValue( );
        }
        else
        {
            m_offsetMaxValDecimalValue = m_offsetMaxValDecimalValue_1H;
            defs.setProperty( TDFDFittingDefaults.OffsetMaxValDecimalValue_1H, "" + m_offsetMaxValDecimalValue );
        }
        ////////////////////
        if ( defs.getProperty( TDFDFittingDefaults.OffsetMinValDecimalValue_1H ) != null )
        {
            m_offsetMinValDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.OffsetMinValDecimalValue_1H ) ).floatValue( );
        }
        else
        {
            m_offsetMinValDecimalValue = m_offsetMinValDecimalValue_1H;
            defs.setProperty( TDFDFittingDefaults.OffsetMinValDecimalValue_1H, "" + m_offsetMinValDecimalValue );
        }
        ////////////////////       
        if ( defs.getProperty( TDFDFittingDefaults.PhaseMaxValDecimalValue_1H ) != null )
        {
            m_phaseMaxValDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.PhaseMaxValDecimalValue_1H ) ).floatValue( );
        }
        else
        {
            m_phaseMaxValDecimalValue = m_phaseMaxValDecimalValue_1H;
            defs.setProperty( TDFDFittingDefaults.PhaseMaxValDecimalValue_1H, "" + m_phaseMaxValDecimalValue );
        }
        ////////////////////
        if ( defs.getProperty( TDFDFittingDefaults.PhaseMinValDecimalValue_1H ) != null )
        {
            m_phaseMinValDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.PhaseMinValDecimalValue_1H ) ).floatValue( );
        }
        else
        {
            m_phaseMinValDecimalValue = m_phaseMinValDecimalValue_1H;
            defs.setProperty( TDFDFittingDefaults.PhaseMinValDecimalValue_1H, "" + m_phaseMinValDecimalValue );
        }
        
        // save the new values into the file!
        defs.save( );
    }
    
    /**
     * Reads the proton default values which where stored before.
     */
    private void getProtonDefaults( )
    {
        TDFDFittingDefaults defs = TDFDFittingDefaults.getInstance( );
        defs.load( );
        
        if ( defs.getProperty( TDFDFittingDefaults.IsAreaMaxDeterminedByFactor_1H ) != null )
        {
            String areaMaxDeterminedByFactor = defs.getProperty( TDFDFittingDefaults.IsAreaMaxDeterminedByFactor_1H );
            if ( areaMaxDeterminedByFactor.compareTo( "yes" ) == 0 )
            {
                m_isAreaMaxDeterminedByFactor = true;
            }
            else
            {
                m_isAreaMaxDeterminedByFactor = false;
            }
        }
        else
        {
            defs.setProperty( TDFDFittingDefaults.IsAreaMaxDeterminedByFactor_1H, "yes" );
        }
        
        if ( defs.getProperty( TDFDFittingDefaults.AreaMaxFactorDecimalValue_1H ) != null )
        {
            m_areaMaxFactorDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.AreaMaxFactorDecimalValue_1H ) ).floatValue( );
        }
        else
        {
            m_areaMaxFactorDecimalValue = m_areaMaxFactorDecimalValue_1H;
            defs.setProperty( TDFDFittingDefaults.AreaMaxFactorDecimalValue_1H, "" + m_areaMaxFactorDecimalValue );
        }
        ////////////////////
        if ( defs.getProperty( TDFDFittingDefaults.AreaMaxValDecimalValue_1H ) != null )
        {
            m_areaMaxValDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.AreaMaxValDecimalValue_1H ) ).floatValue( );
        }
        else
        {
            m_areaMaxValDecimalValue = m_areaMaxValDecimalValue_1H;
            defs.setProperty( TDFDFittingDefaults.AreaMaxValDecimalValue_1H, "" + m_areaMaxValDecimalValue );
        }
        ////////////////////
        if ( defs.getProperty( TDFDFittingDefaults.AreaMinValDecimalValue_1H ) != null )
        {
            m_areaMinValDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.AreaMinValDecimalValue_1H ) ).floatValue( );
        }
        else
        {
            m_areaMinValDecimalValue = m_areaMinValDecimalValue_1H;
            defs.setProperty( TDFDFittingDefaults.AreaMinValDecimalValue_1H, "" + m_areaMinValDecimalValue );
        }
        ////////////////////
        if ( defs.getProperty( TDFDFittingDefaults.GaussianWidthMaxValDecimalValue_1H ) != null )
        {
            m_gaussianWidthMaxValDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.GaussianWidthMaxValDecimalValue_1H ) ).floatValue( );
        }
        else
        {
            m_gaussianWidthMaxValDecimalValue = m_gaussianWidthMaxValDecimalValue_1H;
            defs.setProperty( TDFDFittingDefaults.GaussianWidthMaxValDecimalValue_1H, "" + m_gaussianWidthMaxValDecimalValue );
        }
        ////////////////////
        if ( defs.getProperty( TDFDFittingDefaults.GaussianWidthMinValDecimalValue_1H ) != null )
        {
            m_gaussianWidthMinValDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.GaussianWidthMinValDecimalValue_1H ) ).floatValue( );
        }
        else
        {
            m_gaussianWidthMinValDecimalValue = m_gaussianWidthMinValDecimalValue_1H;
            defs.setProperty( TDFDFittingDefaults.GaussianWidthMinValDecimalValue_1H, "" + m_gaussianWidthMinValDecimalValue );
        }
        ////////////////////
        if ( defs.getProperty( TDFDFittingDefaults.LorentzianWidthMaxValDecimalValue_1H ) != null )
        {
            m_lorentzianWidthMaxValDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.LorentzianWidthMaxValDecimalValue_1H ) ).floatValue( );
        }
        else
        {
            m_lorentzianWidthMaxValDecimalValue = m_lorentzianWidthMaxValDecimalValue_1H;
            defs.setProperty( TDFDFittingDefaults.LorentzianWidthMaxValDecimalValue_1H, "" + m_lorentzianWidthMaxValDecimalValue );
        }
        ////////////////////
        if ( defs.getProperty( TDFDFittingDefaults.LorentzianWidthMinValDecimalValue_1H ) != null )
        {
               m_lorentzianWidthMinValDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.LorentzianWidthMinValDecimalValue_1H ) ).floatValue( );
        }
        else
        {
            m_lorentzianWidthMinValDecimalValue = m_lorentzianWidthMinValDecimalValue_1H;
            defs.setProperty( TDFDFittingDefaults.LorentzianWidthMinValDecimalValue_1H, "" + m_lorentzianWidthMinValDecimalValue );
        }
        ////////////////////
        if ( defs.getProperty( TDFDFittingDefaults.OffsetMaxValDecimalValue_1H ) != null )
        {
            m_offsetMaxValDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.OffsetMaxValDecimalValue_1H ) ).floatValue( );
        }
        else
        {
            m_offsetMaxValDecimalValue = m_offsetMaxValDecimalValue_1H;
            defs.setProperty( TDFDFittingDefaults.OffsetMaxValDecimalValue_1H, "" + m_offsetMaxValDecimalValue );
        }
        ////////////////////
        if ( defs.getProperty( TDFDFittingDefaults.OffsetMinValDecimalValue_1H ) != null )
        {
            m_offsetMinValDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.OffsetMinValDecimalValue_1H ) ).floatValue( );
        }
        else
        {
            m_offsetMinValDecimalValue = m_offsetMinValDecimalValue_1H;
            defs.setProperty( TDFDFittingDefaults.OffsetMinValDecimalValue_1H, "" + m_offsetMinValDecimalValue );
        }
        ////////////////////       
        if ( defs.getProperty( TDFDFittingDefaults.PhaseMaxValDecimalValue_1H ) != null )
        {
            m_phaseMaxValDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.PhaseMaxValDecimalValue_1H ) ).floatValue( );
        }
        else
        {
            m_phaseMaxValDecimalValue = m_phaseMaxValDecimalValue_1H;
            defs.setProperty( TDFDFittingDefaults.PhaseMaxValDecimalValue_1H, "" + m_phaseMaxValDecimalValue );
        }
        ////////////////////
        if ( defs.getProperty( TDFDFittingDefaults.PhaseMinValDecimalValue_1H ) != null )
        {
            m_phaseMinValDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.PhaseMinValDecimalValue_1H ) ).floatValue( );
        }
        else
        {
            m_phaseMinValDecimalValue = m_phaseMinValDecimalValue_1H;
            defs.setProperty( TDFDFittingDefaults.PhaseMinValDecimalValue_1H, "" + m_phaseMinValDecimalValue );
        }
        
    }
    
    /**
     * Sets the carbon default values which were stored before.
     */
    private void setCarbonDefaults( )
    {
        TDFDFittingDefaults defs = TDFDFittingDefaults.getInstance( );
        defs.load( );
        
        if ( defs.getProperty( TDFDFittingDefaults.IsAreaMaxDeterminedByFactor_13C ).compareTo( "yes" ) == 0 )
        {
            m_isAreaMaxDeterminedByFactor = true;
        }
        else
        {
            m_isAreaMaxDeterminedByFactor = false;
        }
        
        if ( defs.getProperty( TDFDFittingDefaults.AreaMaxFactorDecimalValue_13C ) != null )
        {
            m_areaMaxFactorDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.AreaMaxFactorDecimalValue_13C ) ).floatValue( );
        }
        else
        {
            m_areaMaxFactorDecimalValue = m_areaMaxFactorDecimalValue_13C;
            defs.setProperty( TDFDFittingDefaults.AreaMaxFactorDecimalValue_13C, "" + m_areaMaxFactorDecimalValue );
        }
        ////////////////////
        if ( defs.getProperty( TDFDFittingDefaults.AreaMaxValDecimalValue_13C ) != null )
        {
            m_areaMaxValDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.AreaMaxValDecimalValue_13C ) ).floatValue( );
        }
        else
        {
            m_areaMaxValDecimalValue = m_areaMaxValDecimalValue_13C;
            defs.setProperty( TDFDFittingDefaults.AreaMaxValDecimalValue_13C, "" + m_areaMaxValDecimalValue );
        }
        ////////////////////
        if ( defs.getProperty( TDFDFittingDefaults.AreaMinValDecimalValue_13C ) != null )
        {
            m_areaMinValDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.AreaMinValDecimalValue_13C ) ).floatValue( );
        }
        else
        {
            m_areaMinValDecimalValue = m_areaMinValDecimalValue_13C;
            defs.setProperty( TDFDFittingDefaults.AreaMinValDecimalValue_13C, "" + m_areaMinValDecimalValue );
        }
        ////////////////////
        if ( defs.getProperty( TDFDFittingDefaults.GaussianWidthMaxValDecimalValue_13C ) != null )
        {
            m_gaussianWidthMaxValDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.GaussianWidthMaxValDecimalValue_13C ) ).floatValue( );
        }
        else
        {
            m_gaussianWidthMaxValDecimalValue = m_gaussianWidthMaxValDecimalValue_13C;
            defs.setProperty( TDFDFittingDefaults.GaussianWidthMaxValDecimalValue_13C, "" + m_gaussianWidthMaxValDecimalValue );
        }
        ////////////////////
        if ( defs.getProperty( TDFDFittingDefaults.GaussianWidthMinValDecimalValue_13C ) != null )
        {
            m_gaussianWidthMinValDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.GaussianWidthMinValDecimalValue_13C ) ).floatValue( );
        }
        else
        {
            m_gaussianWidthMinValDecimalValue = m_gaussianWidthMinValDecimalValue_13C;
            defs.setProperty( TDFDFittingDefaults.GaussianWidthMinValDecimalValue_13C, "" + m_gaussianWidthMinValDecimalValue );
        }
        ////////////////////
        if ( defs.getProperty( TDFDFittingDefaults.LorentzianWidthMaxValDecimalValue_13C ) != null )
        {
            m_lorentzianWidthMaxValDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.LorentzianWidthMaxValDecimalValue_13C ) ).floatValue( );
     
        }
        else
        {
            m_lorentzianWidthMaxValDecimalValue = m_lorentzianWidthMaxValDecimalValue_13C;
            defs.setProperty( TDFDFittingDefaults.LorentzianWidthMaxValDecimalValue_13C, "" + m_lorentzianWidthMaxValDecimalValue );
        }
        ////////////////////
        if ( defs.getProperty( TDFDFittingDefaults.LorentzianWidthMinValDecimalValue_13C ) != null )
        {
               m_lorentzianWidthMinValDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.LorentzianWidthMinValDecimalValue_13C ) ).floatValue( );
        }
        else
        {
            m_lorentzianWidthMinValDecimalValue = m_lorentzianWidthMinValDecimalValue_13C;
            defs.setProperty( TDFDFittingDefaults.LorentzianWidthMinValDecimalValue_13C, "" + m_lorentzianWidthMinValDecimalValue );
        }
        ////////////////////
        if ( defs.getProperty( TDFDFittingDefaults.OffsetMaxValDecimalValue_13C ) != null )
        {
            m_offsetMaxValDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.OffsetMaxValDecimalValue_13C ) ).floatValue( );
        }
        else
        {
            m_offsetMaxValDecimalValue = m_offsetMaxValDecimalValue_13C;
            defs.setProperty( TDFDFittingDefaults.OffsetMaxValDecimalValue_13C, "" + m_offsetMaxValDecimalValue );
        }
        ////////////////////
        if ( defs.getProperty( TDFDFittingDefaults.OffsetMinValDecimalValue_13C ) != null )
        {
            m_offsetMinValDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.OffsetMinValDecimalValue_13C ) ).floatValue( );
        }
        else
        {
            m_offsetMinValDecimalValue = m_offsetMinValDecimalValue_13C;
            defs.setProperty( TDFDFittingDefaults.OffsetMinValDecimalValue_13C, "" + m_offsetMinValDecimalValue );
        }
        ////////////////////       
        if ( defs.getProperty( TDFDFittingDefaults.PhaseMaxValDecimalValue_13C ) != null )
        {
            m_phaseMaxValDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.PhaseMaxValDecimalValue_13C ) ).floatValue( );
        }
        else
        {
            m_phaseMaxValDecimalValue = m_phaseMaxValDecimalValue_13C;
            defs.setProperty( TDFDFittingDefaults.PhaseMaxValDecimalValue_13C, "" + m_phaseMaxValDecimalValue );
        }
        ////////////////////
        if ( defs.getProperty( TDFDFittingDefaults.PhaseMinValDecimalValue_13C ) != null )
        {
            m_phaseMinValDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.PhaseMinValDecimalValue_13C ) ).floatValue( );
        }
        else
        {
            m_phaseMinValDecimalValue = m_phaseMinValDecimalValue_13C;
            defs.setProperty( TDFDFittingDefaults.PhaseMinValDecimalValue_13C, "" + m_phaseMinValDecimalValue );
        }
        // save the new values into the file!
        defs.save( );
    }
    
    /**
     * Sets the phosphorus default values which where stored before.
     */
    private void setPhosphorusDefaults( )
    {
        TDFDFittingDefaults defs = TDFDFittingDefaults.getInstance( );
        defs.load( );
        
        if ( defs.getProperty( TDFDFittingDefaults.IsAreaMaxDeterminedByFactor_31P ).compareTo( "yes" ) == 0 )
        {
            m_isAreaMaxDeterminedByFactor = true;
        }
        else
        {
            m_isAreaMaxDeterminedByFactor = false;
        }
        
        if ( defs.getProperty( TDFDFittingDefaults.AreaMaxFactorDecimalValue_31P ) != null )
        {
            m_areaMaxFactorDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.AreaMaxFactorDecimalValue_31P ) ).floatValue( );
        }
        else
        {
            m_areaMaxFactorDecimalValue = m_areaMaxFactorDecimalValue_31P;
            defs.setProperty( TDFDFittingDefaults.AreaMaxFactorDecimalValue_31P, "" + m_areaMaxFactorDecimalValue );
        }
        ////////////////////
        if ( defs.getProperty( TDFDFittingDefaults.AreaMaxValDecimalValue_31P ) != null )
        {
            m_areaMaxValDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.AreaMaxValDecimalValue_31P ) ).floatValue( );
        }
        else
        {
            m_areaMaxValDecimalValue = m_areaMaxValDecimalValue_31P;
            defs.setProperty( TDFDFittingDefaults.AreaMaxValDecimalValue_31P, "" + m_areaMaxValDecimalValue );
        }
        ////////////////////
        if ( defs.getProperty( TDFDFittingDefaults.AreaMinValDecimalValue_31P ) != null )
        {
            m_areaMinValDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.AreaMinValDecimalValue_31P ) ).floatValue( );
        }
        else
        {
            m_areaMinValDecimalValue = m_areaMinValDecimalValue_31P;
            defs.setProperty( TDFDFittingDefaults.AreaMinValDecimalValue_31P, "" + m_areaMinValDecimalValue );
        }
        ////////////////////
        if ( defs.getProperty( TDFDFittingDefaults.GaussianWidthMaxValDecimalValue_31P ) != null )
        {
            m_gaussianWidthMaxValDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.GaussianWidthMaxValDecimalValue_31P ) ).floatValue( );
        }
        else
        {
            m_gaussianWidthMaxValDecimalValue = m_gaussianWidthMaxValDecimalValue_31P;
            defs.setProperty( TDFDFittingDefaults.GaussianWidthMaxValDecimalValue_31P, "" + m_gaussianWidthMaxValDecimalValue );
        }
        ////////////////////
        if ( defs.getProperty( TDFDFittingDefaults.GaussianWidthMinValDecimalValue_31P ) != null )
        {
            m_gaussianWidthMinValDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.GaussianWidthMinValDecimalValue_31P ) ).floatValue( );
        }
        else
        {
            m_gaussianWidthMinValDecimalValue = m_gaussianWidthMinValDecimalValue_31P;
            defs.setProperty( TDFDFittingDefaults.GaussianWidthMinValDecimalValue_31P, "" + m_gaussianWidthMinValDecimalValue );
        }
        ////////////////////
        if ( defs.getProperty( TDFDFittingDefaults.LorentzianWidthMaxValDecimalValue_31P ) != null )
        {
            m_lorentzianWidthMaxValDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.LorentzianWidthMaxValDecimalValue_31P ) ).floatValue( );
     
        }
        else
        {
            m_lorentzianWidthMaxValDecimalValue = m_lorentzianWidthMaxValDecimalValue_31P;
            defs.setProperty( TDFDFittingDefaults.LorentzianWidthMaxValDecimalValue_31P, "" + m_lorentzianWidthMaxValDecimalValue );
        }
        ////////////////////
        if ( defs.getProperty( TDFDFittingDefaults.LorentzianWidthMinValDecimalValue_31P ) != null )
        {
               m_lorentzianWidthMinValDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.LorentzianWidthMinValDecimalValue_31P ) ).floatValue( );
        }
        else
        {
            m_lorentzianWidthMinValDecimalValue = m_lorentzianWidthMinValDecimalValue_31P;
            defs.setProperty( TDFDFittingDefaults.LorentzianWidthMinValDecimalValue_31P, "" + m_lorentzianWidthMinValDecimalValue );
        }
        ////////////////////
        if ( defs.getProperty( TDFDFittingDefaults.OffsetMaxValDecimalValue_31P ) != null )
        {
            m_offsetMaxValDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.OffsetMaxValDecimalValue_31P ) ).floatValue( );
        }
        else
        {
            m_offsetMaxValDecimalValue = m_offsetMaxValDecimalValue_31P;
            defs.setProperty( TDFDFittingDefaults.OffsetMaxValDecimalValue_31P, "" + m_offsetMaxValDecimalValue );
        }
        ////////////////////
        if ( defs.getProperty( TDFDFittingDefaults.OffsetMinValDecimalValue_31P ) != null )
        {
            m_offsetMinValDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.OffsetMinValDecimalValue_31P ) ).floatValue( );
        }
        else
        {
            m_offsetMinValDecimalValue = m_offsetMinValDecimalValue_31P;
            defs.setProperty( TDFDFittingDefaults.OffsetMinValDecimalValue_31P, "" + m_offsetMinValDecimalValue );
        }
        ////////////////////       
        if ( defs.getProperty( TDFDFittingDefaults.PhaseMaxValDecimalValue_31P ) != null )
        {
            m_phaseMaxValDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.PhaseMaxValDecimalValue_31P ) ).floatValue( );
        }
        else
        {
            m_phaseMaxValDecimalValue = m_phaseMaxValDecimalValue_31P;
            defs.setProperty( TDFDFittingDefaults.PhaseMaxValDecimalValue_31P, "" + m_phaseMaxValDecimalValue );
        }
        ////////////////////
        if ( defs.getProperty( TDFDFittingDefaults.PhaseMinValDecimalValue_31P ) != null )
        {
            m_phaseMinValDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.PhaseMinValDecimalValue_31P ) ).floatValue( );
        }
        else
        {
            m_phaseMinValDecimalValue = m_phaseMinValDecimalValue_31P;
            defs.setProperty( TDFDFittingDefaults.PhaseMinValDecimalValue_31P, "" + m_phaseMinValDecimalValue );
        }
        // save the new values into the file!
        defs.save( );
    }
    
    /**
     * Sets the sodium default values which where stored before.
     */
    private void setSodiumDefaults( )
    {
        TDFDFittingDefaults defs = TDFDFittingDefaults.getInstance( );
        defs.load( );
        
        if ( defs.getProperty( TDFDFittingDefaults.IsAreaMaxDeterminedByFactor_23Na ).compareTo( "yes" ) == 0 )
        {
            m_isAreaMaxDeterminedByFactor = true;
        }
        else
        {
            m_isAreaMaxDeterminedByFactor = false;
        }
        
        if ( defs.getProperty( TDFDFittingDefaults.AreaMaxFactorDecimalValue_23Na ) != null )
        {
            m_areaMaxFactorDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.AreaMaxFactorDecimalValue_23Na ) ).floatValue( );
        }
        else
        {
            m_areaMaxFactorDecimalValue = m_areaMaxFactorDecimalValue_23Na;
            defs.setProperty( TDFDFittingDefaults.AreaMaxFactorDecimalValue_23Na, "" + m_areaMaxFactorDecimalValue );
        }
        ////////////////////
        if ( defs.getProperty( TDFDFittingDefaults.AreaMaxValDecimalValue_23Na ) != null )
        {
            m_areaMaxValDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.AreaMaxValDecimalValue_23Na ) ).floatValue( );
        }
        else
        {
            m_areaMaxValDecimalValue = m_areaMaxValDecimalValue_23Na;
            defs.setProperty( TDFDFittingDefaults.AreaMaxValDecimalValue_23Na, "" + m_areaMaxValDecimalValue );
        }
        ////////////////////
        if ( defs.getProperty( TDFDFittingDefaults.AreaMinValDecimalValue_23Na ) != null )
        {
            m_areaMinValDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.AreaMinValDecimalValue_23Na ) ).floatValue( );
        }
        else
        {
            m_areaMinValDecimalValue = m_areaMinValDecimalValue_23Na;
            defs.setProperty( TDFDFittingDefaults.AreaMinValDecimalValue_23Na, "" + m_areaMinValDecimalValue );
        }
        ////////////////////
        if ( defs.getProperty( TDFDFittingDefaults.GaussianWidthMaxValDecimalValue_23Na ) != null )
        {
            m_gaussianWidthMaxValDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.GaussianWidthMaxValDecimalValue_23Na ) ).floatValue( );
        }
        else
        {
            m_gaussianWidthMaxValDecimalValue = m_gaussianWidthMaxValDecimalValue_23Na;
            defs.setProperty( TDFDFittingDefaults.GaussianWidthMaxValDecimalValue_23Na, "" + m_gaussianWidthMaxValDecimalValue );
        }
        ////////////////////
        if ( defs.getProperty( TDFDFittingDefaults.GaussianWidthMinValDecimalValue_23Na ) != null )
        {
            m_gaussianWidthMinValDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.GaussianWidthMinValDecimalValue_23Na ) ).floatValue( );
        }
        else
        {
            m_gaussianWidthMinValDecimalValue = m_gaussianWidthMinValDecimalValue_23Na;
            defs.setProperty( TDFDFittingDefaults.GaussianWidthMinValDecimalValue_23Na, "" + m_gaussianWidthMinValDecimalValue );
        }
        ////////////////////
        if ( defs.getProperty( TDFDFittingDefaults.LorentzianWidthMaxValDecimalValue_23Na ) != null )
        {
            m_lorentzianWidthMaxValDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.LorentzianWidthMaxValDecimalValue_23Na ) ).floatValue( );
     
        }
        else
        {
            m_lorentzianWidthMaxValDecimalValue = m_lorentzianWidthMaxValDecimalValue_23Na;
            defs.setProperty( TDFDFittingDefaults.LorentzianWidthMaxValDecimalValue_23Na, "" + m_lorentzianWidthMaxValDecimalValue );
        }
        ////////////////////
        if ( defs.getProperty( TDFDFittingDefaults.LorentzianWidthMinValDecimalValue_23Na ) != null )
        {
               m_lorentzianWidthMinValDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.LorentzianWidthMinValDecimalValue_23Na ) ).floatValue( );
        }
        else
        {
            m_lorentzianWidthMinValDecimalValue = m_lorentzianWidthMinValDecimalValue_23Na;
            defs.setProperty( TDFDFittingDefaults.LorentzianWidthMinValDecimalValue_23Na, "" + m_lorentzianWidthMinValDecimalValue );
        }
        ////////////////////
        if ( defs.getProperty( TDFDFittingDefaults.OffsetMaxValDecimalValue_23Na ) != null )
        {
            m_offsetMaxValDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.OffsetMaxValDecimalValue_23Na ) ).floatValue( );
        }
        else
        {
            m_offsetMaxValDecimalValue = m_offsetMaxValDecimalValue_23Na;
            defs.setProperty( TDFDFittingDefaults.OffsetMaxValDecimalValue_23Na, "" + m_offsetMaxValDecimalValue );
        }
        ////////////////////
        if ( defs.getProperty( TDFDFittingDefaults.OffsetMinValDecimalValue_23Na ) != null )
        {
            m_offsetMinValDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.OffsetMinValDecimalValue_23Na ) ).floatValue( );
        }
        else
        {
            m_offsetMinValDecimalValue = m_offsetMinValDecimalValue_23Na;
            defs.setProperty( TDFDFittingDefaults.OffsetMinValDecimalValue_23Na, "" + m_offsetMinValDecimalValue );
        }
        ////////////////////       
        if ( defs.getProperty( TDFDFittingDefaults.PhaseMaxValDecimalValue_23Na ) != null )
        {
            m_phaseMaxValDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.PhaseMaxValDecimalValue_23Na ) ).floatValue( );
        }
        else
        {
            m_phaseMaxValDecimalValue = m_phaseMaxValDecimalValue_23Na;
            defs.setProperty( TDFDFittingDefaults.PhaseMaxValDecimalValue_23Na, "" + m_phaseMaxValDecimalValue );
        }
        ////////////////////
        if ( defs.getProperty( TDFDFittingDefaults.PhaseMinValDecimalValue_23Na ) != null )
        {
            m_phaseMinValDecimalValue = new Double( defs.getProperty( TDFDFittingDefaults.PhaseMinValDecimalValue_23Na ) ).floatValue( );
        }
        else
        {
            m_phaseMinValDecimalValue = m_phaseMinValDecimalValue_23Na;
            defs.setProperty( TDFDFittingDefaults.PhaseMinValDecimalValue_23Na, "" + m_phaseMinValDecimalValue );
        }
        // save the new values into the file!
        defs.save( );
    }
    
    /**
     * Returns the maximum allowed factor of the fitting parameter for 
     * the area parameter.
     * @return 
     */
    public float getAreaMaxFactorDecimalValue( ) 
    {
        return m_areaMaxFactorDecimalValue;
    }
    
    /**
     * Returns the maximum allowed factor of the fitting parameter for 
     * the area parameter.
     * @param areaMaxFactorDecimalValue 
     */
    public void setAreaMaxFactorDecimalValue( float areaMaxFactorDecimalValue ) 
    {
        m_areaMaxFactorDecimalValue = areaMaxFactorDecimalValue;
    }
    
    /**
     * Returns the maximum tolerable area value.
     * @return 
     */
    public float getAreaMaxValDecimalValue( ) 
    {
        return m_areaMaxValDecimalValue;
    }
    
    /**
     * Sets the maximum tolerable area value.
     * @param m_areaMaxValDecimalValue 
     */
    public void setAreaMaxValDecimalValue( float areaMaxValDecimalValue ) 
    {
        m_areaMaxValDecimalValue = areaMaxValDecimalValue;
    }
    
    /**
     * Returns the minimal tolerable area value.
     * @return 
     */
    public float getAreaMinValDecimalValue( )
    {
        return m_areaMinValDecimalValue;
    }
    
   /**
    * Sets the minimal tolerable area value.
    * @param m_areaMinValDecimalValue 
    */
    public void setAreaMinValDecimalValue( float areaMinValDecimalValue ) 
    {
        m_areaMinValDecimalValue = areaMinValDecimalValue;
    }

    /**
     * Returns the maximum tolerable Gaussian width parameter value.
     * @return 
     */
    public float getGaussianWidthMaxValDecimalValue( ) 
    {
        return m_gaussianWidthMaxValDecimalValue;
    }
    
    /**
     * Sets the maximum tolerable Gaussian width parameter value.
     * @param m_gaussianWidthMaxValDecimalValue 
     */
    public void setGaussianWidthMaxValDecimalValue( float gaussianWidthMaxValDecimalValue ) 
    {
        m_gaussianWidthMaxValDecimalValue = gaussianWidthMaxValDecimalValue;
    }

    /**
     * Returns the minimum tolerable Gaussian width parameter value.
     * @return 
     */
    public float getGaussianWidthMinValDecimalValue( ) 
    {
        return m_gaussianWidthMinValDecimalValue;
    }
    
    /**
     * Sets the minimum tolerable Gaussian width parameter value.
     * @param m_gaussianWidthMinValDecimalValue 
     */
    public void setGaussianWidthMinValDecimalValue( float gaussianWidthMinValDecimalValue ) 
    {
        m_gaussianWidthMinValDecimalValue = gaussianWidthMinValDecimalValue;
    }

    /**
     * Returns the maximum tolerable Lorentzian width parameter.
     * @return 
     */
    public float getLorentzianWidthMaxValDecimalValue( ) 
    {
        return m_lorentzianWidthMaxValDecimalValue;
    }
    
    /**
     * Sets the maximum tolerable Lorentzian width parameter.
     * @param m_lorentzianWidthMaxValDecimalValue 
     */
    public void setLorentzianWidthMaxValDecimalValue( float lorentzianWidthMaxValDecimalValue ) 
    {
        m_lorentzianWidthMaxValDecimalValue = lorentzianWidthMaxValDecimalValue;
    }
    
    /**
     * Returns the minimum tolerable Lorentzian width parameter.
     * @return 
     */
    public float getLorentzianWidthMinValDecimalValue( ) 
    {
        return m_lorentzianWidthMinValDecimalValue;
    }
    
    /**
     * Sets the minimum tolerable Lorentzian width parameter. 
     * @param m_lorentzianWidthMinValDecimalValue
     */
    public void setLorentzianWidthMinValDecimalValue( float lorentzianWidthMinValDecimalValue ) 
    {
        m_lorentzianWidthMinValDecimalValue = lorentzianWidthMinValDecimalValue;
    }
   
    /**
     * Returns the maximum tolerable Offset width parameter.
     * @return 
     */
    public float getOffsetMaxValDecimalValue( ) 
    {
        return m_offsetMaxValDecimalValue;
    }
    
    /**
     * Sets the maximum tolerable Offset width parameter. 
     * @param offsetMaxValDecimalValue
     */
    public void set0ffsetMaxValDecimalValue( float offsetMaxValDecimalValue ) 
    {
        m_offsetMaxValDecimalValue = offsetMaxValDecimalValue;
    }
    
    /**
     * Returns the minimum tolerable offset parameter.
     * @return 
     */
    public float getOffsetMinValDecimalValue( ) 
    {
        return m_offsetMinValDecimalValue;
    }
    /**
     * Sets the minimum tolerable offset parameter. 
     * @param offsetMinValDecimalValue
     */
    public void setOffsetMinValDecimalValue( float offsetMinValDecimalValue ) 
    {
        m_offsetMinValDecimalValue = offsetMinValDecimalValue;
    }
    
    /**
     * Returns the maximum tolerable phase parameter.
     * @return 
     */
    public float getPhaseMaxValDecimalValue( ) 
    {
        return m_phaseMaxValDecimalValue;
    }
    
    /**
     * Sets the maximum tolerable phase parameter. 
     * @param phaseMaxValDecimalValue
     */
    public void setPhaseMaxValDecimalValue( float phaseMaxValDecimalValue ) 
    {
        m_phaseMaxValDecimalValue = phaseMaxValDecimalValue;
    }
    
    /**
     * Returns the minimum tolerable phase parameter.
     * @return 
     */
    public float getPhaseMinValDecimalValue( ) 
    {
        return m_phaseMinValDecimalValue;
    }
    
    /**
     * Sets the minimum tolerable phase parameter. 
     * @param phaseMinValDecimalValue
     */
    public void setPhaseMinValDecimalValue( float phaseMinValDecimalValue ) 
    {
        m_phaseMinValDecimalValue = phaseMinValDecimalValue;
    }
    
    /**
     * Returns whether the maximum area parameter is determined by a factor. 
     * @return 
     */
    public boolean isAreaMaxDeterminedByFactor( ) 
    {
        return m_isAreaMaxDeterminedByFactor;
    }
    
    /**
     * Defines whether the area max tolerable amplitude is determined by 
     * a user set factor.
     * @param determineByFactor
     */
    public void setAreaMaxIsDeterminedByFactor( boolean determineByFactor )
    {
        m_isAreaMaxDeterminedByFactor = determineByFactor;
    }
    
    // Defaults for currently selected nucleus type
    private boolean m_isAreaMaxDeterminedByFactor;
    
    private float m_areaMaxFactorDecimalValue;

    private float m_areaMaxValDecimalValue;
    private float m_areaMinValDecimalValue;
    
    private float m_gaussianWidthMaxValDecimalValue;
    private float m_gaussianWidthMinValDecimalValue;
    
    private float m_lorentzianWidthMaxValDecimalValue;
    private float m_lorentzianWidthMinValDecimalValue;
    
    private float m_offsetMaxValDecimalValue;

    private float m_offsetMinValDecimalValue;
    
    private float m_phaseMaxValDecimalValue;
    private float m_phaseMinValDecimalValue;
    
    // Protons
    private float m_areaMaxFactorDecimalValue_1H = 5f;
    
    private float m_areaMaxValDecimalValue_1H = 10000f;
    private float m_areaMinValDecimalValue_1H = 0f;
    
    private float m_gaussianWidthMaxValDecimalValue_1H = 20f;
    private float m_gaussianWidthMinValDecimalValue_1H = 0f;
    
    private float m_lorentzianWidthMaxValDecimalValue_1H = 20f;
    private float m_lorentzianWidthMinValDecimalValue_1H = 0f;
    
    private float m_offsetMaxValDecimalValue_1H =  5f;
    private float m_offsetMinValDecimalValue_1H = -5f;
    
    private float m_phaseMaxValDecimalValue_1H = 25f;
    private float m_phaseMinValDecimalValue_1H = -25f;
    
    /////////////////////////////////////////////
    
    // Phosphorus
    private float m_areaMaxFactorDecimalValue_31P = 5f;
    
    private float m_areaMaxValDecimalValue_31P = 10000f;
    private float m_areaMinValDecimalValue_31P = 0f;
    
    private float m_gaussianWidthMaxValDecimalValue_31P = 20f;
    private float m_gaussianWidthMinValDecimalValue_31P = 0f;
    
    private float m_lorentzianWidthMaxValDecimalValue_31P = 20f;
    private float m_lorentzianWidthMinValDecimalValue_31P = 0f;
    
    private float m_offsetMaxValDecimalValue_31P = -5f;
    private float m_offsetMinValDecimalValue_31P = 5f;
    
    private float m_phaseMaxValDecimalValue_31P = -25f;
    private float m_phaseMinValDecimalValue_31P = 25f;
    /////////////////////////////////////////////

    // Carbon 13 
    private float m_areaMaxFactorDecimalValue_13C = 5f;
    
    private float m_areaMaxValDecimalValue_13C = 10000f;
    private float m_areaMinValDecimalValue_13C = 0f;
    
    private float m_gaussianWidthMaxValDecimalValue_13C = 20f;
    private float m_gaussianWidthMinValDecimalValue_13C = 0f;
    
    private float m_lorentzianWidthMaxValDecimalValue_13C = 20f;
    private float m_lorentzianWidthMinValDecimalValue_13C = 0f;
    
    private float m_offsetMaxValDecimalValue_13C = -5f;
    private float m_offsetMinValDecimalValue_13C = 5f;
    
    private float m_phaseMaxValDecimalValue_13C = -25f;
    private float m_phaseMinValDecimalValue_13C = 25f;
    
    /////////////////////////////////////////////
    
    // Sodium 23
    private float m_areaMaxFactorDecimalValue_23Na = 5f;
    
    private float m_areaMaxValDecimalValue_23Na = 10000f;
    private float m_areaMinValDecimalValue_23Na = 0f;
    
    private float m_gaussianWidthMaxValDecimalValue_23Na = 20f;
    private float m_gaussianWidthMinValDecimalValue_23Na = 0f;
    
    private float m_lorentzianWidthMaxValDecimalValue_23Na = 20f;
    private float m_lorentzianWidthMinValDecimalValue_23Na = 0f;
    
    private float m_offsetMaxValDecimalValue_23Na = -5f;
    private float m_offsetMinValDecimalValue_23Na = 5f;
    
    private float m_phaseMaxValDecimalValue_23Na = -25f;
    private float m_phaseMinValDecimalValue_23Na = 25f;
    /////////////////////////////////////////////
    private static DefaultFittingParameterRangeValues theInstance;

 
}
