/*
 * File: $Id: FileSystemSettingsPanel.java,v 1.1.1.1 2006/04/19 09:38:49 hans Exp $
 * History
 * Date: 03.12.2003, 3:21  Authors: Hans Slotboom
 *
 * Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

package org.spectrim.tdfdfit;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import java.awt.Dimension;
import java.awt.BorderLayout;

import java.io.File;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JFileChooser;
import javax.swing.JPanel;
import javax.swing.BoxLayout;
import javax.swing.BorderFactory;

/**
 * 
 * Allows diverse settings of file system related information.
 *
 * @author  hans
 */
public class FileSystemSettingsPanel extends JPanel implements ActionListener
{

    /** 
     * Creates a new instance of FileSystemSettingsPanel .
     */
    public FileSystemSettingsPanel( )
    {
        setLayout( new BorderLayout( ) );
        
        m_contentPanel = new JPanel( );
        
        m_contentPanel.setLayout( new BoxLayout( m_contentPanel, BoxLayout.Y_AXIS ) );
        
        m_contentPanel.setBorder( BorderFactory.createCompoundBorder( 
                                  BorderFactory.createTitledBorder( "Important file system paths" ),
                                  BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) ) );
               
        m_setPatientDatabaseRootPathPanel = new JPanel( );
        m_setPatientDatabaseRootPathPanel.setLayout( new BoxLayout( m_setPatientDatabaseRootPathPanel, BoxLayout.X_AXIS ) );
        
        m_tdfdftExcutableInstallationPathPanel = new JPanel( );
        m_tdfdftExcutableInstallationPathPanel.setLayout( new BoxLayout( m_tdfdftExcutableInstallationPathPanel, BoxLayout.X_AXIS ) );
        
        m_tdfdftExcutableInstallationPathLabel = new JLabel( "  TDFDFit installation path goes here" );
        
        m_tdfdftExcutableInstallationPathPanel.add( m_tdfdftExcutableInstallationPathLabelTitle );
        m_tdfdftExcutableInstallationPathPanel.add( Box.createHorizontalGlue( ) );
        m_tdfdftExcutableInstallationPathDefinitionButton = new JButton( "TDFDFit path .." );
        m_tdfdftExcutableInstallationPathDefinitionButton.setMaximumSize( new Dimension( 150, 25 ) );
        m_tdfdftExcutableInstallationPathDefinitionButton.setPreferredSize( new Dimension( 150, 25 ) );
        m_tdfdftExcutableInstallationPathDefinitionButton.setMinimumSize( new Dimension( 150, 25 ) );
        m_tdfdftExcutableInstallationPathDefinitionButton.addActionListener( this );
        m_tdfdftExcutableInstallationPathPanel.add( m_tdfdftExcutableInstallationPathDefinitionButton );
        m_tdfdftExcutableInstallationPathPanel.add( Box.createHorizontalGlue( ) );
        m_tdfdftExcutableInstallationPathPanel.add( m_tdfdftExcutableInstallationPathLabel );
        m_tdfdftExcutableInstallationPathPanel.add( Box.createHorizontalGlue( ) );

        // Patient database ..
        m_patientDatabaseRootPathLabel = new JLabel( "  default patient DB path goes here" );
        
        m_setPatientDatabaseRootPathPanel.add( m_patientDatabaseRootPathLabelTitle );
        m_setPatientDatabaseRootPathPanel.add( Box.createHorizontalGlue( ) );
        m_patientDatabaseRootPathDefinitionButton = new JButton( "DBase path .." );
        m_patientDatabaseRootPathDefinitionButton.setMaximumSize( new Dimension( 150, 25 ) );
        m_patientDatabaseRootPathDefinitionButton.setPreferredSize( new Dimension( 150, 25 ) );
        m_patientDatabaseRootPathDefinitionButton.setMinimumSize( new Dimension( 150, 25 ) ); 
        m_patientDatabaseRootPathDefinitionButton.addActionListener( this );
        m_setPatientDatabaseRootPathPanel.add( m_patientDatabaseRootPathDefinitionButton );
        
        m_setPatientDatabaseRootPathPanel.add( m_patientDatabaseRootPathLabel );
        m_setPatientDatabaseRootPathPanel.add( Box.createHorizontalGlue( ) );
        
        m_setPatientDatabaseRootPathPanel.add( Box.createHorizontalGlue( ) );
        
        m_contentPanel.add( m_setPatientDatabaseRootPathPanel );
        m_contentPanel.add( m_tdfdftExcutableInstallationPathPanel );
        
        m_contentPanel.setMaximumSize( m_contentPanel.getPreferredSize( ) );
        
        setMaximumSize( m_contentPanel.getPreferredSize( ) );
        
        add( m_contentPanel, BorderLayout.NORTH );
        
        TDFDFittingDefaults fdef = TDFDFittingDefaults.getInstance( );
        
        m_patientDatabaseRootPathLabel.setText( fdef.getProperty( TDFDFittingDefaults.PATIENT_DATABASE_ROOT_PATH, System.getProperty( "user.dir" ) ) );
        m_tdfdftExcutableInstallationPathLabel.setText( fdef.getProperty( TDFDFittingDefaults.TDFDFIT_INSTALLATION_PATH, System.getProperty( "user.dir" ) ) );
        setVisible( true );
    }
 
    /**
     * Method is called whenever an registered components raises an ActionEvent.
     * @param actionEvent
     */
    @Override
    public void actionPerformed( ActionEvent actionEvent ) 
    {
        Object sender = actionEvent.getSource( );
        if ( sender == this.m_patientDatabaseRootPathDefinitionButton )
        {
            this.definePatientDatabaseRootPath( );
        }
        if ( sender == this.m_tdfdftExcutableInstallationPathDefinitionButton )
        {
            this.defineTDFDFitInstallationPath( );
        }
        
    }
    
    /**
     * Defines the path in which the patient database is stored.
     */
    private File definePatientDatabaseRootPath( )
    {
        TDFDFittingDefaults fdef = TDFDFittingDefaults.getInstance( );
        fdef.load( );
        
        String defaultPathString = fdef.getProperty( TDFDFittingDefaults.PATIENT_DATABASE_ROOT_PATH );
        
        File defaultPatientDBRootPath;
        if ( defaultPathString != null )
        {
            defaultPatientDBRootPath= new File( defaultPathString );
            // check whether previous directory was deleted ; if yes set path back to
            // user.home directory
            if ( !defaultPatientDBRootPath.exists( ) )
            {
                defaultPathString = System.getProperty( "user.home" );
            }
        }
        else
        {
            defaultPathString = System.getProperty( "user.home" );
        }
        
        JFileChooser fileChooser = new JFileChooser( defaultPathString );
        
        fileChooser.setFileSelectionMode( JFileChooser.DIRECTORIES_ONLY );
        
        int result = fileChooser.showOpenDialog( this );
        
        File selected_dir = new File( "" );
        
        switch ( result )
        {
            case JFileChooser.APPROVE_OPTION:   // Approve Open button was pressed
                                                selected_dir = fileChooser.getSelectedFile( );
                                                
                                                TDFDFittingDefaults def = TDFDFittingDefaults.getInstance( );
                                                def.load( );
                                                def.setProperty( "Patient database root path", selected_dir.getPath( ) );
                                                m_patientDatabaseRootPathLabel.setText( selected_dir.getPath( ) );
                                                def.save( );
                                                if ( selected_dir != null )
                                                {
                                                    m_patientDBDirectorySet = true;
                                                }
            
                                                break;
           case JFileChooser.CANCEL_OPTION:     // Cancel or close dialog was selected
                                                break;
           case JFileChooser.ERROR_OPTION:      // the selected process did not terminate properly
                                                break;
        }
        
        return selected_dir;
    }
    
    
    /**
     * Defines the path where the TDFDFit executable is installed. 
     */
    private File defineTDFDFitInstallationPath( )
    {
        TDFDFittingDefaults fdef = TDFDFittingDefaults.getInstance( );
        fdef.load( );
        
        String defaultPathString = fdef.getProperty( TDFDFittingDefaults.TDFDFIT_INSTALLATION_PATH );
        
        File defaultPatientDBRootPath;
        
        if ( defaultPathString != null )
        {   
            defaultPatientDBRootPath = new File( defaultPathString );
        
            // check whether previous directory was deleted ; if yes set path back to
            // user.home directory
            if ( !defaultPatientDBRootPath.exists( ) )
            {
                defaultPathString = System.getProperty( "user.home" );
            }
        }
        else
        {
            defaultPathString = System.getProperty( "user.home" );
        }
        
        JFileChooser fileChooser = new JFileChooser( defaultPathString );
        
        fileChooser.setFileSelectionMode( JFileChooser.DIRECTORIES_ONLY );
        
        int result = fileChooser.showOpenDialog( this );
        
        File selected_dir = new File( "" );
        
        switch ( result )
        {
            case JFileChooser.APPROVE_OPTION:   // Approve Open button was pressed
                                                selected_dir = fileChooser.getSelectedFile( );
                                                
                                                TDFDFittingDefaults def = TDFDFittingDefaults.getInstance( );
                                                def.load( );
                                                def.setProperty( TDFDFittingDefaults.TDFDFIT_INSTALLATION_PATH, selected_dir.getPath( ) );
                                                m_tdfdftExcutableInstallationPathLabel.setText( selected_dir.getPath( ) );
                                                def.save( );
                                                if ( selected_dir != null )
                                                {
                                                    m_tdfdftExcutableInstallationPathSet = true;
                                                }
            
                                                break;
           case JFileChooser.CANCEL_OPTION:     // Cancel or close dialog was selected
                                                break;
           case JFileChooser.ERROR_OPTION:      // the selected process did not terminate properly
                                                break;
        }
        
        return selected_dir;
    }
    
    
    private final JPanel      m_contentPanel;
    private final JPanel      m_setPatientDatabaseRootPathPanel;
    private final JLabel      m_patientDatabaseRootPathLabelTitle = new JLabel( "Patient data base root path :  " );
    private final JLabel      m_patientDatabaseRootPathLabel;
    private final JButton     m_patientDatabaseRootPathDefinitionButton;
    
    private final JLabel      m_tdfdftExcutableInstallationPathLabelTitle = new JLabel( "TDFDFit installation path  : " );
    private final JLabel      m_tdfdftExcutableInstallationPathLabel;
    private final JPanel      m_tdfdftExcutableInstallationPathPanel;
    private final JButton     m_tdfdftExcutableInstallationPathDefinitionButton;
    
    private boolean m_patientDBDirectorySet = false;
    private boolean m_tdfdftExcutableInstallationPathSet = false;
    
    // For proper serialization
    private static final long serialVersionUID = 6482325998125123L;
    
}
