/*
 * File: $Id: FitResultSpectrumViewerPanel.java,v 1.1.1.1 2006/04/19 09:38:49 hans Exp $
 * History
 * Date: 08.07.2003, 09:18  Authors: Hans Slotboom
 *
 * Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

package org.spectrim.tdfdfit;

import org.spectrim.xyplot.XYPlotRange;
import java.awt.Color;
import java.awt.Point;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.Robot;
import java.awt.Dimension;

import java.awt.image.BufferedImage;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;

import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import java.awt.geom.Point2D;

import java.io.RandomAccessFile; 

import java.io.IOException;
import java.io.File;


import javax.swing.JFrame;
import javax.swing.JMenuItem;
import javax.swing.JMenu;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JFileChooser;
import javax.swing.tree.DefaultMutableTreeNode;

import javax.imageio.ImageIO;

import java.text.DecimalFormat;

import java.util.LinkedList; 

import java.awt.print.Book;
import java.awt.print.Printable;
import java.awt.print.PageFormat;
import java.awt.print.PrinterException;
import java.awt.print.PrinterJob;
import java.io.FileWriter;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Class that display the TDFDFit fit results.
 * @author  hans
 */
public class FitResultSpectrumViewerPanel extends SingleVoxelSpectrumViewer implements ActionListener, MouseListener, MouseMotionListener, Printable
{
    
    /** 
     * Creates a new instance of FitResultViewerPanel. 
     * @param title
     * @param numberOfPoints
     * @param fittingFileReference
     * @param parent
     * @param calledDirectlyAfterFitting
     */
    public FitResultSpectrumViewerPanel( String title, int numberOfPoints, 
                                         String fittingFileReference, JFrame parent, 
                                         boolean calledDirectlyAfterFitting ) 
    {
        super( title, parent, null );
        setTitle( title );
        m_fittingFileReference = fittingFileReference.trim( );
        m_numberOfPoints = numberOfPoints;
        
        addMouseListener( this );
        addMouseMotionListener( this );
        addPopupMenuItems( );
        
        m_calledDirectAfterPrototypeModelFitting = calledDirectlyAfterFitting;
        
        loadAndShowTDFDFitDataFiles( );
    }
    
    /**
     * Adds entry point for user interactive spectrum model editing.
     */
    private void addPopupMenuItems( )
    {
        m_printSpectrum = new JMenuItem( "Print" );
        m_printSpectrum.addActionListener( this );
        
        m_overlayNumericValueTypes = new JMenu( "Numeric Overlay Type" );
        m_displayAllSpectralComponents = new JRadioButtonMenuItem( "All components" );
        m_displayAllSpectralComponents.addActionListener( this );
        
        m_saveScreenShot = new JMenuItem( "Save screenshot .." );
        m_saveScreenShot.setToolTipText( "Saves a copy of the currently displayed spectrum to a file" );
        m_saveScreenShot.addActionListener( this );
        
        m_displayPatternsOnly = new JRadioButtonMenuItem( "Patterns only" );
        m_displayPatternsOnly.addActionListener( this );
        m_displayPatternsOnly.setSelected( true );
        
        m_overlayNumericValueTypes.add( m_displayAllSpectralComponents );
        m_overlayNumericValueTypes.add( m_displayPatternsOnly );
        
        m_exportCurvesOfSpectra = new JMenuItem( "Export curves in text file" );
        m_exportCurvesOfSpectra.addActionListener( this );
        
        m_popupmenu.add( m_overlayNumericValueTypes );
        m_popupmenu.add( m_printSpectrum );
        m_popupmenu.add( m_saveScreenShot );
        m_popupmenu.add( m_exportCurvesOfSpectra );
        
    }
      
    /**
     * Called whenever a registered component raises an ActionEvent.
     */
    @Override
    public void actionPerformed( ActionEvent e ) 
    {
        Object sender = e.getSource( );
        
        
        if ( sender == m_exportCurvesOfSpectra )
        {
            exportCurvesAsText( );
        }
        
        
        if ( sender == m_printSpectrum )
        {
             print( );
        }
        
        if ( sender == m_displayAllSpectralComponents )
        {
            m_displayMode = DISPLAY_ALL;
            readAndDisplayFitresultsInXYPlot( );
            refresh( );
        }
        
        if ( sender == m_displayPatternsOnly )
        {
            m_displayMode = PATTERNS_ONLY;
            readAndDisplayFitresultsInXYPlot( );
            refresh( );
        }
        
        if ( sender == m_saveScreenShot )
        {
            m_screenshotFileReference = getScreenShotFileReference( );
            
            m_popupmenu.setVisible( false );
            m_popupmenu.invalidate( );
            
            try
            {
                Dimension dim = this.getSize( );
                Point pos = this.getLocationOnScreen( );
                
                Rectangle screenRect = new Rectangle( pos.x, pos.y, dim.width, dim.height );
                
                Robot robot = new Robot( );
            
                try
                {
                    BufferedImage image = robot.createScreenCapture( screenRect ); 
                    // save an image to a PNG file
                    ImageIO.write( image, "png", m_screenshotFileReference );
                }
                catch( IOException ioe )
                {
                    System.err.println( "write: " + ioe.getMessage( ) );
                }
            }
            catch ( Exception exception )
            { 
                System.out.println( exception );
            }
        }
        
        super.actionPerformed( e );
    }
    
    /**
     * Called when the mouse button is pressed.
     */
    @Override
    public void mousePressed( MouseEvent mouseEvent ) 
    {
        // is only of interest if left button is pressed
        if ( (mouseEvent.getModifiers( ) & InputEvent.BUTTON1_MASK) == InputEvent.BUTTON1_MASK )
        {
            m_mouseButtonPressedX = mouseEvent.getX( );
            m_mouseButtonPressedY = mouseEvent.getY( );
        }
        
        super.mousePressed( mouseEvent );
    }
    
    /**
     * Called when the mouse button is pressed.
     */
    @Override
    public void mouseReleased( MouseEvent mouseEvent ) 
    {
        if ( m_mouseDragged == true )
        {
            m_mouseDragged = false;
            // is only of interest if left button is released
            if ( ( mouseEvent.getModifiers( ) & InputEvent.BUTTON1_MASK ) == InputEvent.BUTTON1_MASK )
            {
            
                m_mouseButtonReleasedX = mouseEvent.getX( );
                m_mouseButtonReleasedY = mouseEvent.getY( );
            
                // continue only if the start and stop position are not the same
                if ( ( Math.abs( m_mouseButtonPressedX - m_mouseButtonReleasedX ) > 5 ) && 
                     ( Math.abs( m_mouseButtonPressedY - m_mouseButtonReleasedY ) > 5 ) )
                {
                    
                    Point2D.Float start = determineValuesAtCursorPosition( new Point( m_mouseButtonPressedX, m_mouseButtonPressedY ) );
                    Point2D.Float stop = determineValuesAtCursorPosition( new Point( m_mouseButtonReleasedX, m_mouseButtonReleasedY ) );
                    
                    float startX = getXPositionOfCurrentRepresentation( m_mouseButtonPressedX, m_mouseButtonPressedY );
                    float stopX = getXPositionOfCurrentRepresentation( m_mouseButtonReleasedX, m_mouseButtonReleasedY );
                    
                    if ( ( startX < stopX ) && ( start.y < stop.y ) )
                    {
                        m_xyForcedPlotRange = new XYPlotRange( startX, stopX, start.y, stop.y );
                    }
                    if ( ( startX < stopX ) && ( start.y > stop.y ) )
                    {
                        m_xyForcedPlotRange = new XYPlotRange( startX, stopX, stop.y, start.y );
                    }
                    if ( ( startX > stopX ) && ( start.y > stop.y ) )
                    {
                        m_xyForcedPlotRange = new XYPlotRange( stopX, startX, stop.y, start.y );
                    }
                    if ( ( startX > stopX ) && ( start.y < stop.y ) )
                    {
                        m_xyForcedPlotRange = new XYPlotRange(stopX, startX, start.y, stop.y );
                    }
                    
                    this.m_XYPlotRangeSelectorMenuItem.setSelected( true );
                    
                    changeDisplayedSignal( );
                    
                }
            }
        }
        
        super.mouseReleased( mouseEvent );
    }
    
    /**
     * Called when the mouse button is clicked.
     */
    @Override
    public void mouseClicked( MouseEvent mouseEvent ) 
    {
        // is only of interest if left button is pressed
        if ( ( mouseEvent.getModifiers( ) & InputEvent.BUTTON1_MASK ) == InputEvent.BUTTON1_MASK )
        {
            m_mouseButtonClickedX = mouseEvent.getX( );
            m_mouseButtonClickedY = mouseEvent.getY( );
        }
        super.mouseClicked( mouseEvent );
    }
    
    /**
     * Called when the mouse is dragged.
     */
    @Override
    public void mouseDragged( MouseEvent mouseEvent )
    {
        m_mouseDragged = true;
        super.mouseDragged( mouseEvent );
    }
    
    /**
     * Called when the mouse enters the control.
     */    
    @Override
    public void mouseEntered( MouseEvent mouseEvent )
    {
        super.mouseEntered( mouseEvent );
    }
    
    /**
     * Called when the mouse exits.
     */    
    @Override
    public void mouseExited( MouseEvent mouseEvent )
    {
        super.mouseExited( mouseEvent );
    }
    
    /**
     * Called when the mouse button is released.
     */    
    @Override
    public void mouseMoved( MouseEvent mouseEvent )
    {
        createStatusLineMessageMouseLocation( mouseEvent );
        super.mouseMoved( mouseEvent );
    }
    
    /**
     * Creates a status line containing the current location 
     * of the mouse pointer.
     */
    private void createStatusLineMessageMouseLocation( MouseEvent mouseEvent )
    {
        int xPos = mouseEvent.getX( );
        int yPos = mouseEvent.getY( );
        
        if ( ( xPos > this.getSize( ).getWidth( ) / 10 ) && ( xPos < 9 * this.getSize( ).getWidth( ) / 10 ) )
        {   
            if ( ( yPos > this.getSize( ).getHeight( ) / 10 ) && ( yPos < 9 * this.getSize( ).getHeight( ) / 10 ) )
            {
                float xrange = 8.0f * (float)getSize( ).getWidth( ) / 10.0f;
                float yrange = 8.0f * (float)getSize( ).getHeight( ) / 10.0f;
                
                MRSignalProperties mrcond = MRSignalProperties.getInstance( );
                
                if ( m_curves != null )
                {
                    if ( m_curves.size( ) > 0 )
                    {
                        float xPosCorrected = ( ( (float)xPos - (float)this.getSize( ).getWidth( ) / 10.0f )  )  / xrange;
                        float yPosCorrected = ( ( (float)yPos - (float)this.getSize( ).getHeight( ) / 10.0f )  )  / yrange;
                        
                        if ( m_XYPlotRangeSelectorMenuItem.isSelected( ) == true )
                        {
                            xPosCorrected = m_xyForcedPlotRange.getMinimumX( ) + xPosCorrected * m_xyForcedPlotRange.getXRange( );
                            yPosCorrected = m_xyForcedPlotRange.getMaximumY( ) - yPosCorrected * m_xyForcedPlotRange.getYRange( );
                        }
                        else
                        {
                            xPosCorrected = m_xyAutoPlotRange.getMinimumX( ) + xPosCorrected * m_xyAutoPlotRange.getXRange( );
                            yPosCorrected = m_xyAutoPlotRange.getMaximumY( ) - yPosCorrected * m_xyAutoPlotRange.getYRange( );
                        }
                        
                        m_statusPanel.setLeftMessage( "( x, y ) =  ( " + xPosCorrected + " , " + yPosCorrected + " )" );
                        
                    }
                }
            }
        }
    }
    
    /**
     * 
     * Returns the x-axis value in the current representation.
     * 
     * @param currentX
     * @param currentY
     * @return 
     */
    public float getXPositionOfCurrentRepresentation( int currentX, int currentY )
    {
        float xPosCorrected = -1f;
        float yPosCorrected = -1f;
        
        if ( ( currentX > this.getSize( ).getWidth( ) / 10 ) && ( currentX < 9 * this.getSize( ).getWidth( ) / 10 ) )
        {   
            if ( ( currentY > this.getSize( ).getHeight( ) / 10 ) && ( currentY < 9 * this.getSize( ).getHeight( ) / 10 ) )
            {
                float xrange = 8.0f * (float)getSize( ).getWidth( ) / 10.0f;
                float yrange = 8.0f * (float)getSize( ).getHeight( ) / 10.0f;
                
                MRSignalProperties mrcond = MRSignalProperties.getInstance( );
                
                if ( m_curves != null )
                {
                    if ( m_curves.size( ) > 0 )
                    {
                        xPosCorrected = ( ( (float)currentX - (float)this.getSize( ).getWidth( ) / 10.0f )  )  / xrange;
                        yPosCorrected = ( ( (float)currentY - (float)this.getSize( ).getHeight( ) / 10.0f )  )  / yrange;
                        
                        if ( m_XYPlotRangeSelectorMenuItem.isSelected( ) == true )
                        {
                            if ( m_displayXReversed == false )
                            {
                                xPosCorrected = m_xyForcedPlotRange.getMinimumX( ) + xPosCorrected * m_xyForcedPlotRange.getXRange( );
                            }
                            else
                            {
                                xPosCorrected = m_xyForcedPlotRange.getMaximumX( ) - xPosCorrected * m_xyForcedPlotRange.getXRange( );
                            }
                            
                            yPosCorrected = m_xyForcedPlotRange.getMaximumY( ) - yPosCorrected * m_xyForcedPlotRange.getYRange( );
                        }
                        else
                        {
                            if ( m_displayXReversed == false )
                            {
                                xPosCorrected = m_xyAutoPlotRange.getMinimumX( ) + xPosCorrected * m_xyAutoPlotRange.getXRange( );
                            }
                            else
                            {
                                xPosCorrected = m_xyAutoPlotRange.getMaximumX( ) - xPosCorrected * m_xyAutoPlotRange.getXRange( );
                            }
                            
                            yPosCorrected = m_xyAutoPlotRange.getMaximumY( ) - yPosCorrected * m_xyAutoPlotRange.getYRange( );
                        }
                    }
                }
            }
        }
        
        return xPosCorrected;
    }
    
    /**
     * Determines the selected values at the position indicated by the
     * mouseEvent.
     */
    private void determineValuesAtCursorPosition( int xPos, int yPos )
    {
        
        if ( ( xPos > this.getSize( ).getWidth( ) / 10 ) && ( xPos < 9 * this.getSize( ).getWidth( ) / 10 ) )
        {   
            if ( ( yPos > this.getSize( ).getHeight( ) / 10 ) && ( yPos < 9 * this.getSize( ).getHeight( ) / 10 ) )
            {
                float xrange = 8.0f * (float)getSize( ).getWidth( ) / 10.0f;
                float yrange = 8.0f * (float)getSize( ).getHeight( ) / 10.0f;
                
                MRSignalProperties mrcond = MRSignalProperties.getInstance( );
                
                if ( m_curves != null )
                {
                    if ( m_curves.size( ) > 0 )
                    {
                        float xPosCorrected = ( ( (float)xPos - (float)this.getSize( ).getWidth( ) / 10.0f )  )  / xrange;
                        float yPosCorrected = ( ( (float)yPos - (float)this.getSize( ).getHeight( ) / 10.0f )  )  / yrange;
                        
                        if ( m_XYPlotRangeSelectorMenuItem.isSelected( ) == true )
                        {
                            m_lastSelectedOffsetValue = m_xyForcedPlotRange.getMinimumX( ) + xPosCorrected * m_xyForcedPlotRange.getXRange( );
                            m_lastSelectedAmplitudeValue = m_xyForcedPlotRange.getMaximumY( ) - yPosCorrected * m_xyForcedPlotRange.getYRange( );
                        }
                        else
                        {
                            m_lastSelectedOffsetValue = m_xyAutoPlotRange.getMinimumX( ) + xPosCorrected * m_xyAutoPlotRange.getXRange( );
                            m_lastSelectedAmplitudeValue = m_xyAutoPlotRange.getMaximumY( ) - yPosCorrected * m_xyAutoPlotRange.getYRange( );
                        }
                        
                        // lorentzian amplitude must be corrected in order to get the correct height in freq. domain
                        m_lastSelectedAmplitudeValue = m_lastSelectedAmplitudeValue * 3.141592f * 2.0f / 1024.0f;                        
                    }
                }
            }
        }
    }
    
    /**
     * Determines the selected values at the position indicated by the
     * mouseEvent.
     */
    private Point2D.Float determineValuesAtCursorPosition( Point p )
    {
        
        if ( ( p.x > this.getSize( ).getWidth( ) / 10 ) && ( p.x < 9 * this.getSize( ).getWidth( ) / 10 ) )
        {   
            if ( ( p.y > this.getSize( ).getHeight( ) / 10 ) && ( p.y < 9 * this.getSize( ).getHeight( ) / 10 ) )
            {
                float xrange = 8.0f * (float)getSize( ).getWidth( ) / 10.0f;
                float yrange = 8.0f * (float)getSize( ).getHeight( ) / 10.0f;
                
                MRSignalProperties mrcond = MRSignalProperties.getInstance( );
                
                if ( m_curves != null )
                {
                    if ( m_curves.size( ) > 0 )
                    {
                        float xPosCorrected = ( ( (float)p.x - (float)this.getSize( ).getWidth( ) / 10.0f )  )  / xrange;
                        float yPosCorrected = ( ( (float)p.y - (float)this.getSize( ).getHeight( ) / 10.0f )  )  / yrange;
                        
                        if ( m_XYPlotRangeSelectorMenuItem.isSelected( ) == true )
                        {
                            m_lastSelectedOffsetValue = m_xyForcedPlotRange.getMinimumX( ) + xPosCorrected * m_xyForcedPlotRange.getXRange( );
                            m_lastSelectedAmplitudeValue = m_xyForcedPlotRange.getMaximumY( ) - yPosCorrected * m_xyForcedPlotRange.getYRange( );
                        }
                        else
                        {
                            m_lastSelectedOffsetValue = m_xyAutoPlotRange.getMinimumX( ) + xPosCorrected * m_xyAutoPlotRange.getXRange( );
                            m_lastSelectedAmplitudeValue = m_xyAutoPlotRange.getMaximumY( ) - yPosCorrected * m_xyAutoPlotRange.getYRange( );
                        }
                    }
                }
            }
        }
        
        return new Point2D.Float( m_lastSelectedOffsetValue, m_lastSelectedAmplitudeValue );
        
    }
    
    /**
     * Adds an ActionListener to the list of action listeners who are notified
     * when an event happens in this object.
     * @param e
     */
    @Override
    public void addActionListener( ActionListener e )
    {
	m_listeners.add( e );
    }
    
    /**
     * Return the measured data object.
     * @return 
     */
    public SingleVoxelSpectralData getMeasuredDataResponse( )
    {
        return m_measuredData;
    }
    
     /**
     * Return the measured data object.
     * @return 
     */
    public SingleVoxelSpectralData getFittedDataResponse( )
    {
        return m_fittedData;
    }
    
    /**
     * Loads the fitted spectrum, the original spectrum and the residual spectrum
     * and displays them on the screen.
     */
    public void loadAndShowTDFDFitDataFiles( )
    {

        m_measuredData = new SingleVoxelMeasuredSpectralData( m_numberOfPoints );
        m_measuredData.readTDFDFitFile( m_numberOfPoints, m_fittingFileReference );
        addSpectralDataSet( "measured", m_measuredData, false );
        float measuredSpectrumAbsRange = m_measuredData.getDynamicSpectrumRangeAbsorption( );
        // set the title of the plot
        int pos = m_fittingFileReference.lastIndexOf( "." );
        String fileName = m_fittingFileReference.substring( 0, pos );
        
        m_fittedData = new SingleVoxelMeasuredSpectralData( m_numberOfPoints, Color.red );
        m_fittedData.readTDFDFitFile( m_numberOfPoints, fileName + ".fit" );
        addSpectralDataSet( "fitted", m_fittedData, false );
        
        m_residualData = new SingleVoxelMeasuredSpectralData( m_numberOfPoints, Color.blue );
        m_residualData.readTDFDFitFile( m_numberOfPoints, fileName + ".res" );
        m_residualData.subtractConstantValueFromBothFDChannels( measuredSpectrumAbsRange );
        
        addSpectralDataSet( "residual", m_residualData, true );
        
        TDFDFittingDefaults defs = TDFDFittingDefaults.getInstance( );
        defs.load( );
        
        if ( defs.getProperty( TDFDFittingDefaults.DISPLAY_DOMAIN ).compareTo( TDFDFittingDefaults.FREQ_DOMAIN ) == 0 )
        {
            if ( defs.getProperty( TDFDFittingDefaults.DISPLAY_UNIT ).compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
            {
                m_displayXReversed = true;
            }
        }
        
        // Clear all comments of the plot
        clearAllLeftUpperInformationLines( );
        if ( m_calledDirectAfterPrototypeModelFitting == true )
        {
            readAndDisplayFitresultsInXYPlot( );
        }
    }
    
    /**
     * Prints the results panel. 
     * @param g
     * @param pf
     * @param pageIndex
     * @return
     * @throws PrinterException 
     */
    @Override
    public int print( Graphics g, PageFormat pf, int pageIndex ) throws PrinterException 
    {
        if ( pageIndex > 0 )
        {
            return Printable.NO_SUCH_PAGE;
        }
        Graphics2D g2d = (Graphics2D)g;
        g2d.translate( pf.getImageableX( ), pf.getImageableY( ) );
                
        Rectangle componentBounds = this.getBounds(null);
        double scaleX = pf.getImageableWidth( ) / componentBounds.width;
        double scaleY = pf.getImageableHeight( ) / componentBounds.height;
        
        g2d.scale( scaleX, scaleY );
        
        paintComponent( g2d );
        
        return Printable.PAGE_EXISTS;  
    }
    
    /**
     * Prints the currently displayed spectrum
     */
    public void print( )
    {
        //--- Create a new PrinterJob object
        PrinterJob printJob = PrinterJob.getPrinterJob( );
        
        //--- Create a new book to add pages to
        Book book = new Book( );
        
        //--- Add the document page using a landscape page format
        PageFormat documentPageFormat = new PageFormat( );
        documentPageFormat.setOrientation( PageFormat.LANDSCAPE );
        book.append ( this, documentPageFormat );
        
        //--- Tell the printJob to use the Book as the Pageable object
        printJob.setPageable( book );
        
        //--- Show the print dialog box. If the user clicks the
        //--- print button, we then proceed to print, else we cancel
        //--- the process.
        try 
        {
            if ( printJob.printDialog( ) )
            {
                printJob.print( );
            }
        }
        catch ( PrinterException e )
        {
        }  
    }

    /**
     * Returns whether the token is a valid double.
     * @param token
     * @return 
     */
    public boolean testIsTokenADouble( String token )
    {
        boolean isADouble = true;
        
        if ( ( token.contains( "1.#QNAN0e+000" ) == true ) || ( token.contains( "-1.#IND00e+000" ) == true ) )
        {
            isADouble = false;
        }
        
        return isADouble;
    }    
    
    /**
     * 
     * @param token
     * @return 
     */
    public boolean testIsTokenAFloat( String token )
    {
        boolean isADouble = true;
        
        if ( ( token.contains( "1.#QNAN0e+000" ) == true ) || ( token.contains( "-1.#IND00e+000" ) == true ) )
        {
            isADouble = false;
        }
        
        return isADouble;
    }     
    
    /**
     * This method reads the fit results from file and updates the 
     * MRFittingParameter objects.
     */
    private void readAndDisplayFitresultsInXYPlot( )
    {
        
        clearAllLeftUpperInformationLines( );
        
        DecimalFormat decformat = new DecimalFormat( "#.####" );
        TDFDFittingDefaults fdef = TDFDFittingDefaults.getInstance( );
        fdef.load( );
        
        String resultsDirectory = fdef.getProperty( TDFDFittingDefaults.TDFDFIT_MODELS_BASE_PATH, System.getProperty( "user.dir" ) );
        // System.out.println( " Results directory : " + resultsDirectory );
        
        // tdfdfit produces in the directory the verbose.par result file
        // this file has to be parsed and the result values should be read 
        // into the FittingParameter objects
        TDFDFitModelTreeWithPopupMenu fitModelTree = TDFDFitPrototypeModelSpectrumParameters.getInstance( ).getFitModelTree( );
        
        DefaultMutableTreeNode root = (DefaultMutableTreeNode)fitModelTree.getModel( ).getRoot( );
        int cntr = 0;
        MRFittingParameters fitPar = new MRLineModelParameters( );
        
        try
        {        
            RandomAccessFile reader;
            if ( m_calledDirectAfterPrototypeModelFitting == true )
            {
                String verboseFileToParseForResults = m_fittingFileReference.replace( ".bin", "_verbose_master.par" );
                reader = new RandomAccessFile( verboseFileToParseForResults, "rw" );
            }
            else
            {
                String parFile = m_fittingFileReference.replace( "bin", "_master.par" );
                reader = new RandomAccessFile( parFile, "rw" );
            }
            
            String str = reader.readLine( );
            // System.out.println( "string read " + str );

            while ( str.trim( ).compareTo( "tdfdfit_starting_values" ) != 0 )
            {
                str = reader.readLine( );
            }
            
            // read the following blank line
            reader.readLine( );
            
            int totalComponents = TDFDFitPrototypeModelSpectrumParameters.getInstance( ).getTotalNumberOfComponents( );
            
            for ( int i = 0; i < totalComponents; i++ )
            {
                
                // local variable needed to identify whether current spectral
                // object is an instance of MRLineModelParameters; if it
                // is an offset shift of half the spectral width has to occur
                // if not, no offset shift is needed
                boolean isObjectMRLineModelParameters = false;
                
                Object obj = ((DefaultMutableTreeNode)root.getChildAt( i ) ).getUserObject( );
            
                if ( obj instanceof MRLineModelParameters )
                {
                    fitPar = (MRFittingParameters)obj;
                    isObjectMRLineModelParameters = true;
                }
                
                if ( obj instanceof MRParametricPattern )
                {
                    fitPar = ((MRParametricPattern)obj).getFittingParameters( );
                }
                
                if ( obj instanceof MRNumericPattern )
                {
                    fitPar = ((MRNumericPattern)obj).getFittingParameters( );
                }
                
                
                str = reader.readLine( );
                // System.out.println( str );
                int ii = 0;
                while ( str.charAt( ii++ ) == ' ' )
                {
                    // do nothing
                }
                str = str.substring( ii - 1 );
                int length = str.indexOf( " " );
                String firstToken = str.substring( 0, length );
                str = str.substring(length);
                
                ////////////////////////////////////////////////////////////////
                ii = 0;
                while ( str.charAt( ii++ ) == ' ' )
                {
                    // do nothing
                }
                str = str.substring( ii - 1 );
                
                length = str.indexOf( " " );
                String secondToken = str.substring( 0, length );
                str = str.substring( length );
                Float area;
                if ( testIsTokenAFloat( secondToken ) == true )
                    area = new Float( secondToken );
                else
                    area = 0.0f;
                
                ////////////////////////////////////////////////////////////////
                
                ii = 0;
                while ( str.charAt( ii++ ) == ' ' )
                {
                    // do nothing
                }
                str = str.substring( ii - 1 );     
                length = str.indexOf( " " );         
                String thirdToken = str.substring( 0, length );
                str = str.substring( length );
                Float offset;
                
                if ( testIsTokenAFloat( thirdToken ) == true )
                    offset = new Float( thirdToken );
                else
                    offset = 0.0f; 
                
                ////////////////////////////////////////////////////////////////////////
                ii = 0;
                while ( str.charAt( ii++ ) == ' ' )
                {
                    // do nothing
                }
                str = str.substring( ii - 1 );
                
                length = str.indexOf( " " );
                String fourthToken = str.substring( 0, length );
                str = str.substring( length );
                Float gauss;
                if ( testIsTokenAFloat( fourthToken ) == true )
                    gauss = new Float( fourthToken );
                else
                    gauss = 0.0f;
                
                ////////////////////////////////////////////////////////////////////////
                ii = 0;
                while ( str.charAt( ii++ ) == ' ' )
                {
                    // do nothing
                }
                str = str.substring( ii - 1 );
                length = str.indexOf( " " );
                String fifthToken = str.substring( 0, length );
                str = str.substring( length );
                
                Float phase;
                if ( testIsTokenAFloat( fifthToken ) == true )
                    phase = new Float( fifthToken );
                else
                    phase = 0.0f;
                ////////////////////////////////////////////////////////////////////////
                str = str.trim( );   
                Float lorentz;
                if ( testIsTokenAFloat( str ) == true )
                    lorentz = new Float( str );
                else
                    lorentz = 0.0f;        
     
                float areaVal = area.floatValue( ) / (float)MRSignalProperties.getInstance( ).getNumberOfSamplePoints( );
                
                if ( ( obj instanceof MRParametricPattern ) || 
                         ( obj instanceof MRNumericPattern ) )
                {
                    if ( areaVal < 0.0 )
                    {
                        // areaVal = 0.0f;
                    }
                }
                
                String disp = fitPar.getName( ) + "   =   " + decformat.format( (double)areaVal ) + " i.u.";
                if ( m_displayMode == FitResultSpectrumViewerPanel.PATTERNS_ONLY )
                {
                    if ( ( obj instanceof MRParametricPattern ) || 
                         ( obj instanceof MRNumericPattern ) )
                    {
                        addLeftUpperInformationLine( disp );
                    }
                }
                else
                {
                    addLeftUpperInformationLine( disp );
                }
            }
            refresh( );
            reader.close( );
        }
        catch ( IOException e )
        {
            e.printStackTrace( );
            System.out.println( "IOException in reading tdfdfit parameterfile" );
        }
    }
    
    /**
     * Returns the file selected by the user.
     */
    private File getScreenShotFileReference( )
    {   
        TDFDFittingDefaults fdef = TDFDFittingDefaults.getInstance( );
        fdef.load( );
        
        JFileChooser fileChooser = new JFileChooser( fdef.getProperty( "File name and path to store screenshot into" ) );
        
        int result = fileChooser.showOpenDialog( this );
        
        m_screenshotFileReference = new File( "" );
        
        switch ( result )
        {
            case JFileChooser.APPROVE_OPTION:   // Approve Open button was pressed
                                                m_screenshotFileReference = fileChooser.getSelectedFile( );
                                                m_screenshotFileReference = new File( m_screenshotFileReference.getPath( ) );
                                                break;
           case JFileChooser.CANCEL_OPTION:     // Cancel or close dialog was selected
                                                break;
           case JFileChooser.ERROR_OPTION:      // the selected process did not terminate properly
                                                break;
        }
        
        return m_screenshotFileReference;
    }
    
    
    /**
     * Returns the file selected by the user.
     */
    private String getExportFileReference( )
    {   
        TDFDFittingDefaults fdef = TDFDFittingDefaults.getInstance( );
        fdef.load( );
        
        JFileChooser fileChooser = new JFileChooser( fdef.getProperty( "File name and path to store export curves into" ) );
        
        int result = fileChooser.showOpenDialog( this );
        
        
        switch ( result )
        {
            case JFileChooser.APPROVE_OPTION:   // Approve Open button was pressed
                                                m_textFileReference = fileChooser.getSelectedFile().getAbsolutePath( );
                                                
                                                break;
           case JFileChooser.CANCEL_OPTION:     // Cancel or close dialog was selected
                                                break;
           case JFileChooser.ERROR_OPTION:      // the selected process did not terminate properly
                                                break;
        }
        
        return m_textFileReference;
    }
    
    private void exportCurvesAsText( ) 
    {
        TextFileWriter writer = null;
        try 
        {
            String txtFile = getExportFileReference( );
            writer = new TextFileWriter( new FileWriter( txtFile ) );
            
            float[] absSigFD    = m_measuredData.getAbsorptionChannelFD( );
            float[] dispSigFD   = m_measuredData.getDispersionChannelFD( );
            float[] absFitFD    = m_fittedData.getAbsorptionChannelFD( );
            float[] dispFitFD   = m_fittedData.getDispersionChannelFD( );
            float[] absResFD    = m_residualData.getAbsorptionChannelFD( );
            float[] dispResFD   = m_residualData.getDispersionChannelFD( );
            writer.writeStringLn( "Number" + "," + "abs_sig_FD" + "," + "disp_sig_FD" + "," + "abs_fit_FD" + "," + "disp_fit_FD" + "," + "abs_res_FD" + "," + "disp_res_FD" );
            for ( int i = 0; i < absSigFD.length; i++ )
            {
                writer.writeInteger( i );
                writer.writeString( "," );
                writer.writeFloat( absSigFD[i] );
                writer.writeString( "," );
                writer.writeFloat( dispSigFD[i] );
                writer.writeString( "," );
                writer.writeFloat( absFitFD[i] );
                writer.writeString( "," );
                writer.writeFloat( dispFitFD[i] );
                writer.writeString( "," );
                writer.writeFloat( absResFD[i] );
                writer.writeString( "," );
                writer.writeFloat( dispResFD[i] );
                writer.writeString( "\n" );
            }
            
            writer.flush( );
            writer.close( );
            
        } 
        catch ( IOException ex ) 
        {
            Logger.getLogger( FitResultSpectrumViewerPanel.class.getName( ) ).log( Level.SEVERE, null, ex );
        } 
        finally 
        {
            try 
            {
                writer.close( );
            } 
            catch ( IOException ex ) 
            {
                Logger.getLogger( FitResultSpectrumViewerPanel.class.getName( ) ).log( Level.SEVERE, null, ex );
            }
        }
        
    }
    
    // member variables
    
    private LinkedList<ActionListener> m_listeners = new LinkedList<ActionListener>( );
    
    private JMenuItem m_printSpectrum;
    
    private JMenu m_overlayNumericValueTypes;
    private JRadioButtonMenuItem m_displayAllSpectralComponents;
    private JRadioButtonMenuItem m_displayPatternsOnly;
    
    private int m_mouseButtonPressedX;
    private int m_mouseButtonPressedY;
    
    protected float m_lastSelectedOffsetValue;
    protected float m_lastSelectedAmplitudeValue;
    
    private String  m_fittingFileReference;
    private int     m_numberOfPoints;
    private SingleVoxelMeasuredSpectralData m_measuredData;
    private SingleVoxelMeasuredSpectralData m_fittedData;
    private SingleVoxelMeasuredSpectralData m_residualData;
    
    private boolean m_mouseDragged;
    
    protected int m_mouseButtonClickedX;
    protected int m_mouseButtonClickedY;
      
    protected int m_mouseButtonReleasedX;
    protected int m_mouseButtonReleasedY;
    
    protected int m_mouseButtonDraggedX;
    protected int m_mouseButtonDraggedY;
    
    public final static int DISPLAY_ALL = 0;
    public final static int PATTERNS_ONLY = 1;
    
    private int m_displayMode = PATTERNS_ONLY;
    
    private JMenuItem m_saveScreenShot;
    
    private JMenuItem m_exportCurvesOfSpectra;
    
    private File m_screenshotFileReference;
    private String m_textFileReference;
    private boolean m_calledDirectAfterPrototypeModelFitting;
    
    // For proper serialization
    private static final long serialVersionUID = 6482323528125113L;

    
    
}
