/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.tdfdfit;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.StringTokenizer;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.spectrim.settings.GlobalSettings;

/**
 * Data container class that handles model parameters for imperfect.
 * This class contains the default parameters
 * @author johan
 */
public class ImperfectSpectrumModelParameters 
{
    private ImperfectSpectrumModelParameters( )
    {
        GlobalSettings globSettings = GlobalSettings.getInstance( );
        
        String TDFDFittingDefaultsPath = globSettings.getSpectrimRootDir( );
        
        String imperfectTDFDFitModelParametersPath = TDFDFittingDefaultsPath +
                                                     System.getProperty( "file.separator" ) +
                                                     "TDFDFit" +
                                                     System.getProperty( "file.separator" ) +
                                                     "ImperfectTDFDFitModelParameters" +
                                                     System.getProperty( "file.separator" );
        
        File newPath = new File( imperfectTDFDFitModelParametersPath );
        boolean bool = newPath.mkdir( );
        
        String imperfectTDFDFitModelParametersFileName = TDFDFittingDefaultsPath +
                                                         System.getProperty( "file.separator" ) +
                                                         "TDFDFit" +
                                                         System.getProperty( "file.separator" ) +
                                                         "ImperfectTDFDFitModelParameters" +
                                                         System.getProperty( "file.separator" ) +
                                                         "ImperfectTDFDFitModelParameters.ini";
        
        File imperfectTDFDFitModelParametersFile = new File( imperfectTDFDFitModelParametersFileName );
        
        if ( imperfectTDFDFitModelParametersFile.exists( ) == false )
        {
            try 
            {
                TextFileWriter writer;
                
                writer = new TextFileWriter( new FileWriter( imperfectTDFDFitModelParametersFileName ) );
                
                writer.writeStringLn( "**************************************************" );
                writer.writeStringLn( "*  Automatic created file by spectrIm-QMRS       *" );
                writer.writeStringLn( "*  Do not edit!!.                                *" );
                writer.writeStringLn( "**************************************************" );
                
                Date today = new Date( );
                SimpleDateFormat formatter = new SimpleDateFormat( "dd-MMM-yyyy" );
                String dateStr = formatter.format( today );
                writer.writeStringLn( "* Date: " + dateStr + "                              *" );
                writer.writeStringLn( "**************************************************" );
                writer.writeStringLn( "" );
                
                writer.writeStringLn( "Reference component:" );
                writer.writeStringLn( m_referenceComponentName );
                
                // Write residual water model paramters
                writer.writeStringLn( "Number of water components: " );
                writer.writeStringLn( "" + m_numberOfWaterResiduals );
                writer.writeStringLn( "water amplitude factors" );
                
                for ( int i = 0; i < m_numberOfWaterResiduals; i++ )
                {
                   writer.writeString( "" + m_maxWaterResidualAmplitudeFactors[i] );
                   if ( i < m_numberOfWaterResiduals - 1 )
                   {
                       writer.writeString( "," );
                   }
                }
                
                writer.writeStringLn( "" );
                writer.writeStringLn( "water offsets" );
                
                for ( int i = 0; i < m_numberOfWaterResiduals; i++ )
                {
                   writer.writeString( "" + m_maxWaterResidualOffsets[i] );
                   if ( i < m_numberOfWaterResiduals - 1 )
                   {
                       writer.writeString( "," );
                   }
                }
                
                writer.writeStringLn( "" );
                writer.writeStringLn( "water phases" );
                
                for ( int i = 0; i < m_numberOfWaterResiduals; i++ )
                {
                   writer.writeString( "" + m_maxWaterResidualPhases[i] );
                   if ( i < m_numberOfWaterResiduals - 1 )
                   {
                       writer.writeString( "," );
                   }
                }
                
                writer.writeStringLn( "" );
                writer.writeStringLn( "water lwidths" );
                
                for ( int i = 0; i < m_numberOfWaterResiduals; i++ )
                {
                   writer.writeString( "" + m_maxWaterResidualLWidths[i] );
                   if ( i < m_numberOfWaterResiduals - 1 )
                   {
                       writer.writeString( "," );
                   }
                }
                writer.writeStringLn( "" );
                
                // Smiley artifact related parameters
                writer.writeStringLn( "Max smiley artefact amplitude factor:" );
                writer.writeStringLn( "" + m_maxSmileyAmplitudeFactor );
                writer.writeStringLn( "Max smiley artefact phase difference:" );
                writer.writeStringLn( "" + m_maxSmileyPhaseDifference );
                writer.writeStringLn( "Max smiley artefact decay constant:" );
                writer.writeStringLn( "" + m_maxSmileyDecayConstant );
                
                 // Here the general baseline model parameter
                writer.writeStringLn( "General baseline amplitude factor:" );
                writer.writeStringLn( "" + m_generalBaseLineAmplitudeFactor );
                writer.writeStringLn( "General baseline max phase:" );
                writer.writeStringLn( "" + m_generalBaseLineMaxPhase );
                writer.writeStringLn( "General baseline decay damping constant:" );
                writer.writeStringLn( "" + m_generalBaseLineDamping );
                writer.writeStringLn( "General baseline offset:" );
                writer.writeStringLn( "" + m_generalBaseLineOffsetVar );
                
                // what of non ideal signal settings are wanted in the simulations?
                
                writer.writeStringLn( "Model has residual water signals:" );
                if ( m_includeWaterResidualArtefactInBaseline == true )
                {
                    writer.writeStringLn( "yes"  );
                }
                else
                {
                    writer.writeStringLn( "no"  );
                }
                writer.writeStringLn( "Model has smiley artifacts:");
                
                if ( m_includeSmileyArtefactInBaseline == true )
                {
                    writer.writeStringLn( "yes" );
                }
                else
                {
                    writer.writeStringLn( "no" );
                }
                
                writer.writeStringLn( "Model has general baseline role artifact:" );
                if ( m_includeGeneralBaselineRole == true )
                {
                    writer.writeStringLn( "yes" );
                }
                else
                {
                    writer.writeStringLn( "no" );
                }
                writer.flush( );
                writer.close( );
            } 
            catch ( IOException ex ) 
            {
                Logger.getLogger( ImperfectSpectrumModelParameters.class.getName( ) ).log( Level.SEVERE, null, ex );
            }
      
        }
        
    }

    /**
     * 
     * This method makes the attributes persistent.
     * 
     */
    public void makeInstancePersistent( )
    {
        
        GlobalSettings globSettings = GlobalSettings.getInstance( );
        
        String TDFDFittingDefaultsPath = globSettings.getSpectrimRootDir( );
        
        String imperfectTDFDFitModelParametersFileName = TDFDFittingDefaultsPath +
                                                         System.getProperty( "file.separator" ) +
                                                         "TDFDFit" +
                                                         System.getProperty( "file.separator" ) +
                                                         "ImperfectTDFDFitModelParameters" +
                                                         System.getProperty( "file.separator" ) +
                                                         "ImperfectTDFDFitModelParameters.ini";
        
        File imperfectTDFDFitModelParametersFile = new File( imperfectTDFDFitModelParametersFileName );
        
        try 
        {
            TextFileWriter writer;

            writer = new TextFileWriter( new FileWriter( imperfectTDFDFitModelParametersFileName ) );

            writer.writeStringLn( "**************************************************" );
            writer.writeStringLn( "*  Automatic created file by spectrIm-QMRS       *" );
            writer.writeStringLn( "*  Do not edit!!.                                *" );
            writer.writeStringLn( "**************************************************" );

            Date today = new Date( );
            SimpleDateFormat formatter = new SimpleDateFormat( "dd-MMM-yyyy" );
            String dateStr = formatter.format( today );
            writer.writeStringLn( "* Date: " + dateStr + "                              *" );
            writer.writeStringLn( "**************************************************" );
            writer.writeStringLn( "" );

            writer.writeStringLn( "Reference component:" );
            writer.writeStringLn( m_referenceComponentName );

            // Write residual water model paramters
            writer.writeStringLn( "Number of water components: " );
            writer.writeStringLn( "" + m_numberOfWaterResiduals );
            writer.writeStringLn( "water amplitude factors" );

            writer.writeStringLn( floatArrayAsCSVString( m_maxWaterResidualAmplitudeFactors ) );

            writer.writeStringLn( "water offsets" );
            writer.writeStringLn( floatArrayAsCSVString( m_maxWaterResidualOffsets ) );

            writer.writeStringLn( "water phases" );
            writer.writeStringLn( floatArrayAsCSVString( m_maxWaterResidualPhases ) );

            writer.writeStringLn( "water lwidths" );
            writer.writeStringLn( floatArrayAsCSVString( m_maxWaterResidualLWidths ) );

            // Smiley artifact related parameters
            writer.writeStringLn( "Max smiley artefact amplitude factor:" );
            writer.writeStringLn( "" + m_maxSmileyAmplitudeFactor );
            writer.writeStringLn( "Max smiley artefact phase difference:" );
            writer.writeStringLn( "" + m_maxSmileyPhaseDifference );
            writer.writeStringLn( "Max smiley artefact decay constant:" );
            writer.writeStringLn( "" + m_maxSmileyDecayConstant );

            // Here the general baseline model parameter
            writer.writeStringLn( "General baseline amplitude factor:" );
            writer.writeStringLn( "" + m_generalBaseLineAmplitudeFactor );
            writer.writeStringLn( "General baseline max phase:" );
            writer.writeStringLn( "" + m_generalBaseLineMaxPhase );
            writer.writeStringLn( "General baseline decay damping constant:" );
            writer.writeStringLn( "" + m_generalBaseLineDamping );
            writer.writeStringLn( "General baseline offset:" );
            writer.writeStringLn( "" + m_generalBaseLineOffsetVar );
            
            // what of non ideal signal settings are wanted in the simulations?

            writer.writeStringLn( "Model has residual water signals:" );
            if ( m_includeWaterResidualArtefactInBaseline == true )
            {
                writer.writeStringLn( "yes"  );
            }
            else
            {
                writer.writeStringLn( "no"  );
            }
            writer.writeStringLn( "Model has smiley artifacts:");

            if ( m_includeSmileyArtefactInBaseline == true )
            {
                writer.writeStringLn( "yes" );
            }
            else
            {
                writer.writeStringLn( "no" );
            }

            writer.writeStringLn( "Model has general baseline role artifact:" );
            if ( m_includeGeneralBaselineRole == true )
            {
                writer.writeStringLn( "yes" );
            }
            else
            {
                writer.writeStringLn( "no" );
            }
            writer.close( );
        } 
        catch ( IOException ex ) 
        {
            Logger.getLogger( ImperfectSpectrumModelParameters.class.getName( ) ).log( Level.SEVERE, null, ex );
        }
    }
    
    public void setMaxSmileyAmplitudeFactor( float maxSmileyFactor )
    {
        m_maxSmileyAmplitudeFactor = maxSmileyFactor;
    }
    public void setMaxSmileyPhaseDifference ( float maxSmileyPhaseDifference )
    {
        m_maxSmileyPhaseDifference = maxSmileyPhaseDifference;
    }
    
    public void setMaxSmileyDecayConstant( float maxSmileyDecayConstant )
    {    
        m_maxSmileyDecayConstant = maxSmileyDecayConstant;
    }
    
    
    public float getMaxSmileyAmplitudeFactor( )
    {
        return m_maxSmileyAmplitudeFactor;
    }
    public float getMaxSmileyPhaseDifference ( )
    {
        return m_maxSmileyPhaseDifference;
    }
    
    public float getMaxSmileyDecayConstant( )
    {    
        return m_maxSmileyDecayConstant;
    }
    
    
    /**
     * Returns the one and only instance of ImperfectSpectrumModelParameters.
     * @return 
     */
    public static ImperfectSpectrumModelParameters getInstance( )
    {
        if ( theInstance == null )
        {
            resetDefaultsWaterResiduals( );
            theInstance = new ImperfectSpectrumModelParameters( );
            loadInstanceFromFile( );
        }

        return theInstance;
    }
    
    
    private static void loadInstanceFromFile( )
    {
        
        GlobalSettings globSettings = GlobalSettings.getInstance( );
        String TDFDFittingDefaultsPath = globSettings.getSpectrimRootDir( );
        String imperfectTDFDFitModelParametersFileName = TDFDFittingDefaultsPath + 
                                                         System.getProperty( "file.separator" ) +
                                                         "TDFDFit" +
                                                         System.getProperty( "file.separator" ) +
                                                         "ImperfectTDFDFitModelParameters" +
                                                         System.getProperty( "file.separator" ) +
                                                         "ImperfectTDFDFitModelParameters.ini";    
        
        String dummy;
        try ( BufferedReader br = new BufferedReader( new FileReader( imperfectTDFDFitModelParametersFileName ) ) ) 
        {
            dummy = br.readLine( );
            dummy = br.readLine( );
            dummy = br.readLine( );
            dummy = br.readLine( );
            dummy = br.readLine( );   
            dummy = br.readLine( );
            dummy = br.readLine( );
            dummy = br.readLine( );
            m_referenceComponentName = br.readLine( );
            br.readLine( );
            m_numberOfWaterResiduals = new Integer( br.readLine( ) );
            br.readLine( );
            
            // amplitude factors ...
            String ampFactors = br.readLine( );
            
            StringTokenizer S = new StringTokenizer( ampFactors, "," );
            int numTokens = S.countTokens( );
            
            m_maxWaterResidualAmplitudeFactors = new float[numTokens];
            String token;
            for ( int k = 0; k < numTokens; k++ )
            {
                token = S.nextToken( );
                m_maxWaterResidualAmplitudeFactors[k] = new Float( token );
            }
            
            // offset values ...
            br.readLine( );
            String waterResidualOffsets = br.readLine( );
            S = new StringTokenizer( waterResidualOffsets, "," );
            m_maxWaterResidualOffsets = new float[numTokens];
            for ( int k = 0; k < numTokens; k++ )
            {
                token = S.nextToken( );
                m_maxWaterResidualOffsets[k] = new Float( token );
            }
            
            // offset values ...
            br.readLine( );
            String waterResidualPhases = br.readLine( );
            S = new StringTokenizer( waterResidualPhases, "," );
            m_maxWaterResidualPhases = new float[numTokens];
            for ( int k = 0; k < numTokens; k++ )
            {
                token = S.nextToken( );
                m_maxWaterResidualPhases[k] = new Float( token );
            }
            
            // lwidths ...
            br.readLine( );
            String waterResidualLWidths = br.readLine( );
            S = new StringTokenizer( waterResidualLWidths, "," );
            m_maxWaterResidualLWidths= new float[numTokens];
            for ( int k = 0; k < numTokens; k++ )
            {
                token = S.nextToken( );
                m_maxWaterResidualLWidths[k] = new Float( token );
            }
            
            // TO DO read general baseline 
            
            // Read whether non ideal componanent should be used in simulations
            br.readLine( );
            m_maxSmileyAmplitudeFactor = new Float( "" + br.readLine( ) );
            
            br.readLine( );
            m_maxSmileyPhaseDifference = new Float( "" + br.readLine( ) );
            
            br.readLine( );
            m_maxSmileyDecayConstant = new Float( "" + br.readLine( ) );
            
            br.readLine( );
            m_generalBaseLineAmplitudeFactor = new Float( "" + br.readLine( ) );
            
            br.readLine( );
            m_generalBaseLineMaxPhase = new Float( "" + br.readLine( ) );
            
            br.readLine( );
            m_generalBaseLineDamping = new Float( "" + br.readLine( ) );
                    
            br.readLine( );
            m_generalBaseLineOffsetVar = new Float( "" + br.readLine( ) );
                       
           // read some boolean logic variables to select the type of  
            br.readLine( );
            String modelHasResidualWaterSignal = br.readLine( );
            if ( modelHasResidualWaterSignal.compareTo( "yes" ) == 0 )
            {
                m_includeWaterResidualArtefactInBaseline = true;
            }
            else
            {
                m_includeWaterResidualArtefactInBaseline = false;
            }
            
            br.readLine( );
            String modelHasSmileyArtifact = br.readLine( );
            if ( modelHasSmileyArtifact.compareTo( "yes" ) == 0 )
            {
                m_includeSmileyArtefactInBaseline = true;
            }
            else
            {
                m_includeSmileyArtefactInBaseline = false;
            }
            
            br.readLine( );
            String modelHasGeneralBaseline = br.readLine( );
            if ( modelHasSmileyArtifact.compareTo( "yes" ) == 0 )
            {
                m_includeGeneralBaselineRole = true;
            }
            else
            {
                m_includeGeneralBaselineRole = false;
            }
            
            br.close( );
        } 
        catch ( FileNotFoundException ex ) 
        {
            Logger.getLogger( ImperfectSpectrumModelParameters.class.getName( ) ).log( Level.SEVERE, null, ex );
        } 
        catch ( IOException ex ) 
        {
            Logger.getLogger( ImperfectSpectrumModelParameters.class.getName( ) ).log( Level.SEVERE, null, ex );
        }
    }
    
    
    public String floatArrayAsCSVString( float[] fltArr )
    {
        String returnString = "";
        
        for ( int i = 0; i < fltArr.length; i++ )
        {
            if ( i < fltArr.length - 1 )
            {
                returnString += fltArr[i] + ",";
            }
            else
            {
                returnString += fltArr[i];
            }
        }
        
        return returnString;
    }
    
    static public void resetDefaultsWaterResiduals( )
    {
        
        TDFDFitPrototypeModelSpectrumParameters protModel = TDFDFitPrototypeModelSpectrumParameters.getInstance( );
        int numberOfSignalSamples = protModel.getMRSignalProperties( ).getNumberOfSamplePoints( );
          
        m_numberOfWaterResiduals = 5;
        
        m_maxWaterResidualAmplitudeFactors    = new float[m_numberOfWaterResiduals];
        m_maxWaterResidualOffsets             = new float[m_numberOfWaterResiduals];
        m_maxWaterResidualPhases              = new float[m_numberOfWaterResiduals];
        m_maxWaterResidualLWidths             = new float[m_numberOfWaterResiduals];
        
        m_maxWaterResidualAmplitudeFactors[0] = 1.0f;
        m_maxWaterResidualAmplitudeFactors[1] = 2.0f;
        m_maxWaterResidualAmplitudeFactors[2] = 3.0f;
        m_maxWaterResidualAmplitudeFactors[3] = 4.0f;
        m_maxWaterResidualAmplitudeFactors[4] = 5.0f;
        
        m_maxWaterResidualOffsets[0] = 2.0f;
        m_maxWaterResidualOffsets[1] = 6.0f;
        m_maxWaterResidualOffsets[2] = 12.0f;
        m_maxWaterResidualOffsets[3] = 15.0f;
        m_maxWaterResidualOffsets[4] = 20.0f;
                
        m_maxWaterResidualPhases[0] = 3.0f;
        m_maxWaterResidualPhases[1] = 10.0f;
        m_maxWaterResidualPhases[2] = 45.0f;
        m_maxWaterResidualPhases[3] = 45.0f;
        m_maxWaterResidualPhases[4] = 90.0f;
        
        m_maxWaterResidualLWidths[0] = 3.0f;
        m_maxWaterResidualLWidths[1] = 10.0f;
        m_maxWaterResidualLWidths[2] = 25.0f;
        m_maxWaterResidualLWidths[3] = 50.0f;
        m_maxWaterResidualLWidths[4] = 75.0f;
        
        // Smiley artefact related parameters
        m_maxSmileyAmplitudeFactor = 0.1f;
        m_maxSmileyPhaseDifference = (float)( Math.PI / 2f );
        m_maxSmileyDecayConstant   = numberOfSignalSamples / 128f;
        
        m_generalBaseLineAmplitudeFactor    = 100f;
        m_generalBaseLineMaxPhase           = 3.141592f;
        m_generalBaseLineOffsetVar          = 4096f;
        m_generalBaseLineDamping            = 1024f;
        
    }
    
    public void setGeneralBaseLineAmplitudeFactor( float amplitudeFactor )
    {
        m_generalBaseLineAmplitudeFactor = amplitudeFactor;
    }
    
    public void setGeneralBaseLineMaxPhase( float phase )
    {
        m_generalBaseLineMaxPhase = phase;
    }
    
    public void setGeneralBaseLineOffsetVar( float offset )
    {
        m_generalBaseLineOffsetVar = offset;
    }
    
    public void setGeneralBaseLineDamping( float damping )
    {
        m_generalBaseLineDamping = damping;
    }
   
    public float getGeneralBaseLineAmplitudeFactor(  )
    {
        return m_generalBaseLineAmplitudeFactor;
    }
    
    public float getGeneralBaseLineMaxPhase( )
    {
        return m_generalBaseLineMaxPhase;
    }
    
    public float getGeneralBaseLineOffsetVar( )
    {
        return m_generalBaseLineOffsetVar;
    }
    
    public float getGeneralBaseLineDamping( )
    {
        return m_generalBaseLineDamping;
    }
    
    // Attributes
    private static int      m_numberOfWaterResiduals;
    
    private static float[]  m_maxWaterResidualAmplitudeFactors;
    private static float[]  m_maxWaterResidualOffsets;
    private static float[]  m_maxWaterResidualPhases;
    private static float[]  m_maxWaterResidualLWidths;
    
    
    private static String   m_referenceComponentName = "not defined yet!"; 
    
    private static float    m_maxSmileyAmplitudeFactor;
    private static float    m_maxSmileyPhaseDifference;
    private static float    m_maxSmileyDecayConstant;
    
    private static ImperfectSpectrumModelParameters theInstance;

    private static boolean  m_includeSmileyArtefactInBaseline = false;
    private static boolean  m_includeWaterResidualArtefactInBaseline = false;
    private static boolean  m_includeGeneralBaselineRole = false;
    
    private static float    m_generalBaseLineAmplitudeFactor;
    private static float    m_generalBaseLineMaxPhase;
    private static float    m_generalBaseLineOffsetVar;
    private static float    m_generalBaseLineDamping;
   
    int getNumberOfWaterCompoments( ) 
    {
        return m_numberOfWaterResiduals;
    }

    float[] getResidualWaterAmplFactors( ) 
    {
        return m_maxWaterResidualAmplitudeFactors;
    }

    float[] getResidualWaterOffsetValues( ) 
    {
        return m_maxWaterResidualOffsets;
    }

    float[] getResidualWaterPhaseValues( ) 
    {
        return m_maxWaterResidualPhases;
    }

    float[] getResidualWaterLWidthValues( ) 
    {
        return m_maxWaterResidualLWidths;
    }

    void setResidualWaterAmplFactors( float[] values ) 
    {
       m_maxWaterResidualAmplitudeFactors = new float[ values.length ]; 
       System.arraycopy( values, 0, m_maxWaterResidualAmplitudeFactors, 0, values.length );
        
    }

    void setResidualWaterOffsetValues( float[] values ) 
    {
        m_maxWaterResidualOffsets = new float[ values.length ]; 
        System.arraycopy( values, 0, m_maxWaterResidualOffsets, 0, values.length );
    }

    void setResidualWaterPhaseValues( float[] values ) 
    {
        m_maxWaterResidualPhases = new float[ values.length ]; 
        System.arraycopy( values, 0, m_maxWaterResidualPhases, 0, values.length );
    }

    void setResidualWaterLWidthValues( float[] values ) 
    {
        m_maxWaterResidualLWidths = new float[ values.length ]; 
        System.arraycopy( values, 0, m_maxWaterResidualLWidths, 0, values.length );
    } 
    
    public boolean getIncludeSmileyArtefacts( ) 
    {
        return m_includeSmileyArtefactInBaseline;
    }
    
    public boolean getIncludeHasResidualWaterSignals( ) 
    {
        return m_includeWaterResidualArtefactInBaseline;
    }
    
    public boolean getIncludeHasGeneralBaselineRoleArtefacts( ) 
    {
        return m_includeGeneralBaselineRole;
    }
    
    public void setIncludeSmileyArtefacts( boolean doSet ) 
    {
        m_includeSmileyArtefactInBaseline = doSet;
    }
    
    public void setIncludeHasResidualWaterSignals( boolean doSet ) 
    {
        m_includeWaterResidualArtefactInBaseline = doSet;
    }
    
    public void setIncludeHasGeneralBaselineRoleArtefacts( boolean doSet  ) 
    {
        m_includeGeneralBaselineRole = doSet;
    }

    public void setReferenceComponentName( String selComponent ) 
    {
        m_referenceComponentName = selComponent;
        makeInstancePersistent( );
    }

    public String getSelectedReferenceComponent( )
    {
        return m_referenceComponentName;
    }

    public void setNumberOfWaterResidualComponents( int rowCount ) 
    {
        m_numberOfWaterResiduals = rowCount;
        makeInstancePersistent( );
    }
    
    public int getNumberOfWaterResidualComponents( ) 
    {
        return m_numberOfWaterResiduals;
    }
    
}
