/*
 * File: $Id: MRLineModelParametersPanel.java,v 1.1.1.1 2006/04/19 09:38:49 hans Exp $
 * History
 * Date: 27.05.2003, 11:29  Authors: Hans Slotboom
 *
 * Changes:
 *
 * Copyright (C) 2003 by
 *
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

package org.spectrim.tdfdfit;

import java.awt.BorderLayout;
import org.spectrim.lib.DecimalTextField;
import java.awt.Dimension;
import java.awt.Component;

import java.io.RandomAccessFile;

import java.util.Iterator;
import java.util.LinkedList;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JRadioButton;
import javax.swing.SwingConstants;
import javax.swing.JComboBox;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JTextField;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import javax.swing.ButtonGroup;
import javax.swing.JOptionPane;

/**
 *
 * This Singleton class allows the viewing and changing of attributes of
 * MRLineModelParameter class.
 *
 * @author  hans
 */

public class MRLineModelParametersPanel extends JPanel implements ActionListener, ItemListener
{
    
    /**
     * Creates a new instance of MRLineModelParametersPanel.
     * @param parent
     */
    public MRLineModelParametersPanel( JFrame parent ) 
    {
        m_parent = parent;
        
        // create model type selector
        setLayout( new BoxLayout( this, BoxLayout.Y_AXIS ) );
        setBorder( BorderFactory.createCompoundBorder(
        BorderFactory.createTitledBorder( "Model Browser" ),
        BorderFactory.createEmptyBorder( 2, 2, 2, 2 ) ) );        
        TDFDFitPrototypeModelSpectrumParameters modelSpectrum = TDFDFitPrototypeModelSpectrumParameters.getInstance( );
        modelSpectrum.addActionListener( this );
        
        m_fitModelTreePanel = new FitModelTreePanel( m_parent, modelSpectrum );
        modelSpectrum.setModelTree( m_fitModelTreePanel.getModelTree( ) );
        // the model can be edited via buttons on "this"; resulting events must
        // be redirected to the FitModelTreeWithPopup via the FitModelTreePanel
        addActionListener( m_fitModelTreePanel );
        
        m_treeAndComponentTypePanel = new JPanel( );
        m_treeAndComponentTypePanel.setPreferredSize( new Dimension( 580, 290 ) );
        m_treeAndComponentTypePanel.setMaximumSize( new Dimension( 580, 290 ) );
        m_treeAndComponentTypePanel.setMinimumSize( new Dimension( 580, 290 ) );
        m_treeAndComponentTypePanel.setBorder( BorderFactory.createCompoundBorder(
        BorderFactory.createTitledBorder( "Model tree" ),
        BorderFactory.createEmptyBorder( 2, 2, 2, 2 ) ) ); 
        
        m_treeAndComponentTypePanel.setLayout( new BorderLayout( ) );
        m_treeAndComponentTypePanel.add( m_fitModelTreePanel, BorderLayout.NORTH );
        
        createModelTypePanel( );
        m_treeAndComponentTypePanel.add( m_modelTypePanel, BorderLayout.CENTER );
        
        add( m_treeAndComponentTypePanel );
        
        this.createAddAndRemoveSpectralComponentPanel( );
        add( m_addAndRemoveSpectralComponentsPanel );
        
        createLineParameterValueAdjustmentPanel( );
        // add( m_lineParameterValueAdjustmentPanel );
        add( m_overallPanel );
        m_actionListeners.add( modelSpectrum );
        
        createFitControlPanel( );
        add( m_fitControlPanel );
        
        add( Box.createVerticalGlue( ) );
        setVisible( true );
        
        // Let this class listen to changes in TDFDFittingDefaults changes
        // e.g. the user wants another type of offset frequency display units
        TDFDFittingDefaults.getInstance( ).addActionListener( this );
    }
    
    /**
     * Creates a control enabling the setting of chemical shift unit 
     * display. 
     */
    private void createChemicalShiftDisplayUnitsPanel( )
    {
        m_chemicalShiftDisplayUnitSelectionPanel    = new JPanel( );
        m_chemicalShiftDisplayUnitSelectionPanel.setBorder( BorderFactory.createCompoundBorder(
        BorderFactory.createTitledBorder( "Select offset display units" ),
        BorderFactory.createEmptyBorder( 2, 2, 2, 2 ) ) );
         
        m_chemicalShiftDisplayUnitSelectionPanel.setLayout(
        new BoxLayout( m_chemicalShiftDisplayUnitSelectionPanel, BoxLayout.X_AXIS ) );
        
        m_displayPointUnitsRadioButton      = new JRadioButton( "Point units" );
        m_displayPointUnitsRadioButton.setAlignmentX( Component.CENTER_ALIGNMENT );
        m_displayPointUnitsRadioButton.addActionListener( this );
        
        m_displayHzUnitsRadioButton         = new JRadioButton( "Hz units" );
        m_displayHzUnitsRadioButton.setAlignmentX( Component.CENTER_ALIGNMENT );
        m_displayHzUnitsRadioButton.addActionListener( this );
        
        m_displayPPMUnitsRadioButton        = new JRadioButton( "ppm units" );
        m_displayPPMUnitsRadioButton.setAlignmentX( Component.CENTER_ALIGNMENT );
        m_displayPPMUnitsRadioButton.addActionListener( this );
        
        m_displayUnitsButtongroup = new ButtonGroup( );
        
        m_displayUnitsButtongroup.add( m_displayPointUnitsRadioButton );
        m_displayUnitsButtongroup.add( m_displayHzUnitsRadioButton );
        m_displayUnitsButtongroup.add( m_displayPPMUnitsRadioButton );
        
        m_chemicalShiftDisplayUnitSelectionPanel.add( Box.createHorizontalGlue( ) );
        m_chemicalShiftDisplayUnitSelectionPanel.add( m_displayPointUnitsRadioButton );
        m_chemicalShiftDisplayUnitSelectionPanel.add( Box.createHorizontalGlue( ) );
        m_chemicalShiftDisplayUnitSelectionPanel.add( m_displayHzUnitsRadioButton );
        m_chemicalShiftDisplayUnitSelectionPanel.add( Box.createHorizontalGlue( ) );
        m_chemicalShiftDisplayUnitSelectionPanel.add( m_displayPPMUnitsRadioButton );
        m_chemicalShiftDisplayUnitSelectionPanel.add( Box.createHorizontalGlue( ) );
        
        setFrequencyUnitsTypeDisplay( );
    }

    /**
     * Sets the radio buttons which determine in which units the frequency 
     * units are displayed.
     */
    public void setFrequencyUnitsTypeDisplay( )
    {   
        TDFDFittingDefaults fitdef = TDFDFittingDefaults.getInstance( );
//        fitdef.load( );
        
        String displayUnits = fitdef.getProperty( TDFDFittingDefaults.DISPLAY_UNIT );
       
        // Initialize if not set ...
        if ( m_currentlyDisplayUnitType == null )
        {
            m_currentlyDisplayUnitType = displayUnits;
            m_previouslyDisplayUnitType = displayUnits;
            
            if ( displayUnits.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_POINTS ) == 0 )
            {
                m_currentlyDisplayUnitType = TDFDFittingDefaults.FREQUENCY_DOMAIN_POINTS;
                m_displayPointUnitsRadioButton.setSelected( true );
            }

            if ( displayUnits.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_HZ ) == 0 )
            {
                m_currentlyDisplayUnitType = TDFDFittingDefaults.FREQUENCY_DOMAIN_HZ;
                m_displayHzUnitsRadioButton.setSelected( true );
            }

            if ( displayUnits.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
            {
                m_currentlyDisplayUnitType = TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM;
                m_displayPPMUnitsRadioButton.setSelected( true );
            }
            
        }
        else if ( displayUnits.compareTo( m_currentlyDisplayUnitType ) != 0 )
        {
           
            if ( displayUnits.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_POINTS ) == 0 )
            {
                m_currentlyDisplayUnitType = TDFDFittingDefaults.FREQUENCY_DOMAIN_POINTS;
                m_displayPointUnitsRadioButton.setSelected( true );
            }

            if ( displayUnits.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_HZ ) == 0 )
            {
                m_currentlyDisplayUnitType = TDFDFittingDefaults.FREQUENCY_DOMAIN_HZ;
                m_displayHzUnitsRadioButton.setSelected( true );
            }

            if ( displayUnits.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
            {
                m_currentlyDisplayUnitType = TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM;
                m_displayPPMUnitsRadioButton.setSelected( true );
            }

            // If there is a change in offset unit display type the GUI values should be updated.
            updateGUIOffsetValues( );

            // But also the plot window displaying the spectrum
            this.notifyActionListeners( this, DISPLAY_FREQ_UNITS_TYPE_CHANGED, 
                                        DISPLAY_FREQ_UNITS_TYPE_CHANGED_CMD );
            
             m_previouslyDisplayUnitType = m_currentlyDisplayUnitType;
             
        }   
    }
    
    /**
     * Creates a fit control panel.
     */
    private void createFitControlPanel( )
    {
        m_fitControlPanel = new JPanel( );
        m_fitControlPanel.setLayout( new BoxLayout( m_fitControlPanel, BoxLayout.X_AXIS ) );
        
        m_fitControlPanel.setBorder( BorderFactory.createCompoundBorder(
        BorderFactory.createTitledBorder( "TDFDFit - Model fitting" ),
        BorderFactory.createEmptyBorder( 2, 2, 2, 2 ) ) );
        m_fitControlPanel.add( Box.createHorizontalGlue( ) );
        
        // First button of the row.
        m_fitWithCurrentModelButton = new JButton( "Optimize prototype model" );
        m_fitWithCurrentModelButton.setToolTipText( "Fits the model spectrum using the current model spectrum parameters as starting values" );
        m_fitWithCurrentModelButton.addActionListener( this );
        m_fitControlPanel.add( m_fitWithCurrentModelButton );
        
        m_fitControlPanel.add( Box.createHorizontalGlue( ) );
        
        // Second button of the row.
        m_verifyCurrentModelButton = new JButton( "Verify TDFDFit model" );
        m_verifyCurrentModelButton.setToolTipText( "Verifies the currently defined model on prior knowledge consistency." );
        m_verifyCurrentModelButton.addActionListener( this );
        m_fitControlPanel.add( m_verifyCurrentModelButton );
        
        m_fitControlPanel.add( Box.createHorizontalGlue( ) );
        
        // Second button of the row.
        m_autoSetParameterRangesButton = new JButton( "Auto-set parameter ranges .." );
        m_autoSetParameterRangesButton.setToolTipText( "Sets the parameter range values of model based on user definable defaults." );
        m_autoSetParameterRangesButton.addActionListener( this );
        // m_autoSetParameterRangesButton.setEnabled( false );
        m_fitControlPanel.add( m_autoSetParameterRangesButton );
        
        m_fitControlPanel.add( Box.createHorizontalGlue( ) );
    }
    
    
    /**
     * Creates a model type selection panel contain three radio buttons.
     */
    private void createModelTypePanel( ) 
    {
        m_modelTypePanel = new JPanel( );
        m_modelTypePanel.setLayout( new BoxLayout( m_modelTypePanel, BoxLayout.X_AXIS ) );
        m_modelTypePanel.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );
        
        m_componentTypeLabel = new JLabel( "Component type  " );
        m_modelTypePanel.add( m_componentTypeLabel );
        
        m_modelTypePanel.add( Box.createHorizontalGlue( ) );
        
        m_lineModel = new JRadioButton( "Line" );
        m_lineModel.setEnabled( false );
        
        m_modelTypePanel.add( m_lineModel );
        
        m_modelTypePanel.add( Box.createHorizontalGlue( ) );
        
        m_parametricPatternModel = new JRadioButton( "Parametric Pattern" );
        m_parametricPatternModel.setEnabled( false );
        m_modelTypePanel.add( m_parametricPatternModel );
        
        m_modelTypePanel.add( Box.createHorizontalGlue( ) );
        
        m_numericPatternModel = new JRadioButton( "Numeric Pattern" );
        m_numericPatternModel.setEnabled( false );
        m_modelTypePanel.add( m_numericPatternModel );
        
        m_modelTypePanel.add( Box.createHorizontalGlue( ) );
        
        m_metaboliteModel = new JRadioButton( "Metabolite" );
        m_metaboliteModel.setEnabled( false );
        m_modelTypePanel.add( m_metaboliteModel );
        
        m_modelTypePanel.add( Box.createHorizontalGlue( ) );
    }
    
    /**
     * Creates the panel in which the name of the spectroscopic component is
     * displayed.
     */
    private void createSpectralComponentNamePanel( )
    {
        m_spectralComponentNamePanel = new JPanel( );
        m_spectralComponentNamePanel.setLayout(
        new BoxLayout( m_spectralComponentNamePanel, BoxLayout.X_AXIS ) );
        m_spectralComponentNamePanel.setPreferredSize( new Dimension( 480,25 ) );
        m_spectralComponentNamePanel.setMaximumSize( new Dimension( 480,25 ) );
        m_spectralComponentNamePanel.setMinimumSize( new Dimension( 480,25 ) );
        
        m_lineNameLabel = new JLabel( "<html><b>Component name</b></html>" );
        m_lineNameLabel.setPreferredSize( new Dimension( 110, 25 ) );
        m_lineNameLabel.setMinimumSize( new Dimension( 110, 25 ) );
        m_lineNameLabel.setMaximumSize( new Dimension( 110, 25 ) );
        m_spectralComponentNamePanel.add( m_lineNameLabel );
        
        m_lineNameTextField = new JTextField( "New line" );
        m_lineNameTextField.setPreferredSize( new Dimension( 200, 25 ) );
        m_lineNameTextField.setMinimumSize( new Dimension( 200, 25 ) );
        m_lineNameTextField.setMaximumSize( new Dimension( 200, 25 ) );
        m_lineNameTextField.setEnabled(false);
        
        m_spectralComponentNamePanel.add( m_lineNameTextField );
        
    }
    
    
    /**
     * Creates the panel containing the line parameter adjustment
     * components. It allows the user to set peak area, offset etc. values
     */
    private void createLineParameterValueAdjustmentPanel( ) 
    {
        
        m_overallPanel = new JPanel( );
        m_overallPanel.setMaximumSize( new Dimension( 580, 125 ) );
        m_overallPanel.setLayout( new BorderLayout( ) );
        m_overallPanel.setBorder( BorderFactory.createCompoundBorder(
        BorderFactory.createTitledBorder( "Fitting Parameter Editor" ),
        BorderFactory.createEmptyBorder( 2, 2, 2, 2 ) ) );
        
        m_lineParameterValueAdjustmentPanel = new JPanel( );
        m_lineParameterValueAdjustmentPanel.setAlignmentY( Component.TOP_ALIGNMENT );
        m_lineParameterValueAdjustmentPanel.setLayout(
        new BoxLayout( m_lineParameterValueAdjustmentPanel, BoxLayout.X_AXIS ) );
        
        m_lineParameterValueAdjustmentPanel.setAlignmentX( Component.LEFT_ALIGNMENT );
        
        createLabelPanel( );
        m_labelPanel.setAlignmentX( Component.LEFT_ALIGNMENT );
        m_lineParameterValueAdjustmentPanel.add( m_labelPanel );
        
        createValuePanel( );
        m_valuePanel.setAlignmentX( Component.LEFT_ALIGNMENT );
        m_lineParameterValueAdjustmentPanel.add( m_valuePanel );
        
        createMinimumValuePanel( );
        m_minimumValuePanel.setAlignmentX( Component.LEFT_ALIGNMENT );
        m_lineParameterValueAdjustmentPanel.add( m_minimumValuePanel );
        
        createMaximumValuePanel( );
        m_maximumValuePanel.setAlignmentX( Component.LEFT_ALIGNMENT );
        m_lineParameterValueAdjustmentPanel.add( m_maximumValuePanel );
        
        createPriorKnowledgeDefinitionPanel( );
        m_priorKnowledgePanel.setAlignmentX( Component.LEFT_ALIGNMENT );
        m_lineParameterValueAdjustmentPanel.add( m_priorKnowledgePanel );
        
        createSpectralComponentNamePanel( );
        
        m_overallPanel.add( m_spectralComponentNamePanel, BorderLayout.NORTH );
        m_overallPanel.add( m_lineParameterValueAdjustmentPanel, BorderLayout.CENTER );
        
        createChemicalShiftDisplayUnitsPanel( );
        m_overallPanel.add( m_chemicalShiftDisplayUnitSelectionPanel, BorderLayout.SOUTH);
    }
                
    /**
     * Creates the panel on which the types of the parameters which can
     * be changed are displayed.
     */
    private void createLabelPanel( ) 
    {
        m_labelPanel = new JPanel( );
        m_labelPanel.setPreferredSize( new Dimension( 110, 175 ) );
        m_labelPanel.setLayout( new BoxLayout( m_labelPanel, BoxLayout.Y_AXIS ) );
        m_labelPanel.setBorder( BorderFactory.createEmptyBorder( 2, 2, 2, 2 ) );
        
        m_newLineNameLabel = new JLabel( "<html><b>Parameter type</b></html>" );
        m_newLineNameLabel.setPreferredSize( new Dimension( 110, 25 ) );
        m_newLineNameLabel.setMinimumSize( new Dimension( 110, 25 ) );
        m_newLineNameLabel.setMaximumSize( new Dimension( 110, 25 ) );
      
        m_labelPanel.add( m_newLineNameLabel );
        
        m_areaLabel = new JLabel( "Area" );
        m_areaLabel.setPreferredSize( new Dimension( 110, 25 ) );
        m_areaLabel.setMinimumSize( new Dimension( 110, 25 ) );
        m_areaLabel.setMaximumSize( new Dimension( 110, 25 ) );
        
        m_labelPanel.add( m_areaLabel );
        
        m_offsetLabel = new JLabel( "Offset" );
        m_offsetLabel.setPreferredSize( new Dimension( 110, 25 ) );
        m_offsetLabel.setMinimumSize( new Dimension( 110, 25 ) );
        m_offsetLabel.setMaximumSize( new Dimension( 110, 25 ) );
        
        m_labelPanel.add( m_offsetLabel );
        
        m_phaseLabel = new JLabel( "Phase" );
        m_phaseLabel.setPreferredSize( new Dimension( 110, 25 ) );
        m_phaseLabel.setMinimumSize( new Dimension( 110, 25 ) );
        m_phaseLabel.setMaximumSize( new Dimension( 110, 25 ) );
        
        m_labelPanel.add( m_phaseLabel );
        
        m_lorentzianWidthLabel = new JLabel( "Lorentzian width" );
        m_lorentzianWidthLabel.setPreferredSize( new Dimension( 110, 25 ) );
        m_lorentzianWidthLabel.setMinimumSize( new Dimension( 110, 25 ) );
        m_lorentzianWidthLabel.setMaximumSize( new Dimension( 110, 25 ) );
        
        m_labelPanel.add( m_lorentzianWidthLabel );
        
        m_gaussWidthLabel = new JLabel( "Gaussian width" );
        m_gaussWidthLabel.setPreferredSize( new Dimension( 110, 25 ) );
        m_gaussWidthLabel.setMinimumSize( new Dimension( 110, 25 ) );
        m_gaussWidthLabel.setMaximumSize( new Dimension( 110, 25 ) );
        
        m_labelPanel.add( m_gaussWidthLabel );
    }
    
    /**
     * Creates the panel on which the parameters value, which can
     * be viewed and changed, are displayed.
     */
    private void createValuePanel( ) 
    {
        m_valuePanel = new JPanel( );
        m_valuePanel.setPreferredSize( new Dimension( 100, 175 ) );
        m_valuePanel.setLayout( new BoxLayout( m_valuePanel, BoxLayout.Y_AXIS ) );
        m_valuePanel.setBorder( BorderFactory.createEmptyBorder( 2, 2, 2, 2 ) );
        
        m_valueLabel = new JLabel( "<html><b>Parameter Value</b></html>", SwingConstants.CENTER );
        m_valueLabel.setAlignmentX( Component.CENTER_ALIGNMENT );
          
        m_valueLabel.setPreferredSize( new Dimension( 100, 25 ) );
        m_valueLabel.setMinimumSize( new Dimension( 100, 25 ) );
        m_valueLabel.setMaximumSize( new Dimension( 100, 25 ) );
        m_valuePanel.add( m_valueLabel, SwingConstants.CENTER );
        m_valueLabel.setAlignmentX( Component.CENTER_ALIGNMENT );
        
        
//        m_placeHolderTextField = new JLabel( "         ");
//        m_placeHolderTextField.setPreferredSize( new Dimension( 100, 25 ) );
//        m_placeHolderTextField.setMinimumSize( new Dimension( 100, 25 ) );
//        m_placeHolderTextField.setMaximumSize( new Dimension( 100, 25 ) );
//        m_valuePanel.add( m_placeHolderTextField );
        
        // parameter values
        m_areaTextField = new DecimalTextField( "#0.000###E0", 0.0f , 1.0f , 10.0f, 100.0f );
        m_areaTextField.setPreferredSize( new Dimension( 100, 25 ) );
        m_areaTextField.setMinimumSize( new Dimension( 100, 25 ) );
        m_areaTextField.setMaximumSize( new Dimension( 100, 25 ) );
        m_areaTextField.addActionListener( this );
        
        m_valuePanel.add( m_areaTextField );
        
        m_offsetTextField = new DecimalTextField( "#0.000###E0", 0.0f, 0.1f, 1.0f, 10.0f );
        m_offsetTextField.setPreferredSize( new Dimension( 100, 25 ) );
        m_offsetTextField.setMinimumSize( new Dimension( 100, 25 ) );
        m_offsetTextField.setMaximumSize( new Dimension( 100, 25 ) );
        m_offsetTextField.addActionListener( this );
        
        m_valuePanel.add( m_offsetTextField );
        
        m_phaseTextField = new DecimalTextField( "#0.000###E0", 0.0f, 1.0f, 5.0f, 45.0f );
        m_phaseTextField.setPreferredSize( new Dimension( 100, 25 ) );
        m_phaseTextField.setMinimumSize( new Dimension( 100, 25 ) );
        m_phaseTextField.setMaximumSize( new Dimension( 100, 25 ) );
        m_phaseTextField.addActionListener( this );
        
        m_valuePanel.add( m_phaseTextField );
        
        m_lorentzianWidthTextField = new DecimalTextField( "#0.000###E0", 0.0f, 0.1f, 0.5f, 1.0f );
        m_lorentzianWidthTextField.setPreferredSize( new Dimension( 100, 25 ) );
        m_lorentzianWidthTextField.setMinimumSize( new Dimension( 100, 25 ) );
        m_lorentzianWidthTextField.setMaximumSize( new Dimension( 100, 25 ) );
        m_lorentzianWidthTextField.addActionListener( this );
        
        m_valuePanel.add( m_lorentzianWidthTextField );
        
        m_gaussWidthTextField = new DecimalTextField( "#0.000###E0", 0.0f, 0.1f, 0.5f, 1.0f );
        m_gaussWidthTextField.setPreferredSize( new Dimension( 100, 25 ) );
        m_gaussWidthTextField.setMinimumSize( new Dimension( 100, 25 ) );
        m_gaussWidthTextField.setMaximumSize( new Dimension( 100, 25 ) );
        m_gaussWidthTextField.addActionListener( this );
        
        m_valuePanel.add( m_gaussWidthTextField );
    }
    
    /**
     * Creates the panel on which the parameters value, which can
     * be viewed and changed, are displayed.
     */
    private void createMinimumValuePanel( ) 
    {
        // Minimum parameter values
        m_minimumValuePanel = new JPanel( );
        m_minimumValuePanel.setPreferredSize( new Dimension( 100, 175 ) );
        m_minimumValuePanel.setLayout( new BoxLayout( m_minimumValuePanel, BoxLayout.Y_AXIS ) );
        m_minimumValuePanel.setBorder( BorderFactory.createEmptyBorder( 2, 2, 2, 2 ) );
        
//        m_minimumPlaceHolderLabel = new JLabel( "       ", SwingConstants.CENTER );
//        // m_minimumPlaceHolderLabel.setAlignmentX( Component.CENTER_ALIGNMENT );
//        m_minimumPlaceHolderLabel.setPreferredSize( new Dimension( 100, 25 ) );
//        m_minimumPlaceHolderLabel.setMinimumSize( new Dimension( 100, 25 ) );
//        m_minimumPlaceHolderLabel.setMaximumSize( new Dimension( 100, 25 ) );
//        
//        m_minimumValuePanel.add( m_minimumPlaceHolderLabel );
        
        m_minimumValueLabel = new JLabel( "<html><b>Minimum</b></html>", SwingConstants.CENTER );
        m_minimumValueLabel.setAlignmentX( Component.CENTER_ALIGNMENT );
        
        m_minimumValueLabel.setPreferredSize( new Dimension( 100, 25 ) );
        m_minimumValueLabel.setMinimumSize( new Dimension( 100, 25 ) );
        m_minimumValueLabel.setMaximumSize( new Dimension( 100, 25 ) );
        
        m_minimumValuePanel.add( m_minimumValueLabel );
        
        m_minimumAreaTextField = new DecimalTextField( "#0.000###E0", 0.0f, 1.0f, 10.0f, 100.0f );
        m_minimumAreaTextField.setPreferredSize( new Dimension( 100, 25 ) );
        m_minimumAreaTextField.setMinimumSize( new Dimension( 100, 25 ) );
        m_minimumAreaTextField.setMaximumSize( new Dimension( 100, 25 ) );
        m_minimumAreaTextField.addActionListener( this );
        m_minimumValuePanel.add( m_minimumAreaTextField );
        
        m_minimumOffsetTextField = new DecimalTextField( "#0.000###E0", 0.0f, 0.1f, 1.0f, 10.0f );
        m_minimumOffsetTextField.setPreferredSize( new Dimension( 100, 25 ) );
        m_minimumOffsetTextField.setMinimumSize( new Dimension( 100, 25 ) );
        m_minimumOffsetTextField.setMaximumSize( new Dimension( 100, 25 ) );
        m_minimumOffsetTextField.addActionListener( this );
        m_minimumValuePanel.add( m_minimumOffsetTextField );
        
        m_minimumPhaseTextField = new DecimalTextField( "#0.000###E0", 0.0f, 1.0f, 5.0f, 45.0f );
        m_minimumPhaseTextField.setPreferredSize( new Dimension( 100, 25 ) );
        m_minimumPhaseTextField.setMinimumSize( new Dimension( 100, 25 ) );
        m_minimumPhaseTextField.setMaximumSize( new Dimension( 100, 25 ) );
        m_minimumPhaseTextField.addActionListener( this );
        m_minimumValuePanel.add( m_minimumPhaseTextField );
        
        m_minimumLorentzianWidthTextField = new DecimalTextField( "#0.000###E0", 0.0f, 0.1f, 0.5f, 1.0f );
        m_minimumLorentzianWidthTextField.setPreferredSize( new Dimension( 100, 25 ) );
        m_minimumLorentzianWidthTextField.setMinimumSize( new Dimension( 100, 25 ) );
        m_minimumLorentzianWidthTextField.setMaximumSize( new Dimension( 100, 25 ) );
        m_minimumLorentzianWidthTextField.addActionListener( this );
        m_minimumValuePanel.add( m_minimumLorentzianWidthTextField );
        
        m_minimumGaussWidthTextField = new DecimalTextField( "#0.000###E0", 0.0f, 0.1f, 0.5f, 1.0f );
        m_minimumGaussWidthTextField.setPreferredSize( new Dimension( 100, 25 ) );
        m_minimumGaussWidthTextField.setMinimumSize( new Dimension( 100, 25 ) );
        m_minimumGaussWidthTextField.setMaximumSize( new Dimension( 100, 25 ) );
        m_minimumGaussWidthTextField.addActionListener( this );
        m_minimumValuePanel.add( m_minimumGaussWidthTextField );
    }
    
    /**
     * Creates the panel on which the parameters value, which can
     * be viewed and changed, are displayed.
     */
    private void createMaximumValuePanel( ) 
    {
        // Maximum parameter values
        m_maximumValuePanel = new JPanel( );
        m_maximumValuePanel.setPreferredSize( new Dimension( 100, 175 ) );
        m_maximumValuePanel.setLayout( new BoxLayout( m_maximumValuePanel, BoxLayout.Y_AXIS ) );
        m_maximumValuePanel.setBorder( BorderFactory.createEmptyBorder( 2, 2, 2, 2 ) );
        
//        m_maximumPlaceHolderLabel = new JLabel( "       ", SwingConstants.CENTER );
//        // m_maximumPlaceHolderLabel.setAlignmentX( Component.CENTER_ALIGNMENT );
//        m_maximumPlaceHolderLabel.setPreferredSize( new Dimension( 100, 25 ) );
//        m_maximumPlaceHolderLabel.setMinimumSize( new Dimension( 100, 25 ) );
//        m_maximumPlaceHolderLabel.setMaximumSize( new Dimension( 100, 25 ) );
//        
//        m_maximumValuePanel.add( m_maximumPlaceHolderLabel );
          
        m_maximumValueLabel = new JLabel( "<html><b>Maximum</b></html>", SwingConstants.CENTER );
        m_maximumValueLabel.setAlignmentX( Component.CENTER_ALIGNMENT );
        
        m_maximumValueLabel.setPreferredSize( new Dimension( 100, 25 ) );
        m_maximumValueLabel.setMinimumSize( new Dimension( 100, 25 ) );
        m_maximumValueLabel.setMaximumSize( new Dimension( 100, 25 ) );
        
        m_maximumValuePanel.add( m_maximumValueLabel );
        
        m_maximumAreaTextField = new DecimalTextField( "#0.000###E0", 0.0f, 1.0f, 10.0f, 100.0f  );
        m_maximumAreaTextField.setPreferredSize( new Dimension( 100, 25 ) );
        m_maximumAreaTextField.setMinimumSize( new Dimension( 100, 25 ) );
        m_maximumAreaTextField.setMaximumSize( new Dimension( 100, 25 ) );
        m_maximumAreaTextField.addActionListener( this );
        m_maximumValuePanel.add( m_maximumAreaTextField );
        
        m_maximumOffsetTextField = new DecimalTextField( "#0.000###E0", 0.0f, 0.1f, 1.0f, 10.0f );
        m_maximumOffsetTextField.setPreferredSize( new Dimension( 100, 25 ) );
        m_maximumOffsetTextField.setMinimumSize( new Dimension( 100, 25 ) );
        m_maximumOffsetTextField.setMaximumSize( new Dimension( 100, 25 ) );
        m_maximumOffsetTextField.addActionListener( this );
        m_maximumValuePanel.add( m_maximumOffsetTextField );
        
        m_maximumPhaseTextField = new DecimalTextField( "#0.000###E0", 0.0f, 1.0f, 5.0f, 45.0f );
        m_maximumPhaseTextField.setPreferredSize( new Dimension( 100, 25 ) );
        m_maximumPhaseTextField.setMinimumSize( new Dimension( 100, 25 ) );
        m_maximumPhaseTextField.setMaximumSize( new Dimension( 100, 25 ) );
        m_maximumPhaseTextField.addActionListener( this );
        m_maximumValuePanel.add( m_maximumPhaseTextField );
        
        m_maximumLorentzianWidthTextField = new DecimalTextField( "#0.000###E0", 0.0f, 0.1f, 0.5f, 1.0f );
        m_maximumLorentzianWidthTextField.setPreferredSize( new Dimension( 100, 25 ) );
        m_maximumLorentzianWidthTextField.setMinimumSize( new Dimension( 100, 25 ) );
        m_maximumLorentzianWidthTextField.setMaximumSize( new Dimension( 100, 25 ) );
        m_maximumLorentzianWidthTextField.addActionListener( this );
        m_maximumValuePanel.add( m_maximumLorentzianWidthTextField );
        
        m_maximumGaussWidthTextField = new DecimalTextField( "#0.000###E0", 0.0f, 0.1f, 0.5f, 1.0f );
        m_maximumGaussWidthTextField.setPreferredSize( new Dimension( 100, 25 ) );
        m_maximumGaussWidthTextField.setMinimumSize( new Dimension( 100, 25 ) );
        m_maximumGaussWidthTextField.setMaximumSize( new Dimension( 100, 25 ) );
        m_maximumGaussWidthTextField.addActionListener( this );
        m_maximumValuePanel.add( m_maximumGaussWidthTextField );
    }
    
    /**
     * Creates the panel on which the parameters value, which can
     * be viewed and changed, are displayed.
     */
    private void createPriorKnowledgeDefinitionPanel( ) 
    {
        m_priorKnowledgePanel = new JPanel( );
        m_priorKnowledgePanel.setPreferredSize( new Dimension( 120, 175 ) );
        m_priorKnowledgePanel.setLayout( new BoxLayout( m_priorKnowledgePanel, BoxLayout.Y_AXIS ) );
        m_priorKnowledgePanel.setBorder( BorderFactory.createEmptyBorder( 2, 2, 2, 2 ) );
        
        String[] priorKnowledge = { "Free" };
        
//        m_priorKnowledgePlaceHolderLabel = new JLabel( "       ", SwingConstants.CENTER );
//        
//        m_priorKnowledgePlaceHolderLabel.setPreferredSize( new Dimension( 120, 25 ) );
//        m_priorKnowledgePlaceHolderLabel.setMinimumSize( new Dimension( 120, 25 ) );
//        m_priorKnowledgePlaceHolderLabel.setMaximumSize( new Dimension( 120, 25 ) );
//        
//        m_priorKnowledgePanel.add( m_priorKnowledgePlaceHolderLabel );
//        
        m_priorKnowledgeTitleLabel = new JLabel( "<html><b>Prior Knowledge</b></html>", SwingConstants.CENTER );
        m_maximumValueLabel.setAlignmentX( Component.CENTER_ALIGNMENT );
        m_priorKnowledgeTitleLabel.setPreferredSize( new Dimension( 120, 25 ) );
        m_priorKnowledgeTitleLabel.setMinimumSize( new Dimension( 120, 25 ) );
        m_priorKnowledgeTitleLabel.setMaximumSize( new Dimension( 120, 25 ) );
        m_priorKnowledgePanel.add( m_priorKnowledgeTitleLabel );
        
        m_areaPriorComboBox = new JComboBox( priorKnowledge );
        m_areaPriorComboBox.setPreferredSize( new Dimension( 120, 25 ) );
        m_areaPriorComboBox.setMinimumSize( new Dimension( 120, 25 ) );
        m_areaPriorComboBox.setMaximumSize( new Dimension( 120, 25 ) );
        m_areaPriorComboBox.setAlignmentX( Component.LEFT_ALIGNMENT );
        m_areaPriorComboBox.addItemListener( this );
        
        m_priorKnowledgePanel.add( m_areaPriorComboBox );
            
        m_offsetPriorComboBox = new JComboBox( priorKnowledge );
        m_offsetPriorComboBox.setPreferredSize( new Dimension( 120, 25 ) );
        m_offsetPriorComboBox.setMinimumSize( new Dimension( 120, 25 ) );
        m_offsetPriorComboBox.setMaximumSize( new Dimension( 120, 25 ) );
        m_offsetPriorComboBox.setAlignmentX( Component.LEFT_ALIGNMENT );
        m_offsetPriorComboBox.addItemListener( this );
        
        m_priorKnowledgePanel.add( m_offsetPriorComboBox );
        
        m_phasePriorComboBox = new JComboBox( priorKnowledge );
        m_phasePriorComboBox.setPreferredSize( new Dimension( 120, 25 ) );
        m_phasePriorComboBox.setMinimumSize( new Dimension( 120, 25 ) );
        m_phasePriorComboBox.setMaximumSize( new Dimension( 120, 25 ) );
        m_phasePriorComboBox.setAlignmentX( Component.LEFT_ALIGNMENT );
        m_phasePriorComboBox.addItemListener( this );
                
        m_priorKnowledgePanel.add( m_phasePriorComboBox );
        
        m_lorentzianPriorComboBox = new JComboBox( priorKnowledge );
        m_lorentzianPriorComboBox.setPreferredSize( new Dimension( 120, 25 ) );
        m_lorentzianPriorComboBox.setMinimumSize( new Dimension( 120, 25 ) );
        m_lorentzianPriorComboBox.setMaximumSize( new Dimension( 120, 25 ) );
        m_lorentzianPriorComboBox.setAlignmentX( Component.LEFT_ALIGNMENT );
        m_lorentzianPriorComboBox.addItemListener( this );
        
        m_priorKnowledgePanel.add( m_lorentzianPriorComboBox );
        
        m_gaussianPriorComboBox = new JComboBox( priorKnowledge );
        m_gaussianPriorComboBox.setPreferredSize( new Dimension( 120, 25 ) );
        m_gaussianPriorComboBox.setMinimumSize( new Dimension( 120, 25 ) );
        m_gaussianPriorComboBox.setMaximumSize( new Dimension( 120, 25 ) );
        m_gaussianPriorComboBox.setAlignmentX( Component.LEFT_ALIGNMENT );
        m_gaussianPriorComboBox.addItemListener( this );
        
        m_priorKnowledgePanel.add( m_gaussianPriorComboBox );
    }
    
    /**
     * Creates a panel that allows the user to add and remove diverse spectral
     * components by simply pressing buttons.
     */
    private void createAddAndRemoveSpectralComponentPanel( ) 
    {
        m_addAndRemoveSpectralComponentsPanel = new JPanel( );
        m_addAndRemoveSpectralComponentsPanel.setPreferredSize( new Dimension( 580, 120 ) );
        m_addAndRemoveSpectralComponentsPanel.setMinimumSize( new Dimension( 580, 120 ) );
        m_addAndRemoveSpectralComponentsPanel.setMaximumSize( new Dimension( 580, 120 ) );
        
        m_addAndRemoveSpectralComponentsPanel.setLayout( 
                        new BoxLayout( m_addAndRemoveSpectralComponentsPanel,
                        BoxLayout.Y_AXIS ) );
        
        m_addAndRemoveSpectralComponentsPanel.setBorder( BorderFactory.createCompoundBorder(
        BorderFactory.createTitledBorder( "Add/removed spectral components" ),
        BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) ) );
        createAddSpectralComponentPanel( );
        m_addAndRemoveSpectralComponentsPanel.add( m_addSpectralComponentsPanel );
        createRemoveSpectralComponentPanel( );
        m_addAndRemoveSpectralComponentsPanel.add( m_removeSpectralComponentsPanel );
    }
    
    /**
     * Creates the panel containing two buttons which cause, when pressed,
     * the display of dialogs allowing the user to define the name of the
     * metabolite or spectral components.
     */
    private void createAddSpectralComponentPanel( ) 
    {
        m_addSpectralComponentsPanel = new JPanel( );
        m_addSpectralComponentsPanel.setLayout( new BoxLayout(
        m_addSpectralComponentsPanel,
        BoxLayout.X_AXIS ) );
        
        m_addSpectralComponentsPanel.add( Box.createHorizontalGlue( ) );
        m_addLineButton = new JButton( "Line" );
        m_addLineButton.setToolTipText( "Adds a line to the spectral model" );
        m_addLineButton.addActionListener( this );
        m_addLineButton.setEnabled( false );
        
        m_addSpectralComponentsPanel.add( m_addLineButton );
        
        m_addSpectralComponentsPanel.add( Box.createHorizontalGlue( ) );
        
        m_addParametricPatternButton = new JButton( "Parametric Pattern" );
        m_addParametricPatternButton.setToolTipText( "Adds a parametric pattern to the spectral model" );
        m_addParametricPatternButton.addActionListener( this );
        m_addParametricPatternButton.setEnabled( false );
        
        m_addSpectralComponentsPanel.add( m_addParametricPatternButton );
        
        m_addSpectralComponentsPanel.add( Box.createHorizontalGlue( ) );
        
        m_addNumericPatternButton = new JButton( "Numeric Pattern" );
        m_addNumericPatternButton.setToolTipText( "Adds a numeric pattern to the spectral model" );
        m_addNumericPatternButton.addActionListener( this );
        m_addNumericPatternButton.setEnabled( false );
        m_addSpectralComponentsPanel.add( m_addNumericPatternButton );
        
        m_addSpectralComponentsPanel.add( Box.createHorizontalGlue( ) );
        
        m_addMetaboliteButton = new JButton( "Metabolite" );
        m_addMetaboliteButton.setToolTipText( "Adds a metabolite to the spectral model" );
        m_addMetaboliteButton.addActionListener( this );
        m_addMetaboliteButton.setEnabled( false );
        
        m_addSpectralComponentsPanel.add( m_addMetaboliteButton );
        
        m_addSpectralComponentsPanel.add( Box.createHorizontalGlue( ) );
        
    }
    
    /**
     * Creates the panel containing two buttons which cause, when pressed,
     * the display of dialogs allowing the user to define the name of the
     * metabolite or spectral components.
     */
    private void createRemoveSpectralComponentPanel( ) 
    {
        m_removeSpectralComponentsPanel = new JPanel( );
        m_removeSpectralComponentsPanel.setLayout( new BoxLayout(
                                                   m_removeSpectralComponentsPanel,
                                                   BoxLayout.X_AXIS ) );
        m_removeSpectralComponentsPanel.setBorder( BorderFactory.createEmptyBorder( 5, 0, 0, 0 ) );
        m_removeSpectralComponentsPanel.add( Box.createHorizontalGlue( ) );
        m_removeSpectralComponentButton = new JButton( "Remove" );
        m_removeSpectralComponentButton.setToolTipText( "Removes the currently selected spectral component" );
        m_removeSpectralComponentButton.addActionListener( this );
        
        m_removeSpectralComponentButton.setEnabled( false );
        
        m_removeSpectralComponentsPanel.add( m_removeSpectralComponentButton );
        
        m_removeSpectralComponentsPanel.add( Box.createHorizontalGlue( ) );
    }
    
    /**
     * Sets the name of the project.
     * @param modelName
     */
    public void setFitModelName( String modelName ) 
    {
        m_fitModelTreePanel.setFitModelName( modelName );
        m_fitModelTreePanel.updateUI( );
    }
    
    /**
     * Change component name: method necessary if user changes the name of the
     * component.
     * @param newComponentName
     */
    public void setCurrentComponentName( String newComponentName )
    {
        m_lineNameTextField.setText( newComponentName );
        this.updateUI( );
    }   
    
    /**
     * Reads the attributes from a RandomAccesFile.
     */
    void read( RandomAccessFile in ) 
    {
        // TO DO
    }
    
    /**
     * Writes attributes to RandomAccessFile.
     */
    void write( RandomAccessFile out ) 
    {
        // TO DO
    }
    
    /**
     * Sets the MRModelSpectrum attribute; method is called after a model
     * spectrum load. Its purpose is to set the GUI with the information.
     * @param modelSpectrum
     */
    public void setMRModelSpectrum( TDFDFitPrototypeModelSpectrumParameters modelSpectrum ) 
    {
       
        m_fitModelTreePanel.setMRModelSpectrum( modelSpectrum );
        simulateDecimalTextFieldChangedEvent( );
    }
    
    /**
     * This forces the simulation of a DecimalTextField-changed event;
     * The method is a hack to get the displaying correct after importing
     * a metabolite to the model.
     */
    public void simulateDecimalTextFieldChangedEvent( )
    {
        setLastNotifiedValues( );
        notifyValueChanged( );
    }
    
    /**
     * Responds to ationEvents raised by registered objects.
     * @param e
     */
    @Override
    public void actionPerformed( ActionEvent e ) 
    {
        
        // Verifies the model on consistency w.r.t. prior knowledge. 
        if ( e.getSource( ) == m_verifyCurrentModelButton )
        {
            boolean retVal = TDFDFitPrototypeModelSpectrumParameters.getInstance( ).isModelConsistent( true );
        }
        
        // Called to set valid enforce parameter ranges ..
        if ( e.getSource( ) == m_autoSetParameterRangesButton )
        {
            TDFDFitPrototypeModelSpectrumParameters.getInstance( ).autoSetParameterRangeValues( );
        }
        
        if ( e.getSource( ) == TDFDFittingDefaults.getInstance( ) )
        {
            setFrequencyUnitsTypeDisplay( );
        }
        
        if ( e.getSource( ) == m_displayPointUnitsRadioButton )
        {
            TDFDFittingDefaults fitdef = TDFDFittingDefaults.getInstance( );
//            fitdef.load( );
            fitdef.setProperty( TDFDFittingDefaults.DISPLAY_UNIT, 
                                TDFDFittingDefaults.FREQUENCY_DOMAIN_POINTS );
            fitdef.save( );
        }
        
        if ( e.getSource( ) == m_displayHzUnitsRadioButton )
        {
            TDFDFittingDefaults fitdef = TDFDFittingDefaults.getInstance( );
            // fitdef.load( );
            fitdef.setProperty( TDFDFittingDefaults.DISPLAY_UNIT, 
                                TDFDFittingDefaults.FREQUENCY_DOMAIN_HZ );
            fitdef.save( );
        }
        
        if ( e.getSource( ) == m_displayPPMUnitsRadioButton )
        {
            TDFDFittingDefaults fitdef = TDFDFittingDefaults.getInstance( );
            // fitdef.load( );
            fitdef.setProperty( TDFDFittingDefaults.DISPLAY_UNIT, 
                                TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM );
            fitdef.save( );
        }
        
        if ( e.getSource( ) instanceof DecimalTextField ) 
        {
            if ( e.getID( ) == DecimalTextField.VALUE_CHANGED_EVENT ) 
            {
                // inform listeners to this component about the change in
                // the fitting parameters/priorknowledge settings
                m_modelValuesChanged = true;
                setLastNotifiedValues( );
                notifyValueChanged( );
            }
        }
        
        if ( e.getSource( ) instanceof MRFittingParameters ) 
        {
            if ( e.getID( ) == TDFDFitPrototypeModelSpectrumParameters.NODE_SELECTED ) 
            {
                setGUIValues( (MRFittingParameters)e.getSource( ) );
                m_modelValuesChanged = true;
                setLastNotifiedValues( );
                notifyValueChanged( );
                
                m_areaTextField.setEnabled( true );
                m_offsetTextField.setEnabled( true );
                m_phaseTextField.setEnabled( true );
                m_gaussWidthTextField.setEnabled( true );
                m_lorentzianWidthTextField.setEnabled( true );
    
                m_minimumAreaTextField.setEnabled( true );
                m_minimumOffsetTextField.setEnabled( true );
                m_minimumPhaseTextField.setEnabled( true );
                m_minimumGaussWidthTextField.setEnabled( true );
                m_minimumLorentzianWidthTextField.setEnabled( true );
    
                m_maximumAreaTextField.setEnabled( true );
                m_maximumOffsetTextField.setEnabled( true );
                m_maximumPhaseTextField.setEnabled( true );
                m_maximumGaussWidthTextField.setEnabled( true );
                m_maximumLorentzianWidthTextField.setEnabled( true );
                
                m_areaPriorComboBox.setEnabled( true );
                m_offsetPriorComboBox.setEnabled( true );
                m_phasePriorComboBox.setEnabled( true );
                m_lorentzianPriorComboBox.setEnabled( true );
                m_gaussianPriorComboBox.setEnabled( true );
            }
        }
        
        if ( e.getSource( ) instanceof String ) 
        {
            m_lineNameTextField.setText( (String)e.getSource( ) );
            
            if ( e.getID( ) == TDFDFitPrototypeModelSpectrumParameters.ROOT_SELECTED ) 
            {
                m_addLineButton.setEnabled( true );
                m_addMetaboliteButton.setEnabled( true );
                m_addNumericPatternButton.setEnabled( true );
                m_addParametricPatternButton.setEnabled( true );
                m_removeSpectralComponentButton.setEnabled( false );
                m_lineModel.setSelected( false );
                m_numericPatternModel.setSelected( false );
                m_parametricPatternModel.setSelected( false );
                m_metaboliteModel.setSelected( false );
                
                m_areaTextField.setText( "" + 0.0 );
                m_offsetTextField.setText( "" + 0.0 );
                m_phaseTextField.setText( "" + 0.0 );
                m_gaussWidthTextField.setText( "" + 0.0 );
                m_lorentzianWidthTextField.setText( "" + 0.0 );
    
                m_minimumAreaTextField.setText( "" + 0.0 );
                m_minimumOffsetTextField.setText( "" + 0.0 );
                m_minimumPhaseTextField.setText( "" + 0.0 );
                m_minimumGaussWidthTextField.setText( "" + 0.0 );
                m_minimumLorentzianWidthTextField.setText( "" + 0.0 );
    
                m_maximumAreaTextField.setText( "" + 0.0 );
                m_maximumOffsetTextField.setText( "" + 0.0 );
                m_maximumPhaseTextField.setText( "" + 0.0 );
                m_maximumGaussWidthTextField.setText( "" + 0.0 );
                m_maximumLorentzianWidthTextField.setText( "" + 0.0 );
              
                
                m_areaTextField.setEnabled( false );
                m_offsetTextField.setEnabled( false );
                m_phaseTextField.setEnabled( false );
                m_gaussWidthTextField.setEnabled( false );
                m_lorentzianWidthTextField.setEnabled( false );
    
                m_minimumAreaTextField.setEnabled( false );
                m_minimumOffsetTextField.setEnabled( false );
                m_minimumPhaseTextField.setEnabled( false );
                m_minimumGaussWidthTextField.setEnabled( false );
                m_minimumLorentzianWidthTextField.setEnabled( false );
    
                m_maximumAreaTextField.setEnabled( false );
                m_maximumOffsetTextField.setEnabled( false );
                m_maximumPhaseTextField.setEnabled( false );
                m_maximumGaussWidthTextField.setEnabled( false );
                m_maximumLorentzianWidthTextField.setEnabled( false );
                
                m_areaPriorComboBox.removeAllItems( );
                m_areaPriorComboBox.setEnabled( false );
                m_offsetPriorComboBox.removeAllItems( );
                m_offsetPriorComboBox.setEnabled( false );
                m_phasePriorComboBox.removeAllItems( );
                m_phasePriorComboBox.setEnabled( false );
                m_lorentzianPriorComboBox.removeAllItems( );
                m_lorentzianPriorComboBox.setEnabled( false );
                m_gaussianPriorComboBox.removeAllItems( );
                m_gaussianPriorComboBox.setEnabled( false );
                
            } 
        }
        
        if ( e.getSource( ) instanceof TDFDFitPrototypeModelSpectrumParameters ) 
        {
            if ( e.getID( ) == TDFDFitPrototypeModelSpectrumParameters.NODE_NAME_CHANGED )
            {
                m_lineNameTextField.setText( (String)e.getActionCommand( ) );
                this.m_modelValuesChanged = true;
                simulateDecimalTextFieldChangedEvent( );
                
            }
            
            if ( e.getID( ) == TDFDFitPrototypeModelSpectrumParameters.ROOT_SELECTED ) 
            {
                m_addLineButton.setEnabled( true );
                m_addMetaboliteButton.setEnabled( true );
                m_addNumericPatternButton.setEnabled( true );
                m_addParametricPatternButton.setEnabled( true );
                m_removeSpectralComponentButton.setEnabled( false );
                
                m_lineModel.setSelected( false );
                m_numericPatternModel.setSelected( false );
                m_parametricPatternModel.setSelected( false );
                m_metaboliteModel.setSelected( false );
            }
            
            if ( e.getID( ) == TDFDFitPrototypeModelSpectrumParameters.PRIOR_KNOWLEDGE_UPDATE ) 
            {
                
                TDFDFitPrototypeModelSpectrumParameters modSpec = (TDFDFitPrototypeModelSpectrumParameters)e.getSource( );
                LinkedList priorKnowledgeSelectionPossibilities = modSpec.getPriorKnowlegdePossibilityLinkedList( );
                
                // Empty the combo boxes!
                
                m_areaPriorComboBox.removeItemListener( this );
                m_offsetPriorComboBox.removeItemListener( this );
                m_phasePriorComboBox.removeItemListener( this );
                m_lorentzianPriorComboBox.removeItemListener( this );        
                m_gaussianPriorComboBox.removeItemListener( this ); 
                
                m_areaPriorComboBox.removeAllItems( );
                m_offsetPriorComboBox.removeAllItems( );
                m_phasePriorComboBox.removeAllItems( );
                m_lorentzianPriorComboBox.removeAllItems( );
                m_gaussianPriorComboBox.removeAllItems( );
                
                
                Iterator it = priorKnowledgeSelectionPossibilities.iterator( );
                
                // Fill them again!
                while ( it.hasNext( ) ) 
                {
                    String itemName = (String)it.next( );
                    m_areaPriorComboBox.addItem( itemName );
                    m_offsetPriorComboBox.addItem( itemName );
                    m_phasePriorComboBox.addItem( itemName );
                    m_lorentzianPriorComboBox.addItem( itemName );
                    m_gaussianPriorComboBox.addItem( itemName );
                }
                
                // TO DO the actual selection of the correct combobox item
                
                m_addLineButton.setEnabled( true );
                m_addMetaboliteButton.setEnabled( true );
                m_addNumericPatternButton.setEnabled( true );
                m_addParametricPatternButton.setEnabled( true );
                m_removeSpectralComponentButton.setEnabled( false );
                
                TDFDFitModelTreeWithPopupMenu fitModelTree = modSpec.getFitModelTreeWithPopUp( );
                Object selObj = fitModelTree.getSelectedObject( );
                
                if  ( selObj instanceof MRLineModelParameters ) 
                {
                    setGUIValues( (MRFittingParameters)selObj );
                    m_addLineButton.setEnabled( false );
                    m_addMetaboliteButton.setEnabled( false );
                    m_addNumericPatternButton.setEnabled( false );
                    m_addParametricPatternButton.setEnabled( false );
                    m_removeSpectralComponentButton.setEnabled( true );
                    
                    m_lineModel.setSelected( true );
                    m_numericPatternModel.setSelected( false );
                    m_parametricPatternModel.setSelected( false );
                    m_metaboliteModel.setSelected( false );
                    
                }
                else if  ( selObj instanceof MRNumericPattern ) 
                {
                    MRFittingParameters fitPar = ((MRNumericPattern)selObj).getFittingParameters( );
                    setGUIValues( fitPar );
                    m_addLineButton.setEnabled( false );
                    m_addMetaboliteButton.setEnabled( false );
                    m_addNumericPatternButton.setEnabled( false );
                    m_addParametricPatternButton.setEnabled( false );
                    m_removeSpectralComponentButton.setEnabled( true );
                    
                    m_lineModel.setSelected( false );
                    m_numericPatternModel.setSelected( true );
                    m_parametricPatternModel.setSelected( false );
                    m_metaboliteModel.setSelected( false );
                }
                else if  ( selObj instanceof MRParametricPattern ) 
                {
                    MRFittingParameters fitPar = ((MRParametricPattern)selObj).getFittingParameters( );
                    setGUIValues( fitPar );
                    m_addLineButton.setEnabled( true );
                    m_addMetaboliteButton.setEnabled( false );
                    m_addNumericPatternButton.setEnabled( false );
                    m_addParametricPatternButton.setEnabled( false );
                    m_removeSpectralComponentButton.setEnabled( true );
                    
                    m_lineModel.setSelected( false );
                    m_numericPatternModel.setSelected( false );
                    m_parametricPatternModel.setSelected( true );
                    m_metaboliteModel.setSelected( false );
                }
                else if ( selObj instanceof MRMetaboliteModel ) 
                {
                    MRFittingParameters fitPar = ((MRMetaboliteModel)selObj).getFittingParameters( );
                    setGUIValues( fitPar );
                    
                    m_addLineButton.setEnabled( true );
                    m_addMetaboliteButton.setEnabled( false );
                    m_addNumericPatternButton.setEnabled( true );
                    m_addParametricPatternButton.setEnabled( true );
                    m_removeSpectralComponentButton.setEnabled( true );
                    
                    m_lineModel.setSelected( false );
                    m_numericPatternModel.setSelected( false );
                    m_parametricPatternModel.setSelected( false );
                    m_metaboliteModel.setSelected( true );
                }
                                
                m_areaPriorComboBox.addItemListener( this );
                m_offsetPriorComboBox.addItemListener( this );
                m_phasePriorComboBox.addItemListener( this );
                m_lorentzianPriorComboBox.addItemListener( this );        
                m_gaussianPriorComboBox.addItemListener( this ); 
                
            }
        }
        
        if ( e.getSource( ) == m_addLineButton ) 
        {
            notifyModelChangeEvent( this, ADD_LINE_EVENT );
        }
        
        if ( e.getSource( ) == m_addParametricPatternButton ) 
        {
            notifyModelChangeEvent( this, ADD_PARAMETRIC_PATTERN_EVENT );
        }
        
        if ( e.getSource( ) == m_addNumericPatternButton ) 
        {
            notifyModelChangeEvent( this, ADD_NUMERIC_PATTERN_EVENT );
        }
        
        if ( e.getSource( ) == m_addMetaboliteButton ) 
        {
            notifyModelChangeEvent( this, ADD_METABOLITE_EVENT );
        }
        
        if ( e.getSource( ) == m_removeSpectralComponentButton ) 
        {
            notifyModelChangeEvent( this, REMOVE_SPECTRAL_COMPONENT_EVENT );
        }
        
        if ( e.getSource( ) instanceof MRParameterSpectrumModelViewer )
        {
            if ( e.getID( ) == MRParameterSpectrumModelViewer.ADD_LINE_EVENT )
            {
                notifyModelChangeEvent( e.getSource( ), ADD_USER_DEFINED_LINE_EVENT );
            }
        }
        
        if ( e.getSource( ) == m_fitWithCurrentModelButton )
        {
            
            if ( TDFDFitPrototypeModelSpectrumParameters.getInstance( ).getNumberOfSpectralComponentsToFit( ) > 0 )
            {
                TDFDFitPrototypeModelSpectrumParameters.getInstance( ).setFitDoesWaterRemoval( false );
                // Select the root node to force the model to be updated 
                // especially this accounts for the comboboxes!
                m_fitModelTreePanel.getModelTree( ).selectRootNode( );

                // Set the vital output files of TDFDFit to be not deleted if selected!
                TDFDFittingDefaults defs = TDFDFittingDefaults.getInstance( );
                defs.load( );
                m_deleteBinFile = defs.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN );
                m_deleteFitFile = defs.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_FIT );
                m_deleteResFile = defs.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_RES );
                m_deleteVerboseFile = defs.getProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_DOT_VERBOSE_PAR );
                if ( m_deleteBinFile == null )
                {
                    m_deleteBinFile = "no";
                    defs.setProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN, "no" );
                }
                if ( m_deleteFitFile == null )
                {
                    m_deleteFitFile = "no";
                    defs.setProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_FIT, "no" );
                }
                if  ( m_deleteResFile == null )
                {
                    m_deleteResFile = "no";
                    defs.setProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_RES , "no" );
                }
                if ( m_deleteVerboseFile == null )
                {
                    m_deleteVerboseFile = "no";
                    defs.setProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_DOT_VERBOSE_PAR, "no"  );
                }
                defs.save( );

                Object obj = m_fitModelTreePanel.getModelTree( ).getModel( ).getRoot( );
                if ( m_fitModelTreePanel.getModelTree( ).getModel( ).getChildCount( obj ) > 0 )
                {
                    String fileName = obj.toString( );

                    if  ( ( (TDFDFitControlFrame)m_parent).getTDFDFitPrototypeModelFilereference( ) == null )
                    {
                        // TDFDFittingDefaults defs = TDFDFittingDefaults.getInstance( );

                        String userModelsHome = defs.getProperty( TDFDFittingDefaults.TDFDFIT_MODELS_BASE_PATH );
                        if ( userModelsHome == null )
                        {
                            userModelsHome = System.getProperty( "user.home" ) + 
                                             System.getProperty( "file.separator" ) + ".SpectrIm" + System.getProperty( "file.separator" ) +
                                             "TDFDFit_FitModelsDir" + 
                                             System.getProperty( "file.separator" );
                            defs.setProperty( TDFDFittingDefaults.TDFDFIT_MODELS_BASE_PATH, userModelsHome );
                            defs.save( );
                        }


                        String pathAndFileReference = defs.getProperty( TDFDFittingDefaults.TDFDFIT_MODELS_BASE_PATH ) + System.getProperty( "file.separator" ) +
                                                      fileName + System.getProperty( "file.separator" ) +
                                                      fileName + ".tfm";

                        ((TDFDFitControlFrame)m_parent).setTDFDFitPrototypeModelFilereference( pathAndFileReference );
                    }
                    ( (TDFDFitControlFrame)m_parent ).saveTDFDFitModelToFile( );
                    // System.out.println( "The user wants to fit this spectrum !!!! " );
                    notifyActionListeners( this, FIT_MODEL_SPECTRUM, FIT_MODEL_SPECTRUM_CMD );
                }             
            }
            else 
            {
                JOptionPane.showMessageDialog( getParent( ), "TDFDFitting cannot be performed: \n first select/define a valid TDFDFit model with at least one component!." );
            }
        }
    }
    
    
    /**
     * During model fitting certain files should not be deleted.
     */
    void restoreFileDeletionSettings( ) 
    {  
        TDFDFittingDefaults defs = TDFDFittingDefaults.getInstance( );   
        defs.setProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN, m_deleteBinFile );
        defs.setProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_FIT, m_deleteFitFile  );
        defs.setProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_BIN_RES , m_deleteResFile );
        defs.setProperty( TDFDFittingDefaults.DELETE_TDFDFIT_OUTPUTFILE_DOT_VERBOSE_PAR, m_deleteVerboseFile  );
    }
    
    /**
     * Sets the values of the GUI components based on the values
     * of the fitPar object.
     */
    private void setGUIValues( MRFittingParameters fitPar ) 
    {
        
        TDFDFittingDefaults defs = TDFDFittingDefaults.getInstance( );
        String fdUnits = defs.getProperty( TDFDFittingDefaults.DISPLAY_UNIT );
            
        // Set the name of the selected object
        m_lineNameTextField.setText( fitPar.getName( ) );
        
        // Set the values of the fit parameters
        m_areaTextField.setFormattedText( ( new Float( fitPar.getArea( ) ) ).toString( ) );      
        m_offsetTextField.setFormattedText( ( new Float( fitPar.getOffsetInDisplayUnits( ) ).toString( ) ) );
        m_phaseTextField.setFormattedText( ( new Float( fitPar.getPhase( ) ) ).toString( ) );
        m_gaussWidthTextField.setFormattedText( ( new Float( fitPar.getGaussianWidth( ) ) ).toString( ) );
        m_lorentzianWidthTextField.setFormattedText( ( new Float( fitPar.getLorentzianLineWidth( ) ) ).toString( ) );
        
        // Set the values of the minimum fit parameters
        m_minimumAreaTextField.setFormattedText( ( new Float( fitPar.getMinimumArea( ) ) ).toString( ) );
        
        m_minimumOffsetTextField.setFormattedText( ( new Float( fitPar.getMinimumOffsetInDisplayUnits( ) ) ).toString( ) );
        
        if ( fdUnits.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
        {
            m_minimumOffsetTextField.setFormattedText( ( new Float( fitPar.getMaximumOffsetInDisplayUnits( ) ) ).toString( ) );
        }
        else
        {
            m_minimumOffsetTextField.setFormattedText( ( new Float( fitPar.getMinimumOffsetInDisplayUnits( ) ) ).toString( ) );
        } 
        
        m_minimumPhaseTextField.setFormattedText( ( new Float( fitPar.getMinimumPhase( ) ) ).toString( ) );;
        m_minimumGaussWidthTextField.setFormattedText( ( new Float( fitPar.getMinimumGaussianWidth( ) ) ).toString( ) );;
        m_minimumLorentzianWidthTextField.setFormattedText( ( new Float( fitPar.getMinimumLorentzianWidth( ) ) ).toString( ) );
        
        // Set the values of the maximum fit parameters
        m_maximumAreaTextField.setFormattedText( ( new Float( fitPar.getMaximumArea( ) ) ).toString( ) );
        
        m_maximumOffsetTextField.setFormattedText( ( new Float( fitPar.getMaximumOffsetInDisplayUnits( ) ) ).toString( ) );;
        
        if ( fdUnits.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
        {
            m_maximumOffsetTextField.setFormattedText( ( new Float( fitPar.getMinimumOffsetInDisplayUnits( ) ) ).toString( ) );
        }
        else
        {
            m_maximumOffsetTextField.setFormattedText( ( new Float( fitPar.getMaximumOffsetInDisplayUnits( ) ) ).toString( ) );
        } 
        
        m_maximumPhaseTextField.setFormattedText( ( new Float( fitPar.getMaximumPhase( ) ) ).toString( ) );;
        m_maximumGaussWidthTextField.setFormattedText( ( new Float( fitPar.getMaximumGaussianWidth( ) ) ).toString( ) );;
        m_maximumLorentzianWidthTextField.setFormattedText( ( new Float( fitPar.getMaximumLorentzianWidth( ) ) ).toString( ) );
        
        // Set the area prior knowledge combobox selections
        for ( int i = 0 ; i < m_areaPriorComboBox.getModel( ).getSize( ); i++ ) 
        {
            String comboValString = (String)( m_areaPriorComboBox.getModel( ).getElementAt( i ) );
            if ( comboValString.compareTo( fitPar.getAreaReference( ) ) == 0 ) 
            {
                m_areaPriorComboBox.setSelectedItem( m_areaPriorComboBox.getModel( ).getElementAt( i ) );
            }
        }
        
        // Set the offset prior knowledge combobox selections
        for ( int i = 0 ; i < m_offsetPriorComboBox.getModel( ).getSize( ); i++ ) 
        {
            String comboValString = (String)( m_offsetPriorComboBox.getModel( ).getElementAt( i ) );
            if ( comboValString.compareTo( fitPar.getOffsetReference( ) ) == 0 ) 
            {
                m_offsetPriorComboBox.setSelectedItem( m_offsetPriorComboBox.getModel( ).getElementAt( i ) );
            }
        }
        
        // Set the lorentzian linewidth prior knowledge combobox selections
        for ( int i = 0 ; i < m_lorentzianPriorComboBox.getModel( ).getSize( ); i++ ) 
        {
            String comboValString = (String)( m_lorentzianPriorComboBox.getModel( ).getElementAt( i ) );
            if ( comboValString.compareTo( fitPar.getLorentzianLineWidthReference( ) ) == 0 ) 
            {
                m_lorentzianPriorComboBox.setSelectedItem( m_lorentzianPriorComboBox.getModel( ).getElementAt( i ) );
            }
        }
        
        // Set the phase prior knowledge combobox selections
        for ( int i = 0 ; i < m_phasePriorComboBox.getModel( ).getSize( ); i++ ) 
        {
            String comboValString = (String)( m_phasePriorComboBox.getModel( ).getElementAt( i ) );
            if ( comboValString.compareTo( fitPar.getPhaseReference( ) ) == 0 ) 
            {
                m_phasePriorComboBox.setSelectedItem( m_phasePriorComboBox.getModel( ).getElementAt( i ) );
            }
        }
        
        // Set the gaussian width prior knowledge combobox selections
        for ( int i = 0 ; i < m_gaussianPriorComboBox.getModel( ).getSize( ); i++ ) 
        {
            String comboValString = (String)( m_gaussianPriorComboBox.getModel( ).getElementAt( i ) );
            if ( comboValString.compareTo( fitPar.getGaussianWidthReference( ) ) == 0 ) 
            {
                m_gaussianPriorComboBox.setSelectedItem( m_gaussianPriorComboBox.getModel( ).getElementAt( i ) );
            }
        }
    }
            
    /**
     * Extract the values from the GUI components and puts them in a
     * MRFittingParameters object.
     */
    public void setLastNotifiedValues( ) 
    {
        
        if ( m_modelValuesChanged == true )
        {
            
            TDFDFittingDefaults defs = TDFDFittingDefaults.getInstance( );
            String fdUnits = defs.getProperty( TDFDFittingDefaults.DISPLAY_UNIT );
            
            // Create a FittingParameters object with default constructor.
            m_lastNotifiedFitParameters = new MRFittingParameters( );

            // Get the name of the component.
            m_lastNotifiedFitParameters.setName( m_lineNameTextField.getText( ) );

            // Get the parameter values from the graphic controls.
            m_lastNotifiedFitParameters.setArea( m_areaTextField.getFloatValue( ) );
            m_lastNotifiedFitParameters.setOffsetFromDisplayUnits( m_offsetTextField.getFloatValue( ) );
            m_lastNotifiedFitParameters.setLorentzianWidth( m_lorentzianWidthTextField.getFloatValue( ) );
            m_lastNotifiedFitParameters.setPhase( m_phaseTextField.getFloatValue( ) );
            m_lastNotifiedFitParameters.setGaussianWidth( m_gaussWidthTextField.getFloatValue( ) );

            // Get the minimum parameter values from the graphic controls.
            m_lastNotifiedFitParameters.setMinimumArea( m_minimumAreaTextField.getFloatValue( ) );
            if ( fdUnits.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
            {
                m_lastNotifiedFitParameters.setMinimumOffsetFromDisplayUnits( m_maximumOffsetTextField.getFloatValue( ) );
            }
            else
            {
                m_lastNotifiedFitParameters.setMinimumOffsetFromDisplayUnits( m_minimumOffsetTextField.getFloatValue( ) );
            }
            
            // m_lastNotifiedFitParameters.setMinimumOffsetFromDisplayUnits( m_minimumOffsetTextField.getFloatValue( ) );
            m_lastNotifiedFitParameters.setMinimumLorentzianWidth( m_minimumLorentzianWidthTextField.getFloatValue( ) );
            m_lastNotifiedFitParameters.setMinimumPhase( m_minimumPhaseTextField.getFloatValue( ) );
            m_lastNotifiedFitParameters.setMinimumGaussianWidth( m_minimumGaussWidthTextField.getFloatValue( ) );

            // Get the maximum parameter values from the graphic controls.
            m_lastNotifiedFitParameters.setMaximumArea( m_maximumAreaTextField.getFloatValue( ) );
            // m_lastNotifiedFitParameters.setMaximumOffsetFromDisplayUnits( m_maximumOffsetTextField.getFloatValue( ) );
            
            if ( fdUnits.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
            {
                m_lastNotifiedFitParameters.setMaximumOffsetFromDisplayUnits( m_minimumOffsetTextField.getFloatValue( ) );
            }
            else
            {
                m_lastNotifiedFitParameters.setMaximumOffsetFromDisplayUnits( m_maximumOffsetTextField.getFloatValue( ) );
            } 
        
            m_lastNotifiedFitParameters.setMaximumLorentzianWidth( m_maximumLorentzianWidthTextField.getFloatValue( ) );
            m_lastNotifiedFitParameters.setMaximumPhase( m_maximumPhaseTextField.getFloatValue( ) );
            m_lastNotifiedFitParameters.setMaximumGaussianWidth( m_maximumGaussWidthTextField.getFloatValue( ) );

            // Get the current priorknowledge selections of the user.
            m_lastNotifiedFitParameters.setAreaReference( (String)m_areaPriorComboBox.getSelectedItem( ) );
            m_lastNotifiedFitParameters.setOffsetReference( (String)m_offsetPriorComboBox.getSelectedItem( ) );
            m_lastNotifiedFitParameters.setLorentzianWidthReference( (String)m_lorentzianPriorComboBox.getSelectedItem( ) );
            m_lastNotifiedFitParameters.setPhaseReference( (String)m_phasePriorComboBox.getSelectedItem( ) );
            m_lastNotifiedFitParameters.setGaussianWidthReference( (String)m_gaussianPriorComboBox.getSelectedItem( ) );
        }
    }
    
    /**
     * Returns the last notified MRFittingParameters object.
     * @return 
     */
    public MRFittingParameters getLastNotifiedParameters( ) 
    {
        return m_lastNotifiedFitParameters;
    }
    
    /**
     * Calls the setMRFittingParametersOfCurrentlySelectedObject method of the
     * MRModelSpectrum object.
     */
    private void notifyValueChanged( ) 
    {
        if ( m_modelValuesChanged == true )
        {
            TDFDFitPrototypeModelSpectrumParameters modSpec = TDFDFitPrototypeModelSpectrumParameters.getInstance( );
            modSpec.setMRFittingParametersOfCurrentlySelectedObject( m_lastNotifiedFitParameters );
            m_modelValuesChanged = false;
        }
        else
        {
            // System.out.println( "Nothing changed: model not updated!" );
        }
    }
    
    /**
     * Is called whenever a actionEvent should be notified to registered objects.
     */
    private void notifyModelChangeEvent( Object obj, int id ) 
    {
        if ( id == ADD_LINE_EVENT ) 
        {
            ((TDFDFitControlFrame)m_parent).setModelChanged( true );
            notifyActionListeners( obj, ADD_LINE_EVENT, "add line node" );
            this.setMRModelSpectrum( TDFDFitPrototypeModelSpectrumParameters.getInstance( ) );
        }
        
        if ( id == ADD_USER_DEFINED_LINE_EVENT ) 
        {
            ((TDFDFitControlFrame)m_parent).setModelChanged( true );
            // System.out.println( "notifyModelChangedEvent ... ADD_USER_DEFINED_LINE_EVENT " );
            notifyActionListeners( obj, ADD_USER_DEFINED_LINE_EVENT, "add user defined line node" );
            //notifyActionListeners( obj, ADD_LINE_EVENT, "add user defined line node" );
        }
        
        if ( id == ADD_PARAMETRIC_PATTERN_EVENT ) 
        {
            ((TDFDFitControlFrame)m_parent).setModelChanged( true );
            notifyActionListeners( obj, ADD_PARAMETRIC_PATTERN_EVENT, "add parametric pattern node" );
        }
        
        if ( id == ADD_NUMERIC_PATTERN_EVENT ) 
        {
            ((TDFDFitControlFrame)m_parent).setModelChanged( true );
            notifyActionListeners( obj, ADD_NUMERIC_PATTERN_EVENT, "add numeric pattern node" );
        }
        
        if ( id == ADD_METABOLITE_EVENT ) 
        {
            ((TDFDFitControlFrame)m_parent).setModelChanged( true );
            notifyActionListeners( obj, ADD_METABOLITE_EVENT, "add metabolite node" );
        }
        
        if ( id == REMOVE_SPECTRAL_COMPONENT_EVENT ) 
        {
            ((TDFDFitControlFrame)m_parent).setModelChanged( true );
            notifyActionListeners( obj, REMOVE_SPECTRAL_COMPONENT_EVENT, "delete node" );
        }
    }
    
    /**
     * 
     * Sends notification to registered listeners.
     * 
     * @param obj
     * @param eventType
     * @param command 
     */
    private void notifyActionListeners( Object obj, int eventType,  String command ) 
    {
        Iterator it = m_actionListeners.iterator( );
        while( it.hasNext( ) ) 
        {
            ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( obj, eventType, command ) );
        }
    }
    
    /**
     * Adds an ActionListener to the list of action listeners who are notified
     * when something happens with this DecimalTextField.
     * @param e
     */
    public void addActionListener( ActionListener e ) 
    {
        m_actionListeners.add( e );
    }
    
    /**
     * Sets a boolean state whether a project has been defined or not.
     * @param state 
     */
    public void setProjectDefined( boolean state ) 
    {
        m_projectDefined = state;
        if ( state == true ) 
        {
            m_addLineButton.setEnabled( true );
            m_addParametricPatternButton.setEnabled( true );
            m_addNumericPatternButton.setEnabled( true );
            m_addMetaboliteButton.setEnabled( true );
            m_removeSpectralComponentsPanel.setEnabled( true );
        }
        else 
        {
            m_addLineButton.setEnabled( false );
            m_addParametricPatternButton.setEnabled( false );
            m_addNumericPatternButton.setEnabled( false );
            m_addMetaboliteButton.setEnabled( false );
            m_removeSpectralComponentsPanel.setEnabled( false );
        }
    }
    
    public String getCurrentlyDisplayedFrequencyUnitType( )
    {
        return m_currentlyDisplayUnitType;
    }
    
    public String getPreviouslyDisplayedFrequencyUnitType( )
    {
        return m_previouslyDisplayUnitType;
    }
    
    // Private attributes
    private final JPanel              m_treeAndComponentTypePanel;  
    private final FitModelTreePanel   m_fitModelTreePanel;
    
    private JPanel              m_modelTypePanel;
    private JLabel              m_componentTypeLabel;
    private JRadioButton        m_lineModel;
    private JRadioButton        m_parametricPatternModel;
    private JRadioButton        m_numericPatternModel;
    private JRadioButton        m_metaboliteModel;
    
    // Private ButtonGroup m_lineModelTypeButtonGroup;
    private JPanel              m_overallPanel;
    private JPanel              m_lineParameterValueAdjustmentPanel;
    private JPanel              m_spectralComponentNamePanel;
    
    private JPanel              m_labelPanel;
    private JPanel              m_valuePanel;
    
    private JLabel              m_lineNameLabel;
    private JLabel              m_newLineNameLabel;

    private JLabel              m_minimumValueLabel;
    private JLabel              m_maximumValueLabel;
 
    // Chemical shift display unit panel
    private JPanel              m_chemicalShiftDisplayUnitSelectionPanel;
    private JRadioButton        m_displayPointUnitsRadioButton;
    private JRadioButton        m_displayHzUnitsRadioButton;
    private JRadioButton        m_displayPPMUnitsRadioButton;
    private ButtonGroup         m_displayUnitsButtongroup;
    // Parameter name labels
    private JLabel              m_areaLabel;
    private JLabel              m_offsetLabel;
    private JLabel              m_phaseLabel;
    private JLabel              m_gaussWidthLabel;
    private JLabel              m_lorentzianWidthLabel;
    
    // Parameter values text fields
    private DecimalTextField    m_areaTextField;
    private DecimalTextField    m_offsetTextField;
    private DecimalTextField    m_phaseTextField;
    private DecimalTextField    m_gaussWidthTextField;
    private DecimalTextField    m_lorentzianWidthTextField;
    
    private JTextField          m_lineNameTextField;
    private JLabel              m_valueLabel;  
 
    // Minimum parameter values attributes
    private JPanel              m_minimumValuePanel;
    private DecimalTextField    m_minimumAreaTextField;
    private DecimalTextField    m_minimumOffsetTextField;
    private DecimalTextField    m_minimumPhaseTextField;
    private DecimalTextField    m_minimumGaussWidthTextField;
    private DecimalTextField    m_minimumLorentzianWidthTextField;
    
    // Maximum parameter values attributes
    private JPanel              m_maximumValuePanel;
    private DecimalTextField    m_maximumAreaTextField;
    private DecimalTextField    m_maximumOffsetTextField;
    private DecimalTextField    m_maximumPhaseTextField;
    private DecimalTextField    m_maximumGaussWidthTextField;
    private DecimalTextField    m_maximumLorentzianWidthTextField;
    
    // Prior knowledge definition panel
    private JPanel              m_priorKnowledgePanel;
    private JLabel              m_priorKnowledgePlaceHolderLabel;
    private JLabel              m_priorKnowledgeTitleLabel;
    
    // Comboboxes to store prior knowledge into
    private JComboBox           m_areaPriorComboBox;
    private JComboBox           m_offsetPriorComboBox;
    private JComboBox           m_phasePriorComboBox;
    private JComboBox           m_lorentzianPriorComboBox;
    private JComboBox           m_gaussianPriorComboBox;
    
    // Components needed for adding spectral components:
    // A spectral component can be a:
    //                          1. metabolite
    //                          2. voigt line
    //                          3. numeric pattern
    //                          4. parametric pattern
    
    JFrame                      m_parent;
    JPanel                      m_addAndRemoveSpectralComponentsPanel;
    JPanel                      m_addSpectralComponentsPanel;
    
    JButton                     m_addLineButton;
    JButton                     m_addParametricPatternButton;
    JButton                     m_addNumericPatternButton;
    JButton                     m_addMetaboliteButton;
    
    JPanel                      m_removeSpectralComponentsPanel;
    JButton                     m_removeSpectralComponentButton;
    MRFittingParameters         m_lastNotifiedFitParameters;

    // Fitting panel for model optimization
    JPanel                      m_fitControlPanel;
    JButton                     m_fitWithCurrentModelButton;
    JButton                     m_verifyCurrentModelButton;
    JButton                     m_autoSetParameterRangesButton;
    
    private LinkedList<ActionListener> m_actionListeners = new LinkedList<ActionListener>( );
    
    boolean                     m_projectDefined = false;
    
    public final static int     ADD_LINE_EVENT                  = 0;
    public final static int     ADD_PARAMETRIC_PATTERN_EVENT    = 1;
    public final static int     ADD_NUMERIC_PATTERN_EVENT       = 2;
    public final static int     ADD_METABOLITE_EVENT            = 3;
    public final static int     REMOVE_SPECTRAL_COMPONENT_EVENT = 4;
    public final static int     ADD_USER_DEFINED_LINE_EVENT     = 5;
    
    public final static int     DISPLAY_FREQ_UNITS_TYPE_CHANGED = 6;
    public final static String  DISPLAY_FREQ_UNITS_TYPE_CHANGED_CMD = "DISPLAY_FREQ_UNITS_CHANGED";
    
    public final static int     FIT_MODEL_SPECTRUM = 100;
    public final static String  FIT_MODEL_SPECTRUM_CMD = "fit model spectrum";
    // For proper serialization
    private static final long   serialVersionUID = 1482232191289213L;
    private boolean             m_modelValuesChanged = false;
    
    private String              m_currentlyDisplayUnitType;
    private String              m_previouslyDisplayUnitType;
    
    String m_deleteBinFile;
    String m_deleteFitFile;
    String m_deleteResFile;
    String m_deleteVerboseFile;
    
    /**
     * Is called whenever one of the JComboboxes raise a ItemEvent.
     * Is necessary update the model only when needed.
     * @param e 
     */
    @Override
    public void itemStateChanged( ItemEvent e )
    {
        // No matter which combobox fires the selection change: the
        // m_modelValuesChanged boolean must be set.
        if ( e.getID( ) == ItemEvent.ITEM_STATE_CHANGED )
        {
            m_modelValuesChanged = true;
            setLastNotifiedValues( );
            notifyValueChanged( );
        }
    }

    /**
     * Updates only those fields related to offset parameters.
     */
    private void updateGUIOffsetValues( )
    {
        if ( ( m_offsetTextField != null ) && ( m_lastNotifiedFitParameters != null ) )
        {
            m_offsetTextField.setFormattedText( ( new Float( m_lastNotifiedFitParameters.getOffsetInDisplayUnits( ) ).toString( ) ) );
            
            TDFDFittingDefaults defs = TDFDFittingDefaults.getInstance( );
            defs.load( );
            String fdUnits = defs.getProperty( TDFDFittingDefaults.DISPLAY_UNIT );
            
            if ( fdUnits.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
            {
                m_minimumOffsetTextField.setFormattedText( ( new Float( m_lastNotifiedFitParameters.getMaximumOffsetInDisplayUnits( ) ).toString( ) ) );
                m_maximumOffsetTextField.setFormattedText( ( new Float( m_lastNotifiedFitParameters.getMinimumOffsetInDisplayUnits( ) ).toString( ) ) );
            }
            else
            {
                m_minimumOffsetTextField.setFormattedText( ( new Float( m_lastNotifiedFitParameters.getMinimumOffsetInDisplayUnits( ) ).toString( ) ) );
                m_maximumOffsetTextField.setFormattedText( ( new Float( m_lastNotifiedFitParameters.getMaximumOffsetInDisplayUnits( ) ).toString( ) ) );
            } 
            
        }
    }
}