/*
 * File: $Id: MRSignalPropertiesPanel.java,v 1.1.1.1 2006/04/19 09:38:49 hans Exp $
 * History
 * Date: 02.06.2003 2003, 12:20    Authors: Hans Slotboom
 * Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */
 
package org.spectrim.tdfdfit;

import java.awt.BorderLayout;
import java.awt.Dimension;

import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;

import java.util.Iterator;
import java.util.LinkedList;
import javax.swing.AbstractAction;
import javax.swing.Action;

import javax.swing.BorderFactory;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.BoxLayout;
import javax.swing.Box;
import javax.swing.JLabel;
import javax.swing.JCheckBox;
import javax.swing.JTextField;
import javax.swing.JComboBox;
import javax.swing.KeyStroke;

/**
 *
 * Allows the graphic definition of the MRSignalProperties object which is 
 * required to be able to calculate the time and frequency domain representation
 * of the MRLineModelParameters
 *
 *  @author  hans
 */
public class MRSignalPropertiesPanel extends JFrame implements ActionListener
{
    /** 
     * Creates a new instance of MRSignalPropertiesPanel. 
     * @param title
     * @param parent
     * @param samplePoints
     */
    public MRSignalPropertiesPanel( String title, JFrame parent, int samplePoints )
    {
        super( );
        m_parent = parent;
        setTitle( title );
        
        m_contentPanel = new JPanel( );
        m_contentPanel.setLayout( new BorderLayout( ) );
        
        createConditionsPanel( );
        m_contentPanel.add( m_conditionsPanel, BorderLayout.NORTH );
        
        createAdvancedPanel( );
        m_contentPanel.add( m_advancedPanel, BorderLayout.CENTER );
        
        createSelectionPanel( ); 
        m_contentPanel.add( m_selectionPanel, BorderLayout.SOUTH );
        
        getRootPane( ).setDefaultButton( m_applyButton );
                  
        Action applyAndExit = new AbstractAction( ) 
        {
            @Override
            public void actionPerformed( ActionEvent e ) 
            {
                applyAndQuit( );
            }
        };

        Action exitWithoutChange = new AbstractAction( ) 
        {
            @Override
            public void actionPerformed( ActionEvent e )
            {
               cancelAndQuit( );
            }
        };

        getRootPane( ).getInputMap( ).put( KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0),
                                            "applyAndExit" );
        
        getRootPane( ).getActionMap( ).put( "applyAndExit",
                                            applyAndExit  );

        getRootPane( ).getInputMap( ).put( KeyStroke.getKeyStroke( KeyEvent.VK_ESCAPE, 0),
                                           "Cancel and quit" );
        
        getRootPane( ).getActionMap( ).put( "Cancel and quit",
                                            exitWithoutChange );
          
        setContentPane( m_contentPanel );
        
        // add window listener necessary to handle the window closing 
        // event properly
        addWindowListener( 
            new WindowAdapter( )
            {
                public void windowClosing( WindowEvent event )
                {
                   dispose( );
                }
            }
        ); 

        m_resonanceFrequencyTextField.setText( "" + MRSignalProperties.getInstance( ).getResonanceFrequency( ) );
        m_ppmValueInCenterOfSpectrumTextField.setText( "" + MRSignalProperties.getInstance( ).getPPMValueInCenterOfSpectrum( ) );
        m_sampleNumberDecimalTextField.setText( "" + MRSignalProperties.getInstance( ).getNumberOfSamplePoints( ) );
        m_sampleBandwidthDecimalTextField.setText( "" + ( MRSignalProperties.getInstance( ).getFrequencyScale( ) * MRSignalProperties.getInstance( ).getNumberOfSamplePoints( ) ) );
        m_firstOrderPhaseDecimalTextField.setText( "0" );
        m_dataTruncationDecimalTextField.setText( "" + MRSignalProperties.getInstance( ).getNumberOfSamplePoints( ) );
        m_dataToIgnoreDecimalTextField.setText( "0" );
        m_FDNoiseIntervalStartTextField.setText( "" + (int)(MRSignalProperties.getInstance( ).getNumberOfSamplePoints( ) * ( 50. / 1024. ) ) );
        m_FDNoiseIntervalStopTextField.setText( "" + (int)(MRSignalProperties.getInstance( ).getNumberOfSamplePoints( ) * ( 250. / 1024. ) ) );
        pack( );
        
        setLocationRelativeTo( null );
        setAlwaysOnTop ( true );
//        // Put the Dialog nicely in the middle of its parent
//        Point parentLoc = m_parent.getLocation( );
//        Point centerParent = new Point( m_parent.getX( ) + ( m_parent.getWidth( ) / 2 ) - this.getWidth( ) / 2,
//                                        m_parent.getY( ) + ( m_parent.getHeight( ) / 2 ) - this.getHeight( ) / 2 );
//        setLocation( centerParent );
        
        setVisible( true );
        
        MRSignalProperties props = MRSignalProperties.getInstance( );
        switch ( props.getNucleusType( ) )
        {
            case MRSignalProperties.PROTON: m_nucleusComboBox.setSelectedIndex( 0 ); break;
            case MRSignalProperties.CARBON13: m_nucleusComboBox.setSelectedIndex( 1 ); break;
            case MRSignalProperties.SODIUM23: m_nucleusComboBox.setSelectedIndex( 2 ); break;
            case MRSignalProperties.PHOSPHORUS31: m_nucleusComboBox.setSelectedIndex( 3 ); break;
            default: m_nucleusComboBox.setSelectedIndex( 0 ); break;
        }
        
        setFocusable( true );
        requestFocus( );
    }
    
    /**
     * Creates the panel allowing the definition and modification  
     * of attributes from the MRCondition class.
     */
    private void createConditionsPanel( )
    {
        m_conditionsPanel = new JPanel( );
        m_conditionsPanel.setLayout( new BoxLayout( m_conditionsPanel, BoxLayout.Y_AXIS ) );
        m_conditionsPanel.setBorder( BorderFactory.createEmptyBorder( 8, 8, 8, 8 ) );
        
        JPanel m_fitModelNamePanel = new JPanel( );
        m_fitModelNamePanel.setLayout( new BoxLayout( m_fitModelNamePanel, BoxLayout.X_AXIS ) );
        
        m_fitModelNameLabel = new JLabel( "Fit model name" );
        
        m_fitModelNameLabel.setPreferredSize( new Dimension( 180, 25 ) );
        m_fitModelNameLabel.setMinimumSize( new Dimension( 180, 25 ) );
        m_fitModelNameLabel.setMaximumSize( new Dimension( 180, 25 ) );  

        m_fitModelNameTextField = new JTextField( "SpectralModel" );
        
        m_fitModelNameTextField.setPreferredSize( new Dimension( 180, 25 ) );
        m_fitModelNameTextField.setMinimumSize( new Dimension( 180, 25 ) );
        m_fitModelNameTextField.setMaximumSize( new Dimension( 180, 25 ) );  
        
        m_fitModelNamePanel.add( m_fitModelNameLabel );
        m_fitModelNamePanel.add( m_fitModelNameTextField );
        
        m_conditionsPanel.add( m_fitModelNamePanel );
        
        m_attributePanel = new JPanel( );
        m_attributePanel.setLayout( new BoxLayout( m_attributePanel, BoxLayout.X_AXIS ) );
        
        m_attributePanel.setBorder( BorderFactory.createTitledBorder( "Basic parameters" ) );
        
        m_attributeNamePanel = new JPanel( );
        m_attributeNamePanel.setLayout( new BoxLayout( m_attributeNamePanel, BoxLayout.Y_AXIS ) );
        
        m_nucluesTypeLabel = new JLabel( "Nucleus" );
        m_nucluesTypeLabel.setPreferredSize( new Dimension( 180, 25 ) );
        m_nucluesTypeLabel.setMinimumSize( new Dimension( 180, 25 ) );
        m_nucluesTypeLabel.setMaximumSize( new Dimension( 180, 25 ) );  
        m_attributeNamePanel.add( m_nucluesTypeLabel );
        
        m_resonanceFrequencyLabel = new JLabel( "Resonance frequency" );
        m_resonanceFrequencyLabel.setPreferredSize( new Dimension( 180, 25 ) );
        m_resonanceFrequencyLabel.setMinimumSize( new Dimension( 180, 25 ) );        
        m_resonanceFrequencyLabel.setMaximumSize( new Dimension( 180, 25 ) );       
        m_attributeNamePanel.add( m_resonanceFrequencyLabel );
        
        m_ppmValueInCenterOfSpectrumLabel = new JLabel( "PPM in spectral center" );
        m_ppmValueInCenterOfSpectrumLabel.setPreferredSize( new Dimension( 180, 25 ) );
        m_ppmValueInCenterOfSpectrumLabel.setMinimumSize( new Dimension( 180, 25 ) );        
        m_ppmValueInCenterOfSpectrumLabel.setMaximumSize( new Dimension( 180, 25 ) );       
        m_attributeNamePanel.add( m_ppmValueInCenterOfSpectrumLabel );
        
        m_sampleNumberLabel = new JLabel( "Number of samples" );
        m_sampleNumberLabel.setPreferredSize( new Dimension( 180, 25 ) );
        m_sampleNumberLabel.setMinimumSize( new Dimension( 180, 25 ) );
        m_sampleNumberLabel.setMaximumSize( new Dimension( 180, 25 ) );  
        
        m_attributeNamePanel.add( m_sampleNumberLabel );
        m_sampleBandwidthLabel = new JLabel( "Acquisition bandwidth" );
        
        m_sampleBandwidthLabel.setPreferredSize( new Dimension( 180, 25 ) );
        m_sampleBandwidthLabel.setMinimumSize( new Dimension( 180, 25 ) );
        m_sampleBandwidthLabel.setMaximumSize( new Dimension( 180, 25 ) );  
        
        m_attributeNamePanel.add( m_sampleBandwidthLabel );
        
        m_firstOrederPhaseLabel = new JLabel( "First order phase" );        
        m_firstOrederPhaseLabel.setPreferredSize( new Dimension( 180, 25 ) );
        m_firstOrederPhaseLabel.setMinimumSize( new Dimension( 180, 25 ) );
        m_firstOrederPhaseLabel.setMaximumSize( new Dimension( 180, 25 ) );

        m_attributeNamePanel.add( m_firstOrederPhaseLabel );
        
        m_dataTruncationLabel = new JLabel( "Data truncation point" );
        m_dataTruncationLabel.setPreferredSize( new Dimension( 180, 25 ) );
        m_dataTruncationLabel.setMinimumSize( new Dimension( 180, 25 ) );
        m_dataTruncationLabel.setMaximumSize( new Dimension( 180, 25 ) );
        
        m_attributeNamePanel.add( m_dataTruncationLabel );
        
        m_dataToIgnoreLabel = new JLabel( "Number of points to ignore" );
        m_dataTruncationLabel.setPreferredSize( new Dimension( 180, 25 ) );
        m_dataTruncationLabel.setMinimumSize( new Dimension( 180, 25 ) );
        m_dataTruncationLabel.setMaximumSize( new Dimension( 180, 25 ) );
        
        m_attributeNamePanel.add( m_dataToIgnoreLabel );
        
        
        m_FDNoiseIntervalStartLabel = new JLabel( "FD start point # (noise estimate)" );
        m_FDNoiseIntervalStartLabel.setPreferredSize( new Dimension( 180, 25 ) );
        m_FDNoiseIntervalStartLabel.setMinimumSize( new Dimension( 180, 25 ) );
        m_FDNoiseIntervalStartLabel.setMaximumSize( new Dimension( 180, 25 ) );
        
        m_attributeNamePanel.add( m_FDNoiseIntervalStartLabel );
                
        m_FDNoiseIntervalStopLabel = new JLabel( "FD stop point # (noise estimate)" );
        m_FDNoiseIntervalStopLabel.setPreferredSize( new Dimension( 180, 25 ) );
        m_FDNoiseIntervalStopLabel.setMinimumSize( new Dimension( 180, 25 ) );
        m_FDNoiseIntervalStopLabel.setMaximumSize( new Dimension( 180, 25 ) );
        
        m_attributeNamePanel.add( m_FDNoiseIntervalStopLabel );
        
        m_attributeValuePanel = new JPanel( );
        m_attributeValuePanel.setLayout( new BoxLayout( m_attributeValuePanel, BoxLayout.Y_AXIS ) );
        m_attributeValuePanel.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );
       
        m_nucleusComboBox = new JComboBox( );
        
        m_nucleusComboBox.addItem( "<HTML><SUP>1</SUP>H</HTML>" );
        m_nucleusComboBox.addItem( "<HTML><SUP>13</SUP>C</HTML>" );
        m_nucleusComboBox.addItem( "<HTML><SUP>23</SUP>Na</HTML>" );
        m_nucleusComboBox.addItem( "<HTML><SUP>31</SUP>P</HTML>" );
        m_nucleusComboBox.setPreferredSize( new Dimension( 100, 25 ) );
        m_nucleusComboBox.setMinimumSize( new Dimension( 100, 25 ) );
        m_nucleusComboBox.setMaximumSize( new Dimension( 100, 25 ) );
        
        m_attributeValuePanel.add( m_nucleusComboBox );
        
        m_resonanceFrequencyTextField = new FormattedTextField( "000000000000", 0, Long.MAX_VALUE );
        m_resonanceFrequencyTextField.setPreferredSize( new Dimension( 100, 25 ) );
        m_resonanceFrequencyTextField.setMinimumSize( new Dimension( 100, 25 ) );
        m_resonanceFrequencyTextField.setMaximumSize( new Dimension( 100, 25 ) );
        m_attributeValuePanel.add( m_resonanceFrequencyTextField );                  
        
        m_ppmValueInCenterOfSpectrumTextField = new FormattedTextField( "0.0",-Float.MAX_VALUE, Float.MAX_VALUE );
        m_ppmValueInCenterOfSpectrumTextField.setPreferredSize( new Dimension( 100, 25 ) );
        m_ppmValueInCenterOfSpectrumTextField.setMinimumSize( new Dimension( 100, 25 ) );
        m_ppmValueInCenterOfSpectrumTextField.setMaximumSize( new Dimension( 100, 25 ) );
        m_attributeValuePanel.add( m_ppmValueInCenterOfSpectrumTextField );  
        
        m_sampleNumberDecimalTextField = new FormattedTextField( "0", 0.0, 8192.0 );
        m_sampleNumberDecimalTextField.setPreferredSize( new Dimension( 100, 25 ) );
        m_sampleNumberDecimalTextField.setMinimumSize( new Dimension( 100, 25 ) );
        m_sampleNumberDecimalTextField.setMaximumSize( new Dimension( 100, 25 ) );
        m_attributeValuePanel.add( m_sampleNumberDecimalTextField );
        
        m_sampleBandwidthDecimalTextField = new FormattedTextField( "0", 1.0, 100000.0 );
        m_sampleBandwidthDecimalTextField.setPreferredSize( new Dimension( 100, 25 ) );
        m_sampleBandwidthDecimalTextField.setMinimumSize( new Dimension( 100, 25 ) );
        m_sampleBandwidthDecimalTextField.setMaximumSize( new Dimension( 100, 25 ) );        
        m_attributeValuePanel.add( m_sampleBandwidthDecimalTextField );
        
        m_firstOrderPhaseDecimalTextField = new FormattedTextField( "0", -100000.0, 100000.0 );
        
        m_firstOrderPhaseDecimalTextField.setPreferredSize( new Dimension( 100, 25 ) );
        m_firstOrderPhaseDecimalTextField.setMinimumSize( new Dimension( 100, 25 ) );
        m_firstOrderPhaseDecimalTextField.setMaximumSize( new Dimension( 100, 25 ) );
        
        m_attributeValuePanel.add( m_firstOrderPhaseDecimalTextField );
        
        m_dataTruncationDecimalTextField = new FormattedTextField( "0", 0, 100000.0 );
        m_dataTruncationDecimalTextField.setPreferredSize( new Dimension( 100, 25 ) );
        m_dataTruncationDecimalTextField.setMinimumSize( new Dimension( 100, 25 ) );
        m_dataTruncationDecimalTextField.setMaximumSize( new Dimension( 100, 25 ) );
        m_attributeValuePanel.add( m_dataTruncationDecimalTextField );
        
        m_dataToIgnoreDecimalTextField = new FormattedTextField( "0", 0, 1024 );
        m_dataToIgnoreDecimalTextField.setPreferredSize( new Dimension( 100, 25 ) );
        m_dataToIgnoreDecimalTextField.setMinimumSize( new Dimension( 100, 25 ) );
        m_dataToIgnoreDecimalTextField.setMaximumSize( new Dimension( 100, 25 ) );
        
        m_attributeValuePanel.add( m_dataToIgnoreDecimalTextField );
        
        m_FDNoiseIntervalStartTextField = new FormattedTextField( "0", 0, 1024 );
        m_FDNoiseIntervalStartTextField.setPreferredSize( new Dimension( 100, 25 ) );
        m_FDNoiseIntervalStartTextField.setMinimumSize( new Dimension( 100, 25 ) );
        m_FDNoiseIntervalStartTextField.setMaximumSize( new Dimension( 100, 25 ) );
        
        m_attributeValuePanel.add( m_FDNoiseIntervalStartTextField );
        
        m_FDNoiseIntervalStopTextField = new FormattedTextField( "0", 0, 1024 );
        m_FDNoiseIntervalStopTextField.setPreferredSize( new Dimension( 100, 25 ) );
        m_FDNoiseIntervalStopTextField.setMinimumSize( new Dimension( 100, 25 ) );
        m_FDNoiseIntervalStopTextField.setMaximumSize( new Dimension( 100, 25 ) );
        
        m_attributeValuePanel.add( m_FDNoiseIntervalStopTextField );
        
        m_attributeUnitPanel = new JPanel( );
        m_attributeUnitPanel.setLayout( new BoxLayout( m_attributeUnitPanel, BoxLayout.Y_AXIS ) );
        
        m_nucluesTypeUnitLabel = new JLabel( "" );
        m_nucluesTypeUnitLabel.setPreferredSize( new Dimension( 160, 25 ) );
        m_nucluesTypeUnitLabel.setMinimumSize( new Dimension( 160, 25 ) );
        m_nucluesTypeUnitLabel.setMaximumSize( new Dimension( 160, 25 ) );
        
        m_attributeUnitPanel.add( m_nucluesTypeUnitLabel );
//        
        m_resonanceFrequencyUnitLabel = new JLabel( "Hz" );
        m_resonanceFrequencyUnitLabel.setPreferredSize( new Dimension( 160, 25 ) );
        m_resonanceFrequencyUnitLabel.setMinimumSize( new Dimension( 160, 25 ) );
        m_resonanceFrequencyUnitLabel.setMaximumSize( new Dimension( 160, 25 ) );
        m_attributeUnitPanel.add( m_resonanceFrequencyUnitLabel );
        
        m_ppmValueInCenterOfSpectrumUnitLabel = new JLabel( "--" );
        m_ppmValueInCenterOfSpectrumUnitLabel.setPreferredSize( new Dimension( 160, 25 ) );
        m_ppmValueInCenterOfSpectrumUnitLabel.setMinimumSize( new Dimension( 160, 25 ) );
        m_ppmValueInCenterOfSpectrumUnitLabel.setMaximumSize( new Dimension( 160, 25 ) );
        m_attributeUnitPanel.add( m_ppmValueInCenterOfSpectrumUnitLabel );
        
        m_sampleNumberUnitLabel = new JLabel( "Sample points" );
        m_sampleNumberUnitLabel.setPreferredSize( new Dimension( 160, 25 ) );
        m_sampleNumberUnitLabel.setMinimumSize( new Dimension( 160, 25 ) );
        m_sampleNumberUnitLabel.setMaximumSize( new Dimension( 160, 25 ) );
        
        m_attributeUnitPanel.add( m_sampleNumberUnitLabel );
        m_sampleBandwidthUnitLabel = new JLabel( "Hz" );
        
        m_sampleBandwidthUnitLabel.setPreferredSize( new Dimension( 160, 25 ) );
        m_sampleBandwidthUnitLabel.setMinimumSize( new Dimension( 160, 25 ) );
        m_sampleBandwidthUnitLabel.setMaximumSize( new Dimension( 160, 25 ) );  
        
        m_attributeUnitPanel.add( m_sampleBandwidthUnitLabel );
        
        m_firstOrederPhaseUnitLabel = new JLabel( "degrees/total spectrum" );        
        m_firstOrederPhaseUnitLabel.setPreferredSize( new Dimension( 160, 25 ) );
        m_firstOrederPhaseUnitLabel.setMinimumSize( new Dimension( 160, 25 ) );
        m_firstOrederPhaseUnitLabel.setMaximumSize( new Dimension( 160, 25 ) );

        m_attributeUnitPanel.add( m_firstOrederPhaseUnitLabel );
        
        m_dataTruncationUnitLabel = new JLabel( "in time domain" );
        m_dataTruncationUnitLabel.setPreferredSize( new Dimension( 160, 25 ) );
        m_dataTruncationUnitLabel.setMinimumSize( new Dimension( 160, 25 ) );
        m_dataTruncationUnitLabel.setMaximumSize( new Dimension( 160, 25 ) );
        
        m_attributeUnitPanel.add( m_dataTruncationUnitLabel );
        
        m_dataToIgnoreUnitLabel = new JLabel( "at begin of time domain signal" );
        m_dataToIgnoreUnitLabel.setPreferredSize( new Dimension( 180, 25 ) );
        m_dataToIgnoreUnitLabel.setMinimumSize( new Dimension( 180, 25 ) );
        m_dataToIgnoreUnitLabel.setMaximumSize( new Dimension( 180, 25 ) );
        
        m_attributeUnitPanel.add( m_dataToIgnoreUnitLabel );
        
        m_FDNoiseStartIntervalUnitLabel = new JLabel( "point number" );
        m_FDNoiseStartIntervalUnitLabel.setPreferredSize( new Dimension( 180, 25 ) );
        m_FDNoiseStartIntervalUnitLabel.setMinimumSize( new Dimension( 180, 25 ) );
        m_FDNoiseStartIntervalUnitLabel.setMaximumSize( new Dimension( 180, 25 ) );
        
        m_attributeUnitPanel.add( m_FDNoiseStartIntervalUnitLabel );
 
        m_FDNoiseStopIntervalUnitLabel = new JLabel( "point number" );
        m_FDNoiseStopIntervalUnitLabel.setPreferredSize( new Dimension( 180, 25 ) );
        m_FDNoiseStopIntervalUnitLabel.setMinimumSize( new Dimension( 180, 25 ) );
        m_FDNoiseStopIntervalUnitLabel.setMaximumSize( new Dimension( 180, 25 ) );        
                
        m_attributeUnitPanel.add( m_FDNoiseStopIntervalUnitLabel );                
        
        m_attributePanel.add( m_attributeNamePanel );
        m_attributePanel.add( m_attributeValuePanel );
        m_attributePanel.add( m_attributeUnitPanel );
        
        m_conditionsPanel.add( m_attributePanel );
        
    }
    
    /**
     * Creates the panel that allows the setting of advanced options of the
     * fitting calculation.
     */
    private void createAdvancedPanel( )
    {
        m_advancedPanel = new JPanel( );
        m_advancedPanel.setLayout( new BoxLayout( m_advancedPanel, BoxLayout.Y_AXIS ) );
        
        m_advancedPanel.setBorder( BorderFactory.createCompoundBorder(
                                   BorderFactory.createEmptyBorder( 8, 8, 8, 8 ),
                                   BorderFactory.createTitledBorder( "Advanced fitting" ) ) );
        
        m_fitUsesReferenceLine = new JCheckBox( "Use reference line during fit" );
        m_advancedPanel.add( m_fitUsesReferenceLine );
        m_fitUsesReferenceLine.setEnabled( false );
        m_fitDeterminesNonAnalyticLineShape = new JCheckBox( "During fit a non analytic lineshape has to be determined" );
        m_fitDeterminesNonAnalyticLineShape.setEnabled( false );
        m_advancedPanel.add( m_fitDeterminesNonAnalyticLineShape );
        
    }
    
    /** 
     * Adds an selection panel to the m_contentPanel.
     */
    private void createSelectionPanel( )
    {
        m_applyButton = new JButton( "Apply" );
        m_applyButton.addActionListener( this );
        m_applyButton.setToolTipText( "Applies the selected zerofilling factor to the currently loaded dataset" );
        m_cancelButton = new JButton( "Cancel" );
        m_cancelButton.addActionListener( this );
        m_cancelButton.setToolTipText( "Cancels this operation without applying the selected zerofilling factor" );
        
        m_selectionPanel = new JPanel( );
        
        m_selectionPanel.setLayout( new BoxLayout( m_selectionPanel, BoxLayout.X_AXIS ) );
        m_selectionPanel.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );
        m_selectionPanel.add( Box.createGlue( ) );
        m_selectionPanel.add( m_applyButton );
        m_selectionPanel.add( Box.createGlue( ) );
        m_selectionPanel.add( m_cancelButton );
        m_selectionPanel.add( Box.createGlue( ) );
    }
    
    /**
     * Called whenever a registered component raises an actionEvent.
     * @param e
     */
    @Override
    public void actionPerformed( ActionEvent e ) 
    {
        Object sender = e.getSource( );
        
        // handle responses on cancel button pressed event
        if ( sender == m_cancelButton )
        {
            cancelAndQuit( );
        }
        
        // handle responses on apply button pressed event
        if ( sender == m_applyButton )
        {
            applyAndQuit( );
        }
    }
    
    public void cancelAndQuit( )
    {
        Iterator it = m_listeners.iterator( );
        while( it.hasNext( ) )
        {
            ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, CANCEL_EVENT, "" ) );
        }
        dispose( );
    }
    
    public void applyAndQuit( )
    {
        
        MRSignalProperties.getInstance( ).setMinimumNoiseIntervalPointNumber( getMinimumNoiseIntervalPointNumber( ) );
        MRSignalProperties.getInstance( ).setMaximumNoiseIntervalPointNumber( getMaximumNoiseIntervalPointNumber( ) );
        
        Iterator it = m_listeners.iterator( );
        while( it.hasNext( ) )
        {
            ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, ACCEPT_EVENT, "" ) );
        }
        dispose( );
    }
    
    /**
     * Adds an ActionListener to the list of action listeners who are notified
     * when something happens within this object.
     * @param e
     */
    public void addActionListener( ActionListener e )
    {
	m_listeners.add( e );
    }
    
    /**
     * Returns the number of sample points.
     * @return 
     */
    public int getNumberOfSamplePoints( )
    {
        return (int)m_sampleNumberDecimalTextField.getValue( );
    }
    
    /**
     * Sets the number of sample points.
     * @param samplePoints
     */
    public void setNumberOfSamplePoints( int samplePoints )
    {
        m_sampleNumberDecimalTextField.setText( "" + samplePoints );
    }
    
    /**
     * Returns the sample bandwidth.
     * @return 
     */
    public float getSampleBandwidth( )
    {
        return (int)m_sampleBandwidthDecimalTextField.getValue( );
    }
    
    
    public int getMaximumNoiseIntervalPointNumber( )
    {
        return (int)( m_FDNoiseIntervalStopTextField.getValue( ) );
    }
    
    public int getMinimumNoiseIntervalPointNumber( )
    {
        return (int)( m_FDNoiseIntervalStartTextField.getValue( ) );
    }
    
    public void setMaximumNoiseIntervalPointNumber( int maxValue )
    {
        m_FDNoiseIntervalStopTextField.setText( "" + maxValue );
    }
    
    public void setMinimumNoiseIntervalPointNumber( int minValue )
    {
        m_FDNoiseIntervalStopTextField.setText( "" + minValue );
    }
    
    /**
     * Sets the sample bandwidth.
     * @param bandwidth
     */
    public void setSampleBandwidth( float bandwidth )
    {
        m_sampleBandwidthDecimalTextField.setText( "" + bandwidth );
    }
    
    
    /**
     * Set resonance frequency in the associated text field. 
     * @param resonancefrequency
     */
    public void setResonanceFrequency( long resonancefrequency )
    {
        this.m_resonanceFrequencyTextField.setText( "" + resonancefrequency );
    }
    
    /**
     * Returns the resonance frequency of the signal.
     * @return 
     */
    public long getResonanceFrequency( )
    {
        return new Long( m_resonanceFrequencyTextField.getText( ) );
    }
    
    /**
     * Returns the first order phase.
     * @return 
     */
    public float getFirstOrderPhase( )
    {
        return (float)m_firstOrderPhaseDecimalTextField.getValue( );
    }
    
    /**
     * Sets the first order phase.
     * @param foPhase
     */
    public void setFirstOrderPhase( float foPhase )
    {
        m_firstOrderPhaseDecimalTextField.setText( "" + foPhase );
    }
    
    
    /**
     * Returns the point after which the response calculation is stopped.
     * @return 
     */
    public int getTruncationPoint( )
    {
        return (int)m_dataTruncationDecimalTextField.getValue( );
    }

    /**
     * Sets the point after which the response calculation is stopped.
     * @param truncationPoint
     */
    public void setTruncationPoint( int truncationPoint )
    {
        m_dataTruncationDecimalTextField.setText( "" + truncationPoint );
    }
    
    /**
     * Returns the points to ignore at the beginning of the time 
     * domain signal.
     * @return 
     */
    public int getPointsToIgnore( )
    {
        return (int)m_dataToIgnoreDecimalTextField.getValue( );
    }
    
    /**
     * 
     * Returns the user set default ppm value in the center of the spectrum.
     * @return 
     */
    public float getPPMValueInCentreOfSpectrum( )
    {
        return new Float( m_ppmValueInCenterOfSpectrumTextField.getText( ) );
    }
        
    /**
     * Returns the points to ignore at the beginning of the time 
     * domain signal.
     * @param ignorePoints
     */
    public void setPointsToIgnore( int ignorePoints )
    {
        m_dataToIgnoreDecimalTextField.setText( "" + ignorePoints );
    }
    
    /**
     * Returns whether the non analytical line shape function is determined or
     * not.
     * @return 
     */
    public boolean getDetermineNonAnalyticLineShapeFuntion( )
    {
        return m_fitDeterminesNonAnalyticLineShape.isSelected( );
    }

    
    
    void selectNucleusType( String nucleusType ) 
    {
        m_nucleusComboBox.setSelectedItem( nucleusType );
    }
    
    
    /**
     * Sets whether in fit a non analytic lsf should be determined.
     * @param determine
     */
    public void setDetermineNonAnalyticLineShapeFuntion( boolean determine )
    {
        m_fitDeterminesNonAnalyticLineShape.setSelected( determine );
    }

    /**
     * Returns whether the fit uses a non analytic line shape function.
     * @return 
     */
    public boolean getFitUsesReferenceLine( )
    {
        return m_fitUsesReferenceLine.isSelected( );
    }

    /**
     * Sets whether in fit a non analytic lsf should be determined or not.
     * @param determine
     */
    public void setFitUsesReferenceLine( boolean determine )
    {
        m_fitUsesReferenceLine.setSelected( determine );
    }
    
    /**
     * Sets the name of TDFDFit model.
     * @param modelName
     */
    public void setFitModelName( String modelName )
    {
        m_fitModelNameTextField.setText( modelName );
    }
    
    /**
     * Returns the TDFDFit model name. 
     * @return Returns the TDFDFit name
     */
    public String getFitModelName(  )
    {
        return m_fitModelNameTextField.getText( );
    }
    
    // Attributes
    private final JFrame m_parent;
    private final JPanel m_contentPanel;
    private JPanel m_selectionPanel;
    
    private JButton m_applyButton;      // When pressed, sends the apply notification to the registered listeners
    private JButton m_cancelButton;     // When pressed, sends the cansel notification to the registered listeners
    
    private JPanel m_infoPanel;
    private JLabel m_infoLabel;
    
    private JPanel m_conditionsPanel;
    
    private JPanel m_attributePanel;
    
    private JPanel m_attributeNamePanel;
    
    private JLabel m_nucluesTypeLabel;
    private JLabel m_resonanceFrequencyLabel;
    private JLabel m_ppmValueInCenterOfSpectrumLabel;
    private JLabel m_sampleNumberLabel;
    private JLabel m_sampleBandwidthLabel;
    private JLabel m_firstOrederPhaseLabel;
    private JLabel m_dataTruncationLabel;
    private JLabel m_dataToIgnoreLabel;
    
    private JLabel m_FDNoiseIntervalStartLabel;
    private JLabel m_FDNoiseIntervalStopLabel;
    
    
    private JComboBox m_nucleusComboBox;
    
    private FormattedTextField m_sampleNumberDecimalTextField;
    private FormattedTextField m_resonanceFrequencyTextField;
    private FormattedTextField m_ppmValueInCenterOfSpectrumTextField;
    private FormattedTextField m_sampleBandwidthDecimalTextField;
    private FormattedTextField m_firstOrderPhaseDecimalTextField;
    private FormattedTextField m_dataTruncationDecimalTextField;
    private FormattedTextField m_dataToIgnoreDecimalTextField;
    
    private FormattedTextField m_FDNoiseIntervalStartTextField;
    private FormattedTextField m_FDNoiseIntervalStopTextField;
    
    private JLabel m_sampleNumberUnitLabel;
    private JLabel m_sampleBandwidthUnitLabel;
    private JLabel m_firstOrederPhaseUnitLabel;
    private JLabel m_dataTruncationUnitLabel;
    private JLabel m_dataToIgnoreUnitLabel;
    
    private JLabel m_FDNoiseStartIntervalUnitLabel;
    private JLabel m_FDNoiseStopIntervalUnitLabel;
    
    private JLabel m_nucluesTypeUnitLabel;
    private JLabel m_resonanceFrequencyUnitLabel;
    private JLabel m_ppmValueInCenterOfSpectrumUnitLabel;
    
    private JPanel m_attributeValuePanel;
    private JPanel m_attributeUnitPanel;
    
    private JPanel m_fitModelNamePanel;
    private JLabel m_fitModelNameLabel;
    private JTextField m_fitModelNameTextField;
    
    private JPanel m_advancedPanel;
    private JCheckBox m_fitUsesReferenceLine;
    private JCheckBox m_fitDeterminesNonAnalyticLineShape;
    
    // Listeners to changes in this class
    private LinkedList<ActionListener> m_listeners = new LinkedList<ActionListener>( );
    
    // Possible Events
    public final static int CANCEL_EVENT = 0;
    public final static int ACCEPT_EVENT = 1;
    
    // For proper serialization
    private static final long serialVersionUID = 1482114591289393L;

    
   
}
