/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.tdfdfit;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.BufferedOutputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JFrame;
import org.spectrim.data.CSIData;
import org.spectrim.lib.DualProgressBar;
import org.spectrim.model.GlobalModel;
import org.spectrim.spectra.FittedSpectra;
import org.spectrim.spectra.Spectra;
import org.spectrim.spectra.Spectrum;

/**
 * 
 * This class performs fitting of spectra using a given valid TDFDFit model 
 * file reference, on the Spectra object of the GlobalModel object;
 * and it returns the a FittedSpectra object.
 * 
 * @author Johannes Slotboom <johannes.slotboom at insel.ch>
 */
public class NonInteractiveTDFDFitExecuter implements PropertyChangeListener, ActionListener 
{
    
    /**
     * Default constructor.
     */
    public NonInteractiveTDFDFitExecuter( )
    {
        // do nothing.
    }
    
    /**
     * Constructor taking model file reference string and a CSIData object 
     * as arguments.
     * @param frame
     * @param TDFDFitModelFileReference
     * @param globModel
     * @param doesWaterRemoval
     */
    public NonInteractiveTDFDFitExecuter( JFrame frame, String TDFDFitModelFileReference,
                                          GlobalModel globModel, boolean doesWaterRemoval )
    {
        m_csiData = globModel.getCSIData( );
        m_globalModel = globModel;
        m_TDFDFitModelFileReference = TDFDFitModelFileReference;
        m_parentFrame = frame;
        m_thisFitDoesWaterRemoval = doesWaterRemoval;
    }
    
    /**
     * 
     * Starts the fitting process of the CSIData object, using the model file
     * stored in TDFDFitModelFileReference String object.
     * 
     * @param parallelProcessing
     * @return 
     */
    public boolean executeTDFDFitting( boolean parallelProcessing )
    {
        boolean processingPerformedSuccessfully = false;
       
        // Load the so-called prototype model spectrum, which is used to fit
        // the spectrum or the spectra with.
        boolean loadModelSuccesfull = loadTDFDFitPrototypeModelSpectrumFile( );
        
        if ( loadModelSuccesfull == true )
        {
            if ( parallelProcessing == true )
            {
                performMultithreadedFittingInSeparateThread( );
            }
            else
            {
                performFittingSingleThreadMode( );
            }
        }
        else
        {
            processingPerformedSuccessfully = false;
            System.out.println( "Model could not be loaded.\n Fitting could not performed." );
        }
        
        return processingPerformedSuccessfully; 
    }
    
    /**
     * Loads the TDFDFit prototype model spectrum from a file which is 
     * measured, and serves as prototype for fitting other signals with.
     */
    public boolean loadTDFDFitPrototypeModelSpectrumFile( )
    {
        if ( m_TDFDFitModelFileReference != null )
        {
            try
            {
                TDFDFitPrototypeModelSpectrumParameters.getInstance( ).resetModel( );
                TDFDFitPrototypeModelSpectrumParameters.getInstance( ).isWaterRemovalModel(  m_thisFitDoesWaterRemoval );
                RandomAccessFile reader = new RandomAccessFile( m_TDFDFitModelFileReference, "rw" );
            
                m_MRSignalProperties = MRSignalProperties.getInstance( );
                
                // Load all essential object attritubutes from this file.
                m_MRSignalProperties.read( reader );
                
                // Load the fitting strategy; if not defined you will get the 
                // default strategy.
                m_fitStrategy = MRFitStrategy.getInstance( );
                m_fitStrategy.read( reader );
                
                // Read the TDFDFitPrototypeModelSpectrum object from file.
                TDFDFitPrototypeModelSpectrumParameters.getInstance( ).read( reader );
                
                // Instantiate a SingleVoxelMeasuredSpectralData object.
                m_singleVoxelMeasuredData = new SingleVoxelMeasuredSpectralData( );
                // Read the single voxel spectral data response
                m_singleVoxelMeasuredData.read( reader, false );
                
                int index = m_TDFDFitModelFileReference.lastIndexOf( System.getProperty( "file.separator" ) );
                
                String modelName = m_TDFDFitModelFileReference.substring( index+1, m_TDFDFitModelFileReference.length( ) );
                index = modelName.lastIndexOf( "." );
                modelName = modelName.substring( 0, index );
                
                // The model spectrum must know its m_MRSignalProperties object
                TDFDFitPrototypeModelSpectrumParameters.getInstance( ).setModelSpectrumName( modelName );
                TDFDFitPrototypeModelSpectrumParameters.getInstance( ).setMRSignalProperties( m_MRSignalProperties );
                TDFDFitPrototypeModelSpectrumParameters.getInstance( ).setMeasuredSpectrum( m_singleVoxelMeasuredData );

            }
            catch ( FileNotFoundException e )
            {
                System.out.println( "Model file " + m_TDFDFitModelFileReference + " not found!" );
                return false;
            } 
        }
        return true;
    }
    
    public SingleVoxelMeasuredSpectralData getSingleVoxelMeasuredSpectralData()
    {
        return m_singleVoxelMeasuredData;
    }
    
    /**
     * Returns the string pointing to the file in which the model is stored. 
     * @return 
     */
    public String getTDFDFitModelFileReference( ) 
    {
        return m_TDFDFitModelFileReference;
    }
    
    /**
     * Sets the file reference string in which the TDFDFit is stored.
     * @param TDFDFitModelFileReference 
     */
    public void setTDFDFitModelFileReference( String TDFDFitModelFileReference ) 
    {
        m_TDFDFitModelFileReference = TDFDFitModelFileReference;
    }
    
    /**
     * Returns the CSIData object.  
     * @return 
    */
    public CSIData getCsiData( ) 
    {
        return m_csiData;
    }
    
    /**
     * Sets the CSIData object.
     * @param csiData 
     */
    public void setCsiData( CSIData csiData ) 
    {
        m_csiData = csiData;
    }
    
    
    /**
     * This method is necessary to keep SpectrIm responsive during the fitting
     * of the spectra in parallel.
     */
    public synchronized void performMultithreadedFittingInSeparateThread( )
    {
        javax.swing.SwingWorker<Void, Void> worker = new javax.swing.SwingWorker<Void, Void>( ) 
        {
            @Override
            protected Void doInBackground( ) throws Exception 
            {
                performFittingInMultiThreadMode( );
                return null;
            }

            @Override
            protected void done( ) 
            {
//                notifyActionListeners( ALL_FITTING_THREADS_DONE, ALL_FITTING_THREADS_DONE_CMD );
            }
	};
	worker.execute( );
    }
    
    
    /**
     * Collects all fitting results after fitting. 
     * @param removeWaterTDFDFit
     * @return 
     */
    public boolean collectAllFittingResults( boolean removeWaterTDFDFit )
    {
        
        m_dualProgressBar.setLowerLabel( "<HTML><BR>     Collecting resulting fit data of sub-data set number : " + 0 + "</HTML>" );
        boolean everythingOK = true;
        int cntr = 0;
        
        // First clear the object with all results 
        m_allTDFDFitData.clear( );
        
        Vector<String> componentNames = TDFDFitPrototypeModelSpectrumParameters.getInstance( ).getComponentNameVector( );
        int numberOfPoints = TDFDFitPrototypeModelSpectrumParameters.getInstance( ).getMeasuredSpectrum( ).getNumberOfSamplePoints( );
        
        for ( int i = 0; i < m_numberOfSpectraToFit; i++ )
        {
            m_dualProgressBar.setUpperProgress( 0 );
            m_dualProgressBar.setLowerProgress( i );
            m_dualProgressBar.setLowerLabel( "<HTML><BR>     Collecting fit result data of sub-data set : " + i + "</HTML>" );
            String fileNameBase = null;
            if ( !m_csiData.getManufacturer( ).contains( "Philips" ) )
            {
                fileNameBase = m_csiData.getPatientName( ).replaceAll( "�", "_" ) + "_" + m_csiData.getSeriesDescription( ) + "_" + m_csiData.getStudyDate( ) + "_";
            }
            else
            {
                fileNameBase = m_csiData.getPatientName( ).replaceAll( "�", "_" ) + "_" + m_csiData.getProtocolName( ) + "_" + m_csiData.getStudyDate( ) + "_";
            }
            
            TDFDFittingDefaults fitDef = TDFDFittingDefaults.getInstance( );
            fitDef.load( );
            
            String path = fitDef.getProperty( TDFDFittingDefaults.PATIENT_DATABASE_ROOT_PATH );
            
            if ( path.endsWith(System.getProperty( "file.separator" ) ) == false )
            {
                path = path + System.getProperty( "file.separator" );
            }

            String fileToOpen = path + fileNameBase + i + ".exl";
            
            fileToOpen = fileToOpen.replace( " ", "_" );
            // System.out.println( "fileToOpen " + fileToOpen );
            
            NumericTDFDFitResults result = new NumericTDFDFitResults( fileToOpen );
            boolean readOK = result.readFile( );
            if ( readOK == false )
            {
                everythingOK = false;
                break;
            }
            
            boolean readOK2 = result.readFittedSpectrumFile( numberOfPoints );
            if ( readOK2 == false )
            {
                everythingOK = false;
                break;
            }
            
            m_allTDFDFitData.put( m_specStringKeyVec.get( i ), result );
            
            // Set all fit quality numbers in a special data structure
            m_allFitQualityNumbers.put( m_specStringKeyVec.get( i ),  result.getFitQualityNumber( ) );
            
            if ( removeWaterTDFDFit == true )
            {
                this.m_allFittedWaterResonanceFrequencies.put( m_specStringKeyVec.get( i ), new Double( result.getFittedParametersVector( ).firstElement( ).getOffset( ) ) );
            }
            else
            {
                if ( !m_allFittedWaterResonanceFrequencies.isEmpty( ) )
                {
                    double frequency = result.getOffsetModelReferenceOf( "NAA" );
                    System.out.println( ">>> frequency of NAA fit : " + frequency );
                }
            }
            
            m_dualProgressBar.dispose( );
            
        }
        
        m_globalModel.setFitQualityNumbersForMapping( m_allFitQualityNumbers );
        
        return everythingOK;
    }
    
    /**
     * Returns the HashMap containing the offset frequencies.
     * @return 
     */
    public HashMap<String, Double> getAllFittedWaterResonanceFrequencies( )
    {
        return m_allFittedWaterResonanceFrequencies;
    }
    
    /**
     * Returns the HashMap containing the fit quality numbers.
     * @return 
     */
    public HashMap<String, Double> getAllFitQualityNumbers( )
    {
        return m_allFitQualityNumbers;
    }
    
    /**
     * This method does the actual fitting using multiple java threads.
     */
    private synchronized boolean performFittingInMultiThreadMode_OLD( ) 
    {
        
        n_cores = Runtime.getRuntime( ).availableProcessors( );
        
        ////
        // Test code
        ////
        float[] absSpectrum = null;
        float[] dispSpectrum = null;
        
        System.out.println( "Number of cores : " + n_cores );
                
        Spectra spectra = m_csiData.getSpectra( );
        
        HashMap<String, Spectrum> data =  spectra.getData( );
        Iterator it = data.entrySet( ).iterator( );
        
        Vector<String> specStringVec = new Vector<>( );
        Vector<Spectrum> specVec = new Vector<>( );
        
        while ( it.hasNext( ) )
        {
            Map.Entry pair = (Map.Entry)it.next( );
            String key = (String)pair.getKey( );
            Spectrum spectrum = (Spectrum) pair.getValue( );
            
            specVec.add( spectrum );
            specStringVec.add( key );
        }

        // Hashmap is not thread safe, first copy to a Vector<String>
        // and a Vector<Spectrum> 
        
        // Inititialize
        m_cntr = 0;
        m_numberOfThreadsActive = 0;

        // Date date = new Date();
        // System.out.println( "Timestamp : " + new Timestamp( date.getTime( ) ) );
        String fileNameBase;
        m_numberOfSpectraToFit = 32;
        
        m_dualProgressBar = new DualProgressBar( m_parentFrame, 
                                                 n_cores, 
                                                 m_numberOfSpectraToFit, 
                                                 new Dimension( 900, 150 ), true );
        
        m_dualProgressBar.setTitle( "Fitting spectra using tdfdfit .."  );
        m_dualProgressBar.setUpperLabel( "<HTML><BR>     Number of threads active : " + m_numberOfThreadsActive  + " (maximal number of threads : " + n_cores + ")</HTML>" );
        m_dualProgressBar.setUpperProgress( 0 );
        m_dualProgressBar.setUpperProgressVisible( true );
        m_dualProgressBar.setLowerLabel( "<HTML><BR>     Finished nmber of spectra : " + 0 + "</HTML>" );
          
        int dotCounter = 0;
        while ( m_cntr < m_numberOfSpectraToFit )
        {
            // magic code to get it working
            if ( dotCounter < 100000 )
            {
                dotCounter++;
            }
            else
            {
                System.out.print( "" );
                dotCounter = 0;
            }
            
            if ( m_cntr == 0 )
            {
                // System.out.println( pair.getKey( ) + " = " + pair.getValue( ) );
                Spectrum spectrum = specVec.get(m_cntr);
                absSpectrum = spectrum.getAbsorptionTD( );
                dispSpectrum = spectrum.getDispersionTD( );

                // There is a difference in internal representation of SpectrIm
                // and the representation within the TDFDFit part:
                // Therefore we must adapt the arrays to get proper represenatation.
                for ( int i=0; i < spectrum.getAbsorptionTD( ).length; i++ )
                {
                    absSpectrum[i] = absSpectrum[i];
                    dispSpectrum[i] = -dispSpectrum[i];
                }
            }

            fileNameBase = m_csiData.getPatientName( ).replaceAll( "�", "_" ) + "_" + m_csiData.getSeriesDescription( ) + "_" + m_csiData.getStudyDate( ) + "_" + m_cntr;

            if ( m_numberOfThreadsActive < n_cores )
            {

                m_numberOfThreadsActive++;
                m_cntr++;
                SingleVoxelMeasuredSpectralData dataToFit = new SingleVoxelMeasuredSpectralData( absSpectrum, dispSpectrum );
                TDFDFitExecutorSwingWorker worker = new TDFDFitExecutorSwingWorker( m_cntr, dataToFit, false, fileNameBase );
                worker.addActionListener( this );
                worker.execute( );

                m_workers.add( worker );
                
                m_dualProgressBar.setUpperProgress( m_numberOfThreadsActive );
            }
        }
        
        // Date date2 = new Date( );
        // System.out.println( "Timestamp : " + new Timestamp( date2.getTime( ) ) );
        
        return true;
    }
    
    /**
     * This method does the actual fitting using multiple java threads.
     */
    private synchronized boolean performFittingInMultiThreadMode( ) 
    {
        
        startComputationTime = System.currentTimeMillis( );
        n_cores = Runtime.getRuntime( ).availableProcessors( );
        
        ////
        // Test code
        ////
        float[] absSpectrum = null;
        float[] dispSpectrum = null;
        
        // System.out.println( "Number of cores          : " + n_cores );
                
        Spectra spectra = m_csiData.getSpectra( );
        
        // System.out.println( "Number of spectra to fit : " + spectra.getVectorSize( ) );
        
        HashMap<String, Spectrum> data =  spectra.getData( );
        Iterator it = data.entrySet( ).iterator( );
        
        
        // Hashmap is not thread safe, first copy to a Vector<String>
        // and a Vector<Spectrum> 
        
        m_specStringKeyVec = new Vector<>( );
        m_specVec = new Vector<>( );
        ArrayList<String> selVoxels = m_globalModel.getSelectedVoxels( );
        
        for ( int i = 0; i < selVoxels.size( ); i++ )
        {
            String key = selVoxels.get( i );
            Spectrum spectrum = data.get( selVoxels.get( i ) );
            
            m_specVec.add( spectrum );
            m_specStringKeyVec.add( key );
        }
        
        m_numberOfSpectraToFit = m_specVec.size( );
                
        // Inititialize
        m_cntr = 0;
        m_numberOfThreadsActive = 0;
  
        // Date date = new Date( );
        // System.out.println( "Timestamp : " + new Timestamp( date.getTime( ) ) );
        String fileNameBase;
        
        m_dualProgressBar = new DualProgressBar( m_parentFrame, 
                                                 n_cores, 
                                                 m_numberOfSpectraToFit, 
                                                 new Dimension( 900, 150 ), true );
        
        m_dualProgressBar.setTitle( "Fitting spectra using tdfdfit .."  );
        m_dualProgressBar.setUpperLabel( "<HTML><BR>     Number of threads active : " + m_numberOfThreadsActive  + " (maximal number of threads : " + n_cores + ")</HTML>" );
        m_dualProgressBar.setUpperProgress( 0 );
        //m_dualProgressBar.setUpperLabel( "        Total progress " );
        m_dualProgressBar.setUpperProgressVisible( true );
        m_dualProgressBar.setLowerLabel( "<HTML><BR>     Finished number of spectra : " + 0 + "</HTML>" );
        
        m_dualProgressBar.addActionListener( this );
        
        int dotCounter = 0;
        
        while ( m_cntr < m_numberOfSpectraToFit )
        {            
            // System.out.println( pair.getKey( ) + " = " + pair.getValue( ) );
            Spectrum spectrum = m_specVec.get( m_cntr );
            absSpectrum = spectrum.getAbsorptionTD( );
            dispSpectrum = spectrum.getDispersionTD( );

            // There is a difference in internal representation of SpectrIm
            // and the representation within the TDFDFit part:
            // Therefore we must adapt the arrays to get proper represenatation.
            for ( int i = 0; i < spectrum.getAbsorptionTD( ).length; i++ )
            {
                absSpectrum[i] = absSpectrum[i];
                dispSpectrum[i] = -dispSpectrum[i];
            }
            
            if ( m_csiData.getManufacturer( ).contains( "Philips" ) == true )
            {
                fileNameBase = m_csiData.getPatientName( ) + "_" + m_csiData.getProtocolName( ) + "_" + m_csiData.getStudyDate( ) + "_" + m_cntr;
            }
            else
            {
                fileNameBase = m_csiData.getPatientName( ) + "_" + m_csiData.getSeriesDescription( ) + "_" + m_csiData.getStudyDate( ) + "_" + m_cntr;
            }
            fileNameBase = fileNameBase.replaceAll( "�", "_" );
            fileNameBase = fileNameBase.replace( " ", "_" );
            
            if ( m_numberOfThreadsActive < n_cores )
            {

                m_numberOfThreadsActive++;
                SingleVoxelMeasuredSpectralData dataToFit = new SingleVoxelMeasuredSpectralData( absSpectrum, dispSpectrum );
                TDFDFitExecutorSwingWorker worker = new TDFDFitExecutorSwingWorker( m_cntr, dataToFit, false, fileNameBase );
                m_workers.add( worker );
                m_cntr++;
                m_numberOfFitTreadsStarted++;
                worker.addActionListener( this );
                worker.execute( );
                
                m_dualProgressBar.setUpperProgress( m_numberOfThreadsActive );
            }
        }
        
        // Date date2 = new Date( );
        // System.out.println( "Timestamp : " + new Timestamp( date2.getTime( ) ) );
         
        return true;
    }
    
    /**
     * This method does the actual fitting using one single additional 
     * java threads.
     */
    private boolean performFittingSingleThreadMode( ) 
    {

        Spectra spectra = m_csiData.getSpectra( );
        
        HashMap<String, Spectrum> data =  spectra.getData( );
        Iterator it = data.entrySet( ).iterator( );
        
        int cntr = 0;
        m_numberOfThreadsActive = 1;
        
        while ( it.hasNext( ) ) 
        {
            if ( m_numberOfThreadsActive < m_currentNumberOfParallelThreads )
            {
                Map.Entry pair = (Map.Entry)it.next( );

                Spectrum spectrum = (Spectrum) pair.getValue( );
                float[] absSpectrum = spectrum.getAbsorptionTD( );
                float[] dispSpectrum = spectrum.getDispersionTD( );

                // There is a difference in internal representation of SpectrIm
                // and the representation within the TDFDFit part:
                // Therefore we must adapt the arrays to get proper represenatation.
                for ( int i = 0; i < spectrum.getAbsorptionTD( ).length; i++ )
                {
                    absSpectrum[i] = absSpectrum[i];
                    dispSpectrum[i] = -dispSpectrum[i];
                }
                System.out.println( "CISDATA file name : " + m_csiData.getPatientName( ) + "_" + m_csiData.getSeriesDescription( ) + "_" + m_csiData.getStudyDate( ) + "_" + cntr );
                String fileNameBase = m_csiData.getPatientName( ).replaceAll( "�", "_" ) + "_" + m_csiData.getSeriesDescription( ) + "_" + m_csiData.getStudyDate( ) + "_" + cntr;
                
                SingleVoxelMeasuredSpectralData dataToFit = new SingleVoxelMeasuredSpectralData( absSpectrum, dispSpectrum );
                TDFDFitExecutorSwingWorker worker = new TDFDFitExecutorSwingWorker( cntr, dataToFit, true, fileNameBase );                
                worker.addActionListener( this );
                m_workers.add( worker );
                
                worker.execute( );
                
                m_numberOfThreadsActive++;
            }
        }
        
        return true;   
    }
    
    /**
     * If a started thread ends, the value that indicates the number of running
     * threads is decremented.
     * @param e 
     */
    @Override
    public void actionPerformed( ActionEvent e ) 
    {
        
        if ( e.getSource( ) == m_dualProgressBar )
        {
            System.out.println( "Force processing to quit pressed! .." );
            // test code
            notifyActionListeners( ALL_FITTING_THREADS_DONE, ALL_FITTING_THREADS_DONE_CMD );
            m_dualProgressBar.setVisible(false);
            m_forceProcessingToQuit = true;
        }
        
        if ( e.getSource( ) instanceof TDFDFitExecutorSwingWorker )
        {
            if ( e.getID( ) == TDFDFitExecutorSwingWorker.IS_PERFORMED )
            {
                m_isPerformedMethodCalled++;
                
                int number = ( (TDFDFitExecutorSwingWorker)e.getSource( ) ).getTDFDFitThreadID( );
                NumericTDFDFitResults result = ( (TDFDFitExecutorSwingWorker)e.getSource( ) ).getNumericTDFDFitResults( );
                m_allTDFDFitData.put( m_specStringKeyVec.get( number ), result );
                
                m_numberOfThreadsActive--;
                
                m_dualProgressBar.setLowerProgress( m_cntr - m_numberOfThreadsActive );
                m_dualProgressBar.setUpperProgress( m_numberOfThreadsActive );
                
                m_dualProgressBar.setUpperLabel( "<HTML><BR>     Number of threads active     : " + m_numberOfThreadsActive  + " (maximal number of threads : " + n_cores + ")</HTML>" );
                m_dualProgressBar.setLowerLabel( "<HTML><BR>     Finished number of spectra : " + ( m_cntr - m_numberOfThreadsActive ) + "</HTML>" );
                
                m_workers.remove( (TDFDFitExecutorSwingWorker)e.getSource( ) );

                if (  m_isPerformedMethodCalled == m_numberOfSpectraToFit  )    
                {
                    long allWorkersStartedTime = System.currentTimeMillis( );
                    long computationTime = allWorkersStartedTime - startComputationTime;
                    long timePerFit = computationTime / ( m_numberOfSpectraToFit - m_workers.size( ) );
                    int aliveWorkers =  m_workers.size( );
                     
                    try 
                    {
                        Thread.sleep( ( m_workers.size( ) + 6 ) * 150 );
                        // After waiting terminate the workers
                        for ( int i = 0; i < aliveWorkers; i++ )
                        {
                            TDFDFitExecutorSwingWorker worker = m_workers.elementAt(i);
                            worker.cancel( true );
                        }
                        
                        m_dualProgressBar.dispose( );
                        notifyActionListeners( ALL_FITTING_THREADS_DONE, ALL_FITTING_THREADS_DONE_CMD );
                       
                    } 
                    catch ( InterruptedException ex )
                    {
                        Logger.getLogger( NonInteractiveTDFDFitExecuter.class.getName( ) ).log( Level.SEVERE, null, ex );
                    }
                    m_dualProgressBar.dispose( );
                }
            }
        }
    }
    
    
     /**
     * Adds an ActionListener to the list of action listeners who are notified
     * when something happens in this component.
     * @param e
     */
    public void addActionListener( ActionListener e ) 
    {
        m_actionListeners.add( e );
    }
    
    /**
     * Notifies registered ActionListeners.
     * @param eventType
     * @param actionCommand
     */
    private void notifyActionListeners( int eventType, String actionCommand )
    {
        Iterator it = m_actionListeners.iterator( );
        while( it.hasNext( ) )
        {
            ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, eventType, actionCommand ) );
        }
    }
    
    /**
     * Puts the computed fitting results into the GlobalModel.
     */
    public void putFittedSpectraIntoGlobalModel( ) 
    {
        
        if ( m_globalModel != null ) 
        {
            Spectra globModelSpectra = m_globalModel.getSpectra( );
            FittedSpectra fittedSpectra = globModelSpectra.cloneAsFittedSpectra( );

            // Iterate over all entries...                    
            for ( String key : m_globalModel.getCSIData( ).getSpectra( ).getData( ).keySet( ) )
            {
                String[] coord = key.split( "_" );
                if ( coord.length == 3 )
                {
                    if ( m_allTDFDFitData.get( key ) != null )
                    {
                        NumericTDFDFitResults result = m_allTDFDFitData.get( key );
                    
                        float[] abs = result.getFittedSpectrumRealTD( );
                        float[] disp = result.getFittedSpectrumImagTD( );
                    
                        // do complex conjugation ..
                        for ( int i = 0; i < disp.length; i++)
                        {
                            disp[i] = -disp[i];
                        }
                        
                        fittedSpectra.setSpectrum( abs, disp, key );
                    }
                }
            }
            
            m_globalModel.setFittedSpectra( fittedSpectra );
            m_globalModel.setTDFDFitResultsStore( m_allTDFDFitData, null, null );
            m_globalModel.updateViewSelectionChange( );
        }
    }
    
    /**
     * Puts the preprocessed data into the results store.
     */
    public void putPreprocessedSpectraIntoResultsStore( ) 
    {
        
        if ( m_globalModel != null ) 
        {
            // Iterate over all entries...        
            for ( String key : m_globalModel.getCSIData( ).getSpectra( ).getData( ).keySet( ) )
            // for ( String key : m_allTDFDFitData.keySet( ) ) 
            {
                String[] coord = key.split( "_" );
                if ( coord.length == 3 )
                {
                    
                    if ( m_allTDFDFitData.get( key ) != null )
                    {
                        NumericTDFDFitResults result = m_allTDFDFitData.get( key );
                    
                        Spectrum spec = m_globalModel.getCSIData( ).getSpectra( ).getData( ).get( key );
                    
                        float[] abs = spec.getAbsorptionTD( );
                        float[] disp = spec.getDispersionTD( );
                                        
                        result.setPreprocessedInputSignalRealTD( abs );
                        result.setPreprocessedInputSignalImagTD( disp );
                        
                    }
                }
            }
        }
    } 
    
    /**
     * Puts the computed fitting results into the GlobalModel.
     */
    public void subtractFittedSpectraIntoGlobalModel( ) 
    {
        
        if ( m_globalModel != null ) 
        {
            
            Spectra globModelSpectra = m_globalModel.getSpectra( );
            FittedSpectra fittedSpectra = globModelSpectra.cloneAsFittedSpectra( );
            
            // Iterate over all entries...        
            for ( String key : m_allTDFDFitData.keySet( ) ) 
            {
                String[] coord = key.split( "_" );
                if ( coord.length == 3 )
                {
                    // Get the measured spectrum
                    float[] measRe = globModelSpectra.getSpectrumData( key, Spectra.Domain.TD , Spectra.Mode.ABS );
                    float[] measIm = globModelSpectra.getSpectrumData( key, Spectra.Domain.TD , Spectra.Mode.DISP );
                    
                    // Returns the TDFDFitResults 
                    NumericTDFDFitResults result = m_allTDFDFitData.get( key );
                    
                    float[] abs = result.getFittedSpectrumRealTD( );
                    float[] disp = result.getFittedSpectrumImagTD( );
                    
                    
                    for ( int i = 0; i < disp.length; i++ )
                    {
                        measRe[i] = measRe[i] - abs[i];
                        measIm[i] = measIm[i] - disp[i];
                        abs[i] = 0f;
                        disp[i] = 0f;
                    }
                    
                    fittedSpectra.setSpectrum( abs, disp, key ); 
                    globModelSpectra.setSpectrum( measRe, measIm, key );
                    
                    SingleVoxelMeasuredSpectralData data = new SingleVoxelMeasuredSpectralData( measRe, measIm );
                    
                    float[] absSpec = data.getAbsorptionChannelFD( );
                    float[] dispSpec = data.getDispersionChannelFD( );
                    
                    float meanAbs = 0f; 
                    float meanDisp = 0f;
                    
                    for ( int i = 0; i < absSpec.length/8; i++ )
                    {
                        meanAbs += absSpec[i]/( absSpec.length/8 );
                        meanDisp += dispSpec[i]/( absSpec.length/8 );
                    }
                    
                    for ( int i = 0; i < absSpec.length; i++ )
                    {
                        absSpec[i] = absSpec[i] - meanAbs;
                        dispSpec[i] = dispSpec[i] - meanDisp;
                    }
                    
                    data.setSpectralDataPointsFD(absSpec, dispSpec);
                    
                    absSpec = data.getAbsorptionChannelTD( );
                    dispSpec = data.getDispersionChannelTD( );
                    globModelSpectra.setSpectrum( absSpec, dispSpec, key );
                }
            }
            
            m_globalModel.setSpectra( globModelSpectra );
            m_globalModel.setFittedSpectra( fittedSpectra );
            m_globalModel.updateViewSelectionChange( );
        }
    }
    
       
    /**
     * This method puts all .fit.bin files in one big binary file. 
     */
    public void reduceNumberOfTFDFitResultDotFitFiles( ) 
    {
        
        // Determine the size of the collection .fit file.
        int arraySize = m_globalModel.getCSIGrid( ).m_gridSizeDir1 * m_globalModel.getCSIGrid( ).m_gridSizeDir2 * m_globalModel.getCSIGrid( ).m_gridSizeDir3;
        arraySize = 2 * m_MRSignalProperties.getNumberOfSamplePoints( ) * arraySize;
        
        int cntr = 0;
        
        float[] allComplexDotFitSignals = new float[ arraySize ];
        
        if ( m_globalModel != null ) 
        {
            for ( int i = 0; i < m_globalModel.getCSIGrid( ).m_gridSizeDir1 ; i++ )
            {
                for ( int j = 0; j < m_globalModel.getCSIGrid( ).m_gridSizeDir2 ; j++ )
                {
                    for ( int k = 0; k < m_globalModel.getCSIGrid( ).m_gridSizeDir3 ; k++ )
                    {
                        String key = "" + i + "_" + j + "_" + k;
                        NumericTDFDFitResults result = m_allTDFDFitData.get( key );
                        if ( result != null )
                        {
                            float[] abs = result.getFittedSpectrumRealTD( );
                            float[] disp = result.getFittedSpectrumImagTD( );
                    
                            // Copy the TD data into the global data array.
                            for ( int ii = 0; ii < m_MRSignalProperties.getNumberOfSamplePoints( ); ii++ )
                            {
                                allComplexDotFitSignals[cntr] = abs[ii];
                                cntr++;
                                allComplexDotFitSignals[cntr] = -disp[ii];
                                cntr++;
                            }
                        }
                        else
                        {
                            for ( int ii = 0; ii < m_MRSignalProperties.getNumberOfSamplePoints( ); ii++ )
                            {
                                allComplexDotFitSignals[cntr] = 0f;
                                cntr++;
                                allComplexDotFitSignals[cntr] = 0f;
                                cntr++;
                            }
                        }
                    }
                }
            }

            TDFDFittingDefaults tdfdfFittingDefaults = TDFDFittingDefaults.getInstance( );
            tdfdfFittingDefaults.load( );
            
            String outputPath = tdfdfFittingDefaults.getProperty( TDFDFittingDefaults.PATIENT_DATABASE_ROOT_PATH );
            String outputFileName = m_csiData.getPatientName( ).replaceAll( "�","_" ) + "_" + 
                                    m_csiData.getProtocolName( ) + "_" + 
                                    m_csiData.getStudyDate( ) + "_overall.fit.bin";

            writeHugeFloatArrayToDisk( allComplexDotFitSignals, outputPath + System.getProperty( "file.separator" ) + outputFileName );
  
        }
    }
    
    /**
     * This is a fast method to write a huge amount of floating point data into
     * binary file.
     * @param hugeArray
     * @param pathAndFileName
     * @return 
     */
    private boolean writeHugeFloatArrayToDisk( float[] hugeArray, String pathAndFileName )
    {
        try 
        {
            BufferedOutputStream  dataOut = new BufferedOutputStream( new FileOutputStream( pathAndFileName ) );
 
            byte buf[]=new byte[4*hugeArray.length];

            long timeBeforeWrite = System.nanoTime( );

            for ( int i = 0; i < hugeArray.length; ++i )
            {
                int val = Float.floatToRawIntBits(hugeArray[i]);
                buf[4 * i] = (byte) (val >> 24);
                buf[4 * i + 1] = (byte) (val >> 16) ;
                buf[4 * i + 2] = (byte) (val >> 8);
                buf[4 * i + 3] = (byte) (val);
            }

            dataOut.write( buf );

            long ct_write = System.nanoTime( );
            long offset_write = ct_write - timeBeforeWrite;
            float mOffsetInMs_write = (float)(offset_write)/1000000;
            // System.out.println( "Time te write huge array : " + Float.toString( mOffsetInMs_write )  + " msec " );
            dataOut.flush( );
            dataOut.close( );
        } catch ( IOException ex ) 
        {
            System.err.println( ex.getMessage( ) );
            return false;
        }
        return true;
    } 
     
    
    // Attributes
    private String                          m_TDFDFitModelFileReference;
    private CSIData                         m_csiData;
    private JFrame                          m_parentFrame;
    private boolean                         m_thisFitDoesWaterRemoval = false;
    private MRSignalProperties              m_MRSignalProperties;
    private MRFitStrategy                   m_fitStrategy;
    private SingleVoxelMeasuredSpectralData m_singleVoxelMeasuredData;
    private int                             m_currentNumberOfParallelThreads;
    private int                             m_numberOfThreadsActive = 1;
    private int                             m_isPerformedMethodCalled = 0;
    private  Vector<TDFDFitExecutorSwingWorker>    m_workers = new Vector<TDFDFitExecutorSwingWorker>( );
    private DualProgressBar                 m_dualProgressBar;
    private int                             m_cntr = 0;
    private int                             n_cores = 1; // at least one core should be present :-))
    private int                             m_numberOfSpectraToFit;
    private int                             m_numberOfFitTreadsStarted = 0;
    private Vector<ActionListener>          m_actionListeners = new Vector<ActionListener>( );
    
    public final static int                 ALL_FITTING_THREADS_DONE = 0;
    public final static String              ALL_FITTING_THREADS_DONE_CMD = "ALL_FITTING_THREADS_DONE";
    
    private HashMap<String,NumericTDFDFitResults> m_allTDFDFitData = new HashMap<String, NumericTDFDFitResults>( );
    // Save water reference position in ppm.
    private HashMap<String, Double>         m_allFittedWaterResonanceFrequencies = new HashMap<String, Double>( );
    private HashMap<String, Double>         m_allFitQualityNumbers = new HashMap<String, Double>( );
    
    private GlobalModel                     m_globalModel;
    
    Vector<String>                          m_specStringKeyVec = new Vector<>( );
    Vector<Spectrum>                        m_specVec          = new Vector<>( );
    private boolean                         m_forceProcessingToQuit = false;
    
    long startComputationTime;
    
    @Override
    public void propertyChange( PropertyChangeEvent evt ) 
    {
        System.out.println( "propertyChanged is called" );
    }

   
}
