/*
 * OrderStatisticsFilterDefinitionControl.java
 *
 * Created on July 14, 2005, 7:21 AM
 */

package org.spectrim.tdfdfit;

import java.awt.BorderLayout;
import java.awt.Point;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;


import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JRadioButton;
import javax.swing.ButtonGroup;
import javax.swing.JCheckBox;

import java.util.LinkedList;
import java.util.Iterator;


import org.spectrim.lib.FormattedTextField;

/**
 *
 * @author  hans
 */
public class OrderStatisticsFilterDefinitionControl extends JFrame implements ActionListener
{
    
    /** 
     * Creates a new instance of OrderStatisticsFilterDefinitionControl 
     */
    public OrderStatisticsFilterDefinitionControl( String title, JFrame parent, int minValue, int maxValue, 
                                                   int startMinValue, int startMaxValue, boolean denoise )
    {
        m_parent = parent;
        setTitle( title );
        
        m_contentPanel = new JPanel( );
        m_contentPanel.setLayout( new BorderLayout( ) );
        
        // add window listener necessary to handle the window closing 
        // event properly
        addWindowListener( 
            new WindowAdapter( )
            {
                public void windowClosing( WindowEvent event )
                {
                   dispose( );
                }
            }
        );

        instantiateOrderStatisticsParameterPanel( minValue, maxValue, startMinValue, startMaxValue, denoise );
        
        m_contentPanel.add( m_orderStatisticsParameterPanel, BorderLayout.CENTER );
        
        instantiateCancelApplyPanel( );
        
        m_contentPanel.add( m_selectionPanel, BorderLayout.SOUTH );
        
        setContentPane( m_contentPanel );   
        
        pack( );
        // Put the Dialog nicely in the middle of its parent
        Point parentLoc = m_parent.getLocation( );
        Point centerParent = new Point( m_parent.getX( ) + ( m_parent.getWidth( ) / 2 ) - this.getWidth( ) / 2,
                                        m_parent.getY( ) + ( m_parent.getHeight( ) / 2 ) - this.getHeight( ) / 2 );
        setLocation( centerParent );
        setVisible( true );
    }
    
    
    /**
     * Instantiates the panel containing the controls that allows the user
     * in which domain filtering is required and whether the offset has to be
     * removed and in which region pure noise can be assumed
     */
    private void instantiateOrderStatisticsParameterPanel( int min, int max, 
                                                           int startMin, int startMax,
                                                           boolean denoise )
    {
        m_orderStatisticsParameterPanel = new JPanel( );
        m_orderStatisticsParameterPanel.setLayout( new BoxLayout( m_orderStatisticsParameterPanel, BoxLayout.Y_AXIS ) );
        m_orderStatisticsParameterPanel.setBorder( BorderFactory.createCompoundBorder(
                                                   BorderFactory.createEmptyBorder( 10, 10, 10, 10 ),
                                                   BorderFactory.createTitledBorder( "OSF parameters" ) )
                                          );
       
        m_domainPanel = new JPanel( );
        m_domainPanel.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5  ) );
        m_domainPanel.setLayout( new BoxLayout( m_domainPanel, BoxLayout.X_AXIS ) );
        m_domainLabel = new JLabel( "Filter Domain" );
        
        m_timeDomainButton = new JRadioButton( "Time domain" );
        m_frequencyDomainButton = new JRadioButton( "Frequency domain" );
        m_frequencyDomainButton.setSelected( true );
        
        m_domainButtonGroup = new ButtonGroup( );
        
        m_domainButtonGroup.add( m_frequencyDomainButton );
        m_domainButtonGroup.add( m_timeDomainButton );
        
        m_domainPanel.add( Box.createHorizontalGlue( ) );
        m_domainPanel.add( m_domainLabel );
        m_domainPanel.add( Box.createHorizontalGlue( ) );
        m_domainPanel.add( m_timeDomainButton );
        m_domainPanel.add( Box.createHorizontalGlue( ) );
        m_domainPanel.add( m_frequencyDomainButton );
        m_domainPanel.add( Box.createHorizontalGlue( ) );
        
        m_orderStatisticsParameterPanel.add( m_domainPanel );
        
        
        m_offsetsPanel = new JPanel( );
        
        m_offsetsPanel.setBorder( BorderFactory.createEmptyBorder( 0, 5, 0, 5  ) );
        m_offsetsPanel.setLayout( new BoxLayout( m_offsetsPanel, BoxLayout.X_AXIS ) );
        
        m_eliminateOffsetCheckBox = new JCheckBox( "Eliminate offsets" );
        m_eliminateOffsetCheckBox.addActionListener( this );
        m_eliminateOffsetCheckBox.setSelected( true );
        
        m_offsetsPanel.add( m_eliminateOffsetCheckBox );
        m_offsetsPanel.add( Box.createHorizontalGlue( ) );
        
        m_minMaxPanel = new JPanel( );
        
        m_minLabel = new JLabel( "Minimum" );
        m_minimumValueFTF = new FormattedTextField( "0", (double)min, (double)max );
        m_minimumValueFTF.setToolTipText( "Array index value indicating start of pure noise band" );
        m_minimumValueFTF.setValue( (double)startMin );
        m_maxLabel = new JLabel( "Maximum" );
        m_maximumValueFTF = new FormattedTextField( "0", (double)min, (double)max ); 
        m_maximumValueFTF.setToolTipText( "Array index value indicating end of pure noise band" );
        
        m_maximumValueFTF.setValue( (double)startMax );
        
        m_minMaxPanel.setBorder( BorderFactory.createEmptyBorder( 0, 5, 0, 5  ) );
        m_minMaxPanel.setLayout( new BoxLayout( m_minMaxPanel, BoxLayout.X_AXIS ) );
        
        m_minMaxPanel.add( Box.createHorizontalGlue( ) );
        m_minMaxPanel.add( m_minLabel );
        m_minMaxPanel.add( Box.createHorizontalGlue( ) );
        m_minMaxPanel.add( m_minimumValueFTF );
        m_minMaxPanel.add( Box.createHorizontalGlue( ) );
        m_minMaxPanel.add( m_maxLabel );
        m_minMaxPanel.add( Box.createHorizontalGlue( ) );
        m_minMaxPanel.add( m_maximumValueFTF ); 
        m_minMaxPanel.add( Box.createHorizontalGlue( ) );
        
        m_orderStatisticsParameterPanel.add( m_offsetsPanel );
        m_orderStatisticsParameterPanel.add( m_minMaxPanel );
        
        m_previewButtonPanel = new JPanel( );
        m_previewButtonPanel.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5  ) );
        m_previewButtonPanel.setLayout( new BoxLayout( m_previewButtonPanel, BoxLayout.X_AXIS ) );
        m_previewButton = new JButton( "Preview Filter Effect" );
        m_previewButton.addActionListener( this );
        
        m_previewButtonPanel.add( Box.createHorizontalGlue( ) );
        m_previewButtonPanel.add( m_previewButton );
        m_previewButtonPanel.add( Box.createHorizontalGlue( ) );
        
        m_orderStatisticsParameterPanel.add( m_previewButtonPanel );
        
        m_denoise2DResponse = denoise;
    }
    
    /**
     * Instantiates an apply/ cancel panel
     */
    private void instantiateCancelApplyPanel( )
    {
        // Selection panel definitions
        m_applyButton = new JButton( "Apply & Quit" );
        m_applyButton.setEnabled( false );
        m_applyButton.addActionListener( this );
        m_applyButton.setToolTipText( "Applies the selected os-filtering to the currently loaded dataset" );
        m_cancelButton = new JButton( "Cancel & Quit" );
        m_cancelButton.addActionListener( this );
        m_cancelButton.setToolTipText( "Cancels the operation without applying the os-filtering" );
        
        m_selectionPanel = new JPanel( );
        
        m_selectionPanel.setLayout( new BoxLayout( m_selectionPanel, BoxLayout.X_AXIS ) );
        m_selectionPanel.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );
        m_selectionPanel.add( Box.createGlue( ) );
        m_selectionPanel.add( m_applyButton );
        m_selectionPanel.add( Box.createGlue( ) );
        m_selectionPanel.add( m_cancelButton );
        m_selectionPanel.add( Box.createGlue( ) );
    }
    
    /**
     * Is called whenever a registered component raises an ActionEvent
     */
    public void actionPerformed( ActionEvent e ) 
    {  
        Object sender = e.getSource( );
        
        // handle responses on cancel button pressed event
        if ( sender == m_cancelButton )
        {
            Iterator it = m_listeners.iterator( );
             while( it.hasNext( ) )
             {
                ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, CANCEL_EVENT, "" ) );
             }
            dispose( );
        }
        // handle responses on apply button pressed event
        if ( sender == m_applyButton )
        {
            Iterator it = m_listeners.iterator( );
             while( it.hasNext( ) )
             {
                ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, ACCEPT_EVENT, "" ) );
             }
            dispose( );
        }    
        
        if ( sender ==  m_eliminateOffsetCheckBox )
        {
            if ( m_eliminateOffsetCheckBox.isSelected( ) )
            {
                m_minimumValueFTF.setEnabled( true );
                m_maximumValueFTF.setEnabled( true );
            }
            else
            {
                m_minimumValueFTF.setEnabled( false );
                m_maximumValueFTF.setEnabled( false );
            }
        }
        
        if ( sender == m_previewButton )
        {
            Iterator it = m_listeners.iterator( );
            while( it.hasNext( ) )
            {
                ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, PREVIEW_OSF_EVENT, "" ) );
            }
            m_applyButton.setEnabled( true );
        }
    }
    
    /**
     * Adds an actionlistener to the list of action listeners who are notified
     * when something happens within this object
     */
    public void addActionListener( ActionListener e )
    {
        m_listeners.add( e );
    }
    
    /**
     * Returns whether the baseline has to be reoved or not
     */
    public boolean removeBaseline( )
    {
        if ( m_eliminateOffsetCheckBox.isSelected( ) == true )
        {
            return true;
        }
        else
        {
            return false;
        }
    }
    
    /**
     * Returns whether the baseline has to be reoved or not
     */
    public int getApplicationDomain( )
    {
        if ( m_timeDomainButton.isSelected( ) == true )
        {
            return TIME_DOMAIN;
        }
        else
        {
            return FREQUENCY_DOMAIN;
        }
    }
    
    /**
     *  Denoise the 2D signal matrix
     */
    public boolean getDenoise2DResponse( )
    {
        return m_denoise2DResponse;
    }
    
    /**
     * Returns the minimum value of the range that determines offset 
     * calculation range
     */
    public int getMinimumRangeValue( )
    {
        return (int)m_minimumValueFTF.getValue( );
    }
    
    /**
     * Returns the maximum value of the range that determines offset 
     * calculation range
     */
    public int getMaximumRangeValue( )
    {
        return (int)m_maximumValueFTF.getValue( );
    }
    
    // Attributes of this class
    private JFrame m_parent;
    
    private JPanel m_previewButtonPanel;
    private JButton m_previewButton;
    
    private JPanel m_contentPanel;
    private JPanel m_orderStatisticsParameterPanel;
    private JPanel m_domainPanel;
    private JLabel m_domainLabel;
    
    private JRadioButton m_timeDomainButton;
    private JRadioButton m_frequencyDomainButton;
    
    private ButtonGroup m_domainButtonGroup;
    
    private JButton m_applyButton;
    private JButton m_cancelButton;
    private JPanel m_selectionPanel;
    
    private JPanel m_offsetsPanel;
    private JCheckBox m_eliminateOffsetCheckBox;
    private JPanel m_minMaxPanel;
    private JLabel m_minLabel;
    private JLabel m_maxLabel;
    
    private FormattedTextField m_minimumValueFTF;
    private FormattedTextField m_maximumValueFTF;
    
    // Listeners to changes in this class
    private LinkedList<ActionListener> m_listeners = new LinkedList<ActionListener>( );
    
    // events raised by accept and cancel button
    public final static int ACCEPT_EVENT = 0;
    public final static int CANCEL_EVENT = 1;
    
    public final static int PREVIEW_OSF_EVENT = 2;
    
    public final static int TIME_DOMAIN = 100;
    public final static int FREQUENCY_DOMAIN = 200;
    
    private boolean m_denoise2DResponse;
    
    
    // For proper serialization
    private static final long serialVersionUID = 1481295812342293L;
}
