/*
 * OrderStatisticsSignalFilter.java
 *
 * Created on June 30, 2005, 7:50 AM
 */

package org.spectrim.tdfdfit;

import java.util.Random;

/**
 *
 * @author  hans
 */
public class OrderStatisticsSignalFilter
{

    /**
     * Creates a new instance of OrderStatisticsSignalFilter
     */
    public OrderStatisticsSignalFilter( )
    {
    }

    /**
     * Creates a new instance of OrderStatisticsSignalFilter
     */
    public OrderStatisticsSignalFilter( int numberOfChannels, int numberOfPoints, float[][] signal2D )
    {
        m_numberOfChannels = numberOfChannels;
        m_numberOfPoints = numberOfPoints;
        m_signal2D = new double[numberOfChannels][numberOfPoints];
        for ( int i = 0 ; i < m_numberOfChannels; i++ )
        {
            for ( int j = 0 ; j < m_numberOfPoints; j++ )
            {
                m_signal2D[i][j] = (double)signal2D[i][j];
            }
        }
    }


    /**
     * Creates a new instance of OrderStatisticsSignalFilter
     */
    public OrderStatisticsSignalFilter( int numberOfChannels, int numberOfPoints, double[][] signal2D )
    {
        m_numberOfChannels = numberOfChannels;
        m_numberOfPoints = numberOfPoints;
        m_signal2D = new double[numberOfChannels][numberOfPoints];
        for ( int i = 0 ; i < m_numberOfChannels; i++ )
        {
            for ( int j = 0 ; j < m_numberOfPoints; j++ )
            {
                m_signal2D[i][j] = signal2D[i][j];
            }
        }
    }


    /**
     * Performs denoising by repetitive application of OSF filtering
     */
    public double[][] performOSFDenoising( boolean beginAndEnd, int minimum, int maximum )
    {

        System.out.println( "performOSFDenoising ... " );

        Random rnd = new Random( );

        double[] absInst = new double[ m_numberOfChannels ];

        double[][] allOrderedSignal2D = new double[m_numberOfChannels][m_numberOfPoints];

        int maxIter = 100;

        float amp = 1000.0f;


        //if ( m_alreadyComputed == false )
        //{
            // generate the signal
        //    for ( int i = 0 ; i < m_numberOfChannels ; i++ )
        //    {
        //        for ( int j = 0; j < m_numberOfPoints; j++ )
        //        {
        //            m_signal2D[i][j] = (float)(amp * Math.cos( 2.0 * Math.PI * j * 100.0 / m_numberOfPoints ) * Math.exp( -8.0 * j / m_numberOfPoints ) ) + (float)rnd.nextGaussian( );
        //        }
        //    }

        //    m_alreadyComputed = true;
        //}


        double[] averageSpectrum = new double[m_numberOfPoints];

        // Compute average
        for ( int i = 0 ; i < m_numberOfChannels ; i++ )
        {
           for ( int j = 0; j < m_numberOfPoints; j++ )
           {
               averageSpectrum[j] += m_signal2D[i][j];
           }
        }

        for ( int j = 0; j < m_numberOfPoints; j++ )
        {
            averageSpectrum[j] = averageSpectrum[j] / m_numberOfChannels;
        }

        double dc = 0.0;
        for ( int j = minimum; j < maximum; j++ )
        {
            dc = averageSpectrum[j] / ( maximum - minimum );
        }

        for ( int j = 0; j < m_numberOfPoints; j++ )
        {
            averageSpectrum[j] -= dc;
        }


        for ( int osfdenoisercounter = 0; osfdenoisercounter < maxIter ; osfdenoisercounter++ )
        {

            // Perform sorting on every array
            for ( int j = 0 ; j < m_numberOfPoints ; j++ )
            {
                for ( int i = 0; i < m_numberOfChannels; i++ )
                {
                    if ( osfdenoisercounter == 0 )
                    {
                        absInst[i] = m_signal2D[i][j];
                    }
                    else
                    {
                        absInst[i] = allOrderedSignal2D[i][j];
                    }
                }

                absInst = m_qSort.quickSort( absInst );

                for ( int i = 0; i < m_numberOfChannels; i++ )
                {
                    allOrderedSignal2D[i][j] = absInst[i];
                }
            }

            m_amplitudeOffsets = new double[m_numberOfChannels];

            // Shift each array to position
            for ( int i = 0; i < m_numberOfChannels; i++ )
            {
                // Determine offset
                float offsetSig = 0f;

                for( int j = minimum; j < maximum; j++ )
                {
                    offsetSig += allOrderedSignal2D[i][j];
                }

                offsetSig /= (maximum - minimum);

                for( int j = 0; j < m_numberOfPoints; j++ )
                {
                    allOrderedSignal2D[i][j] -= offsetSig;
                }

                m_amplitudeOffsets[i] = offsetSig;
            }

            if ( osfdenoisercounter < maxIter-1 )
            {
                // add appropriate noise to all channels
                for ( int i = 0; i < m_numberOfChannels; i++ )
                {
                	double minValue = Float.MAX_VALUE;
                	double maxValue = -Float.MAX_VALUE;
                    for ( int cc = minimum; cc < maximum; cc++ )
                    {
                        if ( allOrderedSignal2D[i][cc] <  minValue )
                        {
                            minValue = allOrderedSignal2D[i][cc];
                        }
                        if ( allOrderedSignal2D[i][cc] > maxValue )
                        {
                            maxValue = allOrderedSignal2D[i][cc];
                        }
                    }


                    if ( i == 10 )
                    {
                    	double var = 0f;
                        for ( int vv = minimum; vv < maximum; vv++ )
                        {
                            var += allOrderedSignal2D[i][vv] * allOrderedSignal2D[i][vv];
                        }
                        System.out.println( "Chan # :" + i + "   (Max-Min)   " + ( maxValue - minValue ) + "  variance = " + var + "  st-dev  " + Math.sqrt( var )/( maximum - minimum - 1) );
                    }

                    for( int j = 0; j < m_numberOfPoints; j++ )
                    {
                        allOrderedSignal2D[i][j] += 0.045 * ( rnd.nextGaussian( ) * ( maxValue - minValue ) ) / (osfdenoisercounter + 1.);
                    }

                    // start ordering again ..

                }
            }
        }

        double minValue = Float.MAX_VALUE;
        double maxValue = -Float.MAX_VALUE;
        for ( int cc = minimum; cc < maximum; cc++ )
        {
            if ( averageSpectrum[cc] <  minValue )
            {
                minValue = averageSpectrum[cc];
            }
            if ( averageSpectrum[cc] > maxValue )
            {
                maxValue = averageSpectrum[cc];
            }
        }

        float var = 0f;
        for ( int vv = minimum; vv < maximum; vv++ )
        {
            var += averageSpectrum[vv] * averageSpectrum[vv];
        }
        System.out.println( "Average  (Max-Min)   " + ( maxValue - minValue ) + "  variance = " + var + "  st-dev  " + Math.sqrt( var )/( maximum - minimum - 1) );

        // Finally store result in member
        for ( int i = 0; i < m_numberOfChannels; i++ )
        {
            for( int j = 0; j < m_numberOfPoints; j++ )
            {
                m_signal2D[i][j] = allOrderedSignal2D[i][j];
            }
        }

        return m_signal2D;
    }


    /**
     * Performs the OSF filtering
     */
    public double[][] performFiltering( boolean removeOffset, boolean beginAndEnd, int minimum, int maximum )
    {

        double[] absInst = new double[ m_numberOfChannels ];

        double[][] allOrderedSignal2D = new double[m_numberOfChannels][m_numberOfPoints];

        // Perform sorting on every array
        for ( int j = 0 ; j < m_numberOfPoints ; j++ )
        {
            for ( int i = 0; i < m_numberOfChannels; i++ )
            {
                absInst[i] = m_signal2D[i][j];
            }

            absInst = m_qSort.quickSort( absInst );

            for ( int i = 0; i < m_numberOfChannels; i++ )
            {
                allOrderedSignal2D[i][j] = absInst[i];
            }
        }

        m_amplitudeOffsets = new double[m_numberOfChannels];

        if ( removeOffset == true )
        {
            // Shift each array to position
            for ( int i = 0; i < m_numberOfChannels; i++ )
            {
                // Determine offset
                float offsetSig = 0f;

                for( int j = minimum; j < maximum; j++ )
                {
                    offsetSig += allOrderedSignal2D[i][j];
                }

                offsetSig /= (maximum - minimum);

                for( int j = 0; j < m_numberOfPoints; j++ )
                {
                    allOrderedSignal2D[i][j] -= offsetSig;
                }

                m_amplitudeOffsets[i] = offsetSig;
            }
        }

        // Finally store result in member
        for ( int i = 0; i < m_numberOfChannels; i++ )
        {
            for( int j = 0; j < m_numberOfPoints; j++ )
            {
                m_signal2D[i][j] = allOrderedSignal2D[i][j];
            }
        }

        return m_signal2D;
    }


    /**
     * Returns the sums of all channels after OSF filtering
     */
    public double[] getAmplitudeOffsets( )
    {
        return m_amplitudeOffsets;
    }

    // Attributes
    private int m_numberOfChannels;
    private int m_numberOfPoints;
    private double[][] m_signal2D;
    private double[] m_amplitudeOffsets;

    private QuickSortDouble m_qSort = new QuickSortDouble( );

    private boolean m_alreadyComputed = false;
}