/*
 * File: $Id: SingleVoxelSignalStabilityAnalyzer.java,v 1.3 2006/11/08 12:18:40 hans Exp $
 * History
 * Date: 09.05.2003, 9:34  Authors: Hans Slotboom
 *
 * Changes:
 *
 * Copyright (C) 2005 by
 *
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

/*
 * SingleVoxelSignalStabalityAnalyzer.java
 *
 * Created on May 9, 2005, 9:34 AM
 */

package org.spectrim.tdfdfit;


import java.awt.Color;

import java.util.Random;

import org.spectrim.xyplot.Curve;
import org.spectrim.xyplot.XYPlotFrame;


/**
 * Performs a signal stability analysis
 * @author hans
 */
public class SingleVoxelSignalStabilityAnalyzer 
{
    
    /**
     * Creates a new instance of SingleVoxelSignalStabalityAnalyzer
     * @param minXValue minimum value of offset range that can be assumed to be stable and has enough SNR
     * @param maxXValue maximum value of offset range that can be assumed to be stable and has enough SNR
     * @param m_measuredPatientSpectrumSeries Reference to the data object containing the series of SVS data
     */
    public SingleVoxelSignalStabilityAnalyzer( int minXValue, int maxXValue, SingleVoxelSpectralDataSeries measuredPatientSpectrumSeries )
    {
        m_minStableOffsetValue = minXValue;
        m_maxStableOffsetValue = maxXValue;
        m_measuredPatientSpectrumSeries = measuredPatientSpectrumSeries;
    }
    
    /**
     * Computes the probability density function of 
     * The order statistics of Gaussian input noise
     */
    public void computeARSOSOutputPDFs( int ARSOSfilterSize ) 
    {
        int samples = 1000;
        double[] pdf = new double[1000];
        double[] cdf = new double[1000];
        double[] OSFpdf = new double[1000];
        double[] xaxis = new double[1000];
        
        double x;
        
        for ( int i = 0 ; i < 1000; i++ )
        {
            x = ( i * 8.0 / 1000.0 ) - 4.0;
            xaxis[i] = x;
            pdf[i] = Math.exp( -0.5 * x * x ) / Math.sqrt( 2.0 * Math.PI );
            cdf[i] = computeNormalCumulaiveDensityFunction( x );
        }
        
        XYPlotFrame plotPDFs = new XYPlotFrame( "ARSOS outputs", null );
        
        plotPDFs.addCurve( new Curve( xaxis, pdf, Color.BLACK ), true );
        plotPDFs.addCurve( new Curve( xaxis, cdf, Color.BLACK ), true );
         
         /* Compute now the pdfs of all order statistics */ 
        for ( int j = 1; j <= ARSOSfilterSize; j++ )
        {
            double factor = factorial( ARSOSfilterSize ) / ( factorial( j - 1 ) * factorial( ARSOSfilterSize - j ) );
            System.out.println( "factor = " + factor );
            for ( int i = 0; i < 1000; i++ )
            {
                OSFpdf[i] = factor * Math.pow( cdf[i], ARSOSfilterSize - j ) * pdf[i] * Math.pow( ( 1.0 - cdf[i] ), j - 1 ) / ARSOSfilterSize;
            }
            
            plotPDFs.addCurve( new Curve( xaxis, OSFpdf, Color.BLUE ), true );
            
            double[] sum = new double[1000]; 
            sum[0] = OSFpdf[0];
            for ( int k = 1; k < 1000; k++ )
            { 
                sum[k] = sum[k-1] + OSFpdf[k];  
            }
            System.out.println( "wrong factor" + 1.0/sum[999]);
            for ( int k = 0; k < 1000; k++ )
            { 
                sum[k] = 8 * sum[k]/1000.0;  
            }
            
            plotPDFs.addCurve( new Curve( xaxis, sum, Color.RED ), true );
            
            TDFDFittingDefaults fdef = TDFDFittingDefaults.getInstance( );
            fdef.load( );
            
            SignalWriter swrOSF0 = new SignalWriter( "Write ARSOS[ " + j +  " of " + ARSOSfilterSize + " ]", xaxis, OSFpdf, fdef.getProperty( "Dicom mount path" ) + System.getProperty( "file.separator" ), null );

        }
        
    }
    
    /**
     * Computes the bias in the inputs
     */
    public void computeBias( )
    {
        int numberOfPoints = m_measuredPatientSpectrumSeries.getAbsorptionChannelTD( 0 ).length;
        int numberOfResponses = m_measuredPatientSpectrumSeries.getNumberOfInstances( );
        
        float[][] allDataAbs = new float[numberOfResponses][numberOfPoints];
        float[][] allDataDisp = new float[numberOfResponses][numberOfPoints];
        double[] means = new double[numberOfResponses];
        double[] xaxis = new double[numberOfResponses];
        
        for ( int i = 0; i < numberOfResponses; i++ ) 
        {
            SingleVoxelSpectralData specData = m_measuredPatientSpectrumSeries.getCurrentInstance( i );
            float[] abs  = specData.getAbsorptionChannelFD( );
            float[] disp = specData.getDispersionChannelFD( );
            
            for ( int j = 0; j < numberOfPoints ; j++ )
            {
                allDataAbs[i][j] = abs[j];
                allDataDisp[i][j] = disp[j];
            } 
            xaxis[i] = (double)i;
        }
        
        for ( int i = 0 ; i < numberOfResponses ; i++ )
        {
            means[i] = (double)i;
            
            double mean = 0;
            
            // Sample Mean Curve
            for ( int j = 0 ; j < numberOfPoints ; j++ )
            {
                mean += allDataAbs[i][j] / numberOfPoints;
            }
            
            means[i]= mean;
        }
        
        double[] SUMPOS = new double[numberOfResponses];
        double[] SUMNEG = new double[numberOfResponses];
        double[] DIFF = new double[numberOfResponses];
        double[] BIAS = new double[numberOfResponses];
        
        for ( int i = 0 ; i < numberOfResponses ; i++ )
        {
            
            double sumpos = 0;
            double sumneg = 0;
            
            // Sample Mean Curve
            for ( int j = 0 ; j < numberOfPoints ; j++ )
            {
                if ( ( allDataAbs[i][j] - means[i] ) > 0 )
                {
                    sumpos += ( allDataAbs[i][j] - means[i] ) * ( allDataAbs[i][j] - means[i] );
                }
                else
                {
                    sumneg += ( allDataAbs[i][j] - means[i] ) * ( allDataAbs[i][j] - means[i] );
                }
                
            }
            SUMPOS[i] = sumpos;
            SUMNEG[i] = sumneg;
            
            DIFF[i] = sumpos-sumneg;
            BIAS[i] = Math.sqrt( Math.abs( DIFF[i] ) ) * Math.signum( DIFF[i] );
        }
        
        XYPlotFrame plotMean = new XYPlotFrame( "Raw signals Bias", null );
        
        plotMean.addCurve( new Curve( xaxis, SUMPOS, Color.BLACK ), true );
        plotMean.addCurve( new Curve( xaxis, SUMNEG, Color.RED ), true );
        plotMean.addCurve( new Curve( xaxis, DIFF, Color.BLUE ), true );
        
        XYPlotFrame plotBias = new XYPlotFrame( "Bias", null );
        plotBias.addCurve( new Curve( xaxis, BIAS, Color.BLUE ), true );
        
        TDFDFittingDefaults fdef = TDFDFittingDefaults.getInstance( );
        fdef.load( );
        
        SignalWriter swrA = new SignalWriter( "SUM POS", SUMPOS, fdef.getProperty( "Dicom mount path" ) + System.getProperty( "file.separator" ), null );
        SignalWriter swrB = new SignalWriter( "SUM NEG", SUMNEG, fdef.getProperty( "Dicom mount path" ) + System.getProperty( "file.separator" ), null );
        SignalWriter swrC = new SignalWriter( "DIFF", DIFF, fdef.getProperty( "Dicom mount path" ) + System.getProperty( "file.separator" ), null );
        SignalWriter swrD = new SignalWriter( "BIAS", BIAS, fdef.getProperty( "Dicom mount path" ) + System.getProperty( "file.separator" ), null );
        
    }
    
    
    /**
     * Computes the factorial ! of an the integer n
     */
    public double factorial( int n )
    {  
        double factorial = 1.0;
        if ( n == 0 )
        {
            return factorial;
        }
        else
        {
            for ( int i = 1; i <= n; i++ )
            {
                factorial *= i;
            }
            return factorial;
        }
    }
    
    /**
     * Computes the cummulative density function of the normal distribution
     */ 
    public double computeNormalCumulaiveDensityFunction( double x )
    {
        double b1 =  0.319381530;
        double b2 = -0.356563782;
        double b3 =  1.781477937;
        double b4 = -1.821255978;
        double b5 =  1.330274429;
        double p  =  0.2316419;
        double c  =  0.39894228;

        if ( x >= 0.0 ) 
        {
            double t = 1.0 / ( 1.0 + p * x );
            return ( 1.0 - c * Math.exp( -x * x / 2.0 ) * t * ( t *( t * ( t * ( t * b5 + b4 ) + b3 ) + b2 ) + b1 ) );
        }
        else 
        {
            double t = 1.0 / ( 1.0 - p * x );
            return ( c * Math.exp( -x * x / 2.0 ) * t * ( t *( t * ( t * ( t * b5 + b4 ) + b3 ) + b2 ) + b1 ) );
        }
    }
    
    /**
     * Determines the first four standardized moments about the mean
     * first standardized moment  -  mean
     * second standardized moment -  variance
     * third standardized moment  -  skewness
     * fourth standardized moment -  kurtosis
     */
    public void analyzeStandardizedMomentsAbs( )
    {
     
        int numberOfPoints = m_measuredPatientSpectrumSeries.getAbsorptionChannelTD( 0 ).length;
        int numberOfResponses = m_measuredPatientSpectrumSeries.getNumberOfInstances( );
        
        float[][] allDataAbs = new float[numberOfResponses][numberOfPoints];
        float[][] allDataDisp = new float[numberOfResponses][numberOfPoints];
        
        for ( int i = 0; i < numberOfResponses; i++ ) 
        {
            SingleVoxelSpectralData specData = m_measuredPatientSpectrumSeries.getCurrentInstance( i );
            float[] abs  = specData.getAbsorptionChannelTD( );
            float[] disp = specData.getDispersionChannelTD( );
            
            for ( int j = 0; j < numberOfPoints ; j++ )
            {
                allDataAbs[i][j] = abs[j];
                allDataDisp[i][j] = disp[j];
            }   
        }
        
        double[] meanAbs = new double[numberOfPoints];
        double[] varianceAbs = new double[numberOfPoints];
        double[] skewnessAbs = new double[numberOfPoints];
        double[] kurtosisAbs = new double[numberOfPoints];
        double[] xaxis = new double[numberOfPoints];
        
        for ( int i = 0 ; i < numberOfPoints ; i++ )
        {
            xaxis[i] = (double)i;
            // Sample Mean Curve
            for ( int j = 0 ; j < numberOfResponses ; j++ )
            {
                meanAbs[i] += allDataAbs[j][i] / numberOfResponses;
            }
            
            // Sample Variance Curve (unbiased)
            for (int  j = 0 ; j < numberOfResponses ; j++ )
            {
                varianceAbs[i] += ( allDataAbs[j][i] - meanAbs[i] )*( allDataAbs[j][i] - meanAbs[i] ) / ( numberOfResponses - 1 );
            }
            
            double numerator = 0.0;
            double dominator = 0.0;
            
            // Sample Skewness Curve (often unfortunately biased)
            for ( int  j = 0 ; j < numberOfResponses ; j++ )
            {
                numerator += ( allDataAbs[j][i] - meanAbs[i] ) * ( allDataAbs[j][i] - meanAbs[i] ) * ( allDataAbs[j][i] - meanAbs[i] ) ;
                dominator += ( allDataAbs[j][i] - meanAbs[i] ) * ( allDataAbs[j][i] - meanAbs[i] );
            }
            
            dominator = Math.pow( dominator, 1.5f );
            skewnessAbs[i] = ( Math.sqrt(numberOfResponses) * Math.sqrt( numberOfResponses * ( numberOfResponses-1 ) )* numerator ) / ( ( numberOfResponses - 2 ) * dominator );
            
            // Kurtosis part
            numerator = 0.0;
            dominator = 0.0;
            
            double kurtosisFactor = ( (double)( numberOfResponses * ( numberOfResponses + 1 ) ) ) / ( ( numberOfResponses - 1 ) * ( numberOfResponses - 2 ) * ( numberOfResponses - 3 ) );
            double kurtosisAdditional = ( numberOfResponses - 1 ) * ( numberOfResponses - 1 ) / ( ( ( numberOfResponses - 2 ) ) * ( ( numberOfResponses - 3 ) ) );
            
            // Sample Kurtosis Curve
            for ( int  j = 0 ; j < numberOfResponses ; j++ )
            {
                numerator += ( allDataAbs[j][i] - meanAbs[i] ) * ( allDataAbs[j][i] - meanAbs[i] ) * ( allDataAbs[j][i] - meanAbs[i] ) * ( allDataAbs[j][i] - meanAbs[i] );
            }
            
            dominator = varianceAbs[i] * varianceAbs[i];
            kurtosisAbs[i] =  ( kurtosisFactor * numerator / dominator ) - 3 * kurtosisAdditional;
            System.out.println( "Kurtosis = " + kurtosisAbs[i] );
        }
        
        XYPlotFrame plotMean = new XYPlotFrame( "First Moment (Average)", null );
        plotMean.addCurve( new Curve( xaxis, meanAbs, Color.BLACK ), true );
        
        XYPlotFrame plotVariance = new XYPlotFrame( "Second Moment (Variance)", null );
        plotVariance.addCurve( new Curve( xaxis, varianceAbs, Color.BLACK ), true );
        
        double[] coefficientOfVariance = new double[numberOfPoints];
        for ( int i = 0 ; i < numberOfPoints; i++ )
        {    
            coefficientOfVariance[ i ] = Math.sqrt( varianceAbs[i] )/ meanAbs[i];
        }
        
        XYPlotFrame plotCoeffOfVariance = new XYPlotFrame( "Phase - Coefficient of Variance", null );
        plotCoeffOfVariance.addCurve( new Curve( xaxis, coefficientOfVariance, Color.BLACK ), true );
        
        XYPlotFrame plotSkewness = new XYPlotFrame( "Third Moment (Skewness)", null );
        plotSkewness.addCurve( new Curve( xaxis, skewnessAbs, Color.BLACK ), true );

        XYPlotFrame plotKurtosis = new XYPlotFrame( "Fourth Moment (Kurtosis)", null );
        plotKurtosis.addCurve( new Curve( xaxis, kurtosisAbs, Color.BLACK ), true );
        
        TDFDFittingDefaults fdef = TDFDFittingDefaults.getInstance( );
        fdef.load( );
        
        SignalWriter swrMean = new SignalWriter( "Mean", meanAbs, fdef.getProperty( "Dicom mount path" ) + System.getProperty( "file.separator" ), null );
        SignalWriter swrVar = new SignalWriter( "Variance", varianceAbs, fdef.getProperty( "Dicom mount path" ) + System.getProperty( "file.separator" ), null );
        SignalWriter swrSkew = new SignalWriter( "Skewness", skewnessAbs, fdef.getProperty( "Dicom mount path" ) + System.getProperty( "file.separator" ), null );
        SignalWriter swrKurt = new SignalWriter( "Kurtosis", kurtosisAbs, fdef.getProperty( "Dicom mount path" ) + System.getProperty( "file.separator" ), null );
       
        meanAbs = SignalTools.fivePointsSmoothenSignal( meanAbs );
        varianceAbs = SignalTools.fivePointsSmoothenSignal( varianceAbs );
        skewnessAbs = SignalTools.fivePointsSmoothenSignal( skewnessAbs );
        kurtosisAbs = SignalTools.fivePointsSmoothenSignal( kurtosisAbs );
        
        XYPlotFrame plotMean2 = new XYPlotFrame( "First Moment (Average)", null );
        plotMean2.addCurve( new Curve( xaxis, meanAbs, Color.BLACK ), true );
        
        XYPlotFrame plotVariance2 = new XYPlotFrame( "Second Moment (Variance)", null );
        plotVariance2.addCurve( new Curve( xaxis, varianceAbs, Color.BLACK ), true );
        
        XYPlotFrame plotSkewness2 = new XYPlotFrame( "Third Moment (Skewness)", null );
        plotSkewness2.addCurve( new Curve( xaxis, skewnessAbs, Color.BLACK ), true );
        
        XYPlotFrame plotKurtosis2 = new XYPlotFrame( "Fourth Moment (Kurtosis)", null );
        plotKurtosis2.addCurve( new Curve( xaxis, kurtosisAbs, Color.BLACK ), true );
    }
    
    /**
     * Determines the first four standardized moments about the mean
     * first standardized moment  -  mean
     * second standardized moment -  variance
     * third standardized moment  -  skewness
     * fourth standardized moment -  kurtosis
     * Analyses the four standard moments of noise input signals 
     */
    public void analyzeStandardizedMomentsAbsNoise( )
    {
     
        int numberOfPoints = m_measuredPatientSpectrumSeries.getAbsorptionChannelTD( 0 ).length;
        int numberOfResponses = m_measuredPatientSpectrumSeries.getNumberOfInstances( );
        
        float[][] allDataAbs = new float[numberOfResponses][numberOfPoints];
        float[][] allDataDisp = new float[numberOfResponses][numberOfPoints];
        
        for ( int i = 0; i < numberOfResponses; i++ ) 
        {
            SingleVoxelSpectralData specData = m_measuredPatientSpectrumSeries.getCurrentInstance( i );
            float[] abs  = specData.getAbsorptionChannelTD( );
            
            for ( int j = 0; j < numberOfPoints ; j++ )
            {
                allDataAbs[i][j] = abs[j];
            }   
        }
        
        double[] meanAbs = new double[numberOfResponses];
        double[] varianceAbs = new double[numberOfResponses];
        double[] skewnessAbs = new double[numberOfResponses];
        double[] kurtosisAbs = new double[numberOfResponses];
        double[] xaxis = new double[numberOfResponses];
        
        for ( int i = 0 ; i < numberOfResponses ; i++ )
        {
            xaxis[i] = (double)i;
            
            // Sample Mean Curve
            for ( int j = 0 ; j < numberOfPoints ; j++ )
            {
                meanAbs[i] += allDataAbs[i][j] / numberOfPoints;
            }
            
            // Sample Variance Curve (unbiased)
            for (int  j = 0 ; j < numberOfPoints ; j++ )
            {
                varianceAbs[i] += ( allDataAbs[i][j] - meanAbs[i] )*( allDataAbs[i][j] - meanAbs[i] ) / ( numberOfPoints - 1 );
            }
            
            double numerator = 0.0;
            double dominator = 0.0;
            
            // Sample Skewness Curve (often unfortunately biased)
            for ( int  j = 0 ; j < numberOfPoints ; j++ )
            {
                numerator += ( allDataAbs[i][j] - meanAbs[i] ) * ( allDataAbs[i][j] - meanAbs[i] ) * ( allDataAbs[i][j] - meanAbs[i] ) ;
                dominator += ( allDataAbs[i][j] - meanAbs[i] ) * ( allDataAbs[i][j] - meanAbs[i] );
            }
            
            dominator = Math.pow( dominator, 1.5f );
            skewnessAbs[i] = ( Math.sqrt(numberOfPoints) * Math.sqrt( numberOfPoints * ( numberOfPoints-1 ) )* numerator ) / ( ( numberOfPoints - 2 ) * dominator );
            
            // Kurtosis part
            numerator = 0.0;
            dominator = 0.0;
            
            double kurtosisFactor = ( (double)( numberOfPoints * ( numberOfPoints + 1 ) ) ) / ( ( numberOfPoints - 1 ) * ( numberOfPoints - 2 ) * (numberOfPoints - 3 ) );
            double kurtosisAdditional = ( numberOfPoints - 1 ) * ( numberOfPoints - 1 ) / ( ( ( numberOfPoints - 2 ) ) * ( ( numberOfPoints - 3 ) ) );
            
            // Sample Kurtosis Curve
            for ( int  j = 0 ; j < numberOfPoints ; j++ )
            {
                numerator += ( allDataAbs[i][j] - meanAbs[i] ) * ( allDataAbs[i][j] - meanAbs[i] ) * ( allDataAbs[i][j] - meanAbs[i] ) * ( allDataAbs[i][j] - meanAbs[i] );
            }
            
            dominator = varianceAbs[i] * varianceAbs[i];
            kurtosisAbs[i] =  ( kurtosisFactor * numerator / dominator ) - 3 * kurtosisAdditional;
            System.out.println( "Kurtosis = " + kurtosisAbs[i] );
        }
        
        
        XYPlotFrame plotMean = new XYPlotFrame( "First Moment (Average)", null );
        plotMean.addCurve( new Curve( xaxis, meanAbs, Color.BLACK ), true );
        
        XYPlotFrame plotVariance = new XYPlotFrame( "Second Moment (Variance)", null );
        plotVariance.addCurve( new Curve( xaxis, varianceAbs, Color.BLACK ), true );
        
        XYPlotFrame plotSkewness = new XYPlotFrame( "Third Moment (Skewness)", null );
        plotSkewness.addCurve( new Curve( xaxis, skewnessAbs, Color.BLACK ), true );
        
        XYPlotFrame plotKurtosis = new XYPlotFrame( "Fourth Moment (Kurtosis)", null );
        plotKurtosis.addCurve( new Curve( xaxis, kurtosisAbs, Color.BLACK ), true );
        
        
        TDFDFittingDefaults fdef = TDFDFittingDefaults.getInstance( );
        fdef.load( );
        
        SignalWriter swrMean = new SignalWriter( "Mean", meanAbs, fdef.getProperty( "Dicom mount path" ) + System.getProperty( "file.separator" ), null );
        SignalWriter swrVar = new SignalWriter( "Variance", varianceAbs, fdef.getProperty( "Dicom mount path" ) + System.getProperty( "file.separator" ), null );
        SignalWriter swrSkew = new SignalWriter( "Skewness", skewnessAbs, fdef.getProperty( "Dicom mount path" ) + System.getProperty( "file.separator" ), null );
        SignalWriter swrKurt = new SignalWriter( "Kurtosis", kurtosisAbs, fdef.getProperty( "Dicom mount path" ) + System.getProperty( "file.separator" ), null );
        
        
    }
    
    
    /**
     * Determines the first four standardized moments about the mean
     * first standardized moment  -  mean
     * second standardized moment -  variance
     * third standardized moment  -  skewness
     * fourth standardized moment -  kurtosis
     */
    public void analyzeStandardizedMomentsPhase( )
    {
     
        int numberOfPoints = m_measuredPatientSpectrumSeries.getAbsorptionChannelTD( 0 ).length;
        int numberOfResponses = m_measuredPatientSpectrumSeries.getNumberOfInstances( );
        
        double[][] phase = new double[numberOfResponses][numberOfPoints];
        
        for ( int i = 0; i < numberOfResponses; i++ ) 
        {
            SingleVoxelSpectralData specData = m_measuredPatientSpectrumSeries.getCurrentInstance( i );
            float[] abs  = specData.getAbsorptionChannelFD( );
            float[] disp = specData.getDispersionChannelFD( );
            
            for ( int j = 0; j < numberOfPoints ; j++ )
            {
                phase[i][j] = Math.atan2( disp[j], abs[j] );
            }
        }
        
        double[] meanPhase = new double[numberOfPoints];
        double[] variancePhase = new double[numberOfPoints];
        double[] skewnessPhase = new double[numberOfPoints];
        double[] kurtosisPhase = new double[numberOfPoints];
        double[] xaxis = new double[numberOfPoints];
        
        for ( int i = 0 ; i < numberOfPoints ; i++ )
        {
            xaxis[i] = (double)i;
            // Sample Mean Curve
            for ( int j = 0 ; j < numberOfResponses ; j++ )
            {
                meanPhase[i] += phase[j][i] / numberOfResponses;
            }
            
            // Sample Variance Curve (unbiased)
            for (int  j = 0 ; j < numberOfResponses ; j++ )
            {
                variancePhase[i] += ( phase[j][i] - meanPhase[i] )*( phase[j][i] - meanPhase[i] ) / ( numberOfResponses - 1 );
            }
            
            double numerator = 0.0;
            double dominator = 0.0;
            
            // Sample Skewness Curve (often unfortunately biased)
            for ( int  j = 0 ; j < numberOfResponses ; j++ )
            {
                numerator += ( phase[j][i] - meanPhase[i] ) * ( phase[j][i] - meanPhase[i] ) * ( phase[j][i] - meanPhase[i] ) ;
                dominator += ( phase[j][i] - meanPhase[i] ) * ( phase[j][i] - meanPhase[i] );
            }
            
            dominator = Math.pow( dominator, 1.5f );
            skewnessPhase[i] = ( Math.sqrt( numberOfResponses ) * Math.sqrt( numberOfResponses * ( numberOfResponses - 1 ) ) * numerator ) / ( ( numberOfResponses - 2 ) * dominator );
            
            // Kurtosis part
            numerator = 0.0;
            dominator = 0.0;
            
            double kurtosisFactor = ( (double)( numberOfResponses * ( numberOfResponses + 1 ) ) ) / ( ( numberOfResponses - 1 ) * ( numberOfResponses - 2 ) * (numberOfResponses - 3 ) );
            double kurtosisAdditional = ( numberOfResponses - 1 ) * ( numberOfResponses - 1 ) / ( ( ( numberOfResponses - 2 ) ) * ( ( numberOfResponses - 3 ) ) );
            
            // Sample Kurtosis Curve
            for ( int  j = 0 ; j < numberOfResponses ; j++ )
            {
                numerator += ( phase[j][i] - meanPhase[i] ) * ( phase[j][i] - meanPhase[i] ) * ( phase[j][i] - meanPhase[i] ) * ( phase[j][i] - meanPhase[i] );
            }
            
            dominator = variancePhase[i] * variancePhase[i];
            kurtosisPhase[i] =  ( kurtosisFactor * numerator / dominator ) - 3 * kurtosisAdditional;
        }
        
        XYPlotFrame plotMean = new XYPlotFrame( "Phase - First Moment (Average)", null );
        plotMean.addCurve( new Curve( xaxis, meanPhase, Color.BLACK ), true );
        
        XYPlotFrame plotVariance = new XYPlotFrame( "Phase - Second Moment (Variance)", null );
        plotVariance.addCurve( new Curve( xaxis, variancePhase, Color.BLACK ), true );
        
        double[] coefficientOfVariance = new double[numberOfPoints];
        for ( int i = 0 ; i < numberOfPoints; i++ )
        {
            coefficientOfVariance[ i ] = Math.sqrt( variancePhase[i] )/ meanPhase[i];
        }
        
        XYPlotFrame plotCoeffOfVariance = new XYPlotFrame( "Phase - Coefficient of Variance", null );
        plotCoeffOfVariance.addCurve( new Curve( xaxis, coefficientOfVariance, Color.BLACK ), true );
        
        
        XYPlotFrame plotSkewness = new XYPlotFrame( "Phase - Third Moment (Skewness)", null );
        plotSkewness.addCurve( new Curve( xaxis, skewnessPhase, Color.BLACK ), true );

        XYPlotFrame plotKurtosis = new XYPlotFrame( "Phase - Fourth Moment (Kurtosis)", null );
        plotKurtosis.addCurve( new Curve( xaxis, kurtosisPhase, Color.BLACK ), true );

    }
    
    
    
    /**
     * Performs the stability analysis and returns a report file
     */
    public String performStabilityAnalysis( ) 
    {
        
        //computeWhiteNoiseOSF( );
        
        // computeAllOrdersFilteredResponseFD( );
        
        computeAllHistogramsFD( );
        
        // computeMinimumMaximumFilteredResponseFD( );
        
        // computeMedianFilteredResponseFD( );
        
//        int cntr = 0;
//        
//        m_magnitudes = new float[m_measuredPatientSpectrumSeries.getNumberOfInstances( )];
//        m_phases =  new float[m_measuredPatientSpectrumSeries.getNumberOfInstances( )];
//        
//        m_magnitudes = getIntegralOfMagnitudeSignal( );
//        m_phases = getMeanOfPhaseSignal( );
//        
//        m_report += performSignalStabilityTest( );
//        
//        determineNoiseCharacteristics( );
//        // test 1 noise test locates ADC problems, variances in broad baseline larger than
//        m_report += performNoiseTest( );
//        
//        
//        XYPlotFrame plotMagnitudes = new XYPlotFrame( "Freq. Domain Magnitude", null );
//        XYPlotFrame plotPhases = new XYPlotFrame( "Freq. Domain Phase", null );
//        float[] outx = new float[m_phases.length];
//        
//        for (int i = 0; i< m_measuredPatientSpectrumSeries.getNumberOfInstances( ); i++) 
//        {
//            outx[i]= (float)i;
//        }
//        
//        Curve magnitudeCurve = new Curve( outx, m_magnitudes, Color.blue );
//        Curve phaseCurve = new Curve( outx, m_phases, Color.blue );
//        
//        plotMagnitudes.addCurve( magnitudeCurve, true );
//        plotPhases.addCurve( phaseCurve, true );
//        
//        
//        System.out.println( m_report );
        
        return m_report;
    }
    
    /**
     * Compute OSF of white noise generated by java's random generator
     */
    private void computeWhiteNoiseOSF( )
    {
        
        int numberOfPoints = m_measuredPatientSpectrumSeries.getAbsorptionChannelTD( 0 ).length;
        
        float[] outx = new float[ numberOfPoints ];
        float[] statFilteredAbs = new float[ numberOfPoints ];
        
        //int numberOfInstances = m_measuredPatientSpectrumSeries.getNumberOfInstances( );
        int numberOfInstances = 256;
        
        float[] absInst = new float[ numberOfInstances ];
        
        float[][] allAbs = new float[numberOfInstances][numberOfPoints];
        
        float[][] allOrderedAbs = new float[numberOfInstances][numberOfPoints];
        
        Random rnd = new Random( );
        
        float[] arithMean = new float[numberOfPoints];
        
        // Create array data structures
        for ( int i = 0; i < numberOfInstances; i++ ) 
        {
            //now on simulated data SingleVoxelSpectralData specData = m_measuredPatientSpectrumSeries.getCurrentInstance( i );
            
            //now on simulated data float[] abs = specData.getAbsorptionChannelFD( );
            //now on simulated data float[] disp = specData.getDispersionChannelFD( );
            
            for ( int j = 0 ; j < numberOfPoints; j++ ) 
            {
                allAbs[i][j] = 1000f * (float)rnd.nextGaussian( );
                
                //allAbs[i][j] = 2000f * ( 1f - 2f * (float)rnd.nextFloat( ) );
                
            }
            
            //System.out.println( "i = " + i );
        }
        
        float[] originalNoise = new float[ numberOfPoints];
        for( int j = 0; j < numberOfPoints; j++ ) 
        {
           originalNoise[j] = allAbs[numberOfInstances/2][j];
        }
        
        
        // Create array data structures
        for ( int j = 0; j < numberOfPoints; j++ )
        {
            
            for ( int i = 0; i < numberOfInstances; i++ ) 
            {
                arithMean[j]+=allAbs[i][j];
            }
            
            arithMean[j] = arithMean[j]/numberOfInstances;
            
        }
        
        float meanArithMean = 0f;
        float varianceArithMean = 0f;
        
        for( int ii = 0 ; ii < numberOfPoints ; ii++ )
        {
            meanArithMean += arithMean[ii];
        }
        
        meanArithMean = meanArithMean/numberOfPoints;
        
        for( int ii = 0 ; ii < numberOfPoints ; ii++ )
        {
            varianceArithMean += ( ( meanArithMean - arithMean[ii] ) * ( meanArithMean - arithMean[ii] ) );
        }
        
        varianceArithMean = (float)Math.sqrt( varianceArithMean ) / numberOfPoints;
        
        System.out.println( "numberOfPoints = " + numberOfPoints );
        System.out.println( "numberOfInstances = " + numberOfInstances );
        System.out.println( "meanArithMean = " + meanArithMean );
        System.out.println( "varianceArithMean = " + varianceArithMean );
        
        // arithmetic mean and variance of arithmetic mean signal
        
        
        float[] xaxis = new float[ numberOfPoints ];
        
        // Perform sorting on every array
        for ( int j = 0 ; j < numberOfPoints ; j++ ) 
        {
            
            for ( int i = 0; i < numberOfInstances; i++ ) 
            {
                absInst[i] = allAbs[i][j];
            }
            
            absInst = m_qSort.quickSort( absInst );
            
            outx[j] = (float)j;
            
            for ( int i = 0; i < numberOfInstances; i++ ) 
            {
                allOrderedAbs[i][j] = absInst[i];
            }
            
            // System.out.println( "Sorting arrays = " + j );
        }
        
        float[] offsetsAbs = new float[numberOfInstances];
        
        
        float[] stdevsAbs = new float[numberOfInstances];
        float[] stdevsArithmeticMeanSignal = new float[numberOfInstances];
        
        // Shift each array to position
        for ( int i = 0; i < numberOfInstances; i++ ) 
        {
            // Determine offset
            float offsetAbs = 0f;
            
            float stdevAbs = 0f;
            
            for( int j = 0; j < numberOfPoints; j++ ) 
            {
                offsetAbs += allOrderedAbs[i][j];
            }
            
            offsetAbs /= numberOfPoints;
            
            for( int j = 0; j < numberOfPoints; j++ ) 
            {
                //allOrderedAbs[i][j] -= offsetAbs;
            }
            
            offsetsAbs[i] = offsetAbs;
            
            for( int j = 0; j < numberOfPoints; j++ ) 
            {
                stdevAbs += ( allOrderedAbs[i][j] * allOrderedAbs[i][j] );
            }
            
            stdevsAbs[i] = (float)Math.sqrt( stdevAbs ) / numberOfPoints;
            // for all elements the same
            stdevsArithmeticMeanSignal[i] = varianceArithMean;
        }
        
        // Finally add all arrays together
        for ( int i = 0; i < numberOfInstances; i++ ) 
        {
            for( int j = 0; j < numberOfPoints; j++ ) 
            {
                statFilteredAbs[j] += allOrderedAbs[i][j];
                xaxis[j] = (float)j;
            }
        }
        
        float[] OSFChannel0 = new float[numberOfPoints];
        for( int j = 0; j < numberOfPoints; j++ ) 
        {
           OSFChannel0[j] = allOrderedAbs[0][j];
        }
        
        // display the means of the OSF signal
        XYPlotFrame plotOSFChannel = new XYPlotFrame( "OSF[0]", null );
        plotOSFChannel.addCurve( new Curve( xaxis, OSFChannel0, Color.BLACK ), true );
        
        float[] OSFChannelHalf = new float[ numberOfPoints];
        for( int j = 0; j < numberOfPoints; j++ ) 
        {
           OSFChannelHalf[j] = allOrderedAbs[numberOfInstances/2][j];
        }
        
        float[] OSFChannel_N = new float[ numberOfPoints];
        for( int j = 0; j < numberOfPoints; j++ ) 
        {
           OSFChannel_N[j] = allOrderedAbs[numberOfInstances-1][j];
        }
        
        float[] OSFChannel_N_4 = new float[ numberOfPoints];
        for( int j = 0; j < numberOfPoints; j++ ) 
        {
           OSFChannel_N_4[j] = allOrderedAbs[3*numberOfInstances/4][j];
        }
        
        // display the arithmetic mean
        XYPlotFrame plotArithMean = new XYPlotFrame( "Arithmetic mean", null );
        plotArithMean.addCurve( new Curve( xaxis, arithMean, Color.BLACK ), true );
        
        // display the means of the OSFN/2 signal
        XYPlotFrame plotOSFChannelHalf = new XYPlotFrame( "OSF[N/2]", null );
        plotOSFChannelHalf.addCurve( new Curve( xaxis, OSFChannelHalf, Color.BLACK ), true );

        // display the means of the OSF[N] signal
        XYPlotFrame plotOSFChannelN = new XYPlotFrame( "OSF[N]", null );
        plotOSFChannelN.addCurve( new Curve( xaxis, OSFChannel_N, Color.BLACK ), true );
        
        
        
        // display the means of the OSF signal
        XYPlotFrame plotOffsetsAbs = new XYPlotFrame( "Offsets White Noise Absorption Channel", null );
        float[] outxOffsets = new float[numberOfInstances];
        
        for ( int i = 0; i < numberOfInstances; i++) 
        {
            outxOffsets[i] = (float)i;
        }
        
        plotOffsetsAbs.addCurve( new Curve( outxOffsets, offsetsAbs, Color.BLACK ), true );
        
               
        // display the stdev of the OSF signal
        XYPlotFrame plotVarianceAbs = new XYPlotFrame( "Variances White Noise Absorption Channel", null );
                
        plotVarianceAbs.addCurve( new Curve( outxOffsets, stdevsAbs, Color.BLACK ), true );
        plotVarianceAbs.addCurve( new Curve( outxOffsets, stdevsArithmeticMeanSignal, Color.BLACK ), true );
        
        TDFDFittingDefaults fdef = TDFDFittingDefaults.getInstance( );
        fdef.load( );
        
//        SignalWriter swrAM = new SignalWriter( "Save White Noise Absorption Offsets", outxOffsets, offsetsAbs, fdef.getProperty( "Dicom mount path" ) + System.getProperty( "file.separator" ), null );        
//        TwoSignalsWriter swrA_STDEV = new TwoSignalsWriter( "Save White Noise Absorption Variances", "Variance", outxOffsets, stdevsAbs, stdevsArithmeticMeanSignal, fdef.getProperty( "Dicom mount path" ) + System.getProperty( "file.separator" ), null );

          SignalWriter swrOSF0 = new SignalWriter( "Save OSF[0]", xaxis, OSFChannel0, fdef.getProperty( "Dicom mount path" ) + System.getProperty( "file.separator" ), null );
          SignalWriter swrArithMean = new SignalWriter( "Save Arithmetic Mean", xaxis, arithMean, fdef.getProperty( "Dicom mount path" ) + System.getProperty( "file.separator" ), null );
          SignalWriter swrOSF_N_2 = new SignalWriter( "Save OSF[N/2]", xaxis, OSFChannelHalf, fdef.getProperty( "Dicom mount path" ) + System.getProperty( "file.separator" ), null );
          SignalWriter swrOSF_N = new SignalWriter( "Save OSF[N]", xaxis, OSFChannel_N, fdef.getProperty( "Dicom mount path" ) + System.getProperty( "file.separator" ), null );
          SignalWriter swrOSF_N_4 = new SignalWriter( "Save OSF[N/4]", xaxis, OSFChannel_N_4, fdef.getProperty( "Dicom mount path" ) + System.getProperty( "file.separator" ), null );
          SignalWriter swrORG = new SignalWriter( "Original noise", xaxis, originalNoise, fdef.getProperty( "Dicom mount path" ) + System.getProperty( "file.separator" ), null );
    }
    
    //Testcode repetitive OSF
    private void performRepetitiveOSFiltering( int numberOfChannels, int numberOfPoints, float[][] signal2D )
    {
        
        // some test code
        computeHistogramof2DSignal( signal2D,  m_measuredPatientSpectrumSeries.getNumberOfInstances( ), numberOfPoints, 0, 400, SignalHistogramCalculator.HISTOGRAM_RANGE_DETERMINED_BY_TOTAL_SIGNAL );
        
        float[] xaxis = new float[ numberOfPoints];
        float[] sig = new float[ numberOfPoints];
        
        XYPlotFrame plotOriginalSignal = new XYPlotFrame( "Original signal", null );
        
        for ( int i = 0; i < numberOfPoints; i++ )
        {
            xaxis[i] = (float)i;
            sig[i] = signal2D[numberOfChannels/2][i];
        }
        
        plotOriginalSignal.addCurve( new Curve( xaxis, sig, Color.BLUE ), true );
        
        
        OrderStatisticsSignalFilter osf = new OrderStatisticsSignalFilter( numberOfChannels, numberOfPoints, signal2D );
        
        double[][] filteredSignal = new double[numberOfChannels][numberOfPoints];
        
        for ( int i = 0 ; i < numberOfPoints; i++ )
        {
            filteredSignal = osf.performFiltering( true, false, numberOfPoints - 100, numberOfPoints );
        }
        
        // some test code
        computeHistogramof2DSignal( filteredSignal,  m_measuredPatientSpectrumSeries.getNumberOfInstances( ), numberOfPoints, 0, 450, SignalHistogramCalculator.HISTOGRAM_RANGE_DETERMINED_BY_TOTAL_SIGNAL );
        
        XYPlotFrame plotRepetitiveFilteredSignal = new XYPlotFrame( "Repetive osf filtered signal", null );
        
        for ( int i = 0; i < numberOfPoints; i++ )
        {
            xaxis[i] = (float)i;
            sig[i] = (float)filteredSignal[numberOfChannels/2][i];
        }
        
        plotRepetitiveFilteredSignal.addCurve( new Curve( xaxis, sig, Color.BLUE ), true );
    }
    
    /**
     * Performs a stability test on the indicated offset frequency range
     */
    private String performSignalStabilityTest( ) 
    {
        String report = new String( "" );
        float[] sortedArray = new float[m_magnitudes.length];
        
        // examine the behavior of the signal mean
        float meanOfMagnitude = SignalTools.mean( m_magnitudes );
        float stDevOfMagnitude = (float)Math.sqrt( SignalTools.computeVariance( m_magnitudes ) / ( m_magnitudes.length - 1 ) );
        float minOfMagnitude = SignalTools.getMinimum( m_magnitudes );
        float maxOfMagnitude = SignalTools.getMaximum( m_magnitudes );
        
        sortedArray = m_magnitudes.clone();
        sortedArray = m_qSort.quickSort( sortedArray );
        
        float medianOfMagnitude = sortedArray[m_magnitudes.length/2];
        
        report += newln + "=================================================" + newln;
        report += "Frequency Domain Signal Magnitude Stability Tests"  + newln;
        report += "=================================================" + newln + newln;
        
        report += "Examined offset frequency range " + newln;
        report += "Minimum offset array index : " + m_minStableOffsetValue + newln;
        report += "Maximum offset array index : " + m_maxStableOffsetValue + newln + newln;
        
        report += "Mean signal value of reponse magnitudes : " + meanOfMagnitude + newln;
        report += "Variance of reponse magnitudes          : " + stDevOfMagnitude + newln;
        report += "Minimum value of response magnitudes    : " + minOfMagnitude + newln;
        report += "Median  value of response magnitudes    : " + medianOfMagnitude + newln;
        report += "Maximum value of response magnitudes    : " + maxOfMagnitude + newln;
        
        if ( Math.abs( meanOfMagnitude - minOfMagnitude ) > 3 * stDevOfMagnitude ) 
        {
            report += "Mean magnitude test FAILED : minimum of magnitude values > 3 * mean-magnitude-variance"  + newln;
            report += "( Response number : " + SignalTools.getMinimumIndex( m_magnitudes ) + " )" + newln;
        }
        else 
        {
            report += "Mean noise test PASSED : minimum of  magnitude values < 3 * mean-magnitude-variance" + newln;
        }
        
        if ( Math.abs( meanOfMagnitude - maxOfMagnitude ) > 3 * stDevOfMagnitude ) 
        {
            report += "Mean magnitude test FAILED : maximum of magnitude values > 3 * mean-magitude-variance" + newln;
            report += "( Response number : " + SignalTools.getMaximumIndex( m_magnitudes ) + " )" + newln;
        }
        else 
        {
            report += "Mean m_magnitudes test PASSED : maximum of magnitudes values < 3 * mean-magnitude-variance" +  newln;
        }
        
        report += calculateTTest( SignalTools.clip( m_magnitudes, 0, m_magnitudes.length/2 - 1 ),
        SignalTools.clip( m_magnitudes, m_magnitudes.length/2 , m_magnitudes.length - 1 ) );
        
        report += newln;
        
        
        // examine the behavior of the signal phase
        float meanOfPhase = SignalTools.mean( m_phases );
        float stDevOfPhase = (float)Math.sqrt( SignalTools.computeVariance( m_phases ) / ( m_phases.length - 1 ) );
        float minOfPhase = SignalTools.getMinimum( m_phases );
        float maxOfPhase = SignalTools.getMaximum( m_phases );
        
        sortedArray = m_phases.clone( );
        sortedArray = m_qSort.quickSort( sortedArray );
        
        float medianOfPhase = sortedArray[m_phases.length/2];
        
        report += "=============================================" + newln;
        report += "Frequency Domain Signal Phase Stability Tests"  + newln;
        report += "=============================================" + newln + newln;
        report += "Examined offset frequency range " + newln;
        report += "Minimum offset array index : " + m_minStableOffsetValue + newln;
        report += "Maximum offset array index : " + m_maxStableOffsetValue + newln + newln;
        report += "Mean signal value of reponse phases : " + meanOfPhase + newln;
        report += "Variance of reponse phases          : " + stDevOfPhase + newln;
        report += "Minimum value of response phases    : " + minOfPhase + newln;
        report += "Median  value of response phases    : " + medianOfPhase + newln;
        report += "Maximum value of response phases    : " + maxOfPhase + newln;
        
        if ( Math.abs( meanOfPhase - minOfPhase ) > 3 * stDevOfPhase ) 
        {
            report += "Mean phase test FAILED : minimum of magnitude values > 3 * mean-phase-variance"  + newln;
            report += "( Response number : " + SignalTools.getMinimumIndex( m_phases ) + " )" + newln;
        }
        else 
        {
            report += "Mean phase test PASSED : minimum of  magnitude values < 3 * mean-phase-variance" + newln;
        }
        
        if ( Math.abs( meanOfPhase - maxOfPhase ) > 3 * stDevOfPhase ) 
        {
            report += "Mean phase test FAILED : maximum of phase values > 3 * mean-phase-variance" + newln;
            report += "( Response number : " + SignalTools.getMaximumIndex( m_phases ) + " )" + newln;
        }
        else 
        {
            report += "Mean phase test PASSED : maximum of phase values < 3 * mean-phase-variance" +  newln;
        }
        
        report += calculateTTest( SignalTools.clip( m_phases, 0, m_phases.length/2 - 1 ),
        SignalTools.clip( m_phases, m_phases.length/2 , m_phases.length - 1 ) );
        
        report += newln + newln;
        
        return report;
    }
    
    
    /**
     * Performs a test on the TD and FD noise parts of the acquired spectra
     */
    private String performNoiseTest( ) 
    {
        boolean testPassed = false;
        float[] sortedArray = new float[m_noiseVariance.length];
        
        // examine the behavior of the noise mean
        float meanOfMeans = SignalTools.mean( m_noiseMean );
        float stDevOfMeans = (float)Math.sqrt( SignalTools.computeVariance( m_noiseMean ) / (m_noiseMean.length - 1 ) );
        float minOfMeans = SignalTools.getMinimum( m_noiseMean );
        float maxOfMeans = SignalTools.getMaximum( m_noiseMean );
        
        sortedArray = m_noiseMean.clone( );
        sortedArray = m_qSort.quickSort( sortedArray );
        
        float medianOfMeans = sortedArray[m_noiseMean.length/2];
        
        
        String report = "=====================================" + newln;
        report  = "Frequency Domain Noise Mean Tests"  + newln;
        report += "=====================================" + newln + newln;
        
        report += "Mean noise value of reponse means : " + meanOfMeans + newln;
        report += "Variance of reponse means         : " + stDevOfMeans + newln;
        report += "Minimum value of response means   : " + minOfMeans + newln;
        report += "Median  value of response means   : " + medianOfMeans + newln;
        report += "Maximum value of response means   : " + maxOfMeans + newln;
        
        if ( Math.abs( meanOfMeans - minOfMeans ) > 3 * stDevOfMeans ) 
        {
            report += "Mean noise test FAILED : minimum of meanvalues > 3 * mean-noise-variance"  + newln;
            report += "( Response number : " +SignalTools.getMinimumIndex( m_noiseMean ) + " )" + newln;
        }
        else 
        {
            report += "Mean noise test PASSED : minimum of meanvalues < 3 * mean-noise-variance" + newln;
        }
        
        if ( Math.abs( meanOfMeans - maxOfMeans ) > 3 * stDevOfMeans ) 
        {
            report += "Mean noise test FAILED : maximum of meanvalues > 3 * mean-noise-variance" + newln;
            report += "( Response number : " + SignalTools.getMaximumIndex( m_noiseMean ) + " )" + newln;
        }
        else 
        {
            report += "Mean noise test PASSED : maximum of meanvalues < 3 * mean-noise-variance" +  newln;
        }
        
        report += calculateTTest( SignalTools.clip( m_noiseMean, 0, m_noiseMean.length/2 - 1 ),
        SignalTools.clip( m_noiseMean, m_noiseMean.length/2 , m_noiseMean.length - 1 ) );
        
        report += newln;
        
        float meanOfVariances = SignalTools.mean( m_noiseVariance );
        float stdevOfVariances = (float)Math.sqrt( SignalTools.computeVariance( m_noiseVariance ) / ( m_noiseVariance.length - 1 ) );
        float minOfVariances = SignalTools.getMinimum( m_noiseVariance );
        
        float maxOfVariances = SignalTools.getMaximum( m_noiseVariance );
        
        sortedArray = m_noiseVariance.clone( );
        sortedArray = m_qSort.quickSort( sortedArray );
        float medianOfVariances = sortedArray[m_noiseVariance.length/2];
        
        report += "=====================================" + newln;
        report += "Frequency Domain Noise Variance Tests" + newln;
        report += "=====================================" + newln + newln;
        
        report += "Mean noise value of reponse variances : " + meanOfVariances + newln;
        report += "Variance of reponse variances         : " + stdevOfVariances + newln;
        report += "Minimum value of response variances   : " + minOfVariances + newln;
        report += "Median  value of response variances   : " + medianOfVariances + newln;
        report += "Maximum value of response variances   : " + maxOfVariances + newln;
        
        if ( Math.abs( meanOfVariances - minOfVariances ) > 3 * stdevOfVariances ) 
        {
            report += "Mean noise test FAILED : minimum of meanvalues > 3 * mean-noise-variance" + newln;
            report += "( Response number : " +SignalTools.getMinimumIndex( m_noiseMean ) +" )" + newln;
        }
        else 
        {
            report += "Mean noise test PASSED : minimum of variance values < 3 * mean-noise-variance" + newln;
        }
        
        if ( Math.abs( meanOfVariances - maxOfVariances ) > 3 * stdevOfVariances ) {
            report += "Mean noise test FAILED : maximum of variance values > 3 * mean-noise-variance" + newln;
            report += "( Response number : " +SignalTools.getMinimumIndex( m_noiseMean ) + " )" + newln;
        }
        else 
        {
            report += "Mean noise test PASSED : maximum of meanvalues < 3 * mean-noise-variance" + newln;
        }
        
        report += calculateTTest( SignalTools.clip( m_noiseVariance, 0, m_noiseVariance.length/2 - 1 ),
        SignalTools.clip( m_noiseVariance, m_noiseVariance.length/2 , m_noiseVariance.length - 1 ) );
        
        report += newln;
        
        return report;
    }
    
    
    /**
     * Returns the integral of magnitude frequency domain signal
     */
    private void determineNoiseCharacteristics( ) 
    {
        m_noiseVariance = new float[ m_measuredPatientSpectrumSeries.getNumberOfInstances( ) ];
        m_noiseMean = new float[ m_measuredPatientSpectrumSeries.getNumberOfInstances( ) ];
        
        for ( int i = 0; i < m_measuredPatientSpectrumSeries.getNumberOfInstances( ); i++ ) 
        {
            SingleVoxelSpectralData specData = m_measuredPatientSpectrumSeries.getCurrentInstance( i );
            float[] abs  = specData.getAbsorptionChannelFD( );
            float[] disp = specData.getDispersionChannelFD( );
            
            float integral = 0f;
            for ( int j = abs.length - 50; j < abs.length; j++ ) 
            {
                integral += abs[j];
            }
            
            m_noiseMean[i] = integral / 50.0f;
            
            float variance = 0f;
            for ( int j = abs.length - 50; j < abs.length; j++ ) 
            {
                variance += ( (integral/50f) - abs[j] ) * ( (integral/50f) - abs[j] );
            }
            
            m_noiseVariance[i] = (float)Math.sqrt( variance / 49f );
        }
    }
    
    
    /**
     * Returns the integral of magnitude frequency domain signal
     */
    private float[] getIntegralOfMagnitudeSignal( ) 
    {
        float[] magnitudeSignal = new float[ m_measuredPatientSpectrumSeries.getNumberOfInstances( ) ];
        
        for ( int i = 0; i < m_measuredPatientSpectrumSeries.getNumberOfInstances( ); i++ ) 
        {
            SingleVoxelSpectralData specData = m_measuredPatientSpectrumSeries.getCurrentInstance( i );
            float[] abs  = specData.getAbsorptionChannelFD( );
            float[] disp = specData.getDispersionChannelFD( );
            
            float integral = 0f;
            for ( int j = m_minStableOffsetValue; j <= m_maxStableOffsetValue; j++ ) 
            {
                integral += Math.sqrt( abs[j] * abs[j] + disp[j] * disp[j] );
            }
            
            magnitudeSignal[i] = integral;
        }
        
        float[] sortedArray = new float[magnitudeSignal.length];
        
        sortedArray = magnitudeSignal.clone( );
        
        sortedArray = m_qSort.quickSort( sortedArray );
        
        for ( int i = 0; i < m_measuredPatientSpectrumSeries.getNumberOfInstances( ); i++ ) 
        {
            magnitudeSignal[i] = magnitudeSignal[i]/sortedArray[sortedArray.length/2];
        }
        
        return magnitudeSignal;
    }
    
    /**
     * Returns the integral of magnitude frequency domain signal
     */
    private float[] getMeanOfPhaseSignal( ) 
    {
        float[] phaseSignal = new float[ m_measuredPatientSpectrumSeries.getNumberOfInstances( ) ];
        
        for ( int i = 0; i < m_measuredPatientSpectrumSeries.getNumberOfInstances( ); i++ ) 
        {
            SingleVoxelSpectralData specData = m_measuredPatientSpectrumSeries.getCurrentInstance( i );
            float[] abs  = specData.getAbsorptionChannelFD( );
            float[] disp = specData.getDispersionChannelFD( );
            
            float average = 0f;
            for ( int j = m_minStableOffsetValue; j <= m_maxStableOffsetValue; j++ ) 
            {
                average += Math.atan2( abs[j], disp[j] );
            }
            phaseSignal[i] = average / ( m_maxStableOffsetValue - m_minStableOffsetValue );
        }
        
        return phaseSignal;
    }
    
    /**
     * Computes pixel by pixel the minimum frequency domain absorption
     * channel
     */
    private void computeMedianFilteredResponseFD( ) 
    {
        int numberOfPoints = m_measuredPatientSpectrumSeries.getAbsorptionChannelTD( 0 ).length;
      
        float[] outx = new float[ numberOfPoints ];
        float[] medianFilteredAbs = new float[ numberOfPoints ];
        float[] medianFilteredDisp = new float[ numberOfPoints ];
        
        float[] medianAbs = new float[ m_measuredPatientSpectrumSeries.getNumberOfInstances( ) ];
        float[] medianDisp = new float[ m_measuredPatientSpectrumSeries.getNumberOfInstances( ) ];
        
        float[][] allAbs = new float[m_measuredPatientSpectrumSeries.getNumberOfInstances( )][numberOfPoints];
        float[][] allDisp = new float[m_measuredPatientSpectrumSeries.getNumberOfInstances( )][numberOfPoints];
 
        for ( int i = 0; i < m_measuredPatientSpectrumSeries.getNumberOfInstances( ); i++ ) 
        {
            SingleVoxelSpectralData specData = m_measuredPatientSpectrumSeries.getCurrentInstance( i );
            
            float[] abs = specData.getAbsorptionChannelFD( );
            float[] disp = specData.getDispersionChannelFD( );
            
            for ( int j = 0 ; j < numberOfPoints; j++ ) 
            {
                allAbs[i][j] = abs[j];
                allDisp[i][j] = disp[j];
            }
            
            System.out.println( "i = " + i );
        }
        
        float[] xaxis = new float[ numberOfPoints ];
        
        for ( int j = 0 ; j < numberOfPoints ;j++ ) 
        {
            
            for ( int i = 0 ; i < m_measuredPatientSpectrumSeries.getNumberOfInstances( ); i++ ) 
            {
                medianAbs[i] = allAbs[i][j];
                medianDisp[i] = allDisp[i][j];
            }
            
            medianAbs = m_qSort.quickSort( medianAbs );
            medianDisp = m_qSort.quickSort( medianDisp );
            
            outx[j] = (float)j;
            medianFilteredAbs[j] = medianAbs[medianAbs.length/2];
            medianFilteredDisp[j] = medianDisp[medianDisp.length/2];
        }
        
        XYPlotFrame plotAbs = new XYPlotFrame( "Median filtered Absorption spectrum", null );
        XYPlotFrame plotDisp = new XYPlotFrame( "Median filtered Dispersion spectrum", null );
        
        Curve absCurve = new Curve( outx, medianFilteredAbs, Color.blue );
        Curve dispCurve = new Curve( outx, medianFilteredDisp, Color.blue );
        
        plotAbs.addCurve( absCurve, true );
        plotDisp.addCurve( dispCurve, true );
        
        TDFDFittingDefaults fdef = TDFDFittingDefaults.getInstance( );
        fdef.load( );
        
        SignalWriter swrA = new SignalWriter( "Save Median Filtered Absorption FD Channel", medianFilteredAbs, fdef.getProperty( "Dicom mount path" ) + System.getProperty( "file.separator" ), null );
        SignalWriter swrD = new SignalWriter( "Save Median Filtered Dispersion FD Channel", medianFilteredDisp, fdef.getProperty( "Dicom mount path" ) + System.getProperty( "file.separator" ), null );
        
        allAbs = null;
        allDisp = null;
    }
    
    
    /**
     * Computes pixel by pixel the minimum frequency domain absorption
     * channel
     */
    private void computeMinimumMaximumFilteredResponseFD( ) 
    {
        int numberOfPoints = m_measuredPatientSpectrumSeries.getAbsorptionChannelTD( 0 ).length;
        
        float[] outx = new float[ numberOfPoints ];
        float[] statFilteredAbs = new float[ numberOfPoints ];
        float[] statFilteredDisp = new float[ numberOfPoints ];
        
        float[] absInst = new float[ m_measuredPatientSpectrumSeries.getNumberOfInstances( ) ];
        float[] dispInst = new float[ m_measuredPatientSpectrumSeries.getNumberOfInstances( ) ];
        
        
        float[][] allAbs = new float[m_measuredPatientSpectrumSeries.getNumberOfInstances( )][numberOfPoints];
        float[][] allDisp = new float[m_measuredPatientSpectrumSeries.getNumberOfInstances( )][numberOfPoints];
        
        float[][] allOrderedAbs = new float[m_measuredPatientSpectrumSeries.getNumberOfInstances( )][numberOfPoints];
        float[][] allOrderedDisp = new float[m_measuredPatientSpectrumSeries.getNumberOfInstances( )][numberOfPoints];
        
        // Create array data structures
        for ( int i = 0; i < m_measuredPatientSpectrumSeries.getNumberOfInstances( ); i++ ) 
        {
            SingleVoxelSpectralData specData = m_measuredPatientSpectrumSeries.getCurrentInstance( i );
            
            float[] abs = specData.getAbsorptionChannelFD( );
            float[] disp = specData.getDispersionChannelFD( );
            
            for ( int j = 0 ; j < numberOfPoints; j++ ) 
            {
                allAbs[i][j] = abs[j];
                allDisp[i][j] = disp[j];
            }
            
            System.out.println( "i = " + i );
        }
        
        // test code:
        performRepetitiveOSFiltering( m_measuredPatientSpectrumSeries.getNumberOfInstances( ), numberOfPoints, allAbs );
        
        float[] xaxis = new float[ numberOfPoints ];
        
        // Perform sorting on every array
        for ( int j = 0 ; j < numberOfPoints ; j++ ) 
        {
            
            for ( int i=0; i< m_measuredPatientSpectrumSeries.getNumberOfInstances( ); i++ ) 
            {
                absInst[i] = allAbs[i][j];
                dispInst[i] = allDisp[i][j];
            }
            
            absInst = m_qSort.quickSort( absInst );
            dispInst = m_qSort.quickSort( dispInst );
            
            
            outx[j] = (float)j;
            
            for ( int i = 0; i < m_measuredPatientSpectrumSeries.getNumberOfInstances( ); i++ ) 
            {
                allOrderedAbs[i][j] = absInst[i];
                allOrderedDisp[i][j] = dispInst[i];
            }
            
            System.out.println( "Sorting arrays = " + j );
        }
        
        float[] offsetsAbs = new float[m_measuredPatientSpectrumSeries.getNumberOfInstances( )];
        float[] offsetsDisp = new float[m_measuredPatientSpectrumSeries.getNumberOfInstances( )];
        
        // Shift each array to position
        for ( int i = 0; i < m_measuredPatientSpectrumSeries.getNumberOfInstances( ); i++ ) 
        {
            // Determine offset
            float offsetAbs = 0f;
            float offsetDisp = 0f;
            for( int j = numberOfPoints - 100; j < numberOfPoints; j++ ) 
            {
                offsetAbs += allOrderedAbs[i][j];
                offsetDisp += allOrderedDisp[i][j];
            }
            
            offsetAbs /= 100f;
            offsetDisp /= 100f;
            
            for( int j = 0; j < numberOfPoints; j++ ) 
            {
                allOrderedAbs[i][j] -= offsetAbs;
                allOrderedDisp[i][j] -= offsetDisp;
            }
            offsetsAbs[i] = offsetAbs;
            offsetsDisp[i] = offsetDisp;
        }
        
        // some test code
        computeHistogramof2DSignal( allAbs,  m_measuredPatientSpectrumSeries.getNumberOfInstances( ), numberOfPoints, 0, 400, SignalHistogramCalculator.HISTOGRAM_RANGE_DETERMINED_BY_TOTAL_SIGNAL );
        
        // some test code
        computeHistogramof2DSignal( allOrderedAbs,  m_measuredPatientSpectrumSeries.getNumberOfInstances( ), numberOfPoints, 0, 400, SignalHistogramCalculator.HISTOGRAM_RANGE_DETERMINED_BY_TOTAL_SIGNAL );
        
        
        
        // Finally add all arrays together
        for ( int i = 0; i < m_measuredPatientSpectrumSeries.getNumberOfInstances( ); i++ ) 
        {
            for( int j = 0; j < numberOfPoints; j++ ) 
            {
                statFilteredAbs[j] = allOrderedAbs[i][j];
                statFilteredDisp[j] = allOrderedDisp[i][j];
            }
        }
        
        XYPlotFrame plotOffsetsAbs = new XYPlotFrame( "Offsets absorption channel", null );
        XYPlotFrame plotOffsetsDisp = new XYPlotFrame( "Offsets dispersion channel", null );
        
        float[] outxOffsets = new float[m_measuredPatientSpectrumSeries.getNumberOfInstances( )];
        
        for ( int i = 0; i < m_measuredPatientSpectrumSeries.getNumberOfInstances( ); i++ ) 
        {
            outxOffsets[i] = (float)i;
        }
        
        plotOffsetsAbs.addCurve( new Curve( outxOffsets, offsetsAbs, Color.BLACK ), true );
        plotOffsetsAbs.addCurve( new Curve( outxOffsets, offsetsDisp, Color.BLUE ), true );
        
        plotOffsetsDisp.addCurve( new Curve( outxOffsets, offsetsDisp, Color.BLACK ), true );
        
        XYPlotFrame plotAbs = new XYPlotFrame( "Statistic Filtered Filtered Absorption Spectrum", null );
        XYPlotFrame plotDisp = new XYPlotFrame( "Statistic Filtered Dispersion Spectrum", null );
        
        Curve statFilteredAbsCurve = new Curve( outx, statFilteredAbs, Color.blue );
        Curve statFilteredDispCurve  = new Curve( outx, statFilteredDisp, Color.blue );
        
        
        plotAbs.addCurve( statFilteredAbsCurve, true );
        plotDisp.addCurve( statFilteredDispCurve, true );
    }
    
    /**
     * Computes pixel by pixel all orders filterd frequency domain absorption
     * channel : Code used for MRM paper 
     */
    private void computeAllOrdersFilteredResponseFDNoiseSample( ) 
    {
        int numberOfPoints = m_measuredPatientSpectrumSeries.getAbsorptionChannelTD( 0 ).length;
        
        float[] outx = new float[ numberOfPoints ];
        float[] statFilteredAbs = new float[ numberOfPoints ];
        float[] statFilteredDisp = new float[ numberOfPoints ];
        
        float[] absInst = new float[ m_measuredPatientSpectrumSeries.getNumberOfInstances( ) ];
        float[] dispInst = new float[ m_measuredPatientSpectrumSeries.getNumberOfInstances( ) ];
        
        
        float[][] allAbs = new float[m_measuredPatientSpectrumSeries.getNumberOfInstances( )][numberOfPoints];
        float[][] allDisp = new float[m_measuredPatientSpectrumSeries.getNumberOfInstances( )][numberOfPoints];
        
        float[][] allOrderedAbs = new float[m_measuredPatientSpectrumSeries.getNumberOfInstances( )][numberOfPoints];
        float[][] allOrderedDisp = new float[m_measuredPatientSpectrumSeries.getNumberOfInstances( )][numberOfPoints];
        
        //int  numberOfInstances = m_measuredPatientSpectrumSeries.getNumberOfInstances( );
        
        int  numberOfInstances = 32;
        
        // Create array data structures of measured MR Signal
//        for ( int i = 0; i < m_measuredPatientSpectrumSeries.getNumberOfInstances( ); i++ ) 
//        {
//            SingleVoxelSpectralData specData = m_measuredPatientSpectrumSeries.getCurrentInstance( i );
//            
//            float[] abs = specData.getAbsorptionChannelFD( );
//            float[] disp = specData.getDispersionChannelFD( );
//            
//            for ( int j = 0 ; j < numberOfPoints; j++ ) 
//            {
//                allAbs[i][j] = abs[j];
//                allDisp[i][j] = disp[j];
//            }
//            
//            System.out.println( "i = " + i );
//        }
        
        Random rnd = new Random( );
        
        // Create array data structures
        for ( int i = 0; i < numberOfInstances; i++ ) 
        {
            //now on simulated data SingleVoxelSpectralData specData = m_measuredPatientSpectrumSeries.getCurrentInstance( i );
            
            //now on simulated data float[] abs = specData.getAbsorptionChannelFD( );
            //now on simulated data float[] disp = specData.getDispersionChannelFD( );
            
            for ( int j = 0 ; j < numberOfPoints; j++ ) 
            {
                allAbs[i][j] = 1000f * (float)rnd.nextGaussian( );
                
                //allAbs[i][j] = 2000f * ( 1f - 2f * (float)rnd.nextFloat( ) );
                
            }
            
            //System.out.println( "i = " + i );
        }
        
        for ( int  k = 0; k < 1; k++ )
        {
            // Eliminate offsets!
            for ( int i = 0; i < numberOfInstances; i++ )
            {
                float offsetAbs = 0f;
                float offsetDisp = 0f;
                for ( int j = 0; j < numberOfPoints / 4; j++ )
                {
                    offsetAbs += allAbs[i][j];
                    offsetDisp += allDisp[i][j];              
                }
            
                offsetAbs = offsetAbs/(numberOfPoints/4);
                offsetDisp = offsetDisp/(numberOfPoints/4);
            
                for ( int j = 0; j < numberOfPoints; j++ )
                {
                    allAbs[i][j] -= offsetAbs;
                    allDisp[i][j] -= offsetDisp;              
                }
            }
        
            // some test code
            // computeHistogramOf1DSignal( test, 0, 400 );
            computeHistogramof2DSignal( allAbs, numberOfInstances, numberOfPoints, 0, numberOfPoints, SignalHistogramCalculator.HISTOGRAM_RANGE_DETERMINED_BY_TOTAL_SIGNAL );
        
            float[] xaxis = new float[ numberOfPoints ];
        
            // Perform sorting on every array
            for ( int j = 0 ; j < numberOfPoints ; j++ ) 
            {
            
                for ( int i = 0; i < numberOfInstances; i++ ) 
                {
                    absInst[i] = allAbs[i][j];
                    dispInst[i] = allDisp[i][j];
                }
            
                absInst = m_qSort.quickSort( absInst );
                dispInst = m_qSort.quickSort( dispInst );
            
            
                outx[j] = (float)j;
            
                for ( int i = 0; i < numberOfInstances; i++ ) 
            
                {
                    allAbs[i][j] = absInst[i];
                    allDisp[i][j] = dispInst[i];
                }
            
                System.out.println( "Sorting arrays = " + j );
            }
        }
        
        for ( int i = 0; i < numberOfInstances; i++ )
        {
            float[] signal = new float[numberOfPoints];
            for ( int j = 0; j < numberOfPoints; j++ )
            {
                signal[j] = allAbs[i][j];
            }
            
            computeHistogramof1DSignal( signal, 0, numberOfPoints / 4,  SignalHistogramCalculator.HISTOGRAM_RANGE_DETERMINED_BY_TOTAL_SIGNAL );
        }
        
        
        // eliminate offsets!
        for ( int i = 0; i < numberOfInstances; i++ )
        {
            float offsetAbs = 0;
            float offsetDisp = 0;
            for ( int j = 0; j < numberOfPoints / 4; j++ )
            {
                offsetAbs += allAbs[i][j];
                offsetDisp += allDisp[i][j];
            }
            
            offsetAbs = offsetAbs/(numberOfPoints/4);
            offsetDisp = offsetDisp/(numberOfPoints/4);
            
            for ( int j = 0; j < numberOfPoints; j++ )
            {
                allAbs[i][j] -= offsetAbs;
                allDisp[i][j] -= offsetDisp;              
            }
        }
        
        computeHistogramof2DSignal( allAbs,  numberOfInstances, numberOfPoints, 0, numberOfPoints / 4, SignalHistogramCalculator.HISTOGRAM_RANGE_DETERMINED_BY_TOTAL_SIGNAL );
        TDFDFittingDefaults fdef = TDFDFittingDefaults.getInstance( );
        fdef.load( );
        
        float[] offsetsAbs = new float[m_measuredPatientSpectrumSeries.getNumberOfInstances( )];
        float[] offsetsDisp = new float[m_measuredPatientSpectrumSeries.getNumberOfInstances( )];
        
        XYPlotFrame plotOffsetsAbs = new XYPlotFrame( "Offsets absorption channel", null );
        XYPlotFrame plotOffsetsDisp = new XYPlotFrame( "Offsets dispersion channel", null );
        float[] outxOffsets = new float[numberOfPoints];
        
        for ( int i = 0; i < numberOfPoints; i++ ) 
        {
            outxOffsets[i] = (float)i;
        }
        
        // Finally display all orders in one single plot
        for ( int i = 0; i < numberOfInstances; i++ ) 
        {
            for( int j = 0; j < numberOfPoints; j++ ) 
            {
                statFilteredAbs[j] = allAbs[i][j];
                statFilteredDisp[j] = allDisp[i][j];
            }
            if ( i == m_measuredPatientSpectrumSeries.getNumberOfInstances( )/2 ) 
            {
                plotOffsetsAbs.addCurve( new Curve( outxOffsets, statFilteredAbs, Color.BLACK ), true );
                plotOffsetsDisp.addCurve( new Curve( outxOffsets, statFilteredDisp, Color.BLACK ), true );
            }
        }   
    }
    
    
    /**
     * Computes pixel by pixel all orders filterd frequency domain absorption
     * channel
     */
    private void computeAllOrdersFilteredResponseFD( ) 
    {
        int numberOfPoints = m_measuredPatientSpectrumSeries.getAbsorptionChannelTD( 0 ).length;
        
        float[] outx = new float[ numberOfPoints ];
        float[] statFilteredAbs = new float[ numberOfPoints ];
        float[] statFilteredDisp = new float[ numberOfPoints ];
        
        float[] absInst = new float[ m_measuredPatientSpectrumSeries.getNumberOfInstances( ) ];
        float[] dispInst = new float[ m_measuredPatientSpectrumSeries.getNumberOfInstances( ) ];
        
        
        float[][] allAbs = new float[m_measuredPatientSpectrumSeries.getNumberOfInstances( )][numberOfPoints];
        float[][] allDisp = new float[m_measuredPatientSpectrumSeries.getNumberOfInstances( )][numberOfPoints];
        
        float[][] allOrderedAbs = new float[m_measuredPatientSpectrumSeries.getNumberOfInstances( )][numberOfPoints];
        float[][] allOrderedDisp = new float[m_measuredPatientSpectrumSeries.getNumberOfInstances( )][numberOfPoints];
        
        int  numberOfInstances = m_measuredPatientSpectrumSeries.getNumberOfInstances( );
        
        
        // Create array data structures of measured MR Signal
        for ( int i = 0; i < m_measuredPatientSpectrumSeries.getNumberOfInstances( ); i++ ) 
        {
            SingleVoxelSpectralData specData = m_measuredPatientSpectrumSeries.getCurrentInstance( i );
            
            float[] abs = specData.getAbsorptionChannelFD( );
            float[] disp = specData.getDispersionChannelFD( );
            
            TDFDFittingDefaults fdef = TDFDFittingDefaults.getInstance( );
            fdef.load( );
            SignalWriter swrHist = new SignalWriter( "Write 1D Input spectrum [" + i + "]", abs, fdef.getProperty( "Dicom mount path" ) + System.getProperty( "file.separator" ), null );

            for ( int j = 0 ; j < numberOfPoints; j++ ) 
            {
                allAbs[i][j] = abs[j];
                allDisp[i][j] = disp[j];
            }
            
            System.out.println( "i = " + i );
        }
        
        float[] geomMean = new float[numberOfPoints];
        // determine geometric mean and write to disk
        for ( int i = 0; i < numberOfPoints; i++ ) 
        {
            for ( int j = 0 ; j < m_measuredPatientSpectrumSeries.getNumberOfInstances( ) ; j++ )
            {
                geomMean[i] +=  allAbs[j][i];   
            }
        }
        
        TDFDFittingDefaults fdef = TDFDFittingDefaults.getInstance( );
        fdef.load( );
        
        SignalWriter geomMeanWrtr = new SignalWriter( "Write 1D Geommetric mean ", geomMean, fdef.getProperty( "Dicom mount path" ) + System.getProperty( "file.separator" ), null );
   
        for ( int  k = 0; k < 1; k++ )
        {
            // Eliminate offsets! ( preprocessing step )
            for ( int i = 0; i < numberOfInstances; i++ )
            {
                float offsetAbs = 0f;
                float offsetDisp = 0f;
                for ( int j = 0; j < numberOfPoints / 4; j++ )
                {
                    offsetAbs += allAbs[i][j];
                    offsetDisp += allDisp[i][j];              
                }
            
                offsetAbs = offsetAbs/(numberOfPoints/4);
                offsetDisp = offsetDisp/(numberOfPoints/4);
            
                for ( int j = 0; j < numberOfPoints; j++ )
                {
                    allAbs[i][j] -= offsetAbs;
                    allDisp[i][j] -= offsetDisp;              
                }
            }
        
            // some test code
            computeHistogramof2DSignal( allAbs, numberOfInstances, numberOfPoints, 0, numberOfPoints/4 , SignalHistogramCalculator.HISTOGRAM_RANGE_DETERMINED_BY_RANGE );
        
            float[] xaxis = new float[ numberOfPoints ];
        
            // Perform sorting on every array
            for ( int j = 0 ; j < numberOfPoints ; j++ ) 
            {            
                for ( int i = 0; i < numberOfInstances; i++ ) 
                {
                    absInst[i] = allAbs[i][j];
                    dispInst[i] = allDisp[i][j];
                }
            
                absInst = m_qSort.quickSort( absInst );
                dispInst = m_qSort.quickSort( dispInst );
            
                outx[j] = (float)j;
            
                for ( int i = 0; i < numberOfInstances; i++ )
                {
                    allAbs[i][j] = absInst[i];
                    allDisp[i][j] = dispInst[i];
                }
            
                System.out.println( "Sorting arrays = " + j );
            }
        }
        
        for ( int i = 0; i < numberOfInstances; i++ )
        {
            float[] signal = new float[numberOfPoints];
            for ( int j = 0; j < numberOfPoints; j++ )
            {
                signal[j] = allAbs[i][j];
            }
            
            computeHistogramof1DSignal( signal, 0, numberOfPoints / 4, SignalHistogramCalculator.HISTOGRAM_RANGE_DETERMINED_BY_RANGE );
        }
        
        
        // eliminate offsets!
        for ( int i = 0; i < numberOfInstances; i++ )
        {
            float offsetAbs = 0;
            float offsetDisp = 0;
            for ( int j = 0; j < numberOfPoints / 4; j++ )
            {
                offsetAbs += allAbs[i][j];
                offsetDisp += allDisp[i][j];              
            }
            
            offsetAbs = offsetAbs/(numberOfPoints/4);
            offsetDisp = offsetDisp/(numberOfPoints/4);
            
            for ( int j = 0; j < numberOfPoints; j++ )
            {
                allAbs[i][j] -= offsetAbs;
                allDisp[i][j] -= offsetDisp;
            }
        }
        
        computeSNRCurve( numberOfInstances, numberOfPoints, allAbs );
        
        computeHistogramof2DSignal( allAbs, numberOfInstances, numberOfPoints, 0, numberOfPoints / 4, SignalHistogramCalculator.HISTOGRAM_RANGE_DETERMINED_BY_RANGE );
        
        float[] offsetsAbs = new float[m_measuredPatientSpectrumSeries.getNumberOfInstances( )];
        float[] offsetsDisp = new float[m_measuredPatientSpectrumSeries.getNumberOfInstances( )];
        
        XYPlotFrame plotOffsetsAbs = new XYPlotFrame( "Offsets absorption channel", null );
        XYPlotFrame plotOffsetsDisp = new XYPlotFrame( "Offsets dispersion channel", null );
        float[] outxOffsets = new float[numberOfPoints];
        
        for ( int i = 0; i < numberOfPoints; i++ ) 
        {
            outxOffsets[i] = (float)i;
        }
        
        // Finally display all orders in one single plot
        for ( int i = 0; i < numberOfInstances; i++ ) 
        {
            for( int j = 0; j < numberOfPoints; j++ ) 
            {
                statFilteredAbs[j] = allAbs[i][j];
                statFilteredDisp[j] = allDisp[i][j];
            }

            SignalWriter osfWrtr = new SignalWriter( "Write 1D OSF-filtered spectrum [" + i + "]", statFilteredAbs, fdef.getProperty( "Dicom mount path" ) + System.getProperty( "file.separator" ), null );

            if ( i == m_measuredPatientSpectrumSeries.getNumberOfInstances( ) / 2 ) 
            {
                plotOffsetsAbs.addCurve( new Curve( outxOffsets, statFilteredAbs, Color.BLACK ), true );
                plotOffsetsDisp.addCurve( new Curve( outxOffsets, statFilteredDisp, Color.BLACK ), true );
            }
        }   
    }
    
    /**
     * Computes pixel by pixel all orders filterd time domain absorption
     * channel
     */
    private void computeAllOrdersFilteredResponseTD( ) 
    {
        int numberOfPoints = m_measuredPatientSpectrumSeries.getAbsorptionChannelTD( 0 ).length;
        
        float[] outx = new float[ numberOfPoints ];
        float[] statFilteredAbs = new float[ numberOfPoints ];
        float[] statFilteredDisp = new float[ numberOfPoints ];
        
        float[] absInst = new float[ m_measuredPatientSpectrumSeries.getNumberOfInstances( ) ];
        float[] dispInst = new float[ m_measuredPatientSpectrumSeries.getNumberOfInstances( ) ];
        
        
        float[][] allAbs = new float[m_measuredPatientSpectrumSeries.getNumberOfInstances( )][numberOfPoints];
        float[][] allDisp = new float[m_measuredPatientSpectrumSeries.getNumberOfInstances( )][numberOfPoints];
        
        float[][] allOrderedAbs = new float[m_measuredPatientSpectrumSeries.getNumberOfInstances( )][numberOfPoints];
        float[][] allOrderedDisp = new float[m_measuredPatientSpectrumSeries.getNumberOfInstances( )][numberOfPoints];
        
        int  numberOfInstances = m_measuredPatientSpectrumSeries.getNumberOfInstances( );
        
        
        // Create array data structures of measured MR Signal
        for ( int i = 0; i < m_measuredPatientSpectrumSeries.getNumberOfInstances( ); i++ ) 
        {
            SingleVoxelSpectralData specData = m_measuredPatientSpectrumSeries.getCurrentInstance( i );
            
            float[] abs = specData.getAbsorptionChannelTD( );
            float[] disp = specData.getDispersionChannelTD( );
            
            TDFDFittingDefaults fdef = TDFDFittingDefaults.getInstance( );
            fdef.load( );
            
            SignalWriter swrHist = new SignalWriter( "Write 1D Input spectrum [" + i + "]", abs, fdef.getProperty( "Dicom mount path" ) + System.getProperty( "file.separator" ), null );

            for ( int j = 0 ; j < numberOfPoints; j++ ) 
            {
                allAbs[i][j] = abs[j];
                allDisp[i][j] = disp[j];
            }
            
            System.out.println( "i = " + i );
        }
        
        float[] geomMean = new float[numberOfPoints];
        // determine geometric mean and write to disk
        for ( int i = 0; i < numberOfPoints; i++ ) 
        {
            for ( int j = 0 ; j < m_measuredPatientSpectrumSeries.getNumberOfInstances( ) ; j++ )
            {
                geomMean[i] +=  allAbs[j][i];   
            }
        }
        
        TDFDFittingDefaults fdef = TDFDFittingDefaults.getInstance( );
        fdef.load( );
        
        SignalWriter geomMeanWrtr = new SignalWriter( "Write 1D Geommetric mean ", 
                                                      geomMean, fdef.getProperty( "Dicom mount path" ) + 
                                                      System.getProperty( "file.separator" ), null );
   
        for ( int  k = 0; k < 1; k++ )
        {
            // Eliminate offsets! ( preprocessing step )
            for ( int i = 0; i < numberOfInstances; i++ )
            {
                float offsetAbs = 0f;
                float offsetDisp = 0f;
                for ( int j = 0; j < numberOfPoints / 4; j++ )
                {
                    offsetAbs += allAbs[i][j];
                    offsetDisp += allDisp[i][j];              
                }
            
                offsetAbs = offsetAbs/(numberOfPoints/4);
                offsetDisp = offsetDisp/(numberOfPoints/4);
            
                for ( int j = 0; j < numberOfPoints; j++ )
                {
                    allAbs[i][j] -= offsetAbs;
                    allDisp[i][j] -= offsetDisp;     
                }
            }
        
            // some test code
            computeHistogramof2DSignal( allAbs, numberOfInstances, numberOfPoints, numberOfPoints/2, numberOfPoints - 1 , SignalHistogramCalculator.HISTOGRAM_RANGE_DETERMINED_BY_RANGE );
        
            float[] xaxis = new float[ numberOfPoints ];
        
            // Perform sorting on every array
            for ( int j = 0 ; j < numberOfPoints ; j++ ) 
            {            
                for ( int i = 0; i < numberOfInstances; i++ ) 
                {
                    absInst[i] = allAbs[i][j];
                    dispInst[i] = allDisp[i][j];
                }
            
                absInst = m_qSort.quickSort( absInst );
                dispInst = m_qSort.quickSort( dispInst );
            
                outx[j] = (float)j;
            
                for ( int i = 0; i < numberOfInstances; i++ )
                {
                    allAbs[i][j] = absInst[i];
                    allDisp[i][j] = dispInst[i];
                }
            
                System.out.println( "Sorting arrays = " + j );
            }
        }
        
        for ( int i = 0; i < numberOfInstances; i++ )
        {
            float[] signal = new float[numberOfPoints];
            for ( int j = 0; j < numberOfPoints; j++ )
            {
                signal[j] = allAbs[i][j];
            }
            
            computeHistogramof1DSignal( signal, numberOfPoints / 2, numberOfPoints - 1, SignalHistogramCalculator.HISTOGRAM_RANGE_DETERMINED_BY_RANGE );
        }
        
        
        // eliminate offsets!
        for ( int i = 0; i < numberOfInstances; i++ )
        {
            float offsetAbs = 0;
            float offsetDisp = 0;
            for ( int j = 0; j < numberOfPoints / 4; j++ )
            {
                offsetAbs += allAbs[i][j];
                offsetDisp += allDisp[i][j];              
            }
            
            offsetAbs = offsetAbs/(numberOfPoints/4);
            offsetDisp = offsetDisp/(numberOfPoints/4);
            
            for ( int j = 0; j < numberOfPoints; j++ )
            {
                allAbs[i][j] -= offsetAbs;
                allDisp[i][j] -= offsetDisp;
            }
        }
        
        computeSNRCurve( numberOfInstances, numberOfPoints, allAbs );
        
        computeHistogramof2DSignal( allAbs, numberOfInstances, numberOfPoints, numberOfPoints / 2, numberOfPoints - 1, SignalHistogramCalculator.HISTOGRAM_RANGE_DETERMINED_BY_RANGE );
        
        float[] offsetsAbs = new float[m_measuredPatientSpectrumSeries.getNumberOfInstances( )];
        float[] offsetsDisp = new float[m_measuredPatientSpectrumSeries.getNumberOfInstances( )];
        
        XYPlotFrame plotOffsetsAbs = new XYPlotFrame( "Offsets absorption channel", null );
        XYPlotFrame plotOffsetsDisp = new XYPlotFrame( "Offsets dispersion channel", null );
        float[] outxOffsets = new float[numberOfPoints];
        
        for ( int i = 0; i < numberOfPoints; i++ ) 
        {
            outxOffsets[i] = (float)i;
        }
        
        // Finally display all orders in one single plot
        for ( int i = 0; i < numberOfInstances; i++ ) 
        {
            for( int j = 0; j < numberOfPoints; j++ ) 
            {
                statFilteredAbs[j] = allAbs[i][j];
                statFilteredDisp[j] = allDisp[i][j];
            }

            SignalWriter osfWrtr = new SignalWriter( "Write 1D OSF-filtered spectrum [" + i + "]", statFilteredAbs, fdef.getProperty( "Dicom mount path" ) + System.getProperty( "file.separator" ), null );

            if ( i == m_measuredPatientSpectrumSeries.getNumberOfInstances( ) / 2 ) 
            {
                plotOffsetsAbs.addCurve( new Curve( outxOffsets, statFilteredAbs, Color.BLACK ), true );
                plotOffsetsDisp.addCurve( new Curve( outxOffsets, statFilteredDisp, Color.BLACK ), true );
            }
        }   
    }

    
     /**
     * Computes pixel by pixel all orders filterd frequency domain absorption
     * channel
     */
    private void computeAllHistogramsFD( ) 
    {
        int numberOfPoints = m_measuredPatientSpectrumSeries.getAbsorptionChannelTD( 0 ).length;
        
//        float[] outx = new float[ numberOfPoints ];
//        float[] statFilteredAbs = new float[ numberOfPoints ];
//        float[] statFilteredDisp = new float[ numberOfPoints ];
//        
//        float[] absInst = new float[ m_measuredPatientSpectrumSeries.getNumberOfInstances( ) ];
//        float[] dispInst = new float[ m_measuredPatientSpectrumSeries.getNumberOfInstances( ) ];
//        
//        
        float[][] allAbs = new float[m_measuredPatientSpectrumSeries.getNumberOfInstances( )][numberOfPoints];
        float[][] allDisp = new float[m_measuredPatientSpectrumSeries.getNumberOfInstances( )][numberOfPoints];
//        
//        float[][] allOrderedAbs = new float[m_measuredPatientSpectrumSeries.getNumberOfInstances( )][numberOfPoints];
//        float[][] allOrderedDisp = new float[m_measuredPatientSpectrumSeries.getNumberOfInstances( )][numberOfPoints];
        
        int  numberOfInstances = m_measuredPatientSpectrumSeries.getNumberOfInstances( );
        
        
        // Create array data structures of measured MR Signal
        for ( int i = 0; i < m_measuredPatientSpectrumSeries.getNumberOfInstances( ); i++ ) 
        {
            SingleVoxelSpectralData specData = m_measuredPatientSpectrumSeries.getCurrentInstance( i );
            
            float[] abs = specData.getAbsorptionChannelFD( );
            float[] disp = specData.getDispersionChannelFD( );
            
            TDFDFittingDefaults fdef = TDFDFittingDefaults.getInstance( );
            fdef.load( );
            
            SignalWriter swrHist = new SignalWriter( "Write 1D Input spectrum [" + i + "]", abs, fdef.getProperty( "Dicom mount path" ) + System.getProperty( "file.separator" ), null );

            for ( int j = 0 ; j < numberOfPoints; j++ ) 
            {
                allAbs[i][j] = abs[j];
                allDisp[i][j] = disp[j];
            }
            
            System.out.println( "i = " + i );
        }
        
        
        TDFDFittingDefaults fdef = TDFDFittingDefaults.getInstance( );
        fdef.load( );
        
        for ( int i = 0; i < numberOfInstances; i++ )
        {
            float[] signal = new float[numberOfPoints];
            for ( int j = 0; j < numberOfPoints; j++ )
            {
                signal[j] = allAbs[i][j];
            }
            
            computeHistogramof1DSignal( signal, 0, 4 * numberOfPoints / 10, SignalHistogramCalculator.HISTOGRAM_RANGE_DETERMINED_BY_RANGE );
            
            SignalWriter geomMeanWrtr = new SignalWriter( "Write 1D Histogram ", signal, 
                                                          fdef.getProperty( "Dicom mount path" ) + System.getProperty( "file.separator" ), null );
        }
                
    }
    
    public float[] computeSNRCurve( int numberOfInstances, int numberOfPoints, float[][] allAbs )
    {
        float[] xaxis = new float[numberOfInstances];
        float[] noisepower = new float[numberOfInstances];
        float[] signalpower = new float[numberOfInstances];
        float[] peakarea = new float[numberOfInstances];
        
        float[] aritMean = new float[numberOfPoints];
        
        
        for ( int j = 0 ; j < numberOfInstances ; j++ )
        {
            for ( int i = 0 ; i < numberOfPoints; i++ )
            {
                aritMean[i] = aritMean[i] + allAbs[j][i];
            }
        }
        
        for ( int i = 0 ; i < numberOfPoints; i++ )
        {
            aritMean[i] = aritMean[i] / numberOfInstances;
        }
        
        float[] aritPower = new float[ numberOfInstances ];
        float sumPower = 0f;
        
        for ( int i = 0 ; i < 30; i++ )
        {
            sumPower = sumPower + aritMean[i] * aritMean[i];
        }
        
        for ( int i = 0 ; i < numberOfInstances; i++ )
        {
            aritPower[i] = (float)Math.sqrt( sumPower ) / 30f;
        }
        
        for ( int j = 0 ; j < numberOfInstances ; j++ )
        {
            
            for ( int i = 0 ; i < 30 ; i++ )
            {
                aritMean[j] = aritMean[j] + allAbs[i][j];
            }
            
            for ( int i = 0 ; i < 30 ; i++ )
            {
                noisepower[j] = noisepower[j] + ( allAbs[j][i] * allAbs[j][i] );
            }
            for ( int i = 1330 ; i < 1360 ; i++ )
            {
                signalpower[j] = signalpower[j] + ( allAbs[j][i] * allAbs[j][i] );
            }
            for ( int i = 1330 ; i < 1360 ; i++ )
            {
                peakarea[j] = peakarea[j] + allAbs[j][i];
            }
        }
        
        
        float[] SNR = new float[numberOfInstances];
        for ( int j = 0 ; j < numberOfInstances ; j++ )
        {
            xaxis[j] = (float)j;
            SNR[j] = (float)(10.0 * Math.log10( (double)( signalpower[j] / noisepower[j] ) ));
        }
        
        XYPlotFrame SNRFrame = new XYPlotFrame( "SNR vs OSF Channel number", null );
        SNRFrame.addCurve( new Curve( xaxis, SNR ), true );
        
        TDFDFittingDefaults fdef = TDFDFittingDefaults.getInstance( );
        fdef.load( );
        
        SignalWriter snrWriter = new SignalWriter( "Write 1D SNR ", SNR, fdef.getProperty( "Dicom mount path" ) + System.getProperty( "file.separator" ), null );
        
        for ( int j = 0 ; j < numberOfInstances ; j++ )
        {
            aritMean[j] = (float)Math.sqrt( aritMean[j] ) / 30f;
            noisepower[j] = (float)Math.sqrt( noisepower[j] ) / 30f;
            signalpower[j] = signalpower[j] / 30f;
        }
        
        XYPlotFrame noiseFrame = new XYPlotFrame( "Noise as function of Channel number", null );
        noiseFrame.addCurve( new Curve( xaxis, noisepower ), true );
        noiseFrame.addCurve( new Curve( xaxis, aritPower ), true );

        SignalWriter OSFNoiseWriter = new SignalWriter( "OSF noise as function of Channel number", noisepower, fdef.getProperty( "Dicom mount path" ) + System.getProperty( "file.separator" ), null );
        SignalWriter AritMeanWriter = new SignalWriter( "AMF Noise as function of Channel number", aritPower, fdef.getProperty( "Dicom mount path" ) + System.getProperty( "file.separator" ), null );
        
        XYPlotFrame signalFrame = new XYPlotFrame( "Signal as function of Channel number", null );
        signalFrame.addCurve( new Curve( xaxis, signalpower ), true );

        XYPlotFrame peakAreaFrame = new XYPlotFrame( "Peak area of Channel number", null );
        peakAreaFrame.addCurve( new Curve( xaxis, peakarea ), true );

        return SNR;
    }
    
//    /**
//     * Computes pixel by pixel all orders filterd frequency domain absorption
//     * channel
//     */
//    private void computeAllOrdersFilteredResponseFD( ) 
//    {
//        int numberOfPoints = m_measuredPatientSpectrumSeries.getAbsorptionChannelTD( 0 ).length;
//        
//        float[] outx = new float[ numberOfPoints ];
//        float[] statFilteredAbs = new float[ numberOfPoints ];
//        float[] statFilteredDisp = new float[ numberOfPoints ];
//        
//        float[] absInst = new float[ m_measuredPatientSpectrumSeries.getNumberOfInstances( ) ];
//        float[] dispInst = new float[ m_measuredPatientSpectrumSeries.getNumberOfInstances( ) ];
//        
//        
//        float[][] allAbs = new float[m_measuredPatientSpectrumSeries.getNumberOfInstances( )][numberOfPoints];
//        float[][] allDisp = new float[m_measuredPatientSpectrumSeries.getNumberOfInstances( )][numberOfPoints];
//        
//        float[][] allOrderedAbs = new float[m_measuredPatientSpectrumSeries.getNumberOfInstances( )][numberOfPoints];
//        float[][] allOrderedDisp = new float[m_measuredPatientSpectrumSeries.getNumberOfInstances( )][numberOfPoints];
//        
//        // Create array data structures
//        for ( int i = 0; i < m_measuredPatientSpectrumSeries.getNumberOfInstances( ); i++ ) 
//        {
//            SingleVoxelSpectralData specData = m_measuredPatientSpectrumSeries.getCurrentInstance( i );
//            
//            float[] abs = specData.getAbsorptionChannelFD( );
//            float[] disp = specData.getDispersionChannelFD( );
//            
//            for ( int j = 0 ; j < numberOfPoints; j++ ) 
//            {
//                allAbs[i][j] = abs[j];
//                allDisp[i][j] = disp[j];
//            }
//            
//            System.out.println( "i = " + i );
//        }
//        
//        float[] test = new float[ numberOfPoints ];
//        for ( int t = 0 ; t < numberOfPoints ; t++ )
//        {
//            test[t] = allAbs[5][t];
//        }
//        
//        // some test code
//        // computeHistogramOf1DSignal( test, 0, 400 );
//        computeHistogramof2DSignal( allAbs,  m_measuredPatientSpectrumSeries.getNumberOfInstances( ), numberOfPoints, 0, 400 );
//        
//        float[] xaxis = new float[ numberOfPoints ];
//        
//        // Perform sorting on every array
//        for ( int j = 0 ; j < numberOfPoints ; j++ ) 
//        {
//            
//            for ( int i = 0; i < m_measuredPatientSpectrumSeries.getNumberOfInstances( ); i++ ) 
//            {
//                absInst[i] = allAbs[i][j];
//                dispInst[i] = allDisp[i][j];
//            }
//            
//            absInst = m_qSort.quickSort( absInst );
//            dispInst = m_qSort.quickSort( dispInst );
//            
//            
//            outx[j] = (float)j;
//            
//            for ( int i = 0; i < m_measuredPatientSpectrumSeries.getNumberOfInstances( ); i++ ) 
//            
//            {
//                allOrderedAbs[i][j] = absInst[i];
//                allOrderedDisp[i][j] = dispInst[i];
//            }
//            
//            System.out.println( "Sorting arrays = " + j );
//        }
//        
//        computeHistogramof2DSignal( allOrderedAbs,  m_measuredPatientSpectrumSeries.getNumberOfInstances( ), numberOfPoints, 0, 400 );
//        MRFittingDefaults fdef = MRFittingDefaults.getInstance( );
//        
//        
//         SignalWriter swr = new SignalWriter( "Raw 2D Absorption Channel Data", allAbs,
//                                             m_measuredPatientSpectrumSeries.getNumberOfInstances( ), 
//                                             numberOfPoints, fdef.getProperty( "Dicom mount path" ) + System.getProperty( "file.separator" ), null );
//        
//        
//        SignalWriter swr1 = new SignalWriter( "Ordered 2D Absorption Channel Data", allOrderedAbs,
//                                              m_measuredPatientSpectrumSeries.getNumberOfInstances( ), 
//                                              numberOfPoints, fdef.getProperty( "Dicom mount path" ) + System.getProperty( "file.separator" ), null );
//        
//       
//        
//        float[] offsetsAbs = new float[m_measuredPatientSpectrumSeries.getNumberOfInstances( )];
//        float[] offsetsDisp = new float[m_measuredPatientSpectrumSeries.getNumberOfInstances( )];
//        
//        XYPlotFrame plotOffsetsAbs = new XYPlotFrame( "Offsets absorption channel", null );
//        XYPlotFrame plotOffsetsDisp = new XYPlotFrame( "Offsets dispersion channel", null );
//        float[] outxOffsets = new float[numberOfPoints];
//        
//        for ( int i = 0; i < numberOfPoints; i++ ) 
//        {
//            outxOffsets[i] = (float)i;
//        }
//        
//        // Finally display all orders in one single plot
//        for ( int i = 0; i < m_measuredPatientSpectrumSeries.getNumberOfInstances( ); i++ ) 
//        {
//            for( int j = 0; j < numberOfPoints; j++ ) 
//            {
//                statFilteredAbs[j] = allOrderedAbs[i][j];
//                statFilteredDisp[j] = allOrderedDisp[i][j];
//            }
//            if ( ( i - 1 ) % 5 == 0 ) 
//            {
//                plotOffsetsAbs.addCurve( new Curve( outxOffsets, statFilteredAbs, Color.BLACK ), true );
//                plotOffsetsDisp.addCurve( new Curve( outxOffsets, statFilteredDisp, Color.BLACK ), true );
//            }
//        }
//        
//    }
    


    /**
     * Computes an histogram for a float signal 'sig'
     * using a SignalHistogramCalculator object
     * @param sig2D
     * @param numberOfChannels
     * @param numberOfPoints
     * @param startIndex
     * @param stopIndex
     * @param computationMethod
     * @return 
     */
    public Curve computeHistogramof2DSignal( float[][] sig2D, int numberOfChannels, 
                                             int numberOfPoints, int startIndex, 
                                             int stopIndex, int computationMethod )
    {
        SignalHistogramCalculator histCalc = new SignalHistogramCalculator( sig2D, numberOfChannels, numberOfPoints, 0, numberOfPoints / 4 );

        Curve cur = histCalc.calculateHistogram( );

        XYPlotFrame plotHistogram = new XYPlotFrame( "Histogram 2D channel", null );
        plotHistogram.addCurve( cur, true );

        TDFDFittingDefaults fdef = TDFDFittingDefaults.getInstance( );
        fdef.load( );
        
        SignalWriter swrHist = new SignalWriter( "Write 2D Sum histogram", 
                                                 cur.getXvalues(), cur.getYvalues( ), 
                                                 fdef.getProperty( "Dicom mount path" ) + 
                                                 System.getProperty( "file.separator" ), null );

        return cur;

    }

    
    /**
     * Computes an histogram for a float signal 'sig'
     * using a SignalHistogramCalculator object
     * @param sig2D
     * @param numberOfChannels
     * @param numberOfPoints
     * @param startIndex
     * @param stopIndex
     * @param computationMethod
     * @return 
     */
    public Curve computeHistogramof2DSignal( double[][] sig2D, int numberOfChannels, 
                                             int numberOfPoints, int startIndex, 
                                             int stopIndex, int computationMethod )
    {
        SignalHistogramCalculator histCalc = new SignalHistogramCalculator( sig2D, numberOfChannels, numberOfPoints, 0, numberOfPoints / 4 );
        
        Curve cur = histCalc.calculateHistogram( );
        
        XYPlotFrame plotHistogram = new XYPlotFrame( "Histogram 2D channel", null );
        plotHistogram.addCurve( cur, true );
        
        TDFDFittingDefaults fdef = TDFDFittingDefaults.getInstance( );
        SignalWriter swrHist = new SignalWriter( "Write 2D Sum histogram", 
                                                 cur.getXvalues( ), cur.getYvalues( ), 
                                                 fdef.getProperty( "Dicom mount path" ) + 
                                                 System.getProperty( "file.separator" ), null );

        return cur;
        
    }
    
    
    /**
     * Computes an histogram for a float signal 'sig'
     * using a SignalHistogramCalculator object
     * @param sig1D
     * @param startIndex
     * @param stopIndex
     * @param computationMethod
     * @return 
     */
    public Curve computeHistogramof1DSignal( float[] sig1D, 
                                             int startIndex, 
                                             int stopIndex, 
                                             int computationMethod )
    {
        
        SignalHistogramCalculator histCalc = new SignalHistogramCalculator( sig1D, startIndex, stopIndex, computationMethod );
        
        Curve cur = histCalc.calculateHistogram( );
        
        XYPlotFrame plotHistogram = new XYPlotFrame( "Histogram 1D channel", null );
        plotHistogram.addCurve( cur, true );
        
        TDFDFittingDefaults fdef = TDFDFittingDefaults.getInstance( );
        fdef.load( );
        
        SignalWriter swrHist = new SignalWriter( "Write 1D Sum histogram", 
                                                 cur.getXvalues( ), 
                                                 cur.getYvalues( ), 
                                                 fdef.getProperty( "Dicom mount path" ) + 
                                                 System.getProperty( "file.separator" ), null );

        return cur;
    }
    
    
    /**
     * Routine that calculates the one sided TTest.
     * @param array1
     * @param array2
     * @return 
     */
    public String calculateTTest( float[] array1, float[] array2 ) 
    {
        String reportString;
        
        // First, set up the random variables
        randomVar rv1 = new randomVar( );
        rv1.mean = SignalTools.mean( array1 );
        rv1.sumSquares = SignalTools.sum2( array1 );
        rv1.n = array1.length;
        
        randomVar rv2 = new randomVar( );
        rv2.mean = SignalTools.mean( array2 );
        rv2.sumSquares = SignalTools.sum2( array2 );
        rv2.n = array2.length;
        
        double sv1 = 0;	// 1st dataset sample variance
        double sv2 = 0;	// 2nd dataset sample variance
        
        // sample--not pop. variance is calculated below
        sv1 = ( rv1.sumSquares / rv1.n - rv1.mean * rv1.mean ) *
        ( rv1.n / ( rv1.n - 1 ) );
        sv2 = ( rv2.sumSquares / rv2.n - rv2.mean * rv2.mean ) *
        ( rv2.n / ( rv2.n - 1 ) );
        
        /*
            Now it remains to be determined if the population variances can
            be pooled or not. To see this, an F-test can be applied to them.
            This decision is determined automatically, with an alpha level of
            1%, in order to not encumber the user with details.
         */
        
        double f = 0;
        double fAlpha = 0;
        SpecialFunctions sf = new SpecialFunctions( );
        
        if ( sv1 > sv2 ) 
        {
            f = sv1 / sv2;
            fAlpha = sf.fDist( rv1.n - 1, rv2.n - 1, f );
        }
        else 
        {
            f = sv2 / sv1;
            fAlpha = sf.fDist( rv2.n - 1, rv1.n - 1, f );
        }
        
        double df = 0;		// t Test degrees of freedom
        double t = 0;		// t value
        String comment = new String( );
        String comment1 = new String( );
        
        if ( fAlpha <= 0.005 ) 
        {
            comment = "An F test on the sample variances indicates that they are " + newln +
            "probably not from the same population (the variances " + newln +
            "can't be pooled), at an alpha level of " + fAlpha + "." + newln +
            "Thus, the t-test was set up for samples with unequal variances. "+ newln +
            "(The degrees of freedom were adjusted.)";
            
            double svn1 = sv1 / rv1.n;
            double svn2 = sv2 / rv2.n;
            
            df = Math.pow( svn1 + svn2, 2 ) /
            ( Math.pow( svn1, 2 ) / ( rv1.n + 1 ) + Math.pow( svn2, 2 ) / ( rv2.n + 1 ) )  - 2;
            t = Math.abs( rv1.mean - rv2.mean ) / Math.sqrt( sv1 / rv1.n + sv2 / rv2.n );
        }
        else 
        {
            comment = "An F test on the sample variances indicates that they could be " + newln +
            "from the same population, (alpha level of 0.005)." + newln +
            "Accordingly, the t-test was set up for samples with equal population variance.";
            df = rv1.n + rv2.n - 2;
            double sp = Math.sqrt( ( ( rv1.n - 1 ) * sv1 + ( rv2.n - 1 ) * sv2 ) /
            ( rv1.n + rv2.n - 2 ) );
            t = Math.abs( rv1.mean - rv2.mean ) * Math.sqrt( rv1.n * rv2.n / ( rv1.n + rv2.n ) ) / sp;
        }
        
        double pVal = sf.stDist( df, t );
        String pValComment = "" + pVal;
        
        if ( pVal <= 0.01 ) 
        {
            comment1 = "This probability indicates that there is a difference in sample means.";
            if ( pVal <= 0.0001 ) {
                pValComment = "< 0.0001";
            }
        }
        else if ( pVal <= 0.05 )
        {
            comment1 = "This probability indicates that there may be a difference in sample means.";
        }
        else {
            comment1 = "There is not a significant difference in the sample means. " + newln +
            "A difference could not be detected due to large variability, small sample size, " + newln +
            "or both. Of course, it's possible that the samples really are from the same population!";
        }
        
        // a hack to take care of garbage data
        if ( ( rv1.n == 0 ) || ( rv2.n == 0 ) ) 
        {
            comment1 = "There is a problem with the roi data. ";
        }
               
        // convert variance to std. deviation for report
        sv1 = Math.sqrt( sv1 );
        sv2 = Math.sqrt( sv2 );
        
        // build the report
        char tab = (char) 9;
        reportString = "Student's t-Test for Comparing Things " + newln +
        "Java Implementation by Bryan Lewis, Kent State University, 1997 " + newln +
        "This test determines if there is a significant "+ newln + "difference between "
        + "the averages of the two samples above. " + newln +
        newln +
        "Mean of first data set                   :" + tab + rv1.mean +   newln +
        "Standard deviation of first data set     :" + tab + sv1 +  newln +
        "Number of observations in the first set  :" + tab + rv1.n +  newln +
        "Mean of second data set                  :" + tab + rv2.mean +  newln +
        "Standard deviation of second data set    :" + tab + sv2 +  newln +
        "Number of observations in the second set :" + tab + rv2.n +  newln +
        newln +
        "Degrees of freedom   :" + tab + df +  newln +
        "t Value (one-tailed) :" + tab + t +  newln +
        "P(x>t)               :" + tab + pValComment +  newln +
        newln + comment1 +  newln + newln  + comment;
        
        
        return reportString;
    }
    
    /**
     * Little helper class
     */
    class randomVar 
    {
        // a generic random variable
        public double mean;
        public double sumSquares;
        public double n;
    }
    
    // private attributes
    private final String newln = System.getProperty("line.separator");
    private String m_report = "";
    
    private int  m_minStableOffsetValue;
    private int  m_maxStableOffsetValue;
    private SingleVoxelSpectralDataSeries m_measuredPatientSpectrumSeries;
    
    private float[] m_magnitudes;
    private float[] m_phases;
    
    private float[] m_noiseVariance;
    private float[] m_noiseMean;
    
    private QuickSortFloat m_qSort = new QuickSortFloat( );
}
