/*
 * File: $Id: SingleVoxelSpectrumSeriesViewer.java,v 1.2 2006/10/02 06:12:21 hans Exp $
 * History
 * Date: 24.03.2005, 11:26  Authors: Hans Slotboom
 *
 * Changes:
 *
 * Copyright (C) 2003 - 2005 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

/*
 * SingleVoxelSpectrumSeriesViewer.java
 *
 * Created on March 24, 2005, 11:34 AM
 */
 
package org.spectrim.tdfdfit;


import org.spectrim.xyplot.Curve;
import org.spectrim.xyplot.XYPlotFrame;

import java.awt.Cursor;
import java.awt.Color;
import java.awt.Point;

import java.awt.geom.Point2D;



import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;

import java.util.Random;

import javax.swing.JMenuItem;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JMenu;
import javax.swing.ButtonGroup;
import javax.swing.JFrame;

import java.util.Iterator;
import java.util.LinkedList;
import org.spectrim.dicom.DicomDirectory;

import org.spectrim.xyplot.XYPlotRange;
import org.spectrim.xyplot.XYPlot;


/**
 *
 * Class that allows the user to view and manipulate single voxel spectra
 * series, which is for instance useful for repetitively measured spectra
 * which are separately stored.
 *
 * @author  hans
 */
public class SingleVoxelSpectrumSeriesViewer extends XYPlot implements ActionListener, MouseListener, MouseMotionListener
{
    
    /** 
     * Creates a new instance of SingleVoxelSpectrumViewer. 
     * @param title
     * @param parent
     * @param ddir
     */
    public SingleVoxelSpectrumSeriesViewer( String title, JFrame parent, DicomDirectory ddir )
    {
        super( title , parent );
        m_parent = parent;
        m_dicomDir = ddir;
        
        this.addMouseListener( this );
        this.addMouseMotionListener( this );
        addPopupMenu( );
        
        m_svsDataClipboard.addActionListener( this );
        
    }

    /**
     * Creates a new instance of SingleVoxelSpectrumViewer.
     * @param title
     * @param specDataName
     * @param specData the spectral data object containing the complex data points
     * @param parent
     */
    public SingleVoxelSpectrumSeriesViewer( String title, String specDataName, 
                                            SingleVoxelMeasuredSpectralData specData, 
                                            JFrame parent )
    {
        super( title , parent );
        
        m_parent = parent;
        
        this.addMouseListener( this );
        this.addMouseMotionListener( this );
        
        addPopupMenu( );
        
        m_responseList.add( specData );
        m_responseNameList.add( specDataName );
        
        if ( m_applyProcessing.isSelected( ) )
        {
            m_sdp = new SingleVoxelSpectralDataProcessing( m_responseList );
            m_responseList = m_sdp.zeroFill( 1.0f );
        }
        
        m_svsDataClipboard.addActionListener( this );
        
        if ( m_svsDataClipboard.containsData( ) == true )
        {
            this.m_pasteSingleVoxelSpectrumDataMenuItem.setEnabled( true );
        }
        
        // get the default Hertz representation
        this.addCurve( specData.getAbsorptionCurveFDHertz( ), true );
    }
    
    
    /**
     * Adds a context or popup menu to the spectrum viewer that allows the
     * manipulation of data.
     */
    private void addPopupMenu( )
    {
        // The popup menu is instantiated in the base class and is extended 
        // below
        
         m_popupmenu.addSeparator( );
        
        m_copySingleVoxelSpectrumDataMenuItem = new JMenuItem( "Copy spectral data" );
        m_copySingleVoxelSpectrumDataMenuItem.addActionListener( this );
        m_copySingleVoxelSpectrumDataMenuItem.setEnabled( false );
        
        m_pasteSingleVoxelSpectrumDataMenuItem = new JMenuItem( "Paste spectral data in new frame" );
        m_pasteSingleVoxelSpectrumDataMenuItem.addActionListener( this );
        m_pasteSingleVoxelSpectrumDataMenuItem.setEnabled( false );
        
        m_popupmenu.add( m_copySingleVoxelSpectrumDataMenuItem );
        m_popupmenu.add( m_pasteSingleVoxelSpectrumDataMenuItem );
        
        m_popupmenu.addSeparator( );
        
        TDFDFittingDefaults fitdef = TDFDFittingDefaults.getInstance( );
        fitdef.load( );
        
        String fittingDomain = fitdef.getProperty( TDFDFittingDefaults.DISPLAY_DOMAIN );
        String displayChannel = fitdef.getProperty( TDFDFittingDefaults.DISPLAY_CHANNEL );
        
        m_domainSelector = new JMenu( "Display domain" );
        m_domainSelector.setToolTipText( "Select time or frequency display domain" );
        m_domainSelector.addActionListener( this );
        
        m_timeDomain = new JRadioButtonMenuItem( "Time domain" );
        m_timeDomain.addActionListener( this );
        
        m_frequencyDomain = new JRadioButtonMenuItem( "Frequency domain" );
        m_frequencyDomain.addActionListener( this );
        
        m_domainButtonGroup = new ButtonGroup( );
        
        m_domainButtonGroup.add( m_timeDomain );
        m_domainButtonGroup.add( m_frequencyDomain );
        
        m_domainSelector.add( m_timeDomain );
        m_domainSelector.add( m_frequencyDomain );
        
        m_popupmenu.add( m_domainSelector );
        
        if ( fittingDomain.compareTo( TDFDFittingDefaults.FREQ_DOMAIN ) == 0 )
        {
            m_frequencyDomain.setSelected( true );
        }
        else
        {
            m_timeDomain.setSelected( true );
        }
        
        JMenuItem  m_channelSelector;
        
        m_channelSelector = new JMenu( "Display channel" );
        m_channelSelector.setToolTipText( "Allows the selection of which channel has to be displayed" );
        
        m_absorption = new JRadioButtonMenuItem( "Absorption" );
        m_absorption.setToolTipText( "Displays the absorption (real) channel" );
        m_absorption.addActionListener( this );
        
        m_dispersion = new JRadioButtonMenuItem( "Dispersion" );
        m_dispersion.setToolTipText( "Displays the dispersion (imaginary) channel" );
        m_dispersion.addActionListener( this );
        
        m_magnitude = new JRadioButtonMenuItem( "Magnitude" );
        m_magnitude.setToolTipText( "Displays the magnitude representation of absorption and dispersion channel" );
        m_magnitude.addActionListener( this );    

        m_power = new JRadioButtonMenuItem( "Power" );
        m_power.setToolTipText( "Displays the power representation of absorption and dispersion channel" );
        m_power.addActionListener( this );          
        
        m_phase = new JRadioButtonMenuItem( "Phase" );
        m_phase.setToolTipText( "Displays the magnitude of absorption and dispersion channel" );
        m_phase.addActionListener( this );    
        
        m_channelSelector.add( m_absorption );
        m_channelSelector.add( m_dispersion );
        m_channelSelector.add( m_magnitude );
        m_channelSelector.add( m_power );
        m_channelSelector.add( m_phase );
        
        m_channelButtonGroup = new ButtonGroup( );
        
        m_channelButtonGroup.add( m_absorption );
        m_channelButtonGroup.add( m_dispersion );
        m_channelButtonGroup.add( m_magnitude );
        m_channelButtonGroup.add( m_power );
        m_channelButtonGroup.add( m_phase );
        
        if ( displayChannel.compareTo( TDFDFittingDefaults.ABSORPTION ) == 0 )
        {
            m_absorption.setSelected( true );
        }

        if ( displayChannel.compareTo( TDFDFittingDefaults.DISPERSION ) == 0 )
        {
            m_dispersion.setSelected( true );
        }

        if ( displayChannel.compareTo( TDFDFittingDefaults.MAGNITUDE ) == 0 )
        {
            m_magnitude.setSelected( true );
        }
        
        if ( displayChannel.compareTo( TDFDFittingDefaults.POWER ) == 0 )
        {
            m_power.setSelected( true );
        }

        if ( displayChannel.compareTo( TDFDFittingDefaults.PHASE ) == 0 )
        {
            m_phase.setSelected( true );
        }

        m_popupmenu.add( m_channelSelector );
        
        m_displayUnitSelector = new JMenu( "Display units" );
        
        m_displayPointsOnlyUnit = new JRadioButtonMenuItem( "Points - Points" );
        m_displayPointsOnlyUnit.addActionListener( this );
        m_displaySecondsOrHertzUnit = new JRadioButtonMenuItem( "Seconds - Hz" );
        m_displaySecondsOrHertzUnit.addActionListener( this );
        m_displaySecondsOrPPMUnit = new JRadioButtonMenuItem( "Seconds - ppm");
        m_displaySecondsOrPPMUnit.addActionListener( this );
        
        m_displayUnitSelector.add( m_displayPointsOnlyUnit );
        m_displayUnitSelector.add( m_displaySecondsOrHertzUnit );
        m_displayUnitSelector.add( m_displaySecondsOrPPMUnit );
        
        m_unitSelectionButtonGroup = new ButtonGroup( );
        m_unitSelectionButtonGroup.add( m_displayPointsOnlyUnit );
        m_unitSelectionButtonGroup.add( m_displaySecondsOrHertzUnit );
        m_unitSelectionButtonGroup.add( m_displaySecondsOrPPMUnit );
        
        String selectedUnit = fitdef.getProperty( TDFDFittingDefaults.DISPLAY_UNIT );

        if ( ( selectedUnit.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_POINTS ) == 0 ) ||
             ( selectedUnit.compareTo( TDFDFittingDefaults.TIME_DOMAIN_POINTS ) == 0 ) )
        {
            m_displayPointsOnlyUnit.setSelected( true );
        }

        if ( ( selectedUnit.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_HZ ) == 0 ) ||
             ( selectedUnit.compareTo( TDFDFittingDefaults.TIME_DOMAIN_SECONDS ) == 0 ) )
        {
            m_displaySecondsOrHertzUnit.setSelected( true );
        }
        
        if ( ( selectedUnit.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 ) ||
             ( selectedUnit.compareTo( TDFDFittingDefaults.TIME_DOMAIN_SECONDS ) == 0 ) )
        {
            m_displaySecondsOrPPMUnit.setSelected( true );
        }
        
        m_popupmenu.add( m_displayUnitSelector );
        
        m_signalProcessing = new JMenu( "Signal processing" );
        m_signalProcessing.setToolTipText( "Allows you to enable processing and define processing parameters" );
        
        m_applyProcessing= new JRadioButtonMenuItem( "Apply signal processing");
        m_applyProcessing.setToolTipText( "Apply zerofilling, apodization and phasing before FFT" );
        m_applyProcessing.addActionListener( this );
        
        m_applyProcessing.setSelected( true );
        
        m_signalProcessing.add( m_applyProcessing );
        
        m_setApodizationParameters = new JMenuItem( "Apodization .." );
        m_setApodizationParameters.setToolTipText( "Allows you to set the apodization time constants" );
        m_setApodizationParameters.addActionListener( this );
        m_signalProcessing.add( m_setApodizationParameters );
        
        
        m_multiplyFirstTDPoint = new JMenuItem( "Multiply first TD samplepoint .." );
        m_multiplyFirstTDPoint.setToolTipText( "Allows for spectral offset removal" );
        m_multiplyFirstTDPoint.addActionListener( this );
        m_signalProcessing.add( m_multiplyFirstTDPoint );
        
                
        m_offsetCorrectionFD = new JMenuItem( "Offset correction FD .." );
        m_offsetCorrectionFD.setToolTipText( "Allows for frequency domain offset removal" );
        m_offsetCorrectionFD.addActionListener( this );
        m_signalProcessing.add( m_offsetCorrectionFD );
        
        m_setZeroFillingParameters = new JMenuItem( "Zero filling .." );
        m_setZeroFillingParameters.setToolTipText( "Allows you to set the zero filling factor" );
        m_setZeroFillingParameters.addActionListener( this );
        m_signalProcessing.add( m_setZeroFillingParameters );
        
        m_setPhasingParameters = new JMenuItem( "Phasing .." );
        m_setPhasingParameters.setToolTipText( "Allows you to set the frequency domain phasing parameters" );
        m_setPhasingParameters.addActionListener( this );
        
        m_frequencyDependentBaselineElimination = new JMenuItem( "Define baseline for MM baseline elimination .." );
        
        m_frequencyDependentBaselineElimination.setToolTipText( "Allows for broad baseline elimination using splines" );
        m_frequencyDependentBaselineElimination.addActionListener( this );
        m_signalProcessing.add( m_frequencyDependentBaselineElimination );
        
        m_removeMacromolecularBaseline = new JMenuItem( "Remove broad baseline" );
        m_removeMacromolecularBaseline.addActionListener( this );
        m_removeMacromolecularBaseline.setEnabled( false );
        m_signalProcessing.add( m_removeMacromolecularBaseline );
        
        m_signalProcessing.add( m_setPhasingParameters );
        
        m_setShiftParameter = new JMenuItem( "Shift .." );
        m_setShiftParameter.setToolTipText( "Allows you to set the amount of Hz the spectrum should be shifted" );
        m_setShiftParameter.addActionListener( this );
        
        m_signalProcessing.add( m_setShiftParameter );
        
        m_performOSFFiltering = new JMenuItem( "Perform OSF filtering .. " );
        m_performOSFFiltering.addActionListener( this );
        m_performOSFFiltering.setToolTipText( "Performs non linear OSF filtering on response series" );
        
        m_signalProcessing.add( m_performOSFFiltering );
        
        m_performARSOSDenoising = new JMenuItem( "Perform ARSOS denoising .. " );
        m_performARSOSDenoising.addActionListener( this );
        m_performARSOSDenoising.setToolTipText( "Performs denoising using repetitive order statistics filtering" );
        
        m_signalProcessing.add( m_performARSOSDenoising );
        
        
        m_performOSFFittingPlainARSOS = new JMenuItem( "Perform plain OSF followed by fitting .. " );
        m_performOSFFittingPlainARSOS.addActionListener( this );
        m_performOSFFittingPlainARSOS.setToolTipText( "Performs a plain ARSOS and fits N-outputs seperately" );
        
        m_signalProcessing.add( m_performOSFFittingPlainARSOS );
        
        m_performOSFFittingCombinedARSOS = new JMenuItem( "Perform combined OSF followed by fitting .. " );
        m_performOSFFittingCombinedARSOS.addActionListener( this );
        m_performOSFFittingCombinedARSOS.setToolTipText( "Performs a combined ARSOS and fits N-outputs seperately" );
        
        m_signalProcessing.add( m_performOSFFittingCombinedARSOS );
        
        m_popupmenu.add( m_signalProcessing );
        
        add( m_popupmenu );
    }
    
    /**
     * Sets the spectral data of the viewer and displays the absorption 
     * channel curve in the view by default.
     *
     * @param specDataName
     * @param specData the spectral data object containing the complex data points
     * @param refresh
     */
    public void addSpectralDataSet( String specDataName, 
                                    SingleVoxelMeasuredSpectralData specData, 
                                    boolean refresh )
    {
        
        m_copySingleVoxelSpectrumDataMenuItem.setEnabled( true );
        
        m_responseNameList.add( specDataName );
        m_responseList.add( specData );
        m_responseListBuffer.add( specData );
        
        TDFDFittingDefaults def = TDFDFittingDefaults.getInstance( );
        def.load( );
        
        // which unit type does the user want to be displayed is stored and updated
        // in the MRFittingDefault class
        String displayType = def.getProperty( TDFDFittingDefaults.DISPLAY_UNIT );
        String displayChannel = def.getProperty( TDFDFittingDefaults.DISPLAY_CHANNEL );
        String displayDomain = def.getProperty( TDFDFittingDefaults.DISPLAY_DOMAIN );
         
        if ( displayDomain.compareTo( TDFDFittingDefaults.FREQ_DOMAIN ) == 0 )
        {
            if ( displayChannel.compareTo( TDFDFittingDefaults.ABSORPTION ) == 0 )
            {
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_POINTS ) == 0 )
                {
                    addCurve( specData.getAbsorptionCurveFDPoints( ) , refresh );
                }
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_HZ ) == 0 )
                {
                    addCurve( specData.getAbsorptionCurveFDHertz( ) , refresh );
                }
        
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
                {
                    addCurve( specData.getAbsorptionCurveFDPPM( ) , refresh );
                }
            }

            if ( displayChannel.compareTo( TDFDFittingDefaults.DISPERSION ) == 0 )
            {
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_POINTS ) == 0 )
                {
                    addCurve( specData.getDispersionCurveFDPoints( ) , refresh );
                }
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_HZ ) == 0 )
                {
                    addCurve( specData.getDispersionCurveFDHertz( ) , refresh );
                }
        
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
                {
                    addCurve( specData.getDispersionCurveFDPPM( ) , refresh );
                }
            }

            if ( displayChannel.compareTo( TDFDFittingDefaults.MAGNITUDE ) == 0 )
            {
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_POINTS ) == 0 )
                {
                    addCurve( specData.getMagnitudeCurveFDPoints( ) , refresh );
                }
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_HZ ) == 0 )
                {
                    addCurve( specData.getMagnitudeCurveFDHertz( ) , refresh );
                }
        
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
                {
                    addCurve( specData.getMagnitudeCurveFDPPM( ) , refresh );
                }
            }

            if ( displayChannel.compareTo( TDFDFittingDefaults.POWER ) == 0 )
            {
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_POINTS ) == 0 )
                {
                    addCurve( specData.getPowerCurveFDPoints( ) , refresh );
                }
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_HZ ) == 0 )
                {
                    addCurve( specData.getPowerCurveFDHertz( ) , refresh );
                }
        
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
                {
                    addCurve( specData.getPowerCurveFDPPM( ) , refresh );
                }
            }

            if ( displayChannel.compareTo( TDFDFittingDefaults.PHASE ) == 0 )
            {
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_POINTS ) == 0 )
                {
                    addCurve( specData.getPhaseCurveFDPoints( ) , refresh );
                }
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_HZ ) == 0 )
                {
                    addCurve( specData.getPhaseCurveFDHertz( ) , refresh );
                }
        
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
                {
                    addCurve( specData.getPhaseCurveFDPPM( ) , refresh );
                }
            }
        }
        
        if ( displayDomain.compareTo( TDFDFittingDefaults.TIME_DOMAIN ) == 0 )
        {
            if ( displayChannel.compareTo( TDFDFittingDefaults.ABSORPTION ) == 0 )
            {
                if ( displayType.compareTo( TDFDFittingDefaults.TIME_DOMAIN_POINTS ) == 0 )
                {
                    addCurve( specData.getAbsorptionCurveTDPoints( ) , refresh );
                }
                if ( displayType.compareTo( TDFDFittingDefaults.TIME_DOMAIN_SECONDS ) == 0 )
                {
                    addCurve( specData.getAbsorptionCurveTDSeconds(), refresh );
                }
            }

            if ( displayChannel.compareTo( TDFDFittingDefaults.DISPERSION ) == 0 )
            {
                if ( displayType.compareTo( TDFDFittingDefaults.TIME_DOMAIN_POINTS ) == 0 )
                {
                    addCurve( specData.getDispersionCurveTDPoints( ) , refresh );
                }
                if ( displayType.compareTo( TDFDFittingDefaults.TIME_DOMAIN_SECONDS ) == 0 )
                {
                    addCurve( specData.getDispersionCurveTDSeconds( ), refresh );
                }
            }

            if ( displayChannel.compareTo( TDFDFittingDefaults.MAGNITUDE ) == 0 )
            {
                if ( displayType.compareTo( TDFDFittingDefaults.TIME_DOMAIN_POINTS ) == 0 )
                {
                    addCurve( specData.getMagnitudeCurveTDPoints( ) , refresh );
                }
                if ( displayType.compareTo( TDFDFittingDefaults.TIME_DOMAIN_SECONDS ) == 0 )
                {
                    addCurve( specData.getMagnitudeCurveTDSeconds( ), refresh );
                }
            }
            
            if ( displayChannel.compareTo( TDFDFittingDefaults.POWER ) == 0 )
            {
                if ( displayType.compareTo( TDFDFittingDefaults.TIME_DOMAIN_POINTS ) == 0 )
                {
                    addCurve( specData.getPowerCurveTDPoints( ) , refresh );
                }
                if ( displayType.compareTo( TDFDFittingDefaults.TIME_DOMAIN_SECONDS ) == 0 )
                {
                    addCurve( specData.getPowerCurveTDSeconds( ), refresh );
                }
            }

            if ( displayChannel.compareTo( TDFDFittingDefaults.PHASE ) == 0 )
            {
                if ( displayType.compareTo( TDFDFittingDefaults.TIME_DOMAIN_POINTS ) == 0 )
                {
                    addCurve( specData.getPhaseCurveTDPoints( ) , refresh );
                }
                if ( displayType.compareTo( TDFDFittingDefaults.TIME_DOMAIN_SECONDS ) == 0 )
                {
                    addCurve( specData.getPhaseCurveTDSeconds( ), refresh );
                }
            }
            
        }
        
        // After a response has been added the m_firstTimeChanged must be set
        // to true in order to be able to update the m_responseListBuffer correctly
        m_firstTimeChanged = true;
    }
        
    /**
     * Called when the mouse button is clicked.
     */
    @Override
    public void mouseClicked( MouseEvent mouseEvent ) 
    {
        super.mouseClicked( mouseEvent );
    }
    
    /**
     * Called when the mouse is dragged.
     */
    @Override
    public void mouseDragged( MouseEvent mouseEvent )
    {
        super.mouseDragged( mouseEvent );
    }
    
    /**
     * Called when the mouse enters the control.
     */    
    @Override
    public void mouseEntered( MouseEvent mouseEvent )
    {
        super.mouseEntered( mouseEvent );
    }
    
    /**
     * Called when the mouse exits.
     */    
    @Override
    public void mouseExited( MouseEvent mouseEvent )
    {
        super.mouseExited( mouseEvent );
    }
    
    /**
     * Called when the mouse button is released.
     */    
    @Override
    public void mouseMoved( MouseEvent mouseEvent )
    {
        super.mouseMoved( mouseEvent );
    }
    
    /**
     * Called when the mouse button is released.
     */    
    @Override
    public void mousePressed( MouseEvent mouseEvent )
    { 
        
        super.mousePressed( mouseEvent );
        
        // check whether the GUI is in the mode where mouse clicks are awaited 
        if ( m_baseLineDefinitionModeActive == true )
        {
            // is only of interest if left button is pressed
            if ( ( mouseEvent.getModifiers( ) & InputEvent.BUTTON1_MASK ) == InputEvent.BUTTON1_MASK )
            {
                int mouseButtonPressedX = mouseEvent.getX( );
                int mouseButtonPressedY = mouseEvent.getY( );
           
                determineValuesAtCursorPosition( new Point( mouseButtonPressedX, mouseButtonPressedY ) );            
                setCursor( Cursor.getPredefinedCursor(  Cursor.DEFAULT_CURSOR ) );
                notifyActionListeners( BASELINE_POINT_SELECTED_EVENT, "add user indicated line" );
            }
        }
        else
        {
            super.mousePressed( mouseEvent );
            if ( mouseEvent.isPopupTrigger( ) )
            {
                m_popupmenu.show( mouseEvent.getComponent( ), 
                                  mouseEvent.getX( ), mouseEvent.getY( ) );
            }
        }
    }
    
    /**
     * Called when the mouse button is released.
     */
    @Override
    public void mouseReleased( MouseEvent mouseEvent )
    {
         super.mouseReleased( mouseEvent );
         
        if ( mouseEvent.isPopupTrigger( ) )
        {
            m_popupmenu.show( mouseEvent.getComponent( ), 
                              mouseEvent.getX( ), mouseEvent.getY( ) );
        }
    }
    
    /**
     * Changes the displayed signal based on the state members that
     * indicate channel type and domain type.
     */
    protected void changeDisplayedSignal( )
    {
        
        // delete all curves from the vector of curves to display
        deleteAllCurves( );
        
        TDFDFittingDefaults fitdef = TDFDFittingDefaults.getInstance( );
        fitdef.load( );
        
        String displayDomain = fitdef.getProperty( TDFDFittingDefaults.DISPLAY_DOMAIN );
        String displayChannel =  fitdef.getProperty( TDFDFittingDefaults.DISPLAY_CHANNEL );
        String displayUnits = fitdef.getProperty( TDFDFittingDefaults.DISPLAY_UNIT );
        
        if ( displayDomain.compareTo( TDFDFittingDefaults.FREQ_DOMAIN ) == 0 )
        {
            if ( displayChannel.compareTo( TDFDFittingDefaults.ABSORPTION ) == 0 )
            {
                Iterator it = this.m_responseList.iterator( );
                while ( it.hasNext( ) )
                { 
                    SingleVoxelSpectralData specData = (SingleVoxelSpectralData)it.next( );
                    
                    // Change the x-axis
                    specData.autoSetXaxis( );
                    
                    if ( displayUnits.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_POINTS ) == 0 )
                    {
                        addCurve( specData.getAbsorptionCurveFDPoints( ), false );
                    }
                    if ( displayUnits.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_HZ ) == 0 )
                    {
                        addCurve( specData.getAbsorptionCurveFDHertz( ), false );
                    }
                    if ( displayUnits.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
                    {
                        addCurve( specData.getAbsorptionCurveFDPPM( ), false );
                    }
                }
            }
            
            if ( displayChannel.compareTo( TDFDFittingDefaults.DISPERSION ) == 0 )
            {
                Iterator it = this.m_responseList.iterator( );
                while ( it.hasNext( ) )
                {
                    SingleVoxelSpectralData specData = (SingleVoxelSpectralData)it.next( );
                    
                    // change the x-axis
                    specData.autoSetXaxis( );
                    
                    if ( displayUnits.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_POINTS ) == 0 )
                    {
                        addCurve( specData.getDispersionCurveFDPoints( ), false );
                    }
                    
                    if ( displayUnits.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_HZ ) == 0 )
                    {
                        addCurve( specData.getDispersionCurveFDHertz( ), false );
                    }
                    
                    if ( displayUnits.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
                    {
                        addCurve( specData.getDispersionCurveFDPPM( ), false );
                    }
                }
            }
            
            if ( displayChannel.compareTo( TDFDFittingDefaults.MAGNITUDE ) == 0)
            {
               Iterator it = this.m_responseList.iterator( );
                while ( it.hasNext( ) )
                {
                    SingleVoxelSpectralData specData = (SingleVoxelSpectralData)it.next( );
                    
                    // change the x-axis
                    specData.autoSetXaxis( );
                    
                    if ( displayUnits.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_POINTS ) == 0 )
                    { 
                        addCurve( specData.getMagnitudeCurveFDPoints( ), false );
                    }
                    
                    if ( displayUnits.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_HZ ) == 0 )
                    { 
                        addCurve( specData.getMagnitudeCurveFDHertz( ), false );
                    }
                    
                    if ( displayUnits.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
                    { 
                        addCurve( specData.getMagnitudeCurveFDPPM( ), false );
                    }
                } 
            }
            
            if ( displayChannel.compareTo( TDFDFittingDefaults.POWER ) == 0 )
            {
                Iterator it = this.m_responseList.iterator( );
                while ( it.hasNext( ) )
                {
                    SingleVoxelSpectralData specData = (SingleVoxelSpectralData)it.next( );
                    
                    // change the x-axis
                    specData.autoSetXaxis( );
                    
                    if ( displayUnits.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_POINTS ) == 0 )
                    { 
                        addCurve( specData.getPowerCurveFDPoints( ), false );
                    }
                    
                    if ( displayUnits.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_HZ ) == 0 )
                    { 
                        addCurve( specData.getPowerCurveFDHertz( ), false );
                    }
                    
                    if ( displayUnits.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
                    { 
                        addCurve( specData.getPowerCurveFDPPM( ), false );
                    }
                }
            }

            if ( displayChannel.compareTo( TDFDFittingDefaults.PHASE ) == 0 )
            {
                Iterator it = this.m_responseList.iterator( );
                while ( it.hasNext( ) )
                {
                    SingleVoxelSpectralData specData = (SingleVoxelSpectralData)it.next( );
                    // change the x-axis
                    specData.autoSetXaxis( );
                    
                    if (  displayUnits.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_POINTS ) == 0 )
                    {
                        addCurve( specData.getPhaseCurveFDPoints( ), false );
                    }
                    
                    if (  displayUnits.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_HZ ) == 0)
                    {
                        addCurve( specData.getPhaseCurveFDHertz( ), false );
                    }

                    if ( displayUnits.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
                    {
                        addCurve( specData.getPhaseCurveFDPPM( ), false );
                    }
                }
            }
        }
        
        if ( displayDomain.compareTo( TDFDFittingDefaults.TIME_DOMAIN ) == 0 )
        {
            if ( displayChannel.compareTo( TDFDFittingDefaults.ABSORPTION ) == 0 )
            {
                Iterator it = this.m_responseList.iterator( );
                while ( it.hasNext( ) )
                {
                    
                    SingleVoxelSpectralData specData = (SingleVoxelSpectralData)it.next( );
                    // change the x-axis
                    specData.autoSetXaxis( );
                    
                    if ( displayUnits.compareTo( TDFDFittingDefaults.TIME_DOMAIN_POINTS ) == 0)
                    {
                        addCurve( specData.getAbsorptionCurveTDPoints( ), false );
                    }
                    
                    if ( displayUnits.compareTo( TDFDFittingDefaults.TIME_DOMAIN_SECONDS ) == 0)
                    {
                        addCurve( specData.getAbsorptionCurveTDPoints( ), false );
                    }
                    
                }
            }
            
            if ( displayChannel.compareTo( TDFDFittingDefaults.DISPERSION ) == 0 )
            {
                Iterator it = this.m_responseList.iterator( );
                while ( it.hasNext( ) )
                {
                    
                    SingleVoxelSpectralData specData = (SingleVoxelSpectralData)it.next( );
                    // change the x-axis
                    specData.autoSetXaxis( );
                    
                    if ( displayUnits.compareTo( TDFDFittingDefaults.TIME_DOMAIN_POINTS ) == 0 )
                    {
                        addCurve( specData.getDispersionCurveTDPoints( ), false );
                    }
                    
                    if ( displayUnits.compareTo( TDFDFittingDefaults.TIME_DOMAIN_SECONDS ) == 0 )
                    {
                        addCurve( specData.getDispersionCurveTDSeconds( ), false );
                    }
                }
            }
            
            if ( displayChannel.compareTo( TDFDFittingDefaults.MAGNITUDE ) == 0 )
            {
                Iterator it = this.m_responseList.iterator( );
                while ( it.hasNext( ) )
                {
                    SingleVoxelSpectralData specData = (SingleVoxelSpectralData)it.next( );
                    
                    // change the x-axis
                    specData.autoSetXaxis( );
                    
                    if ( displayUnits.compareTo( TDFDFittingDefaults.TIME_DOMAIN_POINTS ) == 0 )
                    {
                        addCurve( specData.getMagnitudeCurveTDPoints( ), false );
                    }

                    if ( displayUnits.compareTo( TDFDFittingDefaults.TIME_DOMAIN_SECONDS ) == 0)
                    {
                        addCurve( specData.getMagnitudeCurveTDSeconds( ), false );
                    }
                }
            }
            
            if ( displayChannel.compareTo( TDFDFittingDefaults.POWER ) == 0 )
            {
                Iterator it = this.m_responseList.iterator( );
                while ( it.hasNext( ) )
                {
                    
                    SingleVoxelSpectralData specData = (SingleVoxelSpectralData)it.next( );
                    
                    // change the x-axis
                    specData.autoSetXaxis( );
                    
                    if ( displayUnits.compareTo( TDFDFittingDefaults.TIME_DOMAIN_POINTS ) == 0 )
                    {
                        addCurve( specData.getPowerCurveTDPoints( ), false );
                    }
                    
                    if ( displayUnits.compareTo( TDFDFittingDefaults.TIME_DOMAIN_SECONDS ) == 0 )
                    {
                        addCurve( specData.getPowerCurveTDSeconds( ), false );
                    }
                }
            }

            if ( displayChannel.compareTo( TDFDFittingDefaults.PHASE ) == 0 )
            {
                Iterator it = this.m_responseList.iterator( );
                while ( it.hasNext( ) )
                { 
                    SingleVoxelSpectralData specData = (SingleVoxelSpectralData)it.next( );
                    // change the x-axis
                    specData.autoSetXaxis( );
                    
                    if ( displayUnits.compareTo( TDFDFittingDefaults.TIME_DOMAIN_SECONDS ) == 0 )
                    {
                        addCurve( specData.getPhaseCurveTDPoints( ), false );
                    }
                    
                    if ( displayUnits.compareTo( TDFDFittingDefaults.TIME_DOMAIN_SECONDS ) == 0 )
                    {
                        addCurve( specData.getPhaseCurveTDSeconds( ), false );
                    }
                }
            }
        }
        
        // directly call refresh in order to minimize flickering
        autoDeterminePlotRanges( );  
        refresh( );
    }
    
    
    /**
     * All registered objects are notified that an event of type
     * actionEventType happened.
     */
    private void notifyActionListeners( int actionEventType, String actionCommand )
    {
        Iterator it = m_listeners.iterator( );
        while( it.hasNext( ) )
        {
            ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, actionEventType, actionCommand ) );
        }
    }
    
    
    /**
     * Performs 1000 simulations.
     */
    public void perform1000simulations( )
    {
     
        double[][] sig = new double[m_measuredPatientSpectrumSeries.getNumberOfInstances( )][2048];
        double[][] filteredSig = new double[m_measuredPatientSpectrumSeries.getNumberOfInstances( )][2048];
        
        float[] avIntegral = new float[m_measuredPatientSpectrumSeries.getNumberOfInstances( )];
        float[] xaxis = new float[m_measuredPatientSpectrumSeries.getNumberOfInstances( )];
        
        //XYPlotFrame plotDuring = new XYPlotFrame( "Signals during", null );
        
        for ( int cnt = 0 ; cnt < 1000 ; cnt++ )
        {
            System.out.println( "cnt = " + cnt  );
            
            sig = createSimulatedSpectraNew( );
            
            OrderStatisticsSignalFilter osf = new OrderStatisticsSignalFilter( m_measuredPatientSpectrumSeries.getNumberOfInstances( ), 2048, sig );
            
            filteredSig = osf.performFiltering( true, true, 1800, 2047 );
            
            for ( int i = 0 ; i < m_measuredPatientSpectrumSeries.getNumberOfInstances( ) ; i++ )
            {
                float val=0;
                for ( int j = 1000; j < 1048; j++ )
                {
                    val += filteredSig[i][j];
                }
                
                avIntegral[i]+=val*val;
                xaxis[i] = i;
            }
             //plotDuring.addCurve( new Curve( xaxis, avIntegral, Color.BLACK ), true );   
        }
            
        XYPlotFrame plotMean = new XYPlotFrame( "Raw signals Bias", null );
        plotMean.addCurve( new Curve( xaxis, avIntegral, Color.BLACK ), true );
    }
    
    
    /**
     * Create Simulated Spectra.
     */
    private void createSimulatedSpectra( )
    {
        
        Random rnd = new Random( );
        
        SingleVoxelSpectralData data = m_measuredPatientSpectrumSeries.getCurrentInstance(1);
        m_measuredPatientSpectrumSeries.clear( );
        
        m_responseList.clear( );
        m_responseNameList.clear( );
        m_responseListBuffer.clear( );
        
        float[][] twoDsig = new float[m_measuredPatientSpectrumSeries.getNumberOfInstances( )][2048];
        
        for ( int k = 0; k < m_measuredPatientSpectrumSeries.getNumberOfInstances( ); k++ )
        {
            float[] abs = new float[2048];
            float[] disp = new float[2048];
            for ( int i = 0; i < 2048; i++ )
            {
                abs[i] = (float)( 1000.0 * Math.cos( (2.0 * Math.PI * 00.0 * i ) / 1024. ) * Math.exp( -i/5.0 ) ); /* + 200.0 * Math.cos( 2.0 * Math.PI * 250.0 * i / 1024 ) * Math.exp( -i / 64.0 ) + 200.0 * Math.cos( 2.0 * Math.PI * 257.0 * i / 1024 ) * Math.exp( -i / 64.0 ) ); */
                abs[i] += (100*(float)rnd.nextGaussian( ) );
                disp[i] = (float)( 1000.0 * Math.sin( (2.0 * Math.PI * 00.0 * i ) / 1024. ) * Math.exp( -i/5.0 ) ); /* + 200.0 * Math.sin( 2.0 * Math.PI * 250.0 * i / 1024 ) * Math.exp( -i / 64.0 ) + 200.0 * Math.sin( 2.0 * Math.PI * 257.0 * i / 1024 ) * Math.exp( -i / 64.0 ) ); */
                disp[i] += (100*(float)rnd.nextGaussian( ) );
            }
            
            abs[0] = abs[0]/2;
            disp[0] = disp[0]/2;
            m_measuredPatientSpectrumSeries.setNumberOfSamplePoints( 2048 );
            m_measuredPatientSpectrumSeries.setSpectralDataPointsTD( k, abs, disp );
            
            SingleVoxelMeasuredSpectralData sVoxData = new SingleVoxelMeasuredSpectralData( 
                                                                            abs,
                                                                            disp,
                                                                            SingleVoxelSpectralData.TIME_DOMAIN );
            sVoxData.setDicomHeaderInfo( data.getDicomHeaderInfo( ) );
            sVoxData.setSiemensSpecificDicomHeaderInfo( data.getSiemensSpecificDicomHeaderInfo( ) );
            
            
            m_responseList.add( sVoxData );
            
            SingleVoxelMeasuredSpectralData sVoxData2 = 
                    new SingleVoxelMeasuredSpectralData( abs,
                                                         disp,
                                                         SingleVoxelSpectralData.TIME_DOMAIN );
            sVoxData2.setDicomHeaderInfo( data.getDicomHeaderInfo( ) );
            sVoxData2.setSiemensSpecificDicomHeaderInfo( data.getSiemensSpecificDicomHeaderInfo( ) );
            
            for ( int i=0 ; i < 2048; i++ )
            {
                twoDsig[k][i]= sVoxData2.getAbsorptionChannelFD()[i];
            }
            
            m_responseListBuffer.add( sVoxData2 );
        }
         
        m_firstTimeChanged = false;
        changeDisplayedSignal( );
        notifyMeasuredSpectrumChanged( this, -999 );
    }
    
    /**
     * Create Simulated Spectra.
     */
    private double[][] createSimulatedSpectraNew( )
    {
        
        Random rnd = new Random( );
        
        SingleVoxelSpectralData data = m_measuredPatientSpectrumSeries.getCurrentInstance(1);
        m_measuredPatientSpectrumSeries.clear( );
        
        m_responseList.clear( );
        m_responseNameList.clear( );
        m_responseListBuffer.clear( );
        
        double[][] twoDsig = new double[m_measuredPatientSpectrumSeries.getNumberOfInstances( )][2048];
        
        for ( int k = 0; k < m_measuredPatientSpectrumSeries.getNumberOfInstances( ); k++ )
        {
            float[] abs = new float[2048];
            float[] disp = new float[2048];
            for ( int i = 0; i < 2048; i++ )
            {
                abs[i] = (float)( 1000.0 * Math.cos( (2.0 * Math.PI * 00.0 * i ) / 1024. ) * Math.exp( -i/5.0 ) ); /* + 200.0 * Math.cos( 2.0 * Math.PI * 250.0 * i / 1024 ) * Math.exp( -i / 64.0 ) + 200.0 * Math.cos( 2.0 * Math.PI * 257.0 * i / 1024 ) * Math.exp( -i / 64.0 ) ); */
                abs[i] += (100*(float)rnd.nextGaussian( ) );
                disp[i] = (float)( 1000.0 * Math.sin( (2.0 * Math.PI * 00.0 * i ) / 1024. ) * Math.exp( -i/5.0 ) ); /* + 200.0 * Math.sin( 2.0 * Math.PI * 250.0 * i / 1024 ) * Math.exp( -i / 64.0 ) + 200.0 * Math.sin( 2.0 * Math.PI * 257.0 * i / 1024 ) * Math.exp( -i / 64.0 ) ); */
                disp[i] += (100*(float)rnd.nextGaussian( ) );
            }
            
            abs[0] = abs[0]/2;
            disp[0] = disp[0]/2;
            m_measuredPatientSpectrumSeries.setNumberOfSamplePoints( 2048 );
            m_measuredPatientSpectrumSeries.setSpectralDataPointsTD( k, abs, disp );
            
            
            SingleVoxelMeasuredSpectralData sVoxData = new SingleVoxelMeasuredSpectralData( abs,
                                                                            disp,
                                                                            SingleVoxelSpectralData.TIME_DOMAIN );
            sVoxData.setDicomHeaderInfo( data.getDicomHeaderInfo( ) );
            sVoxData.setSiemensSpecificDicomHeaderInfo( data.getSiemensSpecificDicomHeaderInfo( ) );
            
            
            m_responseList.add( sVoxData );
            
            SingleVoxelMeasuredSpectralData sVoxData2 = new SingleVoxelMeasuredSpectralData( abs,
                                                                             disp,
                                                                             SingleVoxelSpectralData.TIME_DOMAIN );
            sVoxData2.setDicomHeaderInfo( data.getDicomHeaderInfo( ) );
            sVoxData2.setSiemensSpecificDicomHeaderInfo( data.getSiemensSpecificDicomHeaderInfo( ) );
            
            for ( int i = 0 ; i < 2048; i++ )
            {
                twoDsig[k][i]= (double)sVoxData2.getAbsorptionChannelFD()[i];
            }
            
            m_responseListBuffer.add( sVoxData2 );
        }
         
        return twoDsig;
    }
    
    /**
     * Called whenever a actionEvent occurs.
     */
    @Override
    public void actionPerformed( ActionEvent e )
    {
        // call base class
        super.actionPerformed( e );
        
        Object sender = e.getSource( );
        
        if ( sender == m_svsDataClipboard )
        {
            if ( e.getID( ) ==  m_svsDataClipboard.ADDED_SINGLEVOXELSPECTRALDATA_TO_CLIPBOARD )
            {
                m_pasteSingleVoxelSpectrumDataMenuItem.setEnabled( true );
            }
        }
        
        if ( sender == m_copySingleVoxelSpectrumDataMenuItem )
        {
            m_svsDataClipboard.setSingleVoxelSpectralDataList( m_responseList );
            m_pasteSingleVoxelSpectrumDataMenuItem.setEnabled( true );
        }
        
        if ( sender == m_pasteSingleVoxelSpectrumDataMenuItem )
        {
            LinkedList<SingleVoxelMeasuredSpectralData> list = m_svsDataClipboard.getSingleVoxelSpectralDataList( );
            
            SingleVoxelSpectrumViewerFrame frame = new SingleVoxelSpectrumViewerFrame( list, m_dicomDir, null );
            
            m_pasteSingleVoxelSpectrumDataMenuItem.setEnabled( true );
            
        }
        
        if ( sender == m_performOSFFiltering )
        {
            SingleVoxelSpectralData specData = m_responseList.getFirst( );
            int arrayLength = specData.getAbsorptionChannelTD().length;
            
            m_controlOSF = new OrderStatisticsFilterDefinitionControl( "Perform Order Statistics Filtering", m_parent, 
                                                                       0, arrayLength, 
                                                                       3 * arrayLength / 4, arrayLength, false );
                                                        
            m_controlOSF.addActionListener( this );
        }
        
        if ( sender == this.m_performARSOSDenoising )
        {
            SingleVoxelSpectralData specData = m_responseList.getFirst( );
            int arrayLength = specData.getAbsorptionChannelTD( ).length;
            
            m_controlOSF = new OrderStatisticsFilterDefinitionControl( "Perform Order Statistics Filtering", m_parent, 
                                                                       0, arrayLength, 
                                                                       3 * arrayLength / 4, arrayLength, true );
                                                        
            m_controlOSF.addActionListener( this );
        }
        
        if ( sender == m_performOSFFittingPlainARSOS )
        {
            this.perform1000simulations( );
            createSimulatedSpectra( );
        }
        
        if ( sender == m_performOSFFittingCombinedARSOS )
        {
            createSimulatedSpectra( );
        }
        
        if ( sender == m_controlOSF )
        {
            if ( e.getID( ) == OrderStatisticsFilterDefinitionControl.PREVIEW_OSF_EVENT )
            {
                
                System.out.println( "preview event!!" );
                
                m_originalDataSeriesAbs = new float[m_measuredPatientSpectrumSeries.getNumberOfInstances( )][m_measuredPatientSpectrumSeries.getAbsorptionChannelTD( 0 ).length];
                m_originalDataSeriesDisp = new float[m_measuredPatientSpectrumSeries.getNumberOfInstances( )][m_measuredPatientSpectrumSeries.getAbsorptionChannelTD( 0 ).length];
                
                for ( int i = 0; i < m_measuredPatientSpectrumSeries.getNumberOfInstances( ); i++ )
                {
                    float[] abs = m_measuredPatientSpectrumSeries.getAbsorptionChannelTD( i );
                    float[] disp = m_measuredPatientSpectrumSeries.getDispersionChannelTD( i );
                    for ( int j = 0; j < abs.length; j++ )
                    {
                        m_originalDataSeriesAbs[i][j] = abs[j];
                        m_originalDataSeriesDisp[i][j] = disp[j];
                    }
                }
                
                if ( m_firstTimeChanged == true )
                {  
                    m_responseList.clear( );
                    m_responseNameList.clear( );
                    for ( int i = 0; i < m_measuredPatientSpectrumSeries.getNumberOfInstances( ); i++ )
                    {
                        SingleVoxelMeasuredSpectralData sVoxData = 
                                new SingleVoxelMeasuredSpectralData( m_measuredPatientSpectrumSeries.getAbsorptionChannelTD( i ),
                                                                     m_measuredPatientSpectrumSeries.getDispersionChannelTD( i ),
                                                                     SingleVoxelSpectralData.TIME_DOMAIN );
                    
                        sVoxData.setDicomHeaderInfo( sVoxData.getDicomHeaderInfo( ) );
                        sVoxData.setSiemensSpecificDicomHeaderInfo( sVoxData.getSiemensSpecificDicomHeaderInfo( ) );
                        
                        m_responseList.add( sVoxData );
                    }
                
                    m_responseListBuffer.clear( );
                
                    for ( int i = 0; i < m_measuredPatientSpectrumSeries.getNumberOfInstances( ); i++ )
                    {
                        SingleVoxelMeasuredSpectralData sVoxData = 
                                new SingleVoxelMeasuredSpectralData( m_measuredPatientSpectrumSeries.getAbsorptionChannelTD( i ),
                                                                     m_measuredPatientSpectrumSeries.getDispersionChannelTD( i ),
                                                                     SingleVoxelSpectralData.TIME_DOMAIN );
                    
                        sVoxData.setDicomHeaderInfo( sVoxData.getDicomHeaderInfo( ) );
                        sVoxData.setSiemensSpecificDicomHeaderInfo( sVoxData.getSiemensSpecificDicomHeaderInfo( ) );
                        
                        m_responseListBuffer.add( sVoxData );
                    }     
               
                    m_firstTimeChanged = false;
                }
                
                // All data of the series must be changed accordingly
                // Iterate over all responses of the SingleVoxelSpectralDataSeries
                m_sdp = new SingleVoxelSpectralDataProcessing( m_responseList );
                
                LinkedList<SingleVoxelMeasuredSpectralData> linkedList = new LinkedList<SingleVoxelMeasuredSpectralData>( );
                
                for ( int i = 0; i < m_measuredPatientSpectrumSeries.getNumberOfInstances( ); i++ )
                {
                    SingleVoxelMeasuredSpectralData sVoxData = 
                            new SingleVoxelMeasuredSpectralData( m_measuredPatientSpectrumSeries.getAbsorptionChannelTD( i ),
                                                                 m_measuredPatientSpectrumSeries.getDispersionChannelTD( i ),
                                                                 SingleVoxelSpectralData.TIME_DOMAIN);
                    
                    linkedList.add( sVoxData );
                }
                
                m_sdp.setResponseList( linkedList );
                
                if ( m_controlOSF.getDenoise2DResponse( ) == false )
                {
                    // Perform the OSF to all responsers of the series
                    linkedList = m_sdp.applyOrderStatisticsFilter( m_controlOSF.getApplicationDomain( ) , 
                                                                   m_controlOSF.removeBaseline( ),
                                                                   true, m_controlOSF.getMinimumRangeValue( ), 
                                                                   m_controlOSF.getMaximumRangeValue( ) );
                }
                else
                {
                    // Perform the OSF to all responsers of the series
                    linkedList = m_sdp.applyOrderStatisticsDenoising( m_controlOSF.getApplicationDomain( ) , 
                                                                      m_controlOSF.removeBaseline( ),
                                                                      true, m_controlOSF.getMinimumRangeValue( ), 
                                                                      m_controlOSF.getMaximumRangeValue( ) );
                }
                
                Iterator it1 = linkedList.iterator( );
                int cntr = 0;
                
                while ( it1.hasNext( ) )
                {
                    SingleVoxelSpectralData sVoxData = (SingleVoxelSpectralData)it1.next( );
                    this.m_measuredPatientSpectrumSeries.setSpectralDataPointsTD( cntr, sVoxData.getAbsorptionChannelTD( ), sVoxData.getDispersionChannelTD( ) );
                    
                    cntr++;
                }
                
                // Redefine the attribute containing the sum spectrum
                m_measuredPatientSumSpectrum = m_measuredPatientSpectrumSeries.getSumSpectrum( );
                
                changeDisplayedSignal( );
                
                notifyMeasuredSpectrumChanged( this, -999 );
                
            } // end of preview event
            
            if ( e.getID( ) == OrderStatisticsFilterDefinitionControl.CANCEL_EVENT )
            {
                // Restore old values   
                m_responseList.clear( );
                m_responseNameList.clear( );
                m_responseListBuffer.clear( );
                m_firstTimeChanged = true;
                
                for ( int i = 0 ; i < m_measuredPatientSpectrumSeries.getNumberOfInstances(); i++ )
                {
                    
                    float[] abs = new float[ m_measuredPatientSpectrumSeries.getAbsorptionChannelTD( 0 ).length ];
                    float[] disp = new float[ m_measuredPatientSpectrumSeries.getAbsorptionChannelTD( 0 ).length ];
                    for ( int j=0;j<abs.length;j++)
                    {
                        abs[j] = m_originalDataSeriesAbs[i][j];
                        disp[j] = m_originalDataSeriesDisp[i][j];
                    }
                    
                    m_measuredPatientSpectrumSeries.setSpectralDataPointsTD( i, abs, disp );
                }
                                
                // Redefine the attribute containing the sum spectrum
                m_measuredPatientSumSpectrum = m_measuredPatientSpectrumSeries.getSumSpectrum( );

                changeDisplayedSignal( );
                notifyMeasuredSpectrumChanged( this, -999 );
            }

            if ( e.getID( ) == OrderStatisticsFilterDefinitionControl.ACCEPT_EVENT )
            {
                m_responseList.clear( );
                m_responseNameList.clear( );
                m_responseListBuffer.clear( );
                m_firstTimeChanged = true;
                 
                changeDisplayedSignal( );
                notifyMeasuredSpectrumChanged( this, -999 );
            }
        }
        
        // Handles response user selects time domain representation.
        if ( sender == m_timeDomain )
        {
            TDFDFittingDefaults fitdef = TDFDFittingDefaults.getInstance( );
            fitdef.load( );
            
            fitdef.setProperty( TDFDFittingDefaults.DISPLAY_DOMAIN, TDFDFittingDefaults.TIME_DOMAIN );
            fitdef.setProperty( TDFDFittingDefaults.DISPLAY_UNIT, TDFDFittingDefaults.TIME_DOMAIN_SECONDS );
            fitdef.save( );
            
            m_displayXReversed = false;
            
            // Save the current range values
            if ( m_xyForcedPlotRange != null )
            {
                m_xyForcedPlotRangeFD = m_xyForcedPlotRange;
            }

            // Get the old forced plotting range
            if ( m_xyForcedPlotRangeTD != null )
            {
                m_xyForcedPlotRange = m_xyForcedPlotRangeTD;
            }
            else
            {
                autoDeterminePlotRanges( );
                m_xyForcedPlotRange = this.m_xyAutoPlotRange;
            }
            m_responseListBuffer.clear( );
            changeDisplayedSignal( );            
        }
       
        // Handles response user selects frequency domain representation
        if ( sender == m_frequencyDomain )
        {
            
            TDFDFittingDefaults fitdef = TDFDFittingDefaults.getInstance( );
            fitdef.load( );
            
            fitdef.setProperty( TDFDFittingDefaults.DISPLAY_DOMAIN, TDFDFittingDefaults.FREQ_DOMAIN );
            fitdef.setProperty( TDFDFittingDefaults.DISPLAY_UNIT, TDFDFittingDefaults.FREQUENCY_DOMAIN_HZ );
            fitdef.save( );
            
            // save the current range values
            if ( m_xyForcedPlotRange != null )
            {
                m_xyForcedPlotRangeTD = m_xyForcedPlotRange;
            }
            
            // get the old forced plotting range
            if ( m_xyForcedPlotRangeFD != null )
            {
                m_xyForcedPlotRange = m_xyForcedPlotRangeFD;
            }
            else
            {
                autoDeterminePlotRanges( );
                m_xyForcedPlotRange = this.m_xyAutoPlotRange;
            }
            m_responseListBuffer.clear( );
            changeDisplayedSignal( );
        }
        
        // Handles response user selects display of absorption channel
        if ( sender == m_absorption )
        {
            TDFDFittingDefaults fitdef = TDFDFittingDefaults.getInstance( );
            fitdef.load( );
            fitdef.setProperty( TDFDFittingDefaults.DISPLAY_CHANNEL, TDFDFittingDefaults.ABSORPTION );
            fitdef.save( );
            m_responseListBuffer.clear( );
            changeDisplayedSignal( );
        }

        // Handles response user selects display of absorption channel
        if ( sender == m_dispersion )
        {
            TDFDFittingDefaults fitdef = TDFDFittingDefaults.getInstance( );
            fitdef.load( );
            fitdef.setProperty( TDFDFittingDefaults.DISPLAY_CHANNEL, TDFDFittingDefaults.DISPERSION );
            fitdef.save( );
            m_responseListBuffer.clear( );
            changeDisplayedSignal( );            
        }

        // Handles response user selects display of the magnitude representation
        if ( sender == m_magnitude )
        {
            TDFDFittingDefaults fitdef = TDFDFittingDefaults.getInstance( );
            fitdef.load( );
            fitdef.setProperty( TDFDFittingDefaults.DISPLAY_CHANNEL, TDFDFittingDefaults.MAGNITUDE );
            fitdef.save( );
            m_responseListBuffer.clear( );
            changeDisplayedSignal( );
        }
        
        // Handles response user selects display of the power representation
        if ( sender == m_power )
        {
            TDFDFittingDefaults fitdef = TDFDFittingDefaults.getInstance( );
            fitdef.load( );
            fitdef.setProperty( TDFDFittingDefaults.DISPLAY_CHANNEL, TDFDFittingDefaults.POWER );
            fitdef.save( );
            m_responseListBuffer.clear( );
            changeDisplayedSignal( );            
        }
        
        // Handles response user selects display of the phase representation
        if ( sender == m_phase )
        {
            TDFDFittingDefaults fitdef = TDFDFittingDefaults.getInstance( );
            fitdef.load( );
            fitdef.setProperty( TDFDFittingDefaults.DISPLAY_CHANNEL, TDFDFittingDefaults.PHASE );
            fitdef.save( );
            m_responseListBuffer.clear( );
            changeDisplayedSignal( );
        }
 
        if ( sender == m_setApodizationParameters )
        {
            copyResponseList( );
            m_apodizationDefiner = new ApodizationParameterDefinitionControl( "Define Apodization Parameters", m_parent, ApodizationParameterDefinitionControl.MULTI_RESPONSE_DISPLAY );
            m_apodizationDefiner.addActionListener( this );
        }
        
        if ( sender ==  m_multiplyFirstTDPoint )
        {
            copyResponseList( );
            m_firstPointMultiplicationControl = new SingleVoxelFirstSamplePointMultiplicatorControl( "Set TD first time sample point multiplication factor", m_parent, SpectrumPhasingDefinitionControl.MULTI_RESPONSE_DISPLAY );
            m_firstPointMultiplicationControl.addActionListener( this );
        }
        
        if ( sender == m_setZeroFillingParameters )
        {
            copyResponseList( );
            m_zerofillDefiner = new ZeroFillParameterDefinitionControl( "Define Zerofilling factor", m_parent );
            m_zerofillDefiner.addActionListener( this );
        }
        
        if ( sender == m_setPhasingParameters )
        {
            copyResponseList( );
            m_phasingControl = new SpectrumPhasingDefinitionControl( "Zero and First Order Phasing", m_parent, SpectrumPhasingDefinitionControl.MULTI_RESPONSE_DISPLAY );
            m_phasingControl.addActionListener( this );
        }
        
        if ( sender == m_zerofillDefiner )
        {

            if ( e.getID( ) == m_zerofillDefiner.CANCEL_EVENT )
            {
               
                // Restore old values   
                m_responseList.clear( );
                m_responseNameList.clear( );
                
                if ( !m_responseListBuffer.isEmpty( ) )
                {
                    Iterator it = this.m_responseListBuffer.iterator( );
                    while ( it.hasNext( ) )
                    {
                        SingleVoxelMeasuredSpectralData specDataBuffer = (SingleVoxelMeasuredSpectralData)it.next( );
                        
                        SingleVoxelMeasuredSpectralData specData = new SingleVoxelMeasuredSpectralData( );
                        
                        specData.setSpectralDataPointsTD( specDataBuffer.getAbsorptionChannelTD( ), specDataBuffer.getDispersionChannelTD( ) );
                        specData.setSiemensSpecificDicomHeaderInfo( specDataBuffer.getSiemensSpecificDicomHeaderInfo( ) );
                        specData.setDicomHeaderInfo( specDataBuffer.getDicomHeaderInfo( ) );
                        
                        m_responseList.add( specData );
                    }
                }
                
                m_firstTimeChanged = true;
                changeDisplayedSignal( );
            }
            
            if ( e.getID( ) == m_zerofillDefiner.ACCEPT_EVENT )
            {   
                m_responseListBuffer.clear( );
                m_firstTimeChanged = true;
                m_responseList = m_sdp.getResponseList( );
                
                // sent notification that measured spectrum has changed
                int index = -1;
                int cntr = 0;
                Iterator it = m_responseNameList.iterator( );
                while ( it.hasNext( ) )
                {
                    String str = (String)it.next( );
                    if ( str.compareTo( "measured" ) == 0 )
                    {
                        index = cntr;
                    }
                    cntr++;
                }
                
                if ( index != -1 )
                {
                    notifyMeasuredSpectrumChanged( m_responseList.get( index ), 0 );
                }
                
                
                // All data of the series must be changed accordingly
                // Iterate over all responses of the SingleVoxelSpectralDataSeries
                
                
                float zeroFillFactor =  m_zerofillDefiner.getSelectedZerofillfactor( );
                
                m_measuredPatientSpectrumSeries.setSpectralDataPointsTD( zeroFillFactor );
                
                // Redefine the attribute containing the sum spectrum
                m_measuredPatientSumSpectrum = m_measuredPatientSpectrumSeries.getSumSpectrum( );
                
                changeDisplayedSignal( );
            }
            
            if ( e.getID( ) == m_zerofillDefiner.CHANGE_EVENT )
            {
                if ( m_firstTimeChanged == true )
                {
                    m_responseListBuffer.clear( );
                    
                    Iterator it = this.m_responseList.iterator( );
                    while ( it.hasNext( ) )
                    {
                        SingleVoxelMeasuredSpectralData specData = (SingleVoxelMeasuredSpectralData)it.next( );
                        SingleVoxelMeasuredSpectralData specDataBuffer = new SingleVoxelMeasuredSpectralData( specData );

                        m_responseListBuffer.add( specDataBuffer );
                    }
                    m_firstTimeChanged = false;
                }
                
                m_sdp = new SingleVoxelSpectralDataProcessing( m_responseListBuffer );
                m_responseList = m_sdp.zeroFill( m_zerofillDefiner.getSelectedZerofillfactor( ) );
                changeDisplayedSignal( );
            }
        }
        
        if ( sender == m_apodizationDefiner )
        {
            if ( e.getID( ) == ApodizationParameterDefinitionControl.CANCEL_EVENT )
            {
                // Restore old values
                m_responseList.clear( );
                m_responseNameList.clear( );

                if ( !m_responseListBuffer.isEmpty( ) )
                {
                    Iterator it = this.m_responseListBuffer.iterator( );
                    while ( it.hasNext( ) )
                    {    
                        SingleVoxelMeasuredSpectralData specDataBuffer = (SingleVoxelMeasuredSpectralData)it.next( );     
                        SingleVoxelMeasuredSpectralData specData = new SingleVoxelMeasuredSpectralData( specDataBuffer );
                        
                        m_responseList.add( specData );
                    }
                }
              
                m_firstTimeChanged = true;
                changeDisplayedSignal( );
            }
            
            if ( e.getID( ) == m_apodizationDefiner.ACCEPT_EVENT )
            {
                //m_responseListBuffer.clear( );
                m_firstTimeChanged = true;
                m_responseList = m_sdp.getResponseList( );
                
                int index = -1;
                int cntr = 0;
                Iterator it = m_responseNameList.iterator( );
                while ( it.hasNext( ) )
                {
                    String str = (String)it.next( );
                    if ( str.compareTo( "measured" ) == 0 )
                    {
                        index = cntr;
                    }
                    cntr++;
                }
                if ( index != -1 )
                {
                    notifyMeasuredSpectrumChanged( m_responseList.get( index ), 0 );
                }

                // All data of the series must be changed accordingly
                // Iterate over all responses of the SingleVoxelSpectralDataSeries
                
                LinkedList<SingleVoxelMeasuredSpectralData> linkedList = new LinkedList<SingleVoxelMeasuredSpectralData>( );
                
                for ( int i = 0; i < m_measuredPatientSpectrumSeries.getNumberOfInstances( ); i++ )
                {
                    SingleVoxelMeasuredSpectralData sVoxData = new SingleVoxelMeasuredSpectralData( m_measuredPatientSpectrumSeries.getAbsorptionChannelTD( i ),
                                                                                    m_measuredPatientSpectrumSeries.getDispersionChannelTD( i ),
                                                                                    SingleVoxelSpectralData.TIME_DOMAIN );
                    
                    sVoxData.setDicomHeaderInfo( m_measuredPatientSpectrumSeries.getDicomHeaderInfo( i ) );
                    sVoxData.setSiemensSpecificDicomHeaderInfo(m_measuredPatientSpectrumSeries.getSiemensSpecificDicomHeaderInfo( i ) );
                    linkedList.add( sVoxData );
                    
                }
                
                m_sdp.setResponseList( linkedList );

                // Perform the td apodization to all responses of the series
                linkedList = m_sdp.apodize( m_apodizationDefiner.getSelectedLorentzianTC( ), 
                                            m_apodizationDefiner.getSelectedGaussianTC( ) );
                
                
                Iterator it1 = linkedList.iterator( );
                cntr = 0;
                
                while ( it1.hasNext( ) )
                {
                    SingleVoxelSpectralData sVoxData = (SingleVoxelSpectralData)it1.next( );
                    this.m_measuredPatientSpectrumSeries.setSpectralDataPointsTD( cntr, sVoxData.getAbsorptionChannelTD(), sVoxData.getDispersionChannelTD( ) );
                    
                    cntr++;
                }

                // Redefine the attribute containing the sum spectrum
                m_measuredPatientSumSpectrum = m_measuredPatientSpectrumSeries.getSumSpectrum( );
                
                changeDisplayedSignal( );
            }
            
            if ( e.getID( ) == m_apodizationDefiner.GAUSSIAN_TIME_CONSTANT_CHANGE_EVENT  )
            {
                if ( m_firstTimeChanged == true )
                {
                    m_responseListBuffer.clear( );
                    
                    Iterator it = this.m_responseList.iterator( );
                    while ( it.hasNext( ) )
                    {
                        SingleVoxelMeasuredSpectralData specData = (SingleVoxelMeasuredSpectralData)it.next( );
                        SingleVoxelMeasuredSpectralData specDataBuffer = new SingleVoxelMeasuredSpectralData( specData );
                        
                        m_responseListBuffer.add( specDataBuffer );
                    }
                    m_firstTimeChanged = false;

                }
                
                m_sdp = new SingleVoxelSpectralDataProcessing( m_responseListBuffer );
                m_responseList = m_sdp.apodize( m_apodizationDefiner.getSelectedLorentzianTC( ), 
                                                m_apodizationDefiner.getSelectedGaussianTC( ) );
                changeDisplayedSignal( );                
                
            }
            
            if ( e.getID( ) == m_apodizationDefiner.LORENTZIAN_TIME_CONSTANT_CHANGED_EVENT  )
            {
                if ( m_firstTimeChanged == true )
                {
                    m_responseListBuffer.clear( );
                    
                    Iterator it = this.m_responseList.iterator( );
                    while ( it.hasNext( ) )
                    {
                        SingleVoxelMeasuredSpectralData specData = (SingleVoxelMeasuredSpectralData)it.next( );
                        SingleVoxelMeasuredSpectralData specDataBuffer = new SingleVoxelMeasuredSpectralData( specData );
                        
                        m_responseListBuffer.add( specDataBuffer );
                    }
                    m_firstTimeChanged = false;
                }
                
                m_sdp = new SingleVoxelSpectralDataProcessing( m_responseListBuffer );
                m_responseList = m_sdp.apodize( m_apodizationDefiner.getSelectedLorentzianTC( ), 
                                                m_apodizationDefiner.getSelectedGaussianTC( ) );
                changeDisplayedSignal( );
            }
        }
        
        if ( sender == m_phasingControl )
        {
            if ( e.getID( ) == m_phasingControl.ZERO_ORDER_PHASING_EVENT )
            {
                if ( m_firstTimeChanged == true )
                {

                    m_responseListBuffer.clear( );
                    
                    Iterator it = this.m_responseList.iterator( );
                    while ( it.hasNext( ) )
                    {
                        SingleVoxelMeasuredSpectralData specData = (SingleVoxelMeasuredSpectralData)it.next( );
                        SingleVoxelMeasuredSpectralData specDataBuffer = new SingleVoxelMeasuredSpectralData( specData );
                                                
                        m_responseListBuffer.add( specDataBuffer );
                    }
                    m_firstTimeChanged = false;
                }
                
                m_sdp = new SingleVoxelSpectralDataProcessing( m_responseListBuffer );
                m_responseList = m_sdp.adjustZeroOrderPhase( m_phasingControl.getZeroOrderPhase( ) );
                changeDisplayedSignal( );
                
            }
            
            if ( e.getID( ) == m_phasingControl.FIRST_ORDER_PHASING_EVENT )
            {
                if ( m_firstTimeChanged == true )
                {
                    m_responseListBuffer.clear( );
                    
                    Iterator it = this.m_responseList.iterator( );
                    while ( it.hasNext( ) )
                    {
                        SingleVoxelMeasuredSpectralData specData = (SingleVoxelMeasuredSpectralData)it.next( );
                        SingleVoxelMeasuredSpectralData specDataBuffer = new SingleVoxelMeasuredSpectralData( specData );
                                                
                        m_responseListBuffer.add( specDataBuffer );
                    }
                    m_firstTimeChanged = false;
                }
                
                m_sdp = new SingleVoxelSpectralDataProcessing( m_responseListBuffer );
                m_responseList = m_sdp.adjustFirstOrderPhase( m_phasingControl.getFirstOrderPhase( ) );
                changeDisplayedSignal( );
            }
            
            if ( e.getID( ) == m_phasingControl.CANCEL_EVENT )
            {
                // Restore old values
                m_responseList.clear( );
                m_responseNameList.clear( );

                if ( !m_responseListBuffer.isEmpty( ) )
                {
                    Iterator it = this.m_responseListBuffer.iterator( );
                    while ( it.hasNext( ) )
                    {
                        SingleVoxelMeasuredSpectralData specDataBuffer = (SingleVoxelMeasuredSpectralData)it.next( );
                        SingleVoxelMeasuredSpectralData specData = new SingleVoxelMeasuredSpectralData( specDataBuffer );
                        
                        m_responseList.add( specData );
                    }
                }
                
                deleteAllCurves( );
                
                m_firstTimeChanged = true;
                changeDisplayedSignal( );
            }
            
            if ( e.getID( ) == m_phasingControl.ACCEPT_EVENT )
            {
                m_responseListBuffer.clear( );
                m_firstTimeChanged = true;
                m_responseList = m_sdp.getResponseList( );
                
                int index = -1;
                int cntr = 0;
                Iterator it = m_responseNameList.iterator( );
                while ( it.hasNext( ) )
                {
                    String str = (String)it.next( );
                    if ( str.compareTo( "measured" ) == 0 )
                    {
                        index = cntr;
                    }
                    cntr++;
                }
                
                if ( index != -1 )
                {
                    notifyMeasuredSpectrumChanged( m_responseList.get( index ), 0 );
                }
                
                // All data of the series must be changed accordingly
                // Iterate over all responses of the SingleVoxelSpectralDataSeries
                
                LinkedList<SingleVoxelMeasuredSpectralData> linkedList = new LinkedList<>( );
                
                for ( int i = 0; i < m_measuredPatientSpectrumSeries.getNumberOfInstances( ); i++ )
                {
                    SingleVoxelMeasuredSpectralData sVoxData = 
                            new SingleVoxelMeasuredSpectralData( m_measuredPatientSpectrumSeries.getAbsorptionChannelTD( i ),
                                                                 m_measuredPatientSpectrumSeries.getDispersionChannelTD( i ),
                                                                 SingleVoxelSpectralData.TIME_DOMAIN );
                     
                    linkedList.add( sVoxData );
                }
                
                m_sdp.setResponseList( linkedList );
                // Perform the zero order phasing to all responses of the series
                linkedList = m_sdp.adjustZeroOrderPhase( m_phasingControl.getZeroOrderPhase( ) );
                
                m_sdp.setResponseList( linkedList );
                // Perform the first order phasing to all responses of the series
                linkedList = m_sdp.adjustFirstOrderPhase( m_phasingControl.getFirstOrderPhase( ) );
                
                Iterator it1 = linkedList.iterator( );
                cntr = 0;
                
                while ( it1.hasNext( ) )
                {
                    SingleVoxelSpectralData sVoxData = (SingleVoxelSpectralData)it1.next( );
                    this.m_measuredPatientSpectrumSeries.setSpectralDataPointsTD( cntr, sVoxData.getAbsorptionChannelTD(), sVoxData.getDispersionChannelTD( ) );
                    
                    cntr++;
                }
                
                // Redefine the attribute containing the sum spectrum
                m_measuredPatientSumSpectrum = m_measuredPatientSpectrumSeries.getSumSpectrum( );
                
                changeDisplayedSignal( );
            }
        }
        
        if ( sender == m_setShiftParameter )
        {
            m_shiftControl = new SingleVoxelSpectrumShiftControl( "Define frequency shift [Hz]", m_parent, SingleVoxelSpectrumShiftControl.MULTI_RESPONSE_DISPLAY );
            m_shiftControl.addActionListener( this );
        }
        
        // Handle events from the spectral shift gui control
        
        if ( sender == m_shiftControl )
        {
            if ( e.getID( ) == m_shiftControl.CHANGE_EVENT )
            {
                System.out.println( "change event!!" );
                
                if ( m_firstTimeChanged == true )
                {
                    m_responseListBuffer.clear( );
                    
                    Iterator it = this.m_responseList.iterator( );
                    while ( it.hasNext( ) )
                    {
                        SingleVoxelMeasuredSpectralData specData = (SingleVoxelMeasuredSpectralData)it.next( );                        
                        SingleVoxelMeasuredSpectralData specDataBuffer = new SingleVoxelMeasuredSpectralData( specData );
                                                
                        m_responseListBuffer.add( specDataBuffer );
                    }
                    m_firstTimeChanged = false;
                }
                
                m_sdp = new SingleVoxelSpectralDataProcessing( m_responseListBuffer );
                
                m_responseList = m_sdp.frequencyShift( m_shiftControl.getShiftValue( ) );
                
                changeDisplayedSignal( );
            }
            
            if ( e.getID( ) == m_shiftControl.ACCEPT_EVENT )
            {
                System.out.println( "accept event!!" );
                m_responseListBuffer.clear( );
                m_firstTimeChanged = true;
                m_responseList = m_sdp.getResponseList( );
                
                int index = -1;
                int cntr = 0;
                Iterator it = m_responseNameList.iterator( );
                while ( it.hasNext( ) )
                {
                    String str = (String)it.next( );
                    if ( str.compareTo( "measured" ) == 0 )
                    {
                        index = cntr;
                    }
                    cntr++;
                }
                
                if ( index != -1 )
                {
                    notifyMeasuredSpectrumChanged( m_responseList.get( index ), 0 );
                }
                
                // All data of the series must be changed accordingly
                // Iterate over all responses of the SingleVoxelSpectralDataSeries
                
                LinkedList<SingleVoxelMeasuredSpectralData> linkedList = new LinkedList<SingleVoxelMeasuredSpectralData>( );
                
                for ( int i = 0; i < m_measuredPatientSpectrumSeries.getNumberOfInstances( ); i++ )
                {
                    SingleVoxelMeasuredSpectralData sVoxData = 
                            new SingleVoxelMeasuredSpectralData( m_measuredPatientSpectrumSeries.getAbsorptionChannelTD( i ),
                                                                 m_measuredPatientSpectrumSeries.getDispersionChannelTD( i ),
                                                                 SingleVoxelSpectralData.TIME_DOMAIN );
                    
                    linkedList.add( sVoxData );
                }
                
                m_sdp.setResponseList( linkedList );
                
                // Perform the frequency shift to all responsers of the series
                linkedList = m_sdp.frequencyShift( m_shiftControl.getShiftValue( ) );
                 
                Iterator it1 = linkedList.iterator( );
                cntr = 0;
                
                while ( it1.hasNext( ) )
                {
                    SingleVoxelSpectralData sVoxData = (SingleVoxelSpectralData)it1.next( );
                    this.m_measuredPatientSpectrumSeries.setSpectralDataPointsTD( cntr, sVoxData.getAbsorptionChannelTD(), sVoxData.getDispersionChannelTD( ) );
                    
                    cntr++;
                }
                
                // Redefine the attribute containing the sum spectrum
                m_measuredPatientSumSpectrum = m_measuredPatientSpectrumSeries.getSumSpectrum( );
                
                changeDisplayedSignal( );
            }
            
            if ( e.getID( ) == m_shiftControl.CANCEL_EVENT )
            {
                System.out.println( "cancel event!!" );
                
                // Restore old values
                m_responseList.clear( );
                m_responseNameList.clear( );

                if ( !m_responseListBuffer.isEmpty( ) )
                {
                    Iterator it = this.m_responseListBuffer.iterator( );
                    while ( it.hasNext( ) )
                    {
                        SingleVoxelMeasuredSpectralData specDataBuffer = (SingleVoxelMeasuredSpectralData)it.next( );
                        SingleVoxelMeasuredSpectralData specData = new SingleVoxelMeasuredSpectralData( specDataBuffer );
                        
                        m_responseList.add( specData );
                    }
                }
                
                deleteAllCurves( );
                
                m_firstTimeChanged = true;
                changeDisplayedSignal( );
                
            }
        }
        
        if ( e.getSource( ) == m_plotRangeSelector )
        {
            m_responseListBuffer.clear( );
            m_firstTimeChanged = true;
            changeDisplayedSignal( );   
        }

        // Handle the event that the user wants to display point units only
        if ( sender == m_displayPointsOnlyUnit )
        {
            
            // in this mode the scales are draw normally
            setDisplayXReversed( false );
            // save the new display settings
            TDFDFittingDefaults fitdef = TDFDFittingDefaults.getInstance( );
            fitdef.load( );
            String displayDomain = fitdef.getProperty( TDFDFittingDefaults.DISPLAY_DOMAIN );
            
            if ( displayDomain.compareTo( TDFDFittingDefaults.FREQ_DOMAIN ) == 0 )
            {
                fitdef.setProperty( TDFDFittingDefaults.DISPLAY_UNIT, TDFDFittingDefaults.FREQUENCY_DOMAIN_POINTS );
            }
            else
            {
                fitdef.setProperty( TDFDFittingDefaults.DISPLAY_UNIT, TDFDFittingDefaults.TIME_DOMAIN_POINTS );
            }
            fitdef.save( );
            
            // redraw the display with the new x-axis units
            m_responseListBuffer.clear( );
            m_firstTimeChanged = true;
            changeDisplayedSignal( );
        }

        // handle the event that the user wants to display seconds - Hz units
        if ( sender == m_displaySecondsOrHertzUnit )
        {
            // in this mode the x-scale is drawn normally
            setDisplayXReversed( false );
            
            // save the new display settings
            TDFDFittingDefaults fitdef = TDFDFittingDefaults.getInstance( );
            fitdef.load( );
            
            String displayDomain = fitdef.getProperty( TDFDFittingDefaults.DISPLAY_DOMAIN );
            
            if ( displayDomain.compareTo( TDFDFittingDefaults.FREQ_DOMAIN ) == 0 )
            {
                fitdef.setProperty( TDFDFittingDefaults.DISPLAY_UNIT, TDFDFittingDefaults.FREQUENCY_DOMAIN_HZ );
            }
            else
            {
                fitdef.setProperty( TDFDFittingDefaults.DISPLAY_UNIT, TDFDFittingDefaults.TIME_DOMAIN_SECONDS );
            }            
            fitdef.save( );
            // redraw the display with the new x-axis units
            m_responseListBuffer.clear( );
            m_firstTimeChanged = true;
            changeDisplayedSignal( );

        }

        // handle the event that throws e user wants to display seconds ppm units
        if ( sender == m_displaySecondsOrPPMUnit )
        {
            // save the new display settings
            TDFDFittingDefaults fitdef = TDFDFittingDefaults.getInstance( );
            fitdef.load( );
            
            String displayDomain = fitdef.getProperty( TDFDFittingDefaults.DISPLAY_DOMAIN );
            
            if ( displayDomain.compareTo( TDFDFittingDefaults.FREQ_DOMAIN ) == 0 )
            {
                setDisplayXReversed( true );
                fitdef.setProperty( TDFDFittingDefaults.DISPLAY_UNIT, TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM );
            }
            else
            {
                this.setDisplayXReversed( false );
                fitdef.setProperty( TDFDFittingDefaults.DISPLAY_UNIT, TDFDFittingDefaults.TIME_DOMAIN_SECONDS );
            }
            fitdef.save( );
            
            // redraw the display with the new x-axis units
            m_responseListBuffer.clear( );
            m_firstTimeChanged = true;
            changeDisplayedSignal( );
        }
 
        if ( sender ==  this.m_offsetCorrectionFD )
        {
            m_fdOffsetRemoverControl = new OffsetRemovalParameterControl( "Frequency Domain Offset Removal", 
                                                                           m_parent, 
                                                                           SpectrumPhasingDefinitionControl.MULTI_RESPONSE_DISPLAY,
                                                                           0.0f, 0.2f * getNumberOfSamplePoints( ) );
            
            m_fdOffsetRemoverControl.addActionListener( this );
        }
        
        
        // Allows the user to define baseline ranges and subtraction of a spline approximation
        if ( sender ==  m_frequencyDependentBaselineElimination )
        {
            //set display to FD and total range
            
            m_baseLineDefinitionModeActive = true;
            
            m_baselineDefFrame = new BaseLineSplineSupportPointsDefinitionFrame( m_responseList.getFirst( ) );
            
            addActionListener( m_baselineDefFrame );
            
            m_baselineDefFrame.addActionListener( this );
             
        }
        
        if ( sender == m_removeMacromolecularBaseline )
        {
            removeBaseline( );
        }
        
        // called whenever the baseline definition frame raises an accept event
        if ( sender == m_baselineDefFrame )
        {
            
            if ( e.getID( ) == m_baselineDefFrame.ACCEPT_EVENT )
            {
                m_baseLineRangesDataModel = m_baselineDefFrame.getRangeModel( );
                
                m_baseLineDefinitionModeActive = false;
                
                if ( m_baseLineRangesDataModel.getSize() > 2 )
                {
                    // Based on the range information
                    determineBaseLineSpline( );
                    m_removeMacromolecularBaseline.setEnabled( true );
                }
                
                m_parent.update( m_parent.getGraphics( ) );
            }
            
            if ( e.getID( ) == m_baselineDefFrame.CANCEL_EVENT )
            {
                m_baseLineDefinitionModeActive = false;
                m_parent.update( m_parent.getGraphics( ) );
            }
            
            if ( e.getID( ) == m_baselineDefFrame.PREVIEW_EVENT )
            {
                m_parent.update( m_parent.getGraphics( ) );
            }
            
        }
        
        
        // handles fd domain offset elimination events ...
       
        if ( sender ==  m_fdOffsetRemoverControl )
        {
            if ( e.getID( ) == m_fdOffsetRemoverControl.CANCEL_EVENT )
            {
                // Restore old values
                m_responseList.clear( );
                m_responseNameList.clear( );
                
                if ( !m_responseListBuffer.isEmpty( ) )
                {
                    Iterator it = this.m_responseListBuffer.iterator( );
                    while ( it.hasNext( ) )
                    {    
                        SingleVoxelMeasuredSpectralData specDataBuffer = (SingleVoxelMeasuredSpectralData)it.next( );                     
                        SingleVoxelMeasuredSpectralData specData = new SingleVoxelMeasuredSpectralData( specDataBuffer );

                        m_responseList.add( specData );
                    }
                }
              
                m_firstTimeChanged = true;
                changeDisplayedSignal( );
            }
            
            if ( e.getID( ) == m_fdOffsetRemoverControl.ACCEPT_EVENT )
            {
                m_responseListBuffer.clear( );
                m_firstTimeChanged = true;
                m_responseList = m_sdp.getResponseList( );
                
                int index = -1;
                int cntr = 0;
                Iterator it = m_responseNameList.iterator( );
                while ( it.hasNext( ) )
                {
                    String str = (String)it.next( );
                    if ( str.compareTo( "measured" ) == 0 )
                    {
                        index = cntr;
                    }
                    cntr++;
                }
                
                if ( index != -1 )
                {
                    notifyMeasuredSpectrumChanged( m_responseList.get( index ), 0 );
                }
                
                // All data of the series must be changed accordingly
                // Iterate over all responses of the SingleVoxelSpectralDataSeries
                
                LinkedList<SingleVoxelMeasuredSpectralData> linkedList = new LinkedList<SingleVoxelMeasuredSpectralData>( );
                
                for ( int i = 0; i < m_measuredPatientSpectrumSeries.getNumberOfInstances( ); i++ )
                {
                    SingleVoxelMeasuredSpectralData sVoxData = 
                            new SingleVoxelMeasuredSpectralData( m_measuredPatientSpectrumSeries.getAbsorptionChannelTD( i ),
                                                                 m_measuredPatientSpectrumSeries.getDispersionChannelTD( i ),
                                                                 SingleVoxelSpectralData.TIME_DOMAIN );
                     
                    linkedList.add( sVoxData );
                }
                
                m_sdp.setResponseList( linkedList );
                
                // Perform the frequency domain offset correction
                linkedList = m_sdp.dc_correct( (int)m_fdOffsetRemoverControl.getMinimumRangeValue( ), 
                                               (int)m_fdOffsetRemoverControl.getMaximumRangeValue( ) );
                
                Iterator it1 = linkedList.iterator( );
                cntr = 0;
                
                while ( it1.hasNext( ) )
                {
                    SingleVoxelSpectralData sVoxData = (SingleVoxelSpectralData)it1.next( );
                    m_measuredPatientSpectrumSeries.setSpectralDataPointsTD( cntr, sVoxData.getAbsorptionChannelTD( ), sVoxData.getDispersionChannelTD( ) );
                    cntr++;
                }
                
                // Redefine the attribute containing the sum spectrum
                m_measuredPatientSumSpectrum = m_measuredPatientSpectrumSeries.getSumSpectrum( );
                
                changeDisplayedSignal( );
            }
            
            if ( e.getID( ) == m_fdOffsetRemoverControl.MINIMUM_RANGE_VALUE_CHANGED_EVENT  )
            {
                if ( m_firstTimeChanged == true )
                {
                    m_responseListBuffer.clear( );
                    
                    Iterator it = this.m_responseList.iterator( );
                    while ( it.hasNext( ) )
                    {
                        SingleVoxelMeasuredSpectralData specData = (SingleVoxelMeasuredSpectralData)it.next( );
                        SingleVoxelMeasuredSpectralData specDataBuffer = new SingleVoxelMeasuredSpectralData( specData );
                                                
                        m_responseListBuffer.add( specDataBuffer );
                    }
                    m_firstTimeChanged = false;

                }
                
                m_sdp = new SingleVoxelSpectralDataProcessing( m_responseListBuffer );
                
                m_responseList = m_sdp.dc_correct( (int)m_fdOffsetRemoverControl.getMinimumRangeValue( ), 
                                                   (int)m_fdOffsetRemoverControl.getMaximumRangeValue( ) );
                
                changeDisplayedSignal( );
                
            }
            
            if ( e.getID( ) == m_fdOffsetRemoverControl.MAXIMUM_RANGE_VALUE_CHANGED_EVENT  )
            {
                if ( m_firstTimeChanged == true )
                {
                    m_responseListBuffer.clear( );
                    
                    Iterator it = this.m_responseList.iterator( );
                    while ( it.hasNext( ) )
                    {
                        SingleVoxelMeasuredSpectralData specData = (SingleVoxelMeasuredSpectralData)it.next( );
                        SingleVoxelMeasuredSpectralData specDataBuffer = new SingleVoxelMeasuredSpectralData( specData );
                                                
                        m_responseListBuffer.add( specDataBuffer );
                    }
                    m_firstTimeChanged = false;
                }
                
                m_sdp = new SingleVoxelSpectralDataProcessing( m_responseListBuffer );
                
                m_responseList = m_sdp.dc_correct( (int)m_fdOffsetRemoverControl.getMinimumRangeValue( ), 
                                                   (int)m_fdOffsetRemoverControl.getMaximumRangeValue( ) );
                changeDisplayedSignal( );
            }
        }
        
        // Handle events from the spectral multiply first sample point gui control
        
        if ( sender == m_firstPointMultiplicationControl )
        {
            if ( e.getID( ) == m_firstPointMultiplicationControl.CHANGE_EVENT )
            {
                
                if ( m_firstTimeChanged == true )
                {
                    m_responseListBuffer.clear( );
                    
                    Iterator it = this.m_responseList.iterator( );
                    while ( it.hasNext( ) )
                    {
                        SingleVoxelMeasuredSpectralData specData = (SingleVoxelMeasuredSpectralData)it.next( );
                        SingleVoxelMeasuredSpectralData specDataBuffer = new SingleVoxelMeasuredSpectralData( specData );
                        
                        m_responseListBuffer.add( specDataBuffer );
                    }
                    m_firstTimeChanged = false;
                }
                
                m_sdp = new SingleVoxelSpectralDataProcessing( m_responseListBuffer );
                
                m_responseList = m_sdp.multiplyFirstTDPoint( m_firstPointMultiplicationControl.getMultiplicationFactor( ) );
                
                changeDisplayedSignal( );
            }
            
            if ( e.getID( ) == SingleVoxelFirstSamplePointMultiplicatorControl.ACCEPT_EVENT )
            {
                
                m_responseListBuffer.clear( );
                m_firstTimeChanged = true;
                m_responseList = m_sdp.getResponseList( );
                
                int index = -1;
                int cntr = 0;
                Iterator it = m_responseNameList.iterator( );
                while ( it.hasNext( ) )
                {
                    String str = (String)it.next( );
                    if ( str.compareTo( "measured" ) == 0 )
                    {
                        index = cntr;
                    }
                    cntr++;
                }
                
                if ( index != -1 )
                {
                    notifyMeasuredSpectrumChanged( m_responseList.get( index ), 0 );
                }
                
                changeDisplayedSignal( );
            }
            
            if ( e.getID( ) == m_firstPointMultiplicationControl.CANCEL_EVENT )
            {
                
                // Restore old values
                m_responseList.clear( );
                m_responseNameList.clear( );
                if ( !m_responseListBuffer.isEmpty( ) )
                {
                    Iterator it = this.m_responseListBuffer.iterator( );
                    while ( it.hasNext( ) )
                    {
                        SingleVoxelMeasuredSpectralData specDataBuffer = (SingleVoxelMeasuredSpectralData)it.next( );                       
                        SingleVoxelMeasuredSpectralData specData = new SingleVoxelMeasuredSpectralData( specDataBuffer );
                        m_responseList.add( specData );
                    }
                }
                
                deleteAllCurves( );
                
                m_firstTimeChanged = true;
                changeDisplayedSignal( );       
            }
        }
    }
    
   /**
    * Based on the private attribute  
    *
    *      m_baseLineRangesDataModel
    *
    * this routine computes the baseline spline.
    */
    private void determineBaseLineSpline( )
    {
        
        SingleVoxelSpectralData currentInstance = m_measuredPatientSpectrumSeries.getCurrentInstance( m_currentlyDisplayedInstance );
        
        float[] absCurve = currentInstance.getAbsorptionChannelFD( );
        
        CatmullRomSpline b = new CatmullRomSpline( );
        
        // in order to guarantee continuity we have to preceed the interval 
        // model by two pre-points and two post points
        OrderedBaseLinePointsDataModel model = OrderedBaseLinePointsDataModel.getInstance( );
        model.setArraySize( getNumberOfSamplePoints( ) );
        // Determine the points of the spline
        for ( int i = 0 ; i < m_baseLineRangesDataModel.getSize( ); i++ )
        {
            Point2D pnt = model.getPoint( i );
            b.addPoint( (float)pnt.getX(), (float)pnt.getY() );
        }
        
        float[] baselineSplineAbs = b.computeSplineCurve( absCurve.length );
        float[] baselineSplineDisp = new float[absCurve.length];
        
        for ( int i = 0; i < absCurve.length;  i++ )
        {
            baselineSplineDisp[i] = 0f;
        }
        
        // Define a member variable to model the baseline spline
        m_baselineSpline = new SingleVoxelMeasuredSpectralData( );
        
        SingleVoxelSpectralData dummy =  m_responseList.getFirst( );
        
        m_baselineSpline.setSpectralDataPointsFD( baselineSplineAbs, baselineSplineDisp );
        m_baselineSpline.setDicomHeaderInfo( dummy.getDicomHeaderInfo( ) );
        m_baselineSpline.setSiemensSpecificDicomHeaderInfo( dummy.getSiemensSpecificDicomHeaderInfo( ) );
        
        this.addSpectralDataSet( "baseline_spline", m_baselineSpline , true );
        
        // System.out.println( "spline computed" );
        
    }
    
    
    /**
     * Adds a listener to the list of listeners to action events.
     * @param listener
     */
    protected void addActionListener( ActionListener listener )
    {
        m_listeners.add( listener );
    }
    
    /**
     * Notifies all registered objects that a change has taken place.
     */
    private void notifyMeasuredSpectrumChanged( Object obj, int id )
    {
        Iterator it = m_listeners.iterator( );
        while( it.hasNext( ) )
	{
            ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( obj, id, "measured spectrum changed" ) );
        }
    }
    
    /**
     * Returns the response having name respName.
     * @param respName
     * @return 
     */
    public SingleVoxelSpectralData getNamedResponse( String respName )
    {
        int index = 0;
        Iterator it = m_responseNameList.iterator( );
        SingleVoxelSpectralData specData = new SingleVoxelSpectralData( );
        
        while ( it.hasNext( ) )
        {
            if ( ( (String)it.next( ) ).compareTo( respName ) == 0 )
            {
                specData = m_responseList.get( index );
            }
            index++;
        }
        
        return specData;
    }
   
    ////////////////////////////////////////////////////////////////////////////
    
            
    /**
     * Determines the selected values at the position indicated by the
     * mouseEvent.
     */
    /**
     * 
     * Determines the selected values at the position indicated by the
     * mouseEvent.
     * 
     * @param p
     * @return 
     */
    private Point2D.Double determineValuesAtCursorPosition( Point p )
    {
        
        if ( ( p.x > this.getSize( ).getWidth( ) / 10 ) && ( p.x < 9 * this.getSize( ).getWidth( ) / 10 ) )
        {   
            if ( ( p.y > this.getSize( ).getHeight( ) / 10 ) && ( p.y < 9 * this.getSize( ).getHeight( ) / 10 ) )
            {
                float xrange = 8.0f * (float)getSize( ).getWidth( ) / 10.0f;
                float yrange = 8.0f * (float)getSize( ).getHeight( ) / 10.0f;
                
                MRSignalProperties mrcond = MRSignalProperties.getInstance( );
                
                if ( m_curves != null )
                {
                    if ( m_curves.size( ) > 0 )
                    {
                        float xPosCorrected = ( ( (float)p.x - (float)this.getSize( ).getWidth( ) / 10.0f )  )  / xrange;
                        float yPosCorrected = ( ( (float)p.y - (float)this.getSize( ).getHeight( ) / 10.0f )  )  / yrange;
                        
                        if ( m_XYPlotRangeSelectorMenuItem.isSelected( ) == true )
                        {
                            if ( this.m_displayXReversed == false )
                            {
                                m_lastSelectedOffsetValue = m_xyForcedPlotRange.getMinimumX( ) + xPosCorrected * m_xyForcedPlotRange.getXRange( );
                            }
                            else
                            {
                                m_lastSelectedOffsetValue = m_xyForcedPlotRange.getMaximumX( ) + xPosCorrected * m_xyForcedPlotRange.getXRange( );
                            }
                            
                            m_lastSelectedAmplitudeValue = m_xyForcedPlotRange.getMaximumY( ) - yPosCorrected * m_xyForcedPlotRange.getYRange( );
                        }
                        else
                        {
                            if ( this.m_displayXReversed == false )
                            {
                                m_lastSelectedOffsetValue = m_xyAutoPlotRange.getMinimumX( ) + xPosCorrected * m_xyAutoPlotRange.getXRange( );                 
                            }
                            else
                            {   
                                m_lastSelectedOffsetValue = m_xyAutoPlotRange.getMinimumX( ) + xPosCorrected * m_xyAutoPlotRange.getXRange( );
                            }
                            m_lastSelectedAmplitudeValue = m_xyAutoPlotRange.getMaximumY( ) - yPosCorrected * m_xyAutoPlotRange.getYRange( );
                        }
                    }
                }
            }
        }
        
        return new Point2D.Double( m_lastSelectedOffsetValue, m_lastSelectedAmplitudeValue );
    }
    
    /**
     * Returns the Point2D.Float object of the last user selected point.
     * @return 
     */
    public Point2D.Double getLastSelectedPoint( )
    {
        return new Point2D.Double( m_lastSelectedOffsetValue, m_lastSelectedAmplitudeValue );
    }
    
    /**
     * Returns the number of sample points of the currently displayed signal.
     * @return 
     */
    public int getNumberOfSamplePoints( )
    {
        return m_measuredPatientSpectrumSeries.getAbsorptionChannelTD( 0 ).length;
    }
    
    /**
     * Removes the baseline by subtraction of a spline 
     * Spline is defined by the model or user defined points where the absorption
     * spectrum should dissappear.
     */
    private void removeBaseline( )
    {
        
        if ( m_sdp == null )
        {
            m_sdp = new SingleVoxelSpectralDataProcessing( m_responseList );
        }
        
        if ( this.m_baselineSpline != null )
        {
            m_responseListBuffer.clear( );
            m_firstTimeChanged = true;
            m_responseList = m_sdp.getResponseList( );
                
            int index = -1;
            int cntr = 0;
            Iterator it = m_responseNameList.iterator( );
            while ( it.hasNext( ) )
            {
                String str = (String)it.next( );
                if ( str.compareTo( "measured" ) == 0 )
                {
                    index = cntr;
                }
                cntr++;
            }
                
            if ( index != -1 )
            {
                notifyMeasuredSpectrumChanged( m_responseList.get( index ), 0 );
            }
                
            // All data of the series must be changed accordingly
            // Iterate over all responses of the SingleVoxelSpectralDataSeries
                
            LinkedList<SingleVoxelMeasuredSpectralData> linkedList = new LinkedList<>( );
               
            for ( int i = 0; i < m_measuredPatientSpectrumSeries.getNumberOfInstances( ); i++ )
            {
                SingleVoxelMeasuredSpectralData sVoxData = new SingleVoxelMeasuredSpectralData( 
                                                                                m_measuredPatientSpectrumSeries.getAbsorptionChannelTD( i ),
                                                                                m_measuredPatientSpectrumSeries.getDispersionChannelTD( i ),
                                                                                SingleVoxelSpectralData.TIME_DOMAIN );
                     
                linkedList.add( sVoxData );
            }
                
            m_sdp.setResponseList( linkedList );
            // Subtract the baseline from all responses of this series
            linkedList = m_sdp.removeBaseLineSpline( m_baseLineRangesDataModel );
                
               
            Iterator it1 = linkedList.iterator( );
            cntr = 0;
                
            while ( it1.hasNext( ) )
            {
                SingleVoxelSpectralData sVoxData = (SingleVoxelSpectralData)it1.next( );
                this.m_measuredPatientSpectrumSeries.setSpectralDataPointsTD( cntr, sVoxData.getAbsorptionChannelTD(), sVoxData.getDispersionChannelTD( ) );
                   
                cntr++;
            }
                
            // Redefine the attribute containing the sum spectrum
            m_measuredPatientSumSpectrum = m_measuredPatientSpectrumSeries.getSumSpectrum( );
                
            changeDisplayedSignal( );
        }
    }
    
    /**
     * This method copies the response list: this is necessary in case the user
     * cancels a post processing step without having applied it at least once.
     */
    private void copyResponseList( ) 
    {
        Iterator it = m_responseList.iterator( );
        while ( it.hasNext( ) )
        {
            SingleVoxelMeasuredSpectralData data = (SingleVoxelMeasuredSpectralData)it.next( );
            m_responseListBuffer.add( new SingleVoxelMeasuredSpectralData( data ) );
        }
    }
    
    // Attributes
    private JFrame m_parent;
    
    protected LinkedList<ActionListener> m_listeners = new LinkedList<>( );
    
    private OrderedBaseLinePointsDataModel m_baseLineRangesDataModel = OrderedBaseLinePointsDataModel.getInstance( );
    
    protected LinkedList<String> m_responseNameList = new LinkedList<>( );
    protected LinkedList<SingleVoxelMeasuredSpectralData> m_responseList = new LinkedList<>( );
    protected LinkedList<SingleVoxelMeasuredSpectralData> m_responseListBuffer = new LinkedList<>( );
       
    protected boolean m_firstTimeChanged = true;
    
    // Popup menu : derived classes should be able to extend the popup menu
    // protected JPopupMenu m_popupmenu;
    protected JMenuItem m_pasteSingleVoxelSpectrumDataMenuItem;
    protected JMenuItem m_copySingleVoxelSpectrumDataMenuItem;
    protected JMenu m_domainSelector;
    protected JMenu m_channelSelector;
    
    protected JMenu m_displayUnitSelector;
    
    protected JMenu m_signalProcessing;
    
    // The displayer can show time or frequency domain data
    protected JRadioButtonMenuItem m_timeDomain;
    protected JRadioButtonMenuItem m_frequencyDomain;
    
    // The channel selector allows the user to select the following channels
    protected JRadioButtonMenuItem m_absorption;
    protected JRadioButtonMenuItem m_dispersion;
    protected JRadioButtonMenuItem m_magnitude;
    protected JRadioButtonMenuItem m_power;
    protected JRadioButtonMenuItem m_phase;
     
    // Signal prosessing menu items
    protected JRadioButtonMenuItem m_applyProcessing;
    
    protected JMenuItem m_setApodizationParameters;
    protected JMenuItem m_multiplyFirstTDPoint;
    
    protected JMenuItem m_offsetCorrectionFD;
    
    protected JMenuItem m_frequencyDependentBaselineElimination;
    protected JMenuItem m_removeMacromolecularBaseline;
    
    protected BaseLineSplineSupportPointsDefinitionFrame m_baselineDefFrame;
    protected JMenuItem m_setZeroFillingParameters;
    protected JMenuItem m_setPhasingParameters;
    protected JMenuItem m_setShiftParameter;
    protected JMenuItem m_performOSFFiltering;
    protected JMenuItem m_performARSOSDenoising;
    
    protected JMenuItem m_performOSFFittingPlainARSOS;
    protected JMenuItem m_performOSFFittingCombinedARSOS;
    
    protected OrderStatisticsFilterDefinitionControl m_controlOSF;
    
    protected JRadioButtonMenuItem m_displayPointsOnlyUnit;
    protected JRadioButtonMenuItem m_displaySecondsOrHertzUnit;
    protected JRadioButtonMenuItem m_displaySecondsOrPPMUnit;
    
    protected ButtonGroup m_unitSelectionButtonGroup;
    
    protected ButtonGroup m_domainButtonGroup;
    protected ButtonGroup m_channelButtonGroup;
    
    // Members to store the name and the   data series itself 
    public String m_measuredPatientSpectrumFileName;
    public SingleVoxelSpectralDataSeries m_measuredPatientSpectrumSeries;
    
    private float[][] m_originalDataSeriesAbs;
    private float[][] m_originalDataSeriesDisp;
    
    private ZeroFillParameterDefinitionControl m_zerofillDefiner;
    private ApodizationParameterDefinitionControl m_apodizationDefiner; 
    private SpectrumPhasingDefinitionControl m_phasingControl;
    private SingleVoxelSpectrumShiftControl m_shiftControl;
    private SingleVoxelFirstSamplePointMultiplicatorControl m_firstPointMultiplicationControl;
    private OffsetRemovalParameterControl m_fdOffsetRemoverControl;
    
    // There is a separate plot range for time domain as well as for frequency domain
    private XYPlotRange m_xyAutoPlotRangeTD;
    private XYPlotRange m_xyForcedPlotRangeTD;

    private XYPlotRange m_xyAutoPlotRangeFD;
    private XYPlotRange m_xyForcedPlotRangeFD;
    
    private SingleVoxelSpectralDataProcessing m_sdp;
    
    protected int m_currentlyDisplayedInstance = 0;
    
    // For proper serialization
    private static final long serialVersionUID = 1487225897889393L;
    
     // Attribute defining the sum spectrum of all responses
    protected SingleVoxelMeasuredSpectralData m_measuredPatientSumSpectrum;
    protected SingleVoxelMeasuredSpectralData m_baselineSpline;
    // Attribute that allows for copying and pasting of SingleVoxelSpectralData dataobjects
    private SingleVoxelSpectralDataClipboard m_svsDataClipboard = SingleVoxelSpectralDataClipboard.getInstance( );
    
   // members necessary to realize baseline point selection with the mouse
    protected float m_lastSelectedOffsetValue;
    protected float m_lastSelectedAmplitudeValue;
    protected boolean m_baseLineDefinitionModeActive = false;
    public final static int BASELINE_POINT_SELECTED_EVENT = 500;
    
    private DicomDirectory m_dicomDir;

}
