/*
 * File: $Id: SpectrumPhasingDefinitionControl.java,v 1.1.1.1 2006/04/19 09:38:51 hans Exp $
 * History
 * Date: 22.05.2003, 11:22 Hans Slotboom  Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

package org.spectrim.tdfdfit;

import java.awt.Dimension;
import java.awt.BorderLayout;
import java.awt.Point;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import java.util.LinkedList;
import java.util.Iterator;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JLabel;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JRadioButton;
import javax.swing.ButtonGroup;


import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * SpectrumPhasingDefinitionControl
 * Performs phasing.
 * @author  hans
 */
public class SpectrumPhasingDefinitionControl extends JFrame implements ChangeListener, ActionListener
{
    
    /** 
     * Creates a new instance of SpectrumPhasingDefinitionControl. 
     * @param title
     * @param parent
     * @param displayMode
     */
    public SpectrumPhasingDefinitionControl( String title, JFrame parent, int displayMode  ) 
    {      
        m_parent = parent;
        setTitle( title );
        
        m_displayMode = displayMode;
        
        m_contentPanel = new JPanel( );
        m_contentPanel.setLayout( new BorderLayout( ) );

        // add window listener necessary to handle the window closing 
        // event properly
        addWindowListener( 
            new WindowAdapter( )
            {
                public void windowClosing( WindowEvent event )
                {
                   dispose( );
                }
            }
        );
        
        instantiateZeroOrderPhasingPanel( );
        instantiateFirstOrderPhasingPanel( );
        
        m_phaseSettingCollectionPanel = new JPanel( );
        m_phaseSettingCollectionPanel.setLayout( new BoxLayout( m_phaseSettingCollectionPanel, BoxLayout.Y_AXIS ) );
        
        m_phaseSettingCollectionPanel.add( m_zeroOrderPhasingPanel );
        m_phaseSettingCollectionPanel.add( m_firstOrderPhasingPanel );
        
        m_contentPanel.add( m_phaseSettingCollectionPanel, BorderLayout.NORTH );
        
        if ( m_displayMode == MULTI_RESPONSE_DISPLAY )
        {
            createInstanceSpecificPanel( );
        
            m_contentPanel.add( m_performActionToCurrentOnlyPanel, BorderLayout.CENTER );
        }
        
        instantiateCancelApplyPanel( );
        
        m_contentPanel.add( m_selectionPanel, BorderLayout.SOUTH );
        
        setContentPane( m_contentPanel );   
        
        pack( );
        
        // Put the Dialog nicely in the middle of its parent
        Point parentLoc = m_parent.getLocation( );
        Point centerParent = new Point( m_parent.getX( ) + ( m_parent.getWidth( ) / 2 ) - this.getWidth( ) / 2,
                                        m_parent.getY( ) + ( m_parent.getHeight( ) / 2 ) - this.getHeight( ) / 2 );
        setLocation( centerParent );
        setVisible( true );

    }
    
    
    /**
     * Instantiates the panel containing the controls that allows  zero
     * order phasing in the frequency domain
     */
    private void instantiateZeroOrderPhasingPanel( )
    {
        m_zeroOrderPhasingPanel = new JPanel( );
        m_zeroOrderPhasingPanel.setLayout( new BoxLayout( m_zeroOrderPhasingPanel, BoxLayout.Y_AXIS ) );
        m_zeroOrderPhasingPanel.setBorder( BorderFactory.createCompoundBorder(
                                           BorderFactory.createEmptyBorder( 10, 10, 10, 10 ),
                                           BorderFactory.createTitledBorder( "Zero order phasing" ) )
                                          );
        
        m_zeroOrderPhasingPanel.setPreferredSize( new Dimension( 430, 130 ) );
        m_zeroOrderPhasingPanel.setMaximumSize( new Dimension( 430, 130 ) );
        m_zeroOrderPhasingPanel.setMinimumSize( new Dimension( 430, 130 ) );
               
        m_zeroOrderPhasingSliderPanel = new JPanel( );
        m_zeroOrderPhasingSliderPanel.setLayout( new BoxLayout( m_zeroOrderPhasingSliderPanel, BoxLayout.X_AXIS ) ); 
        
        m_zeroOrderPhasingSliderLabel = new JLabel( "Horizontal center" );
        m_zeroOrderPhasingSliderPanel.add( m_zeroOrderPhasingSliderLabel );
        
        m_zeroOrderPhasingSliderPanel.add( Box.createHorizontalGlue( ) );
        
        m_zeroOrderPhasingSlider = new JSlider( JSlider.HORIZONTAL, -180, 180, 0 );
        m_zeroOrderPhasingSlider.setMajorTickSpacing( 90 );
        m_zeroOrderPhasingSlider.setMinorTickSpacing( 10 );
        m_zeroOrderPhasingSlider.setPaintTicks( true );
        m_zeroOrderPhasingSlider.setPaintLabels( true );
        m_zeroOrderPhasingSlider.addChangeListener( this );
        m_zeroOrderPhasingSlider.setMinimumSize( m_zeroOrderPhasingSlider.getPreferredSize( ) );
        
        m_zeroOrderPhasingSlider.setMinimumSize( new Dimension( 250, 50 ) );
        m_zeroOrderPhasingSlider.setMaximumSize( new Dimension( 250, 50 ) );
        m_zeroOrderPhasingSlider.setPreferredSize( new Dimension( 250, 50 ) );
        
        m_zeroOrderPhasingSliderPanel.add( m_zeroOrderPhasingSlider );
        
        m_zeroOrderPhasingPanel.add( m_zeroOrderPhasingSliderPanel );
    }
    
    
    /**
     * Instantiates the panel containing the controls that allows  zero
     * order phasing in the frequency domain.
     */
    private void instantiateFirstOrderPhasingPanel( )
    {
        m_firstOrderPhasingPanel = new JPanel( );
        m_firstOrderPhasingPanel.setLayout( new BoxLayout( m_firstOrderPhasingPanel, BoxLayout.Y_AXIS ) );
        m_firstOrderPhasingPanel.setBorder( BorderFactory.createCompoundBorder(
                                            BorderFactory.createEmptyBorder( 10, 10, 10, 10 ),
                                            BorderFactory.createTitledBorder( "First order phasing" ) )
                                          );
        
        m_firstOrderPhasingPanel.setPreferredSize( new Dimension( 430, 130 ) );
        m_firstOrderPhasingPanel.setMaximumSize( new Dimension( 430, 130 ) );
        m_firstOrderPhasingPanel.setMinimumSize( new Dimension( 430, 130 ) );
               
        m_firstOrderPhasingSliderPanel = new JPanel( );
        m_firstOrderPhasingSliderPanel.setLayout( new BoxLayout( m_firstOrderPhasingSliderPanel, BoxLayout.X_AXIS ) ); 
        
        m_firstOrderPhasingSliderLabel = new JLabel( "First order phase" );
        m_firstOrderPhasingSliderPanel.add( m_firstOrderPhasingSliderLabel );
        
        m_firstOrderPhasingSliderPanel.add( Box.createHorizontalGlue( ) );
        
        m_firstOrderPhasingSlider = new JSlider( JSlider.HORIZONTAL, -360, 360, 0 );
        m_firstOrderPhasingSlider.setMajorTickSpacing( 90 );
        m_firstOrderPhasingSlider.setMinorTickSpacing( 10 );
        m_firstOrderPhasingSlider.setPaintTicks( true );
        m_firstOrderPhasingSlider.setPaintLabels( true );
        m_firstOrderPhasingSlider.addChangeListener( this );
        m_firstOrderPhasingSlider.setMinimumSize( m_firstOrderPhasingSlider.getPreferredSize( ) );
        
        m_firstOrderPhasingSlider.setMinimumSize( new Dimension( 250, 50 ) );
        m_firstOrderPhasingSlider.setMaximumSize( new Dimension( 250, 50 ) );
        m_firstOrderPhasingSlider.setPreferredSize( new Dimension( 250, 50 ) );
        
        m_firstOrderPhasingSliderPanel.add( m_firstOrderPhasingSlider );
        
        m_firstOrderPhasingPanel.add( m_firstOrderPhasingSliderPanel );
       
        m_contentPanel.add( m_firstOrderPhasingPanel, BorderLayout.CENTER );
    }
    
    /**
     * Instantiates an apply/ cancel panel.
     */
    private void instantiateCancelApplyPanel( )
    {
        // Selection panel definitions
        m_applyButton = new JButton( "Apply" );
        m_applyButton.addActionListener( this );
        m_applyButton.setToolTipText( "Applies the selected zerofilling factor to the currently loaded dataset" );
        m_cancelButton = new JButton( "Cancel" );
        m_cancelButton.addActionListener( this );
        m_cancelButton.setToolTipText( "Cancels this operation without applying the selected zerofilling factor" );
        
        m_selectionPanel = new JPanel( );
        
        m_selectionPanel.setLayout( new BoxLayout( m_selectionPanel, BoxLayout.X_AXIS ) );
        m_selectionPanel.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );
        m_selectionPanel.add( Box.createGlue( ) );
        m_selectionPanel.add( m_applyButton );
        m_selectionPanel.add( Box.createGlue( ) );
        m_selectionPanel.add( m_cancelButton );
        m_selectionPanel.add( Box.createGlue( ) );
    }
    
    /**
     * Called whenever a registered component raises a changeEvent.
     */
    @Override
    public void stateChanged( ChangeEvent e ) 
    {
        Object sender = e.getSource( );
        
        // Handles responses on horizontal slider changed and released event
        if ( sender == m_zeroOrderPhasingSlider )
        {
             if ( !m_zeroOrderPhasingSlider.getValueIsAdjusting( ) )
            {
                Iterator it = m_listeners.iterator( );
                while( it.hasNext( ) )
                {
                    ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, ZERO_ORDER_PHASING_EVENT, "" ) );
                }         
            }
        }        
            
        // Handles responses on horizontal slider changed and released event
        if ( sender == m_firstOrderPhasingSlider )
        {
             if ( !m_firstOrderPhasingSlider.getValueIsAdjusting( ) )
            {
                Iterator it = m_listeners.iterator( );
                while( it.hasNext( ) )
                {
                    ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, FIRST_ORDER_PHASING_EVENT, "" ) );
                }         
            }
        }   
    }    
    
    /**
     * Adds an ActionListener to the list of action listeners who are notified
     * when something happens within this object.
     * @param e
     */
    public void addActionListener( ActionListener e )
    {
	m_listeners.add( e );
    }
    
    /**
     * Returns the value of the zero order slider.
     * @return 
     */  
    public int getZeroOrderPhase( )
    {
        return m_zeroOrderPhasingSlider.getValue( );
    }

    /**
     * Returns the value of the zero order slider.
     * @return 
     */  
    public int getFirstOrderPhase( )
    {
        return m_firstOrderPhasingSlider.getValue( );
    }

    /**
     * Is called whenever a registered component raises an ActionEvent.
     * @param e
     */
    @Override
    public void actionPerformed( ActionEvent e ) 
    {
       
        Object sender = e.getSource( );
       
       // handle responses on cancel button pressed event
        if ( sender == m_cancelButton )
        {
            Iterator it = m_listeners.iterator( );
             while( it.hasNext( ) )
             {
                ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, CANCEL_EVENT, "" ) );
             }
            dispose( );
        }
        // handle responses on apply button pressed event
        if ( sender == m_applyButton )
        {
            Iterator it = m_listeners.iterator( );
             while( it.hasNext( ) )
             {
                ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, ACCEPT_EVENT, "" ) );
             }
            dispose( );
        }
    }
    
    
    /**
     * Creates the panel that allows the user to perform the operation to 
     * the selected instance only.
     */
    public void createInstanceSpecificPanel( )
    {
        m_performActionToCurrentOnlyPanel =  new JPanel( );
        
        m_performActionToCurrentOnlyPanel.setLayout( new BoxLayout( m_performActionToCurrentOnlyPanel, BoxLayout.X_AXIS ) );
        
        m_applyToSelectedInstanceOnly = new JRadioButton( "Selected instance only" );
        m_applyToSelectedInstanceOnly.setSelected( true );
        
        m_applyToAllInstances = new JRadioButton( "All instances" );
        m_applyToButtonGroup.add( m_applyToAllInstances );
        m_applyToButtonGroup.add( m_applyToSelectedInstanceOnly );
        m_performActionToCurrentOnlyPanel.add( Box.createHorizontalGlue( ) );
        m_performActionToCurrentOnlyPanel.add( m_applyToSelectedInstanceOnly );
        m_applyToSelectedInstanceOnly.setToolTipText( "Applies frequency shift only to the selected instance" );
        m_performActionToCurrentOnlyPanel.add( Box.createHorizontalGlue( ) );
        m_performActionToCurrentOnlyPanel.add( m_applyToAllInstances );
        m_applyToAllInstances.setToolTipText( "Applies frequency shift to all instances of the spectrum series" );
        m_performActionToCurrentOnlyPanel.add( Box.createHorizontalGlue( ) );
    }
    
    
    // Member attributes
    private JFrame m_parent;
    
    private JPanel  m_contentPanel;
    private JPanel  m_zeroOrderPhasingPanel;
    private JPanel  m_zeroOrderPhasingSliderPanel;
    private JSlider m_zeroOrderPhasingSlider;
    private JLabel  m_zeroOrderPhasingSliderLabel;

    private JPanel  m_firstOrderPhasingPanel;
    private JPanel  m_firstOrderPhasingSliderPanel;
    private JSlider m_firstOrderPhasingSlider;
    private JLabel  m_firstOrderPhasingSliderLabel;    
    
    private JPanel m_phaseSettingCollectionPanel;
    
    private JButton m_applyButton;
    private JButton m_cancelButton;
    private JPanel m_selectionPanel;
    private JPanel m_performActionToCurrentOnlyPanel;
    
    private JRadioButton m_applyToSelectedInstanceOnly;
    private JRadioButton m_applyToAllInstances;
    
    private ButtonGroup m_applyToButtonGroup = new ButtonGroup( );
    
    // Listeners to changes in this class
    private LinkedList<ActionListener> m_listeners = new LinkedList<ActionListener>( );
    
    // events raised by accept and cancel button
    public final static int ACCEPT_EVENT = 0;
    public final static int CANCEL_EVENT = 1;
    
    // events raised by controls on gui
    public final static int ZERO_ORDER_PHASING_EVENT = 2;
    public final static int FIRST_ORDER_PHASING_EVENT = 3;
    
    private int m_displayMode = SINGLE_RESPONSE_DISPLAY;
    
    public final static int SINGLE_RESPONSE_DISPLAY = 0;
    public final static int MULTI_RESPONSE_DISPLAY = 1;
    
    // For proper serialization
    private static final long serialVersionUID = 1481295812349393L;
}
