/*
 * File: $Id: ZeroFillParameterDefinitionControl.java,v 1.1.1.1 2006/04/19 09:38:51 hans Exp $
 * History
 * Date: 13.05.2003, 16:27  Authors: Hans Slotboom  
 *
 * Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

/*
 * ZeroFillParameterDefinitionControl.java
 *
 * Created on May 13, 2003, 4:27 PM
 */

package org.spectrim.tdfdfit;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Point;

import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import java.util.Arrays;
import java.util.LinkedList;
import java.util.Iterator;

import javax.swing.BorderFactory;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.SpinnerListModel;
import javax.swing.JSpinner;
import javax.swing.BoxLayout;
import javax.swing.Box;

import javax.swing.JRadioButton;
import javax.swing.ButtonGroup;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 *
 * @author  hans
 */
public class ZeroFillParameterDefinitionControl extends JFrame implements ActionListener, ChangeListener
{
    
    /** 
     * Creates a new instance of ZeroFillParameterDefinitionControl 
     */
    public ZeroFillParameterDefinitionControl( String title, JFrame parent ) 
    {
        
        super( );
        m_parent = parent;
        setTitle( title );
        
        m_contentPanel = new JPanel( );
        m_contentPanel.setLayout( new BorderLayout( ) );
        
        m_controlPanel = new JPanel( );
        m_controlPanel.setLayout( new BoxLayout( m_controlPanel, BoxLayout.X_AXIS ) );
        m_controlPanel.setBorder( BorderFactory.createEmptyBorder( 10, 10, 10, 10 ) );
        m_controlPanel.setPreferredSize( new Dimension( 210, 45 ) );
        m_spinnerLabel = new JLabel( "Zerofill factor " );
        
        Object[] values = { new Float( 0.125f ), new Float( 0.25f ), new Float( 0.5f ), 
                            new Float( 1.0f ), new Float( 2.0f ), 
                            new Float( 4.0f ), new Float( 8.0f ) };
        
        m_spinnerListModel = new SpinnerListModel( Arrays.asList( values ) );
        
        m_spinnerListModel.setValue( new Float(1.0f ) );
        
        m_zerofillSpinner = new JSpinner( m_spinnerListModel );
        m_zerofillSpinner.addChangeListener( this );
        
        m_controlPanel.add( m_spinnerLabel );
        m_controlPanel.add( m_zerofillSpinner );
        
        m_contentPanel.add( m_controlPanel, BorderLayout.NORTH );
        
        m_applyButton = new JButton( "Apply" );
        m_applyButton.addActionListener( this );
        m_applyButton.setToolTipText( "Applies the selected zerofilling factor to the currently loaded dataset" );
        m_cancelButton = new JButton( "Cancel" );
        m_cancelButton.addActionListener( this );
        m_cancelButton.setToolTipText( "Cancels this operation without applying the selected zerofilling factor" );
        
        m_selectionPanel = new JPanel( );
        
        m_selectionPanel.setLayout( new BoxLayout( m_selectionPanel, BoxLayout.X_AXIS ) );
        m_selectionPanel.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );
        m_selectionPanel.add( Box.createGlue( ) );
        m_selectionPanel.add( m_applyButton );
        m_selectionPanel.add( Box.createGlue( ) );
        m_selectionPanel.add( m_cancelButton );
        m_selectionPanel.add( Box.createGlue( ) );
        
        m_contentPanel.add( m_selectionPanel, BorderLayout.SOUTH );
        
        setContentPane( m_contentPanel );
        
        // add window listener necessary to handle the window closing 
        // event properly
        addWindowListener( 
            new WindowAdapter( )
            {
                public void windowClosing( WindowEvent event )
                {
                   dispose( );
                }
            }
        );
        
        pack( );
        if ( m_parent != null )
        {
            // Put the Dialog nicely in the middle of its parent
            Point parentLoc = m_parent.getLocation( );
            Point centerParent = new Point( m_parent.getX( ) + ( m_parent.getWidth( ) / 2 ) - this.getWidth( ) / 2,
                                        m_parent.getY( ) + ( m_parent.getHeight( ) / 2 ) - this.getHeight( ) / 2 );
            setLocation( centerParent );
        }
        setVisible( true );
    }
     
    /**
     * Called whenever an action event occurs
     */
    public void actionPerformed( ActionEvent e ) 
    {
        Object sender = e.getSource( );
        
        // handle responses on cancel button pressed event
        if ( sender == m_cancelButton )
        {
            Iterator it = m_listeners.iterator( );
             while( it.hasNext( ) )
             {
                ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, CANCEL_EVENT, "" ) );
             }
        }
        // handle responses on apply button pressed event
        if ( sender == m_applyButton )
        {
            Iterator it = m_listeners.iterator( );
             while( it.hasNext( ) )
             {
                ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, ACCEPT_EVENT, "" ) );
             }
        }
        
        dispose( );
    }
    
    /**
     * Handles events dealing with a state change of a gui component
     * e.g. JSpinner button is pressed
     */
    public void stateChanged( ChangeEvent e ) 
    {
        Object sender = e.getSource( );
        if ( sender == m_zerofillSpinner )
        {
             Iterator it = m_listeners.iterator( );
             while( it.hasNext( ) )
             {
                ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, CHANGE_EVENT, "" ) );
             }
        }
    }

    /**
     * Adds an actionlistener to the list of action listeners who are notified
     * when something happens within this object
     */
    public void addActionListener( ActionListener e )
    {
	m_listeners.add( e );
    }
    
    /**
     * Returns the currently selected zero filling factor
     */
    public float getSelectedZerofillfactor( )
    {
       return ( (Float)m_zerofillSpinner.getValue( ) ).floatValue( );
    }
    
    
    // Attributes
    private JFrame m_parent;
    
    private JPanel m_contentPanel;      // Contains all other panels
    private JPanel m_controlPanel;      // Contains the controls (and/or control subpanels)
    private JPanel m_selectionPanel;    // Contains Apply/Cancel buttons
    
    private JButton m_applyButton;      // When pressed, sends the apply notification to the registered listeners
    private JButton m_cancelButton;     // When pressed, sends the cansel notification to the registered listeners
    
    private JLabel m_spinnerLabel;
    private SpinnerListModel m_spinnerListModel;
    private JSpinner m_zerofillSpinner;
    
    private int m_pressedButton;
    
    private JPanel m_performActionToCurrentOnlyPanel;
    private JRadioButton m_applyToSelectedInstanceOnly;
    private JRadioButton m_applyToAllInstances;
    
    private ButtonGroup m_applyToButtonGroup = new ButtonGroup( );

    
    // Possible Events
    public final static int CANCEL_EVENT = 0;
    public final static int ACCEPT_EVENT = 1;
    public final static int CHANGE_EVENT = 2;
    
    // Listeners to changes in this class
    private LinkedList<ActionListener> m_listeners = new LinkedList<ActionListener>( );
    
    // For proper serialization
    private static final long serialVersionUID = 9982212346688883L;
}
