/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.spectrim.tdfdfit.quantitativeresults;

/**
 * Class that handles the T1 and T2 relaxation times of a metabolite model.
 * @author slotboom
 */
public class MetaboliteRelaxationTimeProperties
{
    /**
     * Default constructor of a MetaboliteRelaxationTimeProperties object.
     */
    public MetaboliteRelaxationTimeProperties( )
    {

    }

    /**
     * Constructor for metabolite with one relaxation time.
     * @param name
     * @param T1
     * @param T2
     * @param multiplicity
     */
    public MetaboliteRelaxationTimeProperties( String name, double T1, double T2, int multiplicity )
    {
        m_metaboliteName                = name;
        m_longitudinalRelaxationTime    = T1;
        m_transversalRelaxationTime     = T2;
        m_multiplicity                  = multiplicity;
    }

    /**
     * Returns the metabolite name.
     * @return 
     */
    public String getMetaboliteName( )
    {
        return m_metaboliteName;
    }
    /**
     * Returns the first T1 from the vector of T1-values.
     * @return
     */
    public double getT1( )
    {
        return m_longitudinalRelaxationTime;
    }

    /**
     * Returns the first T2 from the vector of T2-values.
     * @return
     */
    public double getT2( )
    {
        return m_transversalRelaxationTime;
    }

    /**
     * Sets the name of the metabolite.
     * @param metaboliteName
     */
    public void setMetaboliteName( String metaboliteName )
    {
        m_metaboliteName = metaboliteName;
    }

    /**
     * Sets the first T1 of the metabolite.
     * @param longitudinalRelaxationTime
     */
    public void setT1( double longitudinalRelaxationTime )
    {
        m_longitudinalRelaxationTime = longitudinalRelaxationTime;
    }

    /**
     * Sets the first T2 of the metabolite.
     * @param transversalRelaxationTime
     */
    public void setT2( double transversalRelaxationTime )
    {
        m_transversalRelaxationTime = transversalRelaxationTime;
    }

    /**
     * The multiplicity is the number of protons that have to be assumed for
     * the metabolite, or part of the metabolite spectrum.
     * @return 
     */
    public int getMultiplicity( )
    {
        return m_multiplicity;
    }
    
    /**
     * The multiplicity is the number of protons that have to be assumed for
     * the metabolite, or part of the metabolite spectrum.
     * @param multiplicity 
     */
    public void setMultiplicity( int multiplicity )
    {
        m_multiplicity = multiplicity;
    }
      
    // Attributes.
    private String m_metaboliteName;

    // Attributes name component, T1, T2 in [sec]
    private double m_longitudinalRelaxationTime = 2.0;
    private double m_transversalRelaxationTime  = 0.3;
    private int    m_multiplicity               = 1;

}
