/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.spectrim.tdfdfit.quantitativeresults;

import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.LinkedList;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * Contains the parameter values of the fitted metabolites.
 *
 * @author slotboom
 */
public class NormalDatabaseMetaboliteParameters
{

    /**
     * Default constructor.
     */
    public NormalDatabaseMetaboliteParameters( )
    {

    }

    /**
     * Default constructor.
     * @param numberOfMetabolites
     * @param metaboliteNames
     * @param rawPeakAreas
     * @param rawFractionCorrectedPeakAreas
     * @param trueTwoCompartmentModelConcentrations
     * @param jMRUITypeConcentrations
     * @param dFreqs
     * @param phases
     * @param lorentzWidths
     * @param lorentzRWidths
     */
    public NormalDatabaseMetaboliteParameters( int      numberOfMetabolites,
                                               LinkedList<String> metaboliteNames,
                                               double[] rawPeakAreas,
                                               double[] rawFractionCorrectedPeakAreas,
                                               double[] trueTwoCompartmentModelConcentrations,
                                               double[] jMRUITypeConcentrations,
                                               double[] dFreqs,
                                               double[] phases,
                                               double[] lorentzWidths,
                                               double[] lorentzRWidths )
    {
        m_numberOfMetabolites = numberOfMetabolites;
        m_metaboliteNames = metaboliteNames;

        m_rawPeakAreas = rawPeakAreas;
        m_rawFractionCorrectedPeakAreas = rawFractionCorrectedPeakAreas;
        m_trueTwoCompartmentModelConcentrations = trueTwoCompartmentModelConcentrations;
        m_jMRUITypeConcentrations = jMRUITypeConcentrations;

        m_dFreq = dFreqs;
        m_phase = phases;
        m_lorentzWidth = lorentzWidths;
        m_lorentzRWidth = lorentzRWidths;
    }


    /**
     * Reads and parses the attributes from a RandomAccesFile.
     * @param in
     */
    void read( RandomAccessFile in )
    {
        try
        {
            //<NormalDatabaseMetaboliteParameters>
            String dummy = in.readUTF( );

            m_numberOfMetabolites = in.readInt( );

            // Now we can instantiate the data objects with correct size
            m_metaboliteNames = new LinkedList<String>();

            m_rawPeakAreas = new double[m_numberOfMetabolites];
            m_rawFractionCorrectedPeakAreas = new double[m_numberOfMetabolites];
            m_trueTwoCompartmentModelConcentrations = new double[m_numberOfMetabolites];
            m_jMRUITypeConcentrations = new double[m_numberOfMetabolites];

            m_dFreq = new double[m_numberOfMetabolites];
            m_phase = new double[m_numberOfMetabolites];
            m_lorentzWidth = new double[m_numberOfMetabolites];
            m_lorentzRWidth = new double[m_numberOfMetabolites];

            //<MetaboliteNames>
            dummy = in.readUTF( );
            for ( int i = 0; i < m_numberOfMetabolites; i++ )
            {
                m_metaboliteNames.add( in.readUTF( ) );
            }

            //</MetaboliteNames>
            dummy = in.readUTF( );

            //<RawPeakAreas>
            dummy = in.readUTF( );

            for ( int i = 0; i < m_numberOfMetabolites; i++ )
            {
                m_rawPeakAreas[i] = in.readDouble( );
            }
            //</RawPeakAreas>
            dummy = in.readUTF( );

            //<FractionCorrectedRawPeakAreas>
            dummy = in.readUTF( );
            
            for ( int i = 0; i < m_numberOfMetabolites; i++ )
            {
                m_rawFractionCorrectedPeakAreas[i] = in.readDouble( );
            }
            //</FractionCorrectedRawPeakAreas>
            dummy = in.readUTF( );
            
            //<TrueTwoCompartmentConcentrations>
            dummy = in.readUTF( );
            for ( int i = 0; i < m_numberOfMetabolites; i++ )
            {
               m_trueTwoCompartmentModelConcentrations[i] = in.readDouble( );
            }
            //</TrueTwoCompartmentConcentrations>
            dummy = in.readUTF( );

            //<JMRUITypeConcentrations>
            dummy = in.readUTF( );
            for ( int i = 0; i < m_numberOfMetabolites; i++ )
            {
               m_jMRUITypeConcentrations[i] = in.readDouble( );
            }
            //</JMRUITypeConcentrations>
            dummy = in.readUTF( );

            //<DFreq>
            dummy = in.readUTF( );
            for ( int i = 0; i < m_numberOfMetabolites; i++ )
            {
               m_dFreq[i] = in.readDouble( );
            }
            //</DFreq>
            dummy = in.readUTF( );
            
            //<Lorentzwidth>
            dummy = in.readUTF( );
            for ( int i = 0; i < m_numberOfMetabolites; i++ )
            {
               m_lorentzWidth[i] = in.readDouble( );
            }
            //</Lorentzwidth>
            dummy = in.readUTF( );


            //<Gausswidth>
            dummy = in.readUTF( );
            for ( int i = 0; i < m_numberOfMetabolites; i++ )
            {
               m_lorentzRWidth[i] = in.readDouble( );
            }
            //</Gausswidth>
            dummy = in.readUTF( );

            //<Phases>
            dummy = in.readUTF( );
            for ( int i = 0; i < m_numberOfMetabolites; i++ )
            {
               m_phase[i] = in.readDouble( );
            }
            //</Phases>
            dummy = in.readUTF( );
            dummy = in.readUTF( );

        }
        catch ( IOException ex )
        {
            Logger.getLogger( NormalDatabaseMetaboliteParameters.class.getName( ) ).log( Level.SEVERE, null, ex );
        }

    }

    /**
     * Writes attributes to RandomAccessFile.
     * @param out
     */
    public void write( RandomAccessFile out )
    {
        try
        {
            out.writeUTF( "<NormalDatabaseMetaboliteParameters>" );
            out.writeInt( m_numberOfMetabolites );

            //names
            out.writeUTF( "<MetaboliteNames>" );
            for ( int i = 0; i < m_numberOfMetabolites; i++ )
            {
                out.writeUTF( m_metaboliteNames.get(i) );
            }
            out.writeUTF( "</MetaboliteNames>" );

            // raw peak areas
            out.writeUTF( "<RawPeakAreas>" );
            for ( int i = 0; i < m_numberOfMetabolites; i++ )
            {
                out.writeDouble( m_rawPeakAreas[i] );
            }
            out.writeUTF( "</RawPeakAreas>" );


            // fraction corrected raw peak areas
            out.writeUTF( "<FractionCorrectedRawPeakAreas>" );
            for ( int i = 0; i < m_numberOfMetabolites; i++ )
            {
                out.writeDouble( m_rawFractionCorrectedPeakAreas[i] );
            }
            out.writeUTF( "</FractionCorrectedRawPeakAreas>" );

            // true two compartment corrected concentrations
            out.writeUTF( "<TrueTwoCompartmentConcentrations>" );
            for ( int i = 0; i < m_numberOfMetabolites; i++ )
            {
                out.writeDouble( m_trueTwoCompartmentModelConcentrations[i] );
            }
            out.writeUTF( "</TrueTwoCompartmentConcentrations>" );

            // jmrui type concentrations
            out.writeUTF( "<JMRUITypeConcentrations>" );
            for ( int i = 0; i < m_numberOfMetabolites; i++ )
            {
                out.writeDouble( m_jMRUITypeConcentrations[i] );
            }
            out.writeUTF( "</JMRUITypeConcentrations>" );

            // jmrui type dfreq
            out.writeUTF( "<DFreq>" );
            for ( int i = 0; i < m_numberOfMetabolites; i++ )
            {
                out.writeDouble( m_dFreq[i] );
            }
            out.writeUTF( "</DFreq>" );

            // jmrui type lorentzwidth
            out.writeUTF( "<Lorentzwidth>" );
            for ( int i = 0; i < m_numberOfMetabolites; i++ )
            {
                out.writeDouble( m_lorentzWidth[i] );
            }
            out.writeUTF( "</Lorentzwidth>" );

            // jmrui type gausswidth
            out.writeUTF( "<LorentzRwidth>" );
            for ( int i = 0; i < m_numberOfMetabolites; i++ )
            {
                out.writeDouble( m_lorentzRWidth[i] );
            }
            out.writeUTF( "</LorentzRwidth>" );

            // jmrui type phase
            out.writeUTF( "<Phases>" );
            for ( int i = 0; i < m_numberOfMetabolites; i++ )
            {
                out.writeDouble( m_phase[i] );
            }
            out.writeUTF( "</Phases>" );

            out.writeUTF( "</NormalDatabaseMetaboliteParameters>" );
        }
        catch ( IOException ex )
        {
            Logger.getLogger( NormalMetaboliteValuesAndSpectrumDatabase.class.getName( ) ).log( Level.SEVERE, null, ex );
        }
    }

    /**
     * Returns the number of metabolites in the model.
     * @return
     */
    public int getNumberOfMetabolites( )
    {
        return m_numberOfMetabolites;
    }

    /**
     * Returns the array of metabolite names.
     * @return
     */
    public LinkedList<String> getMetaboliteNames( )
    {
        return m_metaboliteNames;
    }

    /**
     * Returns the array of peak area parameters.
     * @return
     */
    public double[] getRawPeakAreas( )
    {
        return m_rawPeakAreas;
    }

    /**
     * Returns the array of raw fraction corrected peak areas.
     * @return
     */
    public double[] getRawFractionCorrectedPeakAreas( )
    {
        return m_rawFractionCorrectedPeakAreas;
    }

    /**
     * Returns the array of getTrueTwoCompartmentModelConcentrations.
     * @return
     */
    public double[] getTrueTwoCompartmentModelConcentrations( )
    {
        return m_trueTwoCompartmentModelConcentrations;
    }

    /**
     * Returns the array of getTrueTwoCompartmentModelConcentrations.
     * @return
     */
    public double[] getJMRUITypeConcentrations( )
    {
        return m_jMRUITypeConcentrations;
    }

    /**
     * Returns the array of delta frequency.
     * @return
     */
    public double[] getDFreqs( )
    {
        return m_dFreq;
    }

    /**
     * Returns the array of phase.
     * @return
     */
    public double[] getPhases( )
    {
        return m_phase;
    }

    /**
     * Returns the array of Lorentzian widths.
     * @return
     */
    public double[] getLorentzWidths( )
    {
        return m_lorentzWidth;
    }

    /**
     * Returns the array of LorentzianR widths.
     * @return
     */
    public double[] getLorentzRWidths( )
    {
        return m_lorentzRWidth;
    }

    // Attributes of the 
    private int      m_numberOfMetabolites;
    private LinkedList<String> m_metaboliteNames;

    private double[] m_rawPeakAreas;
    private double[] m_rawFractionCorrectedPeakAreas;
    private double[] m_trueTwoCompartmentModelConcentrations;
    private double[] m_jMRUITypeConcentrations;

    private double[] m_dFreq;
    private double[] m_phase;
    private double[] m_lorentzWidth;
    private double[] m_lorentzRWidth;

    /**
     * Sets the average averageDFreqs array
     * @param averageDFreqs
     */
    public void setDFreqs( double[] averageDFreqs )
    {
        m_dFreq = new double[ averageDFreqs.length ];
        m_numberOfMetabolites = averageDFreqs.length;

        System.arraycopy(averageDFreqs, 0, m_dFreq, 0, averageDFreqs.length);
    }

    /**
     * Set JMRUI-type conform concentration values.
     * @param averageJMRUITypeConcentrations 
     */
    public void setJMRUITypeConcentrations( double[] averageJMRUITypeConcentrations )
    {
        m_jMRUITypeConcentrations = new double[ averageJMRUITypeConcentrations.length ];
        m_numberOfMetabolites = averageJMRUITypeConcentrations.length;

        System.arraycopy(averageJMRUITypeConcentrations, 0, m_jMRUITypeConcentrations, 0, averageJMRUITypeConcentrations.length);
    }

    /**
     * Sets the average Lorentz width values.
     * @param averageLorentzRWidths 
     */
    public void setLorentzRWidths( double[] averageLorentzRWidths )
    {
        m_lorentzRWidth = new double[ averageLorentzRWidths.length ];
        m_numberOfMetabolites = averageLorentzRWidths.length;

        System.arraycopy(averageLorentzRWidths, 0, m_lorentzRWidth, 0, averageLorentzRWidths.length);
    }

    /**
     * Sets the Lorentz width values. 
     * @param averageLorentzWidths
     */
    public void setLorentzWidths( double[] averageLorentzWidths )
    {
        m_lorentzWidth = new double[ averageLorentzWidths.length ];
        m_numberOfMetabolites = averageLorentzWidths.length;

        System.arraycopy(averageLorentzWidths, 0, m_lorentzWidth, 0, averageLorentzWidths.length);
    }

    /**
     * Sets the average phase values.
     * @param averagePhases 
     */
    public void setPhases( double[] averagePhases )
    {
        m_phase = new double[ averagePhases.length ];
        m_numberOfMetabolites = averagePhases.length;

        System.arraycopy(averagePhases, 0, m_phase, 0, averagePhases.length);
    }
    
    
    /**
     * Sets and use the true two compartment model concentrations. 
     * @param averageTrueTwoCompartmentModelConcentrations 
     */
    public void setTrueTwoCompartmentModelConcentrations( double[] averageTrueTwoCompartmentModelConcentrations )
    {
        m_trueTwoCompartmentModelConcentrations = new double[ averageTrueTwoCompartmentModelConcentrations.length ];
        m_numberOfMetabolites = averageTrueTwoCompartmentModelConcentrations.length;

        System.arraycopy(averageTrueTwoCompartmentModelConcentrations, 0, m_trueTwoCompartmentModelConcentrations, 0, averageTrueTwoCompartmentModelConcentrations.length);
    }

    /**
     * Sets the metabolite names object.
     * @param metaboliteNames
     */
    public void setMetaboliteNames( LinkedList<String> metaboliteNames )
    {
        if ( metaboliteNames != null )
        {
            m_metaboliteNames = new LinkedList<>( );
            m_numberOfMetabolites = metaboliteNames.size( );

            for (String metaboliteName : metaboliteNames) 
            {
                m_metaboliteNames.add(metaboliteName);
            }
        }
    }

    public void setRawFractionCorrectedPeakAreas( double[] rawAverageFractionCorrectedPeakAreas )
    {
        m_rawFractionCorrectedPeakAreas = new double[ rawAverageFractionCorrectedPeakAreas.length ];
        m_numberOfMetabolites = rawAverageFractionCorrectedPeakAreas.length;

        System.arraycopy(rawAverageFractionCorrectedPeakAreas, 0, m_rawFractionCorrectedPeakAreas, 0, rawAverageFractionCorrectedPeakAreas.length);
    }

    public void setRawPeakAreas( double[] rawPeakAreas )
    {
        m_rawPeakAreas = new double[ rawPeakAreas.length ];
        m_numberOfMetabolites = rawPeakAreas.length;

        System.arraycopy(rawPeakAreas, 0, m_rawPeakAreas, 0, rawPeakAreas.length);
    }

}