/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.spectrim.tdfdfit.quantitativeresults;

import org.spectrim.dicom.DicomImageData;
import org.spectrim.dicom.DicomImageSeriesData;
import org.spectrim.dicom.FileInfo;
import org.spectrim.dicom.SpectroscopicVoxelProjectionDicomSliceCanvasPolyDraw;
import org.spectrim.imagegraphicobjects.Spectroscopic3DVoxel;

import java.awt.Color;
import java.awt.Dimension;
import java.io.File;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import javax.swing.BoxLayout;
import javax.swing.JFrame;
import javax.swing.JPanel;
import org.spectrim.tdfdfit.SingleVoxelMeasuredSpectralData;
import org.spectrim.dicom.DicomDirectory;


/**
 *
 * Special panel to display the reference images on which the 
 * spectroscopy was planned.
 *
 * @author slotboom
 */
public class SVSReferenceImagesPanel extends JPanel
{


    /**
     * Creates a new instance of SVSReferenceImagesPanel for displaying those
     * images on which the examination was planned.
     *
     * @param parent
     * @param refImage0
     * @param refImage1
     * @param refImage2
     * @param rawSpectroscopyFileName
     */
    public SVSReferenceImagesPanel( JFrame parent,
                                    String refImage0,
                                    String refImage1,
                                    String refImage2,
                                    DicomDirectory dicomDir,
                                    String rawSpectroscopyFileName )
    {

        setBackground( Color.BLACK );
        setPreferredSize( new Dimension( 1024, 256 ) );
        setMinimumSize( new Dimension( 1024, 256 ) );

        DicomImageData d_ima_data = new DicomImageData( rawSpectroscopyFileName );

        LinkedHashMap ssd = d_ima_data.getSiemensSpecificDataFromFile( );

        // we are dealing with an Single Voxel Spectroscopy measurement
        FileInfo fi = d_ima_data.getHeader( );
        SingleVoxelMeasuredSpectralData specData =  new SingleVoxelMeasuredSpectralData( fi.getNonImageData( ) );
        specData.setDicomHeaderInfo( fi );
        specData.setSiemensSpecificDicomHeaderInfo( d_ima_data.getSiemensSpecificDataFromFile( ) );

        m_refImage0 = refImage0;
        m_refImage1 = refImage1;
        m_refImage2 = refImage2;

        LinkedList<String> fileNames = new LinkedList<String>( );
        fileNames.add( rawSpectroscopyFileName );
        
        m_specVoxel = specData.getSpectroscopic3DVoxel( );

        setLayout( new BoxLayout( this, BoxLayout.X_AXIS ) );

        if ( ( m_refImage0 != null ) && ( new File( m_refImage0 ).exists( ) ) )
        {
            // First reference image
            LinkedList<String> fileReferencesLinkedList0 = new LinkedList<String>( );
            fileReferencesLinkedList0.add( refImage0 );

            DicomImageSeriesData imaDat0 = new DicomImageSeriesData( fileReferencesLinkedList0 );
            LinkedHashMap hashMap0 = imaDat0.getDicomImageDataObject( 0 ).getSiemensSpecificDataFromFile( );
            FileInfo finfo0 = imaDat0.getFileInfo( 0 );
            
            m_refIma0Displayer = new SpectroscopicVoxelProjectionDicomSliceCanvasPolyDraw( parent,
                                                                                           m_specVoxel,
                                                                                           hashMap0,
                                                                                           finfo0 );
            m_refIma0Displayer.setDicomImages( imaDat0 );
            m_refIma0Displayer.setFileInfo( finfo0 );
            
            m_refIma0Displayer.setPreferredSize( new Dimension( 256, 256 ) );
            m_refIma0Displayer.displayDicomImage( 0 );
            m_refIma0Displayer.setVisible( true );
            LinkedList<String> dicomRefs = new LinkedList<String>( );
            dicomRefs.add( refImage0 );        
        }
        else
        {
             m_refIma0Displayer = new SpectroscopicVoxelProjectionDicomSliceCanvasPolyDraw( parent );
             m_refIma0Displayer.setPreferredSize( new Dimension( 256, 256 ) );
        }

        add( m_refIma0Displayer );

        // ======================================================================
        // Second image
        // ======================================================================
        if ( ( m_refImage1 != null ) && ( new File( m_refImage1 ).exists( ) ) )
        {
            LinkedList<String> fileReferencesLinkedList1 = new LinkedList<>( );
            fileReferencesLinkedList1.add( refImage1 );

            DicomImageSeriesData imaDat1 = new DicomImageSeriesData( fileReferencesLinkedList1 );
            LinkedHashMap hashMap1 = imaDat1.getDicomImageDataObject( 0 ).getSiemensSpecificDataFromFile( );
            FileInfo finfo1 = imaDat1.getFileInfo( 0 );

            m_refIma1Displayer = new SpectroscopicVoxelProjectionDicomSliceCanvasPolyDraw( parent,
                                                                                           m_specVoxel,
                                                                                           hashMap1,
                                                                                           finfo1 );

            m_refIma1Displayer.setDicomImages( imaDat1 );
            m_refIma1Displayer.setFileInfo( finfo1 );
            m_refIma1Displayer.setPreferredSize( new Dimension( 256, 256 ) );
            m_refIma1Displayer.displayDicomImage( 0 );
            m_refIma1Displayer.setVisible( true );

        }
        else
        {
             m_refIma1Displayer = new SpectroscopicVoxelProjectionDicomSliceCanvasPolyDraw( parent );
             m_refIma1Displayer.setPreferredSize( new Dimension( 256, 256 ) );
        }

        add( m_refIma1Displayer );

        // ======================================================================
        // Third image
        // ======================================================================
        if ( ( m_refImage2 != null ) && ( new File( m_refImage2 ).exists( ) ) )
        {
            LinkedList<String> fileReferencesLinkedList2 = new LinkedList<String>( );
            fileReferencesLinkedList2.add( refImage2 );

            DicomImageSeriesData imaDat2 = new DicomImageSeriesData( fileReferencesLinkedList2 );
            LinkedHashMap hashMap2 = imaDat2.getDicomImageDataObject( 0 ).getSiemensSpecificDataFromFile( );
            FileInfo finfo2 = imaDat2.getFileInfo( 0 );

            m_refIma2Displayer = new SpectroscopicVoxelProjectionDicomSliceCanvasPolyDraw( parent,
                                                                                           m_specVoxel,
                                                                                           hashMap2,
                                                                                           finfo2 );
            
            // m_refIma2Displayer.setImage( imaDat2.getSeriesPixelData(), 256, 256, 0, finfo2, hashMap2 );
            
            m_refIma2Displayer.setDicomImages( imaDat2 );
            m_refIma2Displayer.setFileInfo( finfo2 );
            
            m_refIma2Displayer.setPreferredSize( new Dimension( 256, 256 ) );
            
            m_refIma2Displayer.setVisible( true );
            m_refIma2Displayer.displayDicomImage( 0 );
            
            // m_refIma2Displayer.setDicomImages( imaDat2 );
        }
        else
        {
            m_refIma2Displayer = new SpectroscopicVoxelProjectionDicomSliceCanvasPolyDraw( parent );
            m_refIma2Displayer.setPreferredSize( new Dimension( 256, 256 ) );
        }

        add( m_refIma2Displayer );
        
    }

    /**
     * Set the zoom factor of the images and forces an redraw of the images.
     */
    void setImageMagnifications( )
    {
        
        double zoom = 1.0 / ( m_refIma0Displayer.getWidth( ) / 256 );
        m_refIma0Displayer.setZoomNoDraw( (float)zoom );

        zoom = 1.0 / ( m_refIma1Displayer.getWidth( ) / 256 );
        m_refIma1Displayer.setZoomNoDraw( (float)zoom );

        zoom = 1.0 / ( m_refIma2Displayer.getWidth( ) / 256 );
        m_refIma2Displayer.setZoomNoDraw( (float)zoom );

        // m_refIma0Displayer.paintComponent( m_refIma0Displayer.getGraphics( ) );
        // m_refIma1Displayer.paintComponent( m_refIma1Displayer.getGraphics( ) );
        // m_refIma2Displayer.paintComponent( m_refIma2Displayer.getGraphics( ) );

        m_refIma0Displayer.displayDicomImage( 0 );
        m_refIma1Displayer.displayDicomImage( 0 );
        m_refIma2Displayer.displayDicomImage( 0 );
        
        m_refIma0Displayer.drawImage( true );
        m_refIma1Displayer.drawImage( true );
        m_refIma2Displayer.drawImage( true );
       
//        m_refIma0Displayer.repaint( );
//        m_refIma1Displayer.repaint( );
//        m_refIma2Displayer.repaint( );;
    }


    // Private attributes
    private final String m_refImage0;
    private final String m_refImage1;
    private final String m_refImage2;

    private final Spectroscopic3DVoxel m_specVoxel;
    private final SpectroscopicVoxelProjectionDicomSliceCanvasPolyDraw m_refIma0Displayer;
    private final SpectroscopicVoxelProjectionDicomSliceCanvasPolyDraw m_refIma1Displayer;
    private final SpectroscopicVoxelProjectionDicomSliceCanvasPolyDraw m_refIma2Displayer;
 
}
