/*
 * File: $Id: DicomFileSelectionControl.java,v 1.1.1.1 2006/04/19 09:38:47 hans Exp $
 * History
 * Date: 12. April 2003, 13:43 Hans Slotboom  Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */
  
package org.spectrim.tdfdfit.waterreferencemodelling;

import java.awt.Dimension;
import java.awt.event.MouseListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JTree;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;
import javax.swing.JPopupMenu;
import javax.swing.JMenuItem;

import javax.swing.JScrollPane;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.table.DefaultTableModel;

import javax.swing.BoxLayout;
import javax.swing.BorderFactory;

import java.util.TreeMap;

import java.util.LinkedList;
import java.util.Iterator;
import java.util.Collection;
import java.util.Enumeration;
import javax.swing.tree.TreeNode;
import org.spectrim.dicom.DicomDirectory;


/**
 * 
 * Control allowing the use to select a complete image series
 * or some file of a image series.
 * 
 * @author hans
 */

public class TwoCompartmentWaterReferenceDicomFileSelectionControl extends JPanel
{
    /** 
     * Creates a new instance of 
     * TwoCompartmentWaterReferenceDicomFileSelectionControl. 
     */
    public TwoCompartmentWaterReferenceDicomFileSelectionControl( ) 
    {
        m_treePanel = new JPanel( );
        m_treePanel.setLayout( new BoxLayout( m_treePanel, BoxLayout.X_AXIS ) );
        m_treePanel.setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ) );
         
        createNewTreeControl( );
        
        m_tablePanel = new JPanel( );
        m_tablePanel.setLayout( new BoxLayout( m_tablePanel, BoxLayout.X_AXIS ) );
        m_tablePanel.setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ) );
        
//        Object[][] cellData = { { "", "", "", "", "", "", "" } };
//        String[] columnNames = { "<HTML><B>File name</B></HTML>",
//                                 "<HTML><B>Type</B></HTML>",
//                                 "<HTML><B>Instance number</B></HTML>",
//                                 "<HTML><B>Slice Position [mm]</B></HTML>",
//                                 "<HTML><B>FOV [mm]x[mm]</B></HTML>",
//                                 "<HTML><B>TE [ms]</B></HTML>",
//                                 "<HTML><B>TR [ms]</B></HTML>"
//                               };
//        
//        m_dicomImageSeriesTable = new JTable( cellData, columnNames );
        
        m_dicomImageSeriesTable = new TwoCompartmentWaterReferenceDicomFileSelectionTable( );
        
        // JTree does not provide a JScrollPane automatically so we have to 
        // add it ourselves
        m_treeScrollPane = new JScrollPane( );
        m_treeScrollPane.setMinimumSize( new Dimension( 200, 200 ) );
        m_treeScrollPane.setPreferredSize( new Dimension( 200, 200 ) );
        m_treeScrollPane.getViewport( ).setView( m_dicomDirectoryTree );
        
        m_treePanel.add( m_treeScrollPane );
        
        // JList does not provide a scroll list automatically so we have to 
        // add it ourselves.
        m_listScrollPane = new JScrollPane( );
        m_listScrollPane.getViewport( ).setView( m_dicomImageSeriesTable );        
        m_listScrollPane.setPreferredSize( new Dimension( 600, 200 ) );
        
        m_tablePanel.add( m_listScrollPane );
        
        setLayout( new BoxLayout( this, BoxLayout.X_AXIS ) );

        setMinimumSize( new Dimension( 810, 210 ) );
        setPreferredSize( new Dimension( 810, 210 ) );
        
        setBorder( BorderFactory.createCompoundBorder(
                   BorderFactory.createLoweredBevelBorder( ),
                   BorderFactory.createTitledBorder( "Dicom file/series selector" ) ) );
        
        add( m_treePanel );
        add( m_tablePanel );
    }
    
    /**
     * Sets the series names present in the selected DICOM directory.
     * @param dicomDir
     */
    public void setDicomDir( DicomDirectory dicomDir )
    {
        m_dicomDir = dicomDir;
        setDicomDirectoryTree( );
        m_directoryIsMounted = true;
    }
    
    /**
     * Redefines the selection behavior of the JTree.
     * @param multiSelectable
     */
    public void setTreeMultiSelectable( boolean multiSelectable )
    {
        m_dicomDirectoryTree.getSelectionModel( ).setSelectionMode( TreeSelectionModel.DISCONTIGUOUS_TREE_SELECTION );
    }
    
    /**
     * Creates the tree control.
     */
    private void createNewTreeControl( )
    {
        m_treeRoot = new DefaultMutableTreeNode( "Mounted Dicom Directory" );
        
        m_dicomDirectoryTree = null;
        
        m_dicomDirectoryTree = new JTree( m_treeRoot );
        // m_dicomDirectoryTree.getSelectionModel( ).setSelectionMode( TreeSelectionModel.SINGLE_TREE_SELECTION );
        m_dicomDirectoryTree.getSelectionModel( ).setSelectionMode( TreeSelectionModel.CONTIGUOUS_TREE_SELECTION );
              
        // Adds a mouse listener for the 
        m_treeMouseListener = new MouseAdapter( ) 
        {
            @Override
            public void mousePressed( MouseEvent e )
            {
                int selRow = m_dicomDirectoryTree.getRowForLocation( e.getX( ), e.getY( ) );
                TreePath selPath = m_dicomDirectoryTree.getPathForLocation( e.getX( ), e.getY( ) );

                if ( selPath != null )
                {
                    DefaultMutableTreeNode node = (DefaultMutableTreeNode)selPath.getLastPathComponent( );
                
                    if ( ( selRow != -1 ) && ( !node.isRoot( ) ) )
                    {
                        if ( e.getClickCount( ) == 1 )
                        {
                            updateDicomImageSeriesList( selRow, selPath );
                            notifyListeners( TREE_ENTRY_SELECTED_EVENT, TREE_ENTRY_SELECTED_CMD );
                        }
                        else if( e.getClickCount( ) == 2 )
                        {
                            notifyListeners( TREE_ENTRY_DOUBLE_CLICKED_EVENT, TREE_ENTRY_DBL_CLICKED_CMD );
                        }
                    }
                }
                
                if ( e.isPopupTrigger( ) )
                {
                    if ( m_directoryIsMounted == true )
                    {
                        // Enable all other popup menu elements
                        for ( int i = 0; i < m_popupMenu.getComponentCount( ); i++ )
                        {
                            if ( m_popupMenu.getComponent( i ) instanceof JMenuItem )
                            {
                                ( (JMenuItem)m_popupMenu.getComponent( i ) ).setEnabled( true );
                            }
                        }
                    }
                    m_popupMenu.show( (JComponent)e.getSource( ), e.getX( ), e.getY( ) );
                }
            }
            // Called whenever the right mouse button is pressed
            @Override
            public void mouseReleased( MouseEvent e )
            {
                if ( e.isPopupTrigger( ) )
                {  
                    if ( m_directoryIsMounted == true )
                    {
                        // Enable all other popup menu elements
                        for ( int i = 0; i < m_popupMenu.getComponentCount( ); i++ )
                        {
                            if ( m_popupMenu.getComponent( i ) instanceof JMenuItem )
                            {
                                ( (JMenuItem)m_popupMenu.getComponent( i ) ).setEnabled( true );
                            }
                        }
                    }
                    m_popupMenu.show( (JComponent)e.getSource( ), e.getX( ), e.getY( ) );
                }
            }
        };
        
        m_dicomDirectoryTree.addMouseListener( m_treeMouseListener );
    }
    
    /**
     * Sets the series names present in the selected DICOM directory.
     */
    public void clearDicomDir( )
    {
        m_seriesNames.clear( );
        
        Object[][] cellData = { { "", "", "", "", "", "", "" } };
        
        m_cellData = cellData;
        
        String[] columnNames = { "<HTML><B>File name</B></HTML>",
                                 "<HTML><B>Type</B></HTML>",
                                 "<HTML><B>Instance number</B></HTML>",
                                 "<HTML><B>Slice Position [mm]</B><HTML>",
                                 "<HTML><B>FOV [mm]x[mm]</B><HTML>",
                                 "<HTML><B>TE [ms]</B><HTML>",
                                 "<HTML><B>TR [ms]</B><HTML>"
                               };
        
        m_tableModel = new DefaultTableModel( cellData, columnNames );
        
        m_dicomImageSeriesTable.setModel( m_tableModel );
        
        m_dicomImageSeriesTable.updateUI( );
        
        m_treeRoot.removeAllChildren( );
        m_dicomDirectoryTree.updateUI( );
        
    }
    
    /**
     * Adds elements to the DICOM study -> seriesN tree.
     */
    private synchronized void setDicomDirectoryTree( )
    {
        m_dicomDir.resolveSeriesNumberToSeriesDescriptionMap( );
        m_seriesNames = m_dicomDir.getSeriesDescriptions( );
        
        Collection smap = m_seriesNames.keySet( );
        Iterator it = smap.iterator( );
        
        for ( int i = 0; i < m_seriesNames.size( ); i++ )
        {
            Integer intje = (Integer)it.next( );
            String nodeDescription = intje + " - " + m_seriesNames.get( intje );
            m_treeRoot.add( new DefaultMutableTreeNode(  nodeDescription ) );
        }

        // m_dicomDirectoryTree = new JTree( m_treeRoot );
        
        expandAll( m_dicomDirectoryTree, true );
        m_dicomDirectoryTree.updateUI( );
    }

    /**
     * If expand is true, expands all nodes in the tree;
     * otherwise, collapses all nodes in the tree.
     * @param tree
     * @param expand
     */
    public void expandAll( JTree tree, boolean expand )
    {
        TreeNode root = (TreeNode)tree.getModel( ).getRoot( );

        // Traverse tree from root
        expandAll( tree, new TreePath(root), expand );
    }

    /**
     * Does the actual expansion of the list.
     * @param tree
     * @param parent
     * @param expand
     */
    private void expandAll( JTree tree, TreePath parent, boolean expand )
    {
        // Traverse children
        TreeNode node = (TreeNode)parent.getLastPathComponent( );
        if ( node.getChildCount( ) >= 0 )
        {
            for ( Enumeration e=node.children( ); e.hasMoreElements( ); )
            {
                TreeNode n = (TreeNode)e.nextElement( );
                TreePath path = parent.pathByAddingChild( n );
                expandAll( tree, path, expand );
            }
        }

        // Expansion or collapse must be done bottom-up
        if ( expand )
        {
            tree.expandPath( parent );
        }
        else
        {
            tree.collapsePath( parent );
        }
    }

    /**
     * Updates the JList component that indicates the images in the selected
     * Image series.
     */
    private void updateDicomImageSeriesList( int selRow, TreePath selPath )
    {
        
        DefaultMutableTreeNode dmtn = (DefaultMutableTreeNode)selPath.getLastPathComponent( );
        if ( !dmtn.isRoot( ) )
        {
            m_selectedNodeName = selPath.getLastPathComponent( ).toString( );
            selPath.getLastPathComponent( );
            int beginSeriesNameIndex = m_selectedNodeName.indexOf( " " );
            m_selectedSeriesNumber = new Integer( m_selectedNodeName.substring( 0, beginSeriesNameIndex ) );
            // eliminate the - sign
            m_selectedNodeName = m_selectedNodeName.substring( beginSeriesNameIndex + 3 );
            
//            System.out.println( "Selected Series Name : " + m_selectedNodeName );
//            System.out.println( "Selected Series Number : " + m_selectedSeriesNumber.intValue( ) );
           
            m_fileSelectedSeriesFileReferences = m_dicomDir.getImageFileReferencesOfSeriesNumber( m_selectedSeriesNumber.intValue( ) );
            m_dicomImageSeriesTable.setListData( m_fileSelectedSeriesFileReferences );
        }
        else
        {
            if ( m_fileSelectedSeriesFileReferences != null )
            {
                m_fileSelectedSeriesFileReferences.clear( );
                m_dicomImageSeriesTable.setListData( m_fileSelectedSeriesFileReferences );
            }
        }
    }
    
    /**
     * Returns the selected series name of the selected treenode.
     * @return 
     */
    public String getSelectedSeriesName( )
    {
        return m_selectedNodeName;
    }
    
    /**
     * Returns the selected series name of the selected treenode.
     * @return 
     */
    public int getSelectedSeriesNumber( )
    {
        return m_selectedSeriesNumber;
    }
    
    
//    /**
//     * Sets the cell values of the list. Since it is a public method it can be 
//     * called from a other object
//     */
//    public void setListData( TreeSet<String> fileReferences ) 
//    {
//        
//        m_fileSelectedSeriesFileReferences = fileReferences;
//        
//        m_cellData = new Object[ m_fileSelectedSeriesFileReferences.size( ) ][ 7 ];
//                
//        // reimplement the constructor code
//        int counter = 0;
//        Iterator it = fileReferences.iterator( );
//        while ( it.hasNext( ) )
//        {
//            String fileName = (String)it.next( );
//            DicomImageData data = new DicomImageData( fileName );
//            FileInfo finfo = data.getHeader( );
//            
//            m_cellData[counter][0] = fileName;
//            
//            if ( finfo.isImageData( ) == true )
//            {
//                m_cellData[counter][1] = "Image";
//                m_isImageSeries = true;
//            }
//            else
//            {
//                m_cellData[counter][1] = "Spectrum";
//                m_isImageSeries = false;
//            }
//            
//            m_cellData[counter][2] = new Integer( finfo.getInstanceNumber( ) );
//            m_cellData[counter][3] = new Double( finfo.getSlicePosition( ) ).toString( );
//            String fov = new Double( finfo.getPixelWidth( ) * finfo.getImageWidth( ) ).toString( );
//            fov += " x " + ( finfo.getPixelHeight( ) * finfo.getImageHeight( ) );
//            m_cellData[counter][4] = fov;
//            m_cellData[counter][5] = new Integer( finfo.getEchoTime( ) ).toString( );
//            m_cellData[counter][6] = new Integer( finfo.getTR( ) ).toString( );
//            counter++;
//        }
//         
//        String[] columnNames = { "<HTML><B>File name</B></HTML>",
//                                 "<HTML><B>Type</B></HTML>",
//                                 "<HTML><B>Instance number</B></HTML>",
//                                 "<HTML><B>Slice Position [mm]</B><HTML>",
//                                 "<HTML><B>FOV [mm]x[mm]</B><HTML>",
//                                 "<HTML><B>TE [ms]</B><HTML>",
//                                 "<HTML><B>TR [ms]</B><HTML>"
//                                };
//                                 
//        m_tableModel = new DefaultTableModel( m_cellData, columnNames );
//        
//        m_dicomImageSeriesTable.setModel( m_tableModel );
//        
//        m_dicomImageSeriesTable.selectAll( );
//        
//        m_dicomImageSeriesTable.updateUI( );
//    }
    
    
    /**
     * Returns the file references of all selected files from the table.
     * @return
     */
    public LinkedList<String> getSelectedFileReferencesFromTable( )
    {        
        return m_dicomImageSeriesTable.getSelectedFileReferencesFromTable( );
    }
    
    
    /**
     * Notify all listeners that a change has taken place in this object.
     * @param evType
     * @param cmd
     */ 
    protected void notifyListeners( int evType, String cmd )
    {
        Iterator it = m_listeners.iterator( );
        // notify registered listeners
        while( it.hasNext( ) )
        {
            ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, evType, cmd ) );
        }
    }
     
    /**
     * Adds an ActionListener to the list of action listeners who are notified
     * when something happens in this component.
     * @param e
     */
    public void addActionListener( ActionListener e )
    {
        m_listeners.add( e );
    }

    /**
     * Returns the TreeSet contain the file references in the selected series.
     * @return 
     */
    public LinkedList<String> getSelectedSeriesFileReferences( )
    {
        return m_dicomImageSeriesTable.getSelectedSeriesFileReferences( );
    }
    
    /**
     * Returns whether the selected series contains image data or
     * not.
     * @return 
     */
    public boolean isImageSeries( )
    {
        return m_dicomImageSeriesTable.isImageSeries( );
    }
    
    
    /**
     * Returns the number of different slice positions in the selected series.
     * @return 
     */
    public int getNumberOfDifferentSlicePositions( )
    {
        return m_dicomImageSeriesTable.getNumberOfDifferentSlicePositions( );
    }
    
    /**
     * Returns the number of series in the selected DICOM directory having 
     * the same series description "seriesDescription".
     * @param seriesDescription
     * @return 
     */
    public int getNumberOfSeries( String seriesDescription )
    {
        int numberOfSeries = 0;
        m_seriesNames = m_dicomDir.getSeriesDescriptions( );
        
        Collection smap = m_seriesNames.keySet( );
        Iterator it = smap.iterator( );
        
        for ( int i = 0; i < m_seriesNames.size( ); i++ )
        {
            Integer intje = (Integer)it.next( );
            if ( ((String)m_seriesNames.get( intje )).compareTo( seriesDescription ) == 0 )
            {
                numberOfSeries++;
            }
        }
        
        return numberOfSeries;
    }
    
    /**
     * Returns an array of all selected series numbers.
     * @return 
     */
    public int[] getSelectedSeriesNumbers( )
    {
        TreePath[] paths = this.m_dicomDirectoryTree.getSelectionPaths( );
        int[] selSeriesNumbers = new int[paths.length];
        for ( int i = 0; i < paths.length; i++ )
        {
            String str = paths[i].getLastPathComponent( ).toString( );
            int beginSeriesNameIndex = str.indexOf( " " );
            selSeriesNumbers[i] = ( new Integer( str.substring( 0, beginSeriesNameIndex ) ) ).intValue( );
        }
        
        return selSeriesNumbers;
    }

    /**
     * Sets the signals for the 2 compartment model.
     * @return 
     */
    public LinkedList<String> getFileReferencesOfAllSelectedSpectroSeries( )
    {

        m_allSpectroscopyFileReferences = new LinkedList<>( );

        // m_xySignalsPlot.deleteAllCurves( );

        int[] selectedSeries = getSelectedSeriesNumbers( );

        //m_svsDataSeries = new SingleVoxelMeasuredSpectralData[ selectedSeries.length ];
        //m_svsDataSeriesTEs = new double[ selectedSeries.length ];

        //m_numberOf2ComponentWaterReferences = selectedSeries.length;

        for ( int i = 0 ; i < selectedSeries.length; i++ )
        {
            LinkedList<String> selectedSeriesFileReferences;
            selectedSeriesFileReferences = m_dicomDir.getImageFileReferencesOfSeriesNumber( selectedSeries[i] );

            //DicomImageSeriesData dicomData = new DicomImageSeriesData( selectedSeriesFileReferences );
            //dicomData.orderBySlicePosition( );

            Iterator it = selectedSeriesFileReferences.iterator( );

            while ( it.hasNext( ) )
            {
                String fileRef = (String)it.next( );
                m_allSpectroscopyFileReferences.add( fileRef );
                if ( selectedSeries.length == 1 )
                {
                    m_singleWaterFileReference = fileRef;
                }
            }
        }

        return m_allSpectroscopyFileReferences;
    }
    
    /**
     * Returns the TreeSet contain the file references in the selected series.
     * @return 
     */
    public LinkedList<String> getSelected2CompartmentWaterReferences( )
    {
        return m_allSpectroscopyFileReferences;
    }
    
    /**
     * Returns the TreeSet contain the file references in the selected series.
     * @return 
     */
    public String getSelectedSingleWaterReference( )
    {
        return m_singleWaterFileReference;
    }    
    
    // Member variables
    // Listeners to textfield changes
    private final LinkedList<ActionListener> m_listeners = new LinkedList<>( );
    
    private String m_selectedNodeName;
    private Integer m_selectedSeriesNumber;
    // Two sub panels to put the treeview and the listview upon
    JPanel m_treePanel;
    JPanel m_tablePanel;
    
    // The JTree contains the list of series belonging to the selected directory
    JTree m_dicomDirectoryTree;
    JScrollPane m_treeScrollPane;
    
    // Contains the list of images belonging to a series study 
    TwoCompartmentWaterReferenceDicomFileSelectionTable m_dicomImageSeriesTable;
    DefaultTableModel       m_tableModel;
    // Object to store the list items of the image series data into
    Object[][] m_cellData;
    
    JScrollPane m_listScrollPane;
    
    // TreeMap containing the names of the series present in the mounted 
    // directory
    TreeMap m_seriesNames;
    DefaultMutableTreeNode m_treeRoot;
    
    // Data structure containing all file informations of the mounted directory
    DicomDirectory m_dicomDir;
    protected LinkedList<String> m_fileSelectedSeriesFileReferences;
    
    MouseListener m_treeMouseListener;
    
    // variable indicating whether series contains image data or else
    boolean m_isImageSeries;
    
    // variable to indicate that a file system is mounted
    protected boolean m_directoryIsMounted = false;
    
    protected JPopupMenu m_popupMenu;
    
    public final static int TREE_ENTRY_SELECTED_EVENT = 0;
    public final static int TREE_ENTRY_DOUBLE_CLICKED_EVENT = 1;
    
    public final static String TREE_ENTRY_SELECTED_CMD = "SERIES_TREE_SELECTED_CMD";
    public final static String TREE_ENTRY_DBL_CLICKED_CMD = "SERIES_TREE_DBL_CLICKED_CMD";
    
    // For proper serialization
    private static final long serialVersionUID = 9982128799688883L;
    
    private LinkedList<String> m_allSpectroscopyFileReferences = new LinkedList<>( );
    private String             m_singleWaterFileReference;
    
}