/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.spectrim.toolbars;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.Iterator;
import java.util.LinkedList;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.JToolBar;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;

/**
 *
 * Toolbar for basic TDFDfit model file oriented operations.
 *
 * @author slotboom
 */
public final class TDFDFitFileMenuToolbar extends JPanel implements ActionListener
{
    

    public TDFDFitFileMenuToolbar ( )
    {
        super( new BorderLayout( ) );

        // Create the toolbar.
        JToolBar toolBar = new JToolBar( "File-menu toolbar " );
        addButtons( toolBar );

        add( toolBar, BorderLayout.PAGE_START );
        
        setVisible( true );
    }

    /**
     * Add the buttons to the Toolbar.
     * @param toolBar
     */
    protected void addButtons( JToolBar toolBar )
    {
        
        // first button
        m_newModelButton = makeBasicFileToolBarButton( "New24", NEW_MODEL,
                                                       "Create a new TDFDFit model.",
                                                       "Open" );
        toolBar.add( m_newModelButton );
        // m_newModelButton.setEnabled( false );
        
        // second button
        m_newModelKeepResponseButton = makeBasicFileToolBarButton( "NewButKeepResponse24", OPEN_MODEL_KEEP_RESPONSE,
                                                                   "Open exciting TDFDFit mode but keep response.",
                                                                   "OpenKeepResponse" );
        toolBar.add( m_newModelKeepResponseButton );
        // m_newModelKeepResponseButton.setEnabled( false );
        
        // third button
        m_openModelButton = makeBasicFileToolBarButton( "Open24", OPEN_MODEL,
                                                        "Open exciting TDFDFit model.",
                                                        "Open" );
        toolBar.add( m_openModelButton );
        // m_openModelButton.setEnabled( false );
        
        // fourth button
        m_saveModelButton = makeBasicFileToolBarButton( "Save24", SAVE_MODEL,
                                                        "Saves current TDFDFit model.",
                                                        "Save" );
        toolBar.add( m_saveModelButton );
        // m_saveModelButton.setEnabled( false );
        
        // fifth button
        m_closeModelButton = makeBasicFileToolBarButton( "Close24", CLOSE_MODEL,
                                                         "Close currently selected TDFDFit model.",
                                                         "Close");
        toolBar.add( m_closeModelButton );
        // m_closeModelButton.setEnabled( false );

        // sixth button
        m_printModelButton = makeBasicFileToolBarButton( "Print24", PRINT_MODEL,
                                                         "Print currently loaded TDFDFit model ",
                                                         "Print" );
        toolBar.add( m_printModelButton );
        //m_printModelButton.setEnabled( false );
    }


    /**
     *
     * Creates all buttons of the Toolbar
     *
     * @param imageName
     * @param actionCommand
     * @param toolTipText
     * @param altText
     * @return the button to add
     */
    protected JButton makeBasicFileToolBarButton( String imageName,
                                                  String actionCommand,
                                                  String toolTipText,
                                                  String altText )
    {
        //Look for the image.
        String workingDir = System.getProperty( "user.dir" );
        String imgLocation = workingDir + System.getProperty( "file.separator" )
                                        + "images"
                                        + System.getProperty( "file.separator" )
                                        + imageName
                                        + ".gif";
        File imageFile = new File( imgLocation );

        //Create and initialize the button.
        JButton button = new JButton( );
        button.setActionCommand( actionCommand );
        button.setToolTipText( toolTipText );
        button.addActionListener( this );

        if ( imageFile.exists( ) )
        {                      //image found
            button.setIcon( new ImageIcon( imgLocation ) );
        } 
        else
        {                                     //no image found
            button.setText( altText );
            System.err.println( "Resource not found: "
                               + imgLocation );
        }

        return button;
    }


    /**
     *
     * Is called whenever a registered component raises an ActionEvent
     *
     * @param e
     */
    @Override
    public void actionPerformed( ActionEvent e )
    {
        String cmd = e.getActionCommand( );
        String description = null;

        // Handle each button.
        if ( NEW_MODEL.equals( cmd ) )
        {   //first button clicked
            notifyActionListeners( NEW_MODEL, NEW_MODEL_ID );
        }
        else if ( OPEN_MODEL.equals( cmd ) )
        {   //first button clicked
            notifyActionListeners( OPEN_MODEL, OPEN_MODEL_ID );
        }
        else if ( SAVE_MODEL.equals( cmd ) )
        {   //first button clicked
            notifyActionListeners( SAVE_MODEL, SAVE_MODEL_ID );
        } 
        else if ( CLOSE_MODEL.equals( cmd ) )
        {   // second button clicked
            notifyActionListeners( CLOSE_MODEL, CLOSE_MODEL_ID );
        }
        else if ( PRINT_MODEL.equals( cmd ) )
        {   // third button clicked
            notifyActionListeners( PRINT_MODEL, PRINT_MODEL_ID );
        }
    }

     /**
     * Notifies actionEvent to all registered listeners
     */
    private void notifyActionListeners( String actionCommand, int actionID )
    {
        Iterator it = m_actionListeners.iterator( );
        while( it.hasNext( ) )
        {
            ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, actionID, actionCommand ) );
        }
    }

    /**
     * Adds an ActionListener to the list of listeners
     */
    public void addActionListener( ActionListener e )
    {
        m_actionListeners.add( e );
    }

    /**
     * Create the GUI and show it.  For thread safety,
     * this method should be invoked from the
     * event dispatch thread.
     */
    private static void createAndShowGUI( )
    {
        //Create and set up the window.
        JFrame frame = new JFrame( "BasicFileMenuToolbar" );
        frame.setDefaultCloseOperation( JFrame.EXIT_ON_CLOSE );

        //Add content to the window.
        frame.add(new TDFDFitFileMenuToolbar( ) );

        //Display the window.
        frame.pack( );
        frame.setVisible( true );
    }

    public static void main( String[] args )
    {
        //Schedule a job for the event dispatch thread:
        //creating and showing this application's GUI.
        SwingUtilities.invokeLater( new Runnable( )
        {
            public void run( )
            {
                //Turn off metal's use of bold fonts
                UIManager.put( "swing.boldMetal", Boolean.FALSE );
                createAndShowGUI( );
            }
        });
    }

    protected JTextArea textArea;
    protected String newline = "\n";
    
    static final public String NEW_MODEL      = "New";
    static final public String OPEN_MODEL     = "Open";
    static final public String OPEN_MODEL_KEEP_RESPONSE     = "Open-Keep-Response";
    static final public String SAVE_MODEL     = "Save";
    static final public String CLOSE_MODEL    = "Close";
    static final public String PRINT_MODEL    = "Print";
   

    static final public int NEW_MODEL_ID                    = 0;
    static final public int OPEN_MODEL_ID                   = 1;
    static final public int OPEN_MODEL_KEEP_RESPONSE_ID     = 2;
    static final public int SAVE_MODEL_ID                   = 3;
    static final public int CLOSE_MODEL_ID                  = 4;
    static final public int PRINT_MODEL_ID                  = 5;

    JButton m_newModelButton;
    JButton m_newModelKeepResponseButton;
    JButton m_openModelButton;
    JButton m_saveModelButton;
    JButton m_closeModelButton;
    JButton m_printModelButton;
    
    LinkedList<ActionListener> m_actionListeners = new LinkedList<ActionListener>( );

}
