/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.view;

import org.spectrim.imagegraphicobjects.Point3D;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Polygon;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.awt.image.IndexColorModel;
import java.awt.image.WritableRaster;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.TimerTask;
import javax.swing.BorderFactory;
import javax.swing.SwingUtilities;
import org.spectrim.geometricobjects.IntersectionLine;
import org.spectrim.main.ClinicalViewer;
import org.spectrim.model.GlobalModel;
import org.spectrim.spectra.Spectra;
import org.spectrim.utilities.ViewingTools;

/**
 *
 * @author nunopbarros
 */
public class ImageViewerPanel extends JFractionPanel implements MouseListener, MouseMotionListener, MouseWheelListener, KeyListener, ComponentListener, FocusListener {

    private GlobalModel model;
    private int window;
    private int level;
    private int[] pressedCoord = new int[2];
    public BufferedImage buffImage;
    public Image image;
    private ArrayList<double[]> gridLinerProjection;
    private boolean hideGrid = false;
    private boolean hideSaturationBands = false;
    private int selectMode = GlobalModel.SELECTION_SINGLE;
    private IndexColorModel colModel;
    private int colmode = 0;
    private int virtualSize = 512;
    private double zoom = (double) this.getHeight() / (double) this.virtualSize;
    private double panelRatio = zoom;
    private double translx;
    private double transly;
    private double colRowRatio = 1;
    private boolean isSpacialInfoVisible = false;
    private boolean isIntersectionsVisible = false;
    private boolean isInfoSeriesVisible = true;
    private boolean isNameVisible = true;
    private boolean isSelectionActive = true;
    private boolean isSpectraThumbnailVisible = true;
    private String patientName = "";
    private String infoImage = "";
    private String infoGrid = "";
    private String voxelCoordTxt = "";
    private java.util.Timer timer;
    private boolean longpress = false;
    private boolean isFreeHandSelectionActive = false;
    private Polygon freeHandRegion = new Polygon();
    private SpectraPanel spectraThumbnail;
    private int[] hoveredVoxel = new int[]{-1, -1, -1};
    private LinkedList<ActionListener> m_listener = new LinkedList<>();
    
    public final static int LEAVE_DIFFENCE_MODE_ID      = 999;
    public final static String LEAVE_DIFFENCE_MODE_CMD  = "LEAVE_DIFFENCE_MODE_ID";
    
    
//    private ArrayList<double[]> ExcitationVoxelProjection;
    public ImageViewerPanel() {
        super();
        this.window = 255;
        this.level = 128;
        colModel = ColorModelGenerator.createColorModel(255, 128, 256, colmode);
        addMouseListener(this);
        addMouseMotionListener(this);
        addMouseWheelListener(this);
        addKeyListener(this);
        addComponentListener(this);
        addFocusListener(this);
        spectraThumbnail = new SpectraPanel(false);
        this.add(spectraThumbnail);
        spectraThumbnail.setBorder(BorderFactory.createLineBorder(Color.BLUE, 2, true));
    }

    public ImageViewerPanel(int weightWidth, int weightHeight) {
        super(weightWidth, weightHeight);
        this.window = 255;
        this.level = 128;
        colModel = ColorModelGenerator.createColorModel(255, 128, 256, colmode);
        addMouseListener(this);
        addMouseMotionListener(this);
        addMouseWheelListener(this);
        addKeyListener(this);
        addComponentListener(this);
        addFocusListener(this);
    }

    public double getZoom( )
    {
        return zoom;
    }
    
    
    public void setSpacialInfoVisible(boolean bool) {
        isSpacialInfoVisible = bool;
        setView();
        repaint();
    }

    public void setIntersectionsVisible(boolean bool) {
        isIntersectionsVisible = bool;
        repaint();
    }

    public void setInfoSeriesVisible(boolean bool) {
        isInfoSeriesVisible = bool;
        setView();
        repaint();
    }

    public void setSpectraThumbnailVisible(boolean bool) {
        isSpectraThumbnailVisible = bool;
        setView();
        repaint();
    }

    public void setMouseSelectionActive(boolean bool) {
        isSelectionActive = bool;
    }

    public boolean isMouseSelectionActive() {
        return isSelectionActive;
    }

    public void setNameVisible(boolean bool) {
        isNameVisible = bool;
        repaint();
    }

    public void setPatientName(String txt) {
        this.patientName = txt;
        repaint();
    }

    public void setImageInfo(String txt) {
        this.infoImage = txt;
        repaint();
    }

    public void setGridInfo(String txt) {
        this.infoGrid = txt;
        repaint();
    }

    public void setModel(GlobalModel model) {
        this.model = model;
    }

    public GlobalModel getModel() {
        return model;
    }

    public int getVirtualSize() {
        return this.virtualSize;
    }

    public AffineTransform getAppliedTransform() {
        AffineTransform transf = ((Graphics2D) (this.getGraphics())).getTransform();
        transf.translate(translx + ((double) this.getWidth() / 2 * (1 - colRowRatio)) + (this.getWidth() - this.getHeight()) / 2, transly);
        transf.scale(zoom * colRowRatio, zoom);
        return transf;
    }

    public int[] getCorrectedCoordinates(int x, int y) {
        int xcorrect = (int) (((float) x - (float) translx - ((float) this.getWidth() / 2 * (1 - (float) colRowRatio)) - (this.getWidth() - this.getHeight()) / 2) / ((float) zoom * (float) colRowRatio));
        int ycorrect = (int) (((float) y - (float) transly) / ((float) zoom));
        return new int[]{xcorrect, ycorrect};
    }

    /**
     * transform image coordinates into scanner coordinates
     */
    public Point3D getMachineSpaceCoordinates(int x, int y) {
        double xval = Math.floor(((float) x * (float) model.getCurrentImage().image.getWidth() / (float) this.virtualSize));
        double yval = Math.floor(((float) y * (float) model.getCurrentImage().image.getHeight() / (float) this.virtualSize));
        Point3D position = model.getCurrentImage().imagePosition;
        position = position.addPoint(model.getCurrentImage().imageOrientationX.scalarMultiplication(xval * model.getCurrentImage().pixelSpacingX)).addPoint(model.getCurrentImage().imageOrientationY.scalarMultiplication(yval * model.getCurrentImage().pixelSpacingY));
        return position;
    }

    public int[] getCorrectedCoordinatesAbsRef(int x, int y) {
        int xcorrect = (int) (((float) x - ((float) this.getWidth() / 2 * (1 - (float) colRowRatio)) - (this.getWidth() - this.getHeight()) / 2) / ((float) zoom * (float) colRowRatio));
        int ycorrect = (int) (((float) y) / ((float) zoom));
        return new int[]{xcorrect, ycorrect};
    }

    public void autoWindowing() {
        if (model.getCurrentImage() != null) {
            ImageSlice imageInfo = model.getCurrentImage();
            window = (int) (imageInfo.getMaxVal()[0] - imageInfo.getMinVal()[0] + 1);
            level = (int) (imageInfo.getMinVal()[0] + (imageInfo.getMaxVal()[0] - imageInfo.getMinVal()[0]) / 2);
            if (window < 1) {
                window = 1;
            }
            if (level < 1) {
                level = 1;
            }
            colModel = ColorModelGenerator.createColorModel(window, level, (int) model.getImageStack().getMaxIntensity()[0], colmode);
            setView();
            repaint();
        }
    }

    public void focusOnExcitedVoxels() {
        // TODO: add check before rezoom
        /*if(!model.())
         {*/
        double[] min = new double[]{Double.MAX_VALUE, Double.MAX_VALUE};
        double[] max = new double[]{Double.MIN_VALUE, Double.MIN_VALUE};
        for (double[] line : model.getExcitationVoxelProjection()) {//getGridLinerProjection
            min[0] = Math.min(min[0], Math.min(line[0], line[2]));
            min[1] = Math.min(min[1], Math.min(line[1], line[3]));
            max[0] = Math.max(max[0], Math.max(line[0], line[2]));
            max[1] = Math.max(max[1], Math.max(line[1], line[3]));
        }
        min[0] *= this.virtualSize;
        min[1] *= this.virtualSize;
        max[0] *= this.virtualSize;
        max[1] *= this.virtualSize;
        System.out.println("max: (" + max[0] + "," + max[1] + "), min: (" + min[0] + "," + min[1] + ")");
        System.out.println("Height: " + this.getHeight() + ", Width: " + this.getWidth() + ", virtual size: " + this.virtualSize);
        double[] side = new double[]{max[0] - min[0], max[1] - min[1]};
        System.out.println("Size: " + side[0] + ", " + side[1]);

        double oldZoom = zoom;
        if (side[0] > side[1]) {
            zoom = this.getWidth() / side[0] - 1;
        } else {
            zoom = this.getHeight() / side[1] - 1;
        }
        System.out.println("t: " + translx + " " + transly);
        translx = (1 - zoom / oldZoom) * (min[0] + side[0] / 2.0) / 1.04;
        transly = (1 - zoom / oldZoom) * (min[1] + side[1] / 2.0) * 1.065;
        System.out.println("zoom ratio: " + (1 - zoom / oldZoom));

        repaint();
        //}
    }

    public void resetZoom() {
        zoom = (double) this.getHeight() / (double) this.virtualSize;
//        if(z)
//            zoom = 1.0;
        panelRatio = zoom;
        translx = 0;
        transly = 0;
    }

    public void setColorMode(int mode) {
        colmode = mode;
        autoWindowing();
    }

    public void setGridVisible(boolean bool) {
        hideGrid = !bool;
    }

    public void setSaturationBandsVisible(boolean bool) {
        hideSaturationBands = !bool;
    }

    public void newFreeHandSelection() {
        if (model != null) {
            if ((model.getGridDim1() * model.getGridDim2() * model.getGridDim3() > 1)) {
                isFreeHandSelectionActive = true;
            }
        }
    }

    @Override
    public void paintComponent(Graphics g) {
        super.paintComponent(g);

        if (this.buffImage != null) {
            Graphics2D g2D = (Graphics2D) g;
            g2D.setFont(new Font("Calibri", Font.PLAIN, 14));
            g2D.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                    RenderingHints.VALUE_ANTIALIAS_ON);
//            g2D.setRenderingHint(RenderingHints.KEY_RENDERING,
//                    RenderingHints.VALUE_RENDER_SPEED);
            g2D.setRenderingHint(RenderingHints.KEY_RENDERING,
                    RenderingHints.VALUE_RENDER_DEFAULT);
            AffineTransform transfOrig = (AffineTransform) g2D.getTransform().clone();
            AffineTransform transf = g2D.getTransform();
            transf.translate(translx + ((double) this.getWidth() / 2 * (1 - colRowRatio)) + (this.getWidth() - this.getHeight()) / 2, transly);
            transf.scale(zoom * colRowRatio, zoom);
            g2D.setTransform(transf);
            g2D.drawImage(this.buffImage, 0, 0, this);
            ArrayList<IntersectionLine> linesaux = model.getLocalizersIntersections();
            if (linesaux.size() > 0 && isIntersectionsVisible) {
                for (int i = 0; i < linesaux.size(); i++) {
                    g2D.setStroke(new BasicStroke(2));
                    linesaux.get(i).draw(g2D);
                }
            }
            if (model != null && !hideGrid) {
                if (!longpress || isFreeHandSelectionActive) {
                    model.drawIntersections(g2D, selectMode != GlobalModel.SELECTION_SINGLE, !hideSaturationBands);
                    g2D.setStroke(new BasicStroke(2));
                    drawProjection(g2D, new Color(0f, 0.45f, 0f, 0.6f), gridLinerProjection);
                }
                
                if ( model != null && model.getSegmentationOverlayAvailable( ) )
                {
                    model.drawSegmentationOverlays( g2D );
                }
                
                if ( this.isFreeHandSelectionActive ) 
                {
                    if ( freeHandRegion.npoints > 1 ) 
                    {
                        g2D.setStroke(new BasicStroke(2f, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND));
                        g2D.setColor(Color.YELLOW);
                        g2D.drawPolygon(freeHandRegion);
                    }
                }
            }
            g2D.setTransform(transfOrig);

            g2D.setStroke(new BasicStroke(1));
            g2D.setFont(new Font("Calibri", Font.BOLD, 14));
            ViewingTools.drawShadowedString(g2D, (model.getSlice() + 1) + "/" + model.getImageStackSize(), 12, 20);
            g2D.setFont(new Font("Calibri", Font.PLAIN, 14));
            
            // System.out.println( "voxelCoordTxt : " + voxelCoordTxt );
            
            ViewingTools.drawShadowedString(g2D, voxelCoordTxt, this.getWidth() - 112, this.getHeight() - 10);
            
            if (isSpacialInfoVisible) {
                Point3D p = model.getCurrentImage().imagePosition;
                Point3D e1 = model.getCurrentImage().imageOrientationX;
                Point3D e2 = model.getCurrentImage().imageOrientationY;
                DecimalFormat df = new DecimalFormat("#.###");
                String ptxt = "p   = (" + df.format(p.m_x1) + ", " + df.format(p.m_x2) + ", " + df.format(p.m_x3) + ")";
                String e1txt = "e1 = (" + df.format(e1.m_x1) + ", " + df.format(e1.m_x2) + ", " + df.format(e1.m_x3) + ")";
                String e2txt = "e2 = (" + df.format(e2.m_x1) + ", " + df.format(e2.m_x2) + ", " + df.format(e2.m_x3) + ")";
                ViewingTools.drawShadowedString(g2D, e1txt, 12, this.getHeight() - 38);
                ViewingTools.drawShadowedString(g2D, e2txt, 12, this.getHeight() - 24);
                ViewingTools.drawShadowedString(g2D, ptxt, 12, this.getHeight() - 10);
            }
            if (isInfoSeriesVisible) {
                ViewingTools.drawShadowedString(g2D, this.infoImage, 12, 40);
                ViewingTools.drawShadowedString(g2D, this.infoGrid, 12, 56);
            }
            if (isNameVisible) {
                FontMetrics fmetrics = g2D.getFontMetrics();
                g2D.setFont(new Font("Calibri", Font.BOLD, 14));
                ViewingTools.drawShadowedString(g2D, this.patientName, this.getWidth() / 2 - fmetrics.stringWidth(patientName) / 2, 20);
            }
            if ( model.isColorMapActive( ) && model.isColorBarVisible( ) && !model.getColorMap( ).isTriChannel ) 
            {
                model.getColorMap( ).getColorBar( ).paintComponent( g2D );
            }
            if (this.hoveredVoxel[0] != -1) {
                spectraThumbnail.setVisible(true);
            } else {
                spectraThumbnail.setVisible(false);
            }
        }
    }

    public void setView() 
    {
        try 
        {
            if ( model != null && model.getImageStackSize() > 0 ) 
            {
//                buffImage = new BufferedImage(this.virtualSize, this.virtualSize, BufferedImage.TYPE_INT_ARGB);
                buffImage = new BufferedImage(this.virtualSize, this.virtualSize, BufferedImage.TYPE_4BYTE_ABGR);
                if (model.getCurrentImage().image != null) 
                {
                    if (this.virtualSize > 0 && this.virtualSize > 0) 
                    {

                        ImageSlice imageInfo = model.getCurrentImage();
                        colRowRatio = (double) imageInfo.sizeX / (double) imageInfo.sizeY;

                        BufferedImage buffered = new BufferedImage(colModel, (WritableRaster) model.getCurrentImage().image, colModel.isAlphaPremultiplied(), null);

////                        TESTE COLOR SCALE
//                        WritableRaster compatRaster = colModel.createCompatibleWritableRaster(model.getCurrentImage().image.getWidth(), model.getCurrentImage().image.getHeight());
//                        double[] min = model.getImageStack().getMinIntensity();
//                        double[] max = model.getImageStack().getMaxIntensity();
//                        for (int i = 0; i < compatRaster.getWidth(); i++) {
//                            for (int j = 0; j < compatRaster.getHeight(); j++) {
//                                int[] val = model.getCurrentImage().image.getPixel(i, j, new int[] {0,0,0});
//                                int val1 = (int)(255*((double)val[0]-min[0])/max[0]);
//                                if(val1<0){
//                                    val1=0;
//                                }else if(val1>255){
//                                    val1 = 255;
//                                }
//                                int[] newval = new int[] {val1,val1,val1};
////                                if(newval[1]<0){
////                                    newval[1]=0;
////                                }
////                                if(newval[2]<0){
////                                    newval[2]=0;
////                                }
//                                compatRaster.setPixel(i, j,newval);
//                            }
//                        }
//                        BufferedImage buffered = new BufferedImage(colModel, (WritableRaster) compatRaster, colModel.isAlphaPremultiplied(), null);
//                        RESCALE
                        Image img = buffered.getScaledInstance(this.virtualSize, this.virtualSize, BufferedImage.SCALE_FAST);
//                        buffImage = new BufferedImage(this.getWidth(), this.getHeight(), BufferedImage.TYPE_INT_ARGB);
                        buffImage.getGraphics().drawImage(img, 0, 0, null);

                        if ( model.hasOverlay() ) 
                        {
//                            BufferedImage bufferedOverlay = new BufferedImage(model.getCurrentOverlay().image.getWidth(),model.getCurrentOverlay().image.getHeight(),BufferedImage.TYPE_INT_ARGB);
//                            bufferedOverlay.setData(model.getCurrentOverlay().imageRepresentation);
                            img = model.getCurrentOverlay().buffImg.getScaledInstance(this.virtualSize, this.virtualSize, BufferedImage.SCALE_FAST);
                            buffImage.getGraphics().drawImage(img, 0, 0, null);
                        }

                        model.calcLocalizersIntersections();
                        if ( model.getGridSize() > 0 ) 
                        {
//                            model.setAmplificationFact((double) this.getWidth()  / ((double) imageInfo.SizeX * (double) imageInfo.PixelSpacingX), (double) this.getHeight() * 5 / ((double) imageInfo.SizeY * (double) imageInfo.PixelSpacingY));
                            model.setAmplificationFact((double) virtualSize / ((double) imageInfo.sizeX * (double) imageInfo.pixelSpacingX), (double) virtualSize / ((double) imageInfo.sizeY * (double) imageInfo.pixelSpacingY));
                            if (model.getGridSize() > 1) 
                            {
                                gridLinerProjection = model.getGridLinerProjection();
                            } 
                            else if (model.getGridSize() == 1) 
                            {
                                gridLinerProjection = model.getExcitationVoxelProjection();
                            }
                        }
                    }
                    if ( model.isColorMapActive() && model.isColorBarVisible() && !model.getColorMap().isTriChannel ) 
                    {
                        int startyCBar = 82;
                        int heightCBar = this.getHeight() - 140;
                        if (!this.isSpacialInfoVisible) 
                        {
                            heightCBar += 38;
                        }
                        if (!this.isInfoSeriesVisible) 
                        {
                            startyCBar = 52;
                            heightCBar += 30;
                        }
                        model.getColorMap().getColorBar().setBounds(12, startyCBar, 20, heightCBar);
                    }
                }
            }
        } catch (Exception e) 
        {
            e.printStackTrace();
        }
    }

    public void clearImage() {
        buffImage = null;
        this.gridLinerProjection.clear();
        this.infoImage = "";
        repaint();
    }

    public void clearGridProjection() {
        if (this.gridLinerProjection != null) {
            this.gridLinerProjection.clear();
        }
        this.infoGrid = "";
        repaint();
    }

    public void drawProjection(Graphics2D g, Color c, ArrayList<double[]> lines) {
        double[] aux;
        if (lines != null) {
            for (int i = 0; i < lines.size(); i++) {
                aux = lines.get(i);
                g.setColor(c);
                g.drawLine((int) Math.round(aux[0] * this.virtualSize), (int) Math.round(aux[1] * this.virtualSize), (int) Math.round(aux[2] * this.virtualSize), (int) Math.round(aux[3] * this.virtualSize));
            }
        }
    }

    @Override
    public void mouseClicked( MouseEvent me ) 
    {
        notifyActionlisteners( LEAVE_DIFFENCE_MODE_ID, LEAVE_DIFFENCE_MODE_CMD );
//         System.out.println( "ImageViewer::mouseClicked : " + me.getX( ) + "_" + me.getY( ) );
        this.requestFocus( );
        if ( model != null ) 
        {
            this.pressedCoord = new int[]{ me.getX( ), me.getY( ) };
            if ( model.getCurrentImage( ).image != null ) 
            {
                ImageSlice imageInfo = model.getCurrentImage();

                int xcorrect = (int) (((float) me.getX() - (float) translx - ((float) this.getWidth() / 2 * (1 - (float) colRowRatio)) - (this.getWidth() - this.getHeight()) / 2) / ((float) zoom * (float) colRowRatio));
                int ycorrect = (int) (((float) me.getY() - (float) transly) / ((float) zoom));

//                System.out.println();
//                System.out.println(">>coord correct: " + xcorrect + " " + ycorrect);
                int xval = (int) Math.floor(((float) xcorrect * (float) model.getCurrentImage().image.getWidth() / (float) this.virtualSize));
                int yval = (int) Math.floor(((float) ycorrect * (float) model.getCurrentImage().image.getHeight() / (float) this.virtualSize));
                if (xcorrect >= 0 && xcorrect < this.virtualSize && ycorrect >= 0 && ycorrect < this.virtualSize) {
//                    System.out.println(">>coord: " + xval + " " + yval);
                    int[] val = model.getCurrentImage().image.getPixel(xval, yval, new int[]{0, 0, 0});
//                    System.out.println(">>Pixel Value: " + val[0]);
                    Point3D position = imageInfo.imagePosition;
                    position = position.addPoint(imageInfo.imageOrientationX.scalarMultiplication(xval * imageInfo.pixelSpacingX)).addPoint(imageInfo.imageOrientationY.scalarMultiplication(yval * imageInfo.pixelSpacingY));
//                    System.out.println(">>Position: " + position.m_x1 + "" + position.m_x2 + "" + position.m_x3);
                    if (model != null && isSelectionActive) {
                        if (me.isShiftDown()) {
                            if (me.isControlDown()) {
                                model.selectVoxel(xcorrect, ycorrect, GlobalModel.SELECTION_RANGE_ADD);
                            } else if (me.isAltDown()) {
                                model.selectVoxel(xcorrect, ycorrect, GlobalModel.SELECTION_RANGE_REMOVE);
                            } else {
                                model.selectVoxel(xcorrect, ycorrect, GlobalModel.SELECTION_RANGE_SINGLE);
                            }
                        } else {
                            if (me.isControlDown()) {
                                model.selectVoxel(xcorrect, ycorrect, GlobalModel.SELECTION_ADD);
                            } else if (me.isAltDown()) {
                                model.selectVoxel(xcorrect, ycorrect, GlobalModel.SELECTION_REMOVE);
                            } else {
                                model.SetShowDifference(false);
                                model.selectVoxel(xcorrect, ycorrect, GlobalModel.SELECTION_SINGLE);
                            }
                        }
                    }
                    setView();
                    repaint();
                }
            }
        }
    }

    @Override
    public void mousePressed(MouseEvent me) {
        this.pressedCoord = new int[]{me.getX(), me.getY()};
        if (timer == null) {
            timer = new java.util.Timer();
        }

        if (!me.isControlDown() && !me.isAltDown()) {
            timer.schedule(new TimerTask() {
                public void run() {
                    longpress = true;
                    repaint();
                }
            }, 500);
        }
        if (me.getModifiersEx() == (MouseEvent.BUTTON1_DOWN_MASK + MouseEvent.BUTTON3_DOWN_MASK)) {
            zoom = (double) this.getHeight() / (double) this.virtualSize;
            translx = 0;
            transly = 0;
            repaint();
        }
    }

    @Override
    public void mouseReleased(MouseEvent me) {
        if (timer != null) {
            timer.cancel();
            timer = null;
        }
        if (longpress) {
            longpress = false;
            repaint();
        }
        if (this.isFreeHandSelectionActive & this.model != null) {
            selectMode = GlobalModel.SELECTION_SINGLE;
            if (me.isControlDown()) {
                selectMode = GlobalModel.SELECTION_ADD;
            } else if (me.isAltDown()) {
                selectMode = GlobalModel.SELECTION_REMOVE;
            }
            model.selectVoxel(freeHandRegion, selectMode);
            this.isFreeHandSelectionActive = false;
            this.freeHandRegion.reset();
        }
        
    }

    @Override
    public void mouseEntered(MouseEvent me) {
    }

    @Override
    public void mouseExited(MouseEvent me) {
        if (model != null) {
            model.setXLabelText("");
            model.setYLabelText("");
            model.setPixelValLabelText("");
            if (model.isColorMapActive() && model.isColorBarVisible() && !model.getColorMap().isTriChannel) {
                model.getColorMap().getColorBar().mouseListener(0, 0);
            }
            voxelCoordTxt = "";
            spectraThumbnail.setVisible(false);
            hoveredVoxel = new int[]{-1, -1, -1};
            this.repaint();
        }

    }

    @Override
    public void mouseDragged(MouseEvent me) {
        voxelCoordTxt = "";
        spectraThumbnail.setVisible(false);
        hoveredVoxel = new int[]{-1, -1, -1};
        if (model != null) {
            if (model.getCurrentImage() != null) {
                this.requestFocus();
                int xcorrect = (int) (((float) me.getX() - (float) translx - ((float) this.getWidth() / 2 * (1 - (float) colRowRatio)) - (this.getWidth() - this.getHeight()) / 2) / ((float) zoom * (float) colRowRatio));
                int ycorrect = (int) (((float) me.getY() - (float) transly) / ((float) zoom));
                if (this.isFreeHandSelectionActive) {
                    double dist = 0;
                    if (freeHandRegion.npoints > 1) {
                        dist = Math.sqrt(Math.pow(freeHandRegion.xpoints[freeHandRegion.npoints - 1] - xcorrect, 2) + Math.pow(freeHandRegion.ypoints[freeHandRegion.npoints - 1] - ycorrect, 2));
                    } else {
                        dist = 1;
                    }
                    if (dist >= 1) {
                        this.freeHandRegion.addPoint(xcorrect, ycorrect);
                    }
                } else if (me.isAltDown()) { //Alt and Middle mouse button are equivalent
                    if (this.selectMode == GlobalModel.SELECTION_REMOVE && isSelectionActive) {
                        model.selectVoxel(xcorrect, ycorrect, selectMode);
                    } else {
                        this.window = this.window + (me.getX() - this.pressedCoord[0]);
                        this.level = this.level + (me.getY() - this.pressedCoord[1]);
                        if (window < 1) {
                            window = 1;
                        }
                        if (level < 1) {
                            level = 1;
                        }
                        colModel = ColorModelGenerator.createColorModel(window, level, (int) this.model.getImageStack().getMaxIntensity()[0], colmode);
                        this.pressedCoord = new int[]{me.getX(), me.getY()};
                    }
                    setView();
                } else if (me.isControlDown() && isSelectionActive) {
                    model.selectVoxel(xcorrect, ycorrect, GlobalModel.SELECTION_ADD);
                    setView();
                } else {
                    translx = translx + (me.getX() - this.pressedCoord[0]);
                    transly = transly + (me.getY() - this.pressedCoord[1]);
                    this.pressedCoord[0] = me.getX();
                    this.pressedCoord[1] = me.getY();
                }
                repaint();
            }
        }
    }

    @Override
    public void mouseMoved(MouseEvent me) {
        if (model != null) {
            if (model.getCurrentImage().image != null && isSelectionActive) {
                int xcorrect = (int) (((float) me.getX() - (float) translx - ((float) this.getWidth() / 2 * (1 - (float) colRowRatio)) - (this.getWidth() - this.getHeight()) / 2) / ((float) zoom * (float) colRowRatio));
                int ycorrect = (int) (((float) me.getY() - (float) transly) / ((float) zoom));
                int xval = (int) Math.floor(((float) xcorrect * (float) model.getCurrentImage().image.getWidth() / (float) this.virtualSize));
                int yval = (int) Math.floor(((float) ycorrect * (float) model.getCurrentImage().image.getHeight() / (float) this.virtualSize));
                if (xcorrect >= 0 && xcorrect < this.virtualSize && ycorrect >= 0 && ycorrect < this.virtualSize) {
                    model.setXLabelText(String.valueOf(xval));
                    model.setYLabelText(String.valueOf(yval));
                    int[] val;
                    if (model.hasOverlay()) {
                        val = model.getCurrentOverlay().image.getPixel(xval, yval, new int[]{0, 0, 0});
                    } else {
                        val = model.getCurrentImage().image.getPixel(xval, yval, new int[]{0, 0, 0});
                    }
                    model.setPixelValLabelText(String.valueOf(val[0]));
                }

                if (model.getGridSize() > 1) {
                    int[] coord = model.getVoxelIndex(xcorrect, ycorrect);

                    if (coord[0] != -1) {
                        voxelCoordTxt = "(" + (coord[0] + 1) + "/" + model.getGridDim1() + "," + (coord[1] + 1) + "/" + model.getGridDim2() + "," + (coord[2] + 1) + "/" + model.getGridDim3() + ")";
                        if (isSpectraThumbnailVisible && model.getSpectra().containsID(coord[0] + "_" + coord[1] + "_" + coord[2])) {
                            if (!Arrays.equals(coord, hoveredVoxel)) {
                                spectraThumbnail.setXAxis(model.getxAxis());
                                if (model.getSpectra().getDefaultDomain() == Spectra.Domain.TD) {
                                    spectraThumbnail.setInvertedAxis(false);
                                } else {
                                    spectraThumbnail.setInvertedAxis(true);
                                }
                                spectraThumbnail.clearData();
                                if (model.isAbsorptionVisible()) {
                                    spectraThumbnail.setData(model.getSpectra().getSpectrumData(coord[0], coord[1], coord[2], Spectra.Mode.ABS), SpectraPanel.DataType.ABS_BLUE);
                                }
                                if (model.isDispersionVisible()) {
                                    spectraThumbnail.setData(model.getSpectra().getSpectrumData(coord[0], coord[1], coord[2], Spectra.Mode.DISP), SpectraPanel.DataType.DISP_BLUE);
                                }
                                if (model.isMagnitudeVisible()) {
                                    spectraThumbnail.setData(model.getSpectra().getSpectrumData(coord[0], coord[1], coord[2], Spectra.Mode.MAG), SpectraPanel.DataType.MAG_BLUE);
                                }
                                spectraThumbnail.updateSeriesColor();
                                double[] zoom = model.getSpectraPanel().getZoom();
                                spectraThumbnail.setZoom(new double[]{zoom[0], zoom[1]});

                                hoveredVoxel = coord;
                            }

                            int width = this.getWidth() / 4;
                            int height = (int) Math.round(width / 1.618);
                            spectraThumbnail.setSize(width, height);
                            int posX = (me.getX() + 18 + width) > this.getWidth() ? me.getX() - width - 8 : me.getX() + 18;
                            int posY = (me.getY() + 26 + height) > this.getHeight() ? me.getY() - height - 8 : me.getY() + 26;
                            spectraThumbnail.setLocation(posX, posY);
                        }
                    } else {
                        voxelCoordTxt = "";
                        spectraThumbnail.setVisible(false);
                        hoveredVoxel = new int[]{-1, -1, -1};
                    }
                    repaint();
                    if (model.isColorMapActive() && model.isColorBarVisible() && !model.getColorMap().isTriChannel) {
                        model.getColorMap().getColorBar().mouseListener(me.getX(), me.getY());
                        this.repaint();
                    }
                }
            }
        }
    }

    @Override
    public void mouseWheelMoved(MouseWheelEvent mwe) 
    {
        int aux;
        if ( model != null ) 
        {
            if ( mwe.isControlDown( ) ) 
            {
                float xcorrect = ( ( (float) mwe.getX( ) - (float) translx ) / ( (float) zoom ) );
                float ycorrect = ( ( (float) mwe.getY( ) - (float) transly ) / ( (float) zoom ) );
                zoom = zoom - (double) mwe.getWheelRotation() / 5;
                if (mwe.getWheelRotation() > 0 && zoom < 1) 
                {
                    zoom = 1;
                } 
                else if ( mwe.getWheelRotation( ) < 0 && zoom > 40 ) 
                {
                    zoom = 40;
                }
                translx = mwe.getX( ) - zoom * xcorrect;
                transly = mwe.getY( ) - zoom * ycorrect;
                repaint( );
            } 
            else 
            {
                if ( model.getCurrentImage( ) != null ) 
                {
                    aux = model.getSlice( ) + mwe.getWheelRotation( );
                    if ( aux < 0 ) 
                    {
                        model.setSlice( 0 );
                    } 
                    else if ( aux > ( model.getImageStackSize( ) - 1 ) ) 
                    {
                        model.setSlice( model.getImageStackSize( ) - 1 );
                    } 
                    else 
                    {
                        model.setSlice( aux );
                    }
                }
            }
        }
        mouseMoved( mwe );
    }

    @Override
    public void keyTyped(KeyEvent ke) {
    }

    @Override
    public void keyPressed(KeyEvent ke) {
        selectMode = GlobalModel.SELECTION_SINGLE;
        if (ke.isControlDown()) {
            selectMode = GlobalModel.SELECTION_ADD;
        } else if (ke.isAltDown()) {
            selectMode = GlobalModel.SELECTION_REMOVE;
        }

        if (model != null && model.getGridSize() > 1 && isSelectionActive) {
            Point3D increment = new Point3D();
            switch (ke.getKeyCode()) {
                case KeyEvent.VK_RIGHT:
                    increment = calculateCoordinateIncrement(model.getCurrentImage().imageOrientationX);
                    break;
                case KeyEvent.VK_LEFT:
                    increment = calculateCoordinateIncrement(model.getCurrentImage().imageOrientationX.Clone().scalarMultiplication(-1));
                    break;
                case KeyEvent.VK_DOWN:
                    if (ke.isShiftDown()) {
                        increment = calculateCoordinateIncrement(Point3D.crossProduct(model.getCurrentImage().imageOrientationY,model.getCurrentImage().imageOrientationX));
                    } else {
                        increment = calculateCoordinateIncrement(model.getCurrentImage().imageOrientationY);
                    }
                    break;
                case KeyEvent.VK_UP:
                    if (ke.isShiftDown()) {
                        increment = calculateCoordinateIncrement(Point3D.crossProduct(model.getCurrentImage().imageOrientationX,model.getCurrentImage().imageOrientationY));
                    } else {
                        increment = calculateCoordinateIncrement(model.getCurrentImage().imageOrientationY.Clone().scalarMultiplication(-1));
                    }
                    break;
            }
            if (increment.length() > 0) {
                int[] lastVoxel = model.getLastSelectedVoxel();
                String newCoord = (lastVoxel[0] + (int) increment.m_x1) + "_" + (lastVoxel[1] + (int) increment.m_x2) + "_" + (lastVoxel[2] + (int) increment.m_x3);
                if (model.getSpectra().containsID(newCoord)) {
                    model.selectVoxel(newCoord, selectMode);
                    setView();
                }
            }
            repaint();
        }
    }

    public Point3D calculateCoordinateIncrement(Point3D vect) {
        double dir1Prod = Point3D.dotProduct(model.getCSIGrid().getDir1().normalize(), vect);
        double dir2Prod = Point3D.dotProduct(model.getCSIGrid().getDir2().normalize(), vect);
        double dir3Prod = Point3D.dotProduct(model.getCSIGrid().getDir3().normalize(), vect);
        if (Math.abs(dir1Prod) >= Math.abs(dir2Prod) && Math.abs(dir1Prod) >= Math.abs(dir3Prod)) {
            return new Point3D(dir1Prod > 0 ? 1 : -1, 0, 0);
        } else if (Math.abs(dir2Prod) >= Math.abs(dir3Prod)) {
            return new Point3D(0, dir2Prod > 0 ? 1 : -1, 0);
        } else {
            return new Point3D(0, 0, dir3Prod > 0 ? 1 : -1);
        }
    }

    @Override
    public void keyReleased(KeyEvent ke) {
        if ((ke.getKeyCode() == 17 || ke.getKeyCode() == 18) && !ke.isControlDown() && !ke.isAltDown()) {
            selectMode = GlobalModel.SELECTION_SINGLE;
            repaint();
        }
    }

    @Override
    public void componentResized(ComponentEvent ce) {
        if (!Double.isNaN(zoom)) {
            double newRatio = (double) this.getHeight() / (double) this.virtualSize;
            zoom = newRatio / panelRatio * zoom;
            translx *= newRatio / panelRatio;
            transly *= newRatio / panelRatio;
            panelRatio = newRatio;
        } else {
            resetZoom();
        }
        if (model != null) {
            if (model.isColorMapActive() && model.isColorBarVisible() && !model.getColorMap().isTriChannel) {
                int startyCBar = 82;
                int heightCBar = this.getHeight() - 140;
                if (!this.isSpacialInfoVisible) {
                    heightCBar += 38;
                }
                if (!this.isInfoSeriesVisible) {
                    startyCBar = 52;
                    heightCBar += 30;
                }
                model.getColorMap().getColorBar().setBounds(12, startyCBar, 20, heightCBar);
            }
        }
        setView();
        repaint();
    }

    @Override
    public void componentMoved(ComponentEvent ce) {
    }

    @Override
    public void componentShown(ComponentEvent ce) {
    }

    @Override
    public void componentHidden(ComponentEvent ce) {
    }

    @Override
    public void focusGained(FocusEvent fe) {
    }

    @Override
    public void focusLost(FocusEvent fe) {
        selectMode = GlobalModel.SELECTION_SINGLE;
        repaint();
    }

    public void addActionListener( ActionListener listener ) 
    {
        m_listener.add( listener );
    }
    
    private void notifyActionlisteners( int ID, String cmdString )
    {
        Iterator it = m_listener.iterator();
        while ( it.hasNext() )
        {
            ((ActionListener)it.next( ) ).actionPerformed(new ActionEvent(this, ID, cmdString));
        }

    }
}
