/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.waterremoval;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.spectrim.fidsvd.SVDMVCModel;
import org.spectrim.jampack.JampackException;
import org.spectrim.tdfdfit.SingleVoxelMeasuredSpectralData;
import org.spectrim.xyplot.Curve;
import org.spectrim.xyplot.XYPlotFrame;

/**
 * 
 * Gives access to simple SVD water removal routine.
 * 
 * @author Johannes Slotboom <johannes.slotboom at insel.ch>
 */
public class SVDWaterLineRemover 
{

    private final LinkedList<ActionListener> m_listeners = new LinkedList<>( );
    
    public final static int FIRST_PASS_SVD_COMPUTED = 0;
    public final static String FIRST_PASS_SVD_COMPUTED_CMD = "FIRST_PASS_SVD_COMPUTED";
    
    public final static int FIRST_PASS_FINISHED = 1;
    public final static String FIRST_PASS_FINISHED_CMD = "FIRST_PASS_FINISHED";
    
    public final static int SECOND_PASS_SVD_COMPUTED = 2;
    public final static String SECOND_PASS_SVD_COMPUTED_CMD = "SECOND_PASS_SVD_COMPUTED";
    
    public final static int SECOND_PASS_FINISHED = 3;
    public final static String SECOND_PASS_FINISHED_CMD = "SECOND_PASS_FINISHED";
    
    double[] m_singularValues1stPass;
    double[] m_singularValues2ndPass;
    double[] m_fid_water_first_pass_sum;
    double[] m_fid_water_second_pass_sum;
    int      m_numberOfSingularValuesToCheck;
    
    /**
     * Default constructor.
     */
    public SVDWaterLineRemover( )
    {    
    }
    
    /**
     * This routine perform a SVD on the specData object and investigates
     * whether the responses belonging to individual eigenvalues have their
     * maximum in the water resonance frequency range: if yes this signal is 
     * subtracted from the measured signal.
     * @param specData
     * @param numberOfSingularValuesToCheck
     * @return 
     */
    public SingleVoxelMeasuredSpectralData performSVDLineRemover( SingleVoxelMeasuredSpectralData specData,
                                                                  int numberOfSingularValuesToCheck )
    {
        
        m_numberOfSingularValuesToCheck = numberOfSingularValuesToCheck;
        
        double[] real = new double[specData.getNumberOfSamplePoints( )];
        double[] imag = new double[specData.getNumberOfSamplePoints( )];
        m_fid_water_first_pass_sum = new double[2*specData.getNumberOfSamplePoints( )];
        m_fid_water_second_pass_sum = new double[2*specData.getNumberOfSamplePoints( )];
        
        // double[] fid_water_first_pass_sum   = null;
        double[] fid_water_second_pass      = null;
        // double[] fid_water_second_pass_sum  = null;
        
        for( int i = 0; i < specData.getNumberOfSamplePoints( ); i++ )
        {
            real[i] = (double)specData.getAbsorptionChannelTD( )[i];
            imag[i] = (double)specData.getDispersionChannelTD( )[i];
        }
        
        SVDMVCModel svdModel = new SVDMVCModel( );
        try 
        {
            svdModel.essaisetBaseIndex( );
        } 
        catch ( JampackException ex ) 
        {
            Logger.getLogger( SVDWaterLineRemover.class.getName( ) ).log( Level.SEVERE, null, ex );
        }
        svdModel.setmaxNbComplex( specData.getNumberOfSamplePoints( ) );
        svdModel.setfidRe( real );
        svdModel.setfidIm( imag );
        
        //SVD call
        
        try 
        {
            svdModel.essaiZsvd( );
        }
        catch ( JampackException e ) 
        {
            System.out.println( "An exception occurred while computation the SVD of the signal." );
        }

        m_singularValues1stPass = svdModel.getSVD( );

        notifyActionListeners( this, FIRST_PASS_SVD_COMPUTED, FIRST_PASS_FINISHED_CMD );
        
        boolean once = false;
        boolean once2ndPass = false;
        double[] fid_water_first_pass = null;
        double[] xaxis;
        double[] xaxis2;
        double[] usedSVD = new double[svdModel.getmvalue( )];
        for ( int j = 0; j < m_numberOfSingularValuesToCheck; j++ )
        {
            for( int i=0; i < m_numberOfSingularValuesToCheck; i++ )
            {
                if ( i == j )
                {
                    usedSVD[i] = m_singularValues1stPass[i];
                }
                else
                {
                    usedSVD[i] = 0.0;
                }
            }
        
            svdModel.setSVD( usedSVD );
        
            try 
            {
                
                fid_water_first_pass = svdModel.getWaterSignal( );
                
                if ( fid_water_first_pass != null )
                {
                    if ( m_fid_water_first_pass_sum == null )
                    {
                        m_fid_water_first_pass_sum = new double[fid_water_first_pass.length];
                    }
                    
                    for ( int k = 0; k < fid_water_first_pass.length; k++ )
                    {
                        m_fid_water_first_pass_sum[k] += fid_water_first_pass[k];
                    }
                    
                    double[] fidReal = new double[specData.getNumberOfSamplePoints( )];
                    for ( int k = 0; k < fid_water_first_pass.length/2 - 1; k++ )
                    {
                        fidReal[k] = fid_water_first_pass[2*k];
                    }
                }             
            }
            catch ( JampackException ex ) 
            {
                Logger.getLogger(SVDWaterLineRemover.class.getName( ) ).log( Level.SEVERE, null, ex );
            }
        }
        
        notifyActionListeners( this, FIRST_PASS_FINISHED, FIRST_PASS_FINISHED_CMD );
        
        
        //////////////////////////////////////////////////////////////////////
        // Now the second SVD must be applied to the sum water 
        // signal of the first pass.
        /////////////////////////////////////////////////////////////////////
        
        SVDMVCModel svdModel2ndPass= new SVDMVCModel( );
        svdModel2ndPass.setmaxNbComplex( specData.getNumberOfSamplePoints( ) );

        for( int i = 0; i < m_fid_water_first_pass_sum.length/2; i++ )
        {
            real[i] = (double)m_fid_water_first_pass_sum[2*i];
            imag[i] = (double)m_fid_water_first_pass_sum[2*i+1];
        }

        svdModel2ndPass.setfidRe( real );
        svdModel2ndPass.setfidIm( imag );
        
        try 
        {
            svdModel2ndPass.essaiZsvd( );
        }
        catch ( JampackException e ) 
        {
            System.out.println( "An exception occurred while computation the SVD of the signal." );
        }

        m_singularValues2ndPass = svdModel2ndPass.getSVD( );
        notifyActionListeners( this, SECOND_PASS_SVD_COMPUTED, SECOND_PASS_SVD_COMPUTED_CMD );
       
        double[] usedSVD2ndPass = new double[svdModel2ndPass.getmvalue( )];
        
        for ( int j = 0; j < m_numberOfSingularValuesToCheck/2; j++ )
        {
            for( int i = 0; i < m_numberOfSingularValuesToCheck/2; i++ )
            {
                if( i == j )
                {
                    usedSVD2ndPass[i] = m_singularValues2ndPass[i];
                }
                else
                {
                    usedSVD2ndPass[i] = 0.0;
                }
            }
        
            svdModel2ndPass.setSVD( usedSVD2ndPass );
        
            try 
            {
                
                fid_water_second_pass = svdModel2ndPass.getWaterSignal( );
                
                if ( fid_water_second_pass != null )
                {
                    if ( m_fid_water_second_pass_sum == null )
                    {
                        m_fid_water_second_pass_sum = new double[specData.getNumberOfSamplePoints( )];
                    }
                    
                    for ( int k=0; k < fid_water_second_pass.length; k++ )
                    {
                        m_fid_water_second_pass_sum[k] += fid_water_second_pass[k];
                    }
                    
                    double[] fidReal = new double[fid_water_second_pass.length/2];
                    for ( int k = 0; k < fid_water_second_pass.length/2 - 1; k++ )
                    {
                        fidReal[k] = m_fid_water_second_pass_sum[2*k];
                    }
                
                    if ( fidReal != null )
                    {
                        xaxis2 = new double[ fidReal.length ];
                        for ( int i = 0; i < fidReal.length; i++ )
                        {
                            xaxis2[i] = (double)i;
                        }                 
                    }
                }
            }
            catch ( JampackException ex ) 
            {
                Logger.getLogger(SVDWaterLineRemover.class.getName( ) ).log( Level.SEVERE, null, ex );
            }
        }
        
        notifyActionListeners( this, SECOND_PASS_FINISHED, SECOND_PASS_FINISHED_CMD );
        
        // finally we can return the cleaned water signal
        
        float[] cleanedSpectrumAbs = new float[specData.getNumberOfSamplePoints( )];
        float[] cleanedSpectrumDisp = new float[specData.getNumberOfSamplePoints( )];
        float[] xaxisfloat = new float[specData.getNumberOfSamplePoints( )];
        
        for ( int k = 0; k < specData.getNumberOfSamplePoints( ); k++)
        {
            xaxisfloat[k] = (float)k;
            cleanedSpectrumAbs[k] = specData.getAbsorptionChannelTD()[k] - (float)m_fid_water_second_pass_sum[2*k];
            cleanedSpectrumDisp[k] = specData.getDispersionChannelTD()[k] - (float)m_fid_water_second_pass_sum[2*k+1];
        }
        
        SingleVoxelMeasuredSpectralData cleanedSpectrum = new SingleVoxelMeasuredSpectralData( cleanedSpectrumAbs, cleanedSpectrumDisp );
        XYPlotFrame xyPlotCleanedSpectrumFrame = new XYPlotFrame( "Cleaned Spectrum", null );
        
        xyPlotCleanedSpectrumFrame.addCurve( new Curve( xaxisfloat, cleanedSpectrum.getAbsorptionChannelFD( ) ), true );
        xyPlotCleanedSpectrumFrame.addCurve( new Curve( xaxisfloat, specData.getAbsorptionChannelFD( ), Color.red ), true );
        
        return cleanedSpectrum;
    }
    
    /**
     * 
     * Returns the double array containing the singular values 
     * of the first pass.
     * 
     * @return 
     */
    public double[] getSingularValues1stPass( )
    {
        return m_singularValues1stPass;
    }
    
    /**
     * 
     * Returns the double array containing the singular values 
     * of the first pass.
     * 
     * @return 
     */
    public double[] getSingularValues2ndPass( )
    {
        return m_singularValues2ndPass;
    }
    
    /**
     * 
     * Returns the water signal as result of the first pass of the water 
     * removal algorithm.
     * 
     * @return 
     */
    public double[] getWaterSignal1stPass( )
    {
        return m_fid_water_first_pass_sum;
    }
    
     /**
     * 
     * Returns the water signal as result of the first pass of the water 
     * removal algorithm.
     * 
     * @return 
     */
    public double[] getWaterSignal2ndPass( )
    {
        return m_fid_water_second_pass_sum;
    }
    
    
    
    /**
     * Adds a listener to the list of listeners to this class.
     * @param listener 
     */
    public void addActionListener( ActionListener listener ) 
    {
        m_listeners.add( listener );
    }
    
    /**
     * Notifies actionEvent to all registered listeners.
     */
    private void notifyActionListeners( Object obj, int eventType, String actionCommand ) 
    {
        Iterator it = m_listeners.iterator( );
        while ( it.hasNext( ) ) 
        {
            ( ( ActionListener ) it.next( ) ).actionPerformed( new ActionEvent( obj, eventType, actionCommand ) );
        }
    } 
    
}
