/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.waterremoval;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Iterator;
import java.util.LinkedList;
import javax.swing.JFrame;
import org.spectrim.tdfdfit.SingleVoxelMeasuredSpectralData;
import org.spectrim.xyplot.Curve;

/**
 * 
 * Graphical user interface to control residual water removal using 
 * SVDWaterLineRemoverGUI.
 * 
 * @author Johannes Slotboom <johannes.slotboom at insel.ch>
 */
public class SVDWaterLineRemoverGUI extends javax.swing.JFrame implements ActionListener 
{

    public final int CANCEL_EVENT = 0;
    public final int ACCEPT_EVENT = 1;
    public final int CHANGE_EVENT = 2;
    public LinkedList<SingleVoxelMeasuredSpectralData> m_responseList;
    SingleVoxelMeasuredSpectralData m_filteredOutputSignal = null;
    
    /**
     * Creates new form SVDWaterLineRemoverGUI.
     * @param responseList
     */
    public SVDWaterLineRemoverGUI( LinkedList<SingleVoxelMeasuredSpectralData> responseList ) 
    {
        initComponents( );
        setDefaultCloseOperation( JFrame.DISPOSE_ON_CLOSE );
        m_responseList = responseList;
        m_originalSignalXYPlot.setDisplayXReversed( true );
        m_originalSignalXYPlot.setTitle( "Input spectrum" );
        m_originalSignalXYPlot.addCurve( m_responseList.getFirst( ).getAbsorptionCurveFDPoints( ), true );
        setTitle( "SVD residual water remover" );
        
        addWindowListener( 
            new WindowAdapter( )
            {
                @Override
                public void windowClosing( WindowEvent event )
                {
                  ((SVDWaterLineRemoverGUI)event.getSource()).dispose( );
                }
            }
        );
         
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings( "unchecked" )
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        m_setFrequencyWidthToRemovePanel = new javax.swing.JPanel();
        m_setFrequencyWidthLabel = new javax.swing.JLabel();
        m_svdValuesToBeRemovedTextField = new javax.swing.JTextField();
        m_optionsPanel = new javax.swing.JPanel();
        m_applyButton = new javax.swing.JButton();
        m_quitButton = new javax.swing.JButton();
        m_testWaterRemovalButton = new javax.swing.JButton();
        jPanel3 = new javax.swing.JPanel();
        m_resultXYPlot = new org.spectrim.xyplot.XYPlot();
        m_waterFIDXYPlot = new org.spectrim.xyplot.XYPlot();
        m_originalSignalXYPlot = new org.spectrim.xyplot.XYPlot();
        m_singularValuesXYPlot = new org.spectrim.xyplot.XYPlot();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("HLSVDpro residual water removal");

        m_setFrequencyWidthLabel.setText("Frequency range to remove (points)");

        m_svdValuesToBeRemovedTextField.setText("10");

        javax.swing.GroupLayout m_setFrequencyWidthToRemovePanelLayout = new javax.swing.GroupLayout(m_setFrequencyWidthToRemovePanel);
        m_setFrequencyWidthToRemovePanel.setLayout(m_setFrequencyWidthToRemovePanelLayout);
        m_setFrequencyWidthToRemovePanelLayout.setHorizontalGroup(
            m_setFrequencyWidthToRemovePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(m_setFrequencyWidthToRemovePanelLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(m_setFrequencyWidthLabel)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addComponent(m_svdValuesToBeRemovedTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 59, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );
        m_setFrequencyWidthToRemovePanelLayout.setVerticalGroup(
            m_setFrequencyWidthToRemovePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(m_setFrequencyWidthToRemovePanelLayout.createSequentialGroup()
                .addGap(26, 26, 26)
                .addGroup(m_setFrequencyWidthToRemovePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_setFrequencyWidthLabel)
                    .addComponent(m_svdValuesToBeRemovedTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        m_applyButton.setText("Apply");
        m_applyButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_applyButtonActionPerformed(evt);
            }
        });

        m_quitButton.setText("Quit");
        m_quitButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_quitButtonActionPerformed(evt);
            }
        });

        m_testWaterRemovalButton.setText("Test water removal");
        m_testWaterRemovalButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_testWaterRemovalButtonActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout m_optionsPanelLayout = new javax.swing.GroupLayout(m_optionsPanel);
        m_optionsPanel.setLayout(m_optionsPanelLayout);
        m_optionsPanelLayout.setHorizontalGroup(
            m_optionsPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(m_optionsPanelLayout.createSequentialGroup()
                .addGap(67, 67, 67)
                .addComponent(m_applyButton)
                .addGap(310, 310, 310)
                .addComponent(m_testWaterRemovalButton)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addComponent(m_quitButton)
                .addGap(76, 76, 76))
        );
        m_optionsPanelLayout.setVerticalGroup(
            m_optionsPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(m_optionsPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(m_optionsPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_applyButton)
                    .addComponent(m_quitButton)
                    .addComponent(m_testWaterRemovalButton))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        m_resultXYPlot.setPreferredSize(new java.awt.Dimension(384, 384));

        m_waterFIDXYPlot.setPreferredSize(new java.awt.Dimension(384, 384));

        javax.swing.GroupLayout jPanel3Layout = new javax.swing.GroupLayout(jPanel3);
        jPanel3.setLayout(jPanel3Layout);
        jPanel3Layout.setHorizontalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel3Layout.createSequentialGroup()
                .addComponent(m_resultXYPlot, javax.swing.GroupLayout.PREFERRED_SIZE, 512, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(m_waterFIDXYPlot, javax.swing.GroupLayout.PREFERRED_SIZE, 512, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(0, 0, Short.MAX_VALUE))
        );
        jPanel3Layout.setVerticalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel3Layout.createSequentialGroup()
                .addGroup(jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(m_resultXYPlot, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(m_waterFIDXYPlot, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        m_originalSignalXYPlot.setPreferredSize(new java.awt.Dimension(384, 384));

        m_singularValuesXYPlot.setPreferredSize(new java.awt.Dimension(384, 384));

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(m_setFrequencyWidthToRemovePanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addGroup(layout.createSequentialGroup()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                                .addComponent(m_optionsPanel, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                .addComponent(jPanel3, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                            .addGroup(layout.createSequentialGroup()
                                .addComponent(m_originalSignalXYPlot, javax.swing.GroupLayout.PREFERRED_SIZE, 512, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(m_singularValuesXYPlot, javax.swing.GroupLayout.PREFERRED_SIZE, 512, javax.swing.GroupLayout.PREFERRED_SIZE)))
                        .addGap(0, 12, Short.MAX_VALUE))))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addComponent(m_setFrequencyWidthToRemovePanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(30, 30, 30)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(m_originalSignalXYPlot, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(m_singularValuesXYPlot, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jPanel3, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(m_optionsPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void m_applyButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_applyButtonActionPerformed
        actionPerformed( evt );
    }//GEN-LAST:event_m_applyButtonActionPerformed

    private void m_quitButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_quitButtonActionPerformed
        actionPerformed( evt );
    }//GEN-LAST:event_m_quitButtonActionPerformed

    private void m_testWaterRemovalButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_testWaterRemovalButtonActionPerformed
        m_waterRemover = new SVDWaterLineRemover( );
        m_waterRemover.addActionListener( this );
        m_waterRemover.performSVDLineRemover( m_responseList.getFirst( ), new Integer( m_svdValuesToBeRemovedTextField.getText() ) );
    }//GEN-LAST:event_m_testWaterRemovalButtonActionPerformed

    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        /* Set the Nimbus look and feel */
        //<editor-fold defaultstate="collapsed" desc=" Look and feel setting code (optional) ">
        /* If Nimbus (introduced in Java SE 6) is not available, stay with the default look and feel.
         * For details see http://download.oracle.com/javase/tutorial/uiswing/lookandfeel/plaf.html 
         */
        try {
            for (javax.swing.UIManager.LookAndFeelInfo info : javax.swing.UIManager.getInstalledLookAndFeels()) {
                if ("Nimbus".equals(info.getName())) {
                    javax.swing.UIManager.setLookAndFeel(info.getClassName());
                    break;
                }
            }
        } catch (ClassNotFoundException ex) {
            java.util.logging.Logger.getLogger(SVDWaterLineRemoverGUI.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (InstantiationException ex) {
            java.util.logging.Logger.getLogger(SVDWaterLineRemoverGUI.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (IllegalAccessException ex) {
            java.util.logging.Logger.getLogger(SVDWaterLineRemoverGUI.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (javax.swing.UnsupportedLookAndFeelException ex) {
            java.util.logging.Logger.getLogger(SVDWaterLineRemoverGUI.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        }
        //</editor-fold>
        //</editor-fold>

        /* Create and display the form */
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
                new SVDWaterLineRemoverGUI(null).setVisible(true);
            }
        });
    }
    
    /**
     * Returns the integer value of the number of singular values to be removed. 
     * @return 
     */
    public int getNumberOfSVDsToBeRemoved( )
    {
        return new Integer( m_svdValuesToBeRemovedTextField.getText( ) );
    }
    
    /**
     * Adds an ActionListener to the lists of ActionListeners
     * @param listener
     */
    public void addActionListener( ActionListener listener )
    {
        m_listeners.add( listener );
    }
    
    /**
     * Handles all ActionEvents of registered components.
     * @param e 
     */
    @Override
    public void actionPerformed( ActionEvent e ) 
    {
        
        Object sender = e.getSource( );
        
        if ( sender == m_waterRemover )
        {
            if ( e.getID( ) == SVDWaterLineRemover.FIRST_PASS_SVD_COMPUTED )
            {
                double[] array = ((SVDWaterLineRemover)sender).getSingularValues1stPass( );
                double[] xaxis = new double[array.length];
                for ( int i = 0; i < array.length; i++ )
                {
                    xaxis[i]=(double)i;
                }
                m_singularValuesXYPlot.setTitle( "Singular values first pass" );
                m_singularValuesXYPlot.addCurve( new Curve( xaxis, array ), true );
            }
            
            if ( e.getID( ) == SVDWaterLineRemover.FIRST_PASS_FINISHED )
            {
                double[] array = ((SVDWaterLineRemover)sender).getWaterSignal1stPass( );
                
                float[] abs   = new float[array.length/2];
                float[] disp  = new float[array.length/2];
                for ( int i = 0; i < array.length/2; i++ )
                {
                    abs[i] = (float)array[2*i];
                    disp[i] = (float)array[2*i+1];
                }
                m_waterFIDXYPlot.setTitle( "Water signal after first pass." );
                m_waterFIDXYPlot.setDisplayXReversed( true );
                SingleVoxelMeasuredSpectralData data = new SingleVoxelMeasuredSpectralData( abs, disp );
                m_waterFIDXYPlot.addCurve( data.getAbsorptionCurveFDPoints( ), true );
            }
            
            if ( e.getID( ) == SVDWaterLineRemover.SECOND_PASS_SVD_COMPUTED )
            {
                double[] array = ((SVDWaterLineRemover)sender).getSingularValues2ndPass( );
                double[] xaxis = new double[array.length];
                for ( int i = 0; i < array.length; i++ )
                {
                    xaxis[i]=(double)i;
                }
                
                m_singularValuesXYPlot.setTitle( "Singular values first pass" );
                m_singularValuesXYPlot.addCurve( new Curve( xaxis, array, Color.RED ), true );
            }
            
            if ( e.getID( ) == SVDWaterLineRemover.SECOND_PASS_FINISHED )
            {
                double[] array = ((SVDWaterLineRemover)sender).getWaterSignal2ndPass( );
                
                float[] abs   = new float[array.length/2];
                float[] disp  = new float[array.length/2];
                for ( int i = 0; i < array.length/2; i++ )
                {
                    abs[i] = (float)array[2*i];
                    disp[i] = (float)array[2*i+1];
                }
                
                SingleVoxelMeasuredSpectralData inputData = m_responseList.getFirst( );
                
                SingleVoxelMeasuredSpectralData data = new SingleVoxelMeasuredSpectralData( abs, disp );
                m_waterFIDXYPlot.setTitle( "Water signal after first and second pass." );
                m_waterFIDXYPlot.addCurve( data.getAbsorptionCurveFDPoints( ), true );
                
                inputData.subtractSingleVoxelSpectralDataObject(data);
                m_resultXYPlot.setTitle( "Water removed response" );
                m_resultXYPlot.setDisplayXReversed( true );
                m_resultXYPlot.addCurve( inputData.getAbsorptionCurveFDPoints( ), true );
                
                float[] real = inputData.getAbsorptionChannelFD( );
                float[] xaxis = inputData.getAbsorptionCurveFDPoints( ).getXvalues();
                
                m_originalSignalXYPlot.setDisplayXReversed( true );
                m_originalSignalXYPlot.setTitle( "Input spectrum & water removed specrum" );
                m_originalSignalXYPlot.addCurve( new Curve( xaxis, real, Color.RED ), true );
                
                
                m_filteredOutputSignal = new SingleVoxelMeasuredSpectralData( inputData );
            }
        }
        
        if ( sender == m_applyButton )
        {
            Iterator it = m_listeners.iterator( );
            while( it.hasNext( ) )
            {
                ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, CHANGE_EVENT, "" ) );
            }
            dispose();
        }
        
        if ( sender == m_quitButton )
        {
            Iterator it = m_listeners.iterator( );
            while( it.hasNext( ) )
            {
                ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, CANCEL_EVENT, "" ) );
            }
        }
        
    }
    
    /**
     * Returns the SingleVoxelMeasuredSpectralData object that contains the
     * water removed response.
     * @return 
     */
    public SingleVoxelMeasuredSpectralData getFilteredOutputSpectrum( )
    {
        return m_filteredOutputSignal;
    }
    
    // Listeners to changes in this class
    private final LinkedList<ActionListener> m_listeners = new LinkedList<>( );
    private SVDWaterLineRemover m_waterRemover;
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel jPanel3;
    private javax.swing.JButton m_applyButton;
    private javax.swing.JPanel m_optionsPanel;
    private org.spectrim.xyplot.XYPlot m_originalSignalXYPlot;
    private javax.swing.JButton m_quitButton;
    private org.spectrim.xyplot.XYPlot m_resultXYPlot;
    private javax.swing.JLabel m_setFrequencyWidthLabel;
    private javax.swing.JPanel m_setFrequencyWidthToRemovePanel;
    private org.spectrim.xyplot.XYPlot m_singularValuesXYPlot;
    private javax.swing.JTextField m_svdValuesToBeRemovedTextField;
    private javax.swing.JButton m_testWaterRemovalButton;
    private org.spectrim.xyplot.XYPlot m_waterFIDXYPlot;
    // End of variables declaration//GEN-END:variables
}
