/*
 * File: $Id: XYPlotSlider.java,v 1.1.1.1 2006/04/19 09:38:51 hans Exp $
 * History
 * Date: 05.04.2005, 3:36 PM Hans Slotboom  Changes:
 *
 * Copyright (C) 2003 - 2005 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

/*
 * XYPlotSlider.java
 *
 * Created on April 5, 2005, 3:36 PM
 */

package org.spectrim.xyplot;

import java.awt.Graphics;
import java.awt.Color;

import javax.swing.JPanel;

/**
 * An XYPlotSlider is an object that allows for being dragged over an XYPlot
 * and can serve as a marker for a border, i.e. a minimal or maximal value
 * indicator
 * @author  hans
 */
public class XYPlotSlider 
{
    
    /**
     * Creates a new instance of XYPlotSlider 
     */
    public XYPlotSlider( JPanel parent )
    {
    }

    /**
     * Creates a new instance of XYPlotSlider 
     */
    public XYPlotSlider( JPanel parent,
                         int currentPosition, int orientation )
    {
        
        m_parent = parent;

        m_currentPosition = currentPosition;
        m_orientation = orientation;
    }
    
    /**
     * Draws the XYPlotSlider on the Graphics object
     */
    public void drawSlider( Graphics g )
    {
         
        if ( m_currentPlotWidth == -1 )
        {
            m_currentPlotWidth = m_parent.getWidth( );   
        }
        
        if ( m_currentPlotHeight == -1 )
        {
            m_currentPlotHeight = m_parent.getHeight( );
        }
        
        if ( m_parent.getWidth() != m_currentPlotWidth  )
        {
            
            if ( this.m_orientation == this.VERTICAL_SLIDER )
            {                
                m_currentPosition = Math.round( (float)m_currentPosition * (float)m_parent.getWidth( )  / (float)m_currentPlotWidth );
            }
            
            m_currentPlotWidth = m_parent.getWidth( );
        }
        
        if ( m_parent.getHeight() != m_currentPlotHeight  )
        {
            if ( this.m_orientation == this.HORIZONTAL_SLIDER )
            {
                m_currentPosition = Math.round( (float)m_currentPosition * (float)m_parent.getHeight( ) / (float)m_currentPlotHeight );
            }
            m_currentPlotHeight = m_parent.getHeight( );
        }
        
        // set color back to original
        Color c = g.getColor( );
        
        m_minPosibleXValue = m_parent.getWidth( ) / 10;
        m_maxPosibleXValue = 9 * m_parent.getWidth( ) / 10;
        m_minPosibleYValue = m_parent.getHeight( ) / 10;
        m_maxPosibleYValue = 9 * m_parent.getHeight( ) / 10;
        
        if ( m_orientation == HORIZONTAL_SLIDER ) 
        {
            if ( m_isSelected == true )
            {
                g.setColor( Color.BLUE );
                g.drawLine( m_minPosibleXValue, m_currentPosition, m_maxPosibleXValue, m_currentPosition );
                g.setColor( Color.RED );
                g.drawRect( ( m_minPosibleXValue + m_maxPosibleXValue ) / 2, m_currentPosition - 4, 8, 8 );
            }
            else
            {
                g.setColor( m_lineColor );
                g.drawLine( m_minPosibleXValue, m_currentPosition, m_maxPosibleXValue, m_currentPosition );
                g.drawRect( ( m_minPosibleXValue + m_maxPosibleXValue ) / 2, m_currentPosition - 4, 8, 8 );                
            }
        }
       
        if ( m_orientation == VERTICAL_SLIDER ) 
        {
            if ( m_isSelected == true )
            {
                g.setColor( Color.BLUE );
                g.drawLine( m_currentPosition, m_minPosibleYValue, m_currentPosition, m_maxPosibleYValue );
                g.setColor( Color.RED );
                g.drawRect( m_currentPosition - 4, ( m_minPosibleYValue + m_maxPosibleYValue ) / 2, 8, 8 );
            }
            else
            {
                g.setColor( m_lineColor );
                g.drawLine( m_currentPosition, m_minPosibleYValue, m_currentPosition, m_maxPosibleYValue );
                g.drawRect( m_currentPosition - 4, ( m_minPosibleYValue + m_maxPosibleYValue ) / 2, 8, 8 );
            }
        }
        // set color back to original
        g.setColor( c );
    }
    
    /**
     * Sets the slider selected, i.e. it is drawn in black with a yellow anchor
     */
    public void setSelected( boolean selected )
    {
        m_isSelected = selected;
    }
    
    /**
     * Sets the slider selected, i.e. it is drawn in black with a yellow anchor
     */
    public boolean isSelected( )
    {
        return m_isSelected;
    }
    
    /**
     * Detects whether one of the sliders is selected
     */
    public boolean isSliderSelected( int xCoord, int yCoord )
    {
        
        if ( m_orientation == VERTICAL_SLIDER )
        {
            if ( ( xCoord > m_currentPosition - 4 ) && ( xCoord <  m_currentPosition + 4 ) &&
                 ( yCoord > ( ( m_minPosibleYValue + m_maxPosibleYValue ) / 2 ) - 4 ) &&
                 ( yCoord < ( ( m_minPosibleYValue + m_maxPosibleYValue ) / 2 ) + 4  ) )
            {
                m_isSelected = true;
                return true;
            }
        }
        
        if ( m_orientation == HORIZONTAL_SLIDER )
        {
            if ( ( yCoord > m_currentPosition - 4 ) && ( yCoord <  m_currentPosition + 4 ) &&
                 ( xCoord > ( ( m_minPosibleXValue + m_maxPosibleXValue ) / 2 ) - 4 ) &&
                 ( xCoord < ( ( m_minPosibleXValue + m_maxPosibleXValue ) / 2 ) + 4  ) )
            {
                m_isSelected = true;
                return true;
            }
        }

        m_isSelected = false;
        return false;
    }
    
    /**
     * Sets a new value for the slider to be drawn; it does not envoke any redrawing !
     */
    public void setNewSliderPosition( int newPosition )
    {
        m_currentPosition = newPosition;
    }
    
    /**
     * Sets a new value for the slider to be drawn; it does not envoke any redrawing !
     */
    public void setNewSliderPosition( int xCoord, int yCoord )
    {
        if ( m_orientation == this.VERTICAL_SLIDER )
        {
            m_currentPosition = xCoord;
        }
        
        if ( m_orientation == this.HORIZONTAL_SLIDER )
        {
            m_currentPosition = yCoord;
        }
    }
    
    /**
     * Returns the cuurent position of the slider
     */
    public int getCurrentPosition( )
    {
        return m_currentPosition;
    }
    
    // Attributes
    private boolean m_isSelected = false;
    
    private JPanel m_parent;
    public final static int HORIZONTAL_SLIDER = 0;
    public final static int VERTICAL_SLIDER   = 1;
    
    private int m_orientation = VERTICAL_SLIDER;
    private int m_currentPosition;

    private int m_minPosibleXValue;
    private int m_maxPosibleXValue;
    
    private int m_minPosibleYValue;
    private int m_maxPosibleYValue;
    
    private Color m_lineColor = Color.GRAY;
    
    private int m_currentPlotWidth = -1;
    private int m_currentPlotHeight = -1;
    
}
