/*
 * File: $Id: DicomSliceCanvasPolyDraw.java,v 1.6 2006/11/09 10:11:32 hans Exp $
 * History
 * Date: 31.01.2003  Authors: Martin Buerki / Hans Slotboom  
 *
 * Changes:
 *
 * Copyright (C) 2003 - 2011 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

package org.spectrim.dicom;


import org.spectrim.imageprocessing.MorphologicFilterFrame;
import org.spectrim.imageprocessing.AdvancedInteractiveSegmentationAnalyzerFrame;
import java.awt.*;
import java.awt.event.MouseWheelEvent;
import java.awt.image.BufferedImage;

import java.awt.image.PixelGrabber;
import java.awt.image.ImageObserver;

import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.ActionEvent;
import java.awt.geom.Point2D;

import java.text.DecimalFormat;
import java.text.NumberFormat;

import java.util.Vector;
import java.util.LinkedList;
import java.util.Iterator;

import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JPopupMenu;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.ButtonGroup;
import javax.swing.JFrame;

import org.spectrim.lib.Tiff;
import org.spectrim.lib.EnterStringFrame;

import org.spectrim.imagegraphicobjects.Distance;
import org.spectrim.imagegraphicobjects.PolygonEventCaster;
import org.spectrim.imagegraphicobjects.RegioPolygon;
import org.spectrim.imagegraphicobjects.PolygonClipboard;
import org.spectrim.imagegraphicobjects.PolygonListener;
import org.spectrim.imagegraphicobjects.PolygonEvent;
import org.spectrim.imagegraphicobjects.RegioPolygonPixelDistributionHistogram;
import org.spectrim.imagegraphicobjects.MirrorAxis;
import org.spectrim.imagegraphicobjects.Point3D;
import org.spectrim.imagegraphicobjects.RegioPolygonConnections;
import org.spectrim.imagegraphicobjects.RegionsInVectorFileFormatReader;
import org.spectrim.imagegraphicobjects.ScaledROIDefintionFrame;
import org.spectrim.imageprocessing.ExtendedRegionPropertiesTableFrame;
import org.spectrim.imageprocessing.ImageOverlayPixelRemover;
import org.spectrim.imageprocessing.ImageOverlaySegmentHistogramCalculator;
import org.spectrim.imageprocessing.ImageParametersWithinARegioPolygon;
import org.spectrim.imageprocessing.ImageRegionProperties;
import org.spectrim.imageprocessing.RegionTTesterFrame;
import org.spectrim.imageprocessing.SimpleThressholdSegmentationAnalyzerFrame;
import org.spectrim.imageprocessing.ImageRestorationFilterFrame;
import org.spectrim.imageprocessing.ImageSegmentProperties;
import org.spectrim.imageprocessing.ImageStackTextureAnalyzerFrame;
import org.spectrim.imageprocessing.MHAFileOverlayReader;
import org.spectrim.imageprocessing.orderstatisticsegmentation.OrderStatisticsSegmentationAnalyzerFrame;
import org.spectrim.imageprocessing.MeasureDistancesFrame;
import org.spectrim.imageprocessing.MeasuredDistancesFile;
import org.spectrim.imageprocessing.MomentAboutTheMeanOfArrayCalculator;
import org.spectrim.imageprocessing.MomentsAboutTheMean;
import org.spectrim.imageprocessing.MomentsAboutTheMeanOfRegionCalculator;
import org.spectrim.imageprocessing.MomentsAboutTheMeanOfVOICalculator;
import org.spectrim.imageprocessing.orderstatisticsegmentation.OrderStatisticSegmentationCalculator;
import org.spectrim.imageprocessing.OverlayPropertiesTable;
import org.spectrim.imageprocessing.PixelValueReplacementFrame;
import org.spectrim.imageprocessing.ProbingRegionPropertiesPanel;
import org.spectrim.imageprocessing.QuickSort;
import org.spectrim.imageprocessing.RegioContoursByMorphologicFiltering;
import org.spectrim.imageprocessing.RegionPropertiesTable;
import org.spectrim.imageprocessing.SimpleThressholdSegmentationCalculator;
import org.spectrim.imageprocessing.VolumeOfInterestOverlayManipulationFrame;
import org.spectrim.imageprocessing.VolumeOfInterestOverlayPropertiesTable;
import org.spectrim.imageprocessing.VolumeOfInterestStatistics;
import org.spectrim.lib.BrainVoyagerFBRFileParserAndLoader;
import org.spectrim.lib.BrainVoyagerVOIFileParserAndLoader;
import org.spectrim.lib.Dialogs;
import org.spectrim.lib.DualProgressBar;
import org.spectrim.lib.EnterDecimalValueFrame;
import org.spectrim.lib.FileUtils;
import org.spectrim.lib.Globals;
import org.spectrim.lib.MessageBox;
import org.spectrim.lib.RegFile;
import org.spectrim.lib.SwingWorker;
import org.spectrim.lib.TextFileWriter;
import org.spectrim.lib.QuestionDialog;
import org.spectrim.signalprocessing.SignalTools;
import org.spectrim.signalprocessing.SignalWriter;
import org.spectrim.xyplot.Curve;
import org.spectrim.xyplot.XYPlotFrame;

import java.awt.event.MouseWheelListener;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedHashMap;
import javax.imageio.ImageIO;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileNameExtensionFilter;
import org.spectrim.imagegraphicobjects.Dicom3DSlicerControlApp;


public class DicomSliceCanvasPolyDraw extends DicomSliceCanvas implements MouseListener, MouseMotionListener, ActionListener, MouseWheelListener
{
    
    
    /**
     * Main class for displaying and manipulation of Dicom image stack 
     * information.
     * @param parent 
     */
    @SuppressWarnings("LeakingThisInConstructor")
    public DicomSliceCanvasPolyDraw( JFrame parent )
    {
        super( parent );
        
        m_parent = parent;
        
        this.setDoubleBuffered( true );
        
        this.setOpaque( true );
        
        m_polygonsVector = new Vector< LinkedList<RegioPolygon> >( );

        m_distancesVector = new Vector< LinkedList<Distance> >( );

        multicaster = new PolygonEventCaster( );
        uniqueNames = true;
	

        // Add a popup menu to the SliceCanvasPolydraw object
        m_popupmenu = new JPopupMenu( );
        
        m_zoomMenu = new JMenu( "Zoom & Flip " );
        
        m_doubleImageDisplay = new JMenuItem( "Double display" ) ;
        m_doubleImageDisplay.addActionListener( this );
        m_zoomMenu.add( m_doubleImageDisplay );
        
        m_halfImageDisplay = new JMenuItem( "Half display" );
        m_halfImageDisplay.addActionListener( this );
        m_zoomMenu.add( m_halfImageDisplay );
        
        m_flipHorizontally = new JMenuItem( "Flip horizontally" );
        m_flipHorizontally.addActionListener( this );
        m_zoomMenu.add( m_flipHorizontally );
        
        m_flipVertically = new JMenuItem( "Flip vertically" );
        m_flipVertically.addActionListener( this );
        m_zoomMenu.add( m_flipVertically );
        
        m_rotateAbout90Degrees = new JMenuItem( "Rotate 90 degrees" );
        m_rotateAbout90Degrees.addActionListener( this );
        m_zoomMenu.add( m_rotateAbout90Degrees );
        
        m_popupmenu.add( m_zoomMenu );
        
        m_dicomHeaderPropertiesMenuItem = new JMenuItem( "View Dicom Header .. " );
        m_dicomHeaderPropertiesMenuItem.addActionListener( this );
        m_popupmenu.add( m_dicomHeaderPropertiesMenuItem );

        if ( m_isJMRUIRelease == false )
        {
            m_editDicomHeaderPropertiesMenuItem = new JMenuItem( "Edit Dicom Header .." );
            m_editDicomHeaderPropertiesMenuItem.addActionListener( this );
            m_popupmenu.add( m_editDicomHeaderPropertiesMenuItem );
        }

        m_siemensSpecificPropertiesMenuItem = new JMenuItem( "View Siemens Specific Dicom Header .." );
        m_siemensSpecificPropertiesMenuItem.addActionListener( this );
        m_popupmenu.add( m_siemensSpecificPropertiesMenuItem );
        
        m_popupmenu.addSeparator( );
        m_showWindowingScale = new JRadioButtonMenuItem( "Windowing scale" );
        m_showWindowingScale.addActionListener( this );
        
        m_popupmenu.add( m_showWindowingScale );
        m_popupmenu.addSeparator( );

        if ( m_isJMRUIRelease == false )
        {
            m_showImageAnnotation = new JRadioButtonMenuItem( "Image annotation" );
            m_showImageAnnotation.addActionListener( this );
         
            m_popupmenu.add( m_showImageAnnotation );
        }

        m_popupmenu.addSeparator( );
            
        m_colorTableSelection = new JMenu( "Color Table" );
        
        m_displayGrayMenuItem = new JRadioButtonMenuItem( "Gray scale" );
        m_displayGrayMenuItem.addActionListener( this );
        m_colorTableSelection.add( m_displayGrayMenuItem ); 

        m_displayColorMenuItem = new JRadioButtonMenuItem( "Color" );
        m_displayColorMenuItem.addActionListener( this );
        m_colorTableSelection.add( m_displayColorMenuItem ); 
        
        m_displayColorInverseMenuItem = new JRadioButtonMenuItem( "Color Inverse" );
        m_displayColorInverseMenuItem.addActionListener( this );
        m_colorTableSelection.add( m_displayColorInverseMenuItem );
        
        m_popupmenu.add( m_colorTableSelection );
        m_popupmenu.addSeparator( );
        
        m_colorTableChoice = new ButtonGroup( );
        m_colorTableChoice.add( m_displayGrayMenuItem );
        m_colorTableChoice.add( m_displayColorMenuItem );
        m_colorTableChoice.add( m_displayColorInverseMenuItem );
        
        
        // main menu entry for activities related to region manipulations
        m_imageProcessing = new JMenu( "Image Processing" );
        m_imageProcessing.setToolTipText( "Allows image processing operations" );
        
        m_imageRestorationFilters = new JMenuItem( "Image Restoration Filtering" );
        m_imageRestorationFilters.addActionListener( this );
        
        m_imageProcessing.add( m_imageRestorationFilters );
        
        m_morphologicFilter = new JMenuItem( "Morphologic Filtering" );
        m_morphologicFilter.addActionListener( this );
        
        m_imageProcessing.add( m_morphologicFilter );
        
        m_changePixelValuesBasedonUserThressholds = new JMenuItem( "Change Pixel values .." );
        m_changePixelValuesBasedonUserThressholds.addActionListener( this );
        
        m_imageProcessing.add( m_changePixelValuesBasedonUserThressholds );

        m_createTextureAnalyzerMenuItem = new JMenuItem( "Open image stack in texture analyzer" );
        m_createTextureAnalyzerMenuItem.addActionListener( this );

        m_imageProcessing.add( m_createTextureAnalyzerMenuItem );


        m_popupmenu.add( m_imageProcessing );

        if ( m_isJMRUIRelease == false )
        {
            // main menu entry for activities related to region manipulations
            m_region = new JMenu( "Regions" );

            // Submenu that contain
            m_regionMode = new JMenu( "ROI Mode" );
            m_canvasMode = new ButtonGroup( );

            m_fieldOfInterestMenuItem = new JRadioButtonMenuItem( "Currently selected ROI is field of interest" );
            m_region.add( m_fieldOfInterestMenuItem );
            m_fieldOfInterestMenuItem.addActionListener( this );
            
            
            m_canvasMode = new ButtonGroup( );
            
            m_selectROIButton = new JRadioButtonMenuItem( "Select ROI" );
            m_regionMode.add( m_selectROIButton );
            m_selectROIButton.addActionListener( this );
            m_canvasMode.add( m_selectROIButton );
            m_selectROIButton.setSelected( true );

            m_addROI = new JRadioButtonMenuItem( "Add ROI" );
            m_regionMode.add( m_addROI );
            m_addROI.addActionListener( this );
            m_canvasMode.add( m_addROI );

            m_editROI = new JRadioButtonMenuItem( "Edit ROI" );
            m_regionMode.add( m_editROI  );
            m_editROI.addActionListener( this );
            m_canvasMode.add( m_editROI  );

            m_deleteROI = new JRadioButtonMenuItem( "Delete ROI" );
            m_regionMode.add( m_deleteROI );
            m_deleteROI.addActionListener( this );
            m_canvasMode.add( m_deleteROI );

            m_moveROI = new JRadioButtonMenuItem( "Move ROI" );
            m_regionMode.add( m_moveROI );
            m_moveROI.addActionListener( this );
            m_canvasMode.add( m_moveROI );

            m_cutROI = new JRadioButtonMenuItem( "Cut ROI" );
            m_regionMode.add( m_cutROI );
            m_cutROI.addActionListener( this );
            m_canvasMode.add( m_cutROI );

            m_addStackedROIMode = new JRadioButtonMenuItem( "Add stacked ROIs" );
            m_regionMode.add( m_addStackedROIMode );
            m_addStackedROIMode.addActionListener( this );
            m_canvasMode.add( m_addStackedROIMode );        
            
            m_addOneCircularROIMode = new JRadioButtonMenuItem( "Add single circular ROI" );
            m_regionMode.add( m_addOneCircularROIMode );
            m_addOneCircularROIMode.addActionListener( this );
            m_canvasMode.add( m_addOneCircularROIMode );   
            
            m_region.add( m_regionMode );

            m_saveConvertedRegioPolygonsOfAllImagesMenuItem = new JMenuItem( "Save polygons of all images" );
            m_saveConvertedRegioPolygonsOfAllImagesMenuItem.addActionListener( this );
            m_region.add( m_saveConvertedRegioPolygonsOfAllImagesMenuItem );

            m_loadConvertedRegioPolygonsOfAllImagesMenuItem = new JMenuItem( "Load polygons of all images" );
            m_loadConvertedRegioPolygonsOfAllImagesMenuItem.addActionListener( this );
            m_region.add( m_loadConvertedRegioPolygonsOfAllImagesMenuItem );

            m_saveRegioPolygonsAsIOVOfAllImagesMenuItem = new JMenuItem( "Save polygons in IOV-format" );
            m_saveRegioPolygonsAsIOVOfAllImagesMenuItem.addActionListener( this );
            m_region.add( m_saveRegioPolygonsAsIOVOfAllImagesMenuItem );

            m_loadIOVOfAllImagesMenuItem = new JMenuItem( "Load IOV of all images" );
            m_loadIOVOfAllImagesMenuItem.addActionListener( this );
            m_region.add( m_loadIOVOfAllImagesMenuItem );
          
            m_saveConvertedRegioPolygonsOfCurrentlyDisplayedImageOnlyMenuItem = new JMenuItem( "Save polygons of currently displayed images only" );
            m_saveConvertedRegioPolygonsOfCurrentlyDisplayedImageOnlyMenuItem.addActionListener( this );
            m_region.add( m_saveConvertedRegioPolygonsOfCurrentlyDisplayedImageOnlyMenuItem );

            m_loadConvertedRegioPolygonsOfCurrentlyDisplayedImageOnlyMenuItem = new JMenuItem( "Load polygons of currently displayed images only" );
            m_loadConvertedRegioPolygonsOfCurrentlyDisplayedImageOnlyMenuItem.addActionListener( this );
            m_region.add( m_loadConvertedRegioPolygonsOfCurrentlyDisplayedImageOnlyMenuItem );

            m_statisticalPropertiesOfSelectedRegioPolygonMenuItem = new JMenuItem( "Statistical properties of polygons" );
            m_statisticalPropertiesOfSelectedRegioPolygonMenuItem.addActionListener( this );
            m_region.add( m_statisticalPropertiesOfSelectedRegioPolygonMenuItem );
            
            m_statisticalPropertiesOf3CircularRegioPolygonMenuItem = new JMenuItem( "Statistical properties 3 concentric polygons" );
            m_statisticalPropertiesOf3CircularRegioPolygonMenuItem.addActionListener( this );
            m_region.add( m_statisticalPropertiesOf3CircularRegioPolygonMenuItem );
            
        }

        if ( m_isJMRUIRelease == false )
        {
            // image stack volumes
            m_volumeOfInterestMenu = new JMenu( "Volume of interest" );

            m_loadVolumeOfInterestFromDefault = new JMenuItem( "Load default volume of interest" );
            m_loadVolumeOfInterestFromDefault.addActionListener( this );

            m_volumeOfInterestMenu.add( m_loadVolumeOfInterestFromDefault );

            m_saveVolumeOfInterestToDefault = new JMenuItem( "Save to default volume of interest" );
            m_saveVolumeOfInterestToDefault.addActionListener( this );

            m_volumeOfInterestMenu.add( m_saveVolumeOfInterestToDefault );

            m_loadVolumeOfInterestUserDefined = new JMenuItem( "Load volume of interest .." );
            m_loadVolumeOfInterestUserDefined.addActionListener( this );

            m_volumeOfInterestMenu.add( m_loadVolumeOfInterestUserDefined );

            m_saveVolumeOfInterestUserDefined = new JMenuItem( "Save volume of interest .." );
            m_saveVolumeOfInterestUserDefined.addActionListener( this );

            m_volumeOfInterestMenu.add( m_saveVolumeOfInterestUserDefined );

            m_projectionOfVolumeOfInterestInOtherImageStacks = new JMenuItem( "Projection VOI in other image stacks" );
            m_projectionOfVolumeOfInterestInOtherImageStacks.addActionListener( this );
            m_volumeOfInterestMenu.add( m_projectionOfVolumeOfInterestInOtherImageStacks );

            m_selectVolumesToOverlay = new JMenuItem( "Select volumes to overlay .." );
            m_selectVolumesToOverlay.addActionListener( this );
            m_volumeOfInterestMenu.add( m_selectVolumesToOverlay );


            m_opaquacyOfVolumesOverlays = new JMenuItem( "Opacacy of volumes overlays .." );
            m_opaquacyOfVolumesOverlays.addActionListener( this );
            m_volumeOfInterestMenu.add( m_opaquacyOfVolumesOverlays );

            m_eliminatePixelsFromVolumeOfInterestOverlaysByContourDrawingModeRadioButton = new JRadioButtonMenuItem( "Eliminate VOI-overlay pixels by contour drawing" );
            m_volumeOfInterestMenu.add( m_eliminatePixelsFromVolumeOfInterestOverlaysByContourDrawingModeRadioButton );
            
            m_copyVolumeOfInterest = new JMenuItem( "Copy selected volume of interest" );
            m_copyVolumeOfInterest.addActionListener( this );
            m_copyVolumeOfInterest.setEnabled( false );
            m_volumeOfInterestMenu.add( m_copyVolumeOfInterest );

            m_pasteVolumeOfInterest = new JMenuItem( "Paste volume of interest" );
            m_pasteVolumeOfInterest.addActionListener( this );
            m_pasteVolumeOfInterest.setEnabled( false );
            m_volumeOfInterestMenu.add( m_pasteVolumeOfInterest );

            m_deleteVolumeOfInterest = new JMenuItem( "Delete selected volume of interest" );
            m_deleteVolumeOfInterest.addActionListener( this );
            m_volumeOfInterestMenu.add( m_deleteVolumeOfInterest );


            m_mirrorVOI = new JMenu( "Mirrored volumes of interest" );

            m_showVOIMirrorAxis = new JRadioButtonMenuItem( "Show current VOI mirror axis" );
            m_showVOIMirrorAxis.addActionListener( this );
            m_mirrorVOI.add( m_showVOIMirrorAxis );

            // do not show the mirror axis per default
            m_showVOIMirrorAxis.setSelected( false );

            m_editVOIMirrorAxis = new JMenuItem( "Edit mirror axis for VOI.." );
            m_mirrorVOI.add( m_editVOIMirrorAxis );
            m_editVOIMirrorAxis.addActionListener( this );

            m_addMirroredVOI = new JMenuItem( "Add Mirrored VOIs .." );
            m_mirrorVOI.add( m_addMirroredVOI );
            m_addMirroredVOI.addActionListener( this );

            m_volumeOfInterestMenu.add( m_mirrorVOI );

            m_loadMHAOverlay = new JMenuItem( "Load MHA overlay" );
            m_loadMHAOverlay.addActionListener( this );
            m_volumeOfInterestMenu.add( m_loadMHAOverlay );  
            
            
            m_popupmenu.add( m_volumeOfInterestMenu );
        }

        if ( m_isJMRUIRelease == false )
        {
            m_ROI_reportingMenu = new JMenu( "Reporting" );
        
            m_writeDetailedReportOfAllROIsofAllImagesMenuItem = new JMenuItem( "Detailed ROI-report of all images" );
            m_writeDetailedReportOfAllROIsofAllImagesMenuItem.addActionListener( this );
            m_ROI_reportingMenu.add( m_writeDetailedReportOfAllROIsofAllImagesMenuItem );
        
            m_writeCondensedReportOfAllROIsofAllImagesMenuItem = new JMenuItem( "Condensed ROI-report of all images" );
            m_writeCondensedReportOfAllROIsofAllImagesMenuItem.addActionListener( this );
            m_ROI_reportingMenu.add( m_writeCondensedReportOfAllROIsofAllImagesMenuItem );
        
            m_writeDetailedReportOfAllROIsofCurrrentlyDisplayedImageMenuItem = new JMenuItem( "Detailed ROI-report of currently displayed image only" );
            m_writeDetailedReportOfAllROIsofCurrrentlyDisplayedImageMenuItem.addActionListener( this );
            m_ROI_reportingMenu.add( m_writeDetailedReportOfAllROIsofCurrrentlyDisplayedImageMenuItem );
        
            m_writeCondensedReportOfAllROIsofCurrrentlyDisplayedImageMenuItem = new JMenuItem( "Condensed ROI-report of currently displayed image only" );
            m_writeCondensedReportOfAllROIsofCurrrentlyDisplayedImageMenuItem.addActionListener( this );
            m_ROI_reportingMenu.add( m_writeCondensedReportOfAllROIsofCurrrentlyDisplayedImageMenuItem );
        
            m_region.add( m_ROI_reportingMenu );
        }


        if ( m_isJMRUIRelease == false )
        {
            m_affineTransformationRegioPolygon = new JMenuItem( "Affine transformation" );
            m_affineTransformationRegioPolygon.addActionListener( this );
            m_affineTransformationRegioPolygon.setEnabled( false );
            m_region.add( m_affineTransformationRegioPolygon );

            m_copyRegioPolygon = new JMenuItem( "Copy region" );
            m_copyRegioPolygon.addActionListener( this );
            m_copyRegioPolygon.setEnabled( false );
            m_region.add( m_copyRegioPolygon );

            m_copyAllRegioPolygons = new JMenuItem( "Copy all regions" );
            m_copyAllRegioPolygons.addActionListener( this );
            m_copyAllRegioPolygons.setEnabled( false );
            m_region.add( m_copyAllRegioPolygons );

            m_pasteRegioPolygon = new JMenuItem( "Paste region" );
            m_pasteRegioPolygon.addActionListener( this );
            m_pasteRegioPolygon.setEnabled( false );
            m_region.add( m_pasteRegioPolygon );

            m_pasteSpecialRegioPolygon = new JMenuItem( "Paste special region .. " );
            m_pasteSpecialRegioPolygon.addActionListener( this );
            m_pasteSpecialRegioPolygon.setEnabled( false );
            m_region.add( m_pasteSpecialRegioPolygon );

            m_pasteAllRegioPolygons = new JMenuItem( "Paste all regions" );
            m_pasteAllRegioPolygons.addActionListener( this );
            m_pasteAllRegioPolygons.setEnabled( false );
            m_region.add( m_pasteAllRegioPolygons );

            m_deleteAllRegioPolygonsOfCurrentlyDisplayedImageOnly = new JMenuItem( "Delete all regions of currently selected image only" );
            m_deleteAllRegioPolygonsOfCurrentlyDisplayedImageOnly.addActionListener( this );
            m_deleteAllRegioPolygonsOfCurrentlyDisplayedImageOnly.setEnabled( false );
            m_region.add( m_deleteAllRegioPolygonsOfCurrentlyDisplayedImageOnly  );

            m_deleteAllRegioPolygonsOfAllImages = new JMenuItem( "Delete all regions of all images " );
            m_deleteAllRegioPolygonsOfAllImages.addActionListener( this );
            m_deleteAllRegioPolygonsOfAllImages.setEnabled( false );
            m_region.add( m_deleteAllRegioPolygonsOfAllImages  );


            m_renameRegion = new JMenuItem( "Rename region .." );
            m_renameRegion.addActionListener( this );
            m_renameRegion.setEnabled( false );
            m_region.add( m_renameRegion );

            m_mirrorROI = new JMenu( "Mirrored ROI" );

            m_showROIMirrorAxis = new JRadioButtonMenuItem( "Show current mirror axis" );
            m_mirrorROI.add( m_showROIMirrorAxis );
            m_showROIMirrorAxis.addActionListener( this );

            // do not show the mirror axis per default
            m_showROIMirrorAxis.setSelected( false );

            m_editMirrorAxis = new JMenuItem( "Edit mirror axis .." );
            m_mirrorROI.add( m_editMirrorAxis );
            m_editMirrorAxis.addActionListener( this );

            m_addMirroredROI = new JMenuItem( "Add Mirrored ROI .." );
            m_mirrorROI.add( m_addMirroredROI );
            m_addMirroredROI.addActionListener( this );

            m_region.add( m_mirrorROI );

            m_showHistogramOfSelectedRegioPolygon = new JMenuItem( "Histogram of ROI" );
            m_showHistogramOfSelectedRegioPolygon.addActionListener( this );
            m_showHistogramOfSelectedRegioPolygon.setEnabled( false );
            m_region.add( m_showHistogramOfSelectedRegioPolygon );

            m_tTestRegioPolygon = new JMenuItem( "tTest ROIs ..." );
            m_tTestRegioPolygon.addActionListener( this );
            m_tTestRegioPolygon.setEnabled( false );
            m_region.add( m_tTestRegioPolygon );
            
            m_pixelDumpSelectedRegioPolygon = new JMenuItem( "Pixel dump selected ROI ..." );
            m_pixelDumpSelectedRegioPolygon.addActionListener( this );
            m_pixelDumpSelectedRegioPolygon.setEnabled( false );
            m_region.add( m_pixelDumpSelectedRegioPolygon );
            
            m_scaleRegioPolygon = new JMenuItem( "Scale selected ROI ..." );
            m_scaleRegioPolygon.addActionListener( this );
            m_scaleRegioPolygon.setEnabled( false );
            m_region.add( m_scaleRegioPolygon );

            m_importROI = new JMenuItem( "Import ROI" );

            m_importROI.addActionListener( this );
            m_importROI.setEnabled( true );
            m_region.add( m_importROI );

            m_popupmenu.add( m_region );
        }
        
        m_distanceMeasurementMenu = new JMenu( "Distance Measurements .. " );

        m_measureDistanceMenuItem = new JMenuItem( "Measure distances in images .." );
        m_measureDistanceMenuItem.addActionListener( this );

        m_loadDistancesOfAllImagesFromFileMenuItem =  new JMenuItem( "Load Distances of all images.." );
        m_loadDistancesOfAllImagesFromFileMenuItem.addActionListener( this );

        m_loadDistancesOfCurrentlyDisplayedImageInFilesMenuItem = new JMenuItem( "Load Distances of currently displayed image only .." );
        m_loadDistancesOfCurrentlyDisplayedImageInFilesMenuItem.addActionListener( this );

        m_saveDistancesOfAllImagesInFilesMenuItem = new JMenuItem( "Save distances of all images.." );
        m_saveDistancesOfAllImagesInFilesMenuItem.addActionListener( this );
        
        m_saveDistancesOfCurrentlyDisplayedImageInFilesMenuItem = new JMenuItem( "Save distances of currently displayed image only .." );
        m_saveDistancesOfCurrentlyDisplayedImageInFilesMenuItem.addActionListener( this );

        m_createReportFileOfDistancesMenuItem = new JMenuItem( "Create report file of all distances .." );
        m_createReportFileOfDistancesMenuItem.addActionListener( this );
        
        m_distanceMeasurementMenu.add( m_measureDistanceMenuItem );
        m_distanceMeasurementMenu.add( m_loadDistancesOfAllImagesFromFileMenuItem );
        m_distanceMeasurementMenu.add( m_loadDistancesOfCurrentlyDisplayedImageInFilesMenuItem );
        m_distanceMeasurementMenu.add( m_saveDistancesOfAllImagesInFilesMenuItem );
        m_distanceMeasurementMenu.add( m_saveDistancesOfCurrentlyDisplayedImageInFilesMenuItem );
        m_distanceMeasurementMenu.add( m_createReportFileOfDistancesMenuItem );

        m_popupmenu.add( m_distanceMeasurementMenu );

        m_popupmenu.addSeparator( );
        
        if ( m_isJMRUIRelease == false )
        {
            m_openImageStackIn3DControl = new JMenuItem( "View data in 3D slicer control" );
            m_openImageStackIn3DControl.addActionListener( this );
        
            m_popupmenu.add( m_openImageStackIn3DControl );
        
            m_popupmenu.addSeparator( );
        }

        if ( m_isJMRUIRelease == false )
        {
            m_segmentation = new JMenu( "Segmentation" );

            m_simpleThressholdingSegmentationMenuItem = new JMenuItem( "Simple Thressholding .." );
            m_simpleThressholdingSegmentationMenuItem.addActionListener( this );
            m_simpleThressholdingSegmentationMenuItem.setEnabled( true );

            m_segmentation.add( m_simpleThressholdingSegmentationMenuItem );

            m_advancedInteractiveSegmentationMenuItem = new JMenuItem( "Advanced interactive segmentation .. " );
            m_advancedInteractiveSegmentationMenuItem.addActionListener( this );
            m_advancedInteractiveSegmentationMenuItem.setEnabled( true );

            m_segmentation.add( m_advancedInteractiveSegmentationMenuItem );

            m_orderStatisticsSegmentationMenuItem = new JMenuItem( "Order statistics segmentation .. " );
            m_orderStatisticsSegmentationMenuItem.addActionListener( this );
            m_orderStatisticsSegmentationMenuItem.setEnabled( true );

            m_segmentation.add( m_orderStatisticsSegmentationMenuItem );

            m_popupmenu.add( m_segmentation );
        }
        
        m_displayGrayMenuItem.setSelected( true );
        
        addMouseListener( this );
        addMouseMotionListener( this );
        addMouseWheelListener( this );
        
        add( m_popupmenu );
        
        m_clipBoard = PolygonClipboard.getInstance( );
        m_clipBoard.addActionListener( this );
        setMode( SELECT_MODE );
        
    }

    /**
     * Computes the statistics object for the given volume of interest name
     * @param imageVolumeOfInterestName
     */
    public void computeStatisticsOfSelectedVolume( String imageVolumeOfInterestName )
    {
        int voiNumber = -1;

        for ( int i= 0; i < m_imageVolumeOfInterestNameList.size( ); i++ )
        {
            if ( m_imageVolumeOfInterestNameList.get( i ).compareTo( imageVolumeOfInterestName ) == 0 )
            {
                voiNumber = i;
                break;
            }
        }

    }

    /**
     * Renames a given volume number 
     * @param selectedVolumeNumber
     * @param newNameOfVolume
     */
    public void remameVolume( int selectedVolumeNumber, String newNameOfVolume )
    {
        String name = m_imageVolumeOfInterestNameList.get( selectedVolumeNumber );
        m_imageVolumeOfInterestNameList.set( selectedVolumeNumber, newNameOfVolume );
        System.out.println( );
    }
    
    /**
     * Sets the pixel data object ; no displaying just setting
     */
    @Override
    public void setDicomImages( DicomImageSeriesData imageSeriesData )
    {
        super.setDicomImages( imageSeriesData );
        
        // initialize the vector of linkedlists of RegioPolygons
        m_polygonsVector = new Vector< LinkedList <RegioPolygon> >( );
        m_distancesVector = new Vector< LinkedList <Distance> >( );
        
        for ( int i = 0 ; i < m_numberOfImages; i++ )
        {
            m_polygonsVector.add( new LinkedList<RegioPolygon>( ) );
            m_distancesVector.add( new LinkedList<Distance>( ) );
        }

    }
 
    /**
     * Sets the image data and causes slicenumber 'slice' to be displayed
     */
     @Override
    public void setImage( int[] data, int width, int height, int slice ) 
    {

        super.setImage( data, width, height, slice );


        if ( m_polygonsVector == null )
        {
            // initialize the vector of linkedlists of RegioPolygons
            m_polygonsVector = new Vector< LinkedList <RegioPolygon> >( );
        
            for ( int i = 0 ; i < m_numberOfImages; i++ )
            {
                m_polygonsVector.add( new LinkedList<RegioPolygon>( ) );
            }
        }
        if ( ( m_distancesVector == null ) || ( m_distancesVector.size( ) == 0 ) )
        {
            // initialize the vector of linkedlists of Distance
            m_distancesVector = new Vector< LinkedList <Distance> >( );

            for ( int i = 0 ; i < m_numberOfImages; i++ )
            {
                m_distancesVector.add( new LinkedList<Distance>( ) );
            }
        }

    }
    
    /**
     * Sets the subsegment scope of the sub segmentation.
     * 
     * @param segmentationScope
     */
    public void setSubSegmentationScope( int segmentationScope )
    {
        m_subSegmentationScope = segmentationScope;
    }
    
    /**
     * Sets the pixel range computation type
     * 
     * @param type
     */
    public void setPixelRangeComputationType( int type )
    {
        
    }
    
    
    public void setUniqueNames( boolean mode ) 
    {
        uniqueNames = mode;
    }

    public boolean hasUniqueNames( ) 
    {
        return uniqueNames;
    }

    /**
     * Sets the mode of the object. Depending on this the user 
     * can add draw move delete polygons
     */
    public void setMode( int mode ) 
    {
        
        m_current_mode = mode;

        if ( m_isJMRUIRelease == false )
        {
            inPolygonDrawMode = inRectangleDrawMode = inResizeMode =
            inDeleteMode = inCutMode = showVertices = inSelectMode = false;
            
            if ( mode == ADD_STACKED_ROIS_MODE )
            {
                System.out.println( "ADD_STACKED_ROI_MODE" );
            }
            
            if ( mode == DRAG_MODE )
            {
                setCursor( new Cursor( Cursor.DEFAULT_CURSOR ) );
                m_selectROIButton.setSelected( true );
                m_moveROI.setSelected( true );
            }
            else if ( mode == POLYGON_DRAW_MODE )
            {
                inPolygonDrawMode = true;
                m_addROI.setSelected( true );
                setCursor( new Cursor( Cursor.CROSSHAIR_CURSOR ) );
                notifyActionListeners( ADD_ROI_MODE_SELECTED_EVENT );
            }
            else if( mode == ADD_SINGLE_CIRCULAR_ROI_MODE )
            {
                inPolygonDrawMode = true;
                m_addOneCircularROIMode.setSelected( true );
                setCursor( new Cursor( Cursor.CROSSHAIR_CURSOR ) );
                notifyActionListeners( ADD_ROI_MODE_SELECTED_EVENT );
            }
            else if ( mode == RECTANGLE_DRAW_MODE )
            {
                inRectangleDrawMode = true;
                setCursor( new Cursor( Cursor.CROSSHAIR_CURSOR ) );
            }
            else if ( mode == RESIZE_MODE )
            {
                inResizeMode = true;
                showVertices = true;
                setCursor( new Cursor( Cursor.HAND_CURSOR ) );
                this.m_editROI.setSelected( true );
                if ( m_selectedPolygon !=null )
                {
                    m_selectedPolygon.reduceVertices( 10.0 );
                }
                notifyActionListeners( RESIZE_ROI_MODE_SELECTED_EVENT );
            }
            else if ( mode == DELETE_MODE )
            {
                inDeleteMode = true;
                m_deleteROI.setSelected( true );
                setCursor( new Cursor( Cursor.MOVE_CURSOR ) );
                notifyActionListeners( DELETE_ROI_MODE_SELECTED_EVENT );
            }
            else if ( mode == CUT_MODE )
            {
                inCutMode = true;
                
                setCursor( new Cursor( Cursor.CROSSHAIR_CURSOR ) );
                m_cutROI.setSelected( true );
                notifyActionListeners( CUT_ROI_MODE_SELECTED_EVENT );
            }
            else if ( mode == SELECT_MODE )
            {
                inSelectMode = true;
                setCursor( new  Cursor( Cursor.DEFAULT_CURSOR ) );
                m_selectROIButton.setSelected( true );
                notifyActionListeners( SELECT_ROI_MODE_SELECTED_EVENT );
            }
        }
    }
    
    /**
     * Returns the current mode of the object
     */
    public int getMode( )
    {
        return m_current_mode;
    }
    
    /**
     * Returns a new polygon with one point added
     */
    public RegioPolygon newPolygon( int x, int y ) 
    {
        LinkedList< RegioPolygon > currentPolygons = new LinkedList< RegioPolygon >( );
        if ( m_polygonsVector.size( ) > 1 )
        {
            currentPolygons = m_polygonsVector.get( m_currentlyDisplayedImageNumber );
        }
        else
        {
            currentPolygons = m_polygonsVector.get( 0 );
        }
        
        RegioPolygon p = new RegioPolygon( currentPolygons.size( ) + 1 );
        p.addPoint( x, y );
        isPolygonDrawing = true;
        return p;
    }
    
    
    /**
     * Is called whenever the user releases the mouse button after drawing
     * a ROI (polygon) in case standard circular ROIs need to be defined.
     */
    public void finishPolygon( int x, int y ) 
    {
        isPolygonDrawing = false;
        
        currentPoly = addCircularShapedROI(10, x, y);
        addPolygon( currentPoly );
        
        notifyActionListeners( POLYGON_CLOSED_EVENT );
	
        currentPoly = null;
    }
    
    
    
    /**
     * Is called whenever the user releases the mouse button after drawing
     * a ROI (polygon).
     */
    public void finishPolygon( ) 
    {
        isPolygonDrawing = false;
        if ( !currentPoly.isEmpty( ) )
        { 
                addPolygon( currentPoly ); 
        }
        else
        {
            if ( m_addOneCircularROIMode.isSelected( ) == true )
            {
                currentPoly = addCircularShapedROI(100, 128, 128);
                addPolygon( currentPoly );
            }
        }
        
        notifyActionListeners( POLYGON_CLOSED_EVENT );
	
        currentPoly = null;
    }
    
    /**
     * Converts the current polygon to rectangle
     */
    public RegioPolygon convertCurrentPolygonToRectangle( )
    {
        
        Rectangle rect = m_selectedPolygon.getBounds( );
        
        int x[] = new int[4];
        int y[] = new int[4];
        
        x[0] = rect.x;
        x[1] = rect.x + rect.width;
        x[2] = rect.x + rect.width;
        x[3] = rect.x;
        
        y[0] = rect.y;
        y[1] = rect.y;
        y[2] = rect.y + rect.height;
        y[3] = rect.y + rect.height;
        
        RegioPolygon regPol = new RegioPolygon( 1, x, y, 4 );
        
        deletePoly( m_selectedPolygon );
        
        m_selectedPolygon = createUniquePolygon( regPol );
        
        m_selectedPolygon.setColor( Color.blue );
        
        addPolygon( m_selectedPolygon );
        
        repaint( );
        
        return m_selectedPolygon;
        
    }
    
    /**
     * Converts the current polygon to rectangle
     */
    public RegioPolygon convertPolygonToRectangle( RegioPolygon polygon )
    {
        Rectangle rect = polygon.getBounds( );
        
        int x[] = new int[4];
        int y[] = new int[4];
        
        x[0] = rect.x;
        x[1] = rect.x + rect.width;
        x[2] = rect.x + rect.width;
        x[3] = rect.x;
        
        y[0] = rect.y;
        y[1] = rect.y;
        y[2] = rect.y + rect.height;
        y[3] = rect.y + rect.height;
        
        RegioPolygon regPol = new RegioPolygon( 1, x, y, 4 );
        
        return regPol;
    }
    
    
    /**
     * Returns the current rectangle
     */
     public Rectangle getCurrentRectangle( ) 
     {
        if ( currentRectangle == null ) return null;
        Rectangle r = new Rectangle( currentRectangle );
        r.x /= m_zoom;
        r.y /= m_zoom;
        r.width /= m_zoom;
        r.height /= m_zoom;
        return r;
     }
     
     
    /**
     * Rescales all polygons of the polygon list
     */ 
    public void rescalePolygons( ) 
    {
        float r = m_zoom / m_oldZoom;
        // System.out.println( "scaling factor" + r );
        // iterate over all images of the slice stack!
        for ( int j = 0; j < m_polygonsVector.size( ) ; j++ )
        {
            LinkedList< RegioPolygon > currentPolygons = m_polygonsVector.get( j );
        
            for ( int k = 0; k < currentPolygons.size( ); k++ ) 
            {
                RegioPolygon p = currentPolygons.get( k );
                p.setScalingFactor( r );
            }
        }
        
        if ( m_fieldOfInterest != null )
        {
            m_fieldOfInterest.setScalingFactor( r );
        }
    }
    
    /**
     * Sets the zoom factor (float) of the displayed image
     */
    @Override
    public void setZoom( float zoom ) 
    {
        m_oldZoom = m_zoom;
        m_zoom = zoom;
	
        rescalePolygons( );
        
        // redraw the image if the complete image is larger than 1 pixel 
        if ( ( this.m_imageHeight > 1 ) && ( this.m_imageHeight > 1 ) )
        {
            drawImage( false );
        }
        
        calculateImagePixels( );
        
        if ( m_imageCrossLines != null )
        {
            m_imageCrossLines.setZoom( zoom );
        }
    }
    
    /**
     * Sets the index value of the selected polygon
     */ 
    public void setSelectedPolygonIndex( int newIndex ) 
    {

        if ( m_selectedPolygon == null )
        { 
            m_renameRegion.setEnabled( false );
            return;
        }

        m_selectedPolygon.setIndex( newIndex );
        m_renameRegion.setEnabled( true );
        m_showHistogramOfSelectedRegioPolygon.setEnabled( true );
        
        LinkedList< RegioPolygon > currentPolygons = m_polygonsVector.get(m_currentlyDisplayedImageNumber );
        
        if ( currentPolygons.size( ) > 1 )
        {
            m_tTestRegioPolygon.setEnabled( true );
            m_pixelDumpSelectedRegioPolygon.setEnabled( true );
        }
        else
        {
            m_tTestRegioPolygon.setEnabled( false );
            m_pixelDumpSelectedRegioPolygon.setEnabled( false );
        }
        repaint( );
    }


    /**
     * Returns the old name before renaming!
     * @return
     */
    public String getRegioNamebeforeRenaming( )
    {
        return m_regioNameBeforeRenaming;
    }

    /**
     * Renames the selected RegioPolygon giving it the name "newName"
     */
    public void renameSelectedPolygon( String newName ) 
    {
        m_regioNameBeforeRenaming = m_selectedPolygon.getLabel( );

        if ( m_selectedPolygon == null )
        { 
            return;
        }
        
        if ( uniqueNames )
        {
            // check if there exists a polygon with the new name in the list
            RegioPolygon existingPoly = null;
            
            // select the correct LinkedList-element!
            LinkedList< RegioPolygon > currentPolygons = m_polygonsVector.get(m_currentlyDisplayedImageNumber );
        
            // iterate over all RegioPolygons
            for ( int k = 0; k < currentPolygons.size( ); k++ )
            {
                
                RegioPolygon p1 = currentPolygons.get( k );
                
                if ( p1 != m_selectedPolygon && p1.m_label.equals( newName ) )
                {
                    existingPoly = p1;
                    break;
                }
            }
            
            if ( existingPoly != null )
            {
                //exchange the names
                existingPoly.setLabel( m_selectedPolygon.m_label );
            }
            
        }
       
        // select the correct LinkedList from the vector! 
        LinkedList< RegioPolygon > currentPolygons = m_polygonsVector.get( m_currentlyDisplayedImageNumber );
        
        for ( int k = 0; k < currentPolygons.size( ); k++ ) 
        {
            RegioPolygon p1 = currentPolygons.get( k );
            if ( p1 == m_selectedPolygon  ) 
            {
                p1.setLabel( newName );
                break;
            }
        }
        
        m_selectedPolygon.setLabel( newName );
        
        //notifyActionListeners( POLYGON_CHANGED_EVENT );

        // this event is a hack: is only caught in MultiDicomSliceView
        notifyActionListeners( POLYGON_RENAME_EVENT );
        repaint( );
    }

    /**
     * Returns the RegioPolygon having index "index"
     */
    public RegioPolygon getPolygon( int index )
    {
        LinkedList< RegioPolygon > currentPolygons = new LinkedList< RegioPolygon >( );
        
        if ( m_polygonsVector.size( ) > 1 )
        {
            currentPolygons = m_polygonsVector.get( m_currentlyDisplayedImageNumber );
        }
        else
        {
            currentPolygons = m_polygonsVector.get( 0 );
        }
        
        if ( index < 0 || index >= currentPolygons.size( ) )
        {
            return null;
        }
        
        return currentPolygons.get( index );
    }

    
    /**
     * Returns the RegioPolygon of image belonging to imageNumber having 
     * index "polygonNumber"
     */
    public RegioPolygon getPolygon( int imageNumber, int polygonNumber )
    {
        LinkedList< RegioPolygon > currentPolygons = new LinkedList< RegioPolygon >( );
      
        currentPolygons = m_polygonsVector.get( imageNumber );
        
        if ( polygonNumber < 0 || polygonNumber >= currentPolygons.size( ) )
        {
            return null;
        }
        
        return currentPolygons.get( polygonNumber );
    }
    
    
    /**
     * Return a copy of polygon with index "index"
     */ 
    public RegioPolygon copyPolygon( int index ) 
    {
        LinkedList< RegioPolygon > currentPolygons = new LinkedList< RegioPolygon >( );
        
        if ( m_polygonsVector.size( ) > 1 ) 
        {
            currentPolygons = m_polygonsVector.get(m_currentlyDisplayedImageNumber );
        }
        else
        {
            currentPolygons = m_polygonsVector.get( 0 );
        }
        
        if ( index < 0 || index >= currentPolygons.size( ) )
        { 
            return null;
        }
	
        RegioPolygon p = currentPolygons.get( index );
	
        return p.copy( );
    }
    
    /**
     * Returns a ImageParametersWithinRegioPoygon object to caller
     */
    public ImageParametersWithinARegioPolygon getPolygonImageInformation( int imageNumber, int index ) 
    {
        
        LinkedList< RegioPolygon > polygons = new LinkedList< RegioPolygon >( );
        
        if ( m_polygonsVector.size( ) > 1 ) 
        {
            polygons = m_polygonsVector.get( imageNumber );
        }
        else
        {
            polygons = m_polygonsVector.get( 0 );
        }
        
        RegioPolygon rpol = polygons.get( index );
        
        calculateROIParameters( imageNumber, rpol );
	
        ImageParametersWithinARegioPolygon imPar = new ImageParametersWithinARegioPolygon( );
        
        imPar.setROIName( rpol.getLabel( ) );
        imPar.setNumberOfPixels( rpol.getInnerVolume( ) );
        imPar.setROIVolume( rpol.getInnerVolume( ) );
        imPar.setMeanPixelValue( m_av );
        imPar.setStandardDeviationOfPixelValues( m_stdev );
        
        imPar.setMeanRawPixelValue( m_av_raw );
        imPar.setStandardDeviationRawOfPixelValues( m_stdev_raw );
        
        imPar.setMinimumMaximumRawRangeOfRegion( m_range_raw );
        imPar.setCenterPixelRawRangeValueOfRegion( m_center_raw );
        
        return imPar;   
    }

    /**
     * Returns a ImageParametersWithinRegioPoygon object to caller
     */
    public MomentsAboutTheMean getMomentsAboutTheMeanOfRegioPolygon( int imageNumber, int index )
    {

        LinkedList< RegioPolygon > polygons = new LinkedList< RegioPolygon >( );

        if ( m_polygonsVector.size( ) > 1 )
        {
            polygons = m_polygonsVector.get( imageNumber );
        }
        else
        {
            polygons = m_polygonsVector.get( 0 );
        }

        RegioPolygon rpol = polygons.get( index );

        // calculateROIParameters( imageNumber, rpol );

        int[] image = new int[m_imageWidth*m_imageHeight];

        int dataOffset = m_imageWidth * m_imageHeight * imageNumber;

        for ( int i = 0; i < m_imageWidth * m_imageHeight; i++ )
        {
            image[i] = m_imagePixelData[ dataOffset + i ];
        }

        MomentsAboutTheMeanOfRegionCalculator calc = new MomentsAboutTheMeanOfRegionCalculator( rpol, image, m_imageWidth, m_imageHeight, m_zoom, m_nordicScaleFactor );
        MomentsAboutTheMean fourMoments = calc.computeStatistics( );

        return fourMoments;
    }





    /**
     * Returns the LinkedList containing all information of the user defined different
     * regios (i.e. tissue types)
     * @return
     */
    public LinkedList <ImageParametersWithinARegioPolygon> getAllTissueTypeParametersAsLinkedList( )
    {
        LinkedList <ImageParametersWithinARegioPolygon> allPolygonRegionInformation = new LinkedList<ImageParametersWithinARegioPolygon>( );
        
        for ( int i = 0; i < m_numberOfImages; i++ )
        {
             LinkedList<RegioPolygon> polygons = m_polygonsVector.get( i );
             
             for ( int j = 0; j < polygons.size( ); j++ )
             {
                RegioPolygon rpol = polygons.get( j );
        
                calculateROIParameters( i, rpol );
	
                ImageParametersWithinARegioPolygon imPar = new ImageParametersWithinARegioPolygon( );
        
                imPar.setROIName( rpol.getLabel( ) );
                imPar.setNumberOfPixels( rpol.getInnerVolume( ) );
                imPar.setROIVolume( rpol.getInnerVolume( ) );
                
                imPar.setMeanPixelValue( m_av );
                imPar.setStandardDeviationOfPixelValues( m_stdev );
                
                imPar.setMeanRawPixelValue( m_av_raw );
                imPar.setStandardDeviationRawOfPixelValues( m_stdev_raw );
                
                imPar.setMinimumMaximumRawRangeOfRegion( m_range_raw );
                imPar.setCenterPixelRawRangeValueOfRegion( m_center_raw );
                
                allPolygonRegionInformation.add( imPar );
             }
        }
        
        return allPolygonRegionInformation;
    }
    
    
    
    
    /**
     * Returns the current number of polygons
     */
    public int getNumberOfPolygons( )
    {
        
        LinkedList< RegioPolygon > currentPolygons = new LinkedList< RegioPolygon >( );
        
        if ( m_polygonsVector.size( ) > 1 ) 
        {
            currentPolygons = m_polygonsVector.get(m_currentlyDisplayedImageNumber );
        }
        else
        {
            currentPolygons = m_polygonsVector.get( 0 );
        }
        
        return currentPolygons.size( );
    }
    
    /**
     * Returns the current number of polygons
     */
    public int getNumberOfPolygons( int imageNumber )
    {
        
        LinkedList< RegioPolygon > currentPolygons = new LinkedList< RegioPolygon >( );
        
        if ( m_polygonsVector.size( ) > 1 ) 
        {
            currentPolygons = m_polygonsVector.get( imageNumber );
        }
        else
        {
            currentPolygons = m_polygonsVector.get( 0 );
        }
        
        return currentPolygons.size( );
    }
    
    
    
    /**
     * Returns the polygon object having name 'm_label'
     */
    public RegioPolygon getPolygon( String label ) 
    {
        RegioPolygon p = null;
        LinkedList< RegioPolygon > currentPolygons = m_polygonsVector.get(m_currentlyDisplayedImageNumber );
        
        for ( int k = 0; k < currentPolygons.size( ); k++ )
        {
            p = currentPolygons.get( k );
            
            if ( p.m_label.equals( label ) )
            { 
                return p;
            }
        }
        
        return null;
    }

    /**
     * Sets polygon having name 'm_label' selected
     */
    public void selectPolygons( String[] roiNames ) 
    {
        LinkedList< RegioPolygon > currentPolygons = m_polygonsVector.get(m_currentlyDisplayedImageNumber );
        
        // reset m_color all to default red
        for ( int k = 0; k < currentPolygons.size( ); k++ ) 
        {
            currentPolygons.get( k ).setColor( Color.red );
        }
        
        RegioPolygon p = null;
        for ( int i = 0 ; i < roiNames.length; i++ )
        {
            for ( int k = 0; k < currentPolygons.size( ); k++ ) 
            {
                p = currentPolygons.get( k );
                if ( p.m_label.equals( roiNames[i] ) )
                {
                    p.setColor( Color.blue );
                }
            }
        }
        
        // draw them all now
        repaint( );
    }
    
    /**
     * Returns a reference to the currently selected polygon
     */
    public RegioPolygon getSelectedPolygon( ) 
    {
        return m_selectedPolygon;
    }

    /**
     *  Returns the index of the currently selected polygon
     * @param p
     * @return integer index
     */
    public int getPolygonListIndex( RegioPolygon p ) 
    {
        LinkedList< RegioPolygon > currentPolygons = m_polygonsVector.get(m_currentlyDisplayedImageNumber );
        
        if ( p == null ) return -1;
        for ( int k = 0; k < currentPolygons.size( ); k++ )
        {
            if ( currentPolygons.get( k ) == p )
            { 
                return k;
            }
        }
        return -1;
    }
    
    /**
     * Returns the index of the currently selected RegioPolygon
     */
    public int getSelectedPolygonListIndex( ) 
    {
        return getPolygonListIndex( m_selectedPolygon );
    }

    /**
     * Returns the array of region names
     */
    public String[] getRegionNames( ) 
    {
        LinkedList< RegioPolygon > currentPolygons = new  LinkedList< RegioPolygon >( );
        
        if( m_polygonsVector.size( ) > 1 )
        {
            currentPolygons = m_polygonsVector.get(m_currentlyDisplayedImageNumber );
        }
        else
        {
            currentPolygons = m_polygonsVector.get( 0 );    
        }
        
        
        String[] tmpNames = new String[currentPolygons.size( )];
	
        if ( currentPolygons.size( ) == 0 )
        { 
            return tmpNames;
        }
       
        tmpNames[0] = new String( currentPolygons.get( 0 ).m_label );
        int n = 1;
        for ( int k = 1; k < currentPolygons.size( ); k++ )
        {
            RegioPolygon p1 = currentPolygons.get( k );
            boolean isThere = false;
            for ( int l = 0; l < k; l++ )
            {
                RegioPolygon p2 = currentPolygons.get( l );
                if ( p2.m_label.equals( p1.m_label ) )
                {
                    isThere = true;
                    break;
                }
            }
            if ( !isThere )
            { 
                tmpNames[n++] = new String( p1.m_label );
            }
        }
        String[] names = new String[n];
        for ( int k = 0; k < n; k++ )
        {
            names[k] = new String( tmpNames[k] );
        }
        return names;
    }
    
    /**
     * Adds a polygon to the list of polygons
     */
    public void addPolygon( int[] x, int[] y ) 
    {
        int nx = x.length;
        int ny = y.length;
        int n = ( nx <= ny ? nx : ny );
        addPolygon( x, y, n );
    }
    
    /**
     * Adds a polygon to the list of polygons
     */
    public void addPolygon( int[] x, int[] y, int n ) 
    {
        LinkedList< RegioPolygon > currentPolygons = m_polygonsVector.get(m_currentlyDisplayedImageNumber );
        
        RegioPolygon P = new RegioPolygon( currentPolygons.size( ) + 1, x, y, n );
        addPolygon( P );
    }

    /**
     * Method enables or disables zooming of image via popupmenu item
     * 
     * @param b
     */
    void enableZoomPopupMenuItem( boolean enable ) 
    {
        m_doubleImageDisplay.setEnabled( enable );
        m_halfImageDisplay.setEnabled( enable );
    }

    /**
     * Adds an empty volume to the list of volumes
     */
    public void addEmptyVolume( )
    {

        if ( m_imageVolumeOfInterestOverlayList.size( ) == 0 )
        {
            for ( int i = 0; i < m_numberOfImages; i++ )
            {
                m_imageVolumeOfInterestOverlayList.add( new LinkedList<boolean[][]>() );
            }
        }

        for ( int i = 0; i < m_numberOfImages; i++ )
        {
            LinkedList<boolean[][]> volumeList = m_imageVolumeOfInterestOverlayList.get( i );

            boolean[][] volOverlay = new boolean[m_imageWidth][m_imageHeight];
            for ( int j = 0; j < m_imageWidth; j++ )
            {
                for ( int k = 0; k < m_imageHeight; k++ )
                {
                    volOverlay[j][k] = false;
                }
            }

            volumeList.add( volOverlay );
        }


        m_selectedImageVolumeOfInterestList.add( true );

        if ( m_imageVolumeOfInterestOverlayList.get( 0 ) != null )
        {
            m_imageVolumeOfInterestNameList.add( "Volume_" + m_imageVolumeOfInterestOverlayList.get( 0 ).size( ) );
            m_imageVolumeOfInterestColorList.add( defaultColors[ m_imageVolumeOfInterestOverlayList.get( 0 ).size( ) - 1] );
        }
        else
        {
            m_imageVolumeOfInterestNameList.add( "Volume_" + 0 );
            m_imageVolumeOfInterestColorList.add( defaultColors[ 0 ] );
        } 
    }

    /**
     *
     * Sets the volume pixel element number to true
     *
     * @param minDistSliceNumber
     * @param p2D
     */
    void setVolumeElementPixel( int volNumber, int minDistSliceNumber, Point p2D )
    {
        // if the length is zero no volume was defined yet

        if ( m_imageVolumeOfInterestOverlayList.size( ) == 0 )
        {
            addEmptyVolume( );
        }
        else if ( volNumber == (  m_imageVolumeOfInterestOverlayList.get( 0 ).size( ) ) )
        {
            addEmptyVolume( );
        }

        // in case there are multiple slices at the same location these points have to be set as well
        FileInfo finfo = m_dicomImageSeriesData.getFileInfo( minDistSliceNumber );

        String sliceLocation = finfo.getSliceLocation( );

        for ( int i = 0; i < m_dicomImageSeriesData.getNumberOfImages( ); i++ )
        {

            FileInfo finfo2 = m_dicomImageSeriesData.getFileInfo( i );

            if ( sliceLocation.compareTo( finfo2.getSliceLocation( ) ) == 0 )
            {
                LinkedList<boolean[][]> volumeList = m_imageVolumeOfInterestOverlayList.get( i );

                boolean[][] overlay = volumeList.get( volNumber );

                if ( ( p2D.x > -1 ) &&
                     ( p2D.x < m_imageWidth ) &&
                     ( p2D.y > -1 ) &&
                     ( p2D.y < m_imageHeight ) )
                {
                    overlay[p2D.x][p2D.y] = true;
                }
            }
        }

    }

    /**
     * Determines whether m_label of polygon is unique
     */
    private RegioPolygon createUniquePolygon( RegioPolygon P )
    {
        LinkedList< RegioPolygon > currentPolygons = new LinkedList< RegioPolygon >( );
        
        if ( m_polygonsVector.size( ) > 1 )
        {
            currentPolygons = m_polygonsVector.get(m_currentlyDisplayedImageNumber );
        }
        else
        {
            currentPolygons = m_polygonsVector.get( 0 );
        }
        
        Iterator it = currentPolygons.iterator( );
        
        while ( it.hasNext( ) )
        {
            RegioPolygon rpol = (RegioPolygon)it.next( );
            // check whether polygon name allready exists
            if ( P.getLabel( ).compareTo( rpol.getLabel( ) ) == 0 )
            {
                // if yes then look whether the bounds are the same
                P.setLabel( P.getLabel( ) + "*" );
                Rectangle rec = P.getBounds( );
                if ( ( rec.getX( ) == rpol.getBounds( ).getX( ) ) && 
                     ( rec.getY( ) == rpol.getBounds( ).getY( ) ) &&
                     ( rec.getWidth( ) == rpol.getBounds( ).getWidth( ) ) &&
                     ( rec.getHeight( ) == rpol.getBounds( ).getHeight( ) ) )
                {
                    P.translate( 10, 10 );
                }
            }
        }
        return P;
    }

    /**
     * Determines whether m_label of polygon is unique
     */
    private RegioPolygon createUniquePolygonNoTrans( RegioPolygon P )
    {
        LinkedList< RegioPolygon > currentPolygons = new LinkedList< RegioPolygon >( );

        if ( m_polygonsVector.size( ) > 1 )
        {
            currentPolygons = m_polygonsVector.get(m_currentlyDisplayedImageNumber );
        }
        else
        {
            currentPolygons = m_polygonsVector.get( 0 );
        }

        Iterator it = currentPolygons.iterator( );

        while ( it.hasNext( ) )
        {
            RegioPolygon rpol = (RegioPolygon)it.next( );
            // check whether polygon name allready exists
            if ( P.getLabel( ).compareTo( rpol.getLabel( ) ) == 0 )
            {
                // if yes then look whether the bounds are the same
                P.setLabel( P.getLabel( ) );
                Rectangle rec = P.getBounds( );
                if ( ( rec.getX( ) == rpol.getBounds( ).getX( ) ) &&
                     ( rec.getY( ) == rpol.getBounds( ).getY( ) ) &&
                     ( rec.getWidth( ) == rpol.getBounds( ).getWidth( ) ) &&
                     ( rec.getHeight( ) == rpol.getBounds( ).getHeight( ) ) )
                {
                    // P.translate( 10, 10 );
                }
            }
        }
        return P;
    }



    /**
     * Adds a polygon to the list of polygons
     */
    public void addPolygon( RegioPolygon P ) 
    {

        if ( m_polygonsVector.size( ) == 0 )
        {
            m_polygonsVector = new Vector< LinkedList <RegioPolygon> >( );
            m_polygonsVector.add( new LinkedList< RegioPolygon >( ) );
        }

        LinkedList< RegioPolygon > currentPolygons;

        if ( m_polygonsVector.size( ) > 1 )
        {
            currentPolygons = m_polygonsVector.get( m_currentlyDisplayedImageNumber );
        }
        else
        {
//            m_polygonsVector = new Vector< LinkedList <RegioPolygon> >( );
//            for ( int i = 0 ; i < this.m_numberOfImages; i++ )
//            {
//                m_polygonsVector.add( new LinkedList<RegioPolygon>( ) );
//            }
//            currentPolygons = m_polygonsVector.get( m_currentlyDisplayedImage );
            currentPolygons = m_polygonsVector.get( 0 );
        }

        String label = P.getLabel( );

        P = createUniquePolygon( P );
        currentPolygons.add( P );

        // update the colors of the other polygons
        for ( int k = 0; k < currentPolygons.size( ); k++ )
        {
           RegioPolygon p = currentPolygons.get( k );
           p.setColor( Color.red );
        }

        // notify the listeners that a polygon has been selected
        multicaster.polygonSelect( new PolygonEvent( this,
                                   PolygonEvent.POLYGON_SELECT,
                                   m_selectedPolygon ) );

        m_selectedPolygon = P;
        m_selectedPolygon.setColor( Color.blue );

        repaint( );

        if ( m_renameRegion != null )
        {
            m_renameRegion.setEnabled( true );
        }

        if ( m_showHistogramOfSelectedRegioPolygon != null )
        {
            m_showHistogramOfSelectedRegioPolygon.setEnabled( true );
        }

        if ( m_enablePaging == true )
        {

            if ( m_reduceImageMatrixAndWriteAsDICOM != null )
            {
                m_reduceImageMatrixAndWriteAsDICOM.setEnabled( true );
            }
        }

        if ( ( m_enablePaging == true ) && ( m_performSimple3DCut != null ) )
        {
            m_performSimple3DCut.setEnabled( true );
        }

        if ( ( m_enablePaging == true ) && ( m_selectedPolygon != null ) )
        {
            if ( m_computePropertiesofSelectedROIForAllSlicesOfCurrentlySelectedRegionOnly != null )
            {
                m_computePropertiesofSelectedROIForAllSlicesOfCurrentlySelectedRegionOnly.setEnabled( true );
            }

            if ( m_computePropertiesofSelectedROIForAllSlicesForAllRegions != null )
            {
                m_computePropertiesofSelectedROIForAllSlicesForAllRegions.setEnabled( true );
            }
        }

        if ( currentPolygons.size( ) > 1 )
        {
            if ( m_tTestRegioPolygon != null )
            {
                m_tTestRegioPolygon.setEnabled( true );
            }
        }
        else
        {
            if ( m_tTestRegioPolygon != null )
            {
                m_tTestRegioPolygon.setEnabled( false );
            }
        }

        if ( P != null )
        {
            if ( m_copyRegioPolygon != null )
            {
                m_copyRegioPolygon.setEnabled( true );
                m_pixelDumpSelectedRegioPolygon.setEnabled( true );
            }
            if ( m_affineTransformationRegioPolygon != null )
            {
                m_affineTransformationRegioPolygon.setEnabled( true );
            }
        }

        if ( currentPolygons.size( ) > 1 )
        {
            if ( m_deleteAllRegioPolygonsOfAllImages != null )
            {
                m_deleteAllRegioPolygonsOfAllImages.setEnabled( true );
            }

            if (m_copyAllRegioPolygons != null )
            {
                m_copyAllRegioPolygons.setEnabled( true );
            }
            if ( m_deleteAllRegioPolygonsOfCurrentlyDisplayedImageOnly != null )
            {
                m_deleteAllRegioPolygonsOfCurrentlyDisplayedImageOnly.setEnabled( true );
            }
        }

        if ( ( m_imageSegmentsMenu != null ) && ( m_selectedPolygon != null ) )
        {
            if ( m_eliminateOverlayPixelsOutsideFocusRegion != null )
            {
                m_eliminateOverlayPixelsOutsideFocusRegion.setEnabled( true );
            }
            if ( m_eliminateOverlayPixelsInsideFocusRegion != null )
            {
                m_eliminateOverlayPixelsInsideFocusRegion.setEnabled( true );
            }
        }

        if  ( !m_addOneCircularROIMode.isSelected( ) == true  )
        {
            if ( currentPolygons.size( ) == 1 )
            {
                setMode( DRAG_MODE  );
            }
            else
            {
                setMode( SELECT_MODE );
            }
        }
    }
    
    
    /**
     * Adds a polygon to the list of polygons
     */
    public void addPolygon( int imageNumber, RegioPolygon P ) 
    {

        if ( m_isJMRUIRelease == false )
        {

            LinkedList< RegioPolygon > currentPolygons;

            if ( m_polygonsVector.size( ) > 1 )
            {
                currentPolygons = m_polygonsVector.get( imageNumber );
            }
            else
            {
    //            m_polygonsVector = new Vector< LinkedList <RegioPolygon> >( );
    //            for ( int i = 0 ; i < this.m_numberOfImages; i++ )
    //            {
    //                m_polygonsVector.add( new LinkedList<RegioPolygon>( ) );
    //            }
    //            currentPolygons = m_polygonsVector.get( m_currentlyDisplayedImage );
                currentPolygons = m_polygonsVector.get( 0 );
            }

            String label = P.getLabel( );

            P = createUniquePolygon( P );
            currentPolygons.add( P );

            // update the colors of the other polygons
            for ( int k = 0; k < currentPolygons.size( ); k++ )
            {
               RegioPolygon p = currentPolygons.get( k );
               p.setColor( Color.red );
            }

            // notify the listeners that a polygon has been selected
            multicaster.polygonSelect( new PolygonEvent( this,
                                       PolygonEvent.POLYGON_SELECT,
                                       m_selectedPolygon ) );

            m_selectedPolygon = P;
            m_selectedPolygon.setColor( Color.blue );

            repaint( );

            m_renameRegion.setEnabled( true );

            m_showHistogramOfSelectedRegioPolygon.setEnabled( true );

            if ( ( m_enablePaging == true ) && ( m_performSimple3DCut != null ) )
            {
                m_performSimple3DCut.setEnabled( true );
            }

            if ( ( m_enablePaging == true ) && ( m_selectedPolygon != null ) )
            {

                if ( m_computePropertiesofSelectedROIForAllSlicesOfCurrentlySelectedRegionOnly != null )
                {
                    m_computePropertiesofSelectedROIForAllSlicesOfCurrentlySelectedRegionOnly.setEnabled( true );
                }

                if ( m_computePropertiesofSelectedROIForAllSlicesForAllRegions != null )
                {
                    m_computePropertiesofSelectedROIForAllSlicesForAllRegions.setEnabled( true );
                }

            }

            if ( currentPolygons.size( ) > 1 )
            {
                m_tTestRegioPolygon.setEnabled( true );
            }
            else
            {
                m_tTestRegioPolygon.setEnabled( false );
            }

            if ( P != null )
            {
                m_copyRegioPolygon.setEnabled( true );
                m_pixelDumpSelectedRegioPolygon.setEnabled( true );
                m_affineTransformationRegioPolygon.setEnabled( true );
            }
            if ( currentPolygons.size( ) > 1 )
            {
                m_copyAllRegioPolygons.setEnabled( true );
                m_deleteAllRegioPolygonsOfCurrentlyDisplayedImageOnly.setEnabled( true );
                m_deleteAllRegioPolygonsOfAllImages.setEnabled( true );
            }

            
            if  ( !m_addOneCircularROIMode.isSelected( ) == true  )
            {
                if ( currentPolygons.size( ) == 1 )
                {
                    setMode( DRAG_MODE  );
                }
                else
                {
                    setMode( SELECT_MODE );
                }
            }
            
            if ( ( m_imageSegmentsMenu != null ) && ( m_selectedPolygon != null ) )
            {
                m_eliminateOverlayPixelsOutsideFocusRegion.setEnabled( true );
                m_eliminateOverlayPixelsInsideFocusRegion.setEnabled( true );
            }
        }
    }


    /**
     * Adds a polygon to the list of polygons
     */
    public void addPolygonNoTrans( int imageNumber, RegioPolygon P )
    {

        if ( m_isJMRUIRelease == false )
        {

            LinkedList< RegioPolygon > currentPolygons;

            if ( m_polygonsVector.size( ) > 1 )
            {
                currentPolygons = m_polygonsVector.get( imageNumber );
            }
            else
            {
    //            m_polygonsVector = new Vector< LinkedList <RegioPolygon> >( );
    //            for ( int i = 0 ; i < this.m_numberOfImages; i++ )
    //            {
    //                m_polygonsVector.add( new LinkedList<RegioPolygon>( ) );
    //            }
    //            currentPolygons = m_polygonsVector.get( m_currentlyDisplayedImage );
                currentPolygons = m_polygonsVector.get( 0 );
            }

            String label = P.getLabel( );

            P = createUniquePolygonNoTrans( P );
            currentPolygons.add( P );

            // update the colors of the other polygons
            for ( int k = 0; k < currentPolygons.size( ); k++ )
            {
               RegioPolygon p = currentPolygons.get( k );
               p.setColor( Color.red );
            }

            // notify the listeners that a polygon has been selected
            multicaster.polygonSelect( new PolygonEvent( this,
                                       PolygonEvent.POLYGON_SELECT,
                                       m_selectedPolygon ) );

            m_selectedPolygon = P;
            m_selectedPolygon.setColor( Color.blue );

            repaint( );

            m_renameRegion.setEnabled( true );

            m_showHistogramOfSelectedRegioPolygon.setEnabled( true );

            if ( ( m_enablePaging == true ) && ( m_performSimple3DCut != null ) )
            {
                m_performSimple3DCut.setEnabled( true );
            }

            if ( ( m_enablePaging == true ) && ( m_selectedPolygon != null ) )
            {

                if ( m_computePropertiesofSelectedROIForAllSlicesOfCurrentlySelectedRegionOnly != null )
                {
                    m_computePropertiesofSelectedROIForAllSlicesOfCurrentlySelectedRegionOnly.setEnabled( true );
                }

                if ( m_computePropertiesofSelectedROIForAllSlicesForAllRegions != null )
                {
                    m_computePropertiesofSelectedROIForAllSlicesForAllRegions.setEnabled( true );
                }

            }

            if ( currentPolygons.size( ) > 1 )
            {
                m_tTestRegioPolygon.setEnabled( true );
            }
            else
            {
                m_tTestRegioPolygon.setEnabled( false );
            }

            if ( P != null )
            {
                m_copyRegioPolygon.setEnabled( true );
                m_pixelDumpSelectedRegioPolygon.setEnabled( true );
                m_affineTransformationRegioPolygon.setEnabled( true );
            }
            if ( currentPolygons.size( ) > 1 )
            {
                m_copyAllRegioPolygons.setEnabled( true );
                m_deleteAllRegioPolygonsOfCurrentlyDisplayedImageOnly.setEnabled( true );
                m_deleteAllRegioPolygonsOfAllImages.setEnabled( true );
            }

            if  ( !m_addOneCircularROIMode.isSelected( ) == true  )
            {
                if ( currentPolygons.size( ) == 1 )
                {
                    setMode( DRAG_MODE  );
                }
                else
                {
                    setMode( SELECT_MODE );
                }
            }

            if ( ( m_imageSegmentsMenu != null ) && ( m_selectedPolygon != null ) )
            {
                m_eliminateOverlayPixelsOutsideFocusRegion.setEnabled( true );
                m_eliminateOverlayPixelsInsideFocusRegion.setEnabled( true );
            }
        }
    }


    
    /**
     * Deletes a selected polygon
     */
    public void deletePoly( RegioPolygon p ) 
    {

        if ( m_isJMRUIRelease == false )
        {
            if ( p == m_selectedPolygon )
            {
                m_selectedPolygon = null;
                m_showHistogramOfSelectedRegioPolygon.setEnabled( false );
                m_renameRegion.setEnabled( false );
            }

            LinkedList< RegioPolygon > currentPolygons = m_polygonsVector.get(m_currentlyDisplayedImageNumber );

            currentPolygons.remove( p );

            // let all listenrs know that a polygon has been removed!
            notifyActionListeners( POLYGON_REMOVED_EVENT );

            if ( currentPolygons.size( ) < 2 )
            {
                m_copyAllRegioPolygons.setEnabled( false );
                if ( currentPolygons.size( ) == 1 )
                {
                    m_copyRegioPolygon.setEnabled( true );
                    m_pixelDumpSelectedRegioPolygon.setEnabled( true );
                    m_affineTransformationRegioPolygon.setEnabled( true );
                }
                else
                {
                    m_copyRegioPolygon.setEnabled( false );
                    m_pixelDumpSelectedRegioPolygon.setEnabled( false );
                    m_affineTransformationRegioPolygon.setEnabled( false );
                }
            }

            if ( ( m_imageSegmentsMenu != null ) && ( m_selectedPolygon == null ) )
            {
                m_eliminateOverlayPixelsOutsideFocusRegion.setEnabled( false );
                m_eliminateOverlayPixelsInsideFocusRegion.setEnabled( false );
            }

            repaint( );
        }
    }
    
    /**
     * Deletes the selected polygon
     */
    public void deleteSelectedPoly( ) 
    {
        if ( m_isJMRUIRelease == false )
        {
            LinkedList< RegioPolygon > currentPolygons = new LinkedList< RegioPolygon >( );

            if ( m_polygonsVector.size() > 1 )
            {
                currentPolygons = m_polygonsVector.get( m_currentlyDisplayedImageNumber );
            }
            else
            {
                currentPolygons = m_polygonsVector.get( 0 );
            }


            if ( m_selectedPolygon == null )
            {
                return;
            }
            
            m_lastRemovedRegioPolygonFileName = m_selectedPolygon.getLabel( );

            currentPolygons.remove( m_selectedPolygon );

            // let all listeners know that a polygon has been removed!
            notifyActionListeners( POLYGON_REMOVED_EVENT );

            m_selectedPolygon = null;

            m_showHistogramOfSelectedRegioPolygon.setEnabled( false );
            m_renameRegion.setEnabled( false );
            if ( currentPolygons.size( ) < 2 )
            {
                m_copyAllRegioPolygons.setEnabled( false );
                if ( currentPolygons.size( ) == 1 )
                {
                    m_copyRegioPolygon.setEnabled( true );
                    m_pixelDumpSelectedRegioPolygon.setEnabled( true );
                    m_affineTransformationRegioPolygon.setEnabled( true );
                }
                else
                {
                    m_copyRegioPolygon.setEnabled( false );
                    m_pixelDumpSelectedRegioPolygon.setEnabled( false );
                    m_affineTransformationRegioPolygon.setEnabled( false );
                }
            }

            if ( ( m_imageSegmentsMenu != null ) && ( m_selectedPolygon == null ) )
            {
                m_eliminateOverlayPixelsOutsideFocusRegion.setEnabled( false );
                m_eliminateOverlayPixelsInsideFocusRegion.setEnabled( false );
            }

            repaint( );

            notifyActionListeners( POLYGON_CHANGED_EVENT );
            //notifyActionListeners( POLYGON_REMOVED_EVENT );
        }
    }
    
    /***
     * replicates the RegioPolygons of the currently selected image 
     * to all other images
     */
    public void replicatePolygonListOfImageNumberToAllOther( )
    {
        for ( int i = 0; i < getNumberOfImages( ); i++ )
        {
            
            LinkedList<RegioPolygon> list = m_polygonsVector.get( m_currentlyDisplayedImageNumber );
            
            LinkedList<RegioPolygon> clonedList = new LinkedList<RegioPolygon>( ); 
            
            // first make a deep copy
            for ( int j = 0; j < list.size( ); j++ )
            {
                clonedList.add( new RegioPolygon( list.get( j ) ) );
            }
            
            if ( i != m_currentlyDisplayedImageNumber )
            {
                m_polygonsVector.set( i, clonedList );
            }    
        }
        
    }
    
    
    public void setVectorOfArrayListOfPolygons( Vector< LinkedList< RegioPolygon > > polygons )
    {
        if ( m_polygonsVector.size( ) <= polygons.size( ) )
        {
            for ( int i = 0; i < m_polygonsVector.size( ); i++ )
            {
                
                LinkedList<RegioPolygon> list = polygons.get( i );
                LinkedList<RegioPolygon> clonedList = new LinkedList<RegioPolygon>( );
         
                for ( int j = 0; j < list.size( ); j++ )
                {
                    clonedList.add( new RegioPolygon( list.get( j ) ) );
                }
                m_polygonsVector.set( i, clonedList );
            }
        }
    }
    
    /**
     * Deletes all polygons from the LinkedList of polygons for all
     * images of the image stack
     */
    public void deleteAllPolygonsOfAllImages( ) 
    {
        m_polygonsVector = new Vector< LinkedList<RegioPolygon> >( );
        
        m_selectedPolygon = null;
        
        m_showHistogramOfSelectedRegioPolygon.setEnabled( false );
        m_renameRegion.setEnabled( false );
        m_copyAllRegioPolygons.setEnabled( false );
        m_copyRegioPolygon.setEnabled( false );
        m_pixelDumpSelectedRegioPolygon.setEnabled( false );
        m_affineTransformationRegioPolygon.setEnabled( false );
        m_deleteAllRegioPolygonsOfCurrentlyDisplayedImageOnly.setEnabled( false );
        m_deleteAllRegioPolygonsOfAllImages.setEnabled( false );
        m_scaleRegioPolygon.setEnabled( false );
        
        if ( m_imageSegmentsMenu != null )
        {
            m_eliminateOverlayPixelsOutsideFocusRegion.setEnabled( false );
            m_eliminateOverlayPixelsOutsideFocusRegion.setEnabled( false );
        }
        
        // let all listenrs know that a polygon has been removed!
        notifyActionListeners( POLYGON_REMOVED_EVENT );
        
        repaint( );
        
    }
    
    /**
     * Deletes all polygons from the LinkedList of polygons for all
     * images of the image stack
     */
    public void deleteAllPolygonsOfAllImagesWithoutNotification( ) 
    {
        
        m_polygonsVector = new Vector< LinkedList<RegioPolygon> >( );
        
        for ( int i = 0; i < this.m_numberOfImages; i++ )
        {
            m_polygonsVector.add( new LinkedList<RegioPolygon>( ) );
        }
        
        m_selectedPolygon = null;
        
        m_showHistogramOfSelectedRegioPolygon.setEnabled( false );
        m_renameRegion.setEnabled( false );
        m_copyAllRegioPolygons.setEnabled( false );
        m_copyRegioPolygon.setEnabled( false );
        m_pixelDumpSelectedRegioPolygon.setEnabled( false );
        m_affineTransformationRegioPolygon.setEnabled( false );
        m_deleteAllRegioPolygonsOfCurrentlyDisplayedImageOnly.setEnabled( false );
        m_deleteAllRegioPolygonsOfAllImages.setEnabled( false );
        m_scaleRegioPolygon.setEnabled( false );
        
        if ( m_imageSegmentsMenu != null )
        {
            m_eliminateOverlayPixelsOutsideFocusRegion.setEnabled( false );
            m_eliminateOverlayPixelsOutsideFocusRegion.setEnabled( false );
        }
        
        // let all listenrs know that a polygon has been removed!
        // notifyActionListeners( POLYGON_REMOVED_EVENT );
        
        repaint( );
        
    }
    
    
    /**
     * Deletes all polygons from the LinkedList of polygons for the currently
     * selected image only
     */
    public void deleteAllPolygonsOfCurrentlyDisplayedImageOnly( ) 
    {

        if ( m_isJMRUIRelease == false )
        {
            LinkedList< RegioPolygon > currentPolygons = new LinkedList< RegioPolygon >( );
            if ( ( m_polygonsVector != null ) && ( m_polygonsVector.size( ) > 0 ) )
            {
                currentPolygons = m_polygonsVector.get(m_currentlyDisplayedImageNumber );

                currentPolygons.clear( );
            }

            m_selectedPolygon = null;

            m_showHistogramOfSelectedRegioPolygon.setEnabled( false );
            m_renameRegion.setEnabled( false );
            m_copyAllRegioPolygons.setEnabled( false );
            m_copyRegioPolygon.setEnabled( false );
            m_pixelDumpSelectedRegioPolygon.setEnabled( false );
            m_affineTransformationRegioPolygon.setEnabled( false );
            m_deleteAllRegioPolygonsOfCurrentlyDisplayedImageOnly.setEnabled( false );
            m_scaleRegioPolygon.setEnabled( false );

            if ( m_imageSegmentsMenu != null )
            {
                m_eliminateOverlayPixelsOutsideFocusRegion.setEnabled( false );
                m_eliminateOverlayPixelsOutsideFocusRegion.setEnabled( false );
            }

            // let all listenrs know that a polygon has been removed!
            notifyActionListeners( POLYGON_REMOVED_EVENT );

            repaint( );
        }
    }
    
    /**
     * Deletes a polygon with name polyName
     */
    public void deletePolygon( String polyName )
    {
        if ( m_isJMRUIRelease == false )
        {
            LinkedList< RegioPolygon > currentPolygons = new LinkedList< RegioPolygon >( );

            if ( m_polygonsVector.size( ) > 1 )
            {
                currentPolygons = m_polygonsVector.get(m_currentlyDisplayedImageNumber );
            }
            else
            {
                currentPolygons = m_polygonsVector.get( 0 );
            }

            for ( int i = 0; i < currentPolygons.size( ); i++ )
            {
                if ( currentPolygons.get( i ).getLabel( ).compareTo( polyName ) == 0 )
                {
                    currentPolygons.remove( currentPolygons.get( i ) );
                    // let all listeners know that a polygon has been removed!
                    notifyActionListeners( POLYGON_REMOVED_EVENT );
                }
            }
            repaint( );
        }
    }
    
    /**
     * Deletes a polygon with name polyName
     */
    public void deletePolygonWithoutNotifying( String polyName )
    {
        if ( m_isJMRUIRelease == false )
        {
            LinkedList< RegioPolygon > currentPolygons = new LinkedList< RegioPolygon >( );

            if ( m_polygonsVector.size( ) > 1 )
            {
                currentPolygons = m_polygonsVector.get(m_currentlyDisplayedImageNumber );
            }
            else
            {
                currentPolygons = m_polygonsVector.get( 0 );
            }

            for ( int i = 0; i < currentPolygons.size( ); i++ )
            {
                if ( currentPolygons.get( i ).getLabel( ).compareTo( polyName ) == 0 )
                {
                    currentPolygons.remove( currentPolygons.get( i ) );
                }
            }

            repaint( );
        }
    }
    
    
    
    
    /**
     * Imports a polygon from a specially defined polygon file
     */
    public void importPolysFromFile( String fileName ) 
    {
        if ( m_isJMRUIRelease == false )
        {
            LinkedList< RegioPolygon > currentPolygons = new LinkedList< RegioPolygon >( );

            // first check whether m_polygonVector is defined
            if ( m_polygonsVector.size( ) == 0 )
            {
                m_polygonsVector = new Vector< LinkedList <RegioPolygon> >( );
                for ( int i = 0 ; i < m_numberOfImages; i++ )
                {
                    m_polygonsVector.add( new LinkedList<RegioPolygon>( ) );
                }
            }

            if ( m_polygonsVector.size( ) > 1 )
            {
                currentPolygons = m_polygonsVector.get(m_currentlyDisplayedImageNumber );
            }
            else
            {
                currentPolygons = m_polygonsVector.get( 0 );
            }

            currentPoly = null;
            RegioPolygon p;
            RegFile f = new RegFile( fileName, m_imageWidth, m_imageHeight );
            f.open( "r" );
            f.readHeader( );
            while ( null != ( p = f.readPolygonData( m_zoom ) ) )
            {
                currentPolygons.add( p );
                m_copyRegioPolygon.setEnabled( true );
                m_pixelDumpSelectedRegioPolygon.setEnabled( false );
                m_affineTransformationRegioPolygon.setEnabled( true );
                m_scaleRegioPolygon.setEnabled( true );
                m_showHistogramOfSelectedRegioPolygon.setEnabled( true );
                m_renameRegion.setEnabled( true );
            }
            f.close( );
            repaint( );

            // If there exists regio polygons for this slice then changes
            // of the context popupmenu must occur
            if ( currentPolygons.size( ) == 1 )
            {
                m_copyRegioPolygon.setEnabled( true );
                m_pixelDumpSelectedRegioPolygon.setEnabled( true );
                m_affineTransformationRegioPolygon.setEnabled( true );
                m_scaleRegioPolygon.setEnabled( true );
                setMode( SELECT_MODE );
            }
            if ( currentPolygons.size( ) > 1 )
            {
                m_copyRegioPolygon.setEnabled( true );
                m_pixelDumpSelectedRegioPolygon.setEnabled( true );
                m_affineTransformationRegioPolygon.setEnabled( true );
                m_scaleRegioPolygon.setEnabled( true );
                m_copyAllRegioPolygons.setEnabled( true );
                setMode( SELECT_MODE );
            }
        }
    }
    
    /**
     * Adds a mirrored polygon of the currently selected polygon to the
     * list of polygons
     */
    public void addMirroredPolygon( ) 
    {
        if ( m_isJMRUIRelease == false )
        {
            if ( m_selectedPolygon == null )
            {
                m_affineTransformationRegioPolygon.setEnabled( false );
                return;
            }
            if ( m_mirrorAxis == null )
            {
                m_mirrorAxis = new MirrorAxis( m_parent, m_imageWidth, m_imageHeight, m_zoom );
                m_mirrorAxis.addActionListener( this );
            }

            int[] x = m_selectedPolygon.xpoints.clone( );
            int[] y = m_selectedPolygon.ypoints.clone( );


            for ( int k = 0; k < x.length; k++ )
            {
            // x[k] = (int)( m_imageWidth * m_zoom ) - x[k];

                Point2D.Double point = m_mirrorAxis.getMirrorPoint( (double)x[k], (double)y[k] );
                x[k] = (int)point.x;
                y[k] = (int)point.y;

            }

            addPolygon( x, y, m_selectedPolygon.npoints );

            repaint( );
        }
    }

    /**
     * Adds a mirrored VOIs of the currently selected VOI
     * and adds them to the liost of VOIs
     */
    public void addMirroredVOIs( )
    {
        if ( m_isJMRUIRelease == false )
        {
            for ( int j = 0; j < m_numberOfImages; j++ )
            {
                LinkedList<boolean[][]> overlaysOfImage = m_imageVolumeOfInterestOverlayList.get( j );

                for ( int k = 0; k < m_selectedImageVolumeOfInterestList.size( ); k++ )
                {
                    boolean[][] mirroredOverlay = new boolean[m_imageWidth][m_imageHeight];
                    if ( m_selectedImageVolumeOfInterestList.get( k ).booleanValue( ) == true )
                    {
                        boolean[][] overlayToMirror = overlaysOfImage.get( k );

                        for ( int ii = 0; ii < m_imageHeight; ii++ )
                        {
                            for ( int jj = 0; jj < m_imageWidth; jj++ )
                            {
                                if ( overlayToMirror[ii][jj] == true )
                                {

                                    if ( m_mirrorVOIAxis == null )
                                    {
                                        m_mirrorVOIAxis = new MirrorAxis( m_parent, m_imageWidth, m_imageHeight, m_zoom );
                                        m_mirrorVOIAxis.addActionListener( this );
                                    }

                                    Point2D.Double point = m_mirrorVOIAxis.getMirrorPoint( (double)ii, (double)jj );
                                    mirroredOverlay[(int)( point.x + 0.5 )][(int)( point.y + 0.5 ) ] = true;
                                }
                            }
                        }
                        overlaysOfImage.add( mirroredOverlay );
                    }
                }
            }
        }

        // update related structures
        int numberOfVolumesAdded = m_selectedImageVolumeOfInterestList.size( );
        
        for ( int k = 0; k < numberOfVolumesAdded; k++ )
        {
            m_selectedImageVolumeOfInterestList.add( true );
            m_imageVolumeOfInterestNameList.add( "Volume_" + ( m_imageVolumeOfInterestNameList.size( ) ) );
            m_imageVolumeOfInterestColorList.add( defaultColors[m_imageVolumeOfInterestColorList.size( ) ] );
        }

        repaint( );
    }




    /**
     * Deletes the currently selected triangle
     */
    public void deleteRectangle( ) 
    {
        currentRectangle = null;
    }
    
    /**
     * Returns the bounding RegioPolygon p
     */
    public Polygon getBoundingRect( RegioPolygon p ) 
    {
        Rectangle bb = p.getBounds( );
        return new Polygon( new int[] { bb.x, bb.x + bb.width, bb.x + bb.width, bb.x },
                            new int[] { bb.y, bb.y, bb.y + bb.height, bb.y + bb.height }, 4 );
    }
    
    /**
     * Closing the RegioPolygon p
     */
    private int getCloseVertex( RegioPolygon p, int x, int y ) 
    {
        for ( int k = 0; k < p.npoints; k++ )
        {
            if ( ( p.xpoints[k] - x ) * ( p.xpoints[k] - x ) +
                 ( p.ypoints[k] - y ) * ( p.ypoints[k] - y ) < MAX_DISTANCE2 ) 
            {
            return k;
            }
        }
        return -1;
    }

    
   
    /**
     * Stores the (Dicom) image in tiff format
     */
    public int writeAsTrueDicom( String fileName )
    {
        try 
        {
            RandomAccessFile fout = new RandomAccessFile( fileName, "rw" );

            byte[] leadingZeroes = new byte[128];

            fout.write( leadingZeroes );

            String dicm = new String( "DICM" ).trim( );
                  
            fout.write( dicm.getBytes( ) );

            m_fileInfo.writePatientRelatedTags( fout );
            m_fileInfo.writeStudySeriesInstancesRelatedTags( fout );
            m_fileInfo.writeMRRelatedTags( fout );
            m_fileInfo.writeGeometryRelatedTags( fout );
            m_fileInfo.writeOtherTags( fout );
            if ( m_fileInfo.isJMRUICreatedDICOMReportFile( ) == true )
            {
                writeJMRUIDICOMTagsToFile( fout );
            }


            writePixelInformation( fout );

            fout.close( );

            return IO_OK;
        }
        catch ( IOException ex )
        {
            Logger.getLogger( DicomSliceCanvasPolyDraw.class.getName( ) ).log( Level.SEVERE, null, ex );
        }

        return 0;
    }


    /**
     * If this object is a JMRUI DICOM report object, then this TAG is written
     * automatically to the File.
     */
    private void writeJMRUIDICOMTagsToFile(  RandomAccessFile fout ) throws IOException
    {
        System.out.println("!!! >>> Writing -> writeJMRUIDICOMTagsToFile" );

        byte[] jmruiInfoAsBytes = m_fileInfo.getJMRUIInfo( ).getBytes( );

        fout.writeInt( Integer.rotateLeft( Integer.reverseBytes( DicomDecoder.JMRUI_INFO_TAG ), 16 ) );

        fout.writeShort( DicomDecoder.OW );

        byte[] byteArr = new byte[2];

        fout.write( byteArr );

        fout.writeInt( Integer.reverseBytes( jmruiInfoAsBytes.length ) );

        fout.write( jmruiInfoAsBytes );
    }

    /**
     * Depending on the type of the pixel information,
     * @param fout
     */
    private void writePixelInformation( RandomAccessFile fout ) throws IOException
    {

        //int outp0 = Integer.rotateLeft( DicomDecoder.PIXEL_DATA, 8 );
        fout.writeInt( Integer.rotateLeft( Integer.reverseBytes( DicomDecoder.PIXEL_DATA ), 16 ) );

        fout.writeShort( DicomDecoder.OW );

        byte[] pixels2write = new byte[ 2 * m_imageHeight * m_imageWidth ];

        for ( int i = 0; i < m_imageHeight * m_imageWidth; i++ )
        {
            org.spectrim.lib.Convert.int2wordl( pixels2write, 2 * i, m_rawData[ m_currentlyDisplayedImageNumber * m_imageHeight * m_imageWidth + i ] );
        }

        byte[] byteArr = new byte[2];

        fout.write( byteArr );

        fout.writeInt( Integer.reverseBytes( pixels2write.length ) );

        fout.write( pixels2write );

    }

    /**
     * Stores the (Dicom) image in tiff format
     */
    public int writeAsTiff( String fileName ) 
    {
        int w = (int)( m_imageWidth * m_zoom );
        int h = (int)( m_imageHeight * m_zoom );
        Image imageCopy = getImageInstance( );
        Graphics2D g = (Graphics2D)imageCopy.getGraphics( );

        boolean drawPolygons = true;

        if ( m_displayMeasureDistanceMode == true )
        {
           this.drawDistances( g );
        }

        if ( drawPolygons )
        {
            drawPolygons( g );
        }

        int[] pixels = new int[ w * h ];
        PixelGrabber pg = new PixelGrabber( imageCopy, 0, 0, w, h, pixels, 0, w );
        try
        {
            pg.grabPixels( );
        }
        catch ( InterruptedException e )
        {
            return  IO_INTERRUPTED;
        }
        if ( ( pg.getStatus( ) & ImageObserver.ABORT ) != 0 )
        {
            return IO_ERROR;
        }
        Tiff T = new Tiff( );
        T.setImage( pixels, w, h );
        if ( T.TIFF_OK != T.writeFullRGB( fileName ) )
        {
            return IO_ERROR;
        }
        return IO_OK;
    }

    /**
     * Draws all visible distances defined by the user
     */
    private void drawDistances( Graphics g )
    {
        if ( ( m_displayMeasureDistanceMode == true ) && ( m_distancesVector.size( ) > 0 ) )
        {
  
            LinkedList<Distance> distLinkedList = m_distancesVector.get(m_currentlyDisplayedImageNumber );

            for( int i= 0; i < distLinkedList.size( ); i++ )
            {
                Distance dst = distLinkedList.get( i );

                if ( dst.getImageNumber( ) == m_currentlyDisplayedImageNumber )
                {
                    g.setColor( Color.MAGENTA );

                    Point begin = dst.getBeginPoint( );
                    g.drawLine( (int)( m_zoom * begin.x - 3 ), (int)( m_zoom * begin.y ),
                                (int)( m_zoom * begin.x + 3 ) , (int)( m_zoom * begin.y ) );
                    g.drawLine( (int)( m_zoom * begin.x ), (int)( m_zoom * begin.y - 3 ),
                                (int)( m_zoom * begin.x ), (int)( m_zoom * begin.y + 3 ) );

                    Point end = dst.getEndPoint( );
                    g.drawLine( (int)( m_zoom * end.x - 3), (int)( m_zoom * end.y ),
                                (int)( m_zoom * end.x + 3), (int)( m_zoom * end.y ) );
                    g.drawLine( (int)( m_zoom * end.x ), (int)( m_zoom * end.y - 3 ),
                                (int)( m_zoom * end.x ), (int)( m_zoom * end.y + 3 ) );


                    g.drawLine( (int)( m_zoom * begin.x ), (int)( m_zoom * begin.y ),
                                (int)( m_zoom * end.x ), (int)( m_zoom * end.y ) );

                    DecimalFormat fm = new DecimalFormat( "#0.0" );
                    String formattedString = fm.format( (double)dst.getDistanceValue( ) );
                    g.setColor( Color.YELLOW );
                    g.drawString( formattedString + " mm", (int)( m_zoom * ( begin.x + end.x ) / 2 ), (int)( m_zoom * ( begin.y + end.y ) / 2 ) );
                }
            }
        }
        else
        {
            m_distancesVector = new Vector< LinkedList <Distance> >( );
            m_distancesVector.add( new LinkedList <Distance>( ) );
        }
    }


    /**
     * Routine for drawing polygons
     */
    private void drawPolygons( Graphics g ) 
    {
        FontMetrics fm = g.getFontMetrics( );
        int labelAscent = fm.getAscent( );
        LinkedList< RegioPolygon > currentPolygons = new LinkedList<RegioPolygon>( );
        if ( m_polygonsVector.size( ) > 0 )
        {
            // System.out.println( "DrawPolygons :: m_currently displayed image : " +  m_currentlyDisplayedImage );
            if ( m_polygonsVector.size( ) > 1 )
            {
                currentPolygons = m_polygonsVector.get(m_currentlyDisplayedImageNumber );
            }
            else
            {
                currentPolygons = m_polygonsVector.get( 0 );
            }
            
            if ( currentPolygons.size( ) > 0 )
            {
                for ( int k = 0; k < currentPolygons.size( ); k++ ) 
                {
                    RegioPolygon p = currentPolygons.get( k );
                    Rectangle r = p.getBounds( );
            
                    p.translate( m_imageTop, m_imageLeft );
                    g.setColor( p.m_color );
                    g.drawPolygon( (Polygon)p );
	    
                    int labelLength = fm.stringWidth( p.m_label );
                    int labelX = Math.min( r.x + r.width, (int)( m_imageWidth * m_zoom ) - labelLength );
                    int labelY = Math.max( r.y, labelAscent );
                    g.setColor( p.m_labelColor );
            
                    g.drawString( p.m_label, m_imageTop + labelX , m_imageLeft + labelY );
            
                    p.translate( -m_imageTop, -m_imageLeft );
                }
            }
        }
        else
        {
            m_polygonsVector = new Vector< LinkedList <RegioPolygon> >( );
            for ( int i = 0 ; i < this.m_numberOfImages; i++ )
            {
                m_polygonsVector.add( new LinkedList<RegioPolygon>( ) );
            }
        }
        
        // also draw the region of special interest
        if ( m_roiToSegment != null )
        {
            Rectangle r = m_roiToSegment.getBounds( );
            
            m_roiToSegment.translate( m_imageTop, m_imageLeft );
            g.setColor( m_roiToSegment.m_color );
            g.drawPolygon( (Polygon)m_roiToSegment );
	    
            int labelLength = fm.stringWidth( m_roiToSegment.m_label );
            int labelX = Math.min( r.x + r.width, (int)( m_imageWidth * m_zoom ) - labelLength );
            int labelY = Math.max( r.y, labelAscent );
            g.setColor( m_roiToSegment.m_labelColor );
            
            g.drawString( m_roiToSegment.m_label, m_imageTop + labelX , m_imageLeft + labelY );
            
            m_roiToSegment.translate( -m_imageTop, -m_imageLeft );
        }
    }

    /**
     * Routine for painting the component
     */
    @Override
    public synchronized void paintComponent( Graphics g ) 
    {
        
        definePagerLocation( );
         
        super.paintComponent( g );
        if ( g != null )
        {    

            if ( ( m_slopes != null ) && ( m_slopes.length == m_numberOfImages ) )
            {
                if ( ( m_intercepts != null ) && ( m_intercepts.length == m_numberOfImages ) )
                {
                    for ( int i = 0; i < (m_zoom * m_imageWidth)-1; i++ )
                    {   
                        g.drawLine( (int)(m_zoom * (m_intercepts[m_currentlyDisplayedImageNumber] + m_slopes[m_currentlyDisplayedImageNumber]*i)), (int)(m_zoom * i ),
                                    (int)(m_zoom * (m_intercepts[m_currentlyDisplayedImageNumber] + m_slopes[m_currentlyDisplayedImageNumber]*(i+1))), (int)(m_zoom * (i+1) ) );
                    }
                }
            }
            
            if ( m_doubleClickedPoint3D != null )
            {
                g.drawOval( (int)( m_zoom * m_doubleClickedPoint.x ), (int)( m_zoom * m_doubleClickedPoint.y ), 5, 5 );
            }

            if ( previousPoly != null ) 
            {
                g.setColor( m_selectedPolygon.m_color );
                g.setXORMode( Color.white );
                g.drawPolygon( previousPoly );
                previousPoly = null;
            }
            else
            {
                g.setPaintMode( );
            }
            
            if ( inPolygonDrawMode == false )
            {
                drawColoredThresholdOverlay( g );
        
                drawBufferedImageThresholdOverlay( g );
                
                drawBufferedImageMHAOverlay( g );

                if ( m_dtiFiberColors != null )
                {
                    drawDTIFiberImageThresholdOverlay( g );
                }

                if ( this.m_activationImageOverlays != null )
                {
                    drawfMRIActivationImageThresholdOverlay( g );
                }

                drawBufferedImageImageSegmentsOverlay( g );

                // drawBufferedImageVolumeOfInterestOverlay( g );
                
            }
            
            drawBufferedImageVolumeOfInterestOverlay( g );

            if ( m_localStatisticsDisplayOverlayType != -1 )
            {
                drawLocalStatisticsOverlay( g );
            }

            if ( ( m_leftSideImageSegmentOverlays != null ) || ( m_rightSideImageSegmentOverlays != null ) )
            {
                drawLeftandRightSideOverlay( g );
            }
            
            // too time consuming; obsolete drawTissueSegmentOverlays( g );

            if ( m_interactiveSegmentationMode == true )
            {
                if ( m_interactiveSegmentationOverlay != null )
                {
                    drawInteractiveOverlay( g );
                }
            }
            // draw the contour containing image is 
            if ( m_contoursAsBooleanImageOverlays != null )
            {
                drawContoursAsBooleanImageOverlay( g );
            }

            if ( m_displayMeasureDistanceMode == true )
            {
                this.drawDistances( g );
            }

            drawPolygons( g );

            if ( m_selectedPolygon != null && showVertices ) 
            {
                g.setColor( Color.yellow );
                for ( int i = 0; i < m_selectedPolygon.npoints; i++ )
                { 
                    g.fillOval( m_selectedPolygon.xpoints[i] - 2, m_selectedPolygon.ypoints[i] - 2, 4, 4 );
                }
            }
            if ( currentPoly != null ) 
            {
                if ( isPolygonDrawing || isCutting ) 
                {
                    g.setColor( Color.yellow );
                    g.drawPolyline( currentPoly.xpoints, 
                                    currentPoly.ypoints, 
                                    currentPoly.npoints );
                }
            }
        
            if ( currentRectangle != null ) 
            {
                g.setColor( Color.orange );
                if ( previousRectangle != null ) 
                {
                    g.setXORMode( Color.magenta );
                    g.drawRect( previousRectangle.x, previousRectangle.y,
                                previousRectangle.width, previousRectangle.height );
                    previousRectangle = null;
                }
                else
                { 
                    g.setPaintMode( );
                }
                g.drawRect( currentRectangle.x, currentRectangle.y,
                            currentRectangle.width, currentRectangle.height );
            }
        
            drawImageAnnotation( g );

            if ( m_showROIMirrorAxis != null)
            {

                if ( m_showROIMirrorAxis.isSelected( ) == true )
                {
                    drawMirrorAxis( g );
                }
            }

            if ( m_showVOIMirrorAxis != null )
            {
                if ( m_showVOIMirrorAxis != null )
                {
                    if ( m_showVOIMirrorAxis.isSelected( ) == true )
                    {
                        drawVOIMirrorAxis( g );
                    }
                }
            }
            
            drawTopTitle( g );
            
        }
    }
    
    /**
     * Draws the mirror axis
     */
    private void drawMirrorAxis( Graphics g )
    {
        m_mirrorAxis.draw( g );
    }

    /**
     * Draws the mirror axis
     */
    private void drawVOIMirrorAxis( Graphics g )
    {
        m_mirrorVOIAxis.draw( g );
    }


    /**
     * Draws the image annotaton on the image
     */
    public void drawImageAnnotation( Graphics g  )
    {
        g.drawString( this.imageNumber, 10, 20 );
        g.drawString( this.imageTime, 10, 34 );
    }
    
    /**
     * Draws the image comment on the image
     */
    public void drawTopTitle( Graphics g )
    {
        if ( m_topTitle != null )
        {
            FontMetrics fontMetrics = g.getFontMetrics( );
            int width = fontMetrics.stringWidth( m_topTitle ); 
            g.drawString( m_topTitle, m_imageWidth/2 - width/2, 10 );
        }
    }
    
   
    
    /**
     * Sets the image number
     */
    public void setImageNumber( String number )
    {
        imageNumber = number;     
    }

    /**
     * Sets the image time
     */
    public void setImageTime( String time )
    {
        imageTime = time;     
    }
     
    /**
     * Is called whenever the mouse is dragged
     */
    public void mouseDragged( MouseEvent e ) 
    {
        // super.mouseDragged( e );
        int modifiers = e.getModifiers( );
        int x = e.getX( );
        int y = e.getY( );
        
        if ( ( modifiers & MouseEvent.BUTTON3_MASK ) > 0 )
        {
            // nothing is done!
        }
        
        // middle mouse button was pressed and dragged:
        // adjustment of windowing
        if ( ( modifiers & MouseEvent.BUTTON2_MASK ) > 0 )
        {
            int dX = x - startMouseDragX;
            int dY = y - startMouseDragY;
            setColormap( m_startBrightness - dX, m_startContrast + dY );
            update( true );
        }
        
        // System.out.println( "Mouse pressed at ( " + x + " / " + y + ")" );

        if ( ( modifiers & MouseEvent.BUTTON1_MASK ) > 0 )
        {
	    
            if ( inPolygonDrawMode ) 
            {
                if ( !isPolygonDrawing ) return;
                currentPoly.addPoint( x, y );
                this.repaint( x - 2, y - 2, 5 ,5 );
            }
            else if ( inRectangleDrawMode )
            {
                if ( !isRectangleDrawing ) return;
                previousRectangle = new Rectangle( currentRectangle );
                int w = x - lastMouseDragX;
                int h = y - lastMouseDragY;
                if ( w < 0 )
                {
                    currentRectangle.x = x;
                    currentRectangle.width = -w;
                }
                else
                {
                    currentRectangle.width = w;
                }
                if ( h < 0 )
                {
                    currentRectangle.y = y;
                    currentRectangle.height = -h;
                }
                else currentRectangle.height = h;
		
                update( true );
            }
            else if ( inResizeMode )
            {
                if ( !isResizing || m_selectedPolygon == null )
                { 
                    return;
                }
                m_selectedPolygon.xpoints[resizeVertex] = x;
                m_selectedPolygon.ypoints[resizeVertex] = y;
                //www update( false );
                repaint( x - 100, y - 100, 200 , 200 );
                notifyActionListeners( POLYGON_CHANGED_EVENT );
            }
            else if ( inCutMode )
            {
                if ( !isCutting ) return;
                    currentPoly.addPoint( x, y );
                    update( true );
            }
            else if ( !inSelectMode && !inDeleteMode && m_selectedPolygon != null )
            {
                previousPoly = m_selectedPolygon.copy( );
                m_selectedPolygon.translate( x - lastMouseDragX, y - lastMouseDragY );
                lastMouseDragX = x;
                lastMouseDragY = y;
		
                this.repaint( x - 50, y - 50, 100 , 100 );
                
                // notify the listeners that the polygon is being dragged
                multicaster.polygonDrag( new PolygonEvent( this,
                                         PolygonEvent.POLYGON_DRAG, 
                                         m_selectedPolygon ) );
                
                notifyActionListeners( POLYGON_CHANGED_EVENT );
            }
        }
    }
    
    /**
     * Is called whenever a mouse button is pressed
     * @param e
     */
    public void mousePressed( MouseEvent e ) 
    {
        // super.mousePressed( e );

        
        if ( e.isPopupTrigger( ) )
        {
            m_popupmenu.show( e.getComponent( ), e.getX( ), e.getY( ) );
        }
        if ( ( e.getModifiers( ) & MouseEvent.BUTTON2_MASK ) > 0 ) 
        {
            startMouseDragX = e.getX( );
            startMouseDragY = e.getY( );
            m_startBrightness = m_brightness;
            m_startContrast = m_contrast;
            prevCursor = getCursor( );
            setCursor( new Cursor( Cursor.MOVE_CURSOR ) );
        }
        
        if ( ( e.getModifiers( ) & MouseEvent.BUTTON1_MASK ) > 0 )
        {
            int x = e.getX( );
            int y = e.getY( );

            // System.out.println( "Mouse pressed at (" + x + " / " + y + ")" );

            lastMouseDragX = x;
            lastMouseDragY = y;
            if ( inPolygonDrawMode )
            {
                currentPoly = newPolygon( x, y );
                if ( ( currentPoly != null ) )
                {
                    m_copyRegioPolygon.setEnabled( true );
                    m_pixelDumpSelectedRegioPolygon.setEnabled( true );
                    m_affineTransformationRegioPolygon.setEnabled( true );
                    m_scaleRegioPolygon.setEnabled( true );
                }
            }
            else if ( inRectangleDrawMode )
            {
                isRectangleDrawing = true;
                currentRectangle = new Rectangle( x, y, 0, 0 );
            }
            else if ( inResizeMode )
            {
                if ( m_selectedPolygon == null ) return;
                resizeVertex = getCloseVertex( m_selectedPolygon, e.getX( ), e.getY( ) );
                isResizing = ( resizeVertex >= 0 );
            }
            else if ( inDeleteMode )
            {
                // nothing
            }
            else if ( inCutMode )
            {
                
                LinkedList< RegioPolygon > currentPolygons = new LinkedList< RegioPolygon >( );
        
                if ( m_polygonsVector.size( ) > 1 )
                {
                    currentPolygons = m_polygonsVector.get( m_currentlyDisplayedImageNumber );
                }
                else
                {
                    currentPolygons = m_polygonsVector.get( 0 );
                }
                
                
                if ( currentPolygons.size( ) == 0 )
                { 
                    return; 
                }
              
                RegioPolygon p = currentPolygons.get( 0 );
                m_selectedPolygon = p;
                int[] vInfo = p.getNearestVertex( x, y );
                for ( int k = 1; k < currentPolygons.size( ); k++ )
                {
                    p = currentPolygons.get( k );
                    int[] i = p.getNearestVertex( x, y );
                    if ( i[3] < vInfo[3] )
                    {
                        System.arraycopy( i, 0, vInfo, 0, i.length );
                        m_selectedPolygon = p;
                    }
                }
                // if we are too far from a polygon, cancel the action
                if ( vInfo[3] > 100 )
                {
                    m_selectedPolygon = null;
                    return;
                }
                isCutting = true;
                cutStartIndex = vInfo[0];
                currentPoly = new RegioPolygon(99);
                currentPoly.addPoint( m_selectedPolygon.xpoints[vInfo[0]],
                                      m_selectedPolygon.ypoints[vInfo[0]] );
            }
            else
            {
		
                m_selectedPolygon = null;
                LinkedList< RegioPolygon > currentPolygons = new LinkedList< RegioPolygon >( );
                
                if ( m_polygonsVector.size( ) > 1 )
                {
                    currentPolygons = m_polygonsVector.get(m_currentlyDisplayedImageNumber );
                }
                else
                {
                    currentPolygons = m_polygonsVector.get( 0 );
                }
                
                for ( int k = 0; k < currentPolygons.size( ); k++ )
                {
                    RegioPolygon p = currentPolygons.get( k );
                    if ( p.contains( x, y ) )
                    {
                    	p.m_color = Color.blue;
                        m_selectedPolygon = p;
                    }
                    else p.m_color = Color.red;
                }
                // notify the listeners that a polygon has been selected
                multicaster.polygonSelect( new PolygonEvent( this,
                                           PolygonEvent.POLYGON_SELECT, 
                                           m_selectedPolygon ) );
                repaint( );
            }
        }
    }
    
    /**
     * Is called whenever the mouse button is released
     */
    @Override
    public void mouseReleased( MouseEvent e ) 
    {
        int x = e.getX( );
        int y = e.getY( );

        
        if ( m_addOneCircularROIMode.isSelected() )
        {
            finishPolygon( x, y );
        }
         
        if ( e.isPopupTrigger( ) )
        {
            m_popupmenu.show( e.getComponent( ), e.getX( ), e.getY( ) );
        }
        
        if ( ( e.getModifiers( ) & MouseEvent.BUTTON2_MASK ) > 0 )
        {
            setCursor( prevCursor );
        }

        int modifiers = e.getModifiers( );
        if ( ( modifiers & MouseEvent.BUTTON1_MASK ) > 0 )
        {
            if ( inPolygonDrawMode )
            {
                if ( !isPolygonDrawing ) return;
                finishPolygon( );
                if ( m_selectedPolygon != null )
                {
                    update( true );
                    // notify the listeners that the polygon has changed
                    multicaster.polygonChanged( new PolygonEvent( this, 
                                                PolygonEvent.POLYGON_CHANGED, 
                                                m_selectedPolygon ) );

                    if ( m_eliminatePixelsFromOverlaysByContourDrawingModeRadioButton != null )
                    {
                        if ( m_eliminatePixelsFromOverlaysByContourDrawingModeRadioButton.isSelected( ) == true )
                        {
                            handleRemovePixelsFromSelectedImageSegmentOverlays( );

                            deleteSelectedPoly( );
                            update( true );
                            repaint( );
                        }
                    }

                    if ( m_eliminatePixelsFromVolumeOfInterestOverlaysByContourDrawingModeRadioButton != null )
                    {
                        if ( m_eliminatePixelsFromVolumeOfInterestOverlaysByContourDrawingModeRadioButton.isSelected( ) == true )
                        {
                            handleRemovePixelsFromSelectedVolumeOfInterestOverlays( );

                            deleteSelectedPoly( );
                            update( true );
                            repaint( );
                        }
                    }

                    

                }
            }
            else if ( inRectangleDrawMode )
            {
                if ( !isRectangleDrawing ) return;
                int w = x - lastMouseDragX;
                int h = y - lastMouseDragY;
                if ( w == 0 || h == 0 )
                {
                    isRectangleDrawing = false;
                    setMode( DRAG_MODE  );
                return;
                }
                if ( w < 0 )
                {
                    currentRectangle.x = x;
                    currentRectangle.width = -w;
                }
                else currentRectangle.width = w;
                if ( h < 0 )
                {
                    currentRectangle.y = 0;
                    currentRectangle.height = -h;
                }
                else currentRectangle.height = h;
                setMode( DRAG_MODE  );
                update( true );
                // notify the listeners that the polygon has changed
                multicaster.polygonChanged( new PolygonEvent( this,
                                            PolygonEvent.RECTANGLE_CHANGED, 
                                            currentRectangle ) );
            }
            else if ( inResizeMode )
            {
                if ( !isResizing || m_selectedPolygon == null ) return;
                isResizing = false;
                m_selectedPolygon.calculateBounds( );
            update( true );
            // notify the listeners that the polygon has changed
            multicaster.polygonChanged( new PolygonEvent( this,
                                            PolygonEvent.POLYGON_CHANGED, 
                                            m_selectedPolygon ) );
            // reset the cursor
            setCursor( new Cursor( Cursor.HAND_CURSOR ) );
            }
            else if ( inDeleteMode )
            {
                m_selectedPolygon = null;
                LinkedList< RegioPolygon > currentPolygons = new LinkedList< RegioPolygon >( );
                
                if ( m_polygonsVector.size( ) > 1 )
                {
                    currentPolygons = m_polygonsVector.get(m_currentlyDisplayedImageNumber );
                }
                else
                {
                    currentPolygons = m_polygonsVector.get( 0 );
                }
                
                for ( int k = 0; k < currentPolygons.size( ); k++ )
                {
                    RegioPolygon p = currentPolygons.get( k );
                    if ( p.contains( x,y ) )
                    {
                        m_selectedPolygon = p;
                        break;
                    }
                }
                // delete the polygon that was clicked
                deleteSelectedPoly( );
                update( true );
                // notify the listeners that the polygon has changed
                multicaster.polygonRemoved( new PolygonEvent( this,
                                            PolygonEvent.POLYGON_CHANGED, 
                                            null ) );
            }
            else if ( inCutMode )
            {
                if ( m_selectedPolygon != null )
                {
                    m_parentPolyonToCut = new RegioPolygon( m_selectedPolygon );
                 
                    if ( !isCutting ) return;
                    int[] i = m_selectedPolygon.getNearestVertex( e.getX( ), e.getY( ) );
                    // if we are too far from the polygon, cancel the action
                    if ( i[3] > 100 )
                    {
                        m_selectedPolygon = null;
                        currentPoly = null;
                        return;
                    }
                    cutEndIndex = i[0];
                    int i1, i2, j1, j2, dj;
                    if ( cutStartIndex <= cutEndIndex )
                    {
                        i1 = cutStartIndex;
                        i2 = cutEndIndex;
                        j1 = 1;
                        j2 = currentPoly.npoints - 1;
                        dj = 1;
                    }
                    else
                    {
                        i2 = cutStartIndex;
                        i1 = cutEndIndex;
                        j2 = 1;
                        j1 = currentPoly.npoints - 1;
                        dj = -1;
                    }
                    RegioPolygon tmpPoly = new RegioPolygon( 99 );
                    tmpPoly.appendPoints( m_selectedPolygon.xpoints, m_selectedPolygon.ypoints, m_selectedPolygon.npoints );
                    m_selectedPolygon.clear( );
                    m_selectedPolygon.appendPoints( tmpPoly.xpoints,
                                                    tmpPoly.ypoints, 
                                                    i1 + 1 );
                    m_selectedPolygon.appendPoints( currentPoly.xpoints,
                                                    currentPoly.ypoints, 
                                                    j1, 
                                                    j2 );
                    m_selectedPolygon.appendPoints( tmpPoly.xpoints,
                                                    tmpPoly.ypoints, 
                                                    i2, 
                                                    tmpPoly.npoints - 1 );
                
                    String parentLabel = m_selectedPolygon.getLabel( );
                
                    m_selectedPolygon.setLabel( parentLabel + "_1" );
                    m_childOneAfterCut = new RegioPolygon( m_selectedPolygon );
                
                    RegioPolygon newPoly = new RegioPolygon( m_polygonsVector.size( ) + 1 );
                    newPoly.appendPoints( tmpPoly.xpoints, tmpPoly.ypoints, i1, i2 );
                    newPoly.appendPoints( currentPoly.xpoints, currentPoly.ypoints, j2, j1 );
                
                    newPoly.setLabel( parentLabel + "_2" );
                
                    // polygons.addElement( newPoly );
                
                    m_childTwoAfterCut = new RegioPolygon( newPoly );
                
                    LinkedList< RegioPolygon > currentPolygons = new LinkedList< RegioPolygon >( );
        
                    if ( m_polygonsVector.size( ) > 1 )
                    {
                        currentPolygons = m_polygonsVector.get( m_currentlyDisplayedImageNumber );
                    }
                    else
                    {
                        currentPolygons = m_polygonsVector.get( 0 );
                    }
                
                    currentPolygons.add( newPoly );
                    m_selectedPolygon = newPoly;
                    currentPoly = null;
                    update( true );
                
                    // Notify the listeners that the polygon has changed
                    multicaster.polygonChanged( new PolygonEvent( this, PolygonEvent.POLYGON_CUT_EVENT, m_parentPolyonToCut ) );
                    // also inform listeners to this
                    notifyActionListeners( POLYGON_CUT_EVENT );
                }
            }
            else if ( m_selectedPolygon != null )
            {
                if ( !inSelectMode == true )
                {
                    m_selectedPolygon.translate( e.getX( ) - lastMouseDragX, e.getY( ) - lastMouseDragY );
                    update( true );
                    // notify the listeners that the polygon has changed
                    multicaster.polygonChanged( new PolygonEvent( this, 
                                                PolygonEvent.POLYGON_CHANGED, 
                                                m_selectedPolygon ) );
                }
            }
        }
    }

    
    
    public RegioPolygon getLastCutPolygon( )
    {
        return m_parentPolyonToCut;
    }        
            
    public RegioPolygon getPolygonSegmentOneAfterCut( )
    {
        return m_childOneAfterCut;
    }
    
    public RegioPolygon getPolygonSegementTwoAfterCut( )
    {
        return m_childTwoAfterCut;
    }
    

    /**
     * Returns the number to be displayed next
     * @return
     */
    public int getImageNumberTobeDisplayedNext( )
    {
        return this.m_currentlyDisplayedImageNumber;
    }

    /**
     *
     * @return whether their is a next larger image number be displayed?
     */
    public boolean canNextLargerImageNumberBeDisplayed( )
    {
        if ( m_currentlyDisplayedImageNumber + 1 > m_numberOfImages )
        {
            return false;
        }
        else
        {
            return true;
        }
    }

    /**
     *
     * @return whether their is a next smaller image number be displayed?
     */
    public boolean canNextSmallerImageNumberBeDisplayed( )
    {
        if ( m_currentlyDisplayedImageNumber - 1 < -1 )
        {
            return false;
        }
        else
        {
            return true;
        }
    }

    /**
     * Returns the currently displayed image number.
     * @return m_currentlyDisplayedImageNumber
     */
    public int getDisplayedImageNumber( )
    {
        return m_currentlyDisplayedImageNumber;
    }
    
    /**
     * Displays the next image number
     */
    public void displayNextDicomImage( )
    {
        m_currentlyDisplayedImageNumber = m_currentlyDisplayedImageNumber + 1;
        displayDicomImage(m_currentlyDisplayedImageNumber );
        drawImage( false );
    }

    /**
     * Displays the previous image number
     */
    public void displayPreviousDicomImage( )
    {
        m_currentlyDisplayedImageNumber = m_currentlyDisplayedImageNumber - 1;
        displayDicomImage(m_currentlyDisplayedImageNumber );
        drawImage( false );
    }


     /**
      * 
      * Is called when the user moves the mouse wheel
      * 
      * @param e
      */
     public void mouseWheelMoved( MouseWheelEvent e ) 
     {
        int diff = e.getWheelRotation( );

        m_doubleClickedPoint = null;
        m_doubleClickedPoint3D = null;

//        if ( ( m_currentlyDisplayedImage - diff < getNumberOfImages( ) ) && ( m_currentlyDisplayedImage - diff > -1 ) )
//        {
//            m_currentlyDisplayedImage -= diff;
//        }
        
        boolean didChange = false;
        
        if ( ( diff < 0 ) && ( m_currentlyDisplayedImageNumber != 0 ) )
        {
            m_currentlyDisplayedImageNumber = m_currentlyDisplayedImageNumber - 1;
            
            didChange = true;
            m_currentlyDisplayedDTIOverlayImageNumber = m_currentlyDisplayedImageNumber;
            m_currentlyDisplayedfMRIOverlayImageNumber = m_currentlyDisplayedImageNumber;
        }
        
        if ( ( diff > 0 ) && ( m_currentlyDisplayedImageNumber + 1 != m_numberOfImages ) )
        {
            m_currentlyDisplayedImageNumber = m_currentlyDisplayedImageNumber + 1;
            didChange = true;
            m_currentlyDisplayedDTIOverlayImageNumber = m_currentlyDisplayedImageNumber;
            m_currentlyDisplayedfMRIOverlayImageNumber = m_currentlyDisplayedImageNumber;
        }
        
        if ( didChange == true )
        {

            if ( diff > 0 )
            {
                notifyActionListeners( DISPLAY_NEXT_IMAGE_OF_SLICE_STACK_EVENT, 
                                       DISPLAY_NEXT_IMAGE_OF_SLICE_STACK_EVENT_ACTION_COMMAND );
            }

            if ( diff < 0 )
            {
                notifyActionListeners( DISPLAY_PREVIOUS_IMAGE_OF_SLICE_STACK_EVENT, 
                                       DISPLAY_PREVIOUS_IMAGE_OF_SLICE_STACK_EVENT_ACTION_COMMAND );
            }
        }
//        else
//        {
//            if ( m_currentlyDisplayedImage < 0 )
//            {
//                m_currentlyDisplayedImage = 0;
//                m_currentlyDisplayedDTIOverlayImageNumber = m_currentlyDisplayedImage;
//                m_currentlyDisplayedfMRIOverlayImageNumber = m_currentlyDisplayedImage;
//            }
//            
//            if ( m_currentlyDisplayedImage > m_numberOfImages - 1 )
//            {
//                m_currentlyDisplayedImage = m_numberOfImages - 1;
//                m_currentlyDisplayedDTIOverlayImageNumber = m_currentlyDisplayedImage;
//                m_currentlyDisplayedfMRIOverlayImageNumber = m_currentlyDisplayedImage;
//            }
//            
//        }
        
        System.out.println( "m_currentlyDisplayedImage " + m_currentlyDisplayedImageNumber );

        displayDicomImage( m_currentlyDisplayedImageNumber );
        updateRegionPopupMenu( );
        drawImage( false );
     }
    
     /**
      * Updates the setEnabled properties
      */
     public void updateRegionPopupMenu( )
     {

        if ( m_isJMRUIRelease == false )
        {
            LinkedList< RegioPolygon > currentPolygons = new LinkedList< RegioPolygon >( );

            if ( m_polygonsVector.size( ) == 1 )
            {
                currentPolygons = m_polygonsVector.get( 0 );
            }
            else
            {
                currentPolygons = m_polygonsVector.get(m_currentlyDisplayedImageNumber );
            }

            if ( currentPolygons.size( ) > 0 )
            {
      //           m_selectedPolygon = currentPolygons.get( 0 );
            }

            if ( currentPolygons.size( ) == 0 )
            {
               m_copyRegioPolygon.setEnabled( false );
               m_pixelDumpSelectedRegioPolygon.setEnabled( false );
               m_affineTransformationRegioPolygon.setEnabled( false );
               m_copyAllRegioPolygons.setEnabled( false );
               m_deleteAllRegioPolygonsOfCurrentlyDisplayedImageOnly.setEnabled( false );
               m_scaleRegioPolygon.setEnabled( false );
               m_renameRegion.setEnabled( false );
               m_showHistogramOfSelectedRegioPolygon.setEnabled( false );
               m_tTestRegioPolygon.setEnabled( false );

               m_mirrorROI.setEnabled( false );
               m_addMirroredROI.setEnabled( false );
               m_showROIMirrorAxis.setEnabled( false );
               m_editMirrorAxis.setEnabled( false );

            }

            if ( !m_addOneCircularROIMode.isSelected( ) ) 
            {
                if ( currentPolygons.size( ) == 1 )
                {
                   m_copyRegioPolygon.setEnabled( true );
                   m_pixelDumpSelectedRegioPolygon.setEnabled( true );
                   m_affineTransformationRegioPolygon.setEnabled( true );
                   m_copyAllRegioPolygons.setEnabled( false );
                   m_deleteAllRegioPolygonsOfCurrentlyDisplayedImageOnly.setEnabled( true );
                   m_deleteAllRegioPolygonsOfAllImages.setEnabled( true );
                   m_scaleRegioPolygon.setEnabled( true );
                   m_renameRegion.setEnabled( true );
                   m_showHistogramOfSelectedRegioPolygon.setEnabled( true );
                   m_tTestRegioPolygon.setEnabled( false );

                   m_mirrorROI.setEnabled( true );
                   m_addMirroredROI.setEnabled( true );
                   m_showROIMirrorAxis.setEnabled( true );
                   m_editMirrorAxis.setEnabled( true );

                   setMode( SELECT_MODE );
                }
            }
            
            if ( !m_addOneCircularROIMode.isSelected( ) )
            {
                if ( currentPolygons.size( ) > 1 )
                {
                    m_copyRegioPolygon.setEnabled( true );
                    m_pixelDumpSelectedRegioPolygon.setEnabled( true );
                    m_affineTransformationRegioPolygon.setEnabled( true );
                    m_scaleRegioPolygon.setEnabled( true );
                    m_copyAllRegioPolygons.setEnabled( true );
                    m_deleteAllRegioPolygonsOfCurrentlyDisplayedImageOnly.setEnabled( true );
                    m_deleteAllRegioPolygonsOfAllImages.setEnabled( true );
                    m_tTestRegioPolygon.setEnabled( true );
                    m_showHistogramOfSelectedRegioPolygon.setEnabled( true );
                    m_renameRegion.setEnabled( true );

                    m_mirrorROI.setEnabled( true );
                    m_addMirroredROI.setEnabled( true );
                    m_showROIMirrorAxis.setEnabled( true );
                    m_editMirrorAxis.setEnabled( true );
                    setMode( SELECT_MODE );
                }
            }
            
            if ( m_enablePaging == true )
            {
                if ( m_reduceImageMatrixAndWriteAsDICOM != null )
                {
                    m_reduceImageMatrixAndWriteAsDICOM.setEnabled( true );
                }
            }

            if ( ( m_enablePaging == true ) && ( m_performSimple3DCut != null ) )
            {
               m_performSimple3DCut.setEnabled( true );
            }

            if ( ( m_enablePaging == true ) && ( m_selectedPolygon != null ) )
            {

                if ( m_computePropertiesofSelectedROIForAllSlicesOfCurrentlySelectedRegionOnly != null )
                {
                    m_computePropertiesofSelectedROIForAllSlicesOfCurrentlySelectedRegionOnly.setEnabled( true );
                }

                if ( m_computePropertiesofSelectedROIForAllSlicesForAllRegions != null )
                {
                    m_computePropertiesofSelectedROIForAllSlicesForAllRegions.setEnabled( true );
                }
            }

            if ( !m_addOneCircularROIMode.isSelected( ) )
            {
                if ( currentPolygons.size( ) == 1 )
                {
                    setMode( DRAG_MODE  );
                }
                else
                {
                    setMode( SELECT_MODE );
                }
            }
            
            if ( ( m_imageSegmentsMenu != null ) && ( m_selectedPolygon != null ) )
            {
                m_eliminateOverlayPixelsOutsideFocusRegion.setEnabled( true );
                m_eliminateOverlayPixelsInsideFocusRegion.setEnabled( true );
            }
        }
     }
     
    /**
     *
     * event multicaster management
     *
     **/
    public void addPolygonListener( PolygonListener l )
    {
        multicaster.add( l );
    }
    
    public void removePolygonListener( PolygonListener l ) 
    {
        multicaster.remove( l );
    }
    
    /**
     * Method to calculate an relative image. Normal value is the average
     * pixel value calculated over the selected region
     */
    public void calculateRelativeImageRepresentation( )
    {
        calculateSelectedROIParameters( );
        calculateRelativeImage( );
        setImage( m_rawData, m_imageWidth, m_imageHeight, 0 ); 
    }
    /**
     * Calculates the ROI parameters of the selected polygon and
     * returns the variance it
     */
    public float getVarianceOfSelectedPolygon( )
    {
        calculateSelectedROIParameters( );
        return m_stdev * m_stdev;
    }
    
    
    /**
     *  Calculates the average pixel value and standard deviation of the
     *  currently selected polygon (which encloses a ROI).
     */
    public void calculateSelectedROIParameters( )
    {
        
        // get a reference on the selected polygon        
	
        float zoom = 1f / getZoom( );
        
        RegioPolygon rpol = getSelectedPolygon( );
        
        Polygon roi = null;
        
        if ( rpol != null )
        {
            roi = (Polygon)rpol.scaled( zoom );
        }
        
        if ( roi != null )
        {
            
            Rectangle bb = roi.getBounds( );
            if ( bb.x < 0 )
            {    
                bb.x = 0;
            }
            if ( bb.x + bb.width >= m_imageWidth ) 
            {
                bb.width = m_imageWidth - bb.x - 1;
            }
            if ( bb.y < 0 )
            {
                bb.y = 0;
            }
        
            if ( bb.y + bb.height >= m_imageHeight ) 
            {
                bb.height = m_imageHeight - bb.y - 1;
            }
        
            int imageSize = m_imageWidth * m_imageHeight;

            m_roiSize = 0;
            m_av = 0f;
            m_stdev = 0f;
            
            // Calculate the average of the ROI
            for ( int y = bb.y; y <= bb.y + bb.height; y++ ) 
            {
                int yoffset = y * m_imageWidth;
                for ( int x = bb.x; x <= bb.x + bb.width; x++ ) 
                {
                    if ( roi.contains( x, y ) ) 
                    {
                        m_roiSize++;
                        int xyoffset = yoffset;
                        m_av += m_rawData[ x + m_imageWidth * y ] * m_nordicScaleFactor;
                    }
                }
            }

            m_av = m_av / m_roiSize;

            // Calculate the standard deviation of the current ROI
            float diff = 0.0f;
            // reset the roi size
            m_roiSize = 0;
        
            for ( int y = bb.y; y <= bb.y + bb.height; y++ )
            {
                int yoffset = y * m_imageWidth;
                for ( int x = bb.x; x <= bb.x + bb.width; x++ )
                {
                    if ( roi.contains( x, y ) )
                    {
                        m_roiSize++;
                        diff = m_av - ( m_rawData[ x + m_imageWidth * y ] * m_nordicScaleFactor );
                        m_stdev += diff * diff;
                    }
                }
            }

            m_stdev = (float)Math.sqrt( (double) m_stdev / ( (float)m_roiSize - 1.0 ) ) ;
        }
    }
    
    /**
     *  Calculates the average pixel value and standard deviation of the
     *  polygon number (polygon_number) which encloses a ROI).
     */
    public void calculateROIParameters( int polygon_number )
    { 
        // get a reference on the selected polygon        
	
        float zoom = 1f / getZoom( );
        
        RegioPolygon rpol = getPolygon( polygon_number );
        Polygon roi = (Polygon)rpol.scaled( zoom );
        
        if ( roi != null )
        {
            Rectangle bb = roi.getBounds( );
            if ( bb.x < 0 )
            {    
                bb.x = 0;
            }
            if ( bb.x + bb.width >= m_imageWidth ) 
            {
                bb.width = m_imageWidth - bb.x - 1;
            }
            if ( bb.y < 0 )
            {
                bb.y = 0;
            }
        
            if ( bb.y + bb.height >= m_imageHeight ) 
            {
                bb.height = m_imageHeight - bb.y - 1;
            }
        
            int imageSize = m_imageWidth * m_imageHeight;

            m_roiSize = 0;
            m_av = 0f;
            this.m_stdev = 0f;
            
            // Calculate the average of the ROI
            for ( int y = bb.y; y <= bb.y + bb.height; y++ ) 
            {
                int yoffset = y * m_imageWidth;
                for ( int x = bb.x; x <= bb.x + bb.width; x++ ) 
                {
                    if ( roi.contains( x, y ) ) 
                    {
                        m_roiSize++;
                        int xyoffset = yoffset;
                        m_av += m_rawData[ x + m_imageWidth * y ] * m_nordicScaleFactor;
                    }
                }
            }

            m_av = m_av / m_roiSize;

            // Calculate the standard deviation of the current ROI
            float diff = 0.0f;
            // reset the roi size
            m_roiSize = 0;
        
            for ( int y = bb.y; y <= bb.y + bb.height; y++ )
            {
                int yoffset = y * m_imageWidth;
                for ( int x = bb.x; x <= bb.x + bb.width; x++ )
                {
                    if ( roi.contains( x, y ) )
                    {
                        m_roiSize++;
                        diff = m_av - (m_rawData[ x + m_imageWidth * y ] * m_nordicScaleFactor );
                        m_stdev += diff * diff;
                    }
                }
            }

            m_stdev = (float)Math.sqrt( (double) m_stdev / ( (float)m_roiSize - 1.0 ) ) ;
        }
    }
    
    
    /**
     *  Calculates the average pixel value and standard deviation of the
     *  polygon number (polygon_number) which encloses a ROI).
     */
    public void calculateROIParameters( int imageNumber, RegioPolygon rpol )
    { 
        // get a reference on the selected polygon        
	
        float minimum = Float.MAX_VALUE;
        float maximum = -Float.MIN_VALUE;
        
        
        int dataOffset = m_imageWidth * m_imageHeight * imageNumber;
        
        float zoom = 1f / getZoom( );

        Polygon roi = (Polygon)rpol.scaled( zoom );
        
        if ( roi != null )
        {
            Rectangle bb = roi.getBounds( );
            if ( bb.x < 0 )
            {    
                bb.x = 0;
            }
            if ( bb.x + bb.width >= m_imageWidth ) 
            {
                bb.width = m_imageWidth - bb.x - 1;
            }
            if ( bb.y < 0 )
            {
                bb.y = 0;
            }
        
            if ( bb.y + bb.height >= m_imageHeight ) 
            {
                bb.height = m_imageHeight - bb.y - 1;
            }
        
            int imageSize = m_imageWidth * m_imageHeight;

            m_roiSize = 0;
            
            m_av_raw = 0f;
            m_stdev_raw = 0f;
            
            m_av = 0f;
            m_stdev = 0f;
            
            // Calculate the average of the ROI
            for ( int y = bb.y; y <= bb.y + bb.height; y++ ) 
            {
                int yoffset = y * m_imageWidth;
                for ( int x = bb.x; x <= bb.x + bb.width; x++ ) 
                {
                    if ( roi.contains( x, y ) ) 
                    {
                        m_roiSize++;
                        int xyoffset = yoffset;
                        
                        m_av_raw += m_imagePixelData[ dataOffset + x + m_imageWidth * y ];
                        
                        float value = m_imagePixelData[ dataOffset + x + m_imageWidth * y ] * m_nordicScaleFactor;                        
                        m_av += value;
                        
                        if ( m_imagePixelData[ dataOffset + x + m_imageWidth * y ] < minimum )
                        {
                            minimum = m_imagePixelData[ dataOffset + x + m_imageWidth * y ];
                        }
                        
                        if ( m_imagePixelData[ dataOffset + x + m_imageWidth * y ] > maximum )
                        {
                            maximum = m_imagePixelData[ dataOffset + x + m_imageWidth * y ];
                        }
                        
                    }
                }
            }

            m_av = m_av / m_roiSize;
            m_av_raw = m_av_raw / m_roiSize;
            
            // Calculate the standard deviation of the current ROI
            float diff = 0.0f;
            float diff_raw = 0.0f;
            
            // reset the roi size
            m_roiSize = 0;
        
            for ( int y = bb.y; y <= bb.y + bb.height; y++ )
            {
                int yoffset = y * m_imageWidth;
                for ( int x = bb.x; x <= bb.x + bb.width; x++ )
                {
                    if ( roi.contains( x, y ) )
                    {
                        m_roiSize++;
                        
                        diff = m_av - ( m_imagePixelData[ dataOffset + x + m_imageWidth * y ] * m_nordicScaleFactor );
                        diff_raw = m_av_raw - m_imagePixelData[ dataOffset + x + m_imageWidth * y ];
                        
                        m_stdev += diff * diff;
                        m_stdev_raw += diff_raw * diff_raw;
                    }
                }
            }

            m_stdev = (float)Math.sqrt( (double) m_stdev / ( (float)m_roiSize - 1.0 ) ) ;
            m_stdev_raw = (float)Math.sqrt( (double) m_stdev_raw / ( (float)m_roiSize - 1.0 ) ) ;
            
            m_range_raw  = ( maximum - minimum );
            m_center_raw = ( maximum + minimum ) / 2f;
            
        }
    }
    
    
    /**
     * Calculate relative images, i.e. the rawData is devided by the average 
     * value over the 
     */
    private void calculateRelativeImage( )
    {
        float dummy;
        for ( int i = 0; i < m_rawData.length; i++ )
        {
            dummy = 100.0f * (float)m_rawData[i] / m_av;
            m_rawData[i] = (int)dummy;
        }
    }
    
    /**
     * Perform sub segmentation to currently displayed image only
     * 
     */
    private boolean[][] performSubSegmentationToCurrentlyDisplayedImageOnly( Point p )
    {
        
        int selectedTissueType = m_selectedTissueTypes[0];
        
        // declare a new data structure to store result int
        int newSize = 0;
        
        if ( m_userSelectedSegmentArrayList == null )
        {
            m_userSelectedSegmentArrayList = new ArrayList<boolean[][]>( );
            for ( int i = 0 ; i < m_numberOfImages; i++ )
            {
                boolean[][] overLayMap = new boolean[m_imageWidth][m_imageHeight];
                m_userSelectedSegmentArrayList.add( overLayMap );
            }
            
            // if this routine is called for the first time than a 
            // new tissue type entry must be created
            
            m_userAlreadySelectedImageSegmentsOnce = true;
            
            for ( int i = 0; i < m_numberOfImages; i++ )
            {
                m_imageSegmentsOverlayList.get( i ).addLast( m_userSelectedSegmentArrayList.get( i ) );
            }
        }
        
        m_connectedPixels = m_userSelectedSegmentArrayList.get(m_currentlyDisplayedImageNumber );
        
        // Start the boolean logic
        LinkedList<boolean[][]> arrayList = m_imageSegmentsOverlayList.get(m_currentlyDisplayedImageNumber );
                
        boolean[][] overlay = arrayList.get( selectedTissueType );
        
        if ( m_connectedPixels[(int)(p.x/m_zoom)][(int)(p.y/m_zoom)] == false )
        {
            Point pnt = new Point( (int)( p.x/m_zoom ), (int)( p.y/m_zoom ) ); 
            m_connectedPixels = determineConnectedPixels( m_connectedPixels, overlay, pnt, m_imageWidth, m_imageHeight );
        }
        else
        {
            boolean[][] connectedPixels = new boolean[m_imageWidth][m_imageHeight];
            
            Point pnt = new Point( (int)( p.x/m_zoom ), (int)( p.y/m_zoom ) );
            connectedPixels = determineConnectedPixels( connectedPixels, overlay, pnt, m_imageWidth, m_imageHeight );
            
            for( int i = 0; i < m_imageWidth; i++ )
            {
                for( int j = 0; j < m_imageHeight; j++ )
                { 
                    if ( connectedPixels[i][j] == true )
                    {
                        m_connectedPixels[i][j] = !( m_connectedPixels[i][j] && connectedPixels[i][j] );
                    }
                }  
            }
            
        }
        
        m_userSelectedSegmentArrayList.set(m_currentlyDisplayedImageNumber, m_connectedPixels );
            
        //are these two expressions below correct?
            
        if ( m_userAlreadySelectedImageSegmentsOnce == true )
        {
            arrayList.removeLast( );     
        }
            
        arrayList.addLast( m_connectedPixels );
            
        if ( m_userAlreadySelectedImageSegmentsOnce == true )
        {
            notifyActionListeners( ADDED_NEW_COMPUTED_OVERLAY_EVENT, "" + arrayList.size( ) );
        }
        else
        {
            m_userAlreadySelectedImageSegmentsOnce = false;
        }
        
        return m_connectedPixels;
    }
    
    /**
     * Returns 
     * @param matrix
     * @param sizeX
     * @param sizeY
     * @return
     */
    private boolean[][] logicOR( boolean[][] matA, boolean[][] matB, int sizeX, int sizeY )
    {
        boolean[][] matOut = new boolean[sizeX][sizeY];
        for ( int i = 0; i < sizeX; i++ )
        {
            for ( int j = 0 ; j < sizeY; j++ )
            {
                matOut[i][j] = matA[i][j] || matB[i][j]; 
            }
        }
        return matOut;
    }

    /**
     *
     * Determine connected image segements in parallel images and perform segmentation!
     *
     * @param p
     */
    private void performSubSegmentationAllToParrallelImagesInSeperateWorkerThread( final Point p )
    {
        m_dualProgressBar = new DualProgressBar( m_parent, 100, 100, new Dimension( 900, 150 ), false );
        m_dualProgressBar.setTitle( "Searching for connected image segments in other images "  );

        m_dualProgressBar.setUpperProgress( 0 );
        m_dualProgressBar.setUpperLabel( "        Total progress " );
        m_dualProgressBar.setUpperProgressVisible( false );
        m_dualProgressBar.setLowerLabel( "<HTML><BR>  Processing image number : " +  0 + "</HTML>" );
       
       final SwingWorker worker = new SwingWorker( )
       {
            public Object construct( )
            {
                try
                {
                    // segment the image
                    performSubSegmentationAllToParrallelImages( p );

                }
                catch ( OutOfMemoryError e )
                {
                    Dialogs d = new Dialogs( );
                    Dialogs.messageBox( m_parent, "<HTML> An out of memory exception occurred <BR> Contact your program supplier! </HTML>" );
                }
                return null;
            }

            // runs in the event dispatching thread
            @Override
            public void finished( )
            {
                 // After calculation set enabled again
                 m_dualProgressBar.setVisible( false );

                 Dialogs d = new Dialogs( );
                 Dialogs.messageBox( m_parent, "<HTML> Connected ready! </HTML>" );
            }

        };

        worker.start( );
    }
    
    /**
     * 
     * Performs sub segmentation to all images !
     * 
     * @param p
     */
    private void performSubSegmentationAllToParrallelImages( Point p )
    {
        int selectedTissueType = m_selectedTissueTypes[0];
        
        boolean[][] connectedPixels = performSubSegmentationToCurrentlyDisplayedImageOnly( p );
        int cnt = 0;
        
        boolean[][][] _3DOverlay = new boolean[m_imageWidth][m_imageHeight][m_numberOfImages];
        boolean[][][] _3DOutputOverlay = new boolean[m_imageWidth][m_imageHeight][m_numberOfImages];
        
        // create a 3D structure to determine quickly pixel connectivity
        for ( int i = 0; i < m_imageWidth; i++ )
        { 
            for ( int j = 0; j < m_imageHeight; j++ )
            {
                for ( int k = 0; k < m_numberOfImages; k++ )
                {  
                    LinkedList<boolean[][]> arrayList = m_imageSegmentsOverlayList.get( k );
                    boolean[][] overlay = arrayList.get( selectedTissueType ); 
                    
                    _3DOverlay[i][j][k] = overlay[i][j];
                }
            }
        }
        
         // create a 3D structure to determine quickly pixel connectivity
        for ( int i = 0; i < m_imageWidth; i++ )
        { 
            for ( int j = 0; j < m_imageHeight; j++ )
            {
                for ( int k = 0; k < m_numberOfImages; k++ )
                {  
                    if ( k != m_currentlyDisplayedImageNumber )
                    {
                        _3DOutputOverlay[i][j][k] = false;
                    }
                    else
                    {
                        _3DOutputOverlay[i][j][k] = connectedPixels[i][j];
                    }
                }
            }
        }

        int progressCounter = 0;

        // now we have to compute for each perpendicular image segment
        // the connected pixelmap using an arbitrary seed point (having value
        // true) in the map
        
        for ( int kk = m_currentlyDisplayedImageNumber - 1; kk > -1; kk-- )
        {
            System.out.println( "Counter kk = " + kk );

            m_dualProgressBar.setLowerProgress( (int)( 100.0 * (double)progressCounter / m_numberOfImages ) );
            m_dualProgressBar.setLowerLabel( "<HTML><BR>  Processing image number : " +  kk + "</HTML>" );

            boolean[][] parallelOverlay = new boolean[m_imageWidth][m_imageHeight];
            boolean[][] parallelConnectedPixelsStep = new boolean[m_imageWidth][m_imageHeight];
            boolean[][] parallelConnectedPixelsTotal = new boolean[m_imageWidth][m_imageHeight];
            
            for ( int ii = 0; ii < m_imageWidth; ii++ )
            {
                for ( int jj = 0; jj < m_imageHeight ; jj++ )
                {
                    parallelOverlay[ii][jj] = _3DOverlay[ii][jj][kk];
                }
            }
            
            // Now determine connectivities for all perpendicular slices
            boolean alreadyFound = false;
            int cntr = 0;
            int cntr2 = 0;
            for ( int ii = 0 ; ii < m_imageWidth ; ii++ )
            {
                if ( cntr > m_imageWidth / 2 )
                {
                    alreadyFound = false;
                    cntr=0;
                }
                cntr++; 
                for ( int jj = 0; jj < m_imageHeight; jj++ )
                {
                    // start search close to user picked point
                    if ( cntr2 > m_imageHeight / 2 )
                    {
                        alreadyFound = false;
                        cntr2 = 0;
                    }
                    cntr2++; 
                    if ( ( _3DOutputOverlay[ii][jj][kk+1] == true ) && ( parallelOverlay[ii][jj] == true ) && ( alreadyFound == false ) )
                    {
                    
                        alreadyFound = true;    
                        
                        Point pnt = new Point( ii, jj );
                    
                        // do the computation
                        parallelConnectedPixelsStep = determineConnectedPixels( parallelConnectedPixelsStep, parallelOverlay, pnt, m_imageWidth, m_imageHeight );
                        
                        parallelConnectedPixelsTotal = logicOR( parallelConnectedPixelsTotal, parallelConnectedPixelsStep, m_imageWidth, m_imageHeight );
                    
                    }
                }
            }
            
            for ( int ii = 0 ; ii < m_imageWidth ; ii++ )
            {
            
                for ( int jj = 0; jj < m_imageHeight; jj++ )
                {
                    _3DOutputOverlay[ii][jj][kk] = parallelConnectedPixelsTotal[ii][jj];
                }
            }
            progressCounter++;
        }
        
        for ( int kk = m_currentlyDisplayedImageNumber + 1; kk < m_numberOfImages; kk++ )
        {
            
            System.out.println( "Counter kk = " + kk );

            m_dualProgressBar.setLowerProgress( (int)( 100.0 * (double)progressCounter / m_numberOfImages ) );
            m_dualProgressBar.setLowerLabel( "<HTML><BR>  Processing image number : " +  kk + "</HTML>" );

            boolean[][] parallelOverlay = new boolean[m_imageWidth][m_imageHeight];
            boolean[][] parallelConnectedPixelsStep = new boolean[m_imageWidth][m_imageHeight];
            boolean[][] parallelConnectedPixelsTotal = new boolean[m_imageWidth][m_imageHeight];
            
            for ( int ii = 0; ii < m_imageWidth; ii++ )
            {
                for ( int jj = 0; jj < m_imageHeight ; jj++ )
                {
                    parallelOverlay[ii][jj] = _3DOverlay[ii][jj][kk];
                }
            }
            
            // Now determine connectivities for all perpendicular slices
            boolean alreadyFound = false;
            int cntr = 0;
            int cntr2 = 0;
            for ( int ii = 0 ; ii < m_imageWidth ; ii++ )
            {
                
                if ( cntr > m_imageWidth / 2 )
                {
                    alreadyFound = false;
                    cntr = 0;
                }
                cntr++; 
                
                for ( int jj = 0; jj < m_imageHeight; jj++ )
                {
                    // start search close to user picked point
                    if ( cntr2 > m_imageHeight / 2 )
                    {
                        alreadyFound = false;
                        cntr2 = 0;
                    }
                    cntr2++; 
                    if ( ( _3DOutputOverlay[ii][jj][kk-1] == true ) && ( parallelOverlay[ii][jj] == true ) && ( alreadyFound == false ) )
                    {
                   
                        alreadyFound = true;
                        
                        Point pnt = new Point( ii, jj );
                    
                        // do the computation
                        parallelConnectedPixelsStep = determineConnectedPixels( parallelConnectedPixelsStep, parallelOverlay, pnt, m_imageWidth, m_imageHeight );
                        
                        parallelConnectedPixelsTotal = logicOR( parallelConnectedPixelsTotal, parallelConnectedPixelsStep, m_imageWidth, m_imageHeight );
                    
                    }
                }
            }
            
            for ( int ii = 0 ; ii < m_imageWidth ; ii++ )
            {
            
                for ( int jj = 0; jj < m_imageHeight; jj++ )
                {
                    _3DOutputOverlay[ii][jj][kk] = parallelConnectedPixelsTotal[ii][jj];
                }
            }
            progressCounter++;
        }
        
       
        // copy back
        for ( int k = 0; k < m_numberOfImages; k++ )
        {
            boolean[][] newConnectedPixels = new boolean[m_imageWidth][m_imageHeight];
            
            for ( int i = 0; i < m_imageWidth; i++)
            {
                for ( int j = 0; j < m_imageHeight; j++ )
                {
                    newConnectedPixels[i][j] = _3DOutputOverlay[i][j][k];
                }
                
                m_userSelectedSegmentArrayList.set( k, newConnectedPixels );
            }
        }
        
        // update the m_imageSegment overlay list
        for ( int i = 0; i < m_numberOfImages; i++ )
        {
            m_imageSegmentsOverlayList.get( i ).removeLast( );
            m_imageSegmentsOverlayList.get( i ).addLast( m_userSelectedSegmentArrayList.get( i ) );
        }
                
    }


    /**
     * Determines for all points of the image taken as seed the amount of pixels
     * which corresponds to the range defined by this point
     */
    private void determineStatisticsOfAllPointsAsSeed( )
    {
         int cnt=0;
         int[] xAxis = new int[ getImageWidth( ) * getImageHeight( ) ];
         int[] numberOfPixelsCount = new int[ getImageWidth( ) * getImageHeight( ) ];

         for ( int i = 1; i < getImageHeight( ) - 2; i++ )
         {
             for ( int j = 1; j < getImageWidth( ) - 2; j++ )
             {
                 Point p = new Point( i, j );
                 xAxis[cnt] = cnt;
                 if ( this.m_imagePixelData[ getImageWidth( ) * i + j ] > 0.05 * this.getMaxRawDataValue( ) )
                 {
                    numberOfPixelsCount[cnt] = handleInteractiveSegmentationPrototypeRegionClicked( p );
                    cnt++;
                 }
             }
         }

         QuickSort qsort= new QuickSort( );
         numberOfPixelsCount = qsort.quickSort( numberOfPixelsCount );

         XYPlotFrame avPlot = new XYPlotFrame( "Counts", null );
         Curve avCurve = new Curve( xAxis, numberOfPixelsCount, Color.black );
         avPlot.addCurve( avCurve, true );

         calculateHistogram( numberOfPixelsCount );

    }


    /**
     * Calculates the histogram of the intensity distribution of the pixels
     * enclosed
     */
    private void calculateHistogram( int[] values )
    {
        int numberOfClasses = 0;
        
        SignalTools sTool  = new SignalTools( );

        int maxValue = sTool.getMaximum( values );
        int minValue = sTool.getMinimum( values );
        float classWidth = 0f;
        int[] occurences;


        int rangeWidth = ( maxValue - minValue );

        if ( rangeWidth != 0 )
        {
            numberOfClasses = ( values.length / 10 );

            if ( numberOfClasses < 10 )
            {
                numberOfClasses = 10;
            }

            if ( numberOfClasses > 4096 )
            {
                numberOfClasses = 4096;
            }

            classWidth = (float)rangeWidth / (float)numberOfClasses;

            occurences = new int[ numberOfClasses + 1];

            for ( int i = 0; i < values.length ; i++ )
            {
                if ( values[i] != 0 )
                {
                    int xVal = (int)( (values[i] - minValue)  /  classWidth );
                    occurences[ xVal ]++;
                }
            }

            float[] xVals = new float[ 2 * occurences.length ];
            float[] yVals = new float[ 2 * occurences.length ];

            for ( int i = 0; i < occurences.length; i++ )
            {
                xVals[2*i] = minValue + (float)i * classWidth;
                yVals[2*i] = occurences[i];
                xVals[2*i + 1] = minValue + (float)(i+1) * classWidth;
                yVals[2*i + 1] = occurences[i];
            }

            XYPlotFrame xyplot = new XYPlotFrame( "Histogram", null );

            xyplot.addCurve( new Curve( xVals, yVals ), true );

            xyplot.repaint( );

        }
    }

    /**
     * This method handles the reaction on a mouse-click of the user
     * @param p
     */
    private int handleInteractiveSegmentationPrototypeRegionClicked( Point p )
    {

        m_interactiveSegmentationOverlay = new boolean[ getImageWidth( ) ][ getImageHeight( ) ];

        int pixVal = 0;

        int minPixVal = Integer.MAX_VALUE;
        int maxPixVal = Integer.MIN_VALUE;

        int pixelOffset = m_currentlyDisplayedImageNumber * m_imageHeight * m_imageWidth;

        for ( int i = -1; i < 2; i++ )
        {
            for ( int j = -1; j < 2; j++ )
            {
                pixVal = m_imagePixelData[ pixelOffset + ( (int)(p.y/m_zoom) + j ) * m_imageHeight + ( (int)(p.x / m_zoom) + i ) ];
                if ( pixVal < minPixVal )
                {
                    minPixVal = pixVal;
                }
                if ( pixVal > maxPixVal )
                {
                    maxPixVal = pixVal;
                }
            }
        }

        int count = 0;
        for ( int i = 0; i < m_imageWidth; i++ )
        {
            for ( int j = 0; j < m_imageHeight; j++ )
            {
                if ( ( m_imagePixelData[ pixelOffset + j * m_imageHeight + i] < maxPixVal ) &&
                     ( m_imagePixelData[ pixelOffset + j * m_imageHeight + i] > minPixVal ) )
                {
                    m_interactiveSegmentationOverlay[i][j] = true;
                    count++;
                }
                else
                {
                    m_interactiveSegmentationOverlay[i][j] = false;
                }
            }
        }

        return count;

    }

    /**
     * Handles the selection of a subregion in the cuurently selected image only
     */
    private void handleSelectionSubregionInImageSegment( Point p )
    {
        
        if ( m_subSegmentationScope == ProbingRegionPropertiesPanel.SUBSEGMENT_IN_CURRENTLY_SELECTED_SLICE_ONLY )
        {
            performSubSegmentationToCurrentlyDisplayedImageOnly( p ); 
        }
        else
        {
            // performSubSegmentationAllImages( p );
            
            performSubSegmentationAllToParrallelImagesInSeperateWorkerThread( p );
        }
    }
    
    /**
     * 
     * @param inputMap boolean image map
     * @param p Starting point (seed)
     * @return a map containing all connected pixels to point p
     */
    private boolean[][] determineConnectedPixels( boolean[][] connectedPixels, boolean[][] inputMap, Point p, int dimension_1, int dimension_2 )       
    {        
                
        // set seed point
        if ( inputMap[p.x][p.y] == true )
        {   
            connectedPixels[p.x][p.y] = true;

            for( int k = 0; k < dimension_1 ; k++ )
            {
                
                for ( int l = 0; l < dimension_2; l++ )
                {


                    int kkm1 = ( dimension_1 + p.x + k - 1 ) % ( dimension_1 );
                    int llm1 = ( dimension_2 + p.y + l - 1 ) % ( dimension_2 );


                    int kk = ( p.x + k ) % ( dimension_1 );
                    int ll = ( p.y + l ) % ( dimension_2 );

                    int kkp1 = ( dimension_1 + p.x + k + 1 ) % ( dimension_1 );
                    int llp1 = ( dimension_2 + p.y + l + 1 ) % ( dimension_2 );

                    if ( inputMap[kk][ll] == true )
                    {
                      
                        if ( connectedPixels[kkm1][llm1] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kkm1][ll] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kkm1][llp1] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kk][llm1] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kk][llp1] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kkp1][llm1] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kkp1][ll] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kkp1][llp1] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                    
                    } 
                }
            }
            
            for( int k = dimension_1 - 1; k > -1 ; k-- )
            {
                for ( int l = dimension_2 - 1; l > -1; l-- )
                {
                    
//                    int kk = ( 1 + ( p.x + k ) ) % ( dimension_1 - 1 );
//                    int ll = ( 1 + ( p.y + l ) ) % ( dimension_2 - 1 );


                    int kkm1 = ( dimension_1 + p.x + k - 1 ) % ( dimension_1 );
                    int llm1 = ( dimension_2 + p.y + l - 1 ) % ( dimension_2 );


                    int kk = ( p.x + k ) % ( dimension_1 );
                    int ll = ( p.y + l ) % ( dimension_2 );

                    int kkp1 = ( dimension_1 + p.x + k + 1 ) % ( dimension_1 );
                    int llp1 = ( dimension_2 + p.y + l + 1 ) % ( dimension_2 );

                    if ( inputMap[kk][ll] == true )
                    {
                      
                        if ( connectedPixels[kkm1][llm1] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kkm1][ll] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kkm1][llp1] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kk][llm1] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kk][llp1] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kkp1][llm1] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kkp1][ll] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kkp1][llp1] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                    
                    } 
                }
            }
            
            for( int k = 0; k < dimension_1 - 1 ; k++ )
            {
                
                for ( int l = dimension_2 - 1; l > -1; l-- )
                {
                    
//                    int kk = ( 1 + ( p.x + k ) ) % ( dimension_1 - 1 );
//                    int ll = ( 1 + ( p.y + l ) ) % ( dimension_2 - 1 );



                    int kkm1 = ( dimension_1 + p.x + k - 1 ) % ( dimension_1 );
                    int llm1 = ( dimension_2 + p.y + l - 1 ) % ( dimension_2 );


                    int kk = ( p.x + k ) % ( dimension_1 );
                    int ll = ( p.y + l ) % ( dimension_2 );

                    int kkp1 = ( dimension_1 + p.x + k + 1 ) % ( dimension_1 );
                    int llp1 = ( dimension_2 + p.y + l + 1 ) % ( dimension_2 );

                    if ( inputMap[kk][ll] == true )
                    {
                      
                        if ( connectedPixels[kkm1][llm1] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kkm1][ll] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kkm1][llp1] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kk][llm1] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kk][llp1] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kkp1][llm1] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kkp1][ll] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kkp1][llp1] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                    } 
                }
            }
            
            for( int k = dimension_1 - 1; k >  -1 ; k-- )
            {
                
                for ( int l = dimension_2 - 1; l > -1; l-- )
                {
                    
//                    int kk = ( 1 + ( p.x + k ) ) % ( dimension_1 - 1 );
//                    int ll = ( 1 + ( p.y + l ) ) % ( dimension_2 - 1 );



                    int kkm1 = ( dimension_1 + p.x + k - 1 ) % ( dimension_1 );
                    int llm1 = ( dimension_2 + p.y + l - 1 ) % ( dimension_2 );


                    int kk = ( p.x + k ) % ( dimension_1 );
                    int ll = ( p.y + l ) % ( dimension_2 );

                    int kkp1 = ( dimension_1 + p.x + k + 1 ) % ( dimension_1 );
                    int llp1 = ( dimension_2 + p.y + l + 1 ) % ( dimension_2 );

                    if ( inputMap[kk][ll] == true )
                    {             
                        if ( connectedPixels[kkm1][llm1] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kkm1][ll] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kkm1][llp1] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kk][llm1] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kk][llp1] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kkp1][llm1] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kkp1][ll] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kkp1][llp1] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                    } 
                }
            }
            
            //////////////////////////////////////////////////////////////////
            
            for( int k = 0; k <  dimension_1 ; k++ )
            {
                
                for ( int l = 0; l < dimension_2 ; l++ )
                {
                    
//                    int kk = ( 1 + ( p.x + k ) ) % ( dimension_1 - 1 );
//                    int ll = ( 1 + ( p.y + l ) ) % ( dimension_2 - 1 );


                    int kkm1 = ( dimension_1 + p.x + k - 1 ) % ( dimension_1 );
                    int llm1 = ( dimension_2 + p.y + l - 1 ) % ( dimension_2 );


                    int kk = ( p.x + k ) % ( dimension_1 );
                    int ll = ( p.y + l ) % ( dimension_2 );

                    int kkp1 = ( dimension_1 + p.x + k + 1 ) % ( dimension_1 );
                    int llp1 = ( dimension_2 + p.y + l + 1 ) % ( dimension_2 );


                    if ( inputMap[kk][ll] == true )
                    {
                      
                        if ( connectedPixels[kkm1][llm1] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kkm1][ll] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kkm1][llp1] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kk][llm1] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kk][llp1] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kkp1][llm1] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kkp1][ll] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kkp1][llp1] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                    
                    } 
                }
            }
            
            for( int k = dimension_1 - 1; k >  -1 ; k-- )
            {
                
                for ( int l = dimension_2 - 1; l > -1; l-- )
                {
                    
//                    int kk = ( 1 + ( p.x + k ) ) % ( dimension_1 - 1 );
//                    int ll = ( 1 + ( p.y + l ) ) % ( dimension_2 - 1 );


                    int kkm1 = ( dimension_1 + p.x + k - 1 ) % ( dimension_1 );
                    int llm1 = ( dimension_2 + p.y + l - 1 ) % ( dimension_2 );


                    int kk = ( p.x + k ) % ( dimension_1 );
                    int ll = ( p.y + l ) % ( dimension_2 );

                    int kkp1 = ( dimension_1 + p.x + k + 1 ) % ( dimension_1 );
                    int llp1 = ( dimension_2 + p.y + l + 1 ) % ( dimension_2 );


                    if ( inputMap[kk][ll] == true )
                    {
                      
                        if ( connectedPixels[kkm1][llm1] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kkm1][ll] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kkm1][llp1] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kk][llm1] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kk][llp1] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kkp1][llm1] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kkp1][ll] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kkp1][llp1] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                    } 
                }
            }
            
            for( int k = 0; k < dimension_1 - 1 ; k++ )
            {
                
                for ( int l = dimension_2 - 1; l > -1; l-- )
                {
                    
//                    int kk = ( 1 + ( p.x + k ) ) % ( dimension_1 - 1 );
//                    int ll = ( 1 + ( p.y + l ) ) % ( dimension_2 - 1 );



                    int kkm1 = ( dimension_1 + p.x + k - 1 ) % ( dimension_1 );
                    int llm1 = ( dimension_2 + p.y + l - 1 ) % ( dimension_2 );


                    int kk = ( p.x + k ) % ( dimension_1 );
                    int ll = ( p.y + l ) % ( dimension_2 );

                    int kkp1 = ( dimension_1 + p.x + k + 1 ) % ( dimension_1 );
                    int llp1 = ( dimension_2 + p.y + l + 1 ) % ( dimension_2 );


                    if ( inputMap[kk][ll] == true )
                    {
                      
                        if ( connectedPixels[kkm1][llm1] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kkm1][ll] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kkm1][llp1] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kk][llm1] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kk][llp1] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kkp1][llm1] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kkp1][ll] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kkp1][llp1] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                    
                    } 
                }
            }
            
            for( int k = dimension_1 - 1; k >  -1 ; k-- )
            {
                
                for ( int l = dimension_2 - 1; l > -1; l-- )
                {
//
//                    int kk = ( 1 + ( p.x + k ) ) % ( dimension_1 - 1 );
//                    int ll = ( 1 + ( p.y + l ) ) % ( dimension_2 - 1 );

                    int kkm1 = ( dimension_1 + p.x + k - 1 ) % ( dimension_1 );
                    int llm1 = ( dimension_2 + p.y + l - 1 ) % ( dimension_2 );


                    int kk = ( p.x + k ) % ( dimension_1 );
                    int ll = ( p.y + l ) % ( dimension_2 );

                    int kkp1 = ( dimension_1 + p.x + k + 1 ) % ( dimension_1 );
                    int llp1 = ( dimension_2 + p.y + l + 1 ) % ( dimension_2 );

                    if ( inputMap[kk][ll] == true )
                    {
                      
                        if ( connectedPixels[kkm1][llm1] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kkm1][ll] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kkm1][llp1] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kk][llm1] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kk][llp1] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kkp1][llm1] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kkp1][ll] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                        if ( connectedPixels[kkp1][llp1] && inputMap[kk][ll] )
                        {
                            connectedPixels[kk][ll] = true;
                        }
                    } 
                }
            }    
        }
        return connectedPixels;
    }
            
    /** 
     * MouseListener implementation 
     */
    @Override
    public void mouseClicked( MouseEvent e ) 
    {  
        super.mouseClicked( e );
  
        // super.mouseDragged( e );
        int modifiers = e.getModifiers( );
        int x = e.getX( );
        int y = e.getY( );
       
        if ( m_current_mode == ADD_STACKED_ROI_MODE_EVENT )
        {
            // addCircularROIs( 10, e.getX( ), e.getY( ) );
            if ( m_scaledReferenceFrame.getSelectedContourToAddType( ) == ScaledROIDefintionFrame.STANDARD_TRHEE_CONCENTRIC_CIRCULAR_CONTOURS )
            {
                System.out.println( "Add Circular ROIs" );
                addThreeStandardCircularROIs( e.getX( ), e.getY( ) );
            }
            
            if ( m_scaledReferenceFrame.getSelectedContourToAddType( ) == ScaledROIDefintionFrame.N_CONCENTRIC_CIRCULAR_CONTOURS )
            {
                
                addArbitraryNumberofConcentricalCircularShapedROIs( m_scaledReferenceFrame.getBaseCircularRadius( ), m_scaledReferenceFrame.getNumberOfConcentricalContours( ), e.getX( ), e.getY( ) );
            }
            
            if ( m_scaledReferenceFrame.getSelectedContourToAddType( ) == ScaledROIDefintionFrame.N_ARBITRARY_SHAPED_CONTOURS )
            {
                addArbitraryNumberofConcentricalArbitraryShapedROIs( m_scaledReferenceFrame.getNumberOfConcentricalContours( ), e.getX( ), e.getY( ) );
            }
        }

        if ( m_measureDistanceMode == true )
        {
            m_measureDistancesFrame.setClickedPoint( m_currentlyDisplayedImageNumber,
                                                     (int)(x/m_zoom), (int)(y/m_zoom),
                                                     compute3DCoordinate( ),
                                                     "distance_" + m_currentlyDisplayedImageNumber + "_" + m_distancesVector.get(m_currentlyDisplayedImageNumber ).size( ),
                                                     Color.CYAN );
        }

        if ( m_interactiveSegmentationMode == true )
        {
            handleInteractiveSegmentationPrototypeRegionClicked( e.getPoint( ) );
        }
        else
        {
            updateRegionPopupMenu( );
            // if the user double clicks the control, and the
            // m_imageSegments menu is active than the user wants
            // to select a subregion of a image segment!
            // and than possibly convert it into a region
            if ( ( modifiers & MouseEvent.BUTTON1_MASK ) > 0 )
            {
                if ( e.getClickCount( ) == 2 )
                {
                    if ( m_imageSegmentsMenu != null )
                    {
                        if ( ( m_selectedTissueTypes.length >= 1 ) && ( m_selectedTissueTypes.length < 3 ) )
                        {
                            handleSelectionSubregionInImageSegment( e.getPoint( ) );
                        }
                    }
                }
            }

            // segment magic!
            if ( ( modifiers & MouseEvent.BUTTON2_MASK ) > 0 )
            {
                // nothing is done!
                if ( e.getClickCount( ) == 2 )
                {
                    System.out.println( "middle button double clicked!" );
                }
            }
        }

        if ( e.getClickCount( ) == 2 )
        {
            // System.out.println( "Mouse double clicked!" );
            m_doubleClickedPoint3D = compute3DCoordinate( );
            m_doubleClickedPoint = e.getPoint( );
            drawImage( false );

            notifyActionListeners( MOUSE_DOUBLE_CLICKED_EVENT, MOUSE_DOUBLE_CLICKED_COMMAND );
        }
        
    }
    
    /**
     * Adds circular ROIs to currently displayed image
     */
    public void addArbitraryNumberofConcentricalArbitraryShapedROIs( int numberOfContours, int x_center, int y_center )
    {
        
        setZoom(1f);
         
        double r_0_f = 1.0f/Math.sqrt(numberOfContours);
        double r_0 = 1.0f;
        
        double r_n = 0.0f;
        
        double currentscale = 1.0f;
        
        float meanRadius = m_selectedPolygon.getMeanRadius( );
        
        RegioPolygon basePolygon = new RegioPolygon( m_selectedPolygon );
        
        NumberFormat format = new DecimalFormat( "###.###" );
        
        
        int cntr = 2;
        
        for ( int i = 0; i < numberOfContours; i++ )
        {
            
            currentscale = r_0_f * Math.sqrt( i + 1 );
            
            RegioPolygon polygon = new RegioPolygon( basePolygon );
            RegioPolygon polygon2 = polygon.scaledToPointOfGravity( currentscale );
            polygon2.setLabel( "scaled_" + i /* + "_" + format.format( currentscale ) */ );
            polygon2.setIndex( cntr );
            this.addPolygon( polygon2 );
            cntr++;
            
        }

        for ( int i = 0; i < numberOfContours; i++ )
        {
            
            currentscale = r_0 * Math.sqrt( i + 1 );
            
            RegioPolygon polygon = new RegioPolygon( basePolygon );
            RegioPolygon polygon2 = polygon.scaledToPointOfGravity( currentscale );
            polygon2.setLabel( "scaled_" + ( i  +  numberOfContours ) /* + "_" + format.format( currentscale ) */ );
            polygon2.setIndex( cntr );
            this.addPolygon( polygon2 );
            cntr++;
            
        }
        
        
        LinkedList<MomentsAboutTheMean> avMoments = new LinkedList<MomentsAboutTheMean>( );
        
        // now we determine the average pixel intensity statistics
        for ( int i = 1; i < ( cntr - 1 ); i++ )
        {
            RegioPolygon poly1 = getPolygon( i );
            RegioPolygon poly2 = getPolygon( i + 1 );
            if ( ( poly1 != null ) && ( poly2 != null ) )
            {
                avMoments.add( determineStatiscticsOfAreaBetweenTwoContours( poly1, poly2 ) );
            }
        }
        
        String fileNameExt = "";
        
        // Write results to hard disc
        String outputFileReference = getFilePathAndName( fileNameExt + ".txt", "roi_properties", "Save" );
        
        // String outputFileReference = System.getProperty( "user.home" ) + System.getProperty( "file.separator" ) + ".txt";

        TextFileWriter writer;
            
        try 
        {
            
            writer = new TextFileWriter( new FileWriter( outputFileReference ) );
            writer.writeStringLn( "**************************************************" );
            writer.writeStringLn( "*  Statistical properties of a ROI or VOI.       *" );
            writer.writeStringLn( "*  Do not edit!!.                                *" );
            writer.writeStringLn( "**************************************************" );

            Date today = new Date( );
            SimpleDateFormat formatter = new SimpleDateFormat( "dd-MMM-yyyy" );
            String dateStr = formatter.format( today );
            writer.writeStringLn( "* Date: " + dateStr + "                              *" );
            writer.writeStringLn( "**************************************************" );
            writer.writeStringLn( "" );
            
            for ( int i = 1; i < avMoments.size( ) ; i++ )
            {
                if ( i == 1 )
                {
                    avMoments.get( i ).write_HistogramStatistics_To_ASCII_File( writer, "ROI:(x;y)>Contour(" + i + ")_AND_(x;y)<Contour(" + (i+1) + ")", true );
                }
                else
                {
                    avMoments.get( i ).write_HistogramStatistics_To_ASCII_File( writer, "ROI:(x;y)>Contour(" + i + ")_AND_(x;y)<Contour(" + (i+1) + ")", false );
                }
            }
            
            writer.close( );
            
        }
        catch ( IOException ex ) 
        {
            Logger.getLogger( DicomSliceCanvasPolyDraw.class.getName( ) ).log( Level.SEVERE, null, ex );
        }
        
        
        for ( int i = 0 ; i < m_imageWidth ; i++ )
        {
            for ( int j = 0 ; j < m_imageHeight ; j++ )
            {
                if ( m_fieldOfInterest.contains( (double)i, (double)j ) == true )
                {
                    
                    m_thresholdedOverlay[j][i] = true;
                }
                else
                {
                    m_thresholdedOverlay[j][i] = false;
                }
            }
        }
               
        displayDicomImage( m_currentlyDisplayedImageNumber );
        drawImage( false );
              
    }
    
    
    public MomentsAboutTheMean determineStatiscticsOfAreaBetweenTwoContours( RegioPolygon pol_1, RegioPolygon pol_2 )
    {
        int[] image = new int[m_imageWidth * m_imageHeight];
        image = getImagePixelData( );
        
        m_thresholdedOverlay = new boolean[m_imageWidth][m_imageHeight];
        MomentsAboutTheMean fourMoments_1_to_2;
        
        if ( m_fieldOfInterest != null )
        {
            for ( int i = 0 ; i < m_imageWidth ; i++ )
            {
                for ( int j = 0 ; j < m_imageHeight ; j++ )
                {
                    if ( m_fieldOfInterest.contains( (double)i, (double)j ) == true )
                    {
                        if ( ( pol_2.contains( (double)i, (double)j ) == true ) && 
                             (!pol_1.contains( (double)i, (double)j ) ) )
                        {
                            m_thresholdedOverlay[j][i] = true;
                        }
                        else
                        {
                            m_thresholdedOverlay[j][i] = false;
                        }
                    }
                }
            }
        
            MomentsAboutTheMeanOfVOICalculator momCalc_1_2 = new MomentsAboutTheMeanOfVOICalculator( m_thresholdedOverlay, image, m_imageWidth, m_imageHeight, m_zoom );
            fourMoments_1_to_2 = momCalc_1_2.computeStatistics( );
        }
        else
        {
            for ( int i = 0 ; i < m_imageWidth ; i++ )
            {
                for ( int j = 0 ; j < m_imageHeight ; j++ )
                {
                    
                    if ( ( pol_2.contains( (double)i, (double)j ) == true ) && 
                         (!pol_1.contains( (double)i, (double)j ) ) )
                    {
                        m_thresholdedOverlay[j][i] = true;
                    }
                    else
                    {
                        m_thresholdedOverlay[j][i] = false;
                    }
                }
            }
        
            MomentsAboutTheMeanOfVOICalculator momCalc_1_2 = new MomentsAboutTheMeanOfVOICalculator( m_thresholdedOverlay, image, m_imageWidth, m_imageHeight, m_zoom );
            fourMoments_1_to_2 = momCalc_1_2.computeStatistics( );
        }
        
        return fourMoments_1_to_2;
    }
    
    
    /**
     * Adds circular ROIs to currently displayed image
     */
    public RegioPolygon addCircularShapedROI( float baseRadius, int x_center, int y_center )
    {
        
        int number_of_points = 200;
        
        float pixel_width   = new Float( m_fileInfo.getPixelWidth( ) ).floatValue( );
        float pixel_height = new Float( m_fileInfo.getPixelHeight( ) ).floatValue( );
        float mm_per_pixel;
        
        if ( pixel_width!= pixel_height )
        {
            mm_per_pixel = 0.5f * ( pixel_width + pixel_height );
        }
        else
        {
            mm_per_pixel = pixel_width;
        }
        
        float radius_in_pixel_number = baseRadius / mm_per_pixel;
        
        int[] x_coords = new int[number_of_points];
        int[] y_coords = new int[number_of_points];

        for ( int n = 0; n < number_of_points; n++ )
        {

            double phase = 2 * Math.PI * n / number_of_points;
            x_coords[n] = x_center + (int) ( radius_in_pixel_number * Math.cos( phase ) );
            y_coords[n] = y_center + (int) ( radius_in_pixel_number * Math.sin( phase ) );

        }

        RegioPolygon pol = new RegioPolygon( this.getNumberOfPolygons() + 1 , x_coords, y_coords, number_of_points );
        
        return pol;
    }
    
    
    /**
     * Adds circular ROIs to currently displayed image
     */
    public void addArbitraryNumberofConcentricalCircularShapedROIs( int baseRadius, int numberOfContours, int x_center, int y_center )
    {
        
        int number_of_points = 200;
        
        double r_0 = baseRadius;
        double r_n = r_0;
        
        for ( int c = 1; c < numberOfContours; c++ )
        {
            
            int[] x_coords = new int[number_of_points];
            int[] y_coords = new int[number_of_points];
            
            
            for ( int n = 0; n < number_of_points; n++ )
            {
                
                double phase = 2 * Math.PI * n / number_of_points;
                x_coords[n] = x_center + (int) ( r_n * Math.cos( phase ) );
                y_coords[n] = y_center + (int) ( r_n * Math.sin( phase ) );
                
            }
            
            r_n = r_0 * Math.sqrt( c + 1 );
            
            RegioPolygon pol = new RegioPolygon( c, x_coords, y_coords, number_of_points );
            this.addPolygon( pol );
            
        }
        
    }
    
    
    
    /**
     * Adds circular ROIs to currently displayed image
     */
    public void addThreeStandardCircularROIs( int x_center, int y_center )
    {
        
        int selectedImageType = m_scaledReferenceFrame.getSelectedImageType( );
        
        int number_of_points = 200;
        
        float height = ( new Float( m_fileInfo.getPixelHeight( ) ) ).floatValue( );
        float width  = ( new Float( m_fileInfo.getPixelWidth( ) ) ).floatValue( );
        
        
        double r_0 = 10 / height;
        double r_1 = 25 / height;
        double r_2 = 50 / height;
        
            
        int[] x_coords = new int[number_of_points];
        int[] y_coords = new int[number_of_points];

        for ( int n = 0; n < number_of_points; n++ )
        {
            double phase = 2 * Math.PI * n / number_of_points;
            x_coords[n] = x_center + (int) ( r_0 * Math.cos( phase ) );
            y_coords[n] = y_center + (int) ( r_0 * Math.sin( phase ) );
        }
        
        // 
        
        RegioPolygon pol_1 = new RegioPolygon( 1, x_coords, y_coords, number_of_points );
        this.addPolygon( pol_1 );

        for ( int n = 0; n < number_of_points; n++ )
        {
            double phase = 2 * Math.PI * n / number_of_points;
            x_coords[n] = x_center + (int) ( r_1 * Math.cos( phase ) );
            y_coords[n] = y_center + (int) ( r_1 * Math.sin( phase ) );
        }
            
        RegioPolygon pol_2 = new RegioPolygon( 2, x_coords, y_coords, number_of_points );
        this.addPolygon( pol_2 );
        
        //
        
        for ( int n = 0; n < number_of_points; n++ )
        {
            double phase = 2 * Math.PI * n / number_of_points;
            x_coords[n] = x_center + (int) ( r_2 * Math.cos( phase ) );
            y_coords[n] = y_center + (int) ( r_2 * Math.sin( phase ) );
        }
            
        RegioPolygon pol_3 = new RegioPolygon( 3, x_coords, y_coords, number_of_points );
        this.addPolygon( pol_3 );
        
        MomentsAboutTheMean fourMoments_1 = this.getMomentsAboutTheMeanOfRegioPolygon( m_currentlyDisplayedImageNumber, 1 );
        
        m_thresholdedOverlay = new boolean[m_imageHeight][m_imageWidth];
        
        int[] image = new int[m_imageWidth * m_imageHeight];
        image = getImagePixelData( );
        for ( int i = 0 ; i < m_imageWidth ; i++ )
        {
            for ( int j = 0 ; j < m_imageHeight ; j++ )
            {
                if ( ( pol_2.contains( i, j ) ) && ( !pol_1.contains( i, j ) ) )
                {
                    m_thresholdedOverlay[j][i] = true;
                }
                else
                {
                    m_thresholdedOverlay[j][i] = false;
                }
            }
        }
        
        MomentsAboutTheMeanOfVOICalculator momCalc_1_2 = new MomentsAboutTheMeanOfVOICalculator( m_thresholdedOverlay, image, m_imageWidth, m_imageHeight, m_zoom );
        MomentsAboutTheMean fourMoments_1_to_2 = momCalc_1_2.computeStatistics( );
        
        m_thresholdedOverlay = new boolean[m_imageHeight][m_imageWidth];
        
        for ( int i = 0 ; i < m_imageWidth ; i++ )
        {
            for ( int j = 0 ; j < m_imageHeight ; j++ )
            {
                if ( ( pol_3.contains( i, j ) ) && ( !pol_2.contains( i, j ) ) )
                {
                    m_thresholdedOverlay[j][i] = true;
                }
                else
                {
                    m_thresholdedOverlay[j][i] = false;
                }
            }
        }
          
        MomentsAboutTheMeanOfVOICalculator momCalc_2_3 = new MomentsAboutTheMeanOfVOICalculator( m_thresholdedOverlay, image, m_imageWidth, m_imageHeight, m_zoom );
        MomentsAboutTheMean fourMoments_2_to_3 = momCalc_2_3.computeStatistics( );
        
        for ( int i = 0 ; i < m_imageWidth ; i++ )
        {
            for ( int j = 0 ; j < m_imageHeight ; j++ )
            {
                m_thresholdedOverlay[j][i] = false;       
            }
        }     
        
        String fileNameExt = "";
        
        if ( selectedImageType == 0 )
        {
            fileNameExt = "_org"; 
        }
        
        if ( selectedImageType == 1 )
        {
            fileNameExt = "_cor"; 
        }
        
        if ( selectedImageType == 2 )
        {
            fileNameExt = "_dif"; 
        }
        
        String outputFileReference = getFilePathAndName( fileNameExt + ".txt", "roi_properties", "Save" );
        
        // String outputFileReference = System.getProperty( "user.home" ) + System.getProperty( "file.separator" ) + ".txt";

        TextFileWriter writer;
            
        try 
        {
            
            writer = new TextFileWriter( new FileWriter( outputFileReference ) );
            writer.writeStringLn( "**************************************************" );
            writer.writeStringLn( "*  Statistical properties of a ROI or VOI.       *" );
            writer.writeStringLn( "*  Do not edit!!.                                *" );
            writer.writeStringLn( "**************************************************" );

            Date today = new Date( );
            SimpleDateFormat formatter = new SimpleDateFormat( "dd-MMM-yyyy" );
            String dateStr = formatter.format( today );
            writer.writeStringLn( "* Date: " + dateStr + "                              *" );
            writer.writeStringLn( "**************************************************" );
            writer.writeStringLn( "" );
            
            fourMoments_1.write_HistogramStatistics_To_ASCII_File( writer, "ROI < 20mm", true );
            fourMoments_1_to_2.write_HistogramStatistics_To_ASCII_File( writer, "10mm < ROI < 20mm", false );
            fourMoments_2_to_3.write_HistogramStatistics_To_ASCII_File( writer, "20mm < ROI < 100mm", false );
            
            writer.close( );
            
        }
        catch ( IOException ex ) 
        {
            Logger.getLogger( DicomSliceCanvasPolyDraw.class.getName( ) ).log( Level.SEVERE, null, ex );
        }
        
        displayDicomImage( m_currentlyDisplayedImageNumber );
        drawImage( false );
    
    }
    
    
    /**
     * Adds circular ROIs to currently displayed image
     */
    public void saveThreeStandardCircularROIInfos( )
    {
        
        float height = ( new Float( m_fileInfo.getPixelHeight( ) ) ).floatValue( );
        float width  = ( new Float( m_fileInfo.getPixelWidth( ) ) ).floatValue( );
                
        LinkedList<RegioPolygon> list = this.m_polygonsVector.get( m_currentlyDisplayedImageNumber );
        
        RegioPolygon pol_1 = list.get( 0 );
        RegioPolygon pol_2 = list.get( 1 );
        RegioPolygon pol_3 = list.get( 2 );
        
        MomentsAboutTheMean fourMoments_1 = this.getMomentsAboutTheMeanOfRegioPolygon( m_currentlyDisplayedImageNumber, 1 );
        
        m_thresholdedOverlay = new boolean[m_imageHeight][m_imageWidth];
        
        int[] image = new int[m_imageWidth * m_imageHeight];
        image = getImagePixelData( );
        for ( int i = 0 ; i < m_imageWidth ; i++ )
        {
            for ( int j = 0 ; j < m_imageHeight ; j++ )
            {
                if ( ( pol_2.contains( i, j ) ) && ( !pol_1.contains( i, j ) ) )
                {
                    m_thresholdedOverlay[j][i] = true;
                }
                else
                {
                    m_thresholdedOverlay[j][i] = false;
                }
            }
        }
        
        MomentsAboutTheMeanOfVOICalculator momCalc_1_2 = new MomentsAboutTheMeanOfVOICalculator( m_thresholdedOverlay, image, m_imageWidth, m_imageHeight, m_zoom );
        MomentsAboutTheMean fourMoments_1_to_2 = momCalc_1_2.computeStatistics( );
        
        m_thresholdedOverlay = new boolean[m_imageHeight][m_imageWidth];
        
        for ( int i = 0 ; i < m_imageWidth ; i++ )
        {
            for ( int j = 0 ; j < m_imageHeight ; j++ )
            {
                if ( ( pol_3.contains( i, j ) ) && ( !pol_2.contains( i, j ) ) )
                {
                    m_thresholdedOverlay[j][i] = true;
                }
                else
                {
                    m_thresholdedOverlay[j][i] = false;
                }
            }
        }
          
        MomentsAboutTheMeanOfVOICalculator momCalc_2_3 = new MomentsAboutTheMeanOfVOICalculator( m_thresholdedOverlay, image, m_imageWidth, m_imageHeight, m_zoom );
        MomentsAboutTheMean fourMoments_2_to_3 = momCalc_2_3.computeStatistics( );
        
        for ( int i = 0 ; i < m_imageWidth ; i++ )
        {
            for ( int j = 0 ; j < m_imageHeight ; j++ )
            {
                m_thresholdedOverlay[j][i] = false;
            }
        }
        
        String fileNameExt = "";
        
        String outputFileReference = getFilePathAndName( fileNameExt + ".txt", "roi_properties", "Save" );
        
        // String outputFileReference = System.getProperty( "user.home" ) + System.getProperty( "file.separator" ) + ".txt";

        TextFileWriter writer;
            
        try 
        {
            
            writer = new TextFileWriter( new FileWriter( outputFileReference ) );
            writer.writeStringLn( "**************************************************" );
            writer.writeStringLn( "*  Statistical properties of a ROI or VOI.       *" );
            writer.writeStringLn( "*  Do not edit!!.                                *" );
            writer.writeStringLn( "**************************************************" );

            Date today = new Date( );
            SimpleDateFormat formatter = new SimpleDateFormat( "dd-MMM-yyyy" );
            String dateStr = formatter.format( today );
            writer.writeStringLn( "* Date: " + dateStr + "                              *" );
            writer.writeStringLn( "**************************************************" );
            writer.writeStringLn( "" );
            
            fourMoments_1.write_HistogramStatistics_To_ASCII_File( writer, "ROI < 20mm", true );
            fourMoments_1_to_2.write_HistogramStatistics_To_ASCII_File( writer, "10mm < ROI < 20mm", false );
            fourMoments_2_to_3.write_HistogramStatistics_To_ASCII_File( writer, "20mm < ROI < 100mm", false );
            
            writer.close( );
            
        }
        catch ( IOException ex ) 
        {
            Logger.getLogger( DicomSliceCanvasPolyDraw.class.getName( ) ).log( Level.SEVERE, null, ex );
        }
        
        displayDicomImage( m_currentlyDisplayedImageNumber );
        drawImage( false );
    
    }
    
    /** 
     * MouseListener implementation
     */
    @Override
    public void mouseEntered( MouseEvent e ) 
    {
        super.mouseEntered( e );
    }
    
    /** MouseListener implementation */
    @Override
    public void mouseExited( MouseEvent e ) 
    {
        super.mouseExited( e );
    }

    
    /**
     * Sets the last interactively indicated point
     * @return
     */
    private void setLastInteractivelyIndicatedPoint( int x, int y )
    {
        m_lastInteractivelyIndicatedPoint = new Point( x, y );
    }


    /**
     * Returns the last interactively indicated point
     * @return
     */
    public Point getLastInteractivelyIndicatedPoint( )
    {
        return m_lastInteractivelyIndicatedPoint;
    }


    /**
     * Implementation mousemotion listener
     */
    public void mouseMoved( MouseEvent e ) 
    {
       
        m_X = e.getX( );
        m_Y = e.getY( );

        if ( m_interactiveSegmentationMode == true )
        {
            setLastInteractivelyIndicatedPoint( m_X, m_Y );
            notifyActionListeners( NOTIFY_MOUSE_MOVED_IN_INTERACTIVE_MODE_ENABLED_EVENT );
        }

        // the 3D coordinates of the currently pointed coordinate
        Point3D p = compute3DCoordinate( );
        
        
        if ( m_X < (int)( m_imageWidth * m_zoom ) && ( m_Y  < (int)( m_imageHeight * m_zoom ) ) )
        {        
            if ( m_showToolTipText == true )
            {      
                if ( m_imagePixels == null )
                {
                    calculateImagePixels( );
                }
        
                float display_value = 0.0f;
                boolean valid_value = false;
                
                int pixelOffset = m_currentlyDisplayedImageNumber * m_imageHeight * m_imageWidth;
                
                if ( ( m_imagePixels != null ) && ( m_imagePixels.length != 0 ) )
                {
                    if ( ( m_X < m_zoom * m_imageWidth - 1 ) && ( m_Y < m_zoom * m_imageHeight - 1 ) )
                    {
                        if ( m_numberOfImages == 1 )
                        {
                            display_value = (float)( m_imagePixels[ m_X + (int)( m_imageWidth * m_zoom ) * m_Y ] + 16777216.0 ) / (float)16777216.0;
                        }
                        else
                        {
                            if ( m_imagePixelData != null )
                            {
                                display_value = (float)( m_imagePixelData[ pixelOffset + (int) (m_X / m_zoom ) + (int)( m_imageWidth * m_Y  / m_zoom ) ] );
                            }
                            else
                            {
                                display_value = (float)( m_imagePixels[ m_X + (int)( m_imageWidth * m_zoom ) * m_Y ] + 16777216.0 ) / (float)16777216.0;
                            }
                        }

                        //display_value = (float)( m_rawData[ (int)( m_X / m_zoom ) + (int)( m_imageWidth * ( m_Y  / m_zoom ) ) ] ) ;
                        valid_value = true;
                    }
                    else
                    {
                        valid_value = false;
                    }
                    
                    // display_value = m_rawDataMinVal + display_value * ( m_rawDataMaxVal - m_rawDataMinVal );
                    
                    if ( m_regioIsParent == true )
                    {
                        if ( ( m_X < m_zoom * m_imageWidth ) && ( m_Y < m_zoom * m_imageHeight ) )
                        {
                            notifyActionListeners( MOUSE_MOVED_EVENT );
                            
                        }
                    }
                    else
                    {
                        if ( ( valid_value == true ) && ( m_fileInfo != null ) )
                        {   
                            
                            NumberFormat format = new DecimalFormat( "##.#" );

                            String displayText = "<HTML>X = " + m_X + " Y = " + m_Y + "<BR><B>"
                                                 + "Pixel value          = " + display_value + "</B><BR>";
                                                 
                            if ( m_fileInfo.getNordicPixelScaling( ) != 1.0f  )
                            {
                                displayText = displayText + "Pixel value (scaled) = " + ( display_value * m_nordicScaleFactor ) + "</B><BR>";
                            }
                            
                            displayText = displayText +  "x = " + format.format( p.m_x1 ) 
                                                      +  "  y = " + format.format( p.m_x2 )
                                                      +  "z = " + format.format( p.m_x3 ) + "</HTML>";
                            
                            setToolTipText( displayText );
                            
                        }
                        else
                        {
                            setToolTipText( "<HTML>X = " + m_X + " Y = " + m_Y + "<BR><B>" +  "Pixel value = out of range" + "</B></HTML>" );  
                        }
                    }   
                }
            }
        }
    }


    /**
     * Computes the 3-dimensional coordinate of the point ( X, Y )
     */
    public Point3D compute3DCoordinate( int sliceNumber, int X, int Y )
    {

        FileInfo finfo = m_dicomImageSeriesData.getFileInfo( sliceNumber );

        // force the defintion of the required attributes
        finfo.setGeometricSliceAttributes( );

        Point3D topLeftPoint = finfo.getTopLeftPoint( );

        Point3D p = new Point3D( );

        if ( ( m_dirVecX != null ) && ( m_dirVecY != null ) )
        {
            if ( ( X > -1 ) && ( X < m_imageWidth * m_zoom ) &&
                 ( Y > -1 ) && ( Y < m_imageHeight * m_zoom ) )
            {

                double factorX = ( X / ( (double)m_imageWidth * m_zoom ) ) * m_FOVX;
                double factorY = ( Y / ( (double)m_imageHeight * m_zoom ) ) * m_FOVY;

                Point3D hori = new Point3D( m_dirVecX );
                hori.scalarMultiply( factorX );

                Point3D verti = new Point3D( m_dirVecY );
                verti.scalarMultiply( factorY );

                p.translate( topLeftPoint );
                p.translate( hori );
                p.translate( verti );
            }
        }
        else
        {
            p = null;
        }

        return p;
    }


    
    /**
     * Computes the 3-dimensional coordinate of the point ( X, Y )
     */
    public Point3D compute3DCoordinate( int X, int Y )
    {
        
        Point3D p = new Point3D( );
        
        if ( ( m_dirVecX != null ) && ( m_dirVecY != null ) )
        {
            if ( ( X > -1 ) && ( X < m_imageWidth * m_zoom ) && 
                 ( Y > -1 ) && ( Y < m_imageHeight * m_zoom ) )
            {
            
                double factorX = ( X / ( (double)m_imageWidth * m_zoom ) ) * m_FOVX;
                double factorY = ( Y / ( (double)m_imageHeight * m_zoom ) ) * m_FOVY;
        
                Point3D hori = new Point3D( this.m_dirVecX );
                hori.scalarMultiply( factorX );
        
                Point3D verti = new Point3D( this.m_dirVecY );
                verti.scalarMultiply( factorY );
        
                p.translate( this.m_topLeftPoint );
                p.translate( hori );
                p.translate( verti );
            }
        }
        else
        {
            p = null;
        }
        
        return p;
    }
    
    /**
     * Computes the 3-dimensional coordinate of the point ( m_X, m_Y )
     */
    private Point3D compute3DCoordinate( )
    {
        Point3D p = new Point3D( );
        
        if ( ( m_dirVecX != null ) && ( m_dirVecY != null ) )
        {
            double factorX = ( m_X / ( this.m_imageWidth * m_zoom ) ) * m_FOVX;
            double factorY = ( m_Y / ( this.m_imageHeight * m_zoom ) ) * m_FOVY;
        
            Point3D hori = new Point3D( this.m_dirVecX );
            hori.scalarMultiply( factorX );
        
            Point3D verti = new Point3D( this.m_dirVecY );
            verti.scalarMultiply( factorY );
        
            p.translate( this.m_topLeftPoint );
            p.translate( hori );
            p.translate( verti );
        }
        
        return p;
    }
    
    
    /**
     * Returns the value of the selected point as string
     */
    public String get3DCoordinateAsString( )
    {
        Point3D p = compute3DCoordinate( );
        NumberFormat format = new DecimalFormat( "##.#" );

        String str =  "x = " + format.format( p.m_x1 ) 
                   +  "  y = " + format.format( p.m_x2 )
                   +  "z = " + format.format( p.m_x3 );
        
        return str;
    }

    
    
    /**
     * Replaces the raw data pixel values
     * @param replacementType
     */
    private void replaceRawDataPixels( int replacementType, int userMinimum, int userMaximum, int newPixelValue )
    {
       if ( replacementType == PixelValueReplacementFrame.REPLACE_ONLY_LOWER_THAN )
       {
           for ( int i = 0 ; i < m_imagePixelData.length; i++ )
           {
               if ( m_imagePixelData[i] < userMinimum )
               {
                   m_imagePixelData[i] = newPixelValue;
               }
           }    
       }
       
       if ( replacementType == PixelValueReplacementFrame.REPLACE_ONLY_HIGHER_THAN )
       {
           for ( int i = 0 ; i < m_imagePixelData.length; i++ )
           {
               if ( m_imagePixelData[i] > userMaximum )
               {
                   m_imagePixelData[i] = newPixelValue;
               }
           }    
       }
       
       if ( replacementType == PixelValueReplacementFrame.REPLACE_ONLY_HIGHER_THAN )
       {
           for ( int i = 0 ; i < m_imagePixelData.length; i++ )
           {
               if ( m_imagePixelData[i] > userMaximum )
               {
                   m_imagePixelData[i] = newPixelValue;
               }
           }    
       }
       
       if ( replacementType == PixelValueReplacementFrame.REPLACE_INSIDE_INTERVAL )
       {
           for ( int i = 0 ; i < m_imagePixelData.length; i++ )
           {
               if ( ( m_imagePixelData[i] > userMinimum ) && ( m_imagePixelData[i] < userMaximum ) )
               {
                   m_imagePixelData[i] = newPixelValue;
               }
           }    
       }
       
       if ( replacementType == PixelValueReplacementFrame.REPLACE_OUTSIDE_INTERVAL )
       {
           for ( int i = 0 ; i < m_imagePixelData.length; i++ )
           {
               if ( !( ( m_imagePixelData[i] > userMinimum ) && ( m_imagePixelData[i] < userMaximum ) ) )
               {
                   m_imagePixelData[i] = newPixelValue;
               }
           }    
       }
        
       // force a redraw of the currently displayed image
        displayDicomImage(m_currentlyDisplayedImageNumber );
        drawImage( false ); 
       
    }
    
    /**
     * Segments the image in separate threadth
     */
    public void performConversionOfImageSegmentOverlayToRegioPolygons( final boolean allImages )
    {
       
        
        m_dualProgressBar = new DualProgressBar( m_parent, 100, 100, new Dimension( 900, 150 ), false );
        m_dualProgressBar.setTitle( "Conversion to contours "  );
            
        m_dualProgressBar.setUpperProgress( 0 );
        m_dualProgressBar.setUpperLabel( "        Total progress " );
        m_dualProgressBar.setUpperProgressVisible( false );
        m_dualProgressBar.setLowerLabel( "<HTML><BR>  Converting image number : " +  0 + "</HTML>" );
         
        
       final SwingWorker worker = new SwingWorker( )
       {
            public Object construct( )
            {
                try
                {
                    // segment the image
                    convertImageSegmentOverlayToRegioPolygons( allImages );
                    excludeSmallInnerRegionsWithinOneSliceForAllSlices( );
                }
                catch ( OutOfMemoryError e )
                {
                    Dialogs d = new Dialogs( );
                    Dialogs.messageBox( m_parent, "<HTML> Too much ROI's are found to be handled <BR> Change pixel thressholds ranges & roi size ranges </HTML>" );
                }
                return null;
            }
                
            // runs in the event dispatching thread
            @Override
            public void finished( )
            {
                 // After calculation set enabled again 
                 m_dualProgressBar.setVisible( false );
                 
                 Dialogs d = new Dialogs( );
                 Dialogs.messageBox( m_parent, "<HTML> Conversion ready </HTML>" );
            }
 
        };
        
        worker.start( );
    }
    
    /**
     * Converts Image Segment -images to RegioPolygons 
     */
    private void convertImageSegmentOverlayToRegioPolygons( boolean allImages )
    {
        SimpleThressholdSegmentationCalculator simpCalc = new SimpleThressholdSegmentationCalculator( m_parent, this );
                
        if ( ( m_imageSegmentsOverlayList != null ) && ( m_selectedTissueTypes != null ) )
        {
            if ( ( !m_imageSegmentsOverlayList.isEmpty( ) ) && ( m_selectedTissueTypes.length > 0 ) )
            {
                LinkedList<boolean[][]> colorOverlays = m_imageSegmentsOverlayList.get(m_currentlyDisplayedImageNumber );
        
                int numbersOfTissues = colorOverlays.size( );
        
                Color[][] colormap = new Color[m_imageHeight][m_imageWidth];
        
                for ( int k = 0 ; k < m_selectedTissueTypes.length ; k++ )
                {
                    if ( m_selectedTissueTypes[k] < numbersOfTissues )
                    {
                        if ( allImages == false )
                        {
                            m_dualProgressBar.setLowerProgress( (int)( 100.0 ) );
                            m_dualProgressBar.setLowerLabel( "<HTML><BR>  Converting image number : " +  0 + "</HTML>"  );
                            
                            simpCalc.computeRegioPolygonObjectsOfSegmentOverlayOfView( getCurrentlyDisplayedImageNumber( ), m_selectedTissueTypes[k], false );
                        }
                        else
                        {
                            for( int i = 0; i < m_numberOfImages; i++ )
                            {
                                simpCalc.computeRegioPolygonObjectsOfSegmentOverlayOfView( i, m_selectedTissueTypes[k], false );
                                m_dualProgressBar.setLowerProgress( (int)( 100.0 * (double)i / m_numberOfImages ) );
                                m_dualProgressBar.setLowerLabel( "<HTML><BR>  Converting image number : " +  i + "</HTML>"  );
                            }
                        }
                    }
                }
            }
        }
    }
    

    
    /**
     * Converts a boolean[][] to a byte[]
     * @param h
     * @param w
     * @param array
     */
    private byte[] convertToByteArray( int w, int h, boolean[][] array )
    {
        byte[] outputByteArray = new byte[w*h];
        int cntr = 0;
        for ( int i = 0; i < h; i++ )
        {
            for( int j = 0; j < w; j++ )
            {
                outputByteArray[cntr] = (byte)(array[i][j]?1:0);
                cntr++;
            }
        }
        return outputByteArray;
    }
    
    /**
     * 
     * Loads all overlays to file. For each image of a slice stack one 
     * overlay file is assumed having extension .ovl
     * 
     */
    private void loadOverlaysFromFiles( )
    {
        
        // create a new instance for image segemnt overlays
        m_imageSegmentsOverlayList = new ArrayList< LinkedList<boolean[][] > >( );
        
        // Determine path to load the overlay files from
        String filePath = m_dicomImageSeriesData.getFileInfo( 0 ).getDirectory( );
        
        File outputPath = new File( filePath + System.getProperty( "file.separator" ) + "ovl" );
        
        if ( outputPath.exists( ) )
        {
            // if the path exists then try to read overlays
            for( int i = 0; i < m_numberOfImages; i++ )
            {
                String fileName = m_dicomImageSeriesData.getFileInfo( i ).getFileName( );
                fileName = fileName + ".ovl";
                try
                {
                    try
                    {
                        //  if file alreay exists than delete it before writing to it!
                        File f = new File( outputPath.getAbsolutePath( )+ System.getProperty( "file.separator" ) + fileName );
                        
                        if ( f.exists( ) )
                        {
                            
                            RandomAccessFile reader = new RandomAccessFile( new File( outputPath.getAbsolutePath( )+ System.getProperty( "file.separator" ) + fileName ) , "rw" );
                            
                            //System.out.println( "reading overlayFile" + fileName );
                            
                            // first thing written in the file is the filename the overlay belongs to
                            String testString;
                            
                            testString = reader.readUTF( );
                            if ( fileName.compareTo( testString ) == 0 )
                            {
                                // write the number of overlays the file contains
                                int numberOfOverlays = reader.readInt( );
                        
                                int imageWidth = reader.readInt( );
                        
                                // write the y dimension (height) of the image
                                int imageHeight = reader.readInt( );
                                
                                LinkedList<boolean[][]> arrayList = new LinkedList<boolean[][]>( );
                                
                                for ( int k = 0; k < numberOfOverlays; k++ )
                                {
                                    boolean[][] overLay = readOverlay( imageWidth, imageHeight, reader );
                                    arrayList.add( overLay );
                                }
                                
                                m_imageSegmentsOverlayList.add( arrayList );
                                
                                reader.close( );
                            }
                        }
                    }
                    catch ( FileNotFoundException e )
                    {
                         e.printStackTrace( );
                    }
                 }
                 catch( IOException ioe )
                 {
                     ioe.printStackTrace( );   
                 }
            }
        }
        
        notifyActionListeners( OVERLAYS_READ_FROM_FILE_EVENT,  "" + m_imageSegmentsOverlayList.get(0).size( ) );
        
        m_imageSegmentColors = defaultColors;
        m_alpha = 0.3f;
        
        // set the thresshold to true for all pixels otherwise control cannot paint the overlays properly
        m_thressholdedImageSegmentOverlays = new boolean[m_numberOfImages][m_imageWidth][m_imageHeight];
        for ( int i = 0; i < m_numberOfImages; i++ )
        {
            for ( int j = 0; j < m_imageWidth; j++ )
            {
                for ( int k = 0; k < m_imageHeight; k++ )
                {
                    m_thressholdedImageSegmentOverlays[i][j][k] = true;
                }
            }
        }

        // notityListeners that a tissue type was eliminated
        // force an update
        displayDicomImage(m_currentlyDisplayedImageNumber );
        drawImage( false ); 
    }

    /**
     * Returns the number of volumes of the view
     * @return
     */
    public int getNumberOfVolumes( )
    {
        return m_imageVolumeOfInterestNameList.size( );
    }


    /**
     *
     * Loads all overlays to file. For each image of a slice stack one
     * overlay file is assumed having extension .ovl
     *
     */
    private void loadVolumesOfInterestOverlaysFromFiles( )
    {

        int numberOfOverlays = 0;

        // create a new instance for image segemnt overlays
        m_imageVolumeOfInterestOverlayList = new ArrayList< LinkedList<boolean[][] > >( );

        // Determine path to load the overlay files from
        String filePath = m_dicomImageSeriesData.getFileInfo( 0 ).getDirectory( );

        File outputPath = new File( filePath + System.getProperty( "file.separator" ) + "ovl" );

        if ( outputPath.exists( ) )
        {
            // if the path exists then try to read overlays
            for ( int i = 0; i < m_numberOfImages; i++ )
            {
                String fileName = m_dicomImageSeriesData.getFileInfo( i ).getFileName( );
                fileName = fileName + ".ovl";
                try
                {
                    try
                    {
                        //  if file alreay exists than delete it before writing to it!
                        File f = new File( outputPath.getAbsolutePath( ) + System.getProperty( "file.separator" ) + fileName );

                        if ( f.exists( ) )
                        {

                            RandomAccessFile reader = new RandomAccessFile( new File( outputPath.getAbsolutePath( )+ System.getProperty( "file.separator" ) + fileName ) , "rw" );

                            //System.out.println( "reading overlayFile" + fileName );

                            // first thing written in the file is the filename the overlay belongs to
                            String testString;

                            testString = reader.readUTF( );
                            if ( fileName.compareTo( testString ) == 0 )
                            {
                                // read the number of overlays the file contains
                                numberOfOverlays = reader.readInt( );

                                int imageWidth = reader.readInt( );

                                // read the y dimension (height) of the image
                                int imageHeight = reader.readInt( );

                                LinkedList<boolean[][]> arrayList = new LinkedList<boolean[][]>( );

                                for ( int k = 0; k < numberOfOverlays; k++ )
                                {
                                    boolean[][] overLay = readOverlay( imageWidth, imageHeight, reader );
                                    arrayList.add( overLay );

                                }

                                m_imageVolumeOfInterestOverlayList.add( arrayList );

                                reader.close( );
                            }
                        }
                    }
                    catch ( FileNotFoundException e )
                    {
                         e.printStackTrace( );
                    }
                 }
                 catch( IOException ioe )
                 {
                     ioe.printStackTrace( );
                 }
            }
        }

        m_selectedImageVolumeOfInterestList.clear( );
        m_imageVolumeOfInterestNameList.clear( );
        m_imageVolumeOfInterestColorList.clear( );

        for ( int i = 0; i < numberOfOverlays; i++ )
        {
            m_selectedImageVolumeOfInterestList.add( true );
            m_imageVolumeOfInterestNameList.add( "Volume_" + i );
            m_imageVolumeOfInterestColorList.add( defaultColors[i] );
        }

        // notityListeners that a tissue type was eliminated
        // force an update
        displayDicomImage(m_currentlyDisplayedImageNumber );
        drawImage( false );
    }

    
    
    /**
     * Loads the volumes from an mha file
     * (SNF tumor project related)
     */ 
    private void load_Volumes_Overlays_From_MHA_File( )
    {
        MHAFileOverlayReader mhaReader = new MHAFileOverlayReader( );    
        
        if ( mhaReader.doLoading( m_fileInfo.getDirectory( ) ) == true )
        {
            m_mhaImageSegmentOverlays = mhaReader.getBooleanOverlay( );
        }
        
        displayDicomImage(m_currentlyDisplayedImageNumber );
        drawImage( false );
        
    }
    
    
    /**
     *
     * Loads all overlays to file. For each image of a slice stack one
     * overlay file is assumed having extension .ovl
     *
     */
    private void loadVolumesOfInterestOverlaysFromFile( )
    {

        int numberOfOverlays = 0;

        // create a new instance for image segemnt overlays
        m_imageVolumeOfInterestOverlayList = new ArrayList< LinkedList<boolean[][] > >( );

        // Determine path to load the overlay files from
        String filePath = m_fileInfo.getDirectory( );

        File inputPath = new File( filePath + System.getProperty( "file.separator" ) + "ovl" );

        if ( inputPath.exists( ) )
        {
//            if the path exists then try to read overlays
//            for ( int i = 0; i < m_numberOfImages; i++ )
//            {
            String fileName = m_fileInfo.getFileName( );
            fileName = fileName + ".ovl";
            try
            {
                try
                {
                    //  if file alreay exists than delete it before writing to it!
                    File f = new File( inputPath.getAbsolutePath( ) + System.getProperty( "file.separator" ) + fileName );

                    if ( f.exists( ) )
                    {

                        RandomAccessFile reader = new RandomAccessFile( new File( inputPath.getAbsolutePath( )+ System.getProperty( "file.separator" ) + fileName ) , "rw" );

                        //System.out.println( "reading overlayFile" + fileName );

                        // first thing written in the file is the filename the overlay belongs to
                        String testString;

                        testString = reader.readUTF( );
                        if ( fileName.compareTo( testString ) == 0 )
                        {
                            // load the number of overlays the file contains
                            numberOfOverlays = reader.readInt( );

                            int imageWidth = reader.readInt( );

                            // load the y dimension (height) of the image
                            int imageHeight = reader.readInt( );

                            LinkedList<boolean[][]> arrayList = new LinkedList<boolean[][]>( );

                            for ( int k = 0; k < numberOfOverlays; k++ )
                            {
                                boolean[][] overLay = readOverlay( imageWidth, imageHeight, reader );
                                arrayList.add( overLay );

                            }

                            m_imageVolumeOfInterestOverlayList.add( arrayList );

                            reader.close( );
                        }
                    }
                }
                catch ( FileNotFoundException e )
                {
                     e.printStackTrace( );
                }
             }
             catch( IOException ioe )
             {
                 ioe.printStackTrace( );
             }
            //}
        }

        m_selectedImageVolumeOfInterestList.clear( );
        m_imageVolumeOfInterestNameList.clear( );
        m_imageVolumeOfInterestColorList.clear( );

        for ( int i = 0; i < numberOfOverlays; i++ )
        {
            m_selectedImageVolumeOfInterestList.add( true );
            m_imageVolumeOfInterestNameList.add( "Volume_" + i );
            m_imageVolumeOfInterestColorList.add( defaultColors[i] );
        }

        // notityListeners that a tissue type was eliminated
        // force an update
        displayDicomImage(m_currentlyDisplayedImageNumber );
        drawImage( false );
    }

    /**
     *
     * Loads all overlays to file. For each image of a slice stack one
     * overlay file is assumed having extension .ovl
     *
     */
//    private void loadVolumeOfInterestOverlaysFromFiles( )
//    {
//
//        System.out.println( "loadVolumeOfInterestOverlaysFromFiles( )" );
//
//        // create a new instance for image segemnt overlays
//        m_imageVolumeOfInterestOverlayList = new ArrayList< LinkedList<boolean[][] > >( );
//
//        // Determine path to load the overlay files from
//        String filePath = m_dicomImageSeriesData.getFileInfo(0).getDirectory( );
//
//        File outputPath = new File( filePath + System.getProperty( "file.separator" ) + "ovl" );
//
//        if ( outputPath.exists( ) )
//        {
//            // if the path exists then try to read overlays
//            for( int i = 0; i < m_numberOfImages; i++ )
//            {
//                String fileName = m_dicomImageSeriesData.getFileInfo( i ).getFileName( );
//                fileName = fileName + ".ovl";
//                try
//                {
//                    try
//                    {
//                        //  if file alreay exists than delete it before writing to it!
//                        File f = new File( outputPath.getAbsolutePath( )+ System.getProperty( "file.separator" ) + fileName );
//
//                        if ( f.exists( ) )
//                        {
//
//                            RandomAccessFile reader = new RandomAccessFile( new File( outputPath.getAbsolutePath( )+ System.getProperty( "file.separator" ) + fileName ) , "rw" );
//
//                            System.out.println( "reading overlayFile" + fileName );
//
//                            // first thing written in the file is the filename the overlay belongs to
//                            String testString;
//
//                            testString = reader.readUTF( );
//                            if ( fileName.compareTo( testString ) == 0 )
//                            {
//                                // write the number of overlays the file contains
//                                int numberOfOverlays = reader.readInt( );
//
//                                int imageWidth = reader.readInt( );
//
//                                // write the y dimension (height) of the image
//                                int imageHeight = reader.readInt( );
//
//                                LinkedList<boolean[][]> arrayList = new LinkedList<boolean[][]>( );
//
//                                for ( int k = 0; k < numberOfOverlays; k++ )
//                                {
//                                    boolean[][] overLay = readOverlay( imageWidth, imageHeight, reader );
//                                    arrayList.add( overLay );
//                                }
//
//                                m_imageVolumeOfInterestOverlayList.add( arrayList );
//
//                                reader.close( );
//                            }
//                        }
//                    }
//                    catch ( FileNotFoundException e )
//                    {
//                         e.printStackTrace( );
//                    }
//                 }
//                 catch( IOException ioe )
//                 {
//                     ioe.printStackTrace( );
//                 }
//            }
//        }
//
//        // notityListeners that a tissue type was eliminated
//        // force an update
//        displayDicomImage( m_currentlyDisplayedImage );
//        drawImage( false );
//    }



    /**
     * Return which volumes should be displayed
     * @return
     */
     public LinkedList<Boolean> getSelectedImageVolumeOfInterestList( )
     {
         return m_selectedImageVolumeOfInterestList;
     }

     /**
      * Return which the name list of the volumes
      */
     public  LinkedList<String> getImageVolumeOfInterestNameList( )
     {
         return m_imageVolumeOfInterestNameList;
     }

     /**
      * Returns the colorlist of the volumes
      */
     public  LinkedList<Color> getImageVolumeOfInterestColorList( )
     {
        return m_imageVolumeOfInterestColorList;
     }

     /**
     * Return which volumes should be displayed
     * @return
     */
     public void setSelectedImageVolumeOfInterestList( LinkedList<Boolean> selectedImageVolumeOfInterestList )
     {
         m_selectedImageVolumeOfInterestList = selectedImageVolumeOfInterestList;
     }

     /**
      * Return which the name list of the volumes
      */
     public void setImageVolumeOfInterestNameList( LinkedList<String> imageVolumeOfInterestNameList )
     {
         m_imageVolumeOfInterestNameList = imageVolumeOfInterestNameList;
     }

     /**
      * Returns the colorlist of the volumes
      */
     public void setImageVolumeOfInterestColorList( LinkedList<Color> imageVolumeOfInterestColorList )
     {
        m_imageVolumeOfInterestColorList = imageVolumeOfInterestColorList;
     }

    /**
     * Returns the dicom directory selected by the user
     */
    private File getDirectory( )
    {
        DicomDefaults fdef = DicomDefaults.getInstance( );
        fdef.load( );

        String path = fdef.getProperty( "Overlay mount path" );

        JFileChooser fileChooser = new JFileChooser( fdef.getProperty( "Overlay mount path" ) + System.getProperty( "file.separator" ) );

        fileChooser.setName( "Select Overlay Directory" );

        fileChooser.setFileSelectionMode( JFileChooser.DIRECTORIES_ONLY );

        int result = fileChooser.showOpenDialog( this );

        File selected_dir = new File( "" );

        switch ( result )
        {
            case JFileChooser.APPROVE_OPTION:   // Approve Open button was pressed
                                                selected_dir = fileChooser.getSelectedFile( );
                                                DicomDefaults def = DicomDefaults.getInstance( );
                                                def.setProperty( "Overlay mount path", selected_dir.getPath( ) );
                                                def.save( );

                                                break;
           case JFileChooser.CANCEL_OPTION:     // Cancel or close dialog was selected
                                                break;
           case JFileChooser.ERROR_OPTION:      // the selected process did not terminate properly
                                                break;
        }

        return selected_dir;
    }



    /**
     *
     * Loads all overlays to file. For each image of a slice stack one
     * overlay file is assumed having extension .ovl
     *
     */
    private void loadOverlaysFromFilesUserDefinedPath( )
    {

        // create a new instance for image segemnt overlays
        m_imageSegmentsOverlayList = new ArrayList< LinkedList<boolean[][] > >( );

        // Determine path to load the overlay files from
        //String filePath = m_dicomImageSeriesData.getFileInfo(0).getDirectory( );

        //File outputPath = new File( filePath + System.getProperty( "file.separator" ) + "ovl" );
        File inputPath = getDirectory( );

        if ( inputPath.exists( ) )
        {
            // if the path exists then try to read overlays
            for( int i = 0; i < m_numberOfImages; i++ )
            {
                String fileName = m_dicomImageSeriesData.getFileInfo( i ).getFileName( );
                fileName = fileName + ".ovl";
                try
                {
                    try
                    {
                        //  if file alreay exists than delete it before writing to it!
                        File f = new File( inputPath.getAbsolutePath( )+ System.getProperty( "file.separator" ) + fileName );

                        if ( f.exists( ) )
                        {

                            RandomAccessFile reader = new RandomAccessFile( new File( inputPath.getAbsolutePath( )+ System.getProperty( "file.separator" ) + fileName ) , "rw" );

                            // System.out.println( "reading overlayFile" + fileName );

                            // first thing written in the file is the filename the overlay belongs to
                            String testString;

                            testString = reader.readUTF( );
                            if ( fileName.compareTo(testString) == 0 )
                            {
                                // write the number of overlays the file contains
                                int numberOfOverlays = reader.readInt( );

                                int imageWidth = reader.readInt( );

                                // write the y dimension (height) of the image
                                int imageHeight = reader.readInt( );

                                LinkedList<boolean[][]> arrayList = new LinkedList<boolean[][]>( );

                                for ( int k = 0; k < numberOfOverlays; k++ )
                                {
                                    boolean[][] overLay = readOverlay( imageWidth, imageHeight, reader );
                                    arrayList.add( overLay );
                                }

                                m_imageSegmentsOverlayList.add( arrayList );

                                reader.close( );
                            }
                        }
                    }
                    catch ( FileNotFoundException e )
                    {
                         e.printStackTrace( );
                    }
                 }
                 catch( IOException ioe )
                 {
                     ioe.printStackTrace( );
                 }
            }
        }

        notifyActionListeners( OVERLAYS_READ_FROM_FILE_EVENT,  "" + m_imageSegmentsOverlayList.get( 0 ).size( ) );

        m_imageSegmentColors = defaultColors;
        m_alpha = 0.3f;

        // set the thresshold to true for all pixels otherwise control cannot paint the overlays properly
        m_thressholdedImageSegmentOverlays = new boolean[m_numberOfImages][m_imageWidth][m_imageHeight];
        for ( int i = 0; i < m_numberOfImages; i++ )
        {
            for ( int j = 0; j < m_imageWidth; j++ )
            {
                for ( int k = 0; k < m_imageHeight; k++ )
                {
                    m_thressholdedImageSegmentOverlays[i][j][k] = true;
                }
            }
        }

        // notityListeners that a tissue type was eliminated
        // force an update
        displayDicomImage(m_currentlyDisplayedImageNumber );
        drawImage( false );
    }

    /**
     *
     * Loads all overlays to file. For each image of a slice stack one
     * overlay file is assumed having extension .ovl
     *
     */
    private void loadVolumesOfInterestFromFilesUserDefinedPath( )
    {
        int numberOfOverlays = 0;
        // create a new instance for image segemnt overlays
        m_imageSegmentsOverlayList = new ArrayList< LinkedList<boolean[][] > >( );

        File inputPath = getDirectory( );

        if ( inputPath.exists( ) )
        {
            // if the path exists then try to read overlays
            for( int i = 0; i < m_numberOfImages; i++ )
            {
                String fileName = m_dicomImageSeriesData.getFileInfo( i ).getFileName( );
                fileName = fileName + ".ovl";
                try
                {
                    try
                    {
                        //  if file alreay exists than delete it before writing to it!
                        File f = new File( inputPath.getAbsolutePath( )+ System.getProperty( "file.separator" ) + fileName );

                        if ( f.exists( ) )
                        {

                            RandomAccessFile reader = new RandomAccessFile( new File( inputPath.getAbsolutePath( ) + System.getProperty( "file.separator" ) + fileName ) , "rw" );

                            // first thing written in the file is the filename the overlay belongs to
                            String testString;

                            testString = reader.readUTF( );
                            if ( fileName.compareTo( testString ) == 0 )
                            {
                                
                                int imageWidth = reader.readInt( );

                                // write the y dimension (height) of the image
                                int imageHeight = reader.readInt( );

                                LinkedList<boolean[][]> arrayList = new LinkedList<boolean[][]>( );

                                for ( int k = 0; k < numberOfOverlays; k++ )
                                {
                                    boolean[][] overLay = readOverlay( imageWidth, imageHeight, reader );
                                    arrayList.add( overLay );
                                }

                                m_imageVolumeOfInterestOverlayList.add( arrayList );

                                reader.close( );
                            }
                        }
                    }
                    catch ( FileNotFoundException e )
                    {
                         e.printStackTrace( );
                    }
                 }
                 catch( IOException ioe )
                 {
                     ioe.printStackTrace( );
                 }
            }
        }

        m_selectedImageVolumeOfInterestList.clear( );
        m_imageVolumeOfInterestNameList.clear( );
        m_imageVolumeOfInterestColorList.clear( );

        for ( int i = 0; i < numberOfOverlays; i++ )
        {
            m_selectedImageVolumeOfInterestList.add( true );
            m_imageVolumeOfInterestNameList.add( "Volume_" + i );
            m_imageVolumeOfInterestColorList.add( defaultColors[i] );
        }

        // notityListeners that a tissue type was eliminated
        // force an update
        displayDicomImage(m_currentlyDisplayedImageNumber );
        drawImage( false );
        
    }
   
    /**
     * Reads the overlay from a random access file and return the overlay
     * as a boolean[][]
     */
    private boolean[][] readOverlay( int w, int h, RandomAccessFile reader )
    {
        
        byte[] byteArr = new byte[w*h];
        boolean[][] overlay = new boolean[w][h];
        // read the complete overlay once!
        try
        {
            reader.read( byteArr, 0, w * h );
            int cntr = 0;
        
            // do some reformatting
            for ( int i = 0; i < w; i++ )
            {
                for ( int j = 0; j < h; j++ )
                {
                    if( ((int)byteArr[cntr]) == 1 )
                    {
                        overlay[i][j]= true;
                    }
                    else
                    {
                        overlay[i][j]= false;
                    }
                    cntr++;
                }
            }
        }
        catch ( IOException ioe )
        {
            ioe.printStackTrace( );
        }
        
        return overlay;
    }
     
    /**
     * Converts the user defined ROI information 
     * @return roiInformation
     */
    LinkedList<int[]> getROIInformationAsIntegerOverlay( )
    {
        
        LinkedList<int[]> roiInformation = new LinkedList<int[]>( );

        for ( int i = 0; i < m_dicomImageSeriesData.getNumberOfImages( ); i++ )
        {
            int[] iov = new int[m_dicomImageSeriesData.getFileInfo( i ).getImageWidth( ) * m_dicomImageSeriesData.getFileInfo( i ).getImageHeight( )];
            // First get the reference to the polygons of the image
            LinkedList<RegioPolygon> polygonList = this.m_polygonsVector.get( i );
            
             System.out.println( "called .. getROIInformationAsIntegerOverlay( )" );
            
            // create integer overlay datastructure 
            for ( int pol_count = 0; pol_count < polygonList.size( ); pol_count++ )
            {
                int cntr = 0;
                
                RegioPolygon polygon = polygonList.get( pol_count );
                int tst = 0;
                for ( int j = 0; j < m_dicomImageSeriesData.getFileInfo( i ).getImageHeight( ); j++ )
                {
                    for ( int k = 0; k < m_dicomImageSeriesData.getFileInfo( i ).getImageWidth( ); k++ )
                    {
                        
                        if ( polygon.contains( k, j ) == true )
                        {
                            iov[cntr] = polygon.getIndex( );
                            // System.out.println( "pol index = " + polygon.getIndex( ) );
                            tst++;
                        }
                        
                        cntr++;
                    }
                }
                
                System.out.println( "tst (output) = " + tst );
                
            }
            
            // add the roi information object to the return object
            roiInformation.add( iov );
        }
        
        return roiInformation;
    }

    /**
     * 
     * Saves all overlays to file. For each image of a slice stack one 
     * integer typed overlay file is assumed having extension .iov
     * The value 0 is assigned to pixels outside any user draws ROI 
     * The value i is assigned to pixels on and within a user drawn ROI
     */
    private void saveRegionsToIntegerOverlaysFiles( )
    {
        // Determine path to write overlay files to
        String filePath = m_dicomImageSeriesData.getFileInfo( 0 ).getDirectory( );
        
        File outputPath = new File( filePath + System.getProperty( "file.separator" ) + "iov" );
        
        // create the directory
        outputPath.mkdir( );
        
        // if the directory exists than writing can start
        if ( outputPath.exists( ) == true )
        {   
            
            // Compute the integer typed out outdata structures
            LinkedList<int[]> roiInfoAsIntegerOverlay = getROIInformationAsIntegerOverlay( );
                                    
            for( int i = 0; i < roiInfoAsIntegerOverlay.size( ); i++ )
            {
                
                String fileName = m_dicomImageSeriesData.getFileInfo( i ).getFileName( );
                
                int[] iov = roiInfoAsIntegerOverlay.get( i );
                
                fileName = fileName + ".iov";
                try
                {
                    try
                    {
                        // if file alreay exists than delete it before writing to it!
                        File f = new File( outputPath.getAbsolutePath( )+ System.getProperty( "file.separator" ) + fileName );
                                 
                        if ( f.exists( ) )
                        {
                            f.delete( );
                        }
                        
                        RandomAccessFile writer = new RandomAccessFile( new File( outputPath.getAbsolutePath( )+ System.getProperty( "file.separator" ) + fileName ) , "rw" );
                        
                        // first thing written in the file is the filename the overlay belongs to
                        writer.writeUTF( fileName );
                        
                        // write the x dimension (width) of the image
                        writer.writeInt( m_imageWidth );
                        
                        // write the y dimension (height) of the image
                        writer.writeInt( m_imageHeight );
                        
                        int cntr = 0;
                        
                        for (int j = 0;j < m_imageHeight; j++ )
                        {
                            for ( int k = 0; k < m_imageWidth; k++ )
                            {
                                writer.writeInt(iov[cntr]);
                                cntr++;
                            }
                        }
                                                
                        writer.close( );
                        
                    }
                    catch ( FileNotFoundException e )
                    {
                        // not handled yet
                    }
                }
            
                catch( IOException ioe )
                {
                    // to be implemented
                }
            }
        }
    }

    
    
    /**
     * 
     * Load all integer overlays to file. For each image of a slice stack one 
     * boolean typed overlay file is assumed having extension .iov
     * The value 0 is assigned to pixels outside any user draws ROI 
     * The value 1 is assigned to pixels a user drawn ROIs
     * no distinctions between different ROIs can be made;
     * This routine was made to test the output of its save counterpart
     *          this.saveRegionsToIntegerOverlaysFiles( )
     */
    private void loadRegionsFromIntegerOverlaysFiles( )
    {
         
        int numberOfOverlays = 0;

        // create a new instance for image segment overlays
        m_imageVolumeOfInterestOverlayList = new ArrayList< LinkedList<boolean[][] > >( );
        
        String filePath = m_dicomImageSeriesData.getFileInfo( 0 ).getDirectory( );
        
        for ( int i = 0; i < m_dicomImageSeriesData.getNumberOfImages( ); i++ )
        {
            LinkedList<boolean[][]> localBooleanLinkedList = new LinkedList<boolean[][]>();
            boolean[][] ovl = new boolean[ m_dicomImageSeriesData.getFileInfo( 0 ).getImageWidth( ) ][ m_dicomImageSeriesData.getFileInfo( 0 ).getImageWidth( ) ];
        
            String fileName = m_dicomImageSeriesData.getFileInfo( i ).getFileName( );
            fileName = fileName + ".iov";
        
            File inputPath = new File( filePath + System.getProperty( "file.separator" ) + "iov"  + System.getProperty( "file.separator" ) );
            int[] iov = new int[ m_dicomImageSeriesData.getFileInfo( 0 ).getImageWidth( ) * m_dicomImageSeriesData.getFileInfo( 0 ).getImageWidth( ) ];
            
            if ( inputPath.exists( ) )
            {
                // if the path exists then try to read integer overlay
                String inputFileName = fileName;
                //fileName = inputFileName + ".iov";
                try
                {
                    try
                    {
                        //  if file alreay exists than delete it before writing to it!
                        File f = new File( inputPath.getAbsolutePath( ) + System.getProperty( "file.separator" ) + fileName );

                        if ( f.exists( ) )
                        {

                            RandomAccessFile reader = new RandomAccessFile( new File( inputPath.getAbsolutePath( )+ System.getProperty( "file.separator" ) + fileName ) , "rw" );
                      
                            // first thing to read is the file is the filename the overlay belongs to
                            String testString;

                            testString = reader.readUTF( );
                            
                            if ( fileName.compareTo( testString ) == 0 )
                            {
                                
                                int imageWidth = reader.readInt( );

                                // load the y dimension (height) of the image
                                int imageHeight = reader.readInt( );
                                
                                int cntr = 0;
                        
                                for ( int j = 0;j < m_imageHeight; j++ )
                                {
                                    for ( int k = 0; k < m_imageWidth; k++ )
                                    {
                                        iov[cntr] = reader.readInt( );
                                        cntr++;
                                    }
                                }
                                reader.close( );
                                
                                // convert to displayable boolean overlay
                                cntr=0;
                                int tst = 0;
                                // Convert int overlay list to boolean overlay list
                                for ( int j = 0; j < imageHeight; j++ )
                                {
                                    for ( int k = 0; k < imageWidth; k++ )
                                    {
                                        if ( iov[cntr] != 0 )
                                        {
                                            ovl[k][j] = true;
                                            tst++;
                                        }
                                        else
                                        {
                                            ovl[k][j] = false;
                                        }
                                        cntr++;
                                    }
                                }
                                System.out.println( "tst (input) = " + tst );
                            }
                            
                            localBooleanLinkedList.add( ovl );
                            m_imageVolumeOfInterestOverlayList.add( localBooleanLinkedList );
                        }
                    }
                
                    catch ( FileNotFoundException e )
                    {
                        e.printStackTrace( );
                    }
                }
                catch( IOException ioe )
                {
                    ioe.printStackTrace( );
                }
            }
        }
        
        m_selectedImageVolumeOfInterestList.clear( );
        m_imageVolumeOfInterestNameList.clear( );
        m_imageVolumeOfInterestColorList.clear( );
        
        numberOfOverlays = 1;
        
        for ( int i = 0; i < numberOfOverlays; i++ )
        {
            m_selectedImageVolumeOfInterestList.add( true );
            m_imageVolumeOfInterestNameList.add( "Volume_" + i );
            m_imageVolumeOfInterestColorList.add( Color.YELLOW );
        }

        // notityListeners that a tissue type was eliminated
        // force an update
        displayDicomImage(m_currentlyDisplayedImageNumber );
        drawImage( false );

    }
    
    /**
     * 
     * Saves all overlays to file. For each image of a slice stack one 
     * overlay file is assumed having extension .ovl
     * 
     */
    private void saveOverlaysToFiles( )
    {
        // Determine path to write overlay files to
        String filePath = m_dicomImageSeriesData.getFileInfo( 0 ).getDirectory( );
        
        File outputPath = new File( filePath + System.getProperty( "file.separator" ) + "ovl" );
        
        // create the directory
        outputPath.mkdir( );
        
        // if the directory exists than writing can start
        if ( outputPath.exists( ) == true )
        {   
        
            for( int i = 0; i < m_numberOfImages; i++ )
            {
                
                String fileName = m_dicomImageSeriesData.getFileInfo( i ).getFileName( );
                
                fileName = fileName + ".ovl";
                try
                {
                    try
                    {
                        //  if file alreay exists than delete it before writing to it!
                        File f = new File( outputPath.getAbsolutePath( )+ System.getProperty( "file.separator" ) + fileName );
                        
                        
                        if ( f.exists( ) )
                        {
                            f.delete( );
                        }
                        
                        RandomAccessFile writer = new RandomAccessFile( new File( outputPath.getAbsolutePath( )+ System.getProperty( "file.separator" ) + fileName ) , "rw" );
                        // first thing written in the file is the filename the overlay belongs to
                        writer.writeUTF( fileName );
                        
                        // write the number of overlays the file contains
                        writer.writeInt( m_imageSegmentsOverlayList.get( i ).size( ) );
                        
                        // write the x dimension (width) of the image
                        writer.writeInt( this.m_imageWidth );
                        
                        // write the y dimension (height) of the image
                        writer.writeInt( this.m_imageHeight );
                        
                        // for ( int k = 0; k < m_imageSegmentsOverlayList.get( i ).size( ); k++ )
                        for ( int k = 0; k < m_selectedTissueTypes.length; k++ )
                        {
                            boolean[][] overLay = m_imageSegmentsOverlayList.get( i ).get( m_selectedTissueTypes[k] );
                            // only write those overlays selected by the user!
                            
                            byte[] overlayAsByteArray = convertToByteArray( m_imageWidth, m_imageHeight, overLay );
                            writer.write( overlayAsByteArray );

                        }
                        
                        writer.close( );
                    }
                    catch ( FileNotFoundException e )
                    {
                        // not handled yet
                    }
                }
            
                catch( IOException ioe )
                {
                    // to be implemented
                }
            }
        }
    }


    /**
     *
     * Saves all overlays to file. For each image of a slice stack one
     * overlay file is assumed having extension .ovl
     *
     */
    private void saveVolumeOfInterestOverlaysToFiles( )
    {
        // Determine path to write overlay files to
        String filePath = m_dicomImageSeriesData.getFileInfo( 0 ).getDirectory( );

        File outputPath = new File( filePath + System.getProperty( "file.separator" ) + "ovl" );

        // create the directory
        outputPath.mkdir( );

        // if the directory exists than writing can start
        if ( outputPath.exists( ) == true )
        {

            for( int i = 0; i < m_numberOfImages; i++ )
            {

                String fileName = m_dicomImageSeriesData.getFileInfo( i ).getFileName( );

                fileName = fileName + ".ovl";
                try
                {
                    try
                    {
                        //  if file alreay exists than delete it before writing to it!
                        File f = new File( outputPath.getAbsolutePath( ) + System.getProperty( "file.separator" ) + fileName );


                        if ( f.exists( ) )
                        {
                            f.delete( );
                        }

                        RandomAccessFile writer = new RandomAccessFile( new File( outputPath.getAbsolutePath( )+ System.getProperty( "file.separator" ) + fileName ) , "rw" );
                        // first thing written in the file is the filename the overlay belongs to
                        writer.writeUTF( fileName );

                        // write the number of overlays the file contains
                        writer.writeInt( m_imageVolumeOfInterestOverlayList.get( i ).size( ) );

                        // write the x dimension (width) of the image
                        writer.writeInt( this.m_imageWidth );

                        // write the y dimension (height) of the image
                        writer.writeInt( this.m_imageHeight );

                        for ( int k = 0; k < m_imageVolumeOfInterestOverlayList.get( i ).size( ); k++ )
                        {
                            
                            boolean[][] overLay = m_imageVolumeOfInterestOverlayList.get( i ).get( k );

                            if ( m_selectedImageVolumeOfInterestList.get( k ).booleanValue( ) == true )
                            {
                                byte[] overlayAsByteArray = convertToByteArray( m_imageWidth, m_imageHeight, overLay );
                                writer.write( overlayAsByteArray );
                            }

                        }

                        writer.close( );
                    }
                    catch ( FileNotFoundException e )
                    {
                        // not handled yet
                    }
                }

                catch( IOException ioe )
                {
                    // to be implemented
                }
            }
        }
    }


    /**
     *
     * Saves all overlays to file. For each image of a slice stack one
     * overlay file is assumed having extension .ovl
     *
     */
    private void saveOverlaysFilesToUserDefinedPath( )
    {
        // Determine path to write overlay files to
        // String filePath = m_dicomImageSeriesData.getFileInfo( 0 ).getDirectory( );

        // File outputPath = new File( filePath + System.getProperty( "file.separator" ) + "ovl" );

        File outputPath = getDirectory( );

        // create the directory
        outputPath.mkdir( );

        // if the directory exists than writing can start
        if ( outputPath.exists( ) == true )
        {

            for( int i = 0; i < m_numberOfImages; i++ )
            {

                String fileName = m_dicomImageSeriesData.getFileInfo( i ).getFileName( );

                fileName = fileName + ".ovl";
                try
                {
                    try
                    {
                        //  if file alreay exists than delete it before writing to it!
                        File f = new File( outputPath.getAbsolutePath( )+ System.getProperty( "file.separator" ) + fileName );


                        if ( f.exists( ) )
                        {
                            f.delete( );
                        }

                        RandomAccessFile writer = new RandomAccessFile( new File( outputPath.getAbsolutePath( )+ System.getProperty( "file.separator" ) + fileName ) , "rw" );
                        // first thing written in the file is the filename the overlay belongs to
                        writer.writeUTF( fileName );

                        // write the number of overlays the file contains
                        writer.writeInt( m_imageSegmentsOverlayList.get( i ).size( ) );

                        // write the x dimension (width) of the image
                        writer.writeInt( this.m_imageWidth );

                        // write the y dimension (height) of the image
                        writer.writeInt( this.m_imageHeight );

                        for ( int k = 0; k < m_imageSegmentsOverlayList.get( i ).size( ); k++ )
                        {
                            boolean[][] overLay = m_imageSegmentsOverlayList.get( i ).get( k );

                            byte[] overlayAsByteArray = convertToByteArray( m_imageWidth, m_imageHeight, overLay );
                            writer.write( overlayAsByteArray );
                        }

                        writer.close( );
                    }
                    catch ( FileNotFoundException e )
                    {
                        // not handled yet
                    }
                }

                catch( IOException ioe )
                {
                    // to be implemented
                }
            }
        }
    }


    /**
     *
     * Saves all overlays to file. For each image of a slice stack one
     * overlay file is assumed having extension .ovl
     *
     */
    private void saveVolumeOfInterestFilesToUserDefinedPath( )
    {
        // Determine path to write overlay files to
        // String filePath = m_dicomImageSeriesData.getFileInfo( 0 ).getDirectory( );

        // File outputPath = new File( filePath + System.getProperty( "file.separator" ) + "ovl" );

        File outputPath = getDirectory( );

        // create the directory
        outputPath.mkdir( );

        // if the directory exists than writing can start
        if ( outputPath.exists( ) == true )
        {

            for( int i = 0; i < m_numberOfImages; i++ )
            {

                String fileName = m_dicomImageSeriesData.getFileInfo( i ).getFileName( );

                fileName = fileName + ".ovl";
                try
                {
                    try
                    {
                        //  if file alreay exists than delete it before writing to it!
                        File f = new File( outputPath.getAbsolutePath( )+ System.getProperty( "file.separator" ) + fileName );


                        if ( f.exists( ) )
                        {
                            f.delete( );
                        }

                        RandomAccessFile writer = new RandomAccessFile( new File( outputPath.getAbsolutePath( )+ System.getProperty( "file.separator" ) + fileName ) , "rw" );
                        // first thing written in the file is the filename the overlay belongs to
                        writer.writeUTF( fileName );

                        // write the number of overlays the file contains
                        writer.writeInt( m_imageSegmentsOverlayList.get( i ).size( ) );

                        // write the x dimension (width) of the image
                        writer.writeInt( this.m_imageWidth );

                        // write the y dimension (height) of the image
                        writer.writeInt( this.m_imageHeight );

                        for ( int k = 0; k < m_imageVolumeOfInterestOverlayList.get( i ).size( ); k++ )
                        {
                            boolean[][] overLay = m_imageVolumeOfInterestOverlayList.get( i ).get( k );

                            byte[] overlayAsByteArray = convertToByteArray( m_imageWidth, m_imageHeight, overLay );
                            writer.write( overlayAsByteArray );
                        }

                        writer.close( );
                    }
                    catch ( FileNotFoundException e )
                    {
                        // not handled yet
                    }
                }

                catch( IOException ioe )
                {
                    // to be implemented
                }
            }
        }
    }


    /**
     * Returns the selected tissue types of the  
     * @return
     */
    public int[] getSelectedTissueTypes( )
    {
        return m_selectedTissueTypes;
    }



    /**
     * Called when user wants to perform the boolena NOT operation on
     * a selected tissue type. Effect one "tissue type" is added
     */
    private void handleBooleanOperationNOT( )
    {

        int selectedTissueType = m_selectedTissueTypes[0];

        // declare a new data structure to store result into
        LinkedList<boolean[][]> newList = new LinkedList<boolean[][]>( );
        int newSize = 0;
        for ( int i = 0; i < m_numberOfImages; i++ )
        {
            // start the boolean logic
            LinkedList<boolean[][]> arrayList = m_imageSegmentsOverlayList.get( i );

            boolean[][] result = new boolean[m_imageWidth][m_imageHeight];

            boolean[][] overlay = arrayList.get( selectedTissueType );

            for( int k = 0; k < m_imageWidth; k++ )
            {
                for ( int l = 0; l < m_imageHeight; l++ )
                {
                    result[k][l] = !overlay[k][l];
                }
            }

            arrayList.add( result );
            newSize = arrayList.size( );
        }

        //m_imageSegmentsOverlayList.add( newList );
        notifyActionListeners( ADDED_NEW_COMPUTED_OVERLAY_EVENT, "" + newSize );
    }

    
    /**
     * Converts the 
     */
    private void handlePromoteCurrentThressholdOverlayToTissueType( )
    {
        //This is only allowed once, in case m_selectedTissueTypes == null
        if ( m_selectedTissueTypes.length == 0 )
        {
            m_selectedTissueTypes = new int[1];
            m_selectedTissueTypes[0] = 0;
            
            int selectedTissueType = m_selectedTissueTypes[0];

            // declare a new data structure to store result into
            LinkedList<boolean[][]> newList = new LinkedList<boolean[][]>( );
            int newSize = 0;
            for ( int i = 0; i < m_numberOfImages; i++ )
            {
                // start the boolean logic
//                LinkedList<boolean[][]> arrayList = m_imageSegmentsOverlayList.get( i );

                boolean[][] result = new boolean[m_imageWidth][m_imageHeight];

//                boolean[][] overlay = arrayList.get( selectedTissueType );

                for( int k = 0; k < m_imageWidth; k++ )
                {
                    for ( int l = 0; l < m_imageHeight; l++ )
                    {
                        result[k][l] = m_thressholdedImageSegmentOverlays[i][k][l];
                    }
                }

                newList.add( result );  
            }
            
            // we have now one tissue type with size 1
            newSize = 1;
            
            m_imageSegmentsOverlayList.add( newList );
            notifyActionListeners( ADDED_NEW_COMPUTED_OVERLAY_EVENT, "" + newSize );
            
        }
    }
    
    
    /**
     * Called when user wants to perform the boolena AND operation on
     * two selected tissue types. Effect one "tissue type" is added
     */
    private void handleBooleanOperationAND( )
    {

        int selectedTissueType_1 = m_selectedTissueTypes[0];
        int selectedTissueType_2 = m_selectedTissueTypes[1];

        // declare a new data structure to store result into
        LinkedList<boolean[][]> newList = new LinkedList<boolean[][]>( );
        int newSize = 0;
        for ( int i = 0; i < m_numberOfImages; i++ )
        {
            // start the boolean logic
            LinkedList<boolean[][]> arrayList = m_imageSegmentsOverlayList.get( i );

            boolean[][] result = new boolean[m_imageWidth][m_imageHeight];

            boolean[][] overlay_1 = arrayList.get( selectedTissueType_1 );
            boolean[][] overlay_2 = arrayList.get( selectedTissueType_2 );

            for( int k = 0; k < m_imageWidth; k++ )
            {
                for ( int l = 0; l < m_imageHeight; l++ )
                {
                    result[k][l] = ( overlay_1[k][l] && overlay_2[k][l] );
                }
            }

            arrayList.add( result );
            newSize = arrayList.size( );
        }

        //m_imageSegmentsOverlayList.add( newList );
        notifyActionListeners( ADDED_NEW_COMPUTED_OVERLAY_EVENT, "" + newSize );
    }

    /**
     * Called when user wants to perform the boolena AND operation on
     * two selected tissue types. Effect one "tissue type" is added
     */
    private void handleBooleanOperationOR( )
    {

        int selectedTissueType_1 = m_selectedTissueTypes[0];
        int selectedTissueType_2 = m_selectedTissueTypes[1];

        // declare a new data structure to store result into
        LinkedList<boolean[][]> newList = new LinkedList<boolean[][]>( );
        int newSize = 0;
        for ( int i = 0; i < m_numberOfImages; i++ )
        {
            // start the boolean logic
            LinkedList<boolean[][]> arrayList = m_imageSegmentsOverlayList.get( i );

            boolean[][] result = new boolean[m_imageWidth][m_imageHeight];

            boolean[][] overlay_1 = arrayList.get( selectedTissueType_1 );
            boolean[][] overlay_2 = arrayList.get( selectedTissueType_2 );

            for( int k = 0; k < m_imageWidth; k++ )
            {
                for ( int l = 0; l < m_imageHeight; l++ )
                {
                    result[k][l] = ( overlay_1[k][l] || overlay_2[k][l] );
                }
            }

            arrayList.add( result );
            newSize = arrayList.size( );
        }

        //m_imageSegmentsOverlayList.add( newList );
        notifyActionListeners( ADDED_NEW_COMPUTED_OVERLAY_EVENT, "" + newSize );
    }

        /**
     * Called when user wants to perform the boolena AND operation on
     * two selected tissue types. Effect one "tissue type" is added
     */
    private void handleBooleanOperationEXOR( )
    {

        int selectedTissueType_1 = m_selectedTissueTypes[0];
        int selectedTissueType_2 = m_selectedTissueTypes[1];

        // declare a new data structure to store result into
        LinkedList<boolean[][]> newList = new LinkedList<boolean[][]>( );
        int newSize = 0;
        for ( int i = 0; i < m_numberOfImages; i++ )
        {
            // start the boolean logic
            LinkedList<boolean[][]> arrayList = m_imageSegmentsOverlayList.get( i );

            boolean[][] result = new boolean[m_imageWidth][m_imageHeight];

            boolean[][] overlay_1 = arrayList.get( selectedTissueType_1 );
            boolean[][] overlay_2 = arrayList.get( selectedTissueType_2 );

            for( int k = 0; k < m_imageWidth; k++ )
            {
                for ( int l = 0; l < m_imageHeight; l++ )
                {
                    // EXOR is OR plus additional check!
                    result[k][l] = ( overlay_1[k][l] || overlay_2[k][l] );

                    // extra check for exclusive or: if both true than false!
                    if ( ( overlay_1[k][l] == true ) && (overlay_2[k][l] == true ) )
                    {
                        result[k][l] = false;
                    }
                }

            }

            arrayList.add( result );
            newSize = arrayList.size( );
        }

        //m_imageSegmentsOverlayList.add( newList );
        notifyActionListeners( ADDED_NEW_COMPUTED_OVERLAY_EVENT, "" + newSize );
    }



    /**
     * Called when user wants to perform the boolena NOT operation on
     * a selected tissue type. Effect one "tissue type" is added
     */
    private void handleVolumeOfInterestBooleanOperationNOT( int[] selectedVolumes )
    {
        
        int selectedOverlayVolume = selectedVolumes[0];
        
        // declare a new data structure to store result into
        LinkedList<boolean[][]> newList = new LinkedList<boolean[][]>( );
        int newSize = 0;
        for ( int i = 0; i < m_numberOfImages; i++ )
        {
            // start the boolean logic
            LinkedList<boolean[][]> arrayList = m_imageVolumeOfInterestOverlayList.get( i );
            
            boolean[][] result = new boolean[m_imageWidth][m_imageHeight];
                
            boolean[][] overlay = arrayList.get( selectedOverlayVolume );
                
            for( int k = 0; k < m_imageWidth; k++ )
            {
                for ( int l = 0; l < m_imageHeight; l++ )
                {
                    result[k][l] = !overlay[k][l];
                }
            }
                
            arrayList.add( result );
            newSize = arrayList.size( );
        }

        m_selectedImageVolumeOfInterestList.add( new Boolean(true ) );
        m_imageVolumeOfInterestNameList.add( "NOT of volume " + selectedOverlayVolume );
        m_imageVolumeOfInterestColorList.add( defaultColors[ m_selectedImageVolumeOfInterestList.size( ) ] );
        
        if ( m_voi_ManipulationFrame != null )
        {
            m_voi_ManipulationFrame.setDicomSliceCanvasPolyDraw( this );
        }
        
        // m_imageSegmentsOverlayList.add( newList );
        // notifyActionListeners( ADDED_NEW_COMPUTED_OVERLAY_EVENT, "" + newSize );
    }
    
    /**
     * Called when user wants to perform the boolena AND operation on
     * two selected tissue types. Effect one "tissue type" is added
     */
    private void handleVolumeOfInterestBooleanOperationAND( int[] selectedVolumes )
    {
        
        int selectedOverlayVolume_1 = selectedVolumes[0];
        int selectedOverlayVolume_2 = selectedVolumes[1];
        
        // declare a new data structure to store result into
        LinkedList<boolean[][]> newList = new LinkedList<boolean[][]>( );
        int newSize = 0;
        for ( int i = 0; i < m_numberOfImages; i++ )
        {
            // start the boolean logic
            LinkedList<boolean[][]> arrayList = m_imageVolumeOfInterestOverlayList.get( i );
            
            boolean[][] result = new boolean[m_imageWidth][m_imageHeight];
                
            boolean[][] overlay_1 = arrayList.get( selectedOverlayVolume_1 );
            boolean[][] overlay_2 = arrayList.get( selectedOverlayVolume_2 );
                
            for( int k = 0; k < m_imageWidth; k++ )
            {
                for ( int l = 0; l < m_imageHeight; l++ )
                {
                    result[k][l] = ( overlay_1[k][l] && overlay_2[k][l] );
                }
            }
                
            arrayList.add( result );
            newSize = arrayList.size( );
        }

        m_selectedImageVolumeOfInterestList.add( new Boolean(true ) );
        m_imageVolumeOfInterestNameList.add( "AND of volume " + selectedOverlayVolume_1 + " and " + selectedOverlayVolume_2 );
        m_imageVolumeOfInterestColorList.add( defaultColors[ m_selectedImageVolumeOfInterestList.size( ) ] );

        if ( m_voi_ManipulationFrame != null )
        {
            m_voi_ManipulationFrame.setDicomSliceCanvasPolyDraw( this );
        }

    }
    
    /**
     * Called when user wants to perform the boolena AND operation on
     * two selected tissue types. Effect one "tissue type" is added
     */
    private void handleVolumeOfInterestBooleanOperationOR( int[] selectedVolumes )
    {
        
        int selectedOverlayVolume_1 = selectedVolumes[0];
        int selectedOverlayVolume_2 = selectedVolumes[1];
        
        // declare a new data structure to store result into
        LinkedList<boolean[][]> newList = new LinkedList<boolean[][]>( );
        int newSize = 0;
        for ( int i = 0; i < m_numberOfImages; i++ )
        {
            // start the boolean logic
            LinkedList<boolean[][]> arrayList = m_imageVolumeOfInterestOverlayList.get( i );
            
            boolean[][] result = new boolean[m_imageWidth][m_imageHeight];
                
            boolean[][] overlay_1 = arrayList.get( selectedOverlayVolume_1 );
            boolean[][] overlay_2 = arrayList.get( selectedOverlayVolume_2 );
                
            for( int k = 0; k < m_imageWidth; k++ )
            {
                for ( int l = 0; l < m_imageHeight; l++ )
                {
                    result[k][l] = ( overlay_1[k][l] || overlay_2[k][l] );
                }
            }
                
            arrayList.add( result );
            newSize = arrayList.size( );
        }

        m_selectedImageVolumeOfInterestList.add( new Boolean(true ) );
        m_imageVolumeOfInterestNameList.add( "OR of volume " + selectedOverlayVolume_1 + " and " + selectedOverlayVolume_2 );
        m_imageVolumeOfInterestColorList.add( defaultColors[ m_selectedImageVolumeOfInterestList.size( ) ] );

        if ( m_voi_ManipulationFrame != null )
        {
            m_voi_ManipulationFrame.setDicomSliceCanvasPolyDraw( this );
        }
    }

        /**
     * Called when user wants to perform the boolena AND operation on
     * two selected tissue types. Effect one "tissue type" is added
     */
    private void handleVolumeOfInterestBooleanOperationEXOR( int[] selectedVolumes )
    {
        
        int selectedOverlayVolume_1 = selectedVolumes[0];
        int selectedOverlayVolume_2 = selectedVolumes[1];
        
        // declare a new data structure to store result into
        LinkedList<boolean[][]> newList = new LinkedList<boolean[][]>( );
        int newSize = 0;
        for ( int i = 0; i < m_numberOfImages; i++ )
        {
            // start the boolean logic
            LinkedList<boolean[][]> arrayList = m_imageVolumeOfInterestOverlayList.get( i );
            
            boolean[][] result = new boolean[m_imageWidth][m_imageHeight];
                
            boolean[][] overlay_1 = arrayList.get( selectedOverlayVolume_1 );
            boolean[][] overlay_2 = arrayList.get( selectedOverlayVolume_2 );
                
            for( int k = 0; k < m_imageWidth; k++ )
            {
                for ( int l = 0; l < m_imageHeight; l++ )
                {
                    // EXOR is OR plus additional check!
                    result[k][l] = ( overlay_1[k][l] || overlay_2[k][l] );
                    
                    // extra check for exclusive or: if both true than false!
                    if ( ( overlay_1[k][l] == true ) && (overlay_2[k][l] == true ) )
                    {
                        result[k][l] = false;
                    }
                }
                
            }
                
            arrayList.add( result );
            newSize = arrayList.size( );
        }

        m_selectedImageVolumeOfInterestList.add( new Boolean( true ) );
        m_imageVolumeOfInterestNameList.add( "EXOR of volume " + selectedOverlayVolume_1 + " and " + selectedOverlayVolume_2 );
        m_imageVolumeOfInterestColorList.add( defaultColors[ m_selectedImageVolumeOfInterestList.size( ) ] );

        if ( m_voi_ManipulationFrame != null )
        {
            m_voi_ManipulationFrame.setDicomSliceCanvasPolyDraw( this );
        }
    }
    
    
    /**
     * Eliminates those overlay pixels which are within the selected ROI.
     * Operation is only performed on selected overlay types.
     */
    public void eliminateOverlayPixelsOfSelectedOverlaysInsideCurrentlySelectedROI( )
    {
        if ( m_selectedTissueTypes.length > 0 )
        {
            LinkedList<boolean[][]> allOverlaysOfCurrentlyDisplayedImage = m_imageSegmentsOverlayList.get(m_currentlyDisplayedImageNumber );
            
            // iterate over all tissue types which are selected
            for ( int i = 0 ; i < m_selectedTissueTypes.length ; i++ )
            {
                boolean[][] overlay = allOverlaysOfCurrentlyDisplayedImage.get( m_selectedTissueTypes[i] );
                
                // now compute the logic AND between overlay pixels and currently selected overlay
                RegioPolygon p = new RegioPolygon( this.m_selectedPolygon );
                p.setScalingFactor( (float) ( 1.0 / m_zoom ) );
                
                for( int k = 0; k < m_imageWidth; k++ )
                {
                    for( int l = 0; l < m_imageWidth; l++ )
                    {
                        if ( p.contains( k, l ) && ( overlay[k][l]) == true )
                        {
                            overlay[k][l] = false;
                        }
                    }
                }
            }
            
            displayDicomImage(m_currentlyDisplayedImageNumber );
            drawImage( false );    
        }
    }
    
    /**
     * Eliminates those overlay pixels which are poutside the selected ROI.
     * Operation is only performed on selected overlay types.
     */
    public void eliminateOverlayPixelsOfSelectedOverlaysOutsideCurrentlySelectedROI( )
    {
        
        if ( m_selectedTissueTypes.length > 0 )
        {
            LinkedList<boolean[][]> allOverlaysOfCurrentlyDisplayedImage = m_imageSegmentsOverlayList.get(m_currentlyDisplayedImageNumber );
            // iterate over all tissue types which are selected
            for ( int i = 0 ; i < m_selectedTissueTypes.length ; i++ )
            {
                boolean[][] overlay = allOverlaysOfCurrentlyDisplayedImage.get( m_selectedTissueTypes[i] );
                
                // now compute the logic AND between overlay pixels and currently selected overlay
                RegioPolygon p = new RegioPolygon( this.m_selectedPolygon );
                p.setScalingFactor( (float) (1.0 / m_zoom ) );
                for( int k = 0; k < m_imageWidth; k++ )
                {
                    for( int l = 0; l < m_imageWidth; l++ )
                    {
                        if ( !p.contains( k, l ) && ( overlay[k][l]) == true )
                        {
                            overlay[k][l] = false;
                        }
                    }
                }
            }
            
            displayDicomImage(m_currentlyDisplayedImageNumber );
            drawImage( false );
            
        }
        
    }
    
   
    /**
     * Loads the user defined regions in the normal (non relative) image group
     */
    public LinkedList< RegioPolygon > loadRegionsFromFile( boolean ofCurrentlyDisplayImageOnly, boolean doLoad ) 
    {
        
        LinkedList< RegioPolygon > additionallyLoadedPolys = null;
        
        if ( ofCurrentlyDisplayImageOnly == true )
        {
            additionallyLoadedPolys = loadRegions( m_currentlyDisplayedImageNumber, doLoad );
        }
        else
        {
            for ( int i = 0; i < m_numberOfImages; i++ )
            {
                loadRegions( i, doLoad );
            }
        }
        
        return additionallyLoadedPolys;
    }
    
    
    /**
     * 
     * Writes the regions belonging to image number i to file
     * 
     * @param imageNumber
     */
    private LinkedList< RegioPolygon > loadRegions( int imageNumber, boolean doLoad )
    {
        // Determine path to load the overlay files from
        String filePath = m_dicomImageSeriesData.getFileInfo(0).getDirectory( );
        
        LinkedList< RegioPolygon > polysAdditionallyLoaded = null;
        
        File outputPath = new File( filePath + System.getProperty( "file.separator" ) + "reg" );
        
        if ( outputPath.exists( ) == false )
        {    
            Dialogs.messageBox( m_parent, "No region files defined!", "" + outputPath, MessageBox.MB_OK ); 
        }
        else
        {
            polysAdditionallyLoaded = importPolysFromFile( imageNumber, outputPath.getAbsolutePath( ), doLoad );
        }
        
        return polysAdditionallyLoaded;
        
    }
    
    
    /**
     * Imports a polygon from a specially defined polygon file
     */
    public LinkedList< RegioPolygon > importPolysFromFile( int imageNumber, String path, boolean doLoad ) 
    {
	
        String fileName = m_dicomImageSeriesData.getFileInfo( imageNumber ).getFileName( );
        
        int i = fileName.lastIndexOf( "." );
        if (i == -1) i = fileName.length( );
        String nameRoot = fileName.substring( 0, i );
        String regName = nameRoot + ".reg";
        
        fileName = path + System.getProperty( "file.separator" )+ regName;
        
        LinkedList< RegioPolygon > currentPolygons = new LinkedList< RegioPolygon >( );
        
        if ( doLoad == true )
        {    
            currentPolygons = m_polygonsVector.get( imageNumber );
        }
        
        File refFileName = new File( fileName );

        if ( refFileName.exists( ) == true )
        {
            currentPoly = null;
            RegioPolygon p;
            RegFile f = new RegFile( fileName, m_imageWidth, m_imageHeight );
            f.open( "r" );
            f.readHeader( );
            while ( null != ( p = f.readPolygonData( m_zoom ) ) )
            {
                currentPolygons.add( p );

                // note the last loaded polygon and slice number
                m_lastLoadedPolygon = new RegioPolygon( p );
                m_lastLoadedPolygonSliceNumber = imageNumber;

                notifyActionListeners( POLYGON_FROM_FILE_LOAD_EVENT, POLYGON_FROM_FILE_LOAD_CMD );

                m_copyRegioPolygon.setEnabled( true );
                m_pixelDumpSelectedRegioPolygon.setEnabled( true );
                m_affineTransformationRegioPolygon.setEnabled( true );
                m_scaleRegioPolygon.setEnabled( true );
                m_showHistogramOfSelectedRegioPolygon.setEnabled( true );
                m_renameRegion.setEnabled( true );
            }
            f.close( );
            repaint( );
        
            // If there exists regio polygons for this slice then changes
            // of the context popupmenu must occur
            if ( currentPolygons.size( ) == 1 )
            {
                m_copyRegioPolygon.setEnabled( true );
                m_pixelDumpSelectedRegioPolygon.setEnabled( true );
                m_affineTransformationRegioPolygon.setEnabled( true );
                m_scaleRegioPolygon.setEnabled( true );
                setMode( SELECT_MODE );
            }
            if ( currentPolygons.size( ) > 1 )
            {
                m_copyRegioPolygon.setEnabled( true );
                m_pixelDumpSelectedRegioPolygon.setEnabled( true );
                m_affineTransformationRegioPolygon.setEnabled( true );
                m_scaleRegioPolygon.setEnabled( true );
                m_copyAllRegioPolygons.setEnabled( true );
                setMode( SELECT_MODE );
            } 
        }
        
        return currentPolygons;
    }

    /**
     * Returns the polgon which was last loaded
     * @return
     */
    public RegioPolygon getLastLoadedPolygon( )
    {
        return m_lastLoadedPolygon;
    }

    public int getLastLoadedPolygonSliceNumber( )
    {
        return m_lastLoadedPolygonSliceNumber;
    }

    /**
     * 
     * Removes all selected distances from list and from memory
     * 
     * @param sliceNumbers
     * @param names
     */
    public void deleteSelectedDistances( Vector<Integer> sliceNumbers, Vector<String> names )
    {
        for ( int i = 0 ; i < sliceNumbers.size( ); i++ )
        {
            LinkedList<Distance> list = m_distancesVector.get( sliceNumbers.get( i ) );

            for ( int j = 0; j < list.size( ); j++ )
            {
                if ( list.get( j ).getDistanceName( ).compareTo( names.get( i ) ) == 0 )
                {
                    list.remove( j );
                    break;
                }
            }
        }

        repaint( );

    }

    /**
     * Loads the user defined distances in the normal (non relative) image group
     */
    public void loadDistanceFromFile( boolean ofCurrentlyDisplayImageOnly )
    {

        if ( ofCurrentlyDisplayImageOnly == true )
        {
            loadDistances(m_currentlyDisplayedImageNumber );
        }
        else
        {
            for ( int i = 0; i < m_numberOfImages; i++ )
            {
                loadDistances( i );
            }
        }
    }


    /**
     *
     * Writes the regions belonging to image number i to file
     *
     * @param imageNumber
     */
    private void loadDistances( int imageNumber )
    {
        // Determine path to load the overlay files from
        String filePath = m_dicomImageSeriesData.getFileInfo(0).getDirectory( );

        File outputPath = new File( filePath + System.getProperty( "file.separator" ) + "dst" );

        if ( outputPath.exists( ) == false )
        {
            Dialogs.messageBox( m_parent, "No distances files defined!", "" + outputPath, MessageBox.MB_OK );
        }
        else
        {
            importDistancesFromFile( imageNumber, outputPath.getAbsolutePath( ) );
        }
    }

    /**
     * Imports a polygon from a specially defined polygon file
     */
    public void importDistancesFromFile( int imageNumber, String path )
    {

        String fileName = m_dicomImageSeriesData.getFileInfo( imageNumber ).getFileName( );

        int i = fileName.lastIndexOf( "." );
        if (i == -1) i = fileName.length( );
        String nameRoot = fileName.substring( 0, i );
        String regName = nameRoot + ".dst";

        fileName = path + System.getProperty( "file.separator" )+ regName;

        LinkedList< Distance > currentDistances = m_distancesVector.get( imageNumber );

        Distance dst;

        MeasuredDistancesFile f = new MeasuredDistancesFile( fileName, m_imageWidth, m_imageHeight,  fileName );
        f.open( "r" );
        f.readHeader( );

        while ( null != ( dst = f.readDistanceData( m_zoom ) ) )
        {
            currentDistances.add( dst );
        }
        
        m_displayMeasureDistanceMode = true;

        f.close( );
        repaint( );
    }

    /**
     * Saves the user defined distances
     */
    public void saveDistancesToFiles( boolean ofCurrentlyDisplayImageOnly )
    {
        if ( m_measureDistancesFrame != null )
        {
            m_measureDistancesFrame.setAlwaysOnTop( false );
        }
        
        if ( ofCurrentlyDisplayImageOnly == true )
        {
            saveDistances(m_currentlyDisplayedImageNumber );
        }
        else
        {
            for ( int i = 0; i < m_numberOfImages; i++ )
            {
                saveDistances( i );
            }
        }
        
        if ( m_measureDistancesFrame != null )
        {
            m_measureDistancesFrame.setAlwaysOnTop( true );
        }
    }

    /**
     * Creates a human readable file containing all distance information
     */
    private void createReportFileOfAllDistances( )
    {
        if ( m_dicomImageSeriesData != null )
        {
            
            String fileName = m_dicomImageSeriesData.getFileInfo( 0 ).getFileName( );

            // Determine path to load the overlay files from
            String filePath = m_dicomImageSeriesData.getFileInfo( 0 ).getDirectory( );

            File outputPath = new File( filePath + System.getProperty( "file.separator" ) + "dst" );
            if ( outputPath.exists( ) == false )
            {
                outputPath.mkdir( );
            }

            // save to dst file
            // String fileName = m_currentView.getTitle( );
            int i = fileName.lastIndexOf( "." );
            if (i == -1) i = fileName.length( );
            String nameRoot = fileName.substring( 0, i );
            String dstName = nameRoot + "_all_images.dst";

            // check if such a file already exists
            if ( FileUtils.fileExists( outputPath.getAbsolutePath( ) + System.getProperty( "file.separator" ) + dstName ) )
            {
                QuestionDialog qDlg = new QuestionDialog( m_parent );
                if (! qDlg.ask( "File exists. Ok to overwrite?" ) ) return;
                FileUtils.fileDelete( outputPath.getAbsolutePath( ) + System.getProperty( "file.separator" ) + dstName  );
            }

            MeasuredDistancesFile f = new MeasuredDistancesFile( ( outputPath.getAbsolutePath( ) + System.getProperty( "file.separator" ) + dstName ),
                                                                 m_imageWidth, m_imageHeight, filePath + fileName );

            if ( f.open( "rw" ) )
            {
                for ( int ii = 0; ii < this.m_numberOfImages; ii++ )
                {
                    LinkedList<Distance> dsts = m_distancesVector.get( ii );

                    for ( int k = 0; k < dsts.size( ); k++ )
                    {
                        f.writeDistanceDataOnOneLine( dsts.get( k ), m_zoom );
                    }
                }
                boolean state = f.finish( );

                Globals globs = Globals.getInstance( );

                if ( globs.getConfirmAfterSave( ) )
                {
                    if ( state )
                    {
                        Dialogs.messageBox( m_parent, "Succesfully saved", "Save distances", MessageBox.MB_OK );
                    }
                    else
                    {
                        Dialogs.messageBox( m_parent, "Save failed", "Save distances", MessageBox.MB_OK );
                    }
                }
            }     
        }
    }
    
    /**
     *
     * Writes the distances belonging to image number i to file
     *
     * @param imageNumber
     */
    private void saveDistances( int imageNumber )
    {
        if ( m_dicomImageSeriesData != null )
        {
            if ( ( imageNumber >= 0 ) && ( imageNumber < m_numberOfImages ) )
            {
                String fileName = m_dicomImageSeriesData.getFileInfo( imageNumber ).getFileName( );

                // Determine path to load the overlay files from
                String filePath = m_dicomImageSeriesData.getFileInfo( 0 ).getDirectory( );

                File outputPath = new File( filePath + System.getProperty( "file.separator" ) + "dst" );
                if ( outputPath.exists( ) == false )
                {
                    outputPath.mkdir( );
                }

                // save to dst file
                // String fileName = m_currentView.getTitle( );
                int i = fileName.lastIndexOf( "." );
                if (i == -1) i = fileName.length( );
                String nameRoot = fileName.substring( 0, i );
                String dstName = nameRoot + ".dst";

                // check if such a file already exists
                if ( FileUtils.fileExists( outputPath.getAbsolutePath( ) + System.getProperty( "file.separator" ) + dstName ) )
                {
                    QuestionDialog qDlg = new QuestionDialog( m_parent );
                    qDlg.setAlwaysOnTop( true );
                    if (! qDlg.ask( "File exists. Ok to overwrite?" ) ) return;
                    FileUtils.fileDelete( outputPath.getAbsolutePath( ) + System.getProperty( "file.separator" ) + dstName  );
                }

                MeasuredDistancesFile f = new MeasuredDistancesFile( ( outputPath.getAbsolutePath( ) + System.getProperty( "file.separator" ) + dstName ),
                                                                     m_imageWidth, m_imageHeight, filePath + fileName );

                if ( f.open( "rw" ) )
                {
                    f.writeHeader( );
                    LinkedList<Distance> dsts = m_distancesVector.get( imageNumber );

                    for ( int k = 0; k < dsts.size( ); k++ )
                    {
                        if ( dsts.get( k ).getImageNumber( ) == imageNumber )
                        {
                            f.writeDistanceData( dsts.get( k ), m_zoom );
                        }
                    }

                    boolean state = f.finish( );

                    Globals globs = Globals.getInstance( );
                    if ( globs.getConfirmAfterSave( ) )
                    {
                        if ( state )
                        {
                            Dialogs.messageBox( m_parent, "Succesfully saved", "Save distances", MessageBox.MB_OK );
                        }
                        else
                        {
                            Dialogs.messageBox( m_parent, "Save failed", "Save distances", MessageBox.MB_OK );
                        }
                    }
                }
            }
        }
        else
        {
            Dialogs.messageBox( m_parent, "Cannot save distances. Contact developer " );
        }
    }




    /**
     * Saves the user defined regions in the normal (non relative) image group
     */
    public void saveRegionsToFile( boolean ofCurrentlyDisplayImageOnly ) 
    {
        
        if ( ofCurrentlyDisplayImageOnly == true )
        {
            saveRegions(m_currentlyDisplayedImageNumber );
        }
        else
        {
            for ( int i = 0; i < m_numberOfImages; i++ )
            {
                saveRegions( i );
            }
        }
    }



    /**
     * Asks the user to overwrite the already existing files.
     * @return
     */
    private boolean showOverWriteQuestionDialog( )
    {

        boolean choice = false;
        JOptionPane pane = new JOptionPane( "Regio file already exists!\nOverwrite file?" );
        Object[] options = new String[] { "Yes", "No" };
        pane.setOptions( options );
        JDialog dialog = pane.createDialog( new JFrame( ), "Overwrite regio file?" );
        dialog.show( );
        String obj = (String)pane.getValue( );
        
        if  ( obj.compareTo( "Yes" ) == 0 )
        {
            choice = true;
        }

        return choice;
    }

    /**
     * Asks the user to overwrite the already existing files.
     * @return
     */
    private boolean showOverWriteAllQuestionDialog( )
    {

        boolean choice = false;
        JOptionPane pane = new JOptionPane( "Overwite all regio files?" );
        Object[] options = new String[] { "Yes", "No" };
        pane.setOptions( options );
        JDialog dialog = pane.createDialog( new JFrame( ), "Overwrite regio file?" );
        dialog.show( );
        String obj = (String)pane.getValue( );

        if  ( obj.compareTo( "Yes" ) == 0 )
        {
            choice = true;
        }
        
        m_alreadyAskedToOverwriteAllRegioFiles = true;
        
        return choice;
    }


    
    /**
     * 
     * Writes the regions belonging to image number i to file
     * 
     * @param imageNumber
     */
    private void saveRegions( int imageNumber )
    {
        
        if ( ( imageNumber >= 0 ) && ( imageNumber < m_numberOfImages ) )
        {
            String fileName = m_dicomImageSeriesData.getFileInfo( imageNumber ).getFileName( );
            
            
            // Determine path to load the overlay files from
            String filePath = m_dicomImageSeriesData.getFileInfo(0).getDirectory( );
        
            File outputPath = new File( filePath + System.getProperty( "file.separator" ) + "reg" );
            if ( outputPath.exists( ) == false )
            {    
                outputPath.mkdir( );
            }                
             
            // save to reg file
            // String fileName = m_currentView.getTitle( );
            int i = fileName.lastIndexOf( "." );
            if (i == -1) i = fileName.length( );
            String nameRoot = fileName.substring( 0, i );
            String regName = nameRoot + ".reg";
            
            // check if such a file already exists
            if ( FileUtils.fileExists( outputPath.getAbsolutePath( ) + System.getProperty( "file.separator" ) + regName ) ) 
            {
                //QuestionDialog qDlg = new QuestionDialog( m_parent );

                //if (! qDlg.ask( "File exists. Ok to overwrite?" ) ) return;

                if ( m_alreadyAskedToOverwriteAllRegioFiles == false )
                {
                    if ( showOverWriteQuestionDialog( ) == false )
                    {
                        return;
                    }

                    // FileUtils.fileDelete( outputPath.getAbsolutePath( ) + System.getProperty( "file.separator" ) + regName  );

                    if ( m_numberOfImages > 1 )
                    {
                        m_overwriteAllRegioFiles = showOverWriteAllQuestionDialog( );
                    }
                }
            }

            if ( m_overwriteAllRegioFiles == true )
            {
                FileUtils.fileDelete( outputPath.getAbsolutePath( ) + System.getProperty( "file.separator" ) + regName  );
            }

            RegFile f = new RegFile( ( outputPath.getAbsolutePath( ) + System.getProperty( "file.separator" ) + regName ) , m_imageWidth, m_imageHeight );
            
            if ( f.open( "rw" ) ) 
            {
                f.writeHeader( );
                for ( int k = 0; k < getNumberOfPolygons( imageNumber ); k++ ) 
                {
                    f.writePolygonData( getPolygon( imageNumber, k ), m_zoom );
                }
                boolean state = f.finish( );
                Globals globs = Globals.getInstance( );

                if ( globs.getConfirmAfterSave( ) )
                {
                    if ( state )
                    { 
                        Dialogs.messageBox( m_parent, "Succesfully saved", "Save ROIs", MessageBox.MB_OK ); 
                    }
                    else 
                    {
                        Dialogs.messageBox( m_parent, "Save failed", "Save ROIs", MessageBox.MB_OK );
                    }
                }
            }
        }
    }
    
    /**
     * 
     * Writes a report file to hard disc in the directory ../rap
     * .rap files are created
     * @param condensed
     * @param allSegmentTypes
     */
    private void createReportFiles( boolean condensed )
    { 
        if ( condensed == true )
        {
            createCondensedReportFileForCurrentlySelectedSegmentTypesOnly( );
        }
        
        if ( condensed == false  )
        {
            createDetailedReportFileForCurrentlySelectedSegmentTypesOnly( );
        }
    }
    
      
    /**
     * Creates one report file per image containing information on all selected 
     * segments
     */
    private void createDetailedReportFileForCurrentlySelectedSegmentTypesOnly( )
    {

        boolean giveWarning = false;
        FileInfo finfo = m_dicomImageSeriesData.getFileInfo( 0 );
        
        double volumePerPixel = new Double( finfo.getPixelDepth( ) ).doubleValue( ) * 
                                new Double( finfo.getPixelHeight( ) ).doubleValue( ) *
                                new Double( finfo.getPixelWidth( ) ).doubleValue( );

        if ( volumePerPixel == 0.0 )
        {
            System.out.println( "!! Warning : volume per pixel in DICOM header is zero (inconsistent, illegal, on non complete DICOM file!) " );
            System.out.println( "!! Automatically set volume per pixel to 1.0 : CORRECT volume for true volume-per-pixel value!" );
            Dialogs.messageBox( m_parent, "Warning : volume per pixel in DICOM header is zero (inconsistent, illegal, on non complete DICOM file!) " );
            volumePerPixel = 1.0;
            giveWarning = true;
        }

        //for ( int i = 0 ; i < m_numberOfImages ; i++ )
        for ( int i = m_minimumImageNumberToFocus ; i < this.m_maximumImageNumberToFocus ; i++ )
        {

            LinkedList<ImageSegmentProperties> segmentProperties = computeStatisticalPropertiesOfImageSegmentsOfImageNumber( i );

            LinkedList<boolean[][]> colorOverlays = m_imageSegmentsOverlayList.get( i );
            
            finfo = m_dicomImageSeriesData.getFileInfo( i );
            String fileName = finfo.getFileName( );
            
            String patientName = finfo.getPatientName( );
            String examinationDate = finfo.getAcquisitionDate( );

            // save to reg file
            // String fileName = m_currentView.getTitle( );
            int ii = fileName.lastIndexOf( "." );
            if (ii == -1) ii = fileName.length( );
            String nameRoot = fileName.substring( 0, ii );
            String reportName = nameRoot + ".rep";
            
            // Determine path to load the overlay files from
            String filePath = m_dicomImageSeriesData.getFileInfo(0).getDirectory( );
        
            File outputPath = new File( filePath + System.getProperty( "file.separator" ) + "rep" );
            if ( outputPath.exists( ) == false )
            {    
                outputPath.mkdir( );
            }
            try
            {
                TextFileWriter writer = new TextFileWriter( new FileWriter( outputPath + System.getProperty( "file.separator" ) + reportName ) );
            
                writer.writeStringLn( "Report of        : " + fileName );
                writer.writeStringLn( "Patient name     : " + patientName );
                writer.writeStringLn( "Examination date : " + examinationDate );
                writer.writeStringLn( "Pixel depth      : " + finfo.getPixelDepth( ) + "mm  | pixel width : " + finfo.getPixelWidth( ) + "mm  | pixel heigth : " + finfo.getPixelHeight( ) + "mm" );

                if ( giveWarning == true )
                {
                    writer.writeStringLn( "" );
                    writer.writeStringLn( "!! Warning : volume per pixel in DICOM header is zero (inconsistent, illegal, on non complete DICOM file!) " );
                    writer.writeStringLn( "!! Automatically set volume per pixel to 1.0 : Manually CORRECT volume for TRUE volume-per-pixel value!" );
                    writer.writeStringLn( "" );
                }

                writer.writeStringLn( "--------------------------------" );
                for ( int j = 0; j < m_selectedTissueTypes.length; j++ )
                {
                    boolean[][] imSegment = colorOverlays.get( m_selectedTissueTypes[j] );
                
                    int numberOfPixels = 0;
                    for ( int k = 0; k < m_imageWidth ; k++ )
                    {
                        for ( int l = 0; l < m_imageHeight; l++ )
                        {
                            if ( imSegment[k][l] == true )
                            {
                                numberOfPixels++;
                            }
                        }
                    }
                    
                    writer.writeString( "Segment_number : " + j + "\t");
                    writer.writeString( "Number_of_pixels : " + numberOfPixels + "\t");
                    writer.writeString( "Volume_: \t" + ( numberOfPixels * volumePerPixel / 1000.0 ) + "\t cm^3\t" );
                    writer.writeString( "Average_pixel_value_:\t" + segmentProperties.get( j ).getMeanPixelValue( ) + "\t" );
                    writer.writeStringLn( "Standard_deviation_pixel_value:\t" + segmentProperties.get( j ).getStandardDeviationOfPixelValues( ) );

                }
        
                writer.flush( );
                writer.close( );
            }
            catch ( IOException e )
            {
                e.printStackTrace( );
            }
        }
    }
    
    /**
     * Creates one report file per image containing information on all selected 
     * segments
     */
    private void createCondensedReportFileForCurrentlySelectedSegmentTypesOnly( )
    {
        
        boolean giveWarning = false;

        double[] grandVolume;
        double[] averagePixelValuesPerSegmentType = new double[m_selectedTissueTypes.length];

        for ( int i = 0 ; i < m_selectedTissueTypes.length; i++ )
        {
            averagePixelValuesPerSegmentType[i] = 0f;
        }


        if ( m_selectedTissueTypes != null )
        {
            grandVolume = new double[m_selectedTissueTypes.length];
        }
        else
        {
            grandVolume = new double[0];
        }
        
        FileInfo finfo2 = m_dicomImageSeriesData.getFileInfo( 0 );
        String fileName2 = finfo2.getFileName( );
        String patientName = finfo2.getPatientName( );
        String examinationDate = finfo2.getAcquisitionDate( );
        String seriesDescription = finfo2.getSeriesDescription( );
        
        double volumePerPixel = new Double( finfo2.getPixelDepth( ) ).doubleValue( ) * new Double( finfo2.getPixelHeight( ) ).doubleValue( ) * new Double ( finfo2.getPixelWidth( ) ).doubleValue( );

        if ( volumePerPixel == 0.0 )
        {
            System.out.println( "Warning : volume per pixel in DICOM header is zero (inconsistent, illegal, on non complete DICOM file!) " );
            System.out.println( "Automatically set volume per pixel to 1.0 : CORRECT volume for true volume-per-pixel value!" );
            Dialogs.messageBox( m_parent, "Warning : volume per pixel in DICOM header is zero (inconsistent, illegal, on non complete DICOM file!) " );
            volumePerPixel = 1.0;
            giveWarning = true;
        }

        double volumeInCurrentImage = 0.0;
        
        // save to reg file
        // String fileName = m_currentView.getTitle( );
        int ii = fileName2.lastIndexOf( "." );
        if (ii == -1) ii = fileName2.length( );
        String nameRoot = fileName2.substring( 0, ii );
        String reportName = patientName + "_" + seriesDescription + "_" + nameRoot + ".vol";
        
        String filePath = m_dicomImageSeriesData.getFileInfo( 0 ).getDirectory( );
        
        File outputPath = new File( filePath + System.getProperty( "file.separator" ) + "rep" );
        
        if ( outputPath.exists( ) == false )
        {    
            outputPath.mkdir( );
        }
        
        try
        {
            TextFileWriter writer = new TextFileWriter( new FileWriter( outputPath + System.getProperty( "file.separator" ) + reportName ) );
            
            writer.writeStringLn( "Report of file   : " + fileName2 );
            writer.writeStringLn( "Patient name     : " + patientName );
            writer.writeStringLn( "Examination date : " + examinationDate );
            writer.writeStringLn( "Slice thickness : " + finfo2.getSliceThickness( ) + "mm  | pixel width : " + finfo2.getPixelWidth( ) + "mm  | pixel heigth : " + finfo2.getPixelHeight( ) + "mm" );
            
            writer.writeStringLn( "Series description : " + seriesDescription );
            if ( giveWarning == true )
            {
                writer.writeStringLn( "" );
                writer.writeStringLn( "!! Warning : volume per pixel in DICOM header is zero (inconsistent, illegal, on non complete DICOM file!) " );
                writer.writeStringLn( "!! Automatically set volume per pixel to 1.0 : Manually CORRECT volume for TRUE volume-per-pixel value!" );
                writer.writeStringLn( "" );
            }

            writer.writeStringLn( "--------------------------------" );
            
            int[] numberOfNonZeroSlices;
            
            if ( m_selectedTissueTypes != null )
            {
                numberOfNonZeroSlices = new int[m_selectedTissueTypes.length];
            }
            else
            {
                numberOfNonZeroSlices = new int[0];
            }

            //for ( int i = 0 ; i < m_numberOfImages ; i++ )
            for ( int i = m_minimumImageNumberToFocus ; i < m_maximumImageNumberToFocus ; i++ )
            {
                
                if ( m_selectedTissueTypes != null )
                {

                    LinkedList<ImageSegmentProperties> segmentProperties = computeStatisticalPropertiesOfImageSegmentsOfImageNumber( i );

                    LinkedList<boolean[][]> colorOverlays = m_imageSegmentsOverlayList.get( i );
                    
                    for ( int j = 0; j < m_selectedTissueTypes.length; j++ )
                    {
                        boolean[][] imSegment = colorOverlays.get( m_selectedTissueTypes[ j ] );

                        //averagePixelValuesPerSegmentType[j] += segmentProperties.get(j).getMeanPixelValue( );

                        int numberOfPixels = 0;
                        for ( int k = 0; k < m_imageWidth ; k++ )
                        {
                            for ( int l = 0; l < m_imageHeight; l++ )
                            {
                                if ( imSegment[k][l] == true )
                                {
                                    numberOfPixels++;
                                }
                            }
                        }
                    
                        volumeInCurrentImage = ( numberOfPixels * volumePerPixel );
                        grandVolume[j] = grandVolume[j] + volumeInCurrentImage;
                        if ( volumeInCurrentImage != 0 )
                        {
                            // use weighted mean!
                            averagePixelValuesPerSegmentType[j] += segmentProperties.get( j ).getMeanPixelValue( ) * volumeInCurrentImage;

                            //also per tissue type!
                            numberOfNonZeroSlices[j]++;
                        }
                    }
                }
            }
            
            if ( m_selectedTissueTypes != null )
            {
                for ( int j = 0; j < m_selectedTissueTypes.length; j++ )
                {
                    if ( m_selectedTissueTypes.length > 0 )
                    {
                        writer.writeStringLn( "" );
                        writer.writeStringLn( "Total volume ( " + numberOfNonZeroSlices[j] + " out of " + m_numberOfImages + " non zero slices ) of Tissue-type[" + j + "]\t\t: " + ( grandVolume[j]/1000.0 ) + " cm^3" );
                        writer.writeStringLn( "Average pixel value ( " + numberOfNonZeroSlices[j] + " out of " + m_numberOfImages + " non zero slices ) of Tissue-type[" + j + "]\t: " + ( averagePixelValuesPerSegmentType[j] / grandVolume[j] ) + " a.u." );
                    }
                    else
                    {
                        writer.writeStringLn( "Total volume ( " + "0" + " out of " + m_numberOfImages + " non zero slices ) : " + "0.000" + " cm^3" );    
                    }
                }
            }
            
            if ( ( m_selectedTissueTypes == null ) || ( m_selectedTissueTypes.length == 0 ) )
            {
                writer.writeStringLn( "Total volume ( " + "0" + " out of " + m_numberOfImages + " non zero slices ) : " + "0.000" + " cm^3" );    
            }
            
            writer.flush( );
            writer.close( );
        }
        catch ( IOException e )
        {
            e.printStackTrace( );
        }
    }

    /**
     * If the data in DicomSliceCanvasPolyDraw has more than 64 images,
     * they can be viewed in a 3D fashion
     */
    private void viewImageStackIn3DControl( )
    {
        Dicom3DSlicerControlApp app = new Dicom3DSlicerControlApp( m_dicomImageSeriesData.getFileReferences( ),
                                                                   m_activationImageOverlays,
                                                                   m_dtiFiberImageOverlays,
                                                                   m_dtiFiberColors );
    }
    
    /**
     * Handle import of ROI data from a file having 
     * the .vec-format
     */
    public void handleImportOfROIData( )
    {
        FileInfo fInfo1 = m_dicomImageSeriesData.getFileInfo( 0 );
        String fileName = getFilePathAndName( fInfo1.getPatientName( ) + "_" + fInfo1.getSeriesDescription( ) + "_" + "detailed_report_file_of_all_regions_for_all_images.txt", "regio_rep", "Import" );
        
        RegionsInVectorFileFormatReader reader = new RegionsInVectorFileFormatReader( m_imageWidth, m_imageHeight );
        
        LinkedHashMap<String, RegioPolygon> polyLinkedHashMap = reader.parseFile( fileName );
        Iterator iter = polyLinkedHashMap.keySet( ).iterator( );
        
        while ( iter.hasNext( ) )
        {
            RegioPolygon poly = polyLinkedHashMap.get( iter.next( ) );
            this.addPolygon(poly);
        }
    }

    /**
     * Defines for all slices and all regiopolygons the instance number the
     * RegioPolygon belongs to
     */
    public void setAllInstanceNumbersOfPolygonsOfAllSlices( )
    {

        for ( int i = 0; i < m_numberOfImages; i++ )
        {
            LinkedList <RegioPolygon> imageRegioPolygonsSliceLinkedList = m_polygonsVector.get( i );

            // set the instance number attribute
            for( int kk = 0; kk < imageRegioPolygonsSliceLinkedList.size( ); kk++ )
            {
                RegioPolygon poly = imageRegioPolygonsSliceLinkedList.get( kk );
                poly.setInstanceNumber( i );
            }
        }
    }



    /**
     * Resets all linking information
     */
    private void resetAllPolygonLinkingInformation( )
    {
        for ( int i = 1; i < m_numberOfImages; i++ )
        {
            LinkedList <RegioPolygon> imageRegioPolygonsSlice_1 = m_polygonsVector.get( i - 1 );

            for ( int j = 0; j < imageRegioPolygonsSlice_1.size( ); j++ )
            {
                RegioPolygon poly = imageRegioPolygonsSlice_1.get( j );
                poly.resetConnectedRegionPolyInformation( );
            }
        }
    }


    /**
     * Determines all Linked RegioPolygons between adjacent slices
     */
    public void resolveAllLinksBetweenRegioPolygons( )
    {
         // define the links between the regions in the different slices
        System.out.println(  "1. Resolve connections" );
        for ( int i = 1; i < m_numberOfImages; i++ )
        {
            LinkedList <RegioPolygon> imageRegioPolygonsSlice_1 = m_polygonsVector.get( i - 1 );
            LinkedList <RegioPolygon> imageRegioPolygonsSlice_2 = m_polygonsVector.get( i );

            // Resolve linking
            for ( int kk = 0 ; kk < imageRegioPolygonsSlice_1.size( ); kk++ )
            {
                RegioPolygon poly_1 = imageRegioPolygonsSlice_1.get( kk );

                for ( int ll = 0; ll < imageRegioPolygonsSlice_2.size( ); ll++ )
                {
                    RegioPolygon poly_2 = imageRegioPolygonsSlice_2.get( ll );

                     boolean intersect = poly_1.intersects( poly_2 );

                     if ( intersect == true )
                     {
                         poly_2.addConnectedRegioPolygon( poly_1 );

                          System.out.println(  "Polygon " + poly_1.getLabel( ) + " is connected with Polygon " + poly_2.getLabel( ) );
                     }
                     else
                     {
                         System.out.println(  "Polygon " + poly_1.getLabel( ) + " is NOT connected with Polygon " + poly_2.getLabel( ) );
                     }
                }
            }
        }
    }

    /**
     * Excludes small ROIs which are completely within another bigger ROI
     * This is necessary in volume counting, where we are not interested in
     * the ROI-volume and values itself but how many volumes there are
     * and whats the total volume
     */
    private void excludeSmallInnerRegionsWithinOneSliceForAllSlices( )
    {
        boolean[] regionsToRemove;

        for ( int k = 0; k < m_numberOfImages;  k++ )
        {

            LinkedList <RegioPolygon> imageRegioPolygonsSlice_1 = m_polygonsVector.get( k );

            regionsToRemove = new boolean[ m_polygonsVector.size( ) ];

            for ( int i = 0; i < imageRegioPolygonsSlice_1.size( ); i++ )
            {
                for ( int j = 0; j < imageRegioPolygonsSlice_1.size( ); j++ )
                {
                    RegioPolygon poly_1 = imageRegioPolygonsSlice_1.get( i );
                    RegioPolygon poly_2 = imageRegioPolygonsSlice_1.get( j );

                    if ( poly_1.getInnerVolume( ) < poly_2.getInnerVolume( ) )
                    {
                        // if ( poly_1.isTotallyWithin( poly_2 ) == true )
                        if ( poly_1.intersects( poly_2 ) == true )
                        {
                            regionsToRemove[i] = true;
                        }
                    }
                }
            }

            int length = imageRegioPolygonsSlice_1.size( );

            for ( int i = length - 1; i > -1; i-- )
            {
                if ( regionsToRemove[i] == true )
                {
                    imageRegioPolygonsSlice_1.remove( i );
                }
            }
            
        }

    }


    /**
     * Resolves the number of distinct not connected volumes
     */
    public void resolveAndSetNumberOfUniqueVolumes( )
    {
        int uniqueVolumeCounter = 0;

        for ( int i = m_numberOfImages - 1; i > -1; i-- )
        {

            LinkedList <RegioPolygon> imageRegioPolygonsSlice_1 = m_polygonsVector.get( i );
            
            //System.out.println(  "1. Slice number : " + i );

            for ( int j = 0; j < imageRegioPolygonsSlice_1.size( ); j++ )
            {

                RegioPolygon poly = imageRegioPolygonsSlice_1.get( j );

                //System.out.println( "2. Analysing : " + poly.getLabel( ) + "  " + ( poly.getIndex( ) ) );
                //System.out.println( "3. uniqueVolumeCounter = " + uniqueVolumeCounter );

                // two case current node does not belong to a defined volume yet
                // CASE 1 volume is not defined yet
                if ( poly.getConnectedRegionPolyInformation( ).getVolumeNumberBelongingTo( ) == -1 )
                {
                    
                    System.out.println(  "4. Still analysing : " + poly.getLabel( ) + "  " + "(" + poly.getIndex( ) +")" + " does not belong to a volume number yet" );

                    if  ( ( poly.getConnectedRegionPolyInformation( ).getConnectedPolygons( ).size( ) == 0 ) &&
                          ( poly.getConnectedRegionPolyInformation( ).getVolumeNumberBelongingTo( ) == -1 ) )
                    {

                        //System.out.println(  "5. Still analysing : " + poly.getLabel( ) + "  " + "(" + poly.getIndex( ) + ")" + " has NO connected polygons" );
                        //System.out.println(  "6. Still analysing : " + poly.getLabel( ) + "  " + "(" + poly.getIndex( ) + ")" + " assignening to volume : " + uniqueVolumeCounter );

                        poly.getConnectedRegionPolyInformation( ).setVolumeNumberBelongingTo( uniqueVolumeCounter );
                        uniqueVolumeCounter++;
                    }

                    if ( ( poly.getConnectedRegionPolyInformation( ).getConnectedPolygons( ).size( ) != 0 ) )
                    {

                        //System.out.println(  "7. Still analysing : " + poly.getLabel( ) + "  " + "(" + poly.getIndex( ) + ")" + " has connected polygons" );
                        //System.out.println(  "8. Still analysing : " + poly.getLabel( ) + "  " + "(" + poly.getIndex( ) + ")" + " assignening to volume : " + uniqueVolumeCounter );


                        for ( int k = 0; k < poly.getConnectedRegionPolyInformation( ).getConnectedPolygons( ).size( ); k++ )
                        {

                            if ( poly.getConnectedRegionPolyInformation( ).getVolumeNumberBelongingTo( ) == -1 )
                            {

                                //System.out.println(  "9. Still analysing : " + poly.getLabel( ) + "  " + "(" + poly.getIndex( ) + ")" + " has connected polygons" );
                                //System.out.println(  "10. Still analysing : " + poly.getLabel( ) + "  " + "(" + poly.getIndex( ) + ")" + " assignening to volume : " + uniqueVolumeCounter );
                                poly.getConnectedRegionPolyInformation( ).setVolumeNumberBelongingTo( uniqueVolumeCounter );



                                RegioPolygon linked_poly = poly.getConnectedRegionPolyInformation( ).getConnectedPolygons( ).get( k );
                                linked_poly.getConnectedRegionPolyInformation( ).setVolumeNumberBelongingTo( uniqueVolumeCounter );
                                
                                //System.out.println(  "11. Assigning linked polygon : " + linked_poly.getLabel( ) + "  " + "(" + linked_poly.getIndex( ) + ")" + "  " + "to volume " + uniqueVolumeCounter );

                            }
                            else
                            {

                                RegioPolygon linked_poly = poly.getConnectedRegionPolyInformation( ).getConnectedPolygons( ).get( k );
                                linked_poly.getConnectedRegionPolyInformation( ).setVolumeNumberBelongingTo( poly.getConnectedRegionPolyInformation( ).getVolumeNumberBelongingTo( ) );

                                System.out.println(  "12. Assigning linked polygon : " + linked_poly.getLabel( ) + "  " + "(" + linked_poly.getIndex( ) + ")" + "  " + "to volume " + uniqueVolumeCounter );
                            }
                        }
                        uniqueVolumeCounter++;
                    }
                }
                else
                {

                     //System.out.println(  "13. Still analysing : " + poly.getLabel( ) + "  " + ( poly.getIndex( ) ) + " already belongs to a volume number" );
                     
                    // this case that the node already belongs to a defined volume
                    for ( int k = 0; k < poly.getConnectedRegionPolyInformation( ).getConnectedPolygons( ).size( ); k++ )
                    {
                       RegioPolygon linked_poly = poly.getConnectedRegionPolyInformation( ).getConnectedPolygons( ).get( k );
                       linked_poly.getConnectedRegionPolyInformation( ).setVolumeNumberBelongingTo( poly.getConnectedRegionPolyInformation( ).getVolumeNumberBelongingTo( ) );

                       //System.out.println(  "14. Assigning linked polygon : " + linked_poly.getLabel( ) + "  " + "(" + linked_poly.getIndex( ) + ")" + "  " + " to already assigned volume " + poly.getConnectedRegionPolyInformation( ).getVolumeNumberBelongingTo( ) );
                    }
                }
            }
        }
        
        m_uniqueVolumeCounter = uniqueVolumeCounter;
        
        System.out.print( "Number of unique volumes = " + uniqueVolumeCounter );

    }

    /**
     * Creates a text-file with an overview of all volumes and volume properties
     *
     */
    private void createFileOfDistinctVolumes( )
    {

        // local variables to be written to file
        double total_volume_per_lesion = 0f;
        double average_of_average_pixel_values = 0f;
        double average_of_st_devs = 0f;

        double globalLesionVolume = 0f;
        double  globalAveragePixIntensity = 0f;
        double  globalStDevPixIntensity = 0f;

        FileInfo finfo = m_dicomImageSeriesData.getFileInfo( 0 );
        String fileName = finfo.getFileName( );

        String patientName = finfo.getPatientName( );
        String examinationDate = finfo.getAcquisitionDate( );

        double volumePerPixel = new Double( finfo.getSliceThickness( ) ).doubleValue( ) * 
                                new Double( finfo.getPixelHeight( ) ).doubleValue( ) *
                                new Double( finfo.getPixelWidth( ) ).doubleValue( );

        // save to reg file
        // String fileName = m_currentView.getTitle( );
        int ii = fileName.lastIndexOf( "." );
        if (ii == -1) ii = fileName.length( );
        String nameRoot = fileName.substring( 0, ii );
        String reportName = nameRoot + ".cnt";

        // Determine path to load the overlay files from
        String filePath = m_dicomImageSeriesData.getFileInfo( 0 ).getDirectory( );

        File outputPath = new File( filePath + System.getProperty( "file.separator" ) + "cnt" );
        if ( outputPath.exists( ) == false )
        {
            outputPath.mkdir( );
        }
        try
        {
            TextFileWriter writer = new TextFileWriter( new FileWriter( outputPath + System.getProperty( "file.separator" ) + reportName ) );

            writer.writeStringLn( "Lesion-Report of " + fileName );
            writer.writeStringLn( "Patient name     : " + patientName );
            writer.writeStringLn( "Examination date : " + examinationDate );
            writer.writeStringLn( "Slice thickness : " + finfo.getSliceThickness( ) + "mm  | pixel width : " + finfo.getPixelWidth( ) + "mm  | pixel heigth : " + finfo.getPixelHeight( ) + "mm" );
            writer.writeStringLn(  "*****************************************************************************************************" );

            // for each unique volume all properties are determined
            for ( int k = 0; k < m_uniqueVolumeCounter; k++ )
            {

                int    totalNumberOfPixels = 0;
                double  averageOfPixelIntensityValues = 0f;
                double  averageOfStDevOfPixelValues = 0f;
                double totalLesionVolume = 0f;
                int    cntr = 0;

                for ( int i = m_numberOfImages - 1; i > -1; i-- )
                {
                    boolean delta = false;
                    LinkedList <RegioPolygon> imageRegioPolygonsSlice_1 = m_polygonsVector.get( i );

                    for ( int j = 0; j < imageRegioPolygonsSlice_1.size( ); j++ )
                    {
                        RegioPolygon poly = imageRegioPolygonsSlice_1.get( j );

                        RegioPolygonConnections regPolConnections = poly.getConnectedRegionPolyInformation( );

                        if ( regPolConnections.getVolumeNumberBelongingTo( ) == k )
                        {
                            cntr++;

                            ImageRegionProperties imageROIProperty = computeRegioProperties( i, poly );

                            if ( imageROIProperty.getNumberOfPixels( ) > 1 )
                            {
                                delta = true;
                                writer.writeString( "Unique volume number :\t" + ( k + 1 ) );
                                writer.writeString( "\tSlice number : " + ( i + 1 ) );
                            
                                writer.writeString( "\tNumber of points :\t" + imageROIProperty.getNumberOfPixels( ) );
                                totalNumberOfPixels = totalNumberOfPixels + imageROIProperty.getNumberOfPixels( );
                            
                                writer.writeString( "\tVolume :\t" + imageROIProperty.getImageRegionVolume( ) );
                                totalLesionVolume = totalLesionVolume + imageROIProperty.getImageRegionVolume( );

                                writer.writeString( "\tMean of pix values\t: " + imageROIProperty.getMeanPixelValue( ) );
                                averageOfPixelIntensityValues = averageOfPixelIntensityValues + imageROIProperty.getMeanPixelValue( );

                                writer.writeStringLn( "\tSt.dev pix values\t: " + imageROIProperty.getStandardDeviationOfPixelValues( ) );
                                averageOfStDevOfPixelValues = averageOfStDevOfPixelValues + imageROIProperty.getStandardDeviationOfPixelValues( );
                            }
                        }
                    }

                    // write only to file non zero results!
                    //if ( ( totalNumberOfPixels != 0 ) && ( delta == true ) )
                    //{
                    //    writer.writeString( "Unique volume number : " + k );
                    //    writer.writeString( "    Slice number         : " + i );
                    //    writer.writeString( "    Number of points     : " + totalNumberOfPixels );
                    //    writer.writeString( "    Sub total Volume     : " + totalLesionVolume );
                    //    writer.writeString( "    Mean of mean pix values   : " + averageOfPixelIntensityValues / cntr );
                    //    writer.writeStringLn( "    Mean of St.dev pix values : " + averageOfStDevOfPixelValues / cntr );
                    //    writer.writeStringLn( "" ); // after each lesion one empty line
                    //}
                }

                globalLesionVolume = globalLesionVolume + totalLesionVolume;
                globalAveragePixIntensity = globalAveragePixIntensity + ( averageOfPixelIntensityValues / cntr );
                globalStDevPixIntensity = globalStDevPixIntensity + ( averageOfStDevOfPixelValues / cntr );
            }

            writer.writeStringLn( "" );
            writer.writeStringLn( "*****************************************************************************************************" );
            writer.writeStringLn( "" );
            writer.writeStringLn( "Total number of lesions :\t" + m_uniqueVolumeCounter );
            writer.writeStringLn( "Grand total lesion volume :\t" + globalLesionVolume );
            writer.writeStringLn( "Grand Mean of mean pix values :\t" + globalAveragePixIntensity / m_uniqueVolumeCounter );
            writer.writeStringLn( "Mean of St.dev pix values :\t" + globalStDevPixIntensity / m_uniqueVolumeCounter );
       
            writer.flush( );
            writer.close( );
        }
        catch ( IOException e )
        {
            e.printStackTrace( );
        }
    }


    /**
     * This method creates a report file of all distict, i.e. non-connected 
     * volumes in the image stack. The algorthm verifies whether ROI's in 
     * different slices are connected or not.
     */
    private void determineDistinctVolumesAndCreateReportFile( )
    {

         // excludes all small ROI (normally outerborders within a big ROI)
        excludeSmallInnerRegionsWithinOneSliceForAllSlices( );

        resetAllPolygonLinkingInformation( );

        setAllInstanceNumbersOfPolygonsOfAllSlices( );
        
        // define the links between the regions in the different slices
        resolveAllLinksBetweenRegioPolygons( );

        // now the number of unique volumes can be resolved
        resolveAndSetNumberOfUniqueVolumes( );

        // now the file can be made
        createFileOfDistinctVolumes( );

    }


    
    /**
     * Returns the filename and directory selected by the user
     */
    private String getROIPixelValueDumpFilePathAndName( String defaultString, String buttonText )
    {

        FileNameExtensionFilter voiFilter = new FileNameExtensionFilter( "ROI file", "roi", "ROI" );


        JFileChooser fileChooser = new JFileChooser( );
        String selectedFilePathAndName = "";

        // only image file can be choosen!
        fileChooser.addChoosableFileFilter( voiFilter );
        
        // Create a File object containing the canonical path of the
        // desired directory

        String filePath = System.getProperty( "user.dir" );

        File f = new File( filePath );

        // Set the current directory
        fileChooser.setCurrentDirectory( f );

        int result;
        
        result = fileChooser.showDialog( this, defaultString );

        File selected_file = new File( "" );

        switch ( result )
        {
            case JFileChooser.APPROVE_OPTION:   // Approve Open button was pressed
                                                selected_file = fileChooser.getSelectedFile( );
                                                selectedFilePathAndName = selected_file.getPath( );
                                                break;
            case JFileChooser.CANCEL_OPTION:    // Cancel or close dialog was selected
                                                break;
            case JFileChooser.ERROR_OPTION:     // the selected process did not terminate properly
                                                break;
        }

        return selectedFilePathAndName;
    }
    
    

    /**
     * Returns the filename and directory selected by the user
     */
    private String getBrainVoyagerVOIFilePathAndName( String defaultString, String buttonText )
    {

        
        FileNameExtensionFilter voiFilter = new FileNameExtensionFilter( "VOI file", "voi", "VOI" );


        JFileChooser fileChooser = new JFileChooser( );
        String selectedFilePathAndName = "";

        // only image file can be choosen!
        fileChooser.addChoosableFileFilter( voiFilter );
        
        // Create a File object containing the canonical path of the
        // desired directory

        String filePath = System.getProperty( "user.dir" );

        File f = new File( filePath );

        // Set the current directory
        fileChooser.setCurrentDirectory( f );

        int result;
        //fileChooser.setSelectedFile( new File( patientName+ "_" + examinationDate + "_" + defaultString ) );

        result = fileChooser.showDialog( this, defaultString );

        File selected_file = new File( "" );

        switch ( result )
        {
            case JFileChooser.APPROVE_OPTION:   // Approve Open button was pressed
                                                selected_file = fileChooser.getSelectedFile( );
                                                selectedFilePathAndName = selected_file.getPath( );
                                                break;
            case JFileChooser.CANCEL_OPTION:    // Cancel or close dialog was selected
                                                break;
            case JFileChooser.ERROR_OPTION:     // the selected process did not terminate properly
                                                break;
        }

        return selectedFilePathAndName;
    }

    /**
     * Returns the filename and directory selected by the user
     */
    private String getBrainVoyagerFBRFilePathAndName( String defaultString, String buttonText )
    {

        FileNameExtensionFilter voiFilter = new FileNameExtensionFilter( "FBR file", "fbr", "FBR" );


        JFileChooser fileChooser = new JFileChooser( );
        String selectedFilePathAndName = "";

        // only image file can be choosen!
        fileChooser.addChoosableFileFilter( voiFilter );

        // Create a File object containing the canonical path of the
        // desired directory

        String filePath = System.getProperty( "user.dir" );

        File f = new File( filePath );

        // Set the current directory
        fileChooser.setCurrentDirectory( f );

        int result;
        //fileChooser.setSelectedFile( new File( patientName+ "_" + examinationDate + "_" + defaultString ) );

        result = fileChooser.showDialog( this, defaultString );

        File selected_file = new File( "" );

        switch ( result )
        {
            case JFileChooser.APPROVE_OPTION:   // Approve Open button was pressed
                                                selected_file = fileChooser.getSelectedFile( );
                                                selectedFilePathAndName = selected_file.getPath( );
                                                break;
            case JFileChooser.CANCEL_OPTION:    // Cancel or close dialog was selected
                                                break;
            case JFileChooser.ERROR_OPTION:     // the selected process did not terminate properly
                                                break;
        }

        return selectedFilePathAndName;
    }



    /**
     * Loads the BrainVoyager overlay file containing fMRI activation maps
     * which can be overlayed onto a 3D dataset which currently loaded.
     * @return
     */
    private boolean importBrainvoyagerVOIFileAsImageOverlay( )
    {
        String voiFile = getBrainVoyagerVOIFilePathAndName( "Brainvoyager - fMRI-activation voi-file", "Import" );

        BrainVoyagerVOIFileParserAndLoader voiLoader = new BrainVoyagerVOIFileParserAndLoader( voiFile,  m_numberOfImages, m_imageWidth, m_imageHeight );
        boolean[][][] b = voiLoader.loadBrainVoyagerVOIFile( );
        setActivationOverlays( b );
        
        return true;
    }


    /**
     * Loads a BrainVoyager FBR file and fiber overlay positions and
     * the fiber directions as color codings
     * @return
     */
    private boolean importBrainvoyagerFBRFileAsImageOverlay( )
    {

        String fbrFile = getBrainVoyagerFBRFilePathAndName( "Brainvoyager - DTI-fiber fbr-file", "Import" );

        BrainVoyagerFBRFileParserAndLoader fbrLoader = new BrainVoyagerFBRFileParserAndLoader( fbrFile,  m_numberOfImages, m_imageWidth, m_imageHeight );

        boolean[][][] pos = fbrLoader.loadBrainVoyagerFBRFile( );
        Color[][][] colors = fbrLoader.getOverlayColors( );
        setDTIFiberOverlays( pos, colors );

        return true;
    }


    /**
     * If m_eliminatePixelsFromOverlaysByContourDrawingModeRadioButton.isSelected() is
     * true, the user is drawing ROIs for which the image segments must be set to
     * false; user wants them to remove from the image segments
     */
    public void handleRemovePixelsFromSelectedImageSegmentOverlays( )
    {
        m_selectedPolygon.setScalingFactor( 1.0f / m_zoom );
        
        ImageOverlayPixelRemover pixelRemover = new ImageOverlayPixelRemover( m_parent,
                                                                              m_imageSegmentsOverlayList,
                                                                              m_selectedPolygon,
                                                                              m_selectedTissueTypes,
                                                                              m_currentlyDisplayedImageNumber,
                                                                              m_imageWidth,
                                                                              m_imageHeight,
                                                                              m_numberOfImages );
    }
    
    
    /**
     * Handles the removal of pixel from the m_imageVolumeOfInterestOverlayList
     */
    public void handleRemovePixelsFromSelectedVolumeOfInterestOverlays( )
    {
        m_selectedPolygon.setScalingFactor( 1.0f / m_zoom );

        int[] selectedVolumeOverlays = new int[ m_selectedImageVolumeOfInterestList.size( ) ];

        for ( int i = 0; i<  m_selectedImageVolumeOfInterestList.size( ); i++ )
        {
            if ( m_selectedImageVolumeOfInterestList.get(i) == true )
            {
                selectedVolumeOverlays[i] = i;
            }
        }

        ImageOverlayPixelRemover pixelRemover = new ImageOverlayPixelRemover( m_parent,
                                                                              m_imageVolumeOfInterestOverlayList,
                                                                              m_selectedPolygon,
                                                                              selectedVolumeOverlays,
                                                                              m_currentlyDisplayedImageNumber,
                                                                              m_imageWidth,
                                                                              m_imageHeight,
                                                                              m_numberOfImages );
    }

    /**
     * Routine that computes pixel intensity histograms for selected 
     * image segment types. Additionally it displays the histograms
     * and allows the user to store the histograms.
     */
    public void handleCreateHistogramsForSelectedSegmentTypes( )
    {
        
//         public ImageOverlaySegmentHistogramCalculator( JFrame parent,
//                                                   ArrayList< LinkedList<boolean[][]> > imageSegmentsOverlayArrayList,
//                                                   int[] imagePixelValues, int[] selectedTissueTypes,
//                                                   int imageWidth, int imageHeight, int numberOfImages )
        
        
        ImageOverlaySegmentHistogramCalculator imOverlaySegmentHistogramCalculator =
                new ImageOverlaySegmentHistogramCalculator( m_parent,
                                                            m_imageSegmentsOverlayList,
                                                            m_imagePixelData,
                                                            m_selectedTissueTypes,
                                                            m_imageWidth,
                                                            m_imageHeight,
                                                            m_numberOfImages,
                                                            m_nordicScaleFactor );
    }



    /**
     *  Called when e.g. a button is pressed or a menu item is selected etc.
     */
    @Override
    @SuppressWarnings( "empty-statement" )
    public void actionPerformed( ActionEvent actionEvent ) 
    {
        // first call base class
        super.actionPerformed( actionEvent );

        Object sender = actionEvent.getSource( );
        
        if ( sender == m_fieldOfInterestMenuItem )
        {
            if (  getSelectedPolygon( ) != null )
            {
                m_fieldOfInterest = new RegioPolygon( getSelectedPolygon( ) );
            }
        }
        
        if ( sender == m_statisticalPropertiesOf3CircularRegioPolygonMenuItem )
        {
            saveThreeStandardCircularROIInfos( );
        }
        
        if ( sender == m_scaledReferenceFrame )
        {
            if ( actionEvent.getID( ) == ScaledROIDefintionFrame.ENABLE_ADDMODE )
            {
                setMode( ADD_STACKED_ROI_MODE_EVENT );
            }
        }
        
        if ( sender == m_saveRegioPolygonsAsIOVOfAllImagesMenuItem )
        {
            saveRegionsToIntegerOverlaysFiles( );
        }
        
        if ( sender == this.m_loadIOVOfAllImagesMenuItem )
        {
            loadRegionsFromIntegerOverlaysFiles( );
        }
        
        if ( sender == m_promoteThressholdOverlayToTissueTypeJMenuItem )
        {
            notifyActionListeners( PROMOTE_CURRENT_THRESSHOLD_OVERLAY_TO_TISSUE_TYPE );
            handlePromoteCurrentThressholdOverlayToTissueType( );
        }
        
        if ( sender == m_createTextureAnalyzerMenuItem )
        {
            ImageStackTextureAnalyzerFrame frm = new ImageStackTextureAnalyzerFrame( m_parent, m_dicomImageSeriesData, false, 0, 0, 0, 0 );
        }

        if ( sender == m_deleteVolumeOfInterest )
        {
            for ( int i = m_selectedImageVolumeOfInterestList.size( ) - 1; i >= 0; i-- )
            {
                if ( m_selectedImageVolumeOfInterestList.get(i).booleanValue( ) == true )
                {
                    m_selectedImageVolumeOfInterestList.remove( i );
                    m_imageVolumeOfInterestNameList.remove( i );
                    m_imageVolumeOfInterestColorList.remove( i );

                    LinkedList<boolean[][]> overlayListOfOneImage;

                    // also remove the overlay from the data containers..
                    for ( int j = 0; j < m_numberOfImages; j++ )
                    {
                        overlayListOfOneImage = m_imageVolumeOfInterestOverlayList.get( j );
                        overlayListOfOneImage.remove( i );
                    }

                }
            }
            repaint( );
        }

        if ( sender == m_selectVolumesToOverlay )
        {
            if ( ( m_imageVolumeOfInterestOverlayList != null ) && ( m_imageVolumeOfInterestOverlayList.size( ) > 0 ) )
            {
                LinkedList<boolean[][]> overlaysOfImage = m_imageVolumeOfInterestOverlayList.get( 0 );
            
                m_voi_ManipulationFrame = new VolumeOfInterestOverlayManipulationFrame( this );
                m_voi_ManipulationFrame.addActionListener( this );
            }
            else
            {
                Dialogs.messageBox( m_parent, "No volumes of interes defined: first load them and retry!!", "No VOIs defined!" );
            }
        }

        if ( sender == m_voi_ManipulationFrame )
        {

            if ( actionEvent.getID( ) == VolumeOfInterestOverlayPropertiesTable.DELETE_SELECTED_VOLUME_EVENT )
            {
                for ( int i = m_selectedImageVolumeOfInterestList.size( ) - 1; i >= 0; i-- )
                {
                    if ( m_selectedImageVolumeOfInterestList.get(i).booleanValue( ) == true )
                    {
                        m_selectedImageVolumeOfInterestList.remove( i );
                        m_imageVolumeOfInterestNameList.remove( i );
                        m_imageVolumeOfInterestColorList.remove( i );

                        LinkedList<boolean[][]> overlayListOfOneImage;

                        // also remove the overlay from the data containers..
                        for ( int j = 0; j < m_numberOfImages; j++ )
                        {
                            overlayListOfOneImage = m_imageVolumeOfInterestOverlayList.get( j );
                            overlayListOfOneImage.remove( i );
                        }

                    }
                }
                repaint( );
            }

            if ( actionEvent.getID( ) == VolumeOfInterestOverlayPropertiesTable.RENAME_SELECTED_VOLUME_EVENT )
            {
                System.out.println( "RENAME EVENT CALLED!" );

            }
        }

        if ( sender == m_opaquacyOfVolumesOverlays )
        {
            m_defineOpaquacyFactorDefinitionFrame = new EnterDecimalValueFrame( m_parent, "Opacacy factor (0 < factor < 1.0)", m_opaquacyOfVolumeOverlays, "Define opaquacy of volume overlays", "0.000" );
            m_defineOpaquacyFactorDefinitionFrame.addActionListener( this );
        }
       
        if ( sender == m_defineOpaquacyFactorDefinitionFrame )
        {
             if ( actionEvent.getID( ) == EnterDecimalValueFrame.APPLY_EVENT )
             {
                m_opaquacyOfVolumeOverlays = new Float( m_defineOpaquacyFactorDefinitionFrame.getText( ) ).floatValue( );
                repaint( );
             }
        }

        if ( sender == m_loadVolumeOfInterestFromDefault )
        {
            if ( m_dicomImageSeriesData != null )
            {
                loadVolumesOfInterestOverlaysFromFiles( );
            }
            else
            {
                loadVolumesOfInterestOverlaysFromFile( );
            }
        }
        
        if ( sender == m_loadMHAOverlay )
        {
            if ( m_dicomImageSeriesData != null )
            {
                load_Volumes_Overlays_From_MHA_File( );
            }
        }

        if ( sender == m_loadVolumeOfInterestUserDefined )
        {
            loadVolumesOfInterestFromFilesUserDefinedPath( );
        }

        if ( sender == m_saveVolumeOfInterestToDefault )
        {
            saveVolumeOfInterestOverlaysToFiles( );
        }

        if ( sender == m_saveVolumeOfInterestUserDefined )
        {
            saveVolumeOfInterestFilesToUserDefinedPath( );
        }

        if ( sender == m_projectionOfVolumeOfInterestInOtherImageStacks )
        {
            notifyActionListeners( PROJECT_VOLUME_IN_OTHER_IMAGE_STACKS_EVENT );
        }

        if ( sender == m_createHistogramFilesOfCurrentlySelectedImageSegmentTypes )
        {
            handleCreateHistogramsForSelectedSegmentTypes( );
        }

        if ( sender == m_measureDistancesFrame )
        {
            
            if ( actionEvent.getID( ) == MeasureDistancesFrame.DISTANCES_CHANGED )
            {
                LinkedList distanceLinkedList = m_distancesVector.get(m_currentlyDisplayedImageNumber );

                Vector<Point> startPoints2D = m_measureDistancesFrame.getStartingPoints2D( );
                Vector<Point> endPoints2D = m_measureDistancesFrame.getEndingPoints2D( );

                Vector<Point3D> startPoints3D = m_measureDistancesFrame.getStartingPoints3D( );
                Vector<Point3D> endPoints3D = m_measureDistancesFrame.getEndingPoints3D( );
                Vector<Float> distanceValues = m_measureDistancesFrame.getDistances( );

                Vector<Integer> sliceNumbers = m_measureDistancesFrame.getSliceNumbers( );
                Vector<String> names = m_measureDistancesFrame.getDistancesNames( );
                Vector<Color> colors = m_measureDistancesFrame.getDistancesColors( );

                // build the linked list newly
                distanceLinkedList.clear( );
                for ( int i = 0; i < endPoints2D.size( ) ; i++ )
                {
                    distanceLinkedList.add( new Distance( distanceLinkedList.size( ),
                                                          names.get( i ),
                                                          distanceValues.get(i).floatValue( ),
                                                          sliceNumbers.get( i ),
                                                          startPoints2D.get( i ),
                                                          endPoints2D.get( i ),
                                                          startPoints3D.get( i ),
                                                          endPoints3D.get( i ),
                                                          Color.CYAN ) );
                }

                update( true );
            }





            if ( actionEvent.getID( ) == MeasureDistancesFrame.ADDED_POINT )
            {
                LinkedList distanceLinkedList = m_distancesVector.get(m_currentlyDisplayedImageNumber );

                Vector<Point> startPoints2D = m_measureDistancesFrame.getStartingPoints2D( );
                Vector<Point> endPoints2D = m_measureDistancesFrame.getEndingPoints2D( );

                Vector<Point3D> startPoints3D = m_measureDistancesFrame.getStartingPoints3D( );
                Vector<Point3D> endPoints3D = m_measureDistancesFrame.getEndingPoints3D( );
                Vector<Float> distanceValues = m_measureDistancesFrame.getDistances( );

                Vector<Integer> sliceNumbers = m_measureDistancesFrame.getSliceNumbers( );
                Vector<String> names = m_measureDistancesFrame.getDistancesNames( );
                Vector<Color> colors = m_measureDistancesFrame.getDistancesColors( );

                // build the linked list newly
                distanceLinkedList.clear( );
                for ( int i = 0; i < endPoints2D.size( ) ; i++ )
                {
                    distanceLinkedList.add( new Distance( distanceLinkedList.size( ),
                                                          names.get( i ),
                                                          distanceValues.get(i).floatValue( ),
                                                          sliceNumbers.get( i ),
                                                          startPoints2D.get( i ),
                                                          endPoints2D.get( i ),
                                                          startPoints3D.get( i ),
                                                          endPoints3D.get( i ),
                                                          Color.CYAN ) );
                }

                update( true );
            }

            if ( actionEvent.getID( ) == MeasureDistancesFrame.SAVE_DISTANCES )
            {
                saveDistancesToFiles( false );
                update( true );
                repaint( );
            }

            if ( actionEvent.getID( ) == MeasureDistancesFrame.STOP_MEASURING_DISTANCES )
            {
                m_measureDistanceMode = false;
                m_displayMeasureDistanceMode = true;

                update( true );
                repaint( );
            }

            if ( actionEvent.getID( ) == MeasureDistancesFrame.DELETE_SELECTED_DISTANCES )
            {
                Vector<Integer> selSliceNumbers = m_measureDistancesFrame.getSelectedSliceNumbers( );
                Vector<String> selNames = m_measureDistancesFrame.getSelectedDistancesNames( );

                deleteSelectedDistances( selSliceNumbers, selNames );
            }
        }

        if ( sender == m_loadDistancesOfAllImagesFromFileMenuItem )
        {
            loadDistanceFromFile( false );
        }

        if ( sender == m_loadDistancesOfCurrentlyDisplayedImageInFilesMenuItem )
        {
            loadDistanceFromFile( true );
        }

        if ( sender == m_saveDistancesOfCurrentlyDisplayedImageInFilesMenuItem )
        {
             saveDistancesToFiles( true );
        }
        
        if ( sender == m_createReportFileOfDistancesMenuItem )
        {
            createReportFileOfAllDistances( );
        }
        
        if ( sender == m_saveDistancesOfAllImagesInFilesMenuItem )
        {
            saveDistancesToFiles( false );
        }

        if ( sender == m_measureDistanceMenuItem )
        {
            m_measureDistancesFrame = new MeasureDistancesFrame( m_parent, m_distancesVector );
            m_measureDistancesFrame.setAlwaysOnTop( true );
            m_measureDistanceMode = true;

            m_displayMeasureDistanceMode = true;
            m_measureDistancesFrame.addActionListener( this );
        }

        if ( sender == m_convertImageSegmentsByMorphologicFiltering )
        {
            computeContoursAsBooleanImageOverlays( );
            update( );
        }

        // user wants conversion to true dicom; not a dicomized (colored) screenshot
        if ( sender == m_trueDicomFileCreatorOptionsFrame )
        {
            if ( actionEvent.getID( ) == DicomFileCreatorOptionsFrame.NOTIFY_CANCEL_BUTTON_PRESSED_EVENT )
            {
                m_trueDicomFileCreatorOptionsFrame.dispose( );
            }

            if ( actionEvent.getID( ) == DicomFileCreatorOptionsFrame.NOTIFY_DICOMIZE_AND_SEND_BUTTON_PRESSED )
            {
                int seriesNumberToUseForDicomization = m_trueDicomFileCreatorOptionsFrame.getUserSelectedSeriesNumberForDicomization( );
                String seriesDescriptionForDicomization = m_trueDicomFileCreatorOptionsFrame.getUserSelectedSeriesDescriptionForDicomization( );

                if ( actionEvent.getActionCommand( ).compareTo( DicomFileCreatorOptionsFrame.ALSO_INCLUDE_SPATIAL_TAGS ) == 0 )
                {
                    redicomizeCompleteImageStackAsTrueDICOMInWorkerThread( true, true, seriesNumberToUseForDicomization, seriesDescriptionForDicomization, m_numberOfImages );
                }
                else
                {
                    redicomizeCompleteImageStackAsTrueDICOMInWorkerThread( true, false, seriesNumberToUseForDicomization, seriesDescriptionForDicomization, m_numberOfImages );
                }
            }

            if ( actionEvent.getID( ) == DicomFileCreatorOptionsFrame.NOTIFY_DICOMIZE_ONLY_BUTTON_PRESSED )
            {

                int seriesNumberToUseForDicomization = m_trueDicomFileCreatorOptionsFrame.getUserSelectedSeriesNumberForDicomization( );
                String seriesDescriptionForDicomization = m_trueDicomFileCreatorOptionsFrame.getUserSelectedSeriesDescriptionForDicomization( );

                if ( actionEvent.getActionCommand( ).compareTo( DicomFileCreatorOptionsFrame.ALSO_INCLUDE_SPATIAL_TAGS ) == 0 )
                {
                    redicomizeCompleteImageStackAsTrueDICOMInWorkerThread( false, true, seriesNumberToUseForDicomization, seriesDescriptionForDicomization, m_numberOfImages );
                }
                else
                {
                    redicomizeCompleteImageStackAsTrueDICOMInWorkerThread( false, false, seriesNumberToUseForDicomization, seriesDescriptionForDicomization, m_numberOfImages );
                }
            }
        }


        if ( sender == m_dicomFileAsSecondaryCaptureCreatorOptionsFrame )
        {
            if ( actionEvent.getID( ) == DicomFileCreatorOptionsFrame.NOTIFY_CANCEL_BUTTON_PRESSED_EVENT )
            {
                m_dicomFileAsSecondaryCaptureCreatorOptionsFrame.dispose( );
            }

            if ( actionEvent.getID( ) == DicomFileCreatorOptionsFrame.NOTIFY_DICOMIZE_AND_SEND_BUTTON_PRESSED )
            {
                int seriesNumberToUseForDicomization = m_dicomFileAsSecondaryCaptureCreatorOptionsFrame.getUserSelectedSeriesNumberForDicomization( );
                String seriesDescriptionForDicomization = m_dicomFileAsSecondaryCaptureCreatorOptionsFrame.getUserSelectedSeriesDescriptionForDicomization( );

                if ( actionEvent.getActionCommand( ).compareTo( DicomFileCreatorOptionsFrame.ALSO_INCLUDE_SPATIAL_TAGS ) == 0 )
                {
                    redicomizeCompleteImageStackAsSecondaryCaptureInWorkerThread( true, true, seriesNumberToUseForDicomization, seriesDescriptionForDicomization, m_numberOfImages );
                }
                else
                {
                    redicomizeCompleteImageStackAsSecondaryCaptureInWorkerThread( true, false, seriesNumberToUseForDicomization, seriesDescriptionForDicomization, m_numberOfImages );
                }
            }

            if ( actionEvent.getID( ) == DicomFileCreatorOptionsFrame.NOTIFY_DICOMIZE_ONLY_BUTTON_PRESSED )
            {

                int seriesNumberToUseForDicomization = m_dicomFileAsSecondaryCaptureCreatorOptionsFrame.getUserSelectedSeriesNumberForDicomization( );
                String seriesDescriptionForDicomization = m_dicomFileAsSecondaryCaptureCreatorOptionsFrame.getUserSelectedSeriesDescriptionForDicomization( );

                if ( actionEvent.getActionCommand( ).compareTo( DicomFileCreatorOptionsFrame.ALSO_INCLUDE_SPATIAL_TAGS ) == 0 )
                {
                    redicomizeCompleteImageStackAsSecondaryCaptureInWorkerThread( false, true, seriesNumberToUseForDicomization, seriesDescriptionForDicomization, m_numberOfImages );
                }
                else
                {
                    redicomizeCompleteImageStackAsSecondaryCaptureInWorkerThread( false, false, seriesNumberToUseForDicomization, seriesDescriptionForDicomization, m_numberOfImages );
                }
            }
        }

        if ( ( sender instanceof DicomSliceCanvasPolyDraw ) && ( sender != this ) )
        {
            if ( actionEvent.getID( ) == NOTIFY_MOUSE_MOVED_IN_INTERACTIVE_MODE_ENABLED_EVENT )
            {
                System.out.println( "Action event NOTIFY_MOUSE_MOVED_IN_INTERACTIVE_MODE_ENABLED_EVENT !");

                handleInteractiveSegmentationPrototypeRegionClicked( ((DicomSliceCanvasPolyDraw) sender ).getLastInteractivelyIndicatedPoint( ) );
                determineStatisticsOfAllPointsAsSeed( );
                update( true );
            }
        }


        if ( sender == m_importROI )
        {
            handleImportOfROIData( );
        }
        
        if (  sender == m_writeDetailedReportOfAllROIsofAllImagesMenuItem )
        {
            createDetailedReportFilesOfAllRegioPolygonsOfAllImages( );
        }
        
        if ( sender == m_writeCondensedReportOfAllROIsofAllImagesMenuItem )
        {
            createCondensedReportFilesOfAllRegioPolygonsOfAllImages( );
        }
    
        if ( sender == m_writeDetailedReportOfAllROIsofCurrrentlyDisplayedImageMenuItem )
        {
            createDetailedReportFilesOfAllRegioPolygonsForImageNumber(m_currentlyDisplayedImageNumber );
        }
    
        if ( sender == m_writeCondensedReportOfAllROIsofCurrrentlyDisplayedImageMenuItem )
        {
            createCondensedReportFilesOfAllRegioPolygonsOfImageNumber(m_currentlyDisplayedImageNumber );
        }
        
        if ( sender == m_openImageStackIn3DControl )
        {
            if ( m_numberOfImages > 64 )
            {
                viewImageStackIn3DControl( );
            }
            else
            {
                Dialogs.messageBox( m_parent, "3D-control can only be view if number of slices is larger than 64" );
            }
        }
        
        if ( sender == m_createReportFileOfCurrentlySelectedImageSegmentTypes )
        {
            createReportFiles( false );
        }
        
        if ( sender == m_createCondensedReportFileOfCurrentlySelectedImageSegmentType )
        {
            createReportFiles( true );
        }

        if ( sender == m_createReportFileOfDistinct3DVolumes )
        {
            determineDistinctVolumesAndCreateReportFile( );
        }

        if ( sender == m_deleteAllConvertedRegioPolygonsOfAllImagesMenuItem )
        {
            deleteAllPolygonsOfAllImages( );
        }
        
        if ( sender == m_saveConvertedRegioPolygonsOfCurrentlyDisplayedImageOnlyMenuItem )
        {
            saveRegionsToFile( true );
        }
        
        if ( sender == m_saveConvertedRegioPolygonsOfAllImagesMenuItem )
        {
            saveRegionsToFile( false );
        }

        if ( sender == m_statisticalPropertiesOfSelectedRegioPolygonMenuItem )
        {
            MomentsAboutTheMeanOfRegionCalculator statCalc = new MomentsAboutTheMeanOfRegionCalculator( m_selectedPolygon, m_rawData, m_imageWidth, m_imageHeight, m_zoom, m_nordicScaleFactor );
            statCalc.computeStatistics( );
            ExtendedRegionPropertiesTableFrame frame = new ExtendedRegionPropertiesTableFrame( m_parent, this, "Statistical properties of regions" );
        }

        if ( sender == m_loadConvertedRegioPolygonsOfCurrentlyDisplayedImageOnlyMenuItem )
        {
            loadRegionsFromFile( true, true );
        }
        
        if ( sender == m_loadConvertedRegioPolygonsOfAllImagesMenuItem )
        {
            loadRegionsFromFile( false, true );
        }
        
        if ( sender == m_eliminateOverlayPixelsInsideFocusRegion )
        {
            eliminateOverlayPixelsOfSelectedOverlaysInsideCurrentlySelectedROI( );
        }

        if ( sender == this.m_importDTIOverlayFile )
        {
            importBrainvoyagerFBRFileAsImageOverlay( );
        }

        if ( sender == m_importFMRIOverlayFile )
        {
            importBrainvoyagerVOIFileAsImageOverlay( );
        }

        if ( sender == m_reDicomizeImageStackWithOverlays )
        {
            handleDicomizationOfImagesStackAsSecondaryCapture( );
        }

        if ( sender == m_reDicomizeImageStackWithOverlaysAsPlainDicom )
        {
            handleDicomizationOfImagesStackToTrueDicom( );
        }


        if ( sender == m_eliminateOverlayPixelsOutsideFocusRegion )
        {
            eliminateOverlayPixelsOfSelectedOverlaysOutsideCurrentlySelectedROI( );
        }
        
        if ( sender == m_applyLogic_NOT_toImageSegments )
        {
            handleBooleanOperationNOT( );
        }
        
        if ( sender == m_applyLogic_AND_toImageSegments )
        {
            handleBooleanOperationAND( );
        }
        
        if ( sender == m_applyLogic_OR_toImageSegments )
        {
            handleBooleanOperationOR( );
        }
        
        if ( sender == m_applyLogic_OR_toImageSegments )
        {
            handleBooleanOperationEXOR( );
        }
        
        if ( sender == m_loadImageSegmentsFromDefaultFiles )
        {
            loadOverlaysFromFiles( );
        }

        if ( sender == m_loadImageSegmentsFromUserDefinedDefaultFiles )
        {
            loadOverlaysFromFilesUserDefinedPath( );
        }
        
        if ( sender == m_saveImageSegmentsToDefaultFiles )
        {
            saveOverlaysToFiles( );
        }

        if ( sender == m_saveImageSegmentsToUserDefinedFiles )
        {
            saveOverlaysFilesToUserDefinedPath( );
        }


        if ( sender == m_deleteAllImageSegments )
        {
            deleteSelectedOverlayImageSegments( );
        }
        
        if ( sender == m_convertToRegioPolygonsCurrentlyDisplayedOnly )
        {
            performConversionOfImageSegmentOverlayToRegioPolygons( false );           
        } 

        if ( sender == m_convertToRegioPolygonsForAllImages )
        {
            performConversionOfImageSegmentOverlayToRegioPolygons( true );
        }
        
        if ( sender == m_scaleRegioPolygon )
        {
            m_scaleFactorDefinitionFrame = new EnterDecimalValueFrame( m_parent, "Contour scaling factor", 1, "Scale contour", "0.000" );
            m_scaleFactorDefinitionFrame.addActionListener( this );
        }
        if( sender == m_scaleFactorDefinitionFrame )
        {
            double scaleFactor = new Double( m_scaleFactorDefinitionFrame.getText( ) ).doubleValue( );
            // System.out.println( "scale factor " + scaleFactor );
            m_selectedPolygon.setScalingFactor( (float)scaleFactor );
            repaint( );
        }
        
        if ( sender == m_computePropertiesofSelectedROIForAllSlicesForAllRegions )
        {
            LinkedList< LinkedList <ImageRegionProperties> > allROIInformation = computePropertiesOfSelectedROIForAllImagesAndAllRegionsOfCurrentImage( );
            Iterator it = allROIInformation.iterator( );
            
            String[] columnNames = new String[ 2 * m_polygonsVector.get(m_currentlyDisplayedImageNumber ).size( ) ];
            
            
            // 2D data structure to store results into
            double[][] values = new double[ columnNames.length ][ m_numberOfImages ];
            int counter = 0;
            
            while ( it.hasNext( ) )
            {
                LinkedList<ImageRegionProperties> llist = ( LinkedList<ImageRegionProperties> )it.next( );
                
                columnNames[2 * counter ] = llist.get( counter ).getImageRegionName( ) + " (Average)";
                columnNames[2 * counter + 1] = llist.get( counter ).getImageRegionName( ) + " (St. dev.)";
                
                for ( int i = 0; i < m_numberOfImages; i++ )
                {
                    values[2 * counter][i] = llist.get( i ).getMeanPixelValue( );
                    values[2 * counter + 1][i] = llist.get( i ).getStandardDeviationOfPixelValues( );
                }    
                counter++;
            }
            
            // Now write the information into a text file
            FileInfo fInfo1 = m_dicomImageSeriesData.getFileInfo( 0 );
            String fileName = getFilePathAndName( fInfo1.getPatientName( ) + "_" + fInfo1.getSeriesDescription( ) + "_" + "detailed_report_file_of_all_regions_for_all_images.txt", "regio_rep", "Save" );
            try
            {
             
               
                // try to create create and open a RandomAccessfile
                FileWriter writer = new FileWriter( new File( fileName ) );
                    
                for ( int i = 0; i < columnNames.length; i++ )
                {
                    writer.write( columnNames[i] );
                    writer.write( "\t" );
                }
            
                writer.write( "\n" );
            
                for ( int j = 0; j < m_numberOfImages; j++ )
                {
                    for ( int i = 0; i < columnNames.length; i++ )
                    {
                        writer.write( "" + values[i][j] );
                        writer.write( "\t" );
                    }
                    writer.write( "\n" );
                }
                writer.close( );
            } 
            catch ( IOException ioe )
            {   
                ioe.printStackTrace( );
            }
        }
        
        if ( sender == m_computePropertiesofSelectedROIForAllSlicesOfCurrentlySelectedRegionOnly )
        {
            if ( m_selectedPolygon != null )
            {
                
                LinkedList<ImageRegionProperties> regionPropertiesList = computePropertiesOfSelectedROIForAllImages( );
                
                double[] xaxis = new double[m_numberOfImages];
                double[] meanROIValues = new double[m_numberOfImages];
                double[] stdDevsOfROIValues = new double[m_numberOfImages];
                
                for ( int i = 0; i < m_numberOfImages; i++ )
                {
                    //xaxis[i] = regionPropertiesList.get( i ).getImagePosition( );
                    xaxis[i] = (double)i;
                    meanROIValues[i] = regionPropertiesList.get( i ).getMeanPixelValue( );
                    stdDevsOfROIValues[i] = regionPropertiesList.get( i ).getStandardDeviationOfPixelValues( );
                }
                
                XYPlotFrame avPlot = new XYPlotFrame( "Average pixel values for all images", null );
                Curve avCurve = new Curve( xaxis, meanROIValues, Color.black );
                avPlot.addCurve( avCurve, true );
                
                XYPlotFrame stDevPlot = new XYPlotFrame( "St.dev. of pixel values for all images", null );
                Curve stDevCurve = new Curve( xaxis, stdDevsOfROIValues, Color.black );
                stDevPlot.addCurve( stDevCurve, true );
                
                SignalWriter averageWriter = new SignalWriter( "Write average ROI-pixel values of all images", xaxis, meanROIValues, null, null );
                SignalWriter stDevWriter = new SignalWriter( "Write standard deviations of ROI-pixel values of all images", xaxis, stdDevsOfROIValues, null, null );
                
            }
        }
        
        
        if ( sender == m_performSimple3DCut )
        {
            m_cutting3DPolygon =  m_selectedPolygon;
            perform3DCut( m_cutting3DPolygon );
            notifyActionListeners( CUT_3D_PERFORMED_EVENT );
        }
        
        if ( sender == m_displaySliceNumber )
        {
            m_selectImageNumberToDisplayDefined = new EnterDecimalValueFrame( m_parent, "Image nummer to display", m_currentlyDisplayedImageNumber, "Change displayed image number", "0" );
            m_selectImageNumberToDisplayDefined.addActionListener( this );
        }
        
        if ( sender == m_selectImageNumberToDisplayDefined )
        {
            if ( actionEvent.getID( ) == EnterDecimalValueFrame.APPLY_EVENT )
            {
                displayDicomImage( new Integer( m_selectImageNumberToDisplayDefined.getText( ) ).intValue( ) );   
                drawImage( false );
                notifyActionListeners( DISPLAYED_IMAGE_NUMBER_CHANGED_EVENT  );
            }
        }
        
        
        // check whether sender is of DicomSliceCanvas-type
        if ( sender instanceof DicomSliceCanvasPolyDraw )
        {

//
//            if ( ( actionEvent.getID( ) == NOTIFY_MINIMUM_IMAGE_NUMBER_FOCUS_CHANGE_EVENT ) && ( sender != this ) )
//            {
//                m_minimumImageNumberToFocus = ((DicomSliceCanvasPolyDraw)actionEvent.getSource( ) ).getMinimumImageNumberToFocus( );
//                m_ImageStackMinimumImageNumberThatHasFocus.setText( "Set minimum image number having focus ( " + m_minimumImageNumberToFocus + " )" );
//
//                notifyActionListeners( NOTIFY_MINIMUM_IMAGE_NUMBER_FOCUS_CHANGE_EVENT );
//            }
//
//            if ( ( actionEvent.getID( ) == NOTIFY_MAXIMUM_IMAGE_NUMBER_FOCUS_CHANGE_EVENT ) && ( sender != this ) )
//            {
//                m_maximumImageNumberToFocus = ((DicomSliceCanvasPolyDraw)actionEvent.getSource( ) ).getMaximumImageNumberToFocus( );
//                m_ImageStackMaximumImageNumberThatHasFocus.setText( "Set maximum image number having focus ( " + m_maximumImageNumberToFocus + " )" );
//
//                notifyActionListeners( NOTIFY_MAXIMUM_IMAGE_NUMBER_FOCUS_CHANGE_EVENT );
//            }
//
//
            if ( actionEvent.getID( ) == DISPLAYED_IMAGE_NUMBER_CHANGED_EVENT )
            {
                displayDicomImage( ((DicomSliceCanvasPolyDraw)sender).getCurrentlyDisplayedImageNumber( ) );
                updateRegionPopupMenu( );
                drawImage( false ); 
            }
            
            if ( actionEvent.getID( ) == DISPLAY_NEXT_IMAGE_OF_SLICE_STACK_EVENT )
            {
                System.out.println( "next" );
                
                m_currentlyDisplayedImageNumber = m_currentlyDisplayedImageNumber + 1;

                m_currentlyDisplayedDTIOverlayImageNumber = m_currentlyDisplayedImageNumber;
                m_currentlyDisplayedfMRIOverlayImageNumber = m_currentlyDisplayedImageNumber;

                displayDicomImage(m_currentlyDisplayedImageNumber );
                updateRegionPopupMenu( );
                drawImage( false ); 
                
            }
            
            if ( actionEvent.getID( ) == DISPLAY_PREVIOUS_IMAGE_OF_SLICE_STACK_EVENT )
            {
                System.out.println( "previous" );
                
                m_currentlyDisplayedImageNumber = m_currentlyDisplayedImageNumber - 1;

                m_currentlyDisplayedDTIOverlayImageNumber = m_currentlyDisplayedImageNumber;
                m_currentlyDisplayedfMRIOverlayImageNumber = m_currentlyDisplayedImageNumber;

                displayDicomImage(m_currentlyDisplayedImageNumber );
                updateRegionPopupMenu( );
                drawImage( false ); 
            }
            
        }
        
        if ( sender == m_changePixelValuesBasedonUserThressholds )
        {
            m_pixValReplacementFrame = new PixelValueReplacementFrame( m_rawDataMinVal, m_rawDataMaxVal );
            m_pixValReplacementFrame.addActionListener( this );
        }  
        
        if ( sender == m_pixValReplacementFrame )
        {
            if ( actionEvent.getID( ) == PixelValueReplacementFrame.APPLY_EVENT )
            {
                if ( m_pixValReplacementFrame.getSeletedReplcementOption( ) == PixelValueReplacementFrame.REPLACE_ONLY_LOWER_THAN )
                {
                    replaceRawDataPixels( m_pixValReplacementFrame.getSeletedReplcementOption( ), 
                                          m_pixValReplacementFrame.getMinimumPixelValue( ),
                                          m_pixValReplacementFrame.getMaximumPixelValue( ),
                                          m_pixValReplacementFrame.getNewPixelValue( ) );
                }
                else if ( m_pixValReplacementFrame.getSeletedReplcementOption( ) == PixelValueReplacementFrame.REPLACE_ONLY_HIGHER_THAN )
                {
                    replaceRawDataPixels( m_pixValReplacementFrame.getSeletedReplcementOption( ), 
                                          m_pixValReplacementFrame.getMinimumPixelValue( ),
                                          m_pixValReplacementFrame.getMaximumPixelValue( ),
                                          m_pixValReplacementFrame.getNewPixelValue( ) );
                }
                else if ( m_pixValReplacementFrame.getSeletedReplcementOption( ) == PixelValueReplacementFrame.REPLACE_INSIDE_INTERVAL )
                {
                    replaceRawDataPixels( m_pixValReplacementFrame.getSeletedReplcementOption( ), 
                                          m_pixValReplacementFrame.getMinimumPixelValue( ),
                                          m_pixValReplacementFrame.getMaximumPixelValue( ),
                                          m_pixValReplacementFrame.getNewPixelValue( ) );
                }
                else if ( m_pixValReplacementFrame.getSeletedReplcementOption( ) == PixelValueReplacementFrame.REPLACE_OUTSIDE_INTERVAL )
                {
                    replaceRawDataPixels( m_pixValReplacementFrame.getSeletedReplcementOption( ), 
                                          m_pixValReplacementFrame.getMinimumPixelValue( ),
                                          m_pixValReplacementFrame.getMaximumPixelValue( ),
                                          m_pixValReplacementFrame.getNewPixelValue( ) );
                }
            }
        }
        
        if ( sender instanceof RegionPropertiesTable )
        {
            if ( actionEvent.getID( ) == RegionPropertiesTable.DELETE_EVENT )
            {
                String regionToDelete = actionEvent.getActionCommand( );
                
                //both not good
                        
                deletePolygon( regionToDelete );
                
            }
        }
        
        if ( sender instanceof DicomSliceCanvas )
        {
            if ( actionEvent.getID( ) == DicomSliceCanvas.WINDOWING_EVENT )
            {
                setColormapWithoutNotification( ((DicomSliceCanvas)sender).getBrightness( ), ((DicomSliceCanvas)sender).getContrast() );
            }
        }
        
        if ( sender instanceof DicomSliceCanvas )
        {
            if ( actionEvent.getID( ) == DicomSliceCanvas.COLOR_MAP_CHANGED_EVENT )
            {
                setColormapTypeWithoutNotification( ((DicomSliceCanvas)sender).getColormapType( ) );
            }
        }
        
         if ( sender instanceof DicomSliceCanvas )
        {
            if ( actionEvent.getID( ) == DicomSliceCanvasPolyDraw.IMAGE_ZOOM_EVENT )
            {
                setZoomWithoutNotification( ((DicomSliceCanvas)sender).getZoom( ) );
            }
            m_parent.pack( );
        }
        
        if ( sender == m_doubleImageDisplay )
        {
            setZoom( m_zoom * 2 );
            notifyActionListeners( IMAGE_ZOOM_EVENT );
            definePagerLocation( );
        }
        
        if ( sender == m_halfImageDisplay )
        {
            setZoom( m_zoom / 2 );
            notifyActionListeners( IMAGE_ZOOM_EVENT );
            definePagerLocation( );
        }
        if ( sender == m_rotateAbout90Degrees )
        {
            rotateImageCounterClockwise90Degrees( );
        }
        
        if ( sender == m_flipHorizontally )
        {
            flipHorizontally( );
        }
        
        if ( sender == m_flipVertically )
        {
            flipVertically( );
        }
        
        if ( sender == m_editDicomHeaderPropertiesMenuItem  )
        {
            DicomTagsManipulationFrame m_DicomTagsManipulationFrame = new DicomTagsManipulationFrame( (JFrame)m_parent, 
                                                                                                      m_fileInfo.getPatientRelatedDicomTagsAndValues( ),
                                                                                                      m_fileInfo.getMRRelatedlDicomTagsAndValues( ),
                                                                                                      m_fileInfo.getStudySeriesInstanceTagsAndValues( ),
                                                                                                      m_fileInfo.getOtherDicomTagsAndValues( ),
                                                                                                      m_fileInfo.getGeomtryRelatedDicomTagsAndValues( ) );
        }
        
        if ( sender == m_dicomHeaderPropertiesMenuItem )
        {
            DicomHeaderViewer headerViewer = new DicomHeaderViewer( m_parent, m_fileInfo );
        }
        else if ( sender == m_siemensSpecificPropertiesMenuItem )
        {
            DicomSiemensSpecificFileHeaderViewer siemensViewer = new DicomSiemensSpecificFileHeaderViewer( m_parent, m_siemensSpecificData );
        }
        else if ( sender == m_displayGrayMenuItem )
        {
            setColormapType( CMAP_GRAYSCALE );
            update( true );
        }
        else if ( sender == m_displayColorMenuItem )
        {
            setColormapType( CMAP_COLOR );
            update( true );
        }
        else if ( sender == m_displayColorInverseMenuItem )
        {
            setColormapType( CMAP_COLOR_INVERSE );
            update( true );
        }
        else if ( sender == m_addROI )
        {
            setMode( POLYGON_DRAW_MODE );
            notifyActionListeners( ADD_ROI_MODE_SELECTED_EVENT );
        }
        else if ( sender == m_editROI )
        {
            setMode( RESIZE_MODE );
            notifyActionListeners( RESIZE_ROI_MODE_SELECTED_EVENT );
        }
        else if ( sender == m_deleteROI )
        {
            setMode( DELETE_MODE );
            notifyActionListeners( DELETE_ROI_MODE_SELECTED_EVENT );
        }
        else if ( sender == m_moveROI )
        {
            setMode( DRAG_MODE  );
            notifyActionListeners( MOVE_ROI_MODE_SELECTED_EVENT );
        }
        else if ( sender == m_selectROIButton )
        {
            setMode( SELECT_MODE );
            notifyActionListeners( SELECT_ROI_MODE_SELECTED_EVENT );
        }
        else if ( sender == m_cutROI )
        {
            setMode( CUT_MODE );
            notifyActionListeners( CUT_ROI_MODE_SELECTED_EVENT );
        }
        else if ( sender == m_addStackedROIMode )
        {
            setMode( ADD_STACKED_ROIS_MODE );
            
            m_scaledReferenceFrame = new ScaledROIDefintionFrame( );
            m_scaledReferenceFrame.addActionListener( this );
            m_scaledReferenceFrame.setVisible( true );
            
            notifyActionListeners( ADD_STACKED_ROI_MODE_EVENT );
        }
        else if ( sender == m_showWindowingScale )
        {
            if ( m_showWindowingScale.isSelected( ) == true )
            {
                scaleSetVisible( true );
                update( );
            }
            else
            {
                scaleSetVisible( false );
                update( );
            }
        }
        else if ( sender == m_showImageAnnotation )
        {
            if ( m_showImageAnnotation.isSelected( ) == true )
            {
                m_drawAnnotation = true;
                update( );
            }
            else
            {
                m_drawAnnotation = false;
                update( );
            }
        }
        else if ( sender == m_copyRegioPolygon )
        {
            m_clipBoard.setPolygon( m_selectedPolygon, getZoom( ) );
        }
        else if ( sender == m_affineTransformationRegioPolygon )
        {
            this.handleAffineTransformOfPolygon( );
//            if ( getSelectedPolygon( ) != null )
//            {
//                m_regionManipulationFrame = new RegionManupulationFrame(  m_parent, this.getSelectedPolygon( ), this, false, "Affine transformation of regions" );
//            }
//            else
//            {
//                Dialogs.messageBox( m_parent, "No currently selected polygon: no affine transform is possible" );
//            }
        }
        else if ( sender == m_copyAllRegioPolygons )
        {
            
            LinkedList< RegioPolygon > currentPolygons = new LinkedList< RegioPolygon >( );
        
            if ( m_polygonsVector.size( ) > 1 )
            {
                currentPolygons = m_polygonsVector.get(m_currentlyDisplayedImageNumber );
            }
            else
            {
                currentPolygons = m_polygonsVector.get( 0 );
            }
           
            m_clipBoard.setPolygonList( currentPolygons, getZoom( )  );
        }
        else if ( sender == m_pasteRegioPolygon )
        {
            RegioPolygon poly = m_clipBoard.getPolygon( );
            if ( poly != null )
            {
                // Create a new instance, i.e. use copy contructor!
                RegioPolygon polyCopy = new RegioPolygon( poly );
                
                // rescale the polygon
                polyCopy.setScalingFactor( getZoom( ) / m_clipBoard.getZoom( ) );
                addPolygon( polyCopy );
                repaint( );
                m_copyRegioPolygon.setEnabled( true );
                m_pixelDumpSelectedRegioPolygon.setEnabled( true );
                m_affineTransformationRegioPolygon.setEnabled( true );
                m_scaleRegioPolygon.setEnabled( true );
            }
        }
        else if ( sender == m_pasteSpecialRegioPolygon )
        {

// ??         m_regionManipulationFrame = new RegionManupulationFrame( m_parent, m_clipBoard.getPolygon( ),
//                                                                     this, true, "Paste special" );
// what is this code doing here?
//            RegioPolygon poly = m_clipBoard.getPolygon( );
//            if ( poly != null )
//            {
//                // Create a new instance, i.e. use copy contructor!
//                RegioPolygon polyCopy = new RegioPolygon( poly );
//
//                // rescale the polygon
//                polyCopy.setScalingFactor( getZoom( ) / m_clipBoard.getZoom( ) );
//                addPolygon( polyCopy );
//                repaint( );
//                m_copyRegioPolygon.setEnabled( true );
//                m_scaleRegioPolygon.setEnabled( true );
//            }
        }
        else if ( sender == m_pasteAllRegioPolygons )
        {
            LinkedList poly = m_clipBoard.getPolygonList( );
            Iterator it = poly.iterator( );
            while ( it.hasNext( ) )
            {
                RegioPolygon regPoly = new RegioPolygon( (RegioPolygon)it.next( ) );
                if ( regPoly != null )
                {
                    // Create a complete new RegioPolygon object 
                    // via a copy constructor rescale the polygon
                    RegioPolygon polyCopy = new RegioPolygon( regPoly );
                    polyCopy.setScalingFactor( this.getZoom( ) / m_clipBoard.getZoom( ) );
                    
                    addPolygon( polyCopy );
                    
                }
            }
            repaint( );
            m_copyRegioPolygon.setEnabled( true );
            m_pixelDumpSelectedRegioPolygon.setEnabled( true );
            m_affineTransformationRegioPolygon.setEnabled( true );
            m_scaleRegioPolygon.setEnabled( true );
        }
        else if ( sender == m_clipBoard )
        {
            if ( actionEvent.getID( ) == PolygonClipboard.ADDED_SINGLE_POLYGON_TO_CLIPBOARD )
            {
                m_pasteRegioPolygon.setEnabled( true );
                m_pasteSpecialRegioPolygon.setEnabled( true );
            }
            if ( actionEvent.getID( ) == PolygonClipboard.ADDED_MULTIPLE_POLYGONS_TO_CLIPBOARD )
            {
                m_pasteAllRegioPolygons.setEnabled( true );
                m_pasteSpecialRegioPolygon.setEnabled( true );

            }
        }
        else if ( sender == m_addMirroredROI )
        {
            addMirroredPolygon( );
        }
        else if ( sender == m_addMirroredVOI )
        {
            addMirroredVOIs( );
        }
        else if ( sender == m_showHistogramOfSelectedRegioPolygon )
        {
            RegioPolygonPixelDistributionHistogram hist = new RegioPolygonPixelDistributionHistogram( m_parent, 
                                                                                                      m_selectedPolygon, 
                                                                                                      getZoom(), 
                                                                                                      m_rawData, 
                                                                                                      getImageWidth( ), 
                                                                                                      getImageHeight( ), 
                                                                                                      m_nordicScaleFactor );
        }
        else if ( sender == m_renameRegion )
        {
            if ( m_selectedPolygon != null )
            {
                EnterStringFrame frame = new EnterStringFrame( m_parent, "New region name", this.m_selectedPolygon.getLabel( ), "Change region name" );
                frame.addActionListener( this );
            }
        }
        else if ( sender instanceof EnterStringFrame )
        {
            if ( actionEvent.getID( ) == 0 )
            {
                renameSelectedPolygon( ((EnterStringFrame)sender).getText( ) );   
            }
        }
        else if ( sender == m_showROIMirrorAxis )
        {
            if ( m_mirrorAxis == null )
            {
                m_mirrorAxis = new MirrorAxis( m_parent, m_imageWidth, m_imageHeight, m_zoom );
                m_mirrorAxis.addActionListener( this );
            }
            repaint( );
        }
        else if ( sender == m_showVOIMirrorAxis )
        {
            if ( m_mirrorVOIAxis == null )
            {
                m_mirrorVOIAxis = new MirrorAxis( m_parent, m_imageWidth, m_imageHeight, m_zoom );
                m_mirrorVOIAxis.addActionListener( this );
            }
            repaint( );
        }
        else if ( sender == m_editMirrorAxis )
        {
            if ( m_mirrorAxis != null )
            {
                m_mirrorAxis.changeMirrorAxis( );
            }
        }
        else if ( sender == m_editVOIMirrorAxis )
        {
            if ( m_mirrorVOIAxis != null )
            {
                m_mirrorVOIAxis.changeMirrorAxis( );
            }
        }
        else if ( sender == m_mirrorAxis )
        {
            update( );
        }
        else if ( sender == m_mirrorVOIAxis )
        {
            update( );
        }
        else if ( sender == m_tTestRegioPolygon )
        {
            
            LinkedList< RegioPolygon > currentPolygons = new LinkedList< RegioPolygon >( );
        
            if ( m_polygonsVector.size( ) > 1 )
            {
                currentPolygons = m_polygonsVector.get(m_currentlyDisplayedImageNumber );
            }
            else
            {
                currentPolygons = m_polygonsVector.get( 0 );
            }
            
            RegionTTesterFrame tTFrame = new RegionTTesterFrame( m_parent, m_rawData, 
                                                                 currentPolygons, m_imageWidth, 
                                                                 m_imageHeight, m_zoom, 
                                                                 m_nordicScaleFactor );
        }
        else if( sender == m_pixelDumpSelectedRegioPolygon )
        {
            
            DicomDefaults fdef = DicomDefaults.getInstance( );
            fdef.load( );
            
            RegioPolygonPixelDistributionHistogram hist = new RegioPolygonPixelDistributionHistogram( m_parent, m_selectedPolygon, getZoom(), m_rawData, getImageWidth( ) , getImageHeight( ), m_nordicScaleFactor );
            
            float[] selectedPixels = hist.getImagePixels( );
            
            for ( int i=0; i < selectedPixels.length; i++ )
            {
                selectedPixels[i] = selectedPixels[i]* m_nordicScaleFactor;
            }
            
            SignalWriter pixelDumpWriter = new SignalWriter( "Save Pixel of Selected ROI", selectedPixels, fdef.getProperty( "Dicom mount path" ) + System.getProperty( "file.separator" ), null );
                        
        }
        else if ( sender == m_simpleThressholdingSegmentationMenuItem )
        {
            //  also ImageSeries should be allowed : TODO
           m_simpleThressholdingSegmentationFrame = new SimpleThressholdSegmentationAnalyzerFrame( m_parent, m_rawData, getDicomFileInfo( ), m_siemensSpecificData, m_numberOfImages );
           m_simpleThressholdingSegmentationFrame.addActionListener( this );
        }
        else if ( sender == m_advancedInteractiveSegmentationMenuItem )
        {
            
            if ( m_dicomImageSeriesData == null )
            {
                m_advancedInteractiveSegmentationSegmentationFrame = new AdvancedInteractiveSegmentationAnalyzerFrame( m_parent, m_rawData, getDicomFileInfo( ), 
                                                                                                                       m_siemensSpecificData, m_numberOfImages,
                                                                                                                       m_minimumImageNumberToFocus, m_maximumImageNumberToFocus );
                m_advancedInteractiveSegmentationSegmentationFrame.addActionListener( this );
            }
            else
            {
                m_advancedInteractiveSegmentationSegmentationFrame = new AdvancedInteractiveSegmentationAnalyzerFrame( m_parent, m_dicomImageSeriesData,
                                                                                                                       m_minimumImageNumberToFocus, m_maximumImageNumberToFocus );
                m_advancedInteractiveSegmentationSegmentationFrame.addActionListener( this );

                if ( m_hasProcessedImagePixelData == true )
                {
                    m_advancedInteractiveSegmentationSegmentationFrame.setImageSeriesPixelData( m_imagePixelData );
                }

            }
        }
        else if ( sender == m_orderStatisticsSegmentationMenuItem )
        {

            // m_interactiveSegmentationMode = true;

            if ( m_dicomImageSeriesData == null )
            {
                m_orderStatisticsSegmentationAnalyzerFrame = new OrderStatisticsSegmentationAnalyzerFrame( m_parent, m_rawData, getDicomFileInfo( ), m_siemensSpecificData, m_numberOfImages );
                m_orderStatisticsSegmentationAnalyzerFrame.addActionListener( this );
            }
            else
            {
                m_orderStatisticsSegmentationAnalyzerFrame = new OrderStatisticsSegmentationAnalyzerFrame( m_parent, m_dicomImageSeriesData );
                m_orderStatisticsSegmentationAnalyzerFrame.addActionListener( this );

                if ( m_hasProcessedImagePixelData == true )
                {
                    m_orderStatisticsSegmentationAnalyzerFrame.setImageSeriesPixelData( m_imagePixelData );
                    m_orderStatisticsSegmentationAnalyzerFrame.initializeOSFFilter( );
                }
                else
                {
                    m_orderStatisticsSegmentationAnalyzerFrame.initializeOSFFilter( );
                }
            }
        }
        else if ( sender == m_deleteAllRegioPolygonsOfCurrentlyDisplayedImageOnly )
        {
            deleteAllPolygonsOfCurrentlyDisplayedImageOnly( );
        }
        else if ( sender == m_deleteAllRegioPolygonsOfAllImages )
        {
            deleteAllPolygonsOfAllImages( );
        }
        else if ( sender == m_advancedInteractiveSegmentationSegmentationFrame )
        {
            
            if  ( actionEvent.getID( ) == AdvancedInteractiveSegmentationAnalyzerFrame.ACCEPT_EVENT )
            {
                LinkedList persPolys = m_advancedInteractiveSegmentationSegmentationFrame.getPersistentPolygons( );
            
                LinkedList< RegioPolygon > currentPolygons = new LinkedList< RegioPolygon >( );
        
                if ( m_polygonsVector.size( ) > 1 )
                {
                    currentPolygons = m_polygonsVector.get(m_currentlyDisplayedImageNumber );
                }
                else
                {
                    currentPolygons = m_polygonsVector.get( 0 );
                }
                
                for ( int i = 0 ; i < persPolys.size( ) ; i++ )
                {
                    ( (RegioPolygon)persPolys.get( i ) ).setScalingFactor( 1f / m_advancedInteractiveSegmentationSegmentationFrame.getAppliedZoom( ) );
                    currentPolygons.addLast( (RegioPolygon)persPolys.get( i ) );
                }
            }
            
            LinkedList< RegioPolygon > currentPolygons = new LinkedList< RegioPolygon >( );
        
            if ( m_polygonsVector.size( ) > 1 )
            {
                currentPolygons = m_polygonsVector.get(m_currentlyDisplayedImageNumber );
            }
            else
            {
                currentPolygons = m_polygonsVector.get( 0 );
            }
            // update the context menu status
            if ( currentPolygons.size( ) > 0 )
            {
                
                m_selectedPolygon = currentPolygons.get( 0 );
                setSelectedPolygonIndex( 1 );
                
                m_renameRegion.setEnabled( true );
        
                m_showHistogramOfSelectedRegioPolygon.setEnabled( true );
        
                if ( currentPolygons.size( ) > 1 )
                {
                    m_tTestRegioPolygon.setEnabled( true );
                }
                else
                {
                    m_tTestRegioPolygon.setEnabled( false );
                }
        
                m_copyRegioPolygon.setEnabled( true );
                m_pixelDumpSelectedRegioPolygon.setEnabled( true );
                m_affineTransformationRegioPolygon.setEnabled( true );
                m_scaleRegioPolygon.setEnabled( true );
                
                if ( currentPolygons.size( ) > 1 )
                {
                    m_copyAllRegioPolygons.setEnabled( true );
                    m_deleteAllRegioPolygonsOfCurrentlyDisplayedImageOnly.setEnabled( true );
                    m_deleteAllRegioPolygonsOfAllImages.setEnabled( true );
                }
        
                if ( !m_addOneCircularROIMode.isSelected( ) )
                {
                    if ( currentPolygons.size( ) == 1 )
                    {
                        setMode( DRAG_MODE  );
                    }
                    else
                    {
                        setMode( SELECT_MODE );
                    }
                }    
            }    
            update( );
        }
        
        else if ( sender == this.m_simpleThressholdingSegmentationFrame )
        {
            
            LinkedList< RegioPolygon > currentPolygons = new LinkedList< RegioPolygon >( );
        
            if ( m_polygonsVector.size( ) > 1 )
            {
                currentPolygons = m_polygonsVector.get(m_currentlyDisplayedImageNumber );
            }
            else
            {
                currentPolygons = m_polygonsVector.get( 0 );
            }
            
            if  ( actionEvent.getID( ) == SimpleThressholdSegmentationAnalyzerFrame.ACCEPT_EVENT )
            {
                LinkedList persPolys = m_simpleThressholdingSegmentationFrame.getPersistentPolygons( );
                
                for ( int i = 0 ; i < persPolys.size( ) ; i++ )
                {
                    ( (RegioPolygon)persPolys.get( i ) ).setScalingFactor( 1f / m_simpleThressholdingSegmentationFrame.getAppliedZoom( ) );
                    currentPolygons.addLast( (RegioPolygon)persPolys.get( i ) );
                }
            }
            
            // update the context menu status
            if ( currentPolygons.size( ) > 0 )
            {
                
                m_selectedPolygon = currentPolygons.get( 0 );
                this.setSelectedPolygonIndex( 1 );
                
                m_renameRegion.setEnabled( true );
        
                m_showHistogramOfSelectedRegioPolygon.setEnabled( true );
        
                if ( currentPolygons.size( ) > 1 )
                {
                    m_tTestRegioPolygon.setEnabled( true );
                }
                else
                {
                    m_tTestRegioPolygon.setEnabled( false );
                }
        
                m_copyRegioPolygon.setEnabled( true );
                m_pixelDumpSelectedRegioPolygon.setEnabled( true );
                m_affineTransformationRegioPolygon.setEnabled( true );
                m_scaleRegioPolygon.setEnabled( true );
                
                if ( currentPolygons.size( ) > 1 )
                {
                    m_copyAllRegioPolygons.setEnabled( true );
                    m_deleteAllRegioPolygonsOfCurrentlyDisplayedImageOnly.setEnabled( true );
                    m_deleteAllRegioPolygonsOfAllImages.setEnabled( true );
                }
        
                if ( !m_addOneCircularROIMode.isSelected( ) )
                {
                    if ( currentPolygons.size( ) == 1 )
                    {
                        setMode( DRAG_MODE  );
                    }
                    else
                    {
                        setMode( SELECT_MODE );
                    }
                }
            }    
                
            update( );
        }
        else if ( sender == m_imageRestorationFilters )
        {

            m_imageRestorationFrame = new ImageRestorationFilterFrame( m_parent, m_dicomImageSeriesData );
            m_imageRestorationFrame.addActionListener( this );

            if ( m_hasProcessedImagePixelData == true )
            {
                m_imageRestorationFrame.setImageSeriesPixelData( m_imagePixelData );
            }

        }
        else if ( sender == m_morphologicFilter )
        {
            if ( m_numberOfImages == 1 )
            {
                m_morphologicFilterFrame = new MorphologicFilterFrame( m_parent, m_rawData, 
                                                                       getDicomFileInfo( ), 
                                                                       m_siemensSpecificData, 
                                                                       m_numberOfImages );
            }
            else
            {
                m_morphologicFilterFrame = new MorphologicFilterFrame( m_parent, m_dicomImageSeriesData );

                if ( m_hasProcessedImagePixelData == true )
                {
                    m_morphologicFilterFrame.setImageSeriesPixelData( m_imagePixelData );
                }

            }
            
            m_morphologicFilterFrame.addActionListener( this );
        }
        else if ( sender == m_imageRestorationFrame )
        {
            if ( actionEvent.getID( ) == ImageRestorationFilterFrame.APPLY_FILTER )
            {

                m_hasProcessedImagePixelData = true;

                setImageSeriesPixelData( m_imageRestorationFrame.getFilteredImageData( ) );
                displayDicomImage( getCurrentlyDisplayedImageNumber( ) );
                update( true );
            }
        }
        else if ( sender == m_morphologicFilterFrame )
        {
            if ( actionEvent.getID( ) == MorphologicFilterFrame.APPLY_FILTER )
            {

                m_hasProcessedImagePixelData = true;

                setImageSeriesPixelData( m_morphologicFilterFrame.getFilteredImageData( ) );
                displayDicomImage( getCurrentlyDisplayedImageNumber( ) );
                update( true );
            }
        }
    }

    /**
     * Handles the affine transform of polygon
     */
    public void handleAffineTransformOfPolygon( )
    {
        if ( getSelectedPolygon( ) != null )
        {
            m_regionManipulationFrame = new RegionManipulationFrame(  m_parent, this.getSelectedPolygon( ), this, false, "Affine transformation of regions" );
        }
        else
        {
            Dialogs.messageBox( m_parent, "No currently selected polygon: no affine transform is possible" );
        }
    }


    /**
     * Replaces all blanks by "\  " in order to be interpreted
     * correctly by the Windows operating system
     * @param cmd
     * @return
     */
    private String eliminateIllegalFileSystemCharacters( String cmd )
    {

        // first get rid ofm blancs
        cmd = cmd.trim( );

        cmd = cmd.replace( ' ', '_' );
        cmd = cmd.replace( ',', '_' );
        cmd = cmd.replace( ';', '_' );
        cmd = cmd.replace( '%', '_' );
        //cmd.replace( '*', '_' );

        cmd = cmd.replace( 'é', 'e' ); // problematic on some linux systems
        cmd = cmd.replace( 'è', 'e' ); // problematic on some linux systems
        cmd = cmd.replace( 'ö', 'o' ); // problematic on some linux systems
        cmd = cmd.replace( 'ü', 'u' ); // problematic on some linux systems
        cmd = cmd.replace( 'à', 'a' ); // problematic on some linux systems
        cmd = cmd.replace( 'ä', 'a' ); // problematic on some linux systems
        cmd = cmd.replace( 'Ö', 'O' ); // problematic on some linux systems
        cmd = cmd.replace( 'Ü', 'U' ); // problematic on some linux systems

        cmd = cmd.replace( '@', '_' ); // problematic on some linux systems
        cmd = cmd.replace( '#', '_' ); // problematic on some linux systems

        cmd = cmd.replace( '?', '_' );  // problematic on some linux systems

        cmd = cmd.replace( '¢', 'c' ); // problematic on some linux systems
        cmd = cmd.replace( '!', '_' );  // problematic on some linux systems
        cmd = cmd.replace( '§', '_' );  // problematic on some linux systems
        cmd = cmd.replace( '¦', '_' );  // problematic on some linux systems

        cmd = cmd.replace( '$', '_' );  // problematic on some linux systems
        cmd = cmd.replace( '[', '_' );  // problematic on some linux systems
        cmd = cmd.replace( ']', '_' );  // problematic on some linux systems

        cmd = cmd.replace( '^', '_' );  // problematic on some linux systems
        cmd = cmd.replace( '~', '_' );  // problematic on some linux systems

        cmd = cmd.replace( 'ç', 'c' );  // problematic on some linux systems
        cmd = cmd.replace( '~', '_' );  // problematic on some linux systems
        cmd = cmd.replace( '"', '_' );  // problematic on some linux systems

        cmd = cmd.replace("____", "_" );
        cmd = cmd.replace("___", "_" );
        cmd = cmd.replace("__", "_" );

        return cmd;
    }

   /**
    *
    * @param sentToSelectedDicomNetworkNode
    */
    private void handleDicomizationOfImagesStackToTrueDicom( )
    {
        m_trueDicomFileCreatorOptionsFrame = new DicomFileCreatorOptionsFrame( "Redicomize to new unique true DICOM", m_parent, m_fileInfo );
        m_trueDicomFileCreatorOptionsFrame.addActionListener( this );
    }
    
   /**
    *
    * @param sentToSelectedDicomNetworkNode
    */
    public void handleDicomizationOfImagesStackAsSecondaryCapture( )
    {
        m_dicomFileAsSecondaryCaptureCreatorOptionsFrame = new DicomFileCreatorOptionsFrame( "Redicomize to secondary capture DICOM", m_parent, m_fileInfo );
        m_dicomFileAsSecondaryCaptureCreatorOptionsFrame.addActionListener( this );
    }


/**
     * Performs the dicomization in an seperate thread to keep the GUI responsive!
     * @param sentToSelectedDicomNetworkNode
     * @param includeGeometryTags
     */
    public void redicomizeCompleteImageStackAsTrueDICOMInWorkerThread(  final boolean sentToSelectedDicomNetworkNode,
                                                                        final boolean includeGeometryTags,
                                                                        final int seriesNumberToUseForDicomization,
                                                                        final String seriesDescriptionForDicomization,
                                                                        final int totalNumberOfSlices )
    {
        m_dualProgressBar = new DualProgressBar( m_parent, 100, 100, new Dimension( 900, 150 ), false );
        m_dualProgressBar.setTitle( "Dicomizing files"  );

        m_dualProgressBar.setUpperProgress( 0 );
        m_dualProgressBar.setUpperLabel( "        Total progress " );
        m_dualProgressBar.setUpperProgressVisible( false );
        m_dualProgressBar.setLowerLabel( "<HTML><BR>     Dicomizing image number : " + 0 + "</HTML>" );

        final org.spectrim.lib.SwingWorker worker = new org.spectrim.lib.SwingWorker( )
        {

            public Object construct( )
            {
                redicomizeCompleteImageStackAsTrueDICOM( sentToSelectedDicomNetworkNode,
                                                         includeGeometryTags,
                                                         seriesNumberToUseForDicomization,
                                                         seriesDescriptionForDicomization,
                                                         totalNumberOfSlices );
                return null;
            }

            // runs in the event dispatching thread
            @Override
            public void finished( )
            {
                // After calculation set enabled again
                m_dualProgressBar.setVisible( false );
                Dialogs.messageBox( m_parent, "Redicomizing image stack finished!" );
            }
        };
        worker.start( );
    }


    /**
     * Performs the dicomization in an seperate thread to keep the GUI responsive!
     * @param sentToSelectedDicomNetworkNode
     * @param includeGeometryTags
     */
    public void redicomizeCompleteImageStackAsSecondaryCaptureInWorkerThread(  final boolean sentToSelectedDicomNetworkNode,
                                                             final boolean includeGeometryTags,
                                                             final int seriesNumberToUseForDicomization,
                                                             final String seriesDescriptionForDicomization,
                                                             final int totalNumberOfSlices )
    {
        m_dualProgressBar = new DualProgressBar( m_parent, 100, 100, new Dimension( 900, 150 ), false );
        m_dualProgressBar.setTitle( "Dicomizing files"  );

        m_dualProgressBar.setUpperProgress( 0 );
        m_dualProgressBar.setUpperLabel( "        Total progress " );
        m_dualProgressBar.setUpperProgressVisible( false );
        m_dualProgressBar.setLowerLabel( "<HTML><BR>     Dicomizing image number : " + 0 + "</HTML>" );

        final org.spectrim.lib.SwingWorker worker = new org.spectrim.lib.SwingWorker( )
        {

            public Object construct( )
            {
                redicomizeCompleteImageStackAsSecondaryCapture( sentToSelectedDicomNetworkNode,
                                              includeGeometryTags,
                                              seriesNumberToUseForDicomization,
                                              seriesDescriptionForDicomization,
                                              totalNumberOfSlices );
                return null;
            }

            // runs in the event dispatching thread
            @Override
            public void finished( )
            {
                // After calculation set enabled again
                m_dualProgressBar.setVisible( false );
                Dialogs.messageBox( m_parent, "Redicomizing image stack finished!" );
            }
        };
        worker.start( );
    }

    /**
     * Redicomizes all images with overlays and store them in the file system.
     */
    private void redicomizeCompleteImageStackAsSecondaryCapture( boolean sentToSelectedDicomNetworkNode,
                                                                 boolean includeGeometryTags,
                                                                 int seriesNumberToUseForDicomization,
                                                                 String seriesDescriptionForDicomization,
                                                                 int totalNumberOfSlices )
    {
        
        // do something usefull with infos one gets from the panel ...

        int currentlyDisplayedImage = m_currentlyDisplayedImageNumber;

        m_currentlyDisplayedDTIOverlayImageNumber = m_currentlyDisplayedImageNumber;
        m_currentlyDisplayedfMRIOverlayImageNumber = m_currentlyDisplayedImageNumber;
        String uid_prefix = "1.3.12.2.1107.5.2";

        DicomNetworkNodeDefaults dnwnd = DicomNetworkNodeDefaults.getInstance( );
        String IP_address = dnwnd.getProperty( DicomNetworkNodeDefaults.LOCAL_HOST_IP_ADDRESS );
        long currentTime = System.currentTimeMillis( );
        String seriesInstanceUID = uid_prefix + IP_address + currentTime;


        for ( int i = 0; i < totalNumberOfSlices; i++ )
        {

            m_dualProgressBar.setLowerProgress( (int)( 100.0 * (double)i / totalNumberOfSlices ) );
            m_dualProgressBar.setLowerLabel( "    Dicomizing image number : " +  i );

            // first display the image such that the user sees that something is happening...
            displayDicomImage( i );
            update( true );


            String patName = getDicomFileInfo( ).getPatientName( );
            String accessionNumber = getDicomFileInfo( ).getAccessionNumber( );
            String examDate = getDicomFileInfo( ).getStudyDate( );
            String examTime = getDicomFileInfo( ).getStudyTime( );
            int seriesNumber = new Integer( getDicomFileInfo( ).getSeriesNumber( ).trim( ) ).intValue( );
            int instanceNumber = new Integer( getDicomFileInfo( ).getInstanceNumber( ).trim( ) ).intValue( );

            DicomNetworkNodeDefaults fdef = DicomNetworkNodeDefaults.getInstance( );

            String workingDir = fdef.getProperty( DicomNetworkNodeDefaults.CURRENT_INTERMEDIATE_DICOM_CREATION_DIRECTORY );

            System.out.println( System.getProperty( "file.separator" ) );
            
            
            String imageSubDirString  = patName + "_" + examDate + "_" + accessionNumber;
            imageSubDirString = eliminateIllegalFileSystemCharacters( imageSubDirString );
            
            imageSubDirString = workingDir +  System.getProperty( "file.separator" ) + imageSubDirString;

            File subDir = new File( imageSubDirString );

            if ( !subDir.exists( ) )
            {
                subDir.mkdir( );
            }

            String localImageFileNameBase = System.getProperty( "file.separator" ) +
                                            patName + "_" + accessionNumber + "_" + examDate +
                                            "_" + examTime + "_" + ( seriesNumber + 1000 ) + "_" + instanceNumber ;

            localImageFileNameBase = imageSubDirString + System.getProperty( "file.separator" ) + eliminateIllegalFileSystemCharacters( localImageFileNameBase );
            
            // now paint the graphics to the image
            BufferedImage image = new BufferedImage( this.getWidth( ) , this.getHeight( ),
                                                     BufferedImage.TYPE_3BYTE_BGR );

            Graphics g = image.createGraphics( );
            g.translate( 0, 0 );
            this.printAll( g );

            if ( g != null )
            {
                g.dispose( );
            }

            String totalJPGFileName = localImageFileNameBase + ".jpg";
            
                    
            File f = new File( totalJPGFileName );
            
            
            try
            {
                ImageIO.write( image, "jpg", f );
            }
            catch ( IOException e )
            {
                System.out.print( "Problem writing local image file in DicomFormattedScreenShotCreator::createIntermediateImageOfScreenshot :\n" );
                e.printStackTrace( );
            }

            // DicomFormattedScreenShotCreator dicomCreator = new DicomFormattedScreenShotCreator( this, getDicomFileInfo( ), "DIN Bildlabor", includeGeometryTags );

            DicomFormattedScreenShotCreator dicomCreator = new DicomFormattedScreenShotCreator( this, getDicomFileInfo( ), includeGeometryTags,
                                                                                                totalJPGFileName, seriesNumberToUseForDicomization,
                                                                                                seriesDescriptionForDicomization, seriesInstanceUID,
                                                                                                instanceNumber );

            // boolean returnValue = dicomCreator.createDicomFileInLocalFileSystem( );

            String m_dicomFileName = dicomCreator.getStampedDicomFileReference( );
            if ( sentToSelectedDicomNetworkNode == true )
            {
                int selectedRemoteDicomNodeNumber = DicomNetworkNodes.getInstance( ).getSelectedRemoteDicomNodeNumber( );
                DicomFileOverNetworkTransferrer dcmTransferrer = new DicomFileOverNetworkTransferrer( selectedRemoteDicomNodeNumber, dicomCreator.getStampedDicomFileReference( ) );
                dcmTransferrer.performDicomSendOfLocalDicomFileToSelectedRemoteNode( );
            }
        }

        // set value back to keep GUI in sync ..
        m_currentlyDisplayedImageNumber = currentlyDisplayedImage;
        m_currentlyDisplayedDTIOverlayImageNumber = m_currentlyDisplayedImageNumber;
        m_currentlyDisplayedfMRIOverlayImageNumber = m_currentlyDisplayedImageNumber;

        displayDicomImage( m_currentlyDisplayedImageNumber );
        update( true );
    }



    /**
     * Redicomizes all images with overlays and store them in the file system.
     */
    private void redicomizeCompleteImageStackAsTrueDICOM( boolean sentToSelectedDicomNetworkNode,
                                                          boolean includeGeometryTags,
                                                          int seriesNumberToUseForDicomization,
                                                          String seriesDescriptionForDicomization,
                                                          int totalNumberOfSlices )
    {

        // do something usefull with infos one gets from the panel ...

        int currentlyDisplayedImage = m_currentlyDisplayedImageNumber;

        m_currentlyDisplayedDTIOverlayImageNumber = m_currentlyDisplayedImageNumber;
        m_currentlyDisplayedfMRIOverlayImageNumber = m_currentlyDisplayedImageNumber;
        String uid_prefix = "1.3.12.2.1107.5.2";

        DicomNetworkNodeDefaults dnwnd = DicomNetworkNodeDefaults.getInstance( );
        String IP_address = dnwnd.getProperty( DicomNetworkNodeDefaults.LOCAL_HOST_IP_ADDRESS );
        long currentTime = System.currentTimeMillis( );
        String seriesInstanceUID = uid_prefix + IP_address + currentTime;

        String newSeriesUID = createUID( );
        for ( int i = 0; i < totalNumberOfSlices; i++ )
        {

            m_dualProgressBar.setLowerProgress( (int)( 100.0 * (double)i / totalNumberOfSlices ) );
            m_dualProgressBar.setLowerLabel( "    Dicomizing image number : " +  i );

            // first display the image such that the user sees that something is happening...
            displayDicomImage( i );
            update( true );

            String fileRef = m_dicomImageSeriesData.getDicomImageDataObject( i ).getFileNameAndPath( );

            String path = fileRef.substring( 0, fileRef.lastIndexOf( "" + System.getProperty( "file.separator" ) ) );
            String fileName = fileRef.substring( fileRef.lastIndexOf( "" + System.getProperty( "file.separator" ) ) + 1, fileRef.length( ) );

            String newPath = path + "_ovl";

            File newPathInFileSystem = new File( newPath );

            if ( newPathInFileSystem.exists( ) == false )
            {
                newPathInFileSystem.mkdir( );
            }

            fileRef = newPath + System.getProperty( "file.separator" ) + eliminateIllegalFileSystemCharacters( fileName );
                 
//            String legalFileRef = eliminateIllegalFileSystemCharacters( fileRef );
//        
//            if ( fileRef.compareTo( legalFileRef ) != 0 )
//            {
//                // there were illegal symbols
//                File oldFile = new File( fileRef );
//                oldFile.renameTo( new File( legalFileRef ) );
//                fileRef = legalFileRef;
//            }
            
            createDICOMImagesWithDTIandfMRIOverlayInformation( i, seriesNumberToUseForDicomization, newSeriesUID, seriesDescriptionForDicomization, fileRef );            
                                                            
//            String fileRef = m_dicomImageSeriesData.getDicomImageDataObject( i ).getFileNameAndPath( );

            if ( sentToSelectedDicomNetworkNode == true )
            {
                int selectedRemoteDicomNodeNumber = DicomNetworkNodes.getInstance( ).getSelectedRemoteDicomNodeNumber( );
                DicomFileOverNetworkTransferrer dcmTransferrer = new DicomFileOverNetworkTransferrer( selectedRemoteDicomNodeNumber, fileRef );
                dcmTransferrer.performDicomSendOfLocalDicomFileToSelectedRemoteNode( );
            }

        }

        // set value back to keep GUI in sync ..
        m_currentlyDisplayedImageNumber = currentlyDisplayedImage;
        m_currentlyDisplayedDTIOverlayImageNumber = m_currentlyDisplayedImageNumber;
        m_currentlyDisplayedfMRIOverlayImageNumber = m_currentlyDisplayedImageNumber;

        displayDicomImage(m_currentlyDisplayedImageNumber );
        update( true );
    }


    /**
     * Returns the vector of RegioPolygons.
     */
    public LinkedList<RegioPolygon> getPolygonList( int imageNumber )
    {
        LinkedList< RegioPolygon > currentPolygons;
        if ( m_polygonsVector.size( ) == 1 )
        {
            currentPolygons = m_polygonsVector.get( 0 );
        }
        else
        {
            currentPolygons = m_polygonsVector.get( imageNumber );
        }
        return currentPolygons;
    }

    /**
     * Returns the vector of RegioPolygons
     */
    public LinkedList<RegioPolygon> getPolygonList( )
    {
        LinkedList< RegioPolygon > currentPolygons;
        if ( m_polygonsVector.size() == 1 )
        {
            currentPolygons = m_polygonsVector.get( 0 );
        }
        else
        {
            currentPolygons = m_polygonsVector.get(m_currentlyDisplayedImageNumber );
        }
        return currentPolygons;
    }
    
    /**
     * Returs the vector of RegioPolygons
     */  
    public Vector< LinkedList< RegioPolygon > > getAllPolygonsVector( )
    {
        return m_polygonsVector;
    }
    
    
    /**
     * Set the draw annotation member; if true informations about the
     * currently displayed image wil be displayed in the left upper
     * corner
     */
    public void setDrawAnnotation( boolean draw_annotation )
    {
        if ( m_showImageAnnotation != null )
        {
            m_drawAnnotation = draw_annotation;
            if ( draw_annotation == true )
            {
                m_showImageAnnotation.setSelected( true );
            }
            else
            {
                m_showImageAnnotation.setSelected( false );
            }
        }
    }
    
    /**
     * Method that returns the value of draw annotation member variable
     */
    public boolean getDrawAnnotation( )
    {
        return m_drawAnnotation;
    }
    
    /**
     * Method that the returns the name of polygon with number "polNumber"
     */
    public String getPolygonName( int polNumber )
    {
        
        LinkedList< RegioPolygon > currentPolygons = new LinkedList< RegioPolygon >( );
        
        if ( m_polygonsVector.size() > 1 )
        {
            currentPolygons = m_polygonsVector.get(m_currentlyDisplayedImageNumber );
        }
        else
        {
            currentPolygons = m_polygonsVector.get( 0 );
        }
        
        if ( polNumber < 0 || polNumber >= currentPolygons.size( ) )
        { 
            return null;
        }
        
        RegioPolygon regPol = currentPolygons.get( polNumber );
	return regPol.getLabel( );
    }
        
    /**
     * Method that enables the copy regio JMenuItem
     */
    public void enableCopyPolygon( boolean enable )
    {
        m_copyRegioPolygon.setEnabled( enable );
    }
    
    
    /**
     * Sets the image comment string
     */
    public void setImageTopTitle( String topTitle )
    {
        m_topTitle = topTitle;
    }
    
    /**
     * Sets the image comment string
     */
    public String getImageTopTitle(  )
    {
        return m_topTitle;
    }
    
    /**
     * Returns the FOV size in the X-direction
     */
    public double getFOVX( )
    {
        return m_FOVX;
    }
    
    /**
     * Returns the FOV size in the Y-direction
     */
    public double getFOVY( )
    {
        return m_FOVY;
    }
    
    /**
     * Sets the region which should be segmented by some algorithm
     */
    public void setRegionToSegment( RegioPolygon roiToSegment )
    {
        m_roiToSegment = roiToSegment;
    }
    
    /**
     * Sets the region which should be segmented by some algorithm
     */
    public RegioPolygon getRegionToSegment( )
    {
        return m_roiToSegment;
    }
    
    /**
     * Sets the region which should be segmented by some algorithm
     */
    public void removeRegionToSegment(  )
    {
        m_roiToSegment = null;
    }
    
    /**
     * Set the colored overlay 
     */
    public void setColoredOverlay( boolean[][] overlay, Color col )
    {
        m_thresholdedOverlay = overlay;
        m_colorOfThressholdedOverlay = new Color( col.getRed( )/255f, col.getGreen( )/255f, col.getBlue( )/255f, 0.3f );
    }
    
    /**
     * Draws the colored overlay over the image
     */
    private void drawColoredThresholdOverlay( Graphics g )
    {   
        if ( m_thresholdedOverlay != null )
        {
            // store current color
            Color curCol = g.getColor( );
            g.setColor( m_colorOfThressholdedOverlay );
            
            for ( int i = 0; i < m_imageWidth ; i++ )
            {
                for ( int j = 0; j < m_imageHeight; j++ )
                {
                    if ( m_thresholdedOverlay[j][i] == true )
                    {
                        g.fillRect( (int)(m_zoom * i), (int)(m_zoom * j), (int)m_zoom , (int)m_zoom );
                    }
                }
            }
            // set colot back to current before call
            g.setColor( curCol ); 
        }
    }
    
    /**
     *  Sets the overlay which contains the thresshold
     * @param overlays
     */
    public void setBinaryPixelIntensityThressholdOverlays( boolean[][][] overlays )
    {
        m_thressholdedImageSegmentOverlays = overlays;
    }   

    /**
     *  Sets the overlay which contains the thresshold
     * @param overlays
     */
    public boolean[][][]  getBinaryPixelIntensityThressholdOverlays( )
    {
        return m_thressholdedImageSegmentOverlays;
    }

     /**
     *  Sets the activation overlay 3D array
     * @param overlays
     */
    public void setActivationOverlays( boolean[][][] overlays )
    {
        // m_activationImageOverlays = overlays;
        
        if ( ( m_activationImageOverlays == null ) && ( overlays != null ) )
        {
            
            m_activationImageOverlays = new boolean[overlays.length][m_imageWidth][m_imageHeight];
            
            for ( int i = 0; i < overlays.length; i++ )
            {
                for ( int j = 0; j < m_imageWidth; j++ )
                {
                    for ( int k = 0; k < m_imageHeight; k++ )
                    {
                    
                        m_activationImageOverlays[i][j][k]= false;
                    }
                } 
            }
        

            for ( int i = 0; i < overlays.length; i++ )
            {
                for ( int j = 0; j < m_imageWidth; j++ )
                {
                    for ( int k = 0; k < m_imageHeight; k++ )
                    {

                        m_activationImageOverlays[i][j][k]= m_activationImageOverlays[i][j][k] || overlays[i][j][k];
                    }
                }
            }
        }
    }

    /**
     *  Sets DTI fiber overlay 3D array
     * @param overlays
     */
    public void setDTIFiberOverlays( boolean[][][] overlays, Color[][][] colors )
    {
        // define a new object if the current object == null
        // else merge the new overlays with the already existant overlay
        // the last defined dti fiber information is decisive
        if ( overlays != null )
        {
            if ( ( m_dtiFiberImageOverlays == null ) && ( m_dtiFiberColors == null ) )
            {
                m_dtiFiberImageOverlays = overlays;
                m_dtiFiberColors = colors;
            }
            else
            {
                for ( int i = 0; i < overlays.length; i++ )
                {
                    for ( int j = 0; j < m_imageWidth; j++ )
                    {
                        for ( int k = 0; k < m_imageHeight; k++ )
                        {
                            if ( overlays[i][j][k] == true )
                            {
                                m_dtiFiberImageOverlays[i][j][k] =  overlays[i][j][k];
                                m_dtiFiberColors[i][j][k] = colors[i][j][k];
                            }
                        }
                    }
                }
            }
        }
    }

    /**
     * In interactive segmenation mode draws the interactively determined
     * overlay based on the mouse click of the user.
     * @param g
     */
    private void drawContoursAsBooleanImageOverlay( Graphics g )
    {
        if ( m_contoursAsBooleanImageOverlays != null )
        {
            Color overlayColor = Color.RED;
            float alpha = 0.8f;
            overlayColor = new Color( overlayColor.getRed( )/255f, overlayColor.getGreen( )/255f, overlayColor.getBlue( )/255f, alpha );

            BufferedImage overlay = new BufferedImage( (int)m_zoom * m_imageWidth, (int)m_zoom * m_imageHeight, BufferedImage.TYPE_INT_ARGB );

            Graphics gb = overlay.getGraphics( );

            // store current color

            gb.setColor( overlayColor );

            for ( int i = 0; i < m_imageWidth ; i++ )
            {
                for ( int j = 0; j < m_imageHeight; j++ )
                {
                    if ( m_contoursAsBooleanImageOverlays[m_currentlyDisplayedImageNumber][i][j] == true )
                    {
                        gb.fillRect( (int)(m_zoom * i), (int)(m_zoom * j), (int)m_zoom , (int)m_zoom );
                    }
                }
            }

            g.drawImage( overlay, 0, 0, null );
        }
    }


    /**
     * In interactive segmenation mode draws the interactively determined
     * overlay based on the mouse click of the user.
     * @param g
     */
    private void drawInteractiveOverlay( Graphics g )
    {
        if ( m_interactiveSegmentationOverlay != null )
        {
            Color overlayColor = Color.GREEN;
            float alpha = 0.3f;
            overlayColor = new Color( overlayColor.getRed( )/255f, overlayColor.getGreen( )/255f, overlayColor.getBlue( )/255f, alpha );

            BufferedImage overlay = new BufferedImage( (int)m_zoom * m_imageWidth, (int)m_zoom * m_imageHeight, BufferedImage.TYPE_INT_ARGB );

            Graphics gb = overlay.getGraphics( );

            // store current color

            gb.setColor( overlayColor );

            for ( int i = 0; i < m_imageWidth ; i++ )
            {
                for ( int j = 0; j < m_imageHeight; j++ )
                {
                    if ( m_interactiveSegmentationOverlay[i][j] == true )
                    {
                        gb.fillRect( (int)(m_zoom * i), (int)(m_zoom * j), (int)m_zoom , (int)m_zoom );
                    }
                } 
            }

            g.drawImage( overlay, 0, 0, null );
        }
    }

    
    /**
     * Draws the BufferedImage object on the image context
     */
    private void drawBufferedImageMHAOverlay( Graphics g ) 
    {
        
        if ( m_mhaImageSegmentOverlays != null )
        {
            Color overlayColor = Color.YELLOW;
            float alpha = 0.3f;
            overlayColor = new Color( overlayColor.getRed( )/255f, overlayColor.getGreen( )/255f, overlayColor.getBlue( )/255f, alpha ); 
        
            BufferedImage overlay = new BufferedImage( (int)m_zoom * m_imageWidth, (int)m_zoom * m_imageHeight, BufferedImage.TYPE_INT_ARGB );
        
            Graphics gb = overlay.getGraphics( );
            
            // store current color
            
            gb.setColor( overlayColor );
            
            for ( int i = 0; i < m_imageWidth ; i++ )
            {
                for ( int j = 0; j < m_imageHeight; j++ )
                {
                    if ( m_mhaImageSegmentOverlays[m_currentlyDisplayedImageNumber][i][j] == true )
                    {
                        gb.fillRect( (int)(m_zoom * i), (int)(m_zoom * j), (int)m_zoom , (int)m_zoom );
                    }
                } 
            }
            
            g.drawImage( overlay, 0, 0, null );
        }
    }

    
    

    /**
     * Draws the BufferedImage object on the image context
     */
    private void drawBufferedImageThresholdOverlay( Graphics g ) 
    {
        if ( m_thressholdedImageSegmentOverlays != null )
        {
            Color overlayColor = Color.YELLOW;
            float alpha = 0.3f;
            overlayColor = new Color( overlayColor.getRed( )/255f, overlayColor.getGreen( )/255f, overlayColor.getBlue( )/255f, alpha ); 
        
            BufferedImage overlay = new BufferedImage( (int)m_zoom * m_imageWidth, (int)m_zoom * m_imageHeight, BufferedImage.TYPE_INT_ARGB );
        
            Graphics gb = overlay.getGraphics( );
            
            // store current color
            
            gb.setColor( overlayColor );
            
            for ( int i = 0; i < m_imageWidth ; i++ )
            {
                for ( int j = 0; j < m_imageHeight; j++ )
                {
                    if ( m_thressholdedImageSegmentOverlays[m_currentlyDisplayedImageNumber][i][j] == true )
                    {
                        gb.fillRect( (int)(m_zoom * i), (int)(m_zoom * j), (int)m_zoom , (int)m_zoom );
                    }
                } 
            }
            
            g.drawImage( overlay, 0, 0, null );
        }
    }

    /**
     * Creates a UID (e.g. SOP instance UID or other)
     * @return
     */
    private String createUID( )
    {
        String uid_prefix = "1.3.12.2.1107.5.2.99.99999.";

        DicomNetworkNodeDefaults dnwnd = DicomNetworkNodeDefaults.getInstance( );
        String IP_address = dnwnd.getProperty( DicomNetworkNodeDefaults.LOCAL_HOST_IP_ADDRESS );
        long currentTime = System.currentTimeMillis( );

        int prefixLenth = uid_prefix.length( );
        int uniqueLength = (IP_address + currentTime).length( );
        String newSOPInstanceUID = uid_prefix + IP_address + currentTime;

        String UID = uid_prefix + IP_address + currentTime;

        return UID;
    }


    /**
     * Writes DICOM files to disk having DTI and fMRI data integrated
     */

    public void createDICOMImagesWithDTIandfMRIOverlayInformation( int imageNumber, int newSeriesNumber, String seriesUID,
                                                                   String newSeriesDescription, String fileRef )
    {

        int[] imagePixelData = m_dicomImageSeriesData.getDicomImageDataObject( imageNumber ).getPixelData( );

        int maxPixelVal = org.spectrim.signalprocessing.SignalTools.getMaximum( imagePixelData );

        for ( int j = 0; j < m_imageWidth ; j++ )
        {
            for ( int k = 0; k < m_imageHeight; k++ )
            {
                if ( m_dtiFiberImageOverlays != null )
                {
                    if ( m_dtiFiberImageOverlays[imageNumber][k][j] == true )
                    {
                        imagePixelData[ j * m_imageWidth + k ] = maxPixelVal + 100;
                    }
                }
                if ( m_activationImageOverlays != null )
                {
                    if ( m_activationImageOverlays[imageNumber][k][j] == true )
                    {
                        imagePixelData[ j * m_imageWidth + k ] = maxPixelVal + 100;
                    }
                }
            }
        }
        
        m_dicomImageSeriesData.getDicomImageDataObject( imageNumber ).saveDicomImageWithNewPixelData( imagePixelData, fileRef );

        DicomTagChanger dcmTagChanger = new DicomTagChanger( fileRef );

        dcmTagChanger.convertDicom2XML( );

        try
        {
            // set new series description
            dcmTagChanger.changeDicomTaginXMLFile( "0008103E", newSeriesDescription );

            // set new series number
            dcmTagChanger.changeDicomTaginXMLFile( "00200011", "" + newSeriesNumber );

            // set new SOP Instance UID
            dcmTagChanger.changeDicomTaginXMLFile( "00080018", createUID( ) );

            // set the new Series Instance UID
            dcmTagChanger.changeDicomTaginXMLFile( "0020000E", seriesUID );

            // set new Manufacturer to bildlabor
            dcmTagChanger.changeDicomTaginXMLFile( "00080070", "DEVELOPMENTAL TOOL CURRENTLY NOT FOR CLINICAL USE" );

            // set new Institution name
            dcmTagChanger.changeDicomTaginXMLFile( "00080080", "Bildlabor DIN - DEVELOPMENTAL TOOL CURRENTLY NOT FOR CLINICAL USE" );

            dcmTagChanger.convertXML2Dicom( );

            dcmTagChanger.clean( );
        }
        catch ( Exception ex )
        {
            Logger.getLogger( DicomSliceCanvasPolyDraw.class.getName( ) ).log( Level.SEVERE, null, ex );
        }

    }

    /**
     * Writes DICOM files to disk having DTI and fMRI data integrated
     */    
    public void createAllDICOMImagesWithDTIandfMRIOverlayInformation( )
    {

        for ( int i = 0; i < m_numberOfImages; i++ )
        {
            
            int[] imagePixelData = m_dicomImageSeriesData.getDicomImageDataObject( i ).getPixelData( );
            
            int maxPixelVal = org.spectrim.signalprocessing.SignalTools.getMaximum( imagePixelData );

            for ( int j = 0; j < m_imageWidth ; j++ )
            {
                for ( int k = 0; k < m_imageHeight; k++ )
                {
                    if ( m_dtiFiberImageOverlays != null )
                    {
                        if ( m_dtiFiberImageOverlays[i][k][j] == true )
                        {
                            imagePixelData[ j * m_imageWidth + k ] = maxPixelVal + 100;
                        }
                    }
                    if ( m_activationImageOverlays != null )
                    {
                        if ( m_activationImageOverlays[i][k][j] == true )
                        {
                            imagePixelData[ j * m_imageWidth + k ] = maxPixelVal + 10;
                        }
                    }
                }
            }
            
            String fileRef = m_dicomImageSeriesData.getDicomImageDataObject( i ).getFileNameAndPath( );

            String path = fileRef.substring( 0, fileRef.lastIndexOf( "" + System.getProperty( "file.separator" ) ) );
            String fileName = fileRef.substring( fileRef.lastIndexOf( "" + System.getProperty( "file.separator" ) ) + 1, fileRef.length( ) );

            String newPath = path + "_ovl";

            File newPathInFileSystem = new File( newPath );

            if ( newPathInFileSystem.exists( ) == false )
            {
                newPathInFileSystem.mkdir( );
            }

            fileRef = newPath + System.getProperty( "file.separator" ) + fileName;

            m_dicomImageSeriesData.getDicomImageDataObject( i ).saveDicomImageWithNewPixelData( imagePixelData, fileRef );
            
        }
    }
        
        
    /**
     * Draws the BufferedImage object on the image context
     */
    private void drawDTIFiberImageThresholdOverlay( Graphics g )
    {
        if ( m_dtiFiberImageOverlays != null )
        {
            Color overlayColor = Color.RED;

            float alpha = 0.3f;

            BufferedImage overlay = new BufferedImage( (int)m_zoom * m_imageWidth, (int)m_zoom * m_imageHeight, BufferedImage.TYPE_INT_ARGB );

            Graphics gb = overlay.getGraphics( );

            for ( int i = 0; i < m_imageWidth ; i++ )
            {
                for ( int j = 0; j < m_imageHeight; j++ )
                {

                    gb.setColor( m_dtiFiberColors[m_currentlyDisplayedDTIOverlayImageNumber][i][j] );
                    if ( m_dtiFiberImageOverlays[m_currentlyDisplayedDTIOverlayImageNumber][i][j] == true )
                    {
                        gb.fillRect( (int)(m_zoom * i), (int)(m_zoom * j), (int)m_zoom , (int)m_zoom );
                    }
                }
            }

            g.drawImage( overlay, 0, 0, null );
        }
    }




     /**
     * Draws the BufferedImage object on the image context
     */
    private void drawfMRIActivationImageThresholdOverlay( Graphics g )
    {

        if ( m_activationImageOverlays != null )
        {
            Color overlayColor = Color.ORANGE;
            float alpha = 0.3f;
            overlayColor = new Color( overlayColor.getRed( )/255f, overlayColor.getGreen( )/255f, overlayColor.getBlue( )/255f, alpha );

            BufferedImage overlay = new BufferedImage( (int)m_zoom * m_imageWidth, (int)m_zoom * m_imageHeight, BufferedImage.TYPE_INT_ARGB );

            Graphics gb = overlay.getGraphics( );

            // store current color

            gb.setColor( overlayColor );
            int cntr=0;
            for ( int i = 0; i < m_imageWidth ; i++ )
            {
                for ( int j = 0; j < m_imageHeight; j++ )
                {
                    if ( m_activationImageOverlays[m_currentlyDisplayedfMRIOverlayImageNumber][i][j] == true )
                    {
                        gb.fillRect( (int)(m_zoom * i), (int)(m_zoom * j), (int)m_zoom , (int)m_zoom );
                        cntr++;
                    }
                }
            }

            System.out.println( "Currently displayedImage = " + m_currentlyDisplayedfMRIOverlayImageNumber + "   cntr = " + cntr );

            g.drawImage( overlay, 0, 0, null );
        }
    }




    /**
     * Set the Linkedlist of image segments : list conmtains the image segments 
     * as BufferedImage objects
     */
    public void setImageSegmentsAsBufferedImage( BufferedImage[] imageSegments )
    {
        m_imageSegmentsAsBufferedImage = imageSegments;
    }
    
    
    
    
    /**
     * Sets the linked list containing a of boolean[][] image segments
     * with corresponding colors
     */
    public void setImageSegmentOverlayLinkedLists( ArrayList< LinkedList<boolean[][]> > imageSegmentsOverlayList, Color[] imageSegmentColors, int[] selectedTissueTypes, float alpha )
    {
        m_imageSegmentsOverlayList = imageSegmentsOverlayList;
        m_imageSegmentColors = imageSegmentColors;
        m_selectedTissueTypes = selectedTissueTypes;
        m_alpha = alpha;
    }
    
    /**
     * Sets the linked list containing a of boolean[][] image segements
     * with corresponding colors
     */
    public ArrayList< LinkedList<boolean[][]> > getImageSegmentOverlayLinkedLists( )
    {
        return m_imageSegmentsOverlayList;
    }
    
    /**
     * Sets the composite image segments overlay image.
     * This image is an overlay in which every image segment is drawn in a 
     * different color (drawn offline) 
     */
    public void setCompositeImageSegmentsOverlay( BufferedImage compositeImage )
    {
        m_compositeImageSegmentsOverlay = compositeImage;
    }
    
    /**
     * Computes the properties of all 
     * @return
     */
    public LinkedList<ImageRegionProperties> computePropertiesOfSelectedROIForAllImages( )
    {
        LinkedList<ImageRegionProperties> roiProps = new LinkedList<ImageRegionProperties>( );
        
        // get a reference on the selected polygon        
        if ( m_selectedPolygon != null )
        {
            float zoom = 1f / getZoom( );
        
            RegioPolygon rpol = new RegioPolygon( m_selectedPolygon );
            
            Polygon roi = (Polygon)rpol.scaled( zoom );
        
            if ( roi != null )
            {
                for ( int i = 0; i < m_numberOfImages; i++ )
                {
                    FileInfo fInfo = m_dicomImageSeriesData.getFileInfo( i );
                   
                    ImageRegionProperties roiProp = new ImageRegionProperties( fInfo.getImageWidth( ), fInfo.getImageHeight( ), new Double( fInfo.getPixelDepth( ) ).floatValue( ) );
                    
                    roiProp.setImageRegionName( rpol.getLabel( ) );
                    
                    Rectangle bb = roi.getBounds( );
                    if ( bb.x < 0 )
                    {    
                        bb.x = 0;
                    }
                    if ( bb.x + bb.width >= m_imageWidth ) 
                    {
                        bb.width = m_imageWidth - bb.x - 1;
                    }
                    if ( bb.y < 0 )
                    {
                        bb.y = 0;
                    }
        
                    if ( bb.y + bb.height >= m_imageHeight ) 
                    {
                        bb.height = m_imageHeight - bb.y - 1;
                    }
        
                    int imageSize = m_imageWidth * m_imageHeight;

                    int   roiSize = 0;
                    float av = 0f;
                    float stdev = 0f;
            
                    // Calculate the average of the ROI
                    for ( int y = bb.y; y <= bb.y + bb.height; y++ ) 
                    {
                        int yoffset = y * m_imageWidth;
                        for ( int x = bb.x; x <= bb.x + bb.width; x++ ) 
                        {
                            if ( roi.contains( x, y ) ) 
                            {
                                roiSize++;
                                int xyoffset = yoffset;
                                av += m_imagePixelData[ i * m_imageHeight * m_imageWidth + x + m_imageWidth * y ];
                            }
                        }
                    }

                    av = av / roiSize;

                    // Calculate the standard deviation of the current ROI
                    float diff = 0.0f;
                    // reset the roi size
                    roiSize = 0;
        
                    for ( int y = bb.y; y <= bb.y + bb.height; y++ )
                    {
                        int yoffset = y * m_imageWidth;
                        for ( int x = bb.x; x <= bb.x + bb.width; x++ )
                        {
                            if ( roi.contains( x, y ) )
                            {
                                roiSize++;
                                diff = av - m_imagePixelData[ i * m_imageHeight * m_imageWidth + x + m_imageWidth * y ];
                                stdev += diff * diff;
                            }
                        }
                    }

                    stdev = (float)Math.sqrt( (double) stdev / ( (float)roiSize - 1.0 ) ) ;
                    
                    roiProp.setNumberOfPixels( roiSize );
                    roiProp.setImageRegionVolume( (double)(roiSize) * new Double ( m_fileInfo.getSliceThickness( ) ).doubleValue( ) *
                                                                      new Double ( m_fileInfo.getPixelHeight( ) ).doubleValue( ) *
                                                                      new Double ( m_fileInfo.getPixelWidth( ) ).doubleValue( ) );
                    roiProp.setMeanPixelValue( av );
                    roiProp.setStandardDeviationOfPixelValues( stdev );
                    roiProp.setImagePosition( (float)new Double( fInfo.getSliceLocation( ) ).doubleValue( ) );
                    roiProps.add( roiProp );
                }
            }
        }
        
        return roiProps; 
    }
    
    /**
     * Computes the properties of all 
     * @return
     */
    public LinkedList<ImageRegionProperties> computePropertiesOfPolygonROIForAllImages( RegioPolygon polygon )
    {
        LinkedList<ImageRegionProperties> roiProps = new LinkedList<ImageRegionProperties>( );
        
        // get a reference on the selected polygon        
        if ( polygon != null )
        {
            float zoom = 1f / getZoom( );
            
            Polygon roi = (Polygon)polygon.scaled( zoom );
        
            if ( roi != null )
            {
                for ( int i = 0; i < m_numberOfImages; i++ )
                {
                    FileInfo fInfo = m_dicomImageSeriesData.getFileInfo( i );
                   
                    ImageRegionProperties roiProp = new ImageRegionProperties( fInfo.getImageWidth( ),
                                                                               fInfo.getImageHeight( ),
                                                                               new Double( fInfo.getPixelDepth( ) ).floatValue( ) );
                    
                    roiProp.setImageRegionName( polygon.getLabel( ) );
                    
                    Rectangle bb = roi.getBounds( );
                    if ( bb.x < 0 )
                    {    
                        bb.x = 0;
                    }
                    if ( bb.x + bb.width >= m_imageWidth ) 
                    {
                        bb.width = m_imageWidth - bb.x - 1;
                    }
                    if ( bb.y < 0 )
                    {
                        bb.y = 0;
                    }
        
                    if ( bb.y + bb.height >= m_imageHeight ) 
                    {
                        bb.height = m_imageHeight - bb.y - 1;
                    }
        
                    int imageSize = m_imageWidth * m_imageHeight;

                    int   roiSize = 0;
                    float av = 0f;
                    float stdev = 0f;
            
                    // Calculate the average of the ROI
                    for ( int y = bb.y; y <= bb.y + bb.height; y++ ) 
                    {
                        int yoffset = y * m_imageWidth;
                        for ( int x = bb.x; x <= bb.x + bb.width; x++ ) 
                        {
                            if ( roi.contains( x, y ) ) 
                            {
                                roiSize++;
                                int xyoffset = yoffset;
                                av += m_imagePixelData[ i * m_imageHeight * m_imageWidth + x + m_imageWidth * y ];
                            }
                        }
                    }

                    av = av / roiSize;

                    // Calculate the standard deviation of the current ROI
                    float diff = 0.0f;
                    // reset the roi size
                    roiSize = 0;
        
                    for ( int y = bb.y; y <= bb.y + bb.height; y++ )
                    {
                        int yoffset = y * m_imageWidth;
                        for ( int x = bb.x; x <= bb.x + bb.width; x++ )
                        {
                            if ( roi.contains( x, y ) )
                            {
                                roiSize++;
                                diff = av - m_imagePixelData[ i * m_imageHeight * m_imageWidth + x + m_imageWidth * y ];
                                stdev += diff * diff;
                            }
                        }
                    }

                    stdev = (float)Math.sqrt( (double) stdev / ( (float)roiSize - 1.0 ) ) ;
                    
                    roiProp.setNumberOfPixels( roiSize );
                    roiProp.setImageRegionVolume( (double)( roiSize ) * new Double( m_fileInfo.getSliceThickness( ) ).doubleValue( ) *
                                                                        new Double( m_fileInfo.getPixelHeight( ) ).doubleValue( ) *
                                                                        new Double( m_fileInfo.getPixelWidth( ) ).doubleValue( ) );
                    roiProp.setMeanPixelValue( av );
                    roiProp.setStandardDeviationOfPixelValues( stdev );
                    roiProp.setImagePosition( (float) new Double( fInfo.getSliceLocation( ) ).doubleValue( ) );
                    roiProps.add( roiProp );
                }
            }
        }
        
        return roiProps; 
    }
    
    
    
    /**
     * Computes the properties of all 
     * @return
     */
    public LinkedList< LinkedList <ImageRegionProperties> > computePropertiesOfSelectedROIForAllImagesAndAllRegionsOfCurrentImage( )
    {
        // declare a structure for the output information
        LinkedList< LinkedList<ImageRegionProperties > > allROIProps = new LinkedList< LinkedList<ImageRegionProperties > >( );
        
        // get the linked list of RegioPolygons which are in currently displayed image
        LinkedList <RegioPolygon> imageRegioPolygons = m_polygonsVector.get(m_currentlyDisplayedImageNumber );
        
        for ( int i = 0; i < imageRegioPolygons.size( ) ; i++ )
        {
            // compute ROI-props for the given region
            LinkedList<ImageRegionProperties> roiProps = computePropertiesOfPolygonROIForAllImages( imageRegioPolygons.get( i ) );
            // add the ROI props to the container object
            allROIProps.add( roiProps );
        }
        
        // return all information
        return allROIProps;
    }
    
    /**
     * Computes the statistical properties of the defined image segments
     */
    public void computeStatisticalPropertiesOfImageSegmentsCurrentlyDisplayedImage( int currentlyDisplayedImage )
    {

        int backupCurrentlyDisplayedImageNumber = m_currentlyDisplayedImageNumber;

        // Following line of code is a hack to get the displaying of table
        // results in sync with displayed image!

        m_currentlyDisplayedImageNumber = currentlyDisplayedImage;
        displayDicomImage(m_currentlyDisplayedImageNumber );
        
        // Create a list to store ImagesSegmentProperties into
        m_imageSegmentPropertiesList = new LinkedList<ImageSegmentProperties>( );
        
        if ( m_imageSegmentsOverlayList.size( ) > 0 )
        {
            // First get the list of all image overlays
            LinkedList< boolean[][] > imageSegmentList = m_imageSegmentsOverlayList.get(m_currentlyDisplayedImageNumber );
        
            for ( int i = 0; i < imageSegmentList.size( ); i++ )
            {
                boolean[][] imageSegment = imageSegmentList.get( i );
            
                int cntr = 0;
                for ( int j = 0; j < m_imageWidth; j++ )
                {
                    for ( int k = 0; k < m_imageHeight; k++ )
                    {        
                        if ( imageSegment[j][k] == true )
                        {
                            cntr++;
                        }
                    }
                }
            
                int[] segmentPixelValues = new int[cntr]; 
                int cntr2 = 0;
                for ( int j = 0; j < m_imageWidth; j++ )
                {
                    for ( int k = 0; k < m_imageHeight; k++ )
                    {
                        int pos = j * m_imageWidth + k;
                        if ( imageSegment[k][j] == true )
                        {
                            segmentPixelValues[cntr2] = m_rawData[pos];
                            cntr2++;
                        }
                    }
                }
            
                ImageSegmentProperties imageSegmentProperty = new ImageSegmentProperties( );
            
                imageSegmentProperty.setImageSegmentName( "Tissue " + i );
            
                imageSegmentProperty.setNumberOfPixels( segmentPixelValues.length );
            
                imageSegmentProperty.setImageSegmentVolume( (double)( segmentPixelValues.length ) *
                                                            new Double( m_fileInfo.getPixelWidth( ) ).doubleValue( ) *
                                                            new Double( m_fileInfo.getPixelHeight( ) ).doubleValue( ) *
                                                            new Double( m_fileInfo.getSliceThickness( ) ).doubleValue( ) );
            
                float av = 0f;
                for ( int j = 0; j < segmentPixelValues.length; j++ )
                {
                    av += segmentPixelValues[j] * m_nordicScaleFactor;
                }
            
                av = av / segmentPixelValues.length;
            
                imageSegmentProperty.setMeanPixelValue( av );
            
            
                float stdev = 0f;
            
                for ( int j = 0; j< segmentPixelValues.length; j++ )
                {
                    stdev +=  ( segmentPixelValues[j] * m_nordicScaleFactor - av )* ( segmentPixelValues[j] * m_nordicScaleFactor - av );
                }
            
                stdev = (float)Math.sqrt( stdev / ( segmentPixelValues.length - 1 ) );
                    
                imageSegmentProperty.setStandardDeviationOfPixelValues( stdev );
            
                m_imageSegmentPropertiesList.add( imageSegmentProperty  ); 
            }
        }
        
        // Following line of code is a hack to get the displaying of table
        // results in sync with displayed image!
        m_currentlyDisplayedImageNumber = backupCurrentlyDisplayedImageNumber;
        displayDicomImage(m_currentlyDisplayedImageNumber );
    }
    
    /**
     * Returns the statistical properties of the defined image segments
     */
    public LinkedList<ImageSegmentProperties> getImageSegmentPropertiesofCurrentlyDisplayedImage( )
    {
        return m_imageSegmentPropertiesList;
    }


        /**
     * Computes the statistical properties of the defined image segments
     */
    public LinkedList<ImageSegmentProperties> computeStatisticalPropertiesOfImageSegmentsOfImageNumber( int imageNumber )
    {


//        int backupCurrentlyDisplayedImageNumber = m_currentlyDisplayedImage;
//
//        // Following line of code is a hack to get the displaying of table
//        // results in sync with displayed image!
//
//        m_currentlyDisplayedImage = imageNumber;
//        displayDicomImage( m_currentlyDisplayedImage );
//        drawImage( true );

        // Create a list to store ImagesSegmentProperties into
        LinkedList<ImageSegmentProperties> imageSegmentPropertiesList = new LinkedList<ImageSegmentProperties>( );

        if ( m_imageSegmentsOverlayList.size( ) > 0 )
        {
            // First get the list of all image overlays
            LinkedList< boolean[][] > imageSegmentList = m_imageSegmentsOverlayList.get( imageNumber );

            for ( int i = 0; i < imageSegmentList.size( ); i++ )
            {
                boolean[][] imageSegment = imageSegmentList.get( i );

                int cntr = 0;
                for ( int j = 0; j < m_imageWidth; j++ )
                {
                    for ( int k = 0; k < m_imageHeight; k++ )
                    {
                        if ( imageSegment[j][k] == true )
                        {
                            cntr++;
                        }
                    }
                }

                int[] segmentPixelValues = new int[cntr];
                int cntr2 = 0;
                for ( int j = 0; j < m_imageWidth; j++ )
                {
                    for ( int k = 0; k < m_imageHeight; k++ )
                    {

                        int pos = ( imageNumber * m_imageWidth * m_imageHeight ) + j * m_imageHeight + k;

                        if ( imageSegment[k][j] == true )
                        {
                            segmentPixelValues[cntr2] = m_imagePixelData[pos];
                            cntr2++;
                        }
                    }
                }

                ImageSegmentProperties imageSegmentProperty = new ImageSegmentProperties( );

                imageSegmentProperty.setImageSegmentName( "Tissue " + i );

                imageSegmentProperty.setNumberOfPixels( segmentPixelValues.length );

                imageSegmentProperty.setImageSegmentVolume( (double)( segmentPixelValues.length ) *
                                                            new Double( m_fileInfo.getPixelWidth( ) ).doubleValue( ) *
                                                            new Double( m_fileInfo.getPixelHeight( ) ).doubleValue( ) *
                                                            new Double( m_fileInfo.getSliceThickness( ) ).doubleValue( ) );

                float av = 0f;
                for ( int j = 0; j < segmentPixelValues.length; j++ )
                {
                    av += segmentPixelValues[j] * m_nordicScaleFactor;
                }

                av = av / segmentPixelValues.length;

                imageSegmentProperty.setMeanPixelValue( av );

                float stdev = 0f;

                for ( int j = 0; j < segmentPixelValues.length; j++ )
                {
                    stdev +=  ( segmentPixelValues[j] * m_nordicScaleFactor - av )* ( segmentPixelValues[j] * m_nordicScaleFactor - av );
                }

                stdev = (float)Math.sqrt( stdev / ( segmentPixelValues.length - 1 ) );

                imageSegmentProperty.setStandardDeviationOfPixelValues( stdev );

                imageSegmentPropertiesList.add( imageSegmentProperty  );
            }
        }

//        // Following line of code is a hack to get the displaying of table
//        // results in sync with displayed image!
//        m_currentlyDisplayedImage = backupCurrentlyDisplayedImageNumber;
//        displayDicomImage( m_currentlyDisplayedImage );

        return imageSegmentPropertiesList;
    }


    /**
     * Computes the statistical properties of the defined image segments
     */
    public VolumeOfInterestStatistics computeStatisticalPropertiesOfVolumeOfInterest( int volumeNumber )
    {

        // first get the
        //private ArrayList< LinkedList<boolean[][]> > m_imageVolumeOfInterestOverlayList = new ArrayList< LinkedList<boolean[][] > >( );

        double average = 0.0;
        int cntr = 0;
        double stDev = 0.0;


        for ( int i = 0; i < m_numberOfImages; i++ )
        {
            LinkedList<boolean[][]> voiList = m_imageVolumeOfInterestOverlayList.get( i );
            boolean[][] volumeOfInterest = voiList.get( volumeNumber );

            for ( int j = 0; j < m_imageWidth; j++ )
            {
                for ( int k = 0; k < m_imageHeight; k++ )
                {

                    int pos = ( i * m_imageWidth * m_imageHeight ) + j * m_imageHeight + k;

                    if ( volumeOfInterest[k][j] == true )
                    {
                        if ( m_imagePixelData != null )
                        {
                            average += m_imagePixelData[pos];
                        }
                        else
                        {
                            average += m_rawData[pos];
                        }
                        cntr++;
                    }
                }
            }
        }

        average = average / cntr;

        for ( int i = 0; i < m_numberOfImages; i++ )
        {
            LinkedList<boolean[][]> voiList = m_imageVolumeOfInterestOverlayList.get( i );
            boolean[][] volumeOfInterest = voiList.get( volumeNumber );

            for ( int j = 0; j < m_imageWidth; j++ )
            {
                for ( int k = 0; k < m_imageHeight; k++ )
                {

                    int pos = ( i * m_imageWidth * m_imageHeight ) + j * m_imageHeight + k;

                    if ( volumeOfInterest[k][j] == true )
                    {
                        if ( m_imagePixelData != null )
                        {
                            stDev +=  ( m_imagePixelData[pos] - average ) * ( m_imagePixelData[pos] - average );
                        }
                        else
                        {
                            stDev +=  ( m_rawData[pos] - average ) * ( m_rawData[pos] - average );
                        }
                    }
                }
            }
        }

        stDev = (float)Math.sqrt( (double) stDev / ( (float)cntr - 1.0 ) ) ;
        
        VolumeOfInterestStatistics volStat = new VolumeOfInterestStatistics( );

        volStat.setVolumeOfInterestName( m_imageVolumeOfInterestNameList.get( volumeNumber ) );

        FileInfo finfo;
        if ( m_dicomImageSeriesData != null )
        {
            finfo = m_dicomImageSeriesData.getFileInfo( 0 );
        }
        else
        {
            finfo = m_fileInfo;
        }

        volStat.setVolumeOfInterestVolume( cntr * new Double( finfo.getSliceThickness( ) ).doubleValue( ) *
                                                  new Double( finfo.getPixelWidth( ) ).doubleValue( ) *
                                                  new Double( finfo.getPixelHeight( ) ).doubleValue( ) );

        volStat.setVOIAverageOfPixelValues( average );
        volStat.setVOIStandardDeviationOfPixelValues( stDev );
        volStat.setNumberOfPixels( cntr );

        return volStat;
    }
    
    /**
     * Creates a report file of all RegioPolygons of all images of the image stack
     */
    public void createCondensedReportFilesOfAllRegioPolygonsOfAllImages( )
    {
        
        FileInfo fInfo1 = m_dicomImageSeriesData.getFileInfo( 0 );
        
        String fileName = getFilePathAndName( fInfo1.getSeriesDescription( ) + "_" + "condensed_report_file_of_all_regions_of_all_images.txt" , "regio_rep", "Save" );
        
        LinkedList<ImageRegionProperties> roiProps = new LinkedList<ImageRegionProperties>( );
        
        if ( fileName.length( ) != 0 )
        {
            try
            {
                DecimalFormat formatter = new DecimalFormat( "###.00" );
                
                FileWriter fileWriter = new FileWriter( fileName );
                
                FileInfo fInfo = m_dicomImageSeriesData.getFileInfo( 0 );
                
                fileWriter.write( "Patient name       : " + fInfo.getPatientName( ) + "\r\n");
                fileWriter.write( "Patient ID         : " + fInfo.getPatientID( ) + "\r\n");
                fileWriter.write( "Date of Birth      : " + fInfo.getPatientBirthDate( ) + " " );
                fileWriter.write( "Age                : " + fInfo.getPatientAge( ) + "\r\n" );
                fileWriter.write( "Examination Date   : " + fInfo.getAcquisitionDate( ) + "\r\n" );
                fileWriter.write( "Examination Time   : " + fInfo.getAcquisitionTime( ) + "\r\n" );
                fileWriter.write( "Series Number      : " + fInfo.getSeriesNumber( ).trim( ) + "\r\n" );
                fileWriter.write( "Series Description : " + fInfo.getSeriesDescription( ) + "\r\n" );
                fileWriter.write( "Pixel height       : " + fInfo.getPixelHeight( ) + "\r\n" );
                fileWriter.write( "Pixel width        : " + fInfo.getPixelWidth( ) + "\r\n" );
                fileWriter.write( "Pixel depth        : " + fInfo.getPixelDepth( ) + "\r\n" );
                fileWriter.write( "(Slice thickness   : " + fInfo.getSliceThickness( ) + ")\r\n\r\n" );
                
                double totalVolume = 0;
                int totalNumberOfPixels = 0;
                double totalMean = 0;
                int numberOfMeansAdded = 0;
                double totalStDev = 0f;
                int numberOfStDevsAdded = 0;
                
                // for ( int i = 0; i < m_numberOfImages; i++ )
                for ( int i = m_minimumImageNumberToFocus; i < m_maximumImageNumberToFocus; i++ )
                {
                   
                    LinkedList <RegioPolygon> imageRegioPolygons = m_polygonsVector.get( i );
                    
                    for ( int j = 0; j < imageRegioPolygons.size( ); j++ )
                    {
                        RegioPolygon poly = imageRegioPolygons.get( j );
                        
                        ImageRegionProperties regProp = computeRegioProperties( i, poly );
                              
                        totalVolume = totalVolume + (double)regProp.getVolumeEnclosedByRegio( );
                        totalNumberOfPixels = totalNumberOfPixels + regProp.getNumberOfPixels( );
                        
                        if ( regProp.getMeanPixelValue( ) != 0f )
                        {
                            totalMean = totalMean + regProp.getMeanPixelValue( );
                            numberOfMeansAdded++;
                        }
                        if ( regProp.getStandardDeviationOfPixelValues( ) != 0f )
                        {
                            totalStDev = totalStDev + regProp.getStandardDeviationOfPixelValues( );
                            numberOfStDevsAdded++;
                        }
                    }
                }
               
                fileWriter.write( "Values for all regions and all slices" + "\r");
                fileWriter.write( "Total number of pixels      : " + totalNumberOfPixels +  "\r");
                fileWriter.write( "Total volume                : " + totalVolume + "\r");
                fileWriter.write( "Mean  (average of all ROIs) : " + ( totalMean / numberOfMeansAdded ) + "    (N=" + numberOfMeansAdded + ")\r");
                fileWriter.write( "St.dev (average of all    ) : " + ( totalStDev / numberOfStDevsAdded ) + "    (N=" + numberOfStDevsAdded +")\r\n\r\n");

                fileWriter.write( "\r\n" );
                
                fileWriter.close( );
            }
            catch ( IOException e )
            {
                System.out.println( "Error creating file : " + fileName );
            }
        }
    }
    
    
    
    
    /**
     * Creates a report file of all RegioPolygons of all images of the image stack
     */
    public void createDetailedReportFilesOfAllRegioPolygonsForImageNumber( int imageNumber )
    {
        String fileName = getFilePathAndName( "detailed_report_file_of_all_regions_for_image_number_" + imageNumber + ".txt", "regio_rep", "Save" );
        
        LinkedList<ImageRegionProperties> roiProps = new LinkedList<ImageRegionProperties>( );
        
        if ( fileName.length( ) != 0 )
        {
            try
            {
                DecimalFormat formatter = new DecimalFormat( "###.00" );
                
                FileWriter fileWriter = new FileWriter( fileName );
                
                FileInfo fInfo = m_dicomImageSeriesData.getFileInfo( 0 );
                
                fileWriter.write( "Patient name       : " + fInfo.getPatientName( ) + "\r\n");
                fileWriter.write( "Patient ID         : " + fInfo.getPatientID( ) + "\r\n");
                fileWriter.write( "Date of Birth      : " + fInfo.getPatientBirthDate( ) + " " );
                fileWriter.write( "Age                : " + fInfo.getPatientAge( ) + "\r\n" );
                fileWriter.write( "Examination Date   : " + fInfo.getAcquisitionDate( ) + "\r\n" );
                fileWriter.write( "Examination Time   : " + fInfo.getAcquisitionTime( ) + "\r\n" );
                fileWriter.write( "Series Number      : " + fInfo.getSeriesNumber( ).trim( ) + "\r\n\r\n" );
                fileWriter.write( "Series Description : " + fInfo.getSeriesDescription( ) + "\r\n\r\n" );
                
                float totalVolume = 0f;
                
                
                fInfo = m_dicomImageSeriesData.getFileInfo( imageNumber );
                fileWriter.write( "-------------------------------------------------------\r\n" );
                fileWriter.write( "Series number  : " + fInfo.getSeriesNumber( ).trim( ) + " image number : " + imageNumber + "\r\n" );
                fileWriter.write( "Slice position : " + fInfo.getSliceLocation( ) + "\r\n" );
                fileWriter.write( "Filename       : " + fInfo.getFileName( ) + "\r\n");
                fileWriter.write( "-------------------------------------------------------\r\n\r\n" );
                    
                    
                LinkedList <RegioPolygon> imageRegioPolygons = m_polygonsVector.get( imageNumber );
                    
                for ( int j = 0; j < imageRegioPolygons.size( ); j++ )
                {
                    RegioPolygon poly = imageRegioPolygons.get( j );
                        
                    ImageRegionProperties regProp = computeRegioProperties( imageNumber, poly );
                        
                    fileWriter.write( "Name   " + regProp.getImageRegionName( ) + "\r");
                    fileWriter.write( "Pixels " + regProp.getNumberOfPixels( ) + "\r");
                    fileWriter.write( "Volume " + regProp.getVolumeEnclosedByRegio( ) + "\r");
                    fileWriter.write( "Mean   " + regProp.getMeanPixelValue( ) + "\r");
                    fileWriter.write( "St.dev " + regProp.getStandardDeviationOfPixelValues( ) + "\r\n\r\n");
                }
                
               
                fileWriter.write( "\r\n" );
                
                fileWriter.close( );
            }
            catch ( IOException e )
            {
                System.out.println( "Error creating file : " + fileName );
            }
        }
    }
    
    /**
     * Creates a report file of all RegioPolygons of all images of the image stack
     */
    public void createCondensedReportFilesOfAllRegioPolygonsOfImageNumber( int imageNumber )
    {
        String fileName = getFilePathAndName( "condensed_report_file_of_all_regions_for_image_number_" + imageNumber + ".txt", "regio_rep", "Save" );
        
        LinkedList<ImageRegionProperties> roiProps = new LinkedList<ImageRegionProperties>( );
        
        if ( fileName.length( ) != 0 )
        {
            try
            {
                DecimalFormat formatter = new DecimalFormat( "###.00" );
                
                FileWriter fileWriter = new FileWriter( fileName );
                
                FileInfo fInfo = m_dicomImageSeriesData.getFileInfo( 0 );
                
                fileWriter.write( "Patient name       : " + fInfo.getPatientName( ) + "\r\n");
                fileWriter.write( "Patient ID         : " + fInfo.getPatientID( ) + "\r\n");
                fileWriter.write( "Date of Birth      : " + fInfo.getPatientBirthDate( ) + " " );
                fileWriter.write( "Age                : " + fInfo.getPatientAge( ) + "\r\n" );
                fileWriter.write( "Examination Date   : " + fInfo.getAcquisitionDate( ) + "\r\n" );
                fileWriter.write( "Examination Time   : " + fInfo.getAcquisitionTime( ) + "\r\n" );
                fileWriter.write( "Series Number      : " + fInfo.getSeriesNumber( ).trim( ) + "\r\n\r\n" );
                fileWriter.write( "Image  Number      : " + imageNumber + "\r\n\r\n" );
                fileWriter.write( "Series Description : " + fInfo.getSeriesDescription( ) + "\r\n\r\n" );
                
                double totalVolume = 0f;
                int totalNumberOfPixels = 0;
                double totalMean = 0f;
                int numberOfMeansAdded = 0;
                double totalStDev = 0f;
                int numberOfStDevsAdded = 0;
                 
                LinkedList <RegioPolygon> imageRegioPolygons = m_polygonsVector.get( imageNumber );
                    
                for ( int j = 0; j < imageRegioPolygons.size( ); j++ )
                {
                    RegioPolygon poly = imageRegioPolygons.get( j );
                        
                    ImageRegionProperties regProp = computeRegioProperties( imageNumber, poly );
                              
                    totalVolume = totalVolume + (float)regProp.getVolumeEnclosedByRegio( );
                    totalNumberOfPixels = totalNumberOfPixels + regProp.getNumberOfPixels( );
                       
                    if ( regProp.getMeanPixelValue( ) != 0f )
                    {
                        totalMean = totalMean + regProp.getMeanPixelValue( );
                        numberOfMeansAdded++;
                    }
                    if ( regProp.getStandardDeviationOfPixelValues( ) != 0f )
                    {
                        totalStDev = totalStDev + regProp.getStandardDeviationOfPixelValues( );
                        numberOfStDevsAdded++;
                    }
                }
                
                fileWriter.write( "Values for all regions and all slices" + "\r");
                fileWriter.write( "Total number of pixels      : " + totalNumberOfPixels +  "\r");
                fileWriter.write( "Total volume                : " + totalVolume + "\r");
                fileWriter.write( "Mean  (average of all ROIs) : " + ( totalMean / numberOfMeansAdded ) + "    (N=" + numberOfMeansAdded + ")\r");
                fileWriter.write( "St.dev (average of all    ) : " + ( totalStDev / numberOfStDevsAdded ) + "    (N=" + numberOfStDevsAdded +")\r\n\r\n");

                fileWriter.write( "\r\n" );
                
                fileWriter.close( );
            }
            catch ( IOException e )
            {
                System.out.println( "Error creating file : " + fileName );
            }
        }
    }


    /**
     *
     * Writes all information of all to the ROIs to a text file.
     *
     * @param fileWriter
     */
    public void writeROIInformation( FileWriter fileWriter, boolean verbose, int beginBaseLine, int endBaseLine, int maxPerf, int startReperfusion )
    {

        try
        {
            
            if ( beginBaseLine == 0 )
            {
                beginBaseLine = 1;
            }
            
            DecimalFormat formatter = new DecimalFormat( "###.###" );

            float totalVolume = 0f;


            // output as huge list!
            fileWriter.write( "View_Title__" + "\t" + "Series_Number" + "\t" + "Instance_Number" + "\t" + "Slice_Location" + "\t" +
                              "ROI_Name____" + "\t" + "#_of_Pixels__" + "\t" + "ROI_Volume_____" + "\t" + "Mean_Pixel_Val" + "\t" +
                              "Standard_Dev" + "\t" + "Variance_____" + "\t" + "Var_of_Variance" + "\t" + "Skewness______" + "\t" +
                              "Var(Skew)___" + "\t" + "Kurtosis_____" + "\t" + "Var_of_Kurtosis" + "\n" );


            MomentsAboutTheMean[][] momentsList = new MomentsAboutTheMean[m_maximumImageNumberToFocus-m_minimumImageNumberToFocus][m_polygonsVector.get( 0 ).size( )];
            int numberOfRegions = 0;

            FileInfo fInfo = null;
            LinkedList <RegioPolygon> imageRegioPolygons = null;
            float volPerPixel = 0;

            for ( int i = m_minimumImageNumberToFocus ; i < m_maximumImageNumberToFocus ; i++ )
            {

                fInfo = m_dicomImageSeriesData.getFileInfo( i );
                int sliceNumber = i;

                imageRegioPolygons = m_polygonsVector.get( i );

                volPerPixel = new Float( fInfo.getPixelDepth( ) ).floatValue( ) * new Float( fInfo.getPixelWidth( ) ).floatValue( ) * new Float( fInfo.getPixelHeight( ) ).floatValue( );

                numberOfRegions = imageRegioPolygons.size( );

                for ( int j = 0; j < imageRegioPolygons.size( ); j++ )
                {

                    RegioPolygon poly = imageRegioPolygons.get( j );

                    int[] pixelsWithinPolygon = getPixelValues( sliceNumber, poly );

                    if ( pixelsWithinPolygon != null )
                    {

                        MomentAboutTheMeanOfArrayCalculator momCalc = new MomentAboutTheMeanOfArrayCalculator( pixelsWithinPolygon );
                        MomentsAboutTheMean moments = momCalc.getMomentsAboutTheMean( );
                        momentsList[i][j] = moments;

                        if ( verbose == true )
                        {
                            fileWriter.write( getTitle( ) + "\t" +
                                              fInfo.getSeriesNumber( ) + "\t" +
                                              i + "\t" +
                                              formatter.format( new Float( fInfo.getSliceLocation( ) ).floatValue( ) ) + "\t" );

//                            MomentAboutTheMeanOfArrayCalculator momCalc = new MomentAboutTheMeanOfArrayCalculator( pixelsWithinPolygon );
//                            MomentsAboutTheMean moments = momCalc.getMomentsAboutTheMean( );

                            fileWriter.write( poly.getLabel( )        + "\t" );
                            fileWriter.write( poly.getInnerVolume( )  + "\t" );

                            fileWriter.write( formatter.format( poly.getInnerVolume( ) * volPerPixel ) + "\t" );

                            fileWriter.write( formatter.format( moments.getMean( ) )               + "\t" );
                            fileWriter.write( formatter.format( moments.getStDev( ) )              + "\t" );
                            fileWriter.write( formatter.format( moments.getVariance( ) )           + "\t" );
                            fileWriter.write( formatter.format( moments.getVarianceOfVariance( ) ) + "\t" );
                            if ( new Double( moments.getSkewness( ) ).isInfinite( ) )
                            {
                                fileWriter.write( formatter.format( moments.getSkewness( ) )           + "\t" );
                            }
                            else
                            {
                                fileWriter.write( formatter.format( 0.0 )           + "\t" );
                            }

                            fileWriter.write( formatter.format( moments.getVarianceOfSkewness( ) ) + "\t" );

                            if ( new Double( moments.getKurtosis( ) ).isInfinite( ) )
                            {
                                fileWriter.write( formatter.format( moments.getKurtosis( ) )           + "\t" );
                            }
                            else
                            {
                                fileWriter.write( formatter.format( 0.0 )           + "\t" );
                            }

                            fileWriter.write( formatter.format( moments.getVarianceOfKurtosis( ) ) + "\t\n" );
                        }
                    }

                }    
            }

            fileWriter.write( "\r\n" );

            // compute all summarizing statistics ..
            for ( int j = 0; j < numberOfRegions; j++ )
            {

                int cntr = 0;

                double meanBase         = 0;
                double stDevBase        = 0;
                double varianceBase     = 0;
                double varOfVarBase     = 0;
                double skewnessBase     = 0;
                double varOfSkewBase    = 0;
                double kutosisBase      = 0;
                double varOfKurtosisBase = 0;

                double meanInflow       = 0;
                double stDevInflow      = 0;
                double varianceInflow   = 0;
                double varOfVarInflow   = 0;
                double skewnessInflow   = 0;
                double varOfSkewInflow  = 0;
                double kutosisInflow    = 0;
                double varOfKurtosisInflow = 0;

                double meanOutflow      = 0;
                double stDevOutflow     = 0;
                double varianceOutflow  = 0;
                double varOfVarOutflow  = 0;
                double skewnessOutflow  = 0;
                double varOfSkewOutflow = 0;
                double kutosisOutflow   = 0;
                double varOfKurtosisOutflow = 0;

                double meanReperf       = 0;
                double stDevReperf      = 0;
                double varianceReperf   = 0;
                double varOfVarReperf   = 0;
                double skewnessReperf   = 0;
                double varOfSkewReperf  = 0;
                double kutosisReperf    = 0;
                double varOfKurtosisReperf = 0;


                for ( int i = 0; i < m_maximumImageNumberToFocus - m_minimumImageNumberToFocus; i++ )
                {
                    MomentsAboutTheMean moments = momentsList[i][j];

                    if ( ( cntr >= beginBaseLine ) && ( cntr < endBaseLine ) )
                    {
                        meanBase        += moments.getMean( );
                        stDevBase       += moments.getStDev( );
                        varianceBase    += moments.getVariance( );
                        varOfVarBase    += moments.getVarianceOfVariance( );

                        if ( new Double(moments.getSkewness( ) ).isInfinite( ) )
                        {
                            skewnessBase    += moments.getSkewness( );
                        }

                        varOfSkewBase   += moments.getVarianceOfSkewness( );
                        
                        if ( new Double( moments.getKurtosis( ) ).isInfinite( ) )
                        {
                            kutosisBase     += moments.getKurtosis( );
                        }
                       
                        varOfKurtosisBase += moments.getVarianceOfKurtosis( );
                    }

                    if ( ( cntr >= endBaseLine ) && ( cntr <= maxPerf ) )
                    {
                        meanInflow        += moments.getMean( );
                        stDevInflow       += moments.getStDev( );
                        varianceInflow    += moments.getVariance( );
                        varOfVarInflow    += moments.getVarianceOfVariance( );
                        skewnessInflow    += moments.getSkewness( );
                        varOfSkewInflow   += moments.getVarianceOfSkewness( );
                        kutosisInflow     += moments.getKurtosis( );
                        varOfKurtosisInflow += moments.getVarianceOfKurtosis( );
                    }

                    if ( ( cntr >= maxPerf ) && ( cntr < startReperfusion ) )
                    {
                        meanOutflow        += moments.getMean( );
                        stDevOutflow       += moments.getStDev( );
                        varianceOutflow    += moments.getVariance( );
                        varOfVarOutflow    += moments.getVarianceOfVariance( );
                        skewnessOutflow    += moments.getSkewness( );
                        varOfSkewOutflow   += moments.getVarianceOfSkewness( );
                        kutosisOutflow     += moments.getKurtosis( );
                        varOfKurtosisOutflow += moments.getVarianceOfKurtosis( );
                    }

                    if ( ( cntr >= startReperfusion ) && ( cntr <  m_maximumImageNumberToFocus - m_minimumImageNumberToFocus ) )
                    {
                        meanReperf          += moments.getMean( );
                        stDevReperf         += moments.getStDev( );
                        varianceReperf      += moments.getVariance( );
                        varOfVarReperf      += moments.getVarianceOfVariance( );
                        skewnessReperf      += moments.getSkewness( );
                        varOfSkewReperf     += moments.getVarianceOfSkewness( );
                        kutosisReperf       += moments.getKurtosis( );
                        varOfKurtosisReperf += moments.getVarianceOfKurtosis( );
                    }

                    cntr++;
                }

                meanBase        = meanBase / ( endBaseLine - beginBaseLine );
                stDevBase       = stDevBase / ( endBaseLine - beginBaseLine );
                varianceBase    = varianceBase / ( endBaseLine - beginBaseLine );
                varOfVarBase    = varOfVarBase / ( endBaseLine - beginBaseLine );
                skewnessBase    = skewnessBase / ( endBaseLine - beginBaseLine );
                varOfSkewBase   = varOfSkewBase / ( endBaseLine - beginBaseLine );
                kutosisBase     = kutosisBase / ( endBaseLine - beginBaseLine );
                varOfKurtosisBase = varOfKurtosisBase / ( endBaseLine - beginBaseLine );

                meanInflow          = meanInflow / ( maxPerf - endBaseLine + 1 );
                stDevInflow         = stDevInflow / ( maxPerf - endBaseLine + 1 );
                varianceInflow      = varianceInflow / ( maxPerf - endBaseLine + 1 );
                varOfVarInflow      = varOfVarInflow / ( maxPerf - endBaseLine  + 1 );
                skewnessInflow      = skewnessInflow / ( maxPerf - endBaseLine + 1 );
                varOfSkewInflow     = varOfSkewInflow / ( maxPerf - endBaseLine + 1 );
                kutosisInflow       = kutosisInflow / ( maxPerf - endBaseLine + 1 );
                varOfKurtosisInflow = varOfKurtosisInflow / ( maxPerf - endBaseLine + 1 );

                meanOutflow          = meanOutflow / ( startReperfusion - maxPerf );
                stDevOutflow         = stDevOutflow / ( startReperfusion - maxPerf );
                varianceOutflow      = varianceOutflow / ( startReperfusion - maxPerf );
                varOfVarOutflow      = varOfVarOutflow / ( startReperfusion - maxPerf );
                skewnessOutflow      = skewnessOutflow / ( startReperfusion - maxPerf );
                varOfSkewOutflow     = varOfSkewOutflow / ( startReperfusion - maxPerf );
                kutosisOutflow       = kutosisOutflow / ( startReperfusion - maxPerf );
                varOfKurtosisOutflow = varOfKurtosisOutflow / ( startReperfusion - maxPerf );

                meanReperf           = meanReperf / ( m_maximumImageNumberToFocus - m_minimumImageNumberToFocus - startReperfusion );
                stDevReperf          = stDevReperf / ( m_maximumImageNumberToFocus - m_minimumImageNumberToFocus - startReperfusion );
                varianceReperf       = varianceReperf / ( m_maximumImageNumberToFocus - m_minimumImageNumberToFocus - startReperfusion );
                varOfVarReperf       = varOfVarReperf / ( m_maximumImageNumberToFocus - m_minimumImageNumberToFocus - startReperfusion );
                skewnessReperf       = skewnessReperf / ( m_maximumImageNumberToFocus - m_minimumImageNumberToFocus - startReperfusion );
                varOfSkewReperf      = varOfSkewReperf / ( m_maximumImageNumberToFocus - m_minimumImageNumberToFocus - startReperfusion );
                kutosisReperf        = kutosisReperf / ( m_maximumImageNumberToFocus - m_minimumImageNumberToFocus - startReperfusion );
                varOfKurtosisReperf  = varOfKurtosisReperf / ( m_maximumImageNumberToFocus - m_minimumImageNumberToFocus - startReperfusion );


                fileWriter.write( getTitle( ) +  "_BASELINE"  + "\t" +
                                  fInfo.getSeriesNumber( )   + "\t" +
                                  beginBaseLine  + "-" + ( endBaseLine - 1 ) + "\t" +
                                  formatter.format( new Float( fInfo.getSliceLocation( ) ).floatValue( ) ) + "\t" );

                fileWriter.write( "baseline_" );
                
                RegioPolygon poly = imageRegioPolygons.get( j );
                fileWriter.write( poly.getLabel( )        + "\t" );
                fileWriter.write( poly.getInnerVolume( )  + "\t" );

                fileWriter.write( formatter.format( poly.getInnerVolume( ) * volPerPixel ) + "\t" );

                fileWriter.write( formatter.format( meanBase )               + "\t" );
                fileWriter.write( formatter.format( stDevBase )              + "\t" );
                fileWriter.write( formatter.format( varianceBase )           + "\t" );
                fileWriter.write( formatter.format( varOfVarBase )           + "\t" );
                fileWriter.write( formatter.format( skewnessBase )           + "\t" );
                fileWriter.write( formatter.format( varOfSkewBase )          + "\t" );
                fileWriter.write( formatter.format( kutosisBase )            + "\t" );
                fileWriter.write( formatter.format( varOfKurtosisBase )      + "\t\n" );

                ////////////////////////////////////////////////////////////////
                fileWriter.write( getTitle( ) + "_INFLOW"      + "\t" +
                                  fInfo.getSeriesNumber( )    + "\t" +
                                  endBaseLine + "-" + (maxPerf-1) + "\t" +
                                  formatter.format( new Float( fInfo.getSliceLocation( ) ).floatValue( ) ) + "\t" );


                fileWriter.write( "inflow_" );

                fileWriter.write( poly.getLabel( )        + "\t" );
                fileWriter.write( poly.getInnerVolume( )  + "\t" );

                fileWriter.write( formatter.format( poly.getInnerVolume( ) * volPerPixel ) + "\t" );

                fileWriter.write( formatter.format( meanInflow )               + "\t" );
                fileWriter.write( formatter.format( stDevInflow )              + "\t" );
                fileWriter.write( formatter.format( varianceInflow )           + "\t" );
                fileWriter.write( formatter.format( varOfVarInflow )           + "\t" );
                fileWriter.write( formatter.format( skewnessInflow )           + "\t" );
                fileWriter.write( formatter.format( varOfSkewInflow )          + "\t" );
                fileWriter.write( formatter.format( kutosisInflow )            + "\t" );
                fileWriter.write( formatter.format( varOfKurtosisInflow )      + "\t\n" );


                ////////////////////////////////////////////////////////////////
                fileWriter.write( getTitle( ) + "_OUTFLOW" + "\t" +
                                  fInfo.getSeriesNumber( ) + "\t" +
                                  maxPerf + "-" + ( startReperfusion - 1 ) + "\t" +
                                  formatter.format( new Float( fInfo.getSliceLocation( ) ).floatValue( ) ) + "\t" );

                fileWriter.write( "outflow_" );

                fileWriter.write( poly.getLabel( )        + "\t" );
                fileWriter.write( poly.getInnerVolume( )  + "\t" );

                fileWriter.write( formatter.format( poly.getInnerVolume( ) * volPerPixel ) + "\t" );

                fileWriter.write( formatter.format( meanOutflow )               + "\t" );
                fileWriter.write( formatter.format( stDevOutflow )              + "\t" );
                fileWriter.write( formatter.format( varianceOutflow )           + "\t" );
                fileWriter.write( formatter.format( varOfVarOutflow )           + "\t" );
                fileWriter.write( formatter.format( skewnessOutflow )           + "\t" );
                fileWriter.write( formatter.format( varOfSkewOutflow )          + "\t" );
                fileWriter.write( formatter.format( kutosisOutflow )            + "\t" );
                fileWriter.write( formatter.format( varOfKurtosisOutflow )      + "\t\n" );

                ////////////////////////////////////////////////////////////////
                fileWriter.write( getTitle( ) + "_REPERFUSION" + "\t" +
                                  fInfo.getSeriesNumber( ) + "\t" +
                                  startReperfusion + "-" + ( m_maximumImageNumberToFocus - m_minimumImageNumberToFocus - 1 ) + "\t" +
                                  formatter.format( new Float( fInfo.getSliceLocation( ) ).floatValue( ) ) + "\t" );


                fileWriter.write( "reperfusion_" );
                fileWriter.write( poly.getLabel( )        + "\t" );
                fileWriter.write( poly.getInnerVolume( )  + "\t" );

                fileWriter.write( formatter.format( poly.getInnerVolume( ) * volPerPixel ) + "\t" );

                fileWriter.write( formatter.format( meanReperf )               + "\t" );
                fileWriter.write( formatter.format( stDevReperf )              + "\t" );
                fileWriter.write( formatter.format( varianceReperf )           + "\t" );
                fileWriter.write( formatter.format( varOfVarReperf )           + "\t" );
                fileWriter.write( formatter.format( skewnessReperf )           + "\t" );
                fileWriter.write( formatter.format( varOfSkewReperf )          + "\t" );
                fileWriter.write( formatter.format( kutosisReperf )            + "\t" );
                fileWriter.write( formatter.format( varOfKurtosisReperf )      + "\t\n" );

            }

            fileWriter.write( "\t\n" );

        }
        catch ( IOException e )
        {
            System.out.println( "DicomSliceCanvasPolyDraw.writeROIInformation : IO-exception in file! " + e.getMessage( ) );
        }

    }


    /**
     *
     * Writes all information of all to the ROIs to a text file.
     *
     * @param fileWriter
     */
    public void writeROIInformation( FileWriter fileWriter )
    {

        try
        {
            DecimalFormat formatter = new DecimalFormat( "###.00" );

            float totalVolume = 0f;


            // output as huge list!
            fileWriter.write( "View_Title__" + "\t" + "Series_Number" + "\t" + "Instance_Number" + "\t" + "Slice_Location" + "\t" +
                              "ROI_Name____" + "\t" + "#_of_Pixels__" + "\t" + "ROI_Volume_____" + "\t" + "Mean_Pixel_Val" + "\t" +
                              "Standard_Dev" + "\t" + "Variance_____" + "\t" + "Var_of_Variance" + "\t" + "Skewness______" + "\t" +
                              "Var(Skew)___" + "\t" + "Kurtosis_____" + "\t" + "Var_of_Kurtosis" + "\n" );


            for ( int i = m_minimumImageNumberToFocus ; i < m_maximumImageNumberToFocus ; i++ )
            {

                FileInfo fInfo = m_dicomImageSeriesData.getFileInfo( i );
                int sliceNumber = i;

                LinkedList <RegioPolygon> imageRegioPolygons = m_polygonsVector.get( i );
                
                if ( fInfo.getPixelDepth( ).contains( "N.A." ) == true )
                {
                    fInfo.setPixelDepth( "1.0" );
                }
                
                if ( fInfo.getSliceLocation( ).contains( "N.A.") == true )
                {
                    fInfo.setSliceLocation( "1.0" );
                }
                    
                float volPerPixel = new Float( fInfo.getPixelDepth( ) ).floatValue( ) * new Float( fInfo.getPixelWidth( ) ).floatValue( ) * new Float( fInfo.getPixelHeight( ) ).floatValue( );

                for ( int j = 0; j < imageRegioPolygons.size( ); j++ )
                {

                    RegioPolygon poly = imageRegioPolygons.get( j );

                    int[] pixelsWithinPolygon = getPixelValues( sliceNumber, poly );

                    if ( pixelsWithinPolygon != null )
                    {

                        fileWriter.write( getTitle( ) + "\t" +
                                          fInfo.getSeriesNumber( ) + "\t" +
                                          fInfo.getInstanceNumber( ) + "\t" +
                                          formatter.format( new Float( fInfo.getSliceLocation( ) ).floatValue( ) ) + "\t" );

                        MomentAboutTheMeanOfArrayCalculator momCalc = new MomentAboutTheMeanOfArrayCalculator( pixelsWithinPolygon );
                        MomentsAboutTheMean moments = momCalc.getMomentsAboutTheMean( );

                        fileWriter.write( poly.getLabel( )        + "\t" );
                        fileWriter.write( poly.getInnerVolume( )  + "\t" );

                        fileWriter.write( formatter.format( poly.getInnerVolume( ) * volPerPixel ) + "\t" );

                        fileWriter.write( formatter.format( moments.getMean( ) )               + "\t" );
                        fileWriter.write( formatter.format( moments.getStDev( ) )              + "\t" );
                        fileWriter.write( formatter.format( moments.getVariance( ) )           + "\t" );
                        fileWriter.write( formatter.format( moments.getVarianceOfVariance( ) ) + "\t" );
                        fileWriter.write( formatter.format( moments.getSkewness( ) )           + "\t" );
                        fileWriter.write( formatter.format( moments.getVarianceOfSkewness( ) ) + "\t" );
                        fileWriter.write( formatter.format( moments.getKurtosis( ) )           + "\t" );
                        fileWriter.write( formatter.format( moments.getVarianceOfKurtosis( ) ) + "\t\n" );
                    }
                }
            }

            fileWriter.write( "\r\n" );
            
        }
        catch ( IOException e )
        {
            System.out.println( "DicomSliceCanvasPolyDraw.writeROIInformation : IO-exception in file! " + e.getMessage( ) );
        }

    }
    
    /**
     *
     * Writes all information of all to the ROIs to a text file.
     *
     * @param none
     */
    public LinkedList<MomentsAboutTheMean> getROI_MAM_Information( )
    {
        // the MAMinformation of all ROIs in the 
        LinkedList<MomentsAboutTheMean> allMAMInformationList = new LinkedList<MomentsAboutTheMean>( );
        
        MomentsAboutTheMean moments = null;
        
        float totalVolume = 0f;

        for ( int i = m_minimumImageNumberToFocus ; i < m_maximumImageNumberToFocus ; i++ )
        {

            FileInfo fInfo = m_dicomImageSeriesData.getFileInfo( i );
            int sliceNumber = i;

            LinkedList <RegioPolygon> imageRegioPolygons = m_polygonsVector.get( i );
                
            if ( fInfo.getPixelDepth( ).contains( "N.A." ) == true )
            {
                fInfo.setPixelDepth( "1.0" );
            }
                
            if ( fInfo.getSliceLocation( ).contains( "N.A." ) == true )
            {
                fInfo.setSliceLocation( "1.0" );
            }
                    
            float volPerPixel = new Float( fInfo.getPixelDepth( ) ).floatValue( ) * new Float( fInfo.getPixelWidth( ) ).floatValue( ) * new Float( fInfo.getPixelHeight( ) ).floatValue( );

            for ( int j = 0; j < imageRegioPolygons.size( ); j++ )
            {

                RegioPolygon poly = imageRegioPolygons.get( j );

                int[] pixelsWithinPolygon = getPixelValues( sliceNumber, poly );

                if ( pixelsWithinPolygon != null )
                {

                    String output = getTitle( )                 + "\t" +
                                    fInfo.getSeriesNumber( )    + "\t" +
                                    fInfo.getInstanceNumber( )  + "\t";

                    MomentAboutTheMeanOfArrayCalculator momCalc = new MomentAboutTheMeanOfArrayCalculator( pixelsWithinPolygon );
                    
                    // set the other parameters as well
                    moments = momCalc.getMomentsAboutTheMean( );
                    
                    moments.setTPMTypeString( this.getShortImageTitle( ) );
                    
                    moments.setDicomImageFileReference( getDicomFileInfo( ).getFileName( ) );
                    moments.setPatientID( getDicomFileInfo( ).getPatientID( ) );
                    moments.setInstanceSeriesUID( getDicomFileInfo( ).getSOPInstanceUID( ) );
                    moments.setNumberOfROIPixels( pixelsWithinPolygon.length );
                    moments.setSliceNumber( sliceNumber );
                    moments.setPolygonName( poly.getLabel( ) );
                    moments.setExaminedVolumeSize( volPerPixel * pixelsWithinPolygon.length );
                    
                    // add to the hashmap containing all momment information
                    allMAMInformationList.add( moments );
                    
                 }
            }
        }
        return allMAMInformationList;
    }
    
    /**
     *
     * Writes all information of all to the ROIs to a text file.
     *
     * @param fileWriter
     */
//    public void writeROIInformationToMongoDB( java.util.List<BasicDBObject> data )
//    {
//
//        
//        
//        
//        
//        /////////////////////////////////////////////////////////////////////////
//        
//        try
//        {
//            DecimalFormat formatter = new DecimalFormat( "###.00" );
//
//            float totalVolume = 0f;
//
//
//            // output as huge list!
//            //fileWriter.write( "View_Title__" + "\t" + "Series_Number" + "\t" + "Instance_Number" + "\t" + "Slice_Location" + "\t" +
//            //                  "ROI_Name____" + "\t" + "#_of_Pixels__" + "\t" + "ROI_Volume_____" + "\t" + "Mean_Pixel_Val" + "\t" +
//            //                  "Standard_Dev" + "\t" + "Variance_____" + "\t" + "Var_of_Variance" + "\t" + "Skewness______" + "\t" +
//            //                  "Var(Skew)___" + "\t" + "Kurtosis_____" + "\t" + "Var_of_Kurtosis" + "\n" );
//
//
//            for ( int i = m_minimumImageNumberToFocus ; i < m_maximumImageNumberToFocus ; i++ )
//            {
//
//                FileInfo fInfo = m_dicomImageSeriesData.getFileInfo( i );
//                int sliceNumber = i;
//
//                LinkedList <RegioPolygon> imageRegioPolygons = m_polygonsVector.get( i );
//                
//                if ( fInfo.getPixelDepth( ).contains( "N.A." ) == true )
//                {
//                    fInfo.setPixelDepth( "1.0" );
//                }
//                
//                if ( fInfo.getSliceLocation( ).contains( "N.A.") == true )
//                {
//                    fInfo.setSliceLocation( "1.0" );
//                }
//                    
//                float volPerPixel = new Float( fInfo.getPixelDepth( ) ).floatValue( ) * new Float( fInfo.getPixelWidth( ) ).floatValue( ) * new Float( fInfo.getPixelHeight( ) ).floatValue( );
//
//                for ( int j = 0; j < imageRegioPolygons.size( ); j++ )
//                {
//
//                    RegioPolygon poly = imageRegioPolygons.get( j );
//
//                    int[] pixelsWithinPolygon = getPixelValues( sliceNumber, poly );
//
//                    if ( pixelsWithinPolygon != null )
//                    {
//
//                        fileWriter.write( getTitle( ) + "\t" +
//                                          fInfo.getSeriesNumber( ) + "\t" +
//                                          fInfo.getInstanceNumber( ) + "\t" +
//                                          formatter.format( new Float( fInfo.getSliceLocation( ) ).floatValue( ) ) + "\t" );
//
//                        MomentAboutTheMeanOfArrayCalculator momCalc = new MomentAboutTheMeanOfArrayCalculator( pixelsWithinPolygon );
//                        MomentsAboutTheMean moments = momCalc.getMomentsAboutTheMean( );
//
//                        fileWriter.write( poly.getLabel( )        + "\t" );
//                        fileWriter.write( poly.getInnerVolume( )  + "\t" );
//
//                        fileWriter.write( formatter.format( poly.getInnerVolume( ) * volPerPixel ) + "\t" );
//
//                        fileWriter.write( formatter.format( moments.getMean( ) )               + "\t" );
//                        fileWriter.write( formatter.format( moments.getStDev( ) )              + "\t" );
//                        fileWriter.write( formatter.format( moments.getVariance( ) )           + "\t" );
//                        fileWriter.write( formatter.format( moments.getVarianceOfVariance( ) ) + "\t" );
//                        fileWriter.write( formatter.format( moments.getSkewness( ) )           + "\t" );
//                        fileWriter.write( formatter.format( moments.getVarianceOfSkewness( ) ) + "\t" );
//                        fileWriter.write( formatter.format( moments.getKurtosis( ) )           + "\t" );
//                        fileWriter.write( formatter.format( moments.getVarianceOfKurtosis( ) ) + "\t\n" );
//                    }
//                }
//            }
//            
//            ///////////////////////////////////////////////////////////////////
//            
//            BasicDBObject doc = new BasicDBObject();
//            doc.
//            append("patient", new BasicDBObject().
//                append("id", m_fileInfo.getPatientID( ) ).
//                append("name", m_fileInfo.getPatientName( ) ).
//                append("age", m_fileInfo.getPatientAge( ) ).
//                append("birth", new Date(2013, 3, 4, 5, 6, 7))
//            ).
//            append("study", new BasicDBObject().
//                append("id", "1.2234.345.453645.664").
//                append("date", new Date(2013, 3, 4, 5, 6, 7))
//            ).
//            append("sequence", new BasicDBObject().
//                append("type", "t2").
//                append("protocol", "t2_sag_160").
//                append("orientation", "sagittal").
//                append("pixel_spacing", 4).
//                append("slice_spacing", 5).
//                append("slice_thickness", 6)
//            ).
//            append("volume", new BasicDBObject().
//                append("label", "gbm").
//                append("name", "region1").
//                append("data", Arrays.asList(
//                    new BasicDBObject().append("number",5).
//                        append("contour", new BasicDBObject().append("type","Polygon").append("coordinates", JSON.parse("[ [3,1], [1,2], [5,6], [9,2], [4,3], [3,1] ]"))).
//                        append("roi", new BasicDBObject().append("orig_mean", rand.nextFloat()*234.23).append("orig_stdev", 435.89).append("orig_var", 231.43).append("orig_skew", 832.98).append("orig_kurt", 666.66)),
//                    new BasicDBObject().append("number",6).
//                        append("contour", new BasicDBObject().append("type","Polygon").append("coordinates", JSON.parse("[ [3,1], [1,2], [5,6], [9,2], [4,3], [3,1] ]"))).
//                        append("roi", new BasicDBObject().append("orig_mean", rand.nextFloat()*1000).append("orig_stdev", rand.nextFloat()*1000).append("orig_var", rand.nextFloat()*1000).append("orig_skew", rand.nextFloat()*1000).append("orig_kurt", rand.nextFloat()*1000)),
//                    new BasicDBObject().append("number",7).
//                        append("contour", new BasicDBObject().append("type","Polygon").append("coordinates", JSON.parse("[ [3,1], [1,2], [5,6], [9,2], [4,3], [3,1] ]"))).
//                        append("roi", new BasicDBObject().append("orig_mean", rand.nextFloat()*1000).append("orig_stdev", rand.nextFloat()*1000).append("orig_var", rand.nextFloat()*1000).append("orig_skew", rand.nextFloat()*1000).append("orig_kurt", rand.nextFloat()*1000)),
//                    new BasicDBObject().append("number",8).
//                        append("contour", new BasicDBObject().append("type","Polygon").append("coordinates", JSON.parse("[ [3,1], [1,2], [5,6], [9,2], [4,3], [3,1] ]"))).
//                        append("roi", new BasicDBObject().append("orig_mean", rand.nextFloat()*1000).append("orig_stdev", rand.nextFloat()*1000).append("orig_var", rand.nextFloat()*1000).append("orig_skew", rand.nextFloat()*1000).append("orig_kurt", rand.nextFloat()*1000)),
//                    new BasicDBObject().append("number",9).
//                        append("contour", new BasicDBObject().append("type","Polygon").append("coordinates", JSON.parse("[ [3,1], [1,2], [5,6], [9,2], [4,3], [3,1] ]"))).
//                        append("roi", new BasicDBObject().append("orig_mean", rand.nextFloat()*1000).append("orig_stdev", rand.nextFloat()*1000).append("orig_var", rand.nextFloat()*1000).append("orig_skew", rand.nextFloat()*1000).append("orig_kurt", rand.nextFloat()*1000)),
//                    new BasicDBObject().append("number",10).
//                        append("contour", new BasicDBObject().append("type","Polygon").append("coordinates", JSON.parse("[ [3,1], [1,2], [5,6], [9,2], [4,3], [3,1] ]"))).
//                        append("roi", new BasicDBObject().append("orig_mean", rand.nextFloat()*1000).append("orig_stdev", rand.nextFloat()*1000).append("orig_var", rand.nextFloat()*1000).append("orig_skew", rand.nextFloat()*1000).append("orig_kurt", rand.nextFloat()*1000))
//                ))
//                
//            );
//        
//        coll.insert(doc);
//            
//            
//            
//            
//            
//            fileWriter.write( "\r\n" );
//            
//        }
//        catch ( IOException e )
//        {
//            System.out.println( "DicomSliceCanvasPolyDraw.writeROIInformation : IO-exception in file! " + e.getMessage( ) );
//        }
//
//    }
    

    /**
     * Returns the array of pixel values belonging the the Region
     * @param sliceNumber
     * @param poly
     */
    public int[] getPixelValues( int sliceNumber, RegioPolygon polygon )
    {

        int[] pixVals = new int[polygon.getInnerVolume( )];
        int cntr = 0;


        Rectangle bb = polygon.getBounds( );

        if ( bb.x < 0 )
        {
            bb.x = 0;
        }
        if ( bb.x + bb.width >= m_imageWidth )
        {
            bb.width = m_imageWidth - bb.x - 1;
        }
        if ( bb.y < 0 )
        {
            bb.y = 0;
        }

        if ( bb.y + bb.height >= m_imageHeight )
        {
            bb.height = m_imageHeight - bb.y - 1;
        }

        for ( int y = bb.y; y <= bb.y + bb.height; y++ )
        {
            int yoffset = y * m_imageWidth;
            for ( int x = bb.x; x <= bb.x + bb.width; x++ )
            {
                if ( polygon.contains( x, y ) )
                {
                    pixVals[cntr] = m_imagePixelData[ sliceNumber * m_imageHeight * m_imageWidth + x + m_imageWidth * y ];
                    cntr++;
                }
            }
        }

        return pixVals;
    }



    /**
     * Creates a report file of all RegioPolygons of all images of the image stack
     */
    public void createDetailedReportFilesOfAllRegioPolygonsOfAllImages( )
    {
        
        FileInfo fInfo1 = m_dicomImageSeriesData.getFileInfo( 0 );
        
        String fileName = getFilePathAndName( fInfo1.getPatientName( ) + "_" + fInfo1.getSeriesDescription( ) + "_" + "detailed_report_file_of_all_regions_for_all_images.txt", "regio_rep", "Save" );
        
        LinkedList<ImageRegionProperties> roiProps = new LinkedList<ImageRegionProperties>( );
        
        if ( fileName.length( ) != 0 )
        {
            try
            {
                DecimalFormat formatter = new DecimalFormat( "###.00" );
                
                FileWriter fileWriter = new FileWriter( fileName );
                
                FileInfo fInfo = m_dicomImageSeriesData.getFileInfo( 0 );
                
                fileWriter.write( "Patient name       : " + fInfo.getPatientName( ) + "\r\n");
                fileWriter.write( "Patient ID         : " + fInfo.getPatientID( ) + "\r\n");
                fileWriter.write( "Date of Birth      : " + fInfo.getPatientBirthDate( ) + " " );
                fileWriter.write( "Age                : " + fInfo.getPatientAge( ) + "\r\n" );
                fileWriter.write( "Examination Date   : " + fInfo.getAcquisitionDate( ) + "\r\n" );
                fileWriter.write( "Examination Time   : " + fInfo.getAcquisitionTime( ) + "\r\n" );
                fileWriter.write( "Series Number      : " + fInfo.getSeriesNumber( ).trim( ) + "\r\n" );
                fileWriter.write( "Series Description : " + fInfo.getSeriesDescription( ) + "\r\n" );
                fileWriter.write( "Pixel height       : " + fInfo.getPixelHeight( ) + "\r\n" );
                fileWriter.write( "Pixel width        : " + fInfo.getPixelWidth( ) + "\r\n" );
                fileWriter.write( "Pixel depth        : " + fInfo.getPixelDepth( ) + "\r\n" );
                fileWriter.write( "(Slice thickness   : " + fInfo.getSliceThickness( ) + ")\r\n" );
                
                float totalVolume = 0f;
                
                //for ( int i = 0; i < m_numberOfImages; i++ )
                for ( int i = m_minimumImageNumberToFocus ; i < this.m_maximumImageNumberToFocus ; i++ )
                {
                    fInfo = m_dicomImageSeriesData.getFileInfo( i );
                    fileWriter.write( "-------------------------------------------------------\r\n" );
                    fileWriter.write( "Series number  : " + fInfo.getSeriesNumber( ).trim( ) + " image number : " + i + "\r\n" );
                    fileWriter.write( "Slice position : " + fInfo.getSliceLocation( ) + "\r\n" );
                    fileWriter.write( "Filename       : " + fInfo.getFileName( ) + "\r\n");
                    fileWriter.write( "-------------------------------------------------------\r\n\r\n" );
                    
                    //public Vector< LinkedList <RegioPolygon> > m_polygonsVector = null;
                    
                    LinkedList <RegioPolygon> imageRegioPolygons = m_polygonsVector.get( i );
                    
                    for ( int j = 0; j < imageRegioPolygons.size( ); j++ )
                    {
                        RegioPolygon poly = imageRegioPolygons.get( j );
                        
                        ImageRegionProperties regProp = computeRegioProperties( i, poly );
                        
                        fileWriter.write( "Name   " + regProp.getImageRegionName( )       + "\r");
                        fileWriter.write( "Pixels " + regProp.getNumberOfPixels( )        + "\r");
                        fileWriter.write( "Volume " + regProp.getVolumeEnclosedByRegio( ) + "\r");
                        fileWriter.write( "Mean   " + regProp.getMeanPixelValue( )        + "\r");
                        fileWriter.write( "St.dev " + regProp.getStandardDeviationOfPixelValues( ) + "\r\n\r\n");
                    }
                }
               
                fileWriter.write( "\r\n" );
                
                fileWriter.close( );
            }
            catch ( IOException e )
            {
                System.out.println( "Error creating file : " + fileName );
            }
        }
    }
    
    
    /**
     * 
     * Computes the region properties enclosed by the 
     * 
     * @param imageNumber
     * @param polygonNumber
     * @param regPol
     * @return
     * 
     */
    private ImageRegionProperties computeRegioProperties( int imageNumber, RegioPolygon regPol )
    {
        FileInfo fInfo = m_dicomImageSeriesData.getFileInfo( imageNumber );
                    
        ImageRegionProperties roiProp = new ImageRegionProperties( new Double( fInfo.getPixelWidth( ) ).doubleValue( ),
                                                                   new Double( fInfo.getPixelHeight( ) ).doubleValue( ),
                                                                   new Double( fInfo.getPixelDepth( ) ).doubleValue( ) );
                    
        roiProp.setImageRegionName( regPol.getLabel( ) );
        
        float zoom = 1f / getZoom( );
        
        Polygon roi = (Polygon)regPol.scaled( zoom );
        
        Rectangle bb = roi.getBounds( );
        
        if ( bb.x < 0 )
        {    
            bb.x = 0;
        }
        if ( bb.x + bb.width >= m_imageWidth ) 
        {
            bb.width = m_imageWidth - bb.x - 1;
        }
        if ( bb.y < 0 )
        {
            bb.y = 0;
        }
        
        if ( bb.y + bb.height >= m_imageHeight ) 
        {
            bb.height = m_imageHeight - bb.y - 1;
        }
        
        int imageSize = m_imageWidth * m_imageHeight;

        int   roiSize = 0;
        float av = 0f;
        float stdev = 0f;
            
        // Calculate the average of the ROI
        for ( int y = bb.y; y <= bb.y + bb.height; y++ ) 
        {
            int yoffset = y * m_imageWidth;
            for ( int x = bb.x; x <= bb.x + bb.width; x++ ) 
            {
                if ( roi.contains( x, y ) ) 
                {
                    roiSize++;
                    int xyoffset = yoffset;
                    av += m_imagePixelData[ imageNumber * m_imageHeight * m_imageWidth + x + m_imageWidth * y ];
                }
            }
        }

        av = av * m_nordicScaleFactor / roiSize;

        // Calculate the standard deviation of the current ROI
        float diff = 0.0f;
        // reset the roi size
        roiSize = 0;
        
        for ( int y = bb.y; y <= bb.y + bb.height; y++ )
        {
            int yoffset = y * m_imageWidth;
            for ( int x = bb.x; x <= bb.x + bb.width; x++ )
            {
                if ( roi.contains( x, y ) )
                {
                    roiSize++;
                    diff = av * m_nordicScaleFactor - m_imagePixelData[ imageNumber * m_imageHeight * m_imageWidth + x + m_imageWidth * y ] * m_nordicScaleFactor;
                    stdev += diff * diff;
                }
            }
        }

        stdev = (float)Math.sqrt( (double) stdev / ( (float)roiSize - 1.0 ) ) ;
                    
        roiProp.setNumberOfPixels( roiSize );
        roiProp.setImageRegionVolume( (double)( roiSize ) *
                                      new Double( m_fileInfo.getSliceThickness( ) ).doubleValue( ) *
                                      new Double( m_fileInfo.getPixelHeight( ) ).doubleValue( ) *
                                      new Double( m_fileInfo.getPixelWidth( ) ).doubleValue( ) );
        roiProp.setMeanPixelValue( av );
        roiProp.setStandardDeviationOfPixelValues( stdev );
        roiProp.setImagePosition( (float)new Double( fInfo.getSliceLocation( ) ).doubleValue( ) );
        
        return roiProp;
        
    }
    
    /**
     * Creates a report file of all image segments of all images of the image stack
     */
    public void createSegmentPropertiesOfAllImagesReportFile( )
    {
        String fileName = getFilePathAndName( "image_segment_report_file.txt", "rep", "Save" );
        
        if ( fileName.length( ) != 0 )
        {
            try
            {
                DecimalFormat formatter = new DecimalFormat( "###.00" );
                
                FileWriter fileWriter = new FileWriter( fileName );
                
                FileInfo fInfo = m_dicomImageSeriesData.getFileInfo( 0 );
                
                fileWriter.write( "Patient name       : " + fInfo.getPatientName( ) + "\r\n");
                fileWriter.write( "Patient ID         : " + fInfo.getPatientID( ) + "\r\n");
                fileWriter.write( "Date of Birth      : " + fInfo.getPatientBirthDate( ) + " " );
                fileWriter.write( "Age                : " + fInfo.getPatientAge( ) + "\r\n" );
                fileWriter.write( "Examination Date   : " + fInfo.getAcquisitionDate( ) + "\r\n" );
                fileWriter.write( "Examination Time   : " + fInfo.getAcquisitionTime( ) + "\r\n" );
                fileWriter.write( "Series Number      : " + fInfo.getSeriesNumber( ).trim( ) + "\r\n\r\n" );
                fileWriter.write( "Series Description : " + fInfo.getSeriesDescription( ) + "\r\n\r\n" );
                
                // for ( int i = 0; i < m_numberOfImages; i++ )
                for ( int i = m_minimumImageNumberToFocus ; i < this.m_maximumImageNumberToFocus ; i++ )
                {
                    fInfo = m_dicomImageSeriesData.getFileInfo( i );
                    fileWriter.write( "-------------------------------------------------------\r\n" );
                    fileWriter.write( "Series number  : " + fInfo.getSeriesNumber( ).trim( ) + " image number : " + i + "\r\n" );
                    fileWriter.write( "Slice position : " + fInfo.getSliceLocation( ) + "\r\n" );
                    fileWriter.write( "Filename       : " + fInfo.getFileName( ) + "\r\n");
                    fileWriter.write( "-------------------------------------------------------\r\n\r\n" );
                    
                    LinkedList<ImageSegmentProperties> imaProp = computeStatisticalPropertiesOfImageSegmentsOfImageNumber( i );
                    
                    for ( int j = 0; j < imaProp.size( ); j++ )
                    {
                        ImageSegmentProperties segProp = imaProp.get( j );
                        
                        fileWriter.write( "Name   " + segProp.getImageSegmentName( ) + "\r");
                        fileWriter.write( " Pixels " + segProp.getNumberOfPixels( ) + "\r");
                        fileWriter.write( " Volume " + segProp.getImageSegmentVolume( ) + "\r");
                        fileWriter.write( " Mean   " + segProp.getMeanPixelValue( ) + "\r");
                        fileWriter.write( " St.dev " + segProp.getStandardDeviationOfPixelValues( ) + "\r\n\r\n");
                    }
                }
               
                fileWriter.write( "\r\n" );
                
                fileWriter.close( );
            }
            catch ( IOException e )
            {
                System.out.println( "Error creating file : " + fileName );
            }
        }
    }
    
    
    /**
     * Returns the filename and directory selected by the user
     */
    protected String getFilePathAndName( String defaultString, String subDirString, String buttonText )
    {       
        
        JFileChooser fileChooser = new JFileChooser( );
        String selectedFilePathAndName = "";
        
        // Create a File object containing the canonical path of the 
        // desired directory
        
        String filePath = m_dicomImageSeriesData.getFileInfo( 0 ).getDirectory( );
        
        String patientName = m_dicomImageSeriesData.getFileInfo( 0 ).getPatientName( );
        String examinationDate = m_dicomImageSeriesData.getFileInfo( 0 ).getStudyDate( );
        
        String outputPath = new String( filePath + System.getProperty( "file.separator" ) + subDirString );
        
        File f = new File( outputPath );
        if ( f.exists( ) == false )
        {
            f.mkdir( );
        }
        
        // Set the current directory
        fileChooser.setCurrentDirectory( f );
        
        int result;
        fileChooser.setSelectedFile( new File( patientName+ "_" + examinationDate + "_" + defaultString ) );
        
        result = fileChooser.showDialog( this, buttonText );
        
        
        File selected_file = new File( "" );
            
        switch ( result )
        {
            case JFileChooser.APPROVE_OPTION:   // Approve Open button was pressed
                                                selected_file = fileChooser.getSelectedFile( );
                                                selectedFilePathAndName = selected_file.getPath( ); 
                                                break;
            case JFileChooser.CANCEL_OPTION:    // Cancel or close dialog was selected
                                                break;
            case JFileChooser.ERROR_OPTION:     // the selected process did not terminate properly
                                                break;
        }
        
        return selectedFilePathAndName; 
        
    }
    
    /**
     * Draws the BufferedImage object on the image context
     */
    private void drawBufferedImageImageSegmentsOverlay( Graphics g ) 
    {
        if ( m_imageSegmentsOverlayList == null )
        {
            System.out.println( "m_imageSegmentsOverlayList is null" );
        }
        if ( m_selectedTissueTypes == null )
        {
            //System.out.println( "m_selectedTissueTypes is null" );
        }
        else
        {
            if ( m_selectedTissueTypes.length < 1 )
            {
                System.out.println( " m_selectedTissueTypes.length = 0" );
            }
        }
        if ( m_imageSegmentsOverlayList.isEmpty( ) )
        {
            // System.out.println( "m_imageSegmentsOverlayList is empty" );
        }
        

        if ( ( m_imageSegmentsOverlayList != null ) && ( m_selectedTissueTypes != null ) )
        {
            if ( ( !m_imageSegmentsOverlayList.isEmpty( ) ) && ( m_selectedTissueTypes.length > 0 ) )
            {
                LinkedList<boolean[][]> colorOverlays = m_imageSegmentsOverlayList.get(m_currentlyDisplayedImageNumber );
       
                int numbersOfTissues = colorOverlays.size( );
        
                Color[][] colormap = new Color[m_imageHeight][m_imageWidth];
        
                for ( int k = 0 ; k < m_selectedTissueTypes.length ; k++ )
                {
                    if ( m_selectedTissueTypes[k] < numbersOfTissues )
                    {
                        boolean[][] imSegment = colorOverlays.get( m_selectedTissueTypes[k] );

                        for ( int i = 0; i < m_imageWidth ; i++ )
                        {
                            for ( int j = 0; j < m_imageHeight; j++ )
                            { 
                                if ( m_thressholdedImageSegmentOverlays[m_currentlyDisplayedImageNumber][i][j] == false )
                                {
                                    colormap[i][j] = null;
                                }
                                else
                                {
                                    float maxTValue = -Float.MAX_VALUE;
                        
                                    // if there are multiple possibilities the last tissue 
                                    // type gets the color!    
                                    if ( imSegment[i][j] == true )
                                    {
                                        colormap[i][j] = m_imageSegmentColors[ m_selectedTissueTypes[k] ];
                                    }
                                }
                            }
                        }
                    }

                }

                // now draw the image ofline
                BufferedImage image = new BufferedImage( (int)m_zoom * m_imageWidth, (int)m_zoom * m_imageHeight, BufferedImage.TYPE_INT_ARGB );
        
                Graphics bg = image.getGraphics( );
                for ( int i = 0; i < m_imageWidth ; i++ )
                {
                    for ( int j = 0; j < m_imageHeight; j++ )
                    {
                        if ( colormap[i][j] != null )
                        {
                            Color col = new Color( colormap[i][j].getRed()/255, colormap[i][j].getGreen()/255f, colormap[i][j].getBlue()/255f, m_alpha );
                            bg.setColor( col );
                            bg.fillRect( (int)(m_zoom * i), (int)(m_zoom * j), (int)m_zoom , (int)m_zoom );
                        }
                    }
                }
        
                // and draw it
                g.drawImage( image, 0, 0, null );
            }
        }
    }

    
    
    /**
     * Sets the right side overlay
     * @param rightOverlay 
     */
    public void set_rightSideImageSegmentOverlays( boolean[][][] rightOverlay )
    {
        m_rightSideImageSegmentOverlays = rightOverlay;
    }
    
    /**
     * Sets the left side overlay
     * @param leftOverlay 
     */
    public void set_leftSideImageSegmentOverlays( boolean[][][] leftOverlay )
    {
        m_leftSideImageSegmentOverlays = leftOverlay;
    }
    
    
    /**
     * Draws the overlay of the selected local statistics type! 
     */
    private void drawLeftandRightSideOverlay( Graphics g )
    {
        
        Color[][] colormap = new Color[m_imageHeight][m_imageWidth];
        for ( int i = 0; i < m_imageWidth ; i++ )
        {
            for ( int j = 0; j < m_imageHeight; j++ )
            {
                if ( m_rightSideImageSegmentOverlays != null )
                {
                    if ( m_rightSideImageSegmentOverlays[m_currentlyDisplayedImageNumber][i][j] == true )
                    {
                        colormap[i][j] = Color.BLUE;
                    }
                }
                if ( m_leftSideImageSegmentOverlays != null )
                {
                    if ( m_leftSideImageSegmentOverlays[m_currentlyDisplayedImageNumber][i][j] == true )
                    {
                        colormap[i][j] = Color.RED;
                    }
                }
            }
        }
        
        // now draw the image ofline
        BufferedImage image = new BufferedImage( (int)m_zoom * m_imageWidth,
                                                 (int)m_zoom * m_imageHeight,
                                                 BufferedImage.TYPE_INT_ARGB );

        Graphics bg = image.getGraphics( );
        for ( int i = 0; i < m_imageWidth ; i++ )
        {
            for ( int j = 0; j < m_imageHeight; j++ )
            {
                if ( colormap[i][j] != null )
                {
                    Color col = new Color( colormap[i][j].getRed()/255, colormap[i][j].getGreen()/255f, colormap[i][j].getBlue()/255f, m_opaquacyOfVolumeOverlays );
                    bg.setColor( col );
                    bg.fillRect( (int)(m_zoom * i), (int)(m_zoom * j), (int)m_zoom , (int)m_zoom );
                }
            }
        }

        // and draw it
        g.drawImage( image, 0, 0, null );    
        
    }
            
    /**
     * Draws the overlay of the selected local statistics type! 
     */
    private void drawLocalStatisticsOverlay( Graphics g )
    {
        if ( m_localMeanImageSegmentOverlays == null )
        {
            System.out.println( "m_localMeanImageSegmentOverlays is null!" );
        }
        else
        {

            Color[][] colormap = new Color[m_imageHeight][m_imageWidth];
            for ( int i = 0; i < m_imageWidth ; i++ )
            {
                for ( int j = 0; j < m_imageHeight; j++ )
                {
                    if ( m_localStatisticsDisplayOverlayType == 1 )
                    {
                        if ( m_localMeanImageSegmentOverlays[m_currentlyDisplayedImageNumber][i][j] == true )
                        {
                            colormap[i][j] = Color.YELLOW;
                        }
                    }
                    if ( m_localStatisticsDisplayOverlayType == 2 )
                    {
                        if ( m_localVarianceImageSegmentOverlays[m_currentlyDisplayedImageNumber][i][j] == true )
                        {
                            colormap[i][j] = Color.YELLOW;
                        }
                    }
                    if ( m_localStatisticsDisplayOverlayType == 3 )
                    {
                        if ( m_localSkewnessImageSegmentOverlays[m_currentlyDisplayedImageNumber][i][j] == true )
                        {
                            colormap[i][j] = Color.YELLOW;
                        }
                    }
                    if ( m_localStatisticsDisplayOverlayType == 4 )
                    {
                        if ( m_localKurtosisImageSegmentOverlays[m_currentlyDisplayedImageNumber][i][j] == true )
                        {
                            colormap[i][j] = Color.YELLOW;
                        }
                    }
                }
            }

            // now draw the image ofline
            BufferedImage image = new BufferedImage( (int)m_zoom * m_imageWidth,
                                                     (int)m_zoom * m_imageHeight,
                                                     BufferedImage.TYPE_INT_ARGB );

            Graphics bg = image.getGraphics( );
            for ( int i = 0; i < m_imageWidth ; i++ )
            {
                for ( int j = 0; j < m_imageHeight; j++ )
                {
                    if ( colormap[i][j] != null )
                    {
                        Color col = new Color( colormap[i][j].getRed()/255, colormap[i][j].getGreen()/255f, colormap[i][j].getBlue()/255f, m_opaquacyOfVolumeOverlays );
                        bg.setColor( col );
                        bg.fillRect( (int)(m_zoom * i), (int)(m_zoom * j), (int)m_zoom , (int)m_zoom );
                    }
                }
            }

            // and draw it
            g.drawImage( image, 0, 0, null );
        }

    }



    /**
     * Draws the BufferedImage object on the image context
     */
    private void drawBufferedImageVolumeOfInterestOverlay( Graphics g )
    {
        
        if ( m_imageVolumeOfInterestOverlayList == null )
        {
            System.out.println( "m_imageVolumeOfInterestOverlayList is null" );
        }

//        if ( m_imageVolumeOfInterestOverlayList.isEmpty( ) )
//        {
//            System.out.println( "m_imageVolumeOfInterestOverlayList is empty" );
//        }


        if ( m_imageVolumeOfInterestOverlayList != null )
        {
            if ( !m_imageVolumeOfInterestOverlayList.isEmpty( ) )
            {

                LinkedList<boolean[][]> colorOverlays;

                if ( this.m_dicomImageSeriesData != null )
                {
                    colorOverlays = m_imageVolumeOfInterestOverlayList.get(m_currentlyDisplayedImageNumber );
                }
                else
                {
                    colorOverlays = m_imageVolumeOfInterestOverlayList.get( 0 );
                }
                
                int numbersOfVolumes = colorOverlays.size( );

                Color[][] colormap = new Color[m_imageHeight][m_imageWidth];

                for ( int k = 0; k < numbersOfVolumes; k++ )
                {
                    if ( m_selectedImageVolumeOfInterestList.get( k ) == true )
                    {
                        boolean[][] imSegment = colorOverlays.get( k );

                        for ( int i = 0; i < m_imageWidth ; i++ )
                        {
                            for ( int j = 0; j < m_imageHeight; j++ )
                            {

                                if ( imSegment[i][j] == true )
                                {
                                    colormap[i][j] = m_imageVolumeOfInterestColorList.get( k );
                                }
                            }
                        }
                    }
                }

                // now draw the image ofline
                BufferedImage image = new BufferedImage( (int)m_zoom * m_imageWidth,
                                                         (int)m_zoom * m_imageHeight,
                                                         BufferedImage.TYPE_INT_ARGB );

                Graphics bg = image.getGraphics( );
                for ( int i = 0; i < m_imageWidth ; i++ )
                {
                    for ( int j = 0; j < m_imageHeight; j++ )
                    {
                        if ( colormap[i][j] != null )
                        {
                            Color col = new Color( colormap[i][j].getRed()/255, colormap[i][j].getGreen()/255f, colormap[i][j].getBlue()/255f, m_opaquacyOfVolumeOverlays );
                            bg.setColor( col );
                            bg.fillRect( (int)(m_zoom * i), (int)(m_zoom * j), (int)m_zoom , (int)m_zoom );
                        }
                    }
                }

                // and draw it
                g.drawImage( image, 0, 0, null );
            }
        }
    }


    /**
     * Computes the m_contoursAsBooleanImageOverlays attribute
     *
     */
    private boolean computeContoursAsBooleanImageOverlays( )
    {
        
        m_contoursAsBooleanImageOverlays = new boolean[m_numberOfImages][m_imageWidth][m_imageHeight];

        if ( m_imageSegmentsOverlayList == null )
        {
            System.out.println( "m_imageSegmentsOverlayList is null" );
            return false;
        }
        if ( m_selectedTissueTypes == null )
        {
            // System.out.println( "m_selectedTissueTypes is null" );
            return false;
        }
        else
        {
            if ( m_selectedTissueTypes.length < 1 )
            {
                System.out.println( "m_selectedTissueTypes.length = 0" );
                return false;
            }
        }

        if ( m_imageSegmentsOverlayList.isEmpty( ) )
        {
            // System.out.println( "m_imageSegmentsOverlayList is empty" );
            return false;
        }

        if ( ( m_imageSegmentsOverlayList != null ) && ( m_selectedTissueTypes != null ) )
        {
            // compute only if one tissue type has been selected!
            if ( ( !m_imageSegmentsOverlayList.isEmpty( ) ) && ( m_selectedTissueTypes.length == 1 ) )
            {

                for ( int imageCounter = 0; imageCounter < m_numberOfImages; imageCounter++ )
                {

                    LinkedList<boolean[][]> colorOverlays = m_imageSegmentsOverlayList.get( imageCounter );
                    int numbersOfTissues = colorOverlays.size( );

                    for ( int k = 0 ; k < m_selectedTissueTypes.length ; k++ )
                    {
                        if ( m_selectedTissueTypes[k] < numbersOfTissues )
                        {
                            boolean[][] imSegment = colorOverlays.get( m_selectedTissueTypes[k] );
                            boolean[][] contourImage = new boolean[m_imageWidth][m_imageHeight];

                            RegioContoursByMorphologicFiltering contourFinder = new RegioContoursByMorphologicFiltering( imSegment, m_imageWidth, m_imageHeight );

                            contourImage = contourFinder.getContoursAsBooleanImage( );

                            for ( int i = 0; i < m_imageWidth; i++ )
                            {
                                for ( int j = 0; j < m_imageHeight; j++ )
                                {
                                    m_contoursAsBooleanImageOverlays[imageCounter][i][j] = contourImage[i][j];
                                }
                            }
                        }

                    }

                } // end for all images
            }
        }

        // if we made it until here the conversion is ok!
        return true;
    }


    /**
     * Sets the array of overlay numbers to be displayed
     * @param selectedTypes
     */
    public void setSelectedOverlayTypeNumbers( int[] selectedTissueTypes )
    {
        m_selectedTissueTypes = selectedTissueTypes;
        
        // based on the number of selected tissues the popup menu has different 
        // characteristic
        
        if ( ( m_selectedTissueTypes.length == 0 ) || ( m_selectedTissueTypes.length > 2 ) )
        {
            m_applyLogic_AND_toImageSegments.setEnabled( false );
            m_applyLogic_OR_toImageSegments.setEnabled( false );
            m_applyLogic_EXOR_toImageSegments.setEnabled( false);
            m_applyLogic_NOT_toImageSegments.setEnabled( false );
        }
        
        if ( m_selectedTissueTypes.length == 1 )
        {
            m_applyLogic_AND_toImageSegments.setEnabled( false );
            m_applyLogic_OR_toImageSegments.setEnabled( false );
            m_applyLogic_EXOR_toImageSegments.setEnabled( false);
            m_applyLogic_NOT_toImageSegments.setEnabled( true );
        }
        
        if ( m_selectedTissueTypes.length == 2 )
        {
            m_applyLogic_AND_toImageSegments.setEnabled( true );
            m_applyLogic_OR_toImageSegments.setEnabled( true );
            m_applyLogic_EXOR_toImageSegments.setEnabled( true );
            m_applyLogic_NOT_toImageSegments.setEnabled( false );
        }
        
    }



/**
     * Sets the array of overlay numbers to be displayed
     * @param selectedTypes
     */
    public void setSelectedOverlayTypeNumbersAndColors( int[] selectedTissueTypes, Color color )
    {

        m_selectedTissueTypes = selectedTissueTypes;

        for ( int i = 0; i < m_selectedTissueTypes.length; i++ )
        {
            if ( i ==  m_selectedTissueTypes[i] )
            {
                m_imageSegmentColors[i] = color;
            }
        }

        // based on the number of selected tissues the popup menu has different
        // characteristic

        if ( ( m_selectedTissueTypes.length == 0 ) || ( m_selectedTissueTypes.length > 2 ) )
        {
            m_applyLogic_AND_toImageSegments.setEnabled( false );
            m_applyLogic_OR_toImageSegments.setEnabled( false );
            m_applyLogic_EXOR_toImageSegments.setEnabled( false);
            m_applyLogic_NOT_toImageSegments.setEnabled( false );
        }

        if ( m_selectedTissueTypes.length == 1 )
        {
            m_applyLogic_AND_toImageSegments.setEnabled( false );
            m_applyLogic_OR_toImageSegments.setEnabled( false );
            m_applyLogic_EXOR_toImageSegments.setEnabled( false);
            m_applyLogic_NOT_toImageSegments.setEnabled( true );
        }

        if ( m_selectedTissueTypes.length == 2 )
        {
            m_applyLogic_AND_toImageSegments.setEnabled( true );
            m_applyLogic_OR_toImageSegments.setEnabled( true );
            m_applyLogic_EXOR_toImageSegments.setEnabled( true );
            m_applyLogic_NOT_toImageSegments.setEnabled( false );
        }

    }

    /**
     * Sets the array of overlay numbers to be displayed
     * @param selectedTypes
     */
    public void setSelectedVolumeOfInterestOverlayTypeNumbers( int[] selectedTissueTypes )
    {


        // first set to false
        for ( int i = 0; i < m_selectedImageVolumeOfInterestList.size( ); i++ )
        {
            m_selectedImageVolumeOfInterestList.set( i, new Boolean( false ) );
        }

        for ( int i = 0 ; i < selectedTissueTypes.length; i++ )
        {
            m_selectedImageVolumeOfInterestList.set( selectedTissueTypes[i], new Boolean( true ) );
        }

//        // based on the number of selected tissues the popup menu has different
//        // characteristic
//
//        if ( ( m_selectedTissueTypes.length == 0 ) || ( m_selectedTissueTypes.length > 2 ) )
//        {
//            m_applyLogic_AND_toImageSegments.setEnabled( false );
//            m_applyLogic_OR_toImageSegments.setEnabled( false );
//            m_applyLogic_EXOR_toImageSegments.setEnabled( false);
//            m_applyLogic_NOT_toImageSegments.setEnabled( false );
//        }
//
//        if ( m_selectedTissueTypes.length == 1 )
//        {
//            m_applyLogic_AND_toImageSegments.setEnabled( false );
//            m_applyLogic_OR_toImageSegments.setEnabled( false );
//            m_applyLogic_EXOR_toImageSegments.setEnabled( false);
//            m_applyLogic_NOT_toImageSegments.setEnabled( true );
//        }
//
//        if ( m_selectedTissueTypes.length == 2 )
//        {
//            m_applyLogic_AND_toImageSegments.setEnabled( true );
//            m_applyLogic_OR_toImageSegments.setEnabled( true );
//            m_applyLogic_EXOR_toImageSegments.setEnabled( true );
//            m_applyLogic_NOT_toImageSegments.setEnabled( false );
//        }

    }

/**
     * Sets the array of overlay numbers to be displayed
     * @param selectedTypes
     */
    public void setColorOfSelectedVolumeOfInterestOverlays( int[] selectedTissueTypes, Color color )
    {

        // first set to false
        for ( int i = 0; i < m_selectedImageVolumeOfInterestList.size( ); i++ )
        {
            m_selectedImageVolumeOfInterestList.set( i, new Boolean( false ) );
        }

        for ( int i = 0 ; i < selectedTissueTypes.length; i++ )
        {
            m_selectedImageVolumeOfInterestList.set( selectedTissueTypes[i], new Boolean( true ) );
            m_imageVolumeOfInterestColorList.set( selectedTissueTypes[i], color );
        }
    }


    /**
     * 
     * Performs basic boolean operations on different image segments
     * 
     * @param booleanOperationType
     */
    public void performBooleanOperation( int booleanOperationType )
    {
        if ( booleanOperationType == OverlayPropertiesTable.PERFORM_BOOLEAN_AND_OPERATION )
        {
            handleBooleanOperationAND( );
        }
        if ( booleanOperationType == OverlayPropertiesTable.PERFORM_BOOLEAN_OR_OPERATION )
        {
            handleBooleanOperationOR( );
        }
        if ( booleanOperationType == OverlayPropertiesTable.PERFORM_BOOLEAN_EXOR_OPERATION )
        {
            handleBooleanOperationEXOR( );
        }
        if ( booleanOperationType == OverlayPropertiesTable.PERFORM_BOOLEAN_NOT_OPERATION )
        {
            handleBooleanOperationNOT( );
        }
    }

    /**
     *
     * Performs basic boolean operations on different image segments
     *
     * @param booleanOperationType
     */
    public void performBooleanOperationForVolumesOfInterest( int booleanOperationType, int[] selectedVolumeNumbers )
    {
        if ( booleanOperationType == VolumeOfInterestOverlayPropertiesTable.PERFORM_BOOLEAN_AND_OPERATION )
        {
            handleVolumeOfInterestBooleanOperationAND( selectedVolumeNumbers );
        }
        if ( booleanOperationType == VolumeOfInterestOverlayPropertiesTable.PERFORM_BOOLEAN_OR_OPERATION )
        {
            handleVolumeOfInterestBooleanOperationOR( selectedVolumeNumbers );
        }
        if ( booleanOperationType == VolumeOfInterestOverlayPropertiesTable.PERFORM_BOOLEAN_EXOR_OPERATION )
        {
            handleVolumeOfInterestBooleanOperationEXOR( selectedVolumeNumbers );
        }
        if ( booleanOperationType == VolumeOfInterestOverlayPropertiesTable.PERFORM_BOOLEAN_NOT_OPERATION )
        {
            handleVolumeOfInterestBooleanOperationNOT( selectedVolumeNumbers );
        }
    }



    /**
     * This method eliminates all selected overlays  
     */
    private void deleteSelectedOverlayImageSegments( )
    {
        for ( int i = m_selectedTissueTypes.length - 1; i >= 0; i-- )
        {
            for ( int j = 0; j < m_imageSegmentsOverlayList.size( ); j++ )
            {
                m_imageSegmentsOverlayList.get( j ).remove( m_selectedTissueTypes[i] );
            }
        }
        
        notifyActionListeners( IMAGE_SEGMENT_REMOVED_EVENT, "Image segments removed by user" );
        
        // notityListeners that a tissue type was eliminated
        // force an update
        displayDicomImage(m_currentlyDisplayedImageNumber );
        drawImage( false ); 
    }
    
    
    /**
     * 
     *  Performs a cutting out of all pixels withing the polygon cutting3DPolygon
     *  Pixels within the polygon remain there value; pixels outside are set 
     *  to zero.
     * 
     * @param m_cutting3DPolygon
     */
    public void perform3DCut( RegioPolygon cutting3DPolygon ) 
    {

        if ( cutting3DPolygon != null )
        {
            m_cutting3DPolygon = cutting3DPolygon;
        
            cutting3DPolygon.setScalingFactor( 1.0f / m_zoom );
        
            for ( int i = 0; i < this.m_numberOfImages; i++ )
            {
                int offset = i * m_imageWidth * m_imageHeight;
                for ( int j = 0; j < m_imageHeight; j++ )
                {
                    for ( int k = 0; k < m_imageWidth; k++ )
                    {
                        if( !m_cutting3DPolygon.contains( k, j ) )
                        {
                            m_imagePixelData[ offset + j * m_imageWidth + k ] = 0;
                        }
                    }
                }
            }
        
            cutting3DPolygon.setScalingFactor( m_zoom );
        
            // force a redraw of the currently displayed image
            displayDicomImage(m_currentlyDisplayedImageNumber );
            drawImage( false );
        }
        else
        {
            Dialogs.messageBox( m_parent, "No polygon defined", "Cannot perform 3D-Cut" );
        }
    }


    /**
     * Creates the menu which allows image segments related functions
     */
    private void createSegmentModePopupMenu( )
    {
        m_imageSegmentsMenu = new JMenu( "Image Segments" );
        
        m_popupmenu.add( m_imageSegmentsMenu );

        m_eliminatePixelsFromOverlaysByContourDrawingModeRadioButton = new JRadioButtonMenuItem( "Eliminate overlay pixels by contour drawing" );
        m_imageSegmentsMenu.add( m_eliminatePixelsFromOverlaysByContourDrawingModeRadioButton );

        m_promoteThressholdOverlayToTissueTypeJMenuItem = new JMenuItem( "Promote thresshold overlay to image segment type" );
        m_promoteThressholdOverlayToTissueTypeJMenuItem.addActionListener( this );
        m_imageSegmentsMenu.add( m_promoteThressholdOverlayToTissueTypeJMenuItem );
        
        m_loadImageSegmentsFromDefaultFiles = new JMenuItem( "Load image segments (default files)" ) ;
        m_loadImageSegmentsFromDefaultFiles.addActionListener( this );
        m_imageSegmentsMenu.add( m_loadImageSegmentsFromDefaultFiles  );
        
        m_loadImageSegmentsFromDefaultFiles = new JMenuItem( "Load image segments (default files)" ) ;
        m_loadImageSegmentsFromDefaultFiles.addActionListener( this );
        m_imageSegmentsMenu.add( m_loadImageSegmentsFromDefaultFiles  );
        
        m_saveImageSegmentsToDefaultFiles = new JMenuItem( "Save image segments (defaults files)" ) ;
        m_saveImageSegmentsToDefaultFiles.addActionListener( this );
        m_imageSegmentsMenu.add( m_saveImageSegmentsToDefaultFiles );

        m_loadImageSegmentsFromUserDefinedDefaultFiles = new JMenuItem( "Load image segments .." ) ;
        m_loadImageSegmentsFromUserDefinedDefaultFiles.addActionListener( this );
        m_imageSegmentsMenu.add( m_loadImageSegmentsFromUserDefinedDefaultFiles  );

        m_saveImageSegmentsToUserDefinedFiles = new JMenuItem( "Save image segments .." ) ;
        m_saveImageSegmentsToUserDefinedFiles.addActionListener( this );
        m_imageSegmentsMenu.add( m_saveImageSegmentsToUserDefinedFiles );

        m_deleteAllImageSegments = new JMenuItem( "Delete selected segments" ) ;
        m_deleteAllImageSegments.addActionListener( this );
        m_imageSegmentsMenu.add( m_deleteAllImageSegments );

        m_reportingMenu = new JMenu( "Reporting" );

        m_createHistogramFilesOfCurrentlySelectedImageSegmentTypes = new JMenuItem( "Create histograms file of currently selected overlays" );
        m_createHistogramFilesOfCurrentlySelectedImageSegmentTypes.addActionListener( this );
        m_reportingMenu.add( m_createHistogramFilesOfCurrentlySelectedImageSegmentTypes  );


        m_createReportFileOfCurrentlySelectedImageSegmentTypes = new JMenuItem( "Create detailed report file of currently selected overlays" ) ;
        m_createReportFileOfCurrentlySelectedImageSegmentTypes.addActionListener( this );
        m_reportingMenu.add( m_createReportFileOfCurrentlySelectedImageSegmentTypes  );
        
        m_createCondensedReportFileOfCurrentlySelectedImageSegmentType = new JMenuItem( "Create condensed report file of currently selected overlays" ) ;
        m_createCondensedReportFileOfCurrentlySelectedImageSegmentType.addActionListener( this );
        m_reportingMenu.add( m_createCondensedReportFileOfCurrentlySelectedImageSegmentType );
        
        // this menu item allows for counting of the number of distinct volumes + it statistics
        m_createReportFileOfDistinct3DVolumes = new JMenuItem( "Create report on number of distinct volumes (volume count)" ) ;
        m_createReportFileOfDistinct3DVolumes.addActionListener( this );
        m_reportingMenu.add( m_createReportFileOfDistinct3DVolumes );


        m_imageSegmentsMenu.add( m_reportingMenu );
        
        m_convertToRegioPolygonsMenu = new JMenu( "Convert to regio polygons" ) ;
        
        m_convertToRegioPolygonsCurrentlyDisplayedOnly = new JMenuItem( "Currently displayed image only" );
        m_convertToRegioPolygonsCurrentlyDisplayedOnly.addActionListener( this );
        m_convertToRegioPolygonsMenu.add( m_convertToRegioPolygonsCurrentlyDisplayedOnly );
        
        m_convertToRegioPolygonsForAllImages = new JMenuItem( "All images" );
        m_convertToRegioPolygonsForAllImages.addActionListener( this );
        m_convertToRegioPolygonsMenu.add( m_convertToRegioPolygonsForAllImages );
        
        m_imageSegmentsMenu.add( m_convertToRegioPolygonsMenu );
        
        m_convertToRegioPolygonsMenu.addSeparator( );
        
//        m_saveConvertedRegioPolygonsOfCurrentlyDisplayedImageOnlyMenuItem = new JMenuItem( "Save polygons of currently displayed images only" );
//        m_saveConvertedRegioPolygonsOfCurrentlyDisplayedImageOnlyMenuItem.addActionListener( this );
//        m_convertToRegioPolygonsMenu.add( m_saveConvertedRegioPolygonsOfCurrentlyDisplayedImageOnlyMenuItem );
//        
//        m_loadConvertedRegioPolygonsOfCurrentlyDisplayedImageOnlyMenuItem = new JMenuItem( "Load polygons of currently displayed images only" );
//        m_loadConvertedRegioPolygonsOfCurrentlyDisplayedImageOnlyMenuItem.addActionListener( this );
//        m_convertToRegioPolygonsMenu.add( m_loadConvertedRegioPolygonsOfCurrentlyDisplayedImageOnlyMenuItem );
        
//        m_saveConvertedRegioPolygonsOfAllImagesMenuItem = new JMenuItem( "Save polygons of all images" );
//        m_saveConvertedRegioPolygonsOfAllImagesMenuItem.addActionListener( this );
//        m_convertToRegioPolygonsMenu.add( m_saveConvertedRegioPolygonsOfAllImagesMenuItem );
//        
//        m_loadConvertedRegioPolygonsOfAllImagesMenuItem = new JMenuItem( "Load polygons of all images" );
//        m_loadConvertedRegioPolygonsOfAllImagesMenuItem.addActionListener( this );
//        m_convertToRegioPolygonsMenu.add( m_loadConvertedRegioPolygonsOfAllImagesMenuItem );
        
        m_deleteAllConvertedRegioPolygonsOfAllImagesMenuItem = new JMenuItem( "Delete polygons of all images" );
        m_deleteAllConvertedRegioPolygonsOfAllImagesMenuItem.addActionListener( this );
        m_convertToRegioPolygonsMenu.add( m_deleteAllConvertedRegioPolygonsOfAllImagesMenuItem );
        
        m_imageSegmentsMenu.add( m_convertToRegioPolygonsMenu );
        
        m_imageSegmentBooleanLogic = new JMenu( "Boolean logic" ) ;
        m_imageSegmentBooleanLogic.addActionListener( this );
        m_imageSegmentsMenu.add( m_imageSegmentBooleanLogic );
        
        m_applyLogic_AND_toImageSegments = new JMenuItem( "AND" );
        m_applyLogic_AND_toImageSegments.addActionListener( this );
        m_imageSegmentBooleanLogic.add( m_applyLogic_AND_toImageSegments );

        m_applyLogic_OR_toImageSegments = new JMenuItem( "OR" );
        m_applyLogic_OR_toImageSegments.addActionListener( this );
        m_imageSegmentBooleanLogic.add( m_applyLogic_OR_toImageSegments );
        
        m_applyLogic_EXOR_toImageSegments = new JMenuItem( "EXOR" );
        m_applyLogic_EXOR_toImageSegments.addActionListener( this );
        m_imageSegmentBooleanLogic.add( m_applyLogic_EXOR_toImageSegments );
        
        m_applyLogic_NOT_toImageSegments = new JMenuItem( "NOT" );
        m_applyLogic_NOT_toImageSegments.addActionListener( this );
        m_imageSegmentBooleanLogic.add( m_applyLogic_NOT_toImageSegments );
        
        // initialize properly: no segments available after construction -> disable all!
        m_applyLogic_AND_toImageSegments.setEnabled( false );
        m_applyLogic_OR_toImageSegments.setEnabled( false );
        m_applyLogic_EXOR_toImageSegments.setEnabled( false );
        m_applyLogic_NOT_toImageSegments.setEnabled( false );
        
        m_eliminateOverlayPixelsOutsideFocusRegion = new JMenuItem( "Eliminate pixels of selected segment types >>outside<< ROI" );
        m_eliminateOverlayPixelsOutsideFocusRegion.addActionListener( this );
        m_imageSegmentsMenu.add( m_eliminateOverlayPixelsOutsideFocusRegion  );
        
        m_eliminateOverlayPixelsInsideFocusRegion = new JMenuItem( "Eliminate pixels of selected segment types >>inside<< ROI" );
        m_eliminateOverlayPixelsInsideFocusRegion.addActionListener( this );
        m_imageSegmentsMenu.add( m_eliminateOverlayPixelsInsideFocusRegion  );
        
        m_eliminateOverlayPixelsOutsideFocusRegion.setEnabled( false );
        m_eliminateOverlayPixelsInsideFocusRegion.setEnabled( false );

        m_convertImageSegmentsByMorphologicFiltering  = new JMenuItem( "Convert to regions by morphologic filtering" ) ;
        m_convertImageSegmentsByMorphologicFiltering.addActionListener( this );
        m_imageSegmentsMenu.add( m_convertImageSegmentsByMorphologicFiltering  );

        m_importFMRIOverlayFile = new JMenuItem( "Import fMRI .voi-file (Brainvoyager)" ) ;
        m_importFMRIOverlayFile.addActionListener( this );
        m_imageSegmentsMenu.add( m_importFMRIOverlayFile  );

        m_importDTIOverlayFile = new JMenuItem( "Import DTI .fbr-file (Brainvoyager)" ) ;
        m_importDTIOverlayFile.addActionListener( this );
        m_imageSegmentsMenu.add( m_importDTIOverlayFile  );

        m_reDicomizeImageStackWithOverlays = new JMenuItem( "Re-Dicomize (and send) image stack with coloured overlays (Secondary Captures) .." );
        m_reDicomizeImageStackWithOverlays.addActionListener( this );
        m_imageSegmentsMenu.add( m_reDicomizeImageStackWithOverlays  );

        m_reDicomizeImageStackWithOverlaysAsPlainDicom = new JMenuItem( "Re-Dicomize (and send) image stack with overlays (true Dicom) .." );
        m_reDicomizeImageStackWithOverlaysAsPlainDicom.addActionListener( this );
        m_imageSegmentsMenu.add( m_reDicomizeImageStackWithOverlaysAsPlainDicom  );

    }
    
    
    /**
     *  Sets segment display mode enabled (default disabled)
     *  Allows the user to operate on the image segements
     *  (loading, saving, boolean operations, conversion to RegioPolygons etc.)
     */
    public void enableImageSegmentDisplayMode( )
    {
        m_imageSegmentDisplayAndManipulationMode = true;
        createSegmentModePopupMenu( );
    }
    
    /**
     * Clears the overlays of prior segmentation attempts
     */
    public void clearSegmentationResultsOfPriorAttempts( )
    {
        // to be implemented!
        // m_imageSegmentsOverlayList.clear( );
        // m_userAlreadySelectedImageSegmentsOnce = false;
    }

    /**
     * Enables segmenation mode
     * @param bool
     */
    public void enableInteractiveSegmentationMode( )
    {
        m_interactiveSegmentationMode = true; 
    }

    /**
     * Enables segmenation mode
     * @param bool
     */
    public void disablesInteractiveSegmentationMode( )
    {
        m_interactiveSegmentationMode = false;
    }

    /**
     * Sets the logical variable allowing the user to measure a distance
     * @param measureDistanceMode
     */
    public void setMeasureDistanceMode( boolean measureDistanceMode )
    {
        m_measureDistanceMode = measureDistanceMode;
    }

    /**
     * Flips the displayed data vertically
     */
    private void flipVertically( )
    {
        int imageNumberOffSet;
        int imageLineNumberOffsetFlipped;
        int imageLineNumberOffset;
        
        int[] flippedRaw = new int[ m_imagePixelData.length ];
        
        for ( int i = 0; i < m_numberOfImages; i++ )
        {
            imageNumberOffSet = i * m_imageWidth * m_imageHeight;
            
            for ( int j = 0; j < m_imageWidth; j++ )
            {
                
                imageLineNumberOffset = j * m_imageWidth;
                imageLineNumberOffsetFlipped  = ( m_imageWidth - 1 - j ) * m_imageWidth;
                
                for ( int k = 0; k < m_imageHeight; k++ )
                {
                    flippedRaw[ imageNumberOffSet + imageLineNumberOffset + k ] = m_imagePixelData[ imageNumberOffSet + imageLineNumberOffsetFlipped + k ];
                }
            }
        }
        
        System.arraycopy( flippedRaw, 0, m_imagePixelData, 0, flippedRaw.length );
        
        setImageSeriesPixelData( m_imagePixelData );
        displayDicomImage(m_currentlyDisplayedImageNumber );
        update( true );
        
    }
    
    /**
     * Flips the displayed data horizontally
     */
    private void flipHorizontally( )
    {
        int imageNumberOffSet;
        int imageLineNumberOffset;
        
        int[] flippedRaw = new int[ m_imagePixelData.length ];
        
        for ( int i = 0; i < m_numberOfImages; i++ )
        {
            imageNumberOffSet = i * m_imageWidth * m_imageHeight;
            
            for ( int j = 0; j < m_imageWidth; j++ )
            {
                imageLineNumberOffset = j * m_imageWidth;
                
                for ( int k = 0; k < m_imageHeight; k++ )
                {
                    flippedRaw[ imageNumberOffSet + imageLineNumberOffset + k ] = m_imagePixelData[ imageNumberOffSet + imageLineNumberOffset + ( m_imageHeight - 1 - k ) ];
                }
            }
        }
        
        System.arraycopy( flippedRaw, 0, m_imagePixelData, 0, flippedRaw.length );
        
        setImageSeriesPixelData( m_imagePixelData );
        displayDicomImage(m_currentlyDisplayedImageNumber );
        update( true );
        
    }

    /**
     * Rotates the displayed data clockwise about 90 degrees
     */
    private void rotateImageCounterClockwise90Degrees( )
    {
        int imageNumberOffSet;
        int imageLineNumberOffset;
        int imageLineNumberOffset2;
        
        int[] rotatedRaw = new int[ m_imagePixelData.length ];
        
        for ( int i = 0; i < m_numberOfImages; i++ )
        {
            imageNumberOffSet = i * m_imageWidth * m_imageHeight;
            
            for ( int j = 0; j < m_imageWidth; j++ )
            {
                imageLineNumberOffset = j * m_imageWidth;
                
                for ( int k = 0; k < m_imageHeight; k++ )
                {
                    imageLineNumberOffset2 = k * m_imageWidth;
                    rotatedRaw[ imageNumberOffSet + imageLineNumberOffset2 + j ] = m_imagePixelData[ imageNumberOffSet + imageLineNumberOffset + k ];
                }
            }
        }
        
        System.arraycopy( rotatedRaw, 0, m_imagePixelData, 0, rotatedRaw.length );
        
        setImageSeriesPixelData( m_imagePixelData );
        displayDicomImage(m_currentlyDisplayedImageNumber );
        update( true );
        
    }
    /**
     * Disables the GUI elements allowing for flipping and rotation
     */
    public void  enableFlipAndRotateMenuItems( boolean enable )
    {
        m_flipHorizontally.setEnabled( enable );
        m_flipVertically.setEnabled( enable );
        m_rotateAbout90Degrees.setEnabled( enable );
    }

    /**
     * Returns the last double clicked point (3D point)
     * @return
     */
    public Point3D getLastDoubleClickedPoint( )
    {
        return m_doubleClickedPoint3D;
    }
    
    
    /**
     * Sets the last computed double clicked point
     * @param p
     */
    public void setComputedDoubleClicked2DPoint( Point p )
    {
        m_doubleClickedPoint = p;
        if ( p != null )
        {
            m_doubleClickedPoint3D = compute3DCoordinate( p.x, p.y );
        }
    }

    /**
     * 
     * Returns the overlay volume
     * 
     * @return
     */
    public ArrayList< LinkedList<boolean[][]> > getImageVolumeOfInterestOverlayList( )
    {
        return m_imageVolumeOfInterestOverlayList;
    }

    /**
     *
     * Returns the overlay volume
     *
     * @return
     */
    public void setImageVolumeOfInterestOverlayList( ArrayList< LinkedList<boolean[][]> > list )
    {
        m_imageVolumeOfInterestOverlayList = list;
    }


    public boolean isLocalStatisticsAlreadyComputed( )
    {
        return m_localStatisticsIsComputed;
    }


    /**
     * Computes the local statistics of the image stack!
     */
    public void computeLocalStatisics( int xRegionSize, int yRegionSize )
    {
        m_localMeanImageSegmentOverlays     = new boolean[m_numberOfImages][m_imageWidth][m_imageHeight];
        m_localVarianceImageSegmentOverlays = new boolean[m_numberOfImages][m_imageWidth][m_imageHeight];
        m_localSkewnessImageSegmentOverlays = new boolean[m_numberOfImages][m_imageWidth][m_imageHeight];
        m_localKurtosisImageSegmentOverlays = new boolean[m_numberOfImages][m_imageWidth][m_imageHeight];

        m_localMeanValues       = new float[m_numberOfImages*m_imageWidth*m_imageHeight];
        m_localVarianceValues   = new float[m_numberOfImages*m_imageWidth*m_imageHeight];
        m_localSkewnessValues   = new float[m_numberOfImages*m_imageWidth*m_imageHeight];
        m_localKurtosisValues   = new float[m_numberOfImages*m_imageWidth*m_imageHeight];

        for ( int ima = 0; ima < m_numberOfImages; ima++ )
        {
            for ( int i = 0 ; i < m_imageHeight ; i++ )
            {
                for ( int j = 0 ; j < m_imageWidth; j++ )
                {

                    // current coordinate
                    int coord = i * m_imageWidth + j;
                    float[] localImageValues = new float[ ( 2 * xRegionSize + 1 ) * ( 2 * yRegionSize + 1 ) ];
                    int cntr = 0;
                    for ( int k = j - yRegionSize ; k <= j + yRegionSize; k++ )
                    {
                        for ( int l = i - xRegionSize ; l <= i + xRegionSize; l++ )
                        {
                            int ii = ( k + m_imageWidth ) % m_imageWidth;
                            int jj = ( l + m_imageHeight ) % m_imageHeight;
                            localImageValues[cntr] = m_imagePixelData[  ima * m_imageWidth * m_imageHeight + ii + m_imageWidth * jj ];
                            cntr++;
                        }
                    }

                    MomentAboutTheMeanOfArrayCalculator momCalc = new MomentAboutTheMeanOfArrayCalculator( localImageValues );
                    MomentsAboutTheMean moments = momCalc.getMomentsAboutTheMean( );

                    m_localMeanValues[ ima * m_imageWidth * m_imageHeight + m_imageWidth * i + j ] = (float)moments.getMean( );

                    if ( (float)moments.getMean( ) > m_localMeanMaximumValue )
                    {
                        m_localMeanMaximumValue = (float)moments.getMean( );
                    }
                    if ( (float)moments.getMean( ) < m_localMeanMinimumValue )
                    {
                        m_localMeanMinimumValue = (float)moments.getMean( );
                    }

                    m_localVarianceValues[ ima * m_imageWidth * m_imageHeight + m_imageWidth * i + j ] = (float)moments.getVariance( );

                    if ( (float)moments.getVariance( ) > m_localVarianceMaximumValue )
                    {
                        m_localVarianceMaximumValue = (float)moments.getVariance( );
                    }
                    if ( (float)moments.getVariance( ) < m_localVarianceMinimumValue )
                    {
                        m_localVarianceMinimumValue = (float)moments.getVariance( );
                    }

                    m_localSkewnessValues[ ima * m_imageWidth * m_imageHeight + m_imageWidth * i + j ] = (float)moments.getSkewness( );

                    if ( (float)moments.getSkewness( ) > m_localSkewnessMaximumValue )
                    {
                        m_localSkewnessMaximumValue = (float)moments.getSkewness( );
                    }
                    if ( (float)moments.getSkewness( ) < m_localSkewnessMinimumValue )
                    {
                        m_localSkewnessMinimumValue = (float)moments.getSkewness( );
                    }

                    m_localKurtosisValues[ ima * m_imageWidth * m_imageHeight + m_imageWidth * i + j ] = (float)moments.getKurtosis( );

                    if ( (float)moments.getKurtosis( ) > m_localKurtosisMaximumValue )
                    {
                        m_localKurtosisMaximumValue = (float)moments.getKurtosis( );
                    }
                    if ( (float)moments.getKurtosis( ) < m_localKurtosisMinimumValue )
                    {
                        m_localKurtosisMinimumValue = (float)moments.getKurtosis( );
                    }
                }
            }
        }

        m_localStatisticsIsComputed = true;
    }


    /**
     * Computes the local statistics overlays (boolean maps!)
     */
    public void computeLocalStatisticsBooleanOverlays( boolean filterMean,        boolean filterVariance, 
                                                       boolean filterSkewness,    boolean filterKurtosis,
                                                       int minMeanThresshold,     int maxMeanThresshold,
                                                       int minVarianceThresshold, int maxVarianceThresshold,
                                                       int minSkewnessThresshold, int maxSkewnessThresshold,
                                                       int minKurtosisThresshold, int maxKurtosisThresshold,
                                                       int localStatisticsDisplayOverlayType )
    {

        System.out.println( "Computing overlays! minMeanThresshold : " + minMeanThresshold + "maxMeanThresshold : " + maxMeanThresshold );

        m_meanFilterMinimumRangeValue     = minMeanThresshold;
        m_meanFilterMaximumRangeValue     = maxMeanThresshold;

        m_varianceFilterMinimumRangeValue = minVarianceThresshold;
        m_varianceFilterMaximumRangeValue = maxVarianceThresshold;

        m_skewnessFilterMinimumRangeValue = minSkewnessThresshold;
        m_skewnessFilterMaximumRangeValue = maxSkewnessThresshold;

        m_kurtosisFilterMinimumRangeValue = minKurtosisThresshold;
        m_kurtosisFilterMaximumRangeValue = maxKurtosisThresshold;

        m_localMeanRange     = ( m_localMeanMaximumValue     - m_localMeanMinimumValue );
        m_localVarianceRange = ( m_localVarianceMaximumValue - m_localVarianceMinimumValue );
        m_localSkewnessRange = ( m_localSkewnessMaximumValue - m_localSkewnessMinimumValue );
        m_localKurtosisRange = ( m_localKurtosisMaximumValue - m_localKurtosisMinimumValue );

        m_localStatisticsDisplayOverlayType = localStatisticsDisplayOverlayType;

        for ( int ima = 0; ima < m_numberOfImages; ima++ )
        {
            for ( int i = 0 ; i < m_imageHeight ; i++ )
            {
                for ( int j = 0 ; j < m_imageWidth; j++ )
                {
                    // mean
                    int testMeanValue = (int)( ( m_localMeanValues[ ima * m_imageWidth * m_imageHeight + m_imageWidth * j + i ] - m_localMeanMinimumValue ) * 1000f / m_localMeanRange );

                    if ( ( testMeanValue >= m_meanFilterMinimumRangeValue ) &&
                         ( testMeanValue <= m_meanFilterMaximumRangeValue ) )
                    {
                        m_localMeanImageSegmentOverlays[ima][i][j] = true;
                    }
                    else
                    {
                        m_localMeanImageSegmentOverlays[ima][i][j] = false;
                    }

                    // variance            
                    int testVarianceValue = (int)( ( m_localVarianceValues[ ima * m_imageWidth * m_imageHeight + m_imageWidth * j + i ] - m_localVarianceMinimumValue ) * 1000f / m_localVarianceRange );

                    if ( ( testVarianceValue >=  m_varianceFilterMinimumRangeValue ) &&
                         ( testVarianceValue <=  m_varianceFilterMaximumRangeValue ) )
                    {
                        m_localVarianceImageSegmentOverlays[ima][i][j] = true;
                    }
                    else
                    {
                        m_localVarianceImageSegmentOverlays[ima][i][j] = false;
                    }

                    // skewness
                    int testSkewnessValue = (int)( ( m_localSkewnessValues[ ima * m_imageWidth * m_imageHeight + m_imageWidth * j + i ] - m_localSkewnessMinimumValue ) * 1000f / m_localSkewnessRange );

                    if ( ( testSkewnessValue >=  m_skewnessFilterMinimumRangeValue ) &&
                         ( testSkewnessValue <=  m_skewnessFilterMaximumRangeValue ) )
                    {
                        m_localSkewnessImageSegmentOverlays[ima][i][j] = true;
                    }
                    else
                    {
                        m_localSkewnessImageSegmentOverlays[ima][i][j] = false;
                    }

                    // kurtosis
                    int testKurtosisValue = (int)( ( m_localKurtosisValues[ ima * m_imageWidth * m_imageHeight + m_imageWidth * j + i ] - m_localKurtosisMinimumValue ) * 1000f / m_localKurtosisRange );

                    if ( ( testKurtosisValue >=  m_kurtosisFilterMinimumRangeValue ) &&
                         ( testKurtosisValue <=  m_kurtosisFilterMaximumRangeValue ) )
                    {
                        m_localKurtosisImageSegmentOverlays[ima][i][j] = true;
                    }
                    else
                    {
                        m_localKurtosisImageSegmentOverlays[ima][i][j] = false;
                    }
                }
            }
        }

        // update the GUI with the overlays!!!
        update( true );
    }

    /**
     * 
     * Computes the boolean masks for those pixel values which are between
     * 
     * @param minValue minimum value in %
     * @param maxValue maximum value in %
     * 
     */
    public void applyThressholdingAndShowAsVolumeAsOverlay( int minimumValue, int maximumValue )
    {

        m_thressholdedImageSegmentOverlays = new boolean[m_numberOfImages][m_imageWidth][m_imageHeight];

        int maxPixVal = SignalTools.getMaximum( m_imagePixelData );

        for ( int i = 0; i < m_numberOfImages; i++ )
        {
            int pixelOffset = i * m_imageHeight * m_imageWidth;

            for ( int j = 0; j < m_imageWidth; j++ )
            {
                for ( int k = 0; k < m_imageHeight; k++ )
                {
                    if ( ( m_imagePixelData[ j +  m_imageWidth * k + pixelOffset ] > ( minimumValue * maxPixVal ) / 100f ) &&
                         ( m_imagePixelData[ j +  m_imageWidth * k + pixelOffset ] < ( maximumValue * maxPixVal ) / 100f ) )
                    {
                        m_thressholdedImageSegmentOverlays[i][j][k] = true;
                    }
                    else
                    {
                        m_thressholdedImageSegmentOverlays[i][j][k] = false;
                    }
                }
            }
        }

        // notityListeners that a tissue type was eliminated
        // force an update
        displayDicomImage(m_currentlyDisplayedImageNumber );
        drawImage( false );

    }

    /**
     * Returns the name of the last removed RegioPolygon
     * @return
     */
    public String getLastRemovedRegioPolygonFileName( )
    {
        return m_lastRemovedRegioPolygonFileName;
    }


    /**
     * Returns the pixels of the selected ROI
     * @return
     */
    public float[] getPixelsOfCurrentlySelectedROI( )
    {
        RegioPolygonPixelDistributionHistogram hist = new RegioPolygonPixelDistributionHistogram( m_parent, m_selectedPolygon, 
                                                                                                  getZoom( ), m_rawData,
                                                                                                  getImageWidth( ) , getImageHeight( ),
                                                                                                  m_nordicScaleFactor
                                                                                                 );
        return hist.getImagePixels( );
    }
 
    /**
     * The slope of the line found linear regression of pixel sets found 
     * by OSF band pass filtering.
     *
     *      y = slope * x + intercept
     * 
     * @param slopes 
     */
    public void setSlopes( double[] slopes ) 
    {
        m_slopes = new double[slopes.length];
        for ( int i = 0; i < slopes.length; i++ )
        {
            m_slopes[i] = slopes[i];
        }
    }

    /**
     * The intercept of the line found linear regression of pixel sets found 
     * after OSF band pass filtering.
     * 
     *      y = slope * x + intercept
     * 
     * @param slopes 
     */
    public void setIntercepts( double[] intercepts ) 
    {
        
        m_intercepts = new double[intercepts.length];
        
        for ( int i = 0; i < intercepts.length; i++ )
        {
            m_intercepts[i] = intercepts[i];
        }
    }
    
    
    public Vector< LinkedList <RegioPolygon> > getPolygonsVector( )
    {
        return m_polygonsVector;
    }

    public void setPolygonsVector( Vector< LinkedList <RegioPolygon> > polVec )
    {
        
        m_polygonsVector.clear( );
        
        for ( int i = 0; i < polVec.size( ); i++ )
        {
            LinkedList <RegioPolygon> polList = new LinkedList <RegioPolygon>( );
            LinkedList <RegioPolygon> polListSource = polVec.get( i );
            for ( int j = 0; j < polListSource.size( ); j++ )
            {
                polList.add( new RegioPolygon( polListSource.get( j ) ) );
            }
            m_polygonsVector.add( polList );
        }
        
        update( true );
        
    }
    
    
    // Attributes for this class
    public final static int DRAG_MODE             = 0;
    public final static int POLYGON_DRAW_MODE     = 1;
    public final static int RECTANGLE_DRAW_MODE   = 2;
    public final static int RESIZE_MODE           = 3;
    public final static int DELETE_MODE           = 4;   
    public final static int CUT_MODE              = 5;
    public final static int SELECT_MODE           = 6;
    public final static int ADD_STACKED_ROIS_MODE = 7;
    
    public final static int ADD_SINGLE_CIRCULAR_ROI_MODE      = 999;
    
    private PolygonEventCaster multicaster;
    private int lastMouseDragX, lastMouseDragY;
    
    // make private in later step!!
    public Vector< LinkedList <RegioPolygon> > m_polygonsVector = null;

    private boolean inSelectMode;
    
    private boolean inPolygonDrawMode;
    private boolean inRectangleDrawMode;
    private boolean inResizeMode;
    private boolean inDeleteMode;
    private boolean inCutMode;
    
    private Cursor prevCursor;
    
    /* String giving information about the currently visible image */
    private String imageNumber = "";
    private String imageTime = "";
    
    boolean isPolygonDrawing;
    boolean isRectangleDrawing;
    boolean isResizing;
    boolean isCutting;
    boolean showVertices;
    private RegioPolygon currentPoly = null;
    protected RegioPolygon m_selectedPolygon = null;
    
    protected RegioPolygon m_parentPolyonToCut;
    protected RegioPolygon m_childOneAfterCut;
    protected RegioPolygon m_childTwoAfterCut;
    
    private RegioPolygon previousPoly = null;
    public  Rectangle currentRectangle = null;
    private Rectangle previousRectangle = null;
    private float m_oldZoom;
    private boolean uniqueNames;
    private int resizeVertex;
    private int cutStartIndex;
    private int cutEndIndex;
    
    // square of the maximum distance for two points for being close
    private static int MAX_DISTANCE2 = 25;
    
    // member to indicate the current mode
    private int    m_current_mode;

    public float   m_av_raw = 0.0f;
    public float   m_stdev_raw = 0.0f;
    
    public float   m_av = 0.0f;
    public float   m_stdev = 0.0f;
    
    
    public float   m_range_raw = 0.0f;
    public float   m_center_raw = 0.0f;
    
    public int     m_roiSize = 0;
    public int     m_uniqueVolumeCounter = 0;

    private   JMenu     m_zoomMenu;
    
    protected JMenuItem m_doubleImageDisplay;
    protected JMenuItem m_halfImageDisplay;
    
    protected JMenuItem m_flipHorizontally;
    protected JMenuItem m_flipVertically;
    protected JMenuItem m_rotateAbout90Degrees;
    
    
    private JMenu     m_region;
    
    private JMenu       m_colorTableSelection;
    private JMenuItem   m_dicomHeaderPropertiesMenuItem;
    private JMenuItem   m_editDicomHeaderPropertiesMenuItem;
    private JMenuItem   m_siemensSpecificPropertiesMenuItem;
    
    private JMenu       m_imageProcessing;

    // Sub menu
    private JMenuItem   m_imageRestorationFilters;
    private JMenuItem   m_morphologicFilter;
    private JMenuItem   m_changePixelValuesBasedonUserThressholds;
    
    
    // Sub menu 2
    private JMenu       m_meanFilters;
    
    private JMenuItem   m_arithmeticMeanFilter;
    private JMenuItem   m_geometricMeanFilter;
    private JMenuItem   m_ahrmonicMeanFilter;
    private JMenuItem   m_contraharmonicMeanFilter;
    
    private JMenu       m_regionMode;
    
    private JMenu                   m_mirrorROI;
    private JMenuItem               m_addMirroredROI;
    private JRadioButtonMenuItem    m_showROIMirrorAxis;
    private JMenuItem               m_editMirrorAxis;

    private JMenu                   m_mirrorVOI;
    private JMenuItem               m_addMirroredVOI;
    private JRadioButtonMenuItem    m_showVOIMirrorAxis;
    private JMenuItem               m_editVOIMirrorAxis;


    // regio polygon operations allows simple copying of polygons amongst
    // different slice canvas polydraw objects
    protected JMenuItem m_affineTransformationRegioPolygon;
    protected JMenuItem m_copyRegioPolygon;
    private   JMenuItem m_pasteRegioPolygon;
    
    private   JMenuItem m_pasteSpecialRegioPolygon;

    protected JMenuItem m_copyAllRegioPolygons;
    protected JMenuItem m_pasteAllRegioPolygons;
    protected JMenuItem m_deleteAllRegioPolygonsOfCurrentlyDisplayedImageOnly;
    protected JMenuItem m_deleteAllRegioPolygonsOfAllImages;
    
    protected JMenuItem m_renameRegion;
    
    protected JMenuItem m_showHistogramOfSelectedRegioPolygon;
    protected JMenuItem m_tTestRegioPolygon;
    protected JMenuItem m_pixelDumpSelectedRegioPolygon;
    
    protected JMenuItem m_scaleRegioPolygon;
    
    protected JMenuItem m_simpleThressholdingSegmentationMenuItem;
    
    private JRadioButtonMenuItem m_showWindowingScale;
    private JRadioButtonMenuItem m_showImageAnnotation;
    
    private JRadioButtonMenuItem m_selectROIButton;
    private JRadioButtonMenuItem m_addROI;
    private JRadioButtonMenuItem m_editROI;
    private JRadioButtonMenuItem m_deleteROI;
    private JRadioButtonMenuItem m_moveROI;
    private JRadioButtonMenuItem m_cutROI;
    private JRadioButtonMenuItem m_addStackedROIMode;
    
    private JRadioButtonMenuItem m_addOneCircularROIMode;
    
    private JMenuItem            m_importROI;
    
    private SimpleThressholdSegmentationAnalyzerFrame m_simpleThressholdingSegmentationFrame;
    
    private ButtonGroup m_canvasMode;
    
    private JRadioButtonMenuItem m_displayGrayMenuItem;
    private JRadioButtonMenuItem m_displayColorMenuItem;
    private JRadioButtonMenuItem m_displayColorInverseMenuItem;
    private ButtonGroup          m_colorTableChoice;
    
    // attributes needed for image segment manipulation
    private boolean              m_imageSegmentDisplayAndManipulationMode = false;
    
    private JMenu                m_imageSegmentsMenu;

    private JRadioButtonMenuItem m_eliminatePixelsFromOverlaysByContourDrawingModeRadioButton;
    private JRadioButtonMenuItem m_eliminatePixelsFromVolumeOfInterestOverlaysByContourDrawingModeRadioButton;
    
    private JMenuItem m_promoteThressholdOverlayToTissueTypeJMenuItem;
    
    private JMenuItem m_loadImageSegmentsFromDefaultFiles;
    private JMenuItem m_saveImageSegmentsToDefaultFiles;

    private JMenuItem m_loadImageSegmentsFromUserDefinedDefaultFiles;
    private JMenuItem m_saveImageSegmentsToUserDefinedFiles;

    private JMenuItem m_deleteAllImageSegments;
    
    private JMenu     m_convertToRegioPolygonsMenu;
    
    private JMenuItem m_saveConvertedRegioPolygonsOfCurrentlyDisplayedImageOnlyMenuItem;
    private JMenuItem m_loadConvertedRegioPolygonsOfCurrentlyDisplayedImageOnlyMenuItem;
    private JMenuItem m_loadIOVOfAllImagesMenuItem;
    
    private JMenuItem m_saveConvertedRegioPolygonsOfAllImagesMenuItem;
    private JMenuItem m_saveRegioPolygonsAsIOVOfAllImagesMenuItem;
    
    private JRadioButtonMenuItem m_fieldOfInterestMenuItem;
    
    private JMenuItem m_loadConvertedRegioPolygonsOfAllImagesMenuItem;
    private JMenuItem m_statisticalPropertiesOfSelectedRegioPolygonMenuItem;
    private JMenuItem m_statisticalPropertiesOf3CircularRegioPolygonMenuItem;
    private JMenu     m_ROI_reportingMenu;
    
    private JMenuItem m_writeDetailedReportOfAllROIsofAllImagesMenuItem;
    private JMenuItem m_writeCondensedReportOfAllROIsofAllImagesMenuItem;
    
    private JMenuItem m_writeDetailedReportOfAllROIsofCurrrentlyDisplayedImageMenuItem;
    private JMenuItem m_writeCondensedReportOfAllROIsofCurrrentlyDisplayedImageMenuItem;
    
    private JMenuItem m_deleteAllConvertedRegioPolygonsOfAllImagesMenuItem;
    
    private JMenu     m_reportingMenu;
    
    private JMenuItem m_createReportFileOfCurrentlySelectedImageSegmentTypes;
    private JMenuItem m_createCondensedReportFileOfCurrentlySelectedImageSegmentType;
    private JMenuItem m_createReportFileOfDistinct3DVolumes;
    
    private JMenuItem m_convertToRegioPolygonsCurrentlyDisplayedOnly;
    private JMenuItem m_convertToRegioPolygonsForAllImages;
    private DualProgressBar m_dualProgressBar;
    
    private JMenu m_imageSegmentBooleanLogic;
    
    private JMenuItem m_applyLogic_AND_toImageSegments;
    private JMenuItem m_applyLogic_OR_toImageSegments;
    private JMenuItem m_applyLogic_EXOR_toImageSegments;
    private JMenuItem m_applyLogic_NOT_toImageSegments;
    
    private JMenuItem m_eliminateOverlayPixelsOutsideFocusRegion;
    private JMenuItem m_eliminateOverlayPixelsInsideFocusRegion;
    
    private JMenuItem m_convertImageSegmentsByMorphologicFiltering;

    private JMenuItem m_importFMRIOverlayFile;
    private JMenuItem m_importDTIOverlayFile;
    private JMenuItem m_reDicomizeImageStackWithOverlays;

    private JMenuItem m_reDicomizeImageStackWithOverlaysAsPlainDicom; // no color valid Dicom
     
    private JMenuItem m_openImageStackIn3DControl;
    
    private int startMouseDragX, startMouseDragY;

    protected String m_topTitle;
   
    // Variable needed enabling copying of regio's
    PolygonClipboard m_clipBoard;
    
    protected JFrame m_parent;
  
    // Position of mousecursor
    public int m_X, m_Y;
    
    
    // Members needed for mirror axis
    double m_sVecX, m_sVecY, m_rVecX, m_rVecY;
    
    private MirrorAxis m_mirrorAxis;
    private MirrorAxis m_mirrorVOIAxis;

    // EVENT TYPES THAT ARE RAISED BY THIS CLASS
    public final static int MOUSE_MOVED_EVENT = 0;

    public final static int ADD_ROI_MODE_SELECTED_EVENT     = 1;
    public final static int MOVE_ROI_MODE_SELECTED_EVENT    = 2;
    public final static int CUT_ROI_MODE_SELECTED_EVENT     = 3;
    public final static int SELECT_ROI_MODE_SELECTED_EVENT  = 4;
    public final static int DELETE_ROI_MODE_SELECTED_EVENT  = 5;
    public final static int RESIZE_ROI_MODE_SELECTED_EVENT  = 6;
    public final static int POLYGON_CLOSED_EVENT            = 7;
    public final static int ADD_STACKED_ROI_MODE_EVENT      = 8;
    
    
    // is raised whenever the user chooses to zoom the display
    public final static int IMAGE_ZOOM_EVENT                = 8;
    
    // new event type for segmentation: polygon change event
    // let listeners know that the polygon has changed
    public final static int POLYGON_CHANGED_EVENT = 9;
    public final static int POLYGON_REMOVED_EVENT = 10;
    
    public final static int POLYGON_CUT_EVENT = 567;
    
    public final static int POLYGON_FROM_FILE_LOAD_EVENT = 11;
    public final static String POLYGON_FROM_FILE_LOAD_CMD = "Load Polygon";



    public final static int POLYGON_RENAME_EVENT = 4523;

    public final static int DISPLAYED_IMAGE_NUMBER_CHANGED_EVENT = 11;
    
    public final static int CUT_3D_PERFORMED_EVENT = 12;
    public final static int IMAGE_SEGMENT_REMOVED_EVENT = 13;
    
    public final static int OVERLAYS_READ_FROM_FILE_EVENT = 14;
    
    public final static int ADDED_NEW_COMPUTED_OVERLAY_EVENT = 15;

    public final static int PROJECT_VOLUME_IN_OTHER_IMAGE_STACKS_EVENT = 16;

    public final static int PROMOTE_CURRENT_THRESSHOLD_OVERLAY_TO_TISSUE_TYPE = 17;
    
    private PixelValueReplacementFrame m_pixValReplacementFrame;
    
    
    private JMenuItem   m_advancedInteractiveSegmentationMenuItem;
    
    private JMenuItem   m_orderStatisticsSegmentationMenuItem;

    private JMenu       m_distanceMeasurementMenu;
    private JMenuItem   m_measureDistanceMenuItem;

    private JMenuItem   m_loadDistancesOfAllImagesFromFileMenuItem;
    private JMenuItem   m_loadDistancesOfCurrentlyDisplayedImageInFilesMenuItem;

    private JMenuItem   m_saveDistancesOfAllImagesInFilesMenuItem;
    private JMenuItem   m_saveDistancesOfCurrentlyDisplayedImageInFilesMenuItem;
    private JMenuItem   m_createReportFileOfDistancesMenuItem;
    
    private JMenuItem   m_createHistogramFilesOfCurrentlySelectedImageSegmentTypes;

    private JMenu m_segmentation;

    private JMenu m_volumeOfInterestMenu;

    private JMenuItem m_loadVolumeOfInterestFromDefault;
    private JMenuItem m_loadVolumeOfInterestUserDefined;
    private JMenuItem m_saveVolumeOfInterestToDefault;
    private JMenuItem m_saveVolumeOfInterestUserDefined;

    private JMenuItem m_projectionOfVolumeOfInterestInOtherImageStacks;
    private JMenuItem m_selectVolumesToOverlay;
    private JMenuItem m_opaquacyOfVolumesOverlays;
    
    private JMenuItem m_copyVolumeOfInterest;
    private JMenuItem m_pasteVolumeOfInterest;
    private JMenuItem m_deleteVolumeOfInterest;
    private JMenuItem m_loadMHAOverlay;
    
    private EnterDecimalValueFrame m_defineOpaquacyFactorDefinitionFrame;

    private AdvancedInteractiveSegmentationAnalyzerFrame m_advancedInteractiveSegmentationSegmentationFrame;
    private OrderStatisticsSegmentationAnalyzerFrame m_orderStatisticsSegmentationAnalyzerFrame;

    // Two attributes required to draw colored overlays to the image
    private boolean[][]     m_thresholdedOverlay;
    private Color           m_colorOfThressholdedOverlay;
    
    //private BufferedImage m_thressholdedImageSegmentOverlayAsBufferedImage;
    
    private boolean[][][]   m_thressholdedImageSegmentOverlays;
    private boolean[][][]   m_mhaImageSegmentOverlays;
    
    private boolean[][][]   m_activationImageOverlays;
    private boolean[][][]   m_dtiFiberImageOverlays;
    private Color[][][]     m_dtiFiberColors;

    // this member is necessary to store contours as boolean overlay images!
    private boolean[][][]   m_contoursAsBooleanImageOverlays;

    private ArrayList< LinkedList<boolean[][]> > m_imageSegmentsOverlayList = new ArrayList< LinkedList<boolean[][] > >( );
    private ArrayList< LinkedList<boolean[][]> > m_imageVolumeOfInterestOverlayList = new ArrayList< LinkedList<boolean[][] > >( );

    public LinkedList<Boolean> m_selectedImageVolumeOfInterestList = new LinkedList<Boolean>( );
    public LinkedList<String>  m_imageVolumeOfInterestNameList = new LinkedList<String>( );
    public LinkedList<Color>   m_imageVolumeOfInterestColorList = new LinkedList<Color>( );

    private Color[] m_imageSegmentColors;
    private int[]   m_selectedTissueTypes;
    private float   m_alpha = 1.0f;
    
    private Color[] defaultColors = { Color.BLUE, Color.RED, Color.GREEN, Color.ORANGE, Color.PINK, Color.MAGENTA, Color.CYAN,
                                      Color.BLUE.darker( ).darker(), Color.RED.darker( ).darker( ), Color.GREEN.darker( ).darker( ), 
                                      Color.ORANGE.darker( ).darker( ), Color.PINK.darker( ).darker( ), 
                                      Color.MAGENTA.darker( ).darker( ), Color.CYAN.darker( ).darker( ) };
    
    // the following attribute stores the image segments which were selected 
    // by the user by double click
    
    ArrayList<boolean[][]> m_userSelectedSegmentArrayList;
    
    boolean m_userAlreadySelectedImageSegmentsOnce = false;
    
    private BufferedImage[] m_imageSegmentsAsBufferedImage;

    private BufferedImage m_compositeImageSegmentsOverlay;
    
    private ImageRestorationFilterFrame m_imageRestorationFrame;
    private MorphologicFilterFrame m_morphologicFilterFrame;
    
    private EnterDecimalValueFrame m_scaleFactorDefinitionFrame;
    
    public RegioPolygon m_roiToSegment = null;
    
    // contains the ImageSegmentProperties of currently displayed image (if valid)
    LinkedList<ImageSegmentProperties> m_imageSegmentPropertiesList;
    
    private EnterDecimalValueFrame m_selectImageNumberToDisplayDefined;
    private RegioPolygon m_cutting3DPolygon;
    // For proper serialization
    private static final long serialVersionUID = 1482325898865493L;
 
    boolean[][] m_connectedPixels;
    
    int m_subSegmentationScope          = ProbingRegionPropertiesPanel.SUBSEGMENT_IN_CURRENTLY_SELECTED_SLICE_ONLY;
    int m_setPixelRangeComputationType  = ProbingRegionPropertiesPanel.PIXEL_RANGE_DEFINED_BY_STANDARD_DEVIATION;

    private OrderStatisticSegmentationCalculator m_OS_SegmentationCalculator; 
    
    private final static int SHOW_SAVE_DIALOG = 1;
    private final static int SHOW_LOAD_DIALOG = 2;

    private boolean m_interactiveSegmentationMode = false;

    // test code for interactive segmentation overlay
    private boolean[][] m_interactiveSegmentationOverlay;

    //Point indicating the last indicated mouse cursor
    private Point m_lastInteractivelyIndicatedPoint;

    // The user needs to set dicom export and sent before dicomizing...
    private DicomFileCreatorOptionsFrame m_dicomFileAsSecondaryCaptureCreatorOptionsFrame;
    private DicomFileCreatorOptionsFrame m_trueDicomFileCreatorOptionsFrame;
 

    private int m_seriesNumberToUseForDicomization;

    private RegionManipulationFrame m_regionManipulationFrame;

    private boolean m_measureDistanceMode = false;
    private boolean m_displayMeasureDistanceMode = false;

    private MeasureDistancesFrame m_measureDistancesFrame;

    private boolean m_hasProcessedImagePixelData = false;

    private Point3D m_doubleClickedPoint3D;
    private Point   m_doubleClickedPoint;

    // private boolean m_drawBufferedImageThresholdOverlay = true;
    private Point3D[] m_topLeftPoints;

    private float m_opaquacyOfVolumeOverlays = 0.3f;

    private VolumeOfInterestOverlayManipulationFrame m_voi_ManipulationFrame;

    // variables needed for local statistics management
    private boolean[][][] m_localMeanImageSegmentOverlays;
    private boolean[][][] m_localVarianceImageSegmentOverlays;
    private boolean[][][] m_localSkewnessImageSegmentOverlays;
    private boolean[][][] m_localKurtosisImageSegmentOverlays;

    // overlays for left and right side marking
    private boolean[][][] m_leftSideImageSegmentOverlays;
    private boolean[][][] m_rightSideImageSegmentOverlays;

    
    private float[] m_localMeanValues;
    private float[] m_localVarianceValues;
    private float[] m_localSkewnessValues;
    private float[] m_localKurtosisValues;

    private float m_localMeanMinimumValue = Float.MAX_VALUE;
    private float m_localMeanMaximumValue = -Float.MAX_VALUE;

    private float m_localVarianceMinimumValue = Float.MAX_VALUE;
    private float m_localVarianceMaximumValue = -Float.MAX_VALUE;

    private float m_localSkewnessMinimumValue = Float.MAX_VALUE;
    private float m_localSkewnessMaximumValue = -Float.MAX_VALUE;

    private float m_localKurtosisMinimumValue = Float.MAX_VALUE;
    private float m_localKurtosisMaximumValue = -Float.MAX_VALUE;

    private boolean m_localStatisticsIsComputed = false;


    private float     m_meanFilterMinimumRangeValue;
    private float     m_meanFilterMaximumRangeValue;

    private float     m_varianceFilterMinimumRangeValue;
    private float     m_varianceFilterMaximumRangeValue;

    private float     m_skewnessFilterMinimumRangeValue;
    private float     m_skewnessFilterMaximumRangeValue;

    private float     m_kurtosisFilterMinimumRangeValue;
    private float     m_kurtosisFilterMaximumRangeValue;

    private float     m_localMeanRange;
    private float     m_localVarianceRange;
    private float     m_localSkewnessRange;
    private float     m_localKurtosisRange;

    private int       m_localStatisticsDisplayOverlayType = -1;

    private JMenuItem m_createTextureAnalyzerMenuItem;

    private String      m_regioNameBeforeRenaming;
    private String      m_lastRemovedRegioPolygonFileName;

    private RegioPolygon     m_lastLoadedPolygon;
    private int              m_lastLoadedPolygonSliceNumber = -1;
    private boolean     m_overwriteAllRegioFiles = false;
    private boolean     m_alreadyAskedToOverwriteAllRegioFiles = false;

    private double[] m_intercepts;
    private double[] m_slopes;
    
    private ScaledROIDefintionFrame m_scaledReferenceFrame;
    
    private RegioPolygon m_fieldOfInterest;
    
}
