/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

/**
 * QuantitativeFittingResultsOverviewFrame.java
 *
 * Created on 17-Jan-2011, 23:26:00
 */

package org.spectrim.tdfdfit.quantitativeresults;

import org.spectrim.dicom.DicomImageData;
import org.spectrim.dicom.DicomNetworkNodeDefaults;
import org.spectrim.dicom.DicomNetworkNodeDefinerAndSelectionFrame;
import org.spectrim.dicom.FileInfo;
import org.spectrim.lib.Dialogs;

import org.spectrim.tdfdfit.TextFileWriter;
import org.spectrim.tdfdfit.SingleVoxelMeasuredSpectralData;

import java.awt.Dimension;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.text.DecimalFormat;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.TreeMap;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.SpinnerModel;
import javax.swing.SpinnerNumberModel;
import javax.swing.table.DefaultTableModel;
import org.dcm4che2.data.DicomObject;
import org.spectrim.dicom.DicomDirectory;
import org.spectrim.imagegraphicobjects.Point3D;
import org.spectrim.io.dicom.DicomHandler;
import org.spectrim.io.dicom.DicomHeader;
import org.spectrim.io.dicom.DicomHeaderLine;
import org.spectrim.model.GlobalModel;

/**
 * Frame that allows the quantitative results to be viewed and to be 
 * 1: Assigned to a normal value database
 * 2: Compared to normal value databases
 * 3: Generate report files of the results
 * @author johannes slotboom
 */
public class QuantitativeFittingResultsOverviewFrame extends javax.swing.JFrame implements ActionListener 
{

     /**
     * Add hoc solution for reading Dicom Philips Tags.
     * @param header
     * @param tag
     * @return 
     */
    protected String readHeaderValue(DicomHeader header, String tag) {
        DicomHeaderLine headerLine;
        headerLine = (DicomHeaderLine) header.get(tag);
        if (headerLine != null) {
            String val = headerLine.Value.replace("\"", "");
            return val;
        } else {
            // System.out.println("No value found for tag: " + tag);
            return "0";
        }
    }
    
    /** 
     * Creates new form QuantitativeFittingResultsOverviewFrame
     * @param parent
     * @param dicomDir
     * @param spectroscopyDicomFileReferences
     * @param twoCompartmentModelSet
     * @param twoCompartmentModelFitted
     * @param referenceWaterDefined
     * @param rawWaterReferenceSpectrosopyDicomFileName
     * @param numberOfMetabolites
     * @param numberOfSignalsFitted
     * @param metaboliteNames
     * @param concentrations
     * @param errorConcentrations
     * @param chemShift
     * @param errorsChemShift
     * @param phase
     * @param errorsPhase
     * @param lorentzWidth
     * @param errorsLorentzWidth
     * @param lorentzRWidth
     * @param errorsLorentzRWidth
     * @param numberOf2CompWaterRefResponses
     * @param amp1_TCM
     * @param amp2_TCM
     * @param TC1_TCM
     * @param TC2_TCM
     * @param err_amp1_TCM
     * @param err_amp2_TCM
     * @param fittedReal
     * @param err_TC1_TCM
     * @param measuredImag
     * @param measuredReal
     * @param referenceImage2
     * @param referenceImage1
     * @param err_TC2_TCM
     * @param referenceImage0
     * @param waterRefReal
     * @param SVSJMRUIDicomTag
     * @param waterRefImag
     * @param fittedImag
     * @param globalModel
     */
    public QuantitativeFittingResultsOverviewFrame( JFrame parent,
                                                    DicomDirectory dicomDir,
                                                    LinkedList<String> spectroscopyDicomFileReferences,
                                                    boolean twoCompartmentModelSet,
                                                    boolean twoCompartmentModelFitted,
                                                    boolean referenceWaterDefined,
                                                    String  rawWaterReferenceSpectrosopyDicomFileName,
                                                    int numberOfMetabolites,
                                                    int numberOfSignalsFitted,
                                                    LinkedList<String> metaboliteNames,
                                                    double[][] concentrations,
                                                    double[][] errorConcentrations,
                                                    double[][] chemShift,
                                                    double[][] errorsChemShift,
                                                    double[][] phase,
                                                    double[][] errorsPhase,
                                                    double[][] lorentzWidth,
                                                    double[][] errorsLorentzWidth,
                                                    double[][] lorentzRWidth,
                                                    double[][] errorsLorentzRWidth,
                                                    int numberOf2CompWaterRefResponses,
                                                    double amp1_TCM,
                                                    double amp2_TCM,
                                                    double TC1_TCM,
                                                    double TC2_TCM,
                                                    double err_amp1_TCM,
                                                    double err_amp2_TCM,
                                                    double err_TC1_TCM,
                                                    double err_TC2_TCM,
                                                    double[] waterRefReal,
                                                    double[] waterRefImag,
                                                    SVS_JMRUI_DICOM_Tag_FAST SVSJMRUIDicomTag,
                                                    String referenceImage0,
                                                    String referenceImage1,
                                                    String referenceImage2,
                                                    double[][] measuredReal,
                                                    double[][] measuredImag,
                                                    double[][] fittedReal,
                                                    double[][] fittedImag,
                                                    GlobalModel globalModel,
                                                    double fitQualityNumber
                                                   )
    {

        m_fitQualityNumber = fitQualityNumber;
        //InitializeWithUnknownTimeNeededFrame frame = new InitializeWithUnknownTimeNeededFrame( parent );
       
        initComponents( );
       
        searchNormalDataBasesAndFillAvaiableDatabaseTable( );

        // listen to actions in the normal database table! 
        m_normalDataBaseSelectionTable.addActionListener( this );
        m_dicomDir = dicomDir;
        m_rawWaterReferenceSpectrosopyDicomFileName = rawWaterReferenceSpectrosopyDicomFileName;
       
        m_SVSJMRUIDicomTag = SVSJMRUIDicomTag;
       
        m_referenceImage0 = referenceImage0;
        m_referenceImage1 = referenceImage1;
        m_referenceImage2 = referenceImage2;

        m_measuredReal = new double[numberOfSignalsFitted][measuredReal[0].length];
        m_measuredImag = new double[numberOfSignalsFitted][measuredReal[0].length];
        m_fittedReal   = new double[numberOfSignalsFitted][measuredReal[0].length];
        m_fittedImag   = new double[numberOfSignalsFitted][measuredReal[0].length];

        for ( int i = 0; i < numberOfSignalsFitted; i++ )
        {
            for ( int j = 0 ; j < measuredReal[0].length ; j++ )
            {

                m_measuredReal[i][j] = measuredReal[i][j];
                m_measuredImag[i][j] = measuredImag[i][j];
                m_fittedReal[i][j]   = fittedReal[i][j];
                m_fittedImag[i][j]   = fittedImag[i][j];
            }
            
        }

        m_spectroscopyDicomFileReferences = spectroscopyDicomFileReferences;
        
        DicomImageData d_ima_data = new DicomImageData( m_spectroscopyDicomFileReferences.getFirst( ) );

        int dim1, dim2, dim3;
        dim1=dim2=dim3=1;
        if ( !d_ima_data.getHeader().getManufacturer().contains( "Philips" ) )
        {
            LinkedHashMap ssd = d_ima_data.getSiemensSpecificDataFromFile( );
            setSiemensSpecificDicomHeaderInfo( ssd );

            if ( (String)ssd.get( "sSpecPara.lFinalMatrixSizePhase" ) != null )
            {
                dim1 = ( new Integer( (String)ssd.get( "sSpecPara.lFinalMatrixSizePhase" ) ) ).intValue( );
            }

            if ( (String)ssd.get( "sSpecPara.lFinalMatrixSizePhase" ) != null )
            {
                dim2 = ( new Integer( (String)ssd.get( "sSpecPara.lFinalMatrixSizePhase" ) ) ).intValue( );
            }

            // 3D data set not yet available, so set the value to one
            dim3 = 1;
            
            if ( ( dim1 == 1 ) && ( dim2 == 1 ) && ( dim3 == 1 ) )
            {
                m_sequenceTypeLabel.setText( "SVS" );
            }
            else
            {
                 m_sequenceTypeLabel.setText( "MRSI" );
            }
            
            // we are dealing with an Single Voxel Spectroscopy measurement
            FileInfo fi = d_ima_data.getHeader( );
            SingleVoxelMeasuredSpectralData specData =  new SingleVoxelMeasuredSpectralData( fi.getNonImageData( ) );
            specData.setDicomHeaderInfo( fi );
            specData.setSiemensSpecificDicomHeaderInfo( d_ima_data.getSiemensSpecificDataFromFile( ) );

            m_voxelDimensionString = specData.getVoxelDimensionsAsString( );
            m_patientNameLabel.setText( fi.getPatientName( )  + " (" + fi.getPatientSex( ) + ")" );
            m_patientAgeLabel.setText( fi.getPatientBirthDate( ) + " (" + fi.getPatientAge( ) + ")" );
            m_patientIDLabel.setText( fi.getPatientID( ) );
            m_studyDateLabel.setText( fi.getStudyDate( ) );
            m_studyTimeLabel.setText( fi.getStudyTime( ) );
            m_seriesDescriptionLabel.setText( fi.getSeriesDescription( ) );
            m_studyDescription.setText( fi.getStudyDescription( ) );
            m_scannerNameLabel.setText( fi.getScannerModelName( ) );
            m_spectroscopicVoxelSizeLabel.setText( specData.getVoxelDimensionsAsString( ) );
            m_resonanceFrequencyLabel.setText(  "" + specData.getResonanceFrequency( ) );

            m_sequenceTypeLabel.setText( "SVS" );
            m_echoTimeLabel.setText( "" + m_TE );
            m_TRLabel.setText(  "" + m_TR );
            m_globalModel = globalModel;
        }
        else
        {
            // Philips data
            DicomObject object = DicomHandler.getDicomObject( m_spectroscopyDicomFileReferences.getFirst( ) );
            DicomHeader header = DicomHandler.getHeader( object );
            
            dim2 = Integer.valueOf( readHeaderValue( header, "(0028,0010)" ) ); // Nr Rows
            dim1 = Integer.valueOf( readHeaderValue( header, "(0028,0011)" ) ); // Nr Columns
            dim3 = Integer.valueOf( readHeaderValue( header, "(0028,0008)" ) ); // Nr Frames
            
            if ( ( dim1 == 1 ) && ( dim2 == 1 ) && ( dim3 == 1 ) )
            {
                m_sequenceTypeLabel.setText( "SVS" );
            }
            else
            {
                 m_sequenceTypeLabel.setText( "MRSI" );
            }
            Point3D voiSize = new Point3D();
            voiSize.m_x1 = Double.valueOf(readHeaderValue(header, "(0018,9126)[0]->(0018,9104)"));
            voiSize.m_x2 = Double.valueOf(readHeaderValue(header, "(0018,9126)[1]->(0018,9104)"));
            voiSize.m_x3 = Double.valueOf(readHeaderValue(header, "(0018,9126)[2]->(0018,9104)"));
            
            m_voxelDimensionString = "[" + voiSize.m_x1 + "," + voiSize.m_x2 + "," + voiSize.m_x3 + "]";
            
            m_patientNameLabel.setText( readHeaderValue(header, "(0010,0010)")  + " "
                                        + "(" + readHeaderValue(header, "(0010,0040)") + ")" );
            m_patientAgeLabel.setText( readHeaderValue(header, "(0010,0030)") );
            m_patientIDLabel.setText( readHeaderValue(header, "(0010,0020)") );
            m_studyDateLabel.setText( readHeaderValue(header, "(0008,0020)") );
            m_studyTimeLabel.setText( readHeaderValue(header, "(0008,0030)") );
            m_seriesDescriptionLabel.setText( readHeaderValue( header, "(0008,103E)" ) );
            m_studyDescription.setText( "Study ID : " + readHeaderValue(header, "(0020,0010)") );
            m_scannerNameLabel.setText( readHeaderValue( header, "(0008,0070)" ) + " " + readHeaderValue( header, "(0008,1090)" ) );
            m_spectroscopicVoxelSizeLabel.setText( m_voxelDimensionString );
            m_resonanceFrequencyLabel.setText(  "" + Double.valueOf( readHeaderValue( header, "(0018,9098)" ) ) * 1000000 );
            
            m_resonanceFrequency =  new Double( Double.valueOf( readHeaderValue( header, "(0018,9098)" ) ) * 1000000 ).longValue();
            m_TE = Double.valueOf( readHeaderValue( header, "(0018,9114)->(0018,9082)" ) ).longValue( );
            m_TR = Double.valueOf( readHeaderValue( header, "(0018,9112)->(0018,0080)" ) ).longValue( );
            
            m_sequenceTypeLabel.setText( "SVS" );
            m_echoTimeLabel.setText( "" + m_TE );
            m_TRLabel.setText(  "" + m_TR );    
            m_globalModel = globalModel;
        }
        
        /////////////////////////////////////////////////////////////////////////
        
        setSelectedNodeValue( );

        if ( m_selectedHostName != null )
        {
            m_selectedDICOMNodeValue.setText( m_selectedHostName + " (" + m_selectedHostIP + ")") ;
        }
        
        m_twoCompartmentModelSet    = twoCompartmentModelSet;
        m_twoCompartmentModelFitted = twoCompartmentModelFitted;
        m_referenceWaterDefined     = referenceWaterDefined;

        m_numberOfMetabolites   = numberOfMetabolites;
        m_numberOfSignalsFitted = numberOfSignalsFitted;

        m_liquorTimeConstantThressholdTextField.setText( "" + m_TC_Thresshold );

        m_metaboliteNames = metaboliteNames;
        SpinnerModel model = new SpinnerNumberModel( 1, 1, m_numberOfSignalsFitted, 1 ); // initial, minimum, maximum, step
        m_selectResponseSpinner.setModel( model );
        m_displayResponseNumberLabel.setText( "" + m_numberOfSignalsFitted );

        // instantiate data structures..
        m_concentrations = new double[m_numberOfSignalsFitted][m_numberOfMetabolites];
        m_errorConcentrations= new double[m_numberOfSignalsFitted][m_numberOfMetabolites];

        // storing a copy of the concentrations
        m_initialConcentrations = new double[m_numberOfSignalsFitted][m_numberOfMetabolites];
        m_initialErrorConcentrations= new double[m_numberOfSignalsFitted][m_numberOfMetabolites];

        m_chemShift = new double[m_numberOfSignalsFitted][m_numberOfMetabolites];
        m_errorsChemShift = new double[m_numberOfSignalsFitted][m_numberOfMetabolites];

        m_phase = new double[m_numberOfSignalsFitted][m_numberOfMetabolites];
        m_errorsPhase = new double[m_numberOfSignalsFitted][m_numberOfMetabolites];

        m_lorentzWidth = new double[m_numberOfSignalsFitted][m_numberOfMetabolites];
        m_errorsLorentzWidth = new double[m_numberOfSignalsFitted][m_numberOfMetabolites];

        m_lorentzWidthR = new double[m_numberOfSignalsFitted][m_numberOfMetabolites];
        m_errorsLorentzWidthR = new double[m_numberOfSignalsFitted][m_numberOfMetabolites];


        for ( int j = 0; j < m_numberOfSignalsFitted; j++ )
        {
            for ( int i = 0; i < m_numberOfMetabolites; i++ )
            {

                m_initialConcentrations[j][i] = concentrations[j][i];
                m_initialErrorConcentrations[j][i] = errorConcentrations[j][i];

                m_concentrations[j][i]  = concentrations[j][i];
                m_errorConcentrations[j][i] = errorConcentrations[j][i];

                m_initialConcentrations[j][i]  = concentrations[j][i];
                m_initialErrorConcentrations[j][i] = errorConcentrations[j][i];

                m_chemShift[j][i] = chemShift[j][i];
                m_errorsChemShift[j][i] = errorsChemShift[j][i];

                m_phase[j][i] = phase[j][i];
                m_errorsPhase[j][i] = errorsPhase[j][i];

                m_lorentzWidth[j][i] = lorentzWidth[j][i];
                m_errorsLorentzWidth[j][i] = errorsLorentzWidth[j][i];

                m_lorentzWidthR[j][i] = lorentzRWidth[j][i];
                m_errorsLorentzWidthR[j][i] = errorsLorentzRWidth[j][i];
            }
        }

        m_numberOfSignalsValueLabel.setText( "" + numberOf2CompWaterRefResponses );

        DecimalFormat formatter = new DecimalFormat( "0.00E00" );
        DecimalFormat formatter2 = new DecimalFormat( "#########0.000" );
        
        m_Amp1 = amp1_TCM;
        m_Amp2 = amp2_TCM;
        m_TC1 = TC1_TCM;
        m_TC2 = TC2_TCM;

        m_amp_1_ValueLabel.setText( "" + formatter.format( amp1_TCM ) + " +/- " + formatter.format( err_amp1_TCM ) );
        m_amp_2_ValueLabel.setText( "" + formatter.format( amp2_TCM ) + " +/- " + formatter.format( err_amp2_TCM ) );
        m_TC_1_ValueLabel.setText( "" + formatter2.format( TC1_TCM )  + " +/- " + formatter2.format( err_TC1_TCM ) );
        m_TC_2_ValueLabel.setText( "" + formatter2.format( TC2_TCM )  + " +/- " + formatter2.format( err_TC2_TCM ) );


        if ( ( waterRefReal != null ) && ( waterRefReal.length > 0 ) )
        {
            m_waterRefReal = new double[waterRefReal.length];
            m_waterRefImag = new double[waterRefReal.length];

            for ( int i = 0; i < waterRefReal.length; i++ )
            {
                 m_waterRefReal[i] =  waterRefReal[i];
                 m_waterRefImag[i] =  waterRefImag[i];
            }

            if ( Math.sqrt( m_waterRefReal[0] * m_waterRefReal[0] + m_waterRefImag[0] * m_waterRefImag[0] )  != 0.0 )
            {
                m_waterReferenceLoadedStatusLabel.setText( "defined" );
                m_waterAmplitudeValueLabel.setText( "" + formatter.format( Math.sqrt( m_waterRefReal[0] * m_waterRefReal[0] + m_waterRefImag[0] * m_waterRefImag[0] ) ) );
            }
        }
        
        if ( ( twoCompartmentModelSet == false ) || ( twoCompartmentModelFitted == false ) )
        {
            m_twoCompartmentModelFractionOnlyCorrectedOnlyRadioButton.setEnabled( false );
            m_trueTwoCompartmentCorrectedRadioButton.setEnabled( false );
        }

        if ( referenceWaterDefined == false )
        {
            m_singleWaterReferenceOnlyCorrectedRadioButton.setEnabled( false );
        }

        // Initialize the metabolite database depending on B0
        // 0.5T
        if ( ( m_resonanceFrequency > 18000000 ) && ( m_resonanceFrequency < 24000000 ) )
        {
            m_fieldStrengthDatabaseComboBox.setSelectedIndex( 0 );
        }

        // 1.0T
        if ( ( m_resonanceFrequency > 39000000 ) && ( m_resonanceFrequency < 45000000 ) )
        {
            m_fieldStrengthDatabaseComboBox.setSelectedIndex( 1 );
        }

        // 1.5T
        if ( ( m_resonanceFrequency > 60000000 ) && ( m_resonanceFrequency < 66000000 ) )
        {
            m_fieldStrengthDatabaseComboBox.setSelectedIndex( 2 );
        }

        // 2.0T
        if ( ( m_resonanceFrequency > 81000000 ) && ( m_resonanceFrequency < 87000000 ) )
        {
            m_fieldStrengthDatabaseComboBox.setSelectedIndex( 3 );
        }

        // 3.0T
        if ( ( m_resonanceFrequency > 121000000 ) && ( m_resonanceFrequency < 131000000 ) )
        {
            m_fieldStrengthDatabaseComboBox.setSelectedIndex( 4 );
        }

        // 4.0T
        if ( ( m_resonanceFrequency > 163000000 ) && ( m_resonanceFrequency < 173000000 ) )
        {
            m_fieldStrengthDatabaseComboBox.setSelectedIndex( 5 );
        }

        // 6.3T
        if ( ( m_resonanceFrequency > 260000000 ) && ( m_resonanceFrequency < 270000000 ) )
        {
            m_fieldStrengthDatabaseComboBox.setSelectedIndex( 6 );
        }
        // 7.0T
        if ( ( m_resonanceFrequency > 290000000 ) && ( m_resonanceFrequency < 310000000 ) )
        {
            m_fieldStrengthDatabaseComboBox.setSelectedIndex( 7 );
        }

        m_currentlySelectedRelaxationTimeDatabaseNameString = (String)m_fieldStrengthDatabaseComboBox.getSelectedItem( );
        m_metabRelaxDataBase = MetaboliteRelaxationTimesDatabase.getInstance( );
        m_currentlySelectedDataBase = m_metabRelaxDataBase.getDataBase( m_currentlySelectedRelaxationTimeDatabaseNameString );
        
        updateMetaboliteRelaxationTimesTable( );

         // add window listener necessary to handle the window closing
        // event properly
        addWindowListener(
            new WindowAdapter( )
            {
                @Override
                public void windowClosing( WindowEvent event )
                {
                   dispose( );
                }
            }
        );

        setMaximumSize( new Dimension( 1016, 682 ) );
        setMinimumSize( new Dimension( 1016, 682 ) );
        
        setUpResultsTable( );
        //pack( );
        
        m_parent = parent;
        // Put the Dialog nicely in the middle of its parent
        Point parentLoc = parent.getLocation( );
        Point centerParent = new Point( parent.getX( ) + ( parent.getWidth( ) / 2 ) - this.getWidth( ) / 2,
                                        parent.getY( ) + ( parent.getHeight( ) / 2 ) - this.getHeight( ) / 2 );

        setTitle( "Quantitative Fitting Results Overview Frame" );
        pack( );
        setLocation( centerParent );

        //frame.dispose( );

        setVisible( true );

    }

    /**
     * Updates the table with the metabolite relaxation times.
     */
    private void updateMetaboliteRelaxationTimesTable( )
    {

        m_T1s = new double[ m_metaboliteNames.size( ) ];
        m_T2s = new double[ m_metaboliteNames.size( ) ];
        m_T1TRCorrFactor = new double[ m_metaboliteNames.size( ) ];
        m_T2CorrFactor = new double[ m_metaboliteNames.size( ) ];
        m_multiplicities = new int[ m_metaboliteNames.size( ) ];
        
        for ( int i = 0; i < m_metaboliteNames.size( ); i++ )
        {
            Collection c = m_currentlySelectedDataBase.keySet( );
            Iterator it = c.iterator( );

            // give the relaxation values proper default value
            m_T1s[i] = 2000.0;
            m_T2s[i] = 300.0;
            m_T1TRCorrFactor[i] = 1.0 - Math.exp( - m_TR / m_T1s[i] );
            m_T2CorrFactor[i] = Math.exp( - m_TE / m_T2s[i] );
            
            // set the default multiplicity to 1 spin
            m_multiplicities[i] = 1;
            
            while ( it.hasNext( ) )
            {
                String metabName = (String)it.next( );

                if ( metabName.compareTo( m_metaboliteNames.get( i ) ) == 0 )
                {
                    // System.out.println( "Match!! metabName : " + metabName + " list metab name : " +  m_metaboliteNames[i] + "." );
                    MetaboliteRelaxationTimeProperties props = m_currentlySelectedDataBase.get( metabName );

                    // System.out.println( "Name " +  props.getMetaboliteName( ) );
                    // System.out.println( "T1 = " +  props.getT1( ) );
                    // System.out.println( "T2 = " +  props.getT2( ) );

                    m_T1s[i] = props.getT1( ) * 1000.0;
                    m_T2s[i] = props.getT2( ) * 1000.0;
                    m_T1TRCorrFactor[i] = 1.0 - Math.exp( - m_TR / m_T1s[i] );
                    m_T2CorrFactor[i] = Math.exp( - m_TE / m_T2s[i] );
                    m_multiplicities[i] = props.getMultiplicity( );
                    
                }
                
            }
        }
        
        // now fill in the table
        m_metabRelaxTimesTableCellData = new Object[m_numberOfMetabolites][6];
        DecimalFormat formatter  = new DecimalFormat(  "#########0.000" );

        for ( int i = 0; i < m_numberOfMetabolites; i++ )
        {
            m_metabRelaxTimesTableCellData[i][0] = m_metaboliteNames.get( i );

            if ( m_T1s[i] != -1000.0 )
            {
                m_metabRelaxTimesTableCellData[i][1] = new String( formatter.format( m_T1s[i] ) );
                m_metabRelaxTimesTableCellData[i][3] = new String( formatter.format( m_T1TRCorrFactor[i] ) );
            }
            else
            {
                m_metabRelaxTimesTableCellData[i][1] = "Unknown";
                m_metabRelaxTimesTableCellData[i][3] = new String( formatter.format( 1.000 ) );
            }

            if ( m_T2s[i] != -1000.0 )
            {
                m_metabRelaxTimesTableCellData[i][2] = new String( formatter.format( m_T2s[i] ) );
                m_metabRelaxTimesTableCellData[i][4] = new String( formatter.format( m_T2CorrFactor[i] ) );
            }
            else
            {
                m_metabRelaxTimesTableCellData[i][2] = "Unknown";
                m_metabRelaxTimesTableCellData[i][4] = new String( formatter.format( 1.000 ) );
            }
            
            m_metabRelaxTimesTableCellData[i][5] = new String( "" + m_multiplicities[i] );
        }

        String[] columnNames = null;


        String[] columnNames1 = { "<HTML><B>Metabolite name</B></HTML>",
                                  "<HTML><B>Relaxation time T1 [msec]</B></HTML>",
                                  "<HTML><B>Relaxation time T2 [msec]</B></HTML>",
                                  "<HTML><B>T1/TR correction factor</B><HTML>",
                                  "<HTML><B>T2 correction factor</B><HTML>",
                                  "<HTML><B>Multiplicity</B><HTML>"
                                };
        columnNames = columnNames1;

        m_tableModelRelaxTimes = new DefaultTableModel( m_metabRelaxTimesTableCellData, columnNames );

        m_metabRelaxTimesTable.setModel( m_tableModelRelaxTimes );

        m_metabRelaxTimesTable.updateUI( );
        
    }


    /**
     * Search normal value databases and fill the table with 
     * observed information.
     */
    private void searchNormalDataBasesAndFillAvaiableDatabaseTable( )
    {
        NormalMetaboliteValuesAndSpectrumDatabase dBase = new NormalMetaboliteValuesAndSpectrumDatabase( );
        Object[][] dBaseInformation = dBase.getAvailableNormalMetaboliteValuesdBaseInformation( );

        ( (NormalDataBaseSelectionTableModel)m_normalDataBaseSelectionTable.getModel( ) ).setDataVector( dBaseInformation.length,
                                                                                                         dBase.getNumberOfColumns( ),
                                                                                                         dBaseInformation );
    }

    /**
     * Set the selected DICOM node.
     */
    private void setSelectedNodeValue( )
    {

        DicomNetworkNodeDefaults def = DicomNetworkNodeDefaults.getInstance( );
        int selectedNode = Integer.parseInt(def.getProperty( DicomNetworkNodeDefaults.SELECTED_REMOTE_DICOM_HOST_NUMBER ) );

        if ( selectedNode == 0 )
        {
            m_selectedHostName = def.getProperty( DicomNetworkNodeDefaults.REMOTE_DICOM_HOST_0_NAME );
            m_selectedHostIP   = def.getProperty( DicomNetworkNodeDefaults.REMOTE_DICOM_HOST_0_IP_ADDRESS );
        }

        if ( selectedNode == 1 )
        {
            m_selectedHostName = def.getProperty( DicomNetworkNodeDefaults.REMOTE_DICOM_HOST_1_NAME );
            m_selectedHostIP   = def.getProperty( DicomNetworkNodeDefaults.REMOTE_DICOM_HOST_1_IP_ADDRESS );
        }

        if ( selectedNode == 2 )
        {
            m_selectedHostName = def.getProperty( DicomNetworkNodeDefaults.REMOTE_DICOM_HOST_2_NAME );
            m_selectedHostIP   = def.getProperty( DicomNetworkNodeDefaults.REMOTE_DICOM_HOST_2_IP_ADDRESS );
        }

       if ( selectedNode == 3 )
        {
            m_selectedHostName = def.getProperty( DicomNetworkNodeDefaults.REMOTE_DICOM_HOST_3_NAME );
            m_selectedHostIP   = def.getProperty( DicomNetworkNodeDefaults.REMOTE_DICOM_HOST_3_IP_ADDRESS );
        }
    }


    /**
     * Sets up the results table.
     */
    private void setUpResultsTable( )
    {
        
        DecimalFormat formatter  = new DecimalFormat(  "0.00000E00" );

        // set the correct data object and size
        m_cellData = new Object[m_numberOfMetabolites][6];

        for ( int i = 0; i < m_numberOfMetabolites; i++ )
        {
            // names of metabs
            m_cellData[i][0] = m_metaboliteNames.get( i );

            // concentrations
            m_cellData[i][1] = formatter.format( m_concentrations[m_currentlySelectedSignalNumber][i] )  + " +/- " + formatter.format( m_errorConcentrations[m_currentlySelectedSignalNumber][i]  );
            
            // chem. shift
            m_cellData[i][2] = formatter.format( m_chemShift[m_currentlySelectedSignalNumber][i] ) + " +/- " + formatter.format( m_errorsChemShift[m_currentlySelectedSignalNumber][i] );

            // phase
            m_cellData[i][3] = formatter.format( m_phase[m_currentlySelectedSignalNumber][i] ) + " +/- " + formatter.format( m_errorsPhase[m_currentlySelectedSignalNumber][i] );

            // lorentz width
            m_cellData[i][4] = formatter.format( m_lorentzWidth[m_currentlySelectedSignalNumber][i] ) + " +/- " + formatter.format( m_errorsLorentzWidth[m_currentlySelectedSignalNumber][i] );

            // lorentzR width
            m_cellData[i][5] = formatter.format( m_lorentzWidthR[m_currentlySelectedSignalNumber][i] ) + " +/- " + formatter.format( m_errorsLorentzWidthR[m_currentlySelectedSignalNumber][i] );
        }

        String[] columnNames = null;

        if ( m_rawjMRUIAmplitudesRadioButton.isSelected( ) == true )
        {
            String[] columnNames1 = { "<HTML><B>Component name</B></HTML>",
                                      "<HTML><B>Peakareas raw [a.u]</B></HTML>",
                                      "<HTML><B>Chem. shift</B></HTML>",
                                      "<HTML><B>Phase</B><HTML>",
                                      "<HTML><B>Lorentzian damping</B><HTML>",
                                      "<HTML><B>Lorentzian damping R</B><HTML>"
                          };

            columnNames = columnNames1;
        }

        if ( m_twoCompartmentModelFractionOnlyCorrectedOnlyRadioButton.isSelected( ) == true )
        {
            String[] columnNames1 = { "<HTML><B>Component name</B></HTML>",
                                      "<HTML><B>Peakareas corr. [a.u]</B></HTML>",
                                      "<HTML><B>Chem. shift</B></HTML>",
                                      "<HTML><B>Phase</B><HTML>",
                                      "<HTML><B>Lorentzian damping</B><HTML>",
                                      "<HTML><B>Lorentzian damping R</B><HTML>"
                                    };
            
            columnNames = columnNames1;
        }

        if ( m_singleWaterReferenceOnlyCorrectedRadioButton.isSelected( ) == true )
        {
            String[] columnNames1 = { "<HTML><B>Component name</B></HTML>",
                                      "<HTML><B>Peakareas [mmol/l]</B></HTML>",
                                      "<HTML><B>Chem. shift</B></HTML>",
                                      "<HTML><B>Phase</B><HTML>",
                                      "<HTML><B>Lorentzian damping</B><HTML>",
                                      "<HTML><B>Lorentzian damping R</B><HTML>"
                                    };
            
            columnNames = columnNames1;
        }


        if ( m_trueTwoCompartmentCorrectedRadioButton.isSelected( ) == true )
        {
            String[] columnNames1 = { "<HTML><B>Component name</B></HTML>",
                                      "<HTML><B>Peakareas corr. [mmol/l]</B></HTML>",
                                      "<HTML><B>Chem. shift</B></HTML>",
                                      "<HTML><B>Phase</B><HTML>",
                                      "<HTML><B>Lorentzian damping</B><HTML>",
                                      "<HTML><B>Lorentzian damping R</B><HTML>"
                                    };
            
            columnNames = columnNames1;
        }


        m_tableModel = new DefaultTableModel( m_cellData, columnNames );

        m_ResultsTable.setModel( m_tableModel );
        m_ResultsTable.updateUI( );

    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        m_peakAreaCorrectionMethodButtonGroup = new javax.swing.ButtonGroup();
        m_tissueTypeButtonGroup = new javax.swing.ButtonGroup();
        m_adultOrBabyButtonGroup = new javax.swing.ButtonGroup();
        m_correctForRelaxationTimesButtonGroup = new javax.swing.ButtonGroup();
        m_dicomImageTypeButtonGroup = new javax.swing.ButtonGroup();
        m_sentOptionsButtonGroup = new javax.swing.ButtonGroup();
        m_sentCompressionYesNobuttonGroup = new javax.swing.ButtonGroup();
        m_tabContainerPanel = new javax.swing.JPanel();
        m_absoluteQuantificationTabbedPane = new javax.swing.JTabbedPane();
        m_fittingParametersTab = new javax.swing.JPanel();
        m_fittingParameterScrollPane = new javax.swing.JScrollPane();
        m_ResultsTable = new javax.swing.JTable();
        m_waterRferenceCorrrectionModelPanel = new javax.swing.JPanel();
        m_rawjMRUIAmplitudesRadioButton = new javax.swing.JRadioButton();
        m_twoCompartmentModelFractionOnlyCorrectedOnlyRadioButton = new javax.swing.JRadioButton();
        m_singleWaterReferenceOnlyCorrectedRadioButton = new javax.swing.JRadioButton();
        m_trueTwoCompartmentCorrectedRadioButton = new javax.swing.JRadioButton();
        m_selectResponseNumberPanel = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        m_selectResponseSpinner = new javax.swing.JSpinner();
        jLabel2 = new javax.swing.JLabel();
        m_displayResponseNumberLabel = new javax.swing.JLabel();
        jLabel4 = new javax.swing.JLabel();
        m_exportAsTextButton = new javax.swing.JButton();
        m_metaboliteRelaxationTimesTab = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        m_metabRelaxTimesTable = new javax.swing.JTable();
        m_relaxationDataBaseOptionPanel = new javax.swing.JPanel();
        m_correctionsPanel = new javax.swing.JPanel();
        m_correctForMetabT1T2RelaxationRadioButton = new javax.swing.JRadioButton();
        m_notCorrectForMetabT1T2RelaxationRadioButton = new javax.swing.JRadioButton();
        m_usedTRLabel = new javax.swing.JLabel();
        m_TRForMetabCorrectionLabel = new javax.swing.JLabel();
        jPanel5 = new javax.swing.JPanel();
        m_fieldStrengthDatabaseComboBox = new javax.swing.JComboBox();
        m_metaboliteDatabaseLabel = new javax.swing.JLabel();
        m_loadRelaxationTimesDatabaseButton = new javax.swing.JButton();
        m_saveRelaxationTimeDatabaseButton = new javax.swing.JButton();
        m_waterReferencePropertiesTab = new javax.swing.JPanel();
        m_twoCompartmentPropertiesOverviewPanel = new javax.swing.JPanel();
        m_numberOfSignalsLabel = new javax.swing.JLabel();
        m_numberOfSignalsValueLabel = new javax.swing.JLabel();
        m_amp_1_Label = new javax.swing.JLabel();
        m_amp_2_Label = new javax.swing.JLabel();
        m_TC_1_Label = new javax.swing.JLabel();
        m_TC_2_Label = new javax.swing.JLabel();
        m_amp_1_ValueLabel = new javax.swing.JLabel();
        m_amp_2_ValueLabel = new javax.swing.JLabel();
        m_TC_1_ValueLabel = new javax.swing.JLabel();
        m_TC_2_ValueLabel = new javax.swing.JLabel();
        m_thressholdLabel = new javax.swing.JLabel();
        m_liquorTimeConstantThressholdTextField = new javax.swing.JTextField();
        m_thressholdUnitLabel = new javax.swing.JLabel();
        m_numberOfWaterAcquisitionsTwoComponentsModelLabel = new javax.swing.JLabel();
        m_numberOfWaterAcquisitionsTwoComponentsModelTextField = new javax.swing.JTextField();
        m_numberOfSpectrumAcquisitionsLabel = new javax.swing.JLabel();
        m_numberOfSpectrumMeasurementsTextField2 = new javax.swing.JTextField();
        m_waterReferencePanel = new javax.swing.JPanel();
        m_waterReferenceLabel = new javax.swing.JLabel();
        m_waterReferenceLoadedStatusLabel = new javax.swing.JLabel();
        m_waterAmplitudeLabel = new javax.swing.JLabel();
        m_waterAmplitudeValueLabel = new javax.swing.JLabel();
        m_numberOfWaterAcquiitionsOneReferenceResponseLabel = new javax.swing.JLabel();
        m_numberOfSpectrumAcquisistionsLabel = new javax.swing.JLabel();
        m_numberOfWaterReferenceAcquisitionsSingleReferenceTextField = new javax.swing.JTextField();
        m_numberOfSpectrumMeasurementsTextField = new javax.swing.JTextField();
        m_tissuePropertiesPanel = new javax.swing.JPanel();
        jPanel9 = new javax.swing.JPanel();
        jLabel21 = new javax.swing.JLabel();
        m_tissueWaterContentTextField = new javax.swing.JTextField();
        m_percentLabel = new javax.swing.JLabel();
        jPanel10 = new javax.swing.JPanel();
        m_whiteMatterRadioButton = new javax.swing.JRadioButton();
        m_grayMatterRadioButton = new javax.swing.JRadioButton();
        m_50_50_mixRadioButton = new javax.swing.JRadioButton();
        m_elseTissueRadioButton = new javax.swing.JRadioButton();
        jPanel1 = new javax.swing.JPanel();
        m_adultRadioButton = new javax.swing.JRadioButton();
        m_babyRadioButton = new javax.swing.JRadioButton();
        m_patientAndScannerInfoTab = new javax.swing.JPanel();
        jLabel3 = new javax.swing.JLabel();
        jLabel5 = new javax.swing.JLabel();
        jLabel6 = new javax.swing.JLabel();
        jLabel7 = new javax.swing.JLabel();
        jLabel8 = new javax.swing.JLabel();
        jLabel9 = new javax.swing.JLabel();
        jLabel10 = new javax.swing.JLabel();
        jLabel11 = new javax.swing.JLabel();
        jLabel12 = new javax.swing.JLabel();
        jLabel13 = new javax.swing.JLabel();
        jLabel14 = new javax.swing.JLabel();
        m_patientNameLabel = new javax.swing.JLabel();
        m_patientAgeLabel = new javax.swing.JLabel();
        m_patientIDLabel = new javax.swing.JLabel();
        m_studyDateLabel = new javax.swing.JLabel();
        m_studyTimeLabel = new javax.swing.JLabel();
        m_seriesDescriptionLabel = new javax.swing.JLabel();
        m_studyDescription = new javax.swing.JLabel();
        m_scannerNameLabel = new javax.swing.JLabel();
        m_spectroscopicVoxelSizeLabel = new javax.swing.JLabel();
        m_resonanceFrequencyLabel = new javax.swing.JLabel();
        m_sequenceTypeLabel = new javax.swing.JLabel();
        jLabel15 = new javax.swing.JLabel();
        m_echoTimeLabel = new javax.swing.JLabel();
        jLabel16 = new javax.swing.JLabel();
        m_TRLabel = new javax.swing.JLabel();
        jLabel17 = new javax.swing.JLabel();
        jLabel18 = new javax.swing.JLabel();
        jLabel19 = new javax.swing.JLabel();
        jLabel20 = new javax.swing.JLabel();
        m_dicomTransferSettingsTab = new javax.swing.JPanel();
        jPanel6 = new javax.swing.JPanel();
        m_seriesNumberDICOMSENDLabel = new javax.swing.JLabel();
        m_seriesDesciptionDICOMSENDLabel = new javax.swing.JLabel();
        m_seriesNumberDICOMSENDTextField = new javax.swing.JTextField();
        m_seriesDesciptionDICOMSENDTextField = new javax.swing.JTextField();
        jPanel7 = new javax.swing.JPanel();
        m_sentAsGrayValueDICOMImageRadioButton = new javax.swing.JRadioButton();
        m_sendAsColouredDICOMImageRadioButton = new javax.swing.JRadioButton();
        jPanel2 = new javax.swing.JPanel();
        m_previewBeforeSentRadioButton = new javax.swing.JRadioButton();
        m_sentWithoutPreviewRadioButton = new javax.swing.JRadioButton();
        jPanel4 = new javax.swing.JPanel();
        m_sentCompressedDICOMRadioButton = new javax.swing.JRadioButton();
        m_sentNonCompressedRadioButton = new javax.swing.JRadioButton();
        jPanel8 = new javax.swing.JPanel();
        m_addjMRUIDICOMTAGCheckBox = new javax.swing.JCheckBox();
        m_normalDatabasePanel = new javax.swing.JPanel();
        m_normalMetaboliteDatabaseSelectionPanel = new javax.swing.JPanel();
        m_createNewNormalValueDatabaseButton = new javax.swing.JButton();
        m_normalDatabaseScrollPane = new javax.swing.JScrollPane();
        m_normalDataBaseSelectionTable = new org.spectrim.tdfdfit.quantitativeresults.NormalDataBaseSelectionTable();
        m_caseOptionsPanel = new javax.swing.JPanel();
        m_addCurrentCaseToSelectedDatabaseButton = new javax.swing.JButton();
        m_browseSelectedNormalValueDatabaseButton = new javax.swing.JButton();
        m_exportSelectedNormalValueDatabaseButton = new javax.swing.JButton();
        m_dicomReportingPanel = new javax.swing.JPanel();
        m_reportTypeSentSelectionPanel = new javax.swing.JPanel();
        m_previewGraphicReportButton = new javax.swing.JButton();
        m_previewNumericReportButton = new javax.swing.JButton();
        m_previewBothReportsButton = new javax.swing.JButton();
        m_dicomNodeSelectionAndDisplayPanel = new javax.swing.JPanel();
        m_changeSelectedDICOMNodeButton = new javax.swing.JButton();
        m_selectedDICOMLabel = new javax.swing.JLabel();
        m_selectedDICOMNodeValue = new javax.swing.JLabel();
        m_sendGraphicsReportToSelectedNodeButton = new javax.swing.JButton();
        m_sendNumericReportButton = new javax.swing.JButton();
        m_sendBothReportsButton = new javax.swing.JButton();
        m_donePanel = new javax.swing.JPanel();
        m_doneButton = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);

        m_absoluteQuantificationTabbedPane.setTabPlacement(javax.swing.JTabbedPane.BOTTOM);

        m_ResultsTable.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {null, null, null, null, null, null},
                {null, null, null, null, null, null},
                {null, null, null, null, null, null},
                {null, null, null, null, null, null}
            },
            new String [] {
                "Metabolite name", "Ampitude", "Chemical shift", "Phase", "Lorentzian damping", "Gaussian damping"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.String.class, java.lang.Float.class, java.lang.Float.class, java.lang.Float.class, java.lang.Float.class, java.lang.Float.class
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }
        });
        m_ResultsTable.setSelectionMode(javax.swing.ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
        m_fittingParameterScrollPane.setViewportView(m_ResultsTable);

        m_waterRferenceCorrrectionModelPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Water reference correction option"));

        m_peakAreaCorrectionMethodButtonGroup.add(m_rawjMRUIAmplitudesRadioButton);
        m_rawjMRUIAmplitudesRadioButton.setSelected(true);
        m_rawjMRUIAmplitudesRadioButton.setText("no (raw jMRUI values)");
        m_rawjMRUIAmplitudesRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_rawjMRUIAmplitudesRadioButtonActionPerformed(evt);
            }
        });

        m_peakAreaCorrectionMethodButtonGroup.add(m_twoCompartmentModelFractionOnlyCorrectedOnlyRadioButton);
        m_twoCompartmentModelFractionOnlyCorrectedOnlyRadioButton.setText("Two compartment (fraction only)");
        m_twoCompartmentModelFractionOnlyCorrectedOnlyRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_twoCompartmentModelFractionOnlyCorrectedOnlyRadioButtonActionPerformed(evt);
            }
        });

        m_peakAreaCorrectionMethodButtonGroup.add(m_singleWaterReferenceOnlyCorrectedRadioButton);
        m_singleWaterReferenceOnlyCorrectedRadioButton.setText("Reference water only");
        m_singleWaterReferenceOnlyCorrectedRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_singleWaterReferenceOnlyCorrectedRadioButtonActionPerformed(evt);
            }
        });

        m_peakAreaCorrectionMethodButtonGroup.add(m_trueTwoCompartmentCorrectedRadioButton);
        m_trueTwoCompartmentCorrectedRadioButton.setText("True two compartment");
        m_trueTwoCompartmentCorrectedRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_trueTwoCompartmentCorrectedRadioButtonActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout m_waterRferenceCorrrectionModelPanelLayout = new javax.swing.GroupLayout(m_waterRferenceCorrrectionModelPanel);
        m_waterRferenceCorrrectionModelPanel.setLayout(m_waterRferenceCorrrectionModelPanelLayout);
        m_waterRferenceCorrrectionModelPanelLayout.setHorizontalGroup(
            m_waterRferenceCorrrectionModelPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(m_waterRferenceCorrrectionModelPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(m_rawjMRUIAmplitudesRadioButton)
                .addGap(33, 33, 33)
                .addComponent(m_twoCompartmentModelFractionOnlyCorrectedOnlyRadioButton)
                .addGap(37, 37, 37)
                .addComponent(m_trueTwoCompartmentCorrectedRadioButton)
                .addGap(30, 30, 30)
                .addComponent(m_singleWaterReferenceOnlyCorrectedRadioButton)
                .addContainerGap(520, Short.MAX_VALUE))
        );
        m_waterRferenceCorrrectionModelPanelLayout.setVerticalGroup(
            m_waterRferenceCorrrectionModelPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(m_waterRferenceCorrrectionModelPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                .addComponent(m_rawjMRUIAmplitudesRadioButton)
                .addComponent(m_singleWaterReferenceOnlyCorrectedRadioButton)
                .addComponent(m_twoCompartmentModelFractionOnlyCorrectedOnlyRadioButton)
                .addComponent(m_trueTwoCompartmentCorrectedRadioButton))
        );

        m_selectResponseNumberPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Select response number"));

        jLabel1.setText("Number of responses:");

        m_selectResponseSpinner.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                m_selectResponseSpinnerStateChanged(evt);
            }
        });

        jLabel2.setText("of");

        m_displayResponseNumberLabel.setText("000");

        jLabel4.setText("reponses.");

        m_exportAsTextButton.setText("Export as Text");
        m_exportAsTextButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_exportAsTextButtonActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout m_selectResponseNumberPanelLayout = new javax.swing.GroupLayout(m_selectResponseNumberPanel);
        m_selectResponseNumberPanel.setLayout(m_selectResponseNumberPanelLayout);
        m_selectResponseNumberPanelLayout.setHorizontalGroup(
            m_selectResponseNumberPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(m_selectResponseNumberPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jLabel1)
                .addGap(18, 18, 18)
                .addComponent(m_selectResponseSpinner, javax.swing.GroupLayout.PREFERRED_SIZE, 78, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jLabel2, javax.swing.GroupLayout.PREFERRED_SIZE, 32, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(22, 22, 22)
                .addComponent(m_displayResponseNumberLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 47, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jLabel4)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 850, Short.MAX_VALUE)
                .addComponent(m_exportAsTextButton)
                .addContainerGap())
        );
        m_selectResponseNumberPanelLayout.setVerticalGroup(
            m_selectResponseNumberPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(m_selectResponseNumberPanelLayout.createSequentialGroup()
                .addGroup(m_selectResponseNumberPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_selectResponseSpinner, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel1)
                    .addComponent(m_displayResponseNumberLabel)
                    .addComponent(jLabel4)
                    .addComponent(jLabel2)
                    .addComponent(m_exportAsTextButton))
                .addContainerGap(17, Short.MAX_VALUE))
        );

        javax.swing.GroupLayout m_fittingParametersTabLayout = new javax.swing.GroupLayout(m_fittingParametersTab);
        m_fittingParametersTab.setLayout(m_fittingParametersTabLayout);
        m_fittingParametersTabLayout.setHorizontalGroup(
            m_fittingParametersTabLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(m_fittingParametersTabLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(m_fittingParametersTabLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(m_fittingParameterScrollPane, javax.swing.GroupLayout.DEFAULT_SIZE, 1436, Short.MAX_VALUE)
                    .addComponent(m_waterRferenceCorrrectionModelPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(m_selectResponseNumberPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addContainerGap())
        );
        m_fittingParametersTabLayout.setVerticalGroup(
            m_fittingParametersTabLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(m_fittingParametersTabLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(m_fittingParameterScrollPane, javax.swing.GroupLayout.PREFERRED_SIZE, 282, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(m_selectResponseNumberPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(m_waterRferenceCorrrectionModelPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(281, 281, 281))
        );

        m_absoluteQuantificationTabbedPane.addTab("Fitting parameters", m_fittingParametersTab);

        m_metabRelaxTimesTable.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {null, null, null, null, null, null},
                {null, null, null, null, null, null},
                {null, null, null, null, null, null},
                {null, null, null, null, null, null}
            },
            new String [] {
                "Metabolite name", "Relaxation time T1", "Relaxation time T2", "T1/TR correction factor", "T2 correction factor", "Multiplicity [number of nuclei]"
            }
        ) {
            boolean[] canEdit = new boolean [] {
                true, true, true, false, false, true
            };

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        jScrollPane1.setViewportView(m_metabRelaxTimesTable);

        m_relaxationDataBaseOptionPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Relaxation time correction options"));

        m_correctionsPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("corrections"));

        m_correctForRelaxationTimesButtonGroup.add(m_correctForMetabT1T2RelaxationRadioButton);
        m_correctForMetabT1T2RelaxationRadioButton.setSelected(true);
        m_correctForMetabT1T2RelaxationRadioButton.setText("Correct for T1 and T2 relaxation");
        m_correctForMetabT1T2RelaxationRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_correctForMetabT1T2RelaxationRadioButtonActionPerformed(evt);
            }
        });

        m_correctForRelaxationTimesButtonGroup.add(m_notCorrectForMetabT1T2RelaxationRadioButton);
        m_notCorrectForMetabT1T2RelaxationRadioButton.setText("Do NOT correct for T1 and T2 relaxation");
        m_notCorrectForMetabT1T2RelaxationRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_notCorrectForMetabT1T2RelaxationRadioButtonActionPerformed(evt);
            }
        });

        m_usedTRLabel.setText("Used TR for corrections:");

        m_TRForMetabCorrectionLabel.setText("1500");

        javax.swing.GroupLayout m_correctionsPanelLayout = new javax.swing.GroupLayout(m_correctionsPanel);
        m_correctionsPanel.setLayout(m_correctionsPanelLayout);
        m_correctionsPanelLayout.setHorizontalGroup(
            m_correctionsPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(m_correctionsPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(m_correctForMetabT1T2RelaxationRadioButton)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(m_notCorrectForMetabT1T2RelaxationRadioButton)
                .addGap(32, 32, 32)
                .addComponent(m_usedTRLabel)
                .addGap(37, 37, 37)
                .addComponent(m_TRForMetabCorrectionLabel)
                .addContainerGap(549, Short.MAX_VALUE))
        );
        m_correctionsPanelLayout.setVerticalGroup(
            m_correctionsPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(m_correctionsPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                .addComponent(m_correctForMetabT1T2RelaxationRadioButton)
                .addComponent(m_notCorrectForMetabT1T2RelaxationRadioButton)
                .addComponent(m_usedTRLabel)
                .addComponent(m_TRForMetabCorrectionLabel))
        );

        m_fieldStrengthDatabaseComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "0.5T", "1.0T", "1.5T", "2.0T", "3.0T", "4.0T", "6.3T", "7.0T" }));
        m_fieldStrengthDatabaseComboBox.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                m_fieldStrengthDatabaseComboBoxItemStateChanged(evt);
            }
        });
        m_fieldStrengthDatabaseComboBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_fieldStrengthDatabaseComboBoxActionPerformed(evt);
            }
        });

        m_metaboliteDatabaseLabel.setText("Database @");

        m_loadRelaxationTimesDatabaseButton.setText("Load database ..");

        m_saveRelaxationTimeDatabaseButton.setText("Save to default database ..");
        m_saveRelaxationTimeDatabaseButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_saveRelaxationTimeDatabaseButtonActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanel5Layout = new javax.swing.GroupLayout(jPanel5);
        jPanel5.setLayout(jPanel5Layout);
        jPanel5Layout.setHorizontalGroup(
            jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel5Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(m_metaboliteDatabaseLabel)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(m_fieldStrengthDatabaseComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, 137, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 738, Short.MAX_VALUE)
                .addComponent(m_loadRelaxationTimesDatabaseButton)
                .addGap(53, 53, 53)
                .addComponent(m_saveRelaxationTimeDatabaseButton)
                .addGap(47, 47, 47))
        );
        jPanel5Layout.setVerticalGroup(
            jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel5Layout.createSequentialGroup()
                .addContainerGap(48, Short.MAX_VALUE)
                .addGroup(jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_loadRelaxationTimesDatabaseButton)
                    .addComponent(m_metaboliteDatabaseLabel)
                    .addComponent(m_fieldStrengthDatabaseComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(m_saveRelaxationTimeDatabaseButton)))
        );

        javax.swing.GroupLayout m_relaxationDataBaseOptionPanelLayout = new javax.swing.GroupLayout(m_relaxationDataBaseOptionPanel);
        m_relaxationDataBaseOptionPanel.setLayout(m_relaxationDataBaseOptionPanelLayout);
        m_relaxationDataBaseOptionPanelLayout.setHorizontalGroup(
            m_relaxationDataBaseOptionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, m_relaxationDataBaseOptionPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(m_relaxationDataBaseOptionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(jPanel5, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(m_correctionsPanel, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addContainerGap())
        );
        m_relaxationDataBaseOptionPanelLayout.setVerticalGroup(
            m_relaxationDataBaseOptionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(m_relaxationDataBaseOptionPanelLayout.createSequentialGroup()
                .addComponent(m_correctionsPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jPanel5, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addContainerGap())
        );

        javax.swing.GroupLayout m_metaboliteRelaxationTimesTabLayout = new javax.swing.GroupLayout(m_metaboliteRelaxationTimesTab);
        m_metaboliteRelaxationTimesTab.setLayout(m_metaboliteRelaxationTimesTabLayout);
        m_metaboliteRelaxationTimesTabLayout.setHorizontalGroup(
            m_metaboliteRelaxationTimesTabLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, m_metaboliteRelaxationTimesTabLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(m_metaboliteRelaxationTimesTabLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 1436, Short.MAX_VALUE)
                    .addComponent(m_relaxationDataBaseOptionPanel, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addContainerGap())
        );
        m_metaboliteRelaxationTimesTabLayout.setVerticalGroup(
            m_metaboliteRelaxationTimesTabLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(m_metaboliteRelaxationTimesTabLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jScrollPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 243, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(m_relaxationDataBaseOptionPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addContainerGap())
        );

        m_absoluteQuantificationTabbedPane.addTab("Relaxation Time Database", m_metaboliteRelaxationTimesTab);

        m_twoCompartmentPropertiesOverviewPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Two component model"));

        m_numberOfSignalsLabel.setText("Number of reference signals :");

        m_numberOfSignalsValueLabel.setText("#signals");

        m_amp_1_Label.setText("Amplitude parameter 1 :");

        m_amp_2_Label.setText("Amplitude parameter 2 :");

        m_TC_1_Label.setText("Time constant parameter 1 :");

        m_TC_2_Label.setText("Time constant parameter 2 :");

        m_amp_1_ValueLabel.setText("0.00");

        m_amp_2_ValueLabel.setText("0.00");

        m_TC_1_ValueLabel.setText("-1.0");

        m_TC_2_ValueLabel.setText("-1.0");

        m_thressholdLabel.setText("Time constant thresshold :");

        m_liquorTimeConstantThressholdTextField.setText("500");
        m_liquorTimeConstantThressholdTextField.setToolTipText("From this thresshold on the wate signal is regarded to be CSF");

        m_thressholdUnitLabel.setText("msec");

        m_numberOfWaterAcquisitionsTwoComponentsModelLabel.setText("Number of water acqs. :");

        m_numberOfWaterAcquisitionsTwoComponentsModelTextField.setText("1");
        m_numberOfWaterAcquisitionsTwoComponentsModelTextField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_numberOfWaterAcquisitionsTwoComponentsModelTextFieldActionPerformed(evt);
            }
        });

        m_numberOfSpectrumAcquisitionsLabel.setText("Number of spectrum acqs. :");

        m_numberOfSpectrumMeasurementsTextField2.setText("1");
        m_numberOfSpectrumMeasurementsTextField2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_numberOfSpectrumMeasurementsTextField2ActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout m_twoCompartmentPropertiesOverviewPanelLayout = new javax.swing.GroupLayout(m_twoCompartmentPropertiesOverviewPanel);
        m_twoCompartmentPropertiesOverviewPanel.setLayout(m_twoCompartmentPropertiesOverviewPanelLayout);
        m_twoCompartmentPropertiesOverviewPanelLayout.setHorizontalGroup(
            m_twoCompartmentPropertiesOverviewPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(m_twoCompartmentPropertiesOverviewPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(m_twoCompartmentPropertiesOverviewPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(m_numberOfSignalsLabel)
                    .addComponent(m_amp_1_Label)
                    .addComponent(m_amp_2_Label)
                    .addComponent(m_TC_1_Label)
                    .addComponent(m_TC_2_Label)
                    .addComponent(m_thressholdLabel)
                    .addComponent(m_numberOfWaterAcquisitionsTwoComponentsModelLabel)
                    .addComponent(m_numberOfSpectrumAcquisitionsLabel))
                .addGap(37, 37, 37)
                .addGroup(m_twoCompartmentPropertiesOverviewPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(m_twoCompartmentPropertiesOverviewPanelLayout.createSequentialGroup()
                        .addGroup(m_twoCompartmentPropertiesOverviewPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(m_numberOfWaterAcquisitionsTwoComponentsModelTextField, javax.swing.GroupLayout.DEFAULT_SIZE, 71, Short.MAX_VALUE)
                            .addComponent(m_liquorTimeConstantThressholdTextField, javax.swing.GroupLayout.DEFAULT_SIZE, 71, Short.MAX_VALUE)
                            .addComponent(m_TC_2_ValueLabel)
                            .addComponent(m_TC_1_ValueLabel)
                            .addComponent(m_amp_2_ValueLabel)
                            .addComponent(m_amp_1_ValueLabel)
                            .addComponent(m_numberOfSignalsValueLabel, javax.swing.GroupLayout.DEFAULT_SIZE, 71, Short.MAX_VALUE))
                        .addGap(18, 18, 18)
                        .addComponent(m_thressholdUnitLabel)
                        .addGap(87, 87, 87))
                    .addGroup(m_twoCompartmentPropertiesOverviewPanelLayout.createSequentialGroup()
                        .addComponent(m_numberOfSpectrumMeasurementsTextField2, javax.swing.GroupLayout.DEFAULT_SIZE, 66, Short.MAX_VALUE)
                        .addGap(147, 147, 147))))
        );
        m_twoCompartmentPropertiesOverviewPanelLayout.setVerticalGroup(
            m_twoCompartmentPropertiesOverviewPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(m_twoCompartmentPropertiesOverviewPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(m_twoCompartmentPropertiesOverviewPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_numberOfSignalsLabel)
                    .addComponent(m_numberOfSignalsValueLabel))
                .addGap(18, 18, 18)
                .addGroup(m_twoCompartmentPropertiesOverviewPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_amp_1_Label)
                    .addComponent(m_amp_1_ValueLabel))
                .addGap(18, 18, 18)
                .addGroup(m_twoCompartmentPropertiesOverviewPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_amp_2_Label)
                    .addComponent(m_amp_2_ValueLabel))
                .addGap(18, 18, 18)
                .addGroup(m_twoCompartmentPropertiesOverviewPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_TC_1_Label)
                    .addComponent(m_TC_1_ValueLabel))
                .addGap(18, 18, 18)
                .addGroup(m_twoCompartmentPropertiesOverviewPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_TC_2_Label)
                    .addComponent(m_TC_2_ValueLabel))
                .addGap(18, 18, 18)
                .addGroup(m_twoCompartmentPropertiesOverviewPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_thressholdLabel)
                    .addComponent(m_liquorTimeConstantThressholdTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(m_thressholdUnitLabel))
                .addGap(18, 18, 18)
                .addGroup(m_twoCompartmentPropertiesOverviewPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_numberOfWaterAcquisitionsTwoComponentsModelLabel)
                    .addComponent(m_numberOfWaterAcquisitionsTwoComponentsModelTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(18, 18, 18)
                .addGroup(m_twoCompartmentPropertiesOverviewPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_numberOfSpectrumAcquisitionsLabel)
                    .addComponent(m_numberOfSpectrumMeasurementsTextField2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(90, Short.MAX_VALUE))
        );

        m_waterReferencePanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Water reference signal"));

        m_waterReferenceLabel.setText("Water reference :");

        m_waterReferenceLoadedStatusLabel.setText("non loaded");

        m_waterAmplitudeLabel.setText("Water amplitude:");

        m_waterAmplitudeValueLabel.setText("0.000E00");

        m_numberOfWaterAcquiitionsOneReferenceResponseLabel.setText("Number of water acqs. :");

        m_numberOfSpectrumAcquisistionsLabel.setText("Number of spectrum acqs. :");

        m_numberOfWaterReferenceAcquisitionsSingleReferenceTextField.setText("1");
        m_numberOfWaterReferenceAcquisitionsSingleReferenceTextField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_numberOfWaterReferenceAcquisitionsSingleReferenceTextFieldActionPerformed(evt);
            }
        });

        m_numberOfSpectrumMeasurementsTextField.setText("1");
        m_numberOfSpectrumMeasurementsTextField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_numberOfSpectrumMeasurementsTextFieldActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout m_waterReferencePanelLayout = new javax.swing.GroupLayout(m_waterReferencePanel);
        m_waterReferencePanel.setLayout(m_waterReferencePanelLayout);
        m_waterReferencePanelLayout.setHorizontalGroup(
            m_waterReferencePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(m_waterReferencePanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(m_waterReferencePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(m_waterReferencePanelLayout.createSequentialGroup()
                        .addGroup(m_waterReferencePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(m_waterAmplitudeLabel)
                            .addComponent(m_waterReferenceLabel))
                        .addGap(105, 105, 105)
                        .addGroup(m_waterReferencePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(m_waterReferenceLoadedStatusLabel)
                            .addComponent(m_waterAmplitudeValueLabel)))
                    .addGroup(m_waterReferencePanelLayout.createSequentialGroup()
                        .addGroup(m_waterReferencePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(m_numberOfWaterAcquiitionsOneReferenceResponseLabel)
                            .addComponent(m_numberOfSpectrumAcquisistionsLabel))
                        .addGap(22, 22, 22)
                        .addGroup(m_waterReferencePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                            .addComponent(m_numberOfWaterReferenceAcquisitionsSingleReferenceTextField)
                            .addComponent(m_numberOfSpectrumMeasurementsTextField, javax.swing.GroupLayout.DEFAULT_SIZE, 80, Short.MAX_VALUE))))
                .addContainerGap(620, Short.MAX_VALUE))
        );
        m_waterReferencePanelLayout.setVerticalGroup(
            m_waterReferencePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(m_waterReferencePanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(m_waterReferencePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_waterReferenceLabel)
                    .addComponent(m_waterReferenceLoadedStatusLabel))
                .addGap(18, 18, 18)
                .addGroup(m_waterReferencePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(m_waterAmplitudeValueLabel)
                    .addComponent(m_waterAmplitudeLabel))
                .addGap(18, 18, 18)
                .addGroup(m_waterReferencePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_numberOfWaterReferenceAcquisitionsSingleReferenceTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(m_numberOfWaterAcquiitionsOneReferenceResponseLabel))
                .addGap(18, 18, 18)
                .addGroup(m_waterReferencePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_numberOfSpectrumAcquisistionsLabel)
                    .addComponent(m_numberOfSpectrumMeasurementsTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        m_tissuePropertiesPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Tissue properties"));

        jLabel21.setText("Tissue water content:");

        m_tissueWaterContentTextField.setText("71.3");
        m_tissueWaterContentTextField.setEnabled(false);

        m_percentLabel.setText("[%]");

        javax.swing.GroupLayout jPanel9Layout = new javax.swing.GroupLayout(jPanel9);
        jPanel9.setLayout(jPanel9Layout);
        jPanel9Layout.setHorizontalGroup(
            jPanel9Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel9Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jLabel21)
                .addGap(66, 66, 66)
                .addComponent(m_tissueWaterContentTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 77, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(m_percentLabel)
                .addContainerGap(596, Short.MAX_VALUE))
        );
        jPanel9Layout.setVerticalGroup(
            jPanel9Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel9Layout.createSequentialGroup()
                .addContainerGap(12, Short.MAX_VALUE)
                .addGroup(jPanel9Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel21)
                    .addComponent(m_percentLabel)
                    .addComponent(m_tissueWaterContentTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );

        m_tissueTypeButtonGroup.add(m_whiteMatterRadioButton);
        m_whiteMatterRadioButton.setText("White matter");
        m_whiteMatterRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_whiteMatterRadioButtonActionPerformed(evt);
            }
        });

        m_tissueTypeButtonGroup.add(m_grayMatterRadioButton);
        m_grayMatterRadioButton.setText("Gray matter");
        m_grayMatterRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_grayMatterRadioButtonActionPerformed(evt);
            }
        });

        m_tissueTypeButtonGroup.add(m_50_50_mixRadioButton);
        m_50_50_mixRadioButton.setSelected(true);
        m_50_50_mixRadioButton.setText("50-50 mix");
        m_50_50_mixRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_50_50_mixRadioButtonActionPerformed(evt);
            }
        });

        m_tissueTypeButtonGroup.add(m_elseTissueRadioButton);
        m_elseTissueRadioButton.setText("else");
        m_elseTissueRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_elseTissueRadioButtonActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanel10Layout = new javax.swing.GroupLayout(jPanel10);
        jPanel10.setLayout(jPanel10Layout);
        jPanel10Layout.setHorizontalGroup(
            jPanel10Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel10Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(m_whiteMatterRadioButton)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(m_grayMatterRadioButton)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(m_50_50_mixRadioButton)
                .addGap(18, 18, 18)
                .addComponent(m_elseTissueRadioButton)
                .addContainerGap(525, Short.MAX_VALUE))
        );
        jPanel10Layout.setVerticalGroup(
            jPanel10Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel10Layout.createSequentialGroup()
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGroup(jPanel10Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_elseTissueRadioButton)
                    .addComponent(m_whiteMatterRadioButton)
                    .addComponent(m_grayMatterRadioButton)
                    .addComponent(m_50_50_mixRadioButton))
                .addContainerGap())
        );

        jPanel1.setBorder(javax.swing.BorderFactory.createEtchedBorder());

        m_adultOrBabyButtonGroup.add(m_adultRadioButton);
        m_adultRadioButton.setSelected(true);
        m_adultRadioButton.setText("Adult");
        m_adultRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_adultRadioButtonActionPerformed(evt);
            }
        });

        m_adultOrBabyButtonGroup.add(m_babyRadioButton);
        m_babyRadioButton.setText("Baby");
        m_babyRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_babyRadioButtonActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(m_adultRadioButton)
                .addGap(101, 101, 101)
                .addComponent(m_babyRadioButton)
                .addContainerGap(698, Short.MAX_VALUE))
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_adultRadioButton)
                    .addComponent(m_babyRadioButton))
                .addContainerGap(50, Short.MAX_VALUE))
        );

        javax.swing.GroupLayout m_tissuePropertiesPanelLayout = new javax.swing.GroupLayout(m_tissuePropertiesPanel);
        m_tissuePropertiesPanel.setLayout(m_tissuePropertiesPanelLayout);
        m_tissuePropertiesPanelLayout.setHorizontalGroup(
            m_tissuePropertiesPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jPanel10, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .addComponent(jPanel9, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .addComponent(jPanel1, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );
        m_tissuePropertiesPanelLayout.setVerticalGroup(
            m_tissuePropertiesPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(m_tissuePropertiesPanelLayout.createSequentialGroup()
                .addComponent(jPanel9, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jPanel10, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        javax.swing.GroupLayout m_waterReferencePropertiesTabLayout = new javax.swing.GroupLayout(m_waterReferencePropertiesTab);
        m_waterReferencePropertiesTab.setLayout(m_waterReferencePropertiesTabLayout);
        m_waterReferencePropertiesTabLayout.setHorizontalGroup(
            m_waterReferencePropertiesTabLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(m_waterReferencePropertiesTabLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(m_twoCompartmentPropertiesOverviewPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(m_waterReferencePropertiesTabLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(m_tissuePropertiesPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(m_waterReferencePanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addContainerGap())
        );
        m_waterReferencePropertiesTabLayout.setVerticalGroup(
            m_waterReferencePropertiesTabLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, m_waterReferencePropertiesTabLayout.createSequentialGroup()
                .addGroup(m_waterReferencePropertiesTabLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addGroup(javax.swing.GroupLayout.Alignment.LEADING, m_waterReferencePropertiesTabLayout.createSequentialGroup()
                        .addContainerGap()
                        .addComponent(m_twoCompartmentPropertiesOverviewPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                    .addGroup(javax.swing.GroupLayout.Alignment.LEADING, m_waterReferencePropertiesTabLayout.createSequentialGroup()
                        .addGap(18, 18, 18)
                        .addComponent(m_waterReferencePanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(m_tissuePropertiesPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)))
                .addContainerGap())
        );

        m_absoluteQuantificationTabbedPane.addTab("Water reference information", m_waterReferencePropertiesTab);

        jLabel3.setText("Patient name: ");

        jLabel5.setText("Patient age:");

        jLabel6.setText("Study date:");

        jLabel7.setText("Patient ID:");

        jLabel8.setText("Study time:");

        jLabel9.setText("Series description:");

        jLabel10.setText("Study description:");

        jLabel11.setText("Scanner type:");

        jLabel12.setText("Spectroscopic voxel size:");

        jLabel13.setText("Resonance frequency:");

        jLabel14.setText("Sequence type:");

        m_patientNameLabel.setText("Unknown");

        m_patientAgeLabel.setText("Unknown");

        m_patientIDLabel.setText("Unknown");

        m_studyDateLabel.setText("Unknown");

        m_studyTimeLabel.setText("Unknown");

        m_seriesDescriptionLabel.setText("Unknown");

        m_studyDescription.setText("Unknown");

        m_scannerNameLabel.setText("Unknown");

        m_spectroscopicVoxelSizeLabel.setText("Unknown");

        m_resonanceFrequencyLabel.setText("Unknown");

        m_sequenceTypeLabel.setText("Unknown");

        jLabel15.setText("Echo time TE:");

        m_echoTimeLabel.setText("Unknown");

        jLabel16.setText("Repetition time TR:");

        m_TRLabel.setText("Unknown");

        jLabel17.setText("Hz");

        jLabel18.setText("mm");

        jLabel19.setText("msec");

        jLabel20.setText("msec");

        javax.swing.GroupLayout m_patientAndScannerInfoTabLayout = new javax.swing.GroupLayout(m_patientAndScannerInfoTab);
        m_patientAndScannerInfoTab.setLayout(m_patientAndScannerInfoTabLayout);
        m_patientAndScannerInfoTabLayout.setHorizontalGroup(
            m_patientAndScannerInfoTabLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(m_patientAndScannerInfoTabLayout.createSequentialGroup()
                .addGap(28, 28, 28)
                .addGroup(m_patientAndScannerInfoTabLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel3)
                    .addComponent(jLabel5)
                    .addComponent(jLabel7)
                    .addComponent(jLabel6)
                    .addComponent(jLabel8)
                    .addComponent(jLabel9)
                    .addComponent(jLabel10)
                    .addComponent(jLabel11)
                    .addComponent(jLabel12)
                    .addComponent(jLabel13)
                    .addComponent(jLabel14)
                    .addComponent(jLabel15)
                    .addComponent(jLabel16))
                .addGap(62, 62, 62)
                .addGroup(m_patientAndScannerInfoTabLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(m_patientAndScannerInfoTabLayout.createSequentialGroup()
                        .addGroup(m_patientAndScannerInfoTabLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(m_resonanceFrequencyLabel)
                            .addComponent(m_spectroscopicVoxelSizeLabel)
                            .addComponent(m_echoTimeLabel)
                            .addComponent(m_TRLabel))
                        .addGap(37, 37, 37)
                        .addGroup(m_patientAndScannerInfoTabLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jLabel20)
                            .addComponent(jLabel19)
                            .addComponent(jLabel18)
                            .addComponent(jLabel17)))
                    .addComponent(m_sequenceTypeLabel)
                    .addComponent(m_scannerNameLabel)
                    .addComponent(m_studyDescription)
                    .addComponent(m_seriesDescriptionLabel)
                    .addComponent(m_studyTimeLabel)
                    .addComponent(m_studyDateLabel)
                    .addComponent(m_patientIDLabel)
                    .addComponent(m_patientAgeLabel)
                    .addComponent(m_patientNameLabel))
                .addContainerGap(1057, Short.MAX_VALUE))
        );
        m_patientAndScannerInfoTabLayout.setVerticalGroup(
            m_patientAndScannerInfoTabLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(m_patientAndScannerInfoTabLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(m_patientAndScannerInfoTabLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel3)
                    .addComponent(m_patientNameLabel))
                .addGap(14, 14, 14)
                .addGroup(m_patientAndScannerInfoTabLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel5)
                    .addComponent(m_patientAgeLabel))
                .addGap(14, 14, 14)
                .addGroup(m_patientAndScannerInfoTabLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel7)
                    .addComponent(m_patientIDLabel))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(m_patientAndScannerInfoTabLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel6)
                    .addComponent(m_studyDateLabel))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(m_patientAndScannerInfoTabLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel8)
                    .addComponent(m_studyTimeLabel))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(m_patientAndScannerInfoTabLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel9)
                    .addComponent(m_seriesDescriptionLabel))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(m_patientAndScannerInfoTabLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel10)
                    .addComponent(m_studyDescription))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(m_patientAndScannerInfoTabLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel11)
                    .addComponent(m_scannerNameLabel))
                .addGap(12, 12, 12)
                .addGroup(m_patientAndScannerInfoTabLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel12)
                    .addComponent(m_spectroscopicVoxelSizeLabel)
                    .addComponent(jLabel18))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(m_patientAndScannerInfoTabLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel13)
                    .addComponent(m_resonanceFrequencyLabel)
                    .addComponent(jLabel17))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(m_patientAndScannerInfoTabLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_sequenceTypeLabel)
                    .addComponent(jLabel14))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(m_patientAndScannerInfoTabLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel15)
                    .addComponent(m_echoTimeLabel)
                    .addComponent(jLabel19))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(m_patientAndScannerInfoTabLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel16)
                    .addComponent(m_TRLabel)
                    .addComponent(jLabel20))
                .addContainerGap(59, Short.MAX_VALUE))
        );

        m_absoluteQuantificationTabbedPane.addTab("Patient & Scanner Information", m_patientAndScannerInfoTab);

        jPanel6.setBorder(javax.swing.BorderFactory.createTitledBorder("DICOM transfer settings"));

        m_seriesNumberDICOMSENDLabel.setText("Series number:");

        m_seriesDesciptionDICOMSENDLabel.setText("Series description:");

        m_seriesNumberDICOMSENDTextField.setText("1000");
        m_seriesNumberDICOMSENDTextField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_seriesNumberDICOMSENDTextFieldActionPerformed(evt);
            }
        });

        m_seriesDesciptionDICOMSENDTextField.setText("jMRUI-spectroscopy Report");

        javax.swing.GroupLayout jPanel6Layout = new javax.swing.GroupLayout(jPanel6);
        jPanel6.setLayout(jPanel6Layout);
        jPanel6Layout.setHorizontalGroup(
            jPanel6Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel6Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel6Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(m_seriesDesciptionDICOMSENDLabel)
                    .addComponent(m_seriesNumberDICOMSENDLabel))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel6Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(m_seriesNumberDICOMSENDTextField)
                    .addComponent(m_seriesDesciptionDICOMSENDTextField, javax.swing.GroupLayout.DEFAULT_SIZE, 526, Short.MAX_VALUE))
                .addContainerGap(759, Short.MAX_VALUE))
        );
        jPanel6Layout.setVerticalGroup(
            jPanel6Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel6Layout.createSequentialGroup()
                .addGap(23, 23, 23)
                .addGroup(jPanel6Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_seriesNumberDICOMSENDLabel)
                    .addComponent(m_seriesNumberDICOMSENDTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(35, 35, 35)
                .addGroup(jPanel6Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_seriesDesciptionDICOMSENDLabel)
                    .addComponent(m_seriesDesciptionDICOMSENDTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        m_dicomImageTypeButtonGroup.add(m_sentAsGrayValueDICOMImageRadioButton);
        m_sentAsGrayValueDICOMImageRadioButton.setText("DICOM gray value image");
        m_sentAsGrayValueDICOMImageRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_sentAsGrayValueDICOMImageRadioButtonActionPerformed(evt);
            }
        });

        m_dicomImageTypeButtonGroup.add(m_sendAsColouredDICOMImageRadioButton);
        m_sendAsColouredDICOMImageRadioButton.setSelected(true);
        m_sendAsColouredDICOMImageRadioButton.setText("DICOM coloured");

        javax.swing.GroupLayout jPanel7Layout = new javax.swing.GroupLayout(jPanel7);
        jPanel7.setLayout(jPanel7Layout);
        jPanel7Layout.setHorizontalGroup(
            jPanel7Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel7Layout.createSequentialGroup()
                .addGap(117, 117, 117)
                .addComponent(m_sentAsGrayValueDICOMImageRadioButton)
                .addGap(271, 271, 271)
                .addComponent(m_sendAsColouredDICOMImageRadioButton)
                .addContainerGap(726, Short.MAX_VALUE))
        );
        jPanel7Layout.setVerticalGroup(
            jPanel7Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel7Layout.createSequentialGroup()
                .addGroup(jPanel7Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_sentAsGrayValueDICOMImageRadioButton)
                    .addComponent(m_sendAsColouredDICOMImageRadioButton))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        m_sentOptionsButtonGroup.add(m_previewBeforeSentRadioButton);
        m_previewBeforeSentRadioButton.setSelected(true);
        m_previewBeforeSentRadioButton.setText("Preview before sent");

        m_sentOptionsButtonGroup.add(m_sentWithoutPreviewRadioButton);
        m_sentWithoutPreviewRadioButton.setText("Sent without preview");

        javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addGap(107, 107, 107)
                .addComponent(m_previewBeforeSentRadioButton)
                .addGap(304, 304, 304)
                .addComponent(m_sentWithoutPreviewRadioButton)
                .addContainerGap(683, Short.MAX_VALUE))
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_previewBeforeSentRadioButton)
                    .addComponent(m_sentWithoutPreviewRadioButton)))
        );

        m_sentCompressionYesNobuttonGroup.add(m_sentCompressedDICOMRadioButton);
        m_sentCompressedDICOMRadioButton.setSelected(true);
        m_sentCompressedDICOMRadioButton.setText(" Compressed DICOM image");

        m_sentCompressionYesNobuttonGroup.add(m_sentNonCompressedRadioButton);
        m_sentNonCompressedRadioButton.setText("Non compressed DICOM image");
        m_sentNonCompressedRadioButton.setEnabled(false);

        javax.swing.GroupLayout jPanel4Layout = new javax.swing.GroupLayout(jPanel4);
        jPanel4.setLayout(jPanel4Layout);
        jPanel4Layout.setHorizontalGroup(
            jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel4Layout.createSequentialGroup()
                .addGap(107, 107, 107)
                .addComponent(m_sentCompressedDICOMRadioButton)
                .addGap(248, 248, 248)
                .addComponent(m_sentNonCompressedRadioButton)
                .addContainerGap(626, Short.MAX_VALUE))
        );
        jPanel4Layout.setVerticalGroup(
            jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel4Layout.createSequentialGroup()
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGroup(jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_sentCompressedDICOMRadioButton)
                    .addComponent(m_sentNonCompressedRadioButton))
                .addGap(31, 31, 31))
        );

        m_addjMRUIDICOMTAGCheckBox.setText("Add jMRUI DICOM-tag to DICOM report files");
        m_addjMRUIDICOMTAGCheckBox.setEnabled(false);

        javax.swing.GroupLayout jPanel8Layout = new javax.swing.GroupLayout(jPanel8);
        jPanel8.setLayout(jPanel8Layout);
        jPanel8Layout.setHorizontalGroup(
            jPanel8Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel8Layout.createSequentialGroup()
                .addGap(105, 105, 105)
                .addComponent(m_addjMRUIDICOMTAGCheckBox)
                .addContainerGap(1007, Short.MAX_VALUE))
        );
        jPanel8Layout.setVerticalGroup(
            jPanel8Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel8Layout.createSequentialGroup()
                .addContainerGap(20, Short.MAX_VALUE)
                .addComponent(m_addjMRUIDICOMTAGCheckBox))
        );

        javax.swing.GroupLayout m_dicomTransferSettingsTabLayout = new javax.swing.GroupLayout(m_dicomTransferSettingsTab);
        m_dicomTransferSettingsTab.setLayout(m_dicomTransferSettingsTabLayout);
        m_dicomTransferSettingsTabLayout.setHorizontalGroup(
            m_dicomTransferSettingsTabLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(m_dicomTransferSettingsTabLayout.createSequentialGroup()
                .addGroup(m_dicomTransferSettingsTabLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jPanel7, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(jPanel6, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addGroup(m_dicomTransferSettingsTabLayout.createSequentialGroup()
                        .addContainerGap()
                        .addGroup(m_dicomTransferSettingsTabLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jPanel4, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(jPanel2, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(jPanel8, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))))
                .addContainerGap())
        );
        m_dicomTransferSettingsTabLayout.setVerticalGroup(
            m_dicomTransferSettingsTabLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(m_dicomTransferSettingsTabLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jPanel6, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(18, 18, 18)
                .addComponent(jPanel7, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jPanel2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jPanel4, javax.swing.GroupLayout.PREFERRED_SIZE, 46, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jPanel8, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(90, Short.MAX_VALUE))
        );

        m_absoluteQuantificationTabbedPane.addTab("Dicom Transfer Settings", m_dicomTransferSettingsTab);

        m_normalMetaboliteDatabaseSelectionPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Edit & Select database"));

        m_createNewNormalValueDatabaseButton.setText("Create normal database ..");
        m_createNewNormalValueDatabaseButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_createNewNormalValueDatabaseButtonActionPerformed(evt);
            }
        });

        m_normalDatabaseScrollPane.setViewportView(m_normalDataBaseSelectionTable);

        javax.swing.GroupLayout m_normalMetaboliteDatabaseSelectionPanelLayout = new javax.swing.GroupLayout(m_normalMetaboliteDatabaseSelectionPanel);
        m_normalMetaboliteDatabaseSelectionPanel.setLayout(m_normalMetaboliteDatabaseSelectionPanelLayout);
        m_normalMetaboliteDatabaseSelectionPanelLayout.setHorizontalGroup(
            m_normalMetaboliteDatabaseSelectionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(m_normalMetaboliteDatabaseSelectionPanelLayout.createSequentialGroup()
                .addComponent(m_createNewNormalValueDatabaseButton)
                .addContainerGap(1246, Short.MAX_VALUE))
            .addComponent(m_normalDatabaseScrollPane, javax.swing.GroupLayout.DEFAULT_SIZE, 1436, Short.MAX_VALUE)
        );
        m_normalMetaboliteDatabaseSelectionPanelLayout.setVerticalGroup(
            m_normalMetaboliteDatabaseSelectionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(m_normalMetaboliteDatabaseSelectionPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(m_createNewNormalValueDatabaseButton)
                .addGap(28, 28, 28)
                .addComponent(m_normalDatabaseScrollPane, javax.swing.GroupLayout.PREFERRED_SIZE, 159, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        m_addCurrentCaseToSelectedDatabaseButton.setText("Add current case to selected database");
        m_addCurrentCaseToSelectedDatabaseButton.setEnabled(false);
        m_addCurrentCaseToSelectedDatabaseButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_addCurrentCaseToSelectedDatabaseButtonActionPerformed(evt);
            }
        });

        m_browseSelectedNormalValueDatabaseButton.setText("Browse selected database");
        m_browseSelectedNormalValueDatabaseButton.setEnabled(false);
        m_browseSelectedNormalValueDatabaseButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_browseSelectedNormalValueDatabaseButtonActionPerformed(evt);
            }
        });

        m_exportSelectedNormalValueDatabaseButton.setText("Export selected database");
        m_exportSelectedNormalValueDatabaseButton.setEnabled(false);
        m_exportSelectedNormalValueDatabaseButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_exportSelectedNormalValueDatabaseButtonActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout m_caseOptionsPanelLayout = new javax.swing.GroupLayout(m_caseOptionsPanel);
        m_caseOptionsPanel.setLayout(m_caseOptionsPanelLayout);
        m_caseOptionsPanelLayout.setHorizontalGroup(
            m_caseOptionsPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(m_caseOptionsPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(m_caseOptionsPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(m_exportSelectedNormalValueDatabaseButton, javax.swing.GroupLayout.DEFAULT_SIZE, 279, Short.MAX_VALUE)
                    .addComponent(m_addCurrentCaseToSelectedDatabaseButton, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(m_browseSelectedNormalValueDatabaseButton, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 279, Short.MAX_VALUE))
                .addGap(1145, 1145, 1145))
        );
        m_caseOptionsPanelLayout.setVerticalGroup(
            m_caseOptionsPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(m_caseOptionsPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(m_addCurrentCaseToSelectedDatabaseButton)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(m_browseSelectedNormalValueDatabaseButton)
                .addGap(12, 12, 12)
                .addComponent(m_exportSelectedNormalValueDatabaseButton)
                .addContainerGap(23, Short.MAX_VALUE))
        );

        javax.swing.GroupLayout m_normalDatabasePanelLayout = new javax.swing.GroupLayout(m_normalDatabasePanel);
        m_normalDatabasePanel.setLayout(m_normalDatabasePanelLayout);
        m_normalDatabasePanelLayout.setHorizontalGroup(
            m_normalDatabasePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(m_normalDatabasePanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(m_normalDatabasePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(m_normalDatabasePanelLayout.createSequentialGroup()
                        .addComponent(m_caseOptionsPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addGap(12, 12, 12))
                    .addComponent(m_normalMetaboliteDatabaseSelectionPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)))
        );
        m_normalDatabasePanelLayout.setVerticalGroup(
            m_normalDatabasePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(m_normalDatabasePanelLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(m_normalMetaboliteDatabaseSelectionPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(m_caseOptionsPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addContainerGap())
        );

        m_absoluteQuantificationTabbedPane.addTab("Normal databases", m_normalDatabasePanel);

        m_dicomReportingPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("DICOM reporting"));

        m_previewGraphicReportButton.setText("Preview Graphic Report ");
        m_previewGraphicReportButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_previewGraphicReportButtonActionPerformed(evt);
            }
        });

        m_previewNumericReportButton.setText("Preview Numeric Report");
        m_previewNumericReportButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_previewNumericReportButtonActionPerformed(evt);
            }
        });

        m_previewBothReportsButton.setText("Preview Both Reports");
        m_previewBothReportsButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_previewBothReportsButtonActionPerformed(evt);
            }
        });

        m_changeSelectedDICOMNodeButton.setText("Change DICOM node to sent to ..");
        m_changeSelectedDICOMNodeButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_changeSelectedDICOMNodeButtonActionPerformed(evt);
            }
        });

        m_selectedDICOMLabel.setText("Selected DICOM node:");

        m_selectedDICOMNodeValue.setText("none");

        javax.swing.GroupLayout m_dicomNodeSelectionAndDisplayPanelLayout = new javax.swing.GroupLayout(m_dicomNodeSelectionAndDisplayPanel);
        m_dicomNodeSelectionAndDisplayPanel.setLayout(m_dicomNodeSelectionAndDisplayPanelLayout);
        m_dicomNodeSelectionAndDisplayPanelLayout.setHorizontalGroup(
            m_dicomNodeSelectionAndDisplayPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(m_dicomNodeSelectionAndDisplayPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(m_selectedDICOMLabel)
                .addGap(49, 49, 49)
                .addComponent(m_selectedDICOMNodeValue)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addComponent(m_changeSelectedDICOMNodeButton))
        );
        m_dicomNodeSelectionAndDisplayPanelLayout.setVerticalGroup(
            m_dicomNodeSelectionAndDisplayPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, m_dicomNodeSelectionAndDisplayPanelLayout.createSequentialGroup()
                .addContainerGap(20, Short.MAX_VALUE)
                .addGroup(m_dicomNodeSelectionAndDisplayPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_selectedDICOMLabel)
                    .addComponent(m_selectedDICOMNodeValue)
                    .addComponent(m_changeSelectedDICOMNodeButton)))
        );

        m_sendGraphicsReportToSelectedNodeButton.setText("Send Graphic Report");
        m_sendGraphicsReportToSelectedNodeButton.setEnabled(false);
        m_sendGraphicsReportToSelectedNodeButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_sendGraphicsReportToSelectedNodeButtonActionPerformed(evt);
            }
        });

        m_sendNumericReportButton.setText("Send Numeric Report");
        m_sendNumericReportButton.setEnabled(false);
        m_sendNumericReportButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_sendNumericReportButtonActionPerformed(evt);
            }
        });

        m_sendBothReportsButton.setText("Send Both Reports");
        m_sendBothReportsButton.setEnabled(false);
        m_sendBothReportsButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_sendBothReportsButtonActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout m_reportTypeSentSelectionPanelLayout = new javax.swing.GroupLayout(m_reportTypeSentSelectionPanel);
        m_reportTypeSentSelectionPanel.setLayout(m_reportTypeSentSelectionPanelLayout);
        m_reportTypeSentSelectionPanelLayout.setHorizontalGroup(
            m_reportTypeSentSelectionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(m_reportTypeSentSelectionPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(m_reportTypeSentSelectionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                    .addComponent(m_dicomNodeSelectionAndDisplayPanel, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addGroup(javax.swing.GroupLayout.Alignment.LEADING, m_reportTypeSentSelectionPanelLayout.createSequentialGroup()
                        .addComponent(m_previewGraphicReportButton)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(m_sendGraphicsReportToSelectedNodeButton, javax.swing.GroupLayout.PREFERRED_SIZE, 211, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(90, 90, 90)
                        .addComponent(m_previewNumericReportButton, javax.swing.GroupLayout.PREFERRED_SIZE, 214, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(18, 18, 18)
                        .addComponent(m_sendNumericReportButton, javax.swing.GroupLayout.PREFERRED_SIZE, 200, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(66, 66, 66)
                        .addComponent(m_previewBothReportsButton, javax.swing.GroupLayout.PREFERRED_SIZE, 204, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(18, 18, 18)
                        .addComponent(m_sendBothReportsButton, javax.swing.GroupLayout.PREFERRED_SIZE, 204, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addGap(12, 12, 12))
        );
        m_reportTypeSentSelectionPanelLayout.setVerticalGroup(
            m_reportTypeSentSelectionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(m_reportTypeSentSelectionPanelLayout.createSequentialGroup()
                .addGroup(m_reportTypeSentSelectionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_previewGraphicReportButton)
                    .addComponent(m_sendGraphicsReportToSelectedNodeButton)
                    .addComponent(m_previewNumericReportButton)
                    .addComponent(m_sendNumericReportButton)
                    .addComponent(m_previewBothReportsButton)
                    .addComponent(m_sendBothReportsButton))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(m_dicomNodeSelectionAndDisplayPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(24, Short.MAX_VALUE))
        );

        javax.swing.GroupLayout m_dicomReportingPanelLayout = new javax.swing.GroupLayout(m_dicomReportingPanel);
        m_dicomReportingPanel.setLayout(m_dicomReportingPanelLayout);
        m_dicomReportingPanelLayout.setHorizontalGroup(
            m_dicomReportingPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(m_dicomReportingPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(m_reportTypeSentSelectionPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addContainerGap())
        );
        m_dicomReportingPanelLayout.setVerticalGroup(
            m_dicomReportingPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, m_dicomReportingPanelLayout.createSequentialGroup()
                .addComponent(m_reportTypeSentSelectionPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );

        javax.swing.GroupLayout m_tabContainerPanelLayout = new javax.swing.GroupLayout(m_tabContainerPanel);
        m_tabContainerPanel.setLayout(m_tabContainerPanelLayout);
        m_tabContainerPanelLayout.setHorizontalGroup(
            m_tabContainerPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, m_tabContainerPanelLayout.createSequentialGroup()
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGroup(m_tabContainerPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(m_dicomReportingPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(m_absoluteQuantificationTabbedPane, javax.swing.GroupLayout.PREFERRED_SIZE, 0, Short.MAX_VALUE)))
        );
        m_tabContainerPanelLayout.setVerticalGroup(
            m_tabContainerPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(m_tabContainerPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(m_absoluteQuantificationTabbedPane, javax.swing.GroupLayout.PREFERRED_SIZE, 491, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(m_dicomReportingPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(16, Short.MAX_VALUE))
        );

        m_doneButton.setText("Done");
        m_doneButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_doneButtonActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout m_donePanelLayout = new javax.swing.GroupLayout(m_donePanel);
        m_donePanel.setLayout(m_donePanelLayout);
        m_donePanelLayout.setHorizontalGroup(
            m_donePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, m_donePanelLayout.createSequentialGroup()
                .addContainerGap(702, Short.MAX_VALUE)
                .addComponent(m_doneButton, javax.swing.GroupLayout.PREFERRED_SIZE, 92, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(697, 697, 697))
        );
        m_donePanelLayout.setVerticalGroup(
            m_donePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(m_donePanelLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(m_doneButton)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                    .addGroup(layout.createSequentialGroup()
                        .addGap(12, 12, 12)
                        .addComponent(m_donePanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                    .addComponent(m_tabContainerPanel, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addComponent(m_tabContainerPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(m_donePanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void m_previewNumericReportButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_previewNumericReportButtonActionPerformed

        double[] normOutputValues = new double[m_concentrations[0].length];
        double[] normOutputErrorValues = new double[m_concentrations[0].length];

        setNormalValuesFromSelectedDatabase( );

        // set the default values to -1
        for ( int i = 0; i < m_concentrations.length; i++ )
        {
            normOutputValues[i] = -1.0;
            normOutputErrorValues[i] = -1.0;
        }
        
        if ( m_normalDataBaseMetaboliteNamesOrder != null )
        {
            for ( int i = 0; i < m_normalDataBaseMetaboliteNamesOrder.size( ); i++ )
            {
                for ( int j = 0; j < m_metaboliteNames.size( ) ; j++ )
                {
                    // look for matching metabolite names:
                    // it is possible that our reference database does not contain the
                    // fitted metabolite!
                    if ( m_normalDataBaseMetaboliteNamesOrder.get(i).compareTo( m_metaboliteNames.get(j) ) == 0 )
                    {
                        normOutputValues[j]      = m_normalConcentrationsFromDB[i];
                        normOutputErrorValues[i] = m_normalErrorConcentrationsFromDB[i];
                    }
                }
            }

            Dimension d = new Dimension( 1024, 270 + m_numberOfMetabolites * 25 );

            double[] concentrationsToStore = new double[m_numberOfMetabolites];
            double[] concentrationErrorsToStore = new double[m_numberOfMetabolites];
       

            for ( int i = 0; i < m_numberOfMetabolites; i++ )
            {
                concentrationsToStore[i]        = m_concentrations[m_currentlySelectedSignalNumber][i];
                concentrationErrorsToStore[i]   = m_errorConcentrations[m_currentlySelectedSignalNumber][i];
            }

            int outputStyle = SVSMetaboliteTextReport.RAW_VALUES;

            if ( m_rawjMRUIAmplitudesRadioButton.isSelected( ) == true )
            {
                outputStyle = SVSMetaboliteTextReport.RAW_VALUES;
            }

            if ( m_trueTwoCompartmentCorrectedRadioButton.isSelected( ) == true )
            {
                outputStyle = SVSMetaboliteTextReport.TRUE_CONCENTRATIONS_VALUES;
            }

            if ( m_twoCompartmentModelFractionOnlyCorrectedOnlyRadioButton.isSelected( ) == true )
            {
                outputStyle = SVSMetaboliteTextReport.RAW_CORRECTED_VALUES;
            }

            if ( m_singleWaterReferenceOnlyCorrectedRadioButton.isSelected( ) == true )
            {
                outputStyle = SVSMetaboliteTextReport.JMRUI_STYLE_VALUES;
            }

            m_SVSMetaboliteTextReportFrame = new SVSMetaboliteTextReportFrame(
                                                this,
                                                m_dicomDir,
            					d,
                                                m_spectroscopyDicomFileReferences,
                                                m_rawWaterReferenceSpectrosopyDicomFileName,

                                                m_numberOfMetabolites,
                                                m_metaboliteNames,
                                                concentrationsToStore,
                                                concentrationErrorsToStore,
                                                normOutputValues,
                                                normOutputErrorValues,
                                                m_referenceImage0,
                                                m_referenceImage1,
                                                m_referenceImage2,
                                                true,
                                                Integer.parseInt(m_seriesNumberDICOMSENDTextField.getText( ) ),
                                                m_seriesDesciptionDICOMSENDTextField.getText(),
                                                m_SVSJMRUIDicomTag,
                                                outputStyle,
                                                m_globalModel
                                              );
        
            m_sendNumericReportButton.setEnabled( true );
//      frame.dicomizeAndSent( m_sendAsColouredDICOMImageRadioButton.isSelected( ), m_sentCompressedDICOMRadioButton.isSelected( ), m_addjMRUIDICOMTAGCheckBox.isSelected( ) );
//
//      // auto increment the series number
//      int curSeriesNumber = new Integer( m_seriesNumberDICOMSENDTextField.getText( ) ).intValue( ) + 1;
//      m_seriesNumberDICOMSENDTextField.setText( "" + curSeriesNumber );
        }
        else
        {
            JOptionPane.showMessageDialog( this, "Please select a reference database first!" );
        }
}//GEN-LAST:event_m_previewNumericReportButtonActionPerformed

    /**
     * User selected to display the raw fitted data values of the algorithm 
     * without any conversion to concentration.
     * @param evt 
     */
    private void m_rawjMRUIAmplitudesRadioButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_rawjMRUIAmplitudesRadioButtonActionPerformed
       for ( int j = 0; j < m_numberOfSignalsFitted; j++ )
        {
            for ( int i = 0; i < m_numberOfMetabolites; i++ )
            {
                m_concentrations[j][i]  = m_initialConcentrations[j][i];
                m_errorConcentrations[j][i] = m_initialErrorConcentrations[j][i];
            }
        }

        setUpResultsTable( );
        
}//GEN-LAST:event_m_rawjMRUIAmplitudesRadioButtonActionPerformed

    /**
     * Returns the raw peak areas.
     * @return
    */
    public double[] getRawPeakAreas( )
    {

        double[] rawPeakAreas = new double[ m_initialConcentrations[0].length ];

        for ( int i = 0; i < m_initialConcentrations[0].length; i++ )
        {
            rawPeakAreas[i] = m_initialConcentrations[m_currentlySelectedSignalNumber][i];
        }

        return rawPeakAreas;
    }

    /**
     * User pressed done button.
     * @param evt 
     */
    private void m_doneButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_doneButtonActionPerformed
       dispose( );
    }//GEN-LAST:event_m_doneButtonActionPerformed

    /**
     * User choose to display the concentration values using the two compartment
     * model.
     * @param evt 
     */
    private void m_twoCompartmentModelFractionOnlyCorrectedOnlyRadioButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_twoCompartmentModelFractionOnlyCorrectedOnlyRadioButtonActionPerformed


        double correctionFactor = 1.0;

        double acquisitionNumberDifferenceCorrection = Double.parseDouble( m_numberOfWaterAcquisitionsTwoComponentsModelTextField.getText( ) ) / Double.parseDouble(m_numberOfSpectrumMeasurementsTextField.getText( ) );

        double thressholdTC;
        thressholdTC = Double.parseDouble(m_liquorTimeConstantThressholdTextField.getText( ) );

        if ( ( m_TC1 > thressholdTC ) && ( ( m_TC2 < thressholdTC ) ) )
        {
            correctionFactor = 1.0 - ( m_Amp1 / ( m_Amp1 + m_Amp2 ) );
            correctionFactor = 1.0 / correctionFactor;
        }

        if ( ( m_TC1 < thressholdTC ) && ( ( m_TC2 > thressholdTC ) ) )
        {
            correctionFactor = 1.0 - ( m_Amp2 / ( m_Amp1 + m_Amp2 ) );
            correctionFactor = 1.0 / correctionFactor;
        }

        if ( ( ( m_TC1 < thressholdTC ) &&  ( m_TC2 < thressholdTC ) ) ||
             ( ( m_TC1 > thressholdTC ) &&  ( m_TC2 > thressholdTC ) ) )
        {
            correctionFactor = 1.0;
        }

        for ( int j = 0; j < m_numberOfSignalsFitted; j++ )
        {
            for ( int i = 0; i < m_numberOfMetabolites; i++ )
            {
                m_concentrations[j][i]      = m_initialConcentrations[j][i] * correctionFactor * acquisitionNumberDifferenceCorrection;
                m_errorConcentrations[j][i] = m_initialErrorConcentrations[j][i] * correctionFactor * acquisitionNumberDifferenceCorrection;
            }
        }

        setUpResultsTable( );
}//GEN-LAST:event_m_twoCompartmentModelFractionOnlyCorrectedOnlyRadioButtonActionPerformed

    /**
     * Returns the array of fraction only raw peak area parameters.
     * @return
     */
    private double[] getFractionOnlyCorrectedPeakAreas( )
    {
        double[] fractionOnlyCorrectedPeakAreas = new double[ m_initialConcentrations[0].length ];

        double correctionFactor = 1.0;

        double acquisitionNumberDifferenceCorrection = Double.parseDouble( m_numberOfWaterAcquisitionsTwoComponentsModelTextField.getText( ) ) / new Double( m_numberOfSpectrumMeasurementsTextField.getText( ) );

        double thressholdTC;
        thressholdTC = Double.parseDouble( m_liquorTimeConstantThressholdTextField.getText( ) );

        if ( ( m_TC1 > thressholdTC ) && ( ( m_TC2 < thressholdTC ) ) )
        {
            correctionFactor = 1.0 - ( m_Amp1 / ( m_Amp1 + m_Amp2 ) );
            correctionFactor = 1.0 / correctionFactor;
        }

        if ( ( m_TC1 < thressholdTC ) && ( ( m_TC2 > thressholdTC ) ) )
        {
            correctionFactor = 1.0 - ( m_Amp2 / ( m_Amp1 + m_Amp2 ) );
            correctionFactor = 1.0 / correctionFactor;
        }

        if ( ( ( m_TC1 < thressholdTC ) &&  ( m_TC2 < thressholdTC ) ) ||
             ( ( m_TC1 > thressholdTC ) &&  ( m_TC2 > thressholdTC ) ) )
        {
            correctionFactor = 1.0;
        }


        for ( int i = 0; i < m_numberOfMetabolites; i++ )
        {
            fractionOnlyCorrectedPeakAreas[i] = m_initialConcentrations[m_currentlySelectedSignalNumber][i] * correctionFactor * acquisitionNumberDifferenceCorrection;
        }

        return fractionOnlyCorrectedPeakAreas;

    }


    /**
     * 
     * @param evt 
     */
    private void m_singleWaterReferenceOnlyCorrectedRadioButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_singleWaterReferenceOnlyCorrectedRadioButtonActionPerformed


        double correctionFactor = Math.sqrt( m_waterRefReal[0] * m_waterRefReal[0] + m_waterRefImag[0] * m_waterRefImag[0] );
        
        double acquisitionNumberDifferenceCorrection;
        acquisitionNumberDifferenceCorrection = Double.parseDouble(m_numberOfWaterReferenceAcquisitionsSingleReferenceTextField.getText( ) ) / new Double( m_numberOfSpectrumMeasurementsTextField2.getText( ) );

        for ( int j = 0; j < m_numberOfSignalsFitted; j++ )
        {
            for ( int i = 0; i < m_numberOfMetabolites; i++ )
            {
                m_concentrations[j][i]  = m_initialConcentrations[j][i] * 1000.0 * ( 2 * 55.556 ) / correctionFactor;
                m_errorConcentrations[j][i] = m_initialErrorConcentrations[j][i] * 1000.0 * ( 2 * 55.556 )  / correctionFactor;
            }
        }

        setUpResultsTable( );

}//GEN-LAST:event_m_singleWaterReferenceOnlyCorrectedRadioButtonActionPerformed


    /**
     * Returns the JMRUI-typed concentration values.
     */
    private double[] getJMRUITypeConcentrations( )
    {

        double[] JMRUITypeConcentrations = new double[m_initialConcentrations[0].length];
        
        double correctionFactor = Math.sqrt( m_waterRefReal[0] * m_waterRefReal[0] + m_waterRefImag[0] * m_waterRefImag[0] );

        double acquisitionNumberDifferenceCorrection = new Double( m_numberOfWaterReferenceAcquisitionsSingleReferenceTextField.getText( ) ).doubleValue( ) / new Double( m_numberOfSpectrumMeasurementsTextField2.getText( ) ).doubleValue( );

        for ( int j = 0; j < m_numberOfSignalsFitted; j++ )
        {
            for ( int i = 0; i < m_numberOfMetabolites; i++ )
            {
                JMRUITypeConcentrations[i]  = m_initialConcentrations[m_currentlySelectedSignalNumber][i] * 1000.0 * ( 2 * 55.556 ) / correctionFactor;
                // m_errorConcentrations[j][i] = m_initialErrorConcentrations[j][i] * 1000.0 * ( 2 * 55.556 )  / correctionFactor;
            }
        }

        return JMRUITypeConcentrations;
    }


    /**
     * The user wants to see true two compartment model based 
     * concentration values.
     * @param evt 
     */
    private void m_trueTwoCompartmentCorrectedRadioButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_trueTwoCompartmentCorrectedRadioButtonActionPerformed


        System.out.println( "m_trueTwoCompartmentCorrectedRadioButtonActionPerformed is called ! " );

        double correctionFactor = 1.0;

        double selectedTissueTypeWaterContent = m_currentlySelectedTissueTypeWaterContent / 100.0;

        double acquisitionNumberDifferenceCorrection = new Double( m_numberOfWaterAcquisitionsTwoComponentsModelTextField.getText( ) ).doubleValue( ) / new Double( m_numberOfSpectrumMeasurementsTextField.getText( ) ).doubleValue( );

        double thressholdTC
                = new Double( m_liquorTimeConstantThressholdTextField.getText( ) ).doubleValue( );

        if ( ( m_TC1 > thressholdTC ) && ( ( m_TC2 < thressholdTC ) ) )
        {
            correctionFactor = m_Amp2;
        }

        if ( ( m_TC1 < thressholdTC ) && ( ( m_TC2 > thressholdTC ) ) )
        {
            correctionFactor = m_Amp1;
        }

        if ( ( m_TC1 < thressholdTC ) &&  ( m_TC2 < thressholdTC ) )     
        {
            correctionFactor = m_Amp1 + m_Amp2;
        }

        if ( ( m_TC1 > thressholdTC ) &&  ( m_TC2 > thressholdTC ) )
        {
            correctionFactor = m_Amp1 + m_Amp2;
        }

        for ( int j = 0; j < m_numberOfSignalsFitted; j++ )
        {
            for ( int i = 0; i < m_numberOfMetabolites; i++ )
            {
               
                if ( m_correctForMetabT1T2RelaxationRadioButton.isSelected( ) == true )
                {
                    
                    System.out.println( "Correction factor T1 : "  + m_T1TRCorrFactor[i] );
                    System.out.println( "Correction factor T2 : "  + m_T2CorrFactor[i] );
                    
                    m_concentrations[j][i]  = m_initialConcentrations[j][i] * 1000.0 * ( 2 * 55.556 ) * acquisitionNumberDifferenceCorrection / ( selectedTissueTypeWaterContent * correctionFactor * m_T2CorrFactor[i] * m_T1TRCorrFactor[i] * m_multiplicities[i] );
                    m_errorConcentrations[j][i] = m_initialErrorConcentrations[j][i] * 1000.0 * ( 2 * 55.556 ) * acquisitionNumberDifferenceCorrection / (selectedTissueTypeWaterContent * correctionFactor * m_T2CorrFactor[i] * m_T1TRCorrFactor[i] * m_multiplicities[i] );
                }
                else
                {
                    m_concentrations[j][i]  = m_initialConcentrations[j][i] * 1000.0 * ( 2 * 55.556 ) * acquisitionNumberDifferenceCorrection / ( selectedTissueTypeWaterContent * correctionFactor );
                    m_errorConcentrations[j][i] = m_initialErrorConcentrations[j][i] * 1000.0 * ( 2 * 55.556 ) * acquisitionNumberDifferenceCorrection / (selectedTissueTypeWaterContent * correctionFactor );
                }
               
            }
        }

        setUpResultsTable( );
        
}//GEN-LAST:event_m_trueTwoCompartmentCorrectedRadioButtonActionPerformed
    /**
     * Returns the true two compartment model based 
     * concentration values.
     * @param evt 
     */
    private double[] getTrueTwoCompartmentCorrectedConcentrations( )
    {

        double[] trueCorrectedMetaboliteConcentrations = new double[ m_initialConcentrations[0].length ];

        double correctionFactor = 1.0;

        double selectedTissueTypeWaterContent = m_currentlySelectedTissueTypeWaterContent / 100.0;

        double acquisitionNumberDifferenceCorrection = new Double( m_numberOfWaterAcquisitionsTwoComponentsModelTextField.getText( ) ).doubleValue( ) / new Double( m_numberOfSpectrumMeasurementsTextField.getText( ) ).doubleValue( );

        double thressholdTC
                = new Double( m_liquorTimeConstantThressholdTextField.getText( ) ).doubleValue( );

        if ( ( m_TC1 > thressholdTC ) && ( ( m_TC2 < thressholdTC ) ) )
        {
            correctionFactor = m_Amp2;
        }

        if ( ( m_TC1 < thressholdTC ) && ( ( m_TC2 > thressholdTC ) ) )
        {
            correctionFactor = m_Amp1;
        }

        if ( ( m_TC1 < thressholdTC ) &&  ( m_TC2 < thressholdTC ) )
        {
            correctionFactor = m_Amp1 + m_Amp2;
        }

        if ( ( m_TC1 > thressholdTC ) &&  ( m_TC2 > thressholdTC ) )
        {
            correctionFactor = m_Amp1 + m_Amp2;
        }

        for ( int j = 0; j < m_numberOfSignalsFitted; j++ )
        {
            for ( int i = 0; i < m_numberOfMetabolites; i++ )
            {

                System.out.println( " getTrueTwoCompartmentCorrectedConcentrations( ) called!" );
                if ( m_correctForMetabT1T2RelaxationRadioButton.isSelected( ) == true )
                {
                    trueCorrectedMetaboliteConcentrations[i]  = m_initialConcentrations[m_currentlySelectedSignalNumber][i] * 1000.0 * ( 2 * 55.556 ) * acquisitionNumberDifferenceCorrection / ( selectedTissueTypeWaterContent * correctionFactor * m_T2CorrFactor[i] * m_T1TRCorrFactor[i] * m_multiplicities[i] );
                    //m_errorConcentrations[j][i] = m_initialErrorConcentrations[j][i] * 1000.0 * ( 2 * 55.556 ) * acquisitionNumberDifferenceCorrection / (selectedTissueTypeWaterContent * correctionFactor * m_T2CorrFactor[i] * m_T1TRCorrFactor[i] );
                }
                else
                {
                    trueCorrectedMetaboliteConcentrations[i]  = m_initialConcentrations[m_currentlySelectedSignalNumber][i] * 1000.0 * ( 2 * 55.556 ) * acquisitionNumberDifferenceCorrection / ( selectedTissueTypeWaterContent * correctionFactor  * m_multiplicities[i] );
                    //m_errorConcentrations[j][i] = m_initialErrorConcentrations[j][i] * 1000.0 * ( 2 * 55.556 ) * acquisitionNumberDifferenceCorrection / (selectedTissueTypeWaterContent * correctionFactor );
                }
            }
        }

        return trueCorrectedMetaboliteConcentrations;
    }

     private void updateTwoCompartmentCorrectedConcentrationValues( )
     {

        m_trueTwoCompartmentCorrectedRadioButton.setSelected( true );

        double correctionFactor = 1.0;

        double selectedTissueTypeWaterContent = m_currentlySelectedTissueTypeWaterContent / 100.0;

        double acquisitionNumberDifferenceCorrection = Double.parseDouble(m_numberOfWaterAcquisitionsTwoComponentsModelTextField.getText( ) ) / new Double( m_numberOfSpectrumMeasurementsTextField.getText( ) );

        double thressholdTC;
        thressholdTC = new Double( m_liquorTimeConstantThressholdTextField.getText( ) );

        if ( ( m_TC1 > thressholdTC ) && ( ( m_TC2 < thressholdTC ) ) )
        {
            correctionFactor = m_Amp2;
        }

        if ( ( m_TC1 < thressholdTC ) && ( ( m_TC2 > thressholdTC ) ) )
        {
            correctionFactor = m_Amp1;
        }

        if ( ( m_TC1 < thressholdTC ) &&  ( m_TC2 < thressholdTC ) )
        {
            correctionFactor = m_Amp1 + m_Amp2;
        }

        if ( ( m_TC1 > thressholdTC ) &&  ( m_TC2 > thressholdTC ) )
        {
            correctionFactor = m_Amp1 + m_Amp2;
        }

        for ( int j = 0; j < m_numberOfSignalsFitted; j++ )
        {
            for ( int i = 0; i < m_numberOfMetabolites; i++ )
            {
                if ( m_correctForMetabT1T2RelaxationRadioButton.isSelected( ) == true )
                {
                    m_concentrations[j][i]      = m_initialConcentrations[j][i] * 1000.0 * ( 2 * 55.556 ) * acquisitionNumberDifferenceCorrection / ( selectedTissueTypeWaterContent * correctionFactor * m_T2CorrFactor[i] * m_T1TRCorrFactor[i] * m_multiplicities[i] );
                    m_errorConcentrations[j][i] = m_initialErrorConcentrations[j][i] * 1000.0 * ( 2 * 55.556 ) * acquisitionNumberDifferenceCorrection / ( selectedTissueTypeWaterContent * correctionFactor * m_T2CorrFactor[i] * m_T1TRCorrFactor[i] * m_multiplicities[i] );
                }
                else
                {
                    m_concentrations[j][i]      = m_initialConcentrations[j][i] * 1000.0 * ( 2 * 55.556 ) * acquisitionNumberDifferenceCorrection / ( selectedTissueTypeWaterContent * correctionFactor );
                    m_errorConcentrations[j][i] = m_initialErrorConcentrations[j][i] * 1000.0 * ( 2 * 55.556 ) * acquisitionNumberDifferenceCorrection / ( selectedTissueTypeWaterContent * correctionFactor );
                }
            }
        }

        setUpResultsTable( );

    }


    /**
     * If user changes selected tissue type the ( non visible GUI at the moment)
     * must be updated.
     */
    private void changeTissueTypeEvent( )
    {
        if ( m_trueTwoCompartmentCorrectedRadioButton.isSelected( ) == true )
        {
            double correctionFactor = 1.0;

            double selectedTissueTypeWaterContent = new Double( m_tissueWaterContentTextField.getText( ) ).doubleValue() / 100.0;

            //System.out.println( "Currently selected tissue type water content : " + selectedTissueTypeWaterContent );

            double acquisitionNumberDifferenceCorrection = new Double( m_numberOfWaterAcquisitionsTwoComponentsModelTextField.getText( ) ).doubleValue( ) / new Double( m_numberOfSpectrumMeasurementsTextField.getText( ) ).doubleValue( );

            double thressholdTC
                    = new Double( m_liquorTimeConstantThressholdTextField.getText( ) ).doubleValue( );

            if ( ( m_TC1 > thressholdTC ) && ( ( m_TC2 < thressholdTC ) ) )
            {
                correctionFactor = m_Amp2;
            }

            if ( ( m_TC1 < thressholdTC ) && ( ( m_TC2 > thressholdTC ) ) )
            {
                correctionFactor = m_Amp1;
            }

            if ( ( m_TC1 < thressholdTC ) &&  ( m_TC2 < thressholdTC ) )
            {
                correctionFactor = m_Amp1 + m_Amp2;
            }

            if ( ( m_TC1 > thressholdTC ) &&  ( m_TC2 > thressholdTC ) )
            {
                correctionFactor = m_Amp1 + m_Amp2;
            }

            for ( int j = 0; j < m_numberOfSignalsFitted; j++ )
            {
                for ( int i = 0; i < m_numberOfMetabolites; i++ )
                {
                    m_concentrations[j][i]  = m_initialConcentrations[j][i] * 1000.0 * ( 2 * 55.556 ) * acquisitionNumberDifferenceCorrection / ( selectedTissueTypeWaterContent * correctionFactor );
                    m_errorConcentrations[j][i] = m_initialErrorConcentrations[j][i] * 1000.0 * ( 2 * 55.556 ) * acquisitionNumberDifferenceCorrection / ( selectedTissueTypeWaterContent * correctionFactor );
                }
            }

            setUpResultsTable( );
        }
    }



    /**
     * Returns the filename and directory selected by the user.
     * @param path
     * @param subDirString
     * @param fileDefaultName
     * @param buttonText
     * @return 
     */
    protected String getFilePathAndName( String path, String subDirString, 
                                         String fileDefaultName, 
                                         String buttonText )
    {
        JFileChooser fileChooser = new JFileChooser( );
        String selectedFilePathAndName = "";

        String outputPath = new String( path + System.getProperty( "file.separator" ) + subDirString );

        File f = new File( outputPath );
        if ( f.exists( ) == false )
        {
            f.mkdir( );
        }

        // Set the current directory
        fileChooser.setCurrentDirectory( f );

        int result;
        fileChooser.setSelectedFile( new File( fileDefaultName ) );

        result = fileChooser.showDialog( this, buttonText );


        File selected_file = new File( "" );

        switch ( result )
        {
            case JFileChooser.APPROVE_OPTION:   // Approve Open button was pressed
                                                selected_file = fileChooser.getSelectedFile( );
                                                selectedFilePathAndName = selected_file.getPath( );
                                                break;
            case JFileChooser.CANCEL_OPTION:    // Cancel or close dialog was selected
                                                break;
            case JFileChooser.ERROR_OPTION:     // the selected process did not terminate properly
                                                break;
        }

        return selectedFilePathAndName;
    }

    /**
     * Replaces all blanks by "\  " in order to be interpreted
     * correctly by the Windows operating system
     * @param cmd
     * @return
     */
    private String eliminateIllegalFileSystemCharacters( String cmd )
    {

        // first get rid ofm blancs
        cmd = cmd.trim( );

        cmd = cmd.replace( ' ', '_' );
        cmd = cmd.replace( ',', '_' );
        cmd = cmd.replace( ';', '_' );
        cmd = cmd.replace( '%', '_' );
        //cmd.replace( '*', '_' );

        cmd = cmd.replace( 'é', 'e' ); // problematic on some linux systems
        cmd = cmd.replace( 'è', 'e' ); // problematic on some linux systems
        cmd = cmd.replace( 'ö', 'o' ); // problematic on some linux systems
        cmd = cmd.replace( 'ü', 'u' ); // problematic on some linux systems
        cmd = cmd.replace( 'à', 'a' ); // problematic on some linux systems
        cmd = cmd.replace( 'ä', 'a' ); // problematic on some linux systems
        cmd = cmd.replace( 'Ö', 'O' ); // problematic on some linux systems
        cmd = cmd.replace( 'Ü', 'U' ); // problematic on some linux systems

        cmd = cmd.replace( '@', '_' ); // problematic on some linux systems
        cmd = cmd.replace( '#', '_' ); // problematic on some linux systems

        cmd = cmd.replace( '?', '_' );  // problematic on some linux systems

        cmd = cmd.replace( '¢', 'c' ); // problematic on some linux systems
        cmd = cmd.replace( '!', '_' );  // problematic on some linux systems
        cmd = cmd.replace( '§', '_' );  // problematic on some linux systems
        cmd = cmd.replace( '¦', '_' );  // problematic on some linux systems

        cmd = cmd.replace( '$', '_' );  // problematic on some linux systems
        cmd = cmd.replace( '[', '_' );  // problematic on some linux systems
        cmd = cmd.replace( ']', '_' );  // problematic on some linux systems

        cmd = cmd.replace( '^', '_' );  // problematic on some linux systems
        cmd = cmd.replace( '~', '_' );  // problematic on some linux systems

        cmd = cmd.replace( 'ç', 'c' );  // problematic on some linux systems
        cmd = cmd.replace( '~', '_' );  // problematic on some linux systems
        cmd = cmd.replace( '"', '_' );  // problematic on some linux systems

        cmd = cmd.replace("____", "_" );
        cmd = cmd.replace("___", "_" );
        cmd = cmd.replace("__", "_" );

        return cmd;
    }

    /**
     * Allows the user to select one of the four remote DICOM nodes to sent DICOM report to.
     * @param evt
     */
    private void m_changeSelectedDICOMNodeButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_changeSelectedDICOMNodeButtonActionPerformed
        m_dicomNodeSelectionFrame = new DicomNetworkNodeDefinerAndSelectionFrame( this );
        m_dicomNodeSelectionFrame.addActionListener( this );
    }//GEN-LAST:event_m_changeSelectedDICOMNodeButtonActionPerformed

    /**
     * An export as text is performed. 
     * @param evt 
     */
    private void m_exportAsTextButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_exportAsTextButtonActionPerformed


        DicomImageData d_ima_data = new DicomImageData( m_spectroscopyDicomFileReferences.getFirst( ) );
        FileInfo fi = d_ima_data.getHeader( );

        String fileName = fi.getPatientName( ) + "_" + fi.getSeriesDescription( ) + "_" + fi.getSeriesDate( ) + "_" + fi.getSeriesTime( ) + "_" + fi.getSeriesNumber( ) + ".txt";
        fileName = eliminateIllegalFileSystemCharacters( fileName );
        String outputFileName = getFilePathAndName( System.getProperty( "user.home" ), "jMRUIResults", fileName, "Save" );

        File file = new File( outputFileName );

        if ( file.getParentFile( ).exists( ) == false )
        {
            file.getParentFile( ).mkdir( );
        }
        try
        {
            TextFileWriter writer = new TextFileWriter( new FileWriter( outputFileName ) );

            writer.writeStringLn( "Report of:\t" + fi.getFileName( ) );
            writer.writeStringLn( "Patient name:\t" + fi.getPatientName( ) );
            writer.writeStringLn( "Patient date of birth:\t" + fi.getPatientBirthDate( ) );
            writer.writeStringLn( "Study date:\t" + fi.getStudyDate( ) );
            writer.writeStringLn( "Spectroscopic voxel size:\t" + m_voxelDimensionString );

            writer.writeStringLn( "===========================================" );
            writer.writeStringLn( "Amplitude 1:\t" + m_Amp1 );
            writer.writeStringLn( "Amplitude 2:\t" + m_Amp2 );
            writer.writeStringLn( "Time constant 1:\t" + m_TC1 );
            writer.writeStringLn( "Time constant 2:\t" + m_TC2 );
            writer.writeStringLn( "===========================================" );
            writer.writeStringLn( "Minimal GM intensity thresshold :\t" + m_globalModel.getMinimalGMSegmentationPixelValue( ) );
            writer.writeStringLn( "Maximal GM intensity thresshold :\t" + m_globalModel.getMaximalGMSegmentationPixelValue( ) );
            writer.writeStringLn( "Minimal WM intensity thresshold :\t" + m_globalModel.getMinimalWMSegmentationPixelValue( ) );
            writer.writeStringLn( "Maximal WM intensity thresshold :\t" + m_globalModel.getMaximalWMSegmentationPixelValue( ) );
            writer.writeStringLn( "Minimal CSF intensity thresshold :\t" + m_globalModel.getMinimalCSFSegmentationPixelValue( ) );
            writer.writeStringLn( "Maximal CSF intensity thresshold :\t" + m_globalModel.getMaximalCSFSegmentationPixelValue( ) );
            writer.writeStringLn( "===========================================" );
            writer.writeStringLn( "Volume of WM :\t" + m_globalModel.getSVSWMSegmentVolume( ) );
            writer.writeStringLn( "Volume of GM :\t" + m_globalModel.getSVSGMSegmentVolume( ) );
            writer.writeStringLn( "Volume of CSF:\t" + m_globalModel.getSVSCSFSegmentVolume( ) );
            float totalSpecVolume = ( m_globalModel.getSVSCSFSegmentVolume( ) + m_globalModel.getSVSGMSegmentVolume( ) + m_globalModel.getSVSWMSegmentVolume( ) );
            writer.writeStringLn( "Total SVS Volume:\t" + totalSpecVolume );
            writer.writeStringLn( "===========================================" );
            writer.writeStringLn( "%Fraction WM :\t" + ( m_globalModel.getSVSWMSegmentVolume( ) * 100f / totalSpecVolume ) );
            writer.writeStringLn( "%Fraction GM :\t" + ( m_globalModel.getSVSGMSegmentVolume( ) * 100f / totalSpecVolume ) );
            writer.writeStringLn( "%Fraction CSF:\t" + ( m_globalModel.getSVSCSFSegmentVolume( ) * 100f / totalSpecVolume ) );
            writer.writeStringLn( "===========================================" );           
            writer.writeStringLn( "Fit quality number:\t" + m_fitQualityNumber );
            ////////////////////////////////////////////////////////////////////
            
            
            
            // Write raw results
            writer.writeStringLn( "" );
            writer.writeStringLn( "Raw_peak_area_values_[arb_units]" );

            for ( int j = 0; j < m_numberOfSignalsFitted; j++ )
            {
                writer.writeStringLn( "\nResponse number:\t" + j );
                for ( int i = 0; i < m_numberOfMetabolites; i++ )
                {
                    writer.writeStringLn( m_metaboliteNames.get(i) + "\t" + m_initialConcentrations[j][i] + "\t+/-\t" + m_initialErrorConcentrations[j][i] );
                }
            }

            ////////////////////////////////////////////////////////////////////
            // Two compartment corrected (fraction only)
            double correctionFactor = 1.0;

            double acquisitionNumberDifferenceCorrection = Double.parseDouble( m_numberOfWaterAcquisitionsTwoComponentsModelTextField.getText( ) ) / new Double( m_numberOfSpectrumMeasurementsTextField.getText( ) );

            double thressholdTC;
            thressholdTC = new Double( m_liquorTimeConstantThressholdTextField.getText( ) );

            if ( ( m_TC1 > thressholdTC ) && ( ( m_TC2 < thressholdTC ) ) )
            {
                correctionFactor = 1.0 - ( m_Amp1 / ( m_Amp1 + m_Amp2 ) );
                correctionFactor = 1.0 / correctionFactor;
            }

            if ( ( m_TC1 < thressholdTC ) && ( ( m_TC2 > thressholdTC ) ) )
            {
                correctionFactor = 1.0 - ( m_Amp2 / ( m_Amp1 + m_Amp2 ) );
                correctionFactor = 1.0 / correctionFactor;
            }

            if ( ( ( m_TC1 < thressholdTC ) &&  ( m_TC2 < thressholdTC ) ) ||
                 ( ( m_TC1 > thressholdTC ) &&  ( m_TC2 > thressholdTC ) ) )
            {
                correctionFactor = 1.0;
            }

            writer.writeStringLn( "" );
            writer.writeStringLn( "Two_compartment_corrected_fraction_only_peak_area_values_[arbitrary_units]" );
            for ( int j = 0; j < m_numberOfSignalsFitted; j++ )
            {
                writer.writeStringLn( "Response_number:\t" + j );
                for ( int i = 0; i < m_numberOfMetabolites; i++ )
                {
                    writer.writeStringLn( m_metaboliteNames.get(i) + "\t" + m_initialConcentrations[j][i] * acquisitionNumberDifferenceCorrection * correctionFactor + "\t+/-\t" + m_initialErrorConcentrations[j][i] * acquisitionNumberDifferenceCorrection * correctionFactor );
                }
            }
            
            ////////////////////////////////////////////////////////////////////
            // write water ref only corrected
            correctionFactor = Math.sqrt( m_waterRefReal[0] * m_waterRefReal[0] + m_waterRefImag[0] * m_waterRefImag[0] );
            acquisitionNumberDifferenceCorrection = new Double( m_numberOfWaterReferenceAcquisitionsSingleReferenceTextField.getText( ) ) / Double.parseDouble(m_numberOfSpectrumMeasurementsTextField2.getText( ) );

            writer.writeStringLn( "" );
            writer.writeStringLn( "Water_ref_corrected_only_peak_area_values_[mmol_l-1.units]" );
            for ( int j = 0; j < m_numberOfSignalsFitted; j++ )
            {
                writer.writeStringLn( "Response number:\t" + j );
                for ( int i = 0; i < m_numberOfMetabolites; i++ )
                {
                     writer.writeStringLn( m_metaboliteNames.get(i) + "\t" + m_initialConcentrations[j][i] * acquisitionNumberDifferenceCorrection * 1000.0 * ( 2 * 55.556 ) / correctionFactor + "\t+/-\t" + m_initialErrorConcentrations[j][i] * acquisitionNumberDifferenceCorrection * 1000.0 * ( 2 * 55.556 )  / correctionFactor );
                }
            }

            ////////////////////////////////////////////////////////////////////
            // true two compartment model corrected
            m_currentlySelectedTissueTypeWaterContent = new Double( m_tissueWaterContentTextField.getText( ) );

            double selectedTissueTypeWaterContent = m_currentlySelectedTissueTypeWaterContent / 100.0;

            correctionFactor = 1.0;
            acquisitionNumberDifferenceCorrection = Double.parseDouble( m_numberOfWaterAcquisitionsTwoComponentsModelTextField.getText( ) ) / new Double( m_numberOfSpectrumMeasurementsTextField.getText( ) );
            thressholdTC
                    = Double.parseDouble( m_liquorTimeConstantThressholdTextField.getText( ) );

            if ( ( m_TC1 > thressholdTC ) && ( ( m_TC2 < thressholdTC ) ) )
            {
                correctionFactor = m_Amp2;
            }

            if ( ( m_TC1 < thressholdTC ) && ( ( m_TC2 > thressholdTC ) ) )
            {
                correctionFactor = m_Amp1;
            }

            if ( ( m_TC1 < thressholdTC ) &&  ( m_TC2 < thressholdTC ) )
            {
                correctionFactor = m_Amp1 + m_Amp2;
            }

            if ( ( m_TC1 > thressholdTC ) &&  ( m_TC2 > thressholdTC ) )
            {
                correctionFactor = m_Amp1 + m_Amp2;
            }

            writer.writeStringLn( "" );
            writer.writeStringLn( "True_two_compartment_model_corrected_[mmol_l-1.units]" );
            for ( int j = 0; j < m_numberOfSignalsFitted; j++ )
            {
                writer.writeStringLn( "Response number:\t" + j );
                for ( int i = 0; i < m_numberOfMetabolites; i++ )
                {
                    // always T1,T2 correct
                    writer.writeStringLn( m_metaboliteNames.get( i ) + "\t" +  m_initialConcentrations[j][i] * 1000.0 * ( 2 * 55.556 ) * acquisitionNumberDifferenceCorrection / ( selectedTissueTypeWaterContent * correctionFactor * m_multiplicities[i] * m_T2CorrFactor[i] * m_T1TRCorrFactor[i] ) + "\t+/-\t" + m_initialErrorConcentrations[j][i] * 1000.0 * ( 2 * 55.556 ) * acquisitionNumberDifferenceCorrection / (selectedTissueTypeWaterContent * correctionFactor * m_T2CorrFactor[i] * m_T1TRCorrFactor[i] ) );
                }
            }

            writer.flush( );
            writer.close( );
        }
        catch ( IOException e )
        {
            System.out.println( "An IO exception occurred in the while attempting to write : " + fileName );
        }
    }//GEN-LAST:event_m_exportAsTextButtonActionPerformed

    private void m_seriesNumberDICOMSENDTextFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_seriesNumberDICOMSENDTextFieldActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_m_seriesNumberDICOMSENDTextFieldActionPerformed


    /**
     * The user wants to sent a graphic report, that means a report with 
     * spectral curves and localization of the spectroscopic voxel in the images.
     * @param evt
     */
    private void m_previewGraphicReportButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_previewGraphicReportButtonActionPerformed

        
        if ( m_normalDataBaseMetaboliteNamesOrder != null )
        {
            setNormalValuesFromSelectedDatabase( );

            double[] normOutputValues = new double[m_normalDataBaseMetaboliteNamesOrder.size()];
            double[] normOutputErrorValues = new double[m_normalDataBaseMetaboliteNamesOrder.size()];

            // set the default values to -1
            for ( int i = 0; i < m_normalDataBaseMetaboliteNamesOrder.size( ); i++ )
            {
                normOutputValues[i] = -1.0;
                normOutputErrorValues[i] = -1.0;
            }

            for ( int i = 0; i < m_normalDataBaseMetaboliteNamesOrder.size( ); i++ )
            {
                for ( int j = 0; j < m_metaboliteNames.size( ); j++ )
                {
                    // look for matching metabolite names:
                    // it is possible that our reference database does not contain the
                    // fitted metabolite!
                    if ( m_normalDataBaseMetaboliteNamesOrder.get( i ).compareTo( m_metaboliteNames.get( j ) ) == 0 )
                    {
                        normOutputValues[j]      = m_normalConcentrationsFromDB[i];
                        normOutputErrorValues[i] = m_normalErrorConcentrationsFromDB[i];
                    }
                }
            }

            String currentSpectrumFileName = m_spectroscopyDicomFileReferences.get( m_currentlySelectedSignalNumber );
            DicomImageData imaDat = new DicomImageData( currentSpectrumFileName );
 
            FileInfo info0 = imaDat.getHeader( );
            LinkedHashMap hmap0 = imaDat.getSiemensSpecificDataFromFile( );

            float[] measuredReal = new float[m_measuredReal[0].length];
            float[] measuredImag = new float[m_measuredReal[0].length];
            float[] fittedReal   = new float[m_measuredReal[0].length];
            float[] fittedImag   = new float[m_measuredReal[0].length];

            for ( int i = 0; i < m_measuredReal[0].length; i++ )
            {
                measuredReal[i] = (float)m_measuredReal[m_currentlySelectedSignalNumber][i];
                measuredImag[i] = (float)m_measuredImag[m_currentlySelectedSignalNumber][i];
                fittedReal[i]   = (float)m_fittedReal[m_currentlySelectedSignalNumber][i];
                fittedImag[i]   = (float)m_fittedImag[m_currentlySelectedSignalNumber][i];
            }

            double[] concentrations = new double[m_concentrations.length];
            double[] errorConcentrations = new double[m_concentrations.length];


            for ( int i = 0; i < m_concentrations.length; i++ )
            {
                concentrations[i]        = m_concentrations[m_currentlySelectedSignalNumber][i];
                errorConcentrations[i]   = m_errorConcentrations[m_currentlySelectedSignalNumber][i];
            }

            m_SVSFitResultGraphicsDisplayFrame = new SVSFitResultGraphicsDisplayFrame(

                                                         this,
                                                         m_dicomDir,
                                                         info0,
                                                         m_spectroscopyDicomFileReferences,

                                                         measuredReal,
                                                         measuredImag,

                                                         fittedReal,
                                                         fittedImag,

                                                         m_normalMimimaSignalArrayAbsorption,
                                                         m_normalMaximaSignalArrayAbsorption,
                                                         m_normalMeanSignalArrayAbsorption,

                                                         m_normalMimimaSignalArrayDispersion,
                                                         m_normalMaximaSignalArrayDispersion,
                                                         m_normalMeanSignalArrayDispersion,

                                                         m_numberOfMetabolites,
                                                         m_metaboliteNames,

                                                         concentrations, 
                                                         errorConcentrations,

                                                         normOutputValues,
                                                         normOutputErrorValues,

                                                         m_referenceImage0,
                                                         m_referenceImage1,
                                                         m_referenceImage2,

                                                         hmap0,
                                                         new Integer( m_seriesNumberDICOMSENDTextField.getText( ) ).intValue( ),
                                                         m_seriesDesciptionDICOMSENDTextField.getText( ),
                                                         m_SVSJMRUIDicomTag,
                                                         m_globalModel
                                                );
            m_sendGraphicsReportToSelectedNodeButton.setEnabled( true );
        }   
        else
        { 
            JOptionPane.showMessageDialog( this, "Please select a reference database first!" );
        }   
        
//        m_SVSFitResultGraphicsDisplayFrame.dicomizeAndSent( m_sendAsColouredDICOMImageRadioButton.isSelected( ), m_sentCompressedDICOMRadioButton.isSelected( ), m_addjMRUIDICOMTAGCheckBox.isSelected( ) );
//        // auto increment the series number
//        int curSeriesNumber = new Integer( m_seriesNumberDICOMSENDTextField.getText( ) ).intValue( ) + 1;
//        m_seriesNumberDICOMSENDTextField.setText( "" + curSeriesNumber );

}//GEN-LAST:event_m_previewGraphicReportButtonActionPerformed

    private void m_numberOfWaterAcquisitionsTwoComponentsModelTextFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_numberOfWaterAcquisitionsTwoComponentsModelTextFieldActionPerformed
        // TODO add your handling code here:
}//GEN-LAST:event_m_numberOfWaterAcquisitionsTwoComponentsModelTextFieldActionPerformed

    private void m_numberOfSpectrumMeasurementsTextField2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_numberOfSpectrumMeasurementsTextField2ActionPerformed
       m_numberOfSpectrumMeasurementsTextField.setText( m_numberOfSpectrumMeasurementsTextField2.getText( ) );
}//GEN-LAST:event_m_numberOfSpectrumMeasurementsTextField2ActionPerformed

    private void m_numberOfWaterReferenceAcquisitionsSingleReferenceTextFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_numberOfWaterReferenceAcquisitionsSingleReferenceTextFieldActionPerformed
        // TODO add your handling code here:
}//GEN-LAST:event_m_numberOfWaterReferenceAcquisitionsSingleReferenceTextFieldActionPerformed

    private void m_numberOfSpectrumMeasurementsTextFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_numberOfSpectrumMeasurementsTextFieldActionPerformed
        m_numberOfSpectrumMeasurementsTextField2.setText( m_numberOfSpectrumMeasurementsTextField.getText( ) );
}//GEN-LAST:event_m_numberOfSpectrumMeasurementsTextFieldActionPerformed

    private void m_elseTissueRadioButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_elseTissueRadioButtonActionPerformed

        m_tissueWaterContentTextField.setEnabled( true );
        m_tissueWaterContentTextField.setText( "100" );
        changeTissueTypeEvent( );
    }//GEN-LAST:event_m_elseTissueRadioButtonActionPerformed

    private void m_50_50_mixRadioButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_50_50_mixRadioButtonActionPerformed

        m_tissueWaterContentTextField.setEnabled(false);

        if ( m_adultRadioButton.isSelected( ) == true )
        {
            m_tissueWaterContentTextField.setText( "" + m_waterContentWM_GMAdult );
        }
        else
        {
            m_tissueWaterContentTextField.setText( "" + m_waterContentWM_GMBaby );
        }
        changeTissueTypeEvent( );
    }//GEN-LAST:event_m_50_50_mixRadioButtonActionPerformed

    private void m_grayMatterRadioButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_grayMatterRadioButtonActionPerformed

        m_tissueWaterContentTextField.setEnabled(false);

        if ( m_adultRadioButton.isSelected( ) == true )
        {
            m_tissueWaterContentTextField.setText( "" + m_waterContentGMAdult );
        }
        else
        {
            m_tissueWaterContentTextField.setText( "" + m_waterContentGMBaby );
        }
        changeTissueTypeEvent( );
    }//GEN-LAST:event_m_grayMatterRadioButtonActionPerformed

    private void m_whiteMatterRadioButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_whiteMatterRadioButtonActionPerformed

        m_tissueWaterContentTextField.setEnabled( false );

        if ( m_adultRadioButton.isSelected( ) == true )
        {
            m_tissueWaterContentTextField.setText( "" + m_waterContentWMAdult );
        }
        else
        {
            m_tissueWaterContentTextField.setText( "" + m_waterContentWMBaby );
        }
        changeTissueTypeEvent( );
        updateTwoCompartmentCorrectedConcentrationValues( );
    }//GEN-LAST:event_m_whiteMatterRadioButtonActionPerformed

    private void m_adultRadioButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_adultRadioButtonActionPerformed


        if ( m_whiteMatterRadioButton.isSelected( ) )
        {
            m_tissueWaterContentTextField.setText( "" + m_waterContentWMAdult );
        }

        if ( m_grayMatterRadioButton.isSelected( ) )
        {
            m_tissueWaterContentTextField.setText( "" + m_waterContentGMAdult );
        }

        if ( m_50_50_mixRadioButton.isSelected( ) )
        {
            m_tissueWaterContentTextField.setText( "" + m_waterContentWM_GMAdult );
        }

        if ( m_elseTissueRadioButton.isSelected( ) )
        {
            m_tissueWaterContentTextField.setText( "100" );
        }
        changeTissueTypeEvent( );
        updateTwoCompartmentCorrectedConcentrationValues( );
    }//GEN-LAST:event_m_adultRadioButtonActionPerformed

    private void m_babyRadioButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_babyRadioButtonActionPerformed

        if ( m_whiteMatterRadioButton.isSelected( ) )
        {
            m_tissueWaterContentTextField.setText( "" + m_waterContentWMBaby );
        }
        
        if ( m_grayMatterRadioButton.isSelected( ) )
        {
            m_tissueWaterContentTextField.setText( "" + m_waterContentGMBaby );
        }

        if ( m_50_50_mixRadioButton.isSelected( ) )
        {
            m_tissueWaterContentTextField.setText( "" + m_waterContentWM_GMBaby );
        }

        if ( m_elseTissueRadioButton.isSelected( ) )
        {
            m_tissueWaterContentTextField.setText( "100" );
        }
        changeTissueTypeEvent( );
        updateTwoCompartmentCorrectedConcentrationValues( );
    }//GEN-LAST:event_m_babyRadioButtonActionPerformed

    private void m_fieldStrengthDatabaseComboBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_fieldStrengthDatabaseComboBoxActionPerformed
        // TODO add your handling code here:
}//GEN-LAST:event_m_fieldStrengthDatabaseComboBoxActionPerformed

    /**
     * Set the changed selected field strength data base to be used.
     * @param evt 
     */
    private void m_fieldStrengthDatabaseComboBoxItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_m_fieldStrengthDatabaseComboBoxItemStateChanged

        m_currentlySelectedRelaxationTimeDatabaseNameString = (String)m_fieldStrengthDatabaseComboBox.getSelectedItem( );
        m_metabRelaxDataBase = MetaboliteRelaxationTimesDatabase.getInstance( );
        m_currentlySelectedDataBase = m_metabRelaxDataBase.getDataBase( m_currentlySelectedRelaxationTimeDatabaseNameString );
        
//        System.out.println( "Currently selected database string : " + m_currentlySelectedDatabaseNameString );
//        System.out.println( "m_currentlySelectedDataBase.length = " + m_currentlySelectedDataBase.keySet( ).size( ) );

        updateMetaboliteRelaxationTimesTable( );

        // setup the results table again
        setUpResultsTable( );
        
    }//GEN-LAST:event_m_fieldStrengthDatabaseComboBoxItemStateChanged

    
    /**
     * Evokes the relaxation database to be stored.
     * @param evt 
     */
    private void m_saveRelaxationTimeDatabaseButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_saveRelaxationTimeDatabaseButtonActionPerformed
        
        // The user wants the T1 and T2 values and multiplicities saved into the database ...
        for ( int i = 0; i < m_metaboliteNames.size( ); i++  )
        {
            Iterator it = m_currentlySelectedDataBase.keySet( ).iterator( );
            while ( it.hasNext( ) )
            {
                String metabInDBase = (String) it.next( );
                if ( metabInDBase.compareTo( m_metaboliteNames.get( i ) ) == 0 )
                {
                    MetaboliteRelaxationTimeProperties props = m_currentlySelectedDataBase.get( metabInDBase );
                    String T1str = (String)m_metabRelaxTimesTable.getModel( ).getValueAt( i, 1 );
                    String T2str = (String)m_metabRelaxTimesTable.getModel( ).getValueAt( i, 2 );
                    String  multiplicity = (String)m_metabRelaxTimesTable.getModel( ).getValueAt( i, 5 );
                    
                    props.setT1( ( new Double( (String)m_metabRelaxTimesTable.getModel( ).getValueAt( i, 1 ) ) ).doubleValue( ) / 1000.0 );
                    props.setT2( ( new Double( (String)m_metabRelaxTimesTable.getModel( ).getValueAt( i, 2 ) ) ) / 1000.0 );
                    props.setMultiplicity( new Integer( (String)m_metabRelaxTimesTable.getModel( ).getValueAt( i, 5 ) ).intValue( ) );
                }
            }
        }
        
        // Step 2: add all new elements to the selected database.
        for ( int i = 0; i < m_metaboliteNames.size( ); i++ )
        {
            if ( m_currentlySelectedDataBase.keySet( ).contains( m_metaboliteNames.get( i ) ) == false )
            {
                double T1str     = new Double( "" + m_metabRelaxTimesTable.getModel( ).getValueAt( i, 1 ) );
                double T2str     = new Double( "" + m_metabRelaxTimesTable.getModel( ).getValueAt( i, 2 ) );
                int multiplicity = new Integer( "" + m_metabRelaxTimesTable.getModel( ).getValueAt( i, 5 ) );
                
                MetaboliteRelaxationTimeProperties new_component_property =  new MetaboliteRelaxationTimeProperties( );
                new_component_property.setMetaboliteName( m_metaboliteNames.get( i ) );
                new_component_property.setT1( ( T1str / 1000.0 ) );
                new_component_property.setT2( ( T2str / 1000.0 ) );
                new_component_property.setMultiplicity( multiplicity );
                
                m_currentlySelectedDataBase.put( m_metaboliteNames.get( i ), new_component_property );
            }
        }
        
        // Save this value to the database
        MetaboliteRelaxationTimesDatabase.saveRelaxationTimeDataBase( );

        m_currentlySelectedRelaxationTimeDatabaseNameString = (String)m_fieldStrengthDatabaseComboBox.getSelectedItem( );
        m_metabRelaxDataBase = MetaboliteRelaxationTimesDatabase.getInstance( );
        m_currentlySelectedDataBase = m_metabRelaxDataBase.getDataBase( m_currentlySelectedRelaxationTimeDatabaseNameString );

//        System.out.println( "Currently selected database string : " + m_currentlySelectedDatabaseNameString );
//        System.out.println( "m_currentlySelectedDataBase.length = " + m_currentlySelectedDataBase.keySet( ).size( ) );

        updateMetaboliteRelaxationTimesTable( );

        // setup the results table again
        setUpResultsTable( );

        updateTwoCompartmentCorrectedConcentrationValues( );
        
    }//GEN-LAST:event_m_saveRelaxationTimeDatabaseButtonActionPerformed

    /**
     * Selects the response number from the Spinner control.
     * @param evt 
     */
    private void m_selectResponseSpinnerStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_m_selectResponseSpinnerStateChanged

        m_currentlySelectedSignalNumber = ((Integer)m_selectResponseSpinner.getModel( ).getValue( ) ).intValue( ) - 1;
        
        m_rawjMRUIAmplitudesRadioButton.setSelected( true );

        setUpResultsTable( );
    }//GEN-LAST:event_m_selectResponseSpinnerStateChanged

    private void m_sentAsGrayValueDICOMImageRadioButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_sentAsGrayValueDICOMImageRadioButtonActionPerformed

    }//GEN-LAST:event_m_sentAsGrayValueDICOMImageRadioButtonActionPerformed
    
    /**
     * Launches the preview both reports views. 
     * @param evt 
     */
    private void m_previewBothReportsButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_previewBothReportsButtonActionPerformed

        m_previewGraphicReportButtonActionPerformed( evt );
        m_previewNumericReportButtonActionPerformed( evt );
        m_sendBothReportsButton.setEnabled( true );
        
}//GEN-LAST:event_m_previewBothReportsButtonActionPerformed
    
    /**
     * Create a new normal value data base.
     * @param evt 
     */
    private void m_createNewNormalValueDatabaseButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_createNewNormalValueDatabaseButtonActionPerformed
        m_newNormalDatabasePropertiesDefinitionFrame = new NewNormalDatabasePropertiesDefinitionFrame( m_parent );
        m_newNormalDatabasePropertiesDefinitionFrame.setInitialAttributes( "new database",
                                                                           ((m_resonanceFrequency/42857100.0) + "T"),
                                                                           ("" + m_resonanceFrequency),
                                                                           "QUEST",
                                                                           m_scannerNameLabel.getText( ),
                                                                           m_sequenceTypeLabel.getText( ),
                                                                           "Brain",
                                                                           m_echoTimeLabel.getText( ),
                                                                           m_TRLabel.getText( ),
                                                                           "Siemens" );

        m_newNormalDatabasePropertiesDefinitionFrame.addActionListener( this );

}//GEN-LAST:event_m_createNewNormalValueDatabaseButtonActionPerformed

    /**
     * Handles add current case to database event.
     * @param evt 
     */
    private void m_addCurrentCaseToSelectedDatabaseButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_addCurrentCaseToSelectedDatabaseButtonActionPerformed


        double[] rawPeakAreas = getRawPeakAreas( );
        double[] rawFractionCorrectedPeakAreas = getFractionOnlyCorrectedPeakAreas( );
        double[] trueTwoCompartmentModelConcentrations = getTrueTwoCompartmentCorrectedConcentrations( );
        double[] jMRUITypeConcentrations = getJMRUITypeConcentrations( );
        double[] dFreq = getFreqs( );
        double[] phase = getPhase( );
        double[] lorentzWidths = getLorentzWidths( );
        double[] lorentzWidthR = getLorentzWidthRs( );

        NormalDatabaseMetaboliteParameters pars = new NormalDatabaseMetaboliteParameters( m_numberOfMetabolites,
                                                                                          m_metaboliteNames,
                                                                                          rawPeakAreas,
                                                                                          rawFractionCorrectedPeakAreas,
                                                                                          trueTwoCompartmentModelConcentrations,
                                                                                          jMRUITypeConcentrations,
                                                                                          dFreq,
                                                                                          phase,
                                                                                          lorentzWidths,
                                                                                          lorentzWidthR );
        
        double[] measuredReal = getCurrentMeasuredRealResponse( );
        double[] measuredImag = getCurrentMeasuredImagResponse( );
        double[] fittedReal   = getCurrentFittedRealResponse( );
        double[] fittedImag   = getCurrentFittedImagResponse( );

        NormalDatabaseSpectrumResponse normalResponse = new NormalDatabaseSpectrumResponse( measuredReal, measuredImag,
                                                                                            fittedReal, fittedImag );
        if ( normalResponse != null )
        {
            //System.out.println( "normalResponse != null" );
        }
        else
        {
            //System.out.println( "normalResponse == null! > problem!" );
        }


        if ( pars != null )
        {
            //System.out.println( "pars != null" );
        }
        else
        {
            //System.out.println( "pars == null! > problem!" );
        }

        if ( m_currentlySelectedNormalValuesDatabase == null )
        {

        }


        m_currentlySelectedNormalValuesDatabase.addDatabaseObject( normalResponse, pars );


    }//GEN-LAST:event_m_addCurrentCaseToSelectedDatabaseButtonActionPerformed

    /**
     * Handles the send graphical report to selected node event.
     * @param evt 
     */
    private void m_sendGraphicsReportToSelectedNodeButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_sendGraphicsReportToSelectedNodeButtonActionPerformed
        // TODO add your handling code here:
        if ( m_SVSFitResultGraphicsDisplayFrame != null )
        {
            m_SVSFitResultGraphicsDisplayFrame.dicomizeAndSent( m_sendAsColouredDICOMImageRadioButton.isSelected( ), m_sentCompressedDICOMRadioButton.isSelected( ), m_addjMRUIDICOMTAGCheckBox.isSelected( ) );
            // auto increment the series number
            int curSeriesNumber = new Integer( m_seriesNumberDICOMSENDTextField.getText( ) ).intValue( ) + 1;
            m_seriesNumberDICOMSENDTextField.setText( "" + curSeriesNumber );
        }
    }//GEN-LAST:event_m_sendGraphicsReportToSelectedNodeButtonActionPerformed

    
    /**
     * Handles sent numerical report to selected Dicom node.
     * @param evt 
     */
    private void m_sendNumericReportButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_sendNumericReportButtonActionPerformed
        if ( m_SVSMetaboliteTextReportFrame != null )
        {
            m_SVSMetaboliteTextReportFrame.dicomizeAndSent( m_sendAsColouredDICOMImageRadioButton.isSelected( ), m_sentCompressedDICOMRadioButton.isSelected( ), m_addjMRUIDICOMTAGCheckBox.isSelected( ) );

            // auto increment the series number
            int curSeriesNumber = new Integer( m_seriesNumberDICOMSENDTextField.getText( ) ).intValue( ) + 1;
            m_seriesNumberDICOMSENDTextField.setText( "" + curSeriesNumber );
        }
    }//GEN-LAST:event_m_sendNumericReportButtonActionPerformed

    
    /**
     * Handles event to sent graphical as well as numerical report to selected
     * Dicom node.
     * @param evt 
     */
    private void m_sendBothReportsButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_sendBothReportsButtonActionPerformed
       
        m_sendGraphicsReportToSelectedNodeButtonActionPerformed( evt );
        
        m_sendGraphicsReportToSelectedNodeButtonActionPerformed( evt );

    }//GEN-LAST:event_m_sendBothReportsButtonActionPerformed

    
    /**
     * Evokes the update of the GUI to show T1 and T2 time correction.
     * @param evt 
     */
    private void m_correctForMetabT1T2RelaxationRadioButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_correctForMetabT1T2RelaxationRadioButtonActionPerformed

        if ( m_trueTwoCompartmentCorrectedRadioButton.isSelected( ) )
        {
            m_trueTwoCompartmentCorrectedRadioButtonActionPerformed( evt );
        }
        
    }//GEN-LAST:event_m_correctForMetabT1T2RelaxationRadioButtonActionPerformed

    
    /**
     * Evokes the update of the GUI to apply no T1 and T2 time correction.
     * @param evt 
     */
    private void m_notCorrectForMetabT1T2RelaxationRadioButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_notCorrectForMetabT1T2RelaxationRadioButtonActionPerformed

        if ( m_trueTwoCompartmentCorrectedRadioButton.isSelected( ) )
        {
            m_trueTwoCompartmentCorrectedRadioButtonActionPerformed( evt );
        }
        
    }//GEN-LAST:event_m_notCorrectForMetabT1T2RelaxationRadioButtonActionPerformed

    
    /**
     * Launches the normal value database viewer frame.
     * @param evt 
     */
    private void m_browseSelectedNormalValueDatabaseButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_browseSelectedNormalValueDatabaseButtonActionPerformed
        NormalMetaboliteValueDatabaseBrowserFrame frame = new NormalMetaboliteValueDatabaseBrowserFrame( m_parent, m_currentlySelectedNormalValuesDatabase.getDatabaseName( ) );
    }//GEN-LAST:event_m_browseSelectedNormalValueDatabaseButtonActionPerformed

    /**
     * 
     * @param evt 
     */
    private void m_exportSelectedNormalValueDatabaseButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_exportSelectedNormalValueDatabaseButtonActionPerformed
        
        String outputFileName = System.getProperty( "user.home" );
        outputFileName = outputFileName + System.getProperty( "file.separator" ) + m_currentlySelectedRelaxationTimeDatabaseNameString;
                
        m_currentlySelectedNormalValuesDatabase.exportToTextFile( outputFileName );
    }//GEN-LAST:event_m_exportSelectedNormalValueDatabaseButtonActionPerformed


    /**
     * Returns true if string is a double value.
     * @param num
     * @return
     */
    public boolean isDoubleNumber( String num )
    {
        try
        {
            Double.parseDouble( num );
        }
        catch( NumberFormatException nfe )
        {
            return false;
        }

        return true;
      }

    /**
     * Sets the normal values from the selected normal value data base.
     */
    private void setNormalValuesFromSelectedDatabase( )
    {
        // first force an update from the normal values!
        if ( ( m_currentlySelectedNormalValuesDatabase != null ) &&
             ( m_currentlySelectedNormalValuesDatabase.getDatabaseParameters( ).size( ) > 0 ) )
        {
            m_currentlySelectedNormalValuesDatabase.setAllAverageAndStDevsAttributes( );

            m_normalDataBaseMetaboliteNamesOrder = m_currentlySelectedNormalValuesDatabase.getMetaboliteNames( );

            m_normalMimimaSignalArrayAbsorption = m_currentlySelectedNormalValuesDatabase.getMinimaOfNormalRangeSpectrumAbsorption( );
            m_normalMaximaSignalArrayAbsorption = m_currentlySelectedNormalValuesDatabase.getMaximaOfNormalRangeSpectrumAbsorption( );
            m_normalMeanSignalArrayAbsorption   = m_currentlySelectedNormalValuesDatabase.getMeanOfNormalRangeSpectrumAbsorption( );

            m_normalMimimaSignalArrayDispersion = m_currentlySelectedNormalValuesDatabase.getMinimaOfNormalRangeSpectrumDispersion( );
            m_normalMaximaSignalArrayDispersion = m_currentlySelectedNormalValuesDatabase.getMaximaOfNormalRangeSpectrumDispersion( );
            m_normalMeanSignalArrayDispersion   = m_currentlySelectedNormalValuesDatabase.getMeanOfNormalRangeSpectrumDispersion( );
        
        }
        else
        {
            // Dialogs.messageBox( this, "First select a normal value database on 'Normal databases'-tab " );
        }
        
        if ( m_currentlySelectedNormalValuesDatabase != null )
        {
            if ( m_rawjMRUIAmplitudesRadioButton.isSelected( ) == true )
            {

                if (  m_currentlySelectedNormalValuesDatabase.getAveragesDatabaseParameters( ) != null )
                {
                    m_normalConcentrationsFromDB = m_currentlySelectedNormalValuesDatabase.getAveragesDatabaseParameters( ).getRawPeakAreas( );
                }

                if ( m_currentlySelectedNormalValuesDatabase.getStDevsDatabaseParameters( ) != null )
                {
                    m_normalErrorConcentrationsFromDB = m_currentlySelectedNormalValuesDatabase.getStDevsDatabaseParameters( ).getRawPeakAreas( );
                }

            }

            if ( m_twoCompartmentModelFractionOnlyCorrectedOnlyRadioButton.isSelected( ) == true )
            {
                //System.out.println( "Setting normal raw but 2 compartment corrected values!" );
                m_normalConcentrationsFromDB = m_currentlySelectedNormalValuesDatabase.getAveragesDatabaseParameters( ).getRawFractionCorrectedPeakAreas( );
                m_normalErrorConcentrationsFromDB = m_currentlySelectedNormalValuesDatabase.getStDevsDatabaseParameters( ).getRawFractionCorrectedPeakAreas( );
            }

            if ( m_trueTwoCompartmentCorrectedRadioButton.isSelected( ) == true )
            {
                //System.out.println( "Setting normal TRUE concentration values!" );
                m_normalConcentrationsFromDB = m_currentlySelectedNormalValuesDatabase.getAveragesDatabaseParameters( ).getTrueTwoCompartmentModelConcentrations( );
                m_normalErrorConcentrationsFromDB = m_currentlySelectedNormalValuesDatabase.getStDevsDatabaseParameters( ).getTrueTwoCompartmentModelConcentrations( );
            }

            if ( m_singleWaterReferenceOnlyCorrectedRadioButton.isSelected( ) == true )
            {
                //System.out.println( "Setting normal jMRUI-style concentration values!" );
                m_normalConcentrationsFromDB = m_currentlySelectedNormalValuesDatabase.getAveragesDatabaseParameters( ).getJMRUITypeConcentrations( );
                m_normalErrorConcentrationsFromDB = m_currentlySelectedNormalValuesDatabase.getStDevsDatabaseParameters( ).getJMRUITypeConcentrations( );
            }
        }


    }

    /**
     * Is called whenever a registered component raises and ActionEvent.
     * @param e
     */
    @Override
    public void actionPerformed( ActionEvent e )
    {
        // user selected a database ..
        if ( e.getSource( ) == m_normalDataBaseSelectionTable )
        {
            String selectedDatabase = (String)m_normalDataBaseSelectionTable.getModel( ).getValueAt( m_normalDataBaseSelectionTable.getSelectedRow( ), 0 );
            //System.out.println( "Selected database = " + selectedDatabase );
            m_currentlySelectedNormalValuesDatabase = new NormalMetaboliteValuesAndSpectrumDatabase( selectedDatabase );
            // load the normal values from the selected database
            setNormalValuesFromSelectedDatabase( );
            m_addCurrentCaseToSelectedDatabaseButton.setEnabled( true );
            m_browseSelectedNormalValueDatabaseButton.setEnabled( true );
            
            m_exportSelectedNormalValueDatabaseButton.setEnabled( true );
            
        }

        if ( e.getSource( ) == m_dicomNodeSelectionFrame )
        {
            if ( e.getID( ) == DicomNetworkNodeDefinerAndSelectionFrame.OK_BUTTON_PRESSED )
            {
                setSelectedNodeValue( );
                if ( m_selectedHostName != null )
                {
                    m_selectedDICOMNodeValue.setText( m_selectedHostName + " (" + m_selectedHostIP + ")") ;
                }
            }
        }

        if ( e.getSource( ) == m_newNormalDatabasePropertiesDefinitionFrame )
        {

            if ( e.getID( ) == NewNormalDatabasePropertiesDefinitionFrame.ADD_NEW_NORMAL_DATABASE )
            {

                String name                             = m_newNormalDatabasePropertiesDefinitionFrame.getNewDatabaseName( );
                String fieldstrength                    = m_newNormalDatabasePropertiesDefinitionFrame.getFieldStrength( );
                String resonanceFrequency               = m_newNormalDatabasePropertiesDefinitionFrame.getResonanceFrequency( );
                String quantificationMethod             = m_newNormalDatabasePropertiesDefinitionFrame.getQuantificationMethodTextString( );
                String scannerType                      = m_newNormalDatabasePropertiesDefinitionFrame.getScannerTypeTextField( );
                String sequenceTypeTextField            = m_newNormalDatabasePropertiesDefinitionFrame.getSequenceTypeTextField( );
                String tissueTypeTextField              = m_newNormalDatabasePropertiesDefinitionFrame.getTissueTypeTextField( );
                String echoTimeTextField                = m_newNormalDatabasePropertiesDefinitionFrame.getEchoTimeTextField( );
                String repetitionTimeTextField          = m_newNormalDatabasePropertiesDefinitionFrame.getRepetitionTimeTextField( );
                String scannerManufacturerTextField     = m_newNormalDatabasePropertiesDefinitionFrame.getScannerManufacturerTextField( );

                // find out whether the database already excists
                m_currentlySelectedNormalValuesDatabase = new NormalMetaboliteValuesAndSpectrumDatabase( name,
                                                                                                         fieldstrength,
                                                                                                         resonanceFrequency,
                                                                                                         quantificationMethod,
                                                                                                         scannerType,
                                                                                                         sequenceTypeTextField,
                                                                                                         tissueTypeTextField,
                                                                                                         echoTimeTextField,
                                                                                                         repetitionTimeTextField,
                                                                                                         scannerManufacturerTextField );

                if ( m_currentlySelectedNormalValuesDatabase.dataBaseAlreadyExists( ) == false )
                {

                    int currentNumberOfRows = ((DefaultTableModel)m_normalDataBaseSelectionTable.getModel( ) ).getRowCount( );
                    int currentNumberOfColumns = ((DefaultTableModel)m_normalDataBaseSelectionTable.getModel( ) ).getColumnCount( );
                    
                    //System.out.println( "Number of columns = " + currentNumberOfColumns );

                    Object[][] tableData = new Object[currentNumberOfRows + 1][currentNumberOfColumns];

                    if ( currentNumberOfRows > 0 )
                    {
                        for ( int i = 0; i< currentNumberOfColumns; i++ )
                        {
                            for ( int j = 0; j < currentNumberOfRows; j++ )
                            {
                                tableData[j][i] = ((DefaultTableModel)m_normalDataBaseSelectionTable.getModel( ) ).getValueAt( j, i );
                            }
                        }

                        tableData[currentNumberOfRows][0] = name;
                        tableData[currentNumberOfRows][1] = quantificationMethod;
                        tableData[currentNumberOfRows][2] = scannerManufacturerTextField;
                        tableData[currentNumberOfRows][3] = fieldstrength;
                        tableData[currentNumberOfRows][4] = scannerType;
                        tableData[currentNumberOfRows][5] = sequenceTypeTextField;
                        tableData[currentNumberOfRows][6] = tissueTypeTextField;
                        tableData[currentNumberOfRows][7] = echoTimeTextField;
                        tableData[currentNumberOfRows][8] = repetitionTimeTextField;
                    }
                    else
                    {
                         tableData[0][0] = name;
                         tableData[0][1] = quantificationMethod;
                         tableData[0][2] = scannerManufacturerTextField; 
                         tableData[0][3] = fieldstrength;
                         tableData[0][4] = scannerType;
                         tableData[0][5] = sequenceTypeTextField;
                         tableData[0][6] = tissueTypeTextField;
                         tableData[0][7] = echoTimeTextField;
                         tableData[0][8] = repetitionTimeTextField;
                    }

                    ( (NormalDataBaseSelectionTableModel)m_normalDataBaseSelectionTable.getModel( ) ).setDataVector( currentNumberOfRows+1, currentNumberOfColumns, tableData );
                    
                    m_addCurrentCaseToSelectedDatabaseButton.setEnabled( true );
                    m_browseSelectedNormalValueDatabaseButton.setEnabled( true );
                    m_exportSelectedNormalValueDatabaseButton.setEnabled( true );
                    
                }
                else
                {
                    Dialogs.messageBox( this, "Database already exists!" );
                }
            }
        }
    }


    /**
     * Sets the LinkedHashMap containing all Siemens specific header 
     * information.
     * @param ssd
     */
    public void setSiemensSpecificDicomHeaderInfo( LinkedHashMap ssd )
    {

        String name;
        
        if ( ssd != null )
        {
            Iterator it = ssd.keySet( ).iterator( );

            while ( it.hasNext( ) )
            {
                Object key = it.next( );

                if ( ((String)key).compareTo(  "sTXSPEC.asNucleusInfo[0].lFrequency" ) == 0 )
                {
                    m_resonanceFrequency = Long.parseLong((String)ssd.get( key ) );
                }
                

                if ( ((String)key).compareTo( "sSpecPara.sVoI.dThickness" ) == 0 )
                {
                    m_thickVoxelSize = new Double( (String)ssd.get( key ) );
                }

                if ( ((String)key).compareTo( "sSpecPara.sVoI.dReadoutFOV" ) == 0 )
                {
                    m_readOutVoxelSize = new Double( (String)ssd.get( key ) );
                }

                if ( ((String)key).compareTo( "sSpecPara.sVoI.dPhaseFOV" ) == 0 )
                {
                    m_phaseVoxelSize = new Double( (String)ssd.get( key ) );
                }

                if ( ((String)key).compareTo( "sSpecPara.sVoI.sPosition.dSag" ) == 0 )
                {
                    if ( ((String)ssd.get( key ) ).compareTo( "" ) != 0 )
                    {
                        Double d = new Double( (String)ssd.get( key ) );
                        m_dSagVoxelPosition =  d;
                    }
                    else
                    {
                        m_dSagVoxelPosition = 0.00;
                    }
                }

                if ( ((String)key).compareTo( "sSpecPara.sVoI.sPosition.dCor" ) == 0 )
                {
                    if ( ((String)ssd.get( key )).compareTo( "" ) != 0 )
                    {
                        Double d = new Double( (String)ssd.get( key ) );
                        m_dCorVoxelPosition = d;
                    }
                    else
                    {
                        m_dCorVoxelPosition = 0.00;
                    }
                }

                if ( ((String)key).compareTo( "sSpecPara.sVoI.sPosition.dTra" ) == 0 )
                {
                    if ( ( (String)ssd.get( key ) ).compareTo( "" ) != 0 )
                    {
                        Double d = new Double( (String)ssd.get( key ) );
                        m_dTraVoxelPosition = d;
                    }
                    else
                    {
                        m_dTraVoxelPosition = 0.00;
                    }
                }


                if ( ((String)key).compareTo( "sSpecPara.sVoI.sNormal.dSag" ) == 0 )
                {
                    if ( ((String)ssd.get( key )).compareTo( "" ) != 0 )
                    {
                        Double d = new Double( (String)ssd.get( key ) );
                        m_dSagVoxelOrientation = d;
                    }
                    else
                    {
                        m_dSagVoxelOrientation = 0.00;
                    }

                }

                if ( ((String)key).compareTo( "sSpecPara.sVoI.sNormal.dCor" ) == 0 )
                {
                    if ( ((String)ssd.get( key )).compareTo( "" ) != 0 )
                    {
                        Double d = new Double( (String)ssd.get( key ) );
                        m_dCorVoxelOrientation = d;
                    }
                    else
                    {
                        m_dCorVoxelOrientation = 0.00;
                    }

                }

                if ( ((String)key).compareTo( "sSpecPara.sVoI.sNormal.dTra" ) == 0 )
                {
                    if ( ( (String)ssd.get( key ) ).compareTo( "" ) != 0 )
                    {
                        Double d = new Double( (String)ssd.get( key ) );
                        m_dTraVoxelOrientation = d;
                    }
                    else
                    {
                        m_dTraVoxelOrientation = 0.00;
                    }
                }

                if ( ( (String)key).compareTo( "sSpecPara.sVoI.dInPlaneRot" ) == 0 )
                {
                    if ( ( (String)ssd.get( key ) ).compareTo( "" ) != 0 )
                    {
                        Double d = new Double( (String)ssd.get( key ) );
                        m_dInPlaneRotation = d;
                    }
                    else
                    {
                        m_dInPlaneRotation = 0.0;
                    }
                }

                if ( ( (String)key).compareTo( "alTE[0]" ) == 0 )
                {
                    m_TE = ( new Long( (String)ssd.get( key ) ) );
                    m_TE = (long)( m_TE / 1000.0 );
                }

                if ( ( (String)key).compareTo( "sTXSPEC.asNucleusInfo[0].lFrequency" ) == 0 )
                {
                    m_resonanceFrequency = ( new Long( (String)ssd.get( key ) ) );
                }

                if ( ( (String)key).compareTo( "sRXSPEC.alDwellTime[0]" ) == 0 )
                {
                    m_sampleDwellTime = ( new Integer( (String)ssd.get( key ) ) );

                    m_acqFrequency =  1000f * 1000000f / ( 2f * m_sampleDwellTime );

                }

                if ( ( (String)key).compareTo( "alTR[0]" ) == 0 )
                {
                    m_TR = ( new Long( (String)ssd.get( key ) ) );
                    m_TR = (long)( m_TR / 1000.0 );
                }
            }
        }
    }


    /**
     *
     * ¨Returns the frequencies of the currently selected response.
     *
     * @return
     */
    public double[] getFreqs( )
    {
        double[] freqs = new double[ m_numberOfMetabolites ];
        System.arraycopy(m_chemShift[m_currentlySelectedSignalNumber], 0, freqs, 0, m_numberOfMetabolites);
        return freqs;
    }

    /**
     *
     * ¨Returns the phases of the currently selected response.
     *
     * @return
     */
    public double[] getPhase( )
    {
        double[] phase = new double[ m_numberOfMetabolites ];
        System.arraycopy(m_phase[m_currentlySelectedSignalNumber], 0, phase, 0, m_numberOfMetabolites);
        return phase;
    }

    /**
     *
     * Returns the Lorentzian widths of the currently selected response.
     *
     * @return
     */
    public double[] getLorentzWidths( )
    {
        double[] lorentz = new double[ m_numberOfMetabolites ];
        for ( int i = 0; i < m_numberOfMetabolites; i++ )
        {
            lorentz[i] =  m_lorentzWidth[m_currentlySelectedSignalNumber][i];
        }
        return lorentz;
    }

    /**
     *
     * ¨Returns the Lorentz WidthsR of the currently selected response.
     *
     * @return
     */
    public double[] getLorentzWidthRs( )
    {
        double[] lorentzR = new double[ m_numberOfMetabolites ];
        System.arraycopy(m_lorentzWidthR[m_currentlySelectedSignalNumber], 0, lorentzR, 0, m_numberOfMetabolites);
        return lorentzR;
    }

    /**
     * Returns response channel.
     * @return
     */
    public double[] getCurrentMeasuredRealResponse( )
    {
        double[] resp = new double[ m_measuredReal[0].length];
        System.arraycopy(m_measuredReal[this.m_currentlySelectedSignalNumber], 0, resp, 0, m_measuredReal[0].length);
        return resp;
    }

    /**
     * Returns the currently measured imaginary part of the response.
     * @return
     */
    public double[] getCurrentMeasuredImagResponse( )
    {
        double[] resp = new double[ m_measuredImag[0].length];
        System.arraycopy(m_measuredImag[this.m_currentlySelectedSignalNumber], 0, resp, 0, m_measuredImag[0].length);
        return resp;
    }

    /**
     * Returns the currently fitted real part of the response.
     * @return
     */
    public double[] getCurrentFittedRealResponse( )
    {
        double[] resp = new double[ m_fittedReal[0].length];
        System.arraycopy(m_fittedReal[this.m_currentlySelectedSignalNumber], 0, resp, 0, m_fittedReal[0].length);
        return resp;
    }

    /**
     * Returns response channel.
     * @return
     */
    public double[] getCurrentFittedImagResponse( )
    {
        double[] resp = new double[ m_fittedImag[0].length];
        System.arraycopy(m_fittedImag[this.m_currentlySelectedSignalNumber], 0, resp, 0, m_fittedImag[0].length);
        return resp;
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel10;
    private javax.swing.JLabel jLabel11;
    private javax.swing.JLabel jLabel12;
    private javax.swing.JLabel jLabel13;
    private javax.swing.JLabel jLabel14;
    private javax.swing.JLabel jLabel15;
    private javax.swing.JLabel jLabel16;
    private javax.swing.JLabel jLabel17;
    private javax.swing.JLabel jLabel18;
    private javax.swing.JLabel jLabel19;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel20;
    private javax.swing.JLabel jLabel21;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel jLabel8;
    private javax.swing.JLabel jLabel9;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel10;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JPanel jPanel5;
    private javax.swing.JPanel jPanel6;
    private javax.swing.JPanel jPanel7;
    private javax.swing.JPanel jPanel8;
    private javax.swing.JPanel jPanel9;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JRadioButton m_50_50_mixRadioButton;
    private javax.swing.JTable m_ResultsTable;
    private javax.swing.JLabel m_TC_1_Label;
    private javax.swing.JLabel m_TC_1_ValueLabel;
    private javax.swing.JLabel m_TC_2_Label;
    private javax.swing.JLabel m_TC_2_ValueLabel;
    private javax.swing.JLabel m_TRForMetabCorrectionLabel;
    private javax.swing.JLabel m_TRLabel;
    private javax.swing.JTabbedPane m_absoluteQuantificationTabbedPane;
    private javax.swing.JButton m_addCurrentCaseToSelectedDatabaseButton;
    private javax.swing.JCheckBox m_addjMRUIDICOMTAGCheckBox;
    private javax.swing.ButtonGroup m_adultOrBabyButtonGroup;
    private javax.swing.JRadioButton m_adultRadioButton;
    private javax.swing.JLabel m_amp_1_Label;
    private javax.swing.JLabel m_amp_1_ValueLabel;
    private javax.swing.JLabel m_amp_2_Label;
    private javax.swing.JLabel m_amp_2_ValueLabel;
    private javax.swing.JRadioButton m_babyRadioButton;
    private javax.swing.JButton m_browseSelectedNormalValueDatabaseButton;
    private javax.swing.JPanel m_caseOptionsPanel;
    private javax.swing.JButton m_changeSelectedDICOMNodeButton;
    private javax.swing.JRadioButton m_correctForMetabT1T2RelaxationRadioButton;
    private javax.swing.ButtonGroup m_correctForRelaxationTimesButtonGroup;
    private javax.swing.JPanel m_correctionsPanel;
    private javax.swing.JButton m_createNewNormalValueDatabaseButton;
    private javax.swing.ButtonGroup m_dicomImageTypeButtonGroup;
    private javax.swing.JPanel m_dicomNodeSelectionAndDisplayPanel;
    private javax.swing.JPanel m_dicomReportingPanel;
    private javax.swing.JPanel m_dicomTransferSettingsTab;
    private javax.swing.JLabel m_displayResponseNumberLabel;
    private javax.swing.JButton m_doneButton;
    private javax.swing.JPanel m_donePanel;
    private javax.swing.JLabel m_echoTimeLabel;
    private javax.swing.JRadioButton m_elseTissueRadioButton;
    private javax.swing.JButton m_exportAsTextButton;
    private javax.swing.JButton m_exportSelectedNormalValueDatabaseButton;
    private javax.swing.JComboBox m_fieldStrengthDatabaseComboBox;
    private javax.swing.JScrollPane m_fittingParameterScrollPane;
    private javax.swing.JPanel m_fittingParametersTab;
    private javax.swing.JRadioButton m_grayMatterRadioButton;
    private javax.swing.JTextField m_liquorTimeConstantThressholdTextField;
    private javax.swing.JButton m_loadRelaxationTimesDatabaseButton;
    private javax.swing.JTable m_metabRelaxTimesTable;
    private javax.swing.JLabel m_metaboliteDatabaseLabel;
    private javax.swing.JPanel m_metaboliteRelaxationTimesTab;
    private org.spectrim.tdfdfit.quantitativeresults.NormalDataBaseSelectionTable m_normalDataBaseSelectionTable;
    private javax.swing.JPanel m_normalDatabasePanel;
    private javax.swing.JScrollPane m_normalDatabaseScrollPane;
    private javax.swing.JPanel m_normalMetaboliteDatabaseSelectionPanel;
    private javax.swing.JRadioButton m_notCorrectForMetabT1T2RelaxationRadioButton;
    private javax.swing.JLabel m_numberOfSignalsLabel;
    private javax.swing.JLabel m_numberOfSignalsValueLabel;
    private javax.swing.JLabel m_numberOfSpectrumAcquisistionsLabel;
    private javax.swing.JLabel m_numberOfSpectrumAcquisitionsLabel;
    private javax.swing.JTextField m_numberOfSpectrumMeasurementsTextField;
    private javax.swing.JTextField m_numberOfSpectrumMeasurementsTextField2;
    private javax.swing.JLabel m_numberOfWaterAcquiitionsOneReferenceResponseLabel;
    private javax.swing.JLabel m_numberOfWaterAcquisitionsTwoComponentsModelLabel;
    private javax.swing.JTextField m_numberOfWaterAcquisitionsTwoComponentsModelTextField;
    private javax.swing.JTextField m_numberOfWaterReferenceAcquisitionsSingleReferenceTextField;
    private javax.swing.JLabel m_patientAgeLabel;
    private javax.swing.JPanel m_patientAndScannerInfoTab;
    private javax.swing.JLabel m_patientIDLabel;
    private javax.swing.JLabel m_patientNameLabel;
    private javax.swing.ButtonGroup m_peakAreaCorrectionMethodButtonGroup;
    private javax.swing.JLabel m_percentLabel;
    private javax.swing.JRadioButton m_previewBeforeSentRadioButton;
    private javax.swing.JButton m_previewBothReportsButton;
    private javax.swing.JButton m_previewGraphicReportButton;
    private javax.swing.JButton m_previewNumericReportButton;
    private javax.swing.JRadioButton m_rawjMRUIAmplitudesRadioButton;
    private javax.swing.JPanel m_relaxationDataBaseOptionPanel;
    private javax.swing.JPanel m_reportTypeSentSelectionPanel;
    private javax.swing.JLabel m_resonanceFrequencyLabel;
    private javax.swing.JButton m_saveRelaxationTimeDatabaseButton;
    private javax.swing.JLabel m_scannerNameLabel;
    private javax.swing.JPanel m_selectResponseNumberPanel;
    private javax.swing.JSpinner m_selectResponseSpinner;
    private javax.swing.JLabel m_selectedDICOMLabel;
    private javax.swing.JLabel m_selectedDICOMNodeValue;
    private javax.swing.JRadioButton m_sendAsColouredDICOMImageRadioButton;
    private javax.swing.JButton m_sendBothReportsButton;
    private javax.swing.JButton m_sendGraphicsReportToSelectedNodeButton;
    private javax.swing.JButton m_sendNumericReportButton;
    private javax.swing.JRadioButton m_sentAsGrayValueDICOMImageRadioButton;
    private javax.swing.JRadioButton m_sentCompressedDICOMRadioButton;
    private javax.swing.ButtonGroup m_sentCompressionYesNobuttonGroup;
    private javax.swing.JRadioButton m_sentNonCompressedRadioButton;
    private javax.swing.ButtonGroup m_sentOptionsButtonGroup;
    private javax.swing.JRadioButton m_sentWithoutPreviewRadioButton;
    private javax.swing.JLabel m_sequenceTypeLabel;
    private javax.swing.JLabel m_seriesDesciptionDICOMSENDLabel;
    private javax.swing.JTextField m_seriesDesciptionDICOMSENDTextField;
    private javax.swing.JLabel m_seriesDescriptionLabel;
    private javax.swing.JLabel m_seriesNumberDICOMSENDLabel;
    private javax.swing.JTextField m_seriesNumberDICOMSENDTextField;
    private javax.swing.JRadioButton m_singleWaterReferenceOnlyCorrectedRadioButton;
    private javax.swing.JLabel m_spectroscopicVoxelSizeLabel;
    private javax.swing.JLabel m_studyDateLabel;
    private javax.swing.JLabel m_studyDescription;
    private javax.swing.JLabel m_studyTimeLabel;
    private javax.swing.JPanel m_tabContainerPanel;
    private javax.swing.JLabel m_thressholdLabel;
    private javax.swing.JLabel m_thressholdUnitLabel;
    private javax.swing.JPanel m_tissuePropertiesPanel;
    private javax.swing.ButtonGroup m_tissueTypeButtonGroup;
    private javax.swing.JTextField m_tissueWaterContentTextField;
    private javax.swing.JRadioButton m_trueTwoCompartmentCorrectedRadioButton;
    private javax.swing.JRadioButton m_twoCompartmentModelFractionOnlyCorrectedOnlyRadioButton;
    private javax.swing.JPanel m_twoCompartmentPropertiesOverviewPanel;
    private javax.swing.JLabel m_usedTRLabel;
    private javax.swing.JLabel m_waterAmplitudeLabel;
    private javax.swing.JLabel m_waterAmplitudeValueLabel;
    private javax.swing.JLabel m_waterReferenceLabel;
    private javax.swing.JLabel m_waterReferenceLoadedStatusLabel;
    private javax.swing.JPanel m_waterReferencePanel;
    private javax.swing.JPanel m_waterReferencePropertiesTab;
    private javax.swing.JPanel m_waterRferenceCorrrectionModelPanel;
    private javax.swing.JRadioButton m_whiteMatterRadioButton;
    // End of variables declaration//GEN-END:variables

    private final double  m_fitQualityNumber;
    private final boolean m_twoCompartmentModelSet;
    private final boolean m_twoCompartmentModelFitted;
    private final boolean m_referenceWaterDefined;

    private final double[][] m_initialConcentrations;
    private final double[][] m_initialErrorConcentrations;

    private final double[][] m_concentrations;
    private final double[][] m_errorConcentrations;

    private double[] m_normalConcentrationsFromDB;
    private double[] m_normalErrorConcentrationsFromDB;

    private final double[][] m_chemShift;
    private final double[][] m_errorsChemShift;

    private double[] m_normalChemShiftFromDB;
    private double[] m_normalErrorsChemShiftFromDB;

    private final double[][] m_phase;
    private final double[][] m_errorsPhase;

    private double[] m_normalPhaseFromDB;
    private double[] m_normalErrorsPhaseFromDB;

    private final double[][] m_lorentzWidth;
    private final double[][] m_errorsLorentzWidth;

    private double[] m_normalLorentzWidthFromDB;
    private double[] m_normalErrorsLorentzWidthFromDB;

    private final double[][] m_lorentzWidthR;
    private final double[][] m_errorsLorentzWidthR;

    private double[] m_normalLorentzWidthRFromDB;
    private double[] m_normalErrorsLorentzWidthRFromDB;


    // there can be one or more signals
    private int        m_currentlySelectedSignalNumber = 0;
    private Object[][] m_cellData;
    private Object[][] m_metabRelaxTimesTableCellData;

    private int        m_numberOfMetabolites    = -1;
    private int        m_numberOfSignalsFitted  = -1;

    private LinkedList<String> m_metaboliteNames = new LinkedList<>( );
    
    private DefaultTableModel  m_tableModel;
    private DefaultTableModel  m_tableModelRelaxTimes;

    private double[] m_waterRefReal;
    private double[] m_waterRefImag;

    private final double m_TC1;
    private final double m_TC2;

    private final double m_TC_Thresshold = 500;

    private final double m_Amp1;
    private final double m_Amp2;
    
    private int m_selectedRemoteDicomNetworkNode;
    private DicomNetworkNodeDefinerAndSelectionFrame m_dicomNodeSelectionFrame;
    private String m_selectedHostName = "";
    private String m_selectedHostIP = "";
    private final LinkedList<String> m_spectroscopyDicomFileReferences;
    private final String  m_rawWaterReferenceSpectrosopyDicomFileName;
    private float   m_acqFrequency;
    private long    m_resonanceFrequency;
    private Double  m_thickVoxelSize;
    private Double  m_readOutVoxelSize;
    private Double  m_phaseVoxelSize;
    private double  m_dSagVoxelPosition;
    private double  m_dTraVoxelOrientation;
    private double  m_dTraVoxelPosition;
    private double  m_dCorVoxelPosition;
    private double  m_dCorVoxelOrientation;
    private double  m_dSagVoxelOrientation;
    private double  m_dInPlaneRotation;
    private long    m_TE;
    private long    m_TR;
    private float   m_sampleDwellTime;
    private final String  m_voxelDimensionString;


    private double m_currentlySelectedTissueTypeWaterContent = m_waterContentWM_GMAdult;

    // used constants
    private final static double m_waterContentWMAdult       = 64.6;
    private final static double m_waterContentGMAdult       = 78.0;
    private final static double m_waterContentWMBaby        = 84.5;
    private final static double m_waterContentGMBaby        = 84.5;
    private final static double m_waterContentWM_GMAdult    = ( 64.6 + 78.0 ) / 2.0; ;
    private final static double m_waterContentWM_GMBaby     = 84.5;


    private String m_currentlySelectedRelaxationTimeDatabaseNameString;
    private MetaboliteRelaxationTimesDatabase m_metabRelaxDataBase;

    private TreeMap<String, MetaboliteRelaxationTimeProperties> m_currentlySelectedDataBase;

    double[]    m_T1s;
    double[]    m_T2s;
    double[]    m_T1TRCorrFactor;
    double[]    m_T2CorrFactor;
    int[]       m_multiplicities;

    private final SVS_JMRUI_DICOM_Tag_FAST m_SVSJMRUIDicomTag;

    private final String m_referenceImage0;
    private final String m_referenceImage1;
    private final String m_referenceImage2;

    double[][] m_measuredReal;
    double[][] m_measuredImag;
    double[][] m_fittedReal;
    double[][] m_fittedImag;

    NewNormalDatabasePropertiesDefinitionFrame m_newNormalDatabasePropertiesDefinitionFrame;
    private final JFrame m_parent;
    private NormalMetaboliteValuesAndSpectrumDatabase m_currentlySelectedNormalValuesDatabase;
    private LinkedList<String> m_normalDataBaseMetaboliteNamesOrder;

    private float[] m_normalMimimaSignalArrayAbsorption;
    private float[] m_normalMaximaSignalArrayAbsorption;
    private float[] m_normalMeanSignalArrayAbsorption;

    private float[] m_normalMimimaSignalArrayDispersion;
    private float[] m_normalMaximaSignalArrayDispersion;
    private float[] m_normalMeanSignalArrayDispersion;

    private SVSFitResultGraphicsDisplayFrame    m_SVSFitResultGraphicsDisplayFrame;
    private SVSMetaboliteTextReportFrame        m_SVSMetaboliteTextReportFrame;

    private final static int METAB_DATA_RAW_UNCORRECTED_TYPE = 0;
    private final static int METAB_DATA_RAW_COMPONENTS_ONLY_CORRECTED_TYPE = 1;
    private final static int METAB_DATA_JMRUI_CONC_STYLE = 2;
    private final static int METAB_DATA_TRUECONC_TWO_COMPONENTS_CORRECTED = 3;
    
    private final DicomDirectory    m_dicomDir;
    private GlobalModel             m_globalModel;
    
}